/* Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Copyright:
 *   2020      Evan Nemerson <evan@nemerson.com>
 */

#define SIMDE_TESTS_CURRENT_ISAX svml
#include <test/x86/avx512/test-avx512.h>
#include <simde/x86/svml.h>

static int
test_simde_mm_acos_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     1.21)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     1.54)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(     1.08)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(     0.96)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     2.61), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.80)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     2.74)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     1.78), SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     2.29)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     0.81)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_acos_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_acos_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    2.42), SIMDE_FLOAT64_C(    1.21)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.76), SIMDE_FLOAT64_C(    1.53)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.88), SIMDE_FLOAT64_C(    1.54)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.05), SIMDE_FLOAT64_C(    0.84)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.84), SIMDE_FLOAT64_C(    1.08)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.59), SIMDE_FLOAT64_C(    1.14)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.49), SIMDE_FLOAT64_C(    0.96)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    2.32), SIMDE_FLOAT64_C(    2.33)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_acos_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_acos_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     0.84),
                         SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     1.54),
                         SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     1.53),
                         SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     1.21)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69),
                         SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42),
                         SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     2.33),
                         SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.14),
                         SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(     1.08)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86),
                         SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(     1.54),
                         SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     2.74),
                         SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     2.61),
                         SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.80)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45),
                         SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21),
                         SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     2.04),
                         SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     1.78),
                         SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     2.29)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.44),
                         SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.38),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.84)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     1.12),
                         SIMDE_FLOAT32_C(     2.19), SIMDE_FLOAT32_C(     1.18),
                         SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     0.14),
                         SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     0.57)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.03)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     1.16),
                         SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     1.22),
                         SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(     1.83),
                         SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     1.60)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.94),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.40)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     1.65),
                         SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     2.79),
                         SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     2.45),
                         SIMDE_FLOAT32_C(     2.15), SIMDE_FLOAT32_C(     1.16)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.25)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     0.61),
                         SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     1.32)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_acos_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_acos_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.76), SIMDE_FLOAT64_C(    1.53),
                         SIMDE_FLOAT64_C(    2.42), SIMDE_FLOAT64_C(    1.21)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.05), SIMDE_FLOAT64_C(    0.84),
                         SIMDE_FLOAT64_C(    1.88), SIMDE_FLOAT64_C(    1.54)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.59), SIMDE_FLOAT64_C(    1.14),
                         SIMDE_FLOAT64_C(    1.84), SIMDE_FLOAT64_C(    1.08)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.32), SIMDE_FLOAT64_C(    2.33),
                         SIMDE_FLOAT64_C(    1.49), SIMDE_FLOAT64_C(    0.96)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.86),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    0.70)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.89), SIMDE_FLOAT64_C(    2.61),
                         SIMDE_FLOAT64_C(    2.00), SIMDE_FLOAT64_C(    0.80)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.38), SIMDE_FLOAT64_C(   -0.92)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.03), SIMDE_FLOAT64_C(    1.54),
                         SIMDE_FLOAT64_C(    1.96), SIMDE_FLOAT64_C(    2.74)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(   -0.66)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.31), SIMDE_FLOAT64_C(    1.78),
                         SIMDE_FLOAT64_C(    2.94), SIMDE_FLOAT64_C(    2.29)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.69)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.39), SIMDE_FLOAT64_C(    2.04),
                         SIMDE_FLOAT64_C(    1.34), SIMDE_FLOAT64_C(    0.81)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_acos_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_acos_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(     1.08),
                         SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     1.54),
                         SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     1.21)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66),
                         SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     1.78), SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     2.29),
                         SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     2.74),
                         SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     2.61), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.80)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.38),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.84)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     1.22),
                         SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     1.60),
                         SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     1.12), SIMDE_FLOAT32_C(     2.19), SIMDE_FLOAT32_C(     1.18),
                         SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     0.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.25),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.94),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.40)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     0.61),
                         SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     1.32),
                         SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     2.79),
                         SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     2.15), SIMDE_FLOAT32_C(     1.16)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.53),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.17)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     1.22),
                         SIMDE_FLOAT32_C(     2.63), SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(     2.13),
                         SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.85),
                         SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     1.74)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.59),
                         SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.62),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(    -0.91),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.74)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(     2.20),
                         SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     0.90),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.43), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     2.71),
                         SIMDE_FLOAT32_C(     1.23), SIMDE_FLOAT32_C(     1.47), SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(     2.40)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.98),
                         SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.10)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     2.18), SIMDE_FLOAT32_C(     3.14),
                         SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(     2.45),
                         SIMDE_FLOAT32_C(     2.19), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.20),
                         SIMDE_FLOAT32_C(     2.43), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     1.67)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.13),
                         SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.70)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     1.60),
                         SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(     1.70),
                         SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     1.17),
                         SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     2.35)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_acos_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_acos_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.66),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.70),
                         SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.35)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.98),
                         SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.27),
                         SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.75)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(    -0.66),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     2.00),
                         SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.84),
                         SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     0.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.55),
                         SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.03)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.99)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.74), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(     1.32), SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(     2.45),
                         SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     1.83),
                         SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(     1.12), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.14)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(     0.10),
                         SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.80),
                         SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.54)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.45),
                         SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    -0.87),
                         SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.07)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(     1.23),
                         SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(    -0.80),
                         SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.54)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.34),
                         SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(    -0.76)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.98)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     3.14),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     0.20)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.18),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.13),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.44)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(    -0.38),
                         SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     0.48)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     1.41),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     1.13), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     1.46),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(     0.44)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -0.81), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(    -0.78),
                         SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.07),
                         SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.43)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.76),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.72),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     2.40), SIMDE_FLOAT32_C(    -0.78),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     1.41),
                         SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.43)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.64),
                         SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.08),
                         SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(    -0.89)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.43),
                         SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.58),
                         SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     0.09)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.64),
                         SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     2.19),
                         SIMDE_FLOAT32_C(     1.61), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     1.48)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.82),
                         SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.20)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.35),
                         SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(    -0.25),
                         SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.85)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     1.93),
                         SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.55)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_acos_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_acos_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.05), SIMDE_FLOAT64_C(    0.84),
                         SIMDE_FLOAT64_C(    1.88), SIMDE_FLOAT64_C(    1.54),
                         SIMDE_FLOAT64_C(    1.76), SIMDE_FLOAT64_C(    1.53),
                         SIMDE_FLOAT64_C(    2.42), SIMDE_FLOAT64_C(    1.21)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.32), SIMDE_FLOAT64_C(    2.33),
                         SIMDE_FLOAT64_C(    1.49), SIMDE_FLOAT64_C(    0.96),
                         SIMDE_FLOAT64_C(    0.59), SIMDE_FLOAT64_C(    1.14),
                         SIMDE_FLOAT64_C(    1.84), SIMDE_FLOAT64_C(    1.08)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.38), SIMDE_FLOAT64_C(   -0.92),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.86),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    0.70)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.03), SIMDE_FLOAT64_C(    1.54),
                         SIMDE_FLOAT64_C(    1.96), SIMDE_FLOAT64_C(    2.74),
                         SIMDE_FLOAT64_C(    1.89), SIMDE_FLOAT64_C(    2.61),
                         SIMDE_FLOAT64_C(    2.00), SIMDE_FLOAT64_C(    0.80)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.69),
                         SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(   -0.66)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.39), SIMDE_FLOAT64_C(    2.04),
                         SIMDE_FLOAT64_C(    1.34), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(    1.31), SIMDE_FLOAT64_C(    1.78),
                         SIMDE_FLOAT64_C(    2.94), SIMDE_FLOAT64_C(    2.29)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(    0.38),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.03), SIMDE_FLOAT64_C(    0.84)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.45), SIMDE_FLOAT64_C(    1.12),
                         SIMDE_FLOAT64_C(    2.19), SIMDE_FLOAT64_C(    1.18),
                         SIMDE_FLOAT64_C(    2.45), SIMDE_FLOAT64_C(    0.14),
                         SIMDE_FLOAT64_C(    1.54), SIMDE_FLOAT64_C(    0.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.34),
                         SIMDE_FLOAT64_C(    0.53), SIMDE_FLOAT64_C(   -0.26),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(   -0.03)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.97), SIMDE_FLOAT64_C(    1.16),
                         SIMDE_FLOAT64_C(    0.85), SIMDE_FLOAT64_C(    1.22),
                         SIMDE_FLOAT64_C(    1.01), SIMDE_FLOAT64_C(    1.83),
                         SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(    1.60)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.20), SIMDE_FLOAT64_C(   -0.08),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.94),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C(    0.40)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.77), SIMDE_FLOAT64_C(    1.65),
                         SIMDE_FLOAT64_C(    1.22), SIMDE_FLOAT64_C(    2.79),
                         SIMDE_FLOAT64_C(    2.42), SIMDE_FLOAT64_C(    2.45),
                         SIMDE_FLOAT64_C(    2.15), SIMDE_FLOAT64_C(    1.16)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.47), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(    0.91), SIMDE_FLOAT64_C(    0.60),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(    0.25)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.08), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(    1.72), SIMDE_FLOAT64_C(    0.61),
                         SIMDE_FLOAT64_C(    0.43), SIMDE_FLOAT64_C(    0.93),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    1.32)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_acos_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_acos_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.69), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.42), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(    0.04), SIMDE_FLOAT64_C(    0.35)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(    0.08),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(   -0.27),
                         SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(   -0.75)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.32), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.42), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(    1.05), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(    1.76), SIMDE_FLOAT64_C(    2.42)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(    0.23),
                         SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(   -0.38),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.42)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.84), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.66), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.92), SIMDE_FLOAT64_C(   -0.86)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.57), SIMDE_FLOAT64_C(    2.04),
                         SIMDE_FLOAT64_C(    0.81), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    1.54),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(    2.61)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.26),
                         SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(    0.99)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C(   -0.39),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.53),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(   -0.77)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.15), SIMDE_FLOAT64_C(    1.97),
                         SIMDE_FLOAT64_C(    0.85), SIMDE_FLOAT64_C(    1.01),
                         SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(    2.45),
                         SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(    2.45)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.12), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.91),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.75)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.54), SIMDE_FLOAT64_C(   -0.17),
                         SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(    0.60), SIMDE_FLOAT64_C(    0.25),
                         SIMDE_FLOAT64_C(   -0.08), SIMDE_FLOAT64_C(   -0.94)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.12), SIMDE_FLOAT64_C(    1.74),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.61),
                         SIMDE_FLOAT64_C(    0.93), SIMDE_FLOAT64_C(    1.32),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(    2.79)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.10), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.34),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.53),
                         SIMDE_FLOAT64_C(   -0.82), SIMDE_FLOAT64_C(    0.66)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.33), SIMDE_FLOAT64_C(    0.46),
                         SIMDE_FLOAT64_C(   -0.18), SIMDE_FLOAT64_C(    0.32),
                         SIMDE_FLOAT64_C(   -0.87), SIMDE_FLOAT64_C(   -0.33),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.56)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.23), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    1.25),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.53),
                         SIMDE_FLOAT64_C(   -0.82), SIMDE_FLOAT64_C(    0.98)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.02),
                         SIMDE_FLOAT64_C(   -0.45), SIMDE_FLOAT64_C(    0.51),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.98)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.98), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.10), SIMDE_FLOAT64_C(    0.84),
                         SIMDE_FLOAT64_C(   -0.59), SIMDE_FLOAT64_C(    0.73),
                         SIMDE_FLOAT64_C(    0.62), SIMDE_FLOAT64_C(    0.14)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    1.14),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.02),
                         SIMDE_FLOAT64_C(    2.20), SIMDE_FLOAT64_C(    0.51),
                         SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(    1.43)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(   -0.70), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.92),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(   -0.07)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.51), SIMDE_FLOAT64_C(    0.01),
                         SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.57), SIMDE_FLOAT64_C(   -0.34),
                         SIMDE_FLOAT64_C(    0.29), SIMDE_FLOAT64_C(   -0.58)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(    1.56),
                         SIMDE_FLOAT64_C(   -0.70), SIMDE_FLOAT64_C(    2.45),
                         SIMDE_FLOAT64_C(    2.18), SIMDE_FLOAT64_C(    1.92),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    2.19)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.48), SIMDE_FLOAT64_C(    0.94),
                         SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(   -0.44),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.93),
                         SIMDE_FLOAT64_C(   -0.33), SIMDE_FLOAT64_C(   -0.18)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.78), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(   -0.36), SIMDE_FLOAT64_C(   -0.03),
                         SIMDE_FLOAT64_C(    0.01), SIMDE_FLOAT64_C(   -0.13)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.47), SIMDE_FLOAT64_C(    1.12),
                         SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(    1.77),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    1.60),
                         SIMDE_FLOAT64_C(   -0.33), SIMDE_FLOAT64_C(    1.70)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_acos_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_acosh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     1.81), SIMDE_FLOAT32_C(     5.44)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     2.17), SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     2.38)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     5.94), SIMDE_FLOAT32_C(     6.51), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(     4.41)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     1.87), SIMDE_FLOAT32_C(     2.16)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     7.02), SIMDE_FLOAT32_C(     5.69), SIMDE_FLOAT32_C(     3.41), SIMDE_FLOAT32_C(     5.84)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     1.90), SIMDE_FLOAT32_C(     2.45)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     2.06), SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     4.58), SIMDE_FLOAT32_C(     6.19)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     2.51)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     6.60)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     2.57)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     1.25)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     2.16), SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     0.69)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     5.16), SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     2.12)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     1.38)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     4.89), SIMDE_FLOAT32_C(     2.81), SIMDE_FLOAT32_C(     5.07), SIMDE_FLOAT32_C(     6.57)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     2.27), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     2.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_acosh_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_acosh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    1.81), SIMDE_FLOAT64_C(    5.44)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.20), SIMDE_FLOAT64_C(    2.38)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    3.69), SIMDE_FLOAT64_C(    4.43)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.98), SIMDE_FLOAT64_C(    2.17)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    3.32), SIMDE_FLOAT64_C(    4.41)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.87), SIMDE_FLOAT64_C(    2.16)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    5.94), SIMDE_FLOAT64_C(    6.51)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    2.47), SIMDE_FLOAT64_C(    2.56)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    3.41), SIMDE_FLOAT64_C(    5.84)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.90), SIMDE_FLOAT64_C(    2.45)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    7.02), SIMDE_FLOAT64_C(    5.69)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    2.64), SIMDE_FLOAT64_C(    2.42)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    4.58), SIMDE_FLOAT64_C(    6.19)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    2.20), SIMDE_FLOAT64_C(    2.51)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    2.06), SIMDE_FLOAT64_C(    2.04)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.35), SIMDE_FLOAT64_C(    1.34)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_acosh_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_acosh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     5.94), SIMDE_FLOAT32_C(     6.51),
                         SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(     4.41),
                         SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(     4.43),
                         SIMDE_FLOAT32_C(     1.81), SIMDE_FLOAT32_C(     5.44)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(     2.56),
                         SIMDE_FLOAT32_C(     1.87), SIMDE_FLOAT32_C(     2.16),
                         SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     2.17),
                         SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     2.38)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.06), SIMDE_FLOAT32_C(     2.04),
                         SIMDE_FLOAT32_C(     4.58), SIMDE_FLOAT32_C(     6.19),
                         SIMDE_FLOAT32_C(     7.02), SIMDE_FLOAT32_C(     5.69),
                         SIMDE_FLOAT32_C(     3.41), SIMDE_FLOAT32_C(     5.84)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     1.34),
                         SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     2.51),
                         SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     2.42),
                         SIMDE_FLOAT32_C(     1.90), SIMDE_FLOAT32_C(     2.45)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(     4.39),
                         SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     1.25),
                         SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     1.46),
                         SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     6.60)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     2.16),
                         SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     2.57)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     4.89), SIMDE_FLOAT32_C(     2.81),
                         SIMDE_FLOAT32_C(     5.07), SIMDE_FLOAT32_C(     6.57),
                         SIMDE_FLOAT32_C(     5.16), SIMDE_FLOAT32_C(     3.60),
                         SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     2.12)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.27), SIMDE_FLOAT32_C(     1.69),
                         SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     2.57),
                         SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     1.95),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     1.38)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     5.76),
                         SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     5.56),
                         SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     7.58),
                         SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     7.08)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     2.44),
                         SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(     2.40),
                         SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     2.71),
                         SIMDE_FLOAT32_C(     2.16), SIMDE_FLOAT32_C(     2.65)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.02), SIMDE_FLOAT32_C(     5.61),
                         SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     5.42),
                         SIMDE_FLOAT32_C(     6.06), SIMDE_FLOAT32_C(     3.43),
                         SIMDE_FLOAT32_C(     6.88), SIMDE_FLOAT32_C(     4.20)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     2.41),
                         SIMDE_FLOAT32_C(     2.55), SIMDE_FLOAT32_C(     2.37),
                         SIMDE_FLOAT32_C(     2.49), SIMDE_FLOAT32_C(     1.90),
                         SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     2.11)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.63), SIMDE_FLOAT32_C(     4.03),
                         SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(     1.18),
                         SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     1.77),
                         SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(     5.62)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     2.07),
                         SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(     1.21), SIMDE_FLOAT32_C(     1.17),
                         SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     2.41)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     5.85), SIMDE_FLOAT32_C(     6.54),
                         SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     7.00),
                         SIMDE_FLOAT32_C(     7.30), SIMDE_FLOAT32_C(     6.28),
                         SIMDE_FLOAT32_C(     6.91), SIMDE_FLOAT32_C(     5.14)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     2.57),
                         SIMDE_FLOAT32_C(     2.01), SIMDE_FLOAT32_C(     2.63),
                         SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     2.52),
                         SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     2.32)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_acosh_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_acosh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.69), SIMDE_FLOAT64_C(    4.43),
                         SIMDE_FLOAT64_C(    1.81), SIMDE_FLOAT64_C(    5.44)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.98), SIMDE_FLOAT64_C(    2.17),
                         SIMDE_FLOAT64_C(    1.20), SIMDE_FLOAT64_C(    2.38)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    5.94), SIMDE_FLOAT64_C(    6.51),
                         SIMDE_FLOAT64_C(    3.32), SIMDE_FLOAT64_C(    4.41)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.47), SIMDE_FLOAT64_C(    2.56),
                         SIMDE_FLOAT64_C(    1.87), SIMDE_FLOAT64_C(    2.16)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    7.02), SIMDE_FLOAT64_C(    5.69),
                         SIMDE_FLOAT64_C(    3.41), SIMDE_FLOAT64_C(    5.84)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.64), SIMDE_FLOAT64_C(    2.42),
                         SIMDE_FLOAT64_C(    1.90), SIMDE_FLOAT64_C(    2.45)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.06), SIMDE_FLOAT64_C(    2.04),
                         SIMDE_FLOAT64_C(    4.58), SIMDE_FLOAT64_C(    6.19)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.35), SIMDE_FLOAT64_C(    1.34),
                         SIMDE_FLOAT64_C(    2.20), SIMDE_FLOAT64_C(    2.51)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.29), SIMDE_FLOAT64_C(    1.46),
                         SIMDE_FLOAT64_C(    2.92), SIMDE_FLOAT64_C(    6.60)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.86), SIMDE_FLOAT64_C(    0.93),
                         SIMDE_FLOAT64_C(    1.73), SIMDE_FLOAT64_C(    2.57)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.83), SIMDE_FLOAT64_C(    4.39),
                         SIMDE_FLOAT64_C(    3.03), SIMDE_FLOAT64_C(    1.25)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.70), SIMDE_FLOAT64_C(    2.16),
                         SIMDE_FLOAT64_C(    1.77), SIMDE_FLOAT64_C(    0.69)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    5.16), SIMDE_FLOAT64_C(    3.60),
                         SIMDE_FLOAT64_C(    1.08), SIMDE_FLOAT64_C(    2.12)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.32), SIMDE_FLOAT64_C(    1.95),
                         SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(    1.38)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    4.89), SIMDE_FLOAT64_C(    2.81),
                         SIMDE_FLOAT64_C(    5.07), SIMDE_FLOAT64_C(    6.57)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.27), SIMDE_FLOAT64_C(    1.69),
                         SIMDE_FLOAT64_C(    2.31), SIMDE_FLOAT64_C(    2.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_acosh_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_acosh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.06), SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     4.58), SIMDE_FLOAT32_C(     6.19),
                         SIMDE_FLOAT32_C(     7.02), SIMDE_FLOAT32_C(     5.69), SIMDE_FLOAT32_C(     3.41), SIMDE_FLOAT32_C(     5.84),
                         SIMDE_FLOAT32_C(     5.94), SIMDE_FLOAT32_C(     6.51), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(     4.41),
                         SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     1.81), SIMDE_FLOAT32_C(     5.44)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     2.51),
                         SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     1.90), SIMDE_FLOAT32_C(     2.45),
                         SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     1.87), SIMDE_FLOAT32_C(     2.16),
                         SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     2.17), SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     2.38)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.89), SIMDE_FLOAT32_C(     2.81), SIMDE_FLOAT32_C(     5.07), SIMDE_FLOAT32_C(     6.57),
                         SIMDE_FLOAT32_C(     5.16), SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     2.12),
                         SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     1.25),
                         SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     6.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.27), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     2.57),
                         SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     1.38),
                         SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     2.16), SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     2.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.02), SIMDE_FLOAT32_C(     5.61), SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     5.42),
                         SIMDE_FLOAT32_C(     6.06), SIMDE_FLOAT32_C(     3.43), SIMDE_FLOAT32_C(     6.88), SIMDE_FLOAT32_C(     4.20),
                         SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     5.76), SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     5.56),
                         SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     7.58), SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     7.08)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     2.41), SIMDE_FLOAT32_C(     2.55), SIMDE_FLOAT32_C(     2.37),
                         SIMDE_FLOAT32_C(     2.49), SIMDE_FLOAT32_C(     1.90), SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     2.11),
                         SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(     2.40),
                         SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     2.71), SIMDE_FLOAT32_C(     2.16), SIMDE_FLOAT32_C(     2.65)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.85), SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     7.00),
                         SIMDE_FLOAT32_C(     7.30), SIMDE_FLOAT32_C(     6.28), SIMDE_FLOAT32_C(     6.91), SIMDE_FLOAT32_C(     5.14),
                         SIMDE_FLOAT32_C(     3.63), SIMDE_FLOAT32_C(     4.03), SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(     1.18),
                         SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(     5.62)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     2.57), SIMDE_FLOAT32_C(     2.01), SIMDE_FLOAT32_C(     2.63),
                         SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     2.32),
                         SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(     1.21), SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     2.41)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     6.80), SIMDE_FLOAT32_C(     5.37), SIMDE_FLOAT32_C(     5.43),
                         SIMDE_FLOAT32_C(     1.41), SIMDE_FLOAT32_C(     1.67), SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(     2.56),
                         SIMDE_FLOAT32_C(     3.67), SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(     6.15), SIMDE_FLOAT32_C(     6.46),
                         SIMDE_FLOAT32_C(     4.07), SIMDE_FLOAT32_C(     6.09), SIMDE_FLOAT32_C(     4.70), SIMDE_FLOAT32_C(     3.73)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     2.60), SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     2.38),
                         SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     1.10), SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(     1.59),
                         SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(     2.55),
                         SIMDE_FLOAT32_C(     2.08), SIMDE_FLOAT32_C(     2.49), SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(     1.99)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     6.58), SIMDE_FLOAT32_C(     7.07), SIMDE_FLOAT32_C(     4.23), SIMDE_FLOAT32_C(     2.35),
                         SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(     6.71), SIMDE_FLOAT32_C(     5.97), SIMDE_FLOAT32_C(     6.36),
                         SIMDE_FLOAT32_C(     7.04), SIMDE_FLOAT32_C(     4.76), SIMDE_FLOAT32_C(     7.53), SIMDE_FLOAT32_C(     1.31),
                         SIMDE_FLOAT32_C(     5.39), SIMDE_FLOAT32_C(     4.63), SIMDE_FLOAT32_C(     5.83), SIMDE_FLOAT32_C(     1.86)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.57), SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     2.12), SIMDE_FLOAT32_C(     1.50),
                         SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     2.59), SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(     2.54),
                         SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     2.24), SIMDE_FLOAT32_C(     2.71), SIMDE_FLOAT32_C(     0.77),
                         SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     2.21), SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     1.23)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     7.01), SIMDE_FLOAT32_C(     2.41), SIMDE_FLOAT32_C(     1.01),
                         SIMDE_FLOAT32_C(     3.19), SIMDE_FLOAT32_C(     7.35), SIMDE_FLOAT32_C(     5.27), SIMDE_FLOAT32_C(     1.77),
                         SIMDE_FLOAT32_C(     2.40), SIMDE_FLOAT32_C(     4.08), SIMDE_FLOAT32_C(     6.64), SIMDE_FLOAT32_C(     7.53),
                         SIMDE_FLOAT32_C(     1.80), SIMDE_FLOAT32_C(     5.70), SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     3.99)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.14),
                         SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(     1.17),
                         SIMDE_FLOAT32_C(     1.52), SIMDE_FLOAT32_C(     2.08), SIMDE_FLOAT32_C(     2.58), SIMDE_FLOAT32_C(     2.71),
                         SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     2.43), SIMDE_FLOAT32_C(     2.16), SIMDE_FLOAT32_C(     2.06)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.85), SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(     4.19),
                         SIMDE_FLOAT32_C(     7.38), SIMDE_FLOAT32_C(     4.32), SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(     3.89),
                         SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     5.99), SIMDE_FLOAT32_C(     5.60),
                         SIMDE_FLOAT32_C(     4.35), SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     7.32), SIMDE_FLOAT32_C(     2.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.71), SIMDE_FLOAT32_C(     1.80), SIMDE_FLOAT32_C(     1.21), SIMDE_FLOAT32_C(     2.11),
                         SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(     2.03),
                         SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     2.17), SIMDE_FLOAT32_C(     2.48), SIMDE_FLOAT32_C(     2.41),
                         SIMDE_FLOAT32_C(     2.15), SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     1.32)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_acosh_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_acosh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.81), SIMDE_FLOAT32_C(     6.57), SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     2.12),
                         SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(     6.60),
                         SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     6.19), SIMDE_FLOAT32_C(     5.69), SIMDE_FLOAT32_C(     5.84),
                         SIMDE_FLOAT32_C(     6.51), SIMDE_FLOAT32_C(     4.41), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     5.44)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.89), SIMDE_FLOAT32_C(     5.07), SIMDE_FLOAT32_C(     5.16), SIMDE_FLOAT32_C(     1.08),
                         SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     2.92),
                         SIMDE_FLOAT32_C(     2.06), SIMDE_FLOAT32_C(     4.58), SIMDE_FLOAT32_C(     7.02), SIMDE_FLOAT32_C(     3.41),
                         SIMDE_FLOAT32_C(     5.94), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(     1.81)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.27), SIMDE_FLOAT32_C(     6.57), SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     2.12),
                         SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(     1.73),
                         SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     1.90),
                         SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(     4.41), SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     5.44)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.85), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     7.30), SIMDE_FLOAT32_C(     6.91),
                         SIMDE_FLOAT32_C(     3.63), SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     2.47),
                         SIMDE_FLOAT32_C(     3.02), SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     6.06), SIMDE_FLOAT32_C(     6.88),
                         SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     4.39)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(     7.00), SIMDE_FLOAT32_C(     6.28),
                         SIMDE_FLOAT32_C(     5.14), SIMDE_FLOAT32_C(     4.03), SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     1.77),
                         SIMDE_FLOAT32_C(     5.62), SIMDE_FLOAT32_C(     5.61), SIMDE_FLOAT32_C(     5.42), SIMDE_FLOAT32_C(     3.43),
                         SIMDE_FLOAT32_C(     4.20), SIMDE_FLOAT32_C(     5.76), SIMDE_FLOAT32_C(     5.56), SIMDE_FLOAT32_C(     7.58)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     7.30), SIMDE_FLOAT32_C(     6.91),
                         SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.17),
                         SIMDE_FLOAT32_C(     2.41), SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     1.90),
                         SIMDE_FLOAT32_C(     2.11), SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     2.71)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(     7.07), SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(     6.71),
                         SIMDE_FLOAT32_C(     6.36), SIMDE_FLOAT32_C(     4.76), SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     4.63),
                         SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     6.80), SIMDE_FLOAT32_C(     5.43), SIMDE_FLOAT32_C(     1.67),
                         SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     6.09)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     6.58), SIMDE_FLOAT32_C(     4.23), SIMDE_FLOAT32_C(     2.82),
                         SIMDE_FLOAT32_C(     5.97), SIMDE_FLOAT32_C(     7.04), SIMDE_FLOAT32_C(     7.53), SIMDE_FLOAT32_C(     5.39),
                         SIMDE_FLOAT32_C(     5.83), SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     5.37), SIMDE_FLOAT32_C(     1.41),
                         SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(     3.67), SIMDE_FLOAT32_C(     6.15), SIMDE_FLOAT32_C(     4.07)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(     2.57), SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(     6.71),
                         SIMDE_FLOAT32_C(     6.36), SIMDE_FLOAT32_C(     4.76), SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     2.37),
                         SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     6.80), SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     1.67),
                         SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     6.09)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.15), SIMDE_FLOAT32_C(     2.85), SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(     7.38),
                         SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(     5.99), SIMDE_FLOAT32_C(     4.35),
                         SIMDE_FLOAT32_C(     7.32), SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     2.41), SIMDE_FLOAT32_C(     3.19),
                         SIMDE_FLOAT32_C(     5.27), SIMDE_FLOAT32_C(     2.40), SIMDE_FLOAT32_C(     6.64), SIMDE_FLOAT32_C(     1.80)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.26), SIMDE_FLOAT32_C(     3.65), SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(     4.19),
                         SIMDE_FLOAT32_C(     4.32), SIMDE_FLOAT32_C(     3.89), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     5.60),
                         SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     7.01), SIMDE_FLOAT32_C(     1.01),
                         SIMDE_FLOAT32_C(     7.35), SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     4.08), SIMDE_FLOAT32_C(     7.53)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.15), SIMDE_FLOAT32_C(     2.85), SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(     7.38),
                         SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(     5.99), SIMDE_FLOAT32_C(     4.35),
                         SIMDE_FLOAT32_C(     7.32), SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     0.14),
                         SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     2.40), SIMDE_FLOAT32_C(     2.08), SIMDE_FLOAT32_C(     2.71)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.25), SIMDE_FLOAT32_C(     1.87), SIMDE_FLOAT32_C(     3.26), SIMDE_FLOAT32_C(     4.89),
                         SIMDE_FLOAT32_C(     5.44), SIMDE_FLOAT32_C(     7.23), SIMDE_FLOAT32_C(     7.32), SIMDE_FLOAT32_C(     4.74),
                         SIMDE_FLOAT32_C(     5.90), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     1.77),
                         SIMDE_FLOAT32_C(     5.03), SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     1.74), SIMDE_FLOAT32_C(     5.75)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.94), SIMDE_FLOAT32_C(     7.32), SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     4.83),
                         SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(     5.71), SIMDE_FLOAT32_C(     7.38), SIMDE_FLOAT32_C(     4.66),
                         SIMDE_FLOAT32_C(     7.03), SIMDE_FLOAT32_C(     4.05), SIMDE_FLOAT32_C(     5.08), SIMDE_FLOAT32_C(     3.05),
                         SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     2.24), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     5.87)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.25), SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     3.26), SIMDE_FLOAT32_C(     2.26),
                         SIMDE_FLOAT32_C(     5.44), SIMDE_FLOAT32_C(     2.43), SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(     2.22),
                         SIMDE_FLOAT32_C(     5.90), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     1.77),
                         SIMDE_FLOAT32_C(     5.03), SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     5.75)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.21), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     4.07), SIMDE_FLOAT32_C(     1.71),
                         SIMDE_FLOAT32_C(     4.61), SIMDE_FLOAT32_C(     4.98), SIMDE_FLOAT32_C(     7.05), SIMDE_FLOAT32_C(     4.08),
                         SIMDE_FLOAT32_C(     3.36), SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     3.25), SIMDE_FLOAT32_C(     6.89),
                         SIMDE_FLOAT32_C(     2.22), SIMDE_FLOAT32_C(     6.14), SIMDE_FLOAT32_C(     5.75), SIMDE_FLOAT32_C(     5.73)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.12), SIMDE_FLOAT32_C(     2.96), SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(     1.78),
                         SIMDE_FLOAT32_C(     6.91), SIMDE_FLOAT32_C(     4.32), SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(     4.83),
                         SIMDE_FLOAT32_C(     6.21), SIMDE_FLOAT32_C(     4.26), SIMDE_FLOAT32_C(     3.28), SIMDE_FLOAT32_C(     1.93),
                         SIMDE_FLOAT32_C(     5.40), SIMDE_FLOAT32_C(     5.21), SIMDE_FLOAT32_C(     1.27), SIMDE_FLOAT32_C(     2.68)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.21), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     1.23), SIMDE_FLOAT32_C(     1.71),
                         SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     4.98), SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     2.26),
                         SIMDE_FLOAT32_C(     3.36), SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     3.25), SIMDE_FLOAT32_C(     6.89),
                         SIMDE_FLOAT32_C(     2.22), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     5.75), SIMDE_FLOAT32_C(     5.73)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     6.05), SIMDE_FLOAT32_C(     4.22), SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(     6.41),
                         SIMDE_FLOAT32_C(     5.79), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     4.65), SIMDE_FLOAT32_C(     6.25),
                         SIMDE_FLOAT32_C(     4.40), SIMDE_FLOAT32_C(     6.40), SIMDE_FLOAT32_C(     4.02), SIMDE_FLOAT32_C(     4.56),
                         SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     6.31), SIMDE_FLOAT32_C(     5.60), SIMDE_FLOAT32_C(     1.37)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.36), SIMDE_FLOAT32_C(     6.97), SIMDE_FLOAT32_C(     4.78), SIMDE_FLOAT32_C(     2.89),
                         SIMDE_FLOAT32_C(     5.32), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(     6.54),
                         SIMDE_FLOAT32_C(     4.52), SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     4.69), SIMDE_FLOAT32_C(     2.40),
                         SIMDE_FLOAT32_C(     4.17), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     4.12), SIMDE_FLOAT32_C(     4.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     6.05), SIMDE_FLOAT32_C(     4.22), SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(     6.41),
                         SIMDE_FLOAT32_C(     5.79), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     2.57),
                         SIMDE_FLOAT32_C(     2.19), SIMDE_FLOAT32_C(     6.40), SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(     1.52),
                         SIMDE_FLOAT32_C(     2.11), SIMDE_FLOAT32_C(     6.31), SIMDE_FLOAT32_C(     5.60), SIMDE_FLOAT32_C(     2.21)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     5.39), SIMDE_FLOAT32_C(     2.67), SIMDE_FLOAT32_C(     7.01),
                         SIMDE_FLOAT32_C(     7.46), SIMDE_FLOAT32_C(     7.45), SIMDE_FLOAT32_C(     7.02), SIMDE_FLOAT32_C(     1.61),
                         SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     1.90), SIMDE_FLOAT32_C(     2.91),
                         SIMDE_FLOAT32_C(     4.63), SIMDE_FLOAT32_C(     4.64), SIMDE_FLOAT32_C(     5.75), SIMDE_FLOAT32_C(     3.63)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.52), SIMDE_FLOAT32_C(     4.25), SIMDE_FLOAT32_C(     7.02), SIMDE_FLOAT32_C(     6.92),
                         SIMDE_FLOAT32_C(     1.87), SIMDE_FLOAT32_C(     3.28), SIMDE_FLOAT32_C(     6.71), SIMDE_FLOAT32_C(     3.14),
                         SIMDE_FLOAT32_C(     4.50), SIMDE_FLOAT32_C(     4.66), SIMDE_FLOAT32_C(     6.66), SIMDE_FLOAT32_C(     3.47),
                         SIMDE_FLOAT32_C(     7.42), SIMDE_FLOAT32_C(     5.49), SIMDE_FLOAT32_C(     4.26), SIMDE_FLOAT32_C(     7.11)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     5.39), SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     2.62),
                         SIMDE_FLOAT32_C(     7.46), SIMDE_FLOAT32_C(     7.45), SIMDE_FLOAT32_C(     7.02), SIMDE_FLOAT32_C(     1.81),
                         SIMDE_FLOAT32_C(     2.18), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     2.58), SIMDE_FLOAT32_C(     2.91),
                         SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(     4.64), SIMDE_FLOAT32_C(     5.75), SIMDE_FLOAT32_C(     2.65)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_acosh_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_acosh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.94), SIMDE_FLOAT64_C(    6.51),
                         SIMDE_FLOAT64_C(    3.32), SIMDE_FLOAT64_C(    4.41),
                         SIMDE_FLOAT64_C(    3.69), SIMDE_FLOAT64_C(    4.43),
                         SIMDE_FLOAT64_C(    1.81), SIMDE_FLOAT64_C(    5.44)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.47), SIMDE_FLOAT64_C(    2.56),
                         SIMDE_FLOAT64_C(    1.87), SIMDE_FLOAT64_C(    2.16),
                         SIMDE_FLOAT64_C(    1.98), SIMDE_FLOAT64_C(    2.17),
                         SIMDE_FLOAT64_C(    1.20), SIMDE_FLOAT64_C(    2.38)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.06), SIMDE_FLOAT64_C(    2.04),
                         SIMDE_FLOAT64_C(    4.58), SIMDE_FLOAT64_C(    6.19),
                         SIMDE_FLOAT64_C(    7.02), SIMDE_FLOAT64_C(    5.69),
                         SIMDE_FLOAT64_C(    3.41), SIMDE_FLOAT64_C(    5.84)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.35), SIMDE_FLOAT64_C(    1.34),
                         SIMDE_FLOAT64_C(    2.20), SIMDE_FLOAT64_C(    2.51),
                         SIMDE_FLOAT64_C(    2.64), SIMDE_FLOAT64_C(    2.42),
                         SIMDE_FLOAT64_C(    1.90), SIMDE_FLOAT64_C(    2.45)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.83), SIMDE_FLOAT64_C(    4.39),
                         SIMDE_FLOAT64_C(    3.03), SIMDE_FLOAT64_C(    1.25),
                         SIMDE_FLOAT64_C(    3.29), SIMDE_FLOAT64_C(    1.46),
                         SIMDE_FLOAT64_C(    2.92), SIMDE_FLOAT64_C(    6.60)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.70), SIMDE_FLOAT64_C(    2.16),
                         SIMDE_FLOAT64_C(    1.77), SIMDE_FLOAT64_C(    0.69),
                         SIMDE_FLOAT64_C(    1.86), SIMDE_FLOAT64_C(    0.93),
                         SIMDE_FLOAT64_C(    1.73), SIMDE_FLOAT64_C(    2.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.89), SIMDE_FLOAT64_C(    2.81),
                         SIMDE_FLOAT64_C(    5.07), SIMDE_FLOAT64_C(    6.57),
                         SIMDE_FLOAT64_C(    5.16), SIMDE_FLOAT64_C(    3.60),
                         SIMDE_FLOAT64_C(    1.08), SIMDE_FLOAT64_C(    2.12)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.27), SIMDE_FLOAT64_C(    1.69),
                         SIMDE_FLOAT64_C(    2.31), SIMDE_FLOAT64_C(    2.57),
                         SIMDE_FLOAT64_C(    2.32), SIMDE_FLOAT64_C(    1.95),
                         SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(    1.38)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.76), SIMDE_FLOAT64_C(    5.76),
                         SIMDE_FLOAT64_C(    2.37), SIMDE_FLOAT64_C(    5.56),
                         SIMDE_FLOAT64_C(    1.76), SIMDE_FLOAT64_C(    7.58),
                         SIMDE_FLOAT64_C(    4.39), SIMDE_FLOAT64_C(    7.08)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.17), SIMDE_FLOAT64_C(    2.44),
                         SIMDE_FLOAT64_C(    1.51), SIMDE_FLOAT64_C(    2.40),
                         SIMDE_FLOAT64_C(    1.17), SIMDE_FLOAT64_C(    2.71),
                         SIMDE_FLOAT64_C(    2.16), SIMDE_FLOAT64_C(    2.65)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.02), SIMDE_FLOAT64_C(    5.61),
                         SIMDE_FLOAT64_C(    6.46), SIMDE_FLOAT64_C(    5.42),
                         SIMDE_FLOAT64_C(    6.06), SIMDE_FLOAT64_C(    3.43),
                         SIMDE_FLOAT64_C(    6.88), SIMDE_FLOAT64_C(    4.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.77), SIMDE_FLOAT64_C(    2.41),
                         SIMDE_FLOAT64_C(    2.55), SIMDE_FLOAT64_C(    2.37),
                         SIMDE_FLOAT64_C(    2.49), SIMDE_FLOAT64_C(    1.90),
                         SIMDE_FLOAT64_C(    2.62), SIMDE_FLOAT64_C(    2.11)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.63), SIMDE_FLOAT64_C(    4.03),
                         SIMDE_FLOAT64_C(    5.41), SIMDE_FLOAT64_C(    1.18),
                         SIMDE_FLOAT64_C(    1.83), SIMDE_FLOAT64_C(    1.77),
                         SIMDE_FLOAT64_C(    2.47), SIMDE_FLOAT64_C(    5.62)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.96), SIMDE_FLOAT64_C(    2.07),
                         SIMDE_FLOAT64_C(    2.37), SIMDE_FLOAT64_C(    0.59),
                         SIMDE_FLOAT64_C(    1.21), SIMDE_FLOAT64_C(    1.17),
                         SIMDE_FLOAT64_C(    1.55), SIMDE_FLOAT64_C(    2.41)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.85), SIMDE_FLOAT64_C(    6.54),
                         SIMDE_FLOAT64_C(    3.81), SIMDE_FLOAT64_C(    7.00),
                         SIMDE_FLOAT64_C(    7.30), SIMDE_FLOAT64_C(    6.28),
                         SIMDE_FLOAT64_C(    6.91), SIMDE_FLOAT64_C(    5.14)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.45), SIMDE_FLOAT64_C(    2.57),
                         SIMDE_FLOAT64_C(    2.01), SIMDE_FLOAT64_C(    2.63),
                         SIMDE_FLOAT64_C(    2.68), SIMDE_FLOAT64_C(    2.52),
                         SIMDE_FLOAT64_C(    2.62), SIMDE_FLOAT64_C(    2.32)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_acosh_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_acosh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.04), SIMDE_FLOAT64_C(    6.19),
                         SIMDE_FLOAT64_C(    5.69), SIMDE_FLOAT64_C(    5.84),
                         SIMDE_FLOAT64_C(    6.51), SIMDE_FLOAT64_C(    4.41),
                         SIMDE_FLOAT64_C(    4.43), SIMDE_FLOAT64_C(    5.44)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.06), SIMDE_FLOAT64_C(    4.58),
                         SIMDE_FLOAT64_C(    7.02), SIMDE_FLOAT64_C(    3.41),
                         SIMDE_FLOAT64_C(    5.94), SIMDE_FLOAT64_C(    3.32),
                         SIMDE_FLOAT64_C(    3.69), SIMDE_FLOAT64_C(    1.81)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.35), SIMDE_FLOAT64_C(    6.19),
                         SIMDE_FLOAT64_C(    5.69), SIMDE_FLOAT64_C(    5.84),
                         SIMDE_FLOAT64_C(    2.47), SIMDE_FLOAT64_C(    4.41),
                         SIMDE_FLOAT64_C(    1.98), SIMDE_FLOAT64_C(    1.20)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.89), SIMDE_FLOAT64_C(    5.07),
                         SIMDE_FLOAT64_C(    5.16), SIMDE_FLOAT64_C(    1.08),
                         SIMDE_FLOAT64_C(    2.83), SIMDE_FLOAT64_C(    3.03),
                         SIMDE_FLOAT64_C(    3.29), SIMDE_FLOAT64_C(    2.92)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.08), SIMDE_FLOAT64_C(    2.81),
                         SIMDE_FLOAT64_C(    6.57), SIMDE_FLOAT64_C(    3.60),
                         SIMDE_FLOAT64_C(    2.12), SIMDE_FLOAT64_C(    4.39),
                         SIMDE_FLOAT64_C(    1.25), SIMDE_FLOAT64_C(    1.46)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.65), SIMDE_FLOAT64_C(    1.69),
                         SIMDE_FLOAT64_C(    2.57), SIMDE_FLOAT64_C(    1.08),
                         SIMDE_FLOAT64_C(    2.83), SIMDE_FLOAT64_C(    2.16),
                         SIMDE_FLOAT64_C(    3.29), SIMDE_FLOAT64_C(    0.93)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.62), SIMDE_FLOAT64_C(    5.61),
                         SIMDE_FLOAT64_C(    5.42), SIMDE_FLOAT64_C(    3.43),
                         SIMDE_FLOAT64_C(    4.20), SIMDE_FLOAT64_C(    5.76),
                         SIMDE_FLOAT64_C(    5.56), SIMDE_FLOAT64_C(    7.58)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.47), SIMDE_FLOAT64_C(    3.02),
                         SIMDE_FLOAT64_C(    6.46), SIMDE_FLOAT64_C(    6.06),
                         SIMDE_FLOAT64_C(    6.88), SIMDE_FLOAT64_C(    1.76),
                         SIMDE_FLOAT64_C(    2.37), SIMDE_FLOAT64_C(    1.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.55), SIMDE_FLOAT64_C(    1.77),
                         SIMDE_FLOAT64_C(    2.55), SIMDE_FLOAT64_C(    2.49),
                         SIMDE_FLOAT64_C(    2.62), SIMDE_FLOAT64_C(    1.17),
                         SIMDE_FLOAT64_C(    5.56), SIMDE_FLOAT64_C(    1.17)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.70), SIMDE_FLOAT64_C(    5.85),
                         SIMDE_FLOAT64_C(    3.81), SIMDE_FLOAT64_C(    7.30),
                         SIMDE_FLOAT64_C(    6.91), SIMDE_FLOAT64_C(    3.63),
                         SIMDE_FLOAT64_C(    5.41), SIMDE_FLOAT64_C(    1.83)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    6.09), SIMDE_FLOAT64_C(    3.73),
                         SIMDE_FLOAT64_C(    6.54), SIMDE_FLOAT64_C(    7.00),
                         SIMDE_FLOAT64_C(    6.28), SIMDE_FLOAT64_C(    5.14),
                         SIMDE_FLOAT64_C(    4.03), SIMDE_FLOAT64_C(    1.18)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.70), SIMDE_FLOAT64_C(    1.99),
                         SIMDE_FLOAT64_C(    3.81), SIMDE_FLOAT64_C(    2.63),
                         SIMDE_FLOAT64_C(    2.52), SIMDE_FLOAT64_C(    2.32),
                         SIMDE_FLOAT64_C(    5.41), SIMDE_FLOAT64_C(    0.59)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.63), SIMDE_FLOAT64_C(    1.86),
                         SIMDE_FLOAT64_C(    6.80), SIMDE_FLOAT64_C(    5.43),
                         SIMDE_FLOAT64_C(    1.67), SIMDE_FLOAT64_C(    2.56),
                         SIMDE_FLOAT64_C(    1.59), SIMDE_FLOAT64_C(    6.46)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.39), SIMDE_FLOAT64_C(    5.83),
                         SIMDE_FLOAT64_C(    3.71), SIMDE_FLOAT64_C(    5.37),
                         SIMDE_FLOAT64_C(    1.41), SIMDE_FLOAT64_C(    3.22),
                         SIMDE_FLOAT64_C(    3.67), SIMDE_FLOAT64_C(    6.15)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.37), SIMDE_FLOAT64_C(    1.86),
                         SIMDE_FLOAT64_C(    6.80), SIMDE_FLOAT64_C(    2.37),
                         SIMDE_FLOAT64_C(    1.67), SIMDE_FLOAT64_C(    2.56),
                         SIMDE_FLOAT64_C(    1.59), SIMDE_FLOAT64_C(    2.50)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.80), SIMDE_FLOAT64_C(    4.39),
                         SIMDE_FLOAT64_C(    6.58), SIMDE_FLOAT64_C(    4.23),
                         SIMDE_FLOAT64_C(    2.82), SIMDE_FLOAT64_C(    5.97),
                         SIMDE_FLOAT64_C(    7.04), SIMDE_FLOAT64_C(    7.53)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.53), SIMDE_FLOAT64_C(    5.70),
                         SIMDE_FLOAT64_C(    3.99), SIMDE_FLOAT64_C(    7.07),
                         SIMDE_FLOAT64_C(    2.35), SIMDE_FLOAT64_C(    6.71),
                         SIMDE_FLOAT64_C(    6.36), SIMDE_FLOAT64_C(    4.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.80), SIMDE_FLOAT64_C(    2.43),
                         SIMDE_FLOAT64_C(    6.58), SIMDE_FLOAT64_C(    4.23),
                         SIMDE_FLOAT64_C(    1.50), SIMDE_FLOAT64_C(    5.97),
                         SIMDE_FLOAT64_C(    2.54), SIMDE_FLOAT64_C(    2.24)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.60), SIMDE_FLOAT64_C(    3.29),
                         SIMDE_FLOAT64_C(    2.00), SIMDE_FLOAT64_C(    7.01),
                         SIMDE_FLOAT64_C(    1.01), SIMDE_FLOAT64_C(    7.35),
                         SIMDE_FLOAT64_C(    1.77), SIMDE_FLOAT64_C(    4.08)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.99), SIMDE_FLOAT64_C(    4.35),
                         SIMDE_FLOAT64_C(    7.32), SIMDE_FLOAT64_C(    1.76),
                         SIMDE_FLOAT64_C(    2.41), SIMDE_FLOAT64_C(    3.19),
                         SIMDE_FLOAT64_C(    5.27), SIMDE_FLOAT64_C(    2.40)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.60), SIMDE_FLOAT64_C(    2.15),
                         SIMDE_FLOAT64_C(    2.00), SIMDE_FLOAT64_C(    1.17),
                         SIMDE_FLOAT64_C(    1.53), SIMDE_FLOAT64_C(    1.83),
                         SIMDE_FLOAT64_C(    1.77), SIMDE_FLOAT64_C(    1.52)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.87), SIMDE_FLOAT64_C(    7.39),
                         SIMDE_FLOAT64_C(    3.15), SIMDE_FLOAT64_C(    2.85),
                         SIMDE_FLOAT64_C(    1.82), SIMDE_FLOAT64_C(    7.38),
                         SIMDE_FLOAT64_C(    3.22), SIMDE_FLOAT64_C(    3.70)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.74), SIMDE_FLOAT64_C(    5.75),
                         SIMDE_FLOAT64_C(    7.26), SIMDE_FLOAT64_C(    3.65),
                         SIMDE_FLOAT64_C(    3.11), SIMDE_FLOAT64_C(    4.19),
                         SIMDE_FLOAT64_C(    4.32), SIMDE_FLOAT64_C(    3.89)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.15), SIMDE_FLOAT64_C(    2.43),
                         SIMDE_FLOAT64_C(    3.15), SIMDE_FLOAT64_C(    1.97),
                         SIMDE_FLOAT64_C(    1.82), SIMDE_FLOAT64_C(    2.11),
                         SIMDE_FLOAT64_C(    3.22), SIMDE_FLOAT64_C(    2.03)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_acosh_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_asin_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.36)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.49)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.61)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -1.04), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.78)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -1.17)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -1.37), SIMDE_FLOAT32_C(    -0.72)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.76)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_asin_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_asin_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.85), SIMDE_FLOAT64_C(    0.36)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.52), SIMDE_FLOAT64_C(    0.73)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.49)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.98), SIMDE_FLOAT64_C(    0.43)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.61)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.76)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_asin_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_asin_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04),
                         SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.36)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69),
                         SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42),
                         SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.76),
                         SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.61),
                         SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.43),
                         SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.49)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86),
                         SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -1.17),
                         SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -1.04),
                         SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.78)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45),
                         SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21),
                         SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.47),
                         SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.76),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21),
                         SIMDE_FLOAT32_C(    -1.37), SIMDE_FLOAT32_C(    -0.72)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.44),
                         SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.38),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.84)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.46),
                         SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     1.43),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     1.00)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.03)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.41),
                         SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.35),
                         SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.03)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.94),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.40)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(    -1.22),
                         SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(    -0.88),
                         SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.41)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.25)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.75),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     0.64),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.25)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_asin_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_asin_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.85), SIMDE_FLOAT64_C(    0.36)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.52), SIMDE_FLOAT64_C(    0.73),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.98), SIMDE_FLOAT64_C(    0.43),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.49)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.76),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.61)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.86),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    0.70)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.32), SIMDE_FLOAT64_C(   -1.04),
                         SIMDE_FLOAT64_C(   -0.43), SIMDE_FLOAT64_C(    0.78)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.38), SIMDE_FLOAT64_C(   -0.92)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.46), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(   -1.17)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(   -0.66)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -1.37), SIMDE_FLOAT64_C(   -0.72)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.69)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.47),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.76)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_asin_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_asin_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.61),
                         SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.49),
                         SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.36)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66),
                         SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.76),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -1.37), SIMDE_FLOAT32_C(    -0.72),
                         SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -1.17),
                         SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -1.04), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.78)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.38),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.84)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.35),
                         SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     1.43), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     1.00)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.25),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.94),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.40)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.25),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(    -1.22),
                         SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.41)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.53),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.17)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.35),
                         SIMDE_FLOAT32_C(    -1.06), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(    -0.56),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.72),
                         SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.17)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.59),
                         SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.62),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(    -0.91),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.74)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.63),
                         SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.67),
                         SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(    -1.14),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.83)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.98),
                         SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.10)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -0.88),
                         SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     1.37),
                         SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.10)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.13),
                         SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.70)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(    -0.13),
                         SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(    -0.78)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_asin_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_asin_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.66),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.70),
                         SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.35)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.98),
                         SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.27),
                         SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.75)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.66),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.43),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(    -0.27),
                         SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.55),
                         SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.03)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.99)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -1.22), SIMDE_FLOAT32_C(    -0.88),
                         SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     1.43)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(     0.10),
                         SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.80),
                         SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.54)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.45),
                         SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    -0.87),
                         SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.07)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.80),
                         SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.54)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.34),
                         SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(    -0.76)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.98)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     1.37)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.18),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.13),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.44)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(    -0.38),
                         SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     0.48)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -1.22), SIMDE_FLOAT32_C(     0.44)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -0.81), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(    -0.78),
                         SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.07),
                         SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.43)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.76),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.72),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.78),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.43)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.64),
                         SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.08),
                         SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(    -0.89)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.43),
                         SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.58),
                         SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     0.09)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.64),
                         SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(     0.75),
                         SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.62),
                         SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.09)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.82),
                         SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.20)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.35),
                         SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(    -0.25),
                         SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.85)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.91),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.36),
                         SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     1.02)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_asin_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_asin_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.52), SIMDE_FLOAT64_C(    0.73),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.85), SIMDE_FLOAT64_C(    0.36)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.76),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.61),
                         SIMDE_FLOAT64_C(    0.98), SIMDE_FLOAT64_C(    0.43),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.49)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.38), SIMDE_FLOAT64_C(   -0.92),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.86),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    0.70)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.46), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(   -1.17),
                         SIMDE_FLOAT64_C(   -0.32), SIMDE_FLOAT64_C(   -1.04),
                         SIMDE_FLOAT64_C(   -0.43), SIMDE_FLOAT64_C(    0.78)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.69),
                         SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(   -0.66)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.47),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.76),
                         SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -1.37), SIMDE_FLOAT64_C(   -0.72)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(    0.38),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.03), SIMDE_FLOAT64_C(    0.84)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.88), SIMDE_FLOAT64_C(    0.46),
                         SIMDE_FLOAT64_C(   -0.62), SIMDE_FLOAT64_C(    0.39),
                         SIMDE_FLOAT64_C(   -0.88), SIMDE_FLOAT64_C(    1.43),
                         SIMDE_FLOAT64_C(    0.03), SIMDE_FLOAT64_C(    1.00)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.34),
                         SIMDE_FLOAT64_C(    0.53), SIMDE_FLOAT64_C(   -0.26),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(   -0.03)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.40), SIMDE_FLOAT64_C(    0.41),
                         SIMDE_FLOAT64_C(    0.72), SIMDE_FLOAT64_C(    0.35),
                         SIMDE_FLOAT64_C(    0.56), SIMDE_FLOAT64_C(   -0.26),
                         SIMDE_FLOAT64_C(    0.89), SIMDE_FLOAT64_C(   -0.03)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.20), SIMDE_FLOAT64_C(   -0.08),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.94),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C(    0.40)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.20), SIMDE_FLOAT64_C(   -0.08),
                         SIMDE_FLOAT64_C(    0.35), SIMDE_FLOAT64_C(   -1.22),
                         SIMDE_FLOAT64_C(   -0.85), SIMDE_FLOAT64_C(   -0.88),
                         SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(    0.41)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.47), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(    0.91), SIMDE_FLOAT64_C(    0.60),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(    0.25)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.49), SIMDE_FLOAT64_C(    0.75),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.96),
                         SIMDE_FLOAT64_C(    1.14), SIMDE_FLOAT64_C(    0.64),
                         SIMDE_FLOAT64_C(    0.91), SIMDE_FLOAT64_C(    0.25)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_asin_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_asin_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.69), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.42), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(    0.04), SIMDE_FLOAT64_C(    0.35)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(    0.08),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(   -0.27),
                         SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(   -0.75)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.42), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(    0.52), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(   -0.85)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(    0.23),
                         SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(   -0.38),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.42)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.84), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.66), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.92), SIMDE_FLOAT64_C(   -0.86)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(   -0.47),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -1.04)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.26),
                         SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(    0.99)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C(   -0.39),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.53),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(   -0.77)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(   -0.40),
                         SIMDE_FLOAT64_C(    0.72), SIMDE_FLOAT64_C(    0.56),
                         SIMDE_FLOAT64_C(    0.89), SIMDE_FLOAT64_C(   -0.88),
                         SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(   -0.88)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.12), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.91),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.75)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.54), SIMDE_FLOAT64_C(   -0.17),
                         SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(    0.60), SIMDE_FLOAT64_C(    0.25),
                         SIMDE_FLOAT64_C(   -0.08), SIMDE_FLOAT64_C(   -0.94)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.12), SIMDE_FLOAT64_C(   -0.17),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.96),
                         SIMDE_FLOAT64_C(    0.64), SIMDE_FLOAT64_C(    0.25),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -1.22)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.10), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.34),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.53),
                         SIMDE_FLOAT64_C(   -0.82), SIMDE_FLOAT64_C(    0.66)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.33), SIMDE_FLOAT64_C(    0.46),
                         SIMDE_FLOAT64_C(   -0.18), SIMDE_FLOAT64_C(    0.32),
                         SIMDE_FLOAT64_C(   -0.87), SIMDE_FLOAT64_C(   -0.33),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.56)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.33),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.53),
                         SIMDE_FLOAT64_C(   -0.82), SIMDE_FLOAT64_C(    0.59)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.02),
                         SIMDE_FLOAT64_C(   -0.45), SIMDE_FLOAT64_C(    0.51),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.98)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.98), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.10), SIMDE_FLOAT64_C(    0.84),
                         SIMDE_FLOAT64_C(   -0.59), SIMDE_FLOAT64_C(    0.73),
                         SIMDE_FLOAT64_C(    0.62), SIMDE_FLOAT64_C(    0.14)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.43),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.02),
                         SIMDE_FLOAT64_C(   -0.63), SIMDE_FLOAT64_C(    0.51),
                         SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    0.14)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(   -0.70), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.92),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(   -0.07)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.51), SIMDE_FLOAT64_C(    0.01),
                         SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.57), SIMDE_FLOAT64_C(   -0.34),
                         SIMDE_FLOAT64_C(    0.29), SIMDE_FLOAT64_C(   -0.58)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(    0.01),
                         SIMDE_FLOAT64_C(   -0.70), SIMDE_FLOAT64_C(   -0.88),
                         SIMDE_FLOAT64_C(   -0.61), SIMDE_FLOAT64_C(   -0.35),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(   -0.62)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.48), SIMDE_FLOAT64_C(    0.94),
                         SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(   -0.44),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.93),
                         SIMDE_FLOAT64_C(   -0.33), SIMDE_FLOAT64_C(   -0.18)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.78), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(   -0.36), SIMDE_FLOAT64_C(   -0.03),
                         SIMDE_FLOAT64_C(    0.01), SIMDE_FLOAT64_C(   -0.13)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.89), SIMDE_FLOAT64_C(    0.46),
                         SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.03),
                         SIMDE_FLOAT64_C(   -0.33), SIMDE_FLOAT64_C(   -0.13)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_asin_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_asinh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -5.92), SIMDE_FLOAT32_C(     4.36), SIMDE_FLOAT32_C(    -7.32), SIMDE_FLOAT32_C(     6.54)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     6.90), SIMDE_FLOAT32_C(     7.20), SIMDE_FLOAT32_C(    -6.39), SIMDE_FLOAT32_C(     4.22)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(     6.74), SIMDE_FLOAT32_C(    -6.29), SIMDE_FLOAT32_C(     6.84)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -7.21), SIMDE_FLOAT32_C(    -7.22), SIMDE_FLOAT32_C(     5.13), SIMDE_FLOAT32_C(     7.04)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -6.41), SIMDE_FLOAT32_C(    -7.45), SIMDE_FLOAT32_C(    -6.73), SIMDE_FLOAT32_C(     7.24)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -6.79), SIMDE_FLOAT32_C(     4.04), SIMDE_FLOAT32_C(    -6.64), SIMDE_FLOAT32_C(    -7.52)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     6.26), SIMDE_FLOAT32_C(    -6.05), SIMDE_FLOAT32_C(    -7.58), SIMDE_FLOAT32_C(    -7.19)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     5.88), SIMDE_FLOAT32_C(    -6.80), SIMDE_FLOAT32_C(     6.15), SIMDE_FLOAT32_C(     7.23)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_asinh_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_asinh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -7.32), SIMDE_FLOAT64_C(    6.54)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -5.92), SIMDE_FLOAT64_C(    4.36)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -6.39), SIMDE_FLOAT64_C(    4.22)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    6.90), SIMDE_FLOAT64_C(    7.20)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -6.29), SIMDE_FLOAT64_C(    6.84)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    7.41), SIMDE_FLOAT64_C(    6.74)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    5.13), SIMDE_FLOAT64_C(    7.04)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -7.21), SIMDE_FLOAT64_C(   -7.22)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_asinh_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_asinh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24),
                         SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01),
                         SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     6.90), SIMDE_FLOAT32_C(     7.20),
                         SIMDE_FLOAT32_C(    -6.39), SIMDE_FLOAT32_C(     4.22),
                         SIMDE_FLOAT32_C(    -5.92), SIMDE_FLOAT32_C(     4.36),
                         SIMDE_FLOAT32_C(    -7.32), SIMDE_FLOAT32_C(     6.54)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13),
                         SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21),
                         SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -7.21), SIMDE_FLOAT32_C(    -7.22),
                         SIMDE_FLOAT32_C(     5.13), SIMDE_FLOAT32_C(     7.04),
                         SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(     6.74),
                         SIMDE_FLOAT32_C(    -6.29), SIMDE_FLOAT32_C(     6.84)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95),
                         SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -6.79), SIMDE_FLOAT32_C(     4.04),
                         SIMDE_FLOAT32_C(    -6.64), SIMDE_FLOAT32_C(    -7.52),
                         SIMDE_FLOAT32_C(    -6.41), SIMDE_FLOAT32_C(    -7.45),
                         SIMDE_FLOAT32_C(    -6.73), SIMDE_FLOAT32_C(     7.24)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67),
                         SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54),
                         SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     5.88), SIMDE_FLOAT32_C(    -6.80),
                         SIMDE_FLOAT32_C(     6.15), SIMDE_FLOAT32_C(     7.23),
                         SIMDE_FLOAT32_C(     6.26), SIMDE_FLOAT32_C(    -6.05),
                         SIMDE_FLOAT32_C(    -7.58), SIMDE_FLOAT32_C(    -7.19)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48),
                         SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -7.34), SIMDE_FLOAT32_C(     6.79),
                         SIMDE_FLOAT32_C(    -7.06), SIMDE_FLOAT32_C(     6.63),
                         SIMDE_FLOAT32_C(    -7.34), SIMDE_FLOAT32_C(     7.59),
                         SIMDE_FLOAT32_C(     4.03), SIMDE_FLOAT32_C(     7.43)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92),
                         SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -6.65), SIMDE_FLOAT32_C(     6.67),
                         SIMDE_FLOAT32_C(     7.18), SIMDE_FLOAT32_C(     6.52),
                         SIMDE_FLOAT32_C(     6.97), SIMDE_FLOAT32_C(    -6.27),
                         SIMDE_FLOAT32_C(     7.35), SIMDE_FLOAT32_C(    -4.12)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73),
                         SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -6.01), SIMDE_FLOAT32_C(    -5.08),
                         SIMDE_FLOAT32_C(     6.51), SIMDE_FLOAT32_C(    -7.54),
                         SIMDE_FLOAT32_C(    -7.31), SIMDE_FLOAT32_C(    -7.34),
                         SIMDE_FLOAT32_C(    -7.01), SIMDE_FLOAT32_C(     6.68)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02),
                         SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     7.22),
                         SIMDE_FLOAT32_C(    -5.70), SIMDE_FLOAT32_C(     7.40),
                         SIMDE_FLOAT32_C(     7.51), SIMDE_FLOAT32_C(     7.09),
                         SIMDE_FLOAT32_C(     7.37), SIMDE_FLOAT32_C(     6.23)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_asinh_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_asinh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -5.92), SIMDE_FLOAT64_C(    4.36),
                         SIMDE_FLOAT64_C(   -7.32), SIMDE_FLOAT64_C(    6.54)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    6.90), SIMDE_FLOAT64_C(    7.20),
                         SIMDE_FLOAT64_C(   -6.39), SIMDE_FLOAT64_C(    4.22)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    7.41), SIMDE_FLOAT64_C(    6.74),
                         SIMDE_FLOAT64_C(   -6.29), SIMDE_FLOAT64_C(    6.84)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -7.21), SIMDE_FLOAT64_C(   -7.22),
                         SIMDE_FLOAT64_C(    5.13), SIMDE_FLOAT64_C(    7.04)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -6.41), SIMDE_FLOAT64_C(   -7.45),
                         SIMDE_FLOAT64_C(   -6.73), SIMDE_FLOAT64_C(    7.24)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -6.79), SIMDE_FLOAT64_C(    4.04),
                         SIMDE_FLOAT64_C(   -6.64), SIMDE_FLOAT64_C(   -7.52)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    6.26), SIMDE_FLOAT64_C(   -6.05),
                         SIMDE_FLOAT64_C(   -7.58), SIMDE_FLOAT64_C(   -7.19)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    5.88), SIMDE_FLOAT64_C(   -6.80),
                         SIMDE_FLOAT64_C(    6.15), SIMDE_FLOAT64_C(    7.23)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_asinh_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_asinh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -7.21), SIMDE_FLOAT32_C(    -7.22), SIMDE_FLOAT32_C(     5.13), SIMDE_FLOAT32_C(     7.04),
                         SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(     6.74), SIMDE_FLOAT32_C(    -6.29), SIMDE_FLOAT32_C(     6.84),
                         SIMDE_FLOAT32_C(     6.90), SIMDE_FLOAT32_C(     7.20), SIMDE_FLOAT32_C(    -6.39), SIMDE_FLOAT32_C(     4.22),
                         SIMDE_FLOAT32_C(    -5.92), SIMDE_FLOAT32_C(     4.36), SIMDE_FLOAT32_C(    -7.32), SIMDE_FLOAT32_C(     6.54)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.88), SIMDE_FLOAT32_C(    -6.80), SIMDE_FLOAT32_C(     6.15), SIMDE_FLOAT32_C(     7.23),
                         SIMDE_FLOAT32_C(     6.26), SIMDE_FLOAT32_C(    -6.05), SIMDE_FLOAT32_C(    -7.58), SIMDE_FLOAT32_C(    -7.19),
                         SIMDE_FLOAT32_C(    -6.79), SIMDE_FLOAT32_C(     4.04), SIMDE_FLOAT32_C(    -6.64), SIMDE_FLOAT32_C(    -7.52),
                         SIMDE_FLOAT32_C(    -6.41), SIMDE_FLOAT32_C(    -7.45), SIMDE_FLOAT32_C(    -6.73), SIMDE_FLOAT32_C(     7.24)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -6.65), SIMDE_FLOAT32_C(     6.67), SIMDE_FLOAT32_C(     7.18), SIMDE_FLOAT32_C(     6.52),
                         SIMDE_FLOAT32_C(     6.97), SIMDE_FLOAT32_C(    -6.27), SIMDE_FLOAT32_C(     7.35), SIMDE_FLOAT32_C(    -4.12),
                         SIMDE_FLOAT32_C(    -7.34), SIMDE_FLOAT32_C(     6.79), SIMDE_FLOAT32_C(    -7.06), SIMDE_FLOAT32_C(     6.63),
                         SIMDE_FLOAT32_C(    -7.34), SIMDE_FLOAT32_C(     7.59), SIMDE_FLOAT32_C(     4.03), SIMDE_FLOAT32_C(     7.43)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     7.22), SIMDE_FLOAT32_C(    -5.70), SIMDE_FLOAT32_C(     7.40),
                         SIMDE_FLOAT32_C(     7.51), SIMDE_FLOAT32_C(     7.09), SIMDE_FLOAT32_C(     7.37), SIMDE_FLOAT32_C(     6.23),
                         SIMDE_FLOAT32_C(    -6.01), SIMDE_FLOAT32_C(    -5.08), SIMDE_FLOAT32_C(     6.51), SIMDE_FLOAT32_C(    -7.54),
                         SIMDE_FLOAT32_C(    -7.31), SIMDE_FLOAT32_C(    -7.34), SIMDE_FLOAT32_C(    -7.01), SIMDE_FLOAT32_C(     6.68)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(   343.48),
                         SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   655.67),
                         SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(   120.65), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -5.88), SIMDE_FLOAT32_C(     7.32), SIMDE_FLOAT32_C(     6.48), SIMDE_FLOAT32_C(     6.53),
                         SIMDE_FLOAT32_C(    -7.47), SIMDE_FLOAT32_C(    -7.38), SIMDE_FLOAT32_C(    -6.49), SIMDE_FLOAT32_C(    -6.96),
                         SIMDE_FLOAT32_C(    -5.95), SIMDE_FLOAT32_C(    -7.41), SIMDE_FLOAT32_C(     7.02), SIMDE_FLOAT32_C(     7.18),
                         SIMDE_FLOAT32_C(    -4.95), SIMDE_FLOAT32_C(     6.99), SIMDE_FLOAT32_C(     5.49), SIMDE_FLOAT32_C(    -5.84)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -591.56),
                         SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(  -906.16),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -738.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.23), SIMDE_FLOAT32_C(     7.43), SIMDE_FLOAT32_C(    -3.74), SIMDE_FLOAT32_C(    -7.08),
                         SIMDE_FLOAT32_C(    -6.80), SIMDE_FLOAT32_C(     7.29), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     7.13),
                         SIMDE_FLOAT32_C(     7.42), SIMDE_FLOAT32_C(     5.64), SIMDE_FLOAT32_C(     7.58), SIMDE_FLOAT32_C(    -7.50),
                         SIMDE_FLOAT32_C(     6.50), SIMDE_FLOAT32_C(     5.30), SIMDE_FLOAT32_C(     6.83), SIMDE_FLOAT32_C(    -7.30)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(   977.49),
                         SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -7.34), SIMDE_FLOAT32_C(     7.40), SIMDE_FLOAT32_C(    -7.05), SIMDE_FLOAT32_C(    -7.60),
                         SIMDE_FLOAT32_C(    -6.52), SIMDE_FLOAT32_C(     7.52), SIMDE_FLOAT32_C(     6.38), SIMDE_FLOAT32_C(    -7.34),
                         SIMDE_FLOAT32_C(    -7.05), SIMDE_FLOAT32_C(    -4.91), SIMDE_FLOAT32_C(     7.26), SIMDE_FLOAT32_C(     7.58),
                         SIMDE_FLOAT32_C(    -7.32), SIMDE_FLOAT32_C(     6.74), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(    -5.25)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -696.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -6.78), SIMDE_FLOAT32_C(    -6.58), SIMDE_FLOAT32_C(    -7.32), SIMDE_FLOAT32_C(    -4.21),
                         SIMDE_FLOAT32_C(     7.53), SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(    -6.48), SIMDE_FLOAT32_C(    -5.52),
                         SIMDE_FLOAT32_C(    -5.90), SIMDE_FLOAT32_C(     4.38), SIMDE_FLOAT32_C(     6.93), SIMDE_FLOAT32_C(     6.67),
                         SIMDE_FLOAT32_C(     3.36), SIMDE_FLOAT32_C(    -6.41), SIMDE_FLOAT32_C(     7.51), SIMDE_FLOAT32_C(    -7.24)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_asinh_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_asinh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.88), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(     6.26), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(    -6.73),
                         SIMDE_FLOAT32_C(    -7.21), SIMDE_FLOAT32_C(     5.13), SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(    -6.29),
                         SIMDE_FLOAT32_C(     6.90), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    -5.92), SIMDE_FLOAT32_C(   346.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19),
                         SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   254.31), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(   398.82), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   -30.79), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -5.84), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(     6.23), SIMDE_FLOAT32_C(    -5.08), SIMDE_FLOAT32_C(    -7.54), SIMDE_FLOAT32_C(    -7.34),
                         SIMDE_FLOAT32_C(     6.68), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(     6.52), SIMDE_FLOAT32_C(    -6.27),
                         SIMDE_FLOAT32_C(    -4.12), SIMDE_FLOAT32_C(     6.79), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(     7.59)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    99.93),
                         SIMDE_FLOAT32_C(  -738.19), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -448.89),
                         SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(   331.34),
                         SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -874.31),
                         SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   -70.91)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(     7.23), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(     6.50),
                         SIMDE_FLOAT32_C(     6.83), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(     6.48), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(    -5.95), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -337.60),
                         SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(  -756.42)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(     7.40), SIMDE_FLOAT32_C(    -7.60),
                         SIMDE_FLOAT32_C(     7.52), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(    -4.91), SIMDE_FLOAT32_C(     7.58)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -737.13), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(   177.92),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   888.71), SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   133.52),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(  -775.04), SIMDE_FLOAT32_C(   440.64)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(   915.19), SIMDE_FLOAT32_C(  -718.40), SIMDE_FLOAT32_C(   159.97),
                         SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   110.36),
                         SIMDE_FLOAT32_C(   826.84), SIMDE_FLOAT32_C(   -76.75), SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -378.50),
                         SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(  -942.47), SIMDE_FLOAT32_C(   475.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(     7.51), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(     5.77),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(     6.75), SIMDE_FLOAT32_C(     7.53), SIMDE_FLOAT32_C(     5.40),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(    -7.54), SIMDE_FLOAT32_C(   440.64)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(  -807.28), SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(   834.60), SIMDE_FLOAT32_C(   -65.60),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   556.35), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -964.25), SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(  -743.66), SIMDE_FLOAT32_C(  -764.58),
                         SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(  -818.54), SIMDE_FLOAT32_C(   161.06),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -308.52), SIMDE_FLOAT32_C(  -719.57),
                         SIMDE_FLOAT32_C(   334.00), SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(  -916.82), SIMDE_FLOAT32_C(  -490.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(    -6.70), SIMDE_FLOAT32_C(    -7.30), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(     7.37), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(    -7.40), SIMDE_FLOAT32_C(     5.77),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(     6.31), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   105.79), SIMDE_FLOAT32_C(   590.10),
                         SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(   -84.00), SIMDE_FLOAT32_C(    80.04),
                         SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(  -889.11)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(   144.05), SIMDE_FLOAT32_C(  -427.72),
                         SIMDE_FLOAT32_C(   308.28), SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(  -457.77), SIMDE_FLOAT32_C(   678.24),
                         SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   117.28), SIMDE_FLOAT32_C(  -576.80),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   -53.92), SIMDE_FLOAT32_C(    91.94)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(    -6.82), SIMDE_FLOAT32_C(     7.21),
                         SIMDE_FLOAT32_C(     4.88), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(     5.46), SIMDE_FLOAT32_C(    -7.05),
                         SIMDE_FLOAT32_C(    -4.34), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(     5.21)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(  -493.41), SIMDE_FLOAT32_C(   822.72),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(  -816.27),
                         SIMDE_FLOAT32_C(  -209.34), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(  -204.33)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -841.43), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(   824.88), SIMDE_FLOAT32_C(   793.63),
                         SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -350.72),
                         SIMDE_FLOAT32_C(    60.89), SIMDE_FLOAT32_C(   109.81), SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -250.60),
                         SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   -13.07), SIMDE_FLOAT32_C(   852.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(     7.37),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(    -6.55),
                         SIMDE_FLOAT32_C(     4.80), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(     7.54), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(     7.44)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_asinh_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_asinh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    6.90), SIMDE_FLOAT64_C(    7.20),
                         SIMDE_FLOAT64_C(   -6.39), SIMDE_FLOAT64_C(    4.22),
                         SIMDE_FLOAT64_C(   -5.92), SIMDE_FLOAT64_C(    4.36),
                         SIMDE_FLOAT64_C(   -7.32), SIMDE_FLOAT64_C(    6.54)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -7.21), SIMDE_FLOAT64_C(   -7.22),
                         SIMDE_FLOAT64_C(    5.13), SIMDE_FLOAT64_C(    7.04),
                         SIMDE_FLOAT64_C(    7.41), SIMDE_FLOAT64_C(    6.74),
                         SIMDE_FLOAT64_C(   -6.29), SIMDE_FLOAT64_C(    6.84)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -6.79), SIMDE_FLOAT64_C(    4.04),
                         SIMDE_FLOAT64_C(   -6.64), SIMDE_FLOAT64_C(   -7.52),
                         SIMDE_FLOAT64_C(   -6.41), SIMDE_FLOAT64_C(   -7.45),
                         SIMDE_FLOAT64_C(   -6.73), SIMDE_FLOAT64_C(    7.24)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.88), SIMDE_FLOAT64_C(   -6.80),
                         SIMDE_FLOAT64_C(    6.15), SIMDE_FLOAT64_C(    7.23),
                         SIMDE_FLOAT64_C(    6.26), SIMDE_FLOAT64_C(   -6.05),
                         SIMDE_FLOAT64_C(   -7.58), SIMDE_FLOAT64_C(   -7.19)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(  993.90),
                         SIMDE_FLOAT64_C(   28.08), SIMDE_FLOAT64_C(  841.21)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -7.34), SIMDE_FLOAT64_C(    6.79),
                         SIMDE_FLOAT64_C(   -7.06), SIMDE_FLOAT64_C(    6.63),
                         SIMDE_FLOAT64_C(   -7.34), SIMDE_FLOAT64_C(    7.59),
                         SIMDE_FLOAT64_C(    4.03), SIMDE_FLOAT64_C(    7.43)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C(  -30.79)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -6.65), SIMDE_FLOAT64_C(    6.67),
                         SIMDE_FLOAT64_C(    7.18), SIMDE_FLOAT64_C(    6.52),
                         SIMDE_FLOAT64_C(    6.97), SIMDE_FLOAT64_C(   -6.27),
                         SIMDE_FLOAT64_C(    7.35), SIMDE_FLOAT64_C(   -4.12)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  -80.73),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -767.23),
                         SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C(  398.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -6.01), SIMDE_FLOAT64_C(   -5.08),
                         SIMDE_FLOAT64_C(    6.51), SIMDE_FLOAT64_C(   -7.54),
                         SIMDE_FLOAT64_C(   -7.31), SIMDE_FLOAT64_C(   -7.34),
                         SIMDE_FLOAT64_C(   -7.01), SIMDE_FLOAT64_C(    6.68)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C(  254.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    6.85), SIMDE_FLOAT64_C(    7.22),
                         SIMDE_FLOAT64_C(   -5.70), SIMDE_FLOAT64_C(    7.40),
                         SIMDE_FLOAT64_C(    7.51), SIMDE_FLOAT64_C(    7.09),
                         SIMDE_FLOAT64_C(    7.37), SIMDE_FLOAT64_C(    6.23)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_asinh_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_asinh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45),
                         SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -7.21), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(    6.90), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   -5.92), SIMDE_FLOAT64_C(   -7.32)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  841.21), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  687.09), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -660.80), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -923.64), SIMDE_FLOAT64_C( -860.95)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.43), SIMDE_FLOAT64_C(   -6.80),
                         SIMDE_FLOAT64_C(    7.23), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(    4.04),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C(   -7.45)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  398.82), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  339.21), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  -30.79), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(  993.90)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C( -387.90),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  532.35),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C( -770.35),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C( -770.72)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -7.01), SIMDE_FLOAT64_C(   -6.65),
                         SIMDE_FLOAT64_C(    7.18), SIMDE_FLOAT64_C(    6.97),
                         SIMDE_FLOAT64_C(    7.35), SIMDE_FLOAT64_C(   -7.34),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(   -7.34)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(  469.66),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  910.03),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C( -203.65),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -747.59)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C( -171.51),
                         SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(   -5.84),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(    7.40),
                         SIMDE_FLOAT64_C(    7.09), SIMDE_FLOAT64_C(    6.23),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C(   -7.54)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   99.93), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(  343.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(  655.67)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  331.34), SIMDE_FLOAT64_C(  462.95),
                         SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  324.62),
                         SIMDE_FLOAT64_C( -874.31), SIMDE_FLOAT64_C( -328.54),
                         SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  561.36)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    6.50), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(    6.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(    7.02)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   27.25),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  831.02), SIMDE_FLOAT64_C(  977.36)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  977.49), SIMDE_FLOAT64_C(  424.81),
                         SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  840.65),
                         SIMDE_FLOAT64_C( -591.56), SIMDE_FLOAT64_C(  731.49),
                         SIMDE_FLOAT64_C(  623.70), SIMDE_FLOAT64_C(  140.67)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(    6.74),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(   -7.08), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(    7.13), SIMDE_FLOAT64_C(    5.64)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C( -304.73),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(  822.06),
                         SIMDE_FLOAT64_C( -997.63), SIMDE_FLOAT64_C(  923.64),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(  -67.64)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  510.85), SIMDE_FLOAT64_C(   14.34),
                         SIMDE_FLOAT64_C(  916.26), SIMDE_FLOAT64_C( -769.09),
                         SIMDE_FLOAT64_C( -573.81), SIMDE_FLOAT64_C( -337.60),
                         SIMDE_FLOAT64_C(  293.64), SIMDE_FLOAT64_C( -576.22)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C(    3.36),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(   -7.34),
                         SIMDE_FLOAT64_C(   -7.05), SIMDE_FLOAT64_C(   -6.52),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(   -7.05)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  475.51), SIMDE_FLOAT64_C(  936.65),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C( -438.19),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(  932.66),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C( -182.45)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(  440.64),
                         SIMDE_FLOAT64_C(  897.27), SIMDE_FLOAT64_C( -197.89),
                         SIMDE_FLOAT64_C( -359.76), SIMDE_FLOAT64_C(  -33.67),
                         SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -125.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -7.35), SIMDE_FLOAT64_C(    6.78),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C(   -5.98),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(   -4.21),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(   -5.52)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_asinh_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_atan_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.54)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_atan_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_atan_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.55)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.54)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.56), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_atan_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_atan_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24),
                         SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01),
                         SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.54),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.55),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13),
                         SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21),
                         SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95),
                         SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.54),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67),
                         SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54),
                         SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48),
                         SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92),
                         SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.54)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73),
                         SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.56),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02),
                         SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_atan_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_atan_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.55),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.54)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(    1.56), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.54),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_atan_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_atan_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.54),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.54),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(   343.48),
                         SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   655.67),
                         SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(   120.65), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -1.56)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -591.56),
                         SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(  -906.16),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -738.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.52), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(   977.49),
                         SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(    -1.56)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -696.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.54),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.43), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.56),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_atan_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_atan_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(   346.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19),
                         SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   254.31), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(   398.82), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   -30.79), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.54), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    99.93),
                         SIMDE_FLOAT32_C(  -738.19), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -448.89),
                         SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(   331.34),
                         SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -874.31),
                         SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   -70.91)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -337.60),
                         SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(  -756.42)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(     1.43), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -737.13), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(   177.92),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   888.71), SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   133.52),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(  -775.04), SIMDE_FLOAT32_C(   440.64)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(   915.19), SIMDE_FLOAT32_C(  -718.40), SIMDE_FLOAT32_C(   159.97),
                         SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   110.36),
                         SIMDE_FLOAT32_C(   826.84), SIMDE_FLOAT32_C(   -76.75), SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -378.50),
                         SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(  -942.47), SIMDE_FLOAT32_C(   475.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(     1.56),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.56),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(   440.64)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(  -807.28), SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(   834.60), SIMDE_FLOAT32_C(   -65.60),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   556.35), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -964.25), SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(  -743.66), SIMDE_FLOAT32_C(  -764.58),
                         SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(  -818.54), SIMDE_FLOAT32_C(   161.06),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -308.52), SIMDE_FLOAT32_C(  -719.57),
                         SIMDE_FLOAT32_C(   334.00), SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(  -916.82), SIMDE_FLOAT32_C(  -490.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.56),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   105.79), SIMDE_FLOAT32_C(   590.10),
                         SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(   -84.00), SIMDE_FLOAT32_C(    80.04),
                         SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(  -889.11)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(   144.05), SIMDE_FLOAT32_C(  -427.72),
                         SIMDE_FLOAT32_C(   308.28), SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(  -457.77), SIMDE_FLOAT32_C(   678.24),
                         SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   117.28), SIMDE_FLOAT32_C(  -576.80),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   -53.92), SIMDE_FLOAT32_C(    91.94)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.54), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(     1.56)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(  -493.41), SIMDE_FLOAT32_C(   822.72),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(  -816.27),
                         SIMDE_FLOAT32_C(  -209.34), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(  -204.33)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -841.43), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(   824.88), SIMDE_FLOAT32_C(   793.63),
                         SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -350.72),
                         SIMDE_FLOAT32_C(    60.89), SIMDE_FLOAT32_C(   109.81), SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -250.60),
                         SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   -13.07), SIMDE_FLOAT32_C(   852.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(     1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_atan_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_atan_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.54),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.55),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(    1.56), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.54),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(  993.90),
                         SIMDE_FLOAT64_C(   28.08), SIMDE_FLOAT64_C(  841.21)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.54), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C(  -30.79)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   -1.54)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  -80.73),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -767.23),
                         SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C(  398.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.56),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C(  254.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(   -1.56), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_atan_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_atan_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45),
                         SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  841.21), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  687.09), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -660.80), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -923.64), SIMDE_FLOAT64_C( -860.95)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(    1.54),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C(   -1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  398.82), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  339.21), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  -30.79), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(  993.90)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C( -387.90),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  532.35),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C( -770.35),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C( -770.72)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(   -1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(  469.66),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  910.03),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C( -203.65),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -747.59)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C( -171.51),
                         SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(   -1.56),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C(   -1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   99.93), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(  343.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(  655.67)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  331.34), SIMDE_FLOAT64_C(  462.95),
                         SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  324.62),
                         SIMDE_FLOAT64_C( -874.31), SIMDE_FLOAT64_C( -328.54),
                         SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  561.36)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(    1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   27.25),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  831.02), SIMDE_FLOAT64_C(  977.36)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  977.49), SIMDE_FLOAT64_C(  424.81),
                         SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  840.65),
                         SIMDE_FLOAT64_C( -591.56), SIMDE_FLOAT64_C(  731.49),
                         SIMDE_FLOAT64_C(  623.70), SIMDE_FLOAT64_C(  140.67)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.56)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C( -304.73),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(  822.06),
                         SIMDE_FLOAT64_C( -997.63), SIMDE_FLOAT64_C(  923.64),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(  -67.64)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  510.85), SIMDE_FLOAT64_C(   14.34),
                         SIMDE_FLOAT64_C(  916.26), SIMDE_FLOAT64_C( -769.09),
                         SIMDE_FLOAT64_C( -573.81), SIMDE_FLOAT64_C( -337.60),
                         SIMDE_FLOAT64_C(  293.64), SIMDE_FLOAT64_C( -576.22)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C(    1.50),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(   -1.57)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  475.51), SIMDE_FLOAT64_C(  936.65),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C( -438.19),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(  932.66),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C( -182.45)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(  440.64),
                         SIMDE_FLOAT64_C(  897.27), SIMDE_FLOAT64_C( -197.89),
                         SIMDE_FLOAT64_C( -359.76), SIMDE_FLOAT64_C(  -33.67),
                         SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -125.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C(   -1.57),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(   -1.54),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(   -1.56)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_atan_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_atan2_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 b;
    simde__m128 r;
  } test_vec[9] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     2.71)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -2.35), SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     2.09)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -2.35), SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     2.09)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.08), SIMDE_FLOAT32_C(    -1.96), SIMDE_FLOAT32_C(    -1.91), SIMDE_FLOAT32_C(     2.11)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -2.55)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(   841.21)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(     1.54)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   -30.79)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.04)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(   398.82)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -2.76), SIMDE_FLOAT32_C(    -1.23), SIMDE_FLOAT32_C(    -2.34), SIMDE_FLOAT32_C(     2.52)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   593.11), SIMDE_FLOAT32_C(   480.49), SIMDE_FLOAT32_C(  -877.19), SIMDE_FLOAT32_C(  -326.68)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     -0.00)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    1.57), SIMDE_FLOAT32_C(    1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_atan2_ps(test_vec[i].a, test_vec[i].b);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_atan2_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d b;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    2.94), SIMDE_FLOAT64_C(    2.71)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.93), SIMDE_FLOAT64_C(    3.03)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.47), SIMDE_FLOAT64_C(    2.09)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46)),
      simde_mm_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -2.35), SIMDE_FLOAT64_C(    1.42)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -860.95), SIMDE_FLOAT64_C(  696.87)),
      simde_mm_set_pd(SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -1.91), SIMDE_FLOAT64_C(    2.11)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   28.47), SIMDE_FLOAT64_C( -923.64)),
      simde_mm_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.08), SIMDE_FLOAT64_C(   -1.96)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -212.54), SIMDE_FLOAT64_C( -660.80)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -2.55)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_atan2_pd(test_vec[i].a, test_vec[i].b);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_atan2_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 b;
    simde__m256 r;
  } test_vec[9] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -2.35), SIMDE_FLOAT32_C(     1.42),
                         SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     2.09),
                         SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     3.03),
                         SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     2.71)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(     1.24),
                         SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -2.55),
                         SIMDE_FLOAT32_C(     3.08), SIMDE_FLOAT32_C(    -1.96),
                         SIMDE_FLOAT32_C(    -1.91), SIMDE_FLOAT32_C(     2.11)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(   841.21)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(     0.48),
                         SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.04),
                         SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     2.56),
                         SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(     1.54)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(   398.82)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     1.75),
                         SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.31),
                         SIMDE_FLOAT32_C(    -2.76), SIMDE_FLOAT32_C(    -1.23),
                         SIMDE_FLOAT32_C(    -2.34), SIMDE_FLOAT32_C(     2.52)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48),
                         SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67),
                         SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62),
                         SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -328.54),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36),
                         SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   120.65)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.80), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(    -2.40), SIMDE_FLOAT32_C(    -2.13),
                         SIMDE_FLOAT32_C(    -1.80), SIMDE_FLOAT32_C(     0.86),
                         SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(    -0.96)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56),
                         SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16),
                         SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(  -738.19)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09),
                         SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   505.79),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(   462.95)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -1.61),
                         SIMDE_FLOAT32_C(     2.12), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -0.75),
                         SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -1.01)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49),
                         SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   293.64),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38),
                         SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(    27.25)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(    -2.09),
                         SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(    -1.21),
                         SIMDE_FLOAT32_C(    -3.02), SIMDE_FLOAT32_C(     0.94),
                         SIMDE_FLOAT32_C(     2.63), SIMDE_FLOAT32_C(    -1.29)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43),
                         SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(  -327.22),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85),
                         SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(   916.26)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -2.45), SIMDE_FLOAT32_C(    -3.10),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -2.78),
                         SIMDE_FLOAT32_C(     2.93), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(    -1.52), SIMDE_FLOAT32_C(    -0.65)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24),
                         SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01),
                         SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) }

  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_atan2_ps(test_vec[i].a, test_vec[i].b);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_atan2_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d b;
    simde__m256d r;
  } test_vec[9] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.93), SIMDE_FLOAT64_C(    3.03),
                         SIMDE_FLOAT64_C(    2.94), SIMDE_FLOAT64_C(    2.71)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -2.35), SIMDE_FLOAT64_C(    1.42),
                         SIMDE_FLOAT64_C(    0.47), SIMDE_FLOAT64_C(    2.09)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   28.47), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -860.95), SIMDE_FLOAT64_C(  696.87)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.08), SIMDE_FLOAT64_C(   -1.96),
                         SIMDE_FLOAT64_C(   -1.91), SIMDE_FLOAT64_C(    2.11)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -450.67), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C( -212.54), SIMDE_FLOAT64_C( -660.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.19), SIMDE_FLOAT64_C(    1.24),
                         SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -2.55)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  443.48), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C(  993.90), SIMDE_FLOAT64_C(  841.21)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C( -583.60),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(   28.08)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.62), SIMDE_FLOAT64_C(    2.56),
                         SIMDE_FLOAT64_C(    2.23), SIMDE_FLOAT64_C(    1.54)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  395.92), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C( -263.99), SIMDE_FLOAT64_C(  -30.79)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  655.87),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C(  780.64)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.35), SIMDE_FLOAT64_C(    0.48),
                         SIMDE_FLOAT64_C(   -0.46), SIMDE_FLOAT64_C(   -0.04)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -767.23), SIMDE_FLOAT64_C(  398.82)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  336.73),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -554.19)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -2.76), SIMDE_FLOAT64_C(   -1.23),
                         SIMDE_FLOAT64_C(   -2.34), SIMDE_FLOAT64_C(    2.52)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C( -148.69),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  791.23)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.97), SIMDE_FLOAT64_C(    1.75),
                         SIMDE_FLOAT64_C(    0.58), SIMDE_FLOAT64_C(    0.31)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00),
                         SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_atan2_pd(test_vec[i].a, test_vec[i].b);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_atan2_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 b;
    simde__m512 r;
  } test_vec[9] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -2.55),
                         SIMDE_FLOAT32_C(     3.08), SIMDE_FLOAT32_C(    -1.96), SIMDE_FLOAT32_C(    -1.91), SIMDE_FLOAT32_C(     2.11),
                         SIMDE_FLOAT32_C(    -2.35), SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     2.09),
                         SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     2.71)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(   398.82),
                         SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(   841.21)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19),
                         SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.31),
                         SIMDE_FLOAT32_C(    -2.76), SIMDE_FLOAT32_C(    -1.23), SIMDE_FLOAT32_C(    -2.34), SIMDE_FLOAT32_C(     2.52),
                         SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.04),
                         SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(     1.54)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(  -738.19),
                         SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   505.79),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(   462.95),
                         SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -328.54),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   120.65)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -1.61), SIMDE_FLOAT32_C(     2.12), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -1.01),
                         SIMDE_FLOAT32_C(     1.80), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -2.40), SIMDE_FLOAT32_C(    -2.13),
                         SIMDE_FLOAT32_C(    -1.80), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(    -0.96)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69),
                         SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49), SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(  -327.22),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(   916.26),
                         SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   293.64),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(    27.25)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -2.45), SIMDE_FLOAT32_C(    -3.10), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -2.78),
                         SIMDE_FLOAT32_C(     2.93), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -1.52), SIMDE_FLOAT32_C(    -0.65),
                         SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(    -2.09), SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(    -1.21),
                         SIMDE_FLOAT32_C(    -3.02), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     2.63), SIMDE_FLOAT32_C(    -1.29)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(   177.92), SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   888.71),
                         SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   133.52), SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06),
                         SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93), SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36),
                         SIMDE_FLOAT32_C(  -775.04), SIMDE_FLOAT32_C(   440.64), SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -197.89)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -718.40), SIMDE_FLOAT32_C(   159.97), SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   426.61),
                         SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   110.36), SIMDE_FLOAT32_C(   826.84), SIMDE_FLOAT32_C(   -76.75),
                         SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -378.50), SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -623.50),
                         SIMDE_FLOAT32_C(  -942.47), SIMDE_FLOAT32_C(   475.51), SIMDE_FLOAT32_C(   936.65), SIMDE_FLOAT32_C(  -348.70)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -2.73), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     2.76), SIMDE_FLOAT32_C(     1.12),
                         SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -1.70),
                         SIMDE_FLOAT32_C(    -1.28), SIMDE_FLOAT32_C(    -2.03), SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(    -2.24),
                         SIMDE_FLOAT32_C(    -2.45), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(    -2.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -764.58), SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(  -818.54),
                         SIMDE_FLOAT32_C(   161.06), SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -308.52),
                         SIMDE_FLOAT32_C(  -719.57), SIMDE_FLOAT32_C(   334.00), SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(  -916.82),
                         SIMDE_FLOAT32_C(  -490.00), SIMDE_FLOAT32_C(  -799.40), SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -737.13)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(  -784.34), SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(   206.60),
                         SIMDE_FLOAT32_C(   834.60), SIMDE_FLOAT32_C(   -65.60), SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86),
                         SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48), SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   556.35),
                         SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03), SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(   915.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.66), SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -1.32),
                         SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     1.68), SIMDE_FLOAT32_C(    -3.10), SIMDE_FLOAT32_C(    -2.17),
                         SIMDE_FLOAT32_C(    -1.99), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     2.73), SIMDE_FLOAT32_C(    -1.03),
                         SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -1.07), SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    -0.68)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   638.94), SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   105.79),
                         SIMDE_FLOAT32_C(   590.10), SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(   -84.00),
                         SIMDE_FLOAT32_C(    80.04), SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58),
                         SIMDE_FLOAT32_C(  -889.11), SIMDE_FLOAT32_C(  -964.25), SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(  -743.66)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -427.72), SIMDE_FLOAT32_C(   308.28), SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(  -457.77),
                         SIMDE_FLOAT32_C(   678.24), SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   117.28),
                         SIMDE_FLOAT32_C(  -576.80), SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   -53.92),
                         SIMDE_FLOAT32_C(    91.94), SIMDE_FLOAT32_C(   -78.84), SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(  -807.28)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.16), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -1.80), SIMDE_FLOAT32_C(     2.91),
                         SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(    -0.62),
                         SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(    -1.62), SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     1.71),
                         SIMDE_FLOAT32_C(    -1.47), SIMDE_FLOAT32_C(    -1.65), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -2.40)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -350.72),
                         SIMDE_FLOAT32_C(    60.89), SIMDE_FLOAT32_C(   109.81), SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -250.60),
                         SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   -13.07), SIMDE_FLOAT32_C(   852.60),
                         SIMDE_FLOAT32_C(  -440.06), SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   822.72), SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49),
                         SIMDE_FLOAT32_C(  -816.27), SIMDE_FLOAT32_C(  -209.34), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -728.70),
                         SIMDE_FLOAT32_C(  -420.06), SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77),
                         SIMDE_FLOAT32_C(  -204.33), SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(   144.05)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -0.40),
                         SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     2.66), SIMDE_FLOAT32_C(     2.49), SIMDE_FLOAT32_C(    -2.81),
                         SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     1.09),
                         SIMDE_FLOAT32_C(    -2.01), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    -1.42)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_atan2_ps(test_vec[i].a, test_vec[i].b);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_atan2_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 b;
    simde__m512 r;
  } test_vec[9] = {
   { simde_mm512_set_ps(SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(25611),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   -30.79), SIMDE_FLOAT32_C(  -583.60),
                         SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(   696.87), SIMDE_FLOAT32_C(    84.77),
                         SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   443.48),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   841.21), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(  -212.54),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(  -686.13),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   467.76), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    39.01)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(    -1.52)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   977.36),
                         SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -328.54),
                         SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(  -148.69),
                         SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -554.19)),
      UINT16_C(63749),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   140.67),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(  -738.19), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   120.65), SIMDE_FLOAT32_C(   680.02),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   254.31), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -767.23)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   831.02),
                         SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(  -874.31),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(   469.66),
                         SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -747.59)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.71), SIMDE_FLOAT32_C(    -1.61), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.17),
                         SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(    -2.40),
                         SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(  -148.69),
                         SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(    -2.34)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -765.93), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(  -775.04),
                         SIMDE_FLOAT32_C(   936.65), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(     7.27),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   394.67), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(   822.06),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(  -768.12), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(   424.81)),
      UINT16_C(23119),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -378.50), SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -942.47),
                         SIMDE_FLOAT32_C(   440.64), SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(  -125.20), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -769.09),
                         SIMDE_FLOAT32_C(  -997.63), SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(  -756.42)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -76.75), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -788.36),
                         SIMDE_FLOAT32_C(   475.51), SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(  -696.69),
                         SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   977.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(    -2.70), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(    -2.27),
                         SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(    -2.45), SIMDE_FLOAT32_C(     7.27),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(   822.06),
                         SIMDE_FLOAT32_C(    -2.09), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -3.02), SIMDE_FLOAT32_C(    -0.66)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -743.66), SIMDE_FLOAT32_C(  -784.34), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(   834.60),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -719.57), SIMDE_FLOAT32_C(  -628.82),
                         SIMDE_FLOAT32_C(  -916.82), SIMDE_FLOAT32_C(   434.03), SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -718.40),
                         SIMDE_FLOAT32_C(   177.92), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   826.84)),
      UINT16_C(57786),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -807.28), SIMDE_FLOAT32_C(  -764.58), SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(  -818.54),
                         SIMDE_FLOAT32_C(   -65.60), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   334.00),
                         SIMDE_FLOAT32_C(   556.35), SIMDE_FLOAT32_C(  -490.00), SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(  -737.13),
                         SIMDE_FLOAT32_C(   159.97), SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   133.52)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(   206.60),
                         SIMDE_FLOAT32_C(   161.06), SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -308.52), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(  -799.40), SIMDE_FLOAT32_C(   915.19),
                         SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   888.71), SIMDE_FLOAT32_C(   110.36)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -2.04), SIMDE_FLOAT32_C(    -1.66), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(   834.60),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -719.57), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(   434.03), SIMDE_FLOAT32_C(     2.59), SIMDE_FLOAT32_C(    -0.68),
                         SIMDE_FLOAT32_C(     2.67), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(   826.84)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   852.60),
                         SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -427.72), SIMDE_FLOAT32_C(   450.90),
                         SIMDE_FLOAT32_C(  -457.77), SIMDE_FLOAT32_C(   590.10), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   -84.00),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(    91.94), SIMDE_FLOAT32_C(  -964.25)),
      UINT16_C(25589),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -420.06), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   439.77),
                         SIMDE_FLOAT32_C(  -440.06), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   308.28),
                         SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   678.24), SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   117.28),
                         SIMDE_FLOAT32_C(    80.04), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(   -78.84)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -250.60), SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   -13.07),
                         SIMDE_FLOAT32_C(  -204.33), SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   144.05), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(   105.79), SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(   635.35),
                         SIMDE_FLOAT32_C(  -576.80), SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   -53.92), SIMDE_FLOAT32_C(  -889.11)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(    -2.11), SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(   852.60),
                         SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(    -1.42), SIMDE_FLOAT32_C(     0.45),
                         SIMDE_FLOAT32_C(    -1.80), SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.18),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(    -2.80), SIMDE_FLOAT32_C(    91.94), SIMDE_FLOAT32_C(    -3.05)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   195.04), SIMDE_FLOAT32_C(   266.59), SIMDE_FLOAT32_C(   227.06), SIMDE_FLOAT32_C(   410.49),
                         SIMDE_FLOAT32_C(  -523.93), SIMDE_FLOAT32_C(   762.39), SIMDE_FLOAT32_C(   112.81), SIMDE_FLOAT32_C(   686.52),
                         SIMDE_FLOAT32_C(   719.98), SIMDE_FLOAT32_C(   766.36), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(  -493.41),
                         SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(  -350.72), SIMDE_FLOAT32_C(  -209.34)),
      UINT16_C(43196),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -658.72), SIMDE_FLOAT32_C(  -177.76), SIMDE_FLOAT32_C(  -265.00), SIMDE_FLOAT32_C(  -554.31),
                         SIMDE_FLOAT32_C(   533.87), SIMDE_FLOAT32_C(    51.67), SIMDE_FLOAT32_C(  -492.25), SIMDE_FLOAT32_C(   777.74),
                         SIMDE_FLOAT32_C(   793.81), SIMDE_FLOAT32_C(    15.12), SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   824.88),
                         SIMDE_FLOAT32_C(   822.72), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(    60.89)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -371.53), SIMDE_FLOAT32_C(   353.46), SIMDE_FLOAT32_C(  -605.99), SIMDE_FLOAT32_C(  -513.13),
                         SIMDE_FLOAT32_C(  -390.22), SIMDE_FLOAT32_C(  -973.72), SIMDE_FLOAT32_C(  -469.41), SIMDE_FLOAT32_C(    31.72),
                         SIMDE_FLOAT32_C(   -35.27), SIMDE_FLOAT32_C(  -851.21), SIMDE_FLOAT32_C(  -841.43), SIMDE_FLOAT32_C(   330.43),
                         SIMDE_FLOAT32_C(   793.63), SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -816.27)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -2.08), SIMDE_FLOAT32_C(   266.59), SIMDE_FLOAT32_C(    -2.73), SIMDE_FLOAT32_C(   410.49),
                         SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(   762.39), SIMDE_FLOAT32_C(   112.81), SIMDE_FLOAT32_C(   686.52),
                         SIMDE_FLOAT32_C(     1.62), SIMDE_FLOAT32_C(   766.36), SIMDE_FLOAT32_C(    -2.39), SIMDE_FLOAT32_C(     1.19),
                         SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(  -350.72), SIMDE_FLOAT32_C(  -209.34)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -858.24), SIMDE_FLOAT32_C(  -559.04), SIMDE_FLOAT32_C(  -867.90), SIMDE_FLOAT32_C(   -91.47),
                         SIMDE_FLOAT32_C(  -996.53), SIMDE_FLOAT32_C(     7.89), SIMDE_FLOAT32_C(   519.91), SIMDE_FLOAT32_C(  -788.90),
                         SIMDE_FLOAT32_C(   494.45), SIMDE_FLOAT32_C(   338.97), SIMDE_FLOAT32_C(   858.03), SIMDE_FLOAT32_C(  -607.40),
                         SIMDE_FLOAT32_C(   289.29), SIMDE_FLOAT32_C(   618.46), SIMDE_FLOAT32_C(   413.47), SIMDE_FLOAT32_C(  -978.77)),
      UINT16_C( 4768),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   740.49), SIMDE_FLOAT32_C(  -751.81), SIMDE_FLOAT32_C(    13.69), SIMDE_FLOAT32_C(   786.36),
                         SIMDE_FLOAT32_C(  -616.97), SIMDE_FLOAT32_C(   500.34), SIMDE_FLOAT32_C(  -906.43), SIMDE_FLOAT32_C(   690.06),
                         SIMDE_FLOAT32_C(  -252.06), SIMDE_FLOAT32_C(   828.60), SIMDE_FLOAT32_C(  -203.59), SIMDE_FLOAT32_C(   933.39),
                         SIMDE_FLOAT32_C(   -10.85), SIMDE_FLOAT32_C(  -429.78), SIMDE_FLOAT32_C(   190.25), SIMDE_FLOAT32_C(   546.67)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -969.00), SIMDE_FLOAT32_C(   251.09), SIMDE_FLOAT32_C(   109.97), SIMDE_FLOAT32_C(   792.28),
                         SIMDE_FLOAT32_C(  -643.59), SIMDE_FLOAT32_C(   926.98), SIMDE_FLOAT32_C(  -815.02), SIMDE_FLOAT32_C(   181.20),
                         SIMDE_FLOAT32_C(  -206.24), SIMDE_FLOAT32_C(   378.12), SIMDE_FLOAT32_C(   -36.10), SIMDE_FLOAT32_C(  -538.28),
                         SIMDE_FLOAT32_C(   894.04), SIMDE_FLOAT32_C(    72.41), SIMDE_FLOAT32_C(   681.48), SIMDE_FLOAT32_C(   677.82)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -858.24), SIMDE_FLOAT32_C(  -559.04), SIMDE_FLOAT32_C(  -867.90), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(  -996.53), SIMDE_FLOAT32_C(     7.89), SIMDE_FLOAT32_C(    -2.30), SIMDE_FLOAT32_C(  -788.90),
                         SIMDE_FLOAT32_C(    -2.26), SIMDE_FLOAT32_C(   338.97), SIMDE_FLOAT32_C(    -1.75), SIMDE_FLOAT32_C(  -607.40),
                         SIMDE_FLOAT32_C(   289.29), SIMDE_FLOAT32_C(   618.46), SIMDE_FLOAT32_C(   413.47), SIMDE_FLOAT32_C(  -978.77)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -69.61), SIMDE_FLOAT32_C(  -548.92), SIMDE_FLOAT32_C(   625.99), SIMDE_FLOAT32_C(   381.43),
                         SIMDE_FLOAT32_C(   949.66), SIMDE_FLOAT32_C(  -196.91), SIMDE_FLOAT32_C(    28.28), SIMDE_FLOAT32_C(  -181.88),
                         SIMDE_FLOAT32_C(   536.29), SIMDE_FLOAT32_C(  -985.19), SIMDE_FLOAT32_C(    77.09), SIMDE_FLOAT32_C(   315.82),
                         SIMDE_FLOAT32_C(    11.44), SIMDE_FLOAT32_C(  -742.19), SIMDE_FLOAT32_C(   808.07), SIMDE_FLOAT32_C(  -406.94)),
      UINT16_C(49835),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -137.31), SIMDE_FLOAT32_C(  -142.23), SIMDE_FLOAT32_C(    35.44), SIMDE_FLOAT32_C(  -260.69),
                         SIMDE_FLOAT32_C(  -868.51), SIMDE_FLOAT32_C(  -878.61), SIMDE_FLOAT32_C(   777.12), SIMDE_FLOAT32_C(   132.77),
                         SIMDE_FLOAT32_C(  -396.93), SIMDE_FLOAT32_C(   836.29), SIMDE_FLOAT32_C(  -770.09), SIMDE_FLOAT32_C(   911.50),
                         SIMDE_FLOAT32_C(   393.21), SIMDE_FLOAT32_C(  -291.56), SIMDE_FLOAT32_C(   446.83), SIMDE_FLOAT32_C(   802.68)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -821.75), SIMDE_FLOAT32_C(  -892.28), SIMDE_FLOAT32_C(  -852.69), SIMDE_FLOAT32_C(     9.54),
                         SIMDE_FLOAT32_C(  -850.83), SIMDE_FLOAT32_C(   144.77), SIMDE_FLOAT32_C(   932.71), SIMDE_FLOAT32_C(  -565.94),
                         SIMDE_FLOAT32_C(  -821.82), SIMDE_FLOAT32_C(  -929.08), SIMDE_FLOAT32_C(  -624.00), SIMDE_FLOAT32_C(  -595.23),
                         SIMDE_FLOAT32_C(   666.07), SIMDE_FLOAT32_C(  -246.97), SIMDE_FLOAT32_C(  -517.48), SIMDE_FLOAT32_C(   645.83)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -2.98), SIMDE_FLOAT32_C(    -2.98), SIMDE_FLOAT32_C(   625.99), SIMDE_FLOAT32_C(   381.43),
                         SIMDE_FLOAT32_C(   949.66), SIMDE_FLOAT32_C(  -196.91), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(  -181.88),
                         SIMDE_FLOAT32_C(    -2.69), SIMDE_FLOAT32_C(  -985.19), SIMDE_FLOAT32_C(    -2.25), SIMDE_FLOAT32_C(   315.82),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(  -742.19), SIMDE_FLOAT32_C(     2.43), SIMDE_FLOAT32_C(     0.89)) },
   { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(   346.63)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_atan2_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a, test_vec[i].b);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_atan2_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d b;
    simde__m512d r;
  } test_vec[9] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45),
                         SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -2.35), SIMDE_FLOAT64_C(    1.42),
                         SIMDE_FLOAT64_C(    0.47), SIMDE_FLOAT64_C(    2.09),
                         SIMDE_FLOAT64_C(    0.93), SIMDE_FLOAT64_C(    3.03),
                         SIMDE_FLOAT64_C(    2.94), SIMDE_FLOAT64_C(    2.71)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -450.67), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C( -212.54), SIMDE_FLOAT64_C( -660.80),
                         SIMDE_FLOAT64_C(   28.47), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -860.95), SIMDE_FLOAT64_C(  696.87)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.19), SIMDE_FLOAT64_C(    1.24),
                         SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -2.55),
                         SIMDE_FLOAT64_C(    3.08), SIMDE_FLOAT64_C(   -1.96),
                         SIMDE_FLOAT64_C(   -1.91), SIMDE_FLOAT64_C(    2.11)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  395.92), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C( -263.99), SIMDE_FLOAT64_C(  -30.79),
                         SIMDE_FLOAT64_C(  443.48), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C(  993.90), SIMDE_FLOAT64_C(  841.21)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  655.87),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C(  780.64),
                         SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C( -583.60),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(   28.08)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.35), SIMDE_FLOAT64_C(    0.48),
                         SIMDE_FLOAT64_C(   -0.46), SIMDE_FLOAT64_C(   -0.04),
                         SIMDE_FLOAT64_C(    2.62), SIMDE_FLOAT64_C(    2.56),
                         SIMDE_FLOAT64_C(    2.23), SIMDE_FLOAT64_C(    1.54)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -767.23), SIMDE_FLOAT64_C(  398.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C( -148.69),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  791.23),
                         SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  336.73),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -554.19)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.97), SIMDE_FLOAT64_C(    1.75),
                         SIMDE_FLOAT64_C(    0.58), SIMDE_FLOAT64_C(    0.31),
                         SIMDE_FLOAT64_C(   -2.76), SIMDE_FLOAT64_C(   -1.23),
                         SIMDE_FLOAT64_C(   -2.34), SIMDE_FLOAT64_C(    2.52)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(  343.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(  655.67),
                         SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C( -171.51)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  324.62),
                         SIMDE_FLOAT64_C( -874.31), SIMDE_FLOAT64_C( -328.54),
                         SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  561.36),
                         SIMDE_FLOAT64_C(  -70.91), SIMDE_FLOAT64_C(  120.65)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.80), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(   -2.40), SIMDE_FLOAT64_C(   -2.13),
                         SIMDE_FLOAT64_C(   -1.80), SIMDE_FLOAT64_C(    0.86),
                         SIMDE_FLOAT64_C(    1.70), SIMDE_FLOAT64_C(   -0.96)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  840.65), SIMDE_FLOAT64_C( -591.56),
                         SIMDE_FLOAT64_C(  731.49), SIMDE_FLOAT64_C(  623.70),
                         SIMDE_FLOAT64_C(  140.67), SIMDE_FLOAT64_C( -906.16),
                         SIMDE_FLOAT64_C(   99.93), SIMDE_FLOAT64_C( -738.19)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  831.02), SIMDE_FLOAT64_C(  977.36),
                         SIMDE_FLOAT64_C(  331.34), SIMDE_FLOAT64_C(  462.95)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.88), SIMDE_FLOAT64_C(   -1.61),
                         SIMDE_FLOAT64_C(    2.12), SIMDE_FLOAT64_C(    0.89),
                         SIMDE_FLOAT64_C(    0.17), SIMDE_FLOAT64_C(   -0.75),
                         SIMDE_FLOAT64_C(    0.29), SIMDE_FLOAT64_C(   -1.01)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  822.06), SIMDE_FLOAT64_C( -997.63),
                         SIMDE_FLOAT64_C(  923.64), SIMDE_FLOAT64_C( -768.12),
                         SIMDE_FLOAT64_C(  -67.64), SIMDE_FLOAT64_C(  977.49),
                         SIMDE_FLOAT64_C(  424.81), SIMDE_FLOAT64_C(  -95.15)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -769.09), SIMDE_FLOAT64_C( -573.81),
                         SIMDE_FLOAT64_C( -337.60), SIMDE_FLOAT64_C(  293.64),
                         SIMDE_FLOAT64_C( -576.22), SIMDE_FLOAT64_C(  710.38),
                         SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   27.25)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.32), SIMDE_FLOAT64_C(   -2.09),
                         SIMDE_FLOAT64_C(    1.92), SIMDE_FLOAT64_C(   -1.21),
                         SIMDE_FLOAT64_C(   -3.02), SIMDE_FLOAT64_C(    0.94),
                         SIMDE_FLOAT64_C(    2.63), SIMDE_FLOAT64_C(   -1.29)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -359.76), SIMDE_FLOAT64_C(  -33.67),
                         SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -125.20),
                         SIMDE_FLOAT64_C(   39.93), SIMDE_FLOAT64_C(  394.67),
                         SIMDE_FLOAT64_C( -304.73), SIMDE_FLOAT64_C( -696.69)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -438.19), SIMDE_FLOAT64_C( -752.43),
                         SIMDE_FLOAT64_C(  932.66), SIMDE_FLOAT64_C( -327.22),
                         SIMDE_FLOAT64_C( -182.45), SIMDE_FLOAT64_C(  510.85),
                         SIMDE_FLOAT64_C(   14.34), SIMDE_FLOAT64_C(  916.26)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -2.45), SIMDE_FLOAT64_C(   -3.10),
                         SIMDE_FLOAT64_C(    0.01), SIMDE_FLOAT64_C(   -2.78),
                         SIMDE_FLOAT64_C(    2.93), SIMDE_FLOAT64_C(    0.66),
                         SIMDE_FLOAT64_C(   -1.52), SIMDE_FLOAT64_C(   -0.65)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C(  254.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00),
                         SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00),
                         SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00),
                         SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(    1.57)) }

  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_atan2_pd(test_vec[i].a, test_vec[i].b);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_atan2_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d b;
    simde__m512d r;
  } test_vec[9] = {
   { simde_mm512_set_pd(SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(  346.63)),
      UINT8_C(212),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   28.47), SIMDE_FLOAT64_C( -305.07),
                         SIMDE_FLOAT64_C(  696.87), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  497.31),
                         SIMDE_FLOAT64_C(   34.06), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   39.01)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.08), SIMDE_FLOAT64_C(   -2.82),
                         SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(    3.02),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(  346.63)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  395.92), SIMDE_FLOAT64_C(  532.35),
                         SIMDE_FLOAT64_C(  -30.79), SIMDE_FLOAT64_C( -583.60),
                         SIMDE_FLOAT64_C(  993.90), SIMDE_FLOAT64_C(  178.20),
                         SIMDE_FLOAT64_C(  687.09), SIMDE_FLOAT64_C( -976.55)),
      UINT8_C(126),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(  841.21),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C( -212.54)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  398.82), SIMDE_FLOAT64_C(  655.87),
                         SIMDE_FLOAT64_C( -263.99), SIMDE_FLOAT64_C( -770.35),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(   28.08),
                         SIMDE_FLOAT64_C( -450.67), SIMDE_FLOAT64_C(  261.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  395.92), SIMDE_FLOAT64_C(    0.48),
                         SIMDE_FLOAT64_C(    1.90), SIMDE_FLOAT64_C(    2.62),
                         SIMDE_FLOAT64_C(   -1.11), SIMDE_FLOAT64_C(    1.54),
                         SIMDE_FLOAT64_C(    2.66), SIMDE_FLOAT64_C( -976.55)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  655.67),
                         SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -767.23)),
      UINT8_C( 39),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -525.83), SIMDE_FLOAT64_C(  561.36),
                         SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C(  469.66),
                         SIMDE_FLOAT64_C(  818.66), SIMDE_FLOAT64_C(  791.23),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -747.59)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -328.54), SIMDE_FLOAT64_C( -822.65),
                         SIMDE_FLOAT64_C(  -70.91), SIMDE_FLOAT64_C( -171.51),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C( -944.78)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  655.67),
                         SIMDE_FLOAT64_C(    1.70), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(    0.92),
                         SIMDE_FLOAT64_C(   -2.76), SIMDE_FLOAT64_C(   -2.47)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(  731.49), SIMDE_FLOAT64_C(  831.02),
                         SIMDE_FLOAT64_C( -906.16), SIMDE_FLOAT64_C(  462.95),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C( -874.31)),
      UINT8_C( 45),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   27.25), SIMDE_FLOAT64_C(  840.65),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  623.70),
                         SIMDE_FLOAT64_C(  977.36), SIMDE_FLOAT64_C(   99.93),
                         SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  343.48)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  424.81), SIMDE_FLOAT64_C(  690.12),
                         SIMDE_FLOAT64_C( -591.56), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  140.67), SIMDE_FLOAT64_C(  331.34),
                         SIMDE_FLOAT64_C( -738.19), SIMDE_FLOAT64_C(  324.62)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(   -2.49), SIMDE_FLOAT64_C(  831.02),
                         SIMDE_FLOAT64_C(    1.43), SIMDE_FLOAT64_C(    0.29),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(    0.81)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(   39.93),
                         SIMDE_FLOAT64_C(   14.34), SIMDE_FLOAT64_C( -696.69),
                         SIMDE_FLOAT64_C( -573.81), SIMDE_FLOAT64_C(  923.64),
                         SIMDE_FLOAT64_C( -576.22), SIMDE_FLOAT64_C(  977.49)),
      UINT8_C(108),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -182.45),
                         SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C(  916.26),
                         SIMDE_FLOAT64_C(  822.06), SIMDE_FLOAT64_C( -337.60),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(  710.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  932.66), SIMDE_FLOAT64_C( -125.20),
                         SIMDE_FLOAT64_C(  510.85), SIMDE_FLOAT64_C( -304.73),
                         SIMDE_FLOAT64_C( -769.09), SIMDE_FLOAT64_C( -997.63),
                         SIMDE_FLOAT64_C(  293.64), SIMDE_FLOAT64_C(  -67.64)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(   -2.17),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C( -696.69),
                         SIMDE_FLOAT64_C(    2.32), SIMDE_FLOAT64_C(   -2.82),
                         SIMDE_FLOAT64_C( -576.22), SIMDE_FLOAT64_C(  977.49)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  484.94), SIMDE_FLOAT64_C(  237.58),
                         SIMDE_FLOAT64_C( -765.93), SIMDE_FLOAT64_C( -623.50),
                         SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(  936.65),
                         SIMDE_FLOAT64_C( -197.89), SIMDE_FLOAT64_C( -752.43)),
      UINT8_C(214),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  826.84), SIMDE_FLOAT64_C( -598.06),
                         SIMDE_FLOAT64_C( -378.50), SIMDE_FLOAT64_C(  221.37),
                         SIMDE_FLOAT64_C( -942.47), SIMDE_FLOAT64_C(  440.64),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C( -359.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  133.52), SIMDE_FLOAT64_C(  -76.75),
                         SIMDE_FLOAT64_C( -791.07), SIMDE_FLOAT64_C( -601.68),
                         SIMDE_FLOAT64_C( -788.36), SIMDE_FLOAT64_C(  475.51),
                         SIMDE_FLOAT64_C(  897.27), SIMDE_FLOAT64_C( -438.19)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.41), SIMDE_FLOAT64_C(   -1.70),
                         SIMDE_FLOAT64_C( -765.93), SIMDE_FLOAT64_C(    2.79),
                         SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(    0.75),
                         SIMDE_FLOAT64_C(   -0.37), SIMDE_FLOAT64_C( -752.43)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -628.82), SIMDE_FLOAT64_C( -916.82),
                         SIMDE_FLOAT64_C(  434.03), SIMDE_FLOAT64_C(  -15.61),
                         SIMDE_FLOAT64_C( -718.40), SIMDE_FLOAT64_C(  177.92),
                         SIMDE_FLOAT64_C(  426.61), SIMDE_FLOAT64_C(  915.71)),
      UINT8_C( 31),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  334.00), SIMDE_FLOAT64_C(  556.35),
                         SIMDE_FLOAT64_C( -490.00), SIMDE_FLOAT64_C(  496.57),
                         SIMDE_FLOAT64_C( -737.13), SIMDE_FLOAT64_C(  159.97),
                         SIMDE_FLOAT64_C(  345.93), SIMDE_FLOAT64_C(  932.11)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  783.48), SIMDE_FLOAT64_C(  274.71),
                         SIMDE_FLOAT64_C(  439.43), SIMDE_FLOAT64_C( -799.40),
                         SIMDE_FLOAT64_C(  915.19), SIMDE_FLOAT64_C( -314.93),
                         SIMDE_FLOAT64_C( -861.01), SIMDE_FLOAT64_C(  888.71)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -628.82), SIMDE_FLOAT64_C( -916.82),
                         SIMDE_FLOAT64_C(  434.03), SIMDE_FLOAT64_C(    2.59),
                         SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(    2.67),
                         SIMDE_FLOAT64_C(    2.76), SIMDE_FLOAT64_C(    0.81)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -964.25), SIMDE_FLOAT64_C( -807.28),
                         SIMDE_FLOAT64_C( -764.58), SIMDE_FLOAT64_C(   92.52),
                         SIMDE_FLOAT64_C( -818.54), SIMDE_FLOAT64_C(  -65.60),
                         SIMDE_FLOAT64_C(  -11.78), SIMDE_FLOAT64_C( -318.38)),
      UINT8_C( 46),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  -78.84), SIMDE_FLOAT64_C( -406.33),
                         SIMDE_FLOAT64_C(  -70.05), SIMDE_FLOAT64_C(  789.89),
                         SIMDE_FLOAT64_C(  206.60), SIMDE_FLOAT64_C(  161.06),
                         SIMDE_FLOAT64_C( -286.07), SIMDE_FLOAT64_C( -308.52)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -889.11), SIMDE_FLOAT64_C(  883.05),
                         SIMDE_FLOAT64_C( -743.66), SIMDE_FLOAT64_C( -784.34),
                         SIMDE_FLOAT64_C(    4.83), SIMDE_FLOAT64_C(  834.60),
                         SIMDE_FLOAT64_C(  579.25), SIMDE_FLOAT64_C( -212.86)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -964.25), SIMDE_FLOAT64_C( -807.28),
                         SIMDE_FLOAT64_C(   -3.05), SIMDE_FLOAT64_C(   92.52),
                         SIMDE_FLOAT64_C(    1.55), SIMDE_FLOAT64_C(    0.19),
                         SIMDE_FLOAT64_C(   -0.46), SIMDE_FLOAT64_C( -318.38)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.00), SIMDE_FLOAT64_C(  571.00),
                         SIMDE_FLOAT64_C(  422.00), SIMDE_FLOAT64_C(  468.00),
                         SIMDE_FLOAT64_C(  670.00), SIMDE_FLOAT64_C(   34.00),
                         SIMDE_FLOAT64_C(   39.00), SIMDE_FLOAT64_C(  347.00)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.00), SIMDE_FLOAT64_C(   85.00),
                         SIMDE_FLOAT64_C(  826.00), SIMDE_FLOAT64_C( -269.00),
                         SIMDE_FLOAT64_C(  497.00), SIMDE_FLOAT64_C( -297.00),
                         SIMDE_FLOAT64_C( -186.00), SIMDE_FLOAT64_C( -754.00)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00),
                         SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00),
                         SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00),
                         SIMDE_FLOAT64_C(    0.00), SIMDE_FLOAT64_C(    0.00)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(  571.00),
                         SIMDE_FLOAT64_C(  422.00), SIMDE_FLOAT64_C(  468.00),
                         SIMDE_FLOAT64_C(    1.57), SIMDE_FLOAT64_C(   34.00),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -1.57)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_atan2_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a, test_vec[i].b);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_atanh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.37)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.03)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.51)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.65)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -1.29), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.87)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(    -1.59)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -2.30), SIMDE_FLOAT32_C(    -0.79)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.85)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_atanh_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_atanh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.97), SIMDE_FLOAT64_C(    0.37)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(    0.03)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.55), SIMDE_FLOAT64_C(    0.81)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.28), SIMDE_FLOAT64_C(    0.51)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.19), SIMDE_FLOAT64_C(    0.45)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.65)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.83), SIMDE_FLOAT64_C(   -0.85)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_atanh_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_atanh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04),
                         SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.37)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69),
                         SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42),
                         SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.85),
                         SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.65),
                         SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     0.45),
                         SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.51)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86),
                         SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(    -1.59),
                         SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -1.29),
                         SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.87)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45),
                         SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21),
                         SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.48),
                         SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.85),
                         SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.21),
                         SIMDE_FLOAT32_C(    -2.30), SIMDE_FLOAT32_C(    -0.79)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.44),
                         SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.38),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.84)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.02), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(    -1.02), SIMDE_FLOAT32_C(     2.65),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     1.22)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.03)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.42),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.35),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.27),
                         SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(    -0.03)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.94),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.40)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(    -1.74),
                         SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(    -1.02),
                         SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(     0.42)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.25)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.83),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     1.16),
                         SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(     0.26)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_atanh_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_atanh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.97), SIMDE_FLOAT64_C(    0.37)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.55), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(    0.03)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.19), SIMDE_FLOAT64_C(    0.45),
                         SIMDE_FLOAT64_C(   -0.28), SIMDE_FLOAT64_C(    0.51)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.83), SIMDE_FLOAT64_C(   -0.85),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.65)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.86),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    0.70)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.32), SIMDE_FLOAT64_C(   -1.29),
                         SIMDE_FLOAT64_C(   -0.45), SIMDE_FLOAT64_C(    0.87)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.38), SIMDE_FLOAT64_C(   -0.92)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.47), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.40), SIMDE_FLOAT64_C(   -1.59)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(   -0.66)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.27), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -2.30), SIMDE_FLOAT64_C(   -0.79)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.69)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.48),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.85)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_atanh_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_atanh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.52),
                         SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.67)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     1.05),
                         SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.58),
                         SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.81)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.84),
                         SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.17),
                         SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.04),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.85)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     1.22),
                         SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.17),
                         SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.04),
                         SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     1.26)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.67),
                         SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.48),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.92)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.52),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.85),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     2.65), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     1.59)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.91),
                         SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.63),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.70)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     1.53),
                         SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     1.10), SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.74),
                         SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.87)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.67),
                         SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.24),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.83),
                         SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.41)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     1.38), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.24),
                         SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     1.19),
                         SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     1.02), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.44)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.20),
                         SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.05),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.13)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.20),
                         SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     1.13),
                         SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     2.65), SIMDE_FLOAT32_C(     0.05),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.13)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.12),
                         SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.45)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.12),
                         SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     1.26), SIMDE_FLOAT32_C(     2.65),
                         SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.48)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.48),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.44),
                         SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.70),
                         SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.52),
                         SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.87),
                         SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.15)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_atanh_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_atanh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.17),
                         SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.85),
                         SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.67)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.29),
                         SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.36),
                         SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.12)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.17),
                         SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.30),
                         SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.38),
                         SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.67)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.22),
                         SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.51)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.80),
                         SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.12),
                         SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.37),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.99)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.12),
                         SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     2.65)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.86),
                         SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.55),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.10),
                         SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.77)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.28),
                         SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.06),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.46)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.86),
                         SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.10),
                         SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.77)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.51),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.12)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.48),
                         SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.70),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.99)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.51),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     2.65)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.12),
                         SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.72)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.58),
                         SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.55),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.31),
                         SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.74)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.62),
                         SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.12),
                         SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.72)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.72)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.12),
                         SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.58),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.14),
                         SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.25)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.72)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.54),
                         SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.06)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.29),
                         SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.84),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.21),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.54)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     1.22),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.21),
                         SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.60)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.91),
                         SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.09),
                         SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.29),
                         SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.40)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.90),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.32),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.37),
                         SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.92)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     1.47),
                         SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     0.29),
                         SIMDE_FLOAT32_C(     2.09), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     1.59)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_atanh_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_atanh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.55), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.97), SIMDE_FLOAT64_C(    0.37)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.83), SIMDE_FLOAT64_C(   -0.85),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.65),
                         SIMDE_FLOAT64_C(    1.19), SIMDE_FLOAT64_C(    0.45),
                         SIMDE_FLOAT64_C(   -0.28), SIMDE_FLOAT64_C(    0.51)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.38), SIMDE_FLOAT64_C(   -0.92),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.86),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    0.70)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.47), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.40), SIMDE_FLOAT64_C(   -1.59),
                         SIMDE_FLOAT64_C(   -0.32), SIMDE_FLOAT64_C(   -1.29),
                         SIMDE_FLOAT64_C(   -0.45), SIMDE_FLOAT64_C(    0.87)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.69),
                         SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(   -0.66)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.48),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.85),
                         SIMDE_FLOAT64_C(    0.27), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -2.30), SIMDE_FLOAT64_C(   -0.79)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(    0.38),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.03), SIMDE_FLOAT64_C(    0.84)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.02), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(   -0.66), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(   -1.02), SIMDE_FLOAT64_C(    2.65),
                         SIMDE_FLOAT64_C(    0.03), SIMDE_FLOAT64_C(    1.22)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.34),
                         SIMDE_FLOAT64_C(    0.53), SIMDE_FLOAT64_C(   -0.26),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(   -0.03)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.41), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(    0.35),
                         SIMDE_FLOAT64_C(    0.59), SIMDE_FLOAT64_C(   -0.27),
                         SIMDE_FLOAT64_C(    1.05), SIMDE_FLOAT64_C(   -0.03)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.20), SIMDE_FLOAT64_C(   -0.08),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.94),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C(    0.40)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.20), SIMDE_FLOAT64_C(   -0.08),
                         SIMDE_FLOAT64_C(    0.35), SIMDE_FLOAT64_C(   -1.74),
                         SIMDE_FLOAT64_C(   -0.97), SIMDE_FLOAT64_C(   -1.02),
                         SIMDE_FLOAT64_C(   -0.62), SIMDE_FLOAT64_C(    0.42)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.47), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(    0.91), SIMDE_FLOAT64_C(    0.60),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(    0.25)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.51), SIMDE_FLOAT64_C(    0.83),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    1.16),
                         SIMDE_FLOAT64_C(    1.53), SIMDE_FLOAT64_C(    0.69),
                         SIMDE_FLOAT64_C(    1.07), SIMDE_FLOAT64_C(    0.26)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_atanh_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_atanh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.69), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.42), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(    0.04), SIMDE_FLOAT64_C(    0.35)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(    0.08),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(   -0.27),
                         SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(   -0.75)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.83), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.42), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(    0.55), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(   -0.97)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(    0.23),
                         SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(   -0.38),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.42)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.84), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.66), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.92), SIMDE_FLOAT64_C(   -0.86)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.22), SIMDE_FLOAT64_C(   -0.48),
                         SIMDE_FLOAT64_C(    0.85), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -1.29)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.26),
                         SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(    0.99)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C(   -0.39),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.53),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(   -0.77)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.62), SIMDE_FLOAT64_C(   -0.41),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(    0.59),
                         SIMDE_FLOAT64_C(    1.05), SIMDE_FLOAT64_C(   -1.02),
                         SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(   -1.02)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.12), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.91),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.75)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.54), SIMDE_FLOAT64_C(   -0.17),
                         SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(    0.60), SIMDE_FLOAT64_C(    0.25),
                         SIMDE_FLOAT64_C(   -0.08), SIMDE_FLOAT64_C(   -0.94)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.12), SIMDE_FLOAT64_C(   -0.17),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    1.16),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(    0.26),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -1.74)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.10), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.34),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.53),
                         SIMDE_FLOAT64_C(   -0.82), SIMDE_FLOAT64_C(    0.66)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.33), SIMDE_FLOAT64_C(    0.46),
                         SIMDE_FLOAT64_C(   -0.18), SIMDE_FLOAT64_C(    0.32),
                         SIMDE_FLOAT64_C(   -0.87), SIMDE_FLOAT64_C(   -0.33),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.56)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.33),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.53),
                         SIMDE_FLOAT64_C(   -0.82), SIMDE_FLOAT64_C(    0.63)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.02),
                         SIMDE_FLOAT64_C(   -0.45), SIMDE_FLOAT64_C(    0.51),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.98)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.98), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.10), SIMDE_FLOAT64_C(    0.84),
                         SIMDE_FLOAT64_C(   -0.59), SIMDE_FLOAT64_C(    0.73),
                         SIMDE_FLOAT64_C(    0.62), SIMDE_FLOAT64_C(    0.14)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.45),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.02),
                         SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(    0.51),
                         SIMDE_FLOAT64_C(    0.73), SIMDE_FLOAT64_C(    0.14)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(   -0.70), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.92),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(   -0.07)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.51), SIMDE_FLOAT64_C(    0.01),
                         SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.57), SIMDE_FLOAT64_C(   -0.34),
                         SIMDE_FLOAT64_C(    0.29), SIMDE_FLOAT64_C(   -0.58)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(    0.01),
                         SIMDE_FLOAT64_C(   -0.70), SIMDE_FLOAT64_C(   -1.02),
                         SIMDE_FLOAT64_C(   -0.65), SIMDE_FLOAT64_C(   -0.35),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(   -0.66)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.48), SIMDE_FLOAT64_C(    0.94),
                         SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(   -0.44),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.93),
                         SIMDE_FLOAT64_C(   -0.33), SIMDE_FLOAT64_C(   -0.18)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(   -0.36), SIMDE_FLOAT64_C(   -0.03),
                         SIMDE_FLOAT64_C(    0.01), SIMDE_FLOAT64_C(   -0.13)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.02), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.03),
                         SIMDE_FLOAT64_C(   -0.33), SIMDE_FLOAT64_C(   -0.13)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_atanh_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_cdfnorm_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -993.83), SIMDE_FLOAT32_C(    92.27), SIMDE_FLOAT32_C(   208.35), SIMDE_FLOAT32_C(   761.44) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -963.46), SIMDE_FLOAT32_C(   429.93), SIMDE_FLOAT32_C(   318.99), SIMDE_FLOAT32_C(   532.75) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(   677.31), SIMDE_FLOAT32_C(  -552.55), SIMDE_FLOAT32_C(   344.89), SIMDE_FLOAT32_C(  -275.73) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(  -396.40), SIMDE_FLOAT32_C(   319.50), SIMDE_FLOAT32_C(   348.88), SIMDE_FLOAT32_C(  -732.73) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   638.44), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(  -165.87), SIMDE_FLOAT32_C(   843.45) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -841.80), SIMDE_FLOAT32_C(  -382.17), SIMDE_FLOAT32_C(  -889.98), SIMDE_FLOAT32_C(   238.69) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -193.56), SIMDE_FLOAT32_C(   381.13), SIMDE_FLOAT32_C(  -623.80), SIMDE_FLOAT32_C(   -46.41) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   798.25), SIMDE_FLOAT32_C(  -366.96), SIMDE_FLOAT32_C(   249.70), SIMDE_FLOAT32_C(   804.43) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_cdfnorm_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_cdfnorm_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -954.47), SIMDE_FLOAT64_C(  -900.72) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   375.82), SIMDE_FLOAT64_C(   323.80) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -882.15), SIMDE_FLOAT64_C(  -872.83) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -880.22), SIMDE_FLOAT64_C(   404.86) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(   587.17), SIMDE_FLOAT64_C(   674.97) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -509.08), SIMDE_FLOAT64_C(  -152.91) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -296.61), SIMDE_FLOAT64_C(   576.29) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -858.64), SIMDE_FLOAT64_C(  -995.64) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_cdfnorm_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_cdfnorm_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   818.12), SIMDE_FLOAT32_C(   842.04), SIMDE_FLOAT32_C(  -990.82), SIMDE_FLOAT32_C(  -180.40),
        SIMDE_FLOAT32_C(  -703.48), SIMDE_FLOAT32_C(  -658.67), SIMDE_FLOAT32_C(  -675.01), SIMDE_FLOAT32_C(  -213.67) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(  -312.75), SIMDE_FLOAT32_C(  -440.95), SIMDE_FLOAT32_C(    40.83), SIMDE_FLOAT32_C(  -601.56),
        SIMDE_FLOAT32_C(   516.51), SIMDE_FLOAT32_C(    64.68), SIMDE_FLOAT32_C(   765.54), SIMDE_FLOAT32_C(   383.86) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -264.08), SIMDE_FLOAT32_C(  -961.69), SIMDE_FLOAT32_C(   776.59), SIMDE_FLOAT32_C(  -476.70),
        SIMDE_FLOAT32_C(   398.19), SIMDE_FLOAT32_C(   561.61), SIMDE_FLOAT32_C(  -253.27), SIMDE_FLOAT32_C(   994.83) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -614.21), SIMDE_FLOAT32_C(   933.12), SIMDE_FLOAT32_C(   521.15), SIMDE_FLOAT32_C(    87.99),
        SIMDE_FLOAT32_C(   511.16), SIMDE_FLOAT32_C(   278.58), SIMDE_FLOAT32_C(  -327.57), SIMDE_FLOAT32_C(   329.28) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(   120.61), SIMDE_FLOAT32_C(  -318.39), SIMDE_FLOAT32_C(  -851.12), SIMDE_FLOAT32_C(   417.13),
        SIMDE_FLOAT32_C(    22.95), SIMDE_FLOAT32_C(  -526.13), SIMDE_FLOAT32_C(  -796.54), SIMDE_FLOAT32_C(   710.20) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(    32.92), SIMDE_FLOAT32_C(   244.29), SIMDE_FLOAT32_C(  -891.36), SIMDE_FLOAT32_C(  -450.57),
        SIMDE_FLOAT32_C(  -691.03), SIMDE_FLOAT32_C(   874.17), SIMDE_FLOAT32_C(   933.29), SIMDE_FLOAT32_C(    44.89) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(   912.48), SIMDE_FLOAT32_C(   709.88), SIMDE_FLOAT32_C(   568.19), SIMDE_FLOAT32_C(   310.67),
        SIMDE_FLOAT32_C(   271.49), SIMDE_FLOAT32_C(  -685.08), SIMDE_FLOAT32_C(   305.50), SIMDE_FLOAT32_C(   657.28) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -751.96), SIMDE_FLOAT32_C(  -173.35), SIMDE_FLOAT32_C(  -254.73), SIMDE_FLOAT32_C(   759.20),
        SIMDE_FLOAT32_C(  -894.77), SIMDE_FLOAT32_C(   417.70), SIMDE_FLOAT32_C(    88.48), SIMDE_FLOAT32_C(   225.84) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_cdfnorm_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_cdfnorm_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -924.75), SIMDE_FLOAT64_C(  -974.37), SIMDE_FLOAT64_C(  -748.27), SIMDE_FLOAT64_C(  -367.36) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -632.95), SIMDE_FLOAT64_C(   220.99), SIMDE_FLOAT64_C(   820.62), SIMDE_FLOAT64_C(  -652.24) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -811.15), SIMDE_FLOAT64_C(  -815.96), SIMDE_FLOAT64_C(   903.78), SIMDE_FLOAT64_C(   978.99) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -359.97), SIMDE_FLOAT64_C(  -262.68), SIMDE_FLOAT64_C(  -977.31), SIMDE_FLOAT64_C(  -241.69) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(    96.53), SIMDE_FLOAT64_C(   838.57), SIMDE_FLOAT64_C(   179.14), SIMDE_FLOAT64_C(   108.78) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(   -69.02), SIMDE_FLOAT64_C(   -39.14), SIMDE_FLOAT64_C(    24.34), SIMDE_FLOAT64_C(  -579.34) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(    73.79), SIMDE_FLOAT64_C(    99.84), SIMDE_FLOAT64_C(   430.49), SIMDE_FLOAT64_C(   713.26) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -127.22), SIMDE_FLOAT64_C(  -439.34), SIMDE_FLOAT64_C(  -849.37), SIMDE_FLOAT64_C(   -51.97) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_cdfnorm_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_cdfnorm_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -171.83), SIMDE_FLOAT32_C(   -16.40), SIMDE_FLOAT32_C(  -352.71), SIMDE_FLOAT32_C(  -355.76),
        SIMDE_FLOAT32_C(  -532.92), SIMDE_FLOAT32_C(  -657.24), SIMDE_FLOAT32_C(   -31.51), SIMDE_FLOAT32_C(  -403.96),
        SIMDE_FLOAT32_C(    10.99), SIMDE_FLOAT32_C(  -120.77), SIMDE_FLOAT32_C(   317.51), SIMDE_FLOAT32_C(   262.42),
        SIMDE_FLOAT32_C(   830.85), SIMDE_FLOAT32_C(  -503.76), SIMDE_FLOAT32_C(   762.65), SIMDE_FLOAT32_C(  -301.62) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   330.53), SIMDE_FLOAT32_C(   478.14), SIMDE_FLOAT32_C(  -836.82), SIMDE_FLOAT32_C(   378.71),
        SIMDE_FLOAT32_C(   784.61), SIMDE_FLOAT32_C(   602.57), SIMDE_FLOAT32_C(   441.59), SIMDE_FLOAT32_C(  -912.33),
        SIMDE_FLOAT32_C(  -474.27), SIMDE_FLOAT32_C(   991.91), SIMDE_FLOAT32_C(   893.21), SIMDE_FLOAT32_C(    55.17),
        SIMDE_FLOAT32_C(  -251.62), SIMDE_FLOAT32_C(   632.38), SIMDE_FLOAT32_C(   573.89), SIMDE_FLOAT32_C(   576.55) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -384.02), SIMDE_FLOAT32_C(  -778.82), SIMDE_FLOAT32_C(  -779.21), SIMDE_FLOAT32_C(    83.07),
        SIMDE_FLOAT32_C(  -436.06), SIMDE_FLOAT32_C(   189.28), SIMDE_FLOAT32_C(   679.10), SIMDE_FLOAT32_C(   574.93),
        SIMDE_FLOAT32_C(  -931.49), SIMDE_FLOAT32_C(    -3.39), SIMDE_FLOAT32_C(  -162.65), SIMDE_FLOAT32_C(   899.36),
        SIMDE_FLOAT32_C(   492.85), SIMDE_FLOAT32_C(  -399.99), SIMDE_FLOAT32_C(  -402.27), SIMDE_FLOAT32_C(  -176.62) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(  -921.85), SIMDE_FLOAT32_C(  -239.09), SIMDE_FLOAT32_C(  -797.90), SIMDE_FLOAT32_C(   862.75),
        SIMDE_FLOAT32_C(  -636.52), SIMDE_FLOAT32_C(   643.69), SIMDE_FLOAT32_C(   950.42), SIMDE_FLOAT32_C(  -110.78),
        SIMDE_FLOAT32_C(   635.59), SIMDE_FLOAT32_C(   843.63), SIMDE_FLOAT32_C(   944.39), SIMDE_FLOAT32_C(  -616.03),
        SIMDE_FLOAT32_C(   476.02), SIMDE_FLOAT32_C(   518.27), SIMDE_FLOAT32_C(   960.52), SIMDE_FLOAT32_C(  -908.00) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   739.45), SIMDE_FLOAT32_C(  -818.69), SIMDE_FLOAT32_C(   175.06), SIMDE_FLOAT32_C(  -696.61),
        SIMDE_FLOAT32_C(   370.60), SIMDE_FLOAT32_C(  -145.84), SIMDE_FLOAT32_C(   878.31), SIMDE_FLOAT32_C(   439.11),
        SIMDE_FLOAT32_C(   850.77), SIMDE_FLOAT32_C(  -284.33), SIMDE_FLOAT32_C(   338.47), SIMDE_FLOAT32_C(   343.62),
        SIMDE_FLOAT32_C(   315.67), SIMDE_FLOAT32_C(   936.20), SIMDE_FLOAT32_C(  -832.99), SIMDE_FLOAT32_C(   393.82) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -302.88), SIMDE_FLOAT32_C(  -630.90), SIMDE_FLOAT32_C(   256.57), SIMDE_FLOAT32_C(    60.60),
        SIMDE_FLOAT32_C(  -987.21), SIMDE_FLOAT32_C(   206.99), SIMDE_FLOAT32_C(   949.82), SIMDE_FLOAT32_C(   648.38),
        SIMDE_FLOAT32_C(    50.62), SIMDE_FLOAT32_C(   894.21), SIMDE_FLOAT32_C(  -967.65), SIMDE_FLOAT32_C(  -473.36),
        SIMDE_FLOAT32_C(   412.48), SIMDE_FLOAT32_C(   992.88), SIMDE_FLOAT32_C(  -381.36), SIMDE_FLOAT32_C(   151.93) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -825.81), SIMDE_FLOAT32_C(   793.70), SIMDE_FLOAT32_C(   455.32), SIMDE_FLOAT32_C(   544.79),
        SIMDE_FLOAT32_C(  -352.14), SIMDE_FLOAT32_C(   333.63), SIMDE_FLOAT32_C(   -16.10), SIMDE_FLOAT32_C(  -501.36),
        SIMDE_FLOAT32_C(  -950.70), SIMDE_FLOAT32_C(  -677.63), SIMDE_FLOAT32_C(   842.26), SIMDE_FLOAT32_C(   364.97),
        SIMDE_FLOAT32_C(  -741.43), SIMDE_FLOAT32_C(  -990.74), SIMDE_FLOAT32_C(  -241.21), SIMDE_FLOAT32_C(   -44.31) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(  -621.64), SIMDE_FLOAT32_C(  -984.64), SIMDE_FLOAT32_C(  -983.70), SIMDE_FLOAT32_C(  -608.85),
        SIMDE_FLOAT32_C(   222.35), SIMDE_FLOAT32_C(   966.12), SIMDE_FLOAT32_C(  -960.47), SIMDE_FLOAT32_C(  -727.02),
        SIMDE_FLOAT32_C(   860.32), SIMDE_FLOAT32_C(  -928.11), SIMDE_FLOAT32_C(  -200.38), SIMDE_FLOAT32_C(   272.80),
        SIMDE_FLOAT32_C(  -935.24), SIMDE_FLOAT32_C(   418.26), SIMDE_FLOAT32_C(  -575.27), SIMDE_FLOAT32_C(  -761.04) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_cdfnorm_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cdfnorm_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   742.28), SIMDE_FLOAT32_C(   -10.25), SIMDE_FLOAT32_C(  -827.23), SIMDE_FLOAT32_C(   995.37),
        SIMDE_FLOAT32_C(   256.37), SIMDE_FLOAT32_C(   283.72), SIMDE_FLOAT32_C(  -388.62), SIMDE_FLOAT32_C(  -979.71),
        SIMDE_FLOAT32_C(  -680.17), SIMDE_FLOAT32_C(  -749.87), SIMDE_FLOAT32_C(   -71.05), SIMDE_FLOAT32_C(   -60.71),
        SIMDE_FLOAT32_C(  -405.48), SIMDE_FLOAT32_C(   786.24), SIMDE_FLOAT32_C(  -561.14), SIMDE_FLOAT32_C(   561.28) },
      UINT8_C(133),
      { SIMDE_FLOAT32_C(   409.19), SIMDE_FLOAT32_C(  -492.65), SIMDE_FLOAT32_C(    57.95), SIMDE_FLOAT32_C(  -250.60),
        SIMDE_FLOAT32_C(  -403.16), SIMDE_FLOAT32_C(   437.65), SIMDE_FLOAT32_C(   509.49), SIMDE_FLOAT32_C(   -69.63),
        SIMDE_FLOAT32_C(   308.33), SIMDE_FLOAT32_C(   780.29), SIMDE_FLOAT32_C(  -943.64), SIMDE_FLOAT32_C(   322.23),
        SIMDE_FLOAT32_C(   242.19), SIMDE_FLOAT32_C(   643.12), SIMDE_FLOAT32_C(    64.51), SIMDE_FLOAT32_C(  -768.06) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   -10.25), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   995.37),
        SIMDE_FLOAT32_C(   256.37), SIMDE_FLOAT32_C(   283.72), SIMDE_FLOAT32_C(  -388.62), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(  -680.17), SIMDE_FLOAT32_C(  -749.87), SIMDE_FLOAT32_C(   -71.05), SIMDE_FLOAT32_C(   -60.71),
        SIMDE_FLOAT32_C(  -405.48), SIMDE_FLOAT32_C(   786.24), SIMDE_FLOAT32_C(  -561.14), SIMDE_FLOAT32_C(   561.28) } },
    { { SIMDE_FLOAT32_C(   815.89), SIMDE_FLOAT32_C(    59.87), SIMDE_FLOAT32_C(   488.31), SIMDE_FLOAT32_C(    99.61),
        SIMDE_FLOAT32_C(   671.25), SIMDE_FLOAT32_C(   508.61), SIMDE_FLOAT32_C(   419.45), SIMDE_FLOAT32_C(   921.38),
        SIMDE_FLOAT32_C(  -562.45), SIMDE_FLOAT32_C(  -641.27), SIMDE_FLOAT32_C(  -484.11), SIMDE_FLOAT32_C(  -776.21),
        SIMDE_FLOAT32_C(  -202.41), SIMDE_FLOAT32_C(  -922.83), SIMDE_FLOAT32_C(  -317.45), SIMDE_FLOAT32_C(  -793.22) },
      UINT8_C(110),
      { SIMDE_FLOAT32_C(   740.50), SIMDE_FLOAT32_C(   -43.82), SIMDE_FLOAT32_C(   181.36), SIMDE_FLOAT32_C(   178.15),
        SIMDE_FLOAT32_C(  -534.33), SIMDE_FLOAT32_C(  -888.27), SIMDE_FLOAT32_C(  -513.52), SIMDE_FLOAT32_C(  -754.04),
        SIMDE_FLOAT32_C(  -831.91), SIMDE_FLOAT32_C(   808.71), SIMDE_FLOAT32_C(   488.15), SIMDE_FLOAT32_C(   811.21),
        SIMDE_FLOAT32_C(  -126.78), SIMDE_FLOAT32_C(   720.09), SIMDE_FLOAT32_C(   627.10), SIMDE_FLOAT32_C(   933.09) },
      { SIMDE_FLOAT32_C(   815.89), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(   671.25), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   921.38),
        SIMDE_FLOAT32_C(  -562.45), SIMDE_FLOAT32_C(  -641.27), SIMDE_FLOAT32_C(  -484.11), SIMDE_FLOAT32_C(  -776.21),
        SIMDE_FLOAT32_C(  -202.41), SIMDE_FLOAT32_C(  -922.83), SIMDE_FLOAT32_C(  -317.45), SIMDE_FLOAT32_C(  -793.22) } },
    { { SIMDE_FLOAT32_C(   208.40), SIMDE_FLOAT32_C(  -273.28), SIMDE_FLOAT32_C(   604.34), SIMDE_FLOAT32_C(  -282.99),
        SIMDE_FLOAT32_C(  -853.84), SIMDE_FLOAT32_C(   525.72), SIMDE_FLOAT32_C(   154.57), SIMDE_FLOAT32_C(  -495.10),
        SIMDE_FLOAT32_C(  -958.39), SIMDE_FLOAT32_C(   378.36), SIMDE_FLOAT32_C(   302.49), SIMDE_FLOAT32_C(  -881.22),
        SIMDE_FLOAT32_C(  -939.09), SIMDE_FLOAT32_C(   509.27), SIMDE_FLOAT32_C(  -296.70), SIMDE_FLOAT32_C(   801.40) },
      UINT8_C(108),
      { SIMDE_FLOAT32_C(   884.66), SIMDE_FLOAT32_C(   -20.45), SIMDE_FLOAT32_C(   -68.88), SIMDE_FLOAT32_C(   996.39),
        SIMDE_FLOAT32_C(   466.03), SIMDE_FLOAT32_C(   177.08), SIMDE_FLOAT32_C(  -835.52), SIMDE_FLOAT32_C(   274.74),
        SIMDE_FLOAT32_C(  -334.77), SIMDE_FLOAT32_C(   975.69), SIMDE_FLOAT32_C(  -852.04), SIMDE_FLOAT32_C(  -614.68),
        SIMDE_FLOAT32_C(   602.80), SIMDE_FLOAT32_C(  -918.95), SIMDE_FLOAT32_C(   593.73), SIMDE_FLOAT32_C(  -670.48) },
      { SIMDE_FLOAT32_C(   208.40), SIMDE_FLOAT32_C(  -273.28), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(  -853.84), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  -495.10),
        SIMDE_FLOAT32_C(  -958.39), SIMDE_FLOAT32_C(   378.36), SIMDE_FLOAT32_C(   302.49), SIMDE_FLOAT32_C(  -881.22),
        SIMDE_FLOAT32_C(  -939.09), SIMDE_FLOAT32_C(   509.27), SIMDE_FLOAT32_C(  -296.70), SIMDE_FLOAT32_C(   801.40) } },
    { { SIMDE_FLOAT32_C(   685.39), SIMDE_FLOAT32_C(  -689.26), SIMDE_FLOAT32_C(  -524.32), SIMDE_FLOAT32_C(   211.10),
        SIMDE_FLOAT32_C(   465.30), SIMDE_FLOAT32_C(   -19.43), SIMDE_FLOAT32_C(   252.72), SIMDE_FLOAT32_C(  -156.34),
        SIMDE_FLOAT32_C(  -716.94), SIMDE_FLOAT32_C(   371.50), SIMDE_FLOAT32_C(   -95.43), SIMDE_FLOAT32_C(   792.33),
        SIMDE_FLOAT32_C(  -925.20), SIMDE_FLOAT32_C(  -294.03), SIMDE_FLOAT32_C(  -742.21), SIMDE_FLOAT32_C(   959.46) },
      UINT8_C(216),
      { SIMDE_FLOAT32_C(   188.91), SIMDE_FLOAT32_C(   955.85), SIMDE_FLOAT32_C(   151.56), SIMDE_FLOAT32_C(  -634.01),
        SIMDE_FLOAT32_C(  -879.66), SIMDE_FLOAT32_C(  -573.70), SIMDE_FLOAT32_C(    31.23), SIMDE_FLOAT32_C(  -903.97),
        SIMDE_FLOAT32_C(  -425.74), SIMDE_FLOAT32_C(   416.55), SIMDE_FLOAT32_C(   698.83), SIMDE_FLOAT32_C(  -344.69),
        SIMDE_FLOAT32_C(    10.28), SIMDE_FLOAT32_C(  -971.65), SIMDE_FLOAT32_C(  -659.31), SIMDE_FLOAT32_C(   321.02) },
      { SIMDE_FLOAT32_C(   685.39), SIMDE_FLOAT32_C(  -689.26), SIMDE_FLOAT32_C(  -524.32), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   -19.43), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(  -716.94), SIMDE_FLOAT32_C(   371.50), SIMDE_FLOAT32_C(   -95.43), SIMDE_FLOAT32_C(   792.33),
        SIMDE_FLOAT32_C(  -925.20), SIMDE_FLOAT32_C(  -294.03), SIMDE_FLOAT32_C(  -742.21), SIMDE_FLOAT32_C(   959.46) } },
    { { SIMDE_FLOAT32_C(  -495.97), SIMDE_FLOAT32_C(   551.80), SIMDE_FLOAT32_C(  -213.68), SIMDE_FLOAT32_C(   484.60),
        SIMDE_FLOAT32_C(  -195.49), SIMDE_FLOAT32_C(   629.98), SIMDE_FLOAT32_C(   767.66), SIMDE_FLOAT32_C(  -823.99),
        SIMDE_FLOAT32_C(  -465.45), SIMDE_FLOAT32_C(   560.00), SIMDE_FLOAT32_C(  -749.18), SIMDE_FLOAT32_C(   240.52),
        SIMDE_FLOAT32_C(   817.78), SIMDE_FLOAT32_C(  -789.72), SIMDE_FLOAT32_C(   -73.95), SIMDE_FLOAT32_C(     6.69) },
      UINT8_C(202),
      { SIMDE_FLOAT32_C(  -922.39), SIMDE_FLOAT32_C(   372.68), SIMDE_FLOAT32_C(  -713.53), SIMDE_FLOAT32_C(  -496.09),
        SIMDE_FLOAT32_C(  -596.09), SIMDE_FLOAT32_C(  -617.49), SIMDE_FLOAT32_C(    78.17), SIMDE_FLOAT32_C(   820.46),
        SIMDE_FLOAT32_C(  -918.66), SIMDE_FLOAT32_C(   733.47), SIMDE_FLOAT32_C(  -169.26), SIMDE_FLOAT32_C(  -890.32),
        SIMDE_FLOAT32_C(  -925.83), SIMDE_FLOAT32_C(  -848.24), SIMDE_FLOAT32_C(  -386.29), SIMDE_FLOAT32_C(   625.96) },
      { SIMDE_FLOAT32_C(  -495.97), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(  -213.68), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(  -195.49), SIMDE_FLOAT32_C(   629.98), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(  -465.45), SIMDE_FLOAT32_C(   560.00), SIMDE_FLOAT32_C(  -749.18), SIMDE_FLOAT32_C(   240.52),
        SIMDE_FLOAT32_C(   817.78), SIMDE_FLOAT32_C(  -789.72), SIMDE_FLOAT32_C(   -73.95), SIMDE_FLOAT32_C(     6.69) } },
    { { SIMDE_FLOAT32_C(   -61.91), SIMDE_FLOAT32_C(  -901.69), SIMDE_FLOAT32_C(  -569.52), SIMDE_FLOAT32_C(  -431.93),
        SIMDE_FLOAT32_C(   865.97), SIMDE_FLOAT32_C(  -393.51), SIMDE_FLOAT32_C(   102.62), SIMDE_FLOAT32_C(   425.97),
        SIMDE_FLOAT32_C(  -142.69), SIMDE_FLOAT32_C(  -656.86), SIMDE_FLOAT32_C(   243.75), SIMDE_FLOAT32_C(    67.59),
        SIMDE_FLOAT32_C(   269.19), SIMDE_FLOAT32_C(  -749.56), SIMDE_FLOAT32_C(   233.72), SIMDE_FLOAT32_C(   346.79) },
      UINT8_C(117),
      { SIMDE_FLOAT32_C(   520.19), SIMDE_FLOAT32_C(   850.70), SIMDE_FLOAT32_C(  -972.96), SIMDE_FLOAT32_C(   902.70),
        SIMDE_FLOAT32_C(   -71.13), SIMDE_FLOAT32_C(   847.50), SIMDE_FLOAT32_C(   984.04), SIMDE_FLOAT32_C(  -337.66),
        SIMDE_FLOAT32_C(  -321.75), SIMDE_FLOAT32_C(  -906.28), SIMDE_FLOAT32_C(  -263.49), SIMDE_FLOAT32_C(  -169.99),
        SIMDE_FLOAT32_C(  -292.57), SIMDE_FLOAT32_C(  -637.53), SIMDE_FLOAT32_C(   768.10), SIMDE_FLOAT32_C(  -194.26) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(  -901.69), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  -431.93),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   425.97),
        SIMDE_FLOAT32_C(  -142.69), SIMDE_FLOAT32_C(  -656.86), SIMDE_FLOAT32_C(   243.75), SIMDE_FLOAT32_C(    67.59),
        SIMDE_FLOAT32_C(   269.19), SIMDE_FLOAT32_C(  -749.56), SIMDE_FLOAT32_C(   233.72), SIMDE_FLOAT32_C(   346.79) } },
    { { SIMDE_FLOAT32_C(  -207.05), SIMDE_FLOAT32_C(  -663.84), SIMDE_FLOAT32_C(  -328.29), SIMDE_FLOAT32_C(   399.44),
        SIMDE_FLOAT32_C(   438.78), SIMDE_FLOAT32_C(  -902.33), SIMDE_FLOAT32_C(  -743.25), SIMDE_FLOAT32_C(   781.93),
        SIMDE_FLOAT32_C(   341.42), SIMDE_FLOAT32_C(   324.33), SIMDE_FLOAT32_C(    51.11), SIMDE_FLOAT32_C(   591.87),
        SIMDE_FLOAT32_C(  -441.94), SIMDE_FLOAT32_C(  -602.09), SIMDE_FLOAT32_C(   214.99), SIMDE_FLOAT32_C(  -921.75) },
         UINT8_MAX,
      { SIMDE_FLOAT32_C(   242.04), SIMDE_FLOAT32_C(   980.95), SIMDE_FLOAT32_C(   177.48), SIMDE_FLOAT32_C(    89.54),
        SIMDE_FLOAT32_C(   964.99), SIMDE_FLOAT32_C(   839.82), SIMDE_FLOAT32_C(   767.79), SIMDE_FLOAT32_C(  -941.29),
        SIMDE_FLOAT32_C(  -423.68), SIMDE_FLOAT32_C(  -402.20), SIMDE_FLOAT32_C(  -233.86), SIMDE_FLOAT32_C(   -61.21),
        SIMDE_FLOAT32_C(  -634.11), SIMDE_FLOAT32_C(   571.87), SIMDE_FLOAT32_C(   731.74), SIMDE_FLOAT32_C(  -297.94) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(   341.42), SIMDE_FLOAT32_C(   324.33), SIMDE_FLOAT32_C(    51.11), SIMDE_FLOAT32_C(   591.87),
        SIMDE_FLOAT32_C(  -441.94), SIMDE_FLOAT32_C(  -602.09), SIMDE_FLOAT32_C(   214.99), SIMDE_FLOAT32_C(  -921.75) } },
    { { SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(   131.18), SIMDE_FLOAT32_C(  -859.16), SIMDE_FLOAT32_C(  -658.75),
        SIMDE_FLOAT32_C(   387.93), SIMDE_FLOAT32_C(   922.77), SIMDE_FLOAT32_C(   682.68), SIMDE_FLOAT32_C(  -287.73),
        SIMDE_FLOAT32_C(   -26.12), SIMDE_FLOAT32_C(   274.55), SIMDE_FLOAT32_C(   270.32), SIMDE_FLOAT32_C(   371.79),
        SIMDE_FLOAT32_C(  -510.46), SIMDE_FLOAT32_C(   348.57), SIMDE_FLOAT32_C(   620.40), SIMDE_FLOAT32_C(   731.58) },
      UINT8_C(111),
      { SIMDE_FLOAT32_C(  -202.12), SIMDE_FLOAT32_C(  -178.88), SIMDE_FLOAT32_C(   294.51), SIMDE_FLOAT32_C(  -362.30),
        SIMDE_FLOAT32_C(  -411.10), SIMDE_FLOAT32_C(   353.22), SIMDE_FLOAT32_C(   214.02), SIMDE_FLOAT32_C(   186.70),
        SIMDE_FLOAT32_C(  -880.64), SIMDE_FLOAT32_C(  -847.18), SIMDE_FLOAT32_C(   552.59), SIMDE_FLOAT32_C(   691.24),
        SIMDE_FLOAT32_C(   884.56), SIMDE_FLOAT32_C(  -745.35), SIMDE_FLOAT32_C(   934.82), SIMDE_FLOAT32_C(    15.74) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(   387.93), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(  -287.73),
        SIMDE_FLOAT32_C(   -26.12), SIMDE_FLOAT32_C(   274.55), SIMDE_FLOAT32_C(   270.32), SIMDE_FLOAT32_C(   371.79),
        SIMDE_FLOAT32_C(  -510.46), SIMDE_FLOAT32_C(   348.57), SIMDE_FLOAT32_C(   620.40), SIMDE_FLOAT32_C(   731.58) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_cdfnorm_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_cdfnorm_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   515.78), SIMDE_FLOAT64_C(  -190.13), SIMDE_FLOAT64_C(  -905.08), SIMDE_FLOAT64_C(   734.43),
        SIMDE_FLOAT64_C(  -737.45), SIMDE_FLOAT64_C(    98.47), SIMDE_FLOAT64_C(   -95.41), SIMDE_FLOAT64_C(  -675.32) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -274.83), SIMDE_FLOAT64_C(   838.86), SIMDE_FLOAT64_C(  -796.42), SIMDE_FLOAT64_C(   478.49),
        SIMDE_FLOAT64_C(   554.96), SIMDE_FLOAT64_C(  -640.77), SIMDE_FLOAT64_C(   -29.13), SIMDE_FLOAT64_C(   -94.09) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   398.68), SIMDE_FLOAT64_C(   316.09), SIMDE_FLOAT64_C(   332.14), SIMDE_FLOAT64_C(   590.41),
        SIMDE_FLOAT64_C(  -417.40), SIMDE_FLOAT64_C(  -789.19), SIMDE_FLOAT64_C(  -493.08), SIMDE_FLOAT64_C(   967.90) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -877.90), SIMDE_FLOAT64_C(    49.76), SIMDE_FLOAT64_C(   604.59), SIMDE_FLOAT64_C(  -550.52),
        SIMDE_FLOAT64_C(  -548.72), SIMDE_FLOAT64_C(   124.59), SIMDE_FLOAT64_C(   499.19), SIMDE_FLOAT64_C(   967.06) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(   934.46), SIMDE_FLOAT64_C(   594.11), SIMDE_FLOAT64_C(   701.49), SIMDE_FLOAT64_C(  -802.98),
        SIMDE_FLOAT64_C(  -307.42), SIMDE_FLOAT64_C(  -393.92), SIMDE_FLOAT64_C(  -478.30), SIMDE_FLOAT64_C(   417.75) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -555.06), SIMDE_FLOAT64_C(  -274.72), SIMDE_FLOAT64_C(  -103.76), SIMDE_FLOAT64_C(   999.90),
        SIMDE_FLOAT64_C(    84.51), SIMDE_FLOAT64_C(   867.11), SIMDE_FLOAT64_C(   -94.19), SIMDE_FLOAT64_C(  -516.80) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   183.20), SIMDE_FLOAT64_C(  -762.05), SIMDE_FLOAT64_C(  -926.39), SIMDE_FLOAT64_C(   765.80),
        SIMDE_FLOAT64_C(  -551.23), SIMDE_FLOAT64_C(  -419.47), SIMDE_FLOAT64_C(   733.70), SIMDE_FLOAT64_C(  -429.13) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   630.29), SIMDE_FLOAT64_C(   338.28), SIMDE_FLOAT64_C(    20.35), SIMDE_FLOAT64_C(  -918.43),
        SIMDE_FLOAT64_C(  -537.13), SIMDE_FLOAT64_C(  -480.46), SIMDE_FLOAT64_C(  -951.37), SIMDE_FLOAT64_C(  -602.66) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_cdfnorm_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cdfnorm_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -488.95), SIMDE_FLOAT64_C(   602.82), SIMDE_FLOAT64_C(   180.74), SIMDE_FLOAT64_C(  -325.95),
        SIMDE_FLOAT64_C(  -721.92), SIMDE_FLOAT64_C(   512.04), SIMDE_FLOAT64_C(   182.27), SIMDE_FLOAT64_C(  -392.39) },
      UINT8_C( 25),
      { SIMDE_FLOAT64_C(  -174.69), SIMDE_FLOAT64_C(   219.93), SIMDE_FLOAT64_C(   649.77), SIMDE_FLOAT64_C(  -892.75),
        SIMDE_FLOAT64_C(  -136.71), SIMDE_FLOAT64_C(  -906.14), SIMDE_FLOAT64_C(   643.57), SIMDE_FLOAT64_C(   669.62) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   602.82), SIMDE_FLOAT64_C(   180.74), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   512.04), SIMDE_FLOAT64_C(   182.27), SIMDE_FLOAT64_C(  -392.39) } },
    { { SIMDE_FLOAT64_C(  -655.46), SIMDE_FLOAT64_C(   837.15), SIMDE_FLOAT64_C(   772.04), SIMDE_FLOAT64_C(   272.82),
        SIMDE_FLOAT64_C(   490.61), SIMDE_FLOAT64_C(    38.88), SIMDE_FLOAT64_C(  -668.93), SIMDE_FLOAT64_C(  -501.66) },
      UINT8_C(232),
      { SIMDE_FLOAT64_C(  -130.58), SIMDE_FLOAT64_C(   219.17), SIMDE_FLOAT64_C(   309.61), SIMDE_FLOAT64_C(  -572.70),
        SIMDE_FLOAT64_C(   851.68), SIMDE_FLOAT64_C(   820.66), SIMDE_FLOAT64_C(  -969.88), SIMDE_FLOAT64_C(    32.42) },
      { SIMDE_FLOAT64_C(  -655.46), SIMDE_FLOAT64_C(   837.15), SIMDE_FLOAT64_C(   772.04), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(   490.61), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -505.29), SIMDE_FLOAT64_C(  -691.80), SIMDE_FLOAT64_C(  -455.53), SIMDE_FLOAT64_C(   676.98),
        SIMDE_FLOAT64_C(   -84.19), SIMDE_FLOAT64_C(  -340.34), SIMDE_FLOAT64_C(  -497.71), SIMDE_FLOAT64_C(  -864.27) },
      UINT8_C(183),
      { SIMDE_FLOAT64_C(  -390.46), SIMDE_FLOAT64_C(    -0.97), SIMDE_FLOAT64_C(  -596.71), SIMDE_FLOAT64_C(  -746.89),
        SIMDE_FLOAT64_C(  -331.35), SIMDE_FLOAT64_C(  -252.17), SIMDE_FLOAT64_C(  -909.75), SIMDE_FLOAT64_C(  -559.31) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   676.98),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -497.71), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -979.36), SIMDE_FLOAT64_C(   580.86), SIMDE_FLOAT64_C(   479.57), SIMDE_FLOAT64_C(  -648.29),
        SIMDE_FLOAT64_C(  -920.80), SIMDE_FLOAT64_C(   377.46), SIMDE_FLOAT64_C(   221.14), SIMDE_FLOAT64_C(   298.38) },
      UINT8_C(194),
      { SIMDE_FLOAT64_C(   648.44), SIMDE_FLOAT64_C(   150.06), SIMDE_FLOAT64_C(  -492.27), SIMDE_FLOAT64_C(   678.56),
        SIMDE_FLOAT64_C(  -817.52), SIMDE_FLOAT64_C(     2.44), SIMDE_FLOAT64_C(   986.76), SIMDE_FLOAT64_C(  -273.05) },
      { SIMDE_FLOAT64_C(  -979.36), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(   479.57), SIMDE_FLOAT64_C(  -648.29),
        SIMDE_FLOAT64_C(  -920.80), SIMDE_FLOAT64_C(   377.46), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -320.57), SIMDE_FLOAT64_C(   -97.43), SIMDE_FLOAT64_C(   386.61), SIMDE_FLOAT64_C(   181.71),
        SIMDE_FLOAT64_C(    38.30), SIMDE_FLOAT64_C(   696.05), SIMDE_FLOAT64_C(   791.25), SIMDE_FLOAT64_C(  -962.67) },
      UINT8_C(160),
      { SIMDE_FLOAT64_C(  -955.64), SIMDE_FLOAT64_C(  -294.02), SIMDE_FLOAT64_C(  -152.83), SIMDE_FLOAT64_C(  -865.39),
        SIMDE_FLOAT64_C(   146.67), SIMDE_FLOAT64_C(  -132.19), SIMDE_FLOAT64_C(   715.47), SIMDE_FLOAT64_C(  -373.76) },
      { SIMDE_FLOAT64_C(  -320.57), SIMDE_FLOAT64_C(   -97.43), SIMDE_FLOAT64_C(   386.61), SIMDE_FLOAT64_C(   181.71),
        SIMDE_FLOAT64_C(    38.30), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   791.25), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   219.52), SIMDE_FLOAT64_C(   794.68), SIMDE_FLOAT64_C(  -996.30), SIMDE_FLOAT64_C(  -559.34),
        SIMDE_FLOAT64_C(    93.05), SIMDE_FLOAT64_C(  -309.23), SIMDE_FLOAT64_C(  -910.90), SIMDE_FLOAT64_C(  -756.89) },
      UINT8_C( 25),
      { SIMDE_FLOAT64_C(   767.66), SIMDE_FLOAT64_C(  -574.40), SIMDE_FLOAT64_C(  -799.05), SIMDE_FLOAT64_C(   754.42),
        SIMDE_FLOAT64_C(   152.54), SIMDE_FLOAT64_C(  -119.63), SIMDE_FLOAT64_C(  -343.01), SIMDE_FLOAT64_C(  -460.84) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(   794.68), SIMDE_FLOAT64_C(  -996.30), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(  -309.23), SIMDE_FLOAT64_C(  -910.90), SIMDE_FLOAT64_C(  -756.89) } },
    { { SIMDE_FLOAT64_C(  -937.91), SIMDE_FLOAT64_C(   695.30), SIMDE_FLOAT64_C(  -764.79), SIMDE_FLOAT64_C(   853.34),
        SIMDE_FLOAT64_C(   732.63), SIMDE_FLOAT64_C(  -665.45), SIMDE_FLOAT64_C(   897.70), SIMDE_FLOAT64_C(  -561.39) },
      UINT8_C(185),
      { SIMDE_FLOAT64_C(  -967.69), SIMDE_FLOAT64_C(   585.27), SIMDE_FLOAT64_C(  -950.48), SIMDE_FLOAT64_C(   747.78),
        SIMDE_FLOAT64_C(  -788.49), SIMDE_FLOAT64_C(   269.05), SIMDE_FLOAT64_C(   542.46), SIMDE_FLOAT64_C(  -784.79) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   695.30), SIMDE_FLOAT64_C(  -764.79), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(   897.70), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   709.71), SIMDE_FLOAT64_C(  -364.49), SIMDE_FLOAT64_C(   -94.02), SIMDE_FLOAT64_C(   798.81),
        SIMDE_FLOAT64_C(  -121.37), SIMDE_FLOAT64_C(  -895.52), SIMDE_FLOAT64_C(   566.47), SIMDE_FLOAT64_C(   304.22) },
      UINT8_C(190),
      { SIMDE_FLOAT64_C(   320.89), SIMDE_FLOAT64_C(  -543.23), SIMDE_FLOAT64_C(   185.80), SIMDE_FLOAT64_C(   977.88),
        SIMDE_FLOAT64_C(    -4.07), SIMDE_FLOAT64_C(   247.88), SIMDE_FLOAT64_C(   673.18), SIMDE_FLOAT64_C(   231.13) },
      { SIMDE_FLOAT64_C(   709.71), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(   566.47), SIMDE_FLOAT64_C(     1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_cdfnorm_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_cdfnorminv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.90) },
      { SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     1.28) } },
    { { SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.57) },
      { SIMDE_FLOAT32_C(    -1.41), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.18) } },
    { { SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.19) },
      { SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.88) } },
    { { SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.53) },
      { SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(    -2.05), SIMDE_FLOAT32_C(     0.08) } },
    { { SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.99) },
      { SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     2.33) } },
    { { SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.81) },
      { SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     0.88) } },
    { { SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.92) },
      { SIMDE_FLOAT32_C(    -1.34), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     1.41) } },
    { { SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.84) },
      { SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.99) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_cdfnorminv_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_cdfnorminv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     0.77) },
      { SIMDE_FLOAT64_C(     0.77), SIMDE_FLOAT64_C(     0.74) } },
    { { SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     0.34) },
      { SIMDE_FLOAT64_C(     1.48), SIMDE_FLOAT64_C(    -0.41) } },
    { { SIMDE_FLOAT64_C(     0.02), SIMDE_FLOAT64_C(     0.32) },
      { SIMDE_FLOAT64_C(    -2.05), SIMDE_FLOAT64_C(    -0.47) } },
    { { SIMDE_FLOAT64_C(     0.60), SIMDE_FLOAT64_C(     0.80) },
      { SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.84) } },
    { { SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.03) },
      { SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(    -1.88) } },
    { { SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(     0.02) },
      { SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(    -2.05) } },
    { { SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(     0.81) },
      { SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     0.88) } },
    { { SIMDE_FLOAT64_C(     0.77), SIMDE_FLOAT64_C(     0.04) },
      { SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(    -1.75) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_cdfnorminv_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_cdfnorminv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.39),
        SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.19) },
      { SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.28),
        SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.88) } },
    { { SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     0.37),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.16) },
      { SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(    -0.33),
        SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.99) } },
    { { SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.74),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.62) },
      { SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     0.64),
        SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(     0.31) } },
    { { SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.97),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.22) },
      { SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -1.41), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     1.88),
        SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.77) } },
    { { SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.60),
        SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.55) },
      { SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -1.48), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.25),
        SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.13) } },
    { { SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.32) },
      { SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(    -1.88), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(    -0.41),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.47) } },
    { { SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.89),
        SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.75) },
      { SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(     1.23),
        SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(    -1.75), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.67) } },
    { { SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.85) },
      { SIMDE_FLOAT32_C(    -1.17), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.71),
        SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(     1.04) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    // simde__m256 b = simde_mm256_loadu_ps(test_vec[i].b);
    simde__m256 r = simde_mm256_cdfnorminv_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_cdfnorminv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(     0.82), SIMDE_FLOAT64_C(     0.90) },
      { SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     1.28) } },
    { { SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(     0.69), SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(     0.88) },
      { SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     0.50), SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     1.17) } },
    { { SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.40), SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(     0.78) },
      { SIMDE_FLOAT64_C(    -0.74), SIMDE_FLOAT64_C(    -0.25), SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(     0.77) } },
    { { SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.63), SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.44) },
      { SIMDE_FLOAT64_C(    -0.67), SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(     0.71), SIMDE_FLOAT64_C(    -0.15) } },
    { { SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     0.53) },
      { SIMDE_FLOAT64_C(    -0.23), SIMDE_FLOAT64_C(    -0.92), SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(     0.08) } },
    { { SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     0.63) },
      { SIMDE_FLOAT64_C(    -0.36), SIMDE_FLOAT64_C(    -0.23), SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(     0.33) } },
    { { SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(     0.94), SIMDE_FLOAT64_C(     0.41) },
      { SIMDE_FLOAT64_C(    -0.52), SIMDE_FLOAT64_C(     1.34), SIMDE_FLOAT64_C(     1.55), SIMDE_FLOAT64_C(    -0.23) } },
    { { SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     0.82), SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     0.67) },
      { SIMDE_FLOAT64_C(    -1.34), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.44) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_cdfnorminv_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_cdfnorminv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.80),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.99),
        SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.81) },
      { SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.41),
        SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     0.84),
        SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -1.13), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     2.33),
        SIMDE_FLOAT32_C(    -1.64), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     0.88) } },
    { { SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.13),
        SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.21),
        SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.91),
        SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.08) },
      { SIMDE_FLOAT32_C(    -1.28), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -1.13),
        SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(    -0.81),
        SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(     1.34),
        SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -1.41) } },
    { { SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.21),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.19),
        SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.56),
        SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.01) },
      { SIMDE_FLOAT32_C(     1.13), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.81),
        SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -2.33), SIMDE_FLOAT32_C(    -0.88),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(     0.15),
        SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(    -1.88), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -2.33) } },
    { { SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.91),
        SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.15),
        SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.09),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.34) },
      { SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -1.13), SIMDE_FLOAT32_C(     1.34),
        SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    -1.75), SIMDE_FLOAT32_C(    -1.17), SIMDE_FLOAT32_C(    -1.04),
        SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(    -1.34),
        SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(    -0.41) } },
    { { SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.98),
        SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.47),
        SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.04) },
      { SIMDE_FLOAT32_C(    -2.33), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     2.05),
        SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.08),
        SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -1.88), SIMDE_FLOAT32_C(    -0.99),
        SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(    -1.75) } },
    { { SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.71),
        SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.66),
        SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.07),
        SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.22) },
      { SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     0.55),
        SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -1.48), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     0.41),
        SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     1.41), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(    -1.48),
        SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(    -0.77) } },
    { { SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.46),
        SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.60) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.10),
        SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     0.74),
        SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(    -1.55),
        SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -1.28), SIMDE_FLOAT32_C(    -1.23), SIMDE_FLOAT32_C(     0.25) } },
    { { SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.75),
        SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.74),
        SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.28),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.56) },
      { SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.67),
        SIMDE_FLOAT32_C(    -1.13), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.64),
        SIMDE_FLOAT32_C(    -1.41), SIMDE_FLOAT32_C(    -1.04), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -0.58),
        SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.15) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_cdfnorminv_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cdfnorminv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.18),
        SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.74),
        SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.38),
        SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.03) },
      UINT8_C(249),
      { SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.22),
        SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.21),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.63),
        SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.81) },
      { SIMDE_FLOAT32_C(    -1.08), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(    -0.77),
        SIMDE_FLOAT32_C(    -2.33), SIMDE_FLOAT32_C(    -1.88), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.81),
        SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.38),
        SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.03) } },
    { { SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.20),
        SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.65) },
      UINT8_C(209),
      { SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.66),
        SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.40),
        SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.00) },
      { SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.64),
        SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.65) } },
    { { SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.92),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.65),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.78),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.10) },
      UINT8_C(123),
      { SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.32),
        SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     0.64),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.94),
        SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.10) },
      { SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.47),
        SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     0.65),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.78),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.10) } },
    { { SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.33),
        SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.73),
        SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.67),
        SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.95) },
      UINT8_C( 43),
      { SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.50),
        SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.19),
        SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.44),
        SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.36) },
      { SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -1.41), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.73),
        SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.67),
        SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.95) } },
    { { SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.52),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.29),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.94),
        SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.70) },
      UINT8_C( 66),
      { SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.51),
        SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.46),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.72) },
      { SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.52),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.29),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.94),
        SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.70) } },
    { { SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.01),
        SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.02) },
      UINT8_C(157),
      { SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.99),
        SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.50),
        SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.71),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.39) },
      { SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     1.41), SIMDE_FLOAT32_C(     2.33),
        SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.02) } },
    { { SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.19),
        SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.52),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.89),
        SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.95) },
      UINT8_C( 65),
      { SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.27),
        SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.36),
        SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.41) },
      { SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.19),
        SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    -1.08), SIMDE_FLOAT32_C(     0.52),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.89),
        SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.95) } },
    { { SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.09),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.52) },
      UINT8_C(240),
      { SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.12),
        SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.78),
        SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.33),
        SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.26) },
      { SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     1.41), SIMDE_FLOAT32_C(    -1.08), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.09),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.52) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_cdfnorminv_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_cdfnorminv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(     0.38), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     0.48),
        SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     0.88), SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(     0.89) },
      { SIMDE_FLOAT64_C(    -0.44), SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(     0.77), SIMDE_FLOAT64_C(    -0.05),
        SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(     1.17), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     1.23) } },
    { { SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.87),
        SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.51) },
      { SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(    -0.58), SIMDE_FLOAT64_C(     0.88), SIMDE_FLOAT64_C(     1.13),
        SIMDE_FLOAT64_C(    -0.44), SIMDE_FLOAT64_C(     2.05), SIMDE_FLOAT64_C(    -0.41), SIMDE_FLOAT64_C(     0.03) } },
    { { SIMDE_FLOAT64_C(     0.85), SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(     0.79), SIMDE_FLOAT64_C(     0.79),
        SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(     0.73), SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(     0.96) },
      { SIMDE_FLOAT64_C(     1.04), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.81),
        SIMDE_FLOAT64_C(    -0.58), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(     1.64), SIMDE_FLOAT64_C(     1.75) } },
    { { SIMDE_FLOAT64_C(     0.86), SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.84),
        SIMDE_FLOAT64_C(     0.62), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.60), SIMDE_FLOAT64_C(     0.95) },
      { SIMDE_FLOAT64_C(     1.08), SIMDE_FLOAT64_C(    -0.08), SIMDE_FLOAT64_C(    -1.88), SIMDE_FLOAT64_C(     0.99),
        SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(    -1.75), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     1.64) } },
    { { SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(     0.38), SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(     0.82),
        SIMDE_FLOAT64_C(     0.26), SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(     0.71), SIMDE_FLOAT64_C(     0.83) },
      { SIMDE_FLOAT64_C(    -0.20), SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(    -0.15), SIMDE_FLOAT64_C(     0.92),
        SIMDE_FLOAT64_C(    -0.64), SIMDE_FLOAT64_C(    -0.58), SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.95) } },
    { { SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     0.70), SIMDE_FLOAT64_C(     0.89),
        SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     0.35) },
      { SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     1.23),
        SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(    -1.75), SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(    -0.39) } },
    { { SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.84),
        SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.79), SIMDE_FLOAT64_C(     0.77) },
      { SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(    -0.92), SIMDE_FLOAT64_C(    -1.04), SIMDE_FLOAT64_C(     0.99),
        SIMDE_FLOAT64_C(     1.34), SIMDE_FLOAT64_C(    -1.28), SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.74) } },
    { { SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(     0.82), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(     0.20),
        SIMDE_FLOAT64_C(     0.86), SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.28) },
      { SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(    -0.84),
        SIMDE_FLOAT64_C(     1.08), SIMDE_FLOAT64_C(    -0.81), SIMDE_FLOAT64_C(    -1.04), SIMDE_FLOAT64_C(    -0.58) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_cdfnorminv_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cdfnorminv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.13),
        SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     0.19) },
      UINT8_C( 53),
      { SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     0.21),
        SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     0.26), SIMDE_FLOAT64_C(     0.24) },
      { SIMDE_FLOAT64_C(    -0.58), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(    -0.36), SIMDE_FLOAT64_C(     0.13),
        SIMDE_FLOAT64_C(    -0.92), SIMDE_FLOAT64_C(    -1.23), SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     0.19) } },
    { { SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.49), SIMDE_FLOAT64_C(     0.16),
        SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     0.49), SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(     0.47) },
      UINT8_C( 92),
      { SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(     0.47),
        SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(     0.46) },
      { SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(    -0.81), SIMDE_FLOAT64_C(    -0.08),
        SIMDE_FLOAT64_C(    -0.18), SIMDE_FLOAT64_C(     0.49), SIMDE_FLOAT64_C(    -0.88), SIMDE_FLOAT64_C(     0.47) } },
    { { SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     0.14),
        SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(     0.18) },
      UINT8_C(232),
      { SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.26), SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(     0.19),
        SIMDE_FLOAT64_C(     0.40), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.39) },
      { SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -0.88),
        SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(    -0.41), SIMDE_FLOAT64_C(    -0.28) } },
    { { SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(     0.38), SIMDE_FLOAT64_C(     0.17),
        SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(     0.25) },
      UINT8_C(135),
      { SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     0.39),
        SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.35), SIMDE_FLOAT64_C(     0.15) },
      { SIMDE_FLOAT64_C(    -0.52), SIMDE_FLOAT64_C(    -0.92), SIMDE_FLOAT64_C(    -0.13), SIMDE_FLOAT64_C(     0.17),
        SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(    -1.04) } },
    { { SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(     0.40), SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     0.49),
        SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(     0.25) },
      UINT8_C(111),
      { SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(     0.34),
        SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(     0.31) },
      { SIMDE_FLOAT64_C(    -1.28), SIMDE_FLOAT64_C(    -1.04), SIMDE_FLOAT64_C(    -0.23), SIMDE_FLOAT64_C(    -0.41),
        SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(    -0.15), SIMDE_FLOAT64_C(     0.25) } },
    { { SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(     0.27),
        SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     0.34) },
      UINT8_C( 67),
      { SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     0.38), SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(     0.22),
        SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(     0.29) },
      { SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(     0.27),
        SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(    -1.17), SIMDE_FLOAT64_C(     0.34) } },
    { { SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.37),
        SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.38), SIMDE_FLOAT64_C(     0.30) },
      UINT8_C(205),
      { SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.16),
        SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.36) },
      { SIMDE_FLOAT64_C(    -0.77), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -0.18), SIMDE_FLOAT64_C(    -0.99),
        SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(    -0.92), SIMDE_FLOAT64_C(    -0.36) } },
    { { SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(     0.11),
        SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.16), SIMDE_FLOAT64_C(     0.38), SIMDE_FLOAT64_C(     0.46) },
      UINT8_C( 64),
      { SIMDE_FLOAT64_C(     0.50), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.12),
        SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(     0.31) },
      { SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(     0.11),
        SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.16), SIMDE_FLOAT64_C(    -1.13), SIMDE_FLOAT64_C(     0.46) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_cdfnorminv_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_cexp_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     8.03), SIMDE_FLOAT32_C(     6.08), SIMDE_FLOAT32_C(     9.10) },
      { SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     2.90), SIMDE_FLOAT32_C(  -414.18), SIMDE_FLOAT32_C(   139.46) } },
    { { SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(     5.24), SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     4.31) },
      { SIMDE_FLOAT32_C(     1.81), SIMDE_FLOAT32_C(    -3.11), SIMDE_FLOAT32_C(   -14.33), SIMDE_FLOAT32_C(   -33.68) } },
    { { SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(     6.66), SIMDE_FLOAT32_C(     7.44) },
      { SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     1.15), SIMDE_FLOAT32_C(   313.98), SIMDE_FLOAT32_C(   714.61) } },
    { { SIMDE_FLOAT32_C(     5.32), SIMDE_FLOAT32_C(     1.63), SIMDE_FLOAT32_C(     3.75), SIMDE_FLOAT32_C(     1.94) },
      { SIMDE_FLOAT32_C(   -12.09), SIMDE_FLOAT32_C(   204.03), SIMDE_FLOAT32_C(   -15.34), SIMDE_FLOAT32_C(    39.66) } },
    { { SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     4.84), SIMDE_FLOAT32_C(     7.08), SIMDE_FLOAT32_C(     8.24) },
      { SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -2.54), SIMDE_FLOAT32_C(  -447.27), SIMDE_FLOAT32_C(  1100.55) } },
    { { SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     7.57), SIMDE_FLOAT32_C(     8.46), SIMDE_FLOAT32_C(     6.20) },
      { SIMDE_FLOAT32_C(     6.04), SIMDE_FLOAT32_C(    20.68), SIMDE_FLOAT32_C(  4705.73), SIMDE_FLOAT32_C(  -392.35) } },
    { { SIMDE_FLOAT32_C(     6.65), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     5.84), SIMDE_FLOAT32_C(     9.64) },
      { SIMDE_FLOAT32_C(   596.01), SIMDE_FLOAT32_C(   491.91), SIMDE_FLOAT32_C(  -335.85), SIMDE_FLOAT32_C(   -73.42) } },
    { { SIMDE_FLOAT32_C(     5.18), SIMDE_FLOAT32_C(     4.56), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     6.26) },
      { SIMDE_FLOAT32_C(   -26.97), SIMDE_FLOAT32_C(  -175.62), SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(    -0.07) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_cexp_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_cexp_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     5.22), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     9.44),
        SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     9.55), SIMDE_FLOAT32_C(     7.98) },
      { SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(    -1.33), SIMDE_FLOAT32_C(    -3.86), SIMDE_FLOAT32_C(    -0.06),
        SIMDE_FLOAT32_C(    -1.81), SIMDE_FLOAT32_C(     4.05), SIMDE_FLOAT32_C( -1765.21), SIMDE_FLOAT32_C( 13933.33) } },
    { { SIMDE_FLOAT32_C(     9.68), SIMDE_FLOAT32_C(     3.77), SIMDE_FLOAT32_C(     6.40), SIMDE_FLOAT32_C(     1.44),
        SIMDE_FLOAT32_C(     7.91), SIMDE_FLOAT32_C(     7.80), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     3.48) },
      { SIMDE_FLOAT32_C(-12938.99), SIMDE_FLOAT32_C( -9402.48), SIMDE_FLOAT32_C(    78.49), SIMDE_FLOAT32_C(   596.70),
        SIMDE_FLOAT32_C(   147.00), SIMDE_FLOAT32_C(  2720.42), SIMDE_FLOAT32_C(    -3.20), SIMDE_FLOAT32_C(    -1.12) } },
    { { SIMDE_FLOAT32_C(     2.89), SIMDE_FLOAT32_C(     8.55), SIMDE_FLOAT32_C(     4.24), SIMDE_FLOAT32_C(     4.12),
        SIMDE_FLOAT32_C(     7.15), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     6.80), SIMDE_FLOAT32_C(     3.92) },
      { SIMDE_FLOAT32_C(   -11.54), SIMDE_FLOAT32_C(    13.81), SIMDE_FLOAT32_C(   -38.75), SIMDE_FLOAT32_C(   -57.58),
        SIMDE_FLOAT32_C(   761.70), SIMDE_FLOAT32_C(  1021.35), SIMDE_FLOAT32_C(  -639.30), SIMDE_FLOAT32_C(  -630.42) } },
    { { SIMDE_FLOAT32_C(     4.44), SIMDE_FLOAT32_C(     7.17), SIMDE_FLOAT32_C(     7.74), SIMDE_FLOAT32_C(     2.32),
        SIMDE_FLOAT32_C(     3.91), SIMDE_FLOAT32_C(     7.33), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     4.33) },
      { SIMDE_FLOAT32_C(    53.57), SIMDE_FLOAT32_C(    65.71), SIMDE_FLOAT32_C( -1565.39), SIMDE_FLOAT32_C(  1683.01),
        SIMDE_FLOAT32_C(    24.97), SIMDE_FLOAT32_C(    43.20), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(    -1.12) } },
    { { SIMDE_FLOAT32_C(     2.55), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     3.76), SIMDE_FLOAT32_C(     4.04),
        SIMDE_FLOAT32_C(     3.54), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     3.21) },
      { SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(    12.80), SIMDE_FLOAT32_C(   -26.75), SIMDE_FLOAT32_C(   -33.60),
        SIMDE_FLOAT32_C(   -33.92), SIMDE_FLOAT32_C(    -6.12), SIMDE_FLOAT32_C(    -7.52), SIMDE_FLOAT32_C(    -0.52) } },
    { { SIMDE_FLOAT32_C(     7.08), SIMDE_FLOAT32_C(     8.42), SIMDE_FLOAT32_C(     4.66), SIMDE_FLOAT32_C(     4.99),
        SIMDE_FLOAT32_C(     6.22), SIMDE_FLOAT32_C(     5.87), SIMDE_FLOAT32_C(     8.47), SIMDE_FLOAT32_C(     9.11) },
      { SIMDE_FLOAT32_C(  -637.08), SIMDE_FLOAT32_C(  1002.70), SIMDE_FLOAT32_C(    28.95), SIMDE_FLOAT32_C(  -101.59),
        SIMDE_FLOAT32_C(   460.40), SIMDE_FLOAT32_C(  -201.85), SIMDE_FLOAT32_C( -4535.17), SIMDE_FLOAT32_C(  1476.67) } },
    { { SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     2.71), SIMDE_FLOAT32_C(     3.23), SIMDE_FLOAT32_C(     1.57),
        SIMDE_FLOAT32_C(     3.64), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     5.49), SIMDE_FLOAT32_C(     8.08) },
      { SIMDE_FLOAT32_C(   -75.48), SIMDE_FLOAT32_C(    34.76), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(    25.28),
        SIMDE_FLOAT32_C(    38.07), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(   -54.29), SIMDE_FLOAT32_C(   236.10) } },
    { { SIMDE_FLOAT32_C(     7.19), SIMDE_FLOAT32_C(     3.23), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     1.10),
        SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     5.43), SIMDE_FLOAT32_C(     3.11) },
      { SIMDE_FLOAT32_C( -1320.92), SIMDE_FLOAT32_C(  -117.08), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     1.33),
        SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(  -228.04), SIMDE_FLOAT32_C(     7.21) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_cexp_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_clog_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   467.27), SIMDE_FLOAT32_C(  -810.49), SIMDE_FLOAT32_C(  -408.53), SIMDE_FLOAT32_C(  -463.46) },
      { SIMDE_FLOAT32_C(     6.84), SIMDE_FLOAT32_C(    -1.05), SIMDE_FLOAT32_C(     6.43), SIMDE_FLOAT32_C(    -2.29) } },
    { { SIMDE_FLOAT32_C(  -597.00), SIMDE_FLOAT32_C(   144.37), SIMDE_FLOAT32_C(   819.91), SIMDE_FLOAT32_C(   258.51) },
      { SIMDE_FLOAT32_C(     6.42), SIMDE_FLOAT32_C(     2.90), SIMDE_FLOAT32_C(     6.76), SIMDE_FLOAT32_C(     0.31) } },
    { { SIMDE_FLOAT32_C(  -690.61), SIMDE_FLOAT32_C(  -496.03), SIMDE_FLOAT32_C(  -379.26), SIMDE_FLOAT32_C(   822.50) },
      { SIMDE_FLOAT32_C(     6.75), SIMDE_FLOAT32_C(    -2.52), SIMDE_FLOAT32_C(     6.81), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(   369.47), SIMDE_FLOAT32_C(   917.67), SIMDE_FLOAT32_C(   917.67), SIMDE_FLOAT32_C(   649.13) },
      { SIMDE_FLOAT32_C(     6.90), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     7.02), SIMDE_FLOAT32_C(     0.62) } },
    { { SIMDE_FLOAT32_C(  -165.00), SIMDE_FLOAT32_C(   -18.10), SIMDE_FLOAT32_C(   943.19), SIMDE_FLOAT32_C(   635.72) },
      { SIMDE_FLOAT32_C(     5.11), SIMDE_FLOAT32_C(    -3.03), SIMDE_FLOAT32_C(     7.04), SIMDE_FLOAT32_C(     0.59) } },
    { { SIMDE_FLOAT32_C(   -21.66), SIMDE_FLOAT32_C(   494.23), SIMDE_FLOAT32_C(  -734.58), SIMDE_FLOAT32_C(   417.20) },
      { SIMDE_FLOAT32_C(     6.20), SIMDE_FLOAT32_C(     1.61), SIMDE_FLOAT32_C(     6.74), SIMDE_FLOAT32_C(     2.63) } },
    { { SIMDE_FLOAT32_C(   812.64), SIMDE_FLOAT32_C(  -983.61), SIMDE_FLOAT32_C(    15.40), SIMDE_FLOAT32_C(   505.51) },
      { SIMDE_FLOAT32_C(     7.15), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     6.23), SIMDE_FLOAT32_C(     1.54) } },
    { { SIMDE_FLOAT32_C(  -497.22), SIMDE_FLOAT32_C(   590.38), SIMDE_FLOAT32_C(   600.11), SIMDE_FLOAT32_C(   970.05) },
      { SIMDE_FLOAT32_C(     6.65), SIMDE_FLOAT32_C(     2.27), SIMDE_FLOAT32_C(     7.04), SIMDE_FLOAT32_C(     1.02) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_clog_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_clog_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   312.27), SIMDE_FLOAT32_C(   505.55), SIMDE_FLOAT32_C(   862.46), SIMDE_FLOAT32_C(    31.99),
        SIMDE_FLOAT32_C(   800.53), SIMDE_FLOAT32_C(   181.00), SIMDE_FLOAT32_C(   161.95), SIMDE_FLOAT32_C(   -71.19) },
      { SIMDE_FLOAT32_C(     6.39), SIMDE_FLOAT32_C(     1.02), SIMDE_FLOAT32_C(     6.76), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     6.71), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     5.18), SIMDE_FLOAT32_C(    -0.41) } },
    { { SIMDE_FLOAT32_C(   183.06), SIMDE_FLOAT32_C(   131.57), SIMDE_FLOAT32_C(   568.96), SIMDE_FLOAT32_C(   107.92),
        SIMDE_FLOAT32_C(   898.15), SIMDE_FLOAT32_C(   154.17), SIMDE_FLOAT32_C(   262.39), SIMDE_FLOAT32_C(   850.07) },
      { SIMDE_FLOAT32_C(     5.42), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     6.36), SIMDE_FLOAT32_C(     0.19),
        SIMDE_FLOAT32_C(     6.81), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     6.79), SIMDE_FLOAT32_C(     1.27) } },
    { { SIMDE_FLOAT32_C(   459.40), SIMDE_FLOAT32_C(   479.25), SIMDE_FLOAT32_C(   503.31), SIMDE_FLOAT32_C(   451.65),
        SIMDE_FLOAT32_C(   353.11), SIMDE_FLOAT32_C(   438.44), SIMDE_FLOAT32_C(   777.37), SIMDE_FLOAT32_C(    20.59) },
      { SIMDE_FLOAT32_C(     6.50), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     6.52), SIMDE_FLOAT32_C(     0.73),
        SIMDE_FLOAT32_C(     6.33), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     6.66), SIMDE_FLOAT32_C(     0.03) } },
    { { SIMDE_FLOAT32_C(   -35.16), SIMDE_FLOAT32_C(   449.22), SIMDE_FLOAT32_C(   -48.41), SIMDE_FLOAT32_C(   925.44),
        SIMDE_FLOAT32_C(   309.83), SIMDE_FLOAT32_C(   130.15), SIMDE_FLOAT32_C(    38.89), SIMDE_FLOAT32_C(   722.10) },
      { SIMDE_FLOAT32_C(     6.11), SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(     6.83), SIMDE_FLOAT32_C(     1.62),
        SIMDE_FLOAT32_C(     5.82), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     6.58), SIMDE_FLOAT32_C(     1.52) } },
    { { SIMDE_FLOAT32_C(   735.70), SIMDE_FLOAT32_C(   -98.65), SIMDE_FLOAT32_C(   854.09), SIMDE_FLOAT32_C(   536.23),
        SIMDE_FLOAT32_C(   182.34), SIMDE_FLOAT32_C(    16.04), SIMDE_FLOAT32_C(   565.04), SIMDE_FLOAT32_C(   465.40) },
      { SIMDE_FLOAT32_C(     6.61), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     0.56),
        SIMDE_FLOAT32_C(     5.21), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     6.60), SIMDE_FLOAT32_C(     0.69) } },
    { { SIMDE_FLOAT32_C(   247.61), SIMDE_FLOAT32_C(   134.00), SIMDE_FLOAT32_C(   673.33), SIMDE_FLOAT32_C(   145.76),
        SIMDE_FLOAT32_C(   388.17), SIMDE_FLOAT32_C(   -64.29), SIMDE_FLOAT32_C(    -4.17), SIMDE_FLOAT32_C(   947.57) },
      { SIMDE_FLOAT32_C(     5.64), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(     0.21),
        SIMDE_FLOAT32_C(     5.97), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     1.58) } },
    { { SIMDE_FLOAT32_C(   514.96), SIMDE_FLOAT32_C(   599.14), SIMDE_FLOAT32_C(   399.22), SIMDE_FLOAT32_C(   968.07),
        SIMDE_FLOAT32_C(    37.59), SIMDE_FLOAT32_C(   176.60), SIMDE_FLOAT32_C(   -11.35), SIMDE_FLOAT32_C(   102.43) },
      { SIMDE_FLOAT32_C(     6.67), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     6.95), SIMDE_FLOAT32_C(     1.18),
        SIMDE_FLOAT32_C(     5.20), SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     4.64), SIMDE_FLOAT32_C(     1.68) } },
    { { SIMDE_FLOAT32_C(   725.82), SIMDE_FLOAT32_C(    40.24), SIMDE_FLOAT32_C(    27.87), SIMDE_FLOAT32_C(    35.65),
        SIMDE_FLOAT32_C(   270.39), SIMDE_FLOAT32_C(   166.76), SIMDE_FLOAT32_C(   857.75), SIMDE_FLOAT32_C(     6.09) },
      { SIMDE_FLOAT32_C(     6.59), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     0.91),
        SIMDE_FLOAT32_C(     5.76), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     6.75), SIMDE_FLOAT32_C(     0.01) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_clog_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_csqrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   467.84), SIMDE_FLOAT32_C(   803.96), SIMDE_FLOAT32_C(   261.38), SIMDE_FLOAT32_C(  -142.34) },
      { SIMDE_FLOAT32_C(    26.44), SIMDE_FLOAT32_C(    15.20), SIMDE_FLOAT32_C(    16.72), SIMDE_FLOAT32_C(    -4.26) } },
    { { SIMDE_FLOAT32_C(   742.87), SIMDE_FLOAT32_C(    79.67), SIMDE_FLOAT32_C(   840.90), SIMDE_FLOAT32_C(  -323.18) },
      { SIMDE_FLOAT32_C(    27.29), SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(    29.51), SIMDE_FLOAT32_C(    -5.48) } },
    { { SIMDE_FLOAT32_C(  -240.48), SIMDE_FLOAT32_C(  -541.73), SIMDE_FLOAT32_C(   989.55), SIMDE_FLOAT32_C(   570.06) },
      { SIMDE_FLOAT32_C(    13.27), SIMDE_FLOAT32_C(   -20.41), SIMDE_FLOAT32_C(    32.65), SIMDE_FLOAT32_C(     8.73) } },
    { { SIMDE_FLOAT32_C(    83.09), SIMDE_FLOAT32_C(    -1.32), SIMDE_FLOAT32_C(   106.90), SIMDE_FLOAT32_C(  -376.28) },
      { SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(    15.78), SIMDE_FLOAT32_C(   -11.92) } },
    { { SIMDE_FLOAT32_C(  -403.08), SIMDE_FLOAT32_C(   970.42), SIMDE_FLOAT32_C(  -962.81), SIMDE_FLOAT32_C(   736.64) },
      { SIMDE_FLOAT32_C(    18.00), SIMDE_FLOAT32_C(    26.96), SIMDE_FLOAT32_C(    11.17), SIMDE_FLOAT32_C(    32.98) } },
    { { SIMDE_FLOAT32_C(   711.24), SIMDE_FLOAT32_C(  -757.45), SIMDE_FLOAT32_C(   634.59), SIMDE_FLOAT32_C(   -16.19) },
      { SIMDE_FLOAT32_C(    29.58), SIMDE_FLOAT32_C(   -12.80), SIMDE_FLOAT32_C(    25.19), SIMDE_FLOAT32_C(    -0.32) } },
    { { SIMDE_FLOAT32_C(    81.29), SIMDE_FLOAT32_C(  -815.58), SIMDE_FLOAT32_C(  -317.77), SIMDE_FLOAT32_C(   -90.40) },
      { SIMDE_FLOAT32_C(    21.22), SIMDE_FLOAT32_C(   -19.21), SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(   -18.00) } },
    { { SIMDE_FLOAT32_C(   -84.58), SIMDE_FLOAT32_C(   322.77), SIMDE_FLOAT32_C(   454.95), SIMDE_FLOAT32_C(  -616.74) },
      { SIMDE_FLOAT32_C(    11.16), SIMDE_FLOAT32_C(    14.46), SIMDE_FLOAT32_C(    24.71), SIMDE_FLOAT32_C(   -12.48) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_csqrt_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_csqrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   374.45), SIMDE_FLOAT32_C(  -986.58), SIMDE_FLOAT32_C(   -44.44), SIMDE_FLOAT32_C(  -160.79),
        SIMDE_FLOAT32_C(  -660.98), SIMDE_FLOAT32_C(  -996.70), SIMDE_FLOAT32_C(   -22.70), SIMDE_FLOAT32_C(   -74.73) },
      { SIMDE_FLOAT32_C(    26.74), SIMDE_FLOAT32_C(   -18.45), SIMDE_FLOAT32_C(     7.82), SIMDE_FLOAT32_C(   -10.28),
        SIMDE_FLOAT32_C(    16.36), SIMDE_FLOAT32_C(   -30.47), SIMDE_FLOAT32_C(     5.26), SIMDE_FLOAT32_C(    -7.10) } },
    { { SIMDE_FLOAT32_C(  -335.08), SIMDE_FLOAT32_C(  -387.45), SIMDE_FLOAT32_C(   992.50), SIMDE_FLOAT32_C(   334.99),
        SIMDE_FLOAT32_C(  -373.08), SIMDE_FLOAT32_C(  -939.30), SIMDE_FLOAT32_C(   219.57), SIMDE_FLOAT32_C(  -565.96) },
      { SIMDE_FLOAT32_C(     9.41), SIMDE_FLOAT32_C(   -20.58), SIMDE_FLOAT32_C(    31.94), SIMDE_FLOAT32_C(     5.24),
        SIMDE_FLOAT32_C(    17.85), SIMDE_FLOAT32_C(   -26.30), SIMDE_FLOAT32_C(    20.33), SIMDE_FLOAT32_C(   -13.92) } },
    { { SIMDE_FLOAT32_C(   626.25), SIMDE_FLOAT32_C(  -390.81), SIMDE_FLOAT32_C(   653.44), SIMDE_FLOAT32_C(   423.64),
        SIMDE_FLOAT32_C(   320.72), SIMDE_FLOAT32_C(   749.19), SIMDE_FLOAT32_C(  -605.94), SIMDE_FLOAT32_C(   183.09) },
      { SIMDE_FLOAT32_C(    26.12), SIMDE_FLOAT32_C(    -7.48), SIMDE_FLOAT32_C(    26.76), SIMDE_FLOAT32_C(     7.92),
        SIMDE_FLOAT32_C(    23.83), SIMDE_FLOAT32_C(    15.72), SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(    24.89) } },
    { { SIMDE_FLOAT32_C(   911.79), SIMDE_FLOAT32_C(   134.97), SIMDE_FLOAT32_C(  -550.62), SIMDE_FLOAT32_C(  -842.16),
        SIMDE_FLOAT32_C(   650.87), SIMDE_FLOAT32_C(  -128.95), SIMDE_FLOAT32_C(   295.76), SIMDE_FLOAT32_C(    25.32) },
      { SIMDE_FLOAT32_C(    30.28), SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(    15.09), SIMDE_FLOAT32_C(   -27.90),
        SIMDE_FLOAT32_C(    25.64), SIMDE_FLOAT32_C(    -2.52), SIMDE_FLOAT32_C(    17.21), SIMDE_FLOAT32_C(     0.74) } },
    { { SIMDE_FLOAT32_C(  -115.53), SIMDE_FLOAT32_C(  -748.68), SIMDE_FLOAT32_C(   864.53), SIMDE_FLOAT32_C(   223.49),
        SIMDE_FLOAT32_C(  -745.38), SIMDE_FLOAT32_C(  -158.17), SIMDE_FLOAT32_C(  -851.24), SIMDE_FLOAT32_C(   -80.46) },
      { SIMDE_FLOAT32_C(    17.92), SIMDE_FLOAT32_C(   -20.89), SIMDE_FLOAT32_C(    29.64), SIMDE_FLOAT32_C(     3.77),
        SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(   -27.45), SIMDE_FLOAT32_C(     1.38), SIMDE_FLOAT32_C(   -29.21) } },
    { { SIMDE_FLOAT32_C(   454.37), SIMDE_FLOAT32_C(  -858.75), SIMDE_FLOAT32_C(  -745.47), SIMDE_FLOAT32_C(  -918.71),
        SIMDE_FLOAT32_C(  -798.04), SIMDE_FLOAT32_C(   474.10), SIMDE_FLOAT32_C(  -484.67), SIMDE_FLOAT32_C(   828.20) },
      { SIMDE_FLOAT32_C(    26.70), SIMDE_FLOAT32_C(   -16.08), SIMDE_FLOAT32_C(    14.79), SIMDE_FLOAT32_C(   -31.05),
        SIMDE_FLOAT32_C(     8.07), SIMDE_FLOAT32_C(    29.38), SIMDE_FLOAT32_C(    15.41), SIMDE_FLOAT32_C(    26.87) } },
    { { SIMDE_FLOAT32_C(  -916.70), SIMDE_FLOAT32_C(  -831.23), SIMDE_FLOAT32_C(   251.85), SIMDE_FLOAT32_C(   404.02),
        SIMDE_FLOAT32_C(   917.96), SIMDE_FLOAT32_C(   645.91), SIMDE_FLOAT32_C(  -412.89), SIMDE_FLOAT32_C(   829.74) },
      { SIMDE_FLOAT32_C(    12.66), SIMDE_FLOAT32_C(   -32.82), SIMDE_FLOAT32_C(    19.08), SIMDE_FLOAT32_C(    10.59),
        SIMDE_FLOAT32_C(    31.94), SIMDE_FLOAT32_C(    10.11), SIMDE_FLOAT32_C(    16.03), SIMDE_FLOAT32_C(    25.88) } },
    { { SIMDE_FLOAT32_C(  -219.12), SIMDE_FLOAT32_C(    36.49), SIMDE_FLOAT32_C(   987.58), SIMDE_FLOAT32_C(  -568.25),
        SIMDE_FLOAT32_C(   907.54), SIMDE_FLOAT32_C(   283.34), SIMDE_FLOAT32_C(   457.07), SIMDE_FLOAT32_C(  -207.99) },
      { SIMDE_FLOAT32_C(     1.23), SIMDE_FLOAT32_C(    14.85), SIMDE_FLOAT32_C(    32.61), SIMDE_FLOAT32_C(    -8.71),
        SIMDE_FLOAT32_C(    30.48), SIMDE_FLOAT32_C(     4.65), SIMDE_FLOAT32_C(    21.90), SIMDE_FLOAT32_C(    -4.75) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_csqrt_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_cos_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.49)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.88)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -0.94)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.95)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(     0.85)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     1.00)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.48)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -0.61)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_cos_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_cos_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(    0.49)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.66), SIMDE_FLOAT64_C(    0.26)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.54), SIMDE_FLOAT64_C(   -0.88)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.59), SIMDE_FLOAT64_C(   -0.47)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.75), SIMDE_FLOAT64_C(   -0.94)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.33)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.95)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(    0.30)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_cos_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_cos_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24),
                         SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01),
                         SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.47),
                         SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.88),
                         SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(     0.26),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.49)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13),
                         SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21),
                         SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.30),
                         SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.95),
                         SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -0.94)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95),
                         SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.98),
                         SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     1.00),
                         SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(     0.85)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67),
                         SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54),
                         SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.15),
                         SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -0.61),
                         SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.46),
                         SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.48)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48),
                         SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.87),
                         SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(    -0.95),
                         SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.74)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92),
                         SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(     1.00),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     1.00),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     1.00),
                         SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.81)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73),
                         SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.58),
                         SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.67),
                         SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -0.99)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02),
                         SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.13),
                         SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.27),
                         SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(    -0.91),
                         SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    -0.99)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_cos_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_cos_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.66), SIMDE_FLOAT64_C(    0.26),
                         SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(    0.49)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.59), SIMDE_FLOAT64_C(   -0.47),
                         SIMDE_FLOAT64_C(   -0.54), SIMDE_FLOAT64_C(   -0.88)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.33),
                         SIMDE_FLOAT64_C(    0.75), SIMDE_FLOAT64_C(   -0.94)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(    0.30),
                         SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.95)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.94), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(   -0.96), SIMDE_FLOAT64_C(    0.85)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.27), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(    0.73), SIMDE_FLOAT64_C(    1.00)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.85), SIMDE_FLOAT64_C(    0.46),
                         SIMDE_FLOAT64_C(   -0.88), SIMDE_FLOAT64_C(    0.48)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.64), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(    0.63), SIMDE_FLOAT64_C(   -0.61)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_cos_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_cbrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -233.95), SIMDE_FLOAT32_C(   484.12), SIMDE_FLOAT32_C(  -627.08), SIMDE_FLOAT32_C(  -978.93) },
      { SIMDE_FLOAT32_C(    -6.16), SIMDE_FLOAT32_C(     7.85), SIMDE_FLOAT32_C(    -8.56), SIMDE_FLOAT32_C(    -9.93) } },
    { { SIMDE_FLOAT32_C(  -749.83), SIMDE_FLOAT32_C(   484.28), SIMDE_FLOAT32_C(   749.02), SIMDE_FLOAT32_C(   850.44) },
      { SIMDE_FLOAT32_C(    -9.08), SIMDE_FLOAT32_C(     7.85), SIMDE_FLOAT32_C(     9.08), SIMDE_FLOAT32_C(     9.47) } },
    { { SIMDE_FLOAT32_C(  -517.39), SIMDE_FLOAT32_C(  -725.46), SIMDE_FLOAT32_C(  -558.90), SIMDE_FLOAT32_C(  -267.33) },
      { SIMDE_FLOAT32_C(    -8.03), SIMDE_FLOAT32_C(    -8.99), SIMDE_FLOAT32_C(    -8.24), SIMDE_FLOAT32_C(    -6.44) } },
    { { SIMDE_FLOAT32_C(   569.35), SIMDE_FLOAT32_C(   995.62), SIMDE_FLOAT32_C(   709.27), SIMDE_FLOAT32_C(  -107.57) },
      { SIMDE_FLOAT32_C(     8.29), SIMDE_FLOAT32_C(     9.99), SIMDE_FLOAT32_C(     8.92), SIMDE_FLOAT32_C(    -4.76) } },
    { { SIMDE_FLOAT32_C(   350.06), SIMDE_FLOAT32_C(    89.99), SIMDE_FLOAT32_C(   267.98), SIMDE_FLOAT32_C(  -152.18) },
      { SIMDE_FLOAT32_C(     7.05), SIMDE_FLOAT32_C(     4.48), SIMDE_FLOAT32_C(     6.45), SIMDE_FLOAT32_C(    -5.34) } },
    { { SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(   317.87), SIMDE_FLOAT32_C(  -435.79), SIMDE_FLOAT32_C(  -295.24) },
      { SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     6.82), SIMDE_FLOAT32_C(    -7.58), SIMDE_FLOAT32_C(    -6.66) } },
    { { SIMDE_FLOAT32_C(   382.46), SIMDE_FLOAT32_C(   327.49), SIMDE_FLOAT32_C(  -186.96), SIMDE_FLOAT32_C(   913.54) },
      { SIMDE_FLOAT32_C(     7.26), SIMDE_FLOAT32_C(     6.89), SIMDE_FLOAT32_C(    -5.72), SIMDE_FLOAT32_C(     9.70) } },
    { { SIMDE_FLOAT32_C(   619.00), SIMDE_FLOAT32_C(   936.03), SIMDE_FLOAT32_C(    27.91), SIMDE_FLOAT32_C(  -614.95) },
      { SIMDE_FLOAT32_C(     8.52), SIMDE_FLOAT32_C(     9.78), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(    -8.50) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_cbrt_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_cbrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -517.18), SIMDE_FLOAT64_C(   744.08) },
      { SIMDE_FLOAT64_C(    -8.03), SIMDE_FLOAT64_C(     9.06) } },
    { { SIMDE_FLOAT64_C(   664.94), SIMDE_FLOAT64_C(   255.05) },
      { SIMDE_FLOAT64_C(     8.73), SIMDE_FLOAT64_C(     6.34) } },
    { { SIMDE_FLOAT64_C(    38.42), SIMDE_FLOAT64_C(   432.02) },
      { SIMDE_FLOAT64_C(     3.37), SIMDE_FLOAT64_C(     7.56) } },
    { { SIMDE_FLOAT64_C(  -843.35), SIMDE_FLOAT64_C(  -957.81) },
      { SIMDE_FLOAT64_C(    -9.45), SIMDE_FLOAT64_C(    -9.86) } },
    { { SIMDE_FLOAT64_C(  -560.27), SIMDE_FLOAT64_C(   292.64) },
      { SIMDE_FLOAT64_C(    -8.24), SIMDE_FLOAT64_C(     6.64) } },
    { { SIMDE_FLOAT64_C(   329.56), SIMDE_FLOAT64_C(   633.90) },
      { SIMDE_FLOAT64_C(     6.91), SIMDE_FLOAT64_C(     8.59) } },
    { { SIMDE_FLOAT64_C(  -774.56), SIMDE_FLOAT64_C(   892.85) },
      { SIMDE_FLOAT64_C(    -9.18), SIMDE_FLOAT64_C(     9.63) } },
    { { SIMDE_FLOAT64_C(   705.03), SIMDE_FLOAT64_C(  -332.78) },
      { SIMDE_FLOAT64_C(     8.90), SIMDE_FLOAT64_C(    -6.93) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_cbrt_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_cbrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    14.66), SIMDE_FLOAT32_C(  -346.78), SIMDE_FLOAT32_C(   608.16), SIMDE_FLOAT32_C(  -175.40),
        SIMDE_FLOAT32_C(  -696.64), SIMDE_FLOAT32_C(  -645.46), SIMDE_FLOAT32_C(  -765.98), SIMDE_FLOAT32_C(   391.25) },
      { SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(    -7.03), SIMDE_FLOAT32_C(     8.47), SIMDE_FLOAT32_C(    -5.60),
        SIMDE_FLOAT32_C(    -8.86), SIMDE_FLOAT32_C(    -8.64), SIMDE_FLOAT32_C(    -9.15), SIMDE_FLOAT32_C(     7.31) } },
    { { SIMDE_FLOAT32_C(   -27.85), SIMDE_FLOAT32_C(   887.61), SIMDE_FLOAT32_C(  -720.32), SIMDE_FLOAT32_C(  -702.24),
        SIMDE_FLOAT32_C(  -320.58), SIMDE_FLOAT32_C(  -360.38), SIMDE_FLOAT32_C(   -53.29), SIMDE_FLOAT32_C(   251.62) },
      { SIMDE_FLOAT32_C(    -3.03), SIMDE_FLOAT32_C(     9.61), SIMDE_FLOAT32_C(    -8.96), SIMDE_FLOAT32_C(    -8.89),
        SIMDE_FLOAT32_C(    -6.84), SIMDE_FLOAT32_C(    -7.12), SIMDE_FLOAT32_C(    -3.76), SIMDE_FLOAT32_C(     6.31) } },
    { { SIMDE_FLOAT32_C(   677.19), SIMDE_FLOAT32_C(   865.20), SIMDE_FLOAT32_C(  -346.98), SIMDE_FLOAT32_C(  -605.62),
        SIMDE_FLOAT32_C(  -498.20), SIMDE_FLOAT32_C(   696.85), SIMDE_FLOAT32_C(  -203.22), SIMDE_FLOAT32_C(  -909.19) },
      { SIMDE_FLOAT32_C(     8.78), SIMDE_FLOAT32_C(     9.53), SIMDE_FLOAT32_C(    -7.03), SIMDE_FLOAT32_C(    -8.46),
        SIMDE_FLOAT32_C(    -7.93), SIMDE_FLOAT32_C(     8.87), SIMDE_FLOAT32_C(    -5.88), SIMDE_FLOAT32_C(    -9.69) } },
    { { SIMDE_FLOAT32_C(    46.70), SIMDE_FLOAT32_C(  -557.66), SIMDE_FLOAT32_C(  -327.34), SIMDE_FLOAT32_C(  -489.40),
        SIMDE_FLOAT32_C(   -78.90), SIMDE_FLOAT32_C(  -843.63), SIMDE_FLOAT32_C(  -527.77), SIMDE_FLOAT32_C(   935.75) },
      { SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(    -8.23), SIMDE_FLOAT32_C(    -6.89), SIMDE_FLOAT32_C(    -7.88),
        SIMDE_FLOAT32_C(    -4.29), SIMDE_FLOAT32_C(    -9.45), SIMDE_FLOAT32_C(    -8.08), SIMDE_FLOAT32_C(     9.78) } },
    { { SIMDE_FLOAT32_C(  -190.41), SIMDE_FLOAT32_C(  -919.61), SIMDE_FLOAT32_C(  -239.64), SIMDE_FLOAT32_C(   112.95),
        SIMDE_FLOAT32_C(  -565.07), SIMDE_FLOAT32_C(    -5.63), SIMDE_FLOAT32_C(  -495.80), SIMDE_FLOAT32_C(   407.08) },
      { SIMDE_FLOAT32_C(    -5.75), SIMDE_FLOAT32_C(    -9.72), SIMDE_FLOAT32_C(    -6.21), SIMDE_FLOAT32_C(     4.83),
        SIMDE_FLOAT32_C(    -8.27), SIMDE_FLOAT32_C(    -1.78), SIMDE_FLOAT32_C(    -7.91), SIMDE_FLOAT32_C(     7.41) } },
    { { SIMDE_FLOAT32_C(  -118.02), SIMDE_FLOAT32_C(  -216.12), SIMDE_FLOAT32_C(   704.84), SIMDE_FLOAT32_C(   561.40),
        SIMDE_FLOAT32_C(   423.50), SIMDE_FLOAT32_C(  -348.46), SIMDE_FLOAT32_C(  -186.97), SIMDE_FLOAT32_C(   100.69) },
      { SIMDE_FLOAT32_C(    -4.91), SIMDE_FLOAT32_C(    -6.00), SIMDE_FLOAT32_C(     8.90), SIMDE_FLOAT32_C(     8.25),
        SIMDE_FLOAT32_C(     7.51), SIMDE_FLOAT32_C(    -7.04), SIMDE_FLOAT32_C(    -5.72), SIMDE_FLOAT32_C(     4.65) } },
    { { SIMDE_FLOAT32_C(  -483.26), SIMDE_FLOAT32_C(   466.05), SIMDE_FLOAT32_C(   495.07), SIMDE_FLOAT32_C(    18.54),
        SIMDE_FLOAT32_C(   162.90), SIMDE_FLOAT32_C(  -708.15), SIMDE_FLOAT32_C(   109.34), SIMDE_FLOAT32_C(  -790.40) },
      { SIMDE_FLOAT32_C(    -7.85), SIMDE_FLOAT32_C(     7.75), SIMDE_FLOAT32_C(     7.91), SIMDE_FLOAT32_C(     2.65),
        SIMDE_FLOAT32_C(     5.46), SIMDE_FLOAT32_C(    -8.91), SIMDE_FLOAT32_C(     4.78), SIMDE_FLOAT32_C(    -9.25) } },
    { { SIMDE_FLOAT32_C(  -265.81), SIMDE_FLOAT32_C(   782.01), SIMDE_FLOAT32_C(  -279.80), SIMDE_FLOAT32_C(   655.29),
        SIMDE_FLOAT32_C(   938.38), SIMDE_FLOAT32_C(   192.43), SIMDE_FLOAT32_C(   591.04), SIMDE_FLOAT32_C(  -252.03) },
      { SIMDE_FLOAT32_C(    -6.43), SIMDE_FLOAT32_C(     9.21), SIMDE_FLOAT32_C(    -6.54), SIMDE_FLOAT32_C(     8.69),
        SIMDE_FLOAT32_C(     9.79), SIMDE_FLOAT32_C(     5.77), SIMDE_FLOAT32_C(     8.39), SIMDE_FLOAT32_C(    -6.32) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_cbrt_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_cbrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   286.65), SIMDE_FLOAT64_C(  -385.66), SIMDE_FLOAT64_C(    84.84), SIMDE_FLOAT64_C(   116.45) },
      { SIMDE_FLOAT64_C(     6.59), SIMDE_FLOAT64_C(    -7.28), SIMDE_FLOAT64_C(     4.39), SIMDE_FLOAT64_C(     4.88) } },
    { { SIMDE_FLOAT64_C(   443.79), SIMDE_FLOAT64_C(   321.91), SIMDE_FLOAT64_C(  -219.08), SIMDE_FLOAT64_C(  -924.57) },
      { SIMDE_FLOAT64_C(     7.63), SIMDE_FLOAT64_C(     6.85), SIMDE_FLOAT64_C(    -6.03), SIMDE_FLOAT64_C(    -9.74) } },
    { { SIMDE_FLOAT64_C(   745.74), SIMDE_FLOAT64_C(   694.64), SIMDE_FLOAT64_C(   266.38), SIMDE_FLOAT64_C(   138.63) },
      { SIMDE_FLOAT64_C(     9.07), SIMDE_FLOAT64_C(     8.86), SIMDE_FLOAT64_C(     6.43), SIMDE_FLOAT64_C(     5.18) } },
    { { SIMDE_FLOAT64_C(   417.51), SIMDE_FLOAT64_C(    27.01), SIMDE_FLOAT64_C(  -921.58), SIMDE_FLOAT64_C(    56.73) },
      { SIMDE_FLOAT64_C(     7.47), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(    -9.73), SIMDE_FLOAT64_C(     3.84) } },
    { { SIMDE_FLOAT64_C(   568.89), SIMDE_FLOAT64_C(   355.21), SIMDE_FLOAT64_C(  -243.68), SIMDE_FLOAT64_C(   232.84) },
      { SIMDE_FLOAT64_C(     8.29), SIMDE_FLOAT64_C(     7.08), SIMDE_FLOAT64_C(    -6.25), SIMDE_FLOAT64_C(     6.15) } },
    { { SIMDE_FLOAT64_C(  -964.92), SIMDE_FLOAT64_C(  -649.34), SIMDE_FLOAT64_C(  -100.47), SIMDE_FLOAT64_C(  -303.39) },
      { SIMDE_FLOAT64_C(    -9.88), SIMDE_FLOAT64_C(    -8.66), SIMDE_FLOAT64_C(    -4.65), SIMDE_FLOAT64_C(    -6.72) } },
    { { SIMDE_FLOAT64_C(   -56.31), SIMDE_FLOAT64_C(  -696.56), SIMDE_FLOAT64_C(  -500.81), SIMDE_FLOAT64_C(   866.34) },
      { SIMDE_FLOAT64_C(    -3.83), SIMDE_FLOAT64_C(    -8.86), SIMDE_FLOAT64_C(    -7.94), SIMDE_FLOAT64_C(     9.53) } },
    { { SIMDE_FLOAT64_C(   560.33), SIMDE_FLOAT64_C(   808.06), SIMDE_FLOAT64_C(   566.38), SIMDE_FLOAT64_C(  -153.02) },
      { SIMDE_FLOAT64_C(     8.24), SIMDE_FLOAT64_C(     9.31), SIMDE_FLOAT64_C(     8.27), SIMDE_FLOAT64_C(    -5.35) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_cbrt_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_cbrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   -13.67), SIMDE_FLOAT32_C(   -56.84), SIMDE_FLOAT32_C(   -51.43), SIMDE_FLOAT32_C(   570.17),
        SIMDE_FLOAT32_C(   282.97), SIMDE_FLOAT32_C(  -935.16), SIMDE_FLOAT32_C(   302.89), SIMDE_FLOAT32_C(  -720.37),
        SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(   -98.04), SIMDE_FLOAT32_C(    -1.14), SIMDE_FLOAT32_C(   -24.91),
        SIMDE_FLOAT32_C(   315.22), SIMDE_FLOAT32_C(  -790.04), SIMDE_FLOAT32_C(   -92.39), SIMDE_FLOAT32_C(  -624.42) },
      { SIMDE_FLOAT32_C(    -2.39), SIMDE_FLOAT32_C(    -3.84), SIMDE_FLOAT32_C(    -3.72), SIMDE_FLOAT32_C(     8.29),
        SIMDE_FLOAT32_C(     6.57), SIMDE_FLOAT32_C(    -9.78), SIMDE_FLOAT32_C(     6.72), SIMDE_FLOAT32_C(    -8.96),
        SIMDE_FLOAT32_C(    -6.89), SIMDE_FLOAT32_C(    -4.61), SIMDE_FLOAT32_C(    -1.05), SIMDE_FLOAT32_C(    -2.92),
        SIMDE_FLOAT32_C(     6.81), SIMDE_FLOAT32_C(    -9.24), SIMDE_FLOAT32_C(    -4.52), SIMDE_FLOAT32_C(    -8.55) } },
    { { SIMDE_FLOAT32_C(   534.24), SIMDE_FLOAT32_C(   480.60), SIMDE_FLOAT32_C(  -464.10), SIMDE_FLOAT32_C(   924.79),
        SIMDE_FLOAT32_C(   691.98), SIMDE_FLOAT32_C(   368.05), SIMDE_FLOAT32_C(   181.75), SIMDE_FLOAT32_C(   967.37),
        SIMDE_FLOAT32_C(  -837.71), SIMDE_FLOAT32_C(   -61.77), SIMDE_FLOAT32_C(  -702.36), SIMDE_FLOAT32_C(    76.18),
        SIMDE_FLOAT32_C(   549.27), SIMDE_FLOAT32_C(    36.35), SIMDE_FLOAT32_C(  -116.93), SIMDE_FLOAT32_C(  -464.40) },
      { SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     7.83), SIMDE_FLOAT32_C(    -7.74), SIMDE_FLOAT32_C(     9.74),
        SIMDE_FLOAT32_C(     8.84), SIMDE_FLOAT32_C(     7.17), SIMDE_FLOAT32_C(     5.66), SIMDE_FLOAT32_C(     9.89),
        SIMDE_FLOAT32_C(    -9.43), SIMDE_FLOAT32_C(    -3.95), SIMDE_FLOAT32_C(    -8.89), SIMDE_FLOAT32_C(     4.24),
        SIMDE_FLOAT32_C(     8.19), SIMDE_FLOAT32_C(     3.31), SIMDE_FLOAT32_C(    -4.89), SIMDE_FLOAT32_C(    -7.74) } },
    { { SIMDE_FLOAT32_C(   979.51), SIMDE_FLOAT32_C(   831.64), SIMDE_FLOAT32_C(  -894.23), SIMDE_FLOAT32_C(   262.49),
        SIMDE_FLOAT32_C(   896.48), SIMDE_FLOAT32_C(   408.65), SIMDE_FLOAT32_C(   542.11), SIMDE_FLOAT32_C(  -430.74),
        SIMDE_FLOAT32_C(  -689.38), SIMDE_FLOAT32_C(  -459.03), SIMDE_FLOAT32_C(   544.35), SIMDE_FLOAT32_C(   625.84),
        SIMDE_FLOAT32_C(  -249.07), SIMDE_FLOAT32_C(  -548.04), SIMDE_FLOAT32_C(  -998.58), SIMDE_FLOAT32_C(  -714.83) },
      { SIMDE_FLOAT32_C(     9.93), SIMDE_FLOAT32_C(     9.40), SIMDE_FLOAT32_C(    -9.63), SIMDE_FLOAT32_C(     6.40),
        SIMDE_FLOAT32_C(     9.64), SIMDE_FLOAT32_C(     7.42), SIMDE_FLOAT32_C(     8.15), SIMDE_FLOAT32_C(    -7.55),
        SIMDE_FLOAT32_C(    -8.83), SIMDE_FLOAT32_C(    -7.71), SIMDE_FLOAT32_C(     8.17), SIMDE_FLOAT32_C(     8.55),
        SIMDE_FLOAT32_C(    -6.29), SIMDE_FLOAT32_C(    -8.18), SIMDE_FLOAT32_C(   -10.00), SIMDE_FLOAT32_C(    -8.94) } },
    { { SIMDE_FLOAT32_C(   932.56), SIMDE_FLOAT32_C(  -462.68), SIMDE_FLOAT32_C(  -790.04), SIMDE_FLOAT32_C(   624.53),
        SIMDE_FLOAT32_C(   905.37), SIMDE_FLOAT32_C(   391.72), SIMDE_FLOAT32_C(   591.90), SIMDE_FLOAT32_C(  -932.34),
        SIMDE_FLOAT32_C(  -670.05), SIMDE_FLOAT32_C(   889.54), SIMDE_FLOAT32_C(   143.84), SIMDE_FLOAT32_C(   879.22),
        SIMDE_FLOAT32_C(   -74.11), SIMDE_FLOAT32_C(  -973.09), SIMDE_FLOAT32_C(  -585.18), SIMDE_FLOAT32_C(   -94.60) },
      { SIMDE_FLOAT32_C(     9.77), SIMDE_FLOAT32_C(    -7.73), SIMDE_FLOAT32_C(    -9.24), SIMDE_FLOAT32_C(     8.55),
        SIMDE_FLOAT32_C(     9.67), SIMDE_FLOAT32_C(     7.32), SIMDE_FLOAT32_C(     8.40), SIMDE_FLOAT32_C(    -9.77),
        SIMDE_FLOAT32_C(    -8.75), SIMDE_FLOAT32_C(     9.62), SIMDE_FLOAT32_C(     5.24), SIMDE_FLOAT32_C(     9.58),
        SIMDE_FLOAT32_C(    -4.20), SIMDE_FLOAT32_C(    -9.91), SIMDE_FLOAT32_C(    -8.36), SIMDE_FLOAT32_C(    -4.56) } },
    { { SIMDE_FLOAT32_C(   858.55), SIMDE_FLOAT32_C(  -479.41), SIMDE_FLOAT32_C(  -832.11), SIMDE_FLOAT32_C(   755.02),
        SIMDE_FLOAT32_C(   929.24), SIMDE_FLOAT32_C(   710.00), SIMDE_FLOAT32_C(  -675.72), SIMDE_FLOAT32_C(  -760.15),
        SIMDE_FLOAT32_C(  -749.03), SIMDE_FLOAT32_C(   868.63), SIMDE_FLOAT32_C(   865.69), SIMDE_FLOAT32_C(     1.90),
        SIMDE_FLOAT32_C(  -679.42), SIMDE_FLOAT32_C(   867.11), SIMDE_FLOAT32_C(   287.07), SIMDE_FLOAT32_C(  -746.86) },
      { SIMDE_FLOAT32_C(     9.50), SIMDE_FLOAT32_C(    -7.83), SIMDE_FLOAT32_C(    -9.41), SIMDE_FLOAT32_C(     9.11),
        SIMDE_FLOAT32_C(     9.76), SIMDE_FLOAT32_C(     8.92), SIMDE_FLOAT32_C(    -8.78), SIMDE_FLOAT32_C(    -9.13),
        SIMDE_FLOAT32_C(    -9.08), SIMDE_FLOAT32_C(     9.54), SIMDE_FLOAT32_C(     9.53), SIMDE_FLOAT32_C(     1.24),
        SIMDE_FLOAT32_C(    -8.79), SIMDE_FLOAT32_C(     9.54), SIMDE_FLOAT32_C(     6.60), SIMDE_FLOAT32_C(    -9.07) } },
    { { SIMDE_FLOAT32_C(  -595.56), SIMDE_FLOAT32_C(   497.03), SIMDE_FLOAT32_C(   877.67), SIMDE_FLOAT32_C(  -690.19),
        SIMDE_FLOAT32_C(  -111.25), SIMDE_FLOAT32_C(   469.57), SIMDE_FLOAT32_C(  -622.53), SIMDE_FLOAT32_C(   218.70),
        SIMDE_FLOAT32_C(   359.11), SIMDE_FLOAT32_C(   521.31), SIMDE_FLOAT32_C(    97.92), SIMDE_FLOAT32_C(  -714.99),
        SIMDE_FLOAT32_C(   548.22), SIMDE_FLOAT32_C(   512.74), SIMDE_FLOAT32_C(   190.41), SIMDE_FLOAT32_C(   406.77) },
      { SIMDE_FLOAT32_C(    -8.41), SIMDE_FLOAT32_C(     7.92), SIMDE_FLOAT32_C(     9.57), SIMDE_FLOAT32_C(    -8.84),
        SIMDE_FLOAT32_C(    -4.81), SIMDE_FLOAT32_C(     7.77), SIMDE_FLOAT32_C(    -8.54), SIMDE_FLOAT32_C(     6.02),
        SIMDE_FLOAT32_C(     7.11), SIMDE_FLOAT32_C(     8.05), SIMDE_FLOAT32_C(     4.61), SIMDE_FLOAT32_C(    -8.94),
        SIMDE_FLOAT32_C(     8.18), SIMDE_FLOAT32_C(     8.00), SIMDE_FLOAT32_C(     5.75), SIMDE_FLOAT32_C(     7.41) } },
    { { SIMDE_FLOAT32_C(  -966.68), SIMDE_FLOAT32_C(   358.30), SIMDE_FLOAT32_C(   161.79), SIMDE_FLOAT32_C(   962.56),
        SIMDE_FLOAT32_C(    68.29), SIMDE_FLOAT32_C(   486.07), SIMDE_FLOAT32_C(  -797.58), SIMDE_FLOAT32_C(   319.26),
        SIMDE_FLOAT32_C(   354.70), SIMDE_FLOAT32_C(  -931.89), SIMDE_FLOAT32_C(  -678.84), SIMDE_FLOAT32_C(   675.28),
        SIMDE_FLOAT32_C(   935.22), SIMDE_FLOAT32_C(   608.23), SIMDE_FLOAT32_C(   928.43), SIMDE_FLOAT32_C(  -660.34) },
      { SIMDE_FLOAT32_C(    -9.89), SIMDE_FLOAT32_C(     7.10), SIMDE_FLOAT32_C(     5.45), SIMDE_FLOAT32_C(     9.87),
        SIMDE_FLOAT32_C(     4.09), SIMDE_FLOAT32_C(     7.86), SIMDE_FLOAT32_C(    -9.27), SIMDE_FLOAT32_C(     6.83),
        SIMDE_FLOAT32_C(     7.08), SIMDE_FLOAT32_C(    -9.77), SIMDE_FLOAT32_C(    -8.79), SIMDE_FLOAT32_C(     8.77),
        SIMDE_FLOAT32_C(     9.78), SIMDE_FLOAT32_C(     8.47), SIMDE_FLOAT32_C(     9.76), SIMDE_FLOAT32_C(    -8.71) } },
    { { SIMDE_FLOAT32_C(   105.27), SIMDE_FLOAT32_C(   806.10), SIMDE_FLOAT32_C(  -350.53), SIMDE_FLOAT32_C(   994.02),
        SIMDE_FLOAT32_C(   275.67), SIMDE_FLOAT32_C(    26.95), SIMDE_FLOAT32_C(   212.72), SIMDE_FLOAT32_C(  -365.21),
        SIMDE_FLOAT32_C(  -451.74), SIMDE_FLOAT32_C(  -689.36), SIMDE_FLOAT32_C(   -80.21), SIMDE_FLOAT32_C(  -903.52),
        SIMDE_FLOAT32_C(   823.38), SIMDE_FLOAT32_C(  -889.80), SIMDE_FLOAT32_C(   503.25), SIMDE_FLOAT32_C(   856.70) },
      { SIMDE_FLOAT32_C(     4.72), SIMDE_FLOAT32_C(     9.31), SIMDE_FLOAT32_C(    -7.05), SIMDE_FLOAT32_C(     9.98),
        SIMDE_FLOAT32_C(     6.51), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     5.97), SIMDE_FLOAT32_C(    -7.15),
        SIMDE_FLOAT32_C(    -7.67), SIMDE_FLOAT32_C(    -8.83), SIMDE_FLOAT32_C(    -4.31), SIMDE_FLOAT32_C(    -9.67),
        SIMDE_FLOAT32_C(     9.37), SIMDE_FLOAT32_C(    -9.62), SIMDE_FLOAT32_C(     7.95), SIMDE_FLOAT32_C(     9.50) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_cbrt_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_cbrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -399.60), SIMDE_FLOAT64_C(    73.77), SIMDE_FLOAT64_C(   137.22), SIMDE_FLOAT64_C(  -676.98),
        SIMDE_FLOAT64_C(  -304.40), SIMDE_FLOAT64_C(   -35.84), SIMDE_FLOAT64_C(  -298.40), SIMDE_FLOAT64_C(  -909.21) },
      { SIMDE_FLOAT64_C(    -7.37), SIMDE_FLOAT64_C(     4.19), SIMDE_FLOAT64_C(     5.16), SIMDE_FLOAT64_C(    -8.78),
        SIMDE_FLOAT64_C(    -6.73), SIMDE_FLOAT64_C(    -3.30), SIMDE_FLOAT64_C(    -6.68), SIMDE_FLOAT64_C(    -9.69) } },
    { { SIMDE_FLOAT64_C(  -369.66), SIMDE_FLOAT64_C(   -37.98), SIMDE_FLOAT64_C(   225.69), SIMDE_FLOAT64_C(   708.35),
        SIMDE_FLOAT64_C(   411.81), SIMDE_FLOAT64_C(   -32.59), SIMDE_FLOAT64_C(   605.95), SIMDE_FLOAT64_C(  -309.62) },
      { SIMDE_FLOAT64_C(    -7.18), SIMDE_FLOAT64_C(    -3.36), SIMDE_FLOAT64_C(     6.09), SIMDE_FLOAT64_C(     8.91),
        SIMDE_FLOAT64_C(     7.44), SIMDE_FLOAT64_C(    -3.19), SIMDE_FLOAT64_C(     8.46), SIMDE_FLOAT64_C(    -6.77) } },
    { { SIMDE_FLOAT64_C(   644.51), SIMDE_FLOAT64_C(  -178.16), SIMDE_FLOAT64_C(  -305.15), SIMDE_FLOAT64_C(   654.50),
        SIMDE_FLOAT64_C(  -229.06), SIMDE_FLOAT64_C(  -577.20), SIMDE_FLOAT64_C(   549.91), SIMDE_FLOAT64_C(  -450.26) },
      { SIMDE_FLOAT64_C(     8.64), SIMDE_FLOAT64_C(    -5.63), SIMDE_FLOAT64_C(    -6.73), SIMDE_FLOAT64_C(     8.68),
        SIMDE_FLOAT64_C(    -6.12), SIMDE_FLOAT64_C(    -8.33), SIMDE_FLOAT64_C(     8.19), SIMDE_FLOAT64_C(    -7.66) } },
    { { SIMDE_FLOAT64_C(   336.68), SIMDE_FLOAT64_C(  -367.59), SIMDE_FLOAT64_C(   113.01), SIMDE_FLOAT64_C(  -952.73),
        SIMDE_FLOAT64_C(   958.03), SIMDE_FLOAT64_C(   319.98), SIMDE_FLOAT64_C(  -626.30), SIMDE_FLOAT64_C(  -441.56) },
      { SIMDE_FLOAT64_C(     6.96), SIMDE_FLOAT64_C(    -7.16), SIMDE_FLOAT64_C(     4.83), SIMDE_FLOAT64_C(    -9.84),
        SIMDE_FLOAT64_C(     9.86), SIMDE_FLOAT64_C(     6.84), SIMDE_FLOAT64_C(    -8.56), SIMDE_FLOAT64_C(    -7.61) } },
    { { SIMDE_FLOAT64_C(  -606.25), SIMDE_FLOAT64_C(   510.93), SIMDE_FLOAT64_C(  -118.54), SIMDE_FLOAT64_C(    89.36),
        SIMDE_FLOAT64_C(  -524.91), SIMDE_FLOAT64_C(   583.06), SIMDE_FLOAT64_C(   180.15), SIMDE_FLOAT64_C(   105.43) },
      { SIMDE_FLOAT64_C(    -8.46), SIMDE_FLOAT64_C(     7.99), SIMDE_FLOAT64_C(    -4.91), SIMDE_FLOAT64_C(     4.47),
        SIMDE_FLOAT64_C(    -8.07), SIMDE_FLOAT64_C(     8.35), SIMDE_FLOAT64_C(     5.65), SIMDE_FLOAT64_C(     4.72) } },
    { { SIMDE_FLOAT64_C(  -454.92), SIMDE_FLOAT64_C(  -594.16), SIMDE_FLOAT64_C(  -186.22), SIMDE_FLOAT64_C(   956.89),
        SIMDE_FLOAT64_C(   373.25), SIMDE_FLOAT64_C(  -580.27), SIMDE_FLOAT64_C(  -352.73), SIMDE_FLOAT64_C(    17.77) },
      { SIMDE_FLOAT64_C(    -7.69), SIMDE_FLOAT64_C(    -8.41), SIMDE_FLOAT64_C(    -5.71), SIMDE_FLOAT64_C(     9.85),
        SIMDE_FLOAT64_C(     7.20), SIMDE_FLOAT64_C(    -8.34), SIMDE_FLOAT64_C(    -7.07), SIMDE_FLOAT64_C(     2.61) } },
    { { SIMDE_FLOAT64_C(   241.57), SIMDE_FLOAT64_C(   342.12), SIMDE_FLOAT64_C(  -327.73), SIMDE_FLOAT64_C(  -987.48),
        SIMDE_FLOAT64_C(   764.92), SIMDE_FLOAT64_C(  -777.82), SIMDE_FLOAT64_C(  -437.75), SIMDE_FLOAT64_C(   101.60) },
      { SIMDE_FLOAT64_C(     6.23), SIMDE_FLOAT64_C(     6.99), SIMDE_FLOAT64_C(    -6.89), SIMDE_FLOAT64_C(    -9.96),
        SIMDE_FLOAT64_C(     9.15), SIMDE_FLOAT64_C(    -9.20), SIMDE_FLOAT64_C(    -7.59), SIMDE_FLOAT64_C(     4.67) } },
    { { SIMDE_FLOAT64_C(  -145.41), SIMDE_FLOAT64_C(   675.27), SIMDE_FLOAT64_C(   148.87), SIMDE_FLOAT64_C(  -187.38),
        SIMDE_FLOAT64_C(    -4.75), SIMDE_FLOAT64_C(   522.57), SIMDE_FLOAT64_C(   371.06), SIMDE_FLOAT64_C(   389.00) },
      { SIMDE_FLOAT64_C(    -5.26), SIMDE_FLOAT64_C(     8.77), SIMDE_FLOAT64_C(     5.30), SIMDE_FLOAT64_C(    -5.72),
        SIMDE_FLOAT64_C(    -1.68), SIMDE_FLOAT64_C(     8.05), SIMDE_FLOAT64_C(     7.19), SIMDE_FLOAT64_C(     7.30) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_cbrt_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cbrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    98.98), SIMDE_FLOAT32_C(   913.15), SIMDE_FLOAT32_C(   690.53), SIMDE_FLOAT32_C(  -536.23),
        SIMDE_FLOAT32_C(  -147.17), SIMDE_FLOAT32_C(   971.04), SIMDE_FLOAT32_C(  -365.05), SIMDE_FLOAT32_C(   728.65),
        SIMDE_FLOAT32_C(   340.02), SIMDE_FLOAT32_C(  -288.54), SIMDE_FLOAT32_C(   298.61), SIMDE_FLOAT32_C(  -421.40),
        SIMDE_FLOAT32_C(    62.04), SIMDE_FLOAT32_C(   962.27), SIMDE_FLOAT32_C(  -847.19), SIMDE_FLOAT32_C(  -983.83) },
      UINT8_C( 93),
      { SIMDE_FLOAT32_C(  -474.95), SIMDE_FLOAT32_C(  -938.65), SIMDE_FLOAT32_C(  -941.09), SIMDE_FLOAT32_C(   980.71),
        SIMDE_FLOAT32_C(  -613.85), SIMDE_FLOAT32_C(   304.68), SIMDE_FLOAT32_C(  -395.19), SIMDE_FLOAT32_C(  -357.37),
        SIMDE_FLOAT32_C(   667.44), SIMDE_FLOAT32_C(   353.93), SIMDE_FLOAT32_C(   659.42), SIMDE_FLOAT32_C(   -91.27),
        SIMDE_FLOAT32_C(  -203.61), SIMDE_FLOAT32_C(  -908.72), SIMDE_FLOAT32_C(  -992.29), SIMDE_FLOAT32_C(  -290.45) },
      { SIMDE_FLOAT32_C(    -7.80), SIMDE_FLOAT32_C(   913.15), SIMDE_FLOAT32_C(    -9.80), SIMDE_FLOAT32_C(     9.94),
        SIMDE_FLOAT32_C(    -8.50), SIMDE_FLOAT32_C(   971.04), SIMDE_FLOAT32_C(    -7.34), SIMDE_FLOAT32_C(   728.65),
        SIMDE_FLOAT32_C(   340.02), SIMDE_FLOAT32_C(  -288.54), SIMDE_FLOAT32_C(   298.61), SIMDE_FLOAT32_C(  -421.40),
        SIMDE_FLOAT32_C(    62.04), SIMDE_FLOAT32_C(   962.27), SIMDE_FLOAT32_C(  -847.19), SIMDE_FLOAT32_C(  -983.83) } },
    { { SIMDE_FLOAT32_C(   781.81), SIMDE_FLOAT32_C(  -528.52), SIMDE_FLOAT32_C(   562.38), SIMDE_FLOAT32_C(   752.86),
        SIMDE_FLOAT32_C(   106.43), SIMDE_FLOAT32_C(   291.03), SIMDE_FLOAT32_C(    92.88), SIMDE_FLOAT32_C(   817.89),
        SIMDE_FLOAT32_C(  -410.36), SIMDE_FLOAT32_C(   671.48), SIMDE_FLOAT32_C(  -120.07), SIMDE_FLOAT32_C(  -448.09),
        SIMDE_FLOAT32_C(   824.29), SIMDE_FLOAT32_C(  -103.90), SIMDE_FLOAT32_C(  -767.52), SIMDE_FLOAT32_C(  -650.66) },
      UINT8_C( 13),
      { SIMDE_FLOAT32_C(  -708.61), SIMDE_FLOAT32_C(  -669.94), SIMDE_FLOAT32_C(   343.60), SIMDE_FLOAT32_C(   596.08),
        SIMDE_FLOAT32_C(   -65.13), SIMDE_FLOAT32_C(   986.24), SIMDE_FLOAT32_C(   263.52), SIMDE_FLOAT32_C(  -711.20),
        SIMDE_FLOAT32_C(   645.65), SIMDE_FLOAT32_C(  -827.76), SIMDE_FLOAT32_C(    85.19), SIMDE_FLOAT32_C(   736.94),
        SIMDE_FLOAT32_C(  -820.04), SIMDE_FLOAT32_C(   794.74), SIMDE_FLOAT32_C(   518.75), SIMDE_FLOAT32_C(  -348.56) },
      { SIMDE_FLOAT32_C(    -8.92), SIMDE_FLOAT32_C(  -528.52), SIMDE_FLOAT32_C(     7.00), SIMDE_FLOAT32_C(     8.42),
        SIMDE_FLOAT32_C(   106.43), SIMDE_FLOAT32_C(   291.03), SIMDE_FLOAT32_C(    92.88), SIMDE_FLOAT32_C(   817.89),
        SIMDE_FLOAT32_C(  -410.36), SIMDE_FLOAT32_C(   671.48), SIMDE_FLOAT32_C(  -120.07), SIMDE_FLOAT32_C(  -448.09),
        SIMDE_FLOAT32_C(   824.29), SIMDE_FLOAT32_C(  -103.90), SIMDE_FLOAT32_C(  -767.52), SIMDE_FLOAT32_C(  -650.66) } },
    { { SIMDE_FLOAT32_C(   357.12), SIMDE_FLOAT32_C(   271.61), SIMDE_FLOAT32_C(   757.87), SIMDE_FLOAT32_C(  -351.85),
        SIMDE_FLOAT32_C(  -635.52), SIMDE_FLOAT32_C(   575.76), SIMDE_FLOAT32_C(   237.78), SIMDE_FLOAT32_C(  -964.04),
        SIMDE_FLOAT32_C(  -544.31), SIMDE_FLOAT32_C(   789.69), SIMDE_FLOAT32_C(   860.25), SIMDE_FLOAT32_C(   351.79),
        SIMDE_FLOAT32_C(  -977.83), SIMDE_FLOAT32_C(  -790.40), SIMDE_FLOAT32_C(  -690.76), SIMDE_FLOAT32_C(  -686.43) },
      UINT8_C( 57),
      { SIMDE_FLOAT32_C(   652.85), SIMDE_FLOAT32_C(   909.64), SIMDE_FLOAT32_C(   474.52), SIMDE_FLOAT32_C(   639.08),
        SIMDE_FLOAT32_C(   173.16), SIMDE_FLOAT32_C(   763.32), SIMDE_FLOAT32_C(   284.74), SIMDE_FLOAT32_C(   345.41),
        SIMDE_FLOAT32_C(  -151.49), SIMDE_FLOAT32_C(    21.68), SIMDE_FLOAT32_C(   525.36), SIMDE_FLOAT32_C(  -356.75),
        SIMDE_FLOAT32_C(  -459.57), SIMDE_FLOAT32_C(  -823.20), SIMDE_FLOAT32_C(  -999.64), SIMDE_FLOAT32_C(   812.03) },
      { SIMDE_FLOAT32_C(     8.68), SIMDE_FLOAT32_C(   271.61), SIMDE_FLOAT32_C(   757.87), SIMDE_FLOAT32_C(     8.61),
        SIMDE_FLOAT32_C(     5.57), SIMDE_FLOAT32_C(     9.14), SIMDE_FLOAT32_C(   237.78), SIMDE_FLOAT32_C(  -964.04),
        SIMDE_FLOAT32_C(  -544.31), SIMDE_FLOAT32_C(   789.69), SIMDE_FLOAT32_C(   860.25), SIMDE_FLOAT32_C(   351.79),
        SIMDE_FLOAT32_C(  -977.83), SIMDE_FLOAT32_C(  -790.40), SIMDE_FLOAT32_C(  -690.76), SIMDE_FLOAT32_C(  -686.43) } },
    { { SIMDE_FLOAT32_C(   934.67), SIMDE_FLOAT32_C(  -351.49), SIMDE_FLOAT32_C(  -823.49), SIMDE_FLOAT32_C(   510.43),
        SIMDE_FLOAT32_C(   886.29), SIMDE_FLOAT32_C(  -787.53), SIMDE_FLOAT32_C(   966.12), SIMDE_FLOAT32_C(   675.98),
        SIMDE_FLOAT32_C(  -927.28), SIMDE_FLOAT32_C(   317.91), SIMDE_FLOAT32_C(   698.16), SIMDE_FLOAT32_C(  -717.68),
        SIMDE_FLOAT32_C(   627.15), SIMDE_FLOAT32_C(  -988.28), SIMDE_FLOAT32_C(  -178.03), SIMDE_FLOAT32_C(   279.99) },
      UINT8_C( 81),
      { SIMDE_FLOAT32_C(  -703.51), SIMDE_FLOAT32_C(   -80.92), SIMDE_FLOAT32_C(    94.53), SIMDE_FLOAT32_C(  -940.19),
        SIMDE_FLOAT32_C(  -796.18), SIMDE_FLOAT32_C(  -560.07), SIMDE_FLOAT32_C(   -91.68), SIMDE_FLOAT32_C(   225.49),
        SIMDE_FLOAT32_C(   965.29), SIMDE_FLOAT32_C(   551.56), SIMDE_FLOAT32_C(   765.92), SIMDE_FLOAT32_C(  -857.91),
        SIMDE_FLOAT32_C(   551.93), SIMDE_FLOAT32_C(   577.95), SIMDE_FLOAT32_C(  -923.23), SIMDE_FLOAT32_C(  -799.56) },
      { SIMDE_FLOAT32_C(    -8.89), SIMDE_FLOAT32_C(  -351.49), SIMDE_FLOAT32_C(  -823.49), SIMDE_FLOAT32_C(   510.43),
        SIMDE_FLOAT32_C(    -9.27), SIMDE_FLOAT32_C(  -787.53), SIMDE_FLOAT32_C(    -4.51), SIMDE_FLOAT32_C(   675.98),
        SIMDE_FLOAT32_C(  -927.28), SIMDE_FLOAT32_C(   317.91), SIMDE_FLOAT32_C(   698.16), SIMDE_FLOAT32_C(  -717.68),
        SIMDE_FLOAT32_C(   627.15), SIMDE_FLOAT32_C(  -988.28), SIMDE_FLOAT32_C(  -178.03), SIMDE_FLOAT32_C(   279.99) } },
    { { SIMDE_FLOAT32_C(   754.46), SIMDE_FLOAT32_C(   587.20), SIMDE_FLOAT32_C(  -913.27), SIMDE_FLOAT32_C(   966.93),
        SIMDE_FLOAT32_C(   553.32), SIMDE_FLOAT32_C(   762.71), SIMDE_FLOAT32_C(  -960.34), SIMDE_FLOAT32_C(  -128.78),
        SIMDE_FLOAT32_C(   460.87), SIMDE_FLOAT32_C(  -678.02), SIMDE_FLOAT32_C(  -501.63), SIMDE_FLOAT32_C(   472.59),
        SIMDE_FLOAT32_C(   143.95), SIMDE_FLOAT32_C(   778.36), SIMDE_FLOAT32_C(   393.95), SIMDE_FLOAT32_C(   440.44) },
      UINT8_C(131),
      { SIMDE_FLOAT32_C(  -511.52), SIMDE_FLOAT32_C(   500.25), SIMDE_FLOAT32_C(   -98.74), SIMDE_FLOAT32_C(   -71.59),
        SIMDE_FLOAT32_C(  -591.44), SIMDE_FLOAT32_C(  -873.25), SIMDE_FLOAT32_C(  -106.29), SIMDE_FLOAT32_C(   960.13),
        SIMDE_FLOAT32_C(   892.67), SIMDE_FLOAT32_C(    35.80), SIMDE_FLOAT32_C(   512.05), SIMDE_FLOAT32_C(   470.62),
        SIMDE_FLOAT32_C(   112.57), SIMDE_FLOAT32_C(   712.49), SIMDE_FLOAT32_C(   225.08), SIMDE_FLOAT32_C(  -300.23) },
      { SIMDE_FLOAT32_C(    -8.00), SIMDE_FLOAT32_C(     7.94), SIMDE_FLOAT32_C(  -913.27), SIMDE_FLOAT32_C(   966.93),
        SIMDE_FLOAT32_C(   553.32), SIMDE_FLOAT32_C(   762.71), SIMDE_FLOAT32_C(  -960.34), SIMDE_FLOAT32_C(     9.87),
        SIMDE_FLOAT32_C(   460.87), SIMDE_FLOAT32_C(  -678.02), SIMDE_FLOAT32_C(  -501.63), SIMDE_FLOAT32_C(   472.59),
        SIMDE_FLOAT32_C(   143.95), SIMDE_FLOAT32_C(   778.36), SIMDE_FLOAT32_C(   393.95), SIMDE_FLOAT32_C(   440.44) } },
    { { SIMDE_FLOAT32_C(   799.22), SIMDE_FLOAT32_C(   192.01), SIMDE_FLOAT32_C(  -746.92), SIMDE_FLOAT32_C(   561.93),
        SIMDE_FLOAT32_C(   231.67), SIMDE_FLOAT32_C(   124.30), SIMDE_FLOAT32_C(    22.80), SIMDE_FLOAT32_C(   553.64),
        SIMDE_FLOAT32_C(   622.67), SIMDE_FLOAT32_C(  -504.61), SIMDE_FLOAT32_C(  -302.41), SIMDE_FLOAT32_C(   401.04),
        SIMDE_FLOAT32_C(   889.34), SIMDE_FLOAT32_C(  -861.97), SIMDE_FLOAT32_C(  -901.52), SIMDE_FLOAT32_C(  -622.17) },
      UINT8_C(  8),
      { SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(   306.24), SIMDE_FLOAT32_C(  -953.16), SIMDE_FLOAT32_C(   126.49),
        SIMDE_FLOAT32_C(  -800.06), SIMDE_FLOAT32_C(  -993.04), SIMDE_FLOAT32_C(    19.16), SIMDE_FLOAT32_C(   235.74),
        SIMDE_FLOAT32_C(   519.02), SIMDE_FLOAT32_C(  -510.22), SIMDE_FLOAT32_C(  -651.69), SIMDE_FLOAT32_C(   231.50),
        SIMDE_FLOAT32_C(   714.86), SIMDE_FLOAT32_C(    48.08), SIMDE_FLOAT32_C(    30.72), SIMDE_FLOAT32_C(   -93.13) },
      { SIMDE_FLOAT32_C(   799.22), SIMDE_FLOAT32_C(   192.01), SIMDE_FLOAT32_C(  -746.92), SIMDE_FLOAT32_C(     5.02),
        SIMDE_FLOAT32_C(   231.67), SIMDE_FLOAT32_C(   124.30), SIMDE_FLOAT32_C(    22.80), SIMDE_FLOAT32_C(   553.64),
        SIMDE_FLOAT32_C(   622.67), SIMDE_FLOAT32_C(  -504.61), SIMDE_FLOAT32_C(  -302.41), SIMDE_FLOAT32_C(   401.04),
        SIMDE_FLOAT32_C(   889.34), SIMDE_FLOAT32_C(  -861.97), SIMDE_FLOAT32_C(  -901.52), SIMDE_FLOAT32_C(  -622.17) } },
    { { SIMDE_FLOAT32_C(   301.16), SIMDE_FLOAT32_C(  -407.35), SIMDE_FLOAT32_C(  -861.46), SIMDE_FLOAT32_C(  -574.54),
        SIMDE_FLOAT32_C(   615.45), SIMDE_FLOAT32_C(   692.19), SIMDE_FLOAT32_C(  -951.86), SIMDE_FLOAT32_C(  -889.16),
        SIMDE_FLOAT32_C(  -610.22), SIMDE_FLOAT32_C(   449.17), SIMDE_FLOAT32_C(  -999.81), SIMDE_FLOAT32_C(  -472.20),
        SIMDE_FLOAT32_C(   547.65), SIMDE_FLOAT32_C(  -621.98), SIMDE_FLOAT32_C(  -833.92), SIMDE_FLOAT32_C(  -452.61) },
      UINT8_C( 61),
      { SIMDE_FLOAT32_C(  -787.08), SIMDE_FLOAT32_C(   673.88), SIMDE_FLOAT32_C(   884.20), SIMDE_FLOAT32_C(  -780.12),
        SIMDE_FLOAT32_C(  -306.96), SIMDE_FLOAT32_C(   119.94), SIMDE_FLOAT32_C(   738.89), SIMDE_FLOAT32_C(   182.83),
        SIMDE_FLOAT32_C(   468.25), SIMDE_FLOAT32_C(   -29.60), SIMDE_FLOAT32_C(  -102.31), SIMDE_FLOAT32_C(  -483.67),
        SIMDE_FLOAT32_C(  -998.88), SIMDE_FLOAT32_C(   804.56), SIMDE_FLOAT32_C(   817.49), SIMDE_FLOAT32_C(  -406.23) },
      { SIMDE_FLOAT32_C(    -9.23), SIMDE_FLOAT32_C(  -407.35), SIMDE_FLOAT32_C(     9.60), SIMDE_FLOAT32_C(    -9.21),
        SIMDE_FLOAT32_C(    -6.75), SIMDE_FLOAT32_C(     4.93), SIMDE_FLOAT32_C(  -951.86), SIMDE_FLOAT32_C(  -889.16),
        SIMDE_FLOAT32_C(  -610.22), SIMDE_FLOAT32_C(   449.17), SIMDE_FLOAT32_C(  -999.81), SIMDE_FLOAT32_C(  -472.20),
        SIMDE_FLOAT32_C(   547.65), SIMDE_FLOAT32_C(  -621.98), SIMDE_FLOAT32_C(  -833.92), SIMDE_FLOAT32_C(  -452.61) } },
    { { SIMDE_FLOAT32_C(   943.11), SIMDE_FLOAT32_C(  -757.05), SIMDE_FLOAT32_C(  -790.77), SIMDE_FLOAT32_C(   635.29),
        SIMDE_FLOAT32_C(  -708.91), SIMDE_FLOAT32_C(  -679.93), SIMDE_FLOAT32_C(  -974.93), SIMDE_FLOAT32_C(   740.26),
        SIMDE_FLOAT32_C(  -679.74), SIMDE_FLOAT32_C(  -447.13), SIMDE_FLOAT32_C(   287.91), SIMDE_FLOAT32_C(  -301.72),
        SIMDE_FLOAT32_C(  -281.05), SIMDE_FLOAT32_C(   835.30), SIMDE_FLOAT32_C(  -617.47), SIMDE_FLOAT32_C(   -68.13) },
      UINT8_C(116),
      { SIMDE_FLOAT32_C(  -733.27), SIMDE_FLOAT32_C(   151.75), SIMDE_FLOAT32_C(  -797.77), SIMDE_FLOAT32_C(   386.67),
        SIMDE_FLOAT32_C(  -109.36), SIMDE_FLOAT32_C(   385.06), SIMDE_FLOAT32_C(  -145.07), SIMDE_FLOAT32_C(   861.04),
        SIMDE_FLOAT32_C(  -717.26), SIMDE_FLOAT32_C(   371.26), SIMDE_FLOAT32_C(   862.16), SIMDE_FLOAT32_C(  -912.69),
        SIMDE_FLOAT32_C(   188.75), SIMDE_FLOAT32_C(  -544.07), SIMDE_FLOAT32_C(  -969.58), SIMDE_FLOAT32_C(   431.70) },
      { SIMDE_FLOAT32_C(   943.11), SIMDE_FLOAT32_C(  -757.05), SIMDE_FLOAT32_C(    -9.27), SIMDE_FLOAT32_C(   635.29),
        SIMDE_FLOAT32_C(    -4.78), SIMDE_FLOAT32_C(     7.28), SIMDE_FLOAT32_C(    -5.25), SIMDE_FLOAT32_C(   740.26),
        SIMDE_FLOAT32_C(  -679.74), SIMDE_FLOAT32_C(  -447.13), SIMDE_FLOAT32_C(   287.91), SIMDE_FLOAT32_C(  -301.72),
        SIMDE_FLOAT32_C(  -281.05), SIMDE_FLOAT32_C(   835.30), SIMDE_FLOAT32_C(  -617.47), SIMDE_FLOAT32_C(   -68.13) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_cbrt_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cbrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -759.76), SIMDE_FLOAT64_C(   815.00), SIMDE_FLOAT64_C(  -816.92), SIMDE_FLOAT64_C(   967.48),
        SIMDE_FLOAT64_C(  -635.21), SIMDE_FLOAT64_C(   789.99), SIMDE_FLOAT64_C(  -526.03), SIMDE_FLOAT64_C(  -914.28) },
      UINT8_C( 48),
      { SIMDE_FLOAT64_C(    53.18), SIMDE_FLOAT64_C(   207.38), SIMDE_FLOAT64_C(  -889.97), SIMDE_FLOAT64_C(  -694.52),
        SIMDE_FLOAT64_C(    45.75), SIMDE_FLOAT64_C(    94.09), SIMDE_FLOAT64_C(  -391.74), SIMDE_FLOAT64_C(   959.63) },
      { SIMDE_FLOAT64_C(  -759.76), SIMDE_FLOAT64_C(   815.00), SIMDE_FLOAT64_C(  -816.92), SIMDE_FLOAT64_C(   967.48),
        SIMDE_FLOAT64_C(     3.58), SIMDE_FLOAT64_C(     4.55), SIMDE_FLOAT64_C(  -526.03), SIMDE_FLOAT64_C(  -914.28) } },
    { { SIMDE_FLOAT64_C(   378.72), SIMDE_FLOAT64_C(  -982.35), SIMDE_FLOAT64_C(  -413.18), SIMDE_FLOAT64_C(   706.92),
        SIMDE_FLOAT64_C(   679.73), SIMDE_FLOAT64_C(   156.25), SIMDE_FLOAT64_C(   267.05), SIMDE_FLOAT64_C(  -563.13) },
      UINT8_C( 62),
      { SIMDE_FLOAT64_C(  -595.59), SIMDE_FLOAT64_C(  -667.14), SIMDE_FLOAT64_C(  -678.76), SIMDE_FLOAT64_C(   -24.40),
        SIMDE_FLOAT64_C(   817.42), SIMDE_FLOAT64_C(  -438.52), SIMDE_FLOAT64_C(  -209.40), SIMDE_FLOAT64_C(  -999.49) },
      { SIMDE_FLOAT64_C(   378.72), SIMDE_FLOAT64_C(    -8.74), SIMDE_FLOAT64_C(    -8.79), SIMDE_FLOAT64_C(    -2.90),
        SIMDE_FLOAT64_C(     9.35), SIMDE_FLOAT64_C(    -7.60), SIMDE_FLOAT64_C(   267.05), SIMDE_FLOAT64_C(  -563.13) } },
    { { SIMDE_FLOAT64_C(  -471.03), SIMDE_FLOAT64_C(   155.40), SIMDE_FLOAT64_C(   790.50), SIMDE_FLOAT64_C(     2.94),
        SIMDE_FLOAT64_C(   241.12), SIMDE_FLOAT64_C(   295.11), SIMDE_FLOAT64_C(  -943.89), SIMDE_FLOAT64_C(  -551.50) },
      UINT8_C(142),
      { SIMDE_FLOAT64_C(  -638.40), SIMDE_FLOAT64_C(   494.25), SIMDE_FLOAT64_C(  -500.77), SIMDE_FLOAT64_C(   -30.15),
        SIMDE_FLOAT64_C(   453.88), SIMDE_FLOAT64_C(   877.94), SIMDE_FLOAT64_C(   -12.50), SIMDE_FLOAT64_C(  -959.30) },
      { SIMDE_FLOAT64_C(  -471.03), SIMDE_FLOAT64_C(     7.91), SIMDE_FLOAT64_C(    -7.94), SIMDE_FLOAT64_C(    -3.11),
        SIMDE_FLOAT64_C(   241.12), SIMDE_FLOAT64_C(   295.11), SIMDE_FLOAT64_C(  -943.89), SIMDE_FLOAT64_C(    -9.86) } },
    { { SIMDE_FLOAT64_C(   584.87), SIMDE_FLOAT64_C(  -332.77), SIMDE_FLOAT64_C(   196.95), SIMDE_FLOAT64_C(  -148.09),
        SIMDE_FLOAT64_C(   104.11), SIMDE_FLOAT64_C(  -809.90), SIMDE_FLOAT64_C(   256.33), SIMDE_FLOAT64_C(   436.96) },
      UINT8_C(231),
      { SIMDE_FLOAT64_C(  -768.07), SIMDE_FLOAT64_C(   254.39), SIMDE_FLOAT64_C(    72.83), SIMDE_FLOAT64_C(    22.53),
        SIMDE_FLOAT64_C(   254.89), SIMDE_FLOAT64_C(   601.79), SIMDE_FLOAT64_C(  -822.07), SIMDE_FLOAT64_C(    45.39) },
      { SIMDE_FLOAT64_C(    -9.16), SIMDE_FLOAT64_C(     6.34), SIMDE_FLOAT64_C(     4.18), SIMDE_FLOAT64_C(  -148.09),
        SIMDE_FLOAT64_C(   104.11), SIMDE_FLOAT64_C(     8.44), SIMDE_FLOAT64_C(    -9.37), SIMDE_FLOAT64_C(     3.57) } },
    { { SIMDE_FLOAT64_C(  -395.27), SIMDE_FLOAT64_C(   419.05), SIMDE_FLOAT64_C(  -659.50), SIMDE_FLOAT64_C(  -339.16),
        SIMDE_FLOAT64_C(   867.55), SIMDE_FLOAT64_C(   745.64), SIMDE_FLOAT64_C(    22.44), SIMDE_FLOAT64_C(   361.79) },
      UINT8_C( 20),
      { SIMDE_FLOAT64_C(   992.29), SIMDE_FLOAT64_C(  -184.33), SIMDE_FLOAT64_C(  -877.19), SIMDE_FLOAT64_C(   -20.21),
        SIMDE_FLOAT64_C(  -143.62), SIMDE_FLOAT64_C(   707.68), SIMDE_FLOAT64_C(   647.03), SIMDE_FLOAT64_C(  -946.67) },
      { SIMDE_FLOAT64_C(  -395.27), SIMDE_FLOAT64_C(   419.05), SIMDE_FLOAT64_C(    -9.57), SIMDE_FLOAT64_C(  -339.16),
        SIMDE_FLOAT64_C(    -5.24), SIMDE_FLOAT64_C(   745.64), SIMDE_FLOAT64_C(    22.44), SIMDE_FLOAT64_C(   361.79) } },
    { { SIMDE_FLOAT64_C(  -440.41), SIMDE_FLOAT64_C(  -248.87), SIMDE_FLOAT64_C(  -756.57), SIMDE_FLOAT64_C(   815.92),
        SIMDE_FLOAT64_C(  -811.90), SIMDE_FLOAT64_C(  -245.23), SIMDE_FLOAT64_C(  -952.16), SIMDE_FLOAT64_C(   442.48) },
      UINT8_C( 34),
      { SIMDE_FLOAT64_C(    70.37), SIMDE_FLOAT64_C(  -302.63), SIMDE_FLOAT64_C(   429.40), SIMDE_FLOAT64_C(   248.30),
        SIMDE_FLOAT64_C(   742.77), SIMDE_FLOAT64_C(  -965.87), SIMDE_FLOAT64_C(  -332.65), SIMDE_FLOAT64_C(  -916.73) },
      { SIMDE_FLOAT64_C(  -440.41), SIMDE_FLOAT64_C(    -6.71), SIMDE_FLOAT64_C(  -756.57), SIMDE_FLOAT64_C(   815.92),
        SIMDE_FLOAT64_C(  -811.90), SIMDE_FLOAT64_C(    -9.88), SIMDE_FLOAT64_C(  -952.16), SIMDE_FLOAT64_C(   442.48) } },
    { { SIMDE_FLOAT64_C(  -305.03), SIMDE_FLOAT64_C(  -465.11), SIMDE_FLOAT64_C(   828.91), SIMDE_FLOAT64_C(   717.41),
        SIMDE_FLOAT64_C(   896.69), SIMDE_FLOAT64_C(  -926.23), SIMDE_FLOAT64_C(   709.70), SIMDE_FLOAT64_C(  -287.64) },
      UINT8_C( 68),
      { SIMDE_FLOAT64_C(  -310.50), SIMDE_FLOAT64_C(   568.74), SIMDE_FLOAT64_C(   904.26), SIMDE_FLOAT64_C(  -663.47),
        SIMDE_FLOAT64_C(   622.07), SIMDE_FLOAT64_C(  -536.15), SIMDE_FLOAT64_C(    87.66), SIMDE_FLOAT64_C(   865.50) },
      { SIMDE_FLOAT64_C(  -305.03), SIMDE_FLOAT64_C(  -465.11), SIMDE_FLOAT64_C(     9.67), SIMDE_FLOAT64_C(   717.41),
        SIMDE_FLOAT64_C(   896.69), SIMDE_FLOAT64_C(  -926.23), SIMDE_FLOAT64_C(     4.44), SIMDE_FLOAT64_C(  -287.64) } },
    { { SIMDE_FLOAT64_C(  -720.23), SIMDE_FLOAT64_C(   275.76), SIMDE_FLOAT64_C(  -379.73), SIMDE_FLOAT64_C(  -672.39),
        SIMDE_FLOAT64_C(  -281.76), SIMDE_FLOAT64_C(  -552.12), SIMDE_FLOAT64_C(   397.98), SIMDE_FLOAT64_C(   415.61) },
      UINT8_C(204),
      { SIMDE_FLOAT64_C(  -353.72), SIMDE_FLOAT64_C(   158.38), SIMDE_FLOAT64_C(   911.40), SIMDE_FLOAT64_C(   313.63),
        SIMDE_FLOAT64_C(   241.65), SIMDE_FLOAT64_C(  -393.63), SIMDE_FLOAT64_C(   848.52), SIMDE_FLOAT64_C(    70.56) },
      { SIMDE_FLOAT64_C(  -720.23), SIMDE_FLOAT64_C(   275.76), SIMDE_FLOAT64_C(     9.70), SIMDE_FLOAT64_C(     6.79),
        SIMDE_FLOAT64_C(  -281.76), SIMDE_FLOAT64_C(  -552.12), SIMDE_FLOAT64_C(     9.47), SIMDE_FLOAT64_C(     4.13) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_cbrt_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_cos_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.95),
                         SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -0.94),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.88),
                         SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.49)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -0.61),
                         SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.48),
                         SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     1.00),
                         SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(     0.85)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     1.00),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(    -0.95),
                         SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.74)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.27),
                         SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.67),
                         SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -0.99)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(   343.48),
                         SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   655.67),
                         SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(   120.65), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.50),
                         SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(    -0.38),
                         SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.60),
                         SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -0.29)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -591.56),
                         SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(  -906.16),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -738.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.09),
                         SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(     0.19),
                         SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -1.00)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(   977.49),
                         SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.17),
                         SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(    -0.00),
                         SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.90),
                         SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.62)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -696.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(    -0.63),
                         SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.74)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_cos_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cos_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(    -0.96),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.75),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(   346.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19),
                         SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   254.31), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(   398.82), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   -30.79), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
                         SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(     0.40)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    99.93),
                         SIMDE_FLOAT32_C(  -738.19), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -448.89),
                         SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(   331.34),
                         SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -874.31),
                         SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   -70.91)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    -0.10),
                         SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -337.60),
                         SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(  -756.42)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.17),
                         SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    -0.90)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -737.13), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(   177.92),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   888.71), SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   133.52),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(  -775.04), SIMDE_FLOAT32_C(   440.64)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(   915.19), SIMDE_FLOAT32_C(  -718.40), SIMDE_FLOAT32_C(   159.97),
                         SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   110.36),
                         SIMDE_FLOAT32_C(   826.84), SIMDE_FLOAT32_C(   -76.75), SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -378.50),
                         SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(  -942.47), SIMDE_FLOAT32_C(   475.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(    -0.97),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   440.64)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(  -807.28), SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(   834.60), SIMDE_FLOAT32_C(   -65.60),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   556.35), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -964.25), SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(  -743.66), SIMDE_FLOAT32_C(  -764.58),
                         SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(  -818.54), SIMDE_FLOAT32_C(   161.06),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -308.52), SIMDE_FLOAT32_C(  -719.57),
                         SIMDE_FLOAT32_C(   334.00), SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(  -916.82), SIMDE_FLOAT32_C(  -490.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.67),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   105.79), SIMDE_FLOAT32_C(   590.10),
                         SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(   -84.00), SIMDE_FLOAT32_C(    80.04),
                         SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(  -889.11)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(   144.05), SIMDE_FLOAT32_C(  -427.72),
                         SIMDE_FLOAT32_C(   308.28), SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(  -457.77), SIMDE_FLOAT32_C(   678.24),
                         SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   117.28), SIMDE_FLOAT32_C(  -576.80),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   -53.92), SIMDE_FLOAT32_C(    91.94)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.94),
                         SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(     0.31),
                         SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(    -0.67)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(  -493.41), SIMDE_FLOAT32_C(   822.72),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(  -816.27),
                         SIMDE_FLOAT32_C(  -209.34), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(  -204.33)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -841.43), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(   824.88), SIMDE_FLOAT32_C(   793.63),
                         SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -350.72),
                         SIMDE_FLOAT32_C(    60.89), SIMDE_FLOAT32_C(   109.81), SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -250.60),
                         SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   -13.07), SIMDE_FLOAT32_C(   852.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.37),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(     0.42),
                         SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(    -0.34)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_cos_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_cos_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.59), SIMDE_FLOAT64_C(   -0.47),
                         SIMDE_FLOAT64_C(   -0.54), SIMDE_FLOAT64_C(   -0.88),
                         SIMDE_FLOAT64_C(   -0.66), SIMDE_FLOAT64_C(    0.26),
                         SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(    0.49)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(    0.30),
                         SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.95),
                         SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.33),
                         SIMDE_FLOAT64_C(    0.75), SIMDE_FLOAT64_C(   -0.94)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.27), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(    0.73), SIMDE_FLOAT64_C(    1.00),
                         SIMDE_FLOAT64_C(   -0.94), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(   -0.96), SIMDE_FLOAT64_C(    0.85)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.64), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(    0.63), SIMDE_FLOAT64_C(   -0.61),
                         SIMDE_FLOAT64_C(   -0.85), SIMDE_FLOAT64_C(    0.46),
                         SIMDE_FLOAT64_C(   -0.88), SIMDE_FLOAT64_C(    0.48)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(  993.90),
                         SIMDE_FLOAT64_C(   28.08), SIMDE_FLOAT64_C(  841.21)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.79), SIMDE_FLOAT64_C(   -0.87),
                         SIMDE_FLOAT64_C(    0.74), SIMDE_FLOAT64_C(   -0.95),
                         SIMDE_FLOAT64_C(   -0.51), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(    0.74)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C(  -30.79)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.09), SIMDE_FLOAT64_C(    1.00),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    1.00),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    1.00),
                         SIMDE_FLOAT64_C(    0.05), SIMDE_FLOAT64_C(    0.81)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  -80.73),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -767.23),
                         SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C(  398.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.85), SIMDE_FLOAT64_C(    0.58),
                         SIMDE_FLOAT64_C(   -0.84), SIMDE_FLOAT64_C(   -0.67),
                         SIMDE_FLOAT64_C(    0.99), SIMDE_FLOAT64_C(    0.78),
                         SIMDE_FLOAT64_C(    0.30), SIMDE_FLOAT64_C(   -0.99)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C(  254.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.01), SIMDE_FLOAT64_C(    0.13),
                         SIMDE_FLOAT64_C(   -0.51), SIMDE_FLOAT64_C(   -0.27),
                         SIMDE_FLOAT64_C(    0.51), SIMDE_FLOAT64_C(   -0.91),
                         SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(   -0.99)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_cos_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cos_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45),
                         SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(    0.59), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   -0.66), SIMDE_FLOAT64_C(    0.92)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  841.21), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  687.09), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -660.80), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -923.64), SIMDE_FLOAT64_C( -860.95)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.74), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(   -0.61), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C(    0.99)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  398.82), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  339.21), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  -30.79), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(  993.90)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C( -387.90),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  532.35),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C( -770.35),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C( -770.72)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.30), SIMDE_FLOAT64_C(   -0.09),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(    0.05), SIMDE_FLOAT64_C(   -0.79),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(   -0.51)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(  469.66),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  910.03),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C( -203.65),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -747.59)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C( -171.51),
                         SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(   -0.29),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(   -0.27),
                         SIMDE_FLOAT64_C(   -0.91), SIMDE_FLOAT64_C(   -0.99),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C(   -0.67)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   99.93), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(  343.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(  655.67)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  331.34), SIMDE_FLOAT64_C(  462.95),
                         SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  324.62),
                         SIMDE_FLOAT64_C( -874.31), SIMDE_FLOAT64_C( -328.54),
                         SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  561.36)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.10), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(   -0.51),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(   -0.55)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   27.25),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  831.02), SIMDE_FLOAT64_C(  977.36)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  977.49), SIMDE_FLOAT64_C(  424.81),
                         SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  840.65),
                         SIMDE_FLOAT64_C( -591.56), SIMDE_FLOAT64_C(  731.49),
                         SIMDE_FLOAT64_C(  623.70), SIMDE_FLOAT64_C(  140.67)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(    0.59), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(   -0.09), SIMDE_FLOAT64_C(   -0.76)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C( -304.73),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(  822.06),
                         SIMDE_FLOAT64_C( -997.63), SIMDE_FLOAT64_C(  923.64),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(  -67.64)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  510.85), SIMDE_FLOAT64_C(   14.34),
                         SIMDE_FLOAT64_C(  916.26), SIMDE_FLOAT64_C( -769.09),
                         SIMDE_FLOAT64_C( -573.81), SIMDE_FLOAT64_C( -337.60),
                         SIMDE_FLOAT64_C(  293.64), SIMDE_FLOAT64_C( -576.22)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(   -0.83),
                         SIMDE_FLOAT64_C(   -0.45), SIMDE_FLOAT64_C(   -0.12),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(   -0.26)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  475.51), SIMDE_FLOAT64_C(  936.65),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C( -438.19),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(  932.66),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C( -182.45)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(  440.64),
                         SIMDE_FLOAT64_C(  897.27), SIMDE_FLOAT64_C( -197.89),
                         SIMDE_FLOAT64_C( -359.76), SIMDE_FLOAT64_C(  -33.67),
                         SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -125.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.60), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C(   -1.00),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(   -0.63),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(    0.89)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_cos_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_cosd_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.97)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.83)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.31)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(    -0.85)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.92)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    -0.92)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(     0.51)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.84)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_cosd_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_cosd_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.97)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(    0.78)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.46), SIMDE_FLOAT64_C(    0.83)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.74), SIMDE_FLOAT64_C(    0.65)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.01), SIMDE_FLOAT64_C(   -0.31)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.09), SIMDE_FLOAT64_C(   -0.85)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.75), SIMDE_FLOAT64_C(    0.83)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_cosd_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_cosd_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24),
                         SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01),
                         SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.65),
                         SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.83),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.97)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13),
                         SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21),
                         SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.83),
                         SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(    -0.85),
                         SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.31)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95),
                         SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.88),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.78),
                         SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.92)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67),
                         SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54),
                         SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.01),
                         SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.84),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.84),
                         SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(     0.51)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48),
                         SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     0.94),
                         SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.07),
                         SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -0.52)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92),
                         SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.10),
                         SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.86)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73),
                         SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.71),
                         SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.78)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02),
                         SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.77),
                         SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(    -0.15),
                         SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.49),
                         SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    -0.27)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_cosd_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_cosd_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(    0.78),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.97)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.74), SIMDE_FLOAT64_C(    0.65),
                         SIMDE_FLOAT64_C(    0.46), SIMDE_FLOAT64_C(    0.83)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(   -0.01), SIMDE_FLOAT64_C(   -0.31)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.75), SIMDE_FLOAT64_C(    0.83),
                         SIMDE_FLOAT64_C(    0.09), SIMDE_FLOAT64_C(   -0.85)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.57), SIMDE_FLOAT64_C(   -0.78),
                         SIMDE_FLOAT64_C(    0.54), SIMDE_FLOAT64_C(    0.92)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.09), SIMDE_FLOAT64_C(    0.88),
                         SIMDE_FLOAT64_C(    0.91), SIMDE_FLOAT64_C(   -0.92)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(   -0.84),
                         SIMDE_FLOAT64_C(   -0.23), SIMDE_FLOAT64_C(    0.51)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(   -0.01),
                         SIMDE_FLOAT64_C(   -0.60), SIMDE_FLOAT64_C(    0.84)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_cosd_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_cosd_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(    -0.85),
                         SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.31),
                         SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.83),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.97)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.84),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(     0.51),
                         SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.92)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.86),
                         SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     0.94),
                         SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -0.52)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(    -0.15),
                         SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    -0.27),
                         SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.71),
                         SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.78)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(   343.48),
                         SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   655.67),
                         SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(   120.65), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(    -0.97),
                         SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.43),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.99)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -591.56),
                         SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(  -906.16),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -738.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.62),
                         SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.11),
                         SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(     0.95)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(   977.49),
                         SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(     0.13),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.67),
                         SIMDE_FLOAT32_C(    -0.81), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.22),
                         SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.09)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -696.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.83),
                         SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.58),
                         SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(     0.92)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_cosd_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cosd_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(     0.54),
                         SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(    -0.01),
                         SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(   346.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19),
                         SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   254.31), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(   398.82), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   -30.79), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.10),
                         SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(     0.07)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    99.93),
                         SIMDE_FLOAT32_C(  -738.19), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -448.89),
                         SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(   331.34),
                         SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -874.31),
                         SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   -70.91)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(     0.88),
                         SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -337.60),
                         SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(  -756.42)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(     0.13),
                         SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(    -0.22)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -737.13), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(   177.92),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   888.71), SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   133.52),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(  -775.04), SIMDE_FLOAT32_C(   440.64)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(   915.19), SIMDE_FLOAT32_C(  -718.40), SIMDE_FLOAT32_C(   159.97),
                         SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   110.36),
                         SIMDE_FLOAT32_C(   826.84), SIMDE_FLOAT32_C(   -76.75), SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -378.50),
                         SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(  -942.47), SIMDE_FLOAT32_C(   475.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(    -0.94),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(    -0.35),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(   440.64)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(  -807.28), SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(   834.60), SIMDE_FLOAT32_C(   -65.60),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   556.35), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -964.25), SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(  -743.66), SIMDE_FLOAT32_C(  -764.58),
                         SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(  -818.54), SIMDE_FLOAT32_C(   161.06),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -308.52), SIMDE_FLOAT32_C(  -719.57),
                         SIMDE_FLOAT32_C(   334.00), SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(  -916.82), SIMDE_FLOAT32_C(  -490.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.95),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   105.79), SIMDE_FLOAT32_C(   590.10),
                         SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(   -84.00), SIMDE_FLOAT32_C(    80.04),
                         SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(  -889.11)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(   144.05), SIMDE_FLOAT32_C(  -427.72),
                         SIMDE_FLOAT32_C(   308.28), SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(  -457.77), SIMDE_FLOAT32_C(   678.24),
                         SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   117.28), SIMDE_FLOAT32_C(  -576.80),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   -53.92), SIMDE_FLOAT32_C(    91.94)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(     0.75),
                         SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.80),
                         SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(    -0.03)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(  -493.41), SIMDE_FLOAT32_C(   822.72),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(  -816.27),
                         SIMDE_FLOAT32_C(  -209.34), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(  -204.33)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -841.43), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(   824.88), SIMDE_FLOAT32_C(   793.63),
                         SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -350.72),
                         SIMDE_FLOAT32_C(    60.89), SIMDE_FLOAT32_C(   109.81), SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -250.60),
                         SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   -13.07), SIMDE_FLOAT32_C(   852.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.28),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(    -0.68)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_cosd_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_cosd_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
     { simde_mm512_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.74), SIMDE_FLOAT64_C(    0.65),
                         SIMDE_FLOAT64_C(    0.46), SIMDE_FLOAT64_C(    0.83),
                         SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(    0.78),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.97)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.75), SIMDE_FLOAT64_C(    0.83),
                         SIMDE_FLOAT64_C(    0.09), SIMDE_FLOAT64_C(   -0.85),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(   -0.01), SIMDE_FLOAT64_C(   -0.31)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.09), SIMDE_FLOAT64_C(    0.88),
                         SIMDE_FLOAT64_C(    0.91), SIMDE_FLOAT64_C(   -0.92),
                         SIMDE_FLOAT64_C(    0.57), SIMDE_FLOAT64_C(   -0.78),
                         SIMDE_FLOAT64_C(    0.54), SIMDE_FLOAT64_C(    0.92)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(   -0.01),
                         SIMDE_FLOAT64_C(   -0.60), SIMDE_FLOAT64_C(    0.84),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(   -0.84),
                         SIMDE_FLOAT64_C(   -0.23), SIMDE_FLOAT64_C(    0.51)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(  993.90),
                         SIMDE_FLOAT64_C(   28.08), SIMDE_FLOAT64_C(  841.21)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.64), SIMDE_FLOAT64_C(    0.11),
                         SIMDE_FLOAT64_C(   -0.72), SIMDE_FLOAT64_C(    0.94),
                         SIMDE_FLOAT64_C(    0.63), SIMDE_FLOAT64_C(    0.07),
                         SIMDE_FLOAT64_C(    0.88), SIMDE_FLOAT64_C(   -0.52)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C(  -30.79)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.88), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(    0.44), SIMDE_FLOAT64_C(    0.93),
                         SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(   -0.10),
                         SIMDE_FLOAT64_C(    0.49), SIMDE_FLOAT64_C(    0.86)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  -80.73),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -767.23),
                         SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C(  398.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.92), SIMDE_FLOAT64_C(    0.16),
                         SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(   -0.71),
                         SIMDE_FLOAT64_C(    0.89), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C(   -0.97), SIMDE_FLOAT64_C(    0.78)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C(  254.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.34), SIMDE_FLOAT64_C(    0.77),
                         SIMDE_FLOAT64_C(   -0.85), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(   -0.49),
                         SIMDE_FLOAT64_C(    0.32), SIMDE_FLOAT64_C(   -0.27)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_cosd_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cosd_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45),
                         SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.75), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(   -0.74), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(    0.83)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  841.21), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  687.09), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -660.80), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -923.64), SIMDE_FLOAT64_C( -860.95)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.52), SIMDE_FLOAT64_C(   -0.01),
                         SIMDE_FLOAT64_C(    0.84), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(    0.88),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C(   -0.78)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  398.82), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  339.21), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  -30.79), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(  993.90)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C( -387.90),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  532.35),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C( -770.35),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C( -770.72)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.97), SIMDE_FLOAT64_C(    0.88),
                         SIMDE_FLOAT64_C(    0.44), SIMDE_FLOAT64_C(   -0.99),
                         SIMDE_FLOAT64_C(    0.49), SIMDE_FLOAT64_C(    0.64),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(    0.63)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(  469.66),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  910.03),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C( -203.65),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -747.59)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C( -171.51),
                         SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(   -0.99),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(   -0.49), SIMDE_FLOAT64_C(   -0.27),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C(   -0.71)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   99.93), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(  343.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(  655.67)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  331.34), SIMDE_FLOAT64_C(  462.95),
                         SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  324.62),
                         SIMDE_FLOAT64_C( -874.31), SIMDE_FLOAT64_C( -328.54),
                         SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  561.36)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.88), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(   -0.93)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   27.25),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  831.02), SIMDE_FLOAT64_C(  977.36)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  977.49), SIMDE_FLOAT64_C(  424.81),
                         SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  840.65),
                         SIMDE_FLOAT64_C( -591.56), SIMDE_FLOAT64_C(  731.49),
                         SIMDE_FLOAT64_C(  623.70), SIMDE_FLOAT64_C(  140.67)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(    0.43),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(   -0.62), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(   -0.11), SIMDE_FLOAT64_C(   -0.77)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C( -304.73),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(  822.06),
                         SIMDE_FLOAT64_C( -997.63), SIMDE_FLOAT64_C(  923.64),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(  -67.64)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  510.85), SIMDE_FLOAT64_C(   14.34),
                         SIMDE_FLOAT64_C(  916.26), SIMDE_FLOAT64_C( -769.09),
                         SIMDE_FLOAT64_C( -573.81), SIMDE_FLOAT64_C( -337.60),
                         SIMDE_FLOAT64_C(  293.64), SIMDE_FLOAT64_C( -576.22)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C(    0.97),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(    0.65),
                         SIMDE_FLOAT64_C(   -0.83), SIMDE_FLOAT64_C(    0.92),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(   -0.81)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  475.51), SIMDE_FLOAT64_C(  936.65),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C( -438.19),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(  932.66),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C( -182.45)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(  440.64),
                         SIMDE_FLOAT64_C(  897.27), SIMDE_FLOAT64_C( -197.89),
                         SIMDE_FLOAT64_C( -359.76), SIMDE_FLOAT64_C(  -33.67),
                         SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -125.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.57), SIMDE_FLOAT64_C(    0.16),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C(   -0.95),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(    0.83),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(   -0.58)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_cosd_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_cosh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     3.48), SIMDE_FLOAT32_C(     4.71), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     6.41)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    16.25), SIMDE_FLOAT32_C(    55.53), SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(   303.95)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(     8.19), SIMDE_FLOAT32_C(     2.86), SIMDE_FLOAT32_C(     4.69)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(   697.05), SIMDE_FLOAT32_C(  1802.36), SIMDE_FLOAT32_C(     8.76), SIMDE_FLOAT32_C(    54.43)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(     6.82), SIMDE_FLOAT32_C(     3.02), SIMDE_FLOAT32_C(     7.07)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(  4216.89), SIMDE_FLOAT32_C(   457.99), SIMDE_FLOAT32_C(    10.27), SIMDE_FLOAT32_C(   588.07)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     4.97), SIMDE_FLOAT32_C(     7.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(    72.02), SIMDE_FLOAT32_C(  1039.87)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     8.33)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     8.42), SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C(     4.57), SIMDE_FLOAT32_C(  2073.21)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(     4.66), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(    -0.58)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(    52.82), SIMDE_FLOAT32_C(     5.50), SIMDE_FLOAT32_C(     1.17)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     5.94), SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(     0.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(   189.97), SIMDE_FLOAT32_C(    13.99), SIMDE_FLOAT32_C(     1.40), SIMDE_FLOAT32_C(     1.40)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     5.48), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     5.78), SIMDE_FLOAT32_C(     8.28)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(   119.93), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(   161.88), SIMDE_FLOAT32_C(  1972.10)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_cosh_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }


  return 0;
}

static int
test_simde_mm_cosh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.26), SIMDE_FLOAT64_C(    3.04)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.03), SIMDE_FLOAT64_C(   10.48)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    1.44), SIMDE_FLOAT64_C(    2.12)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    2.23), SIMDE_FLOAT64_C(    4.23)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    1.11), SIMDE_FLOAT64_C(    2.10)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.68), SIMDE_FLOAT64_C(    4.14)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    3.49), SIMDE_FLOAT64_C(    4.01)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   16.41), SIMDE_FLOAT64_C(   27.58)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    1.19), SIMDE_FLOAT64_C(    3.40)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.80), SIMDE_FLOAT64_C(   15.00)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    4.48), SIMDE_FLOAT64_C(    3.27)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   44.12), SIMDE_FLOAT64_C(   13.17)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    2.25), SIMDE_FLOAT64_C(    3.71)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    4.80), SIMDE_FLOAT64_C(   20.44)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(   -0.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    1.00)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_cosh_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_cosh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(     8.19),
                         SIMDE_FLOAT32_C(     2.86), SIMDE_FLOAT32_C(     4.69),
                         SIMDE_FLOAT32_C(     3.48), SIMDE_FLOAT32_C(     4.71),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     6.41)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   697.05), SIMDE_FLOAT32_C(  1802.36),
                         SIMDE_FLOAT32_C(     8.76), SIMDE_FLOAT32_C(    54.43),
                         SIMDE_FLOAT32_C(    16.25), SIMDE_FLOAT32_C(    55.53),
                         SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(   303.95)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(     4.97), SIMDE_FLOAT32_C(     7.64),
                         SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(     6.82),
                         SIMDE_FLOAT32_C(     3.02), SIMDE_FLOAT32_C(     7.07)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     1.28),
                         SIMDE_FLOAT32_C(    72.02), SIMDE_FLOAT32_C(  1039.87),
                         SIMDE_FLOAT32_C(  4216.89), SIMDE_FLOAT32_C(   457.99),
                         SIMDE_FLOAT32_C(    10.27), SIMDE_FLOAT32_C(   588.07)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(     4.66),
                         SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(    -0.58),
                         SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(    -0.24),
                         SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     8.33)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(    52.82),
                         SIMDE_FLOAT32_C(     5.50), SIMDE_FLOAT32_C(     1.17),
                         SIMDE_FLOAT32_C(     8.42), SIMDE_FLOAT32_C(     1.03),
                         SIMDE_FLOAT32_C(     4.57), SIMDE_FLOAT32_C(  2073.21)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     5.48), SIMDE_FLOAT32_C(     2.02),
                         SIMDE_FLOAT32_C(     5.78), SIMDE_FLOAT32_C(     8.28),
                         SIMDE_FLOAT32_C(     5.94), SIMDE_FLOAT32_C(     3.33),
                         SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(     0.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   119.93), SIMDE_FLOAT32_C(     3.84),
                         SIMDE_FLOAT32_C(   161.88), SIMDE_FLOAT32_C(  1972.10),
                         SIMDE_FLOAT32_C(   189.97), SIMDE_FLOAT32_C(    13.99),
                         SIMDE_FLOAT32_C(     1.40), SIMDE_FLOAT32_C(     1.40)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     6.94),
                         SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     6.59),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     9.97),
                         SIMDE_FLOAT32_C(     4.65), SIMDE_FLOAT32_C(     9.13)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C(   516.39),
                         SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(   363.89),
                         SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C( 10687.75),
                         SIMDE_FLOAT32_C(    52.30), SIMDE_FLOAT32_C(  4614.01)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     6.68),
                         SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     6.37),
                         SIMDE_FLOAT32_C(     7.43), SIMDE_FLOAT32_C(     3.05),
                         SIMDE_FLOAT32_C(     8.79), SIMDE_FLOAT32_C(     4.33)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     5.40), SIMDE_FLOAT32_C(   398.16),
                         SIMDE_FLOAT32_C(  1663.79), SIMDE_FLOAT32_C(   292.03),
                         SIMDE_FLOAT32_C(   842.90), SIMDE_FLOAT32_C(    10.58),
                         SIMDE_FLOAT32_C(  3284.12), SIMDE_FLOAT32_C(    37.98)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(     4.06),
                         SIMDE_FLOAT32_C(     6.35), SIMDE_FLOAT32_C(    -0.70),
                         SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.28),
                         SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     6.69)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    14.70), SIMDE_FLOAT32_C(    29.00),
                         SIMDE_FLOAT32_C(   286.25), SIMDE_FLOAT32_C(     1.26),
                         SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     1.04),
                         SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(   402.16)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     7.08), SIMDE_FLOAT32_C(     8.24),
                         SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(     9.00),
                         SIMDE_FLOAT32_C(     9.51), SIMDE_FLOAT32_C(     7.80),
                         SIMDE_FLOAT32_C(     8.85), SIMDE_FLOAT32_C(     5.90)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   593.98), SIMDE_FLOAT32_C(  1894.77),
                         SIMDE_FLOAT32_C(    19.84), SIMDE_FLOAT32_C(  4051.54),
                         SIMDE_FLOAT32_C(  6747.00), SIMDE_FLOAT32_C(  1220.30),
                         SIMDE_FLOAT32_C(  3487.20), SIMDE_FLOAT32_C(   182.52)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_cosh_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_cosh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.44), SIMDE_FLOAT64_C(    2.12),
                         SIMDE_FLOAT64_C(   -0.26), SIMDE_FLOAT64_C(    3.04)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.23), SIMDE_FLOAT64_C(    4.23),
                         SIMDE_FLOAT64_C(    1.03), SIMDE_FLOAT64_C(   10.48)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.49), SIMDE_FLOAT64_C(    4.01),
                         SIMDE_FLOAT64_C(    1.11), SIMDE_FLOAT64_C(    2.10)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   16.41), SIMDE_FLOAT64_C(   27.58),
                         SIMDE_FLOAT64_C(    1.68), SIMDE_FLOAT64_C(    4.14)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    4.48), SIMDE_FLOAT64_C(    3.27),
                         SIMDE_FLOAT64_C(    1.19), SIMDE_FLOAT64_C(    3.40)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   44.12), SIMDE_FLOAT64_C(   13.17),
                         SIMDE_FLOAT64_C(    1.80), SIMDE_FLOAT64_C(   15.00)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(   -0.06),
                         SIMDE_FLOAT64_C(    2.25), SIMDE_FLOAT64_C(    3.71)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    1.00),
                         SIMDE_FLOAT64_C(    4.80), SIMDE_FLOAT64_C(   20.44)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.08), SIMDE_FLOAT64_C(   -0.58),
                         SIMDE_FLOAT64_C(    0.75), SIMDE_FLOAT64_C(    4.09)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.64), SIMDE_FLOAT64_C(    1.17),
                         SIMDE_FLOAT64_C(    1.29), SIMDE_FLOAT64_C(   29.88)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    2.09),
                         SIMDE_FLOAT64_C(    0.85), SIMDE_FLOAT64_C(   -0.77)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.23), SIMDE_FLOAT64_C(    4.10),
                         SIMDE_FLOAT64_C(    1.38), SIMDE_FLOAT64_C(    1.31)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.78), SIMDE_FLOAT64_C(    1.36),
                         SIMDE_FLOAT64_C(   -0.93), SIMDE_FLOAT64_C(    0.02)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    8.09), SIMDE_FLOAT64_C(    2.08),
                         SIMDE_FLOAT64_C(    1.46), SIMDE_FLOAT64_C(    1.00)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.53), SIMDE_FLOAT64_C(    0.65),
                         SIMDE_FLOAT64_C(    2.70), SIMDE_FLOAT64_C(    4.06)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    6.32), SIMDE_FLOAT64_C(    1.22),
                         SIMDE_FLOAT64_C(    7.47), SIMDE_FLOAT64_C(   29.00)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_cosh_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_cosh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     4.97), SIMDE_FLOAT32_C(     7.64),
                         SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(     6.82), SIMDE_FLOAT32_C(     3.02), SIMDE_FLOAT32_C(     7.07),
                         SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(     8.19), SIMDE_FLOAT32_C(     2.86), SIMDE_FLOAT32_C(     4.69),
                         SIMDE_FLOAT32_C(     3.48), SIMDE_FLOAT32_C(     4.71), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     6.41)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(    72.02), SIMDE_FLOAT32_C(  1039.87),
                         SIMDE_FLOAT32_C(  4216.89), SIMDE_FLOAT32_C(   457.99), SIMDE_FLOAT32_C(    10.27), SIMDE_FLOAT32_C(   588.07),
                         SIMDE_FLOAT32_C(   697.05), SIMDE_FLOAT32_C(  1802.36), SIMDE_FLOAT32_C(     8.76), SIMDE_FLOAT32_C(    54.43),
                         SIMDE_FLOAT32_C(    16.25), SIMDE_FLOAT32_C(    55.53), SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(   303.95)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.48), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     5.78), SIMDE_FLOAT32_C(     8.28),
                         SIMDE_FLOAT32_C(     5.94), SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(     0.87),
                         SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(     4.66), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(    -0.58),
                         SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     8.33)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   119.93), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(   161.88), SIMDE_FLOAT32_C(  1972.10),
                         SIMDE_FLOAT32_C(   189.97), SIMDE_FLOAT32_C(    13.99), SIMDE_FLOAT32_C(     1.40), SIMDE_FLOAT32_C(     1.40),
                         SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(    52.82), SIMDE_FLOAT32_C(     5.50), SIMDE_FLOAT32_C(     1.17),
                         SIMDE_FLOAT32_C(     8.42), SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C(     4.57), SIMDE_FLOAT32_C(  2073.21)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     6.68), SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     6.37),
                         SIMDE_FLOAT32_C(     7.43), SIMDE_FLOAT32_C(     3.05), SIMDE_FLOAT32_C(     8.79), SIMDE_FLOAT32_C(     4.33),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     6.94), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     6.59),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     9.97), SIMDE_FLOAT32_C(     4.65), SIMDE_FLOAT32_C(     9.13)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.40), SIMDE_FLOAT32_C(   398.16), SIMDE_FLOAT32_C(  1663.79), SIMDE_FLOAT32_C(   292.03),
                         SIMDE_FLOAT32_C(   842.90), SIMDE_FLOAT32_C(    10.58), SIMDE_FLOAT32_C(  3284.12), SIMDE_FLOAT32_C(    37.98),
                         SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C(   516.39), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(   363.89),
                         SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C( 10687.75), SIMDE_FLOAT32_C(    52.30), SIMDE_FLOAT32_C(  4614.01)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.08), SIMDE_FLOAT32_C(     8.24), SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(     9.00),
                         SIMDE_FLOAT32_C(     9.51), SIMDE_FLOAT32_C(     7.80), SIMDE_FLOAT32_C(     8.85), SIMDE_FLOAT32_C(     5.90),
                         SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(     4.06), SIMDE_FLOAT32_C(     6.35), SIMDE_FLOAT32_C(    -0.70),
                         SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     6.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   593.98), SIMDE_FLOAT32_C(  1894.77), SIMDE_FLOAT32_C(    19.84), SIMDE_FLOAT32_C(  4051.54),
                         SIMDE_FLOAT32_C(  6747.00), SIMDE_FLOAT32_C(  1220.30), SIMDE_FLOAT32_C(  3487.20), SIMDE_FLOAT32_C(   182.52),
                         SIMDE_FLOAT32_C(    14.70), SIMDE_FLOAT32_C(    29.00), SIMDE_FLOAT32_C(   286.25), SIMDE_FLOAT32_C(     1.26),
                         SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(   402.16)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(     8.67), SIMDE_FLOAT32_C(     6.29), SIMDE_FLOAT32_C(     6.39),
                         SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(     1.61),
                         SIMDE_FLOAT32_C(     3.44), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     7.59), SIMDE_FLOAT32_C(     8.11),
                         SIMDE_FLOAT32_C(     4.11), SIMDE_FLOAT32_C(     7.49), SIMDE_FLOAT32_C(     5.16), SIMDE_FLOAT32_C(     3.56)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    16.91), SIMDE_FLOAT32_C(  2912.75), SIMDE_FLOAT32_C(   269.58), SIMDE_FLOAT32_C(   297.93),
                         SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(     7.40), SIMDE_FLOAT32_C(     2.60),
                         SIMDE_FLOAT32_C(    15.61), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   989.16), SIMDE_FLOAT32_C(  1663.79),
                         SIMDE_FLOAT32_C(    30.48), SIMDE_FLOAT32_C(   895.03), SIMDE_FLOAT32_C(    87.09), SIMDE_FLOAT32_C(    17.60)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     8.30), SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     4.38), SIMDE_FLOAT32_C(     1.25),
                         SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(     8.52), SIMDE_FLOAT32_C(     7.28), SIMDE_FLOAT32_C(     7.93),
                         SIMDE_FLOAT32_C(     9.07), SIMDE_FLOAT32_C(     5.27), SIMDE_FLOAT32_C(     9.88), SIMDE_FLOAT32_C(    -0.48),
                         SIMDE_FLOAT32_C(     6.32), SIMDE_FLOAT32_C(     5.05), SIMDE_FLOAT32_C(     7.05), SIMDE_FLOAT32_C(     0.44)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  2011.94), SIMDE_FLOAT32_C(  4568.10), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(     1.89),
                         SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(  2507.03), SIMDE_FLOAT32_C(   725.49), SIMDE_FLOAT32_C(  1389.71),
                         SIMDE_FLOAT32_C(  4345.31), SIMDE_FLOAT32_C(    97.21), SIMDE_FLOAT32_C(  9767.86), SIMDE_FLOAT32_C(     1.12),
                         SIMDE_FLOAT32_C(   277.79), SIMDE_FLOAT32_C(    78.01), SIMDE_FLOAT32_C(   576.43), SIMDE_FLOAT32_C(     1.10)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     9.02), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     9.58), SIMDE_FLOAT32_C(     6.12), SIMDE_FLOAT32_C(     0.28),
                         SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     4.13), SIMDE_FLOAT32_C(     8.41), SIMDE_FLOAT32_C(     9.88),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     6.84), SIMDE_FLOAT32_C(     4.65), SIMDE_FLOAT32_C(     3.98)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(  4133.39), SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     1.53),
                         SIMDE_FLOAT32_C(     7.04), SIMDE_FLOAT32_C(  7236.21), SIMDE_FLOAT32_C(   227.43), SIMDE_FLOAT32_C(     1.04),
                         SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(    31.10), SIMDE_FLOAT32_C(  2245.88), SIMDE_FLOAT32_C(  9767.86),
                         SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(   467.25), SIMDE_FLOAT32_C(    52.30), SIMDE_FLOAT32_C(    26.77)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.09), SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     4.31),
                         SIMDE_FLOAT32_C(     9.63), SIMDE_FLOAT32_C(     4.54), SIMDE_FLOAT32_C(     2.70), SIMDE_FLOAT32_C(     3.81),
                         SIMDE_FLOAT32_C(     3.50), SIMDE_FLOAT32_C(     4.72), SIMDE_FLOAT32_C(     7.31), SIMDE_FLOAT32_C(     6.67),
                         SIMDE_FLOAT32_C(     4.58), SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(     9.54), SIMDE_FLOAT32_C(     0.67)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.10), SIMDE_FLOAT32_C(     6.25), SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(    37.23),
                         SIMDE_FLOAT32_C(  7607.22), SIMDE_FLOAT32_C(    46.85), SIMDE_FLOAT32_C(     7.47), SIMDE_FLOAT32_C(    22.59),
                         SIMDE_FLOAT32_C(    16.57), SIMDE_FLOAT32_C(    56.09), SIMDE_FLOAT32_C(   747.59), SIMDE_FLOAT32_C(   394.20),
                         SIMDE_FLOAT32_C(    48.76), SIMDE_FLOAT32_C(     8.42), SIMDE_FLOAT32_C(  6952.47), SIMDE_FLOAT32_C(     1.23)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_cosh_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cosh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     8.28), SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     0.87),
                         SIMDE_FLOAT32_C(     4.66), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     8.33),
                         SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     7.64), SIMDE_FLOAT32_C(     6.82), SIMDE_FLOAT32_C(     7.07),
                         SIMDE_FLOAT32_C(     8.19), SIMDE_FLOAT32_C(     4.69), SIMDE_FLOAT32_C(     4.71), SIMDE_FLOAT32_C(     6.41)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.48), SIMDE_FLOAT32_C(     5.78), SIMDE_FLOAT32_C(     5.94), SIMDE_FLOAT32_C(    -0.87),
                         SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(     2.20),
                         SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     4.97), SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(     3.02),
                         SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(     2.86), SIMDE_FLOAT32_C(     3.48), SIMDE_FLOAT32_C(     0.35)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   119.93), SIMDE_FLOAT32_C(     8.28), SIMDE_FLOAT32_C(   189.97), SIMDE_FLOAT32_C(     0.87),
                         SIMDE_FLOAT32_C(     4.66), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     4.57),
                         SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(    72.02), SIMDE_FLOAT32_C(  4216.89), SIMDE_FLOAT32_C(    10.27),
                         SIMDE_FLOAT32_C(   697.05), SIMDE_FLOAT32_C(     4.69), SIMDE_FLOAT32_C(    16.25), SIMDE_FLOAT32_C(     6.41)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.08), SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(     9.51), SIMDE_FLOAT32_C(     8.85),
                         SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(     6.35), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     1.45),
                         SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     7.43), SIMDE_FLOAT32_C(     8.79),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     4.65)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(     8.24), SIMDE_FLOAT32_C(     9.00), SIMDE_FLOAT32_C(     7.80),
                         SIMDE_FLOAT32_C(     5.90), SIMDE_FLOAT32_C(     4.06), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(     0.28),
                         SIMDE_FLOAT32_C(     6.69), SIMDE_FLOAT32_C(     6.68), SIMDE_FLOAT32_C(     6.37), SIMDE_FLOAT32_C(     3.05),
                         SIMDE_FLOAT32_C(     4.33), SIMDE_FLOAT32_C(     6.94), SIMDE_FLOAT32_C(     6.59), SIMDE_FLOAT32_C(     9.97)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    17.60), SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(     9.51), SIMDE_FLOAT32_C(     8.85),
                         SIMDE_FLOAT32_C(   182.52), SIMDE_FLOAT32_C(    29.00), SIMDE_FLOAT32_C(     1.26), SIMDE_FLOAT32_C(     1.04),
                         SIMDE_FLOAT32_C(   402.16), SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(   292.03), SIMDE_FLOAT32_C(    10.58),
                         SIMDE_FLOAT32_C(    37.98), SIMDE_FLOAT32_C(   516.39), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C( 10687.75)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     8.52),
                         SIMDE_FLOAT32_C(     7.93), SIMDE_FLOAT32_C(     5.27), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(     5.05),
                         SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     8.67), SIMDE_FLOAT32_C(     6.39), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(     1.61), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     7.49)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.65), SIMDE_FLOAT32_C(     8.30), SIMDE_FLOAT32_C(     4.38), SIMDE_FLOAT32_C(     2.03),
                         SIMDE_FLOAT32_C(     7.28), SIMDE_FLOAT32_C(     9.07), SIMDE_FLOAT32_C(     9.88), SIMDE_FLOAT32_C(     6.32),
                         SIMDE_FLOAT32_C(     7.05), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(     6.29), SIMDE_FLOAT32_C(    -0.31),
                         SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(     3.44), SIMDE_FLOAT32_C(     7.59), SIMDE_FLOAT32_C(     4.11)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(  2011.94), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     8.52),
                         SIMDE_FLOAT32_C(     7.93), SIMDE_FLOAT32_C(     5.27), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(   277.79),
                         SIMDE_FLOAT32_C(   576.43), SIMDE_FLOAT32_C(     8.67), SIMDE_FLOAT32_C(   269.58), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(     1.61), SIMDE_FLOAT32_C(    15.61), SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     7.49)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.58), SIMDE_FLOAT32_C(     2.09), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     9.63),
                         SIMDE_FLOAT32_C(     2.70), SIMDE_FLOAT32_C(     3.50), SIMDE_FLOAT32_C(     7.31), SIMDE_FLOAT32_C(     4.58),
                         SIMDE_FLOAT32_C(     9.54), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     2.64),
                         SIMDE_FLOAT32_C(     6.12), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     8.41), SIMDE_FLOAT32_C(     0.34)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     9.43), SIMDE_FLOAT32_C(     3.41), SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(     4.31),
                         SIMDE_FLOAT32_C(     4.54), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     4.72), SIMDE_FLOAT32_C(     6.67),
                         SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     9.02), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(     9.58), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     4.13), SIMDE_FLOAT32_C(     9.88)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.58), SIMDE_FLOAT32_C(     2.09), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     9.63),
                         SIMDE_FLOAT32_C(    46.85), SIMDE_FLOAT32_C(     3.50), SIMDE_FLOAT32_C(     7.31), SIMDE_FLOAT32_C(     4.58),
                         SIMDE_FLOAT32_C(     9.54), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(  4133.39), SIMDE_FLOAT32_C(     1.53),
                         SIMDE_FLOAT32_C(  7236.21), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(    31.10), SIMDE_FLOAT32_C(  9767.86)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.41), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(     5.48),
                         SIMDE_FLOAT32_C(     6.40), SIMDE_FLOAT32_C(     9.39), SIMDE_FLOAT32_C(     9.54), SIMDE_FLOAT32_C(     5.23),
                         SIMDE_FLOAT32_C(     7.17), SIMDE_FLOAT32_C(     1.21), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     0.29),
                         SIMDE_FLOAT32_C(     5.72), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     6.92)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.23), SIMDE_FLOAT32_C(     9.53), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     5.38),
                         SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     9.63), SIMDE_FLOAT32_C(     5.11),
                         SIMDE_FLOAT32_C(     9.05), SIMDE_FLOAT32_C(     4.08), SIMDE_FLOAT32_C(     5.81), SIMDE_FLOAT32_C(     2.42),
                         SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     7.12)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.41), SIMDE_FLOAT32_C(  6883.29), SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(   108.51),
                         SIMDE_FLOAT32_C(     6.40), SIMDE_FLOAT32_C(   471.94), SIMDE_FLOAT32_C(  7607.22), SIMDE_FLOAT32_C(    82.84),
                         SIMDE_FLOAT32_C(     7.17), SIMDE_FLOAT32_C(     1.21), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     0.29),
                         SIMDE_FLOAT32_C(     5.72), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(     6.92)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     9.36), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     4.11), SIMDE_FLOAT32_C(     0.19),
                         SIMDE_FLOAT32_C(     5.01), SIMDE_FLOAT32_C(     5.64), SIMDE_FLOAT32_C(     9.09), SIMDE_FLOAT32_C(     4.14),
                         SIMDE_FLOAT32_C(     2.93), SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     2.75), SIMDE_FLOAT32_C(     8.81),
                         SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     7.56), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     6.89)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(     2.27), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.29),
                         SIMDE_FLOAT32_C(     8.84), SIMDE_FLOAT32_C(     4.53), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     5.39),
                         SIMDE_FLOAT32_C(     7.69), SIMDE_FLOAT32_C(     4.44), SIMDE_FLOAT32_C(     2.80), SIMDE_FLOAT32_C(     0.54),
                         SIMDE_FLOAT32_C(     6.34), SIMDE_FLOAT32_C(     6.01), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(     1.81)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     9.36), SIMDE_FLOAT32_C(     4.89), SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(     0.19),
                         SIMDE_FLOAT32_C(  3452.50), SIMDE_FLOAT32_C(     5.64), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   109.60),
                         SIMDE_FLOAT32_C(     2.93), SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     2.75), SIMDE_FLOAT32_C(     8.81),
                         SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(   203.74), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     6.89)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(     4.36), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     8.01),
                         SIMDE_FLOAT32_C(     6.98), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     5.08), SIMDE_FLOAT32_C(     7.75),
                         SIMDE_FLOAT32_C(     4.67), SIMDE_FLOAT32_C(     7.99), SIMDE_FLOAT32_C(     4.04), SIMDE_FLOAT32_C(     4.94),
                         SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     7.84), SIMDE_FLOAT32_C(     6.67), SIMDE_FLOAT32_C(    -0.39)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.60), SIMDE_FLOAT32_C(     8.95), SIMDE_FLOAT32_C(     5.29), SIMDE_FLOAT32_C(     2.15),
                         SIMDE_FLOAT32_C(     6.20), SIMDE_FLOAT32_C(     3.53), SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     8.23),
                         SIMDE_FLOAT32_C(     4.86), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     5.15), SIMDE_FLOAT32_C(     1.33),
                         SIMDE_FLOAT32_C(     4.29), SIMDE_FLOAT32_C(     3.12), SIMDE_FLOAT32_C(     4.20), SIMDE_FLOAT32_C(     5.01)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(     4.36), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     8.01),
                         SIMDE_FLOAT32_C(     6.98), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(  1875.92),
                         SIMDE_FLOAT32_C(    64.52), SIMDE_FLOAT32_C(     7.99), SIMDE_FLOAT32_C(    86.22), SIMDE_FLOAT32_C(     2.02),
                         SIMDE_FLOAT32_C(    36.49), SIMDE_FLOAT32_C(     7.84), SIMDE_FLOAT32_C(     6.67), SIMDE_FLOAT32_C(    74.96)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     6.32), SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(     9.02),
                         SIMDE_FLOAT32_C(     9.76), SIMDE_FLOAT32_C(     9.75), SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     3.35), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     2.19),
                         SIMDE_FLOAT32_C(     5.05), SIMDE_FLOAT32_C(     5.07), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     3.38)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(     8.86),
                         SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(     8.51), SIMDE_FLOAT32_C(     2.57),
                         SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(     5.10), SIMDE_FLOAT32_C(     8.44), SIMDE_FLOAT32_C(     3.12),
                         SIMDE_FLOAT32_C(     9.69), SIMDE_FLOAT32_C(     6.49), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     9.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     6.32), SIMDE_FLOAT32_C(  4216.89), SIMDE_FLOAT32_C(  3522.24),
                         SIMDE_FLOAT32_C(     9.76), SIMDE_FLOAT32_C(     9.75), SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(     6.57),
                         SIMDE_FLOAT32_C(    62.61), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(  2314.28), SIMDE_FLOAT32_C(     2.19),
                         SIMDE_FLOAT32_C(  8077.62), SIMDE_FLOAT32_C(     5.07), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(  4899.32)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_cosh_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_cosh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.49), SIMDE_FLOAT64_C(    4.01),
                         SIMDE_FLOAT64_C(    1.11), SIMDE_FLOAT64_C(    2.10),
                         SIMDE_FLOAT64_C(    1.44), SIMDE_FLOAT64_C(    2.12),
                         SIMDE_FLOAT64_C(   -0.26), SIMDE_FLOAT64_C(    3.04)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   16.41), SIMDE_FLOAT64_C(   27.58),
                         SIMDE_FLOAT64_C(    1.68), SIMDE_FLOAT64_C(    4.14),
                         SIMDE_FLOAT64_C(    2.23), SIMDE_FLOAT64_C(    4.23),
                         SIMDE_FLOAT64_C(    1.03), SIMDE_FLOAT64_C(   10.48)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(   -0.06),
                         SIMDE_FLOAT64_C(    2.25), SIMDE_FLOAT64_C(    3.71),
                         SIMDE_FLOAT64_C(    4.48), SIMDE_FLOAT64_C(    3.27),
                         SIMDE_FLOAT64_C(    1.19), SIMDE_FLOAT64_C(    3.40)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    1.00),
                         SIMDE_FLOAT64_C(    4.80), SIMDE_FLOAT64_C(   20.44),
                         SIMDE_FLOAT64_C(   44.12), SIMDE_FLOAT64_C(   13.17),
                         SIMDE_FLOAT64_C(    1.80), SIMDE_FLOAT64_C(   15.00)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    2.09),
                         SIMDE_FLOAT64_C(    0.85), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(    1.08), SIMDE_FLOAT64_C(   -0.58),
                         SIMDE_FLOAT64_C(    0.75), SIMDE_FLOAT64_C(    4.09)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.23), SIMDE_FLOAT64_C(    4.10),
                         SIMDE_FLOAT64_C(    1.38), SIMDE_FLOAT64_C(    1.31),
                         SIMDE_FLOAT64_C(    1.64), SIMDE_FLOAT64_C(    1.17),
                         SIMDE_FLOAT64_C(    1.29), SIMDE_FLOAT64_C(   29.88)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.53), SIMDE_FLOAT64_C(    0.65),
                         SIMDE_FLOAT64_C(    2.70), SIMDE_FLOAT64_C(    4.06),
                         SIMDE_FLOAT64_C(    2.78), SIMDE_FLOAT64_C(    1.36),
                         SIMDE_FLOAT64_C(   -0.93), SIMDE_FLOAT64_C(    0.02)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    6.32), SIMDE_FLOAT64_C(    1.22),
                         SIMDE_FLOAT64_C(    7.47), SIMDE_FLOAT64_C(   29.00),
                         SIMDE_FLOAT64_C(    8.09), SIMDE_FLOAT64_C(    2.08),
                         SIMDE_FLOAT64_C(    1.46), SIMDE_FLOAT64_C(    1.00)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(    3.33),
                         SIMDE_FLOAT64_C(    0.25), SIMDE_FLOAT64_C(    3.14),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(    4.98),
                         SIMDE_FLOAT64_C(    2.08), SIMDE_FLOAT64_C(    4.52)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.05), SIMDE_FLOAT64_C(   13.99),
                         SIMDE_FLOAT64_C(    1.03), SIMDE_FLOAT64_C(   11.57),
                         SIMDE_FLOAT64_C(    1.05), SIMDE_FLOAT64_C(   72.74),
                         SIMDE_FLOAT64_C(    4.06), SIMDE_FLOAT64_C(   45.92)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.84), SIMDE_FLOAT64_C(    3.19),
                         SIMDE_FLOAT64_C(    3.97), SIMDE_FLOAT64_C(    3.02),
                         SIMDE_FLOAT64_C(    3.60), SIMDE_FLOAT64_C(    1.21),
                         SIMDE_FLOAT64_C(    4.34), SIMDE_FLOAT64_C(    1.91)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.37), SIMDE_FLOAT64_C(   12.16),
                         SIMDE_FLOAT64_C(   26.50), SIMDE_FLOAT64_C(   10.27),
                         SIMDE_FLOAT64_C(   18.31), SIMDE_FLOAT64_C(    1.83),
                         SIMDE_FLOAT64_C(   38.36), SIMDE_FLOAT64_C(    3.45)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.39), SIMDE_FLOAT64_C(    1.76),
                         SIMDE_FLOAT64_C(    3.01), SIMDE_FLOAT64_C(   -0.83),
                         SIMDE_FLOAT64_C(   -0.24), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(    3.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.13), SIMDE_FLOAT64_C(    2.99),
                         SIMDE_FLOAT64_C(   10.17), SIMDE_FLOAT64_C(    1.36),
                         SIMDE_FLOAT64_C(    1.03), SIMDE_FLOAT64_C(    1.05),
                         SIMDE_FLOAT64_C(    1.06), SIMDE_FLOAT64_C(   12.29)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.41), SIMDE_FLOAT64_C(    4.04),
                         SIMDE_FLOAT64_C(    1.55), SIMDE_FLOAT64_C(    4.46),
                         SIMDE_FLOAT64_C(    4.73), SIMDE_FLOAT64_C(    3.80),
                         SIMDE_FLOAT64_C(    4.37), SIMDE_FLOAT64_C(    2.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   15.15), SIMDE_FLOAT64_C(   28.42),
                         SIMDE_FLOAT64_C(    2.46), SIMDE_FLOAT64_C(   43.25),
                         SIMDE_FLOAT64_C(   56.65), SIMDE_FLOAT64_C(   22.36),
                         SIMDE_FLOAT64_C(   39.53), SIMDE_FLOAT64_C(    7.93)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_cosh_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_cosh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.06), SIMDE_FLOAT64_C(    3.71),
                         SIMDE_FLOAT64_C(    3.27), SIMDE_FLOAT64_C(    3.40),
                         SIMDE_FLOAT64_C(    4.01), SIMDE_FLOAT64_C(    2.10),
                         SIMDE_FLOAT64_C(    2.12), SIMDE_FLOAT64_C(    3.04)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(    2.25),
                         SIMDE_FLOAT64_C(    4.48), SIMDE_FLOAT64_C(    1.19),
                         SIMDE_FLOAT64_C(    3.49), SIMDE_FLOAT64_C(    1.11),
                         SIMDE_FLOAT64_C(    1.44), SIMDE_FLOAT64_C(   -0.26)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    3.71),
                         SIMDE_FLOAT64_C(    3.27), SIMDE_FLOAT64_C(    3.40),
                         SIMDE_FLOAT64_C(   16.41), SIMDE_FLOAT64_C(    2.10),
                         SIMDE_FLOAT64_C(    2.23), SIMDE_FLOAT64_C(    1.03)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.53), SIMDE_FLOAT64_C(    2.70),
                         SIMDE_FLOAT64_C(    2.78), SIMDE_FLOAT64_C(   -0.93),
                         SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    0.85),
                         SIMDE_FLOAT64_C(    1.08), SIMDE_FLOAT64_C(    0.75)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.52), SIMDE_FLOAT64_C(    0.65),
                         SIMDE_FLOAT64_C(    4.06), SIMDE_FLOAT64_C(    1.36),
                         SIMDE_FLOAT64_C(    0.02), SIMDE_FLOAT64_C(    2.09),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(   -0.58)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   45.92), SIMDE_FLOAT64_C(    1.22),
                         SIMDE_FLOAT64_C(   29.00), SIMDE_FLOAT64_C(   -0.93),
                         SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    4.10),
                         SIMDE_FLOAT64_C(    1.08), SIMDE_FLOAT64_C(    1.17)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.20), SIMDE_FLOAT64_C(    3.19),
                         SIMDE_FLOAT64_C(    3.02), SIMDE_FLOAT64_C(    1.21),
                         SIMDE_FLOAT64_C(    1.91), SIMDE_FLOAT64_C(    3.33),
                         SIMDE_FLOAT64_C(    3.14), SIMDE_FLOAT64_C(    4.98)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(    0.84),
                         SIMDE_FLOAT64_C(    3.97), SIMDE_FLOAT64_C(    3.60),
                         SIMDE_FLOAT64_C(    4.34), SIMDE_FLOAT64_C(   -0.31),
                         SIMDE_FLOAT64_C(    0.25), SIMDE_FLOAT64_C(   -0.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.06), SIMDE_FLOAT64_C(    1.37),
                         SIMDE_FLOAT64_C(   26.50), SIMDE_FLOAT64_C(   18.31),
                         SIMDE_FLOAT64_C(   38.36), SIMDE_FLOAT64_C(    1.05),
                         SIMDE_FLOAT64_C(    3.14), SIMDE_FLOAT64_C(    1.05)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.36), SIMDE_FLOAT64_C(    3.41),
                         SIMDE_FLOAT64_C(    1.55), SIMDE_FLOAT64_C(    4.73),
                         SIMDE_FLOAT64_C(    4.37), SIMDE_FLOAT64_C(    1.39),
                         SIMDE_FLOAT64_C(    3.01), SIMDE_FLOAT64_C(   -0.24)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.63), SIMDE_FLOAT64_C(    1.49),
                         SIMDE_FLOAT64_C(    4.04), SIMDE_FLOAT64_C(    4.46),
                         SIMDE_FLOAT64_C(    3.80), SIMDE_FLOAT64_C(    2.76),
                         SIMDE_FLOAT64_C(    1.76), SIMDE_FLOAT64_C(   -0.83)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.36), SIMDE_FLOAT64_C(    2.33),
                         SIMDE_FLOAT64_C(    1.55), SIMDE_FLOAT64_C(   43.25),
                         SIMDE_FLOAT64_C(   22.36), SIMDE_FLOAT64_C(    7.93),
                         SIMDE_FLOAT64_C(    3.01), SIMDE_FLOAT64_C(    1.36)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.30), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(    4.28), SIMDE_FLOAT64_C(    3.03),
                         SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.47), SIMDE_FLOAT64_C(    3.97)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.99), SIMDE_FLOAT64_C(    3.39),
                         SIMDE_FLOAT64_C(    1.46), SIMDE_FLOAT64_C(    2.97),
                         SIMDE_FLOAT64_C(   -0.62), SIMDE_FLOAT64_C(    1.01),
                         SIMDE_FLOAT64_C(    1.42), SIMDE_FLOAT64_C(    3.68)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    9.97), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(    4.28), SIMDE_FLOAT64_C(    9.77),
                         SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.47), SIMDE_FLOAT64_C(   19.84)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    2.08),
                         SIMDE_FLOAT64_C(    4.07), SIMDE_FLOAT64_C(    1.94),
                         SIMDE_FLOAT64_C(    0.65), SIMDE_FLOAT64_C(    3.52),
                         SIMDE_FLOAT64_C(    4.49), SIMDE_FLOAT64_C(    4.93)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.93), SIMDE_FLOAT64_C(    3.27),
                         SIMDE_FLOAT64_C(    1.71), SIMDE_FLOAT64_C(    4.52),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    4.19),
                         SIMDE_FLOAT64_C(    3.87), SIMDE_FLOAT64_C(    2.42)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(   13.17),
                         SIMDE_FLOAT64_C(    4.07), SIMDE_FLOAT64_C(    1.94),
                         SIMDE_FLOAT64_C(    1.03), SIMDE_FLOAT64_C(    3.52),
                         SIMDE_FLOAT64_C(   23.98), SIMDE_FLOAT64_C(    5.67)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.18), SIMDE_FLOAT64_C(    1.09),
                         SIMDE_FLOAT64_C(   -0.09), SIMDE_FLOAT64_C(    4.47),
                         SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(    4.77),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    1.80)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.53), SIMDE_FLOAT64_C(    2.04),
                         SIMDE_FLOAT64_C(    4.75), SIMDE_FLOAT64_C(   -0.31),
                         SIMDE_FLOAT64_C(    0.28), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    2.88), SIMDE_FLOAT64_C(    0.27)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.18), SIMDE_FLOAT64_C(    3.91),
                         SIMDE_FLOAT64_C(   -0.09), SIMDE_FLOAT64_C(    1.05),
                         SIMDE_FLOAT64_C(    1.04), SIMDE_FLOAT64_C(    1.53),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    1.04)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.43), SIMDE_FLOAT64_C(    4.81),
                         SIMDE_FLOAT64_C(    0.95), SIMDE_FLOAT64_C(    0.69),
                         SIMDE_FLOAT64_C(   -0.26), SIMDE_FLOAT64_C(    4.80),
                         SIMDE_FLOAT64_C(    1.02), SIMDE_FLOAT64_C(    1.45)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.33), SIMDE_FLOAT64_C(    3.32),
                         SIMDE_FLOAT64_C(    4.69), SIMDE_FLOAT64_C(    1.41),
                         SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(    1.90),
                         SIMDE_FLOAT64_C(    2.02), SIMDE_FLOAT64_C(    1.62)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.05), SIMDE_FLOAT64_C(   13.85),
                         SIMDE_FLOAT64_C(    0.95), SIMDE_FLOAT64_C(    2.17),
                         SIMDE_FLOAT64_C(   -0.26), SIMDE_FLOAT64_C(    3.42),
                         SIMDE_FLOAT64_C(    1.02), SIMDE_FLOAT64_C(    2.63)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_cosh_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_x_mm_deg2rad_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   954.59), SIMDE_FLOAT32_C(  -212.53), SIMDE_FLOAT32_C(   -73.32), SIMDE_FLOAT32_C(  -280.66) },
      { SIMDE_FLOAT32_C(    16.66), SIMDE_FLOAT32_C(    -3.71), SIMDE_FLOAT32_C(    -1.28), SIMDE_FLOAT32_C(    -4.90) } },
    { { SIMDE_FLOAT32_C(   908.48), SIMDE_FLOAT32_C(   789.59), SIMDE_FLOAT32_C(   675.09), SIMDE_FLOAT32_C(   164.25) },
      { SIMDE_FLOAT32_C(    15.86), SIMDE_FLOAT32_C(    13.78), SIMDE_FLOAT32_C(    11.78), SIMDE_FLOAT32_C(     2.87) } },
    { { SIMDE_FLOAT32_C(   515.80), SIMDE_FLOAT32_C(  -965.27), SIMDE_FLOAT32_C(   659.44), SIMDE_FLOAT32_C(  -806.83) },
      { SIMDE_FLOAT32_C(     9.00), SIMDE_FLOAT32_C(   -16.85), SIMDE_FLOAT32_C(    11.51), SIMDE_FLOAT32_C(   -14.08) } },
    { { SIMDE_FLOAT32_C(  -402.30), SIMDE_FLOAT32_C(   576.73), SIMDE_FLOAT32_C(  -978.47), SIMDE_FLOAT32_C(   782.95) },
      { SIMDE_FLOAT32_C(    -7.02), SIMDE_FLOAT32_C(    10.07), SIMDE_FLOAT32_C(   -17.08), SIMDE_FLOAT32_C(    13.67) } },
    { { SIMDE_FLOAT32_C(  -948.47), SIMDE_FLOAT32_C(   987.01), SIMDE_FLOAT32_C(   630.41), SIMDE_FLOAT32_C(  -637.23) },
      { SIMDE_FLOAT32_C(   -16.55), SIMDE_FLOAT32_C(    17.23), SIMDE_FLOAT32_C(    11.00), SIMDE_FLOAT32_C(   -11.12) } },
    { { SIMDE_FLOAT32_C(    66.92), SIMDE_FLOAT32_C(   674.00), SIMDE_FLOAT32_C(   -52.88), SIMDE_FLOAT32_C(  -732.15) },
      { SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(    11.76), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(   -12.78) } },
    { { SIMDE_FLOAT32_C(   750.47), SIMDE_FLOAT32_C(  -906.63), SIMDE_FLOAT32_C(   205.33), SIMDE_FLOAT32_C(  -941.95) },
      { SIMDE_FLOAT32_C(    13.10), SIMDE_FLOAT32_C(   -15.82), SIMDE_FLOAT32_C(     3.58), SIMDE_FLOAT32_C(   -16.44) } },
    { { SIMDE_FLOAT32_C(   705.35), SIMDE_FLOAT32_C(   774.66), SIMDE_FLOAT32_C(  -289.06), SIMDE_FLOAT32_C(  -214.64) },
      { SIMDE_FLOAT32_C(    12.31), SIMDE_FLOAT32_C(    13.52), SIMDE_FLOAT32_C(    -5.05), SIMDE_FLOAT32_C(    -3.75) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_x_mm_deg2rad_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_x_mm_deg2rad_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -666.18), SIMDE_FLOAT64_C(  -415.97) },
      { SIMDE_FLOAT64_C(   -11.63), SIMDE_FLOAT64_C(    -7.26) } },
    { { SIMDE_FLOAT64_C(   793.43), SIMDE_FLOAT64_C(  -853.65) },
      { SIMDE_FLOAT64_C(    13.85), SIMDE_FLOAT64_C(   -14.90) } },
    { { SIMDE_FLOAT64_C(   738.56), SIMDE_FLOAT64_C(   967.23) },
      { SIMDE_FLOAT64_C(    12.89), SIMDE_FLOAT64_C(    16.88) } },
    { { SIMDE_FLOAT64_C(   309.17), SIMDE_FLOAT64_C(   265.53) },
      { SIMDE_FLOAT64_C(     5.40), SIMDE_FLOAT64_C(     4.63) } },
    { { SIMDE_FLOAT64_C(   844.47), SIMDE_FLOAT64_C(   938.60) },
      { SIMDE_FLOAT64_C(    14.74), SIMDE_FLOAT64_C(    16.38) } },
    { { SIMDE_FLOAT64_C(  -902.86), SIMDE_FLOAT64_C(  -334.71) },
      { SIMDE_FLOAT64_C(   -15.76), SIMDE_FLOAT64_C(    -5.84) } },
    { { SIMDE_FLOAT64_C(   582.46), SIMDE_FLOAT64_C(  -651.74) },
      { SIMDE_FLOAT64_C(    10.17), SIMDE_FLOAT64_C(   -11.38) } },
    { { SIMDE_FLOAT64_C(   196.36), SIMDE_FLOAT64_C(   200.15) },
      { SIMDE_FLOAT64_C(     3.43), SIMDE_FLOAT64_C(     3.49) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_x_mm_deg2rad_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_x_mm256_deg2rad_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   442.73), SIMDE_FLOAT32_C(  -968.72), SIMDE_FLOAT32_C(   679.13), SIMDE_FLOAT32_C(   114.21),
        SIMDE_FLOAT32_C(  -467.66), SIMDE_FLOAT32_C(   -37.81), SIMDE_FLOAT32_C(   579.12), SIMDE_FLOAT32_C(  -687.98) },
      { SIMDE_FLOAT32_C(     7.73), SIMDE_FLOAT32_C(   -16.91), SIMDE_FLOAT32_C(    11.85), SIMDE_FLOAT32_C(     1.99),
        SIMDE_FLOAT32_C(    -8.16), SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(    10.11), SIMDE_FLOAT32_C(   -12.01) } },
    { { SIMDE_FLOAT32_C(  -896.03), SIMDE_FLOAT32_C(   496.82), SIMDE_FLOAT32_C(    46.75), SIMDE_FLOAT32_C(  -189.63),
        SIMDE_FLOAT32_C(   888.19), SIMDE_FLOAT32_C(  -178.85), SIMDE_FLOAT32_C(   106.49), SIMDE_FLOAT32_C(  -266.59) },
      { SIMDE_FLOAT32_C(   -15.64), SIMDE_FLOAT32_C(     8.67), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -3.31),
        SIMDE_FLOAT32_C(    15.50), SIMDE_FLOAT32_C(    -3.12), SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(    -4.65) } },
    { { SIMDE_FLOAT32_C(  -577.36), SIMDE_FLOAT32_C(   319.48), SIMDE_FLOAT32_C(  -568.91), SIMDE_FLOAT32_C(   369.60),
        SIMDE_FLOAT32_C(  -195.78), SIMDE_FLOAT32_C(  -445.13), SIMDE_FLOAT32_C(   676.76), SIMDE_FLOAT32_C(   270.74) },
      { SIMDE_FLOAT32_C(   -10.08), SIMDE_FLOAT32_C(     5.58), SIMDE_FLOAT32_C(    -9.93), SIMDE_FLOAT32_C(     6.45),
        SIMDE_FLOAT32_C(    -3.42), SIMDE_FLOAT32_C(    -7.77), SIMDE_FLOAT32_C(    11.81), SIMDE_FLOAT32_C(     4.73) } },
    { { SIMDE_FLOAT32_C(   386.69), SIMDE_FLOAT32_C(  -818.31), SIMDE_FLOAT32_C(   697.61), SIMDE_FLOAT32_C(   731.13),
        SIMDE_FLOAT32_C(    89.36), SIMDE_FLOAT32_C(  -163.03), SIMDE_FLOAT32_C(     9.17), SIMDE_FLOAT32_C(    76.19) },
      { SIMDE_FLOAT32_C(     6.75), SIMDE_FLOAT32_C(   -14.28), SIMDE_FLOAT32_C(    12.18), SIMDE_FLOAT32_C(    12.76),
        SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -2.85), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     1.33) } },
    { { SIMDE_FLOAT32_C(   522.23), SIMDE_FLOAT32_C(  -876.19), SIMDE_FLOAT32_C(  -206.90), SIMDE_FLOAT32_C(   647.79),
        SIMDE_FLOAT32_C(  -633.72), SIMDE_FLOAT32_C(  -908.37), SIMDE_FLOAT32_C(   944.64), SIMDE_FLOAT32_C(   520.31) },
      { SIMDE_FLOAT32_C(     9.11), SIMDE_FLOAT32_C(   -15.29), SIMDE_FLOAT32_C(    -3.61), SIMDE_FLOAT32_C(    11.31),
        SIMDE_FLOAT32_C(   -11.06), SIMDE_FLOAT32_C(   -15.85), SIMDE_FLOAT32_C(    16.49), SIMDE_FLOAT32_C(     9.08) } },
    { { SIMDE_FLOAT32_C(   907.89), SIMDE_FLOAT32_C(   849.63), SIMDE_FLOAT32_C(  -208.12), SIMDE_FLOAT32_C(    68.74),
        SIMDE_FLOAT32_C(  -670.75), SIMDE_FLOAT32_C(   677.18), SIMDE_FLOAT32_C(  -644.75), SIMDE_FLOAT32_C(  -292.10) },
      { SIMDE_FLOAT32_C(    15.85), SIMDE_FLOAT32_C(    14.83), SIMDE_FLOAT32_C(    -3.63), SIMDE_FLOAT32_C(     1.20),
        SIMDE_FLOAT32_C(   -11.71), SIMDE_FLOAT32_C(    11.82), SIMDE_FLOAT32_C(   -11.25), SIMDE_FLOAT32_C(    -5.10) } },
    { { SIMDE_FLOAT32_C(   675.40), SIMDE_FLOAT32_C(  -616.47), SIMDE_FLOAT32_C(   962.11), SIMDE_FLOAT32_C(   134.41),
        SIMDE_FLOAT32_C(  -905.98), SIMDE_FLOAT32_C(  -860.48), SIMDE_FLOAT32_C(   -24.28), SIMDE_FLOAT32_C(  -121.44) },
      { SIMDE_FLOAT32_C(    11.79), SIMDE_FLOAT32_C(   -10.76), SIMDE_FLOAT32_C(    16.79), SIMDE_FLOAT32_C(     2.35),
        SIMDE_FLOAT32_C(   -15.81), SIMDE_FLOAT32_C(   -15.02), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -2.12) } },
    { { SIMDE_FLOAT32_C(  -960.63), SIMDE_FLOAT32_C(   687.26), SIMDE_FLOAT32_C(   788.74), SIMDE_FLOAT32_C(   386.45),
        SIMDE_FLOAT32_C(  -901.72), SIMDE_FLOAT32_C(   856.65), SIMDE_FLOAT32_C(  -345.73), SIMDE_FLOAT32_C(  -616.97) },
      { SIMDE_FLOAT32_C(   -16.77), SIMDE_FLOAT32_C(    11.99), SIMDE_FLOAT32_C(    13.77), SIMDE_FLOAT32_C(     6.74),
        SIMDE_FLOAT32_C(   -15.74), SIMDE_FLOAT32_C(    14.95), SIMDE_FLOAT32_C(    -6.03), SIMDE_FLOAT32_C(   -10.77) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_x_mm256_deg2rad_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_x_mm256_deg2rad_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -467.83), SIMDE_FLOAT64_C(  -838.03), SIMDE_FLOAT64_C(  -852.25), SIMDE_FLOAT64_C(   261.37) },
      { SIMDE_FLOAT64_C(    -8.17), SIMDE_FLOAT64_C(   -14.63), SIMDE_FLOAT64_C(   -14.87), SIMDE_FLOAT64_C(     4.56) } },
    { { SIMDE_FLOAT64_C(   838.67), SIMDE_FLOAT64_C(  -424.12), SIMDE_FLOAT64_C(  -236.36), SIMDE_FLOAT64_C(  -471.04) },
      { SIMDE_FLOAT64_C(    14.64), SIMDE_FLOAT64_C(    -7.40), SIMDE_FLOAT64_C(    -4.13), SIMDE_FLOAT64_C(    -8.22) } },
    { { SIMDE_FLOAT64_C(  -834.32), SIMDE_FLOAT64_C(  -357.08), SIMDE_FLOAT64_C(   596.48), SIMDE_FLOAT64_C(   991.10) },
      { SIMDE_FLOAT64_C(   -14.56), SIMDE_FLOAT64_C(    -6.23), SIMDE_FLOAT64_C(    10.41), SIMDE_FLOAT64_C(    17.30) } },
    { { SIMDE_FLOAT64_C(  -638.79), SIMDE_FLOAT64_C(   -95.57), SIMDE_FLOAT64_C(  -262.62), SIMDE_FLOAT64_C(   117.35) },
      { SIMDE_FLOAT64_C(   -11.15), SIMDE_FLOAT64_C(    -1.67), SIMDE_FLOAT64_C(    -4.58), SIMDE_FLOAT64_C(     2.05) } },
    { { SIMDE_FLOAT64_C(   253.25), SIMDE_FLOAT64_C(   332.14), SIMDE_FLOAT64_C(   311.92), SIMDE_FLOAT64_C(   451.40) },
      { SIMDE_FLOAT64_C(     4.42), SIMDE_FLOAT64_C(     5.80), SIMDE_FLOAT64_C(     5.44), SIMDE_FLOAT64_C(     7.88) } },
    { { SIMDE_FLOAT64_C(   635.16), SIMDE_FLOAT64_C(  -795.05), SIMDE_FLOAT64_C(  -458.24), SIMDE_FLOAT64_C(   422.17) },
      { SIMDE_FLOAT64_C(    11.09), SIMDE_FLOAT64_C(   -13.88), SIMDE_FLOAT64_C(    -8.00), SIMDE_FLOAT64_C(     7.37) } },
    { { SIMDE_FLOAT64_C(  -505.84), SIMDE_FLOAT64_C(   400.55), SIMDE_FLOAT64_C(    54.12), SIMDE_FLOAT64_C(  -409.93) },
      { SIMDE_FLOAT64_C(    -8.83), SIMDE_FLOAT64_C(     6.99), SIMDE_FLOAT64_C(     0.94), SIMDE_FLOAT64_C(    -7.15) } },
    { { SIMDE_FLOAT64_C(   241.03), SIMDE_FLOAT64_C(  -950.08), SIMDE_FLOAT64_C(     5.55), SIMDE_FLOAT64_C(  -683.44) },
      { SIMDE_FLOAT64_C(     4.21), SIMDE_FLOAT64_C(   -16.58), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(   -11.93) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_x_mm256_deg2rad_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_x_mm512_deg2rad_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -204.97), SIMDE_FLOAT32_C(  -943.14), SIMDE_FLOAT32_C(   662.36), SIMDE_FLOAT32_C(   286.89),
        SIMDE_FLOAT32_C(  -272.57), SIMDE_FLOAT32_C(   978.11), SIMDE_FLOAT32_C(  -911.94), SIMDE_FLOAT32_C(  -924.18),
        SIMDE_FLOAT32_C(  -626.92), SIMDE_FLOAT32_C(  -721.73), SIMDE_FLOAT32_C(   -41.73), SIMDE_FLOAT32_C(   615.09),
        SIMDE_FLOAT32_C(  -253.85), SIMDE_FLOAT32_C(  -484.20), SIMDE_FLOAT32_C(   130.81), SIMDE_FLOAT32_C(   548.86) },
      { SIMDE_FLOAT32_C(    -3.58), SIMDE_FLOAT32_C(   -16.46), SIMDE_FLOAT32_C(    11.56), SIMDE_FLOAT32_C(     5.01),
        SIMDE_FLOAT32_C(    -4.76), SIMDE_FLOAT32_C(    17.07), SIMDE_FLOAT32_C(   -15.92), SIMDE_FLOAT32_C(   -16.13),
        SIMDE_FLOAT32_C(   -10.94), SIMDE_FLOAT32_C(   -12.60), SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(    10.74),
        SIMDE_FLOAT32_C(    -4.43), SIMDE_FLOAT32_C(    -8.45), SIMDE_FLOAT32_C(     2.28), SIMDE_FLOAT32_C(     9.58) } },
    { { SIMDE_FLOAT32_C(   759.71), SIMDE_FLOAT32_C(   445.37), SIMDE_FLOAT32_C(  -639.90), SIMDE_FLOAT32_C(  -816.54),
        SIMDE_FLOAT32_C(   349.70), SIMDE_FLOAT32_C(  -526.35), SIMDE_FLOAT32_C(  -291.02), SIMDE_FLOAT32_C(   855.10),
        SIMDE_FLOAT32_C(  -382.23), SIMDE_FLOAT32_C(   -58.28), SIMDE_FLOAT32_C(   435.56), SIMDE_FLOAT32_C(   388.92),
        SIMDE_FLOAT32_C(   616.34), SIMDE_FLOAT32_C(   879.74), SIMDE_FLOAT32_C(  -205.65), SIMDE_FLOAT32_C(  -284.03) },
      { SIMDE_FLOAT32_C(    13.26), SIMDE_FLOAT32_C(     7.77), SIMDE_FLOAT32_C(   -11.17), SIMDE_FLOAT32_C(   -14.25),
        SIMDE_FLOAT32_C(     6.10), SIMDE_FLOAT32_C(    -9.19), SIMDE_FLOAT32_C(    -5.08), SIMDE_FLOAT32_C(    14.92),
        SIMDE_FLOAT32_C(    -6.67), SIMDE_FLOAT32_C(    -1.02), SIMDE_FLOAT32_C(     7.60), SIMDE_FLOAT32_C(     6.79),
        SIMDE_FLOAT32_C(    10.76), SIMDE_FLOAT32_C(    15.35), SIMDE_FLOAT32_C(    -3.59), SIMDE_FLOAT32_C(    -4.96) } },
    { { SIMDE_FLOAT32_C(   252.00), SIMDE_FLOAT32_C(  -672.50), SIMDE_FLOAT32_C(  -750.03), SIMDE_FLOAT32_C(   219.53),
        SIMDE_FLOAT32_C(  -348.40), SIMDE_FLOAT32_C(   510.16), SIMDE_FLOAT32_C(   308.72), SIMDE_FLOAT32_C(   669.84),
        SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(   327.67), SIMDE_FLOAT32_C(  -780.79), SIMDE_FLOAT32_C(  -790.56),
        SIMDE_FLOAT32_C(   999.19), SIMDE_FLOAT32_C(  -674.94), SIMDE_FLOAT32_C(   338.16), SIMDE_FLOAT32_C(  -623.42) },
      { SIMDE_FLOAT32_C(     4.40), SIMDE_FLOAT32_C(   -11.74), SIMDE_FLOAT32_C(   -13.09), SIMDE_FLOAT32_C(     3.83),
        SIMDE_FLOAT32_C(    -6.08), SIMDE_FLOAT32_C(     8.90), SIMDE_FLOAT32_C(     5.39), SIMDE_FLOAT32_C(    11.69),
        SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     5.72), SIMDE_FLOAT32_C(   -13.63), SIMDE_FLOAT32_C(   -13.80),
        SIMDE_FLOAT32_C(    17.44), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(     5.90), SIMDE_FLOAT32_C(   -10.88) } },
    { { SIMDE_FLOAT32_C(   210.99), SIMDE_FLOAT32_C(   133.74), SIMDE_FLOAT32_C(  -196.68), SIMDE_FLOAT32_C(   412.53),
        SIMDE_FLOAT32_C(  -531.14), SIMDE_FLOAT32_C(  -816.95), SIMDE_FLOAT32_C(  -550.15), SIMDE_FLOAT32_C(  -344.98),
        SIMDE_FLOAT32_C(   -32.75), SIMDE_FLOAT32_C(  -439.61), SIMDE_FLOAT32_C(  -503.00), SIMDE_FLOAT32_C(    19.70),
        SIMDE_FLOAT32_C(  -850.81), SIMDE_FLOAT32_C(   392.70), SIMDE_FLOAT32_C(    36.21), SIMDE_FLOAT32_C(   667.59) },
      { SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(    -3.43), SIMDE_FLOAT32_C(     7.20),
        SIMDE_FLOAT32_C(    -9.27), SIMDE_FLOAT32_C(   -14.26), SIMDE_FLOAT32_C(    -9.60), SIMDE_FLOAT32_C(    -6.02),
        SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -7.67), SIMDE_FLOAT32_C(    -8.78), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(   -14.85), SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    11.65) } },
    { { SIMDE_FLOAT32_C(   226.81), SIMDE_FLOAT32_C(   -68.31), SIMDE_FLOAT32_C(   -92.58), SIMDE_FLOAT32_C(     1.70),
        SIMDE_FLOAT32_C(   617.13), SIMDE_FLOAT32_C(    53.88), SIMDE_FLOAT32_C(  -383.79), SIMDE_FLOAT32_C(  -333.97),
        SIMDE_FLOAT32_C(   936.36), SIMDE_FLOAT32_C(  -516.23), SIMDE_FLOAT32_C(  -313.77), SIMDE_FLOAT32_C(   516.09),
        SIMDE_FLOAT32_C(   -12.76), SIMDE_FLOAT32_C(  -491.30), SIMDE_FLOAT32_C(   729.84), SIMDE_FLOAT32_C(   483.88) },
      { SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(    -1.62), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(    10.77), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(    -6.70), SIMDE_FLOAT32_C(    -5.83),
        SIMDE_FLOAT32_C(    16.34), SIMDE_FLOAT32_C(    -9.01), SIMDE_FLOAT32_C(    -5.48), SIMDE_FLOAT32_C(     9.01),
        SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    -8.57), SIMDE_FLOAT32_C(    12.74), SIMDE_FLOAT32_C(     8.45) } },
    { { SIMDE_FLOAT32_C(   619.03), SIMDE_FLOAT32_C(   -43.28), SIMDE_FLOAT32_C(   522.00), SIMDE_FLOAT32_C(  -713.37),
        SIMDE_FLOAT32_C(   394.03), SIMDE_FLOAT32_C(   425.58), SIMDE_FLOAT32_C(   710.40), SIMDE_FLOAT32_C(  -291.67),
        SIMDE_FLOAT32_C(  -116.91), SIMDE_FLOAT32_C(  -890.48), SIMDE_FLOAT32_C(  -316.42), SIMDE_FLOAT32_C(   -26.59),
        SIMDE_FLOAT32_C(  -918.69), SIMDE_FLOAT32_C(  -397.83), SIMDE_FLOAT32_C(  -284.98), SIMDE_FLOAT32_C(   339.56) },
      { SIMDE_FLOAT32_C(    10.80), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     9.11), SIMDE_FLOAT32_C(   -12.45),
        SIMDE_FLOAT32_C(     6.88), SIMDE_FLOAT32_C(     7.43), SIMDE_FLOAT32_C(    12.40), SIMDE_FLOAT32_C(    -5.09),
        SIMDE_FLOAT32_C(    -2.04), SIMDE_FLOAT32_C(   -15.54), SIMDE_FLOAT32_C(    -5.52), SIMDE_FLOAT32_C(    -0.46),
        SIMDE_FLOAT32_C(   -16.03), SIMDE_FLOAT32_C(    -6.94), SIMDE_FLOAT32_C(    -4.97), SIMDE_FLOAT32_C(     5.93) } },
    { { SIMDE_FLOAT32_C(  -935.68), SIMDE_FLOAT32_C(   109.78), SIMDE_FLOAT32_C(  -972.99), SIMDE_FLOAT32_C(   894.31),
        SIMDE_FLOAT32_C(   633.79), SIMDE_FLOAT32_C(    41.84), SIMDE_FLOAT32_C(  -852.93), SIMDE_FLOAT32_C(   776.08),
        SIMDE_FLOAT32_C(  -443.88), SIMDE_FLOAT32_C(  -301.71), SIMDE_FLOAT32_C(  -808.76), SIMDE_FLOAT32_C(  -785.15),
        SIMDE_FLOAT32_C(   -67.76), SIMDE_FLOAT32_C(  -895.91), SIMDE_FLOAT32_C(   478.10), SIMDE_FLOAT32_C(  -636.03) },
      { SIMDE_FLOAT32_C(   -16.33), SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(   -16.98), SIMDE_FLOAT32_C(    15.61),
        SIMDE_FLOAT32_C(    11.06), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(   -14.89), SIMDE_FLOAT32_C(    13.55),
        SIMDE_FLOAT32_C(    -7.75), SIMDE_FLOAT32_C(    -5.27), SIMDE_FLOAT32_C(   -14.12), SIMDE_FLOAT32_C(   -13.70),
        SIMDE_FLOAT32_C(    -1.18), SIMDE_FLOAT32_C(   -15.64), SIMDE_FLOAT32_C(     8.34), SIMDE_FLOAT32_C(   -11.10) } },
    { { SIMDE_FLOAT32_C(   320.10), SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(   -21.63), SIMDE_FLOAT32_C(   500.34),
        SIMDE_FLOAT32_C(  -733.82), SIMDE_FLOAT32_C(   741.17), SIMDE_FLOAT32_C(   921.80), SIMDE_FLOAT32_C(   676.47),
        SIMDE_FLOAT32_C(  -545.48), SIMDE_FLOAT32_C(   136.48), SIMDE_FLOAT32_C(  -243.90), SIMDE_FLOAT32_C(   744.83),
        SIMDE_FLOAT32_C(   297.50), SIMDE_FLOAT32_C(   109.44), SIMDE_FLOAT32_C(  -667.13), SIMDE_FLOAT32_C(  -475.76) },
      { SIMDE_FLOAT32_C(     5.59), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(     8.73),
        SIMDE_FLOAT32_C(   -12.81), SIMDE_FLOAT32_C(    12.94), SIMDE_FLOAT32_C(    16.09), SIMDE_FLOAT32_C(    11.81),
        SIMDE_FLOAT32_C(    -9.52), SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(    -4.26), SIMDE_FLOAT32_C(    13.00),
        SIMDE_FLOAT32_C(     5.19), SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(   -11.64), SIMDE_FLOAT32_C(    -8.30) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_x_mm512_deg2rad_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_x_mm512_deg2rad_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   984.73), SIMDE_FLOAT64_C(   383.87), SIMDE_FLOAT64_C(  -286.43), SIMDE_FLOAT64_C(    18.78),
        SIMDE_FLOAT64_C(  -399.99), SIMDE_FLOAT64_C(  -675.58), SIMDE_FLOAT64_C(  -438.55), SIMDE_FLOAT64_C(  -737.71) },
      { SIMDE_FLOAT64_C(    17.19), SIMDE_FLOAT64_C(     6.70), SIMDE_FLOAT64_C(    -5.00), SIMDE_FLOAT64_C(     0.33),
        SIMDE_FLOAT64_C(    -6.98), SIMDE_FLOAT64_C(   -11.79), SIMDE_FLOAT64_C(    -7.65), SIMDE_FLOAT64_C(   -12.88) } },
    { { SIMDE_FLOAT64_C(  -671.93), SIMDE_FLOAT64_C(   826.99), SIMDE_FLOAT64_C(  -830.65), SIMDE_FLOAT64_C(  -694.10),
        SIMDE_FLOAT64_C(   255.50), SIMDE_FLOAT64_C(   118.40), SIMDE_FLOAT64_C(   -39.28), SIMDE_FLOAT64_C(  -160.67) },
      { SIMDE_FLOAT64_C(   -11.73), SIMDE_FLOAT64_C(    14.43), SIMDE_FLOAT64_C(   -14.50), SIMDE_FLOAT64_C(   -12.11),
        SIMDE_FLOAT64_C(     4.46), SIMDE_FLOAT64_C(     2.07), SIMDE_FLOAT64_C(    -0.69), SIMDE_FLOAT64_C(    -2.80) } },
    { { SIMDE_FLOAT64_C(  -422.40), SIMDE_FLOAT64_C(   720.88), SIMDE_FLOAT64_C(  -179.50), SIMDE_FLOAT64_C(  -877.62),
        SIMDE_FLOAT64_C(  -132.27), SIMDE_FLOAT64_C(   998.68), SIMDE_FLOAT64_C(   784.22), SIMDE_FLOAT64_C(   465.33) },
      { SIMDE_FLOAT64_C(    -7.37), SIMDE_FLOAT64_C(    12.58), SIMDE_FLOAT64_C(    -3.13), SIMDE_FLOAT64_C(   -15.32),
        SIMDE_FLOAT64_C(    -2.31), SIMDE_FLOAT64_C(    17.43), SIMDE_FLOAT64_C(    13.69), SIMDE_FLOAT64_C(     8.12) } },
    { { SIMDE_FLOAT64_C(   844.52), SIMDE_FLOAT64_C(   -91.48), SIMDE_FLOAT64_C(   575.23), SIMDE_FLOAT64_C(  -167.13),
        SIMDE_FLOAT64_C(  -906.69), SIMDE_FLOAT64_C(  -808.01), SIMDE_FLOAT64_C(  -191.68), SIMDE_FLOAT64_C(   439.44) },
      { SIMDE_FLOAT64_C(    14.74), SIMDE_FLOAT64_C(    -1.60), SIMDE_FLOAT64_C(    10.04), SIMDE_FLOAT64_C(    -2.92),
        SIMDE_FLOAT64_C(   -15.82), SIMDE_FLOAT64_C(   -14.10), SIMDE_FLOAT64_C(    -3.35), SIMDE_FLOAT64_C(     7.67) } },
    { { SIMDE_FLOAT64_C(  -327.12), SIMDE_FLOAT64_C(    74.58), SIMDE_FLOAT64_C(  -612.17), SIMDE_FLOAT64_C(  -701.50),
        SIMDE_FLOAT64_C(  -128.00), SIMDE_FLOAT64_C(   625.20), SIMDE_FLOAT64_C(  -218.65), SIMDE_FLOAT64_C(  -917.42) },
      { SIMDE_FLOAT64_C(    -5.71), SIMDE_FLOAT64_C(     1.30), SIMDE_FLOAT64_C(   -10.68), SIMDE_FLOAT64_C(   -12.24),
        SIMDE_FLOAT64_C(    -2.23), SIMDE_FLOAT64_C(    10.91), SIMDE_FLOAT64_C(    -3.82), SIMDE_FLOAT64_C(   -16.01) } },
    { { SIMDE_FLOAT64_C(  -997.92), SIMDE_FLOAT64_C(   -38.58), SIMDE_FLOAT64_C(  -337.38), SIMDE_FLOAT64_C(  -285.85),
        SIMDE_FLOAT64_C(  -318.88), SIMDE_FLOAT64_C(   574.80), SIMDE_FLOAT64_C(   587.94), SIMDE_FLOAT64_C(  -489.48) },
      { SIMDE_FLOAT64_C(   -17.42), SIMDE_FLOAT64_C(    -0.67), SIMDE_FLOAT64_C(    -5.89), SIMDE_FLOAT64_C(    -4.99),
        SIMDE_FLOAT64_C(    -5.57), SIMDE_FLOAT64_C(    10.03), SIMDE_FLOAT64_C(    10.26), SIMDE_FLOAT64_C(    -8.54) } },
    { { SIMDE_FLOAT64_C(  -699.61), SIMDE_FLOAT64_C(  -288.00), SIMDE_FLOAT64_C(  -454.37), SIMDE_FLOAT64_C(  -597.58),
        SIMDE_FLOAT64_C(   496.99), SIMDE_FLOAT64_C(   888.51), SIMDE_FLOAT64_C(  -818.76), SIMDE_FLOAT64_C(  -819.32) },
      { SIMDE_FLOAT64_C(   -12.21), SIMDE_FLOAT64_C(    -5.03), SIMDE_FLOAT64_C(    -7.93), SIMDE_FLOAT64_C(   -10.43),
        SIMDE_FLOAT64_C(     8.67), SIMDE_FLOAT64_C(    15.51), SIMDE_FLOAT64_C(   -14.29), SIMDE_FLOAT64_C(   -14.30) } },
    { { SIMDE_FLOAT64_C(  -315.95), SIMDE_FLOAT64_C(  -109.61), SIMDE_FLOAT64_C(  -186.03), SIMDE_FLOAT64_C(  -677.21),
        SIMDE_FLOAT64_C(    98.17), SIMDE_FLOAT64_C(   -43.95), SIMDE_FLOAT64_C(  -639.89), SIMDE_FLOAT64_C(  -591.44) },
      { SIMDE_FLOAT64_C(    -5.51), SIMDE_FLOAT64_C(    -1.91), SIMDE_FLOAT64_C(    -3.25), SIMDE_FLOAT64_C(   -11.82),
        SIMDE_FLOAT64_C(     1.71), SIMDE_FLOAT64_C(    -0.77), SIMDE_FLOAT64_C(   -11.17), SIMDE_FLOAT64_C(   -10.32) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_x_mm512_deg2rad_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_div_epi8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_mm_set_epi8(INT8_C(  80), INT8_C(  26), INT8_C( -96), INT8_C(  63),
                        INT8_C(  84), INT8_C(   0), INT8_C(  86), INT8_C( -92),
                        INT8_C(  19), INT8_C(  73), INT8_C(  49), INT8_C(  84),
                        INT8_C(  93), INT8_C( -26), INT8_C(  48), INT8_C( -85)),
      simde_mm_set_epi8(INT8_C(   4), INT8_C(   4), INT8_C(   3), INT8_C(  27),
                        INT8_C(  44), INT8_C(  48), INT8_C(   3), INT8_C(  53),
                        INT8_C(  11), INT8_C(   6), INT8_C(   2), INT8_C(  14),
                        INT8_C(  89), INT8_C(  10), INT8_C(   3), INT8_C(   1)),
      simde_mm_set_epi8(INT8_C(  20), INT8_C(   6), INT8_C( -32), INT8_C(   2),
                        INT8_C(   1), INT8_C(   0), INT8_C(  28), INT8_C(  -1),
                        INT8_C(   1), INT8_C(  12), INT8_C(  24), INT8_C(   6),
                        INT8_C(   1), INT8_C(  -2), INT8_C(  16), INT8_C( -85)) },
    { simde_mm_set_epi8(INT8_C( -53), INT8_C(-123), INT8_C(  83), INT8_C(  82),
                        INT8_C( -17), INT8_C(  32), INT8_C( -32), INT8_C(  68),
                        INT8_C( -20), INT8_C(   5), INT8_C(  -1), INT8_C( -23),
                        INT8_C( 118), INT8_C(-101), INT8_C(  53), INT8_C(   4)),
      simde_mm_set_epi8(INT8_C(   9), INT8_C(   1), INT8_C( -68), INT8_C(   1),
                        INT8_C(   1), INT8_C(   1), INT8_C(  22), INT8_C(  17),
                        INT8_C(   4), INT8_C(   8), INT8_C(   6), INT8_C(  10),
                        INT8_C(  55), INT8_C(   3), INT8_C(  14), INT8_C(  14)),
      simde_mm_set_epi8(INT8_C(  -5), INT8_C(-123), INT8_C(  -1), INT8_C(  82),
                        INT8_C( -17), INT8_C(  32), INT8_C(  -1), INT8_C(   4),
                        INT8_C(  -5), INT8_C(   0), INT8_C(   0), INT8_C(  -2),
                        INT8_C(   2), INT8_C( -33), INT8_C(   3), INT8_C(   0)) },
    { simde_mm_set_epi8(INT8_C( 122), INT8_C( 103), INT8_C(  28), INT8_C(-102),
                        INT8_C( -41), INT8_C(-105), INT8_C( -14), INT8_C(-120),
                        INT8_C( -71), INT8_C(  84), INT8_C(  90), INT8_C(   8),
                        INT8_C(  84), INT8_C( 120), INT8_C( -59), INT8_C(   9)),
      simde_mm_set_epi8(INT8_C(  59), INT8_C( -21), INT8_C(  22), INT8_C(  53),
                        INT8_C(  22), INT8_C(   3), INT8_C(   5), INT8_C(   6),
                        INT8_C(   2), INT8_C(  21), INT8_C(   3), INT8_C(   3),
                        INT8_C(   2), INT8_C(  10), INT8_C(  10), INT8_C(   3)),
      simde_mm_set_epi8(INT8_C(   2), INT8_C(  -4), INT8_C(   1), INT8_C(  -1),
                        INT8_C(  -1), INT8_C( -35), INT8_C(  -2), INT8_C( -20),
                        INT8_C( -35), INT8_C(   4), INT8_C(  30), INT8_C(   2),
                        INT8_C(  42), INT8_C(  12), INT8_C(  -5), INT8_C(   3)) },
    { simde_mm_set_epi8(INT8_C( 121), INT8_C( -15), INT8_C(-123), INT8_C(  80),
                        INT8_C(  43), INT8_C(  58), INT8_C( 119), INT8_C( -49),
                        INT8_C( 107), INT8_C( -94), INT8_C(  51), INT8_C(-118),
                        INT8_C(  68), INT8_C( 112), INT8_C( -56), INT8_C(-103)),
      simde_mm_set_epi8(INT8_C(  44), INT8_C(  13), INT8_C(  14), INT8_C(   8),
                        INT8_C( -24), INT8_C(  77), INT8_C( 118), INT8_C(  21),
                        INT8_C(   1), INT8_C( -34), INT8_C(   2), INT8_C(  29),
                        INT8_C(  14), INT8_C(  53), INT8_C(   1), INT8_C(  54)),
      simde_mm_set_epi8(INT8_C(   2), INT8_C(  -1), INT8_C(  -8), INT8_C(  10),
                        INT8_C(  -1), INT8_C(   0), INT8_C(   1), INT8_C(  -2),
                        INT8_C( 107), INT8_C(   2), INT8_C(  25), INT8_C(  -4),
                        INT8_C(   4), INT8_C(   2), INT8_C( -56), INT8_C(  -1)) },
    { simde_mm_set_epi8(INT8_C( -42), INT8_C(  14), INT8_C(-113), INT8_C(  62),
                        INT8_C( -34), INT8_C( -16), INT8_C(-103), INT8_C(-122),
                        INT8_C(-128), INT8_C( -77), INT8_C( -15), INT8_C( -38),
                        INT8_C(  87), INT8_C( -72), INT8_C(  57), INT8_C( -40)),
      simde_mm_set_epi8(INT8_C(  30), INT8_C( 124), INT8_C( -94), INT8_C(   4),
                        INT8_C(  46), INT8_C(  11), INT8_C(   3), INT8_C( -54),
                        INT8_C(  11), INT8_C(   8), INT8_C(-114), INT8_C(   3),
                        INT8_C(   6), INT8_C(   1), INT8_C(-121), INT8_C(   4)),
      simde_mm_set_epi8(INT8_C(  -1), INT8_C(   0), INT8_C(   1), INT8_C(  15),
                        INT8_C(   0), INT8_C(  -1), INT8_C( -34), INT8_C(   2),
                        INT8_C( -11), INT8_C(  -9), INT8_C(   0), INT8_C( -12),
                        INT8_C(  14), INT8_C( -72), INT8_C(   0), INT8_C( -10)) },
    { simde_mm_set_epi8(INT8_C( -13), INT8_C( -82), INT8_C(  64), INT8_C( -67),
                        INT8_C(-120), INT8_C(  26), INT8_C(-105), INT8_C(  40),
                        INT8_C(  59), INT8_C( -83), INT8_C(  64), INT8_C( -39),
                        INT8_C(  99), INT8_C( -73), INT8_C( -97), INT8_C(  -1)),
      simde_mm_set_epi8(INT8_C( -27), INT8_C( 114), INT8_C(-109), INT8_C(   8),
                        INT8_C(  12), INT8_C(   4), INT8_C(   2), INT8_C(   2),
                        INT8_C(   3), INT8_C(  11), INT8_C(   3), INT8_C(  11),
                        INT8_C(  82), INT8_C(  14), INT8_C( 120), INT8_C(-107)),
      simde_mm_set_epi8(INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(  -8),
                        INT8_C( -10), INT8_C(   6), INT8_C( -52), INT8_C(  20),
                        INT8_C(  19), INT8_C(  -7), INT8_C(  21), INT8_C(  -3),
                        INT8_C(   1), INT8_C(  -5), INT8_C(   0), INT8_C(   0)) },
    { simde_mm_set_epi8(INT8_C( -57), INT8_C(  53), INT8_C( 114), INT8_C( -35),
                        INT8_C( -22), INT8_C( -59), INT8_C(  52), INT8_C( 113),
                        INT8_C(  25), INT8_C(  16), INT8_C(  -8), INT8_C( -67),
                        INT8_C(   7), INT8_C( -33), INT8_C(  51), INT8_C( 118)),
      simde_mm_set_epi8(INT8_C(  14), INT8_C(  15), INT8_C(  24), INT8_C(  83),
                        INT8_C(   4), INT8_C(  45), INT8_C(   4), INT8_C(  34),
                        INT8_C(   9), INT8_C(  19), INT8_C(   4), INT8_C(  11),
                        INT8_C(   8), INT8_C(  14), INT8_C( 102), INT8_C( -88)),
      simde_mm_set_epi8(INT8_C(  -4), INT8_C(   3), INT8_C(   4), INT8_C(   0),
                        INT8_C(  -5), INT8_C(  -1), INT8_C(  13), INT8_C(   3),
                        INT8_C(   2), INT8_C(   0), INT8_C(  -2), INT8_C(  -6),
                        INT8_C(   0), INT8_C(  -2), INT8_C(   0), INT8_C(  -1)) },
    { simde_mm_set_epi8(INT8_C( -69), INT8_C(  57), INT8_C(   3), INT8_C( 127),
                        INT8_C( -28), INT8_C( -47), INT8_C(-127), INT8_C( -14),
                        INT8_C( -28), INT8_C(  68), INT8_C( -27), INT8_C( -44),
                        INT8_C( -16), INT8_C(   1), INT8_C( -44), INT8_C( 112)),
      simde_mm_set_epi8(INT8_C(  57), INT8_C(   1), INT8_C( -43), INT8_C( 103),
                        INT8_C(   4), INT8_C(   1), INT8_C(   2), INT8_C(  96),
                        INT8_C(   9), INT8_C(  57), INT8_C(  54), INT8_C( 105),
                        INT8_C(   1), INT8_C(  31), INT8_C( -85), INT8_C( 104)),
      simde_mm_set_epi8(INT8_C(  -1), INT8_C(  57), INT8_C(   0), INT8_C(   1),
                        INT8_C(  -7), INT8_C( -47), INT8_C( -63), INT8_C(   0),
                        INT8_C(  -3), INT8_C(   1), INT8_C(   0), INT8_C(   0),
                        INT8_C( -16), INT8_C(   0), INT8_C(   0), INT8_C(   1)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_div_epi8(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_i8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_div_epi16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_mm_set_epi16(INT16_C(  7569), INT16_C(-21774), INT16_C(  5125), INT16_C( 21356),
                         INT16_C(  9222), INT16_C(  7511), INT16_C(-21561), INT16_C( 29102)),
      simde_mm_set_epi16(INT16_C(  6450), INT16_C(    -2), INT16_C(   190), INT16_C(   -44),
                         INT16_C(    -3), INT16_C(    -9), INT16_C(  -911), INT16_C(     3)),
      simde_mm_set_epi16(INT16_C(     1), INT16_C( 10887), INT16_C(    26), INT16_C(  -485),
                         INT16_C( -3074), INT16_C(  -834), INT16_C(    23), INT16_C(  9700)) },
    { simde_mm_set_epi16(INT16_C( 14790), INT16_C(-17845), INT16_C( 12471), INT16_C( 16666),
                         INT16_C( -4541), INT16_C( 18926), INT16_C(  4112), INT16_C( 26905)),
      simde_mm_set_epi16(INT16_C(    -1), INT16_C(    -8), INT16_C(    15), INT16_C(   -16),
                         INT16_C(    -1), INT16_C(   -28), INT16_C( -3387), INT16_C(    -5)),
      simde_mm_set_epi16(INT16_C(-14790), INT16_C(  2230), INT16_C(   831), INT16_C( -1041),
                         INT16_C(  4541), INT16_C(  -675), INT16_C(    -1), INT16_C( -5381)) },
    { simde_mm_set_epi16(INT16_C( 24700), INT16_C( 18820), INT16_C( -6493), INT16_C(-11852),
                         INT16_C(  7293), INT16_C( 18330), INT16_C(-13423), INT16_C( 30834)),
      simde_mm_set_epi16(INT16_C(  9411), INT16_C(    -2), INT16_C(    -2), INT16_C(   -10),
                         INT16_C(   942), INT16_C(  5062), INT16_C(  3712), INT16_C(-24297)),
      simde_mm_set_epi16(INT16_C(     2), INT16_C( -9410), INT16_C(  3246), INT16_C(  1185),
                         INT16_C(     7), INT16_C(     3), INT16_C(    -3), INT16_C(    -1)) },
    { simde_mm_set_epi16(INT16_C( -8188), INT16_C( -5752), INT16_C( -6400), INT16_C(-18754),
                         INT16_C( 26203), INT16_C( 11990), INT16_C( 27655), INT16_C( 30479)),
      simde_mm_set_epi16(INT16_C( -2891), INT16_C(    -9), INT16_C(     1), INT16_C(    24),
                         INT16_C(  1410), INT16_C( -7348), INT16_C(    56), INT16_C(    -8)),
      simde_mm_set_epi16(INT16_C(     2), INT16_C(   639), INT16_C( -6400), INT16_C(  -781),
                         INT16_C(    18), INT16_C(    -1), INT16_C(   493), INT16_C( -3809)) },
    { simde_mm_set_epi16(INT16_C( 27464), INT16_C( 30742), INT16_C(-17463), INT16_C(  5584),
                         INT16_C( 16882), INT16_C(-13221), INT16_C(-30009), INT16_C( 27529)),
      simde_mm_set_epi16(INT16_C(    92), INT16_C(  -245), INT16_C(    87), INT16_C(  2027),
                         INT16_C(  -218), INT16_C(   181), INT16_C(     1), INT16_C(  -448)),
      simde_mm_set_epi16(INT16_C(   298), INT16_C(  -125), INT16_C(  -200), INT16_C(     2),
                         INT16_C(   -77), INT16_C(   -73), INT16_C(-30009), INT16_C(   -61)) },
    { simde_mm_set_epi16(INT16_C(-28312), INT16_C( -6464), INT16_C(  7438), INT16_C(-24771),
                         INT16_C( 27969), INT16_C( 18884), INT16_C( 17235), INT16_C( 31019)),
      simde_mm_set_epi16(INT16_C( -3989), INT16_C(     8), INT16_C(    -1), INT16_C(   -27),
                         INT16_C(    53), INT16_C(   -58), INT16_C(  2274), INT16_C(    -9)),
      simde_mm_set_epi16(INT16_C(     7), INT16_C(  -808), INT16_C( -7438), INT16_C(   917),
                         INT16_C(   527), INT16_C(  -325), INT16_C(     7), INT16_C( -3446)) },
    { simde_mm_set_epi16(INT16_C(-31090), INT16_C( 20346), INT16_C( 14276), INT16_C(-27653),
                         INT16_C( 19203), INT16_C(-24798), INT16_C(-17826), INT16_C( 16379)),
      simde_mm_set_epi16(INT16_C(     3), INT16_C(     8), INT16_C(   -60), INT16_C(    14),
                         INT16_C(  -435), INT16_C(    -1), INT16_C(  -395), INT16_C( -1532)),
      simde_mm_set_epi16(INT16_C(-10363), INT16_C(  2543), INT16_C(  -237), INT16_C( -1975),
                         INT16_C(   -44), INT16_C( 24798), INT16_C(    45), INT16_C(   -10)) },
    { simde_mm_set_epi16(INT16_C( -4012), INT16_C( 17981), INT16_C( 26341), INT16_C(-11451),
                         INT16_C(-22746), INT16_C(-13246), INT16_C( -6273), INT16_C( 15936)),
      simde_mm_set_epi16(INT16_C(    -5), INT16_C(   325), INT16_C(    10), INT16_C( -2018),
                         INT16_C(-26192), INT16_C(   -15), INT16_C(   -29), INT16_C(  2009)),
      simde_mm_set_epi16(INT16_C(   802), INT16_C(    55), INT16_C(  2634), INT16_C(     5),
                         INT16_C(     0), INT16_C(   883), INT16_C(   216), INT16_C(     7)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_div_epi16(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_i16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_div_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_mm_set_epi32(INT32_C(-2101284579), INT32_C( 1788896628), INT32_C(  742774378), INT32_C( -512831871)),
      simde_mm_set_epi32(INT32_C(       -173), INT32_C(  -20613654), INT32_C(      28772), INT32_C(        118)),
      simde_mm_set_epi32(INT32_C(   12146153), INT32_C(        -86), INT32_C(      25815), INT32_C(   -4346032)) },
    { simde_mm_set_epi32(INT32_C(  505370509), INT32_C( -307733024), INT32_C( -192358019), INT32_C( -299231491)),
      simde_mm_set_epi32(INT32_C(      34268), INT32_C(         -6), INT32_C(       6850), INT32_C(    1214711)),
      simde_mm_set_epi32(INT32_C(      14747), INT32_C(   51288837), INT32_C(     -28081), INT32_C(       -246)) },
    { simde_mm_set_epi32(INT32_C(-1154189768), INT32_C(   94538029), INT32_C(  423884488), INT32_C( 1619435962)),
      simde_mm_set_epi32(INT32_C(       -565), INT32_C(    -128659), INT32_C(        -59), INT32_C( -208397178)),
      simde_mm_set_epi32(INT32_C(    2042813), INT32_C(       -734), INT32_C(   -7184482), INT32_C(         -7)) },
    { simde_mm_set_epi32(INT32_C(-1938127942), INT32_C( -553846699), INT32_C(  685427224), INT32_C(  -86375451)),
      simde_mm_set_epi32(INT32_C( 1223981911), INT32_C(    -108113), INT32_C(          3), INT32_C(      -3698)),
      simde_mm_set_epi32(INT32_C(         -1), INT32_C(       5122), INT32_C(  228475741), INT32_C(      23357)) },
    { simde_mm_set_epi32(INT32_C(-1690889220), INT32_C( -667367235), INT32_C( 1220206139), INT32_C(-1217543723)),
      simde_mm_set_epi32(INT32_C(        299), INT32_C(       7724), INT32_C(         -1), INT32_C(  173051558)),
      simde_mm_set_epi32(INT32_C(   -5655147), INT32_C(     -86401), INT32_C(-1220206139), INT32_C(         -7)) },
    { simde_mm_set_epi32(INT32_C(   93323521), INT32_C( 1996592708), INT32_C( 2087305602), INT32_C(   27568495)),
      simde_mm_set_epi32(INT32_C(         -2), INT32_C(   15626723), INT32_C(       1507), INT32_C(       5412)),
      simde_mm_set_epi32(INT32_C(  -46661760), INT32_C(        127), INT32_C(    1385073), INT32_C(       5093)) },
    { simde_mm_set_epi32(INT32_C( 1825211631), INT32_C( 1750705004), INT32_C( 1935103134), INT32_C(-1042289581)),
      simde_mm_set_epi32(INT32_C(     -20153), INT32_C( -109992928), INT32_C(         -4), INT32_C(          3)),
      simde_mm_set_epi32(INT32_C(     -90567), INT32_C(        -15), INT32_C( -483775783), INT32_C( -347429860)) },
    { simde_mm_set_epi32(INT32_C( -836927167), INT32_C(-2031963629), INT32_C( 1244477192), INT32_C(  662038781)),
      simde_mm_set_epi32(INT32_C(       -226), INT32_C(        320), INT32_C(   17085036), INT32_C(       -883)),
      simde_mm_set_epi32(INT32_C(    3703217), INT32_C(   -6349886), INT32_C(         72), INT32_C(    -749760)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_div_epi32(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_i32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_div_epi64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_mm_set_epi64x(INT64_C(-8762915026342605517), INT64_C( 6327019035084041530)),
      simde_mm_set_epi64x(INT64_C( 1040172869250133860), INT64_C(         -3393154419)),
      simde_mm_set_epi64x(INT64_C(                  -8), INT64_C(         -1864642233)) },
    { simde_mm_set_epi64x(INT64_C( 7086115847005357544), INT64_C( 7169462887889416879)),
      simde_mm_set_epi64x(INT64_C(             -402272), INT64_C(            -6362438)),
      simde_mm_set_epi64x(INT64_C(     -17615235082246), INT64_C(      -1126842082844)) },
    { simde_mm_set_epi64x(INT64_C( 3227829673356714047), INT64_C( 5122063021698718134)),
      simde_mm_set_epi64x(INT64_C(              290796), INT64_C(             -647054)),
      simde_mm_set_epi64x(INT64_C(      11099979619240), INT64_C(      -7915974588981)) },
    { simde_mm_set_epi64x(INT64_C( -712959233727550094), INT64_C( 8175697730423622547)),
      simde_mm_set_epi64x(INT64_C(          -114108996), INT64_C(           727492806)),
      simde_mm_set_epi64x(INT64_C(          6248054568), INT64_C(         11238183612)) },
    { simde_mm_set_epi64x(INT64_C( 7475816922473172733), INT64_C(-1631503293395556188)),
      simde_mm_set_epi64x(INT64_C(                   5), INT64_C(        -24770378177)),
      simde_mm_set_epi64x(INT64_C( 1495163384494634546), INT64_C(            65865094)) },
    { simde_mm_set_epi64x(INT64_C(-7220293124938945390), INT64_C( 5345879758546587877)),
      simde_mm_set_epi64x(INT64_C(                -716), INT64_C(             1692902)),
      simde_mm_set_epi64x(INT64_C(   10084208275054393), INT64_C(       3157819979270)) },
    { simde_mm_set_epi64x(INT64_C(-2100788141468237692), INT64_C( 1869244361192362281)),
      simde_mm_set_epi64x(INT64_C(                  -1), INT64_C(         27867346395)),
      simde_mm_set_epi64x(INT64_C( 2100788141468237692), INT64_C(            67076510)) },
    { simde_mm_set_epi64x(INT64_C(-4218200756000910912), INT64_C( 8429274423139369867)),
      simde_mm_set_epi64x(INT64_C(                  25), INT64_C(        -63869567732)),
      simde_mm_set_epi64x(INT64_C( -168728030240036436), INT64_C(          -131976381)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_div_epi64(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_i64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_div_epu8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_x_mm_set_epu8(UINT8_C( 15), UINT8_C( 75), UINT8_C(224), UINT8_C(156),
                          UINT8_C(  1), UINT8_C( 34), UINT8_C( 35), UINT8_C(127),
                          UINT8_C(127), UINT8_C(120), UINT8_C(177), UINT8_C( 31),
                          UINT8_C(136), UINT8_C(180), UINT8_C(141), UINT8_C(206)),
      simde_x_mm_set_epu8(UINT8_C( 45), UINT8_C(  8), UINT8_C(  9), UINT8_C( 13),
                          UINT8_C(246), UINT8_C(  1), UINT8_C( 15), UINT8_C(  2),
                          UINT8_C(152), UINT8_C( 45), UINT8_C( 56), UINT8_C( 26),
                          UINT8_C(  1), UINT8_C(  1), UINT8_C( 16), UINT8_C( 15)),
      simde_x_mm_set_epu8(UINT8_C(  0), UINT8_C(  9), UINT8_C( 24), UINT8_C( 12),
                          UINT8_C(  0), UINT8_C( 34), UINT8_C(  2), UINT8_C( 63),
                          UINT8_C(  0), UINT8_C(  2), UINT8_C(  3), UINT8_C(  1),
                          UINT8_C(136), UINT8_C(180), UINT8_C(  8), UINT8_C( 13)) },
    { simde_x_mm_set_epu8(UINT8_C( 75), UINT8_C(233), UINT8_C(186), UINT8_C(216),
                          UINT8_C(224), UINT8_C( 45), UINT8_C( 40), UINT8_C(134),
                          UINT8_C(  1), UINT8_C( 47), UINT8_C( 23), UINT8_C(119),
                          UINT8_C(229), UINT8_C(107), UINT8_C(175), UINT8_C( 79)),
      simde_x_mm_set_epu8(UINT8_C(  9), UINT8_C( 12), UINT8_C( 46), UINT8_C( 39),
                          UINT8_C( 11), UINT8_C( 15), UINT8_C( 32), UINT8_C( 13),
                          UINT8_C( 21), UINT8_C(239), UINT8_C(  5), UINT8_C(  2),
                          UINT8_C(  1), UINT8_C( 26), UINT8_C(182), UINT8_C( 29)),
      simde_x_mm_set_epu8(UINT8_C(  8), UINT8_C( 19), UINT8_C(  4), UINT8_C(  5),
                          UINT8_C( 20), UINT8_C(  3), UINT8_C(  1), UINT8_C( 10),
                          UINT8_C(  0), UINT8_C(  0), UINT8_C(  4), UINT8_C( 59),
                          UINT8_C(229), UINT8_C(  4), UINT8_C(  0), UINT8_C(  2)) },
    { simde_x_mm_set_epu8(UINT8_C( 75), UINT8_C(109), UINT8_C( 28), UINT8_C(204),
                          UINT8_C( 53), UINT8_C(255), UINT8_C(143), UINT8_C(254),
                          UINT8_C( 82), UINT8_C(109), UINT8_C(205), UINT8_C( 21),
                          UINT8_C( 16), UINT8_C( 18), UINT8_C(221), UINT8_C(119)),
      simde_x_mm_set_epu8(UINT8_C(210), UINT8_C(108), UINT8_C( 89), UINT8_C( 21),
                          UINT8_C(154), UINT8_C( 52), UINT8_C( 17), UINT8_C(  8),
                          UINT8_C( 90), UINT8_C(  6), UINT8_C(  1), UINT8_C(  5),
                          UINT8_C(  1), UINT8_C(201), UINT8_C( 23), UINT8_C(  2)),
      simde_x_mm_set_epu8(UINT8_C(  0), UINT8_C(  1), UINT8_C(  0), UINT8_C(  9),
                          UINT8_C(  0), UINT8_C(  4), UINT8_C(  8), UINT8_C( 31),
                          UINT8_C(  0), UINT8_C( 18), UINT8_C(205), UINT8_C(  4),
                          UINT8_C( 16), UINT8_C(  0), UINT8_C(  9), UINT8_C( 59)) },
    { simde_x_mm_set_epu8(UINT8_C( 23), UINT8_C(229), UINT8_C(200), UINT8_C( 62),
                          UINT8_C(169), UINT8_C(116), UINT8_C(131), UINT8_C(205),
                          UINT8_C(117), UINT8_C( 49), UINT8_C(130), UINT8_C( 21),
                          UINT8_C( 91), UINT8_C(138), UINT8_C(101), UINT8_C(205)),
      simde_x_mm_set_epu8(UINT8_C( 43), UINT8_C( 65), UINT8_C( 28), UINT8_C( 61),
                          UINT8_C( 12), UINT8_C(  4), UINT8_C( 37), UINT8_C(  4),
                          UINT8_C(237), UINT8_C( 25), UINT8_C( 38), UINT8_C( 15),
                          UINT8_C(  9), UINT8_C(  6), UINT8_C(140), UINT8_C( 10)),
      simde_x_mm_set_epu8(UINT8_C(  0), UINT8_C(  3), UINT8_C(  7), UINT8_C(  1),
                          UINT8_C( 14), UINT8_C( 29), UINT8_C(  3), UINT8_C( 51),
                          UINT8_C(  0), UINT8_C(  1), UINT8_C(  3), UINT8_C(  1),
                          UINT8_C( 10), UINT8_C( 23), UINT8_C(  0), UINT8_C( 20)) },
    { simde_x_mm_set_epu8(UINT8_C(140), UINT8_C(170), UINT8_C(150), UINT8_C(208),
                          UINT8_C( 64), UINT8_C(  6), UINT8_C(116), UINT8_C(102),
                          UINT8_C(200), UINT8_C(110), UINT8_C(136), UINT8_C(125),
                          UINT8_C(201), UINT8_C( 22), UINT8_C(166), UINT8_C(235)),
      simde_x_mm_set_epu8(UINT8_C(  1), UINT8_C(  7), UINT8_C( 23), UINT8_C(  2),
                          UINT8_C( 12), UINT8_C(103), UINT8_C( 24), UINT8_C( 18),
                          UINT8_C(234), UINT8_C( 11), UINT8_C(  6), UINT8_C(  2),
                          UINT8_C(  5), UINT8_C( 34), UINT8_C( 60), UINT8_C( 13)),
      simde_x_mm_set_epu8(UINT8_C(140), UINT8_C( 24), UINT8_C(  6), UINT8_C(104),
                          UINT8_C(  5), UINT8_C(  0), UINT8_C(  4), UINT8_C(  5),
                          UINT8_C(  0), UINT8_C( 10), UINT8_C( 22), UINT8_C( 62),
                          UINT8_C( 40), UINT8_C(  0), UINT8_C(  2), UINT8_C( 18)) },
    { simde_x_mm_set_epu8(UINT8_C(143), UINT8_C( 77), UINT8_C(114), UINT8_C( 66),
                          UINT8_C( 82), UINT8_C(133), UINT8_C( 93), UINT8_C(122),
                          UINT8_C(225), UINT8_C(230), UINT8_C(202), UINT8_C(147),
                          UINT8_C(170), UINT8_C(252), UINT8_C(163), UINT8_C(161)),
      simde_x_mm_set_epu8(UINT8_C(  5), UINT8_C(  8), UINT8_C( 15), UINT8_C( 99),
                          UINT8_C( 10), UINT8_C(  4), UINT8_C(  1), UINT8_C(  1),
                          UINT8_C( 15), UINT8_C( 21), UINT8_C(  3), UINT8_C(  1),
                          UINT8_C(  2), UINT8_C( 18), UINT8_C( 18), UINT8_C(  2)),
      simde_x_mm_set_epu8(UINT8_C( 28), UINT8_C(  9), UINT8_C(  7), UINT8_C(  0),
                          UINT8_C(  8), UINT8_C( 33), UINT8_C( 93), UINT8_C(122),
                          UINT8_C( 15), UINT8_C( 10), UINT8_C( 67), UINT8_C(147),
                          UINT8_C( 85), UINT8_C( 14), UINT8_C(  9), UINT8_C( 80)) },
    { simde_x_mm_set_epu8(UINT8_C(125), UINT8_C(134), UINT8_C(114), UINT8_C( 16),
                          UINT8_C(101), UINT8_C( 75), UINT8_C( 71), UINT8_C(136),
                          UINT8_C(137), UINT8_C(104), UINT8_C(249), UINT8_C(115),
                          UINT8_C(110), UINT8_C(132), UINT8_C(229), UINT8_C( 48)),
      simde_x_mm_set_epu8(UINT8_C( 69), UINT8_C( 11), UINT8_C(  3), UINT8_C(  2),
                          UINT8_C(  2), UINT8_C( 21), UINT8_C(  3), UINT8_C(  1),
                          UINT8_C(  5), UINT8_C(  1), UINT8_C(  3), UINT8_C(  2),
                          UINT8_C(  1), UINT8_C(163), UINT8_C(  1), UINT8_C(  2)),
      simde_x_mm_set_epu8(UINT8_C(  1), UINT8_C( 12), UINT8_C( 38), UINT8_C(  8),
                          UINT8_C( 50), UINT8_C(  3), UINT8_C( 23), UINT8_C(136),
                          UINT8_C( 27), UINT8_C(104), UINT8_C( 83), UINT8_C( 57),
                          UINT8_C(110), UINT8_C(  0), UINT8_C(229), UINT8_C( 24)) },
    { simde_x_mm_set_epu8(UINT8_C( 72), UINT8_C(139), UINT8_C(120), UINT8_C(127),
                          UINT8_C(102), UINT8_C(165), UINT8_C( 82), UINT8_C( 63),
                          UINT8_C(192), UINT8_C( 18), UINT8_C(103), UINT8_C(151),
                          UINT8_C( 81), UINT8_C(222), UINT8_C(212), UINT8_C(  1)),
      simde_x_mm_set_epu8(UINT8_C(  7), UINT8_C( 26), UINT8_C( 32), UINT8_C(  1),
                          UINT8_C(  1), UINT8_C(  1), UINT8_C(  3), UINT8_C(  2),
                          UINT8_C( 65), UINT8_C( 24), UINT8_C(  1), UINT8_C( 97),
                          UINT8_C( 14), UINT8_C(  8), UINT8_C( 89), UINT8_C( 11)),
      simde_x_mm_set_epu8(UINT8_C( 10), UINT8_C(  5), UINT8_C(  3), UINT8_C(127),
                          UINT8_C(102), UINT8_C(165), UINT8_C( 27), UINT8_C( 31),
                          UINT8_C(  2), UINT8_C(  0), UINT8_C(103), UINT8_C(  1),
                          UINT8_C(  5), UINT8_C( 27), UINT8_C(  2), UINT8_C(  0)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_div_epu8(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_u8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_div_epu16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_x_mm_set_epu16(UINT16_C(27566), UINT16_C(40504), UINT16_C( 4629), UINT16_C(53715),
                           UINT16_C( 9716), UINT16_C( 9411), UINT16_C(47476), UINT16_C(41385)),
      simde_x_mm_set_epu16(UINT16_C(   13), UINT16_C( 6506), UINT16_C( 2031), UINT16_C( 2041),
                           UINT16_C(   41), UINT16_C( 3089), UINT16_C( 4707), UINT16_C(    3)),
      simde_x_mm_set_epu16(UINT16_C( 2120), UINT16_C(    6), UINT16_C(    2), UINT16_C(   26),
                           UINT16_C(  236), UINT16_C(    3), UINT16_C(   10), UINT16_C(13795)) },
    { simde_x_mm_set_epu16(UINT16_C( 9353), UINT16_C(  761), UINT16_C( 3256), UINT16_C(15648),
                           UINT16_C(54529), UINT16_C(37909), UINT16_C( 6524), UINT16_C(24806)),
      simde_x_mm_set_epu16(UINT16_C(17088), UINT16_C( 3660), UINT16_C(    3), UINT16_C(    9),
                           UINT16_C(  186), UINT16_C(    2), UINT16_C(    7), UINT16_C( 1856)),
      simde_x_mm_set_epu16(UINT16_C(    0), UINT16_C(    0), UINT16_C( 1085), UINT16_C( 1738),
                           UINT16_C(  293), UINT16_C(18954), UINT16_C(  932), UINT16_C(   13)) },
    { simde_x_mm_set_epu16(UINT16_C(19795), UINT16_C(45332), UINT16_C(60579), UINT16_C(32327),
                           UINT16_C(25905), UINT16_C(63671), UINT16_C(  930), UINT16_C(32017)),
      simde_x_mm_set_epu16(UINT16_C(    8), UINT16_C(30488), UINT16_C(   26), UINT16_C( 3397),
                           UINT16_C( 1518), UINT16_C(    2), UINT16_C(   20), UINT16_C(    6)),
      simde_x_mm_set_epu16(UINT16_C( 2474), UINT16_C(    1), UINT16_C( 2329), UINT16_C(    9),
                           UINT16_C(   17), UINT16_C(31835), UINT16_C(   46), UINT16_C( 5336)) },
    { simde_x_mm_set_epu16(UINT16_C(29801), UINT16_C(62435), UINT16_C(31106), UINT16_C(58247),
                           UINT16_C(47275), UINT16_C(34875), UINT16_C(63847), UINT16_C( 8602)),
      simde_x_mm_set_epu16(UINT16_C(    5), UINT16_C(    1), UINT16_C(  842), UINT16_C( 1634),
                           UINT16_C(   11), UINT16_C(   25), UINT16_C( 3640), UINT16_C(  932)),
      simde_x_mm_set_epu16(UINT16_C( 5960), UINT16_C(62435), UINT16_C(   36), UINT16_C(   35),
                           UINT16_C( 4297), UINT16_C( 1395), UINT16_C(   17), UINT16_C(    9)) },
    { simde_x_mm_set_epu16(UINT16_C(41564), UINT16_C(16940), UINT16_C(39647), UINT16_C(59460),
                           UINT16_C(17425), UINT16_C(59711), UINT16_C(30880), UINT16_C(42139)),
      simde_x_mm_set_epu16(UINT16_C(25139), UINT16_C( 3416), UINT16_C(   43), UINT16_C(    6),
                           UINT16_C(    4), UINT16_C( 1256), UINT16_C(   60), UINT16_C(  129)),
      simde_x_mm_set_epu16(UINT16_C(    1), UINT16_C(    4), UINT16_C(  922), UINT16_C( 9910),
                           UINT16_C( 4356), UINT16_C(   47), UINT16_C(  514), UINT16_C(  326)) },
    { simde_x_mm_set_epu16(UINT16_C(39593), UINT16_C(41522), UINT16_C(58894), UINT16_C( 6383),
                           UINT16_C(39956), UINT16_C( 2820), UINT16_C(20260), UINT16_C(57360)),
      simde_x_mm_set_epu16(UINT16_C(    1), UINT16_C(10468), UINT16_C(    2), UINT16_C(   79),
                           UINT16_C(    5), UINT16_C( 1166), UINT16_C(    2), UINT16_C(    3)),
      simde_x_mm_set_epu16(UINT16_C(39593), UINT16_C(    3), UINT16_C(29447), UINT16_C(   80),
                           UINT16_C( 7991), UINT16_C(    2), UINT16_C(10130), UINT16_C(19120)) },
    { simde_x_mm_set_epu16(UINT16_C(58633), UINT16_C(30014), UINT16_C(57061), UINT16_C(60439),
                           UINT16_C(22536), UINT16_C(20868), UINT16_C(20870), UINT16_C(13916)),
      simde_x_mm_set_epu16(UINT16_C(   15), UINT16_C(  490), UINT16_C( 2338), UINT16_C(   64),
                           UINT16_C(  876), UINT16_C(  706), UINT16_C(   65), UINT16_C(  320)),
      simde_x_mm_set_epu16(UINT16_C( 3908), UINT16_C(   61), UINT16_C(   24), UINT16_C(  944),
                           UINT16_C(   25), UINT16_C(   29), UINT16_C(  321), UINT16_C(   43)) },
    { simde_x_mm_set_epu16(UINT16_C( 6697), UINT16_C(21906), UINT16_C(59582), UINT16_C(44845),
                           UINT16_C(35883), UINT16_C(64682), UINT16_C(55100), UINT16_C(57711)),
      simde_x_mm_set_epu16(UINT16_C( 7058), UINT16_C(   10), UINT16_C(60566), UINT16_C(    1),
                           UINT16_C(    1), UINT16_C(  872), UINT16_C(  109), UINT16_C(    1)),
      simde_x_mm_set_epu16(UINT16_C(    0), UINT16_C( 2190), UINT16_C(    0), UINT16_C(44845),
                           UINT16_C(35883), UINT16_C(   74), UINT16_C(  505), UINT16_C(57711)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_div_epu16(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_u16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_div_epu32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_x_mm_set_epu32(UINT32_C(3152261024), UINT32_C(2598586578), UINT32_C(1610828679), UINT32_C(3536337768)),
      simde_x_mm_set_epu32(UINT32_C(     14157), UINT32_C(       947), UINT32_C(1043337665), UINT32_C(     97937)),
      simde_x_mm_set_epu32(UINT32_C(    222664), UINT32_C(   2744019), UINT32_C(         1), UINT32_C(     36108)) },
    { simde_x_mm_set_epu32(UINT32_C(  75140339), UINT32_C(1941562012), UINT32_C( 857740081), UINT32_C(1336535286)),
      simde_x_mm_set_epu32(UINT32_C(        22), UINT32_C(      1682), UINT32_C(        11), UINT32_C(         2)),
      simde_x_mm_set_epu32(UINT32_C(   3415469), UINT32_C(   1154317), UINT32_C(  77976371), UINT32_C( 668267643)) },
    { simde_x_mm_set_epu32(UINT32_C( 948661264), UINT32_C(1195769225), UINT32_C( 694120276), UINT32_C(3517239447)),
      simde_x_mm_set_epu32(UINT32_C(      3949), UINT32_C(       275), UINT32_C(  12430067), UINT32_C(     15794)),
      simde_x_mm_set_epu32(UINT32_C(    240228), UINT32_C(   4348251), UINT32_C(        55), UINT32_C(    222694)) },
    { simde_x_mm_set_epu32(UINT32_C(3023938951), UINT32_C(4109050401), UINT32_C( 287757059), UINT32_C(2648669825)),
      simde_x_mm_set_epu32(UINT32_C(     57756), UINT32_C(        40), UINT32_C(1080216164), UINT32_C(    173312)),
      simde_x_mm_set_epu32(UINT32_C(     52357), UINT32_C( 102726260), UINT32_C(         0), UINT32_C(     15282)) },
    { simde_x_mm_set_epu32(UINT32_C( 864299658), UINT32_C(2427378437), UINT32_C( 823539242), UINT32_C(1758563044)),
      simde_x_mm_set_epu32(UINT32_C(       225), UINT32_C(        75), UINT32_C(     11529), UINT32_C( 119418298)),
      simde_x_mm_set_epu32(UINT32_C(   3841331), UINT32_C(  32365045), UINT32_C(     71431), UINT32_C(        14)) },
    { simde_x_mm_set_epu32(UINT32_C(2662820398), UINT32_C(1208068616), UINT32_C(2158211537), UINT32_C(3417661837)),
      simde_x_mm_set_epu32(UINT32_C(      2367), UINT32_C(    126619), UINT32_C(     55203), UINT32_C(       155)),
      simde_x_mm_set_epu32(UINT32_C(   1124976), UINT32_C(      9540), UINT32_C(     39095), UINT32_C(  22049431)) },
    { simde_x_mm_set_epu32(UINT32_C(1097247740), UINT32_C(3448507951), UINT32_C(4106436665), UINT32_C(3017338787)),
      simde_x_mm_set_epu32(UINT32_C(  61963115), UINT32_C( 238397327), UINT32_C(    245318), UINT32_C(   3312135)),
      simde_x_mm_set_epu32(UINT32_C(        17), UINT32_C(        14), UINT32_C(     16739), UINT32_C(       910)) },
    { simde_x_mm_set_epu32(UINT32_C(3006363325), UINT32_C(2983927188), UINT32_C(2177891039), UINT32_C(1117727917)),
      simde_x_mm_set_epu32(UINT32_C(        24), UINT32_C(        12), UINT32_C(1067413818), UINT32_C(       206)),
      simde_x_mm_set_epu32(UINT32_C( 125265138), UINT32_C( 248660599), UINT32_C(         2), UINT32_C(   5425863)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_div_epu32(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_u32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_div_epu64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_x_mm_set_epu64x(UINT64_C(14823946846053138543), UINT64_C( 2773213006356142856)),
      simde_x_mm_set_epu64x(UINT64_C(   22806630538915743), UINT64_C(                1295)),
      simde_x_mm_set_epu64x(UINT64_C(                 649), UINT64_C(    2141477224985438)) },
    { simde_x_mm_set_epu64x(UINT64_C(16338394746286416599), UINT64_C( 4395568244008230294)),
      simde_x_mm_set_epu64x(UINT64_C(                1610), UINT64_C(         68247035008)),
      simde_x_mm_set_epu64x(UINT64_C(   10148071270985351), UINT64_C(            64406728)) },
    { simde_x_mm_set_epu64x(UINT64_C( 6431957656146818365), UINT64_C(14710883493083458909)),
      simde_x_mm_set_epu64x(UINT64_C(       2399266305377), UINT64_C(   16092627197291141)),
      simde_x_mm_set_epu64x(UINT64_C(             2680801), UINT64_C(                 914)) },
    { simde_x_mm_set_epu64x(UINT64_C( 7920700281052633117), UINT64_C(15482760419196872328)),
      simde_x_mm_set_epu64x(UINT64_C(         45928957131), UINT64_C(              837231)),
      simde_x_mm_set_epu64x(UINT64_C(           172455478), UINT64_C(      18492817895176)) },
    { simde_x_mm_set_epu64x(UINT64_C(  230158309193392347), UINT64_C(18390356791266391163)),
      simde_x_mm_set_epu64x(UINT64_C(                2253), UINT64_C(       1691141090999)),
      simde_x_mm_set_epu64x(UINT64_C(     102156373365908), UINT64_C(            10874525)) },
    { simde_x_mm_set_epu64x(UINT64_C(12307531484633875995), UINT64_C(16695234188854570094)),
      simde_x_mm_set_epu64x(UINT64_C(           131150029), UINT64_C(  516657134296053652)),
      simde_x_mm_set_epu64x(UINT64_C(         93843147260), UINT64_C(                  32)) },
    { simde_x_mm_set_epu64x(UINT64_C(11764896934406933200), UINT64_C(18439918542668248477)),
      simde_x_mm_set_epu64x(UINT64_C(        306481550847), UINT64_C(  776223621938168297)),
      simde_x_mm_set_epu64x(UINT64_C(            38386966), UINT64_C(                  23)) },
    { simde_x_mm_set_epu64x(UINT64_C(15338454595408931369), UINT64_C(14530768559531423502)),
      simde_x_mm_set_epu64x(UINT64_C(                3408), UINT64_C(                   2)),
      simde_x_mm_set_epu64x(UINT64_C(    4500720245131728), UINT64_C( 7265384279765711751)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_div_epu64(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_u64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_div_epi8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_mm256_set_epi8(INT8_C( -27), INT8_C(  46), INT8_C(-122), INT8_C(  87),
                           INT8_C(  34), INT8_C( -53), INT8_C(  64), INT8_C( -70),
                           INT8_C(  25), INT8_C( -17), INT8_C(  56), INT8_C(   3),
                           INT8_C( -75), INT8_C( -17), INT8_C( -12), INT8_C(  60),
                           INT8_C( 100), INT8_C(  -7), INT8_C(-102), INT8_C(  -6),
                           INT8_C( -10), INT8_C(-111), INT8_C( 106), INT8_C( -43),
                           INT8_C( -28), INT8_C( -46), INT8_C(  42), INT8_C( -58),
                           INT8_C(  85), INT8_C( -33), INT8_C(-106), INT8_C(-106)),
      simde_mm256_set_epi8(INT8_C(   1), INT8_C(   4), INT8_C( -31), INT8_C(   6),
                           INT8_C(  13), INT8_C(  15), INT8_C(  20), INT8_C(   3),
                           INT8_C( -77), INT8_C(  32), INT8_C(   5), INT8_C(  55),
                           INT8_C(   5), INT8_C(   1), INT8_C(  16), INT8_C(  49),
                           INT8_C(  43), INT8_C(  83), INT8_C(   5), INT8_C(  16),
                           INT8_C(  34), INT8_C(  20), INT8_C(   2), INT8_C(  13),
                           INT8_C(   8), INT8_C(   2), INT8_C(  90), INT8_C(   2),
                           INT8_C(  23), INT8_C(  12), INT8_C(   2), INT8_C(   5)),
      simde_mm256_set_epi8(INT8_C( -27), INT8_C(  11), INT8_C(   3), INT8_C(  14),
                           INT8_C(   2), INT8_C(  -3), INT8_C(   3), INT8_C( -23),
                           INT8_C(   0), INT8_C(   0), INT8_C(  11), INT8_C(   0),
                           INT8_C( -15), INT8_C( -17), INT8_C(   0), INT8_C(   1),
                           INT8_C(   2), INT8_C(   0), INT8_C( -20), INT8_C(   0),
                           INT8_C(   0), INT8_C(  -5), INT8_C(  53), INT8_C(  -3),
                           INT8_C(  -3), INT8_C( -23), INT8_C(   0), INT8_C( -29),
                           INT8_C(   3), INT8_C(  -2), INT8_C( -53), INT8_C( -21)) },
    { simde_mm256_set_epi8(INT8_C(  64), INT8_C(-114), INT8_C(  66), INT8_C( -73),
                           INT8_C( -80), INT8_C(  97), INT8_C( 103), INT8_C( -46),
                           INT8_C( -83), INT8_C( 104), INT8_C(  22), INT8_C( -39),
                           INT8_C( 114), INT8_C( -82), INT8_C(  83), INT8_C( 122),
                           INT8_C(   1), INT8_C(  51), INT8_C(  75), INT8_C(-100),
                           INT8_C(  17), INT8_C(  37), INT8_C(  53), INT8_C( -57),
                           INT8_C( 121), INT8_C( -35), INT8_C( 108), INT8_C( -68),
                           INT8_C(  25), INT8_C( -78), INT8_C( -54), INT8_C(-104)),
      simde_mm256_set_epi8(INT8_C(  91), INT8_C(  10), INT8_C( -96), INT8_C(  14),
                           INT8_C(  21), INT8_C(  23), INT8_C(   1), INT8_C(   8),
                           INT8_C(   9), INT8_C(   2), INT8_C(   8), INT8_C(  30),
                           INT8_C(   1), INT8_C( -75), INT8_C(  15), INT8_C(   1),
                           INT8_C(  27), INT8_C(   5), INT8_C( 104), INT8_C(  48),
                           INT8_C(  11), INT8_C(   4), INT8_C(  31), INT8_C(   3),
                           INT8_C(  20), INT8_C( 118), INT8_C(   1), INT8_C(  18),
                           INT8_C(   1), INT8_C(  22), INT8_C(  20), INT8_C(  33)),
      simde_mm256_set_epi8(INT8_C(   0), INT8_C( -11), INT8_C(   0), INT8_C(  -5),
                           INT8_C(  -3), INT8_C(   4), INT8_C( 103), INT8_C(  -5),
                           INT8_C(  -9), INT8_C(  52), INT8_C(   2), INT8_C(  -1),
                           INT8_C( 114), INT8_C(   1), INT8_C(   5), INT8_C( 122),
                           INT8_C(   0), INT8_C(  10), INT8_C(   0), INT8_C(  -2),
                           INT8_C(   1), INT8_C(   9), INT8_C(   1), INT8_C( -19),
                           INT8_C(   6), INT8_C(   0), INT8_C( 108), INT8_C(  -3),
                           INT8_C(  25), INT8_C(  -3), INT8_C(  -2), INT8_C(  -3)) },
    { simde_mm256_set_epi8(INT8_C( 123), INT8_C(  92), INT8_C( -58), INT8_C(  47),
                           INT8_C(  51), INT8_C(  47), INT8_C(  69), INT8_C(  12),
                           INT8_C(  68), INT8_C( -99), INT8_C(  76), INT8_C(  32),
                           INT8_C(  85), INT8_C( -81), INT8_C(  -3), INT8_C(  -4),
                           INT8_C( -35), INT8_C( -48), INT8_C(  17), INT8_C( -73),
                           INT8_C( 109), INT8_C(  88), INT8_C( -56), INT8_C( -99),
                           INT8_C(-114), INT8_C( 127), INT8_C(  26), INT8_C( -29),
                           INT8_C( -48), INT8_C( -28), INT8_C(  93), INT8_C( -85)),
      simde_mm256_set_epi8(INT8_C(  86), INT8_C(  12), INT8_C(  90), INT8_C(  46),
                           INT8_C(  10), INT8_C(  18), INT8_C(   1), INT8_C(  58),
                           INT8_C( -94), INT8_C(   4), INT8_C(   2), INT8_C(   1),
                           INT8_C(  20), INT8_C(  20), INT8_C(   1), INT8_C(  10),
                           INT8_C(   4), INT8_C(  13), INT8_C(   1), INT8_C(   1),
                           INT8_C(   1), INT8_C(   3), INT8_C(  16), INT8_C(   4),
                           INT8_C(   4), INT8_C(   2), INT8_C(   8), INT8_C( -96),
                           INT8_C(   1), INT8_C(   5), INT8_C( -98), INT8_C(  11)),
      simde_mm256_set_epi8(INT8_C(   1), INT8_C(   7), INT8_C(   0), INT8_C(   1),
                           INT8_C(   5), INT8_C(   2), INT8_C(  69), INT8_C(   0),
                           INT8_C(   0), INT8_C( -24), INT8_C(  38), INT8_C(  32),
                           INT8_C(   4), INT8_C(  -4), INT8_C(  -3), INT8_C(   0),
                           INT8_C(  -8), INT8_C(  -3), INT8_C(  17), INT8_C( -73),
                           INT8_C( 109), INT8_C(  29), INT8_C(  -3), INT8_C( -24),
                           INT8_C( -28), INT8_C(  63), INT8_C(   3), INT8_C(   0),
                           INT8_C( -48), INT8_C(  -5), INT8_C(   0), INT8_C(  -7)) },
    { simde_mm256_set_epi8(INT8_C( -83), INT8_C(   8), INT8_C(  39), INT8_C(  32),
                           INT8_C( -68), INT8_C(   0), INT8_C(  93), INT8_C(   7),
                           INT8_C( -26), INT8_C( -37), INT8_C(   3), INT8_C( -23),
                           INT8_C(  38), INT8_C( -61), INT8_C(  87), INT8_C(  32),
                           INT8_C(  65), INT8_C(  24), INT8_C( -17), INT8_C( -19),
                           INT8_C( 113), INT8_C( -25), INT8_C(  58), INT8_C(   4),
                           INT8_C(-127), INT8_C(  41), INT8_C( -74), INT8_C( 113),
                           INT8_C(  49), INT8_C( -39), INT8_C( -48), INT8_C( 114)),
      simde_mm256_set_epi8(INT8_C(-102), INT8_C(   1), INT8_C(  22), INT8_C(   1),
                           INT8_C(  15), INT8_C(   2), INT8_C(  19), INT8_C(  69),
                           INT8_C(   1), INT8_C(  49), INT8_C(  66), INT8_C(   2),
                           INT8_C(   1), INT8_C(   2), INT8_C(  10), INT8_C(   8),
                           INT8_C(   1), INT8_C(   1), INT8_C(   4), INT8_C(  66),
                           INT8_C(  11), INT8_C(  22), INT8_C(-126), INT8_C(  49),
                           INT8_C(   1), INT8_C(  38), INT8_C(   1), INT8_C(   3),
                           INT8_C(   7), INT8_C(   3), INT8_C(  21), INT8_C(  21)),
      simde_mm256_set_epi8(INT8_C(   0), INT8_C(   8), INT8_C(   1), INT8_C(  32),
                           INT8_C(  -4), INT8_C(   0), INT8_C(   4), INT8_C(   0),
                           INT8_C( -26), INT8_C(   0), INT8_C(   0), INT8_C( -11),
                           INT8_C(  38), INT8_C( -30), INT8_C(   8), INT8_C(   4),
                           INT8_C(  65), INT8_C(  24), INT8_C(  -4), INT8_C(   0),
                           INT8_C(  10), INT8_C(  -1), INT8_C(   0), INT8_C(   0),
                           INT8_C(-127), INT8_C(   1), INT8_C( -74), INT8_C(  37),
                           INT8_C(   7), INT8_C( -13), INT8_C(  -2), INT8_C(   5)) },
    { simde_mm256_set_epi8(INT8_C(  66), INT8_C( 127), INT8_C(  41), INT8_C(-124),
                           INT8_C( -90), INT8_C(  28), INT8_C(-118), INT8_C(  18),
                           INT8_C(  79), INT8_C(  17), INT8_C( 126), INT8_C( -43),
                           INT8_C( -78), INT8_C(  78), INT8_C(  76), INT8_C(  46),
                           INT8_C(  60), INT8_C(-126), INT8_C( -41), INT8_C( -77),
                           INT8_C( -62), INT8_C(-116), INT8_C(-115), INT8_C(  55),
                           INT8_C(  19), INT8_C( 104), INT8_C(-104), INT8_C( -29),
                           INT8_C(  54), INT8_C(-118), INT8_C( -40), INT8_C( -58)),
      simde_mm256_set_epi8(INT8_C(   3), INT8_C(  53), INT8_C(  28), INT8_C( -96),
                           INT8_C(   1), INT8_C(  91), INT8_C(   7), INT8_C(   1),
                           INT8_C(  29), INT8_C(  30), INT8_C(   1), INT8_C(  10),
                           INT8_C(   1), INT8_C(  36), INT8_C(   7), INT8_C(   1),
                           INT8_C(-101), INT8_C(   5), INT8_C(  13), INT8_C(   5),
                           INT8_C(  85), INT8_C(  11), INT8_C(  34), INT8_C(  48),
                           INT8_C(  17), INT8_C(  42), INT8_C(   3), INT8_C(  87),
                           INT8_C(   1), INT8_C(   2), INT8_C(  74), INT8_C(   8)),
      simde_mm256_set_epi8(INT8_C(  22), INT8_C(   2), INT8_C(   1), INT8_C(   1),
                           INT8_C( -90), INT8_C(   0), INT8_C( -16), INT8_C(  18),
                           INT8_C(   2), INT8_C(   0), INT8_C( 126), INT8_C(  -4),
                           INT8_C( -78), INT8_C(   2), INT8_C(  10), INT8_C(  46),
                           INT8_C(   0), INT8_C( -25), INT8_C(  -3), INT8_C( -15),
                           INT8_C(   0), INT8_C( -10), INT8_C(  -3), INT8_C(   1),
                           INT8_C(   1), INT8_C(   2), INT8_C( -34), INT8_C(   0),
                           INT8_C(  54), INT8_C( -59), INT8_C(   0), INT8_C(  -7)) },
    { simde_mm256_set_epi8(INT8_C(  79), INT8_C( -60), INT8_C( 106), INT8_C( -93),
                           INT8_C(-111), INT8_C( 118), INT8_C( -87), INT8_C( -78),
                           INT8_C( -28), INT8_C( 107), INT8_C( -12), INT8_C( -54),
                           INT8_C( 101), INT8_C( -62), INT8_C(   4), INT8_C( -51),
                           INT8_C( -90), INT8_C(-114), INT8_C(  14), INT8_C( 124),
                           INT8_C( -67), INT8_C(  47), INT8_C(  41), INT8_C(  37),
                           INT8_C( 126), INT8_C( -20), INT8_C( 119), INT8_C( 105),
                           INT8_C( -17), INT8_C(  95), INT8_C( -41), INT8_C(  19)),
      simde_mm256_set_epi8(INT8_C( -34), INT8_C(   4), INT8_C(  32), INT8_C(   1),
                           INT8_C(   4), INT8_C(  10), INT8_C(   7), INT8_C(   5),
                           INT8_C( 120), INT8_C(   1), INT8_C(   1), INT8_C(   1),
                           INT8_C(  26), INT8_C(   6), INT8_C(  44), INT8_C(   2),
                           INT8_C(  55), INT8_C(  14), INT8_C(   4), INT8_C(  41),
                           INT8_C(  41), INT8_C(   6), INT8_C(  10), INT8_C(   7),
                           INT8_C(   7), INT8_C(  21), INT8_C( 126), INT8_C(  59),
                           INT8_C(  13), INT8_C(   8), INT8_C(   2), INT8_C(   6)),
      simde_mm256_set_epi8(INT8_C(  -2), INT8_C( -15), INT8_C(   3), INT8_C( -93),
                           INT8_C( -27), INT8_C(  11), INT8_C( -12), INT8_C( -15),
                           INT8_C(   0), INT8_C( 107), INT8_C( -12), INT8_C( -54),
                           INT8_C(   3), INT8_C( -10), INT8_C(   0), INT8_C( -25),
                           INT8_C(  -1), INT8_C(  -8), INT8_C(   3), INT8_C(   3),
                           INT8_C(  -1), INT8_C(   7), INT8_C(   4), INT8_C(   5),
                           INT8_C(  18), INT8_C(   0), INT8_C(   0), INT8_C(   1),
                           INT8_C(  -1), INT8_C(  11), INT8_C( -20), INT8_C(   3)) },
    { simde_mm256_set_epi8(INT8_C( -48), INT8_C( -29), INT8_C(  23), INT8_C(  39),
                           INT8_C( 106), INT8_C( -37), INT8_C(   1), INT8_C(  62),
                           INT8_C( -21), INT8_C(  -4), INT8_C( -92), INT8_C( -12),
                           INT8_C(  78), INT8_C( -93), INT8_C(  36), INT8_C( -10),
                           INT8_C( -84), INT8_C( 102), INT8_C(   9), INT8_C(  70),
                           INT8_C( -16), INT8_C( -90), INT8_C(  82), INT8_C(-124),
                           INT8_C( -78), INT8_C(  58), INT8_C(  35), INT8_C( 108),
                           INT8_C(-105), INT8_C( -72), INT8_C( -16), INT8_C(-103)),
      simde_mm256_set_epi8(INT8_C(   2), INT8_C(   4), INT8_C(  28), INT8_C( 120),
                           INT8_C(   1), INT8_C(   5), INT8_C(   2), INT8_C(  61),
                           INT8_C(   1), INT8_C(  33), INT8_C( 110), INT8_C(   1),
                           INT8_C( 102), INT8_C(   3), INT8_C(   3), INT8_C(   1),
                           INT8_C(   1), INT8_C(  26), INT8_C(  11), INT8_C(   7),
                           INT8_C(  75), INT8_C(   3), INT8_C(   5), INT8_C(  19),
                           INT8_C(   3), INT8_C( -26), INT8_C(  56), INT8_C(   5),
                           INT8_C(   7), INT8_C(   6), INT8_C(   2), INT8_C(   5)),
      simde_mm256_set_epi8(INT8_C( -24), INT8_C(  -7), INT8_C(   0), INT8_C(   0),
                           INT8_C( 106), INT8_C(  -7), INT8_C(   0), INT8_C(   1),
                           INT8_C( -21), INT8_C(   0), INT8_C(   0), INT8_C( -12),
                           INT8_C(   0), INT8_C( -31), INT8_C(  12), INT8_C( -10),
                           INT8_C( -84), INT8_C(   3), INT8_C(   0), INT8_C(  10),
                           INT8_C(   0), INT8_C( -30), INT8_C(  16), INT8_C(  -6),
                           INT8_C( -26), INT8_C(  -2), INT8_C(   0), INT8_C(  21),
                           INT8_C( -15), INT8_C( -12), INT8_C(  -8), INT8_C( -20)) },
    { simde_mm256_set_epi8(INT8_C( 110), INT8_C(  56), INT8_C(-120), INT8_C( -32),
                           INT8_C( -22), INT8_C(  97), INT8_C( -56), INT8_C(  55),
                           INT8_C( -90), INT8_C(  33), INT8_C(  92), INT8_C(  89),
                           INT8_C(-107), INT8_C(  55), INT8_C( -50), INT8_C( -88),
                           INT8_C(  35), INT8_C(  21), INT8_C(  54), INT8_C(  26),
                           INT8_C(-122), INT8_C( 103), INT8_C(  76), INT8_C(  38),
                           INT8_C(-110), INT8_C(  11), INT8_C(  26), INT8_C( -11),
                           INT8_C(   0), INT8_C(   3), INT8_C(  30), INT8_C(  59)),
      simde_mm256_set_epi8(INT8_C( -31), INT8_C( -83), INT8_C( 101), INT8_C(  17),
                           INT8_C(   8), INT8_C(  15), INT8_C(   2), INT8_C(   7),
                           INT8_C(  37), INT8_C(  84), INT8_C( -52), INT8_C(  25),
                           INT8_C(  42), INT8_C( -27), INT8_C(   1), INT8_C(  10),
                           INT8_C(   7), INT8_C(  37), INT8_C(  54), INT8_C(  31),
                           INT8_C(  54), INT8_C(  62), INT8_C(  11), INT8_C(  54),
                           INT8_C(  43), INT8_C(   1), INT8_C(   4), INT8_C(   5),
                           INT8_C(  93), INT8_C( 124), INT8_C(   2), INT8_C(   3)),
      simde_mm256_set_epi8(INT8_C(  -3), INT8_C(   0), INT8_C(  -1), INT8_C(  -1),
                           INT8_C(  -2), INT8_C(   6), INT8_C( -28), INT8_C(   7),
                           INT8_C(  -2), INT8_C(   0), INT8_C(  -1), INT8_C(   3),
                           INT8_C(  -2), INT8_C(  -2), INT8_C( -50), INT8_C(  -8),
                           INT8_C(   5), INT8_C(   0), INT8_C(   1), INT8_C(   0),
                           INT8_C(  -2), INT8_C(   1), INT8_C(   6), INT8_C(   0),
                           INT8_C(  -2), INT8_C(  11), INT8_C(   6), INT8_C(  -2),
                           INT8_C(   0), INT8_C(   0), INT8_C(  15), INT8_C(  19)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_div_epi8(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_div_epi16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_mm256_set_epi16(INT16_C(-29867), INT16_C(  9314), INT16_C(  7980), INT16_C(  8102),
                            INT16_C(-24663), INT16_C(  4367), INT16_C(-15443), INT16_C( -5657),
                            INT16_C(-20080), INT16_C(-10092), INT16_C(-31734), INT16_C(  6262),
                            INT16_C(  3510), INT16_C(-31811), INT16_C( -4053), INT16_C( -6124)),
      simde_mm256_set_epi16(INT16_C(     1), INT16_C(  1438), INT16_C(    -9), INT16_C(   435),
                            INT16_C(   -11), INT16_C(     2), INT16_C(  -496), INT16_C( 10321),
                            INT16_C( -1000), INT16_C(   -27), INT16_C(    -4), INT16_C(   453),
                            INT16_C(    -2), INT16_C( 19741), INT16_C(  -615), INT16_C( -3265)),
      simde_mm256_set_epi16(INT16_C(-29867), INT16_C(     6), INT16_C(  -886), INT16_C(    18),
                            INT16_C(  2242), INT16_C(  2183), INT16_C(    31), INT16_C(     0),
                            INT16_C(    20), INT16_C(   373), INT16_C(  7933), INT16_C(    13),
                            INT16_C( -1755), INT16_C(    -1), INT16_C(     6), INT16_C(     1)) },
    { simde_mm256_set_epi16(INT16_C( -6800), INT16_C( 13259), INT16_C( -2233), INT16_C(  1354),
                            INT16_C( -8106), INT16_C(-17039), INT16_C(  9504), INT16_C( 22255),
                            INT16_C( 12402), INT16_C( -2677), INT16_C(  4463), INT16_C( 28303),
                            INT16_C(-12322), INT16_C(-19201), INT16_C( 30668), INT16_C( 15284)),
      simde_mm256_set_epi16(INT16_C( 16270), INT16_C(-26534), INT16_C(   -13), INT16_C(   -20),
                            INT16_C(   -12), INT16_C(  -182), INT16_C(   -13), INT16_C(    -2),
                            INT16_C(   399), INT16_C(  -245), INT16_C(    -1), INT16_C(    -1),
                            INT16_C(    -3), INT16_C(    59), INT16_C(    11), INT16_C( -9799)),
      simde_mm256_set_epi16(INT16_C(     0), INT16_C(     0), INT16_C(   171), INT16_C(   -67),
                            INT16_C(   675), INT16_C(    93), INT16_C(  -731), INT16_C(-11127),
                            INT16_C(    31), INT16_C(    10), INT16_C( -4463), INT16_C(-28303),
                            INT16_C(  4107), INT16_C(  -325), INT16_C(  2788), INT16_C(    -1)) },
    { simde_mm256_set_epi16(INT16_C( 23535), INT16_C( 10930), INT16_C( 30193), INT16_C( -8194),
                            INT16_C( -8688), INT16_C(  2183), INT16_C(-14596), INT16_C(-28144),
                            INT16_C(-10670), INT16_C(  1107), INT16_C( 31427), INT16_C( -7322),
                            INT16_C( 17038), INT16_C(-32679), INT16_C( 23368), INT16_C(-24524)),
      simde_mm256_set_epi16(INT16_C(    19), INT16_C(  -388), INT16_C(    -1), INT16_C( -2261),
                            INT16_C( -7651), INT16_C(  1639), INT16_C(   -50), INT16_C( -2059),
                            INT16_C(   -25), INT16_C(   -57), INT16_C(  -952), INT16_C(    17),
                            INT16_C( -4528), INT16_C(  -764), INT16_C(  -925), INT16_C(   -20)),
      simde_mm256_set_epi16(INT16_C(  1238), INT16_C(   -28), INT16_C(-30193), INT16_C(     3),
                            INT16_C(     1), INT16_C(     1), INT16_C(   291), INT16_C(    13),
                            INT16_C(   426), INT16_C(   -19), INT16_C(   -33), INT16_C(  -430),
                            INT16_C(    -3), INT16_C(    42), INT16_C(   -25), INT16_C(  1226)) },
    { simde_mm256_set_epi16(INT16_C( 22767), INT16_C( 28543), INT16_C(-30401), INT16_C( 25623),
                            INT16_C(  2206), INT16_C(-16640), INT16_C(-13607), INT16_C(-30899),
                            INT16_C( -2384), INT16_C( -1714), INT16_C( 12691), INT16_C(  9427),
                            INT16_C( 11864), INT16_C( 29526), INT16_C(  8259), INT16_C(  6808)),
      simde_mm256_set_epi16(INT16_C( 15244), INT16_C(     1), INT16_C(    -1), INT16_C(    -3),
                            INT16_C(   -18), INT16_C(   -10), INT16_C(-15299), INT16_C(  -824),
                            INT16_C(  2005), INT16_C(   471), INT16_C(  2069), INT16_C(   204),
                            INT16_C(    25), INT16_C(   -13), INT16_C(    -3), INT16_C(    11)),
      simde_mm256_set_epi16(INT16_C(     1), INT16_C( 28543), INT16_C( 30401), INT16_C( -8541),
                            INT16_C(  -122), INT16_C(  1664), INT16_C(     0), INT16_C(    37),
                            INT16_C(    -1), INT16_C(    -3), INT16_C(     6), INT16_C(    46),
                            INT16_C(   474), INT16_C( -2271), INT16_C( -2753), INT16_C(   618)) },
    { simde_mm256_set_epi16(INT16_C(-16585), INT16_C(-25277), INT16_C( -4139), INT16_C(-27065),
                            INT16_C(-28777), INT16_C( -9487), INT16_C(-18713), INT16_C(-30387),
                            INT16_C(-14811), INT16_C( 24102), INT16_C(-10162), INT16_C(  7921),
                            INT16_C( 29417), INT16_C( 15464), INT16_C( 24785), INT16_C( -1285)),
      simde_mm256_set_epi16(INT16_C(  -121), INT16_C(   328), INT16_C(    10), INT16_C(  -385),
                            INT16_C(    -1), INT16_C(     4), INT16_C(   388), INT16_C(    -1),
                            INT16_C(     1), INT16_C(  4863), INT16_C(  -499), INT16_C(     3),
                            INT16_C(  -226), INT16_C(-15244), INT16_C(     5), INT16_C(    -5)),
      simde_mm256_set_epi16(INT16_C(   137), INT16_C(   -77), INT16_C(  -413), INT16_C(    70),
                            INT16_C( 28777), INT16_C( -2371), INT16_C(   -48), INT16_C( 30387),
                            INT16_C(-14811), INT16_C(     4), INT16_C(    20), INT16_C(  2640),
                            INT16_C(  -130), INT16_C(    -1), INT16_C(  4957), INT16_C(   257)) },
    { simde_mm256_set_epi16(INT16_C( -8831), INT16_C(-12421), INT16_C( 28092), INT16_C(-15215),
                            INT16_C(  5495), INT16_C( 15560), INT16_C(  8747), INT16_C( 22186),
                            INT16_C(-22634), INT16_C(-23262), INT16_C(   360), INT16_C(-18340),
                            INT16_C(-15939), INT16_C(-18429), INT16_C(-10641), INT16_C(-25953)),
      simde_mm256_set_epi16(INT16_C(  6646), INT16_C(  -440), INT16_C(     5), INT16_C(     9),
                            INT16_C(  5230), INT16_C( 14027), INT16_C(  -115), INT16_C(    -1),
                            INT16_C(  -118), INT16_C(  -466), INT16_C(  -288), INT16_C(    -9),
                            INT16_C(   114), INT16_C( -2656), INT16_C( -2539), INT16_C(  1803)),
      simde_mm256_set_epi16(INT16_C(    -1), INT16_C(    28), INT16_C(  5618), INT16_C( -1690),
                            INT16_C(     1), INT16_C(     1), INT16_C(   -76), INT16_C(-22186),
                            INT16_C(   191), INT16_C(    49), INT16_C(    -1), INT16_C(  2037),
                            INT16_C(  -139), INT16_C(     6), INT16_C(     4), INT16_C(   -14)) },
    { simde_mm256_set_epi16(INT16_C(  2118), INT16_C( 26269), INT16_C( 31059), INT16_C( 17912),
                            INT16_C(-28141), INT16_C(  5202), INT16_C( 30957), INT16_C(-32121),
                            INT16_C( -2609), INT16_C(-12316), INT16_C(-10959), INT16_C( 17018),
                            INT16_C(  4376), INT16_C(  1963), INT16_C( 14912), INT16_C(  8031)),
      simde_mm256_set_epi16(INT16_C( -2197), INT16_C(    11), INT16_C(   -18), INT16_C( -3745),
                            INT16_C(    -1), INT16_C(    -3), INT16_C(     4), INT16_C(  3362),
                            INT16_C( -1965), INT16_C(     2), INT16_C(   574), INT16_C(  1347),
                            INT16_C(  -888), INT16_C(   -15), INT16_C(  1260), INT16_C(  -640)),
      simde_mm256_set_epi16(INT16_C(     0), INT16_C(  2388), INT16_C( -1725), INT16_C(    -4),
                            INT16_C( 28141), INT16_C( -1734), INT16_C(  7739), INT16_C(    -9),
                            INT16_C(     1), INT16_C( -6158), INT16_C(   -19), INT16_C(    12),
                            INT16_C(    -4), INT16_C(  -130), INT16_C(    11), INT16_C(   -12)) },
    { simde_mm256_set_epi16(INT16_C(-28159), INT16_C(  7162), INT16_C(-24830), INT16_C(  4589),
                            INT16_C(  7038), INT16_C(  3178), INT16_C(  4246), INT16_C( -8357),
                            INT16_C( -4695), INT16_C( -9928), INT16_C( -5517), INT16_C(-27023),
                            INT16_C( 18843), INT16_C(   726), INT16_C( 30135), INT16_C( -4871)),
      simde_mm256_set_epi16(INT16_C(   -48), INT16_C(   767), INT16_C(    10), INT16_C(    14),
                            INT16_C( -2039), INT16_C(    -2), INT16_C(   -53), INT16_C(    -1),
                            INT16_C( -1865), INT16_C( -5344), INT16_C(    63), INT16_C(  -505),
                            INT16_C(  2993), INT16_C(-14674), INT16_C(     3), INT16_C(    -2)),
      simde_mm256_set_epi16(INT16_C(   586), INT16_C(     9), INT16_C( -2483), INT16_C(   327),
                            INT16_C(    -3), INT16_C( -1589), INT16_C(   -80), INT16_C(  8357),
                            INT16_C(     2), INT16_C(     1), INT16_C(   -87), INT16_C(    53),
                            INT16_C(     6), INT16_C(     0), INT16_C( 10045), INT16_C(  2435)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_div_epi16(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_div_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_mm256_set_epi32(INT32_C( 1220357195), INT32_C( 1053623553), INT32_C( 1487300768), INT32_C(-1113593972),
                            INT32_C( -270466921), INT32_C( 1339961381), INT32_C(  586340423), INT32_C( 1641199948)),
      simde_mm256_set_epi32(INT32_C(  119685834), INT32_C(         18), INT32_C(   13175516), INT32_C(    2634495),
                            INT32_C(         17), INT32_C(      43789), INT32_C(        -89), INT32_C(         14)),
      simde_mm256_set_epi32(INT32_C(         10), INT32_C(   58534641), INT32_C(        112), INT32_C(       -422),
                            INT32_C(  -15909818), INT32_C(      30600), INT32_C(   -6588094), INT32_C(  117228567)) },
    { simde_mm256_set_epi32(INT32_C( 1446174898), INT32_C( 1812297946), INT32_C(-2020316623), INT32_C(  843765864),
                            INT32_C(-1892632155), INT32_C( -473868741), INT32_C( -150363910), INT32_C(-1673359813)),
      simde_mm256_set_epi32(INT32_C(    2569135), INT32_C(       8168), INT32_C(   -4111977), INT32_C(       -322),
                            INT32_C(  -34091386), INT32_C(       6306), INT32_C(     363174), INT32_C(     -37460)),
      simde_mm256_set_epi32(INT32_C(        562), INT32_C(     221877), INT32_C(        491), INT32_C(   -2620390),
                            INT32_C(         55), INT32_C(     -75145), INT32_C(       -414), INT32_C(      44670)) },
    { simde_mm256_set_epi32(INT32_C( 1015973964), INT32_C( -637033789), INT32_C(-1269659180), INT32_C(-1847076164),
                            INT32_C(  841308417), INT32_C(-1365136816), INT32_C( -621262370), INT32_C( -734285761)),
      simde_mm256_set_epi32(INT32_C(   -1597720), INT32_C(     192391), INT32_C(    2145556), INT32_C(      -4054),
                            INT32_C(         -1), INT32_C(      63753), INT32_C(   24015328), INT32_C(        267)),
      simde_mm256_set_epi32(INT32_C(       -635), INT32_C(      -3311), INT32_C(       -591), INT32_C(     455618),
                            INT32_C( -841308417), INT32_C(     -21412), INT32_C(        -25), INT32_C(   -2750133)) },
    { simde_mm256_set_epi32(INT32_C(   55709148), INT32_C( 1036348942), INT32_C( 1622954205), INT32_C( 1464937075),
                            INT32_C(  309602207), INT32_C(  765487752), INT32_C(-1883826060), INT32_C(  396580110)),
      simde_mm256_set_epi32(INT32_C(      81348), INT32_C(     130432), INT32_C(   -2896201), INT32_C(     130033),
                            INT32_C(       2659), INT32_C(      12656), INT32_C(        -49), INT32_C(      -3976)),
      simde_mm256_set_epi32(INT32_C(        684), INT32_C(       7945), INT32_C(       -560), INT32_C(      11265),
                            INT32_C(     116435), INT32_C(      60484), INT32_C(   38445429), INT32_C(     -99743)) },
    { simde_mm256_set_epi32(INT32_C( -679308904), INT32_C( 1402916027), INT32_C( -568259373), INT32_C( -151984025),
                            INT32_C(-1276596492), INT32_C(  897258790), INT32_C( 1125465930), INT32_C(-1843912592)),
      simde_mm256_set_epi32(INT32_C(        -32), INT32_C(      -3810), INT32_C(        -77), INT32_C(     -56604),
                            INT32_C(       2670), INT32_C(      -7949), INT32_C(       3200), INT32_C(      22045)),
      simde_mm256_set_epi32(INT32_C(   21228403), INT32_C(    -368219), INT32_C(    7379991), INT32_C(       2685),
                            INT32_C(    -478126), INT32_C(    -112876), INT32_C(     351708), INT32_C(     -83643)) },
    { simde_mm256_set_epi32(INT32_C(-2128829075), INT32_C( -944286219), INT32_C(-1801390937), INT32_C( 1597729863),
                            INT32_C( -919883082), INT32_C(  243529930), INT32_C(-1346833089), INT32_C( -703593878)),
      simde_mm256_set_epi32(INT32_C(    -702474), INT32_C(       -505), INT32_C(  -33538370), INT32_C(         98),
                            INT32_C(    -989384), INT32_C(   -3405840), INT32_C(    1441037), INT32_C(         13)),
      simde_mm256_set_epi32(INT32_C(       3030), INT32_C(    1869873), INT32_C(         53), INT32_C(   16303365),
                            INT32_C(        929), INT32_C(        -71), INT32_C(       -934), INT32_C(  -54122606)) },
    { simde_mm256_set_epi32(INT32_C( 2104898600), INT32_C( 1858378377), INT32_C(  427610695), INT32_C( 1702051599),
                            INT32_C( 1832473397), INT32_C(  333005662), INT32_C( 2145787203), INT32_C(-1223503753)),
      simde_mm256_set_epi32(INT32_C( -558822192), INT32_C(   -1119473), INT32_C(         71), INT32_C(         -1),
                            INT32_C(      83208), INT32_C(        -24), INT32_C(        490), INT32_C(    1423105)),
      simde_mm256_set_epi32(INT32_C(         -3), INT32_C(      -1660), INT32_C(    6022685), INT32_C(-1702051599),
                            INT32_C(      22022), INT32_C(  -13875235), INT32_C(    4379157), INT32_C(       -859)) },
    { simde_mm256_set_epi32(INT32_C( 1485879823), INT32_C( -139198096), INT32_C(  325243915), INT32_C( 1406493107),
                            INT32_C(  631640676), INT32_C( -221831503), INT32_C(-1100348538), INT32_C(-1615759789)),
      simde_mm256_set_epi32(INT32_C(         -5), INT32_C(    6019751), INT32_C(  240957918), INT32_C(     -11512),
                            INT32_C(        598), INT32_C(      -2086), INT32_C(       -398), INT32_C(   57524929)),
      simde_mm256_set_epi32(INT32_C( -297175964), INT32_C(        -23), INT32_C(          1), INT32_C(    -122176),
                            INT32_C(    1056255), INT32_C(     106343), INT32_C(    2764694), INT32_C(        -28)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_div_epi32(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_div_epi64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_mm256_set_epi64x(INT64_C(-3334573923423752375), INT64_C( 5523377417165557950),
                             INT64_C( 8907494989684855351), INT64_C(-7237415305059575746)),
      simde_mm256_set_epi64x(INT64_C(      -9171626596647), INT64_C(       -528646059918),
                             INT64_C(             -547414), INT64_C(                -408)),
      simde_mm256_set_epi64x(INT64_C(              363574), INT64_C(           -10448157),
                             INT64_C(     -16271953201205), INT64_C(   17738763002596999)) },
    { simde_mm256_set_epi64x(INT64_C( 1061533355853207499), INT64_C(-6945701440990101118),
                             INT64_C( 2574461366811200995), INT64_C( 5644549884645175906)),
      simde_mm256_set_epi64x(INT64_C(            -7767261), INT64_C(                  10),
                             INT64_C(           703320391), INT64_C(               12482)),
      simde_mm256_set_epi64x(INT64_C(       -136667656185), INT64_C( -694570144099010111),
                             INT64_C(          3660438968), INT64_C(     452215180631723)) },
    { simde_mm256_set_epi64x(INT64_C( 6574854431853233270), INT64_C(-4435882974713226150),
                             INT64_C(-7281891715377237835), INT64_C( 5757222003030846963)),
      simde_mm256_set_epi64x(INT64_C(   -6789037658203169), INT64_C(              -17570),
                             INT64_C(   13607885161437703), INT64_C(            -3435095)),
      simde_mm256_set_epi64x(INT64_C(                -968), INT64_C(     252469150524372),
                             INT64_C(                -535), INT64_C(      -1676000810175)) },
    { simde_mm256_set_epi64x(INT64_C( 8744553519166698091), INT64_C( 1287292031192317940),
                             INT64_C( 3174243940922689145), INT64_C( 1491394686146555130)),
      simde_mm256_set_epi64x(INT64_C( 4922490686897444762), INT64_C(         39224412374),
                             INT64_C(     408105256075342), INT64_C(       -123591096713)),
      simde_mm256_set_epi64x(INT64_C(                   1), INT64_C(            32818644),
                             INT64_C(                7778), INT64_C(           -12067169)) },
    { simde_mm256_set_epi64x(INT64_C( 7799483112595335323), INT64_C(-7884857912053188380),
                             INT64_C( 7107489308993436793), INT64_C( 8695475100908985079)),
      simde_mm256_set_epi64x(INT64_C(                  87), INT64_C(             9826793),
                             INT64_C(          -161255109), INT64_C(   -1858599442623445)),
      simde_mm256_set_epi64x(INT64_C(   89649231179256727), INT64_C(       -802383637474),
                             INT64_C(        -44076056585), INT64_C(               -4678)) },
    { simde_mm256_set_epi64x(INT64_C(-7825910496387937639), INT64_C( -900763466419687908),
                             INT64_C(-4456690812175475739), INT64_C(-5053240277275181299)),
      simde_mm256_set_epi64x(INT64_C(      -6606649764768), INT64_C(              -57398),
                             INT64_C( -568604113828926107), INT64_C(             4737239)),
      simde_mm256_set_epi64x(INT64_C(             1184550), INT64_C(      15693290121950),
                             INT64_C(                   7), INT64_C(      -1066705791553)) },
    { simde_mm256_set_epi64x(INT64_C(-3221953081539923764), INT64_C(-1956032791701614517),
                             INT64_C( 7374977017813000944), INT64_C( 1124803906659433418)),
      simde_mm256_set_epi64x(INT64_C( -339969907608416876), INT64_C(              -15370),
                             INT64_C(         -1321351535), INT64_C(                  -7)),
      simde_mm256_set_epi64x(INT64_C(                   9), INT64_C(     127263031340378),
                             INT64_C(         -5581389072), INT64_C( -160686272379919059)) },
    { simde_mm256_set_epi64x(INT64_C( 2535418176622027197), INT64_C(-1425521063377864898),
                             INT64_C( 5027060343823160394), INT64_C(-2416798548878703366)),
      simde_mm256_set_epi64x(INT64_C(                -250), INT64_C(                  51),
                             INT64_C(                3355), INT64_C(      22043462023905)),
      simde_mm256_set_epi64x(INT64_C(  -10141672706488108), INT64_C(  -27951393399565978),
                             INT64_C(    1498378641974116), INT64_C(             -109637)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_div_epi64(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_div_epu8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_x_mm256_set_epu8(UINT8_C(236), UINT8_C(194), UINT8_C(120), UINT8_C(  0),
                             UINT8_C(238), UINT8_C(197), UINT8_C(223), UINT8_C( 50),
                             UINT8_C(177), UINT8_C( 51), UINT8_C( 14), UINT8_C(208),
                             UINT8_C(118), UINT8_C(136), UINT8_C(234), UINT8_C(162),
                             UINT8_C( 34), UINT8_C(152), UINT8_C( 32), UINT8_C( 62),
                             UINT8_C( 35), UINT8_C(101), UINT8_C( 72), UINT8_C(  2),
                             UINT8_C(139), UINT8_C(150), UINT8_C(255), UINT8_C(  2),
                             UINT8_C( 37), UINT8_C(232), UINT8_C(  3), UINT8_C(210)),
      simde_x_mm256_set_epu8(UINT8_C(218), UINT8_C( 43), UINT8_C(  2), UINT8_C(  2),
                             UINT8_C( 29), UINT8_C( 90), UINT8_C( 30), UINT8_C( 31),
                             UINT8_C( 20), UINT8_C(  1), UINT8_C( 24), UINT8_C( 92),
                             UINT8_C(  3), UINT8_C(  1), UINT8_C( 33), UINT8_C(  6),
                             UINT8_C( 14), UINT8_C( 38), UINT8_C(  5), UINT8_C(  4),
                             UINT8_C( 13), UINT8_C(  2), UINT8_C( 11), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C( 25), UINT8_C(242), UINT8_C(  3),
                             UINT8_C( 12), UINT8_C( 59), UINT8_C( 75), UINT8_C(192)),
      simde_x_mm256_set_epu8(UINT8_C(  1), UINT8_C(  4), UINT8_C( 60), UINT8_C(  0),
                             UINT8_C(  8), UINT8_C(  2), UINT8_C(  7), UINT8_C(  1),
                             UINT8_C(  8), UINT8_C( 51), UINT8_C(  0), UINT8_C(  2),
                             UINT8_C( 39), UINT8_C(136), UINT8_C(  7), UINT8_C( 27),
                             UINT8_C(  2), UINT8_C(  4), UINT8_C(  6), UINT8_C( 15),
                             UINT8_C(  2), UINT8_C( 50), UINT8_C(  6), UINT8_C(  2),
                             UINT8_C(139), UINT8_C(  6), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  3), UINT8_C(  3), UINT8_C(  0), UINT8_C(  1)) },
    { simde_x_mm256_set_epu8(UINT8_C(223), UINT8_C(136), UINT8_C(181), UINT8_C(189),
                             UINT8_C(144), UINT8_C(162), UINT8_C( 60), UINT8_C(122),
                             UINT8_C(180), UINT8_C(157), UINT8_C(255), UINT8_C(  4),
                             UINT8_C(248), UINT8_C( 71), UINT8_C( 45), UINT8_C(231),
                             UINT8_C(108), UINT8_C(100), UINT8_C( 13), UINT8_C(181),
                             UINT8_C(158), UINT8_C(251), UINT8_C(141), UINT8_C( 49),
                             UINT8_C(175), UINT8_C( 90), UINT8_C(251), UINT8_C( 13),
                             UINT8_C(151), UINT8_C(233), UINT8_C(181), UINT8_C(181)),
      simde_x_mm256_set_epu8(UINT8_C(  2), UINT8_C(  7), UINT8_C(  2), UINT8_C(  7),
                             UINT8_C(  6), UINT8_C( 23), UINT8_C(  1), UINT8_C( 22),
                             UINT8_C(  9), UINT8_C( 21), UINT8_C(  6), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C( 27), UINT8_C(  1), UINT8_C(254),
                             UINT8_C( 30), UINT8_C( 92), UINT8_C(  8), UINT8_C( 13),
                             UINT8_C(  7), UINT8_C(  4), UINT8_C( 29), UINT8_C( 24),
                             UINT8_C(  1), UINT8_C( 15), UINT8_C( 31), UINT8_C(  1),
                             UINT8_C(190), UINT8_C(  1), UINT8_C( 20), UINT8_C(  8)),
      simde_x_mm256_set_epu8(UINT8_C(111), UINT8_C( 19), UINT8_C( 90), UINT8_C( 27),
                             UINT8_C( 24), UINT8_C(  7), UINT8_C( 60), UINT8_C(  5),
                             UINT8_C( 20), UINT8_C(  7), UINT8_C( 42), UINT8_C(  4),
                             UINT8_C(248), UINT8_C(  2), UINT8_C( 45), UINT8_C(  0),
                             UINT8_C(  3), UINT8_C(  1), UINT8_C(  1), UINT8_C( 13),
                             UINT8_C( 22), UINT8_C( 62), UINT8_C(  4), UINT8_C(  2),
                             UINT8_C(175), UINT8_C(  6), UINT8_C(  8), UINT8_C( 13),
                             UINT8_C(  0), UINT8_C(233), UINT8_C(  9), UINT8_C( 22)) },
    { simde_x_mm256_set_epu8(UINT8_C(162), UINT8_C(  7), UINT8_C(145), UINT8_C(154),
                             UINT8_C(168), UINT8_C(175), UINT8_C( 61), UINT8_C(  3),
                             UINT8_C( 93), UINT8_C(  6), UINT8_C(114), UINT8_C( 59),
                             UINT8_C( 17), UINT8_C(165), UINT8_C(240), UINT8_C(189),
                             UINT8_C(201), UINT8_C( 90), UINT8_C( 72), UINT8_C( 56),
                             UINT8_C( 98), UINT8_C(155), UINT8_C( 93), UINT8_C(190),
                             UINT8_C( 59), UINT8_C(174), UINT8_C(136), UINT8_C(  6),
                             UINT8_C(153), UINT8_C(172), UINT8_C(102), UINT8_C(120)),
      simde_x_mm256_set_epu8(UINT8_C(110), UINT8_C( 41), UINT8_C(  3), UINT8_C( 12),
                             UINT8_C(210), UINT8_C(  1), UINT8_C(  5), UINT8_C(  6),
                             UINT8_C( 47), UINT8_C( 58), UINT8_C( 48), UINT8_C( 20),
                             UINT8_C(109), UINT8_C(  3), UINT8_C( 34), UINT8_C(  3),
                             UINT8_C(  8), UINT8_C(  5), UINT8_C(  3), UINT8_C(  1),
                             UINT8_C( 20), UINT8_C( 14), UINT8_C(  1), UINT8_C(  6),
                             UINT8_C( 15), UINT8_C(  3), UINT8_C( 95), UINT8_C(  1),
                             UINT8_C(  4), UINT8_C(  1), UINT8_C(  7), UINT8_C(  1)),
      simde_x_mm256_set_epu8(UINT8_C(  1), UINT8_C(  0), UINT8_C( 48), UINT8_C( 12),
                             UINT8_C(  0), UINT8_C(175), UINT8_C( 12), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  2), UINT8_C(  2),
                             UINT8_C(  0), UINT8_C( 55), UINT8_C(  7), UINT8_C( 63),
                             UINT8_C( 25), UINT8_C( 18), UINT8_C( 24), UINT8_C( 56),
                             UINT8_C(  4), UINT8_C( 11), UINT8_C( 93), UINT8_C( 31),
                             UINT8_C(  3), UINT8_C( 58), UINT8_C(  1), UINT8_C(  6),
                             UINT8_C( 38), UINT8_C(172), UINT8_C( 14), UINT8_C(120)) },
    { simde_x_mm256_set_epu8(UINT8_C(  3), UINT8_C( 62), UINT8_C(201), UINT8_C( 91),
                             UINT8_C( 81), UINT8_C(108), UINT8_C(219), UINT8_C(124),
                             UINT8_C(107), UINT8_C(229), UINT8_C(194), UINT8_C(  6),
                             UINT8_C(247), UINT8_C(122), UINT8_C( 69), UINT8_C(216),
                             UINT8_C(192), UINT8_C(132), UINT8_C( 14), UINT8_C(210),
                             UINT8_C(242), UINT8_C(228), UINT8_C( 76), UINT8_C(247),
                             UINT8_C(164), UINT8_C(249), UINT8_C(124), UINT8_C(200),
                             UINT8_C(141), UINT8_C(206), UINT8_C(142), UINT8_C(235)),
      simde_x_mm256_set_epu8(UINT8_C(182), UINT8_C(  3), UINT8_C( 13), UINT8_C( 91),
                             UINT8_C( 12), UINT8_C( 10), UINT8_C(  1), UINT8_C(  3),
                             UINT8_C(  4), UINT8_C(  8), UINT8_C( 93), UINT8_C(  1),
                             UINT8_C(  2), UINT8_C( 38), UINT8_C(  3), UINT8_C(172),
                             UINT8_C( 38), UINT8_C( 15), UINT8_C( 55), UINT8_C( 26),
                             UINT8_C(  4), UINT8_C( 16), UINT8_C( 28), UINT8_C( 54),
                             UINT8_C( 21), UINT8_C( 30), UINT8_C(  3), UINT8_C( 39),
                             UINT8_C( 14), UINT8_C(171), UINT8_C(  2), UINT8_C(  4)),
      simde_x_mm256_set_epu8(UINT8_C(  0), UINT8_C( 20), UINT8_C( 15), UINT8_C(  1),
                             UINT8_C(  6), UINT8_C( 10), UINT8_C(219), UINT8_C( 41),
                             UINT8_C( 26), UINT8_C( 28), UINT8_C(  2), UINT8_C(  6),
                             UINT8_C(123), UINT8_C(  3), UINT8_C( 23), UINT8_C(  1),
                             UINT8_C(  5), UINT8_C(  8), UINT8_C(  0), UINT8_C(  8),
                             UINT8_C( 60), UINT8_C( 14), UINT8_C(  2), UINT8_C(  4),
                             UINT8_C(  7), UINT8_C(  8), UINT8_C( 41), UINT8_C(  5),
                             UINT8_C( 10), UINT8_C(  1), UINT8_C( 71), UINT8_C( 58)) },
    { simde_x_mm256_set_epu8(UINT8_C(168), UINT8_C(  0), UINT8_C(141), UINT8_C(215),
                             UINT8_C( 23), UINT8_C(105), UINT8_C(153), UINT8_C(228),
                             UINT8_C(144), UINT8_C(204), UINT8_C(214), UINT8_C(202),
                             UINT8_C(227), UINT8_C(255), UINT8_C( 22), UINT8_C(115),
                             UINT8_C(131), UINT8_C(142), UINT8_C( 73), UINT8_C(133),
                             UINT8_C( 47), UINT8_C(243), UINT8_C(254), UINT8_C(234),
                             UINT8_C( 91), UINT8_C(217), UINT8_C(119), UINT8_C(247),
                             UINT8_C(245), UINT8_C( 31), UINT8_C( 46), UINT8_C( 19)),
      simde_x_mm256_set_epu8(UINT8_C(  1), UINT8_C(248), UINT8_C(  3), UINT8_C(  9),
                             UINT8_C(  3), UINT8_C( 87), UINT8_C(117), UINT8_C( 58),
                             UINT8_C( 18), UINT8_C(  9), UINT8_C(  7), UINT8_C( 77),
                             UINT8_C( 11), UINT8_C( 11), UINT8_C( 28), UINT8_C( 49),
                             UINT8_C( 64), UINT8_C( 46), UINT8_C(  5), UINT8_C(  1),
                             UINT8_C(115), UINT8_C(  2), UINT8_C(  1), UINT8_C(  1),
                             UINT8_C( 86), UINT8_C( 10), UINT8_C(  3), UINT8_C( 12),
                             UINT8_C( 49), UINT8_C(155), UINT8_C(  1), UINT8_C(  3)),
      simde_x_mm256_set_epu8(UINT8_C(168), UINT8_C(  0), UINT8_C( 47), UINT8_C( 23),
                             UINT8_C(  7), UINT8_C(  1), UINT8_C(  1), UINT8_C(  3),
                             UINT8_C(  8), UINT8_C( 22), UINT8_C( 30), UINT8_C(  2),
                             UINT8_C( 20), UINT8_C( 23), UINT8_C(  0), UINT8_C(  2),
                             UINT8_C(  2), UINT8_C(  3), UINT8_C( 14), UINT8_C(133),
                             UINT8_C(  0), UINT8_C(121), UINT8_C(254), UINT8_C(234),
                             UINT8_C(  1), UINT8_C( 21), UINT8_C( 39), UINT8_C( 20),
                             UINT8_C(  5), UINT8_C(  0), UINT8_C( 46), UINT8_C(  6)) },
    { simde_x_mm256_set_epu8(UINT8_C(163), UINT8_C(117), UINT8_C( 13), UINT8_C( 71),
                             UINT8_C(173), UINT8_C(230), UINT8_C(206), UINT8_C(  2),
                             UINT8_C( 15), UINT8_C(252), UINT8_C( 14), UINT8_C(197),
                             UINT8_C(249), UINT8_C(198), UINT8_C( 30), UINT8_C(180),
                             UINT8_C(128), UINT8_C( 78), UINT8_C(184), UINT8_C(254),
                             UINT8_C(184), UINT8_C(231), UINT8_C(238), UINT8_C( 30),
                             UINT8_C(194), UINT8_C( 37), UINT8_C(226), UINT8_C( 86),
                             UINT8_C(140), UINT8_C( 24), UINT8_C(144), UINT8_C( 16)),
      simde_x_mm256_set_epu8(UINT8_C( 48), UINT8_C(  1), UINT8_C(  7), UINT8_C(  6),
                             UINT8_C(119), UINT8_C( 41), UINT8_C(111), UINT8_C(  8),
                             UINT8_C(135), UINT8_C(  2), UINT8_C( 23), UINT8_C(  1),
                             UINT8_C( 88), UINT8_C( 15), UINT8_C( 65), UINT8_C( 79),
                             UINT8_C( 29), UINT8_C(  5), UINT8_C(  5), UINT8_C(  6),
                             UINT8_C( 44), UINT8_C( 21), UINT8_C(  2), UINT8_C(  3),
                             UINT8_C( 15), UINT8_C(  1), UINT8_C(  3), UINT8_C(  3),
                             UINT8_C(  1), UINT8_C( 10), UINT8_C(  1), UINT8_C( 55)),
      simde_x_mm256_set_epu8(UINT8_C(  3), UINT8_C(117), UINT8_C(  1), UINT8_C( 11),
                             UINT8_C(  1), UINT8_C(  5), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(126), UINT8_C(  0), UINT8_C(197),
                             UINT8_C(  2), UINT8_C( 13), UINT8_C(  0), UINT8_C(  2),
                             UINT8_C(  4), UINT8_C( 15), UINT8_C( 36), UINT8_C( 42),
                             UINT8_C(  4), UINT8_C( 11), UINT8_C(119), UINT8_C( 10),
                             UINT8_C( 12), UINT8_C( 37), UINT8_C( 75), UINT8_C( 28),
                             UINT8_C(140), UINT8_C(  2), UINT8_C(144), UINT8_C(  0)) },
    { simde_x_mm256_set_epu8(UINT8_C(239), UINT8_C(204), UINT8_C( 51), UINT8_C(246),
                             UINT8_C( 77), UINT8_C(149), UINT8_C( 40), UINT8_C( 86),
                             UINT8_C( 29), UINT8_C(  8), UINT8_C(140), UINT8_C(202),
                             UINT8_C(138), UINT8_C(208), UINT8_C(142), UINT8_C( 95),
                             UINT8_C(247), UINT8_C(102), UINT8_C( 63), UINT8_C(232),
                             UINT8_C(115), UINT8_C(187), UINT8_C(122), UINT8_C(179),
                             UINT8_C( 81), UINT8_C(192), UINT8_C( 47), UINT8_C( 34),
                             UINT8_C( 24), UINT8_C(133), UINT8_C( 98), UINT8_C(208)),
      simde_x_mm256_set_epu8(UINT8_C( 11), UINT8_C(  8), UINT8_C(  2), UINT8_C( 10),
                             UINT8_C(  3), UINT8_C(  7), UINT8_C( 38), UINT8_C( 21),
                             UINT8_C(247), UINT8_C( 14), UINT8_C(  4), UINT8_C(  3),
                             UINT8_C( 85), UINT8_C( 59), UINT8_C( 41), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(250), UINT8_C(  1), UINT8_C(  2),
                             UINT8_C(  6), UINT8_C(  8), UINT8_C(  6), UINT8_C( 40),
                             UINT8_C(136), UINT8_C( 10), UINT8_C( 29), UINT8_C(  7),
                             UINT8_C( 36), UINT8_C(  8), UINT8_C(  1), UINT8_C(  7)),
      simde_x_mm256_set_epu8(UINT8_C( 21), UINT8_C( 25), UINT8_C( 25), UINT8_C( 24),
                             UINT8_C( 25), UINT8_C( 21), UINT8_C(  1), UINT8_C(  4),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C( 35), UINT8_C( 67),
                             UINT8_C(  1), UINT8_C(  3), UINT8_C(  3), UINT8_C( 95),
                             UINT8_C(247), UINT8_C(  0), UINT8_C( 63), UINT8_C(116),
                             UINT8_C( 19), UINT8_C( 23), UINT8_C( 20), UINT8_C(  4),
                             UINT8_C(  0), UINT8_C( 19), UINT8_C(  1), UINT8_C(  4),
                             UINT8_C(  0), UINT8_C( 16), UINT8_C( 98), UINT8_C( 29)) },
    { simde_x_mm256_set_epu8(UINT8_C(179), UINT8_C(197), UINT8_C(124), UINT8_C(228),
                             UINT8_C(210), UINT8_C(205), UINT8_C(251), UINT8_C( 37),
                             UINT8_C( 37), UINT8_C( 57), UINT8_C( 27), UINT8_C( 38),
                             UINT8_C( 13), UINT8_C(212), UINT8_C(201), UINT8_C(125),
                             UINT8_C( 84), UINT8_C(229), UINT8_C( 76), UINT8_C(128),
                             UINT8_C(139), UINT8_C(203), UINT8_C(238), UINT8_C(218),
                             UINT8_C( 40), UINT8_C( 95), UINT8_C(243), UINT8_C(110),
                             UINT8_C( 74), UINT8_C(  0), UINT8_C(215), UINT8_C( 43)),
      simde_x_mm256_set_epu8(UINT8_C(  2), UINT8_C(  2), UINT8_C(  4), UINT8_C(  5),
                             UINT8_C(  7), UINT8_C(  2), UINT8_C(195), UINT8_C(  2),
                             UINT8_C( 30), UINT8_C(  1), UINT8_C(  9), UINT8_C( 24),
                             UINT8_C(  6), UINT8_C(  7), UINT8_C( 28), UINT8_C( 58),
                             UINT8_C(  3), UINT8_C( 77), UINT8_C( 90), UINT8_C( 51),
                             UINT8_C( 13), UINT8_C( 12), UINT8_C(  7), UINT8_C( 91),
                             UINT8_C(243), UINT8_C( 40), UINT8_C(  1), UINT8_C( 45),
                             UINT8_C( 77), UINT8_C( 45), UINT8_C( 60), UINT8_C(  3)),
      simde_x_mm256_set_epu8(UINT8_C( 89), UINT8_C( 98), UINT8_C( 31), UINT8_C( 45),
                             UINT8_C( 30), UINT8_C(102), UINT8_C(  1), UINT8_C( 18),
                             UINT8_C(  1), UINT8_C( 57), UINT8_C(  3), UINT8_C(  1),
                             UINT8_C(  2), UINT8_C( 30), UINT8_C(  7), UINT8_C(  2),
                             UINT8_C( 28), UINT8_C(  2), UINT8_C(  0), UINT8_C(  2),
                             UINT8_C( 10), UINT8_C( 16), UINT8_C( 34), UINT8_C(  2),
                             UINT8_C(  0), UINT8_C(  2), UINT8_C(243), UINT8_C(  2),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  3), UINT8_C( 14)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_div_epu8(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_u8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_div_epu16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_x_mm256_set_epu16(UINT16_C( 50042), UINT16_C( 33648), UINT16_C(  7535), UINT16_C( 12279),
                              UINT16_C( 36071), UINT16_C( 18107), UINT16_C( 48674), UINT16_C( 48206),
                              UINT16_C(  9011), UINT16_C( 45275), UINT16_C(  7845), UINT16_C( 54048),
                              UINT16_C( 27322), UINT16_C( 31657), UINT16_C( 43497), UINT16_C( 33598)),
      simde_x_mm256_set_epu16(UINT16_C( 12011), UINT16_C(   249), UINT16_C(     5), UINT16_C(     2),
                              UINT16_C(  1870), UINT16_C(  2904), UINT16_C(  1530), UINT16_C( 42479),
                              UINT16_C( 63442), UINT16_C(  1039), UINT16_C(    54), UINT16_C(     1),
                              UINT16_C(    98), UINT16_C(  7948), UINT16_C(  2053), UINT16_C(    29)),
      simde_x_mm256_set_epu16(UINT16_C(     4), UINT16_C(   135), UINT16_C(  1507), UINT16_C(  6139),
                              UINT16_C(    19), UINT16_C(     6), UINT16_C(    31), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(    43), UINT16_C(   145), UINT16_C( 54048),
                              UINT16_C(   278), UINT16_C(     3), UINT16_C(    21), UINT16_C(  1158)) },
    { simde_x_mm256_set_epu16(UINT16_C( 31411), UINT16_C( 55001), UINT16_C( 38051), UINT16_C( 20389),
                              UINT16_C( 61351), UINT16_C( 22045), UINT16_C( 61939), UINT16_C( 10168),
                              UINT16_C( 65482), UINT16_C( 32951), UINT16_C( 59114), UINT16_C(  9472),
                              UINT16_C( 21787), UINT16_C(  1387), UINT16_C( 60519), UINT16_C( 39038)),
      simde_x_mm256_set_epu16(UINT16_C( 11771), UINT16_C(     1), UINT16_C(   490), UINT16_C( 32408),
                              UINT16_C(  2225), UINT16_C(   134), UINT16_C( 13968), UINT16_C(     1),
                              UINT16_C(   387), UINT16_C( 14591), UINT16_C(    24), UINT16_C(    46),
                              UINT16_C(  8450), UINT16_C(  1053), UINT16_C(   908), UINT16_C(  5686)),
      simde_x_mm256_set_epu16(UINT16_C(     2), UINT16_C( 55001), UINT16_C(    77), UINT16_C(     0),
                              UINT16_C(    27), UINT16_C(   164), UINT16_C(     4), UINT16_C( 10168),
                              UINT16_C(   169), UINT16_C(     2), UINT16_C(  2463), UINT16_C(   205),
                              UINT16_C(     2), UINT16_C(     1), UINT16_C(    66), UINT16_C(     6)) },
    { simde_x_mm256_set_epu16(UINT16_C( 22899), UINT16_C(   630), UINT16_C( 34558), UINT16_C(  7884),
                              UINT16_C( 39724), UINT16_C( 33230), UINT16_C( 54475), UINT16_C( 22805),
                              UINT16_C( 61755), UINT16_C( 34661), UINT16_C( 28373), UINT16_C( 58279),
                              UINT16_C( 22187), UINT16_C( 56981), UINT16_C( 43877), UINT16_C(  3469)),
      simde_x_mm256_set_epu16(UINT16_C( 12306), UINT16_C(   182), UINT16_C( 29239), UINT16_C(  4194),
                              UINT16_C(   818), UINT16_C(    16), UINT16_C(     5), UINT16_C(    38),
                              UINT16_C( 42688), UINT16_C(     8), UINT16_C(     1), UINT16_C(    96),
                              UINT16_C(     3), UINT16_C(     1), UINT16_C(   508), UINT16_C(     1)),
      simde_x_mm256_set_epu16(UINT16_C(     1), UINT16_C(     3), UINT16_C(     1), UINT16_C(     1),
                              UINT16_C(    48), UINT16_C(  2076), UINT16_C( 10895), UINT16_C(   600),
                              UINT16_C(     1), UINT16_C(  4332), UINT16_C( 28373), UINT16_C(   607),
                              UINT16_C(  7395), UINT16_C( 56981), UINT16_C(    86), UINT16_C(  3469)) },
    { simde_x_mm256_set_epu16(UINT16_C( 29363), UINT16_C( 50584), UINT16_C( 56168), UINT16_C( 44370),
                              UINT16_C( 62910), UINT16_C( 23255), UINT16_C( 39479), UINT16_C( 21044),
                              UINT16_C(  7491), UINT16_C( 25737), UINT16_C(  6938), UINT16_C( 40142),
                              UINT16_C( 22210), UINT16_C( 63545), UINT16_C( 33358), UINT16_C(  9014)),
      simde_x_mm256_set_epu16(UINT16_C(    61), UINT16_C(   274), UINT16_C(   365), UINT16_C( 58937),
                              UINT16_C(     2), UINT16_C(   172), UINT16_C(   432), UINT16_C(     2),
                              UINT16_C(   957), UINT16_C(   351), UINT16_C(    18), UINT16_C( 12717),
                              UINT16_C(     4), UINT16_C(   417), UINT16_C(     1), UINT16_C( 10550)),
      simde_x_mm256_set_epu16(UINT16_C(   481), UINT16_C(   184), UINT16_C(   153), UINT16_C(     0),
                              UINT16_C( 31455), UINT16_C(   135), UINT16_C(    91), UINT16_C( 10522),
                              UINT16_C(     7), UINT16_C(    73), UINT16_C(   385), UINT16_C(     3),
                              UINT16_C(  5552), UINT16_C(   152), UINT16_C( 33358), UINT16_C(     0)) },
    { simde_x_mm256_set_epu16(UINT16_C( 22208), UINT16_C( 58940), UINT16_C( 24739), UINT16_C( 29405),
                              UINT16_C(  9863), UINT16_C( 41917), UINT16_C( 30045), UINT16_C( 40634),
                              UINT16_C( 50211), UINT16_C(  4668), UINT16_C( 42314), UINT16_C( 29370),
                              UINT16_C( 57744), UINT16_C( 37787), UINT16_C( 17171), UINT16_C( 34222)),
      simde_x_mm256_set_epu16(UINT16_C(  4256), UINT16_C( 23971), UINT16_C(   171), UINT16_C(    12),
                              UINT16_C(  8070), UINT16_C(  2906), UINT16_C(    22), UINT16_C(   107),
                              UINT16_C(     3), UINT16_C(     1), UINT16_C( 28355), UINT16_C(  2210),
                              UINT16_C(     1), UINT16_C(  1161), UINT16_C(   613), UINT16_C( 51426)),
      simde_x_mm256_set_epu16(UINT16_C(     5), UINT16_C(     2), UINT16_C(   144), UINT16_C(  2450),
                              UINT16_C(     1), UINT16_C(    14), UINT16_C(  1365), UINT16_C(   379),
                              UINT16_C( 16737), UINT16_C(  4668), UINT16_C(     1), UINT16_C(    13),
                              UINT16_C( 57744), UINT16_C(    32), UINT16_C(    28), UINT16_C(     0)) },
    { simde_x_mm256_set_epu16(UINT16_C(  9143), UINT16_C( 55963), UINT16_C( 46820), UINT16_C( 55354),
                              UINT16_C( 21540), UINT16_C( 21596), UINT16_C( 49435), UINT16_C( 42142),
                              UINT16_C( 28170), UINT16_C(  3714), UINT16_C( 39462), UINT16_C( 28043),
                              UINT16_C( 45359), UINT16_C( 22609), UINT16_C( 55149), UINT16_C( 21886)),
      simde_x_mm256_set_epu16(UINT16_C(  3121), UINT16_C(   103), UINT16_C(     1), UINT16_C(   283),
                              UINT16_C(   201), UINT16_C(    53), UINT16_C( 25996), UINT16_C(  3169),
                              UINT16_C(     1), UINT16_C(     2), UINT16_C(    38), UINT16_C(    24),
                              UINT16_C(    55), UINT16_C( 25444), UINT16_C(  5182), UINT16_C(     9)),
      simde_x_mm256_set_epu16(UINT16_C(     2), UINT16_C(   543), UINT16_C( 46820), UINT16_C(   195),
                              UINT16_C(   107), UINT16_C(   407), UINT16_C(     1), UINT16_C(    13),
                              UINT16_C( 28170), UINT16_C(  1857), UINT16_C(  1038), UINT16_C(  1168),
                              UINT16_C(   824), UINT16_C(     0), UINT16_C(    10), UINT16_C(  2431)) },
    { simde_x_mm256_set_epu16(UINT16_C( 51894), UINT16_C(  1840), UINT16_C( 33552), UINT16_C( 50070),
                              UINT16_C( 16848), UINT16_C( 13340), UINT16_C( 25356), UINT16_C( 34016),
                              UINT16_C( 61275), UINT16_C( 22886), UINT16_C( 28292), UINT16_C( 37845),
                              UINT16_C(  1481), UINT16_C(   559), UINT16_C( 12899), UINT16_C( 38851)),
      simde_x_mm256_set_epu16(UINT16_C( 16266), UINT16_C(   376), UINT16_C( 62048), UINT16_C(     8),
                              UINT16_C(    53), UINT16_C(  1573), UINT16_C(     8), UINT16_C(   212),
                              UINT16_C( 15505), UINT16_C(     1), UINT16_C(    10), UINT16_C(  2744),
                              UINT16_C(     2), UINT16_C(     5), UINT16_C(  4478), UINT16_C( 12656)),
      simde_x_mm256_set_epu16(UINT16_C(     3), UINT16_C(     4), UINT16_C(     0), UINT16_C(  6258),
                              UINT16_C(   317), UINT16_C(     8), UINT16_C(  3169), UINT16_C(   160),
                              UINT16_C(     3), UINT16_C( 22886), UINT16_C(  2829), UINT16_C(    13),
                              UINT16_C(   740), UINT16_C(   111), UINT16_C(     2), UINT16_C(     3)) },
    { simde_x_mm256_set_epu16(UINT16_C( 40946), UINT16_C( 11832), UINT16_C( 52869), UINT16_C( 41324),
                              UINT16_C( 41064), UINT16_C( 57085), UINT16_C( 14204), UINT16_C( 23869),
                              UINT16_C( 30467), UINT16_C( 20149), UINT16_C( 58844), UINT16_C( 49602),
                              UINT16_C( 36092), UINT16_C( 39146), UINT16_C( 62840), UINT16_C( 19573)),
      simde_x_mm256_set_epu16(UINT16_C(  7725), UINT16_C(  5897), UINT16_C(    81), UINT16_C(   199),
                              UINT16_C( 33008), UINT16_C( 55443), UINT16_C(   925), UINT16_C(  4043),
                              UINT16_C(   362), UINT16_C(   156), UINT16_C(  2592), UINT16_C(    29),
                              UINT16_C(   213), UINT16_C(    14), UINT16_C(    39), UINT16_C(   178)),
      simde_x_mm256_set_epu16(UINT16_C(     5), UINT16_C(     2), UINT16_C(   652), UINT16_C(   207),
                              UINT16_C(     1), UINT16_C(     1), UINT16_C(    15), UINT16_C(     5),
                              UINT16_C(    84), UINT16_C(   129), UINT16_C(    22), UINT16_C(  1710),
                              UINT16_C(   169), UINT16_C(  2796), UINT16_C(  1611), UINT16_C(   109)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_div_epu16(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_u16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_div_epu32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_x_mm256_set_epu32(UINT32_C( 621216267), UINT32_C(2973447507), UINT32_C(1814279233), UINT32_C(3673557536),
                              UINT32_C(4015780858), UINT32_C(1070914538), UINT32_C(2707640519), UINT32_C(3041291274)),
      simde_x_mm256_set_epu32(UINT32_C(    122731), UINT32_C(  51630147), UINT32_C(    152670), UINT32_C(   7731229),
                              UINT32_C(    711400), UINT32_C(   1744981), UINT32_C( 164943127), UINT32_C(    169494)),
      simde_x_mm256_set_epu32(UINT32_C(      5061), UINT32_C(        57), UINT32_C(     11883), UINT32_C(       475),
                              UINT32_C(      5644), UINT32_C(       613), UINT32_C(        16), UINT32_C(     17943)) },
    { simde_x_mm256_set_epu32(UINT32_C(1084014678), UINT32_C(1666523830), UINT32_C(3454667769), UINT32_C(4029614313),
                              UINT32_C(3425016021), UINT32_C(2449839571), UINT32_C(1601532569), UINT32_C(1519388398)),
      simde_x_mm256_set_epu32(UINT32_C(    130157), UINT32_C(   5585515), UINT32_C(  62691231), UINT32_C(     37123),
                              UINT32_C(   2515600), UINT32_C( 106484982), UINT32_C(4168501606), UINT32_C(   2781814)),
      simde_x_mm256_set_epu32(UINT32_C(      8328), UINT32_C(       298), UINT32_C(        55), UINT32_C(    108547),
                              UINT32_C(      1361), UINT32_C(        23), UINT32_C(         0), UINT32_C(       546)) },
    { simde_x_mm256_set_epu32(UINT32_C(2187853776), UINT32_C( 131263503), UINT32_C(  20338031), UINT32_C(3062800456),
                              UINT32_C(1802896354), UINT32_C(  22231847), UINT32_C(3438214155), UINT32_C(1776513196)),
      simde_x_mm256_set_epu32(UINT32_C(  28353115), UINT32_C(  92496104), UINT32_C(  15335526), UINT32_C(  99105532),
                              UINT32_C(   5905009), UINT32_C(     27824), UINT32_C(     28986), UINT32_C(  12459911)),
      simde_x_mm256_set_epu32(UINT32_C(        77), UINT32_C(         1), UINT32_C(         1), UINT32_C(        30),
                              UINT32_C(       305), UINT32_C(       799), UINT32_C(    118616), UINT32_C(       142)) },
    { simde_x_mm256_set_epu32(UINT32_C( 524596333), UINT32_C(3965897825), UINT32_C(1593754725), UINT32_C( 694203496),
                              UINT32_C(1917650066), UINT32_C(2692610113), UINT32_C(1620259645), UINT32_C( 607116294)),
      simde_x_mm256_set_epu32(UINT32_C(  29757558), UINT32_C(     80117), UINT32_C( 412054571), UINT32_C(    878110),
                              UINT32_C(4124070325), UINT32_C(   8250706), UINT32_C(   7930575), UINT32_C(     51813)),
      simde_x_mm256_set_epu32(UINT32_C(        17), UINT32_C(     49501), UINT32_C(         3), UINT32_C(       790),
                              UINT32_C(         0), UINT32_C(       326), UINT32_C(       204), UINT32_C(     11717)) },
    { simde_x_mm256_set_epu32(UINT32_C( 625862951), UINT32_C( 793130310), UINT32_C(2489185635), UINT32_C(2468815203),
                              UINT32_C(3079066921), UINT32_C( 802958712), UINT32_C(1537818066), UINT32_C(1678295724)),
      simde_x_mm256_set_epu32(UINT32_C(   8259237), UINT32_C(    229091), UINT32_C(   7899398), UINT32_C(  41009690),
                              UINT32_C(  26030333), UINT32_C(    228627), UINT32_C(1200021710), UINT32_C(    186204)),
      simde_x_mm256_set_epu32(UINT32_C(        75), UINT32_C(      3462), UINT32_C(       315), UINT32_C(        60),
                              UINT32_C(       118), UINT32_C(      3512), UINT32_C(         1), UINT32_C(      9013)) },
    { simde_x_mm256_set_epu32(UINT32_C(3334078645), UINT32_C(2226952893), UINT32_C(1901933944), UINT32_C(3456551705),
                              UINT32_C(3394846076), UINT32_C(2592342753), UINT32_C(1822000161), UINT32_C(3060682219)),
      simde_x_mm256_set_epu32(UINT32_C(     55529), UINT32_C(     95077), UINT32_C(  61849330), UINT32_C(     77269),
                              UINT32_C(    181901), UINT32_C(     66287), UINT32_C(     46407), UINT32_C(      1962)),
      simde_x_mm256_set_epu32(UINT32_C(     60042), UINT32_C(     23422), UINT32_C(        30), UINT32_C(     44734),
                              UINT32_C(     18663), UINT32_C(     39107), UINT32_C(     39261), UINT32_C(   1559980)) },
    { simde_x_mm256_set_epu32(UINT32_C(2418478797), UINT32_C(3856569345), UINT32_C(2562700829), UINT32_C(2670510577),
                              UINT32_C(3958231909), UINT32_C(3386864730), UINT32_C(2249491002), UINT32_C( 367242130)),
      simde_x_mm256_set_epu32(UINT32_C( 106591767), UINT32_C( 591565864), UINT32_C(    241208), UINT32_C(    384474),
                              UINT32_C(  63569588), UINT32_C(1007016971), UINT32_C( 701090048), UINT32_C(   4482965)),
      simde_x_mm256_set_epu32(UINT32_C(        22), UINT32_C(         6), UINT32_C(     10624), UINT32_C(      6945),
                              UINT32_C(        62), UINT32_C(         3), UINT32_C(         3), UINT32_C(        81)) },
    { simde_x_mm256_set_epu32(UINT32_C(3497551851), UINT32_C(3538232808), UINT32_C(3581222707), UINT32_C(2092274030),
                              UINT32_C(1202922035), UINT32_C(3381143079), UINT32_C(1645890362), UINT32_C(2497764821)),
      simde_x_mm256_set_epu32(UINT32_C(   7255461), UINT32_C(    387871), UINT32_C( 216379987), UINT32_C(   1108325),
                              UINT32_C(   9779926), UINT32_C( 265173482), UINT32_C(    305369), UINT32_C(1628979148)),
      simde_x_mm256_set_epu32(UINT32_C(       482), UINT32_C(      9122), UINT32_C(        16), UINT32_C(      1887),
                              UINT32_C(       122), UINT32_C(        12), UINT32_C(      5389), UINT32_C(         1)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_div_epu32(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_u32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_div_epu64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_x_mm256_set_epu64x(UINT64_C(10385902570114433083), UINT64_C(14228451038995253976),
                               UINT64_C( 3524803476344021799), UINT64_C( 9008088981795720991)),
      simde_x_mm256_set_epu64x(UINT64_C(      11435629647830), UINT64_C(        134705148152),
                               UINT64_C(                1685), UINT64_C(         72468903699)),
      simde_x_mm256_set_epu64x(UINT64_C(18446744073708846728), UINT64_C(18446744073678236607),
                               UINT64_C(    2091871499313959), UINT64_C(           124302818)) },
    { simde_x_mm256_set_epu64x(UINT64_C( 2776707612149100363), UINT64_C(15446686956822865619),
                               UINT64_C( 8116027459326381863), UINT64_C(10577862568627142107)),
      simde_x_mm256_set_epu64x(UINT64_C(              160900), UINT64_C(                 876),
                               UINT64_C(             6656645), UINT64_C(                 198)),
      simde_x_mm256_set_epu64x(UINT64_C(      17257349982281), UINT64_C(18443319350973379601),
                               UINT64_C(       1219236936824), UINT64_C(18407002247926307124)) },
    { simde_x_mm256_set_epu64x(UINT64_C(17966513918331168112), UINT64_C(15404442576328540960),
                               UINT64_C( 1544001744444053712), UINT64_C(12311626015854130554)),
      simde_x_mm256_set_epu64x(UINT64_C(         73453582701), UINT64_C(       2241703492778),
                               UINT64_C(                 149), UINT64_C(    1898802076338580)),
      simde_x_mm256_set_epu64x(UINT64_C(18446744073703013744), UINT64_C(18446744073708194478),
                               UINT64_C(   10362427815060763), UINT64_C(18446744073709548385)) },
    { simde_x_mm256_set_epu64x(UINT64_C( 4996618049503500636), UINT64_C( 3587306346705364576),
                               UINT64_C( 1416661578746677042), UINT64_C(18012200189266188151)),
      simde_x_mm256_set_epu64x(UINT64_C(       9141117518131), UINT64_C(  259684114065326460),
                               UINT64_C(          3735868918), UINT64_C(      13028085907926)),
      simde_x_mm256_set_epu64x(UINT64_C(              546609), UINT64_C(                  13),
                               UINT64_C(           379205376), UINT64_C(18446744073709518262)) },
    { simde_x_mm256_set_epu64x(UINT64_C(17900245410321819662), UINT64_C(   86463307544105486),
                               UINT64_C( 7004808110937624000), UINT64_C( 5352056724630121100)),
      simde_x_mm256_set_epu64x(UINT64_C(           574976069), UINT64_C(   26168849408611714),
                               UINT64_C(           479458176), UINT64_C(         85883846687)),
      simde_x_mm256_set_epu64x(UINT64_C(18446744072759079601), UINT64_C(                   3),
                               UINT64_C(         14609841820), UINT64_C(            62317384)) },
    { simde_x_mm256_set_epu64x(UINT64_C(18191047755947595201), UINT64_C(11274709867061747164),
                               UINT64_C( 4957427800472277352), UINT64_C( 2636046644056480855)),
      simde_x_mm256_set_epu64x(UINT64_C(           455513034), UINT64_C( 4176708352330988763),
                               UINT64_C(              255407), UINT64_C(   77468887445572755)),
      simde_x_mm256_set_epu64x(UINT64_C(18446744073148214621), UINT64_C(18446744073709551615),
                               UINT64_C(      19409913590748), UINT64_C(                  34)) },
    { simde_x_mm256_set_epu64x(UINT64_C(17236629464649076584), UINT64_C( 6716520602983844465),
                               UINT64_C(12794135593178656259), UINT64_C( 3865374743078695737)),
      simde_x_mm256_set_epu64x(UINT64_C(      13893724010244), UINT64_C(                   1),
                               UINT64_C(           142890905), UINT64_C(        135073488234)),
      simde_x_mm256_set_epu64x(UINT64_C(18446744073709464519), UINT64_C( 6716520602983844465),
                               UINT64_C(18446744034150641408), UINT64_C(            28616827)) },
    { simde_x_mm256_set_epu64x(UINT64_C( 3248934010021333275), UINT64_C( 8464322280604302303),
                               UINT64_C(10783963704762759650), UINT64_C(14288989654597257942)),
      simde_x_mm256_set_epu64x(UINT64_C(      37187973814779), UINT64_C(           988730192),
                               UINT64_C(       9409064941619), UINT64_C(           554649997)),
      simde_x_mm256_set_epu64x(UINT64_C(               87365), UINT64_C(          8560800862),
                               UINT64_C(18446744073708737212), UINT64_C(18446744066213374853)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_div_epi64(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_div_epi8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi8(INT8_C( 114), INT8_C(  89), INT8_C(   1), INT8_C( 122),
                           INT8_C(  12), INT8_C( 107), INT8_C(  92), INT8_C(-102),
                           INT8_C( -63), INT8_C( 120), INT8_C( 107), INT8_C( -43),
                           INT8_C(-119), INT8_C( -10), INT8_C(  98), INT8_C( -26),
                           INT8_C( 122), INT8_C(   1), INT8_C( -83), INT8_C(  43),
                           INT8_C(  82), INT8_C( -59), INT8_C( -43), INT8_C( -10),
                           INT8_C(  77), INT8_C( -22), INT8_C( -72), INT8_C( -94),
                           INT8_C(  75), INT8_C( -23), INT8_C( -92), INT8_C( -69),
                           INT8_C( 108), INT8_C(  26), INT8_C(  71), INT8_C( -21),
                           INT8_C(  15), INT8_C( 107), INT8_C(-112), INT8_C( -22),
                           INT8_C( -24), INT8_C(  35), INT8_C(  87), INT8_C(  75),
                           INT8_C(  27), INT8_C( -73), INT8_C(   9), INT8_C( -72),
                           INT8_C(  35), INT8_C(  -9), INT8_C( -68), INT8_C(  73),
                           INT8_C( -61), INT8_C( 118), INT8_C(  78), INT8_C( -20),
                           INT8_C( -42), INT8_C( -19), INT8_C(-125), INT8_C(  51),
                           INT8_C( -14), INT8_C(  17), INT8_C( -24), INT8_C( -72)),
      simde_mm512_set_epi8(INT8_C(  14), INT8_C(-123), INT8_C(  73), INT8_C(  -6),
                           INT8_C( -78), INT8_C( -38), INT8_C( -82), INT8_C( -80),
                           INT8_C(  31), INT8_C(  -9), INT8_C(  35), INT8_C(-110),
                           INT8_C(  -7), INT8_C(  74), INT8_C( -30), INT8_C( 100),
                           INT8_C(  10), INT8_C(  23), INT8_C( -11), INT8_C(  90),
                           INT8_C(  71), INT8_C(-126), INT8_C( -11), INT8_C(  -5),
                           INT8_C(  26), INT8_C(  58), INT8_C(-123), INT8_C( 125),
                           INT8_C(-104), INT8_C(  39), INT8_C(  75), INT8_C(  69),
                           INT8_C(   5), INT8_C(-119), INT8_C(  20), INT8_C(   6),
                           INT8_C( -18), INT8_C( -87), INT8_C(  95), INT8_C(  24),
                           INT8_C(  15), INT8_C( -48), INT8_C( -40), INT8_C(  79),
                           INT8_C(-107), INT8_C( -73), INT8_C(-108), INT8_C( -43),
                           INT8_C(  53), INT8_C( -95), INT8_C(  75), INT8_C(-123),
                           INT8_C(  61), INT8_C(  28), INT8_C(  20), INT8_C(  -5),
                           INT8_C(-127), INT8_C( -90), INT8_C(  94), INT8_C( -61),
                           INT8_C(  91), INT8_C( -70), INT8_C(-111), INT8_C(  30)),
      simde_mm512_set_epi8(INT8_C(   8), INT8_C(   0), INT8_C(   0), INT8_C( -20),
                           INT8_C(   0), INT8_C(  -2), INT8_C(  -1), INT8_C(   1),
                           INT8_C(  -2), INT8_C( -13), INT8_C(   3), INT8_C(   0),
                           INT8_C(  17), INT8_C(   0), INT8_C(  -3), INT8_C(   0),
                           INT8_C(  12), INT8_C(   0), INT8_C(   7), INT8_C(   0),
                           INT8_C(   1), INT8_C(   0), INT8_C(   3), INT8_C(   2),
                           INT8_C(   2), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(  -1), INT8_C(  -1),
                           INT8_C(  21), INT8_C(   0), INT8_C(   3), INT8_C(  -3),
                           INT8_C(   0), INT8_C(  -1), INT8_C(  -1), INT8_C(   0),
                           INT8_C(  -1), INT8_C(   0), INT8_C(  -2), INT8_C(   0),
                           INT8_C(   0), INT8_C(   1), INT8_C(   0), INT8_C(   1),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(  -1), INT8_C(   4), INT8_C(   3), INT8_C(   4),
                           INT8_C(   0), INT8_C(   0), INT8_C(  -1), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(  -2)) },
    { simde_mm512_set_epi8(INT8_C(  12), INT8_C( -52), INT8_C(  -7), INT8_C(  17),
                           INT8_C(-122), INT8_C(  53), INT8_C( -15), INT8_C(-121),
                           INT8_C( -47), INT8_C(-109), INT8_C( -20), INT8_C(  -5),
                           INT8_C( -34), INT8_C(   6), INT8_C(   3), INT8_C( -49),
                           INT8_C(  63), INT8_C(  48), INT8_C( -18), INT8_C( 117),
                           INT8_C( -63), INT8_C(  63), INT8_C(  77), INT8_C( -90),
                           INT8_C( -12), INT8_C(  83), INT8_C(  69), INT8_C( 113),
                           INT8_C(  28), INT8_C( 104), INT8_C( -69), INT8_C( -69),
                           INT8_C(-128), INT8_C(  96), INT8_C(  18), INT8_C(   9),
                           INT8_C(  99), INT8_C(-100), INT8_C( -63), INT8_C(  74),
                           INT8_C( -69), INT8_C(  22), INT8_C( 126), INT8_C(  62),
                           INT8_C(  46), INT8_C(  88), INT8_C(  24), INT8_C(  21),
                           INT8_C( 121), INT8_C(  64), INT8_C(  24), INT8_C(-125),
                           INT8_C(-125), INT8_C( -56), INT8_C( -13), INT8_C(  51),
                           INT8_C(  53), INT8_C( -41), INT8_C( -85), INT8_C(-121),
                           INT8_C( -44), INT8_C( -43), INT8_C( -24), INT8_C( 102)),
      simde_mm512_set_epi8(INT8_C( 109), INT8_C(-119), INT8_C(  12), INT8_C(  72),
                           INT8_C( -36), INT8_C(-115), INT8_C(  98), INT8_C(-110),
                           INT8_C(  58), INT8_C(  -6), INT8_C( -54), INT8_C(  39),
                           INT8_C( -42), INT8_C(  -8), INT8_C( -77), INT8_C( -22),
                           INT8_C( -49), INT8_C(   4), INT8_C( 119), INT8_C(  82),
                           INT8_C( 112), INT8_C(   3), INT8_C(  74), INT8_C(  94),
                           INT8_C( -27), INT8_C(  90), INT8_C(  17), INT8_C(  13),
                           INT8_C(   5), INT8_C(  89), INT8_C(-121), INT8_C(  56),
                           INT8_C(  46), INT8_C( -66), INT8_C( 124), INT8_C( -23),
                           INT8_C(  38), INT8_C(  53), INT8_C(  18), INT8_C( -68),
                           INT8_C(  -6), INT8_C( -62), INT8_C(  -9), INT8_C(  11),
                           INT8_C(  -6), INT8_C(  56), INT8_C( -81), INT8_C(  41),
                           INT8_C( 112), INT8_C(  58), INT8_C( -21), INT8_C( 108),
                           INT8_C(  17), INT8_C(  40), INT8_C(   4), INT8_C(  80),
                           INT8_C(  75), INT8_C(  35), INT8_C(  80), INT8_C( -85),
                           INT8_C(  88), INT8_C( -11), INT8_C(  23), INT8_C(  51)),
      simde_mm512_set_epi8(INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   3), INT8_C(   0), INT8_C(   0), INT8_C(   1),
                           INT8_C(   0), INT8_C(  18), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   2),
                           INT8_C(  -1), INT8_C(  12), INT8_C(   0), INT8_C(   1),
                           INT8_C(   0), INT8_C(  21), INT8_C(   1), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(   4), INT8_C(   8),
                           INT8_C(   5), INT8_C(   1), INT8_C(   0), INT8_C(  -1),
                           INT8_C(  -2), INT8_C(  -1), INT8_C(   0), INT8_C(   0),
                           INT8_C(   2), INT8_C(  -1), INT8_C(  -3), INT8_C(  -1),
                           INT8_C(  11), INT8_C(   0), INT8_C( -14), INT8_C(   5),
                           INT8_C(  -7), INT8_C(   1), INT8_C(   0), INT8_C(   0),
                           INT8_C(   1), INT8_C(   1), INT8_C(  -1), INT8_C(  -1),
                           INT8_C(  -7), INT8_C(  -1), INT8_C(  -3), INT8_C(   0),
                           INT8_C(   0), INT8_C(  -1), INT8_C(  -1), INT8_C(   1),
                           INT8_C(   0), INT8_C(   3), INT8_C(  -1), INT8_C(   2)) },
    { simde_mm512_set_epi8(INT8_C(-111), INT8_C(  -3), INT8_C( 110), INT8_C( -96),
                           INT8_C( 117), INT8_C( -29), INT8_C(-127), INT8_C( 101),
                           INT8_C(-120), INT8_C(  11), INT8_C(  87), INT8_C(  17),
                           INT8_C(-108), INT8_C(  87), INT8_C(   4), INT8_C( -21),
                           INT8_C(  98), INT8_C(   2), INT8_C( -60), INT8_C( -28),
                           INT8_C(  66), INT8_C(-109), INT8_C(   8), INT8_C( -58),
                           INT8_C(  13), INT8_C( -66), INT8_C( -49), INT8_C(  93),
                           INT8_C(-119), INT8_C(  58), INT8_C(  30), INT8_C(  10),
                           INT8_C( -11), INT8_C(  78), INT8_C(  76), INT8_C( 108),
                           INT8_C( -34), INT8_C( -94), INT8_C( -77), INT8_C(-122),
                           INT8_C(  37), INT8_C( -32), INT8_C( -97), INT8_C( 121),
                           INT8_C( -95), INT8_C( -80), INT8_C( -87), INT8_C( -89),
                           INT8_C(  -4), INT8_C( 115), INT8_C( -42), INT8_C( -55),
                           INT8_C(  95), INT8_C( -63), INT8_C(  31), INT8_C( -74),
                           INT8_C( -45), INT8_C( 119), INT8_C(  57), INT8_C( -52),
                           INT8_C( -69), INT8_C(-123), INT8_C( 106), INT8_C( 119)),
      simde_mm512_set_epi8(INT8_C( -74), INT8_C( -32), INT8_C(  89), INT8_C(  50),
                           INT8_C(-105), INT8_C(  85), INT8_C( -71), INT8_C( 105),
                           INT8_C( -37), INT8_C( -78), INT8_C(-107), INT8_C( -67),
                           INT8_C(   9), INT8_C(   2), INT8_C(  83), INT8_C(  67),
                           INT8_C(  25), INT8_C(-103), INT8_C( -90), INT8_C(  30),
                           INT8_C(  69), INT8_C(-127), INT8_C( 114), INT8_C( -99),
                           INT8_C( -97), INT8_C( -52), INT8_C( 120), INT8_C(  78),
                           INT8_C(  97), INT8_C( 124), INT8_C(  31), INT8_C(  72),
                           INT8_C(  -6), INT8_C(  19), INT8_C(  -4), INT8_C( -65),
                           INT8_C( 107), INT8_C( -15), INT8_C(-116), INT8_C( -13),
                           INT8_C( 106), INT8_C( -71), INT8_C( -14), INT8_C( -87),
                           INT8_C(-122), INT8_C( -59), INT8_C( -65), INT8_C( -58),
                           INT8_C( -26), INT8_C(  55), INT8_C(  28), INT8_C( -31),
                           INT8_C( -20), INT8_C( -40), INT8_C( -47), INT8_C(  58),
                           INT8_C(  -3), INT8_C(  67), INT8_C( -47), INT8_C(  93),
                           INT8_C( -77), INT8_C(  21), INT8_C(  49), INT8_C( -54)),
      simde_mm512_set_epi8(INT8_C(   1), INT8_C(   0), INT8_C(   1), INT8_C(  -1),
                           INT8_C(  -1), INT8_C(   0), INT8_C(   1), INT8_C(   0),
                           INT8_C(   3), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C( -12), INT8_C(  43), INT8_C(   0), INT8_C(   0),
                           INT8_C(   3), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   1), INT8_C(   0), INT8_C(   1),
                           INT8_C(  -1), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   1), INT8_C(   4), INT8_C( -19), INT8_C(  -1),
                           INT8_C(   0), INT8_C(   6), INT8_C(   0), INT8_C(   9),
                           INT8_C(   0), INT8_C(   0), INT8_C(   6), INT8_C(  -1),
                           INT8_C(   0), INT8_C(   1), INT8_C(   1), INT8_C(   1),
                           INT8_C(   0), INT8_C(   2), INT8_C(  -1), INT8_C(   1),
                           INT8_C(  -4), INT8_C(   1), INT8_C(   0), INT8_C(  -1),
                           INT8_C(  15), INT8_C(   1), INT8_C(  -1), INT8_C(   0),
                           INT8_C(   0), INT8_C(  -5), INT8_C(   2), INT8_C(  -2)) },
    { simde_mm512_set_epi8(INT8_C( -91), INT8_C( 110), INT8_C( 126), INT8_C(  44),
                           INT8_C(  21), INT8_C( -84), INT8_C( 100), INT8_C( -15),
                           INT8_C( -61), INT8_C( -53), INT8_C(  75), INT8_C( -30),
                           INT8_C( -56), INT8_C( -86), INT8_C(  52), INT8_C( 108),
                           INT8_C(  96), INT8_C(   6), INT8_C(-100), INT8_C(-109),
                           INT8_C(  -7), INT8_C( -22), INT8_C( 109), INT8_C( 124),
                           INT8_C(  85), INT8_C(  53), INT8_C( -45), INT8_C( 122),
                           INT8_C(   7), INT8_C( -21), INT8_C(-123), INT8_C(   4),
                           INT8_C(   3), INT8_C(  94), INT8_C(-127), INT8_C(  73),
                           INT8_C(  65), INT8_C( -69), INT8_C( -91), INT8_C(-115),
                           INT8_C( 117), INT8_C(-104), INT8_C(  66), INT8_C(  79),
                           INT8_C( -63), INT8_C(-115), INT8_C( -77), INT8_C( -89),
                           INT8_C(-113), INT8_C(  34), INT8_C( 100), INT8_C(  96),
                           INT8_C(-101), INT8_C( -34), INT8_C(  64), INT8_C( -59),
                           INT8_C( -53), INT8_C(  87), INT8_C(  48), INT8_C(  95),
                           INT8_C( -53), INT8_C(  61), INT8_C(  63), INT8_C( 106)),
      simde_mm512_set_epi8(INT8_C(  -1), INT8_C(  95), INT8_C(  91), INT8_C( 117),
                           INT8_C(  15), INT8_C( -50), INT8_C( -39), INT8_C(  74),
                           INT8_C(  36), INT8_C( 100), INT8_C( -62), INT8_C(-111),
                           INT8_C(   9), INT8_C(  41), INT8_C(  36), INT8_C( -21),
                           INT8_C(  71), INT8_C( -85), INT8_C( 120), INT8_C( -33),
                           INT8_C( 125), INT8_C(  38), INT8_C(-127), INT8_C(  39),
                           INT8_C(  28), INT8_C(-118), INT8_C(  31), INT8_C(  92),
                           INT8_C(  22), INT8_C(  48), INT8_C( 122), INT8_C(  -6),
                           INT8_C( 107), INT8_C(-101), INT8_C(  14), INT8_C( -17),
                           INT8_C(  26), INT8_C(  -4), INT8_C( -71), INT8_C(  13),
                           INT8_C( -39), INT8_C( -26), INT8_C( -37), INT8_C( 110),
                           INT8_C(  36), INT8_C(  78), INT8_C( -24), INT8_C( -52),
                           INT8_C(-117), INT8_C( -27), INT8_C( 113), INT8_C(-111),
                           INT8_C( -59), INT8_C(  38), INT8_C( -10), INT8_C( -53),
                           INT8_C( 110), INT8_C(  62), INT8_C(  -4), INT8_C(  19),
                           INT8_C( -15), INT8_C(  42), INT8_C( 122), INT8_C( 105)),
      simde_mm512_set_epi8(INT8_C(  91), INT8_C(   1), INT8_C(   1), INT8_C(   0),
                           INT8_C(   1), INT8_C(   1), INT8_C(  -2), INT8_C(   0),
                           INT8_C(  -1), INT8_C(   0), INT8_C(  -1), INT8_C(   0),
                           INT8_C(  -6), INT8_C(  -2), INT8_C(   1), INT8_C(  -5),
                           INT8_C(   1), INT8_C(   0), INT8_C(   0), INT8_C(   3),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   3),
                           INT8_C(   3), INT8_C(   0), INT8_C(  -1), INT8_C(   1),
                           INT8_C(   0), INT8_C(   0), INT8_C(  -1), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(  -9), INT8_C(  -4),
                           INT8_C(   2), INT8_C(  17), INT8_C(   1), INT8_C(  -8),
                           INT8_C(  -3), INT8_C(   4), INT8_C(  -1), INT8_C(   0),
                           INT8_C(  -1), INT8_C(  -1), INT8_C(   3), INT8_C(   1),
                           INT8_C(   0), INT8_C(  -1), INT8_C(   0), INT8_C(   0),
                           INT8_C(   1), INT8_C(   0), INT8_C(  -6), INT8_C(   1),
                           INT8_C(   0), INT8_C(   1), INT8_C( -12), INT8_C(   5),
                           INT8_C(   3), INT8_C(   1), INT8_C(   0), INT8_C(   1)) },
    { simde_mm512_set_epi8(INT8_C( -55), INT8_C( -14), INT8_C(   9), INT8_C(-109),
                           INT8_C(  77), INT8_C( -36), INT8_C(  82), INT8_C( -60),
                           INT8_C( -11), INT8_C(  52), INT8_C(  95), INT8_C( 118),
                           INT8_C( 124), INT8_C( 103), INT8_C( 108), INT8_C(   5),
                           INT8_C(  -7), INT8_C(  55), INT8_C(   1), INT8_C( -90),
                           INT8_C(  89), INT8_C( 106), INT8_C( -80), INT8_C(-113),
                           INT8_C( -97), INT8_C( 113), INT8_C( 100), INT8_C(   9),
                           INT8_C( 122), INT8_C( -51), INT8_C(-121), INT8_C(  78),
                           INT8_C(-100), INT8_C(  26), INT8_C( -23), INT8_C( -89),
                           INT8_C(  20), INT8_C(  19), INT8_C( -91), INT8_C( -38),
                           INT8_C( -59), INT8_C(  10), INT8_C(-121), INT8_C( -30),
                           INT8_C(  79), INT8_C(  49), INT8_C( 104), INT8_C(  55),
                           INT8_C(   2), INT8_C(  -2), INT8_C( -24), INT8_C( -48),
                           INT8_C( -25), INT8_C( -39), INT8_C(  89), INT8_C(  19),
                           INT8_C( -33), INT8_C( 101), INT8_C(  31), INT8_C( -59),
                           INT8_C(-123), INT8_C(  38), INT8_C( 124), INT8_C( 108)),
      simde_mm512_set_epi8(INT8_C( -47), INT8_C( -85), INT8_C(  13), INT8_C( -86),
                           INT8_C(  92), INT8_C(  23), INT8_C(  69), INT8_C( -53),
                           INT8_C(  11), INT8_C( -74), INT8_C(  93), INT8_C(  45),
                           INT8_C( 123), INT8_C( -37), INT8_C(   6), INT8_C( -51),
                           INT8_C(  52), INT8_C( -77), INT8_C( -79), INT8_C( -50),
                           INT8_C( -32), INT8_C(   4), INT8_C( -47), INT8_C( -53),
                           INT8_C( -18), INT8_C( -18), INT8_C( 115), INT8_C( 117),
                           INT8_C( -67), INT8_C( -53), INT8_C( -72), INT8_C(  83),
                           INT8_C( -37), INT8_C(  34), INT8_C( 127), INT8_C( -10),
                           INT8_C( 126), INT8_C( -99), INT8_C(-106), INT8_C(  33),
                           INT8_C( 106), INT8_C( -41), INT8_C( -43), INT8_C(  -4),
                           INT8_C(-104), INT8_C(  77), INT8_C(-107), INT8_C( -78),
                           INT8_C( 126), INT8_C(  37), INT8_C(-124), INT8_C( -92),
                           INT8_C( -30), INT8_C( -11), INT8_C( -49), INT8_C(  22),
                           INT8_C(  41), INT8_C(  82), INT8_C( -75), INT8_C(  81),
                           INT8_C(  39), INT8_C( -91), INT8_C(  65), INT8_C( -12)),
      simde_mm512_set_epi8(INT8_C(   1), INT8_C(   0), INT8_C(   0), INT8_C(   1),
                           INT8_C(   0), INT8_C(  -1), INT8_C(   1), INT8_C(   1),
                           INT8_C(  -1), INT8_C(   0), INT8_C(   1), INT8_C(   2),
                           INT8_C(   1), INT8_C(  -2), INT8_C(  18), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   1),
                           INT8_C(  -2), INT8_C(  26), INT8_C(   1), INT8_C(   2),
                           INT8_C(   5), INT8_C(  -6), INT8_C(   0), INT8_C(   0),
                           INT8_C(  -1), INT8_C(   0), INT8_C(   1), INT8_C(   0),
                           INT8_C(   2), INT8_C(   0), INT8_C(   0), INT8_C(   8),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(  -1),
                           INT8_C(   0), INT8_C(   0), INT8_C(   2), INT8_C(   7),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   3), INT8_C(  -1), INT8_C(   0),
                           INT8_C(   0), INT8_C(   1), INT8_C(   0), INT8_C(   0),
                           INT8_C(  -3), INT8_C(   0), INT8_C(   1), INT8_C(  -9)) },
    { simde_mm512_set_epi8(INT8_C( 101), INT8_C(  62), INT8_C( -23), INT8_C(  48),
                           INT8_C( 118), INT8_C(  51), INT8_C(  -2), INT8_C(-103),
                           INT8_C( 110), INT8_C( -27), INT8_C( 109), INT8_C(  60),
                           INT8_C(  81), INT8_C(  82), INT8_C(  61), INT8_C( -96),
                           INT8_C( -57), INT8_C( 116), INT8_C(  -5), INT8_C(   0),
                           INT8_C(  28), INT8_C(  71), INT8_C( -24), INT8_C(  46),
                           INT8_C( -73), INT8_C(   2), INT8_C( -88), INT8_C(  76),
                           INT8_C(  95), INT8_C( -58), INT8_C(  94), INT8_C(  46),
                           INT8_C(  20), INT8_C( 112), INT8_C( -69), INT8_C( 111),
                           INT8_C( -44), INT8_C( -74), INT8_C( -18), INT8_C(  53),
                           INT8_C( 127), INT8_C(  36), INT8_C(  79), INT8_C( -48),
                           INT8_C( 114), INT8_C(  84), INT8_C(  65), INT8_C(-112),
                           INT8_C(-112), INT8_C(  23), INT8_C(  37), INT8_C(  63),
                           INT8_C( -88), INT8_C( -57), INT8_C( 100), INT8_C( 121),
                           INT8_C(  97), INT8_C( 122), INT8_C(  12), INT8_C( -79),
                           INT8_C(  47), INT8_C(  60), INT8_C( -36), INT8_C( -83)),
      simde_mm512_set_epi8(INT8_C(  -6), INT8_C(  53), INT8_C(  88), INT8_C( -36),
                           INT8_C(  96), INT8_C(  32), INT8_C(  77), INT8_C(   2),
                           INT8_C(  -8), INT8_C( -42), INT8_C( -69), INT8_C(  40),
                           INT8_C( -69), INT8_C(  97), INT8_C(  30), INT8_C( 102),
                           INT8_C( -84), INT8_C( -54), INT8_C(-126), INT8_C(  91),
                           INT8_C(  69), INT8_C(  35), INT8_C( 100), INT8_C(-118),
                           INT8_C( -93), INT8_C( 108), INT8_C(  21), INT8_C( -16),
                           INT8_C(  32), INT8_C( 106), INT8_C( -36), INT8_C( -46),
                           INT8_C( -28), INT8_C( -81), INT8_C(  80), INT8_C(  14),
                           INT8_C( -78), INT8_C(   3), INT8_C(  82), INT8_C(-104),
                           INT8_C(  13), INT8_C( -56), INT8_C(-106), INT8_C(  89),
                           INT8_C( -24), INT8_C(  42), INT8_C(  41), INT8_C(  68),
                           INT8_C( -88), INT8_C(-107), INT8_C( -36), INT8_C(  52),
                           INT8_C(  32), INT8_C( -59), INT8_C( -33), INT8_C( 120),
                           INT8_C(  47), INT8_C(-127), INT8_C(  64), INT8_C( 114),
                           INT8_C( 107), INT8_C( -75), INT8_C( 127), INT8_C(  23)),
      simde_mm512_set_epi8(INT8_C( -16), INT8_C(   1), INT8_C(   0), INT8_C(  -1),
                           INT8_C(   1), INT8_C(   1), INT8_C(   0), INT8_C( -51),
                           INT8_C( -13), INT8_C(   0), INT8_C(  -1), INT8_C(   1),
                           INT8_C(  -1), INT8_C(   0), INT8_C(   2), INT8_C(   0),
                           INT8_C(   0), INT8_C(  -2), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   2), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(  -4), INT8_C(  -4),
                           INT8_C(   2), INT8_C(   0), INT8_C(  -2), INT8_C(  -1),
                           INT8_C(   0), INT8_C(  -1), INT8_C(   0), INT8_C(   7),
                           INT8_C(   0), INT8_C( -24), INT8_C(   0), INT8_C(   0),
                           INT8_C(   9), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(  -4), INT8_C(   2), INT8_C(   1), INT8_C(  -1),
                           INT8_C(   1), INT8_C(   0), INT8_C(  -1), INT8_C(   1),
                           INT8_C(  -2), INT8_C(   0), INT8_C(  -3), INT8_C(   1),
                           INT8_C(   2), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(  -3)) },
    { simde_mm512_set_epi8(INT8_C( 106), INT8_C( -71), INT8_C(  61), INT8_C(  19),
                           INT8_C(  29), INT8_C(  79), INT8_C(  45), INT8_C(  94),
                           INT8_C(-112), INT8_C(  60), INT8_C(   2), INT8_C(  77),
                           INT8_C(  30), INT8_C( -34), INT8_C( 102), INT8_C(  43),
                           INT8_C( -87), INT8_C(  52), INT8_C(-104), INT8_C(  -8),
                           INT8_C(-103), INT8_C(  79), INT8_C( -22), INT8_C(  31),
                           INT8_C(  11), INT8_C( 124), INT8_C(  70), INT8_C( -64),
                           INT8_C( -91), INT8_C(  88), INT8_C( -70), INT8_C( -61),
                           INT8_C( -84), INT8_C(-108), INT8_C( -57), INT8_C(  13),
                           INT8_C( -58), INT8_C(  -7), INT8_C(  39), INT8_C(  66),
                           INT8_C(  50), INT8_C( -61), INT8_C(  -9), INT8_C( -41),
                           INT8_C(  25), INT8_C( -31), INT8_C(  64), INT8_C(  18),
                           INT8_C(  73), INT8_C(  60), INT8_C( -53), INT8_C(  42),
                           INT8_C(  -1), INT8_C(  50), INT8_C(  95), INT8_C(  78),
                           INT8_C(  39), INT8_C(  -9), INT8_C(-121), INT8_C( -72),
                           INT8_C(  48), INT8_C(  20), INT8_C(  76), INT8_C( -48)),
      simde_mm512_set_epi8(INT8_C(  12), INT8_C(  55), INT8_C(-111), INT8_C( -85),
                           INT8_C( -94), INT8_C( -11), INT8_C(  57), INT8_C(  93),
                           INT8_C(  32), INT8_C(  57), INT8_C(  61), INT8_C( -21),
                           INT8_C(-102), INT8_C(  75), INT8_C( -15), INT8_C(-114),
                           INT8_C(  26), INT8_C(  71), INT8_C(-127), INT8_C( -52),
                           INT8_C( -57), INT8_C( -26), INT8_C( -36), INT8_C(  -4),
                           INT8_C(  -7), INT8_C(  40), INT8_C(  60), INT8_C(  82),
                           INT8_C(   6), INT8_C( -12), INT8_C(  52), INT8_C( -37),
                           INT8_C( -96), INT8_C(-117), INT8_C( 104), INT8_C( -99),
                           INT8_C(  -1), INT8_C(  95), INT8_C(  81), INT8_C( -70),
                           INT8_C( -22), INT8_C( -86), INT8_C( 114), INT8_C( -43),
                           INT8_C(-120), INT8_C( 109), INT8_C( -86), INT8_C( -33),
                           INT8_C( -23), INT8_C(  69), INT8_C( -80), INT8_C(  61),
                           INT8_C( -35), INT8_C( 107), INT8_C( -31), INT8_C(  11),
                           INT8_C( -45), INT8_C( 125), INT8_C( -53), INT8_C(  -7),
                           INT8_C(  88), INT8_C(-111), INT8_C(  86), INT8_C(-105)),
      simde_mm512_set_epi8(INT8_C(   8), INT8_C(  -1), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(  -7), INT8_C(   0), INT8_C(   1),
                           INT8_C(  -3), INT8_C(   1), INT8_C(   0), INT8_C(  -3),
                           INT8_C(   0), INT8_C(   0), INT8_C(  -6), INT8_C(   0),
                           INT8_C(  -3), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   1), INT8_C(  -3), INT8_C(   0), INT8_C(  -7),
                           INT8_C(  -1), INT8_C(   3), INT8_C(   1), INT8_C(   0),
                           INT8_C( -15), INT8_C(  -7), INT8_C(  -1), INT8_C(   1),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(  58), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(  -2), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(  -3), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(  -3), INT8_C(   7),
                           INT8_C(   0), INT8_C(   0), INT8_C(   2), INT8_C(  10),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0)) },
    { simde_mm512_set_epi8(INT8_C( 102), INT8_C(  35), INT8_C(  43), INT8_C( -33),
                           INT8_C( -74), INT8_C(  81), INT8_C(  81), INT8_C( 115),
                           INT8_C( -81), INT8_C(  72), INT8_C(-127), INT8_C( 118),
                           INT8_C(-113), INT8_C( 106), INT8_C(  25), INT8_C(  84),
                           INT8_C( -82), INT8_C(  58), INT8_C(  13), INT8_C( -38),
                           INT8_C(  -3), INT8_C( 104), INT8_C(  85), INT8_C(-112),
                           INT8_C(  -4), INT8_C(  52), INT8_C(  -2), INT8_C( -64),
                           INT8_C( -23), INT8_C(   5), INT8_C(  33), INT8_C( -11),
                           INT8_C( 116), INT8_C( 110), INT8_C(  21), INT8_C(  84),
                           INT8_C(  42), INT8_C(  77), INT8_C(  25), INT8_C(  68),
                           INT8_C(  71), INT8_C(  60), INT8_C( -51), INT8_C( -46),
                           INT8_C(  -1), INT8_C( -12), INT8_C(  88), INT8_C(  19),
                           INT8_C( -70), INT8_C(  27), INT8_C(  -6), INT8_C(  61),
                           INT8_C( -48), INT8_C( 119), INT8_C(-107), INT8_C(-115),
                           INT8_C(  90), INT8_C(  64), INT8_C(  19), INT8_C(  64),
                           INT8_C( -19), INT8_C(  -7), INT8_C(  40), INT8_C( -68)),
      simde_mm512_set_epi8(INT8_C(  66), INT8_C(  58), INT8_C(  74), INT8_C( -51),
                           INT8_C( -69), INT8_C( -59), INT8_C(  84), INT8_C(  27),
                           INT8_C(  43), INT8_C( -40), INT8_C( -56), INT8_C( 125),
                           INT8_C(   1), INT8_C(  92), INT8_C( -82), INT8_C(  49),
                           INT8_C( -14), INT8_C(  14), INT8_C(  52), INT8_C( -25),
                           INT8_C(  47), INT8_C( -55), INT8_C( -54), INT8_C( -50),
                           INT8_C( -40), INT8_C(-118), INT8_C(  97), INT8_C( -86),
                           INT8_C(  93), INT8_C( 116), INT8_C( -54), INT8_C(-127),
                           INT8_C(  17), INT8_C( -57), INT8_C( -81), INT8_C( -49),
                           INT8_C(  73), INT8_C(  79), INT8_C( -43), INT8_C(  61),
                           INT8_C( -14), INT8_C(  18), INT8_C( 125), INT8_C( -11),
                           INT8_C( -70), INT8_C(  81), INT8_C(-107), INT8_C( -13),
                           INT8_C( -75), INT8_C(  46), INT8_C(  17), INT8_C( -39),
                           INT8_C( -35), INT8_C(  57), INT8_C(  -8), INT8_C( -62),
                           INT8_C( -61), INT8_C( 118), INT8_C( -33), INT8_C( 116),
                           INT8_C(  -5), INT8_C( 120), INT8_C( 126), INT8_C( -48)),
      simde_mm512_set_epi8(INT8_C(   1), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   1), INT8_C(  -1), INT8_C(   0), INT8_C(   4),
                           INT8_C(  -1), INT8_C(  -1), INT8_C(   2), INT8_C(   0),
                           INT8_C(-113), INT8_C(   1), INT8_C(   0), INT8_C(   1),
                           INT8_C(   5), INT8_C(   4), INT8_C(   0), INT8_C(   1),
                           INT8_C(   0), INT8_C(  -1), INT8_C(  -1), INT8_C(   2),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   6), INT8_C(  -1), INT8_C(   0), INT8_C(  -1),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(   1),
                           INT8_C(  -5), INT8_C(   3), INT8_C(   0), INT8_C(   4),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(  -1),
                           INT8_C(   0), INT8_C(   0), INT8_C(   0), INT8_C(  -1),
                           INT8_C(   1), INT8_C(   2), INT8_C(  13), INT8_C(   1),
                           INT8_C(  -1), INT8_C(   0), INT8_C(   0), INT8_C(   0),
                           INT8_C(   3), INT8_C(   0), INT8_C(   0), INT8_C(   1)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_div_epi8(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_div_epi16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi16(INT16_C(-20040), INT16_C(  8356), INT16_C(-32332), INT16_C( 10333),
                            INT16_C( -5915), INT16_C( 26879), INT16_C(  2532), INT16_C( 21861),
                            INT16_C(-27724), INT16_C(-13980), INT16_C(-30566), INT16_C(-12851),
                            INT16_C( 30608), INT16_C( 27665), INT16_C(   548), INT16_C(  7224),
                            INT16_C(-23312), INT16_C( -9410), INT16_C(  2838), INT16_C(-28448),
                            INT16_C( 30003), INT16_C(-15914), INT16_C(-27549), INT16_C(  6027),
                            INT16_C( 28687), INT16_C(-19881), INT16_C(  5735), INT16_C(  9519),
                            INT16_C( -3746), INT16_C(-25453), INT16_C(-16345), INT16_C(-27291)),
      simde_mm512_set_epi16(INT16_C(  4335), INT16_C( -8694), INT16_C( 20589), INT16_C( -2761),
                            INT16_C( -3216), INT16_C(-24783), INT16_C(-17777), INT16_C(  -501),
                            INT16_C( 25504), INT16_C( 26559), INT16_C( 27843), INT16_C( 31769),
                            INT16_C(-18807), INT16_C(  5762), INT16_C(-26736), INT16_C( 14349),
                            INT16_C(-15519), INT16_C(  4924), INT16_C(-19685), INT16_C( 31074),
                            INT16_C(-20201), INT16_C( -4452), INT16_C( 11125), INT16_C( 19762),
                            INT16_C(-31890), INT16_C(-20519), INT16_C(-27796), INT16_C(  4844),
                            INT16_C(  1980), INT16_C(-25222), INT16_C(-27366), INT16_C( 20455)),
      simde_mm512_set_epi16(INT16_C(    -4), INT16_C(     0), INT16_C(    -1), INT16_C(    -3),
                            INT16_C(     1), INT16_C(    -1), INT16_C(     0), INT16_C(   -43),
                            INT16_C(    -1), INT16_C(     0), INT16_C(    -1), INT16_C(     0),
                            INT16_C(    -1), INT16_C(     4), INT16_C(     0), INT16_C(     0),
                            INT16_C(     1), INT16_C(    -1), INT16_C(     0), INT16_C(     0),
                            INT16_C(    -1), INT16_C(     3), INT16_C(    -2), INT16_C(     0),
                            INT16_C(     0), INT16_C(     0), INT16_C(     0), INT16_C(     1),
                            INT16_C(    -1), INT16_C(     1), INT16_C(     0), INT16_C(    -1)) },
    { simde_mm512_set_epi16(INT16_C( 30542), INT16_C(-21686), INT16_C(-12987), INT16_C(-10637),
                            INT16_C( -1601), INT16_C(-28302), INT16_C( 15211), INT16_C(-14111),
                            INT16_C( 25976), INT16_C( 21242), INT16_C(-23929), INT16_C(-19059),
                            INT16_C(-25081), INT16_C(  5942), INT16_C(-21376), INT16_C(  4770),
                            INT16_C( -1129), INT16_C(-19990), INT16_C( 26476), INT16_C(-29290),
                            INT16_C(-16617), INT16_C(-24641), INT16_C( 13060), INT16_C(-26392),
                            INT16_C(-31122), INT16_C(  1166), INT16_C(-13169), INT16_C( 10959),
                            INT16_C(  3043), INT16_C(-24353), INT16_C(-25618), INT16_C(  3998)),
      simde_mm512_set_epi16(INT16_C(  8697), INT16_C(  4862), INT16_C(-26319), INT16_C(-11370),
                            INT16_C(  4314), INT16_C(-16926), INT16_C( 26882), INT16_C(  8784),
                            INT16_C(-23412), INT16_C(  6784), INT16_C( 27807), INT16_C( 29358),
                            INT16_C( 28774), INT16_C( -1248), INT16_C( 14871), INT16_C(  4639),
                            INT16_C( 17536), INT16_C( -3921), INT16_C(-31860), INT16_C( 18313),
                            INT16_C( 13025), INT16_C(-15494), INT16_C( -6838), INT16_C(-31563),
                            INT16_C( 10488), INT16_C( 29317), INT16_C(  5913), INT16_C( -5447),
                            INT16_C( 11124), INT16_C(-18588), INT16_C(-20055), INT16_C( 31068)),
      simde_mm512_set_epi16(INT16_C(     3), INT16_C(    -4), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(     1), INT16_C(     0), INT16_C(    -1),
                            INT16_C(    -1), INT16_C(     3), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(    -4), INT16_C(    -1), INT16_C(     1),
                            INT16_C(     0), INT16_C(     5), INT16_C(     0), INT16_C(    -1),
                            INT16_C(    -1), INT16_C(     1), INT16_C(    -1), INT16_C(     0),
                            INT16_C(    -2), INT16_C(     0), INT16_C(    -2), INT16_C(    -2),
                            INT16_C(     0), INT16_C(     1), INT16_C(     1), INT16_C(     0)) },
    { simde_mm512_set_epi16(INT16_C( 10506), INT16_C( 27276), INT16_C( 10689), INT16_C(  7669),
                            INT16_C( -9146), INT16_C(-17193), INT16_C(  7411), INT16_C(  5177),
                            INT16_C( 18940), INT16_C(-16405), INT16_C(  3246), INT16_C(  3104),
                            INT16_C( -7140), INT16_C( 31568), INT16_C( -2399), INT16_C(-28909),
                            INT16_C( 26564), INT16_C(-28507), INT16_C(  3797), INT16_C( -9359),
                            INT16_C(-12946), INT16_C( 18074), INT16_C( -6465), INT16_C(  3679),
                            INT16_C( 17483), INT16_C( -5905), INT16_C(  3591), INT16_C(-20227),
                            INT16_C( -6079), INT16_C( -1639), INT16_C(-29076), INT16_C( 29393)),
      simde_mm512_set_epi16(INT16_C( 11630), INT16_C(  9206), INT16_C(-15696), INT16_C(  3180),
                            INT16_C( 12868), INT16_C(-30976), INT16_C( -5774), INT16_C(-11992),
                            INT16_C(-18085), INT16_C( 32470), INT16_C( 17470), INT16_C(-31399),
                            INT16_C(  9368), INT16_C(  3571), INT16_C(  7161), INT16_C(-27278),
                            INT16_C(  9802), INT16_C( 20270), INT16_C(-19501), INT16_C( 19621),
                            INT16_C( 14613), INT16_C( -6394), INT16_C( -6716), INT16_C( -8239),
                            INT16_C(-25839), INT16_C( 28062), INT16_C( -8851), INT16_C(-12431),
                            INT16_C( -8955), INT16_C(  -676), INT16_C( 10256), INT16_C( 15625)),
      simde_mm512_set_epi16(INT16_C(     0), INT16_C(     2), INT16_C(     0), INT16_C(     2),
                            INT16_C(     0), INT16_C(     0), INT16_C(    -1), INT16_C(     0),
                            INT16_C(    -1), INT16_C(     0), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(     8), INT16_C(     0), INT16_C(     1),
                            INT16_C(     2), INT16_C(    -1), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(    -2), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(     0), INT16_C(     0), INT16_C(     1),
                            INT16_C(     0), INT16_C(     2), INT16_C(    -2), INT16_C(     1)) },
    { simde_mm512_set_epi16(INT16_C( 14453), INT16_C(-27323), INT16_C( 14069), INT16_C(-15038),
                            INT16_C( 29890), INT16_C(-32496), INT16_C( -8033), INT16_C(  2034),
                            INT16_C( 28252), INT16_C(-12993), INT16_C(-12172), INT16_C( 21268),
                            INT16_C(-19693), INT16_C( -3590), INT16_C( -7723), INT16_C(-15496),
                            INT16_C( -5494), INT16_C( 10297), INT16_C( 10325), INT16_C( 32003),
                            INT16_C(-11357), INT16_C( 14609), INT16_C(-13537), INT16_C( 17128),
                            INT16_C(  6812), INT16_C( 32194), INT16_C(   287), INT16_C(  5824),
                            INT16_C( 13352), INT16_C(-19334), INT16_C(  8294), INT16_C(-20267)),
      simde_mm512_set_epi16(INT16_C(-10192), INT16_C(-26586), INT16_C( 32452), INT16_C(  4989),
                            INT16_C(-13693), INT16_C(-13838), INT16_C(  2151), INT16_C( 31183),
                            INT16_C(-12217), INT16_C( 28038), INT16_C( 27497), INT16_C(-25404),
                            INT16_C(-25184), INT16_C(-12134), INT16_C( 25347), INT16_C( -5075),
                            INT16_C( 19038), INT16_C(  9321), INT16_C(-20974), INT16_C( 22487),
                            INT16_C( -3253), INT16_C(-14033), INT16_C( 24624), INT16_C( 14772),
                            INT16_C( 16067), INT16_C(-16101), INT16_C( 12034), INT16_C( 11420),
                            INT16_C(-30652), INT16_C(-30195), INT16_C(-10496), INT16_C( 32407)),
      simde_mm512_set_epi16(INT16_C(    -1), INT16_C(     1), INT16_C(     0), INT16_C(    -3),
                            INT16_C(    -2), INT16_C(     2), INT16_C(    -3), INT16_C(     0),
                            INT16_C(    -2), INT16_C(     0), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(     0), INT16_C(     0), INT16_C(     3),
                            INT16_C(     0), INT16_C(     1), INT16_C(     0), INT16_C(     1),
                            INT16_C(     3), INT16_C(    -1), INT16_C(     0), INT16_C(     1),
                            INT16_C(     0), INT16_C(    -1), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(     0), INT16_C(     0), INT16_C(     0)) },
    { simde_mm512_set_epi16(INT16_C(-12762), INT16_C(  -143), INT16_C( 24201), INT16_C( 27500),
                            INT16_C(-21606), INT16_C(-10954), INT16_C( 30460), INT16_C( 28331),
                            INT16_C(-22171), INT16_C(-30589), INT16_C( 16765), INT16_C(-17393),
                            INT16_C( 31673), INT16_C( 13306), INT16_C( -8624), INT16_C( -3653),
                            INT16_C(-23812), INT16_C(  2378), INT16_C( -6069), INT16_C( -8645),
                            INT16_C(  9750), INT16_C(  6252), INT16_C(-30407), INT16_C(-28082),
                            INT16_C(-14686), INT16_C( -5840), INT16_C( 24502), INT16_C( 12329),
                            INT16_C( -5959), INT16_C(-16932), INT16_C( -4867), INT16_C( 10388)),
      simde_mm512_set_epi16(INT16_C(-30203), INT16_C(-31292), INT16_C(  7054), INT16_C( 31766),
                            INT16_C(-23643), INT16_C( -7634), INT16_C( 23958), INT16_C(-19164),
                            INT16_C( 32358), INT16_C( 32485), INT16_C( -8137), INT16_C(  2854),
                            INT16_C(   443), INT16_C(  3757), INT16_C(-31602), INT16_C( 26770),
                            INT16_C(  1434), INT16_C(-26880), INT16_C(-13137), INT16_C(-25600),
                            INT16_C(  3310), INT16_C( 31739), INT16_C( 22782), INT16_C( 27721),
                            INT16_C(-28215), INT16_C( 10286), INT16_C( 11994), INT16_C(-23317),
                            INT16_C(-11843), INT16_C(  6466), INT16_C(  8900), INT16_C( 11867)),
      simde_mm512_set_epi16(INT16_C(     0), INT16_C(     0), INT16_C(     3), INT16_C(     0),
                            INT16_C(     0), INT16_C(     1), INT16_C(     1), INT16_C(    -1),
                            INT16_C(     0), INT16_C(     0), INT16_C(    -2), INT16_C(    -6),
                            INT16_C(    71), INT16_C(     3), INT16_C(     0), INT16_C(     0),
                            INT16_C(   -16), INT16_C(     0), INT16_C(     0), INT16_C(     0),
                            INT16_C(     2), INT16_C(     0), INT16_C(    -1), INT16_C(    -1),
                            INT16_C(     0), INT16_C(     0), INT16_C(     2), INT16_C(     0),
                            INT16_C(     0), INT16_C(    -2), INT16_C(     0), INT16_C(     0)) },
    { simde_mm512_set_epi16(INT16_C(-29408), INT16_C(  7369), INT16_C( -5051), INT16_C(  7942),
                            INT16_C( 18019), INT16_C(-25065), INT16_C( -8302), INT16_C( 17011),
                            INT16_C(  2762), INT16_C( 27559), INT16_C( 18647), INT16_C( 22035),
                            INT16_C(-10618), INT16_C( -3223), INT16_C( 25352), INT16_C(-32696),
                            INT16_C( -1859), INT16_C(-20090), INT16_C( 18297), INT16_C(-27701),
                            INT16_C(-31478), INT16_C(-13300), INT16_C(-15493), INT16_C(-16792),
                            INT16_C(-23954), INT16_C(-14239), INT16_C(-15716), INT16_C( 12103),
                            INT16_C(-30330), INT16_C( -2111), INT16_C(-26781), INT16_C( 25851)),
      simde_mm512_set_epi16(INT16_C( 11252), INT16_C(-25669), INT16_C(-31001), INT16_C( 13518),
                            INT16_C( 30845), INT16_C(-14200), INT16_C(-30880), INT16_C( 22795),
                            INT16_C(-15552), INT16_C( -1554), INT16_C( 29162), INT16_C( -8371),
                            INT16_C(  5731), INT16_C( 22086), INT16_C(  7870), INT16_C(-26229),
                            INT16_C( 19406), INT16_C(-22832), INT16_C(-14386), INT16_C( 22375),
                            INT16_C( -8274), INT16_C( -9174), INT16_C(-24184), INT16_C( 24847),
                            INT16_C( 26808), INT16_C( -2235), INT16_C(  4293), INT16_C(-30072),
                            INT16_C( 23713), INT16_C( 20910), INT16_C(  6378), INT16_C(-18450)),
      simde_mm512_set_epi16(INT16_C(    -2), INT16_C(     0), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(     1), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(   -17), INT16_C(     0), INT16_C(    -2),
                            INT16_C(    -1), INT16_C(     0), INT16_C(     3), INT16_C(     1),
                            INT16_C(     0), INT16_C(     0), INT16_C(    -1), INT16_C(    -1),
                            INT16_C(     3), INT16_C(     1), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(     6), INT16_C(    -3), INT16_C(     0),
                            INT16_C(    -1), INT16_C(     0), INT16_C(    -4), INT16_C(    -1)) },
    { simde_mm512_set_epi16(INT16_C( -8644), INT16_C(  4438), INT16_C(  1025), INT16_C(-26642),
                            INT16_C( 18378), INT16_C(-13976), INT16_C( 21110), INT16_C( 14955),
                            INT16_C(  2525), INT16_C(-19773), INT16_C( 28133), INT16_C(-32693),
                            INT16_C( 12259), INT16_C(-21141), INT16_C(-27294), INT16_C( 16198),
                            INT16_C( -2640), INT16_C( 31144), INT16_C(-15827), INT16_C( 20747),
                            INT16_C(-19791), INT16_C( 30374), INT16_C( -9055), INT16_C(-20334),
                            INT16_C( 28339), INT16_C( 29800), INT16_C( 32312), INT16_C(-19316),
                            INT16_C(-15043), INT16_C(-27434), INT16_C( 29424), INT16_C(-25521)),
      simde_mm512_set_epi16(INT16_C(-24272), INT16_C( -9025), INT16_C(-17538), INT16_C(-13789),
                            INT16_C(  3646), INT16_C( 17578), INT16_C( -9614), INT16_C(-11054),
                            INT16_C( 23757), INT16_C( -5736), INT16_C(  8067), INT16_C( 10531),
                            INT16_C(-24488), INT16_C( 16639), INT16_C(-22179), INT16_C( -8704),
                            INT16_C(  -927), INT16_C(-31517), INT16_C( 10091), INT16_C( 19448),
                            INT16_C( 12069), INT16_C(  8742), INT16_C( 16653), INT16_C( 31958),
                            INT16_C(-18440), INT16_C(-30513), INT16_C( -3426), INT16_C( -7330),
                            INT16_C( 24804), INT16_C( 18228), INT16_C( 16072), INT16_C(-15326)),
      simde_mm512_set_epi16(INT16_C(     0), INT16_C(     0), INT16_C(     0), INT16_C(     1),
                            INT16_C(     5), INT16_C(     0), INT16_C(    -2), INT16_C(    -1),
                            INT16_C(     0), INT16_C(     3), INT16_C(     3), INT16_C(    -3),
                            INT16_C(     0), INT16_C(    -1), INT16_C(     1), INT16_C(    -1),
                            INT16_C(     2), INT16_C(     0), INT16_C(    -1), INT16_C(     1),
                            INT16_C(    -1), INT16_C(     3), INT16_C(     0), INT16_C(     0),
                            INT16_C(    -1), INT16_C(     0), INT16_C(    -9), INT16_C(     2),
                            INT16_C(     0), INT16_C(    -1), INT16_C(     1), INT16_C(     1)) },
    { simde_mm512_set_epi16(INT16_C( 23232), INT16_C(-29257), INT16_C(  1254), INT16_C( -9317),
                            INT16_C(-20336), INT16_C( 10081), INT16_C( 18681), INT16_C( 12677),
                            INT16_C( 17973), INT16_C(-10276), INT16_C(-23503), INT16_C( 18772),
                            INT16_C(  8312), INT16_C( 15138), INT16_C( -9415), INT16_C(-23183),
                            INT16_C(  4065), INT16_C( 14928), INT16_C( -9505), INT16_C( -3213),
                            INT16_C( -8135), INT16_C(-17864), INT16_C(-23451), INT16_C( -2372),
                            INT16_C( 14548), INT16_C(-10992), INT16_C(  6282), INT16_C(-22066),
                            INT16_C(-11858), INT16_C( 14867), INT16_C( -6173), INT16_C( 24146)),
      simde_mm512_set_epi16(INT16_C(-20244), INT16_C( 14874), INT16_C(  7829), INT16_C( 32218),
                            INT16_C( 17818), INT16_C(   309), INT16_C( 27668), INT16_C(  9211),
                            INT16_C( 15166), INT16_C(  4076), INT16_C( 28109), INT16_C(-30601),
                            INT16_C(  4803), INT16_C(-19074), INT16_C(-23287), INT16_C(-27917),
                            INT16_C(  7634), INT16_C(-13255), INT16_C( 14290), INT16_C( -8590),
                            INT16_C(-11602), INT16_C(  9361), INT16_C(-18559), INT16_C(  3976),
                            INT16_C( 20763), INT16_C( 17266), INT16_C(  8709), INT16_C(-30498),
                            INT16_C( 31994), INT16_C(-17983), INT16_C( 25233), INT16_C( 29991)),
      simde_mm512_set_epi16(INT16_C(    -1), INT16_C(    -1), INT16_C(     0), INT16_C(     0),
                            INT16_C(    -1), INT16_C(    32), INT16_C(     0), INT16_C(     1),
                            INT16_C(     1), INT16_C(    -2), INT16_C(     0), INT16_C(     0),
                            INT16_C(     1), INT16_C(     0), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(    -1), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(    -1), INT16_C(     1), INT16_C(     0),
                            INT16_C(     0), INT16_C(     0), INT16_C(     0), INT16_C(     0),
                            INT16_C(     0), INT16_C(     0), INT16_C(     0), INT16_C(     0)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_div_epi16(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_div_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi32(INT32_C(-1425964510), INT32_C( 1884851068), INT32_C( -245085200), INT32_C(  312441627),
                            INT32_C( 1361020823), INT32_C( -269027644), INT32_C( 2046290516), INT32_C(  253262419),
                            INT32_C(-1435031175), INT32_C( -983397284), INT32_C( 1158205006), INT32_C( 2142968427),
                            INT32_C( -610621785), INT32_C(-1874018384), INT32_C(  408084487), INT32_C(  314643093)),
      simde_mm512_set_epi32(INT32_C(-1816447538), INT32_C( 1352799684), INT32_C(  437452333), INT32_C(-2106809533),
                            INT32_C(  850823800), INT32_C(-1580883911), INT32_C(-2115707304), INT32_C( 1577531711),
                            INT32_C(  801246884), INT32_C(   59025302), INT32_C(  905783489), INT32_C(-1645941779),
                            INT32_C(  962943312), INT32_C( 2128170875), INT32_C(-1348448230), INT32_C( -975134432)),
      simde_mm512_set_epi32(INT32_C(          0), INT32_C(          1), INT32_C(          0), INT32_C(          0),
                            INT32_C(          1), INT32_C(          0), INT32_C(          0), INT32_C(          0),
                            INT32_C(         -1), INT32_C(        -16), INT32_C(          1), INT32_C(         -1),
                            INT32_C(          0), INT32_C(          0), INT32_C(          0), INT32_C(          0)) },
    { simde_mm512_set_epi32(INT32_C( 1427225802), INT32_C(-1035302594), INT32_C( -199744603), INT32_C( 1376388625),
                            INT32_C(-2114897409), INT32_C( 1679349706), INT32_C(-1031333846), INT32_C(-1198347443),
                            INT32_C( -637748341), INT32_C( 1314591131), INT32_C(  282479090), INT32_C( 1660196054),
                            INT32_C(-1167126507), INT32_C(-1998854068), INT32_C(  933881032), INT32_C( -624384653)),
      simde_mm512_set_epi32(INT32_C( 1612321322), INT32_C( 2051698478), INT32_C( 1596883036), INT32_C(-1369467325),
                            INT32_C( 1851004364), INT32_C( 1092388812), INT32_C(  828772877), INT32_C( -259189725),
                            INT32_C( -849691191), INT32_C(-1191458488), INT32_C(  801339023), INT32_C( -104328386),
                            INT32_C(  757083857), INT32_C(-1236967236), INT32_C( -850146114), INT32_C( 1258625824)),
      simde_mm512_set_epi32(INT32_C(          0), INT32_C(          0), INT32_C(          0), INT32_C(         -1),
                            INT32_C(         -1), INT32_C(          1), INT32_C(         -1), INT32_C(          4),
                            INT32_C(          0), INT32_C(         -1), INT32_C(          0), INT32_C(        -15),
                            INT32_C(         -1), INT32_C(          1), INT32_C(         -1), INT32_C(          0)) },
    { simde_mm512_set_epi32(INT32_C(  237418199), INT32_C(  -70579339), INT32_C(-2042257710), INT32_C( 1462546998),
                            INT32_C( -202189538), INT32_C(-1353367648), INT32_C(  304511606), INT32_C( -539003093),
                            INT32_C( 1923205305), INT32_C(  464427515), INT32_C( -694421636), INT32_C(-1729085762),
                            INT32_C( 1377800186), INT32_C( -626233146), INT32_C(-2090091895), INT32_C( 1314335058)),
      simde_mm512_set_epi32(INT32_C(   38009422), INT32_C( -855531694), INT32_C( 1096529400), INT32_C(  740723389),
                            INT32_C( -703601695), INT32_C(-1082310854), INT32_C(  120520136), INT32_C(  494300544),
                            INT32_C(-1280011607), INT32_C(-1943894617), INT32_C( -321878744), INT32_C( -690430536),
                            INT32_C( 1135419008), INT32_C( 1818004981), INT32_C( 1471877533), INT32_C(  559240384)),
      simde_mm512_set_epi32(INT32_C(          6), INT32_C(          0), INT32_C(         -1), INT32_C(          1),
                            INT32_C(          0), INT32_C(          1), INT32_C(          2), INT32_C(         -1),
                            INT32_C(         -1), INT32_C(          0), INT32_C(          2), INT32_C(          2),
                            INT32_C(          1), INT32_C(          0), INT32_C(         -1), INT32_C(          2)) },
    { simde_mm512_set_epi32(INT32_C(-1724745069), INT32_C( 1135206576), INT32_C( 1179583658), INT32_C(-1966673560),
                            INT32_C(  876279100), INT32_C( -587502732), INT32_C( -149418425), INT32_C( -921830900),
                            INT32_C(   17215575), INT32_C(-1719497158), INT32_C(-1349196793), INT32_C( 1245762398),
                            INT32_C(  813297065), INT32_C( -835921648), INT32_C(-1975778091), INT32_C( 2110087211)),
      simde_mm512_set_epi32(INT32_C(-1421142882), INT32_C( -720107087), INT32_C( -533473336), INT32_C(-1235553858),
                            INT32_C( 1997884077), INT32_C(-1507361050), INT32_C(   21786729), INT32_C(  743816821),
                            INT32_C(  150690827), INT32_C(-1210873139), INT32_C( 1036977320), INT32_C( -399295069),
                            INT32_C(-1569884506), INT32_C( -616191901), INT32_C(-1839631465), INT32_C( -912247900)),
      simde_mm512_set_epi32(INT32_C(          1), INT32_C(         -1), INT32_C(         -2), INT32_C(          1),
                            INT32_C(          0), INT32_C(          0), INT32_C(         -6), INT32_C(         -1),
                            INT32_C(          0), INT32_C(          1), INT32_C(         -1), INT32_C(         -3),
                            INT32_C(          0), INT32_C(          1), INT32_C(          1), INT32_C(         -2)) },
    { simde_mm512_set_epi32(INT32_C( -788754092), INT32_C( 1871593252), INT32_C(-1494005905), INT32_C(-1673341020),
                            INT32_C( -802349852), INT32_C( 1483795222), INT32_C( -482009835), INT32_C(  -91245467),
                            INT32_C( 1580169915), INT32_C(  692091070), INT32_C( 1863695169), INT32_C( -863865867),
                            INT32_C(-1394651654), INT32_C( -860864123), INT32_C(  684761994), INT32_C(-1721896503)),
      simde_mm512_set_epi32(INT32_C(-1337054377), INT32_C(   66234694), INT32_C(-1856118156), INT32_C(-1127800230),
                            INT32_C(  814009506), INT32_C(-2034345199), INT32_C( 1765405247), INT32_C(-1048066647),
                            INT32_C( -423083536), INT32_C(-1848382006), INT32_C( -152706477), INT32_C(-1375856509),
                            INT32_C(  -23675804), INT32_C( -242644348), INT32_C( 1836148713), INT32_C(  -17324905)),
      simde_mm512_set_epi32(INT32_C(          0), INT32_C(         28), INT32_C(          0), INT32_C(          1),
                            INT32_C(          0), INT32_C(          0), INT32_C(          0), INT32_C(          0),
                            INT32_C(         -3), INT32_C(          0), INT32_C(        -12), INT32_C(          0),
                            INT32_C(         58), INT32_C(          3), INT32_C(          0), INT32_C(         99)) },
    { simde_mm512_set_epi32(INT32_C( -463247298), INT32_C( -951467140), INT32_C( 1433027324), INT32_C(-1349535490),
                            INT32_C( -916446608), INT32_C(-1679952824), INT32_C(  515026148), INT32_C(  -79374441),
                            INT32_C(-1055204414), INT32_C( 1214763982), INT32_C( -351626877), INT32_C(  427209663),
                            INT32_C( 1651021910), INT32_C( -181051643), INT32_C(-1481830173), INT32_C( 1285378207)),
      simde_mm512_set_epi32(INT32_C( -895026020), INT32_C(-2124493776), INT32_C( -806312731), INT32_C(  721610054),
                            INT32_C(  677519448), INT32_C( 1470235459), INT32_C(-2123699180), INT32_C(  883454038),
                            INT32_C(-2020088518), INT32_C( -300465294), INT32_C( 1493254397), INT32_C( 2062995345),
                            INT32_C(  -10095941), INT32_C(-1400374264), INT32_C( 1068728589), INT32_C(  234142625)),
      simde_mm512_set_epi32(INT32_C(          0), INT32_C(          0), INT32_C(         -1), INT32_C(         -1),
                            INT32_C(         -1), INT32_C(         -1), INT32_C(          0), INT32_C(          0),
                            INT32_C(          0), INT32_C(         -4), INT32_C(          0), INT32_C(          0),
                            INT32_C(       -163), INT32_C(          0), INT32_C(         -1), INT32_C(          5)) },
    { simde_mm512_set_epi32(INT32_C( -939190848), INT32_C(-2083825761), INT32_C( 2014997186), INT32_C(  790185633),
                            INT32_C(-1507225536), INT32_C( -384122450), INT32_C(-1588213257), INT32_C(-1040817544),
                            INT32_C( 1965628193), INT32_C(-2067530457), INT32_C( 1204204418), INT32_C(  -39160501),
                            INT32_C( -605764870), INT32_C(  561973657), INT32_C( 1912174450), INT32_C( 1415728252)),
      simde_mm512_set_epi32(INT32_C( -927506034), INT32_C(  155586444), INT32_C( -406884871), INT32_C( -252994257),
                            INT32_C( 1219028873), INT32_C(-1972688074), INT32_C( -597390303), INT32_C(  291669377),
                            INT32_C( -695882735), INT32_C(  879590202), INT32_C( 1348714758), INT32_C( 1712617745),
                            INT32_C( -236530514), INT32_C( 1880792230), INT32_C( 1810070042), INT32_C(-1599785869)),
      simde_mm512_set_epi32(INT32_C(          1), INT32_C(        -13), INT32_C(         -4), INT32_C(         -3),
                            INT32_C(         -1), INT32_C(          0), INT32_C(          2), INT32_C(         -3),
                            INT32_C(         -2), INT32_C(         -2), INT32_C(          0), INT32_C(          0),
                            INT32_C(          2), INT32_C(          0), INT32_C(          1), INT32_C(          0)) },
    { simde_mm512_set_epi32(INT32_C(-1601700614), INT32_C( 1985924496), INT32_C( -342633815), INT32_C(-2007999861),
                            INT32_C(  297828713), INT32_C( 1383645848), INT32_C(-2056044415), INT32_C(  373512753),
                            INT32_C(  -26545593), INT32_C( -328575199), INT32_C( -462276628), INT32_C( 1976153041),
                            INT32_C( 1430984961), INT32_C(-1934079238), INT32_C(  399344654), INT32_C( 1569206763)),
      simde_mm512_set_epi32(INT32_C(  102595444), INT32_C(  731375272), INT32_C(-1673993680), INT32_C( -406822977),
                            INT32_C( -578959028), INT32_C( 1173139127), INT32_C(-1295304556), INT32_C(  955166905),
                            INT32_C(  270270084), INT32_C(  134608446), INT32_C( -519669996), INT32_C( -265658570),
                            INT32_C(-1584344142), INT32_C( 1279036686), INT32_C(-1076842770), INT32_C(  -44502324)),
      simde_mm512_set_epi32(INT32_C(        -15), INT32_C(          2), INT32_C(          0), INT32_C(          4),
                            INT32_C(          0), INT32_C(          1), INT32_C(          1), INT32_C(          0),
                            INT32_C(          0), INT32_C(         -2), INT32_C(          0), INT32_C(         -7),
                            INT32_C(          0), INT32_C(         -1), INT32_C(          0), INT32_C(        -35)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_div_epi32(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_mask_div_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i src;
    simde__mmask16 k;
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi32(INT32_C(  691121094), INT32_C(  674034227), INT32_C(-1965434887), INT32_C( -920286947),
                            INT32_C( -374673026), INT32_C(-1240805178), INT32_C( 1568850865), INT32_C(-1142977539),
                            INT32_C(-1079516608), INT32_C( -708153743), INT32_C( 1508722402), INT32_C(-2074345640),
                            INT32_C( 1747596798), INT32_C(-2063703989), INT32_C(  527472553), INT32_C(-1403096998)),
      UINT16_C(63371),
      simde_mm512_set_epi32(INT32_C( -341007878), INT32_C(-1764810870), INT32_C( 1179683687), INT32_C(-1646326602),
                            INT32_C( -671967289), INT32_C(-1586327268), INT32_C( 1691051285), INT32_C(   50347892),
                            INT32_C(  728425428), INT32_C( 1192263444), INT32_C(-2086343723), INT32_C( 1322777130),
                            INT32_C(  163989560), INT32_C( 1492341726), INT32_C(  298608154), INT32_C( 1250819173)),
      simde_mm512_set_epi32(INT32_C(-1291033589), INT32_C( 1314482530), INT32_C(-1297250617), INT32_C( -739008036),
                            INT32_C(-1419039999), INT32_C(-1004264650), INT32_C( 1580565751), INT32_C( -471064457),
                            INT32_C( 2081361826), INT32_C(  493161721), INT32_C(-1195115819), INT32_C(  894221337),
                            INT32_C(-1330460172), INT32_C(  492373082), INT32_C(  -13096811), INT32_C(-2087181083)),
      simde_mm512_set_epi32(INT32_C(          0), INT32_C(         -1), INT32_C(          0), INT32_C(          2),
                            INT32_C( -374673026), INT32_C(          1), INT32_C(          1), INT32_C(          0),
                            INT32_C(          0), INT32_C( -708153743), INT32_C( 1508722402), INT32_C(-2074345640),
                            INT32_C(          0), INT32_C(-2063703989), INT32_C(        -22), INT32_C(          0)) },
    { simde_mm512_set_epi32(INT32_C( 1779168063), INT32_C(-1138893231), INT32_C( -687161637), INT32_C( 1828175063),
                            INT32_C( -389420023), INT32_C( -193211433), INT32_C( -857989172), INT32_C( -448329300),
                            INT32_C(-1601364212), INT32_C( 1710148738), INT32_C( 1974123080), INT32_C(-1424367196),
                            INT32_C(  118588227), INT32_C(  542053192), INT32_C(  499863549), INT32_C(  957375358)),
      UINT16_C(36797),
      simde_mm512_set_epi32(INT32_C(-1153303869), INT32_C(  562234020), INT32_C( 1763100483), INT32_C( -518004559),
                            INT32_C(-1450358898), INT32_C(-1409866198), INT32_C(  269910347), INT32_C(  433971495),
                            INT32_C( 1441956227), INT32_C( 1018271575), INT32_C( 1734496959), INT32_C(  380846712),
                            INT32_C( -941967689), INT32_C( -739443621), INT32_C( 1995198557), INT32_C( -980655097)),
      simde_mm512_set_epi32(INT32_C(-2088961787), INT32_C( 1943141679), INT32_C( -665465241), INT32_C( -342195833),
                            INT32_C( 2102184556), INT32_C(  877111492), INT32_C( 1183491905), INT32_C( -576610979),
                            INT32_C(-1061316197), INT32_C( -808097400), INT32_C( -362876916), INT32_C(-1845390533),
                            INT32_C(  -48621016), INT32_C(  201516689), INT32_C(-1435930720), INT32_C(-1932876068)),
      simde_mm512_set_epi32(INT32_C(          0), INT32_C(-1138893231), INT32_C( -687161637), INT32_C( 1828175063),
                            INT32_C(          0), INT32_C(         -1), INT32_C(          0), INT32_C(          0),
                            INT32_C(         -1), INT32_C( 1710148738), INT32_C(         -4), INT32_C(          0),
                            INT32_C(         19), INT32_C(         -3), INT32_C(  499863549), INT32_C(          0)) },
    { simde_mm512_set_epi32(INT32_C( -179829877), INT32_C(  651362699), INT32_C(  495870887), INT32_C( -382126427),
                            INT32_C(  915244711), INT32_C(    5081424), INT32_C( 1422501384), INT32_C( -163979724),
                            INT32_C(-1516900265), INT32_C(  497965579), INT32_C(  910061584), INT32_C( 2002226944),
                            INT32_C( -621963189), INT32_C(  -48343218), INT32_C(  523093293), INT32_C(-1235205724)),
      UINT16_C(46902),
      simde_mm512_set_epi32(INT32_C( -220620904), INT32_C( 1398655610), INT32_C( 1722520923), INT32_C( 1206471293),
                            INT32_C( 1374915518), INT32_C(  531653117), INT32_C( 2075187308), INT32_C( -144618549),
                            INT32_C(-2131865715), INT32_C( 1444783055), INT32_C( 1878625233), INT32_C( 1755684145),
                            INT32_C(-2061726371), INT32_C(-1050443653), INT32_C(-1299940555), INT32_C(-2116696545)),
      simde_mm512_set_epi32(INT32_C(-1106093489), INT32_C( 1982658188), INT32_C(  863153207), INT32_C(-1637276628),
                            INT32_C(  448681074), INT32_C( 1334667053), INT32_C(  502667641), INT32_C(  855395764),
                            INT32_C(-1672092948), INT32_C(  808531712), INT32_C(  454488139), INT32_C(  123547093),
                            INT32_C(  483090439), INT32_C(-1126329757), INT32_C(-1201220189), INT32_C( -136050629)),
      simde_mm512_set_epi32(INT32_C(          0), INT32_C(  651362699), INT32_C(          1), INT32_C(          0),
                            INT32_C(  915244711), INT32_C(          0), INT32_C(          4), INT32_C(          0),
                            INT32_C(-1516900265), INT32_C(  497965579), INT32_C(          4), INT32_C(         14),
                            INT32_C( -621963189), INT32_C(          0), INT32_C(          1), INT32_C(-1235205724)) },
    { simde_mm512_set_epi32(INT32_C( 2113970745), INT32_C( -182128842), INT32_C(  564512596), INT32_C(  604721400),
                            INT32_C( 1471174399), INT32_C(-1803940708), INT32_C(-1765392929), INT32_C(  298473775),
                            INT32_C(-1404600737), INT32_C(-1231334921), INT32_C( -238983338), INT32_C( -145797796),
                            INT32_C( -181019162), INT32_C(-1910480170), INT32_C(-1860760170), INT32_C( -371855625)),
      UINT16_C(38914),
      simde_mm512_set_epi32(INT32_C( 1533151625), INT32_C( 2122196136), INT32_C( 1690360675), INT32_C( 1484935627),
                            INT32_C( 1463758672), INT32_C(  602211615), INT32_C( -464964305), INT32_C(-1430226195),
                            INT32_C(  797104998), INT32_C(-1557543977), INT32_C( -952737410), INT32_C(  178625368),
                            INT32_C(-1203806300), INT32_C( 1095216728), INT32_C(-1215405554), INT32_C(  430790402)),
      simde_mm512_set_epi32(INT32_C( -251141702), INT32_C( 1274901810), INT32_C(  413860084), INT32_C(  550494320),
                            INT32_C( 1997049765), INT32_C(  505563651), INT32_C(  463125220), INT32_C( -451213519),
                            INT32_C(-1948793453), INT32_C(-2137102362), INT32_C(-1703809327), INT32_C(  389679318),
                            INT32_C( -355192167), INT32_C(-1801602389), INT32_C( 2006619059), INT32_C( -903558132)),
      simde_mm512_set_epi32(INT32_C(         -6), INT32_C( -182128842), INT32_C(  564512596), INT32_C(          2),
                            INT32_C(          0), INT32_C(-1803940708), INT32_C(-1765392929), INT32_C(  298473775),
                            INT32_C(-1404600737), INT32_C(-1231334921), INT32_C( -238983338), INT32_C( -145797796),
                            INT32_C( -181019162), INT32_C(-1910480170), INT32_C(          0), INT32_C( -371855625)) },
    { simde_mm512_set_epi32(INT32_C( 1572579389), INT32_C( -783078337), INT32_C(-1895621282), INT32_C( 1967093325),
                            INT32_C(  908815803), INT32_C(-1975591270), INT32_C( 2065037155), INT32_C(  623932649),
                            INT32_C( 1610322797), INT32_C( -842122991), INT32_C( 2031682359), INT32_C(-1300130353),
                            INT32_C(-1950048210), INT32_C(  238137788), INT32_C( 1978166020), INT32_C(   76768592)),
      UINT16_C(  883),
      simde_mm512_set_epi32(INT32_C(-1010119490), INT32_C( -410070063), INT32_C( 2094036024), INT32_C(-1838133114),
                            INT32_C(   69201629), INT32_C( 1228958503), INT32_C( -775379327), INT32_C(-1485462767),
                            INT32_C(-1179177847), INT32_C( 1767270276), INT32_C(  490610321), INT32_C( 1164436618),
                            INT32_C(-1920297499), INT32_C( -690964678), INT32_C( -880248267), INT32_C(-2005634277)),
      simde_mm512_set_epi32(INT32_C(-1911659531), INT32_C(  143428987), INT32_C( -610024215), INT32_C(  582607980),
                            INT32_C( 1609326889), INT32_C( 1245407235), INT32_C( -119962198), INT32_C(-1932052969),
                            INT32_C(-1370414254), INT32_C(-1925960308), INT32_C( 2119408419), INT32_C(-1203088886),
                            INT32_C( -316530353), INT32_C( 1708684203), INT32_C( 1202455481), INT32_C(-2107221827)),
      simde_mm512_set_epi32(INT32_C( 1572579389), INT32_C( -783078337), INT32_C(-1895621282), INT32_C( 1967093325),
                            INT32_C(  908815803), INT32_C(-1975591270), INT32_C(          6), INT32_C(          0),
                            INT32_C( 1610322797), INT32_C(          0), INT32_C(          0), INT32_C(          0),
                            INT32_C(-1950048210), INT32_C(  238137788), INT32_C(          0), INT32_C(          0)) },
    { simde_mm512_set_epi32(INT32_C( 2117071873), INT32_C(-1437889529), INT32_C( -376074104), INT32_C( 1087893388),
                            INT32_C( -443183285), INT32_C( -380695552), INT32_C(  565328458), INT32_C(  -93024748),
                            INT32_C( 1480532604), INT32_C(  -97460760), INT32_C( -582247600), INT32_C( -374749470),
                            INT32_C( 1394313506), INT32_C(  394553965), INT32_C(-2016714120), INT32_C( 1697927724)),
      UINT16_C(12254),
      simde_mm512_set_epi32(INT32_C(   56443211), INT32_C(-2036514643), INT32_C( -510270824), INT32_C( 1139427205),
                            INT32_C( 1090384090), INT32_C(-1905231405), INT32_C(-2079359983), INT32_C( -477294891),
                            INT32_C( -673197028), INT32_C( 2071747620), INT32_C( -442789099), INT32_C( -601334711),
                            INT32_C(  319530416), INT32_C(-2115012481), INT32_C( -501730903), INT32_C(  340519338)),
      simde_mm512_set_epi32(INT32_C( 1219537084), INT32_C( 1349635715), INT32_C(  732887738), INT32_C(-1728641921),
                            INT32_C(-1388433411), INT32_C( 1765754685), INT32_C(-1574983663), INT32_C(  846129112),
                            INT32_C( 1578410935), INT32_C(-1659872458), INT32_C( 1045536663), INT32_C(  957117985),
                            INT32_C(-1265958651), INT32_C( 1309498779), INT32_C(-1001015299), INT32_C( 1022360677)),
      simde_mm512_set_epi32(INT32_C( 2117071873), INT32_C(-1437889529), INT32_C(          0), INT32_C( 1087893388),
                            INT32_C(          0), INT32_C(         -1), INT32_C(          1), INT32_C(          0),
                            INT32_C(          0), INT32_C(         -1), INT32_C( -582247600), INT32_C(          0),
                            INT32_C(          0), INT32_C(         -1), INT32_C(          0), INT32_C( 1697927724)) },
    { simde_mm512_set_epi32(INT32_C( -304885978), INT32_C(  991545752), INT32_C( -143034937), INT32_C(  843112042),
                            INT32_C( -227554783), INT32_C( 2124182542), INT32_C(-1526246088), INT32_C(-1991977382),
                            INT32_C( 1224533822), INT32_C( -819361196), INT32_C( -684010252), INT32_C(-1738921185),
                            INT32_C(-1259570772), INT32_C( -691865929), INT32_C( -973523371), INT32_C(   45581573)),
      UINT16_C(42669),
      simde_mm512_set_epi32(INT32_C( -156799603), INT32_C(-1073012339), INT32_C(-2130532125), INT32_C(  397240391),
                            INT32_C(  200936922), INT32_C(-1030980309), INT32_C(-1758363174), INT32_C( -665586367),
                            INT32_C(  453331046), INT32_C( 1704580573), INT32_C( 1606190487), INT32_C(-1085658047),
                            INT32_C(-1335469644), INT32_C( -368070561), INT32_C(-1419559633), INT32_C( 2069966669)),
      simde_mm512_set_epi32(INT32_C( 1379668640), INT32_C(   66581512), INT32_C( -557301797), INT32_C(  304428974),
                            INT32_C(-1608262788), INT32_C(  532978979), INT32_C(  946958552), INT32_C(-1911324669),
                            INT32_C(-2118093156), INT32_C(  283691898), INT32_C( -446072631), INT32_C( -458781294),
                            INT32_C( 1951055651), INT32_C(  765387914), INT32_C(  822559116), INT32_C(    7445617)),
      simde_mm512_set_epi32(INT32_C(          0), INT32_C(  991545752), INT32_C(          3), INT32_C(  843112042),
                            INT32_C( -227554783), INT32_C(         -1), INT32_C(         -1), INT32_C(-1991977382),
                            INT32_C(          0), INT32_C( -819361196), INT32_C(         -3), INT32_C(-1738921185),
                            INT32_C(          0), INT32_C(          0), INT32_C( -973523371), INT32_C(        278)) },
    { simde_mm512_set_epi32(INT32_C(-1981938926), INT32_C(  869237081), INT32_C( -190053534), INT32_C(-1469275330),
                            INT32_C( -717100794), INT32_C(-1303072888), INT32_C(-2122918671), INT32_C( 1617119933),
                            INT32_C( 1521363431), INT32_C(  553638116), INT32_C( 1036201367), INT32_C(-1187933851),
                            INT32_C( -412155886), INT32_C( -760582943), INT32_C( -423751457), INT32_C( 1273589632)),
      UINT16_C(35103),
      simde_mm512_set_epi32(INT32_C(-1836595644), INT32_C(  260676470), INT32_C( 1724614860), INT32_C( -144514633),
                            INT32_C( -478630580), INT32_C(-2086755061), INT32_C(  932145867), INT32_C(-1862372735),
                            INT32_C( 1756892633), INT32_C(  382632965), INT32_C( 1295078740), INT32_C( -995802034),
                            INT32_C(  152308919), INT32_C( -351555508), INT32_C(   31813624), INT32_C(  807463845)),
      simde_mm512_set_epi32(INT32_C(  615301803), INT32_C(  382786341), INT32_C( 1852603705), INT32_C( 1998007730),
                            INT32_C(  231325888), INT32_C( 1842039329), INT32_C(  968682756), INT32_C(  316335394),
                            INT32_C(-2071382094), INT32_C( -803185337), INT32_C(-2126995500), INT32_C( 1587647099),
                            INT32_C(-1328358584), INT32_C(  320339033), INT32_C(  282380179), INT32_C( -108102092)),
      simde_mm512_set_epi32(INT32_C(         -2), INT32_C(  869237081), INT32_C( -190053534), INT32_C(-1469275330),
                            INT32_C(         -2), INT32_C(-1303072888), INT32_C(-2122918671), INT32_C(         -5),
                            INT32_C( 1521363431), INT32_C(  553638116), INT32_C( 1036201367), INT32_C(          0),
                            INT32_C(          0), INT32_C(         -1), INT32_C(          0), INT32_C(         -7)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_mask_div_epi32(test_vec[i].src, test_vec[i].k, test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_div_epi64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi64(INT64_C(-7120494377185439159), INT64_C( 5095015079852768951),
                            INT64_C( -719755322986504865), INT64_C( 1195398499335632561),
                            INT64_C( 4232475372952240435), INT64_C(-1117570177728981140),
                            INT64_C(-4721763859644106046), INT64_C( 6636524825657073074)),
      simde_mm512_set_epi64(INT64_C( 6283111750805844985), INT64_C(-7772496718970349305),
                            INT64_C(-6967007030435791671), INT64_C( 2761331052478409707),
                            INT64_C(-5439727342880208313), INT64_C(-6280010522852202514),
                            INT64_C(-2361957704355445009), INT64_C(-3413538286934776973)),
      simde_mm512_set_epi64(INT64_C(                  -1), INT64_C(                   0),
                            INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                   1), INT64_C(                  -1)) },
    { simde_mm512_set_epi64(INT64_C( 7047516970419020428), INT64_C( 2488576598769637001),
                            INT64_C( 4233591199077735008), INT64_C( 1735409980007662056),
                            INT64_C(-2964306467966319268), INT64_C(-6472988581173317799),
                            INT64_C( 1870256929123231698), INT64_C(-5453281473672019922)),
      simde_mm512_set_epi64(INT64_C(-6026337221937727695), INT64_C( 8654798725117969005),
                            INT64_C(  743584473088107844), INT64_C( 5114866458456107677),
                            INT64_C( 1917095392115883075), INT64_C( 8815346252210924017),
                            INT64_C(-1666651333186431127), INT64_C( 4973081304470687258)),
      simde_mm512_set_epi64(INT64_C(                  -1), INT64_C(                   0),
                            INT64_C(                   5), INT64_C(                   0),
                            INT64_C(                  -1), INT64_C(                   0),
                            INT64_C(                  -1), INT64_C(                  -1)) },
    { simde_mm512_set_epi64(INT64_C(-1433819957247000466), INT64_C(-7270540428235491436),
                            INT64_C( 3506767658669433751), INT64_C(-6269164040512613371),
                            INT64_C(-2703740818469134807), INT64_C( 3442758576787517783),
                            INT64_C(-4507715808807193748), INT64_C( 4997387685805642122)),
      simde_mm512_set_epi64(INT64_C(-3375611624029359751), INT64_C(  155579560497872257),
                            INT64_C( 4346579001240147982), INT64_C( 8478054430600792515),
                            INT64_C( 7917529543412977905), INT64_C( 6077094839460323156),
                            INT64_C(-3234198817213444484), INT64_C( 5455426772165090925)),
      simde_mm512_set_epi64(INT64_C(                   0), INT64_C(                 -46),
                            INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                   1), INT64_C(                   0)) },
    { simde_mm512_set_epi64(INT64_C( 5060007040297057440), INT64_C(-6547486212696877775),
                            INT64_C( 4083773956347780040), INT64_C(-7582952476466356489),
                            INT64_C( -533799245190218148), INT64_C( 6528011672062484486),
                            INT64_C( 8505594160370567764), INT64_C(-7955306051941505966)),
      simde_mm512_set_epi64(INT64_C( 8381795236484256749), INT64_C(-8094121819208130597),
                            INT64_C(-4463810942012697177), INT64_C( 1695569373680370472),
                            INT64_C( 6457800057248167752), INT64_C( 2509734679188915375),
                            INT64_C(-1817858424181439867), INT64_C(-1140679629593449988)),
      simde_mm512_set_epi64(INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                   0), INT64_C(                  -4),
                            INT64_C(                   0), INT64_C(                   2),
                            INT64_C(                  -4), INT64_C(                   6)) },
    { simde_mm512_set_epi64(INT64_C(-3727073512330556719), INT64_C( 1145199535931310009),
                            INT64_C( 6618746106828964781), INT64_C( -318594899546127361),
                            INT64_C(-8348228873903822999), INT64_C( 6522300981577637255),
                            INT64_C(-2123306667443487570), INT64_C(-4210181406724347525)),
      simde_mm512_set_epi64(INT64_C(-5833250200550208329), INT64_C( 8217300129052611844),
                            INT64_C( -649664904511148711), INT64_C( 3231016623164402124),
                            INT64_C( 8024018119100712605), INT64_C( 4306653136982574157),
                            INT64_C(-5380031023357226466), INT64_C( 2544237471105729967)),
      simde_mm512_set_epi64(INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                 -10), INT64_C(                   0),
                            INT64_C(                  -1), INT64_C(                   1),
                            INT64_C(                   0), INT64_C(                  -1)) },
    { simde_mm512_set_epi64(INT64_C(-6427790700478275098), INT64_C(-3168480089241839861),
                            INT64_C(-5000559488767708993), INT64_C( 2755885615249137538),
                            INT64_C( -821966059249139816), INT64_C( 1089871025732147351),
                            INT64_C( 4566772594003817295), INT64_C(-9114574651084812253)),
      simde_mm512_set_epi64(INT64_C( 1778890864282373370), INT64_C( 5911759041868723302),
                            INT64_C( 4553617065988887085), INT64_C( -523178035921802922),
                            INT64_C( 8875040781716651384), INT64_C( 2040058868339841473),
                            INT64_C(-2732208005963885166), INT64_C(-4435516374878659804)),
      simde_mm512_set_epi64(INT64_C(                  -3), INT64_C(                   0),
                            INT64_C(                  -1), INT64_C(                  -5),
                            INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                  -1), INT64_C(                   2)) },
    { simde_mm512_set_epi64(INT64_C(  423237589908350744), INT64_C( 2795901596537384901),
                            INT64_C( 1719109459006160254), INT64_C(-9093479824318774446),
                            INT64_C(-4511267031708830231), INT64_C(-3402553166296368495),
                            INT64_C( 1216620777318406949), INT64_C( -836102980820378689)),
      simde_mm512_set_epi64(INT64_C( 7782115963838117574), INT64_C(-6846698536887599933),
                            INT64_C( 4072223690207540333), INT64_C(-1026965696159348843),
                            INT64_C( 4340400659569160523), INT64_C(-8299269241811916492),
                            INT64_C( 7360887374546597504), INT64_C(-6651085920823128052)),
      simde_mm512_set_epi64(INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                   0), INT64_C(                   8),
                            INT64_C(                  -1), INT64_C(                   0),
                            INT64_C(                   0), INT64_C(                   0)) },
    { simde_mm512_set_epi64(INT64_C(  453211281016332666), INT64_C( 5434252921191502101),
                            INT64_C(-6060319301844209563), INT64_C(-5254139409542070482),
                            INT64_C(-8624885551201065882), INT64_C( 8329149627836272144),
                            INT64_C( 8516875663163240125), INT64_C(-4575460702098419673)),
      simde_mm512_set_epi64(INT64_C(-5051260979279221837), INT64_C( 6222948671724306809),
                            INT64_C( 6742741209152957138), INT64_C( 5958951964162816685),
                            INT64_C( 2981515940173974322), INT64_C( 3752367916961311345),
                            INT64_C(-2840979297342041250), INT64_C(-2506264265844715430)),
      simde_mm512_set_epi64(INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                   0), INT64_C(                   0),
                            INT64_C(                  -2), INT64_C(                   2),
                            INT64_C(                  -2), INT64_C(                   1)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_div_epi64(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_div_epu8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu8(UINT8_C( 41), UINT8_C( 49), UINT8_C(171), UINT8_C(198),
                             UINT8_C( 40), UINT8_C( 44), UINT8_C(242), UINT8_C( 51),
                             UINT8_C(138), UINT8_C(217), UINT8_C(215), UINT8_C(249),
                             UINT8_C(201), UINT8_C( 37), UINT8_C(137), UINT8_C( 29),
                             UINT8_C(233), UINT8_C(170), UINT8_C(241), UINT8_C(126),
                             UINT8_C(182), UINT8_C( 10), UINT8_C(208), UINT8_C(198),
                             UINT8_C( 93), UINT8_C(130), UINT8_C(195), UINT8_C(177),
                             UINT8_C(187), UINT8_C(223), UINT8_C(139), UINT8_C(253),
                             UINT8_C(191), UINT8_C(167), UINT8_C(226), UINT8_C( 64),
                             UINT8_C(213), UINT8_C(202), UINT8_C(110), UINT8_C(113),
                             UINT8_C( 89), UINT8_C(237), UINT8_C( 70), UINT8_C(226),
                             UINT8_C(132), UINT8_C( 91), UINT8_C(255), UINT8_C( 88),
                             UINT8_C(104), UINT8_C( 42), UINT8_C( 53), UINT8_C(254),
                             UINT8_C(132), UINT8_C(254), UINT8_C( 96), UINT8_C( 75),
                             UINT8_C( 31), UINT8_C(112), UINT8_C(151), UINT8_C(169),
                             UINT8_C(172), UINT8_C( 94), UINT8_C(112), UINT8_C( 90)),
      simde_x_mm512_set_epu8(UINT8_C(195), UINT8_C( 49), UINT8_C( 14), UINT8_C(170),
                             UINT8_C(203), UINT8_C(167), UINT8_C(  3), UINT8_C(215),
                             UINT8_C( 63), UINT8_C(248), UINT8_C( 55), UINT8_C(219),
                             UINT8_C(221), UINT8_C(135), UINT8_C( 61), UINT8_C(191),
                             UINT8_C(209), UINT8_C( 91), UINT8_C( 87), UINT8_C(137),
                             UINT8_C( 87), UINT8_C( 76), UINT8_C( 44), UINT8_C(140),
                             UINT8_C(  2), UINT8_C(200), UINT8_C( 36), UINT8_C(195),
                             UINT8_C(200), UINT8_C(125), UINT8_C(254), UINT8_C(139),
                             UINT8_C(226), UINT8_C( 71), UINT8_C( 92), UINT8_C(129),
                             UINT8_C(182), UINT8_C(119), UINT8_C(247), UINT8_C( 34),
                             UINT8_C(121), UINT8_C( 85), UINT8_C(153), UINT8_C(116),
                             UINT8_C(218), UINT8_C( 21), UINT8_C(101), UINT8_C(122),
                             UINT8_C( 10), UINT8_C(231), UINT8_C( 54), UINT8_C( 71),
                             UINT8_C(156), UINT8_C(149), UINT8_C(244), UINT8_C( 84),
                             UINT8_C(148), UINT8_C( 85), UINT8_C(170), UINT8_C(184),
                             UINT8_C( 94), UINT8_C(154), UINT8_C(229), UINT8_C( 11)),
      simde_x_mm512_set_epu8(UINT8_C(  0), UINT8_C(  1), UINT8_C( 12), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C( 80), UINT8_C(  0),
                             UINT8_C(  2), UINT8_C(  0), UINT8_C(  3), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  2), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  1), UINT8_C(  2), UINT8_C(  0),
                             UINT8_C(  2), UINT8_C(  0), UINT8_C(  4), UINT8_C(  1),
                             UINT8_C( 46), UINT8_C(  0), UINT8_C(  5), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  2), UINT8_C(  2), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  1), UINT8_C(  0), UINT8_C(  3),
                             UINT8_C(  0), UINT8_C(  2), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  4), UINT8_C(  2), UINT8_C(  0),
                             UINT8_C( 10), UINT8_C(  0), UINT8_C(  0), UINT8_C(  3),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  0), UINT8_C(  8)) },
    { simde_x_mm512_set_epu8(UINT8_C(216), UINT8_C( 85), UINT8_C(206), UINT8_C(103),
                             UINT8_C(235), UINT8_C(154), UINT8_C(129), UINT8_C(135),
                             UINT8_C(125), UINT8_C( 76), UINT8_C(202), UINT8_C(108),
                             UINT8_C( 52), UINT8_C( 71), UINT8_C(168), UINT8_C(196),
                             UINT8_C( 70), UINT8_C(138), UINT8_C(167), UINT8_C( 65),
                             UINT8_C(221), UINT8_C(161), UINT8_C(157), UINT8_C( 93),
                             UINT8_C(192), UINT8_C(189), UINT8_C(153), UINT8_C(155),
                             UINT8_C(207), UINT8_C(213), UINT8_C(105), UINT8_C(136),
                             UINT8_C(234), UINT8_C( 94), UINT8_C(240), UINT8_C( 12),
                             UINT8_C(146), UINT8_C(  1), UINT8_C(147), UINT8_C( 59),
                             UINT8_C(253), UINT8_C( 26), UINT8_C( 26), UINT8_C( 40),
                             UINT8_C( 12), UINT8_C(  2), UINT8_C(230), UINT8_C(145),
                             UINT8_C(170), UINT8_C(105), UINT8_C(111), UINT8_C(160),
                             UINT8_C(140), UINT8_C(202), UINT8_C(166), UINT8_C(220),
                             UINT8_C(187), UINT8_C( 65), UINT8_C(250), UINT8_C(195),
                             UINT8_C( 33), UINT8_C(131), UINT8_C(  2), UINT8_C(164)),
      simde_x_mm512_set_epu8(UINT8_C(120), UINT8_C(127), UINT8_C( 28), UINT8_C( 95),
                             UINT8_C(175), UINT8_C(223), UINT8_C(119), UINT8_C(214),
                             UINT8_C(220), UINT8_C(102), UINT8_C( 86), UINT8_C( 22),
                             UINT8_C(119), UINT8_C(207), UINT8_C( 12), UINT8_C(183),
                             UINT8_C(172), UINT8_C(242), UINT8_C(173), UINT8_C(249),
                             UINT8_C( 52), UINT8_C(108), UINT8_C(128), UINT8_C(203),
                             UINT8_C( 85), UINT8_C(135), UINT8_C(227), UINT8_C( 35),
                             UINT8_C(187), UINT8_C( 24), UINT8_C(250), UINT8_C(219),
                             UINT8_C(253), UINT8_C( 62), UINT8_C(125), UINT8_C(236),
                             UINT8_C( 75), UINT8_C( 13), UINT8_C( 79), UINT8_C( 81),
                             UINT8_C(177), UINT8_C(221), UINT8_C(251), UINT8_C(181),
                             UINT8_C(159), UINT8_C(182), UINT8_C( 11), UINT8_C( 11),
                             UINT8_C( 39), UINT8_C( 37), UINT8_C( 39), UINT8_C(208),
                             UINT8_C(136), UINT8_C(180), UINT8_C(215), UINT8_C(139),
                             UINT8_C(144), UINT8_C(128), UINT8_C(203), UINT8_C(206),
                             UINT8_C(173), UINT8_C( 36), UINT8_C(133), UINT8_C(175)),
      simde_x_mm512_set_epu8(UINT8_C(  1), UINT8_C(  0), UINT8_C(  7), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  2), UINT8_C(  4),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C( 14), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  4), UINT8_C(  1), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  2), UINT8_C(  1), UINT8_C(  0), UINT8_C(  4),
                             UINT8_C(  1), UINT8_C(  8), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C( 20), UINT8_C( 13),
                             UINT8_C(  4), UINT8_C(  2), UINT8_C(  2), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  1), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  3), UINT8_C(  0), UINT8_C(  0)) },
    { simde_x_mm512_set_epu8(UINT8_C( 87), UINT8_C( 63), UINT8_C( 47), UINT8_C( 80),
                             UINT8_C( 35), UINT8_C(229), UINT8_C(  5), UINT8_C( 31),
                             UINT8_C(228), UINT8_C( 73), UINT8_C( 53), UINT8_C( 47),
                             UINT8_C(170), UINT8_C(192), UINT8_C(122), UINT8_C(237),
                             UINT8_C( 47), UINT8_C(130), UINT8_C(219), UINT8_C(102),
                             UINT8_C(163), UINT8_C( 41), UINT8_C(195), UINT8_C(215),
                             UINT8_C(199), UINT8_C( 54), UINT8_C( 97), UINT8_C(126),
                             UINT8_C( 10), UINT8_C(165), UINT8_C(155), UINT8_C( 88),
                             UINT8_C(184), UINT8_C( 63), UINT8_C( 95), UINT8_C(164),
                             UINT8_C( 65), UINT8_C( 71), UINT8_C(174), UINT8_C( 88),
                             UINT8_C(183), UINT8_C(142), UINT8_C( 98), UINT8_C( 14),
                             UINT8_C( 25), UINT8_C(173), UINT8_C( 87), UINT8_C(  2),
                             UINT8_C(191), UINT8_C(143), UINT8_C(152), UINT8_C(  2),
                             UINT8_C(126), UINT8_C(  0), UINT8_C(162), UINT8_C( 57),
                             UINT8_C(245), UINT8_C( 36), UINT8_C(239), UINT8_C( 54),
                             UINT8_C( 33), UINT8_C(165), UINT8_C(199), UINT8_C( 84)),
      simde_x_mm512_set_epu8(UINT8_C(131), UINT8_C( 42), UINT8_C(151), UINT8_C(210),
                             UINT8_C( 12), UINT8_C(163), UINT8_C(138), UINT8_C(207),
                             UINT8_C( 43), UINT8_C( 57), UINT8_C( 61), UINT8_C( 62),
                             UINT8_C( 81), UINT8_C(184), UINT8_C(  6), UINT8_C( 93),
                             UINT8_C(167), UINT8_C(  1), UINT8_C(145), UINT8_C(  9),
                             UINT8_C(  4), UINT8_C( 17), UINT8_C( 10), UINT8_C(101),
                             UINT8_C(186), UINT8_C(181), UINT8_C(155), UINT8_C(243),
                             UINT8_C(189), UINT8_C(191), UINT8_C(222), UINT8_C(205),
                             UINT8_C( 59), UINT8_C( 26), UINT8_C(227), UINT8_C(105),
                             UINT8_C(237), UINT8_C(145), UINT8_C(183), UINT8_C( 79),
                             UINT8_C(174), UINT8_C( 60), UINT8_C(132), UINT8_C(208),
                             UINT8_C( 58), UINT8_C(178), UINT8_C(116), UINT8_C(240),
                             UINT8_C( 37), UINT8_C(131), UINT8_C(100), UINT8_C(177),
                             UINT8_C( 19), UINT8_C(102), UINT8_C( 81), UINT8_C( 86),
                             UINT8_C( 25), UINT8_C( 43), UINT8_C( 51), UINT8_C(140),
                             UINT8_C(  9), UINT8_C( 40), UINT8_C(227), UINT8_C( 75)),
      simde_x_mm512_set_epu8(UINT8_C(  0), UINT8_C(  1), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  2), UINT8_C(  1), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  5), UINT8_C(  1), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  2), UINT8_C(  1), UINT8_C( 20), UINT8_C(  2),
                             UINT8_C(  0), UINT8_C(130), UINT8_C(  1), UINT8_C( 11),
                             UINT8_C( 40), UINT8_C(  2), UINT8_C( 19), UINT8_C(  2),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  3), UINT8_C(  2), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  2), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  5), UINT8_C(  1), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  6), UINT8_C(  0), UINT8_C(  2), UINT8_C(  0),
                             UINT8_C(  9), UINT8_C(  0), UINT8_C(  4), UINT8_C(  0),
                             UINT8_C(  3), UINT8_C(  4), UINT8_C(  0), UINT8_C(  1)) },
    { simde_x_mm512_set_epu8(UINT8_C(233), UINT8_C( 79), UINT8_C( 12), UINT8_C(  0),
                             UINT8_C( 33), UINT8_C(178), UINT8_C( 58), UINT8_C( 74),
                             UINT8_C(250), UINT8_C(116), UINT8_C(142), UINT8_C( 20),
                             UINT8_C( 88), UINT8_C( 63), UINT8_C( 34), UINT8_C(124),
                             UINT8_C(250), UINT8_C( 48), UINT8_C(221), UINT8_C(232),
                             UINT8_C(221), UINT8_C( 75), UINT8_C(155), UINT8_C( 80),
                             UINT8_C(233), UINT8_C(169), UINT8_C(198), UINT8_C(226),
                             UINT8_C( 83), UINT8_C( 27), UINT8_C(137), UINT8_C( 34),
                             UINT8_C( 23), UINT8_C(132), UINT8_C(106), UINT8_C(109),
                             UINT8_C(135), UINT8_C(203), UINT8_C( 98), UINT8_C(120),
                             UINT8_C(101), UINT8_C( 52), UINT8_C( 82), UINT8_C( 44),
                             UINT8_C(142), UINT8_C( 14), UINT8_C( 99), UINT8_C(245),
                             UINT8_C(  8), UINT8_C(140), UINT8_C(141), UINT8_C(123),
                             UINT8_C(219), UINT8_C(163), UINT8_C(196), UINT8_C(233),
                             UINT8_C( 34), UINT8_C(185), UINT8_C(228), UINT8_C(108),
                             UINT8_C( 95), UINT8_C(236), UINT8_C( 97), UINT8_C( 41)),
      simde_x_mm512_set_epu8(UINT8_C(193), UINT8_C(230), UINT8_C( 93), UINT8_C( 23),
                             UINT8_C(193), UINT8_C( 52), UINT8_C(223), UINT8_C(175),
                             UINT8_C(205), UINT8_C( 45), UINT8_C(166), UINT8_C( 24),
                             UINT8_C( 71), UINT8_C(234), UINT8_C(161), UINT8_C(142),
                             UINT8_C(184), UINT8_C(218), UINT8_C(190), UINT8_C(212),
                             UINT8_C(116), UINT8_C(159), UINT8_C( 44), UINT8_C( 55),
                             UINT8_C(213), UINT8_C(133), UINT8_C( 60), UINT8_C(  3),
                             UINT8_C( 58), UINT8_C(255), UINT8_C(125), UINT8_C(189),
                             UINT8_C(145), UINT8_C( 88), UINT8_C( 55), UINT8_C(182),
                             UINT8_C( 23), UINT8_C(161), UINT8_C(133), UINT8_C( 27),
                             UINT8_C(125), UINT8_C(229), UINT8_C(203), UINT8_C( 45),
                             UINT8_C( 24), UINT8_C(  5), UINT8_C( 90), UINT8_C( 83),
                             UINT8_C(145), UINT8_C( 85), UINT8_C(156), UINT8_C(164),
                             UINT8_C(149), UINT8_C(201), UINT8_C( 48), UINT8_C(255),
                             UINT8_C( 41), UINT8_C( 42), UINT8_C( 94), UINT8_C(129),
                             UINT8_C(135), UINT8_C(  8), UINT8_C( 12), UINT8_C(203)),
      simde_x_mm512_set_epu8(UINT8_C(  1), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  3), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  2), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  1), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  3), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  1), UINT8_C(  3), UINT8_C( 75),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  5), UINT8_C(  1), UINT8_C(  0), UINT8_C(  4),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  5), UINT8_C(  2), UINT8_C(  1), UINT8_C(  2),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  4), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  4), UINT8_C(  2), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C( 29), UINT8_C(  8), UINT8_C(  0)) },
    { simde_x_mm512_set_epu8(UINT8_C(142), UINT8_C( 19), UINT8_C(128), UINT8_C(  3),
                             UINT8_C(129), UINT8_C(192), UINT8_C(118), UINT8_C(156),
                             UINT8_C( 16), UINT8_C(232), UINT8_C(203), UINT8_C(122),
                             UINT8_C(229), UINT8_C(105), UINT8_C(120), UINT8_C(201),
                             UINT8_C(228), UINT8_C(167), UINT8_C(141), UINT8_C(146),
                             UINT8_C(116), UINT8_C( 74), UINT8_C(191), UINT8_C( 35),
                             UINT8_C( 45), UINT8_C(158), UINT8_C(228), UINT8_C(138),
                             UINT8_C( 49), UINT8_C(  7), UINT8_C( 65), UINT8_C(140),
                             UINT8_C(  0), UINT8_C(113), UINT8_C(156), UINT8_C(113),
                             UINT8_C(246), UINT8_C(167), UINT8_C(109), UINT8_C(141),
                             UINT8_C(192), UINT8_C( 11), UINT8_C( 33), UINT8_C(141),
                             UINT8_C(129), UINT8_C(  2), UINT8_C(168), UINT8_C(227),
                             UINT8_C( 23), UINT8_C(173), UINT8_C(104), UINT8_C( 71),
                             UINT8_C( 11), UINT8_C(250), UINT8_C( 13), UINT8_C(218),
                             UINT8_C(194), UINT8_C(140), UINT8_C(125), UINT8_C( 43),
                             UINT8_C(151), UINT8_C( 49), UINT8_C(129), UINT8_C(218)),
      simde_x_mm512_set_epu8(UINT8_C(  8), UINT8_C( 25), UINT8_C(147), UINT8_C(220),
                             UINT8_C(173), UINT8_C(138), UINT8_C( 38), UINT8_C(150),
                             UINT8_C( 35), UINT8_C( 43), UINT8_C(165), UINT8_C(185),
                             UINT8_C( 50), UINT8_C( 64), UINT8_C(161), UINT8_C(132),
                             UINT8_C(162), UINT8_C( 50), UINT8_C(199), UINT8_C( 84),
                             UINT8_C(251), UINT8_C(200), UINT8_C(217), UINT8_C( 19),
                             UINT8_C(180), UINT8_C(196), UINT8_C(246), UINT8_C( 76),
                             UINT8_C( 55), UINT8_C(204), UINT8_C(139), UINT8_C( 75),
                             UINT8_C(  1), UINT8_C( 89), UINT8_C(133), UINT8_C(212),
                             UINT8_C(206), UINT8_C( 55), UINT8_C(204), UINT8_C(120),
                             UINT8_C( 37), UINT8_C(159), UINT8_C(146), UINT8_C(217),
                             UINT8_C(226), UINT8_C(190), UINT8_C(134), UINT8_C(  8),
                             UINT8_C(113), UINT8_C( 61), UINT8_C(103), UINT8_C(100),
                             UINT8_C( 23), UINT8_C(229), UINT8_C(146), UINT8_C( 97),
                             UINT8_C( 95), UINT8_C( 32), UINT8_C(136), UINT8_C( 91),
                             UINT8_C( 46), UINT8_C(252), UINT8_C(163), UINT8_C( 88)),
      simde_x_mm512_set_epu8(UINT8_C( 17), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  3), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  5), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  4), UINT8_C(  1), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  3), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  3), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  5), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  1), UINT8_C( 28),
                             UINT8_C(  0), UINT8_C(  2), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  0), UINT8_C(  2),
                             UINT8_C(  2), UINT8_C(  4), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  3), UINT8_C(  0), UINT8_C(  0), UINT8_C(  2)) },
    { simde_x_mm512_set_epu8(UINT8_C( 46), UINT8_C( 43), UINT8_C(246), UINT8_C(157),
                             UINT8_C( 80), UINT8_C(154), UINT8_C( 27), UINT8_C(118),
                             UINT8_C(176), UINT8_C(216), UINT8_C( 46), UINT8_C(142),
                             UINT8_C(198), UINT8_C(248), UINT8_C( 88), UINT8_C( 29),
                             UINT8_C(176), UINT8_C( 25), UINT8_C(101), UINT8_C( 54),
                             UINT8_C(103), UINT8_C(120), UINT8_C( 94), UINT8_C( 16),
                             UINT8_C(197), UINT8_C(205), UINT8_C( 71), UINT8_C(246),
                             UINT8_C(158), UINT8_C(176), UINT8_C(218), UINT8_C( 43),
                             UINT8_C(235), UINT8_C(249), UINT8_C(116), UINT8_C(137),
                             UINT8_C( 89), UINT8_C(212), UINT8_C(132), UINT8_C( 56),
                             UINT8_C(230), UINT8_C(137), UINT8_C( 66), UINT8_C( 41),
                             UINT8_C( 44), UINT8_C( 35), UINT8_C(189), UINT8_C(155),
                             UINT8_C(125), UINT8_C(130), UINT8_C(123), UINT8_C(117),
                             UINT8_C(123), UINT8_C(127), UINT8_C(151), UINT8_C( 60),
                             UINT8_C(153), UINT8_C(185), UINT8_C(250), UINT8_C(100),
                             UINT8_C( 83), UINT8_C(112), UINT8_C( 33), UINT8_C(140)),
      simde_x_mm512_set_epu8(UINT8_C( 36), UINT8_C( 33), UINT8_C( 42), UINT8_C( 75),
                             UINT8_C(179), UINT8_C(172), UINT8_C(126), UINT8_C(171),
                             UINT8_C(110), UINT8_C(150), UINT8_C(107), UINT8_C(180),
                             UINT8_C(134), UINT8_C( 73), UINT8_C(207), UINT8_C( 15),
                             UINT8_C(241), UINT8_C(103), UINT8_C(103), UINT8_C(150),
                             UINT8_C(103), UINT8_C( 58), UINT8_C(104), UINT8_C( 35),
                             UINT8_C(249), UINT8_C( 79), UINT8_C(113), UINT8_C( 97),
                             UINT8_C(189), UINT8_C(197), UINT8_C(174), UINT8_C(222),
                             UINT8_C(224), UINT8_C(104), UINT8_C(123), UINT8_C(124),
                             UINT8_C( 49), UINT8_C(226), UINT8_C( 37), UINT8_C( 22),
                             UINT8_C(105), UINT8_C(157), UINT8_C(110), UINT8_C( 52),
                             UINT8_C(254), UINT8_C(103), UINT8_C(162), UINT8_C(210),
                             UINT8_C(202), UINT8_C( 39), UINT8_C(193), UINT8_C(151),
                             UINT8_C(183), UINT8_C( 73), UINT8_C( 97), UINT8_C(187),
                             UINT8_C(102), UINT8_C(195), UINT8_C( 68), UINT8_C(190),
                             UINT8_C( 65), UINT8_C( 60), UINT8_C(165), UINT8_C(126)),
      simde_x_mm512_set_epu8(UINT8_C(  1), UINT8_C(  1), UINT8_C(  5), UINT8_C(  2),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  1), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  3), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  2), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  2), UINT8_C(  0), UINT8_C(  2),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  2), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  3), UINT8_C(  2),
                             UINT8_C(  2), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  3), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  3), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  1), UINT8_C(  0), UINT8_C(  1)) },
    { simde_x_mm512_set_epu8(UINT8_C(240), UINT8_C(169), UINT8_C(  8), UINT8_C( 54),
                             UINT8_C( 66), UINT8_C( 99), UINT8_C( 14), UINT8_C( 32),
                             UINT8_C(148), UINT8_C( 92), UINT8_C(122), UINT8_C(200),
                             UINT8_C(192), UINT8_C(186), UINT8_C(225), UINT8_C( 52),
                             UINT8_C(182), UINT8_C(244), UINT8_C(253), UINT8_C(228),
                             UINT8_C(141), UINT8_C(228), UINT8_C(148), UINT8_C(168),
                             UINT8_C(231), UINT8_C(107), UINT8_C( 47), UINT8_C(205),
                             UINT8_C(126), UINT8_C(  7), UINT8_C(182), UINT8_C(245),
                             UINT8_C(165), UINT8_C(186), UINT8_C(213), UINT8_C( 84),
                             UINT8_C( 19), UINT8_C(131), UINT8_C( 54), UINT8_C( 13),
                             UINT8_C(185), UINT8_C(182), UINT8_C( 72), UINT8_C( 61),
                             UINT8_C(125), UINT8_C(104), UINT8_C(147), UINT8_C( 11),
                             UINT8_C( 89), UINT8_C(204), UINT8_C( 62), UINT8_C(163),
                             UINT8_C(198), UINT8_C(162), UINT8_C(205), UINT8_C(  9),
                             UINT8_C(182), UINT8_C(123), UINT8_C( 65), UINT8_C(208),
                             UINT8_C(145), UINT8_C(179), UINT8_C( 34), UINT8_C(195)),
      simde_x_mm512_set_epu8(UINT8_C(141), UINT8_C(103), UINT8_C(116), UINT8_C( 12),
                             UINT8_C(174), UINT8_C(226), UINT8_C(193), UINT8_C(175),
                             UINT8_C(155), UINT8_C(174), UINT8_C( 73), UINT8_C(  6),
                             UINT8_C(141), UINT8_C(140), UINT8_C(254), UINT8_C(193),
                             UINT8_C(100), UINT8_C(151), UINT8_C( 14), UINT8_C( 19),
                             UINT8_C( 38), UINT8_C(115), UINT8_C(201), UINT8_C(118),
                             UINT8_C( 74), UINT8_C(186), UINT8_C( 89), UINT8_C(183),
                             UINT8_C( 65), UINT8_C(138), UINT8_C( 64), UINT8_C( 90),
                             UINT8_C(152), UINT8_C(241), UINT8_C(229), UINT8_C(218),
                             UINT8_C(126), UINT8_C( 38), UINT8_C(159), UINT8_C( 27),
                             UINT8_C(164), UINT8_C(199), UINT8_C( 25), UINT8_C(253),
                             UINT8_C(181), UINT8_C(104), UINT8_C(  6), UINT8_C(183),
                             UINT8_C( 36), UINT8_C(203), UINT8_C(138), UINT8_C(145),
                             UINT8_C(116), UINT8_C(155), UINT8_C(218), UINT8_C( 24),
                             UINT8_C(205), UINT8_C(238), UINT8_C(242), UINT8_C( 26),
                             UINT8_C(226), UINT8_C( 76), UINT8_C(226), UINT8_C(214)),
      simde_x_mm512_set_epu8(UINT8_C(  1), UINT8_C(  1), UINT8_C(  0), UINT8_C(  4),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  1), UINT8_C( 33),
                             UINT8_C(  1), UINT8_C(  1), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  1), UINT8_C( 18), UINT8_C( 12),
                             UINT8_C(  3), UINT8_C(  1), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  3), UINT8_C(  0), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  2), UINT8_C(  2),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  3), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  2), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C( 24), UINT8_C(  0),
                             UINT8_C(  2), UINT8_C(  1), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  1), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  8),
                             UINT8_C(  0), UINT8_C(  2), UINT8_C(  0), UINT8_C(  0)) },
    { simde_x_mm512_set_epu8(UINT8_C(197), UINT8_C( 52), UINT8_C(145), UINT8_C( 20),
                             UINT8_C( 26), UINT8_C(178), UINT8_C(121), UINT8_C( 16),
                             UINT8_C( 45), UINT8_C(229), UINT8_C( 11), UINT8_C(230),
                             UINT8_C( 53), UINT8_C(  2), UINT8_C(234), UINT8_C(  7),
                             UINT8_C(207), UINT8_C(146), UINT8_C(169), UINT8_C(233),
                             UINT8_C(206), UINT8_C(116), UINT8_C( 55), UINT8_C(156),
                             UINT8_C(180), UINT8_C( 91), UINT8_C( 56), UINT8_C(146),
                             UINT8_C( 55), UINT8_C(137), UINT8_C(200), UINT8_C( 76),
                             UINT8_C( 43), UINT8_C(245), UINT8_C(138), UINT8_C(  3),
                             UINT8_C(213), UINT8_C(156), UINT8_C(166), UINT8_C(234),
                             UINT8_C(199), UINT8_C(  2), UINT8_C( 86), UINT8_C( 72),
                             UINT8_C( 93), UINT8_C(254), UINT8_C(190), UINT8_C(121),
                             UINT8_C(119), UINT8_C( 75), UINT8_C(159), UINT8_C( 76),
                             UINT8_C( 70), UINT8_C(218), UINT8_C( 17), UINT8_C(239),
                             UINT8_C( 43), UINT8_C(152), UINT8_C(222), UINT8_C( 80),
                             UINT8_C(197), UINT8_C(113), UINT8_C(112), UINT8_C( 81)),
      simde_x_mm512_set_epu8(UINT8_C(193), UINT8_C(162), UINT8_C(178), UINT8_C( 36),
                             UINT8_C(178), UINT8_C( 86), UINT8_C( 79), UINT8_C(167),
                             UINT8_C(179), UINT8_C( 45), UINT8_C( 18), UINT8_C(231),
                             UINT8_C(113), UINT8_C(127), UINT8_C(211), UINT8_C(181),
                             UINT8_C(121), UINT8_C(171), UINT8_C( 76), UINT8_C(135),
                             UINT8_C( 15), UINT8_C(133), UINT8_C(247), UINT8_C( 32),
                             UINT8_C(181), UINT8_C(168), UINT8_C(236), UINT8_C( 99),
                             UINT8_C( 85), UINT8_C(151), UINT8_C( 36), UINT8_C( 99),
                             UINT8_C(101), UINT8_C( 42), UINT8_C( 63), UINT8_C( 96),
                             UINT8_C(210), UINT8_C(198), UINT8_C(202), UINT8_C(105),
                             UINT8_C(214), UINT8_C( 74), UINT8_C(199), UINT8_C( 17),
                             UINT8_C(234), UINT8_C( 22), UINT8_C(134), UINT8_C(112),
                             UINT8_C( 62), UINT8_C(141), UINT8_C(156), UINT8_C( 91),
                             UINT8_C( 99), UINT8_C( 24), UINT8_C(198), UINT8_C(131),
                             UINT8_C( 88), UINT8_C(136), UINT8_C( 61), UINT8_C( 94),
                             UINT8_C(189), UINT8_C(213), UINT8_C(249), UINT8_C(131)),
      simde_x_mm512_set_epu8(UINT8_C(  1), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  2), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  5), UINT8_C(  0), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  2), UINT8_C(  1),
                             UINT8_C( 13), UINT8_C(  0), UINT8_C(  0), UINT8_C(  4),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  5), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  5), UINT8_C(  2), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  0), UINT8_C(  2),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(  0), UINT8_C(  4),
                             UINT8_C(  0), UINT8_C( 11), UINT8_C(  1), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C(  0), UINT8_C(  9), UINT8_C(  0), UINT8_C(  1),
                             UINT8_C(  0), UINT8_C(  1), UINT8_C(  3), UINT8_C(  0),
                             UINT8_C(  1), UINT8_C(  0), UINT8_C(  0), UINT8_C(  0)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_div_epu8(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_div_epu16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu16(UINT16_C( 10545), UINT16_C( 43974), UINT16_C( 10284), UINT16_C( 62003),
                              UINT16_C( 35545), UINT16_C( 55289), UINT16_C( 51493), UINT16_C( 35101),
                              UINT16_C( 59818), UINT16_C( 61822), UINT16_C( 46602), UINT16_C( 53446),
                              UINT16_C( 23938), UINT16_C( 50097), UINT16_C( 48095), UINT16_C( 35837),
                              UINT16_C( 49063), UINT16_C( 57920), UINT16_C( 54730), UINT16_C( 28273),
                              UINT16_C( 23021), UINT16_C( 18146), UINT16_C( 33883), UINT16_C( 65368),
                              UINT16_C( 26666), UINT16_C( 13822), UINT16_C( 34046), UINT16_C( 24651),
                              UINT16_C(  8048), UINT16_C( 38825), UINT16_C( 44126), UINT16_C( 28762)),
      simde_x_mm512_set_epu16(UINT16_C( 38607), UINT16_C(  8074), UINT16_C( 18000), UINT16_C( 35687),
                              UINT16_C( 40415), UINT16_C(  3254), UINT16_C( 55282), UINT16_C( 38855),
                              UINT16_C( 41330), UINT16_C( 37148), UINT16_C( 25803), UINT16_C( 25877),
                              UINT16_C(   768), UINT16_C( 16244), UINT16_C( 11114), UINT16_C( 58324),
                              UINT16_C( 18192), UINT16_C( 32532), UINT16_C( 33700), UINT16_C( 60373),
                              UINT16_C( 20183), UINT16_C( 64042), UINT16_C(  2502), UINT16_C( 18488),
                              UINT16_C( 22771), UINT16_C( 21470), UINT16_C(  4556), UINT16_C( 26138),
                              UINT16_C( 19085), UINT16_C( 64613), UINT16_C( 55602), UINT16_C( 63371)),
      simde_x_mm512_set_epu16(UINT16_C(     0), UINT16_C(     5), UINT16_C(     0), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(    16), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     1), UINT16_C(     1), UINT16_C(     1), UINT16_C(     2),
                              UINT16_C(    31), UINT16_C(     3), UINT16_C(     4), UINT16_C(     0),
                              UINT16_C(     2), UINT16_C(     1), UINT16_C(     1), UINT16_C(     0),
                              UINT16_C(     1), UINT16_C(     0), UINT16_C(    13), UINT16_C(     3),
                              UINT16_C(     1), UINT16_C(     0), UINT16_C(     7), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     0)) },
    { simde_x_mm512_set_epu16(UINT16_C( 20057), UINT16_C( 26978), UINT16_C( 45741), UINT16_C( 34503),
                              UINT16_C( 54259), UINT16_C( 41436), UINT16_C( 43883), UINT16_C( 11009),
                              UINT16_C( 50212), UINT16_C(  9014), UINT16_C( 24117), UINT16_C( 34039),
                              UINT16_C( 58348), UINT16_C(  8311), UINT16_C( 31759), UINT16_C(  4002),
                              UINT16_C(  7525), UINT16_C(  3321), UINT16_C( 47299), UINT16_C( 64213),
                              UINT16_C( 13644), UINT16_C( 48153), UINT16_C( 45234), UINT16_C( 51700),
                              UINT16_C(  7513), UINT16_C(  1114), UINT16_C( 65336), UINT16_C( 10389),
                              UINT16_C( 33688), UINT16_C(  9445), UINT16_C( 60332), UINT16_C( 41466)),
      simde_x_mm512_set_epu16(UINT16_C( 48157), UINT16_C( 56913), UINT16_C( 55050), UINT16_C( 48859),
                              UINT16_C( 27895), UINT16_C( 48343), UINT16_C( 59593), UINT16_C( 60425),
                              UINT16_C( 62587), UINT16_C( 54231), UINT16_C( 52444), UINT16_C(  8140),
                              UINT16_C( 58695), UINT16_C(  2476), UINT16_C( 41101), UINT16_C(  7948),
                              UINT16_C( 26094), UINT16_C( 52354), UINT16_C( 30122), UINT16_C( 47688),
                              UINT16_C( 43801), UINT16_C( 57764), UINT16_C(  1809), UINT16_C( 33603),
                              UINT16_C(  8271), UINT16_C(  4936), UINT16_C(  7627), UINT16_C( 20477),
                              UINT16_C( 14608), UINT16_C( 25470), UINT16_C( 45836), UINT16_C( 25611)),
      simde_x_mm512_set_epu16(UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     1), UINT16_C(     0), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     4),
                              UINT16_C(     0), UINT16_C(     3), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     1), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(    25), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     8), UINT16_C(     0),
                              UINT16_C(     2), UINT16_C(     0), UINT16_C(     1), UINT16_C(     1)) },
    { simde_x_mm512_set_epu16(UINT16_C( 26902), UINT16_C( 51011), UINT16_C( 57631), UINT16_C( 57521),
                              UINT16_C( 43405), UINT16_C( 18318), UINT16_C( 44023), UINT16_C(  9770),
                              UINT16_C(  4118), UINT16_C( 33099), UINT16_C(  6621), UINT16_C( 57639),
                              UINT16_C( 22002), UINT16_C( 33155), UINT16_C( 15537), UINT16_C( 38743),
                              UINT16_C( 26466), UINT16_C( 21183), UINT16_C(  5811), UINT16_C( 17016),
                              UINT16_C( 51162), UINT16_C( 46775), UINT16_C( 54252), UINT16_C( 64603),
                              UINT16_C( 30444), UINT16_C( 20573), UINT16_C( 50572), UINT16_C( 25607),
                              UINT16_C( 36721), UINT16_C( 36797), UINT16_C( 27147), UINT16_C( 62271)),
      simde_x_mm512_set_epu16(UINT16_C( 55381), UINT16_C( 52839), UINT16_C( 60314), UINT16_C( 33159),
                              UINT16_C( 32076), UINT16_C( 51820), UINT16_C( 13383), UINT16_C( 43204),
                              UINT16_C( 18058), UINT16_C( 42817), UINT16_C( 56737), UINT16_C( 40285),
                              UINT16_C( 49341), UINT16_C( 39323), UINT16_C( 53205), UINT16_C( 27016),
                              UINT16_C( 59998), UINT16_C( 61452), UINT16_C( 37377), UINT16_C( 37691),
                              UINT16_C( 64794), UINT16_C(  6696), UINT16_C(  3074), UINT16_C( 59025),
                              UINT16_C( 43625), UINT16_C( 28576), UINT16_C( 36042), UINT16_C( 42716),
                              UINT16_C( 47937), UINT16_C( 64195), UINT16_C(  8579), UINT16_C(   676)),
      simde_x_mm512_set_epu16(UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     1),
                              UINT16_C(     1), UINT16_C(     0), UINT16_C(     3), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     6), UINT16_C(    17), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     1), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     3), UINT16_C(    92)) },
    { simde_x_mm512_set_epu16(UINT16_C(  7566), UINT16_C( 25511), UINT16_C( 59705), UINT16_C( 13989),
                              UINT16_C( 13965), UINT16_C( 34471), UINT16_C(    77), UINT16_C( 35152),
                              UINT16_C( 21705), UINT16_C( 42504), UINT16_C( 63033), UINT16_C( 56884),
                              UINT16_C( 42389), UINT16_C( 61527), UINT16_C(  7598), UINT16_C( 23051),
                              UINT16_C( 13886), UINT16_C( 28688), UINT16_C( 30551), UINT16_C( 36608),
                              UINT16_C( 56045), UINT16_C( 38987), UINT16_C( 64798), UINT16_C( 22350),
                              UINT16_C(  7981), UINT16_C( 50477), UINT16_C( 46688), UINT16_C( 16804),
                              UINT16_C( 33660), UINT16_C( 63749), UINT16_C( 29649), UINT16_C( 64815)),
      simde_x_mm512_set_epu16(UINT16_C( 18409), UINT16_C( 19069), UINT16_C( 20979), UINT16_C( 35774),
                              UINT16_C(  8112), UINT16_C( 25085), UINT16_C( 31664), UINT16_C( 55404),
                              UINT16_C( 63329), UINT16_C( 19403), UINT16_C( 33006), UINT16_C( 20365),
                              UINT16_C( 22045), UINT16_C( 41935), UINT16_C( 28665), UINT16_C( 35793),
                              UINT16_C( 26789), UINT16_C( 40241), UINT16_C( 34076), UINT16_C( 36189),
                              UINT16_C( 49507), UINT16_C( 32891), UINT16_C( 45700), UINT16_C( 31541),
                              UINT16_C( 33237), UINT16_C( 50719), UINT16_C( 22782), UINT16_C( 46902),
                              UINT16_C( 62792), UINT16_C(   907), UINT16_C(  9939), UINT16_C(   395)),
      simde_x_mm512_set_epu16(UINT16_C(     0), UINT16_C(     1), UINT16_C(     2), UINT16_C(     0),
                              UINT16_C(     1), UINT16_C(     1), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     2), UINT16_C(     1), UINT16_C(     2),
                              UINT16_C(     1), UINT16_C(     1), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     1),
                              UINT16_C(     1), UINT16_C(     1), UINT16_C(     1), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     2), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(    70), UINT16_C(     2), UINT16_C(   164)) },
    { simde_x_mm512_set_epu16(UINT16_C( 40553), UINT16_C(  9260), UINT16_C(  6846), UINT16_C( 21618),
                              UINT16_C( 20365), UINT16_C( 26413), UINT16_C(  7670), UINT16_C(  6521),
                              UINT16_C( 13052), UINT16_C( 19892), UINT16_C( 40021), UINT16_C( 58092),
                              UINT16_C( 12337), UINT16_C( 14080), UINT16_C(  6934), UINT16_C( 61515),
                              UINT16_C(  1885), UINT16_C( 11733), UINT16_C(  7371), UINT16_C( 24583),
                              UINT16_C( 48349), UINT16_C( 37475), UINT16_C( 47206), UINT16_C( 54691),
                              UINT16_C( 63460), UINT16_C(  2107), UINT16_C( 62169), UINT16_C( 38808),
                              UINT16_C( 21341), UINT16_C( 51834), UINT16_C( 26283), UINT16_C( 38235)),
      simde_x_mm512_set_epu16(UINT16_C(  9227), UINT16_C( 20728), UINT16_C( 22448), UINT16_C( 22271),
                              UINT16_C( 38010), UINT16_C(  3228), UINT16_C( 38598), UINT16_C( 15839),
                              UINT16_C(  4554), UINT16_C( 22831), UINT16_C( 44103), UINT16_C( 32351),
                              UINT16_C( 46747), UINT16_C( 20983), UINT16_C( 61889), UINT16_C( 26454),
                              UINT16_C( 63311), UINT16_C( 19804), UINT16_C( 62773), UINT16_C( 56806),
                              UINT16_C( 36384), UINT16_C( 25302), UINT16_C( 37143), UINT16_C(  3478),
                              UINT16_C( 59861), UINT16_C( 61175), UINT16_C( 48658), UINT16_C( 23119),
                              UINT16_C( 30252), UINT16_C( 63116), UINT16_C( 13170), UINT16_C( 44087)),
      simde_x_mm512_set_epu16(UINT16_C(     4), UINT16_C(     0), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     8), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     2), UINT16_C(     0), UINT16_C(     0), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     2),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     1), UINT16_C(     1), UINT16_C(     1), UINT16_C(    15),
                              UINT16_C(     1), UINT16_C(     0), UINT16_C(     1), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     1), UINT16_C(     0)) },
    { simde_x_mm512_set_epu16(UINT16_C( 22335), UINT16_C( 12112), UINT16_C(  9189), UINT16_C(  1311),
                              UINT16_C( 58441), UINT16_C( 13615), UINT16_C( 43712), UINT16_C( 31469),
                              UINT16_C( 12162), UINT16_C( 56166), UINT16_C( 41769), UINT16_C( 50135),
                              UINT16_C( 50998), UINT16_C( 24958), UINT16_C(  2725), UINT16_C( 39768),
                              UINT16_C( 47167), UINT16_C( 24484), UINT16_C( 16711), UINT16_C( 44632),
                              UINT16_C( 46990), UINT16_C( 25102), UINT16_C(  6573), UINT16_C( 22274),
                              UINT16_C( 49039), UINT16_C( 38914), UINT16_C( 32256), UINT16_C( 41529),
                              UINT16_C( 62756), UINT16_C( 61238), UINT16_C(  8613), UINT16_C( 51028)),
      simde_x_mm512_set_epu16(UINT16_C( 30472), UINT16_C( 36773), UINT16_C(  7714), UINT16_C( 18947),
                              UINT16_C(  7066), UINT16_C( 47844), UINT16_C( 58651), UINT16_C(  1841),
                              UINT16_C( 35799), UINT16_C( 50579), UINT16_C( 32926), UINT16_C( 26598),
                              UINT16_C( 39537), UINT16_C( 61137), UINT16_C(  5946), UINT16_C(  2262),
                              UINT16_C( 60116), UINT16_C( 12953), UINT16_C( 38045), UINT16_C( 47787),
                              UINT16_C( 30618), UINT16_C( 37811), UINT16_C( 51748), UINT16_C( 52236),
                              UINT16_C( 23394), UINT16_C(  2441), UINT16_C( 32382), UINT16_C(  9384),
                              UINT16_C( 25792), UINT16_C( 56163), UINT16_C( 22658), UINT16_C( 20939)),
      simde_x_mm512_set_epu16(UINT16_C(     0), UINT16_C(     0), UINT16_C(     1), UINT16_C(     0),
                              UINT16_C(     8), UINT16_C(     0), UINT16_C(     0), UINT16_C(    17),
                              UINT16_C(     0), UINT16_C(     1), UINT16_C(     1), UINT16_C(     1),
                              UINT16_C(     1), UINT16_C(     0), UINT16_C(     0), UINT16_C(    17),
                              UINT16_C(     0), UINT16_C(     1), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     1), UINT16_C(     0), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     2), UINT16_C(    15), UINT16_C(     0), UINT16_C(     4),
                              UINT16_C(     2), UINT16_C(     1), UINT16_C(     0), UINT16_C(     2)) },
    { simde_x_mm512_set_epu16(UINT16_C( 13867), UINT16_C( 28091), UINT16_C( 35390), UINT16_C( 56986),
                              UINT16_C( 31509), UINT16_C( 63331), UINT16_C(  9520), UINT16_C( 29929),
                              UINT16_C( 24571), UINT16_C( 37741), UINT16_C( 52686), UINT16_C( 14609),
                              UINT16_C( 31001), UINT16_C(   823), UINT16_C( 45697), UINT16_C( 38351),
                              UINT16_C( 35780), UINT16_C( 41006), UINT16_C(  3633), UINT16_C( 45500),
                              UINT16_C( 30184), UINT16_C( 27396), UINT16_C(  1171), UINT16_C( 25936),
                              UINT16_C( 61703), UINT16_C( 57786), UINT16_C( 19453), UINT16_C( 30002),
                              UINT16_C(  6315), UINT16_C(   244), UINT16_C(  8399), UINT16_C( 57456)),
      simde_x_mm512_set_epu16(UINT16_C( 18752), UINT16_C( 27431), UINT16_C( 53704), UINT16_C( 42625),
                              UINT16_C( 42869), UINT16_C( 41745), UINT16_C( 47543), UINT16_C( 11401),
                              UINT16_C( 26966), UINT16_C( 26500), UINT16_C(  7486), UINT16_C(  7825),
                              UINT16_C( 17767), UINT16_C( 58506), UINT16_C( 36234), UINT16_C( 38373),
                              UINT16_C( 54992), UINT16_C( 46906), UINT16_C( 52104), UINT16_C( 31285),
                              UINT16_C( 34932), UINT16_C( 29467), UINT16_C( 33781), UINT16_C(   883),
                              UINT16_C( 23995), UINT16_C( 43069), UINT16_C( 53587), UINT16_C( 11327),
                              UINT16_C( 36611), UINT16_C(  7518), UINT16_C( 30015), UINT16_C( 30285)),
      simde_x_mm512_set_epu16(UINT16_C(     0), UINT16_C(     1), UINT16_C(     0), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     1), UINT16_C(     0), UINT16_C(     2),
                              UINT16_C(     0), UINT16_C(     1), UINT16_C(     7), UINT16_C(     1),
                              UINT16_C(     1), UINT16_C(     0), UINT16_C(     1), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(    29),
                              UINT16_C(     2), UINT16_C(     1), UINT16_C(     0), UINT16_C(     2),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     1)) },
    { simde_x_mm512_set_epu16(UINT16_C( 19003), UINT16_C( 26627), UINT16_C( 63705), UINT16_C( 34218),
                              UINT16_C( 36055), UINT16_C( 13847), UINT16_C( 44625), UINT16_C(  9042),
                              UINT16_C( 36148), UINT16_C( 11660), UINT16_C( 32339), UINT16_C( 39715),
                              UINT16_C( 47178), UINT16_C( 21002), UINT16_C( 60706), UINT16_C(  8527),
                              UINT16_C( 26072), UINT16_C( 29611), UINT16_C( 18348), UINT16_C(   953),
                              UINT16_C( 33382), UINT16_C( 22717), UINT16_C( 50122), UINT16_C( 52414),
                              UINT16_C( 59278), UINT16_C( 54225), UINT16_C( 31952), UINT16_C( 29752),
                              UINT16_C( 37488), UINT16_C( 20614), UINT16_C(  1055), UINT16_C( 61149)),
      simde_x_mm512_set_epu16(UINT16_C( 59727), UINT16_C(  3072), UINT16_C(  8626), UINT16_C( 14922),
                              UINT16_C( 64116), UINT16_C( 36372), UINT16_C( 22591), UINT16_C(  8828),
                              UINT16_C( 64048), UINT16_C( 56808), UINT16_C( 56651), UINT16_C( 39760),
                              UINT16_C( 59817), UINT16_C( 50914), UINT16_C( 21275), UINT16_C( 35106),
                              UINT16_C(  6020), UINT16_C( 27245), UINT16_C( 34763), UINT16_C( 25208),
                              UINT16_C( 25908), UINT16_C( 21036), UINT16_C( 36366), UINT16_C( 25589),
                              UINT16_C(  2188), UINT16_C( 36219), UINT16_C( 56227), UINT16_C( 50409),
                              UINT16_C(  8889), UINT16_C( 58476), UINT16_C( 24556), UINT16_C( 24873)),
      simde_x_mm512_set_epu16(UINT16_C(     0), UINT16_C(     8), UINT16_C(     7), UINT16_C(     2),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     1), UINT16_C(     1),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     0), UINT16_C(     0), UINT16_C(     2), UINT16_C(     0),
                              UINT16_C(     4), UINT16_C(     1), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     1), UINT16_C(     1), UINT16_C(     1), UINT16_C(     2),
                              UINT16_C(    27), UINT16_C(     1), UINT16_C(     0), UINT16_C(     0),
                              UINT16_C(     4), UINT16_C(     0), UINT16_C(     0), UINT16_C(     2)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_div_epu16(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_div_epu32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu32(UINT32_C( 691121094), UINT32_C( 674034227), UINT32_C(2329532409), UINT32_C(3374680349),
                              UINT32_C(3920294270), UINT32_C(3054162118), UINT32_C(1568850865), UINT32_C(3151989757),
                              UINT32_C(3215450688), UINT32_C(3586813553), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C(2891870298)),
      simde_x_mm512_set_epu32(UINT32_C(2530156426), UINT32_C(1179683687), UINT32_C(2648640694), UINT32_C(3623000007),
                              UINT32_C(2708640028), UINT32_C(1691051285), UINT32_C(  50347892), UINT32_C( 728425428),
                              UINT32_C(1192263444), UINT32_C(2208623573), UINT32_C(1322777130), UINT32_C( 163989560),
                              UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(1250819173), UINT32_C(3643996043)),
      simde_x_mm512_set_epu32(UINT32_C(         0), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(         1), UINT32_C(         1), UINT32_C(        31), UINT32_C(         4),
                              UINT32_C(         2), UINT32_C(         1), UINT32_C(         1), UINT32_C(        13),
                              UINT32_C(         1), UINT32_C(         7), UINT32_C(         0), UINT32_C(         0)) },
    { simde_x_mm512_set_epu32(UINT32_C(1314482530), UINT32_C(2997716679), UINT32_C(3555959260), UINT32_C(2875927297),
                              UINT32_C(3290702646), UINT32_C(1580565751), UINT32_C(3823902839), UINT32_C(2081361826),
                              UINT32_C( 493161721), UINT32_C(3099851477), UINT32_C( 894221337), UINT32_C(2964507124),
                              UINT32_C( 492373082), UINT32_C(4281870485), UINT32_C(2207786213), UINT32_C(3953959418)),
      simde_x_mm512_set_epu32(UINT32_C(3156074065), UINT32_C(3607805659), UINT32_C(1828175063), UINT32_C(3905547273),
                              UINT32_C(4101755863), UINT32_C(3436978124), UINT32_C(3846637996), UINT32_C(2693603084),
                              UINT32_C(1710148738), UINT32_C(1974123080), UINT32_C(2870600100), UINT32_C( 118588227),
                              UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358), UINT32_C(3003933707)),
      simde_x_mm512_set_epu32(UINT32_C(         0), UINT32_C(         0), UINT32_C(         1), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(         1), UINT32_C(         0), UINT32_C(        24),
                              UINT32_C(         0), UINT32_C(         8), UINT32_C(         2), UINT32_C(         1)) },
    { simde_x_mm512_set_epu32(UINT32_C(1763100483), UINT32_C(3776962737), UINT32_C(2844608398), UINT32_C(2885101098),
                              UINT32_C( 269910347), UINT32_C( 433971495), UINT32_C(1441956227), UINT32_C(1018271575),
                              UINT32_C(1734496959), UINT32_C( 380846712), UINT32_C(3352999607), UINT32_C(3555523675),
                              UINT32_C(1995198557), UINT32_C(3314312199), UINT32_C(2406584253), UINT32_C(1779168063)),
      simde_x_mm512_set_epu32(UINT32_C(3629502055), UINT32_C(3952771463), UINT32_C(2102184556), UINT32_C( 877111492),
                              UINT32_C(1183491905), UINT32_C(3718356317), UINT32_C(3233651099), UINT32_C(3486869896),
                              UINT32_C(3932090380), UINT32_C(2449576763), UINT32_C(4246346280), UINT32_C( 201516689),
                              UINT32_C(2859036576), UINT32_C(2362091228), UINT32_C(3141663427), UINT32_C( 562234020)),
      simde_x_mm512_set_epu32(UINT32_C(         0), UINT32_C(         0), UINT32_C(         1), UINT32_C(         3),
                              UINT32_C(         0), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(         0), UINT32_C(         0), UINT32_C(        17),
                              UINT32_C(         0), UINT32_C(         1), UINT32_C(         0), UINT32_C(         3)) },
    { simde_x_mm512_set_epu32(UINT32_C( 495870887), UINT32_C(3912840869), UINT32_C( 915244711), UINT32_C(   5081424),
                              UINT32_C(1422501384), UINT32_C(4130987572), UINT32_C(2778067031), UINT32_C( 497965579),
                              UINT32_C( 910061584), UINT32_C(2002226944), UINT32_C(3673004107), UINT32_C(4246624078),
                              UINT32_C( 523093293), UINT32_C(3059761572), UINT32_C(2206005509), UINT32_C(1943141679)),
      simde_x_mm512_set_epu32(UINT32_C(1206471293), UINT32_C(1374915518), UINT32_C( 531653117), UINT32_C(2075187308),
                              UINT32_C(4150348747), UINT32_C(2163101581), UINT32_C(1444783055), UINT32_C(1878625233),
                              UINT32_C(1755684145), UINT32_C(2233240925), UINT32_C(3244523643), UINT32_C(2995026741),
                              UINT32_C(2178270751), UINT32_C(1493088054), UINT32_C(4115137419), UINT32_C( 651362699)),
      simde_x_mm512_set_epu32(UINT32_C(         0), UINT32_C(         2), UINT32_C(         1), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(         1), UINT32_C(         1), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(         0), UINT32_C(         1), UINT32_C(         1),
                              UINT32_C(         0), UINT32_C(         2), UINT32_C(         0), UINT32_C(         2)) },
    { simde_x_mm512_set_epu32(UINT32_C(2657690668), UINT32_C( 448681074), UINT32_C(1334667053), UINT32_C( 502667641),
                              UINT32_C( 855395764), UINT32_C(2622874348), UINT32_C( 808531712), UINT32_C( 454488139),
                              UINT32_C( 123547093), UINT32_C( 483090439), UINT32_C(3168637539), UINT32_C(3093747107),
                              UINT32_C(4158916667), UINT32_C(4074346392), UINT32_C(1398655610), UINT32_C(1722520923)),
      simde_x_mm512_set_epu32(UINT32_C( 604721400), UINT32_C(1471174399), UINT32_C(2491026588), UINT32_C(2529574367),
                              UINT32_C( 298473775), UINT32_C(2890366559), UINT32_C(3063632375), UINT32_C(4055983958),
                              UINT32_C(4149169500), UINT32_C(4113948134), UINT32_C(2384487126), UINT32_C(2434207126),
                              UINT32_C(3923111671), UINT32_C(3188873807), UINT32_C(1982658188), UINT32_C( 863153207)),
      simde_x_mm512_set_epu32(UINT32_C(         4), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(         2), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(         0), UINT32_C(         1), UINT32_C(         1),
                              UINT32_C(         1), UINT32_C(         1), UINT32_C(         0), UINT32_C(         1)) },
    { simde_x_mm512_set_epu32(UINT32_C(1463758672), UINT32_C( 602211615), UINT32_C(3830002991), UINT32_C(2864741101),
                              UINT32_C( 797104998), UINT32_C(2737423319), UINT32_C(3342229886), UINT32_C( 178625368),
                              UINT32_C(3091160996), UINT32_C(1095216728), UINT32_C(3079561742), UINT32_C( 430790402),
                              UINT32_C(3213858818), UINT32_C(2113970745), UINT32_C(4112838454), UINT32_C( 564512596)),
      simde_x_mm512_set_epu32(UINT32_C(1997049765), UINT32_C( 505563651), UINT32_C( 463125220), UINT32_C(3843753777),
                              UINT32_C(2346173843), UINT32_C(2157864934), UINT32_C(2591157969), UINT32_C( 389679318),
                              UINT32_C(3939775129), UINT32_C(2493364907), UINT32_C(2006619059), UINT32_C(3391409164),
                              UINT32_C(1533151625), UINT32_C(2122196136), UINT32_C(1690360675), UINT32_C(1484935627)),
      simde_x_mm512_set_epu32(UINT32_C(         0), UINT32_C(         1), UINT32_C(         8), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(         1), UINT32_C(         1), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(         0), UINT32_C(         1), UINT32_C(         0),
                              UINT32_C(         2), UINT32_C(         0), UINT32_C(         2), UINT32_C(         0)) },
    { simde_x_mm512_set_epu32(UINT32_C( 908815803), UINT32_C(2319376026), UINT32_C(2065037155), UINT32_C( 623932649),
                              UINT32_C(1610322797), UINT32_C(3452844305), UINT32_C(2031682359), UINT32_C(2994836943),
                              UINT32_C(2344919086), UINT32_C( 238137788), UINT32_C(1978166020), UINT32_C(  76768592),
                              UINT32_C(4043825594), UINT32_C(1274901810), UINT32_C( 413860084), UINT32_C( 550494320)),
      simde_x_mm512_set_epu32(UINT32_C(1228958503), UINT32_C(3519587969), UINT32_C(2809504529), UINT32_C(3115789449),
                              UINT32_C(1767270276), UINT32_C( 490610321), UINT32_C(1164436618), UINT32_C(2374669797),
                              UINT32_C(3604002618), UINT32_C(3414719029), UINT32_C(2289333019), UINT32_C(2213872499),
                              UINT32_C(1572579389), UINT32_C(3511888959), UINT32_C(2399346014), UINT32_C(1967093325)),
      simde_x_mm512_set_epu32(UINT32_C(         0), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(         7), UINT32_C(         1), UINT32_C(         1),
                              UINT32_C(         0), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(         2), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0)) },
    { simde_x_mm512_set_epu32(UINT32_C(1245407235), UINT32_C(4175005098), UINT32_C(2362914327), UINT32_C(2924553042),
                              UINT32_C(2369006988), UINT32_C(2119408419), UINT32_C(3091878410), UINT32_C(3978436943),
                              UINT32_C(1708684203), UINT32_C(1202455481), UINT32_C(2187745469), UINT32_C(3284847806),
                              UINT32_C(3884897233), UINT32_C(2094036024), UINT32_C(2456834182), UINT32_C(  69201629)),
      simde_x_mm512_set_epu32(UINT32_C(3914271744), UINT32_C( 565328458), UINT32_C(4201942548), UINT32_C(1480532604),
                              UINT32_C(4197506536), UINT32_C(3712719696), UINT32_C(3920217826), UINT32_C(1394313506),
                              UINT32_C( 394553965), UINT32_C(2278253176), UINT32_C(1697927724), UINT32_C(2383307765),
                              UINT32_C( 143428987), UINT32_C(3684943081), UINT32_C( 582607980), UINT32_C(1609326889)),
      simde_x_mm512_set_epu32(UINT32_C(         0), UINT32_C(         7), UINT32_C(         0), UINT32_C(         1),
                              UINT32_C(         0), UINT32_C(         0), UINT32_C(         0), UINT32_C(         2),
                              UINT32_C(         4), UINT32_C(         0), UINT32_C(         1), UINT32_C(         1),
                              UINT32_C(        27), UINT32_C(         0), UINT32_C(         4), UINT32_C(         0)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_div_epu32(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_mask_div_epu32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i src;
    simde__mmask16 k;
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu32(UINT32_C( 691121094), UINT32_C( 674034227), UINT32_C(2329532409), UINT32_C(3374680349),
                              UINT32_C(3920294270), UINT32_C(3054162118), UINT32_C(1568850865), UINT32_C(3151989757),
                              UINT32_C(3215450688), UINT32_C(3586813553), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C(2891870298)),
      UINT16_C(63371),
      simde_x_mm512_set_epu32(UINT32_C(3953959418), UINT32_C(2530156426), UINT32_C(1179683687), UINT32_C(2648640694),
                              UINT32_C(3623000007), UINT32_C(2708640028), UINT32_C(1691051285), UINT32_C(  50347892),
                              UINT32_C( 728425428), UINT32_C(1192263444), UINT32_C(2208623573), UINT32_C(1322777130),
                              UINT32_C( 163989560), UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(1250819173)),
      simde_x_mm512_set_epu32(UINT32_C(3003933707), UINT32_C(1314482530), UINT32_C(2997716679), UINT32_C(3555959260),
                              UINT32_C(2875927297), UINT32_C(3290702646), UINT32_C(1580565751), UINT32_C(3823902839),
                              UINT32_C(2081361826), UINT32_C( 493161721), UINT32_C(3099851477), UINT32_C( 894221337),
                              UINT32_C(2964507124), UINT32_C( 492373082), UINT32_C(4281870485), UINT32_C(2207786213)),
      simde_x_mm512_set_epu32(UINT32_C(         1), UINT32_C(         1), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(3920294270), UINT32_C(         0), UINT32_C(         1), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(3586813553), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C(         0), UINT32_C(2231263307), UINT32_C(         0), UINT32_C(         0)) },
    { simde_x_mm512_set_epu32(UINT32_C(1779168063), UINT32_C(3156074065), UINT32_C(3607805659), UINT32_C(1828175063),
                              UINT32_C(3905547273), UINT32_C(4101755863), UINT32_C(3436978124), UINT32_C(3846637996),
                              UINT32_C(2693603084), UINT32_C(1710148738), UINT32_C(1974123080), UINT32_C(2870600100),
                              UINT32_C( 118588227), UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358)),
      UINT16_C(36797),
      simde_x_mm512_set_epu32(UINT32_C(3141663427), UINT32_C( 562234020), UINT32_C(1763100483), UINT32_C(3776962737),
                              UINT32_C(2844608398), UINT32_C(2885101098), UINT32_C( 269910347), UINT32_C( 433971495),
                              UINT32_C(1441956227), UINT32_C(1018271575), UINT32_C(1734496959), UINT32_C( 380846712),
                              UINT32_C(3352999607), UINT32_C(3555523675), UINT32_C(1995198557), UINT32_C(3314312199)),
      simde_x_mm512_set_epu32(UINT32_C(2206005509), UINT32_C(1943141679), UINT32_C(3629502055), UINT32_C(3952771463),
                              UINT32_C(2102184556), UINT32_C( 877111492), UINT32_C(1183491905), UINT32_C(3718356317),
                              UINT32_C(3233651099), UINT32_C(3486869896), UINT32_C(3932090380), UINT32_C(2449576763),
                              UINT32_C(4246346280), UINT32_C( 201516689), UINT32_C(2859036576), UINT32_C(2362091228)),
      simde_x_mm512_set_epu32(UINT32_C(         1), UINT32_C(3156074065), UINT32_C(3607805659), UINT32_C(1828175063),
                              UINT32_C(         1), UINT32_C(         3), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(1710148738), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(         0), UINT32_C(        17), UINT32_C( 499863549), UINT32_C(         1)) },
    { simde_x_mm512_set_epu32(UINT32_C(4115137419), UINT32_C( 651362699), UINT32_C( 495870887), UINT32_C(3912840869),
                              UINT32_C( 915244711), UINT32_C(   5081424), UINT32_C(1422501384), UINT32_C(4130987572),
                              UINT32_C(2778067031), UINT32_C( 497965579), UINT32_C( 910061584), UINT32_C(2002226944),
                              UINT32_C(3673004107), UINT32_C(4246624078), UINT32_C( 523093293), UINT32_C(3059761572)),
      UINT16_C(46902),
      simde_x_mm512_set_epu32(UINT32_C(4074346392), UINT32_C(1398655610), UINT32_C(1722520923), UINT32_C(1206471293),
                              UINT32_C(1374915518), UINT32_C( 531653117), UINT32_C(2075187308), UINT32_C(4150348747),
                              UINT32_C(2163101581), UINT32_C(1444783055), UINT32_C(1878625233), UINT32_C(1755684145),
                              UINT32_C(2233240925), UINT32_C(3244523643), UINT32_C(2995026741), UINT32_C(2178270751)),
      simde_x_mm512_set_epu32(UINT32_C(3188873807), UINT32_C(1982658188), UINT32_C( 863153207), UINT32_C(2657690668),
                              UINT32_C( 448681074), UINT32_C(1334667053), UINT32_C( 502667641), UINT32_C( 855395764),
                              UINT32_C(2622874348), UINT32_C( 808531712), UINT32_C( 454488139), UINT32_C( 123547093),
                              UINT32_C( 483090439), UINT32_C(3168637539), UINT32_C(3093747107), UINT32_C(4158916667)),
      simde_x_mm512_set_epu32(UINT32_C(         1), UINT32_C( 651362699), UINT32_C(         1), UINT32_C(         0),
                              UINT32_C( 915244711), UINT32_C(         0), UINT32_C(         4), UINT32_C(         4),
                              UINT32_C(2778067031), UINT32_C( 497965579), UINT32_C(         4), UINT32_C(        14),
                              UINT32_C(3673004107), UINT32_C(         1), UINT32_C(         0), UINT32_C(3059761572)) },
    { simde_x_mm512_set_epu32(UINT32_C(2113970745), UINT32_C(4112838454), UINT32_C( 564512596), UINT32_C( 604721400),
                              UINT32_C(1471174399), UINT32_C(2491026588), UINT32_C(2529574367), UINT32_C( 298473775),
                              UINT32_C(2890366559), UINT32_C(3063632375), UINT32_C(4055983958), UINT32_C(4149169500),
                              UINT32_C(4113948134), UINT32_C(2384487126), UINT32_C(2434207126), UINT32_C(3923111671)),
      UINT16_C(38914),
      simde_x_mm512_set_epu32(UINT32_C(1533151625), UINT32_C(2122196136), UINT32_C(1690360675), UINT32_C(1484935627),
                              UINT32_C(1463758672), UINT32_C( 602211615), UINT32_C(3830002991), UINT32_C(2864741101),
                              UINT32_C( 797104998), UINT32_C(2737423319), UINT32_C(3342229886), UINT32_C( 178625368),
                              UINT32_C(3091160996), UINT32_C(1095216728), UINT32_C(3079561742), UINT32_C( 430790402)),
      simde_x_mm512_set_epu32(UINT32_C(4043825594), UINT32_C(1274901810), UINT32_C( 413860084), UINT32_C( 550494320),
                              UINT32_C(1997049765), UINT32_C( 505563651), UINT32_C( 463125220), UINT32_C(3843753777),
                              UINT32_C(2346173843), UINT32_C(2157864934), UINT32_C(2591157969), UINT32_C( 389679318),
                              UINT32_C(3939775129), UINT32_C(2493364907), UINT32_C(2006619059), UINT32_C(3391409164)),
      simde_x_mm512_set_epu32(UINT32_C(         0), UINT32_C(4112838454), UINT32_C( 564512596), UINT32_C(         2),
                              UINT32_C(         0), UINT32_C(2491026588), UINT32_C(2529574367), UINT32_C( 298473775),
                              UINT32_C(2890366559), UINT32_C(3063632375), UINT32_C(4055983958), UINT32_C(4149169500),
                              UINT32_C(4113948134), UINT32_C(2384487126), UINT32_C(         1), UINT32_C(3923111671)) },
    { simde_x_mm512_set_epu32(UINT32_C(1572579389), UINT32_C(3511888959), UINT32_C(2399346014), UINT32_C(1967093325),
                              UINT32_C( 908815803), UINT32_C(2319376026), UINT32_C(2065037155), UINT32_C( 623932649),
                              UINT32_C(1610322797), UINT32_C(3452844305), UINT32_C(2031682359), UINT32_C(2994836943),
                              UINT32_C(2344919086), UINT32_C( 238137788), UINT32_C(1978166020), UINT32_C(  76768592)),
      UINT16_C(  883),
      simde_x_mm512_set_epu32(UINT32_C(3284847806), UINT32_C(3884897233), UINT32_C(2094036024), UINT32_C(2456834182),
                              UINT32_C(  69201629), UINT32_C(1228958503), UINT32_C(3519587969), UINT32_C(2809504529),
                              UINT32_C(3115789449), UINT32_C(1767270276), UINT32_C( 490610321), UINT32_C(1164436618),
                              UINT32_C(2374669797), UINT32_C(3604002618), UINT32_C(3414719029), UINT32_C(2289333019)),
      simde_x_mm512_set_epu32(UINT32_C(2383307765), UINT32_C( 143428987), UINT32_C(3684943081), UINT32_C( 582607980),
                              UINT32_C(1609326889), UINT32_C(1245407235), UINT32_C(4175005098), UINT32_C(2362914327),
                              UINT32_C(2924553042), UINT32_C(2369006988), UINT32_C(2119408419), UINT32_C(3091878410),
                              UINT32_C(3978436943), UINT32_C(1708684203), UINT32_C(1202455481), UINT32_C(2187745469)),
      simde_x_mm512_set_epu32(UINT32_C(1572579389), UINT32_C(3511888959), UINT32_C(2399346014), UINT32_C(1967093325),
                              UINT32_C( 908815803), UINT32_C(2319376026), UINT32_C(         0), UINT32_C(         1),
                              UINT32_C(1610322797), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0),
                              UINT32_C(2344919086), UINT32_C( 238137788), UINT32_C(         2), UINT32_C(         1)) },
    { simde_x_mm512_set_epu32(UINT32_C(2117071873), UINT32_C(2857077767), UINT32_C(3918893192), UINT32_C(1087893388),
                              UINT32_C(3851784011), UINT32_C(3914271744), UINT32_C( 565328458), UINT32_C(4201942548),
                              UINT32_C(1480532604), UINT32_C(4197506536), UINT32_C(3712719696), UINT32_C(3920217826),
                              UINT32_C(1394313506), UINT32_C( 394553965), UINT32_C(2278253176), UINT32_C(1697927724)),
      UINT16_C(12254),
      simde_x_mm512_set_epu32(UINT32_C(  56443211), UINT32_C(2258452653), UINT32_C(3784696472), UINT32_C(1139427205),
                              UINT32_C(1090384090), UINT32_C(2389735891), UINT32_C(2215607313), UINT32_C(3817672405),
                              UINT32_C(3621770268), UINT32_C(2071747620), UINT32_C(3852178197), UINT32_C(3693632585),
                              UINT32_C( 319530416), UINT32_C(2179954815), UINT32_C(3793236393), UINT32_C( 340519338)),
      simde_x_mm512_set_epu32(UINT32_C(1219537084), UINT32_C(1349635715), UINT32_C( 732887738), UINT32_C(2566325375),
                              UINT32_C(2906533885), UINT32_C(1765754685), UINT32_C(2719983633), UINT32_C( 846129112),
                              UINT32_C(1578410935), UINT32_C(2635094838), UINT32_C(1045536663), UINT32_C( 957117985),
                              UINT32_C(3029008645), UINT32_C(1309498779), UINT32_C(3293951997), UINT32_C(1022360677)),
      simde_x_mm512_set_epu32(UINT32_C(2117071873), UINT32_C(2857077767), UINT32_C(         5), UINT32_C(1087893388),
                              UINT32_C(         0), UINT32_C(         1), UINT32_C(         0), UINT32_C(         4),
                              UINT32_C(         2), UINT32_C(         0), UINT32_C(3712719696), UINT32_C(         3),
                              UINT32_C(         0), UINT32_C(         1), UINT32_C(         1), UINT32_C(1697927724)) },
    { simde_x_mm512_set_epu32(UINT32_C(3990081318), UINT32_C( 991545752), UINT32_C(4151932359), UINT32_C( 843112042),
                              UINT32_C(4067412513), UINT32_C(2124182542), UINT32_C(2768721208), UINT32_C(2302989914),
                              UINT32_C(1224533822), UINT32_C(3475606100), UINT32_C(3610957044), UINT32_C(2556046111),
                              UINT32_C(3035396524), UINT32_C(3603101367), UINT32_C(3321443925), UINT32_C(  45581573)),
      UINT16_C(42669),
      simde_x_mm512_set_epu32(UINT32_C(4138167693), UINT32_C(3221954957), UINT32_C(2164435171), UINT32_C( 397240391),
                              UINT32_C( 200936922), UINT32_C(3263986987), UINT32_C(2536604122), UINT32_C(3629380929),
                              UINT32_C( 453331046), UINT32_C(1704580573), UINT32_C(1606190487), UINT32_C(3209309249),
                              UINT32_C(2959497652), UINT32_C(3926896735), UINT32_C(2875407663), UINT32_C(2069966669)),
      simde_x_mm512_set_epu32(UINT32_C(1379668640), UINT32_C(  66581512), UINT32_C(3737665499), UINT32_C( 304428974),
                              UINT32_C(2686704508), UINT32_C( 532978979), UINT32_C( 946958552), UINT32_C(2383642627),
                              UINT32_C(2176874140), UINT32_C( 283691898), UINT32_C(3848894665), UINT32_C(3836186002),
                              UINT32_C(1951055651), UINT32_C( 765387914), UINT32_C( 822559116), UINT32_C(   7445617)),
      simde_x_mm512_set_epu32(UINT32_C(         2), UINT32_C( 991545752), UINT32_C(         0), UINT32_C( 843112042),
                              UINT32_C(4067412513), UINT32_C(         6), UINT32_C(         2), UINT32_C(2302989914),
                              UINT32_C(         0), UINT32_C(3475606100), UINT32_C(         0), UINT32_C(2556046111),
                              UINT32_C(         1), UINT32_C(         5), UINT32_C(3321443925), UINT32_C(       278)) },
    { simde_x_mm512_set_epu32(UINT32_C(2313028370), UINT32_C( 869237081), UINT32_C(4104913762), UINT32_C(2825691966),
                              UINT32_C(3577866502), UINT32_C(2991894408), UINT32_C(2172048625), UINT32_C(1617119933),
                              UINT32_C(1521363431), UINT32_C( 553638116), UINT32_C(1036201367), UINT32_C(3107033445),
                              UINT32_C(3882811410), UINT32_C(3534384353), UINT32_C(3871215839), UINT32_C(1273589632)),
      UINT16_C(35103),
      simde_x_mm512_set_epu32(UINT32_C(2458371652), UINT32_C( 260676470), UINT32_C(1724614860), UINT32_C(4150452663),
                              UINT32_C(3816336716), UINT32_C(2208212235), UINT32_C( 932145867), UINT32_C(2432594561),
                              UINT32_C(1756892633), UINT32_C( 382632965), UINT32_C(1295078740), UINT32_C(3299165262),
                              UINT32_C( 152308919), UINT32_C(3943411788), UINT32_C(  31813624), UINT32_C( 807463845)),
      simde_x_mm512_set_epu32(UINT32_C( 615301803), UINT32_C( 382786341), UINT32_C(1852603705), UINT32_C(1998007730),
                              UINT32_C( 231325888), UINT32_C(1842039329), UINT32_C( 968682756), UINT32_C( 316335394),
                              UINT32_C(2223585202), UINT32_C(3491781959), UINT32_C(2167971796), UINT32_C(1587647099),
                              UINT32_C(2966608712), UINT32_C( 320339033), UINT32_C( 282380179), UINT32_C(4186865204)),
      simde_x_mm512_set_epu32(UINT32_C(         3), UINT32_C( 869237081), UINT32_C(4104913762), UINT32_C(2825691966),
                              UINT32_C(        16), UINT32_C(2991894408), UINT32_C(2172048625), UINT32_C(         7),
                              UINT32_C(1521363431), UINT32_C( 553638116), UINT32_C(1036201367), UINT32_C(         2),
                              UINT32_C(         0), UINT32_C(        12), UINT32_C(         0), UINT32_C(         0)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_mask_div_epu32(test_vec[i].src, test_vec[i].k, test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_div_epu64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu64(UINT64_C( 2968342496979776051), UINT64_C(10005265515001776413),
                              UINT64_C(16837535683400356038), UINT64_C( 6738163160628300797),
                              UINT64_C(13810255550447513201), UINT64_C( 6479913377553186648),
                              UINT64_C( 7505871096235581515), UINT64_C( 2265477367564496986)),
      simde_x_mm512_set_epu64(UINT64_C(10866939104613927783), UINT64_C(11375825163207743431),
                              UINT64_C(11633520338587575573), UINT64_C(  216242550290965460),
                              UINT64_C( 5120732502404950997), UINT64_C( 5681284513410730040),
                              UINT64_C( 6409558907924801050), UINT64_C( 5372227444888762251)),
      simde_x_mm512_set_epu64(UINT64_C(                   0), UINT64_C(                   0),
                              UINT64_C(                   1), UINT64_C(                  31),
                              UINT64_C(                   2), UINT64_C(                   1),
                              UINT64_C(                   1), UINT64_C(                   0)) },
    { simde_x_mm512_set_epu64(UINT64_C( 5645659480511055559), UINT64_C(15272728730484288257),
                              UINT64_C(14133460247011230967), UINT64_C(16423537638667915170),
                              UINT64_C( 2118113466433927893), UINT64_C( 3840651400764901876),
                              UINT64_C( 2114726288902596757), UINT64_C( 9482369585348649466)),
      simde_x_mm512_set_epu64(UINT64_C(13555234896536583899), UINT64_C( 7851952110853286921),
                              UINT64_C(17616907291198234572), UINT64_C(16521184395064581900),
                              UINT64_C( 7345032902979795528), UINT64_C(12329133549512917827),
                              UINT64_C( 2328100732832272381), UINT64_C( 4111895855610225675)),
      simde_x_mm512_set_epu64(UINT64_C(                   0), UINT64_C(                   1),
                              UINT64_C(                   0), UINT64_C(                   0),
                              UINT64_C(                   0), UINT64_C(                   0),
                              UINT64_C(                   0), UINT64_C(                   2)) },
    { simde_x_mm512_set_epu64(UINT64_C( 7572458917823766705), UINT64_C(12217500042222052906),
                              UINT64_C( 1159256113650983207), UINT64_C( 6193154838246823767),
                              UINT64_C( 7449607714297299576), UINT64_C(14401023659121376347),
                              UINT64_C( 8569312554655704071), UINT64_C(10336200663482757951)),
      simde_x_mm512_set_epu64(UINT64_C(15588592630942564743), UINT64_C( 9028813919053392068),
                              UINT64_C( 5083059030774095197), UINT64_C(13888425720366328200),
                              UINT64_C(16888199589465789243), UINT64_C(18237918400292775569),
                              UINT64_C(12279468594349909724), UINT64_C(13493341674566517412)),
      simde_x_mm512_set_epu64(UINT64_C(                   0), UINT64_C(                   1),
                              UINT64_C(                   0), UINT64_C(                   0),
                              UINT64_C(                   0), UINT64_C(                   0),
                              UINT64_C(                   0), UINT64_C(                   0)) },
    { simde_x_mm512_set_epu64(UINT64_C( 2129749246616352421), UINT64_C( 3930946101587052880),
                              UINT64_C( 6109596926925725236), UINT64_C(11931707044738783755),
                              UINT64_C( 3908684742628183808), UINT64_C(15775432521885308750),
                              UINT64_C( 2246668589251707300), UINT64_C( 9474721517893975343)),
      simde_x_mm512_set_epu64(UINT64_C( 5181754748372749246), UINT64_C( 2283432752406648940),
                              UINT64_C(17825612137522679693), UINT64_C( 6205295972918594513),
                              UINT64_C( 7540605987113962845), UINT64_C(13935122940778806069),
                              UINT64_C( 9355601638871447350), UINT64_C(17674380633802211723)),
      simde_x_mm512_set_epu64(UINT64_C(                   0), UINT64_C(                   1),
                              UINT64_C(                   0), UINT64_C(                   1),
                              UINT64_C(                   0), UINT64_C(                   1),
                              UINT64_C(                   0), UINT64_C(                   0)) },
    { simde_x_mm512_set_epu64(UINT64_C(11414694502393074802), UINT64_C( 5732351344186366329),
                              UINT64_C( 3673896834139808492), UINT64_C( 3472617261273378891),
                              UINT64_C(  530630724433960967), UINT64_C(13609194605976671651),
                              UINT64_C(17862411075628668824), UINT64_C( 6007180105039451483)),
      simde_x_mm512_set_epu64(UINT64_C( 2597258637662508799), UINT64_C(10698877731456040415),
                              UINT64_C( 1281935105229028959), UINT64_C(13158200861647791958),
                              UINT64_C(17820547312174620134), UINT64_C(10241294226337238422),
                              UINT64_C(16849636328689785423), UINT64_C( 8515452077469772855)),
      simde_x_mm512_set_epu64(UINT64_C(                   4), UINT64_C(                   0),
                              UINT64_C(                   2), UINT64_C(                   0),
                              UINT64_C(                   0), UINT64_C(                   1),
                              UINT64_C(                   1), UINT64_C(                   0)) },
    { simde_x_mm512_set_epu64(UINT64_C( 6286795626078602527), UINT64_C(16449737592791923437),
                              UINT64_C( 3423539900625568727), UINT64_C(14354768056262433624),
                              UINT64_C(13276435385586003544), UINT64_C(13226616968333580034),
                              UINT64_C(13803418519385186873), UINT64_C(17664506654225712980)),
      simde_x_mm512_set_epu64(UINT64_C( 8577263429665049091), UINT64_C( 1989107677696558897),
                              UINT64_C(10076739928573503462), UINT64_C(11128938736014461142),
                              UINT64_C(16921205335142546091), UINT64_C( 8618363237326703628),
                              UINT64_C( 6584836091306452136), UINT64_C( 7260043819054420427)),
      simde_x_mm512_set_epu64(UINT64_C(                   0), UINT64_C(                   8),
                              UINT64_C(                   0), UINT64_C(                   1),
                              UINT64_C(                   0), UINT64_C(                   1),
                              UINT64_C(                   2), UINT64_C(                   2)) },
    { simde_x_mm512_set_epu64(UINT64_C( 3903334154292354714), UINT64_C( 8869267046373815529),
                              UINT64_C( 6916283752571091217), UINT64_C( 8726009290759968207),
                              UINT64_C(10071350786374349244), UINT64_C( 8496158362035250512),
                              UINT64_C(17368098678232675634), UINT64_C( 1777515526450307184)),
      simde_x_mm512_set_epu64(UINT64_C( 5278336582045705857), UINT64_C(12066730073134673033),
                              UINT64_C( 7590368039103504017), UINT64_C( 5001217194949514725),
                              UINT64_C(15479073382423099957), UINT64_C( 9832610448471819123),
                              UINT64_C( 6754177049630551103), UINT64_C(10305112663885051469)),
      simde_x_mm512_set_epu64(UINT64_C(                   0), UINT64_C(                   0),
                              UINT64_C(                   0), UINT64_C(                   1),
                              UINT64_C(                   0), UINT64_C(                   0),
                              UINT64_C(                   2), UINT64_C(                   0)) },
    { simde_x_mm512_set_epu64(UINT64_C( 5348983348701791658), UINT64_C(10148639760639402834),
                              UINT64_C(10174807539574872867), UINT64_C(13279516658136916303),
                              UINT64_C( 7338742772279280569), UINT64_C( 9396295244612029630),
                              UINT64_C(16685506566149927992), UINT64_C(10552022463454113501)),
      simde_x_mm512_set_epu64(UINT64_C(16811669128702212682), UINT64_C(18047205824811442812),
                              UINT64_C(18028153300578966352), UINT64_C(16837207357260532002),
                              UINT64_C( 1694596378460381816), UINT64_C( 7292544047935022069),
                              UINT64_C(  616022812148352233), UINT64_C( 2502282222097948969)),
      simde_x_mm512_set_epu64(UINT64_C(                   0), UINT64_C(                   0),
                              UINT64_C(                   0), UINT64_C(                   0),
                              UINT64_C(                   4), UINT64_C(                   1),
                              UINT64_C(                  27), UINT64_C(                   4)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_div_epu64(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_erf_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   449.73), SIMDE_FLOAT32_C(  -898.83), SIMDE_FLOAT32_C(   193.72), SIMDE_FLOAT32_C(  -793.70) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(   434.26), SIMDE_FLOAT32_C(   437.61), SIMDE_FLOAT32_C(   -29.18), SIMDE_FLOAT32_C(  -288.39) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(  -989.93), SIMDE_FLOAT32_C(  -799.36), SIMDE_FLOAT32_C(   150.13), SIMDE_FLOAT32_C(   690.23) },
      { SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -667.63), SIMDE_FLOAT32_C(  -368.07), SIMDE_FLOAT32_C(   316.47), SIMDE_FLOAT32_C(   916.61) },
      { SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(   256.26), SIMDE_FLOAT32_C(  -321.94), SIMDE_FLOAT32_C(   111.81), SIMDE_FLOAT32_C(  -665.54) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(   169.01), SIMDE_FLOAT32_C(  -375.29), SIMDE_FLOAT32_C(  -768.83), SIMDE_FLOAT32_C(   166.33) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(   327.83), SIMDE_FLOAT32_C(  -583.11), SIMDE_FLOAT32_C(   452.18), SIMDE_FLOAT32_C(  -922.36) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(    33.53), SIMDE_FLOAT32_C(  -944.72), SIMDE_FLOAT32_C(  -608.58), SIMDE_FLOAT32_C(  -516.73) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_erf_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_erf_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -733.03), SIMDE_FLOAT64_C(  -222.93) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(  -762.35), SIMDE_FLOAT64_C(  -559.95) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(  -868.93), SIMDE_FLOAT64_C(  -580.21) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   299.67), SIMDE_FLOAT64_C(  -439.96) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(  -152.35), SIMDE_FLOAT64_C(     5.07) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(    40.68), SIMDE_FLOAT64_C(  -726.52) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   642.06), SIMDE_FLOAT64_C(  -970.77) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   563.08), SIMDE_FLOAT64_C(  -718.61) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_erf_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_erf_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   374.20), SIMDE_FLOAT32_C(  -943.32), SIMDE_FLOAT32_C(  -503.43), SIMDE_FLOAT32_C(  -980.91),
        SIMDE_FLOAT32_C(   588.09), SIMDE_FLOAT32_C(   116.98), SIMDE_FLOAT32_C(   159.00), SIMDE_FLOAT32_C(    60.92) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(   517.69), SIMDE_FLOAT32_C(   565.06), SIMDE_FLOAT32_C(   410.42), SIMDE_FLOAT32_C(   802.07),
        SIMDE_FLOAT32_C(  -337.69), SIMDE_FLOAT32_C(   790.63), SIMDE_FLOAT32_C(    48.57), SIMDE_FLOAT32_C(   385.99) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(   695.57), SIMDE_FLOAT32_C(  -950.00), SIMDE_FLOAT32_C(   565.77), SIMDE_FLOAT32_C(  -123.23),
        SIMDE_FLOAT32_C(   205.87), SIMDE_FLOAT32_C(  -194.42), SIMDE_FLOAT32_C(   803.30), SIMDE_FLOAT32_C(  -901.24) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(   429.62), SIMDE_FLOAT32_C(  -530.89), SIMDE_FLOAT32_C(   279.94), SIMDE_FLOAT32_C(   445.55),
        SIMDE_FLOAT32_C(    34.20), SIMDE_FLOAT32_C(   333.48), SIMDE_FLOAT32_C(   841.52), SIMDE_FLOAT32_C(  -591.60) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(   390.15), SIMDE_FLOAT32_C(  -661.91), SIMDE_FLOAT32_C(  -572.50), SIMDE_FLOAT32_C(   -21.76),
        SIMDE_FLOAT32_C(   455.07), SIMDE_FLOAT32_C(   586.50), SIMDE_FLOAT32_C(  -960.84), SIMDE_FLOAT32_C(   -27.24) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(   151.56), SIMDE_FLOAT32_C(   449.58), SIMDE_FLOAT32_C(  -225.17), SIMDE_FLOAT32_C(   813.87),
        SIMDE_FLOAT32_C(   240.21), SIMDE_FLOAT32_C(   823.40), SIMDE_FLOAT32_C(   199.87), SIMDE_FLOAT32_C(   -64.22) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(   873.40), SIMDE_FLOAT32_C(  -234.36), SIMDE_FLOAT32_C(   812.55), SIMDE_FLOAT32_C(    79.27),
        SIMDE_FLOAT32_C(   571.22), SIMDE_FLOAT32_C(   615.85), SIMDE_FLOAT32_C(   178.03), SIMDE_FLOAT32_C(     0.84) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.77) } },
    { { SIMDE_FLOAT32_C(  -915.04), SIMDE_FLOAT32_C(  -542.03), SIMDE_FLOAT32_C(  -553.61), SIMDE_FLOAT32_C(   119.16),
        SIMDE_FLOAT32_C(   791.44), SIMDE_FLOAT32_C(  -712.09), SIMDE_FLOAT32_C(   527.56), SIMDE_FLOAT32_C(   181.60) },
      { SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_erf_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_erf_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -313.70), SIMDE_FLOAT64_C(   714.53), SIMDE_FLOAT64_C(   927.20), SIMDE_FLOAT64_C(  -898.10) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   921.61), SIMDE_FLOAT64_C(   406.65), SIMDE_FLOAT64_C(   519.73), SIMDE_FLOAT64_C(  -550.92) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   655.77), SIMDE_FLOAT64_C(  -305.99), SIMDE_FLOAT64_C(   -29.82), SIMDE_FLOAT64_C(  -266.26) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(    47.11), SIMDE_FLOAT64_C(   991.16), SIMDE_FLOAT64_C(  -298.84), SIMDE_FLOAT64_C(   426.24) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -122.46), SIMDE_FLOAT64_C(   928.48), SIMDE_FLOAT64_C(  -151.69), SIMDE_FLOAT64_C(  -677.70) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(  -184.81), SIMDE_FLOAT64_C(  -799.82), SIMDE_FLOAT64_C(   978.74), SIMDE_FLOAT64_C(  -554.85) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(    83.95), SIMDE_FLOAT64_C(  -400.78), SIMDE_FLOAT64_C(  -165.64), SIMDE_FLOAT64_C(  -926.09) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   941.89), SIMDE_FLOAT64_C(   862.77), SIMDE_FLOAT64_C(   150.41), SIMDE_FLOAT64_C(  -371.81) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_erf_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_erf_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -838.40), SIMDE_FLOAT32_C(   872.70), SIMDE_FLOAT32_C(   438.38), SIMDE_FLOAT32_C(  -298.62),
        SIMDE_FLOAT32_C(   781.61), SIMDE_FLOAT32_C(   970.11), SIMDE_FLOAT32_C(    78.85), SIMDE_FLOAT32_C(   723.02),
        SIMDE_FLOAT32_C(  -818.83), SIMDE_FLOAT32_C(  -579.07), SIMDE_FLOAT32_C(   251.53), SIMDE_FLOAT32_C(  -753.80),
        SIMDE_FLOAT32_C(   319.82), SIMDE_FLOAT32_C(   967.37), SIMDE_FLOAT32_C(   725.05), SIMDE_FLOAT32_C(   873.27) },
      { SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -304.80), SIMDE_FLOAT32_C(   941.81), SIMDE_FLOAT32_C(   -83.14), SIMDE_FLOAT32_C(  -799.93),
        SIMDE_FLOAT32_C(  -339.09), SIMDE_FLOAT32_C(   125.84), SIMDE_FLOAT32_C(   891.08), SIMDE_FLOAT32_C(  -989.54),
        SIMDE_FLOAT32_C(   253.61), SIMDE_FLOAT32_C(   980.01), SIMDE_FLOAT32_C(   634.54), SIMDE_FLOAT32_C(   449.90),
        SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(   271.95), SIMDE_FLOAT32_C(   654.57), SIMDE_FLOAT32_C(   624.56) },
      { SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(   144.65), SIMDE_FLOAT32_C(    92.95), SIMDE_FLOAT32_C(  -674.06), SIMDE_FLOAT32_C(   -73.74),
        SIMDE_FLOAT32_C(    63.06), SIMDE_FLOAT32_C(   404.78), SIMDE_FLOAT32_C(  -350.71), SIMDE_FLOAT32_C(   244.23),
        SIMDE_FLOAT32_C(   825.71), SIMDE_FLOAT32_C(   900.82), SIMDE_FLOAT32_C(   490.43), SIMDE_FLOAT32_C(   145.53),
        SIMDE_FLOAT32_C(   868.18), SIMDE_FLOAT32_C(   215.47), SIMDE_FLOAT32_C(    18.80), SIMDE_FLOAT32_C(  -436.61) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(   157.28), SIMDE_FLOAT32_C(   935.67), SIMDE_FLOAT32_C(  -236.55), SIMDE_FLOAT32_C(   818.19),
        SIMDE_FLOAT32_C(    61.50), SIMDE_FLOAT32_C(  -345.47), SIMDE_FLOAT32_C(   828.65), SIMDE_FLOAT32_C(  -684.89),
        SIMDE_FLOAT32_C(  -365.46), SIMDE_FLOAT32_C(   463.19), SIMDE_FLOAT32_C(   765.01), SIMDE_FLOAT32_C(  -902.51),
        SIMDE_FLOAT32_C(  -264.87), SIMDE_FLOAT32_C(   419.58), SIMDE_FLOAT32_C(   722.05), SIMDE_FLOAT32_C(   879.78) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(  -487.47), SIMDE_FLOAT32_C(  -952.01), SIMDE_FLOAT32_C(  -193.96), SIMDE_FLOAT32_C(   575.59),
        SIMDE_FLOAT32_C(   452.77), SIMDE_FLOAT32_C(   455.33), SIMDE_FLOAT32_C(  -180.18), SIMDE_FLOAT32_C(   278.48),
        SIMDE_FLOAT32_C(   356.14), SIMDE_FLOAT32_C(  -689.76), SIMDE_FLOAT32_C(  -575.99), SIMDE_FLOAT32_C(   224.33),
        SIMDE_FLOAT32_C(   525.72), SIMDE_FLOAT32_C(   442.82), SIMDE_FLOAT32_C(   787.71), SIMDE_FLOAT32_C(  -317.01) },
      { SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(   378.48), SIMDE_FLOAT32_C(  -448.83), SIMDE_FLOAT32_C(  -498.82), SIMDE_FLOAT32_C(  -560.02),
        SIMDE_FLOAT32_C(   205.70), SIMDE_FLOAT32_C(  -670.17), SIMDE_FLOAT32_C(  -244.90), SIMDE_FLOAT32_C(   840.24),
        SIMDE_FLOAT32_C(   793.02), SIMDE_FLOAT32_C(  -479.90), SIMDE_FLOAT32_C(   937.74), SIMDE_FLOAT32_C(  -471.85),
        SIMDE_FLOAT32_C(   939.68), SIMDE_FLOAT32_C(   659.79), SIMDE_FLOAT32_C(  -592.07), SIMDE_FLOAT32_C(  -547.79) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00) } },
    { { SIMDE_FLOAT32_C(   707.78), SIMDE_FLOAT32_C(   213.97), SIMDE_FLOAT32_C(  -972.20), SIMDE_FLOAT32_C(   160.55),
        SIMDE_FLOAT32_C(  -330.70), SIMDE_FLOAT32_C(  -152.38), SIMDE_FLOAT32_C(  -560.98), SIMDE_FLOAT32_C(  -974.56),
        SIMDE_FLOAT32_C(   157.86), SIMDE_FLOAT32_C(  -136.96), SIMDE_FLOAT32_C(   249.77), SIMDE_FLOAT32_C(  -316.43),
        SIMDE_FLOAT32_C(  -694.15), SIMDE_FLOAT32_C(    37.48), SIMDE_FLOAT32_C(   366.57), SIMDE_FLOAT32_C(   684.33) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } },
    { { SIMDE_FLOAT32_C(   588.65), SIMDE_FLOAT32_C(   867.75), SIMDE_FLOAT32_C(  -875.68), SIMDE_FLOAT32_C(  -205.65),
        SIMDE_FLOAT32_C(  -802.42), SIMDE_FLOAT32_C(  -120.59), SIMDE_FLOAT32_C(  -365.41), SIMDE_FLOAT32_C(   990.60),
        SIMDE_FLOAT32_C(   399.52), SIMDE_FLOAT32_C(  -427.67), SIMDE_FLOAT32_C(  -481.25), SIMDE_FLOAT32_C(   339.20),
        SIMDE_FLOAT32_C(  -767.88), SIMDE_FLOAT32_C(   -73.32), SIMDE_FLOAT32_C(   791.41), SIMDE_FLOAT32_C(   939.89) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_erf_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_erf_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    60.80), SIMDE_FLOAT32_C(   224.62), SIMDE_FLOAT32_C(  -945.35), SIMDE_FLOAT32_C(  -219.00),
        SIMDE_FLOAT32_C(   891.11), SIMDE_FLOAT32_C(   761.94), SIMDE_FLOAT32_C(   992.65), SIMDE_FLOAT32_C(   332.00),
        SIMDE_FLOAT32_C(   387.85), SIMDE_FLOAT32_C(  -689.44), SIMDE_FLOAT32_C(   195.76), SIMDE_FLOAT32_C(  -335.77),
        SIMDE_FLOAT32_C(  -349.96), SIMDE_FLOAT32_C(  -675.36), SIMDE_FLOAT32_C(   298.19), SIMDE_FLOAT32_C(   171.46) },
      UINT8_C( 43),
      { SIMDE_FLOAT32_C(  -593.03), SIMDE_FLOAT32_C(   241.03), SIMDE_FLOAT32_C(   550.96), SIMDE_FLOAT32_C(   496.03),
        SIMDE_FLOAT32_C(   -94.31), SIMDE_FLOAT32_C(  -581.85), SIMDE_FLOAT32_C(  -755.59), SIMDE_FLOAT32_C(    80.74),
        SIMDE_FLOAT32_C(   755.01), SIMDE_FLOAT32_C(   520.11), SIMDE_FLOAT32_C(    62.41), SIMDE_FLOAT32_C(  -580.00),
        SIMDE_FLOAT32_C(   448.06), SIMDE_FLOAT32_C(  -303.73), SIMDE_FLOAT32_C(   480.80), SIMDE_FLOAT32_C(  -327.32) },
      { SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(  -945.35), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(   891.11), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(   992.65), SIMDE_FLOAT32_C(   332.00),
        SIMDE_FLOAT32_C(   387.85), SIMDE_FLOAT32_C(  -689.44), SIMDE_FLOAT32_C(   195.76), SIMDE_FLOAT32_C(  -335.77),
        SIMDE_FLOAT32_C(  -349.96), SIMDE_FLOAT32_C(  -675.36), SIMDE_FLOAT32_C(   298.19), SIMDE_FLOAT32_C(   171.46) } },
    { { SIMDE_FLOAT32_C(  -249.08), SIMDE_FLOAT32_C(  -738.20), SIMDE_FLOAT32_C(  -436.21), SIMDE_FLOAT32_C(  -487.13),
        SIMDE_FLOAT32_C(  -745.54), SIMDE_FLOAT32_C(   895.79), SIMDE_FLOAT32_C(   900.71), SIMDE_FLOAT32_C(  -434.99),
        SIMDE_FLOAT32_C(    91.55), SIMDE_FLOAT32_C(  -435.06), SIMDE_FLOAT32_C(   215.05), SIMDE_FLOAT32_C(   416.20),
        SIMDE_FLOAT32_C(   863.14), SIMDE_FLOAT32_C(  -613.49), SIMDE_FLOAT32_C(  -739.87), SIMDE_FLOAT32_C(  -729.89) },
      UINT8_C(228),
      { SIMDE_FLOAT32_C(   811.10), SIMDE_FLOAT32_C(   766.14), SIMDE_FLOAT32_C(  -466.77), SIMDE_FLOAT32_C(  -770.76),
        SIMDE_FLOAT32_C(  -989.45), SIMDE_FLOAT32_C(   613.97), SIMDE_FLOAT32_C(   984.25), SIMDE_FLOAT32_C(   530.66),
        SIMDE_FLOAT32_C(  -323.62), SIMDE_FLOAT32_C(  -595.75), SIMDE_FLOAT32_C(   -21.28), SIMDE_FLOAT32_C(   372.65),
        SIMDE_FLOAT32_C(   885.05), SIMDE_FLOAT32_C(   651.40), SIMDE_FLOAT32_C(  -876.43), SIMDE_FLOAT32_C(  -853.15) },
      { SIMDE_FLOAT32_C(  -249.08), SIMDE_FLOAT32_C(  -738.20), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(  -487.13),
        SIMDE_FLOAT32_C(  -745.54), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(    91.55), SIMDE_FLOAT32_C(  -435.06), SIMDE_FLOAT32_C(   215.05), SIMDE_FLOAT32_C(   416.20),
        SIMDE_FLOAT32_C(   863.14), SIMDE_FLOAT32_C(  -613.49), SIMDE_FLOAT32_C(  -739.87), SIMDE_FLOAT32_C(  -729.89) } },
    { { SIMDE_FLOAT32_C(  -784.80), SIMDE_FLOAT32_C(  -363.56), SIMDE_FLOAT32_C(  -598.70), SIMDE_FLOAT32_C(  -889.01),
        SIMDE_FLOAT32_C(  -462.85), SIMDE_FLOAT32_C(   -33.68), SIMDE_FLOAT32_C(   202.54), SIMDE_FLOAT32_C(   102.09),
        SIMDE_FLOAT32_C(  -818.63), SIMDE_FLOAT32_C(  -381.26), SIMDE_FLOAT32_C(   -34.77), SIMDE_FLOAT32_C(  -432.12),
        SIMDE_FLOAT32_C(  -121.13), SIMDE_FLOAT32_C(   235.34), SIMDE_FLOAT32_C(  -804.58), SIMDE_FLOAT32_C(  -310.04) },
      UINT8_C(218),
      { SIMDE_FLOAT32_C(  -271.35), SIMDE_FLOAT32_C(   -80.79), SIMDE_FLOAT32_C(    12.03), SIMDE_FLOAT32_C(  -657.38),
        SIMDE_FLOAT32_C(   -96.55), SIMDE_FLOAT32_C(  -457.32), SIMDE_FLOAT32_C(    19.00), SIMDE_FLOAT32_C(   307.70),
        SIMDE_FLOAT32_C(   521.41), SIMDE_FLOAT32_C(  -608.35), SIMDE_FLOAT32_C(   192.75), SIMDE_FLOAT32_C(   172.81),
        SIMDE_FLOAT32_C(  -484.78), SIMDE_FLOAT32_C(   339.60), SIMDE_FLOAT32_C(   388.01), SIMDE_FLOAT32_C(   151.65) },
      { SIMDE_FLOAT32_C(  -784.80), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(  -598.70), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(   -33.68), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(  -818.63), SIMDE_FLOAT32_C(  -381.26), SIMDE_FLOAT32_C(   -34.77), SIMDE_FLOAT32_C(  -432.12),
        SIMDE_FLOAT32_C(  -121.13), SIMDE_FLOAT32_C(   235.34), SIMDE_FLOAT32_C(  -804.58), SIMDE_FLOAT32_C(  -310.04) } },
    { { SIMDE_FLOAT32_C(   740.90), SIMDE_FLOAT32_C(   498.99), SIMDE_FLOAT32_C(   688.80), SIMDE_FLOAT32_C(  -292.78),
        SIMDE_FLOAT32_C(  -298.47), SIMDE_FLOAT32_C(  -209.10), SIMDE_FLOAT32_C(  -111.42), SIMDE_FLOAT32_C(   320.27),
        SIMDE_FLOAT32_C(   756.13), SIMDE_FLOAT32_C(   456.46), SIMDE_FLOAT32_C(  -800.86), SIMDE_FLOAT32_C(    -8.53),
        SIMDE_FLOAT32_C(   651.88), SIMDE_FLOAT32_C(  -110.90), SIMDE_FLOAT32_C(   992.95), SIMDE_FLOAT32_C(  -619.48) },
      UINT8_C(168),
      { SIMDE_FLOAT32_C(     4.98), SIMDE_FLOAT32_C(  -276.86), SIMDE_FLOAT32_C(  -288.24), SIMDE_FLOAT32_C(   547.66),
        SIMDE_FLOAT32_C(   742.14), SIMDE_FLOAT32_C(  -980.53), SIMDE_FLOAT32_C(    69.07), SIMDE_FLOAT32_C(  -866.21),
        SIMDE_FLOAT32_C(   212.21), SIMDE_FLOAT32_C(  -758.12), SIMDE_FLOAT32_C(  -351.00), SIMDE_FLOAT32_C(  -448.19),
        SIMDE_FLOAT32_C(   629.88), SIMDE_FLOAT32_C(   800.65), SIMDE_FLOAT32_C(  -707.29), SIMDE_FLOAT32_C(   128.87) },
      { SIMDE_FLOAT32_C(   740.90), SIMDE_FLOAT32_C(   498.99), SIMDE_FLOAT32_C(   688.80), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(  -298.47), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(  -111.42), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(   756.13), SIMDE_FLOAT32_C(   456.46), SIMDE_FLOAT32_C(  -800.86), SIMDE_FLOAT32_C(    -8.53),
        SIMDE_FLOAT32_C(   651.88), SIMDE_FLOAT32_C(  -110.90), SIMDE_FLOAT32_C(   992.95), SIMDE_FLOAT32_C(  -619.48) } },
    { { SIMDE_FLOAT32_C(   489.46), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(   830.41), SIMDE_FLOAT32_C(  -719.65),
        SIMDE_FLOAT32_C(   888.51), SIMDE_FLOAT32_C(   150.68), SIMDE_FLOAT32_C(  -963.52), SIMDE_FLOAT32_C(   344.97),
        SIMDE_FLOAT32_C(   349.82), SIMDE_FLOAT32_C(    27.95), SIMDE_FLOAT32_C(    -3.15), SIMDE_FLOAT32_C(  -761.08),
        SIMDE_FLOAT32_C(    20.90), SIMDE_FLOAT32_C(   377.37), SIMDE_FLOAT32_C(  -952.77), SIMDE_FLOAT32_C(  -974.12) },
      UINT8_C( 30),
      { SIMDE_FLOAT32_C(  -241.01), SIMDE_FLOAT32_C(   573.54), SIMDE_FLOAT32_C(   842.66), SIMDE_FLOAT32_C(  -221.54),
        SIMDE_FLOAT32_C(  -357.39), SIMDE_FLOAT32_C(   976.44), SIMDE_FLOAT32_C(   990.67), SIMDE_FLOAT32_C(  -115.52),
        SIMDE_FLOAT32_C(  -374.55), SIMDE_FLOAT32_C(  -457.51), SIMDE_FLOAT32_C(  -485.63), SIMDE_FLOAT32_C(  -573.90),
        SIMDE_FLOAT32_C(  -164.80), SIMDE_FLOAT32_C(   643.24), SIMDE_FLOAT32_C(   915.55), SIMDE_FLOAT32_C(   835.12) },
      { SIMDE_FLOAT32_C(   489.46), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(   150.68), SIMDE_FLOAT32_C(  -963.52), SIMDE_FLOAT32_C(   344.97),
        SIMDE_FLOAT32_C(   349.82), SIMDE_FLOAT32_C(    27.95), SIMDE_FLOAT32_C(    -3.15), SIMDE_FLOAT32_C(  -761.08),
        SIMDE_FLOAT32_C(    20.90), SIMDE_FLOAT32_C(   377.37), SIMDE_FLOAT32_C(  -952.77), SIMDE_FLOAT32_C(  -974.12) } },
    { { SIMDE_FLOAT32_C(   473.65), SIMDE_FLOAT32_C(  -804.09), SIMDE_FLOAT32_C(   723.64), SIMDE_FLOAT32_C(  -375.67),
        SIMDE_FLOAT32_C(  -767.61), SIMDE_FLOAT32_C(    68.61), SIMDE_FLOAT32_C(   974.15), SIMDE_FLOAT32_C(   260.34),
        SIMDE_FLOAT32_C(  -934.54), SIMDE_FLOAT32_C(  -786.93), SIMDE_FLOAT32_C(  -718.76), SIMDE_FLOAT32_C(   442.83),
        SIMDE_FLOAT32_C(  -739.70), SIMDE_FLOAT32_C(  -692.88), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(    19.29) },
      UINT8_C(120),
      { SIMDE_FLOAT32_C(   386.01), SIMDE_FLOAT32_C(   797.75), SIMDE_FLOAT32_C(  -476.73), SIMDE_FLOAT32_C(   362.46),
        SIMDE_FLOAT32_C(   788.43), SIMDE_FLOAT32_C(   407.75), SIMDE_FLOAT32_C(   987.90), SIMDE_FLOAT32_C(  -669.09),
        SIMDE_FLOAT32_C(   922.12), SIMDE_FLOAT32_C(  -586.00), SIMDE_FLOAT32_C(   166.11), SIMDE_FLOAT32_C(   565.36),
        SIMDE_FLOAT32_C(  -670.44), SIMDE_FLOAT32_C(     1.23), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -474.54) },
      { SIMDE_FLOAT32_C(   473.65), SIMDE_FLOAT32_C(  -804.09), SIMDE_FLOAT32_C(   723.64), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   260.34),
        SIMDE_FLOAT32_C(  -934.54), SIMDE_FLOAT32_C(  -786.93), SIMDE_FLOAT32_C(  -718.76), SIMDE_FLOAT32_C(   442.83),
        SIMDE_FLOAT32_C(  -739.70), SIMDE_FLOAT32_C(  -692.88), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(    19.29) } },
    { { SIMDE_FLOAT32_C(  -275.13), SIMDE_FLOAT32_C(   663.34), SIMDE_FLOAT32_C(  -242.15), SIMDE_FLOAT32_C(   793.48),
        SIMDE_FLOAT32_C(   637.49), SIMDE_FLOAT32_C(  -981.81), SIMDE_FLOAT32_C(   858.94), SIMDE_FLOAT32_C(   850.55),
        SIMDE_FLOAT32_C(  -700.57), SIMDE_FLOAT32_C(   301.77), SIMDE_FLOAT32_C(  -889.15), SIMDE_FLOAT32_C(  -393.45),
        SIMDE_FLOAT32_C(  -154.87), SIMDE_FLOAT32_C(   130.14), SIMDE_FLOAT32_C(  -512.79), SIMDE_FLOAT32_C(  -768.86) },
      UINT8_C( 73),
      { SIMDE_FLOAT32_C(    10.48), SIMDE_FLOAT32_C(   593.59), SIMDE_FLOAT32_C(  -283.68), SIMDE_FLOAT32_C(  -581.77),
        SIMDE_FLOAT32_C(   581.50), SIMDE_FLOAT32_C(    47.23), SIMDE_FLOAT32_C(  -659.65), SIMDE_FLOAT32_C(   995.50),
        SIMDE_FLOAT32_C(  -786.66), SIMDE_FLOAT32_C(   905.71), SIMDE_FLOAT32_C(  -674.95), SIMDE_FLOAT32_C(   214.58),
        SIMDE_FLOAT32_C(   -55.28), SIMDE_FLOAT32_C(  -149.49), SIMDE_FLOAT32_C(   939.45), SIMDE_FLOAT32_C(  -391.94) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   663.34), SIMDE_FLOAT32_C(  -242.15), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(   637.49), SIMDE_FLOAT32_C(  -981.81), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(   850.55),
        SIMDE_FLOAT32_C(  -700.57), SIMDE_FLOAT32_C(   301.77), SIMDE_FLOAT32_C(  -889.15), SIMDE_FLOAT32_C(  -393.45),
        SIMDE_FLOAT32_C(  -154.87), SIMDE_FLOAT32_C(   130.14), SIMDE_FLOAT32_C(  -512.79), SIMDE_FLOAT32_C(  -768.86) } },
    { { SIMDE_FLOAT32_C(   608.36), SIMDE_FLOAT32_C(   732.93), SIMDE_FLOAT32_C(  -754.45), SIMDE_FLOAT32_C(   626.55),
        SIMDE_FLOAT32_C(   591.86), SIMDE_FLOAT32_C(  -903.90), SIMDE_FLOAT32_C(   925.98), SIMDE_FLOAT32_C(  -106.36),
        SIMDE_FLOAT32_C(  -793.05), SIMDE_FLOAT32_C(  -467.47), SIMDE_FLOAT32_C(   738.77), SIMDE_FLOAT32_C(   337.09),
        SIMDE_FLOAT32_C(    19.74), SIMDE_FLOAT32_C(   969.90), SIMDE_FLOAT32_C(  -735.01), SIMDE_FLOAT32_C(  -969.78) },
      UINT8_C(189),
      { SIMDE_FLOAT32_C(   -18.69), SIMDE_FLOAT32_C(  -551.55), SIMDE_FLOAT32_C(   144.99), SIMDE_FLOAT32_C(  -971.46),
        SIMDE_FLOAT32_C(  -211.20), SIMDE_FLOAT32_C(   140.49), SIMDE_FLOAT32_C(  -758.11), SIMDE_FLOAT32_C(  -305.49),
        SIMDE_FLOAT32_C(   465.54), SIMDE_FLOAT32_C(   456.46), SIMDE_FLOAT32_C(   639.24), SIMDE_FLOAT32_C(  -683.94),
        SIMDE_FLOAT32_C(   395.91), SIMDE_FLOAT32_C(  -752.70), SIMDE_FLOAT32_C(   924.42), SIMDE_FLOAT32_C(   128.84) },
      { SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(   732.93), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   925.98), SIMDE_FLOAT32_C(    -1.00),
        SIMDE_FLOAT32_C(  -793.05), SIMDE_FLOAT32_C(  -467.47), SIMDE_FLOAT32_C(   738.77), SIMDE_FLOAT32_C(   337.09),
        SIMDE_FLOAT32_C(    19.74), SIMDE_FLOAT32_C(   969.90), SIMDE_FLOAT32_C(  -735.01), SIMDE_FLOAT32_C(  -969.78) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_erf_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_erf_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   287.12), SIMDE_FLOAT64_C(   923.43), SIMDE_FLOAT64_C(  -235.47), SIMDE_FLOAT64_C(  -270.63),
        SIMDE_FLOAT64_C(   872.91), SIMDE_FLOAT64_C(    62.22), SIMDE_FLOAT64_C(  -259.06), SIMDE_FLOAT64_C(   509.74) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -381.16), SIMDE_FLOAT64_C(  -659.69), SIMDE_FLOAT64_C(   397.49), SIMDE_FLOAT64_C(  -803.01),
        SIMDE_FLOAT64_C(  -467.01), SIMDE_FLOAT64_C(  -777.46), SIMDE_FLOAT64_C(  -995.46), SIMDE_FLOAT64_C(  -455.46) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00),
        SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   412.93), SIMDE_FLOAT64_C(    31.33), SIMDE_FLOAT64_C(   675.90), SIMDE_FLOAT64_C(   842.14),
        SIMDE_FLOAT64_C(   999.42), SIMDE_FLOAT64_C(  -210.59), SIMDE_FLOAT64_C(   469.06), SIMDE_FLOAT64_C(  -204.67) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   194.13), SIMDE_FLOAT64_C(   752.63), SIMDE_FLOAT64_C(   950.43), SIMDE_FLOAT64_C(   627.80),
        SIMDE_FLOAT64_C(     3.93), SIMDE_FLOAT64_C(   -80.48), SIMDE_FLOAT64_C(  -738.99), SIMDE_FLOAT64_C(  -708.95) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(  -157.05), SIMDE_FLOAT64_C(    25.54), SIMDE_FLOAT64_C(    20.42), SIMDE_FLOAT64_C(  -284.15),
        SIMDE_FLOAT64_C(  -912.24), SIMDE_FLOAT64_C(   761.36), SIMDE_FLOAT64_C(  -774.41), SIMDE_FLOAT64_C(  -293.40) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00),
        SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(  -898.33), SIMDE_FLOAT64_C(   623.08), SIMDE_FLOAT64_C(   -96.41), SIMDE_FLOAT64_C(  -365.34),
        SIMDE_FLOAT64_C(   845.62), SIMDE_FLOAT64_C(   -91.87), SIMDE_FLOAT64_C(   179.19), SIMDE_FLOAT64_C(   258.55) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(   939.45), SIMDE_FLOAT64_C(  -144.90), SIMDE_FLOAT64_C(   100.69), SIMDE_FLOAT64_C(   938.87),
        SIMDE_FLOAT64_C(   644.51), SIMDE_FLOAT64_C(  -430.25), SIMDE_FLOAT64_C(  -265.80), SIMDE_FLOAT64_C(  -161.37) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(  -677.63), SIMDE_FLOAT64_C(  -315.37), SIMDE_FLOAT64_C(  -533.56), SIMDE_FLOAT64_C(   326.31),
        SIMDE_FLOAT64_C(   604.15), SIMDE_FLOAT64_C(  -272.55), SIMDE_FLOAT64_C(   617.36), SIMDE_FLOAT64_C(  -552.90) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_erf_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_erf_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -475.71), SIMDE_FLOAT64_C(  -480.68), SIMDE_FLOAT64_C(   251.56), SIMDE_FLOAT64_C(   974.57),
        SIMDE_FLOAT64_C(  -654.33), SIMDE_FLOAT64_C(   974.69), SIMDE_FLOAT64_C(  -443.19), SIMDE_FLOAT64_C(   343.95) },
      UINT8_C(224),
      { SIMDE_FLOAT64_C(  -493.29), SIMDE_FLOAT64_C(  -325.36), SIMDE_FLOAT64_C(  -887.40), SIMDE_FLOAT64_C(  -727.34),
        SIMDE_FLOAT64_C(  -936.73), SIMDE_FLOAT64_C(   654.69), SIMDE_FLOAT64_C(   988.04), SIMDE_FLOAT64_C(  -361.17) },
      { SIMDE_FLOAT64_C(  -475.71), SIMDE_FLOAT64_C(  -480.68), SIMDE_FLOAT64_C(   251.56), SIMDE_FLOAT64_C(   974.57),
        SIMDE_FLOAT64_C(  -654.33), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   370.27), SIMDE_FLOAT64_C(   594.68), SIMDE_FLOAT64_C(  -149.62), SIMDE_FLOAT64_C(  -535.38),
        SIMDE_FLOAT64_C(   277.92), SIMDE_FLOAT64_C(  -615.67), SIMDE_FLOAT64_C(  -531.54), SIMDE_FLOAT64_C(   583.79) },
      UINT8_C(113),
      { SIMDE_FLOAT64_C(  -420.19), SIMDE_FLOAT64_C(  -624.33), SIMDE_FLOAT64_C(  -915.05), SIMDE_FLOAT64_C(  -155.08),
        SIMDE_FLOAT64_C(   757.99), SIMDE_FLOAT64_C(  -390.77), SIMDE_FLOAT64_C(   364.24), SIMDE_FLOAT64_C(     9.55) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(   594.68), SIMDE_FLOAT64_C(  -149.62), SIMDE_FLOAT64_C(  -535.38),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(   583.79) } },
    { { SIMDE_FLOAT64_C(  -416.20), SIMDE_FLOAT64_C(   709.91), SIMDE_FLOAT64_C(   -15.76), SIMDE_FLOAT64_C(   140.62),
        SIMDE_FLOAT64_C(    53.86), SIMDE_FLOAT64_C(  -954.63), SIMDE_FLOAT64_C(   647.32), SIMDE_FLOAT64_C(   728.50) },
      UINT8_C(252),
      { SIMDE_FLOAT64_C(   919.98), SIMDE_FLOAT64_C(   791.78), SIMDE_FLOAT64_C(   812.66), SIMDE_FLOAT64_C(   908.02),
        SIMDE_FLOAT64_C(  -569.39), SIMDE_FLOAT64_C(   182.93), SIMDE_FLOAT64_C(   502.70), SIMDE_FLOAT64_C(   280.99) },
      { SIMDE_FLOAT64_C(  -416.20), SIMDE_FLOAT64_C(   709.91), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(   647.55), SIMDE_FLOAT64_C(  -219.38), SIMDE_FLOAT64_C(   665.32), SIMDE_FLOAT64_C(  -883.99),
        SIMDE_FLOAT64_C(  -635.59), SIMDE_FLOAT64_C(  -276.35), SIMDE_FLOAT64_C(  -304.18), SIMDE_FLOAT64_C(  -259.92) },
      UINT8_C(  7),
      { SIMDE_FLOAT64_C(   540.74), SIMDE_FLOAT64_C(  -501.92), SIMDE_FLOAT64_C(   417.83), SIMDE_FLOAT64_C(   -95.02),
        SIMDE_FLOAT64_C(   507.63), SIMDE_FLOAT64_C(  -998.37), SIMDE_FLOAT64_C(  -385.10), SIMDE_FLOAT64_C(  -508.13) },
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(  -883.99),
        SIMDE_FLOAT64_C(  -635.59), SIMDE_FLOAT64_C(  -276.35), SIMDE_FLOAT64_C(  -304.18), SIMDE_FLOAT64_C(  -259.92) } },
    { { SIMDE_FLOAT64_C(   142.25), SIMDE_FLOAT64_C(   668.76), SIMDE_FLOAT64_C(  -462.76), SIMDE_FLOAT64_C(  -210.42),
        SIMDE_FLOAT64_C(   397.27), SIMDE_FLOAT64_C(  -304.79), SIMDE_FLOAT64_C(  -290.44), SIMDE_FLOAT64_C(   189.04) },
      UINT8_C(184),
      { SIMDE_FLOAT64_C(  -382.42), SIMDE_FLOAT64_C(   619.65), SIMDE_FLOAT64_C(   690.79), SIMDE_FLOAT64_C(  -879.72),
        SIMDE_FLOAT64_C(   -99.35), SIMDE_FLOAT64_C(   338.34), SIMDE_FLOAT64_C(   -99.10), SIMDE_FLOAT64_C(  -434.03) },
      { SIMDE_FLOAT64_C(   142.25), SIMDE_FLOAT64_C(   668.76), SIMDE_FLOAT64_C(  -462.76), SIMDE_FLOAT64_C(    -1.00),
        SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(  -290.44), SIMDE_FLOAT64_C(    -1.00) } },
    { { SIMDE_FLOAT64_C(   454.35), SIMDE_FLOAT64_C(   265.31), SIMDE_FLOAT64_C(   289.62), SIMDE_FLOAT64_C(  -849.83),
        SIMDE_FLOAT64_C(  -994.61), SIMDE_FLOAT64_C(  -901.78), SIMDE_FLOAT64_C(   690.91), SIMDE_FLOAT64_C(  -496.53) },
      UINT8_C( 88),
      { SIMDE_FLOAT64_C(  -404.11), SIMDE_FLOAT64_C(  -988.90), SIMDE_FLOAT64_C(   517.68), SIMDE_FLOAT64_C(   210.79),
        SIMDE_FLOAT64_C(  -497.03), SIMDE_FLOAT64_C(  -340.06), SIMDE_FLOAT64_C(  -120.45), SIMDE_FLOAT64_C(    40.21) },
      { SIMDE_FLOAT64_C(   454.35), SIMDE_FLOAT64_C(   265.31), SIMDE_FLOAT64_C(   289.62), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(  -901.78), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(  -496.53) } },
    { { SIMDE_FLOAT64_C(   449.51), SIMDE_FLOAT64_C(  -723.18), SIMDE_FLOAT64_C(   735.42), SIMDE_FLOAT64_C(  -840.92),
        SIMDE_FLOAT64_C(   465.86), SIMDE_FLOAT64_C(  -756.71), SIMDE_FLOAT64_C(  -223.34), SIMDE_FLOAT64_C(    85.52) },
      UINT8_C(226),
      { SIMDE_FLOAT64_C(  -103.06), SIMDE_FLOAT64_C(   986.16), SIMDE_FLOAT64_C(   272.42), SIMDE_FLOAT64_C(   797.84),
        SIMDE_FLOAT64_C(  -447.86), SIMDE_FLOAT64_C(  -273.23), SIMDE_FLOAT64_C(    63.15), SIMDE_FLOAT64_C(   841.76) },
      { SIMDE_FLOAT64_C(   449.51), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(   735.42), SIMDE_FLOAT64_C(  -840.92),
        SIMDE_FLOAT64_C(   465.86), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00) } },
    { { SIMDE_FLOAT64_C(  -123.06), SIMDE_FLOAT64_C(    68.54), SIMDE_FLOAT64_C(   939.98), SIMDE_FLOAT64_C(  -432.16),
        SIMDE_FLOAT64_C(   572.01), SIMDE_FLOAT64_C(   456.03), SIMDE_FLOAT64_C(   163.74), SIMDE_FLOAT64_C(   583.10) },
      UINT8_C(247),
      { SIMDE_FLOAT64_C(  -625.47), SIMDE_FLOAT64_C(  -913.93), SIMDE_FLOAT64_C(   633.64), SIMDE_FLOAT64_C(   254.08),
        SIMDE_FLOAT64_C(   126.28), SIMDE_FLOAT64_C(    83.16), SIMDE_FLOAT64_C(   530.89), SIMDE_FLOAT64_C(  -138.30) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(  -432.16),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(    -1.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_erf_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_erfinv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.29) },
      { SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.26) } },
    { { SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(     0.67) },
      { SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(     0.69) } },
    { { SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.25) },
      { SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.23) } },
    { { SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.13) },
      { SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.12) } },
    { { SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.35) },
      { SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -1.20), SIMDE_FLOAT32_C(    -1.13), SIMDE_FLOAT32_C(     0.32) } },
    { { SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.27) },
      { SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.24) } },
    { { SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.47) },
      { SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(    -0.44) } },
    { { SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.73) },
      { SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.78) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_erfinv_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_erfinv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(    -0.59) },
      { SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(    -0.58) } },
    { { SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(    -0.15) },
      { SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(    -0.13) } },
    { { SIMDE_FLOAT64_C(     0.69), SIMDE_FLOAT64_C(    -0.24) },
      { SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(    -0.22) } },
    { { SIMDE_FLOAT64_C(    -0.02), SIMDE_FLOAT64_C(     0.81) },
      { SIMDE_FLOAT64_C(    -0.02), SIMDE_FLOAT64_C(     0.93) } },
    { { SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(     0.36) },
      { SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     0.33) } },
    { { SIMDE_FLOAT64_C(    -0.75), SIMDE_FLOAT64_C(     0.21) },
      { SIMDE_FLOAT64_C(    -0.81), SIMDE_FLOAT64_C(     0.19) } },
    { { SIMDE_FLOAT64_C(    -0.67), SIMDE_FLOAT64_C(    -0.11) },
      { SIMDE_FLOAT64_C(    -0.69), SIMDE_FLOAT64_C(    -0.10) } },
    { { SIMDE_FLOAT64_C(    -0.54), SIMDE_FLOAT64_C(    -0.85) },
      { SIMDE_FLOAT64_C(    -0.52), SIMDE_FLOAT64_C(    -1.02) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_erfinv_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_erfinv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.73),
        SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.30) },
      { SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(    -1.20), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.78),
        SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.27) } },
    { { SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.39),
        SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.54) },
      { SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.36),
        SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.52) } },
    { { SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(    -0.65),
        SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -0.20) },
      { SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(    -0.66),
        SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.18) } },
    { { SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(    -0.60),
        SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.71) },
      { SIMDE_FLOAT32_C(    -1.16), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    -0.60),
        SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.75) } },
    { { SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.51),
        SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(    -0.26) },
      { SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     0.49),
        SIMDE_FLOAT32_C(    -1.10), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(    -0.23) } },
    { { SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.95),
        SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.64) },
      { SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(    -1.04), SIMDE_FLOAT32_C(    -1.39),
        SIMDE_FLOAT32_C(    -1.13), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    -0.65) } },
    { { SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.11),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.16) },
      { SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.10),
        SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -1.28), SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     0.14) } },
    { { SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(    -0.08) },
      { SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(    -1.45), SIMDE_FLOAT32_C(    -1.10), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    -0.07) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_erfinv_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_erfinv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(     0.53), SIMDE_FLOAT64_C(    -0.77), SIMDE_FLOAT64_C(     0.01) },
      { SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(    -0.85), SIMDE_FLOAT64_C(     0.01) } },
    { { SIMDE_FLOAT64_C(    -0.06), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(    -0.85), SIMDE_FLOAT64_C(    -0.30) },
      { SIMDE_FLOAT64_C(    -0.05), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(    -1.02), SIMDE_FLOAT64_C(    -0.27) } },
    { { SIMDE_FLOAT64_C(    -0.05), SIMDE_FLOAT64_C(     0.50), SIMDE_FLOAT64_C(     0.71), SIMDE_FLOAT64_C(     0.05) },
      { SIMDE_FLOAT64_C(    -0.04), SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(     0.75), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(    -0.77), SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     0.86) },
      { SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -0.85), SIMDE_FLOAT64_C(     0.50), SIMDE_FLOAT64_C(     1.04) } },
    { { SIMDE_FLOAT64_C(    -0.22), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(    -0.94), SIMDE_FLOAT64_C(    -0.31) },
      { SIMDE_FLOAT64_C(    -0.20), SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(    -1.33), SIMDE_FLOAT64_C(    -0.28) } },
    { { SIMDE_FLOAT64_C(     0.82), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(    -0.14), SIMDE_FLOAT64_C(     0.26) },
      { SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(     0.87), SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     0.23) } },
    { { SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(     0.71), SIMDE_FLOAT64_C(    -0.56) },
      { SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(     0.75), SIMDE_FLOAT64_C(    -0.55) } },
    { { SIMDE_FLOAT64_C(    -0.59), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.85) },
      { SIMDE_FLOAT64_C(    -0.58), SIMDE_FLOAT64_C(     0.69), SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(     1.02) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_erfinv_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_erfinv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.66),
        SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.81), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(     0.98),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.09),
        SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.05) },
      { SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(    -0.67),
        SIMDE_FLOAT32_C(    -1.10), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(     1.64),
        SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.08),
        SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(    -0.04) } },
    { { SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.33),
        SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.85),
        SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(     0.37),
        SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(    -0.91) },
      { SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(    -1.82), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.30),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(     1.02),
        SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -1.16), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -1.20) } },
    { { SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     0.86),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.21),
        SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(    -0.14),
        SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.97) },
      { SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(     1.04),
        SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     1.02), SIMDE_FLOAT32_C(     0.19),
        SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -1.39), SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(    -0.12),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     1.53) } },
    { { SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.85),
        SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.37),
        SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.58),
        SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.70) },
      { SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -1.02),
        SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(    -1.82), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(    -1.13), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(     0.57),
        SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.73) } },
    { { SIMDE_FLOAT32_C(    -0.81), SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.81),
        SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.76),
        SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.56),
        SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.03) },
      { SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -1.13), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.93),
        SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(    -1.82), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(    -0.83),
        SIMDE_FLOAT32_C(    -1.33), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.55),
        SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.03) } },
    { { SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.57),
        SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.93),
        SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.85),
        SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.14) },
      { SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(    -0.56),
        SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     1.28),
        SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     1.02),
        SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(    -1.02), SIMDE_FLOAT32_C(     0.12) } },
    { { SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    -0.79),
        SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(    -0.61),
        SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(    -0.22),
        SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.85) },
      { SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -0.89),
        SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.61),
        SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(    -0.20),
        SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    -1.02) } },
    { { SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.20),
        SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.47),
        SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.88),
        SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.11) },
      { SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.18),
        SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.44),
        SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     1.10),
        SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.10) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_erfinv_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_erfinv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    -0.15), SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(    -0.46),
        SIMDE_FLOAT64_C(    -0.63), SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(    -0.93), SIMDE_FLOAT64_C(    -0.78) },
      { SIMDE_FLOAT64_C(    -0.13), SIMDE_FLOAT64_C(     0.38), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(    -0.43),
        SIMDE_FLOAT64_C(    -0.63), SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(    -1.28), SIMDE_FLOAT64_C(    -0.87) } },
    { { SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(     0.60), SIMDE_FLOAT64_C(    -0.02),
        SIMDE_FLOAT64_C(    -0.14), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(    -0.87), SIMDE_FLOAT64_C(     0.63) },
      { SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(    -0.25), SIMDE_FLOAT64_C(     0.60), SIMDE_FLOAT64_C(    -0.02),
        SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(    -1.07), SIMDE_FLOAT64_C(     0.63) } },
    { { SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(    -0.49), SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.24),
        SIMDE_FLOAT64_C(     0.35), SIMDE_FLOAT64_C(    -0.11), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(    -0.23) },
      { SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(    -0.47), SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(     0.22),
        SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(    -0.10), SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(    -0.21) } },
    { { SIMDE_FLOAT64_C(    -0.23), SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(    -0.76), SIMDE_FLOAT64_C(     0.18),
        SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(    -0.69), SIMDE_FLOAT64_C(    -0.66), SIMDE_FLOAT64_C(    -0.24) },
      { SIMDE_FLOAT64_C(    -0.21), SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(    -0.83), SIMDE_FLOAT64_C(     0.16),
        SIMDE_FLOAT64_C(     1.20), SIMDE_FLOAT64_C(    -0.72), SIMDE_FLOAT64_C(    -0.67), SIMDE_FLOAT64_C(    -0.22) } },
    { { SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(     0.79), SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(    -0.91),
        SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(    -0.68), SIMDE_FLOAT64_C(    -0.09) },
      { SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.89), SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(    -1.20),
        SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(    -0.70), SIMDE_FLOAT64_C(    -0.08) } },
    { { SIMDE_FLOAT64_C(    -0.91), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(     0.89), SIMDE_FLOAT64_C(    -0.05),
        SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(    -0.97), SIMDE_FLOAT64_C(    -0.41), SIMDE_FLOAT64_C(    -0.43) },
      { SIMDE_FLOAT64_C(    -1.20), SIMDE_FLOAT64_C(     1.20), SIMDE_FLOAT64_C(     1.13), SIMDE_FLOAT64_C(    -0.04),
        SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(    -1.53), SIMDE_FLOAT64_C(    -0.38), SIMDE_FLOAT64_C(    -0.40) } },
    { { SIMDE_FLOAT64_C(    -0.46), SIMDE_FLOAT64_C(    -0.84), SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.89),
        SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(    -0.51), SIMDE_FLOAT64_C(    -0.34), SIMDE_FLOAT64_C(     0.82) },
      { SIMDE_FLOAT64_C(    -0.43), SIMDE_FLOAT64_C(    -0.99), SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     1.13),
        SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(    -0.49), SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(     0.95) } },
    { { SIMDE_FLOAT64_C(     0.49), SIMDE_FLOAT64_C(    -0.10), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.40),
        SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(     0.35), SIMDE_FLOAT64_C(    -0.84), SIMDE_FLOAT64_C(    -0.07) },
      { SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(    -0.09), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.37),
        SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(    -0.99), SIMDE_FLOAT64_C(    -0.06) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_erfinv_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_erfinv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.66),
        SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.59),
        SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.15),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.82) },
      UINT8_C(161),
      { SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(    -0.81), SIMDE_FLOAT32_C(    -0.18),
        SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.57),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.64),
        SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(    -0.03) },
      { SIMDE_FLOAT32_C(    -1.28), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.66),
        SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.56),
        SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.15),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.82) } },
    { { SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.62),
        SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.32),
        SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.74),
        SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.19) },
      UINT8_C( 98),
      { SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.08),
        SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.60),
        SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.95),
        SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.10) },
      { SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(    -1.82), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.62),
        SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(    -1.04), SIMDE_FLOAT32_C(     0.32),
        SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.74),
        SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.19) } },
    { { SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.38),
        SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.12),
        SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.56),
        SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.87) },
      UINT8_C( 32),
      { SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.18),
        SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.69),
        SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.81),
        SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(    -0.57) },
      { SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.38),
        SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.12),
        SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.56),
        SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.87) } },
    { { SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.60),
        SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.32),
        SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.89) },
      UINT8_C(177),
      { SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.79),
        SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.58),
        SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(    -0.16),
        SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(    -0.81) },
      { SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.60),
        SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.57),
        SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.89) } },
    { { SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(     0.90),
        SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.57),
        SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.63) },
      UINT8_C( 55),
      { SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(    -0.68),
        SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -0.73),
        SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.59),
        SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.18) },
      { SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     0.90),
        SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.57),
        SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.63) } },
    { { SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.43),
        SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.86),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.58) },
      UINT8_C( 30),
      { SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(    -0.51),
        SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.09),
        SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(    -0.06),
        SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.03) },
      { SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(    -0.49),
        SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.86),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.58) } },
    { { SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.68),
        SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.82),
        SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(     0.84) },
      UINT8_C( 89),
      { SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.78),
        SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     0.22),
        SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.65),
        SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(     0.53) },
      { SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.87),
        SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.68),
        SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.82),
        SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(     0.84) } },
    { { SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.24),
        SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.67),
        SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.84) },
      UINT8_C(239),
      { SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.89),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(     0.31),
        SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.89),
        SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(     0.05) },
      { SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(    -1.33), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     1.13),
        SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.28),
        SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.67),
        SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.84) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_erfinv_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_erfinv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    -0.86), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.87),
        SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.70) },
      UINT8_C(108),
      { SIMDE_FLOAT64_C(     0.53), SIMDE_FLOAT64_C(    -0.20), SIMDE_FLOAT64_C(    -0.78), SIMDE_FLOAT64_C(     0.69),
        SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(    -0.30), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(    -0.50) },
      { SIMDE_FLOAT64_C(    -0.86), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(    -0.87), SIMDE_FLOAT64_C(     0.72),
        SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(    -0.27), SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(     0.70) } },
    { { SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -0.89), SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(     0.35),
        SIMDE_FLOAT64_C(    -0.47), SIMDE_FLOAT64_C(    -0.10), SIMDE_FLOAT64_C(    -0.79), SIMDE_FLOAT64_C(    -0.77) },
      UINT8_C(112),
      { SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(     0.54),
        SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(    -0.95), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(    -0.01) },
      { SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -0.89), SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(     0.35),
        SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(    -1.39), SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(    -0.77) } },
    { { SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(    -0.06), SIMDE_FLOAT64_C(     0.47),
        SIMDE_FLOAT64_C(     0.69), SIMDE_FLOAT64_C(    -0.35), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -0.51) },
      UINT8_C(248),
      { SIMDE_FLOAT64_C(    -0.30), SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(    -0.96),
        SIMDE_FLOAT64_C(    -0.76), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(    -0.85), SIMDE_FLOAT64_C(     0.79) },
      { SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(    -0.06), SIMDE_FLOAT64_C(    -1.45),
        SIMDE_FLOAT64_C(    -0.83), SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(    -1.02), SIMDE_FLOAT64_C(     0.89) } },
    { { SIMDE_FLOAT64_C(    -0.97), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(    -0.76),
        SIMDE_FLOAT64_C(    -0.09), SIMDE_FLOAT64_C(    -0.63), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(    -0.66) },
      UINT8_C( 18),
      { SIMDE_FLOAT64_C(    -0.04), SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(    -0.67), SIMDE_FLOAT64_C(     0.64),
        SIMDE_FLOAT64_C(    -0.44), SIMDE_FLOAT64_C(    -0.75), SIMDE_FLOAT64_C(     0.63), SIMDE_FLOAT64_C(     0.50) },
      { SIMDE_FLOAT64_C(    -0.97), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(    -0.76),
        SIMDE_FLOAT64_C(    -0.41), SIMDE_FLOAT64_C(    -0.63), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(    -0.66) } },
    { { SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(    -0.85), SIMDE_FLOAT64_C(    -0.28),
        SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(     0.55) },
      UINT8_C( 45),
      { SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(     0.60),
        SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(    -0.72), SIMDE_FLOAT64_C(    -0.73) },
      { SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.60),
        SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.28), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(     0.55) } },
    { { SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(    -0.36), SIMDE_FLOAT64_C(    -0.03),
        SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(    -0.08), SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     0.11) },
      UINT8_C( 61),
      { SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     0.50), SIMDE_FLOAT64_C(     0.20),
        SIMDE_FLOAT64_C(     0.16), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(    -0.49), SIMDE_FLOAT64_C(     0.31) },
      { SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(     0.18),
        SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     0.11) } },
    { { SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(    -0.67), SIMDE_FLOAT64_C(    -0.66), SIMDE_FLOAT64_C(     0.37),
        SIMDE_FLOAT64_C(     0.88), SIMDE_FLOAT64_C(    -0.06), SIMDE_FLOAT64_C(    -0.17), SIMDE_FLOAT64_C(     0.67) },
      UINT8_C(215),
      { SIMDE_FLOAT64_C(    -0.57), SIMDE_FLOAT64_C(     0.26), SIMDE_FLOAT64_C(     0.53), SIMDE_FLOAT64_C(    -0.29),
        SIMDE_FLOAT64_C(     0.53), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(    -0.83) },
      { SIMDE_FLOAT64_C(    -0.56), SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(     0.37),
        SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(    -0.06), SIMDE_FLOAT64_C(     1.20), SIMDE_FLOAT64_C(    -0.97) } },
    { { SIMDE_FLOAT64_C(    -0.94), SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.99),
        SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(    -0.65), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(    -0.79) },
      UINT8_C(161),
      { SIMDE_FLOAT64_C(    -0.24), SIMDE_FLOAT64_C(     0.38), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     0.27),
        SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.60), SIMDE_FLOAT64_C(     0.03) },
      { SIMDE_FLOAT64_C(    -0.22), SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.99),
        SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.03) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_erfinv_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_erfc_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -315.30), SIMDE_FLOAT32_C(  -413.87), SIMDE_FLOAT32_C(  -345.31), SIMDE_FLOAT32_C(  -228.93) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(   600.65), SIMDE_FLOAT32_C(  -112.11), SIMDE_FLOAT32_C(   -98.86), SIMDE_FLOAT32_C(    20.55) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(  -949.84), SIMDE_FLOAT32_C(  -802.03), SIMDE_FLOAT32_C(   212.71), SIMDE_FLOAT32_C(  -757.84) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(  -651.52), SIMDE_FLOAT32_C(  -363.93), SIMDE_FLOAT32_C(   876.28), SIMDE_FLOAT32_C(  -203.61) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(   527.04), SIMDE_FLOAT32_C(    57.60), SIMDE_FLOAT32_C(  -839.49), SIMDE_FLOAT32_C(   826.28) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   974.10), SIMDE_FLOAT32_C(   325.71), SIMDE_FLOAT32_C(  -535.87), SIMDE_FLOAT32_C(   230.83) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   348.57), SIMDE_FLOAT32_C(   534.66), SIMDE_FLOAT32_C(   231.47), SIMDE_FLOAT32_C(   673.78) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   954.08), SIMDE_FLOAT32_C(   495.36), SIMDE_FLOAT32_C(   387.10), SIMDE_FLOAT32_C(  -361.22) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_erfc_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_erfc_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -645.17), SIMDE_FLOAT64_C(   211.72) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   715.58), SIMDE_FLOAT64_C(   471.86) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   209.41), SIMDE_FLOAT64_C(  -887.34) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(  -326.89), SIMDE_FLOAT64_C(   772.60) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   574.21), SIMDE_FLOAT64_C(   504.70) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -447.93), SIMDE_FLOAT64_C(  -208.36) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(   404.62), SIMDE_FLOAT64_C(  -998.91) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(  -193.72), SIMDE_FLOAT64_C(   660.84) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_erfc_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_erfc_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   496.19), SIMDE_FLOAT32_C(  -675.69), SIMDE_FLOAT32_C(  -153.22), SIMDE_FLOAT32_C(   -88.71),
        SIMDE_FLOAT32_C(   381.12), SIMDE_FLOAT32_C(  -119.60), SIMDE_FLOAT32_C(   255.09), SIMDE_FLOAT32_C(  -509.70) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(  -168.05), SIMDE_FLOAT32_C(   -24.56), SIMDE_FLOAT32_C(  -778.51), SIMDE_FLOAT32_C(   349.90),
        SIMDE_FLOAT32_C(   925.97), SIMDE_FLOAT32_C(   439.36), SIMDE_FLOAT32_C(  -180.81), SIMDE_FLOAT32_C(   678.48) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(  -580.27), SIMDE_FLOAT32_C(  -258.04), SIMDE_FLOAT32_C(   -62.98), SIMDE_FLOAT32_C(  -953.83),
        SIMDE_FLOAT32_C(   354.49), SIMDE_FLOAT32_C(   914.71), SIMDE_FLOAT32_C(  -173.05), SIMDE_FLOAT32_C(  -256.98) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(   277.83), SIMDE_FLOAT32_C(    49.94), SIMDE_FLOAT32_C(  -710.16), SIMDE_FLOAT32_C(   556.77),
        SIMDE_FLOAT32_C(  -300.30), SIMDE_FLOAT32_C(   375.96), SIMDE_FLOAT32_C(   468.75), SIMDE_FLOAT32_C(  -804.12) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(   700.27), SIMDE_FLOAT32_C(  -684.46), SIMDE_FLOAT32_C(   107.18), SIMDE_FLOAT32_C(    81.39),
        SIMDE_FLOAT32_C(   195.94), SIMDE_FLOAT32_C(  -637.73), SIMDE_FLOAT32_C(   571.69), SIMDE_FLOAT32_C(  -972.11) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(   337.71), SIMDE_FLOAT32_C(   793.18), SIMDE_FLOAT32_C(   377.79), SIMDE_FLOAT32_C(   263.68),
        SIMDE_FLOAT32_C(   232.54), SIMDE_FLOAT32_C(  -803.02), SIMDE_FLOAT32_C(   -57.84), SIMDE_FLOAT32_C(   652.27) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   -61.06), SIMDE_FLOAT32_C(   879.18), SIMDE_FLOAT32_C(   698.44), SIMDE_FLOAT32_C(  -706.57),
        SIMDE_FLOAT32_C(   793.88), SIMDE_FLOAT32_C(  -474.61), SIMDE_FLOAT32_C(    36.44), SIMDE_FLOAT32_C(    71.71) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   575.33), SIMDE_FLOAT32_C(   326.28), SIMDE_FLOAT32_C(  -371.52), SIMDE_FLOAT32_C(  -724.97),
        SIMDE_FLOAT32_C(  -297.76), SIMDE_FLOAT32_C(  -902.77), SIMDE_FLOAT32_C(  -529.09), SIMDE_FLOAT32_C(  -597.49) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_erfc_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_erfc_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   461.51), SIMDE_FLOAT64_C(  -571.50), SIMDE_FLOAT64_C(   241.15), SIMDE_FLOAT64_C(   521.48) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -695.16), SIMDE_FLOAT64_C(  -842.41), SIMDE_FLOAT64_C(   799.26), SIMDE_FLOAT64_C(   685.42) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(     3.40), SIMDE_FLOAT64_C(  -776.18), SIMDE_FLOAT64_C(  -325.62), SIMDE_FLOAT64_C(     7.02) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   948.46), SIMDE_FLOAT64_C(   348.12), SIMDE_FLOAT64_C(   741.43), SIMDE_FLOAT64_C(  -182.81) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(   319.42), SIMDE_FLOAT64_C(    46.64), SIMDE_FLOAT64_C(   792.19), SIMDE_FLOAT64_C(   -94.82) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(  -364.65), SIMDE_FLOAT64_C(  -718.98), SIMDE_FLOAT64_C(   201.33), SIMDE_FLOAT64_C(   634.78) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   348.43), SIMDE_FLOAT64_C(   374.84), SIMDE_FLOAT64_C(   -48.84), SIMDE_FLOAT64_C(  -910.34) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(  -513.67), SIMDE_FLOAT64_C(  -235.62), SIMDE_FLOAT64_C(   -80.01), SIMDE_FLOAT64_C(   947.84) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_erfc_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_erfc_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   430.03), SIMDE_FLOAT32_C(  -494.11), SIMDE_FLOAT32_C(  -522.83), SIMDE_FLOAT32_C(  -160.68),
        SIMDE_FLOAT32_C(  -217.51), SIMDE_FLOAT32_C(   364.22), SIMDE_FLOAT32_C(  -906.03), SIMDE_FLOAT32_C(   335.92),
        SIMDE_FLOAT32_C(  -779.46), SIMDE_FLOAT32_C(  -248.95), SIMDE_FLOAT32_C(   -22.71), SIMDE_FLOAT32_C(  -802.66),
        SIMDE_FLOAT32_C(  -495.02), SIMDE_FLOAT32_C(  -618.65), SIMDE_FLOAT32_C(  -592.74), SIMDE_FLOAT32_C(   774.33) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(  -819.68), SIMDE_FLOAT32_C(  -841.87), SIMDE_FLOAT32_C(   969.10), SIMDE_FLOAT32_C(  -855.15),
        SIMDE_FLOAT32_C(  -473.12), SIMDE_FLOAT32_C(   203.71), SIMDE_FLOAT32_C(  -640.23), SIMDE_FLOAT32_C(  -593.80),
        SIMDE_FLOAT32_C(  -307.51), SIMDE_FLOAT32_C(   246.67), SIMDE_FLOAT32_C(  -893.51), SIMDE_FLOAT32_C(   533.63),
        SIMDE_FLOAT32_C(   217.68), SIMDE_FLOAT32_C(   100.04), SIMDE_FLOAT32_C(   228.82), SIMDE_FLOAT32_C(  -352.29) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(   605.93), SIMDE_FLOAT32_C(   705.99), SIMDE_FLOAT32_C(   487.03), SIMDE_FLOAT32_C(  -611.58),
        SIMDE_FLOAT32_C(    70.21), SIMDE_FLOAT32_C(   581.00), SIMDE_FLOAT32_C(   724.34), SIMDE_FLOAT32_C(   290.75),
        SIMDE_FLOAT32_C(  -667.95), SIMDE_FLOAT32_C(  -298.37), SIMDE_FLOAT32_C(   488.09), SIMDE_FLOAT32_C(  -162.97),
        SIMDE_FLOAT32_C(    82.98), SIMDE_FLOAT32_C(   895.36), SIMDE_FLOAT32_C(  -388.63), SIMDE_FLOAT32_C(   263.30) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(  -946.51), SIMDE_FLOAT32_C(  -419.53), SIMDE_FLOAT32_C(   408.15), SIMDE_FLOAT32_C(  -419.64),
        SIMDE_FLOAT32_C(   784.18), SIMDE_FLOAT32_C(   767.92), SIMDE_FLOAT32_C(   -13.43), SIMDE_FLOAT32_C(  -523.33),
        SIMDE_FLOAT32_C(    14.59), SIMDE_FLOAT32_C(    93.06), SIMDE_FLOAT32_C(  -989.70), SIMDE_FLOAT32_C(  -767.74),
        SIMDE_FLOAT32_C(  -806.91), SIMDE_FLOAT32_C(   239.11), SIMDE_FLOAT32_C(  -120.03), SIMDE_FLOAT32_C(   799.02) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   -54.90), SIMDE_FLOAT32_C(  -633.00), SIMDE_FLOAT32_C(  -812.56), SIMDE_FLOAT32_C(  -984.69),
        SIMDE_FLOAT32_C(   948.00), SIMDE_FLOAT32_C(   911.78), SIMDE_FLOAT32_C(   306.06), SIMDE_FLOAT32_C(  -719.95),
        SIMDE_FLOAT32_C(  -386.59), SIMDE_FLOAT32_C(  -205.84), SIMDE_FLOAT32_C(   117.08), SIMDE_FLOAT32_C(   696.39),
        SIMDE_FLOAT32_C(  -310.49), SIMDE_FLOAT32_C(   728.45), SIMDE_FLOAT32_C(   -40.32), SIMDE_FLOAT32_C(  -257.00) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(  -691.08), SIMDE_FLOAT32_C(  -632.17), SIMDE_FLOAT32_C(   323.36), SIMDE_FLOAT32_C(  -906.91),
        SIMDE_FLOAT32_C(  -864.25), SIMDE_FLOAT32_C(  -690.07), SIMDE_FLOAT32_C(  -430.23), SIMDE_FLOAT32_C(   150.34),
        SIMDE_FLOAT32_C(   402.99), SIMDE_FLOAT32_C(  -419.93), SIMDE_FLOAT32_C(   382.60), SIMDE_FLOAT32_C(   596.09),
        SIMDE_FLOAT32_C(   819.18), SIMDE_FLOAT32_C(  -737.43), SIMDE_FLOAT32_C(   395.11), SIMDE_FLOAT32_C(  -235.72) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(  -370.43), SIMDE_FLOAT32_C(   582.55), SIMDE_FLOAT32_C(  -220.40), SIMDE_FLOAT32_C(  -422.43),
        SIMDE_FLOAT32_C(   494.33), SIMDE_FLOAT32_C(  -914.34), SIMDE_FLOAT32_C(  -142.39), SIMDE_FLOAT32_C(  -892.26),
        SIMDE_FLOAT32_C(  -120.19), SIMDE_FLOAT32_C(   974.69), SIMDE_FLOAT32_C(   804.12), SIMDE_FLOAT32_C(   569.33),
        SIMDE_FLOAT32_C(   703.14), SIMDE_FLOAT32_C(  -236.19), SIMDE_FLOAT32_C(  -687.67), SIMDE_FLOAT32_C(  -987.95) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00) } },
    { { SIMDE_FLOAT32_C(   131.64), SIMDE_FLOAT32_C(   635.69), SIMDE_FLOAT32_C(  -894.85), SIMDE_FLOAT32_C(   267.39),
        SIMDE_FLOAT32_C(   945.62), SIMDE_FLOAT32_C(  -325.08), SIMDE_FLOAT32_C(  -582.27), SIMDE_FLOAT32_C(   348.62),
        SIMDE_FLOAT32_C(   254.98), SIMDE_FLOAT32_C(   800.33), SIMDE_FLOAT32_C(   -55.30), SIMDE_FLOAT32_C(    74.16),
        SIMDE_FLOAT32_C(  -937.10), SIMDE_FLOAT32_C(  -660.19), SIMDE_FLOAT32_C(   838.44), SIMDE_FLOAT32_C(  -307.53) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_erfc_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_erfc_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   956.61), SIMDE_FLOAT32_C(   234.13), SIMDE_FLOAT32_C(   892.38), SIMDE_FLOAT32_C(   414.62),
        SIMDE_FLOAT32_C(  -352.76), SIMDE_FLOAT32_C(    66.22), SIMDE_FLOAT32_C(  -611.87), SIMDE_FLOAT32_C(   409.12),
        SIMDE_FLOAT32_C(   -59.49), SIMDE_FLOAT32_C(   561.33), SIMDE_FLOAT32_C(  -922.08), SIMDE_FLOAT32_C(   538.83),
        SIMDE_FLOAT32_C(  -425.54), SIMDE_FLOAT32_C(  -342.56), SIMDE_FLOAT32_C(  -597.87), SIMDE_FLOAT32_C(   992.17) },
      UINT8_C(125),
      { SIMDE_FLOAT32_C(   513.40), SIMDE_FLOAT32_C(  -248.97), SIMDE_FLOAT32_C(  -181.44), SIMDE_FLOAT32_C(   317.13),
        SIMDE_FLOAT32_C(   267.53), SIMDE_FLOAT32_C(   935.63), SIMDE_FLOAT32_C(   584.65), SIMDE_FLOAT32_C(   221.64),
        SIMDE_FLOAT32_C(  -188.28), SIMDE_FLOAT32_C(   142.72), SIMDE_FLOAT32_C(   400.07), SIMDE_FLOAT32_C(   778.58),
        SIMDE_FLOAT32_C(   216.90), SIMDE_FLOAT32_C(   410.27), SIMDE_FLOAT32_C(   735.18), SIMDE_FLOAT32_C(  -548.98) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   234.13), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   409.12),
        SIMDE_FLOAT32_C(   -59.49), SIMDE_FLOAT32_C(   561.33), SIMDE_FLOAT32_C(  -922.08), SIMDE_FLOAT32_C(   538.83),
        SIMDE_FLOAT32_C(  -425.54), SIMDE_FLOAT32_C(  -342.56), SIMDE_FLOAT32_C(  -597.87), SIMDE_FLOAT32_C(   992.17) } },
    { { SIMDE_FLOAT32_C(   302.65), SIMDE_FLOAT32_C(   149.80), SIMDE_FLOAT32_C(    98.26), SIMDE_FLOAT32_C(  -631.12),
        SIMDE_FLOAT32_C(   537.93), SIMDE_FLOAT32_C(  -492.62), SIMDE_FLOAT32_C(   309.39), SIMDE_FLOAT32_C(    99.26),
        SIMDE_FLOAT32_C(  -414.70), SIMDE_FLOAT32_C(  -151.78), SIMDE_FLOAT32_C(   673.72), SIMDE_FLOAT32_C(   242.74),
        SIMDE_FLOAT32_C(   250.35), SIMDE_FLOAT32_C(   665.88), SIMDE_FLOAT32_C(   646.74), SIMDE_FLOAT32_C(  -236.25) },
      UINT8_C(226),
      { SIMDE_FLOAT32_C(  -534.70), SIMDE_FLOAT32_C(  -919.12), SIMDE_FLOAT32_C(   684.44), SIMDE_FLOAT32_C(  -599.07),
        SIMDE_FLOAT32_C(   665.53), SIMDE_FLOAT32_C(   -93.93), SIMDE_FLOAT32_C(   212.65), SIMDE_FLOAT32_C(  -191.74),
        SIMDE_FLOAT32_C(  -693.86), SIMDE_FLOAT32_C(    -8.77), SIMDE_FLOAT32_C(  -974.85), SIMDE_FLOAT32_C(   716.41),
        SIMDE_FLOAT32_C(  -273.59), SIMDE_FLOAT32_C(  -523.82), SIMDE_FLOAT32_C(    19.06), SIMDE_FLOAT32_C(   876.21) },
      { SIMDE_FLOAT32_C(   302.65), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(    98.26), SIMDE_FLOAT32_C(  -631.12),
        SIMDE_FLOAT32_C(   537.93), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(  -414.70), SIMDE_FLOAT32_C(  -151.78), SIMDE_FLOAT32_C(   673.72), SIMDE_FLOAT32_C(   242.74),
        SIMDE_FLOAT32_C(   250.35), SIMDE_FLOAT32_C(   665.88), SIMDE_FLOAT32_C(   646.74), SIMDE_FLOAT32_C(  -236.25) } },
    { { SIMDE_FLOAT32_C(   574.44), SIMDE_FLOAT32_C(   387.93), SIMDE_FLOAT32_C(   414.13), SIMDE_FLOAT32_C(  -918.18),
        SIMDE_FLOAT32_C(  -302.68), SIMDE_FLOAT32_C(  -486.61), SIMDE_FLOAT32_C(  -332.89), SIMDE_FLOAT32_C(   545.53),
        SIMDE_FLOAT32_C(  -812.89), SIMDE_FLOAT32_C(   909.85), SIMDE_FLOAT32_C(  -204.12), SIMDE_FLOAT32_C(   852.99),
        SIMDE_FLOAT32_C(   556.59), SIMDE_FLOAT32_C(   559.63), SIMDE_FLOAT32_C(  -730.10), SIMDE_FLOAT32_C(  -978.11) },
      UINT8_C(  0),
      { SIMDE_FLOAT32_C(   954.34), SIMDE_FLOAT32_C(  -577.18), SIMDE_FLOAT32_C(   306.05), SIMDE_FLOAT32_C(  -139.59),
        SIMDE_FLOAT32_C(   635.48), SIMDE_FLOAT32_C(  -885.69), SIMDE_FLOAT32_C(   166.55), SIMDE_FLOAT32_C(  -373.29),
        SIMDE_FLOAT32_C(  -860.54), SIMDE_FLOAT32_C(  -117.04), SIMDE_FLOAT32_C(   353.12), SIMDE_FLOAT32_C(  -384.37),
        SIMDE_FLOAT32_C(   902.02), SIMDE_FLOAT32_C(   229.33), SIMDE_FLOAT32_C(  -809.93), SIMDE_FLOAT32_C(   289.95) },
      { SIMDE_FLOAT32_C(   574.44), SIMDE_FLOAT32_C(   387.93), SIMDE_FLOAT32_C(   414.13), SIMDE_FLOAT32_C(  -918.18),
        SIMDE_FLOAT32_C(  -302.68), SIMDE_FLOAT32_C(  -486.61), SIMDE_FLOAT32_C(  -332.89), SIMDE_FLOAT32_C(   545.53),
        SIMDE_FLOAT32_C(  -812.89), SIMDE_FLOAT32_C(   909.85), SIMDE_FLOAT32_C(  -204.12), SIMDE_FLOAT32_C(   852.99),
        SIMDE_FLOAT32_C(   556.59), SIMDE_FLOAT32_C(   559.63), SIMDE_FLOAT32_C(  -730.10), SIMDE_FLOAT32_C(  -978.11) } },
    { { SIMDE_FLOAT32_C(  -356.54), SIMDE_FLOAT32_C(  -728.11), SIMDE_FLOAT32_C(   987.27), SIMDE_FLOAT32_C(   156.85),
        SIMDE_FLOAT32_C(   -61.00), SIMDE_FLOAT32_C(   532.80), SIMDE_FLOAT32_C(   343.96), SIMDE_FLOAT32_C(  -151.15),
        SIMDE_FLOAT32_C(  -671.32), SIMDE_FLOAT32_C(   196.95), SIMDE_FLOAT32_C(  -594.56), SIMDE_FLOAT32_C(   888.32),
        SIMDE_FLOAT32_C(   466.85), SIMDE_FLOAT32_C(  -572.66), SIMDE_FLOAT32_C(   528.83), SIMDE_FLOAT32_C(   421.19) },
      UINT8_C(129),
      { SIMDE_FLOAT32_C(  -165.12), SIMDE_FLOAT32_C(  -718.39), SIMDE_FLOAT32_C(  -514.36), SIMDE_FLOAT32_C(   -50.81),
        SIMDE_FLOAT32_C(   448.16), SIMDE_FLOAT32_C(   112.35), SIMDE_FLOAT32_C(    88.64), SIMDE_FLOAT32_C(  -668.88),
        SIMDE_FLOAT32_C(  -534.54), SIMDE_FLOAT32_C(   704.28), SIMDE_FLOAT32_C(  -766.86), SIMDE_FLOAT32_C(   694.79),
        SIMDE_FLOAT32_C(   894.35), SIMDE_FLOAT32_C(   523.08), SIMDE_FLOAT32_C(  -661.75), SIMDE_FLOAT32_C(  -833.77) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(  -728.11), SIMDE_FLOAT32_C(   987.27), SIMDE_FLOAT32_C(   156.85),
        SIMDE_FLOAT32_C(   -61.00), SIMDE_FLOAT32_C(   532.80), SIMDE_FLOAT32_C(   343.96), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(  -671.32), SIMDE_FLOAT32_C(   196.95), SIMDE_FLOAT32_C(  -594.56), SIMDE_FLOAT32_C(   888.32),
        SIMDE_FLOAT32_C(   466.85), SIMDE_FLOAT32_C(  -572.66), SIMDE_FLOAT32_C(   528.83), SIMDE_FLOAT32_C(   421.19) } },
    { { SIMDE_FLOAT32_C(   510.35), SIMDE_FLOAT32_C(   495.10), SIMDE_FLOAT32_C(   105.23), SIMDE_FLOAT32_C(    43.15),
        SIMDE_FLOAT32_C(  -160.94), SIMDE_FLOAT32_C(   954.08), SIMDE_FLOAT32_C(   371.83), SIMDE_FLOAT32_C(  -963.98),
        SIMDE_FLOAT32_C(  -640.48), SIMDE_FLOAT32_C(   260.15), SIMDE_FLOAT32_C(   502.87), SIMDE_FLOAT32_C(  -213.14),
        SIMDE_FLOAT32_C(  -211.02), SIMDE_FLOAT32_C(   -75.94), SIMDE_FLOAT32_C(   637.02), SIMDE_FLOAT32_C(   623.86) },
      UINT8_C( 36),
      { SIMDE_FLOAT32_C(  -877.34), SIMDE_FLOAT32_C(  -426.95), SIMDE_FLOAT32_C(  -346.17), SIMDE_FLOAT32_C(   235.01),
        SIMDE_FLOAT32_C(   661.70), SIMDE_FLOAT32_C(   -15.05), SIMDE_FLOAT32_C(   700.47), SIMDE_FLOAT32_C(   365.98),
        SIMDE_FLOAT32_C(   218.09), SIMDE_FLOAT32_C(   395.26), SIMDE_FLOAT32_C(   260.32), SIMDE_FLOAT32_C(  -258.83),
        SIMDE_FLOAT32_C(   733.51), SIMDE_FLOAT32_C(   426.55), SIMDE_FLOAT32_C(  -748.48), SIMDE_FLOAT32_C(   228.61) },
      { SIMDE_FLOAT32_C(   510.35), SIMDE_FLOAT32_C(   495.10), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(    43.15),
        SIMDE_FLOAT32_C(  -160.94), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(   371.83), SIMDE_FLOAT32_C(  -963.98),
        SIMDE_FLOAT32_C(  -640.48), SIMDE_FLOAT32_C(   260.15), SIMDE_FLOAT32_C(   502.87), SIMDE_FLOAT32_C(  -213.14),
        SIMDE_FLOAT32_C(  -211.02), SIMDE_FLOAT32_C(   -75.94), SIMDE_FLOAT32_C(   637.02), SIMDE_FLOAT32_C(   623.86) } },
    { { SIMDE_FLOAT32_C(  -468.22), SIMDE_FLOAT32_C(   294.67), SIMDE_FLOAT32_C(  -932.33), SIMDE_FLOAT32_C(  -514.14),
        SIMDE_FLOAT32_C(  -333.50), SIMDE_FLOAT32_C(  -896.31), SIMDE_FLOAT32_C(  -154.62), SIMDE_FLOAT32_C(   926.65),
        SIMDE_FLOAT32_C(   606.56), SIMDE_FLOAT32_C(   632.24), SIMDE_FLOAT32_C(  -284.37), SIMDE_FLOAT32_C(  -469.38),
        SIMDE_FLOAT32_C(   269.27), SIMDE_FLOAT32_C(  -660.50), SIMDE_FLOAT32_C(   736.29), SIMDE_FLOAT32_C(   391.93) },
      UINT8_C(251),
      { SIMDE_FLOAT32_C(  -609.88), SIMDE_FLOAT32_C(  -373.06), SIMDE_FLOAT32_C(  -425.75), SIMDE_FLOAT32_C(   375.07),
        SIMDE_FLOAT32_C(  -672.58), SIMDE_FLOAT32_C(   940.22), SIMDE_FLOAT32_C(  -406.85), SIMDE_FLOAT32_C(   722.68),
        SIMDE_FLOAT32_C(   200.54), SIMDE_FLOAT32_C(   334.32), SIMDE_FLOAT32_C(   456.19), SIMDE_FLOAT32_C(  -372.90),
        SIMDE_FLOAT32_C(   585.84), SIMDE_FLOAT32_C(  -315.20), SIMDE_FLOAT32_C(   158.88), SIMDE_FLOAT32_C(  -119.49) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(  -932.33), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(   606.56), SIMDE_FLOAT32_C(   632.24), SIMDE_FLOAT32_C(  -284.37), SIMDE_FLOAT32_C(  -469.38),
        SIMDE_FLOAT32_C(   269.27), SIMDE_FLOAT32_C(  -660.50), SIMDE_FLOAT32_C(   736.29), SIMDE_FLOAT32_C(   391.93) } },
    { { SIMDE_FLOAT32_C(  -247.53), SIMDE_FLOAT32_C(   644.74), SIMDE_FLOAT32_C(   547.01), SIMDE_FLOAT32_C(  -143.84),
        SIMDE_FLOAT32_C(  -509.87), SIMDE_FLOAT32_C(   473.66), SIMDE_FLOAT32_C(  -537.28), SIMDE_FLOAT32_C(  -877.63),
        SIMDE_FLOAT32_C(  -810.70), SIMDE_FLOAT32_C(    -6.66), SIMDE_FLOAT32_C(   391.64), SIMDE_FLOAT32_C(  -471.21),
        SIMDE_FLOAT32_C(  -270.37), SIMDE_FLOAT32_C(  -216.43), SIMDE_FLOAT32_C(   441.34), SIMDE_FLOAT32_C(   119.74) },
      UINT8_C(113),
      { SIMDE_FLOAT32_C(  -984.41), SIMDE_FLOAT32_C(  -505.19), SIMDE_FLOAT32_C(   737.93), SIMDE_FLOAT32_C(   955.81),
        SIMDE_FLOAT32_C(    87.96), SIMDE_FLOAT32_C(   460.61), SIMDE_FLOAT32_C(   156.35), SIMDE_FLOAT32_C(  -577.72),
        SIMDE_FLOAT32_C(   -83.20), SIMDE_FLOAT32_C(   783.45), SIMDE_FLOAT32_C(  -991.87), SIMDE_FLOAT32_C(   601.60),
        SIMDE_FLOAT32_C(   -57.67), SIMDE_FLOAT32_C(  -111.36), SIMDE_FLOAT32_C(  -645.93), SIMDE_FLOAT32_C(  -412.93) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(   644.74), SIMDE_FLOAT32_C(   547.01), SIMDE_FLOAT32_C(  -143.84),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  -877.63),
        SIMDE_FLOAT32_C(  -810.70), SIMDE_FLOAT32_C(    -6.66), SIMDE_FLOAT32_C(   391.64), SIMDE_FLOAT32_C(  -471.21),
        SIMDE_FLOAT32_C(  -270.37), SIMDE_FLOAT32_C(  -216.43), SIMDE_FLOAT32_C(   441.34), SIMDE_FLOAT32_C(   119.74) } },
    { { SIMDE_FLOAT32_C(  -564.35), SIMDE_FLOAT32_C(   210.23), SIMDE_FLOAT32_C(    77.20), SIMDE_FLOAT32_C(   909.32),
        SIMDE_FLOAT32_C(   672.96), SIMDE_FLOAT32_C(   199.57), SIMDE_FLOAT32_C(  -901.39), SIMDE_FLOAT32_C(  -333.70),
        SIMDE_FLOAT32_C(  -408.79), SIMDE_FLOAT32_C(  -372.60), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   374.78),
        SIMDE_FLOAT32_C(  -931.26), SIMDE_FLOAT32_C(  -484.33), SIMDE_FLOAT32_C(  -214.70), SIMDE_FLOAT32_C(  -915.67) },
      UINT8_C(139),
      { SIMDE_FLOAT32_C(  -476.78), SIMDE_FLOAT32_C(  -959.86), SIMDE_FLOAT32_C(  -901.56), SIMDE_FLOAT32_C(   983.83),
        SIMDE_FLOAT32_C(   196.49), SIMDE_FLOAT32_C(  -479.28), SIMDE_FLOAT32_C(   -99.37), SIMDE_FLOAT32_C(   -20.06),
        SIMDE_FLOAT32_C(  -471.16), SIMDE_FLOAT32_C(  -497.78), SIMDE_FLOAT32_C(   922.27), SIMDE_FLOAT32_C(   417.48),
        SIMDE_FLOAT32_C(  -143.71), SIMDE_FLOAT32_C(  -490.66), SIMDE_FLOAT32_C(   853.13), SIMDE_FLOAT32_C(  -933.47) },
      { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(    77.20), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(   672.96), SIMDE_FLOAT32_C(   199.57), SIMDE_FLOAT32_C(  -901.39), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(  -408.79), SIMDE_FLOAT32_C(  -372.60), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   374.78),
        SIMDE_FLOAT32_C(  -931.26), SIMDE_FLOAT32_C(  -484.33), SIMDE_FLOAT32_C(  -214.70), SIMDE_FLOAT32_C(  -915.67) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_erfc_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_erfc_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   926.55), SIMDE_FLOAT64_C(   763.10), SIMDE_FLOAT64_C(     6.18), SIMDE_FLOAT64_C(   453.38),
        SIMDE_FLOAT64_C(   184.79), SIMDE_FLOAT64_C(   608.12), SIMDE_FLOAT64_C(   303.22), SIMDE_FLOAT64_C(   429.75) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   610.63), SIMDE_FLOAT64_C(  -505.99), SIMDE_FLOAT64_C(  -566.70), SIMDE_FLOAT64_C(  -890.86),
        SIMDE_FLOAT64_C(  -469.61), SIMDE_FLOAT64_C(   -65.43), SIMDE_FLOAT64_C(  -190.70), SIMDE_FLOAT64_C(   797.08) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   883.79), SIMDE_FLOAT64_C(  -999.64), SIMDE_FLOAT64_C(   928.39), SIMDE_FLOAT64_C(  -465.63),
        SIMDE_FLOAT64_C(  -214.31), SIMDE_FLOAT64_C(   650.21), SIMDE_FLOAT64_C(   880.22), SIMDE_FLOAT64_C(  -127.39) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(   687.46), SIMDE_FLOAT64_C(  -738.40), SIMDE_FLOAT64_C(  -655.58), SIMDE_FLOAT64_C(  -737.41),
        SIMDE_FLOAT64_C(  -335.05), SIMDE_FLOAT64_C(  -354.48), SIMDE_FLOAT64_C(  -302.30), SIMDE_FLOAT64_C(  -408.50) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(  -591.38), SIMDE_FLOAT64_C(   703.88), SIMDE_FLOAT64_C(  -955.11), SIMDE_FLOAT64_C(   593.41),
        SIMDE_FLOAT64_C(   311.99), SIMDE_FLOAT64_C(   348.11), SIMDE_FLOAT64_C(    23.16), SIMDE_FLOAT64_C(   -77.38) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(   842.12), SIMDE_FLOAT64_C(   456.45), SIMDE_FLOAT64_C(    31.76), SIMDE_FLOAT64_C(  -627.49),
        SIMDE_FLOAT64_C(  -608.98), SIMDE_FLOAT64_C(   841.06), SIMDE_FLOAT64_C(  -830.41), SIMDE_FLOAT64_C(  -725.19) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(   841.43), SIMDE_FLOAT64_C(  -902.02), SIMDE_FLOAT64_C(  -190.81), SIMDE_FLOAT64_C(  -372.89),
        SIMDE_FLOAT64_C(   748.18), SIMDE_FLOAT64_C(  -310.59), SIMDE_FLOAT64_C(   499.72), SIMDE_FLOAT64_C(   435.64) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   -48.99), SIMDE_FLOAT64_C(   844.14), SIMDE_FLOAT64_C(   698.23), SIMDE_FLOAT64_C(   615.96),
        SIMDE_FLOAT64_C(  -510.34), SIMDE_FLOAT64_C(  -604.07), SIMDE_FLOAT64_C(  -792.54), SIMDE_FLOAT64_C(  -101.72) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_erfc_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_erfc_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -683.28), SIMDE_FLOAT64_C(   804.20), SIMDE_FLOAT64_C(  -404.66), SIMDE_FLOAT64_C(  -472.79),
        SIMDE_FLOAT64_C(  -863.69), SIMDE_FLOAT64_C(  -237.69), SIMDE_FLOAT64_C(  -919.11), SIMDE_FLOAT64_C(   998.91) },
      UINT8_C( 80),
      { SIMDE_FLOAT64_C(   291.91), SIMDE_FLOAT64_C(  -572.21), SIMDE_FLOAT64_C(   220.68), SIMDE_FLOAT64_C(  -193.99),
        SIMDE_FLOAT64_C(   -17.57), SIMDE_FLOAT64_C(   493.29), SIMDE_FLOAT64_C(   557.85), SIMDE_FLOAT64_C(   412.26) },
      { SIMDE_FLOAT64_C(  -683.28), SIMDE_FLOAT64_C(   804.20), SIMDE_FLOAT64_C(  -404.66), SIMDE_FLOAT64_C(  -472.79),
        SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(  -237.69), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   998.91) } },
    { { SIMDE_FLOAT64_C(   986.63), SIMDE_FLOAT64_C(  -515.33), SIMDE_FLOAT64_C(   -32.91), SIMDE_FLOAT64_C(  -333.09),
        SIMDE_FLOAT64_C(  -321.96), SIMDE_FLOAT64_C(   468.63), SIMDE_FLOAT64_C(   439.22), SIMDE_FLOAT64_C(  -104.11) },
      UINT8_C( 73),
      { SIMDE_FLOAT64_C(   199.74), SIMDE_FLOAT64_C(   522.47), SIMDE_FLOAT64_C(   516.01), SIMDE_FLOAT64_C(  -942.26),
        SIMDE_FLOAT64_C(  -623.61), SIMDE_FLOAT64_C(   832.73), SIMDE_FLOAT64_C(   861.94), SIMDE_FLOAT64_C(   -28.27) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -515.33), SIMDE_FLOAT64_C(   -32.91), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(  -321.96), SIMDE_FLOAT64_C(   468.63), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -104.11) } },
    { { SIMDE_FLOAT64_C(  -640.06), SIMDE_FLOAT64_C(   998.25), SIMDE_FLOAT64_C(   734.04), SIMDE_FLOAT64_C(  -559.17),
        SIMDE_FLOAT64_C(   997.17), SIMDE_FLOAT64_C(  -856.00), SIMDE_FLOAT64_C(   732.74), SIMDE_FLOAT64_C(  -575.04) },
      UINT8_C(158),
      { SIMDE_FLOAT64_C(  -461.24), SIMDE_FLOAT64_C(   407.39), SIMDE_FLOAT64_C(  -142.02), SIMDE_FLOAT64_C(  -903.39),
        SIMDE_FLOAT64_C(  -180.35), SIMDE_FLOAT64_C(  -155.40), SIMDE_FLOAT64_C(  -418.72), SIMDE_FLOAT64_C(   786.74) },
      { SIMDE_FLOAT64_C(  -640.06), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(  -856.00), SIMDE_FLOAT64_C(   732.74), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   511.51), SIMDE_FLOAT64_C(   259.32), SIMDE_FLOAT64_C(   255.37), SIMDE_FLOAT64_C(   -49.27),
        SIMDE_FLOAT64_C(  -844.79), SIMDE_FLOAT64_C(   939.27), SIMDE_FLOAT64_C(  -849.53), SIMDE_FLOAT64_C(   677.68) },
      UINT8_C(184),
      { SIMDE_FLOAT64_C(  -791.79), SIMDE_FLOAT64_C(  -945.93), SIMDE_FLOAT64_C(   288.01), SIMDE_FLOAT64_C(  -929.85),
        SIMDE_FLOAT64_C(    25.80), SIMDE_FLOAT64_C(   647.95), SIMDE_FLOAT64_C(  -931.60), SIMDE_FLOAT64_C(  -240.16) },
      { SIMDE_FLOAT64_C(   511.51), SIMDE_FLOAT64_C(   259.32), SIMDE_FLOAT64_C(   255.37), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -849.53), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(  -911.22), SIMDE_FLOAT64_C(  -934.43), SIMDE_FLOAT64_C(   -96.16), SIMDE_FLOAT64_C(   821.52),
        SIMDE_FLOAT64_C(  -509.47), SIMDE_FLOAT64_C(  -731.47), SIMDE_FLOAT64_C(  -639.72), SIMDE_FLOAT64_C(   897.92) },
      UINT8_C(176),
      { SIMDE_FLOAT64_C(  -543.12), SIMDE_FLOAT64_C(  -282.43), SIMDE_FLOAT64_C(   971.11), SIMDE_FLOAT64_C(    38.16),
        SIMDE_FLOAT64_C(  -495.70), SIMDE_FLOAT64_C(   482.61), SIMDE_FLOAT64_C(  -702.52), SIMDE_FLOAT64_C(   759.67) },
      { SIMDE_FLOAT64_C(  -911.22), SIMDE_FLOAT64_C(  -934.43), SIMDE_FLOAT64_C(   -96.16), SIMDE_FLOAT64_C(   821.52),
        SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -639.72), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -566.66), SIMDE_FLOAT64_C(  -547.31), SIMDE_FLOAT64_C(   698.94), SIMDE_FLOAT64_C(  -416.19),
        SIMDE_FLOAT64_C(  -869.63), SIMDE_FLOAT64_C(   154.22), SIMDE_FLOAT64_C(  -207.98), SIMDE_FLOAT64_C(  -815.57) },
      UINT8_C(142),
      { SIMDE_FLOAT64_C(  -137.83), SIMDE_FLOAT64_C(   210.23), SIMDE_FLOAT64_C(  -909.82), SIMDE_FLOAT64_C(   -69.43),
        SIMDE_FLOAT64_C(   970.07), SIMDE_FLOAT64_C(  -821.05), SIMDE_FLOAT64_C(    -3.87), SIMDE_FLOAT64_C(  -126.08) },
      { SIMDE_FLOAT64_C(  -566.66), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(  -869.63), SIMDE_FLOAT64_C(   154.22), SIMDE_FLOAT64_C(  -207.98), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(  -999.53), SIMDE_FLOAT64_C(   486.66), SIMDE_FLOAT64_C(   142.44), SIMDE_FLOAT64_C(  -639.25),
        SIMDE_FLOAT64_C(   384.58), SIMDE_FLOAT64_C(  -731.05), SIMDE_FLOAT64_C(  -182.37), SIMDE_FLOAT64_C(  -897.86) },
      UINT8_C(227),
      { SIMDE_FLOAT64_C(   855.79), SIMDE_FLOAT64_C(  -393.55), SIMDE_FLOAT64_C(   722.67), SIMDE_FLOAT64_C(  -846.73),
        SIMDE_FLOAT64_C(  -633.88), SIMDE_FLOAT64_C(  -843.99), SIMDE_FLOAT64_C(  -394.03), SIMDE_FLOAT64_C(  -934.94) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(   142.44), SIMDE_FLOAT64_C(  -639.25),
        SIMDE_FLOAT64_C(   384.58), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(  -260.18), SIMDE_FLOAT64_C(  -263.67), SIMDE_FLOAT64_C(   219.28), SIMDE_FLOAT64_C(   531.84),
        SIMDE_FLOAT64_C(   -79.23), SIMDE_FLOAT64_C(   661.51), SIMDE_FLOAT64_C(  -605.99), SIMDE_FLOAT64_C(  -869.00) },
      UINT8_C( 64),
      { SIMDE_FLOAT64_C(   324.57), SIMDE_FLOAT64_C(  -898.93), SIMDE_FLOAT64_C(   930.64), SIMDE_FLOAT64_C(  -679.29),
        SIMDE_FLOAT64_C(   -25.01), SIMDE_FLOAT64_C(   931.11), SIMDE_FLOAT64_C(   807.37), SIMDE_FLOAT64_C(  -882.57) },
      { SIMDE_FLOAT64_C(  -260.18), SIMDE_FLOAT64_C(  -263.67), SIMDE_FLOAT64_C(   219.28), SIMDE_FLOAT64_C(   531.84),
        SIMDE_FLOAT64_C(   -79.23), SIMDE_FLOAT64_C(   661.51), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -869.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_erfc_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_erfcinv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     1.16) },
      { SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(    -0.14) } },
    { { SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     1.53) },
      { SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(    -0.51) } },
    { { SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     1.68) },
      { SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.70) } },
    { { SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(     1.66), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     1.42) },
      { SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(     1.10), SIMDE_FLOAT32_C(    -0.39) } },
    { { SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     1.51) },
      { SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(    -0.49) } },
    { { SIMDE_FLOAT32_C(     1.44), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.49) },
      { SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     0.49) } },
    { { SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     1.35) },
      { SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -1.82), SIMDE_FLOAT32_C(    -0.32) } },
    { { SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     1.62) },
      { SIMDE_FLOAT32_C(    -1.33), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -0.62) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_erfcinv_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_erfcinv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.73), SIMDE_FLOAT64_C(     0.13) },
      { SIMDE_FLOAT64_C(     0.24), SIMDE_FLOAT64_C(     1.07) } },
    { { SIMDE_FLOAT64_C(     1.09), SIMDE_FLOAT64_C(     0.70) },
      { SIMDE_FLOAT64_C(    -0.08), SIMDE_FLOAT64_C(     0.27) } },
    { { SIMDE_FLOAT64_C(     1.13), SIMDE_FLOAT64_C(     0.97) },
      { SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     0.03) } },
    { { SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     1.72) },
      { SIMDE_FLOAT64_C(     0.53), SIMDE_FLOAT64_C(    -0.76) } },
    { { SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.82) },
      { SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.16) } },
    { { SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     1.88) },
      { SIMDE_FLOAT64_C(     1.82), SIMDE_FLOAT64_C(    -1.10) } },
    { { SIMDE_FLOAT64_C(     1.11), SIMDE_FLOAT64_C(     0.87) },
      { SIMDE_FLOAT64_C(    -0.10), SIMDE_FLOAT64_C(     0.12) } },
    { { SIMDE_FLOAT64_C(     1.13), SIMDE_FLOAT64_C(     0.05) },
      { SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     1.39) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_erfcinv_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_erfcinv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     1.58), SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     0.95),
        SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     1.58), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.73) },
      { SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.24) } },
    { { SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     1.80), SIMDE_FLOAT32_C(     1.88),
        SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     1.44), SIMDE_FLOAT32_C(     1.26) },
      { SIMDE_FLOAT32_C(    -1.28), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(    -1.10),
        SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -0.23) } },
    { { SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     1.32),
        SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     1.77) },
      { SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.29),
        SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    -1.53), SIMDE_FLOAT32_C(    -0.85) } },
    { { SIMDE_FLOAT32_C(     1.71), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     1.99),
        SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     1.58), SIMDE_FLOAT32_C(     0.11) },
      { SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(    -1.82),
        SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     1.13) } },
    { { SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(     1.66),
        SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.81) },
      { SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.67),
        SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.17) } },
    { { SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     1.05),
        SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     1.23) },
      { SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.04),
        SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(    -0.21) } },
    { { SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.02),
        SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     1.80), SIMDE_FLOAT32_C(     1.19) },
      { SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     1.64),
        SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(    -0.17) } },
    { { SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     1.92),
        SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     1.75) },
      { SIMDE_FLOAT32_C(    -1.09), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(    -1.24),
        SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(    -0.81) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_erfcinv_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_erfcinv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     1.66), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(     1.56) },
      { SIMDE_FLOAT64_C(    -0.67), SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(     1.07), SIMDE_FLOAT64_C(    -0.55) } },
    { { SIMDE_FLOAT64_C(     1.89), SIMDE_FLOAT64_C(     0.26), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(     0.46) },
      { SIMDE_FLOAT64_C(    -1.13), SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     0.52) } },
    { { SIMDE_FLOAT64_C(     1.50), SIMDE_FLOAT64_C(     1.78), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     0.70) },
      { SIMDE_FLOAT64_C(    -0.48), SIMDE_FLOAT64_C(    -0.87), SIMDE_FLOAT64_C(     0.02), SIMDE_FLOAT64_C(     0.27) } },
    { { SIMDE_FLOAT64_C(     1.88), SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     1.75) },
      { SIMDE_FLOAT64_C(    -1.10), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(    -0.81) } },
    { { SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(     1.15), SIMDE_FLOAT64_C(     0.52) },
      { SIMDE_FLOAT64_C(     0.85), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(    -0.13), SIMDE_FLOAT64_C(     0.45) } },
    { { SIMDE_FLOAT64_C(     0.16), SIMDE_FLOAT64_C(     1.48), SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(     1.38) },
      { SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(     1.10), SIMDE_FLOAT64_C(    -0.35) } },
    { { SIMDE_FLOAT64_C(     1.88), SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(     1.09), SIMDE_FLOAT64_C(     0.47) },
      { SIMDE_FLOAT64_C(    -1.09), SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(    -0.08), SIMDE_FLOAT64_C(     0.51) } },
    { { SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     1.43), SIMDE_FLOAT64_C(     1.79), SIMDE_FLOAT64_C(     0.34) },
      { SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(    -0.40), SIMDE_FLOAT64_C(    -0.89), SIMDE_FLOAT64_C(     0.67) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_erfcinv_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_erfcinv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     1.41), SIMDE_FLOAT32_C(     1.20),
        SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(     1.37),
        SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.44),
        SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     1.79) },
      { SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.18),
        SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.34),
        SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.41),
        SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.89) } },
    { { SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(     1.40), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     1.60),
        SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     1.10),
        SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(     1.67), SIMDE_FLOAT32_C(     0.93),
        SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.13) },
      { SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.60),
        SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(    -0.09),
        SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     1.07) } },
    { { SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     1.97),
        SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     1.41),
        SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     1.79),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     1.58), SIMDE_FLOAT32_C(     0.30) },
      { SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -1.53),
        SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -0.38),
        SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(    -0.89),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     0.73) } },
    { { SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(     1.90), SIMDE_FLOAT32_C(     1.78),
        SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(     1.62), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     1.54),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.49),
        SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.95) },
      { SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(    -1.16), SIMDE_FLOAT32_C(    -0.87),
        SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.52),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.49),
        SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.04) } },
    { { SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.90),
        SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.63),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     1.48),
        SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     1.78), SIMDE_FLOAT32_C(     0.64) },
      { SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(    -1.39), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.09),
        SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.45),
        SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(     0.33) } },
    { { SIMDE_FLOAT32_C(     1.12), SIMDE_FLOAT32_C(     1.68), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.58),
        SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.29),
        SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     1.20),
        SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     1.41), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     1.21) },
      { SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.39),
        SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     1.13), SIMDE_FLOAT32_C(     0.75),
        SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.18),
        SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(     1.02), SIMDE_FLOAT32_C(    -0.19) } },
    { { SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.48),
        SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     1.10), SIMDE_FLOAT32_C(     1.81),
        SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(     1.52), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     1.79),
        SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.65) },
      { SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(     1.10), SIMDE_FLOAT32_C(     0.50),
        SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(    -0.93),
        SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(    -0.89),
        SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.32) } },
    { { SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     1.99),
        SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(     1.13), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     1.06),
        SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     1.90) },
      { SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(     1.39),
        SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -1.82),
        SIMDE_FLOAT32_C(    -1.24), SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.05),
        SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(    -1.16) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_erfcinv_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_erfcinv_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     1.62), SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     1.18),
        SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(     0.55),
        SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     1.50),
        SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.58) },
      UINT8_C(239),
      { SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     1.31),
        SIMDE_FLOAT32_C(     1.27), SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     1.64),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     1.77),
        SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.47) },
      { SIMDE_FLOAT32_C(    -1.24), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(    -0.28),
        SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(    -0.65),
        SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     1.50),
        SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.58) } },
    { { SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     1.08),
        SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     0.97),
        SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     1.18) },
      UINT8_C( 23),
      { SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     1.69),
        SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     1.34),
        SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     1.92),
        SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(     0.67) },
      { SIMDE_FLOAT32_C(    -1.82), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     1.08),
        SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     0.97),
        SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     1.18) } },
    { { SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     1.32), SIMDE_FLOAT32_C(     1.88),
        SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     1.49),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     0.96) },
      UINT8_C( 91),
      { SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     1.01),
        SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     1.74),
        SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     0.40),
        SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     1.64) },
      { SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     1.32), SIMDE_FLOAT32_C(    -0.01),
        SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     1.49),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     0.96) } },
    { { SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     1.98),
        SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     1.61),
        SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     1.68),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     1.65) },
      UINT8_C(144),
      { SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     1.07),
        SIMDE_FLOAT32_C(     1.61), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.47),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     1.31),
        SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.88) },
      { SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     1.98),
        SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.51),
        SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     1.68),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     1.65) } },
    { { SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     1.03),
        SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     1.52), SIMDE_FLOAT32_C(     1.62),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     1.61), SIMDE_FLOAT32_C(     0.53),
        SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     1.26), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.08) },
      UINT8_C(233),
      { SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     1.15), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     1.46),
        SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     1.15),
        SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     1.71), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     1.99),
        SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     1.69) },
      { SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.43),
        SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -1.82), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(    -0.13),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     1.61), SIMDE_FLOAT32_C(     0.53),
        SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     1.26), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.08) } },
    { { SIMDE_FLOAT32_C(     1.44), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     1.85),
        SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.29),
        SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     1.74),
        SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     1.67), SIMDE_FLOAT32_C(     0.31) },
      UINT8_C(221),
      { SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     1.19),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     1.88),
        SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.56),
        SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     0.47) },
      { SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(    -0.17),
        SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -1.10),
        SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     1.74),
        SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     1.67), SIMDE_FLOAT32_C(     0.31) } },
    { { SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     0.40),
        SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     1.62),
        SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     1.48) },
      UINT8_C(108),
      { SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.89),
        SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     1.40),
        SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.38), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     1.15),
        SIMDE_FLOAT32_C(     1.38), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     1.22) },
      { SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.10),
        SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     1.62),
        SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     1.48) } },
    { { SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(     0.19),
        SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.58),
        SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.60),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     1.93) },
      UINT8_C(110),
      { SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     1.50),
        SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     1.01),
        SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.53),
        SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     1.61), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.26) },
      { SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(    -0.48),
        SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.58),
        SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.60),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     1.93) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_erfcinv_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_erfcinv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     1.62), SIMDE_FLOAT64_C(     0.30),
        SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     1.24), SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     1.25) },
      { SIMDE_FLOAT64_C(     1.28), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(    -0.62), SIMDE_FLOAT64_C(     0.73),
        SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(    -0.22), SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(    -0.23) } },
    { { SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     1.75), SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(     0.09),
        SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     1.42), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.26) },
      { SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(    -0.81), SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     1.20),
        SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(    -0.39), SIMDE_FLOAT64_C(     1.16), SIMDE_FLOAT64_C(     0.80) } },
    { { SIMDE_FLOAT64_C(     1.39), SIMDE_FLOAT64_C(     1.04), SIMDE_FLOAT64_C(     1.66), SIMDE_FLOAT64_C(     0.51),
        SIMDE_FLOAT64_C(     1.47), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     1.28), SIMDE_FLOAT64_C(     0.25) },
      { SIMDE_FLOAT64_C(    -0.36), SIMDE_FLOAT64_C(    -0.04), SIMDE_FLOAT64_C(    -0.67), SIMDE_FLOAT64_C(     0.47),
        SIMDE_FLOAT64_C(    -0.44), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(    -0.25), SIMDE_FLOAT64_C(     0.81) } },
    { { SIMDE_FLOAT64_C(     1.45), SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     1.98), SIMDE_FLOAT64_C(     0.38),
        SIMDE_FLOAT64_C(     1.21), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     1.28) },
      { SIMDE_FLOAT64_C(    -0.42), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(    -1.64), SIMDE_FLOAT64_C(     0.62),
        SIMDE_FLOAT64_C(    -0.19), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(    -0.25) } },
    { { SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(     0.29), SIMDE_FLOAT64_C(     1.58), SIMDE_FLOAT64_C(     1.25),
        SIMDE_FLOAT64_C(     1.53), SIMDE_FLOAT64_C(     1.76), SIMDE_FLOAT64_C(     0.50), SIMDE_FLOAT64_C(     0.21) },
      { SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.75), SIMDE_FLOAT64_C(    -0.57), SIMDE_FLOAT64_C(    -0.23),
        SIMDE_FLOAT64_C(    -0.51), SIMDE_FLOAT64_C(    -0.83), SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(     0.89) } },
    { { SIMDE_FLOAT64_C(     1.51), SIMDE_FLOAT64_C(     1.01), SIMDE_FLOAT64_C(     0.29), SIMDE_FLOAT64_C(     1.94),
        SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(     1.83) },
      { SIMDE_FLOAT64_C(    -0.49), SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(     0.75), SIMDE_FLOAT64_C(    -1.33),
        SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(    -0.97) } },
    { { SIMDE_FLOAT64_C(     1.43), SIMDE_FLOAT64_C(     1.86), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.90),
        SIMDE_FLOAT64_C(     0.64), SIMDE_FLOAT64_C(     1.62), SIMDE_FLOAT64_C(     1.15), SIMDE_FLOAT64_C(     0.09) },
      { SIMDE_FLOAT64_C(    -0.40), SIMDE_FLOAT64_C(    -1.04), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     0.09),
        SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(    -0.62), SIMDE_FLOAT64_C(    -0.13), SIMDE_FLOAT64_C(     1.20) } },
    { { SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     1.13), SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     1.76),
        SIMDE_FLOAT64_C(     1.48), SIMDE_FLOAT64_C(     1.14), SIMDE_FLOAT64_C(     1.04), SIMDE_FLOAT64_C(     0.21) },
      { SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(    -0.83),
        SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(    -0.04), SIMDE_FLOAT64_C(     0.89) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_erfcinv_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_erfcinv_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.49), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     1.36), SIMDE_FLOAT64_C(     1.68),
        SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(     1.10), SIMDE_FLOAT64_C(     0.08) },
      UINT8_C(117),
      { SIMDE_FLOAT64_C(     0.85), SIMDE_FLOAT64_C(     1.24), SIMDE_FLOAT64_C(     1.32), SIMDE_FLOAT64_C(     0.39),
        SIMDE_FLOAT64_C(     1.15), SIMDE_FLOAT64_C(     1.13), SIMDE_FLOAT64_C(     0.65), SIMDE_FLOAT64_C(     0.74) },
      { SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(     1.68),
        SIMDE_FLOAT64_C(    -0.13), SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(     0.08) } },
    { { SIMDE_FLOAT64_C(     1.75), SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     1.37),
        SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     1.22), SIMDE_FLOAT64_C(     1.43), SIMDE_FLOAT64_C(     0.91) },
      UINT8_C( 90),
      { SIMDE_FLOAT64_C(     0.35), SIMDE_FLOAT64_C(     1.48), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     1.46),
        SIMDE_FLOAT64_C(     1.53), SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(     0.89) },
      { SIMDE_FLOAT64_C(     1.75), SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(    -0.43),
        SIMDE_FLOAT64_C(    -0.51), SIMDE_FLOAT64_C(     1.22), SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.91) } },
    { { SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     1.01), SIMDE_FLOAT64_C(     1.06), SIMDE_FLOAT64_C(     1.33),
        SIMDE_FLOAT64_C(     1.08), SIMDE_FLOAT64_C(     1.27), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(     0.33) },
      UINT8_C(134),
      { SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     1.48), SIMDE_FLOAT64_C(     1.71), SIMDE_FLOAT64_C(     1.22),
        SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     1.46), SIMDE_FLOAT64_C(     1.67), SIMDE_FLOAT64_C(     1.01) },
      { SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(    -0.75), SIMDE_FLOAT64_C(     1.33),
        SIMDE_FLOAT64_C(     1.08), SIMDE_FLOAT64_C(     1.27), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(    -0.01) } },
    { { SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     0.26),
        SIMDE_FLOAT64_C(     1.34), SIMDE_FLOAT64_C(     1.77), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(     0.83) },
      UINT8_C(179),
      { SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     0.51),
        SIMDE_FLOAT64_C(     1.25), SIMDE_FLOAT64_C(     0.60), SIMDE_FLOAT64_C(     1.52), SIMDE_FLOAT64_C(     0.31) },
      { SIMDE_FLOAT64_C(     1.28), SIMDE_FLOAT64_C(     0.65), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     0.26),
        SIMDE_FLOAT64_C(    -0.23), SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(     0.72) } },
    { { SIMDE_FLOAT64_C(     1.92), SIMDE_FLOAT64_C(     0.60), SIMDE_FLOAT64_C(     1.58), SIMDE_FLOAT64_C(     0.10),
        SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     0.16), SIMDE_FLOAT64_C(     0.66), SIMDE_FLOAT64_C(     0.41) },
      UINT8_C(115),
      { SIMDE_FLOAT64_C(     1.87), SIMDE_FLOAT64_C(     0.63), SIMDE_FLOAT64_C(     1.34), SIMDE_FLOAT64_C(     1.54),
        SIMDE_FLOAT64_C(     1.64), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(     1.97), SIMDE_FLOAT64_C(     1.86) },
      { SIMDE_FLOAT64_C(    -1.07), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     1.58), SIMDE_FLOAT64_C(     0.10),
        SIMDE_FLOAT64_C(    -0.65), SIMDE_FLOAT64_C(     0.97), SIMDE_FLOAT64_C(    -1.53), SIMDE_FLOAT64_C(     0.41) } },
    { { SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     1.32), SIMDE_FLOAT64_C(     1.63), SIMDE_FLOAT64_C(     1.04),
        SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     1.46), SIMDE_FLOAT64_C(     1.11), SIMDE_FLOAT64_C(     0.50) },
      UINT8_C( 31),
      { SIMDE_FLOAT64_C(     1.62), SIMDE_FLOAT64_C(     1.75), SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     1.14),
        SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     1.74), SIMDE_FLOAT64_C(     1.64) },
      { SIMDE_FLOAT64_C(    -0.62), SIMDE_FLOAT64_C(    -0.81), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(    -0.12),
        SIMDE_FLOAT64_C(     1.33), SIMDE_FLOAT64_C(     1.46), SIMDE_FLOAT64_C(     1.11), SIMDE_FLOAT64_C(     0.50) } },
    { { SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     1.79), SIMDE_FLOAT64_C(     1.11),
        SIMDE_FLOAT64_C(     1.08), SIMDE_FLOAT64_C(     1.67), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     1.71) },
      UINT8_C(152),
      { SIMDE_FLOAT64_C(     0.53), SIMDE_FLOAT64_C(     1.35), SIMDE_FLOAT64_C(     1.17), SIMDE_FLOAT64_C(     0.50),
        SIMDE_FLOAT64_C(     1.21), SIMDE_FLOAT64_C(     1.60), SIMDE_FLOAT64_C(     1.82), SIMDE_FLOAT64_C(     0.85) },
      { SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     1.79), SIMDE_FLOAT64_C(     0.48),
        SIMDE_FLOAT64_C(    -0.19), SIMDE_FLOAT64_C(     1.67), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     0.13) } },
    { { SIMDE_FLOAT64_C(     0.64), SIMDE_FLOAT64_C(     1.96), SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     1.74),
        SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     1.37), SIMDE_FLOAT64_C(     0.21) },
      UINT8_C( 95),
      { SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.25),
        SIMDE_FLOAT64_C(     1.91), SIMDE_FLOAT64_C(     1.38), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     1.70) },
      { SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.81),
        SIMDE_FLOAT64_C(    -1.20), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.21) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_erfcinv_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_exp_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(    -1.86), SIMDE_FLOAT32_C(     1.40), SIMDE_FLOAT32_C(     3.13) },
      { SIMDE_FLOAT32_C(     2.66), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     4.06), SIMDE_FLOAT32_C(    22.87) } },
    { { SIMDE_FLOAT32_C(    -1.01), SIMDE_FLOAT32_C(    -1.34), SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(    -0.13) },
      { SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    15.96), SIMDE_FLOAT32_C(     0.88) } },
    { { SIMDE_FLOAT32_C(    -2.37), SIMDE_FLOAT32_C(     2.01), SIMDE_FLOAT32_C(    -3.83), SIMDE_FLOAT32_C(    -3.05) },
      { SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     7.46), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(    -1.44), SIMDE_FLOAT32_C(     3.85), SIMDE_FLOAT32_C(     2.66) },
      { SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(    46.99), SIMDE_FLOAT32_C(    14.30) } },
    { { SIMDE_FLOAT32_C(    -1.62), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     2.09) },
      { SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     1.32), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     8.08) } },
    { { SIMDE_FLOAT32_C(    -1.46), SIMDE_FLOAT32_C(    -3.87), SIMDE_FLOAT32_C(    -1.51), SIMDE_FLOAT32_C(    -0.90) },
      { SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.41) } },
    { { SIMDE_FLOAT32_C(    -1.48), SIMDE_FLOAT32_C(     3.26), SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(     2.62) },
      { SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(    26.05), SIMDE_FLOAT32_C(    22.42), SIMDE_FLOAT32_C(    13.74) } },
    { { SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(    -1.27), SIMDE_FLOAT32_C(    -0.09) },
      { SIMDE_FLOAT32_C(    18.54), SIMDE_FLOAT32_C(    12.43), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.91) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_exp_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_exp_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    -2.66), SIMDE_FLOAT64_C(    -2.80) },
      { SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.06) } },
    { { SIMDE_FLOAT64_C(    -3.89), SIMDE_FLOAT64_C(    -1.37) },
      { SIMDE_FLOAT64_C(     0.02), SIMDE_FLOAT64_C(     0.25) } },
    { { SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(    -2.64) },
      { SIMDE_FLOAT64_C(     1.25), SIMDE_FLOAT64_C(     0.07) } },
    { { SIMDE_FLOAT64_C(    -3.57), SIMDE_FLOAT64_C(    -2.12) },
      { SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.12) } },
    { { SIMDE_FLOAT64_C(     1.63), SIMDE_FLOAT64_C(     1.90) },
      { SIMDE_FLOAT64_C(     5.10), SIMDE_FLOAT64_C(     6.69) } },
    { { SIMDE_FLOAT64_C(    -3.29), SIMDE_FLOAT64_C(     2.38) },
      { SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(    10.80) } },
    { { SIMDE_FLOAT64_C(     2.98), SIMDE_FLOAT64_C(    -3.59) },
      { SIMDE_FLOAT64_C(    19.69), SIMDE_FLOAT64_C(     0.03) } },
    { { SIMDE_FLOAT64_C(     1.60), SIMDE_FLOAT64_C(     3.03) },
      { SIMDE_FLOAT64_C(     4.95), SIMDE_FLOAT64_C(    20.70) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_exp_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_exp_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     3.31),
        SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(    -1.86), SIMDE_FLOAT32_C(    -0.07) },
      { SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    27.39),
        SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.93) } },
    { { SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     2.79),
        SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -2.09), SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(    -2.83) },
      { SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    34.81), SIMDE_FLOAT32_C(    16.28),
        SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     6.82), SIMDE_FLOAT32_C(     0.06) } },
    { { SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     3.78), SIMDE_FLOAT32_C(     1.32), SIMDE_FLOAT32_C(     0.41),
        SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     3.24), SIMDE_FLOAT32_C(    -3.08), SIMDE_FLOAT32_C(     1.67) },
      { SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    43.82), SIMDE_FLOAT32_C(     3.74), SIMDE_FLOAT32_C(     1.51),
        SIMDE_FLOAT32_C(    36.60), SIMDE_FLOAT32_C(    25.53), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     5.31) } },
    { { SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(    -0.47),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -3.11), SIMDE_FLOAT32_C(    -2.54), SIMDE_FLOAT32_C(    -2.91) },
      { SIMDE_FLOAT32_C(    30.57), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     0.63),
        SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -3.60), SIMDE_FLOAT32_C(    -3.49),
        SIMDE_FLOAT32_C(    -1.85), SIMDE_FLOAT32_C(    -1.46), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     1.26) },
      { SIMDE_FLOAT32_C(     2.29), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     3.53) } },
    { { SIMDE_FLOAT32_C(     2.29), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -1.11),
        SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(    -2.02), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(    -2.75) },
      { SIMDE_FLOAT32_C(     9.87), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     0.33),
        SIMDE_FLOAT32_C(     6.62), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(     0.06) } },
    { { SIMDE_FLOAT32_C(    -2.25), SIMDE_FLOAT32_C(    -2.61), SIMDE_FLOAT32_C(     1.66), SIMDE_FLOAT32_C(    -2.65),
        SIMDE_FLOAT32_C(    -3.37), SIMDE_FLOAT32_C(     2.59), SIMDE_FLOAT32_C(     3.02), SIMDE_FLOAT32_C(    -3.95) },
      { SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     5.26), SIMDE_FLOAT32_C(     0.07),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    13.33), SIMDE_FLOAT32_C(    20.49), SIMDE_FLOAT32_C(     0.02) } },
    { { SIMDE_FLOAT32_C(    -1.74), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     2.73),
        SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     3.82), SIMDE_FLOAT32_C(    -2.64) },
      { SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    15.33),
        SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(    45.60), SIMDE_FLOAT32_C(     0.07) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_exp_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_exp_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     3.29), SIMDE_FLOAT64_C(    -2.77), SIMDE_FLOAT64_C(     3.69), SIMDE_FLOAT64_C(    -0.61) },
      { SIMDE_FLOAT64_C(    26.84), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(    40.04), SIMDE_FLOAT64_C(     0.54) } },
    { { SIMDE_FLOAT64_C(    -1.69), SIMDE_FLOAT64_C(     0.88), SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(     1.60) },
      { SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     2.41), SIMDE_FLOAT64_C(     2.29), SIMDE_FLOAT64_C(     4.95) } },
    { { SIMDE_FLOAT64_C(    -2.30), SIMDE_FLOAT64_C(     2.39), SIMDE_FLOAT64_C(    -1.55), SIMDE_FLOAT64_C(    -3.39) },
      { SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(    10.91), SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(     0.03) } },
    { { SIMDE_FLOAT64_C(     3.91), SIMDE_FLOAT64_C(    -3.26), SIMDE_FLOAT64_C(    -0.58), SIMDE_FLOAT64_C(    -1.96) },
      { SIMDE_FLOAT64_C(    49.90), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.14) } },
    { { SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     2.77), SIMDE_FLOAT64_C(    -1.45), SIMDE_FLOAT64_C(    -1.25) },
      { SIMDE_FLOAT64_C(     1.08), SIMDE_FLOAT64_C(    15.96), SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.29) } },
    { { SIMDE_FLOAT64_C(    -1.13), SIMDE_FLOAT64_C(     2.76), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     2.44) },
      { SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(    15.80), SIMDE_FLOAT64_C(     2.69), SIMDE_FLOAT64_C(    11.47) } },
    { { SIMDE_FLOAT64_C(    -1.89), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(     0.58) },
      { SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     1.01), SIMDE_FLOAT64_C(     1.72), SIMDE_FLOAT64_C(     1.79) } },
    { { SIMDE_FLOAT64_C(     1.40), SIMDE_FLOAT64_C(    -0.80), SIMDE_FLOAT64_C(     1.70), SIMDE_FLOAT64_C(     0.69) },
      { SIMDE_FLOAT64_C(     4.06), SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     5.47), SIMDE_FLOAT64_C(     1.99) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_exp_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_exp_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -2.09), SIMDE_FLOAT32_C(    -2.90),
        SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(    -2.98), SIMDE_FLOAT32_C(    -3.94),
        SIMDE_FLOAT32_C(    -1.92), SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(    -0.28),
        SIMDE_FLOAT32_C(     2.19), SIMDE_FLOAT32_C(    -2.64), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     3.50) },
      { SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    48.42), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.02),
        SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(    18.92), SIMDE_FLOAT32_C(     6.55), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     8.94), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    33.12) } },
    { { SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     2.28), SIMDE_FLOAT32_C(    -3.90), SIMDE_FLOAT32_C(     1.18),
        SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(    -1.46), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(     2.83),
        SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -2.52), SIMDE_FLOAT32_C(     2.98), SIMDE_FLOAT32_C(     1.96),
        SIMDE_FLOAT32_C(    -2.99), SIMDE_FLOAT32_C(    -2.11), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     2.29) },
      { SIMDE_FLOAT32_C(    21.54), SIMDE_FLOAT32_C(     9.78), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     3.25),
        SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    16.95),
        SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(    19.69), SIMDE_FLOAT32_C(     7.10),
        SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     3.19), SIMDE_FLOAT32_C(     9.87) } },
    { { SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(    -2.62),
        SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(    -3.59), SIMDE_FLOAT32_C(    -2.56), SIMDE_FLOAT32_C(    -2.97),
        SIMDE_FLOAT32_C(     3.35), SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     1.54),
        SIMDE_FLOAT32_C(    -3.32), SIMDE_FLOAT32_C(    -3.62), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     3.75) },
      { SIMDE_FLOAT32_C(     6.96), SIMDE_FLOAT32_C(    21.54), SIMDE_FLOAT32_C(    29.67), SIMDE_FLOAT32_C(     0.07),
        SIMDE_FLOAT32_C(    19.11), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(    28.50), SIMDE_FLOAT32_C(    27.94), SIMDE_FLOAT32_C(     2.12), SIMDE_FLOAT32_C(     4.66),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(    42.52) } },
    { { SIMDE_FLOAT32_C(     2.66), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(    -1.93),
        SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(    -3.45), SIMDE_FLOAT32_C(    -3.09), SIMDE_FLOAT32_C(    -0.37),
        SIMDE_FLOAT32_C(    -1.97), SIMDE_FLOAT32_C(     3.89), SIMDE_FLOAT32_C(    -2.41), SIMDE_FLOAT32_C(    -0.96),
        SIMDE_FLOAT32_C(    -2.21), SIMDE_FLOAT32_C(     2.75), SIMDE_FLOAT32_C(    -2.67), SIMDE_FLOAT32_C(     3.72) },
      { SIMDE_FLOAT32_C(    14.30), SIMDE_FLOAT32_C(     3.13), SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     0.15),
        SIMDE_FLOAT32_C(    40.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.69),
        SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    48.91), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.38),
        SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    15.64), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(    41.26) } },
    { { SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(    -3.27), SIMDE_FLOAT32_C(    -2.90), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(    -2.86), SIMDE_FLOAT32_C(    -1.45), SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(    -3.51),
        SIMDE_FLOAT32_C(    -2.13), SIMDE_FLOAT32_C(    -1.46), SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(    -1.45),
        SIMDE_FLOAT32_C(    -1.08), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -1.69), SIMDE_FLOAT32_C(    -2.41) },
      { SIMDE_FLOAT32_C(     6.17), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     2.16),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     5.99), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     7.61), SIMDE_FLOAT32_C(     0.23),
        SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.09) } },
    { { SIMDE_FLOAT32_C(    -3.79), SIMDE_FLOAT32_C(     3.24), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     3.90),
        SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -2.17),
        SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     1.13), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     2.24),
        SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(    -2.30) },
      { SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(    25.53), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(    49.40),
        SIMDE_FLOAT32_C(    44.70), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.11),
        SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     3.10), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     9.39),
        SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     9.03), SIMDE_FLOAT32_C(     7.10), SIMDE_FLOAT32_C(     0.10) } },
    { { SIMDE_FLOAT32_C(     2.93), SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     2.46), SIMDE_FLOAT32_C(    -3.93),
        SIMDE_FLOAT32_C(    -2.39), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -3.44), SIMDE_FLOAT32_C(    -0.52),
        SIMDE_FLOAT32_C(     2.80), SIMDE_FLOAT32_C(     2.60), SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(    -2.28),
        SIMDE_FLOAT32_C(    -2.33), SIMDE_FLOAT32_C(    -3.65), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(    -2.12) },
      { SIMDE_FLOAT32_C(    18.73), SIMDE_FLOAT32_C(    21.54), SIMDE_FLOAT32_C(    11.70), SIMDE_FLOAT32_C(     0.02),
        SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.59),
        SIMDE_FLOAT32_C(    16.44), SIMDE_FLOAT32_C(    13.46), SIMDE_FLOAT32_C(     7.69), SIMDE_FLOAT32_C(     0.10),
        SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.12) } },
    { { SIMDE_FLOAT32_C(     3.59), SIMDE_FLOAT32_C(     2.96), SIMDE_FLOAT32_C(    -2.22), SIMDE_FLOAT32_C(     3.39),
        SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(     1.32), SIMDE_FLOAT32_C(    -2.78), SIMDE_FLOAT32_C(     3.98),
        SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(     2.09), SIMDE_FLOAT32_C(     2.22), SIMDE_FLOAT32_C(     2.33),
        SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -3.98), SIMDE_FLOAT32_C(    -0.78) },
      { SIMDE_FLOAT32_C(    36.23), SIMDE_FLOAT32_C(    19.30), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    29.67),
        SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     3.74), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    53.52),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     8.08), SIMDE_FLOAT32_C(     9.21), SIMDE_FLOAT32_C(    10.28),
        SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.46) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_exp_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_exp_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(    -2.75), SIMDE_FLOAT32_C(     3.98),
        SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     2.66),
        SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     0.54),
        SIMDE_FLOAT32_C(     2.11), SIMDE_FLOAT32_C(    -1.61), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(     3.46) },
      UINT8_C( 98),
      { SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     1.44), SIMDE_FLOAT32_C(    -0.22),
        SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     3.48), SIMDE_FLOAT32_C(    -2.98), SIMDE_FLOAT32_C(    -3.66),
        SIMDE_FLOAT32_C(    -2.38), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     1.62), SIMDE_FLOAT32_C(    -3.73),
        SIMDE_FLOAT32_C(     3.17), SIMDE_FLOAT32_C(    -2.33), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     3.09) },
      { SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     8.50), SIMDE_FLOAT32_C(    -2.75), SIMDE_FLOAT32_C(     3.98),
        SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(    32.46), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     2.66),
        SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     0.54),
        SIMDE_FLOAT32_C(     2.11), SIMDE_FLOAT32_C(    -1.61), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(     3.46) } },
    { { SIMDE_FLOAT32_C(    -1.08), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -3.83),
        SIMDE_FLOAT32_C(    -2.58), SIMDE_FLOAT32_C(    -1.71), SIMDE_FLOAT32_C(     2.08), SIMDE_FLOAT32_C(    -2.80),
        SIMDE_FLOAT32_C(    -3.29), SIMDE_FLOAT32_C(    -1.38), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(    -0.90),
        SIMDE_FLOAT32_C(    -1.54), SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     2.08) },
      UINT8_C(254),
      { SIMDE_FLOAT32_C(    -1.11), SIMDE_FLOAT32_C(    -2.14), SIMDE_FLOAT32_C(    -2.35), SIMDE_FLOAT32_C(    -1.63),
        SIMDE_FLOAT32_C(    -1.11), SIMDE_FLOAT32_C(    -2.01), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     3.59),
        SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     2.76), SIMDE_FLOAT32_C(    -2.73),
        SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.66) },
      { SIMDE_FLOAT32_C(    -1.08), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.20),
        SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    36.23),
        SIMDE_FLOAT32_C(    -3.29), SIMDE_FLOAT32_C(    -1.38), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(    -0.90),
        SIMDE_FLOAT32_C(    -1.54), SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     2.08) } },
    { { SIMDE_FLOAT32_C(    -2.52), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -0.24),
        SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     1.06),
        SIMDE_FLOAT32_C(     1.27), SIMDE_FLOAT32_C(     3.57), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     1.02), SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(    -3.04), SIMDE_FLOAT32_C(     3.91) },
      UINT8_C(140),
      { SIMDE_FLOAT32_C(    -1.39), SIMDE_FLOAT32_C(    -1.72), SIMDE_FLOAT32_C(    -1.65), SIMDE_FLOAT32_C(     0.59),
        SIMDE_FLOAT32_C(     2.27), SIMDE_FLOAT32_C(    -2.06), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(    -1.47),
        SIMDE_FLOAT32_C(    -3.30), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(    -1.67), SIMDE_FLOAT32_C(     2.55),
        SIMDE_FLOAT32_C(    -2.33), SIMDE_FLOAT32_C(     1.67), SIMDE_FLOAT32_C(    -3.97), SIMDE_FLOAT32_C(     2.04) },
      { SIMDE_FLOAT32_C(    -2.52), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     1.80),
        SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.23),
        SIMDE_FLOAT32_C(     1.27), SIMDE_FLOAT32_C(     3.57), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     1.02), SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(    -3.04), SIMDE_FLOAT32_C(     3.91) } },
    { { SIMDE_FLOAT32_C(    -1.58), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -3.52), SIMDE_FLOAT32_C(    -3.63),
        SIMDE_FLOAT32_C(    -3.74), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     3.83),
        SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(    -2.32), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(    -1.33),
        SIMDE_FLOAT32_C(    -1.36), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -1.87), SIMDE_FLOAT32_C(     1.25) },
      UINT8_C(221),
      { SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -2.16), SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(     2.41),
        SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     3.83), SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(    -0.48),
        SIMDE_FLOAT32_C(    -1.84), SIMDE_FLOAT32_C(     1.66), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     3.83),
        SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -1.75), SIMDE_FLOAT32_C(    -2.52) },
      { SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(     3.67), SIMDE_FLOAT32_C(    11.13),
        SIMDE_FLOAT32_C(     7.69), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(    22.42), SIMDE_FLOAT32_C(     0.62),
        SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(    -2.32), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(    -1.33),
        SIMDE_FLOAT32_C(    -1.36), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -1.87), SIMDE_FLOAT32_C(     1.25) } },
    { { SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -1.37), SIMDE_FLOAT32_C(    -2.26), SIMDE_FLOAT32_C(    -2.75),
        SIMDE_FLOAT32_C(    -3.73), SIMDE_FLOAT32_C(    -2.43), SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(    -2.05),
        SIMDE_FLOAT32_C(     2.41), SIMDE_FLOAT32_C(    -3.02), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.83),
        SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(    -2.16), SIMDE_FLOAT32_C(    -1.80), SIMDE_FLOAT32_C(     3.58) },
      UINT8_C(165),
      { SIMDE_FLOAT32_C(     3.50), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(    -2.28), SIMDE_FLOAT32_C(     3.33),
        SIMDE_FLOAT32_C(     1.10), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(    -2.51), SIMDE_FLOAT32_C(    -1.23),
        SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(    -2.68), SIMDE_FLOAT32_C(    -3.54), SIMDE_FLOAT32_C(     1.66),
        SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -2.06), SIMDE_FLOAT32_C(    -2.63), SIMDE_FLOAT32_C(     2.20) },
      { SIMDE_FLOAT32_C(    33.12), SIMDE_FLOAT32_C(    -1.37), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    -2.75),
        SIMDE_FLOAT32_C(    -3.73), SIMDE_FLOAT32_C(     3.46), SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     0.29),
        SIMDE_FLOAT32_C(     2.41), SIMDE_FLOAT32_C(    -3.02), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.83),
        SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(    -2.16), SIMDE_FLOAT32_C(    -1.80), SIMDE_FLOAT32_C(     3.58) } },
    { { SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -1.38), SIMDE_FLOAT32_C(     2.46), SIMDE_FLOAT32_C(     1.24),
        SIMDE_FLOAT32_C(    -3.06), SIMDE_FLOAT32_C(    -3.58), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(    -2.07),
        SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     2.85),
        SIMDE_FLOAT32_C(    -2.97), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(    -1.47), SIMDE_FLOAT32_C(    -3.47) },
      UINT8_C(155),
      { SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     3.85), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(    -2.50),
        SIMDE_FLOAT32_C(    -2.66), SIMDE_FLOAT32_C(    -1.52), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -1.34),
        SIMDE_FLOAT32_C(    -1.06), SIMDE_FLOAT32_C(    -2.40), SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(     0.88),
        SIMDE_FLOAT32_C(    -1.03), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -3.45), SIMDE_FLOAT32_C(     1.60) },
      { SIMDE_FLOAT32_C(     1.30), SIMDE_FLOAT32_C(    46.99), SIMDE_FLOAT32_C(     2.46), SIMDE_FLOAT32_C(     0.08),
        SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(    -3.58), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     2.85),
        SIMDE_FLOAT32_C(    -2.97), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(    -1.47), SIMDE_FLOAT32_C(    -3.47) } },
    { { SIMDE_FLOAT32_C(    -1.11), SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(     2.54), SIMDE_FLOAT32_C(    -0.69),
        SIMDE_FLOAT32_C(    -2.55), SIMDE_FLOAT32_C(    -3.53), SIMDE_FLOAT32_C(    -3.68), SIMDE_FLOAT32_C(    -3.72),
        SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     3.17), SIMDE_FLOAT32_C(    -2.70), SIMDE_FLOAT32_C(    -1.88),
        SIMDE_FLOAT32_C(    -2.30), SIMDE_FLOAT32_C(    -2.17), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     1.96) },
      UINT8_C(151),
      { SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     2.93),
        SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     1.68), SIMDE_FLOAT32_C(    -2.13), SIMDE_FLOAT32_C(     1.01),
        SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(     2.75), SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(    -3.67),
        SIMDE_FLOAT32_C(     3.30), SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     1.09) },
      { SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    32.14), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(    -0.69),
        SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    -3.53), SIMDE_FLOAT32_C(    -3.68), SIMDE_FLOAT32_C(     2.75),
        SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     3.17), SIMDE_FLOAT32_C(    -2.70), SIMDE_FLOAT32_C(    -1.88),
        SIMDE_FLOAT32_C(    -2.30), SIMDE_FLOAT32_C(    -2.17), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     1.96) } },
    { { SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     2.54), SIMDE_FLOAT32_C(     0.59),
        SIMDE_FLOAT32_C(     2.84), SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(    -1.91), SIMDE_FLOAT32_C(     2.01),
        SIMDE_FLOAT32_C(    -3.88), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(    -2.04),
        SIMDE_FLOAT32_C(    -3.05), SIMDE_FLOAT32_C(     1.68), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     1.40) },
      UINT8_C( 24),
      { SIMDE_FLOAT32_C(     2.67), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -3.44), SIMDE_FLOAT32_C(     0.35),
        SIMDE_FLOAT32_C(     2.21), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -3.74), SIMDE_FLOAT32_C(     0.96),
        SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -3.40), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.86),
        SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(    -2.65), SIMDE_FLOAT32_C(     3.27), SIMDE_FLOAT32_C(    -1.65) },
      { SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     2.54), SIMDE_FLOAT32_C(     1.42),
        SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(    -1.91), SIMDE_FLOAT32_C(     2.01),
        SIMDE_FLOAT32_C(    -3.88), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(    -2.04),
        SIMDE_FLOAT32_C(    -3.05), SIMDE_FLOAT32_C(     1.68), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     1.40) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_exp_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_exp_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     3.06), SIMDE_FLOAT64_C(    -0.78), SIMDE_FLOAT64_C(     1.53), SIMDE_FLOAT64_C(     2.94),
        SIMDE_FLOAT64_C(    -3.88), SIMDE_FLOAT64_C(     3.46), SIMDE_FLOAT64_C(     1.02), SIMDE_FLOAT64_C(    -3.05) },
      { SIMDE_FLOAT64_C(    21.33), SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(     4.62), SIMDE_FLOAT64_C(    18.92),
        SIMDE_FLOAT64_C(     0.02), SIMDE_FLOAT64_C(    31.82), SIMDE_FLOAT64_C(     2.77), SIMDE_FLOAT64_C(     0.05) } },
    { { SIMDE_FLOAT64_C(     1.99), SIMDE_FLOAT64_C(    -3.10), SIMDE_FLOAT64_C(     1.58), SIMDE_FLOAT64_C(     2.87),
        SIMDE_FLOAT64_C(    -2.25), SIMDE_FLOAT64_C(    -0.61), SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     3.71) },
      { SIMDE_FLOAT64_C(     7.32), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     4.85), SIMDE_FLOAT64_C(    17.64),
        SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(     0.89), SIMDE_FLOAT64_C(    40.85) } },
    { { SIMDE_FLOAT64_C(    -3.09), SIMDE_FLOAT64_C(     1.38), SIMDE_FLOAT64_C(    -1.35), SIMDE_FLOAT64_C(    -3.35),
        SIMDE_FLOAT64_C(     2.49), SIMDE_FLOAT64_C(    -1.09), SIMDE_FLOAT64_C(    -3.89), SIMDE_FLOAT64_C(     0.92) },
      { SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     3.97), SIMDE_FLOAT64_C(     0.26), SIMDE_FLOAT64_C(     0.04),
        SIMDE_FLOAT64_C(    12.06), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.02), SIMDE_FLOAT64_C(     2.51) } },
    { { SIMDE_FLOAT64_C(    -1.13), SIMDE_FLOAT64_C(    -1.04), SIMDE_FLOAT64_C(    -0.11), SIMDE_FLOAT64_C(     0.37),
        SIMDE_FLOAT64_C(     1.47), SIMDE_FLOAT64_C(    -3.30), SIMDE_FLOAT64_C(    -0.41), SIMDE_FLOAT64_C(     0.53) },
      { SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(     0.35), SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(     1.45),
        SIMDE_FLOAT64_C(     4.35), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.66), SIMDE_FLOAT64_C(     1.70) } },
    { { SIMDE_FLOAT64_C(    -0.08), SIMDE_FLOAT64_C(    -2.87), SIMDE_FLOAT64_C(    -0.54), SIMDE_FLOAT64_C(     0.04),
        SIMDE_FLOAT64_C(    -3.41), SIMDE_FLOAT64_C(    -3.51), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     2.57) },
      { SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     1.04),
        SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     2.69), SIMDE_FLOAT64_C(    13.07) } },
    { { SIMDE_FLOAT64_C(    -2.62), SIMDE_FLOAT64_C(    -1.43), SIMDE_FLOAT64_C(     1.44), SIMDE_FLOAT64_C(    -0.87),
        SIMDE_FLOAT64_C(     1.96), SIMDE_FLOAT64_C(    -2.68), SIMDE_FLOAT64_C(    -1.16), SIMDE_FLOAT64_C(     2.87) },
      { SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.24), SIMDE_FLOAT64_C(     4.22), SIMDE_FLOAT64_C(     0.42),
        SIMDE_FLOAT64_C(     7.10), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(    17.64) } },
    { { SIMDE_FLOAT64_C(     2.70), SIMDE_FLOAT64_C(     1.49), SIMDE_FLOAT64_C(     3.52), SIMDE_FLOAT64_C(     1.19),
        SIMDE_FLOAT64_C(    -3.59), SIMDE_FLOAT64_C(     3.63), SIMDE_FLOAT64_C(    -1.89), SIMDE_FLOAT64_C(    -0.72) },
      { SIMDE_FLOAT64_C(    14.88), SIMDE_FLOAT64_C(     4.44), SIMDE_FLOAT64_C(    33.78), SIMDE_FLOAT64_C(     3.29),
        SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(    37.71), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.49) } },
    { { SIMDE_FLOAT64_C(    -1.41), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     3.65), SIMDE_FLOAT64_C(    -3.94),
        SIMDE_FLOAT64_C(     2.70), SIMDE_FLOAT64_C(    -0.76), SIMDE_FLOAT64_C(     0.59), SIMDE_FLOAT64_C(    -1.37) },
      { SIMDE_FLOAT64_C(     0.24), SIMDE_FLOAT64_C(     7.39), SIMDE_FLOAT64_C(    38.47), SIMDE_FLOAT64_C(     0.02),
        SIMDE_FLOAT64_C(    14.88), SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     1.80), SIMDE_FLOAT64_C(     0.25) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_exp_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_exp_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    -3.51), SIMDE_FLOAT64_C(    -3.98), SIMDE_FLOAT64_C(     3.54), SIMDE_FLOAT64_C(    -1.79),
        SIMDE_FLOAT64_C(    -1.83), SIMDE_FLOAT64_C(    -3.73), SIMDE_FLOAT64_C(    -3.51), SIMDE_FLOAT64_C(     3.71) },
      UINT8_C(199),
      { SIMDE_FLOAT64_C(     2.33), SIMDE_FLOAT64_C(    -1.17), SIMDE_FLOAT64_C(    -1.77), SIMDE_FLOAT64_C(    -2.21),
        SIMDE_FLOAT64_C(     2.46), SIMDE_FLOAT64_C(     1.54), SIMDE_FLOAT64_C(    -1.07), SIMDE_FLOAT64_C(    -0.25) },
      { SIMDE_FLOAT64_C(    10.28), SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(    -1.79),
        SIMDE_FLOAT64_C(    -1.83), SIMDE_FLOAT64_C(    -3.73), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.78) } },
    { { SIMDE_FLOAT64_C(    -2.63), SIMDE_FLOAT64_C(     1.07), SIMDE_FLOAT64_C(    -1.37), SIMDE_FLOAT64_C(    -0.96),
        SIMDE_FLOAT64_C(    -3.82), SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(    -2.76), SIMDE_FLOAT64_C(    -2.64) },
      UINT8_C(126),
      { SIMDE_FLOAT64_C(     0.87), SIMDE_FLOAT64_C(     0.29), SIMDE_FLOAT64_C(     1.08), SIMDE_FLOAT64_C(     2.46),
        SIMDE_FLOAT64_C(    -0.27), SIMDE_FLOAT64_C(     1.56), SIMDE_FLOAT64_C(     2.48), SIMDE_FLOAT64_C(    -0.72) },
      { SIMDE_FLOAT64_C(    -2.63), SIMDE_FLOAT64_C(     1.34), SIMDE_FLOAT64_C(     2.94), SIMDE_FLOAT64_C(    11.70),
        SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     4.76), SIMDE_FLOAT64_C(    11.94), SIMDE_FLOAT64_C(    -2.64) } },
    { { SIMDE_FLOAT64_C(     3.77), SIMDE_FLOAT64_C(    -3.36), SIMDE_FLOAT64_C(    -0.46), SIMDE_FLOAT64_C(    -3.74),
        SIMDE_FLOAT64_C(    -3.65), SIMDE_FLOAT64_C(     1.21), SIMDE_FLOAT64_C(     2.59), SIMDE_FLOAT64_C(    -0.82) },
      UINT8_C( 39),
      { SIMDE_FLOAT64_C(    -3.62), SIMDE_FLOAT64_C(    -2.35), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(    -0.70),
        SIMDE_FLOAT64_C(     1.40), SIMDE_FLOAT64_C(     2.35), SIMDE_FLOAT64_C(    -3.63), SIMDE_FLOAT64_C(    -3.97) },
      { SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     2.66), SIMDE_FLOAT64_C(    -3.74),
        SIMDE_FLOAT64_C(    -3.65), SIMDE_FLOAT64_C(    10.49), SIMDE_FLOAT64_C(     2.59), SIMDE_FLOAT64_C(    -0.82) } },
    { { SIMDE_FLOAT64_C(    -2.61), SIMDE_FLOAT64_C(    -3.45), SIMDE_FLOAT64_C(     0.02), SIMDE_FLOAT64_C(    -1.37),
        SIMDE_FLOAT64_C(    -2.08), SIMDE_FLOAT64_C(    -3.79), SIMDE_FLOAT64_C(     3.50), SIMDE_FLOAT64_C(     2.21) },
      UINT8_C(165),
      { SIMDE_FLOAT64_C(     1.96), SIMDE_FLOAT64_C(    -2.06), SIMDE_FLOAT64_C(    -1.15), SIMDE_FLOAT64_C(     0.44),
        SIMDE_FLOAT64_C(     1.22), SIMDE_FLOAT64_C(    -1.37), SIMDE_FLOAT64_C(     1.08), SIMDE_FLOAT64_C(    -3.24) },
      { SIMDE_FLOAT64_C(     7.10), SIMDE_FLOAT64_C(    -3.45), SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(    -1.37),
        SIMDE_FLOAT64_C(    -2.08), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     3.50), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(    -1.11), SIMDE_FLOAT64_C(     1.43), SIMDE_FLOAT64_C(     1.97), SIMDE_FLOAT64_C(    -2.52),
        SIMDE_FLOAT64_C(    -3.38), SIMDE_FLOAT64_C(     1.41), SIMDE_FLOAT64_C(    -2.14), SIMDE_FLOAT64_C(    -1.73) },
      UINT8_C(202),
      { SIMDE_FLOAT64_C(     1.17), SIMDE_FLOAT64_C(     3.67), SIMDE_FLOAT64_C(    -3.26), SIMDE_FLOAT64_C(     1.54),
        SIMDE_FLOAT64_C(     3.70), SIMDE_FLOAT64_C(    -1.87), SIMDE_FLOAT64_C(     2.10), SIMDE_FLOAT64_C(    -0.28) },
      { SIMDE_FLOAT64_C(    -1.11), SIMDE_FLOAT64_C(    39.25), SIMDE_FLOAT64_C(     1.97), SIMDE_FLOAT64_C(     4.66),
        SIMDE_FLOAT64_C(    -3.38), SIMDE_FLOAT64_C(     1.41), SIMDE_FLOAT64_C(     8.17), SIMDE_FLOAT64_C(     0.76) } },
    { { SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(    -3.99), SIMDE_FLOAT64_C(    -0.06), SIMDE_FLOAT64_C(     0.26),
        SIMDE_FLOAT64_C(     2.22), SIMDE_FLOAT64_C(    -2.77), SIMDE_FLOAT64_C(    -1.78), SIMDE_FLOAT64_C(    -3.84) },
      UINT8_C(172),
      { SIMDE_FLOAT64_C(     2.66), SIMDE_FLOAT64_C(     1.38), SIMDE_FLOAT64_C(     2.71), SIMDE_FLOAT64_C(    -0.26),
        SIMDE_FLOAT64_C(     2.13), SIMDE_FLOAT64_C(    -2.39), SIMDE_FLOAT64_C(    -2.83), SIMDE_FLOAT64_C(     0.11) },
      { SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(    -3.99), SIMDE_FLOAT64_C(    15.03), SIMDE_FLOAT64_C(     0.77),
        SIMDE_FLOAT64_C(     2.22), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(    -1.78), SIMDE_FLOAT64_C(     1.12) } },
    { { SIMDE_FLOAT64_C(    -0.91), SIMDE_FLOAT64_C(    -2.21), SIMDE_FLOAT64_C(    -2.48), SIMDE_FLOAT64_C(     0.95),
        SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(    -0.09), SIMDE_FLOAT64_C(    -1.88), SIMDE_FLOAT64_C(    -0.27) },
      UINT8_C(244),
      { SIMDE_FLOAT64_C(     3.66), SIMDE_FLOAT64_C(    -0.57), SIMDE_FLOAT64_C(     2.78), SIMDE_FLOAT64_C(     1.75),
        SIMDE_FLOAT64_C(     3.15), SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(     1.76), SIMDE_FLOAT64_C(    -0.91) },
      { SIMDE_FLOAT64_C(    -0.91), SIMDE_FLOAT64_C(    -2.21), SIMDE_FLOAT64_C(    16.12), SIMDE_FLOAT64_C(     0.95),
        SIMDE_FLOAT64_C(    23.34), SIMDE_FLOAT64_C(     0.64), SIMDE_FLOAT64_C(     5.81), SIMDE_FLOAT64_C(     0.40) } },
    { { SIMDE_FLOAT64_C(     3.81), SIMDE_FLOAT64_C(    -0.02), SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(    -1.97),
        SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(    -3.60), SIMDE_FLOAT64_C(    -3.30), SIMDE_FLOAT64_C(    -2.48) },
      UINT8_C(117),
      { SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(     3.65), SIMDE_FLOAT64_C(    -3.28), SIMDE_FLOAT64_C(     1.61),
        SIMDE_FLOAT64_C(    -0.24), SIMDE_FLOAT64_C(    -0.20), SIMDE_FLOAT64_C(     3.40), SIMDE_FLOAT64_C(     1.27) },
      { SIMDE_FLOAT64_C(     1.55), SIMDE_FLOAT64_C(    -0.02), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(    -1.97),
        SIMDE_FLOAT64_C(     0.79), SIMDE_FLOAT64_C(     0.82), SIMDE_FLOAT64_C(    29.96), SIMDE_FLOAT64_C(    -2.48) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_exp_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_expm1_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     6.33), SIMDE_FLOAT32_C(     1.68), SIMDE_FLOAT32_C(     8.16), SIMDE_FLOAT32_C(     5.04) },
      { SIMDE_FLOAT32_C(   560.16), SIMDE_FLOAT32_C(     4.37), SIMDE_FLOAT32_C(  3497.19), SIMDE_FLOAT32_C(   153.47) } },
    { { SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     8.63), SIMDE_FLOAT32_C(     5.23), SIMDE_FLOAT32_C(     4.43) },
      { SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(  5596.08), SIMDE_FLOAT32_C(   185.79), SIMDE_FLOAT32_C(    82.93) } },
    { { SIMDE_FLOAT32_C(     7.85), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(     5.34) },
      { SIMDE_FLOAT32_C(  2564.73), SIMDE_FLOAT32_C(  1011.32), SIMDE_FLOAT32_C(    14.96), SIMDE_FLOAT32_C(   207.51) } },
    { { SIMDE_FLOAT32_C(     6.60), SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     0.13) },
      { SIMDE_FLOAT32_C(   734.10), SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(     0.14) } },
    { { SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     7.36), SIMDE_FLOAT32_C(     9.70), SIMDE_FLOAT32_C(     5.19) },
      { SIMDE_FLOAT32_C(    19.09), SIMDE_FLOAT32_C(  1570.84), SIMDE_FLOAT32_C( 16316.60), SIMDE_FLOAT32_C(   178.47) } },
    { { SIMDE_FLOAT32_C(     2.21), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     8.65), SIMDE_FLOAT32_C(     9.58) },
      { SIMDE_FLOAT32_C(     8.12), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(  5709.14), SIMDE_FLOAT32_C( 14471.42) } },
    { { SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(     4.96), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     7.49) },
      { SIMDE_FLOAT32_C(    39.45), SIMDE_FLOAT32_C(   141.59), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(  1789.05) } },
    { { SIMDE_FLOAT32_C(     7.91), SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     4.16), SIMDE_FLOAT32_C(     4.24) },
      { SIMDE_FLOAT32_C(  2723.39), SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(    63.07), SIMDE_FLOAT32_C(    68.41) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_expm1_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_expm1_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.24), SIMDE_FLOAT64_C(     7.18) },
      { SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(  1311.91) } },
    { { SIMDE_FLOAT64_C(     9.69), SIMDE_FLOAT64_C(     1.13) },
      { SIMDE_FLOAT64_C( 16154.24), SIMDE_FLOAT64_C(     2.10) } },
    { { SIMDE_FLOAT64_C(     6.24), SIMDE_FLOAT64_C(     8.67) },
      { SIMDE_FLOAT64_C(   511.86), SIMDE_FLOAT64_C(  5824.50) } },
    { { SIMDE_FLOAT64_C(     9.69), SIMDE_FLOAT64_C(     7.67) },
      { SIMDE_FLOAT64_C( 16154.24), SIMDE_FLOAT64_C(  2142.08) } },
    { { SIMDE_FLOAT64_C(     4.67), SIMDE_FLOAT64_C(     1.83) },
      { SIMDE_FLOAT64_C(   105.70), SIMDE_FLOAT64_C(     5.23) } },
    { { SIMDE_FLOAT64_C(     2.80), SIMDE_FLOAT64_C(     6.65) },
      { SIMDE_FLOAT64_C(    15.44), SIMDE_FLOAT64_C(   771.78) } },
    { { SIMDE_FLOAT64_C(     8.11), SIMDE_FLOAT64_C(     9.49) },
      { SIMDE_FLOAT64_C(  3326.58), SIMDE_FLOAT64_C( 13225.80) } },
    { { SIMDE_FLOAT64_C(     1.48), SIMDE_FLOAT64_C(     7.85) },
      { SIMDE_FLOAT64_C(     3.39), SIMDE_FLOAT64_C(  2564.73) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_expm1_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_expm1_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     8.24), SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     4.38),
        SIMDE_FLOAT32_C(     3.85), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     8.49), SIMDE_FLOAT32_C(     5.32) },
      { SIMDE_FLOAT32_C(  3788.54), SIMDE_FLOAT32_C(     8.30), SIMDE_FLOAT32_C(    44.15), SIMDE_FLOAT32_C(    78.84),
        SIMDE_FLOAT32_C(    45.99), SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(  4864.86), SIMDE_FLOAT32_C(   203.38) } },
    { { SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     4.59), SIMDE_FLOAT32_C(     9.56), SIMDE_FLOAT32_C(     9.67),
        SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     3.74), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.14) },
      { SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(    97.49), SIMDE_FLOAT32_C( 14184.85), SIMDE_FLOAT32_C( 15834.35),
        SIMDE_FLOAT32_C(    29.57), SIMDE_FLOAT32_C(    41.10), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.15) } },
    { { SIMDE_FLOAT32_C(     6.62), SIMDE_FLOAT32_C(     4.91), SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(     8.04),
        SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     9.84), SIMDE_FLOAT32_C(     7.16), SIMDE_FLOAT32_C(     7.09) },
      { SIMDE_FLOAT32_C(   748.95), SIMDE_FLOAT32_C(   134.64), SIMDE_FLOAT32_C(    21.42), SIMDE_FLOAT32_C(  3101.61),
        SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C( 18768.72), SIMDE_FLOAT32_C(  1285.91), SIMDE_FLOAT32_C(  1198.91) } },
    { { SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     9.95), SIMDE_FLOAT32_C(     7.75),
        SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     4.01), SIMDE_FLOAT32_C(     9.02) },
      { SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     5.89), SIMDE_FLOAT32_C( 20951.22), SIMDE_FLOAT32_C(  2320.57),
        SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(    51.46), SIMDE_FLOAT32_C(    54.15), SIMDE_FLOAT32_C(  8265.78) } },
    { { SIMDE_FLOAT32_C(     6.19), SIMDE_FLOAT32_C(     7.82), SIMDE_FLOAT32_C(     3.40), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     8.52), SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     5.37), SIMDE_FLOAT32_C(     9.06) },
      { SIMDE_FLOAT32_C(   486.85), SIMDE_FLOAT32_C(  2488.91), SIMDE_FLOAT32_C(    28.96), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(  5013.06), SIMDE_FLOAT32_C(     5.62), SIMDE_FLOAT32_C(   213.86), SIMDE_FLOAT32_C(  8603.15) } },
    { { SIMDE_FLOAT32_C(     6.48), SIMDE_FLOAT32_C(     4.92), SIMDE_FLOAT32_C(     8.72), SIMDE_FLOAT32_C(     9.90),
        SIMDE_FLOAT32_C(     8.66), SIMDE_FLOAT32_C(     8.99), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     5.28) },
      { SIMDE_FLOAT32_C(   650.97), SIMDE_FLOAT32_C(   136.00), SIMDE_FLOAT32_C(  6123.18), SIMDE_FLOAT32_C( 19929.36),
        SIMDE_FLOAT32_C(  5766.53), SIMDE_FLOAT32_C(  8021.46), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(   195.37) } },
    { { SIMDE_FLOAT32_C(     3.90), SIMDE_FLOAT32_C(     3.15), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(     4.49),
        SIMDE_FLOAT32_C(     2.99), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     3.12) },
      { SIMDE_FLOAT32_C(    48.40), SIMDE_FLOAT32_C(    22.34), SIMDE_FLOAT32_C(    26.66), SIMDE_FLOAT32_C(    88.12),
        SIMDE_FLOAT32_C(    18.89), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(    21.65) } },
    { { SIMDE_FLOAT32_C(     2.41), SIMDE_FLOAT32_C(     1.52), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     3.20),
        SIMDE_FLOAT32_C(     5.48), SIMDE_FLOAT32_C(     4.88), SIMDE_FLOAT32_C(     2.22), SIMDE_FLOAT32_C(     1.67) },
      { SIMDE_FLOAT32_C(    10.13), SIMDE_FLOAT32_C(     3.57), SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(    23.53),
        SIMDE_FLOAT32_C(   238.85), SIMDE_FLOAT32_C(   130.63), SIMDE_FLOAT32_C(     8.21), SIMDE_FLOAT32_C(     4.31) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_expm1_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_expm1_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     6.68), SIMDE_FLOAT64_C(     7.67), SIMDE_FLOAT64_C(     2.13), SIMDE_FLOAT64_C(     3.50) },
      { SIMDE_FLOAT64_C(   795.32), SIMDE_FLOAT64_C(  2142.08), SIMDE_FLOAT64_C(     7.41), SIMDE_FLOAT64_C(    32.12) } },
    { { SIMDE_FLOAT64_C(     4.83), SIMDE_FLOAT64_C(     1.25), SIMDE_FLOAT64_C(     4.74), SIMDE_FLOAT64_C(     8.00) },
      { SIMDE_FLOAT64_C(   124.21), SIMDE_FLOAT64_C(     2.49), SIMDE_FLOAT64_C(   113.43), SIMDE_FLOAT64_C(  2979.96) } },
    { { SIMDE_FLOAT64_C(     9.68), SIMDE_FLOAT64_C(     1.62), SIMDE_FLOAT64_C(     7.69), SIMDE_FLOAT64_C(     7.36) },
      { SIMDE_FLOAT64_C( 15993.50), SIMDE_FLOAT64_C(     4.05), SIMDE_FLOAT64_C(  2185.37), SIMDE_FLOAT64_C(  1570.84) } },
    { { SIMDE_FLOAT64_C(     8.87), SIMDE_FLOAT64_C(     3.50), SIMDE_FLOAT64_C(     7.63), SIMDE_FLOAT64_C(     8.66) },
      { SIMDE_FLOAT64_C(  7114.28), SIMDE_FLOAT64_C(    32.12), SIMDE_FLOAT64_C(  2058.05), SIMDE_FLOAT64_C(  5766.53) } },
    { { SIMDE_FLOAT64_C(     5.89), SIMDE_FLOAT64_C(     2.15), SIMDE_FLOAT64_C(     8.77), SIMDE_FLOAT64_C(     4.86) },
      { SIMDE_FLOAT64_C(   360.41), SIMDE_FLOAT64_C(     7.58), SIMDE_FLOAT64_C(  6437.17), SIMDE_FLOAT64_C(   128.02) } },
    { { SIMDE_FLOAT64_C(     2.27), SIMDE_FLOAT64_C(     7.65), SIMDE_FLOAT64_C(     5.22), SIMDE_FLOAT64_C(     9.35) },
      { SIMDE_FLOAT64_C(     8.68), SIMDE_FLOAT64_C(  2099.65), SIMDE_FLOAT64_C(   183.93), SIMDE_FLOAT64_C( 11497.82) } },
    { { SIMDE_FLOAT64_C(     3.29), SIMDE_FLOAT64_C(     3.19), SIMDE_FLOAT64_C(     2.91), SIMDE_FLOAT64_C(     3.13) },
      { SIMDE_FLOAT64_C(    25.84), SIMDE_FLOAT64_C(    23.29), SIMDE_FLOAT64_C(    17.36), SIMDE_FLOAT64_C(    21.87) } },
    { { SIMDE_FLOAT64_C(     5.79), SIMDE_FLOAT64_C(     1.89), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     2.47) },
      { SIMDE_FLOAT64_C(   326.01), SIMDE_FLOAT64_C(     5.62), SIMDE_FLOAT64_C(     1.66), SIMDE_FLOAT64_C(    10.82) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_expm1_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_expm1_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     2.06), SIMDE_FLOAT32_C(     8.37), SIMDE_FLOAT32_C(     4.10), SIMDE_FLOAT32_C(     0.86),
        SIMDE_FLOAT32_C(     5.30), SIMDE_FLOAT32_C(     6.13), SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(     0.38),
        SIMDE_FLOAT32_C(     1.44), SIMDE_FLOAT32_C(     6.24), SIMDE_FLOAT32_C(     8.36), SIMDE_FLOAT32_C(     5.93),
        SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     8.82), SIMDE_FLOAT32_C(     8.89), SIMDE_FLOAT32_C(     5.58) },
      { SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(  4314.64), SIMDE_FLOAT32_C(    59.34), SIMDE_FLOAT32_C(     1.36),
        SIMDE_FLOAT32_C(   199.34), SIMDE_FLOAT32_C(   458.44), SIMDE_FLOAT32_C(     5.17), SIMDE_FLOAT32_C(     0.46),
        SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(   511.86), SIMDE_FLOAT32_C(  4271.69), SIMDE_FLOAT32_C(   375.15),
        SIMDE_FLOAT32_C(     9.18), SIMDE_FLOAT32_C(  6767.26), SIMDE_FLOAT32_C(  7258.02), SIMDE_FLOAT32_C(   264.07) } },
    { { SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     7.28), SIMDE_FLOAT32_C(     6.53), SIMDE_FLOAT32_C(     8.60),
        SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     1.74), SIMDE_FLOAT32_C(     4.08), SIMDE_FLOAT32_C(     7.80),
        SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     7.90), SIMDE_FLOAT32_C(     9.34), SIMDE_FLOAT32_C(     7.60),
        SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(     5.46), SIMDE_FLOAT32_C(     8.74), SIMDE_FLOAT32_C(     6.01) },
      { SIMDE_FLOAT32_C(     6.10), SIMDE_FLOAT32_C(  1449.99), SIMDE_FLOAT32_C(   684.40), SIMDE_FLOAT32_C(  5430.66),
        SIMDE_FLOAT32_C(     2.86), SIMDE_FLOAT32_C(     4.70), SIMDE_FLOAT32_C(    58.15), SIMDE_FLOAT32_C(  2439.60),
        SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(  2696.28), SIMDE_FLOAT32_C( 11383.41), SIMDE_FLOAT32_C(  1997.20),
        SIMDE_FLOAT32_C(    50.94), SIMDE_FLOAT32_C(   234.10), SIMDE_FLOAT32_C(  6246.89), SIMDE_FLOAT32_C(   406.48) } },
    { { SIMDE_FLOAT32_C(     3.83), SIMDE_FLOAT32_C(     2.84), SIMDE_FLOAT32_C(     6.87), SIMDE_FLOAT32_C(     9.14),
        SIMDE_FLOAT32_C(     8.97), SIMDE_FLOAT32_C(     8.69), SIMDE_FLOAT32_C(     9.51), SIMDE_FLOAT32_C(     0.41),
        SIMDE_FLOAT32_C(     4.93), SIMDE_FLOAT32_C(     7.87), SIMDE_FLOAT32_C(     6.35), SIMDE_FLOAT32_C(     7.25),
        SIMDE_FLOAT32_C(     6.69), SIMDE_FLOAT32_C(     5.24), SIMDE_FLOAT32_C(     2.83), SIMDE_FLOAT32_C(     8.65) },
      { SIMDE_FLOAT32_C(    45.06), SIMDE_FLOAT32_C(    16.12), SIMDE_FLOAT32_C(   961.95), SIMDE_FLOAT32_C(  9319.77),
        SIMDE_FLOAT32_C(  7862.60), SIMDE_FLOAT32_C(  5942.18), SIMDE_FLOAT32_C( 13493.00), SIMDE_FLOAT32_C(     0.51),
        SIMDE_FLOAT32_C(   137.38), SIMDE_FLOAT32_C(  2616.57), SIMDE_FLOAT32_C(   571.49), SIMDE_FLOAT32_C(  1407.10),
        SIMDE_FLOAT32_C(   803.32), SIMDE_FLOAT32_C(   187.67), SIMDE_FLOAT32_C(    15.95), SIMDE_FLOAT32_C(  5709.14) } },
    { { SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(     9.36), SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(     3.86),
        SIMDE_FLOAT32_C(     1.10), SIMDE_FLOAT32_C(     1.32), SIMDE_FLOAT32_C(     1.66), SIMDE_FLOAT32_C(     2.44),
        SIMDE_FLOAT32_C(     9.22), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     3.17),
        SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     8.77), SIMDE_FLOAT32_C(     9.18), SIMDE_FLOAT32_C(     0.30) },
      { SIMDE_FLOAT32_C(    11.30), SIMDE_FLOAT32_C( 11613.38), SIMDE_FLOAT32_C(  1393.09), SIMDE_FLOAT32_C(    46.47),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     2.74), SIMDE_FLOAT32_C(     4.26), SIMDE_FLOAT32_C(    10.47),
        SIMDE_FLOAT32_C( 10096.07), SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    22.81),
        SIMDE_FLOAT32_C(   638.06), SIMDE_FLOAT32_C(  6437.18), SIMDE_FLOAT32_C(  9700.16), SIMDE_FLOAT32_C(     0.35) } },
    { { SIMDE_FLOAT32_C(     1.62), SIMDE_FLOAT32_C(     6.06), SIMDE_FLOAT32_C(     9.43), SIMDE_FLOAT32_C(     0.59),
        SIMDE_FLOAT32_C(     4.74), SIMDE_FLOAT32_C(     8.94), SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(     9.67),
        SIMDE_FLOAT32_C(     6.81), SIMDE_FLOAT32_C(     7.35), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     3.50),
        SIMDE_FLOAT32_C(     2.59), SIMDE_FLOAT32_C(     9.75), SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     5.10) },
      { SIMDE_FLOAT32_C(     4.05), SIMDE_FLOAT32_C(   427.38), SIMDE_FLOAT32_C( 12455.53), SIMDE_FLOAT32_C(     0.80),
        SIMDE_FLOAT32_C(   113.43), SIMDE_FLOAT32_C(  7630.19), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C( 15834.35),
        SIMDE_FLOAT32_C(   905.87), SIMDE_FLOAT32_C(  1555.20), SIMDE_FLOAT32_C(  1011.32), SIMDE_FLOAT32_C(    32.12),
        SIMDE_FLOAT32_C(    12.33), SIMDE_FLOAT32_C( 17153.23), SIMDE_FLOAT32_C(     7.50), SIMDE_FLOAT32_C(   163.02) } },
    { { SIMDE_FLOAT32_C(     9.11), SIMDE_FLOAT32_C(     9.39), SIMDE_FLOAT32_C(     8.97), SIMDE_FLOAT32_C(     0.21),
        SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     2.64), SIMDE_FLOAT32_C(     9.92),
        SIMDE_FLOAT32_C(     1.63), SIMDE_FLOAT32_C(     2.67), SIMDE_FLOAT32_C(     3.10), SIMDE_FLOAT32_C(     8.09),
        SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     2.28), SIMDE_FLOAT32_C(     8.38), SIMDE_FLOAT32_C(     3.06) },
      { SIMDE_FLOAT32_C(  9044.29), SIMDE_FLOAT32_C( 11967.10), SIMDE_FLOAT32_C(  7862.60), SIMDE_FLOAT32_C(     0.23),
        SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    13.01), SIMDE_FLOAT32_C( 20331.99),
        SIMDE_FLOAT32_C(     4.10), SIMDE_FLOAT32_C(    13.44), SIMDE_FLOAT32_C(    21.20), SIMDE_FLOAT32_C(  3260.69),
        SIMDE_FLOAT32_C(     3.26), SIMDE_FLOAT32_C(     8.78), SIMDE_FLOAT32_C(  4358.01), SIMDE_FLOAT32_C(    20.33) } },
    { { SIMDE_FLOAT32_C(     8.34), SIMDE_FLOAT32_C(     7.81), SIMDE_FLOAT32_C(     3.65), SIMDE_FLOAT32_C(     3.08),
        SIMDE_FLOAT32_C(     6.75), SIMDE_FLOAT32_C(     4.66), SIMDE_FLOAT32_C(     2.75), SIMDE_FLOAT32_C(     3.56),
        SIMDE_FLOAT32_C(     2.01), SIMDE_FLOAT32_C(     9.67), SIMDE_FLOAT32_C(     7.06), SIMDE_FLOAT32_C(     4.60),
        SIMDE_FLOAT32_C(     9.42), SIMDE_FLOAT32_C(     9.20), SIMDE_FLOAT32_C(     9.71), SIMDE_FLOAT32_C(     8.53) },
      { SIMDE_FLOAT32_C(  4187.09), SIMDE_FLOAT32_C(  2464.13), SIMDE_FLOAT32_C(    37.47), SIMDE_FLOAT32_C(    20.76),
        SIMDE_FLOAT32_C(   853.06), SIMDE_FLOAT32_C(   104.64), SIMDE_FLOAT32_C(    14.64), SIMDE_FLOAT32_C(    34.16),
        SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C( 15834.35), SIMDE_FLOAT32_C(  1163.45), SIMDE_FLOAT32_C(    98.48),
        SIMDE_FLOAT32_C( 12331.58), SIMDE_FLOAT32_C(  9896.13), SIMDE_FLOAT32_C( 16480.60), SIMDE_FLOAT32_C(  5063.44) } },
    { { SIMDE_FLOAT32_C(     8.59), SIMDE_FLOAT32_C(     8.67), SIMDE_FLOAT32_C(     8.74), SIMDE_FLOAT32_C(     9.29),
        SIMDE_FLOAT32_C(     9.30), SIMDE_FLOAT32_C(     1.38), SIMDE_FLOAT32_C(     9.22), SIMDE_FLOAT32_C(     0.93),
        SIMDE_FLOAT32_C(     4.05), SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     9.01), SIMDE_FLOAT32_C(     5.49),
        SIMDE_FLOAT32_C(     4.59), SIMDE_FLOAT32_C(     7.39), SIMDE_FLOAT32_C(     8.56), SIMDE_FLOAT32_C(     2.93) },
      { SIMDE_FLOAT32_C(  5376.61), SIMDE_FLOAT32_C(  5824.50), SIMDE_FLOAT32_C(  6246.89), SIMDE_FLOAT32_C( 10828.18),
        SIMDE_FLOAT32_C( 10937.02), SIMDE_FLOAT32_C(     2.97), SIMDE_FLOAT32_C( 10096.07), SIMDE_FLOAT32_C(     1.53),
        SIMDE_FLOAT32_C(    56.40), SIMDE_FLOAT32_C(     9.07), SIMDE_FLOAT32_C(  8183.52), SIMDE_FLOAT32_C(   241.26),
        SIMDE_FLOAT32_C(    97.49), SIMDE_FLOAT32_C(  1618.71), SIMDE_FLOAT32_C(  5217.68), SIMDE_FLOAT32_C(    17.73) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_expm1_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_expm1_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     9.71), SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.81),
        SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     8.23), SIMDE_FLOAT32_C(     6.49), SIMDE_FLOAT32_C(     5.78),
        SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     3.67), SIMDE_FLOAT32_C(     6.70), SIMDE_FLOAT32_C(     7.95),
        SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     7.57), SIMDE_FLOAT32_C(     6.39), SIMDE_FLOAT32_C(     7.96) },
      UINT8_C(170),
      { SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     4.36), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     5.69),
        SIMDE_FLOAT32_C(     4.94), SIMDE_FLOAT32_C(     9.39), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(     4.73),
        SIMDE_FLOAT32_C(     2.21), SIMDE_FLOAT32_C(     5.91), SIMDE_FLOAT32_C(     7.57), SIMDE_FLOAT32_C(     1.54),
        SIMDE_FLOAT32_C(     8.30), SIMDE_FLOAT32_C(     3.13), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     1.72) },
      { SIMDE_FLOAT32_C(     9.71), SIMDE_FLOAT32_C(    77.26), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(   294.89),
        SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C( 11967.10), SIMDE_FLOAT32_C(     6.49), SIMDE_FLOAT32_C(   112.30),
        SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     3.67), SIMDE_FLOAT32_C(     6.70), SIMDE_FLOAT32_C(     7.95),
        SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     7.57), SIMDE_FLOAT32_C(     6.39), SIMDE_FLOAT32_C(     7.96) } },
    { { SIMDE_FLOAT32_C(     3.18), SIMDE_FLOAT32_C(     2.06), SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(     1.40),
        SIMDE_FLOAT32_C(     8.55), SIMDE_FLOAT32_C(     7.72), SIMDE_FLOAT32_C(     4.74), SIMDE_FLOAT32_C(     2.22),
        SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(     3.40), SIMDE_FLOAT32_C(     1.99),
        SIMDE_FLOAT32_C(     9.08), SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(     1.07) },
      UINT8_C( 91),
      { SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(     6.76), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     1.84),
        SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     5.39), SIMDE_FLOAT32_C(     4.05), SIMDE_FLOAT32_C(     6.19),
        SIMDE_FLOAT32_C(     2.97), SIMDE_FLOAT32_C(     5.59), SIMDE_FLOAT32_C(     4.49), SIMDE_FLOAT32_C(     6.09),
        SIMDE_FLOAT32_C(     6.84), SIMDE_FLOAT32_C(     6.20), SIMDE_FLOAT32_C(     9.27), SIMDE_FLOAT32_C(     8.90) },
      { SIMDE_FLOAT32_C(    10.47), SIMDE_FLOAT32_C(   861.64), SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(     5.30),
        SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     7.72), SIMDE_FLOAT32_C(    56.40), SIMDE_FLOAT32_C(     2.22),
        SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(     3.40), SIMDE_FLOAT32_C(     1.99),
        SIMDE_FLOAT32_C(     9.08), SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(     1.07) } },
    { { SIMDE_FLOAT32_C(     8.15), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     7.45), SIMDE_FLOAT32_C(     5.87),
        SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(     9.67), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     8.10),
        SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     2.28), SIMDE_FLOAT32_C(     7.18), SIMDE_FLOAT32_C(     4.44),
        SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(     8.25), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     5.83) },
      UINT8_C( 10),
      { SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     7.67), SIMDE_FLOAT32_C(     5.29), SIMDE_FLOAT32_C(     6.22),
        SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     1.47), SIMDE_FLOAT32_C(     9.19), SIMDE_FLOAT32_C(     7.31),
        SIMDE_FLOAT32_C(     5.96), SIMDE_FLOAT32_C(     5.28), SIMDE_FLOAT32_C(     4.15), SIMDE_FLOAT32_C(     2.16),
        SIMDE_FLOAT32_C(     4.55), SIMDE_FLOAT32_C(     3.05), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     5.23) },
      { SIMDE_FLOAT32_C(     8.15), SIMDE_FLOAT32_C(  2142.08), SIMDE_FLOAT32_C(     7.45), SIMDE_FLOAT32_C(   501.70),
        SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(     9.67), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     8.10),
        SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     2.28), SIMDE_FLOAT32_C(     7.18), SIMDE_FLOAT32_C(     4.44),
        SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(     8.25), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     5.83) } },
    { { SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     6.17), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.17),
        SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     8.74), SIMDE_FLOAT32_C(     3.24), SIMDE_FLOAT32_C(     8.75),
        SIMDE_FLOAT32_C(     5.92), SIMDE_FLOAT32_C(     7.68), SIMDE_FLOAT32_C(     2.13), SIMDE_FLOAT32_C(     4.17),
        SIMDE_FLOAT32_C(     7.84), SIMDE_FLOAT32_C(     7.97), SIMDE_FLOAT32_C(     9.18), SIMDE_FLOAT32_C(     8.67) },
      UINT8_C(236),
      { SIMDE_FLOAT32_C(     4.47), SIMDE_FLOAT32_C(     4.89), SIMDE_FLOAT32_C(     7.36), SIMDE_FLOAT32_C(     5.94),
        SIMDE_FLOAT32_C(     4.08), SIMDE_FLOAT32_C(     4.67), SIMDE_FLOAT32_C(     1.90), SIMDE_FLOAT32_C(     9.36),
        SIMDE_FLOAT32_C(     8.82), SIMDE_FLOAT32_C(     4.06), SIMDE_FLOAT32_C(     3.91), SIMDE_FLOAT32_C(     1.86),
        SIMDE_FLOAT32_C(     4.37), SIMDE_FLOAT32_C(     9.13), SIMDE_FLOAT32_C(     2.36), SIMDE_FLOAT32_C(     0.55) },
      { SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     6.17), SIMDE_FLOAT32_C(  1570.84), SIMDE_FLOAT32_C(   378.93),
        SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(   105.70), SIMDE_FLOAT32_C(     5.69), SIMDE_FLOAT32_C( 11613.38),
        SIMDE_FLOAT32_C(     5.92), SIMDE_FLOAT32_C(     7.68), SIMDE_FLOAT32_C(     2.13), SIMDE_FLOAT32_C(     4.17),
        SIMDE_FLOAT32_C(     7.84), SIMDE_FLOAT32_C(     7.97), SIMDE_FLOAT32_C(     9.18), SIMDE_FLOAT32_C(     8.67) } },
    { { SIMDE_FLOAT32_C(     9.77), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     7.01), SIMDE_FLOAT32_C(     8.51),
        SIMDE_FLOAT32_C(     5.77), SIMDE_FLOAT32_C(     5.76), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     3.45),
        SIMDE_FLOAT32_C(     7.89), SIMDE_FLOAT32_C(     8.61), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     5.86),
        SIMDE_FLOAT32_C(     7.79), SIMDE_FLOAT32_C(     9.96), SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     2.25) },
      UINT8_C( 16),
      { SIMDE_FLOAT32_C(     8.86), SIMDE_FLOAT32_C(     8.20), SIMDE_FLOAT32_C(     8.92), SIMDE_FLOAT32_C(     3.53),
        SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     8.28), SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(     4.16),
        SIMDE_FLOAT32_C(     2.18), SIMDE_FLOAT32_C(     4.21), SIMDE_FLOAT32_C(     8.53), SIMDE_FLOAT32_C(     1.32),
        SIMDE_FLOAT32_C(     6.57), SIMDE_FLOAT32_C(     9.08), SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(     9.10) },
      { SIMDE_FLOAT32_C(     9.77), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     7.01), SIMDE_FLOAT32_C(     8.51),
        SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     5.76), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     3.45),
        SIMDE_FLOAT32_C(     7.89), SIMDE_FLOAT32_C(     8.61), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     5.86),
        SIMDE_FLOAT32_C(     7.79), SIMDE_FLOAT32_C(     9.96), SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     2.25) } },
    { { SIMDE_FLOAT32_C(     6.09), SIMDE_FLOAT32_C(     9.60), SIMDE_FLOAT32_C(     4.88), SIMDE_FLOAT32_C(     1.85),
        SIMDE_FLOAT32_C(     4.03), SIMDE_FLOAT32_C(     8.33), SIMDE_FLOAT32_C(     9.74), SIMDE_FLOAT32_C(     2.64),
        SIMDE_FLOAT32_C(     9.62), SIMDE_FLOAT32_C(     5.60), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     9.57),
        SIMDE_FLOAT32_C(     7.10), SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     5.97) },
      UINT8_C( 45),
      { SIMDE_FLOAT32_C(     3.34), SIMDE_FLOAT32_C(     9.50), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     1.61),
        SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(     5.13), SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(     6.06),
        SIMDE_FLOAT32_C(     3.66), SIMDE_FLOAT32_C(     5.11), SIMDE_FLOAT32_C(     2.63), SIMDE_FLOAT32_C(     2.74),
        SIMDE_FLOAT32_C(     6.20), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     8.83), SIMDE_FLOAT32_C(     5.80) },
      { SIMDE_FLOAT32_C(    27.22), SIMDE_FLOAT32_C(     9.60), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     4.00),
        SIMDE_FLOAT32_C(     4.03), SIMDE_FLOAT32_C(   168.02), SIMDE_FLOAT32_C(     9.74), SIMDE_FLOAT32_C(     2.64),
        SIMDE_FLOAT32_C(     9.62), SIMDE_FLOAT32_C(     5.60), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     9.57),
        SIMDE_FLOAT32_C(     7.10), SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     5.97) } },
    { { SIMDE_FLOAT32_C(     6.61), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     9.83), SIMDE_FLOAT32_C(     4.94),
        SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(     4.55), SIMDE_FLOAT32_C(     6.02),
        SIMDE_FLOAT32_C(     2.90), SIMDE_FLOAT32_C(     4.12), SIMDE_FLOAT32_C(     3.12), SIMDE_FLOAT32_C(     5.58),
        SIMDE_FLOAT32_C(     8.54), SIMDE_FLOAT32_C(     9.08), SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     1.88) },
      UINT8_C(126),
      { SIMDE_FLOAT32_C(     7.43), SIMDE_FLOAT32_C(     3.49), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     2.56),
        SIMDE_FLOAT32_C(     7.28), SIMDE_FLOAT32_C(     6.48), SIMDE_FLOAT32_C(     6.22), SIMDE_FLOAT32_C(     2.40),
        SIMDE_FLOAT32_C(     9.11), SIMDE_FLOAT32_C(     8.96), SIMDE_FLOAT32_C(     8.60), SIMDE_FLOAT32_C(     0.84),
        SIMDE_FLOAT32_C(     7.79), SIMDE_FLOAT32_C(     4.40), SIMDE_FLOAT32_C(     7.45), SIMDE_FLOAT32_C(     8.47) },
      { SIMDE_FLOAT32_C(     6.61), SIMDE_FLOAT32_C(    31.79), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(    11.94),
        SIMDE_FLOAT32_C(  1449.99), SIMDE_FLOAT32_C(   650.97), SIMDE_FLOAT32_C(   501.70), SIMDE_FLOAT32_C(     6.02),
        SIMDE_FLOAT32_C(     2.90), SIMDE_FLOAT32_C(     4.12), SIMDE_FLOAT32_C(     3.12), SIMDE_FLOAT32_C(     5.58),
        SIMDE_FLOAT32_C(     8.54), SIMDE_FLOAT32_C(     9.08), SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(     1.88) } },
    { { SIMDE_FLOAT32_C(     4.23), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     8.88), SIMDE_FLOAT32_C(     6.71),
        SIMDE_FLOAT32_C(     6.94), SIMDE_FLOAT32_C(     4.90), SIMDE_FLOAT32_C(     9.61), SIMDE_FLOAT32_C(     1.06),
        SIMDE_FLOAT32_C(     8.02), SIMDE_FLOAT32_C(     5.19), SIMDE_FLOAT32_C(     9.60), SIMDE_FLOAT32_C(     7.10),
        SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(     5.68), SIMDE_FLOAT32_C(     9.08) },
      UINT8_C( 53),
      { SIMDE_FLOAT32_C(     6.11), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(     2.59),
        SIMDE_FLOAT32_C(     7.86), SIMDE_FLOAT32_C(     4.65), SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     6.82),
        SIMDE_FLOAT32_C(     3.25), SIMDE_FLOAT32_C(     2.55), SIMDE_FLOAT32_C(     4.61), SIMDE_FLOAT32_C(     7.65),
        SIMDE_FLOAT32_C(    10.00), SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(     2.39) },
      { SIMDE_FLOAT32_C(   449.34), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     8.49), SIMDE_FLOAT32_C(     6.71),
        SIMDE_FLOAT32_C(  2590.52), SIMDE_FLOAT32_C(   103.58), SIMDE_FLOAT32_C(     9.61), SIMDE_FLOAT32_C(     1.06),
        SIMDE_FLOAT32_C(     8.02), SIMDE_FLOAT32_C(     5.19), SIMDE_FLOAT32_C(     9.60), SIMDE_FLOAT32_C(     7.10),
        SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(     5.68), SIMDE_FLOAT32_C(     9.08) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_expm1_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_expm1_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     1.40), SIMDE_FLOAT64_C(     0.40), SIMDE_FLOAT64_C(     7.27),
        SIMDE_FLOAT64_C(     9.13), SIMDE_FLOAT64_C(     1.31), SIMDE_FLOAT64_C(     2.56), SIMDE_FLOAT64_C(     1.21) },
      { SIMDE_FLOAT64_C(     0.40), SIMDE_FLOAT64_C(     3.06), SIMDE_FLOAT64_C(     0.49), SIMDE_FLOAT64_C(  1435.55),
        SIMDE_FLOAT64_C(  9227.02), SIMDE_FLOAT64_C(     2.71), SIMDE_FLOAT64_C(    11.94), SIMDE_FLOAT64_C(     2.35) } },
    { { SIMDE_FLOAT64_C(     6.72), SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(     3.99), SIMDE_FLOAT64_C(     2.10),
        SIMDE_FLOAT64_C(     2.80), SIMDE_FLOAT64_C(     5.43), SIMDE_FLOAT64_C(     3.71), SIMDE_FLOAT64_C(     6.65) },
      { SIMDE_FLOAT64_C(   827.82), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(    53.05), SIMDE_FLOAT64_C(     7.17),
        SIMDE_FLOAT64_C(    15.44), SIMDE_FLOAT64_C(   227.15), SIMDE_FLOAT64_C(    39.85), SIMDE_FLOAT64_C(   771.78) } },
    { { SIMDE_FLOAT64_C(     3.81), SIMDE_FLOAT64_C(     4.42), SIMDE_FLOAT64_C(     8.46), SIMDE_FLOAT64_C(     3.88),
        SIMDE_FLOAT64_C(     7.48), SIMDE_FLOAT64_C(     9.11), SIMDE_FLOAT64_C(     0.59), SIMDE_FLOAT64_C(     5.94) },
      { SIMDE_FLOAT64_C(    44.15), SIMDE_FLOAT64_C(    82.10), SIMDE_FLOAT64_C(  4721.06), SIMDE_FLOAT64_C(    47.42),
        SIMDE_FLOAT64_C(  1771.24), SIMDE_FLOAT64_C(  9044.29), SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(   378.93) } },
    { { SIMDE_FLOAT64_C(     7.31), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     9.76), SIMDE_FLOAT64_C(     8.87),
        SIMDE_FLOAT64_C(     7.78), SIMDE_FLOAT64_C(     3.26), SIMDE_FLOAT64_C(     6.27), SIMDE_FLOAT64_C(     8.12) },
      { SIMDE_FLOAT64_C(  1494.18), SIMDE_FLOAT64_C(     0.75), SIMDE_FLOAT64_C( 17325.63), SIMDE_FLOAT64_C(  7114.28),
        SIMDE_FLOAT64_C(  2391.27), SIMDE_FLOAT64_C(    25.05), SIMDE_FLOAT64_C(   527.48), SIMDE_FLOAT64_C(  3360.02) } },
    { { SIMDE_FLOAT64_C(     4.67), SIMDE_FLOAT64_C(     6.67), SIMDE_FLOAT64_C(     5.39), SIMDE_FLOAT64_C(     3.79),
        SIMDE_FLOAT64_C(     7.97), SIMDE_FLOAT64_C(     7.95), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     4.69) },
      { SIMDE_FLOAT64_C(   105.70), SIMDE_FLOAT64_C(   787.40), SIMDE_FLOAT64_C(   218.20), SIMDE_FLOAT64_C(    43.26),
        SIMDE_FLOAT64_C(  2891.86), SIMDE_FLOAT64_C(  2834.57), SIMDE_FLOAT64_C(   147.41), SIMDE_FLOAT64_C(   107.85) } },
    { { SIMDE_FLOAT64_C(     8.47), SIMDE_FLOAT64_C(     9.00), SIMDE_FLOAT64_C(     6.79), SIMDE_FLOAT64_C(     1.27),
        SIMDE_FLOAT64_C(     4.42), SIMDE_FLOAT64_C(     0.49), SIMDE_FLOAT64_C(     7.92), SIMDE_FLOAT64_C(     8.23) },
      { SIMDE_FLOAT64_C(  4768.52), SIMDE_FLOAT64_C(  8102.08), SIMDE_FLOAT64_C(   887.91), SIMDE_FLOAT64_C(     2.56),
        SIMDE_FLOAT64_C(    82.10), SIMDE_FLOAT64_C(     0.63), SIMDE_FLOAT64_C(  2750.77), SIMDE_FLOAT64_C(  3750.83) } },
    { { SIMDE_FLOAT64_C(     4.92), SIMDE_FLOAT64_C(     6.38), SIMDE_FLOAT64_C(     2.12), SIMDE_FLOAT64_C(     2.40),
        SIMDE_FLOAT64_C(     5.49), SIMDE_FLOAT64_C(     2.70), SIMDE_FLOAT64_C(     8.35), SIMDE_FLOAT64_C(     2.80) },
      { SIMDE_FLOAT64_C(   136.00), SIMDE_FLOAT64_C(   588.93), SIMDE_FLOAT64_C(     7.33), SIMDE_FLOAT64_C(    10.02),
        SIMDE_FLOAT64_C(   241.26), SIMDE_FLOAT64_C(    13.88), SIMDE_FLOAT64_C(  4229.18), SIMDE_FLOAT64_C(    15.44) } },
    { { SIMDE_FLOAT64_C(     3.27), SIMDE_FLOAT64_C(     8.10), SIMDE_FLOAT64_C(     1.67), SIMDE_FLOAT64_C(     1.04),
        SIMDE_FLOAT64_C(     1.36), SIMDE_FLOAT64_C(     7.94), SIMDE_FLOAT64_C(     9.16), SIMDE_FLOAT64_C(     6.03) },
      { SIMDE_FLOAT64_C(    25.31), SIMDE_FLOAT64_C(  3293.47), SIMDE_FLOAT64_C(     4.31), SIMDE_FLOAT64_C(     1.83),
        SIMDE_FLOAT64_C(     2.90), SIMDE_FLOAT64_C(  2806.36), SIMDE_FLOAT64_C(  9508.06), SIMDE_FLOAT64_C(   414.72) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_expm1_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_expm1_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     4.06), SIMDE_FLOAT64_C(     9.24), SIMDE_FLOAT64_C(     8.55), SIMDE_FLOAT64_C(     9.59),
        SIMDE_FLOAT64_C(     0.69), SIMDE_FLOAT64_C(     2.26), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     4.06) },
      UINT8_C(110),
      { SIMDE_FLOAT64_C(     5.74), SIMDE_FLOAT64_C(     3.51), SIMDE_FLOAT64_C(     5.07), SIMDE_FLOAT64_C(     6.58),
        SIMDE_FLOAT64_C(     8.73), SIMDE_FLOAT64_C(     4.57), SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(     7.96) },
      { SIMDE_FLOAT64_C(     4.06), SIMDE_FLOAT64_C(    32.45), SIMDE_FLOAT64_C(   158.17), SIMDE_FLOAT64_C(   719.54),
        SIMDE_FLOAT64_C(     0.69), SIMDE_FLOAT64_C(    95.54), SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     4.06) } },
    { { SIMDE_FLOAT64_C(     1.32), SIMDE_FLOAT64_C(     1.91), SIMDE_FLOAT64_C(     7.33), SIMDE_FLOAT64_C(     4.66),
        SIMDE_FLOAT64_C(     3.27), SIMDE_FLOAT64_C(     4.31), SIMDE_FLOAT64_C(     0.71), SIMDE_FLOAT64_C(     9.20) },
      UINT8_C(124),
      { SIMDE_FLOAT64_C(     5.28), SIMDE_FLOAT64_C(     1.80), SIMDE_FLOAT64_C(     4.85), SIMDE_FLOAT64_C(     0.49),
        SIMDE_FLOAT64_C(     1.99), SIMDE_FLOAT64_C(     8.91), SIMDE_FLOAT64_C(     9.72), SIMDE_FLOAT64_C(     0.53) },
      { SIMDE_FLOAT64_C(     1.32), SIMDE_FLOAT64_C(     1.91), SIMDE_FLOAT64_C(   126.74), SIMDE_FLOAT64_C(     0.63),
        SIMDE_FLOAT64_C(     6.32), SIMDE_FLOAT64_C(  7404.66), SIMDE_FLOAT64_C( 16646.24), SIMDE_FLOAT64_C(     9.20) } },
    { { SIMDE_FLOAT64_C(     8.50), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(     2.80), SIMDE_FLOAT64_C(     9.06),
        SIMDE_FLOAT64_C(     4.48), SIMDE_FLOAT64_C(     0.59), SIMDE_FLOAT64_C(     4.80), SIMDE_FLOAT64_C(     7.99) },
      UINT8_C( 51),
      { SIMDE_FLOAT64_C(     1.38), SIMDE_FLOAT64_C(     6.72), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     1.85),
        SIMDE_FLOAT64_C(     4.68), SIMDE_FLOAT64_C(     1.54), SIMDE_FLOAT64_C(     3.76), SIMDE_FLOAT64_C(     2.01) },
      { SIMDE_FLOAT64_C(     2.97), SIMDE_FLOAT64_C(   827.82), SIMDE_FLOAT64_C(     2.80), SIMDE_FLOAT64_C(     9.06),
        SIMDE_FLOAT64_C(   106.77), SIMDE_FLOAT64_C(     3.66), SIMDE_FLOAT64_C(     4.80), SIMDE_FLOAT64_C(     7.99) } },
    { { SIMDE_FLOAT64_C(     6.20), SIMDE_FLOAT64_C(     7.03), SIMDE_FLOAT64_C(     6.32), SIMDE_FLOAT64_C(     6.91),
        SIMDE_FLOAT64_C(     6.23), SIMDE_FLOAT64_C(     3.88), SIMDE_FLOAT64_C(     2.18), SIMDE_FLOAT64_C(     8.02) },
      UINT8_C(179),
      { SIMDE_FLOAT64_C(     2.67), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     7.63), SIMDE_FLOAT64_C(     2.40),
        SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(     6.13), SIMDE_FLOAT64_C(     2.81), SIMDE_FLOAT64_C(     3.34) },
      { SIMDE_FLOAT64_C(    13.44), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     6.32), SIMDE_FLOAT64_C(     6.91),
        SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(   458.44), SIMDE_FLOAT64_C(     2.18), SIMDE_FLOAT64_C(    27.22) } },
    { { SIMDE_FLOAT64_C(     5.19), SIMDE_FLOAT64_C(     7.29), SIMDE_FLOAT64_C(     3.93), SIMDE_FLOAT64_C(    10.00),
        SIMDE_FLOAT64_C(     5.28), SIMDE_FLOAT64_C(     9.58), SIMDE_FLOAT64_C(     1.38), SIMDE_FLOAT64_C(     2.00) },
      UINT8_C(216),
      { SIMDE_FLOAT64_C(     3.23), SIMDE_FLOAT64_C(     6.68), SIMDE_FLOAT64_C(     1.35), SIMDE_FLOAT64_C(     6.99),
        SIMDE_FLOAT64_C(     8.69), SIMDE_FLOAT64_C(     7.55), SIMDE_FLOAT64_C(     4.02), SIMDE_FLOAT64_C(     5.01) },
      { SIMDE_FLOAT64_C(     5.19), SIMDE_FLOAT64_C(     7.29), SIMDE_FLOAT64_C(     3.93), SIMDE_FLOAT64_C(  1084.72),
        SIMDE_FLOAT64_C(  5942.18), SIMDE_FLOAT64_C(     9.58), SIMDE_FLOAT64_C(    54.70), SIMDE_FLOAT64_C(   148.90) } },
    { { SIMDE_FLOAT64_C(     4.45), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     8.89), SIMDE_FLOAT64_C(     6.64),
        SIMDE_FLOAT64_C(     8.27), SIMDE_FLOAT64_C(     7.61), SIMDE_FLOAT64_C(     9.31), SIMDE_FLOAT64_C(     8.28) },
      UINT8_C(185),
      { SIMDE_FLOAT64_C(     1.71), SIMDE_FLOAT64_C(     8.83), SIMDE_FLOAT64_C(     1.38), SIMDE_FLOAT64_C(     4.52),
        SIMDE_FLOAT64_C(     2.17), SIMDE_FLOAT64_C(     6.57), SIMDE_FLOAT64_C(     1.81), SIMDE_FLOAT64_C(     6.09) },
      { SIMDE_FLOAT64_C(     4.53), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     8.89), SIMDE_FLOAT64_C(    90.84),
        SIMDE_FLOAT64_C(     7.76), SIMDE_FLOAT64_C(   712.37), SIMDE_FLOAT64_C(     9.31), SIMDE_FLOAT64_C(   440.42) } },
    { { SIMDE_FLOAT64_C(     6.57), SIMDE_FLOAT64_C(     7.09), SIMDE_FLOAT64_C(     5.68), SIMDE_FLOAT64_C(     7.95),
        SIMDE_FLOAT64_C(     9.09), SIMDE_FLOAT64_C(     5.48), SIMDE_FLOAT64_C(     1.18), SIMDE_FLOAT64_C(     5.77) },
      UINT8_C(171),
      { SIMDE_FLOAT64_C(     8.17), SIMDE_FLOAT64_C(     4.46), SIMDE_FLOAT64_C(     4.37), SIMDE_FLOAT64_C(     2.19),
        SIMDE_FLOAT64_C(     9.47), SIMDE_FLOAT64_C(     8.83), SIMDE_FLOAT64_C(     2.44), SIMDE_FLOAT64_C(     8.36) },
      { SIMDE_FLOAT64_C(  3532.34), SIMDE_FLOAT64_C(    85.49), SIMDE_FLOAT64_C(     5.68), SIMDE_FLOAT64_C(     7.94),
        SIMDE_FLOAT64_C(     9.09), SIMDE_FLOAT64_C(  6835.29), SIMDE_FLOAT64_C(     1.18), SIMDE_FLOAT64_C(  4271.69) } },
    { { SIMDE_FLOAT64_C(     5.47), SIMDE_FLOAT64_C(     0.71), SIMDE_FLOAT64_C(     5.97), SIMDE_FLOAT64_C(     4.78),
        SIMDE_FLOAT64_C(     9.00), SIMDE_FLOAT64_C(     1.22), SIMDE_FLOAT64_C(     6.48), SIMDE_FLOAT64_C(     7.82) },
      UINT8_C(171),
      { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     9.99), SIMDE_FLOAT64_C(     9.17), SIMDE_FLOAT64_C(     2.81),
        SIMDE_FLOAT64_C(     6.09), SIMDE_FLOAT64_C(     5.74), SIMDE_FLOAT64_C(     9.89), SIMDE_FLOAT64_C(     1.76) },
      { SIMDE_FLOAT64_C(     1.72), SIMDE_FLOAT64_C( 21806.30), SIMDE_FLOAT64_C(     5.97), SIMDE_FLOAT64_C(    15.61),
        SIMDE_FLOAT64_C(     9.00), SIMDE_FLOAT64_C(   310.06), SIMDE_FLOAT64_C(     6.48), SIMDE_FLOAT64_C(     4.81) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_expm1_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_exp2_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    -2.08), SIMDE_FLOAT32_C(     1.71), SIMDE_FLOAT32_C(     2.58), SIMDE_FLOAT32_C(    -1.10) },
      { SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     3.27), SIMDE_FLOAT32_C(     5.98), SIMDE_FLOAT32_C(     0.47) } },
    { { SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(    -2.92), SIMDE_FLOAT32_C(    -3.15) },
      { SIMDE_FLOAT32_C(     3.14), SIMDE_FLOAT32_C(     1.21), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.11) } },
    { { SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -3.95), SIMDE_FLOAT32_C(    -1.01) },
      { SIMDE_FLOAT32_C(     4.11), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.50) } },
    { { SIMDE_FLOAT32_C(    -2.84), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -3.08), SIMDE_FLOAT32_C(     0.96) },
      { SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     1.95) } },
    { { SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -1.38), SIMDE_FLOAT32_C(    -3.16), SIMDE_FLOAT32_C(     0.33) },
      { SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     1.26) } },
    { { SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(    -3.70), SIMDE_FLOAT32_C(    -0.75) },
      { SIMDE_FLOAT32_C(     1.21), SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.59) } },
    { { SIMDE_FLOAT32_C(    -1.25), SIMDE_FLOAT32_C(    -2.03), SIMDE_FLOAT32_C(    -1.41), SIMDE_FLOAT32_C(    -1.44) },
      { SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.37) } },
    { { SIMDE_FLOAT32_C(    -2.57), SIMDE_FLOAT32_C(    -1.64), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(    -0.66) },
      { SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     2.36), SIMDE_FLOAT32_C(     0.63) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_exp2_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_exp2_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    -1.05), SIMDE_FLOAT64_C(    -3.96) },
      { SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(     0.06) } },
    { { SIMDE_FLOAT64_C(    -3.17), SIMDE_FLOAT64_C(    -0.18) },
      { SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     0.88) } },
    { { SIMDE_FLOAT64_C(     2.75), SIMDE_FLOAT64_C(    -3.78) },
      { SIMDE_FLOAT64_C(     6.73), SIMDE_FLOAT64_C(     0.07) } },
    { { SIMDE_FLOAT64_C(    -3.43), SIMDE_FLOAT64_C(     0.85) },
      { SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     1.80) } },
    { { SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     1.23) },
      { SIMDE_FLOAT64_C(     1.29), SIMDE_FLOAT64_C(     2.35) } },
    { { SIMDE_FLOAT64_C(     1.92), SIMDE_FLOAT64_C(    -0.38) },
      { SIMDE_FLOAT64_C(     3.78), SIMDE_FLOAT64_C(     0.77) } },
    { { SIMDE_FLOAT64_C(     3.87), SIMDE_FLOAT64_C(     2.98) },
      { SIMDE_FLOAT64_C(    14.62), SIMDE_FLOAT64_C(     7.89) } },
    { { SIMDE_FLOAT64_C(    -1.16), SIMDE_FLOAT64_C(     1.76) },
      { SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     3.39) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_exp2_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_exp2_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(     0.64),
        SIMDE_FLOAT32_C(    -1.90), SIMDE_FLOAT32_C(    -1.78), SIMDE_FLOAT32_C(    -1.91), SIMDE_FLOAT32_C(    -1.34) },
      { SIMDE_FLOAT32_C(     7.36), SIMDE_FLOAT32_C(    15.78), SIMDE_FLOAT32_C(     6.82), SIMDE_FLOAT32_C(     1.56),
        SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.40) } },
    { { SIMDE_FLOAT32_C(    -2.07), SIMDE_FLOAT32_C(    -3.29), SIMDE_FLOAT32_C(    -3.96), SIMDE_FLOAT32_C(     0.14),
        SIMDE_FLOAT32_C(    -3.42), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.63) },
      { SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     1.10),
        SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(     1.55) } },
    { { SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    -2.58), SIMDE_FLOAT32_C(     1.40),
        SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(     2.84), SIMDE_FLOAT32_C(     2.74), SIMDE_FLOAT32_C(    -3.03) },
      { SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     2.64),
        SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     7.16), SIMDE_FLOAT32_C(     6.68), SIMDE_FLOAT32_C(     0.12) } },
    { { SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(    -3.44), SIMDE_FLOAT32_C(    -3.79), SIMDE_FLOAT32_C(     3.43),
        SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(    -3.35), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     2.71) },
      { SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(    10.78),
        SIMDE_FLOAT32_C(    14.32), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     6.54) } },
    { { SIMDE_FLOAT32_C(    -3.37), SIMDE_FLOAT32_C(    -2.06), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(    -1.27),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     1.44), SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     2.10) },
      { SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.41),
        SIMDE_FLOAT32_C(     1.13), SIMDE_FLOAT32_C(     2.71), SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     4.29) } },
    { { SIMDE_FLOAT32_C(     2.15), SIMDE_FLOAT32_C(     1.43), SIMDE_FLOAT32_C(    -1.76), SIMDE_FLOAT32_C(     2.73),
        SIMDE_FLOAT32_C(    -2.98), SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     0.75) },
      { SIMDE_FLOAT32_C(     4.44), SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     6.63),
        SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     6.45), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     1.68) } },
    { { SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -1.85), SIMDE_FLOAT32_C(     2.05),
        SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -3.11), SIMDE_FLOAT32_C(     3.02), SIMDE_FLOAT32_C(    -1.59) },
      { SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     4.14),
        SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     0.33) } },
    { { SIMDE_FLOAT32_C(    -2.54), SIMDE_FLOAT32_C(     3.23), SIMDE_FLOAT32_C(    -2.16), SIMDE_FLOAT32_C(    -2.71),
        SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(     1.02), SIMDE_FLOAT32_C(    -4.00), SIMDE_FLOAT32_C(    -3.49) },
      { SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     9.38), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.15),
        SIMDE_FLOAT32_C(    14.72), SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.09) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_exp2_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_exp2_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     1.66), SIMDE_FLOAT64_C(    -0.38), SIMDE_FLOAT64_C(     1.40), SIMDE_FLOAT64_C(     3.84) },
      { SIMDE_FLOAT64_C(     3.16), SIMDE_FLOAT64_C(     0.77), SIMDE_FLOAT64_C(     2.64), SIMDE_FLOAT64_C(    14.32) } },
    { { SIMDE_FLOAT64_C(    -2.15), SIMDE_FLOAT64_C(    -0.83), SIMDE_FLOAT64_C(    -2.32), SIMDE_FLOAT64_C(     1.94) },
      { SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(     3.84) } },
    { { SIMDE_FLOAT64_C(     3.43), SIMDE_FLOAT64_C(    -1.57), SIMDE_FLOAT64_C(    -0.88), SIMDE_FLOAT64_C(     0.76) },
      { SIMDE_FLOAT64_C(    10.78), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(     1.69) } },
    { { SIMDE_FLOAT64_C(     1.69), SIMDE_FLOAT64_C(     2.74), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(    -2.45) },
      { SIMDE_FLOAT64_C(     3.23), SIMDE_FLOAT64_C(     6.68), SIMDE_FLOAT64_C(     1.99), SIMDE_FLOAT64_C(     0.18) } },
    { { SIMDE_FLOAT64_C(     2.22), SIMDE_FLOAT64_C(     1.74), SIMDE_FLOAT64_C(     3.15), SIMDE_FLOAT64_C(     0.54) },
      { SIMDE_FLOAT64_C(     4.66), SIMDE_FLOAT64_C(     3.34), SIMDE_FLOAT64_C(     8.88), SIMDE_FLOAT64_C(     1.45) } },
    { { SIMDE_FLOAT64_C(     1.30), SIMDE_FLOAT64_C(    -1.80), SIMDE_FLOAT64_C(     2.76), SIMDE_FLOAT64_C(    -4.00) },
      { SIMDE_FLOAT64_C(     2.46), SIMDE_FLOAT64_C(     0.29), SIMDE_FLOAT64_C(     6.77), SIMDE_FLOAT64_C(     0.06) } },
    { { SIMDE_FLOAT64_C(    -2.49), SIMDE_FLOAT64_C(    -1.07), SIMDE_FLOAT64_C(     1.81), SIMDE_FLOAT64_C(     0.86) },
      { SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(     3.51), SIMDE_FLOAT64_C(     1.82) } },
    { { SIMDE_FLOAT64_C(    -2.31), SIMDE_FLOAT64_C(    -2.25), SIMDE_FLOAT64_C(     2.43), SIMDE_FLOAT64_C(     3.36) },
      { SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(     5.39), SIMDE_FLOAT64_C(    10.27) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_exp2_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_exp2_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(    -1.43), SIMDE_FLOAT32_C(     3.90),
        SIMDE_FLOAT32_C(     2.40), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.73),
        SIMDE_FLOAT32_C(     1.74), SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(    -1.28), SIMDE_FLOAT32_C(     1.46),
        SIMDE_FLOAT32_C(     2.43), SIMDE_FLOAT32_C(    -3.47), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(     3.90) },
      { SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(    14.93),
        SIMDE_FLOAT32_C(     5.28), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     1.66),
        SIMDE_FLOAT32_C(     3.34), SIMDE_FLOAT32_C(     4.76), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     2.75),
        SIMDE_FLOAT32_C(     5.39), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    14.93) } },
    { { SIMDE_FLOAT32_C(    -3.96), SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(    -3.40), SIMDE_FLOAT32_C(    -2.09),
        SIMDE_FLOAT32_C(    -2.19), SIMDE_FLOAT32_C(     1.78), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(    -3.80),
        SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(    -1.05), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     2.82),
        SIMDE_FLOAT32_C(    -1.74), SIMDE_FLOAT32_C(     2.82), SIMDE_FLOAT32_C(     3.23), SIMDE_FLOAT32_C(     2.11) },
      { SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     5.43), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.23),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     3.43), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.07),
        SIMDE_FLOAT32_C(     2.97), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(     7.06),
        SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     7.06), SIMDE_FLOAT32_C(     9.38), SIMDE_FLOAT32_C(     4.32) } },
    { { SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -2.20), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(    -2.54),
        SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(    -2.24), SIMDE_FLOAT32_C(     2.19), SIMDE_FLOAT32_C(    -0.24),
        SIMDE_FLOAT32_C(    -3.99), SIMDE_FLOAT32_C(    -3.09), SIMDE_FLOAT32_C(    -2.77), SIMDE_FLOAT32_C(     2.43),
        SIMDE_FLOAT32_C(    -2.56), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(    -2.52) },
      { SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     4.06), SIMDE_FLOAT32_C(     0.17),
        SIMDE_FLOAT32_C(     4.06), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     4.56), SIMDE_FLOAT32_C(     0.85),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     5.39),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     1.71), SIMDE_FLOAT32_C(     5.03), SIMDE_FLOAT32_C(     0.17) } },
    { { SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     2.93), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     1.02),
        SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     3.30), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(    -1.72),
        SIMDE_FLOAT32_C(    -1.76), SIMDE_FLOAT32_C(    -2.42), SIMDE_FLOAT32_C(    -2.90), SIMDE_FLOAT32_C(     0.50),
        SIMDE_FLOAT32_C(    -3.60), SIMDE_FLOAT32_C(    -3.67), SIMDE_FLOAT32_C(    -1.38), SIMDE_FLOAT32_C(    -0.54) },
      { SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     7.62), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     2.03),
        SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     9.85), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     0.30),
        SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     1.41),
        SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.69) } },
    { { SIMDE_FLOAT32_C(    -1.87), SIMDE_FLOAT32_C(    -3.37), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -3.85),
        SIMDE_FLOAT32_C(    -1.61), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(    -1.60),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(    -3.17), SIMDE_FLOAT32_C(     1.48),
        SIMDE_FLOAT32_C(    -2.09), SIMDE_FLOAT32_C(     3.16), SIMDE_FLOAT32_C(     2.96), SIMDE_FLOAT32_C(     1.12) },
      { SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(     0.07),
        SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.33),
        SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     2.79),
        SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     8.94), SIMDE_FLOAT32_C(     7.78), SIMDE_FLOAT32_C(     2.17) } },
    { { SIMDE_FLOAT32_C(     2.09), SIMDE_FLOAT32_C(    -1.64), SIMDE_FLOAT32_C(    -1.86), SIMDE_FLOAT32_C(    -1.20),
        SIMDE_FLOAT32_C(    -2.34), SIMDE_FLOAT32_C(     3.36), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(    -0.10),
        SIMDE_FLOAT32_C(    -3.05), SIMDE_FLOAT32_C(     2.18), SIMDE_FLOAT32_C(    -3.59), SIMDE_FLOAT32_C(    -2.65),
        SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -3.36) },
      { SIMDE_FLOAT32_C(     4.26), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.44),
        SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(    10.27), SIMDE_FLOAT32_C(     2.11), SIMDE_FLOAT32_C(     0.93),
        SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     4.53), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(     5.74), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     0.10) } },
    { { SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(    -2.26), SIMDE_FLOAT32_C(    -3.21), SIMDE_FLOAT32_C(     2.05),
        SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    -3.55), SIMDE_FLOAT32_C(    -3.10),
        SIMDE_FLOAT32_C(    -2.16), SIMDE_FLOAT32_C(    -2.72), SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(    -0.25),
        SIMDE_FLOAT32_C(    -3.56), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(    -3.13), SIMDE_FLOAT32_C(     2.53) },
      { SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     4.14),
        SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(     1.62), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.12),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     5.21), SIMDE_FLOAT32_C(     0.84),
        SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     5.78) } },
    { { SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -2.68), SIMDE_FLOAT32_C(    -2.64),
        SIMDE_FLOAT32_C(    -1.63), SIMDE_FLOAT32_C(     2.40), SIMDE_FLOAT32_C(     1.26), SIMDE_FLOAT32_C(    -0.68),
        SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.67), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.90),
        SIMDE_FLOAT32_C(    -3.31), SIMDE_FLOAT32_C(    -2.52), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.35) },
      { SIMDE_FLOAT32_C(    13.00), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     5.28), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     0.62),
        SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(     3.18), SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(     0.54),
        SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     1.27) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_exp2_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_exp2_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     3.64), SIMDE_FLOAT32_C(     1.77),
        SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(    -2.53), SIMDE_FLOAT32_C(    -1.72), SIMDE_FLOAT32_C(    -1.12),
        SIMDE_FLOAT32_C(    -3.88), SIMDE_FLOAT32_C(     1.32), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(    -0.68),
        SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     3.54), SIMDE_FLOAT32_C(    -2.21) },
      UINT8_C(173),
      { SIMDE_FLOAT32_C(    -3.02), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     3.49), SIMDE_FLOAT32_C(    -2.99),
        SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(    -3.69), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     2.30),
        SIMDE_FLOAT32_C(     3.57), SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -0.19),
        SIMDE_FLOAT32_C(     3.28), SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(    -2.25), SIMDE_FLOAT32_C(    -1.70) },
      { SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    11.24), SIMDE_FLOAT32_C(     0.13),
        SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(    -1.72), SIMDE_FLOAT32_C(     4.92),
        SIMDE_FLOAT32_C(    -3.88), SIMDE_FLOAT32_C(     1.32), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(    -0.68),
        SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     3.54), SIMDE_FLOAT32_C(    -2.21) } },
    { { SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(    -3.95), SIMDE_FLOAT32_C(     2.97),
        SIMDE_FLOAT32_C(    -2.20), SIMDE_FLOAT32_C(    -1.07), SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(     3.12),
        SIMDE_FLOAT32_C(     3.97), SIMDE_FLOAT32_C(    -1.59), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(    -2.05), SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(    -3.38), SIMDE_FLOAT32_C(    -1.07) },
      UINT8_C(225),
      { SIMDE_FLOAT32_C(    -3.89), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -2.82), SIMDE_FLOAT32_C(    -3.58),
        SIMDE_FLOAT32_C(    -3.89), SIMDE_FLOAT32_C(     3.49), SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(     2.55),
        SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(    -1.10),
        SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(    -3.87), SIMDE_FLOAT32_C(    -3.60), SIMDE_FLOAT32_C(     1.07) },
      { SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(    -3.95), SIMDE_FLOAT32_C(     2.97),
        SIMDE_FLOAT32_C(    -2.20), SIMDE_FLOAT32_C(    11.24), SIMDE_FLOAT32_C(    15.89), SIMDE_FLOAT32_C(     5.86),
        SIMDE_FLOAT32_C(     3.97), SIMDE_FLOAT32_C(    -1.59), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(    -2.05), SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(    -3.38), SIMDE_FLOAT32_C(    -1.07) } },
    { { SIMDE_FLOAT32_C(    -3.82), SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(    -0.89),
        SIMDE_FLOAT32_C(     2.46), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -3.13),
        SIMDE_FLOAT32_C(    -1.99), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(    -1.18), SIMDE_FLOAT32_C(     3.80),
        SIMDE_FLOAT32_C(    -3.53), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(    -1.07), SIMDE_FLOAT32_C(    -3.42) },
      UINT8_C(147),
      { SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -3.00), SIMDE_FLOAT32_C(    -2.20), SIMDE_FLOAT32_C(    -0.40),
        SIMDE_FLOAT32_C(    -3.00), SIMDE_FLOAT32_C(    -3.65), SIMDE_FLOAT32_C(    -3.35), SIMDE_FLOAT32_C(     0.80),
        SIMDE_FLOAT32_C(     2.18), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(    -1.64), SIMDE_FLOAT32_C(     2.31),
        SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     3.43), SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(    -0.67) },
      { SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(    -0.89),
        SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(     1.74),
        SIMDE_FLOAT32_C(    -1.99), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(    -1.18), SIMDE_FLOAT32_C(     3.80),
        SIMDE_FLOAT32_C(    -3.53), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(    -1.07), SIMDE_FLOAT32_C(    -3.42) } },
    { { SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(    -2.39), SIMDE_FLOAT32_C(    -2.21), SIMDE_FLOAT32_C(     0.31),
        SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -1.33), SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     0.52),
        SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(     2.12), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.76),
        SIMDE_FLOAT32_C(    -2.95), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     1.17) },
      UINT8_C( 16),
      { SIMDE_FLOAT32_C(     2.73), SIMDE_FLOAT32_C(    -3.23), SIMDE_FLOAT32_C(     3.57), SIMDE_FLOAT32_C(     3.08),
        SIMDE_FLOAT32_C(    -2.59), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     1.26), SIMDE_FLOAT32_C(     0.97),
        SIMDE_FLOAT32_C(     2.73), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -3.08), SIMDE_FLOAT32_C(     2.17),
        SIMDE_FLOAT32_C(    -1.93), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.33) },
      { SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(    -2.39), SIMDE_FLOAT32_C(    -2.21), SIMDE_FLOAT32_C(     0.31),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -1.33), SIMDE_FLOAT32_C(     2.32), SIMDE_FLOAT32_C(     0.52),
        SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(     2.12), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.76),
        SIMDE_FLOAT32_C(    -2.95), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     1.17) } },
    { { SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(    -3.21), SIMDE_FLOAT32_C(    -3.65), SIMDE_FLOAT32_C(    -3.29),
        SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     2.21), SIMDE_FLOAT32_C(     1.23),
        SIMDE_FLOAT32_C(    -2.13), SIMDE_FLOAT32_C(    -2.55), SIMDE_FLOAT32_C(     2.29), SIMDE_FLOAT32_C(     3.44),
        SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     2.01), SIMDE_FLOAT32_C(     1.11) },
      UINT8_C(254),
      { SIMDE_FLOAT32_C(     1.58), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     1.63), SIMDE_FLOAT32_C(    -2.04),
        SIMDE_FLOAT32_C(    -2.55), SIMDE_FLOAT32_C(    -1.40), SIMDE_FLOAT32_C(    -3.31), SIMDE_FLOAT32_C(     1.02),
        SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(     2.86), SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(     3.77),
        SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(    -1.23), SIMDE_FLOAT32_C(     1.81), SIMDE_FLOAT32_C(     0.13) },
      { SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     3.10), SIMDE_FLOAT32_C(     0.24),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     2.03),
        SIMDE_FLOAT32_C(    -2.13), SIMDE_FLOAT32_C(    -2.55), SIMDE_FLOAT32_C(     2.29), SIMDE_FLOAT32_C(     3.44),
        SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     2.01), SIMDE_FLOAT32_C(     1.11) } },
    { { SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     4.00),
        SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -3.52), SIMDE_FLOAT32_C(    -2.14), SIMDE_FLOAT32_C(     2.18),
        SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(    -2.70), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -1.78),
        SIMDE_FLOAT32_C(     3.31), SIMDE_FLOAT32_C(    -2.32), SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(     0.89) },
      UINT8_C(128),
      { SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     2.85), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(     2.67),
        SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -1.81), SIMDE_FLOAT32_C(     2.41),
        SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -2.03), SIMDE_FLOAT32_C(    -2.25), SIMDE_FLOAT32_C(     2.20),
        SIMDE_FLOAT32_C(     3.78), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(    -2.68), SIMDE_FLOAT32_C(     2.31) },
      { SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     4.00),
        SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -3.52), SIMDE_FLOAT32_C(    -2.14), SIMDE_FLOAT32_C(     5.31),
        SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(    -2.70), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -1.78),
        SIMDE_FLOAT32_C(     3.31), SIMDE_FLOAT32_C(    -2.32), SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(     0.89) } },
    { { SIMDE_FLOAT32_C(    -2.87), SIMDE_FLOAT32_C(    -2.68), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -2.39),
        SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(    -2.77), SIMDE_FLOAT32_C(    -3.62), SIMDE_FLOAT32_C(     0.48),
        SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(    -1.40), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(     3.47),
        SIMDE_FLOAT32_C(    -2.97), SIMDE_FLOAT32_C(    -3.32), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     1.11) },
      UINT8_C( 84),
      { SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(    -3.00),
        SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     3.94),
        SIMDE_FLOAT32_C(     3.25), SIMDE_FLOAT32_C(    -1.37), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     1.13),
        SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(     2.26), SIMDE_FLOAT32_C(     1.26) },
      { SIMDE_FLOAT32_C(    -2.87), SIMDE_FLOAT32_C(    -2.68), SIMDE_FLOAT32_C(     8.51), SIMDE_FLOAT32_C(    -2.39),
        SIMDE_FLOAT32_C(     3.94), SIMDE_FLOAT32_C(    -2.77), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     0.48),
        SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(    -1.40), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(     3.47),
        SIMDE_FLOAT32_C(    -2.97), SIMDE_FLOAT32_C(    -3.32), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     1.11) } },
    { { SIMDE_FLOAT32_C(    -2.93), SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(    -3.56), SIMDE_FLOAT32_C(    -1.70),
        SIMDE_FLOAT32_C(    -3.75), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -3.91), SIMDE_FLOAT32_C(    -1.16),
        SIMDE_FLOAT32_C(    -3.29), SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -2.61),
        SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    -3.02), SIMDE_FLOAT32_C(    -3.07), SIMDE_FLOAT32_C(    -2.45) },
      UINT8_C(114),
      { SIMDE_FLOAT32_C(    -3.99), SIMDE_FLOAT32_C(    -1.45), SIMDE_FLOAT32_C(    -1.26), SIMDE_FLOAT32_C(     1.51),
        SIMDE_FLOAT32_C(     2.98), SIMDE_FLOAT32_C(    -1.31), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(    -2.40),
        SIMDE_FLOAT32_C(    -1.59), SIMDE_FLOAT32_C(    -2.11), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(    -3.56),
        SIMDE_FLOAT32_C(    -3.85), SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(    -2.49), SIMDE_FLOAT32_C(    -3.98) },
      { SIMDE_FLOAT32_C(    -2.93), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(    -3.56), SIMDE_FLOAT32_C(    -1.70),
        SIMDE_FLOAT32_C(     7.89), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(    -1.16),
        SIMDE_FLOAT32_C(    -3.29), SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -2.61),
        SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    -3.02), SIMDE_FLOAT32_C(    -3.07), SIMDE_FLOAT32_C(    -2.45) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_exp2_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_exp2_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     3.48), SIMDE_FLOAT64_C(    -0.87), SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(    -1.18),
        SIMDE_FLOAT64_C(    -0.93), SIMDE_FLOAT64_C(    -0.41), SIMDE_FLOAT64_C(    -1.58), SIMDE_FLOAT64_C(    -1.72) },
      { SIMDE_FLOAT64_C(    11.16), SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     1.45), SIMDE_FLOAT64_C(     0.44),
        SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     0.75), SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(     0.30) } },
    { { SIMDE_FLOAT64_C(    -0.88), SIMDE_FLOAT64_C(     1.04), SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(    -3.45),
        SIMDE_FLOAT64_C(     3.01), SIMDE_FLOAT64_C(    -3.59), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     3.12) },
      { SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(     2.06), SIMDE_FLOAT64_C(     1.79), SIMDE_FLOAT64_C(     0.09),
        SIMDE_FLOAT64_C(     8.06), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     1.27), SIMDE_FLOAT64_C(     8.69) } },
    { { SIMDE_FLOAT64_C(    -1.74), SIMDE_FLOAT64_C(    -2.12), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     0.52),
        SIMDE_FLOAT64_C(    -1.12), SIMDE_FLOAT64_C(    -1.89), SIMDE_FLOAT64_C(     2.97), SIMDE_FLOAT64_C(     2.38) },
      { SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     1.06), SIMDE_FLOAT64_C(     1.43),
        SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(     7.84), SIMDE_FLOAT64_C(     5.21) } },
    { { SIMDE_FLOAT64_C(     2.06), SIMDE_FLOAT64_C(     2.07), SIMDE_FLOAT64_C(    -3.17), SIMDE_FLOAT64_C(     1.53),
        SIMDE_FLOAT64_C(    -1.34), SIMDE_FLOAT64_C(     1.50), SIMDE_FLOAT64_C(    -0.18), SIMDE_FLOAT64_C(    -1.86) },
      { SIMDE_FLOAT64_C(     4.17), SIMDE_FLOAT64_C(     4.20), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     2.89),
        SIMDE_FLOAT64_C(     0.40), SIMDE_FLOAT64_C(     2.83), SIMDE_FLOAT64_C(     0.88), SIMDE_FLOAT64_C(     0.28) } },
    { { SIMDE_FLOAT64_C(    -3.38), SIMDE_FLOAT64_C(    -3.65), SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(    -0.31),
        SIMDE_FLOAT64_C(    -0.06), SIMDE_FLOAT64_C(     3.37), SIMDE_FLOAT64_C(     1.97), SIMDE_FLOAT64_C(     3.07) },
      { SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     1.95), SIMDE_FLOAT64_C(     0.81),
        SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(    10.34), SIMDE_FLOAT64_C(     3.92), SIMDE_FLOAT64_C(     8.40) } },
    { { SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(    -1.19), SIMDE_FLOAT64_C(     3.61), SIMDE_FLOAT64_C(    -0.57),
        SIMDE_FLOAT64_C(    -0.78), SIMDE_FLOAT64_C(    -0.04), SIMDE_FLOAT64_C(    -1.46), SIMDE_FLOAT64_C(     1.48) },
      { SIMDE_FLOAT64_C(     1.33), SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(    12.21), SIMDE_FLOAT64_C(     0.67),
        SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.97), SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     2.79) } },
    { { SIMDE_FLOAT64_C(     1.84), SIMDE_FLOAT64_C(     2.63), SIMDE_FLOAT64_C(    -1.99), SIMDE_FLOAT64_C(    -3.28),
        SIMDE_FLOAT64_C(    -3.26), SIMDE_FLOAT64_C(    -3.02), SIMDE_FLOAT64_C(     3.10), SIMDE_FLOAT64_C(     2.79) },
      { SIMDE_FLOAT64_C(     3.58), SIMDE_FLOAT64_C(     6.19), SIMDE_FLOAT64_C(     0.25), SIMDE_FLOAT64_C(     0.10),
        SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(     8.57), SIMDE_FLOAT64_C(     6.92) } },
    { { SIMDE_FLOAT64_C(     3.05), SIMDE_FLOAT64_C(     3.93), SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(    -2.28),
        SIMDE_FLOAT64_C(     1.42), SIMDE_FLOAT64_C(    -3.86), SIMDE_FLOAT64_C(    -0.14), SIMDE_FLOAT64_C(     2.05) },
      { SIMDE_FLOAT64_C(     8.28), SIMDE_FLOAT64_C(    15.24), SIMDE_FLOAT64_C(     1.26), SIMDE_FLOAT64_C(     0.21),
        SIMDE_FLOAT64_C(     2.68), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(     4.14) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_exp2_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_exp2_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     1.08), SIMDE_FLOAT64_C(    -1.78), SIMDE_FLOAT64_C(    -3.94), SIMDE_FLOAT64_C(     2.91),
        SIMDE_FLOAT64_C(    -3.39), SIMDE_FLOAT64_C(    -0.34), SIMDE_FLOAT64_C(    -1.05), SIMDE_FLOAT64_C(    -1.87) },
      UINT8_C( 59),
      { SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     3.70), SIMDE_FLOAT64_C(    -0.34), SIMDE_FLOAT64_C(    -2.49),
        SIMDE_FLOAT64_C(    -3.69), SIMDE_FLOAT64_C(     1.16), SIMDE_FLOAT64_C(    -0.71), SIMDE_FLOAT64_C(     3.16) },
      { SIMDE_FLOAT64_C(     1.39), SIMDE_FLOAT64_C(    13.00), SIMDE_FLOAT64_C(    -3.94), SIMDE_FLOAT64_C(     0.18),
        SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     2.23), SIMDE_FLOAT64_C(    -1.05), SIMDE_FLOAT64_C(    -1.87) } },
    { { SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     0.79), SIMDE_FLOAT64_C(    -1.70), SIMDE_FLOAT64_C(    -1.78),
        SIMDE_FLOAT64_C(    -0.42), SIMDE_FLOAT64_C(    -3.00), SIMDE_FLOAT64_C(    -0.85), SIMDE_FLOAT64_C(     3.64) },
      UINT8_C(181),
      { SIMDE_FLOAT64_C(    -3.64), SIMDE_FLOAT64_C(     1.07), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(    -0.99),
        SIMDE_FLOAT64_C(     2.92), SIMDE_FLOAT64_C(    -2.83), SIMDE_FLOAT64_C(     1.23), SIMDE_FLOAT64_C(     2.98) },
      { SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     0.79), SIMDE_FLOAT64_C(     1.06), SIMDE_FLOAT64_C(    -1.78),
        SIMDE_FLOAT64_C(     7.57), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(    -0.85), SIMDE_FLOAT64_C(     7.89) } },
    { { SIMDE_FLOAT64_C(    -3.92), SIMDE_FLOAT64_C(     1.84), SIMDE_FLOAT64_C(    -1.36), SIMDE_FLOAT64_C(    -0.97),
        SIMDE_FLOAT64_C(     3.97), SIMDE_FLOAT64_C(    -2.62), SIMDE_FLOAT64_C(     3.51), SIMDE_FLOAT64_C(     3.67) },
      UINT8_C( 39),
      { SIMDE_FLOAT64_C(    -2.98), SIMDE_FLOAT64_C(     3.98), SIMDE_FLOAT64_C(    -1.79), SIMDE_FLOAT64_C(     0.31),
        SIMDE_FLOAT64_C(     3.14), SIMDE_FLOAT64_C(     2.73), SIMDE_FLOAT64_C(    -2.90), SIMDE_FLOAT64_C(    -2.56) },
      { SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(    15.78), SIMDE_FLOAT64_C(     0.29), SIMDE_FLOAT64_C(    -0.97),
        SIMDE_FLOAT64_C(     3.97), SIMDE_FLOAT64_C(     6.63), SIMDE_FLOAT64_C(     3.51), SIMDE_FLOAT64_C(     3.67) } },
    { { SIMDE_FLOAT64_C(    -3.05), SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(    -1.56), SIMDE_FLOAT64_C(     0.11),
        SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(    -3.35), SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(    -2.62) },
      UINT8_C(222),
      { SIMDE_FLOAT64_C(     3.48), SIMDE_FLOAT64_C(    -3.70), SIMDE_FLOAT64_C(     1.91), SIMDE_FLOAT64_C(     0.71),
        SIMDE_FLOAT64_C(     3.28), SIMDE_FLOAT64_C(     1.99), SIMDE_FLOAT64_C(    -1.45), SIMDE_FLOAT64_C(    -2.07) },
      { SIMDE_FLOAT64_C(    -3.05), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     3.76), SIMDE_FLOAT64_C(     1.64),
        SIMDE_FLOAT64_C(     9.71), SIMDE_FLOAT64_C(    -3.35), SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     0.24) } },
    { { SIMDE_FLOAT64_C(    -2.98), SIMDE_FLOAT64_C(    -1.47), SIMDE_FLOAT64_C(    -0.69), SIMDE_FLOAT64_C(    -3.47),
        SIMDE_FLOAT64_C(    -1.80), SIMDE_FLOAT64_C(    -3.64), SIMDE_FLOAT64_C(    -2.45), SIMDE_FLOAT64_C(    -1.83) },
      UINT8_C(173),
      { SIMDE_FLOAT64_C(     1.86), SIMDE_FLOAT64_C(    -2.68), SIMDE_FLOAT64_C(    -2.71), SIMDE_FLOAT64_C(     2.96),
        SIMDE_FLOAT64_C(    -1.24), SIMDE_FLOAT64_C(    -1.76), SIMDE_FLOAT64_C(    -0.37), SIMDE_FLOAT64_C(     1.20) },
      { SIMDE_FLOAT64_C(     3.63), SIMDE_FLOAT64_C(    -1.47), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     7.78),
        SIMDE_FLOAT64_C(    -1.80), SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(    -2.45), SIMDE_FLOAT64_C(     2.30) } },
    { { SIMDE_FLOAT64_C(     2.35), SIMDE_FLOAT64_C(     3.95), SIMDE_FLOAT64_C(     1.85), SIMDE_FLOAT64_C(    -1.18),
        SIMDE_FLOAT64_C(    -2.67), SIMDE_FLOAT64_C(    -1.41), SIMDE_FLOAT64_C(    -1.70), SIMDE_FLOAT64_C(    -2.37) },
      UINT8_C(128),
      { SIMDE_FLOAT64_C(     3.01), SIMDE_FLOAT64_C(    -3.08), SIMDE_FLOAT64_C(     2.48), SIMDE_FLOAT64_C(    -2.44),
        SIMDE_FLOAT64_C(    -1.16), SIMDE_FLOAT64_C(     3.50), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     2.16) },
      { SIMDE_FLOAT64_C(     2.35), SIMDE_FLOAT64_C(     3.95), SIMDE_FLOAT64_C(     1.85), SIMDE_FLOAT64_C(    -1.18),
        SIMDE_FLOAT64_C(    -2.67), SIMDE_FLOAT64_C(    -1.41), SIMDE_FLOAT64_C(    -1.70), SIMDE_FLOAT64_C(     4.47) } },
    { { SIMDE_FLOAT64_C(    -3.97), SIMDE_FLOAT64_C(     2.28), SIMDE_FLOAT64_C(     2.51), SIMDE_FLOAT64_C(    -2.42),
        SIMDE_FLOAT64_C(    -3.54), SIMDE_FLOAT64_C(    -2.92), SIMDE_FLOAT64_C(     3.44), SIMDE_FLOAT64_C(    -2.23) },
      UINT8_C( 29),
      { SIMDE_FLOAT64_C(     2.39), SIMDE_FLOAT64_C(     0.53), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(    -1.97),
        SIMDE_FLOAT64_C(    -2.27), SIMDE_FLOAT64_C(    -1.04), SIMDE_FLOAT64_C(    -2.02), SIMDE_FLOAT64_C(     3.58) },
      { SIMDE_FLOAT64_C(     5.24), SIMDE_FLOAT64_C(     2.28), SIMDE_FLOAT64_C(     1.53), SIMDE_FLOAT64_C(     0.26),
        SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(    -2.92), SIMDE_FLOAT64_C(     3.44), SIMDE_FLOAT64_C(    -2.23) } },
    { { SIMDE_FLOAT64_C(     1.78), SIMDE_FLOAT64_C(    -0.69), SIMDE_FLOAT64_C(    -1.84), SIMDE_FLOAT64_C(    -3.92),
        SIMDE_FLOAT64_C(     0.94), SIMDE_FLOAT64_C(    -1.34), SIMDE_FLOAT64_C(     3.09), SIMDE_FLOAT64_C(     1.86) },
      UINT8_C(207),
      { SIMDE_FLOAT64_C(    -3.35), SIMDE_FLOAT64_C(    -3.29), SIMDE_FLOAT64_C(    -3.36), SIMDE_FLOAT64_C(     0.74),
        SIMDE_FLOAT64_C(     2.86), SIMDE_FLOAT64_C(    -3.33), SIMDE_FLOAT64_C(    -0.98), SIMDE_FLOAT64_C(     1.37) },
      { SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     1.67),
        SIMDE_FLOAT64_C(     0.94), SIMDE_FLOAT64_C(    -1.34), SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(     2.58) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_exp2_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_exp10_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    -1.28), SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(     3.28), SIMDE_FLOAT32_C(    -3.13) },
      { SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(  1905.46), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(     1.43), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     1.40), SIMDE_FLOAT32_C(    -2.59) },
      { SIMDE_FLOAT32_C(    26.92), SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(    25.12), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     1.67) },
      { SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(  1071.52), SIMDE_FLOAT32_C(    46.77) } },
    { { SIMDE_FLOAT32_C(    -3.68), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     1.43) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.69), SIMDE_FLOAT32_C(    33.88), SIMDE_FLOAT32_C(    26.92) } },
    { { SIMDE_FLOAT32_C(    -1.86), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(    -1.56) },
      { SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(  5248.07), SIMDE_FLOAT32_C(     0.03) } },
    { { SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(    -1.43), SIMDE_FLOAT32_C(     0.99) },
      { SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(   416.87), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     9.77) } },
    { { SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(    -3.37) },
      { SIMDE_FLOAT32_C(    97.72), SIMDE_FLOAT32_C(  1230.27), SIMDE_FLOAT32_C(   239.88), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(    -2.15), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.56) },
      { SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     1.66), SIMDE_FLOAT32_C(     2.57), SIMDE_FLOAT32_C(     3.63) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_exp10_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_exp10_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     2.71), SIMDE_FLOAT64_C(    -2.06) },
      { SIMDE_FLOAT64_C(   512.86), SIMDE_FLOAT64_C(     0.01) } },
    { { SIMDE_FLOAT64_C(     0.82), SIMDE_FLOAT64_C(     2.37) },
      { SIMDE_FLOAT64_C(     6.61), SIMDE_FLOAT64_C(   234.42) } },
    { { SIMDE_FLOAT64_C(    -1.27), SIMDE_FLOAT64_C(    -2.72) },
      { SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(    -0.02), SIMDE_FLOAT64_C(     1.72) },
      { SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(    52.48) } },
    { { SIMDE_FLOAT64_C(    -2.59), SIMDE_FLOAT64_C(    -1.62) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.02) } },
    { { SIMDE_FLOAT64_C(     1.83), SIMDE_FLOAT64_C(     3.25) },
      { SIMDE_FLOAT64_C(    67.61), SIMDE_FLOAT64_C(  1778.28) } },
    { { SIMDE_FLOAT64_C(    -2.12), SIMDE_FLOAT64_C(     3.99) },
      { SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(  9772.37) } },
    { { SIMDE_FLOAT64_C(    -3.59), SIMDE_FLOAT64_C(     0.94) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     8.71) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_exp10_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_exp10_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    -2.69), SIMDE_FLOAT32_C(     3.91), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -0.80),
        SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(    -1.38), SIMDE_FLOAT32_C(    -3.31), SIMDE_FLOAT32_C(    -0.75) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  8128.31), SIMDE_FLOAT32_C(     4.07), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(   323.59), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.18) } },
    { { SIMDE_FLOAT32_C(     3.51), SIMDE_FLOAT32_C(    -3.93), SIMDE_FLOAT32_C(    -3.82), SIMDE_FLOAT32_C(     1.46),
        SIMDE_FLOAT32_C(    -3.04), SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(    -3.04), SIMDE_FLOAT32_C(    -3.66) },
      { SIMDE_FLOAT32_C(  3235.94), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    28.84),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  1949.84), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(    -2.34), SIMDE_FLOAT32_C(    -3.98), SIMDE_FLOAT32_C(    -1.70), SIMDE_FLOAT32_C(    -1.23),
        SIMDE_FLOAT32_C(    -3.97), SIMDE_FLOAT32_C(    -3.62), SIMDE_FLOAT32_C(     3.06), SIMDE_FLOAT32_C(    -1.19) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  1148.15), SIMDE_FLOAT32_C(     0.06) } },
    { { SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(    -1.22), SIMDE_FLOAT32_C(     2.16), SIMDE_FLOAT32_C(     3.83),
        SIMDE_FLOAT32_C(    -3.41), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -2.66), SIMDE_FLOAT32_C(    -2.09) },
      { SIMDE_FLOAT32_C(  5011.87), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(   144.54), SIMDE_FLOAT32_C(  6760.83),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.01) } },
    { { SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(    -2.06),
        SIMDE_FLOAT32_C(    -3.42), SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(    -3.92) },
      { SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    89.13), SIMDE_FLOAT32_C(    12.88), SIMDE_FLOAT32_C(     0.01),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    61.66), SIMDE_FLOAT32_C(    15.49), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(     2.82),
        SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(     3.16), SIMDE_FLOAT32_C(     2.33) },
      { SIMDE_FLOAT32_C(    72.44), SIMDE_FLOAT32_C(    23.44), SIMDE_FLOAT32_C(    34.67), SIMDE_FLOAT32_C(   660.69),
        SIMDE_FLOAT32_C(     4.57), SIMDE_FLOAT32_C(   316.23), SIMDE_FLOAT32_C(  1445.44), SIMDE_FLOAT32_C(   213.80) } },
    { { SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(    -2.54), SIMDE_FLOAT32_C(    -2.90), SIMDE_FLOAT32_C(     2.55),
        SIMDE_FLOAT32_C(    -2.16), SIMDE_FLOAT32_C(    -3.84), SIMDE_FLOAT32_C(    -2.64), SIMDE_FLOAT32_C(    -2.46) },
      { SIMDE_FLOAT32_C(   331.13), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   354.81),
        SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(    -1.06), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(    -2.64), SIMDE_FLOAT32_C(    -0.47),
        SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -1.29), SIMDE_FLOAT32_C(     1.44), SIMDE_FLOAT32_C(     2.48) },
      { SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(  3311.31), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    27.54), SIMDE_FLOAT32_C(   302.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_exp10_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_exp10_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    -3.01), SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(    -0.62) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     2.34), SIMDE_FLOAT64_C(     1.26), SIMDE_FLOAT64_C(     0.24) } },
    { { SIMDE_FLOAT64_C(     1.29), SIMDE_FLOAT64_C(     2.86), SIMDE_FLOAT64_C(     0.75), SIMDE_FLOAT64_C(    -3.99) },
      { SIMDE_FLOAT64_C(    19.50), SIMDE_FLOAT64_C(   724.44), SIMDE_FLOAT64_C(     5.62), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(    -2.93), SIMDE_FLOAT64_C(     3.81), SIMDE_FLOAT64_C(     3.34), SIMDE_FLOAT64_C(     3.21) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  6456.54), SIMDE_FLOAT64_C(  2187.76), SIMDE_FLOAT64_C(  1621.81) } },
    { { SIMDE_FLOAT64_C(    -2.76), SIMDE_FLOAT64_C(    -1.49), SIMDE_FLOAT64_C(     3.76), SIMDE_FLOAT64_C(    -1.66) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(  5754.40), SIMDE_FLOAT64_C(     0.02) } },
    { { SIMDE_FLOAT64_C(    -0.02), SIMDE_FLOAT64_C(    -2.70), SIMDE_FLOAT64_C(     2.90), SIMDE_FLOAT64_C(    -0.73) },
      { SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   794.33), SIMDE_FLOAT64_C(     0.19) } },
    { { SIMDE_FLOAT64_C(    -1.67), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(    -2.21), SIMDE_FLOAT64_C(    -3.15) },
      { SIMDE_FLOAT64_C(     0.02), SIMDE_FLOAT64_C(     1.58), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(     2.30), SIMDE_FLOAT64_C(     3.98), SIMDE_FLOAT64_C(    -0.86), SIMDE_FLOAT64_C(    -1.96) },
      { SIMDE_FLOAT64_C(   199.53), SIMDE_FLOAT64_C(  9549.93), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.01) } },
    { { SIMDE_FLOAT64_C(     0.50), SIMDE_FLOAT64_C(     0.60), SIMDE_FLOAT64_C(     0.65), SIMDE_FLOAT64_C(     1.49) },
      { SIMDE_FLOAT64_C(     3.16), SIMDE_FLOAT64_C(     3.98), SIMDE_FLOAT64_C(     4.47), SIMDE_FLOAT64_C(    30.90) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_exp10_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_exp10_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     2.15), SIMDE_FLOAT32_C(     3.90), SIMDE_FLOAT32_C(    -3.06), SIMDE_FLOAT32_C(    -3.99),
        SIMDE_FLOAT32_C(    -1.49), SIMDE_FLOAT32_C(     3.34), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.15),
        SIMDE_FLOAT32_C(     3.24), SIMDE_FLOAT32_C(     2.10), SIMDE_FLOAT32_C(    -1.61), SIMDE_FLOAT32_C(    -3.33),
        SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(    -2.51), SIMDE_FLOAT32_C(     3.50), SIMDE_FLOAT32_C(    -1.30) },
      { SIMDE_FLOAT32_C(   141.25), SIMDE_FLOAT32_C(  7943.28), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(  2187.76), SIMDE_FLOAT32_C(     1.20), SIMDE_FLOAT32_C(     1.41),
        SIMDE_FLOAT32_C(  1737.80), SIMDE_FLOAT32_C(   125.89), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    48.98), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  3162.28), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -1.13), SIMDE_FLOAT32_C(    -1.51), SIMDE_FLOAT32_C(     1.48),
        SIMDE_FLOAT32_C(    -3.11), SIMDE_FLOAT32_C(    -2.56), SIMDE_FLOAT32_C(    -2.35), SIMDE_FLOAT32_C(    -0.62),
        SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(    -1.51), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -3.88),
        SIMDE_FLOAT32_C(    -2.12), SIMDE_FLOAT32_C(     3.49), SIMDE_FLOAT32_C(    -2.42), SIMDE_FLOAT32_C(    -3.98) },
      { SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    30.20),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.24),
        SIMDE_FLOAT32_C(   107.15), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(  3090.30), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(    -1.48), SIMDE_FLOAT32_C(    -3.96), SIMDE_FLOAT32_C(    -2.11),
        SIMDE_FLOAT32_C(    -2.14), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(    -2.89),
        SIMDE_FLOAT32_C(    -1.78), SIMDE_FLOAT32_C(    -3.57), SIMDE_FLOAT32_C(    -2.23), SIMDE_FLOAT32_C(     3.90),
        SIMDE_FLOAT32_C(    -2.08), SIMDE_FLOAT32_C(    -2.73), SIMDE_FLOAT32_C(    -1.40), SIMDE_FLOAT32_C(     2.46) },
      { SIMDE_FLOAT32_C(  2398.83), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.01),
        SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(   109.65), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(  7943.28),
        SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(   288.40) } },
    { { SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(     1.03),
        SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(    -3.59), SIMDE_FLOAT32_C(     0.56),
        SIMDE_FLOAT32_C(    -3.91), SIMDE_FLOAT32_C(     1.26), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -2.04),
        SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     2.26), SIMDE_FLOAT32_C(    -2.02), SIMDE_FLOAT32_C(     0.13) },
      { SIMDE_FLOAT32_C(     1.41), SIMDE_FLOAT32_C(    12.30), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    10.72),
        SIMDE_FLOAT32_C(   338.84), SIMDE_FLOAT32_C(    38.90), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     3.63),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    18.20), SIMDE_FLOAT32_C(     4.79), SIMDE_FLOAT32_C(     0.01),
        SIMDE_FLOAT32_C(     5.50), SIMDE_FLOAT32_C(   181.97), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     1.35) } },
    { { SIMDE_FLOAT32_C(    -3.22), SIMDE_FLOAT32_C(    -1.98), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(    -1.36),
        SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -3.65),
        SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     1.52), SIMDE_FLOAT32_C(    -3.75), SIMDE_FLOAT32_C(     2.41),
        SIMDE_FLOAT32_C(     2.80), SIMDE_FLOAT32_C(    -1.15), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    -1.06) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(   104.71), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(   138.04), SIMDE_FLOAT32_C(     1.15), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(    33.11), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   257.04),
        SIMDE_FLOAT32_C(   630.96), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(     0.09) } },
    { { SIMDE_FLOAT32_C(     3.94), SIMDE_FLOAT32_C(    -3.19), SIMDE_FLOAT32_C(     3.97), SIMDE_FLOAT32_C(     2.47),
        SIMDE_FLOAT32_C(     2.41), SIMDE_FLOAT32_C(    -3.62), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     2.49),
        SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(    -3.55), SIMDE_FLOAT32_C(    -1.62),
        SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(     2.74) },
      { SIMDE_FLOAT32_C(  8709.64), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  9332.54), SIMDE_FLOAT32_C(   295.12),
        SIMDE_FLOAT32_C(   257.04), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(   309.03),
        SIMDE_FLOAT32_C(    43.65), SIMDE_FLOAT32_C(  5011.87), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.02),
        SIMDE_FLOAT32_C(    91.20), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(   323.59), SIMDE_FLOAT32_C(   549.54) } },
    { { SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -2.61), SIMDE_FLOAT32_C(    -1.40),
        SIMDE_FLOAT32_C(    -3.41), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(    -1.05), SIMDE_FLOAT32_C(     1.08),
        SIMDE_FLOAT32_C(    -1.34), SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -2.54),
        SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(    -3.64), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     2.00) },
      { SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    13.80), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(    12.02),
        SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(   112.20), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(   100.00) } },
    { { SIMDE_FLOAT32_C(    -2.83), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     3.58),
        SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     3.48), SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(    -1.60),
        SIMDE_FLOAT32_C(     3.19), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     1.15),
        SIMDE_FLOAT32_C(    -3.03), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     1.43) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.34), SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(  3801.89),
        SIMDE_FLOAT32_C(     5.75), SIMDE_FLOAT32_C(  3019.95), SIMDE_FLOAT32_C(   117.49), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(  1548.82), SIMDE_FLOAT32_C(   338.84), SIMDE_FLOAT32_C(     6.03), SIMDE_FLOAT32_C(    14.13),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(    26.92) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_exp10_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_exp10_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(    -3.09), SIMDE_FLOAT32_C(     2.30),
        SIMDE_FLOAT32_C(    -3.02), SIMDE_FLOAT32_C(    -1.71), SIMDE_FLOAT32_C(    -2.65), SIMDE_FLOAT32_C(     2.34),
        SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(    -1.53), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     2.13),
        SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(    -3.96), SIMDE_FLOAT32_C(    -3.24), SIMDE_FLOAT32_C(    -2.96) },
      UINT8_C( 58),
      { SIMDE_FLOAT32_C(     2.80), SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -1.26),
        SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -2.77),
        SIMDE_FLOAT32_C(     3.35), SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     3.33),
        SIMDE_FLOAT32_C(     2.21), SIMDE_FLOAT32_C(    -1.15), SIMDE_FLOAT32_C(    -1.25), SIMDE_FLOAT32_C(     0.74) },
      { SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(    38.90), SIMDE_FLOAT32_C(    -3.09), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     6.46), SIMDE_FLOAT32_C(    56.23), SIMDE_FLOAT32_C(    -2.65), SIMDE_FLOAT32_C(     2.34),
        SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(    -1.53), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     2.13),
        SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(    -3.96), SIMDE_FLOAT32_C(    -3.24), SIMDE_FLOAT32_C(    -2.96) } },
    { { SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(    -2.95), SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     2.05),
        SIMDE_FLOAT32_C(    -1.60), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     1.61), SIMDE_FLOAT32_C(     0.87),
        SIMDE_FLOAT32_C(    -2.25), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(    -1.15), SIMDE_FLOAT32_C(    -2.21),
        SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -3.54), SIMDE_FLOAT32_C(    -0.71) },
      UINT8_C(193),
      { SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(     2.29), SIMDE_FLOAT32_C(    -1.46),
        SIMDE_FLOAT32_C(    -1.68), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(    -2.11), SIMDE_FLOAT32_C(    -3.63),
        SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(    -2.78), SIMDE_FLOAT32_C(     2.58), SIMDE_FLOAT32_C(    -3.29),
        SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     1.02) },
      { SIMDE_FLOAT32_C(     6.17), SIMDE_FLOAT32_C(    -2.95), SIMDE_FLOAT32_C(     1.72), SIMDE_FLOAT32_C(     2.05),
        SIMDE_FLOAT32_C(    -1.60), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    -2.25), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(    -1.15), SIMDE_FLOAT32_C(    -2.21),
        SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -3.54), SIMDE_FLOAT32_C(    -0.71) } },
    { { SIMDE_FLOAT32_C(    -2.96), SIMDE_FLOAT32_C(    -1.49), SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     1.10),
        SIMDE_FLOAT32_C(    -3.88), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     2.86), SIMDE_FLOAT32_C(    -0.15),
        SIMDE_FLOAT32_C(     3.14), SIMDE_FLOAT32_C(    -3.35), SIMDE_FLOAT32_C(    -3.66), SIMDE_FLOAT32_C(    -0.97),
        SIMDE_FLOAT32_C(    -2.89), SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     1.89) },
      UINT8_C(215),
      { SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(    -3.57), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -1.67),
        SIMDE_FLOAT32_C(    -1.68), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -3.82), SIMDE_FLOAT32_C(    -0.45),
        SIMDE_FLOAT32_C(    -1.07), SIMDE_FLOAT32_C(    -3.11), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(     2.25),
        SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     3.86) },
      { SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     1.10),
        SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.35),
        SIMDE_FLOAT32_C(     3.14), SIMDE_FLOAT32_C(    -3.35), SIMDE_FLOAT32_C(    -3.66), SIMDE_FLOAT32_C(    -0.97),
        SIMDE_FLOAT32_C(    -2.89), SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     1.89) } },
    { { SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(    -3.74),
        SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     3.40), SIMDE_FLOAT32_C(    -0.10),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(     3.80),
        SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(    -1.09), SIMDE_FLOAT32_C(    -2.53), SIMDE_FLOAT32_C(    -2.24) },
      UINT8_C(253),
      { SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     1.66), SIMDE_FLOAT32_C(    -2.19),
        SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -2.79), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     1.16),
        SIMDE_FLOAT32_C(    -3.28), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.49), SIMDE_FLOAT32_C(     1.26),
        SIMDE_FLOAT32_C(    -1.71), SIMDE_FLOAT32_C(    -1.63), SIMDE_FLOAT32_C(    -2.77), SIMDE_FLOAT32_C(     2.69) },
      { SIMDE_FLOAT32_C(    28.18), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    45.71), SIMDE_FLOAT32_C(     0.01),
        SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     5.62), SIMDE_FLOAT32_C(    14.45),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -1.56), SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(     3.80),
        SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(    -1.09), SIMDE_FLOAT32_C(    -2.53), SIMDE_FLOAT32_C(    -2.24) } },
    { { SIMDE_FLOAT32_C(    -1.65), SIMDE_FLOAT32_C(    -2.52), SIMDE_FLOAT32_C(    -2.06), SIMDE_FLOAT32_C(     2.18),
        SIMDE_FLOAT32_C(    -3.11), SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(    -1.65), SIMDE_FLOAT32_C(    -0.68),
        SIMDE_FLOAT32_C(    -1.14), SIMDE_FLOAT32_C(    -1.85), SIMDE_FLOAT32_C(    -1.73), SIMDE_FLOAT32_C(     1.76),
        SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -2.90), SIMDE_FLOAT32_C(    -2.93) },
      UINT8_C(202),
      { SIMDE_FLOAT32_C(     2.76), SIMDE_FLOAT32_C(    -1.12), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     3.97),
        SIMDE_FLOAT32_C(     3.63), SIMDE_FLOAT32_C(    -2.46), SIMDE_FLOAT32_C(    -3.31), SIMDE_FLOAT32_C(    -1.37),
        SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     2.42),
        SIMDE_FLOAT32_C(     3.18), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -3.23), SIMDE_FLOAT32_C(    -3.34) },
      { SIMDE_FLOAT32_C(    -1.65), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(    -2.06), SIMDE_FLOAT32_C(  9332.54),
        SIMDE_FLOAT32_C(    -3.11), SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(    -1.14), SIMDE_FLOAT32_C(    -1.85), SIMDE_FLOAT32_C(    -1.73), SIMDE_FLOAT32_C(     1.76),
        SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -2.90), SIMDE_FLOAT32_C(    -2.93) } },
    { { SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(    -2.45), SIMDE_FLOAT32_C(    -0.61),
        SIMDE_FLOAT32_C(    -2.70), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(    -0.54),
        SIMDE_FLOAT32_C(     3.14), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     3.08), SIMDE_FLOAT32_C(    -0.83),
        SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(    -3.85), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(    -0.12) },
      UINT8_C(  4),
      { SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(    -1.34), SIMDE_FLOAT32_C(    -2.78),
        SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(    -1.51),
        SIMDE_FLOAT32_C(    -1.79), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -2.34), SIMDE_FLOAT32_C(     1.81),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -1.67), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.57) },
      { SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -0.61),
        SIMDE_FLOAT32_C(    -2.70), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(    -0.54),
        SIMDE_FLOAT32_C(     3.14), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     3.08), SIMDE_FLOAT32_C(    -0.83),
        SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(    -3.85), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(    -0.12) } },
    { { SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     2.76), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -3.26),
        SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(    -3.81), SIMDE_FLOAT32_C(     3.89), SIMDE_FLOAT32_C(    -2.98),
        SIMDE_FLOAT32_C(     3.27), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     3.42),
        SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(    -1.99), SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(     2.03) },
      UINT8_C( 74),
      { SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(     3.25), SIMDE_FLOAT32_C(    -1.61), SIMDE_FLOAT32_C(    -1.60),
        SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(    -3.77),
        SIMDE_FLOAT32_C(     2.54), SIMDE_FLOAT32_C(    -1.58), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    -3.14),
        SIMDE_FLOAT32_C(     1.78), SIMDE_FLOAT32_C(    -3.87), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.53) },
      { SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(  1778.28), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(     1.01), SIMDE_FLOAT32_C(    -3.81), SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(    -2.98),
        SIMDE_FLOAT32_C(     3.27), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     3.42),
        SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(    -1.99), SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(     2.03) } },
    { { SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(    -2.46), SIMDE_FLOAT32_C(     1.04),
        SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(    -1.44), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -3.58),
        SIMDE_FLOAT32_C(    -3.30), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(    -1.29),
        SIMDE_FLOAT32_C(     2.18), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -3.44), SIMDE_FLOAT32_C(    -0.71) },
      UINT8_C(120),
      { SIMDE_FLOAT32_C(    -1.05), SIMDE_FLOAT32_C(     1.68), SIMDE_FLOAT32_C(    -3.41), SIMDE_FLOAT32_C(     3.82),
        SIMDE_FLOAT32_C(    -1.71), SIMDE_FLOAT32_C(    -3.18), SIMDE_FLOAT32_C(     2.36), SIMDE_FLOAT32_C(     0.70),
        SIMDE_FLOAT32_C(     1.52), SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(    -1.52), SIMDE_FLOAT32_C(     1.64),
        SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(     3.01), SIMDE_FLOAT32_C(    -1.50), SIMDE_FLOAT32_C(    -2.56) },
      { SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(    -2.46), SIMDE_FLOAT32_C(  6606.93),
        SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   229.09), SIMDE_FLOAT32_C(    -3.58),
        SIMDE_FLOAT32_C(    -3.30), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(    -1.29),
        SIMDE_FLOAT32_C(     2.18), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -3.44), SIMDE_FLOAT32_C(    -0.71) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_exp10_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_exp10_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(    -1.02), SIMDE_FLOAT64_C(    -3.98),
        SIMDE_FLOAT64_C(     3.95), SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(    -2.47), SIMDE_FLOAT64_C(    -3.25) },
      { SIMDE_FLOAT64_C(     8.51), SIMDE_FLOAT64_C(     7.94), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(  8912.51), SIMDE_FLOAT64_C(     2.04), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(    -3.33), SIMDE_FLOAT64_C(     1.18), SIMDE_FLOAT64_C(    -1.87), SIMDE_FLOAT64_C(     0.97),
        SIMDE_FLOAT64_C(     2.34), SIMDE_FLOAT64_C(    -3.33), SIMDE_FLOAT64_C(    -0.73), SIMDE_FLOAT64_C(     2.80) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    15.14), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     9.33),
        SIMDE_FLOAT64_C(   218.78), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(   630.96) } },
    { { SIMDE_FLOAT64_C(     2.86), SIMDE_FLOAT64_C(     2.64), SIMDE_FLOAT64_C(    -2.88), SIMDE_FLOAT64_C(     3.99),
        SIMDE_FLOAT64_C(     2.91), SIMDE_FLOAT64_C(    -0.85), SIMDE_FLOAT64_C(    -2.79), SIMDE_FLOAT64_C(     3.08) },
      { SIMDE_FLOAT64_C(   724.44), SIMDE_FLOAT64_C(   436.52), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  9772.37),
        SIMDE_FLOAT64_C(   812.83), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  1202.26) } },
    { { SIMDE_FLOAT64_C(     3.79), SIMDE_FLOAT64_C(     1.10), SIMDE_FLOAT64_C(    -2.75), SIMDE_FLOAT64_C(     2.52),
        SIMDE_FLOAT64_C(     1.05), SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(    -2.28), SIMDE_FLOAT64_C(    -2.02) },
      { SIMDE_FLOAT64_C(  6165.95), SIMDE_FLOAT64_C(    12.59), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   331.13),
        SIMDE_FLOAT64_C(    11.22), SIMDE_FLOAT64_C(     2.34), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.01) } },
    { { SIMDE_FLOAT64_C(    -2.73), SIMDE_FLOAT64_C(     0.70), SIMDE_FLOAT64_C(    -2.00), SIMDE_FLOAT64_C(    -2.78),
        SIMDE_FLOAT64_C(    -2.99), SIMDE_FLOAT64_C(    -0.48), SIMDE_FLOAT64_C(    -2.02), SIMDE_FLOAT64_C(    -2.32) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     5.01), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(    -3.30), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     2.65), SIMDE_FLOAT64_C(     3.04),
        SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(    -2.08), SIMDE_FLOAT64_C(     1.84), SIMDE_FLOAT64_C(    -0.36) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     1.29), SIMDE_FLOAT64_C(   446.68), SIMDE_FLOAT64_C(  1096.48),
        SIMDE_FLOAT64_C(     6.03), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(    69.18), SIMDE_FLOAT64_C(     0.44) } },
    { { SIMDE_FLOAT64_C(    -3.45), SIMDE_FLOAT64_C(     2.96), SIMDE_FLOAT64_C(    -0.37), SIMDE_FLOAT64_C(     3.46),
        SIMDE_FLOAT64_C(    -1.89), SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(     2.54), SIMDE_FLOAT64_C(    -2.10) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   912.01), SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(  2884.03),
        SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     6.92), SIMDE_FLOAT64_C(   346.74), SIMDE_FLOAT64_C(     0.01) } },
    { { SIMDE_FLOAT64_C(    -2.06), SIMDE_FLOAT64_C(     3.79), SIMDE_FLOAT64_C(    -3.58), SIMDE_FLOAT64_C(     2.98),
        SIMDE_FLOAT64_C(     0.16), SIMDE_FLOAT64_C(    -1.86), SIMDE_FLOAT64_C(    -3.04), SIMDE_FLOAT64_C(     1.43) },
      { SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(  6165.95), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   954.99),
        SIMDE_FLOAT64_C(     1.45), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    26.92) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_exp10_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_exp10_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     2.35), SIMDE_FLOAT64_C(     3.51), SIMDE_FLOAT64_C(    -2.45), SIMDE_FLOAT64_C(     0.95),
        SIMDE_FLOAT64_C(    -2.12), SIMDE_FLOAT64_C(    -1.70), SIMDE_FLOAT64_C(     3.27), SIMDE_FLOAT64_C(    -3.97) },
      UINT8_C( 85),
      { SIMDE_FLOAT64_C(    -1.97), SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(    -3.87),
        SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(    -1.78), SIMDE_FLOAT64_C(     2.41), SIMDE_FLOAT64_C(     3.67) },
      { SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     3.51), SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(     0.95),
        SIMDE_FLOAT64_C(     1.51), SIMDE_FLOAT64_C(    -1.70), SIMDE_FLOAT64_C(   257.04), SIMDE_FLOAT64_C(    -3.97) } },
    { { SIMDE_FLOAT64_C(    -1.74), SIMDE_FLOAT64_C(    -3.97), SIMDE_FLOAT64_C(     3.52), SIMDE_FLOAT64_C(    -3.35),
        SIMDE_FLOAT64_C(    -1.31), SIMDE_FLOAT64_C(     1.64), SIMDE_FLOAT64_C(     3.64), SIMDE_FLOAT64_C(     1.35) },
      UINT8_C(237),
      { SIMDE_FLOAT64_C(    -3.09), SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(    -0.49), SIMDE_FLOAT64_C(     1.71),
        SIMDE_FLOAT64_C(    -0.39), SIMDE_FLOAT64_C(    -2.14), SIMDE_FLOAT64_C(     1.22), SIMDE_FLOAT64_C(     1.16) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -3.97), SIMDE_FLOAT64_C(     0.32), SIMDE_FLOAT64_C(    51.29),
        SIMDE_FLOAT64_C(    -1.31), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(    16.60), SIMDE_FLOAT64_C(    14.45) } },
    { { SIMDE_FLOAT64_C(     2.80), SIMDE_FLOAT64_C(     3.11), SIMDE_FLOAT64_C(     3.45), SIMDE_FLOAT64_C(     2.07),
        SIMDE_FLOAT64_C(     3.14), SIMDE_FLOAT64_C(    -1.25), SIMDE_FLOAT64_C(    -3.90), SIMDE_FLOAT64_C(    -0.54) },
      UINT8_C(112),
      { SIMDE_FLOAT64_C(    -3.77), SIMDE_FLOAT64_C(     3.65), SIMDE_FLOAT64_C(    -3.35), SIMDE_FLOAT64_C(     2.64),
        SIMDE_FLOAT64_C(     3.31), SIMDE_FLOAT64_C(    -1.09), SIMDE_FLOAT64_C(     2.67), SIMDE_FLOAT64_C(     2.83) },
      { SIMDE_FLOAT64_C(     2.80), SIMDE_FLOAT64_C(     3.11), SIMDE_FLOAT64_C(     3.45), SIMDE_FLOAT64_C(     2.07),
        SIMDE_FLOAT64_C(  2041.74), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(   467.74), SIMDE_FLOAT64_C(    -0.54) } },
    { { SIMDE_FLOAT64_C(    -0.44), SIMDE_FLOAT64_C(    -2.64), SIMDE_FLOAT64_C(     0.47), SIMDE_FLOAT64_C(    -0.80),
        SIMDE_FLOAT64_C(     2.71), SIMDE_FLOAT64_C(    -3.20), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -1.08) },
      UINT8_C( 28),
      { SIMDE_FLOAT64_C(    -2.18), SIMDE_FLOAT64_C(     2.52), SIMDE_FLOAT64_C(     2.16), SIMDE_FLOAT64_C(     3.05),
        SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(     2.15), SIMDE_FLOAT64_C(    -0.87) },
      { SIMDE_FLOAT64_C(    -0.44), SIMDE_FLOAT64_C(    -2.64), SIMDE_FLOAT64_C(   144.54), SIMDE_FLOAT64_C(  1122.02),
        SIMDE_FLOAT64_C(     0.48), SIMDE_FLOAT64_C(    -3.20), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -1.08) } },
    { { SIMDE_FLOAT64_C(    -0.98), SIMDE_FLOAT64_C(     1.30), SIMDE_FLOAT64_C(     1.89), SIMDE_FLOAT64_C(    -0.87),
        SIMDE_FLOAT64_C(    -3.24), SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(    -0.65), SIMDE_FLOAT64_C(    -3.59) },
      UINT8_C(243),
      { SIMDE_FLOAT64_C(    -2.01), SIMDE_FLOAT64_C(     3.72), SIMDE_FLOAT64_C(     3.87), SIMDE_FLOAT64_C(    -3.34),
        SIMDE_FLOAT64_C(     2.55), SIMDE_FLOAT64_C(    -0.57), SIMDE_FLOAT64_C(    -1.99), SIMDE_FLOAT64_C(    -0.99) },
      { SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(  5248.07), SIMDE_FLOAT64_C(     1.89), SIMDE_FLOAT64_C(    -0.87),
        SIMDE_FLOAT64_C(   354.81), SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.10) } },
    { { SIMDE_FLOAT64_C(     2.63), SIMDE_FLOAT64_C(    -3.28), SIMDE_FLOAT64_C(    -0.19), SIMDE_FLOAT64_C(    -1.27),
        SIMDE_FLOAT64_C(    -0.36), SIMDE_FLOAT64_C(    -3.89), SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(    -1.84) },
      UINT8_C( 79),
      { SIMDE_FLOAT64_C(    -0.40), SIMDE_FLOAT64_C(     1.84), SIMDE_FLOAT64_C(    -0.77), SIMDE_FLOAT64_C(    -2.25),
        SIMDE_FLOAT64_C(    -3.02), SIMDE_FLOAT64_C(     2.26), SIMDE_FLOAT64_C(     3.05), SIMDE_FLOAT64_C(     2.87) },
      { SIMDE_FLOAT64_C(     0.40), SIMDE_FLOAT64_C(    69.18), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(     0.01),
        SIMDE_FLOAT64_C(    -0.36), SIMDE_FLOAT64_C(    -3.89), SIMDE_FLOAT64_C(  1122.02), SIMDE_FLOAT64_C(    -1.84) } },
    { { SIMDE_FLOAT64_C(    -2.62), SIMDE_FLOAT64_C(     3.81), SIMDE_FLOAT64_C(    -0.82), SIMDE_FLOAT64_C(     0.73),
        SIMDE_FLOAT64_C(    -3.78), SIMDE_FLOAT64_C(    -3.86), SIMDE_FLOAT64_C(     2.72), SIMDE_FLOAT64_C(     3.93) },
      UINT8_C(113),
      { SIMDE_FLOAT64_C(     3.38), SIMDE_FLOAT64_C(     2.48), SIMDE_FLOAT64_C(    -0.55), SIMDE_FLOAT64_C(    -2.61),
        SIMDE_FLOAT64_C(    -2.50), SIMDE_FLOAT64_C(    -1.93), SIMDE_FLOAT64_C(    -1.89), SIMDE_FLOAT64_C(     1.31) },
      { SIMDE_FLOAT64_C(  2398.83), SIMDE_FLOAT64_C(     3.81), SIMDE_FLOAT64_C(    -0.82), SIMDE_FLOAT64_C(     0.73),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     3.93) } },
    { { SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(     1.75), SIMDE_FLOAT64_C(     1.42), SIMDE_FLOAT64_C(    -2.64),
        SIMDE_FLOAT64_C(     3.91), SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(     1.76) },
      UINT8_C(247),
      { SIMDE_FLOAT64_C(     2.71), SIMDE_FLOAT64_C(     2.74), SIMDE_FLOAT64_C(     1.18), SIMDE_FLOAT64_C(     1.76),
        SIMDE_FLOAT64_C(     1.61), SIMDE_FLOAT64_C(     2.56), SIMDE_FLOAT64_C(     1.57), SIMDE_FLOAT64_C(    -3.21) },
      { SIMDE_FLOAT64_C(   512.86), SIMDE_FLOAT64_C(   549.54), SIMDE_FLOAT64_C(    15.14), SIMDE_FLOAT64_C(    -2.64),
        SIMDE_FLOAT64_C(    40.74), SIMDE_FLOAT64_C(   363.08), SIMDE_FLOAT64_C(    37.15), SIMDE_FLOAT64_C(     0.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_exp10_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_idivrem_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i rem;
    simde__m128i r;
  } test_vec[8] = {
    { simde_mm_set_epi32(INT32_C( 1747596798), INT32_C(-2063703989), INT32_C(  527472553), INT32_C(-1403096998)),
      simde_mm_set_epi32(INT32_C( -269879152), INT32_C( -177038436), INT32_C(  377180600), INT32_C( -518586410)),
      simde_mm_set_epi32(INT32_C(  128321886), INT32_C( -116281193), INT32_C(  150291953), INT32_C( -365924178)),
      simde_mm_set_epi32(INT32_C(         -6), INT32_C(         11), INT32_C(          1), INT32_C(          2)) },
    { simde_mm_set_epi32(INT32_C( -374673026), INT32_C(-1240805178), INT32_C( 1568850865), INT32_C(-1142977539)),
      simde_mm_set_epi32(INT32_C(  172780273), INT32_C(  168508556), INT32_C( -491358722), INT32_C( -230071737)),
      simde_mm_set_epi32(INT32_C(  -29112480), INT32_C(  -61245286), INT32_C(   94774699), INT32_C( -222690591)),
      simde_mm_set_epi32(INT32_C(         -2), INT32_C(         -7), INT32_C(         -3), INT32_C(          4)) },
    { simde_mm_set_epi32(INT32_C( 1492341726), INT32_C(  298608154), INT32_C( 1250819173), INT32_C( -650971253)),
      simde_mm_set_epi32(INT32_C(  298065861), INT32_C( -521585931), INT32_C(  330694282), INT32_C(   40997390)),
      simde_mm_set_epi32(INT32_C(    2012421), INT32_C(  298608154), INT32_C(  258736327), INT32_C(  -36010403)),
      simde_mm_set_epi32(INT32_C(          5), INT32_C(          0), INT32_C(          3), INT32_C(        -15)) },
    { simde_mm_set_epi32(INT32_C(-1586327268), INT32_C( 1691051285), INT32_C(   50347892), INT32_C(  728425428)),
      simde_mm_set_epi32(INT32_C( -441202718), INT32_C(  294920921), INT32_C( -411581651), INT32_C( -167991823)),
      simde_mm_set_epi32(INT32_C( -262719114), INT32_C(  216446680), INT32_C(   50347892), INT32_C(   56458136)),
      simde_mm_set_epi32(INT32_C(          3), INT32_C(          5), INT32_C(          0), INT32_C(         -4)) },
    { simde_mm_set_epi32(INT32_C(  492373082), INT32_C(  -13096811), INT32_C(-2087181083), INT32_C( -341007878)),
      simde_mm_set_epi32(INT32_C(  123290430), INT32_C( -298778955), INT32_C(  223555334), INT32_C( -332615043)),
      simde_mm_set_epi32(INT32_C(  122501792), INT32_C(  -13096811), INT32_C(  -75183077), INT32_C(   -8392835)),
      simde_mm_set_epi32(INT32_C(          3), INT32_C(          0), INT32_C(         -9), INT32_C(          1)) },
    { simde_mm_set_epi32(INT32_C(-1004264650), INT32_C( 1580565751), INT32_C( -471064457), INT32_C( 2081361826)),
      simde_mm_set_epi32(INT32_C(  328620632), INT32_C( -324312655), INT32_C( -184752009), INT32_C( -354760000)),
      simde_mm_set_epi32(INT32_C(  -18402754), INT32_C(  283315131), INT32_C( -101560439), INT32_C(  307561826)),
      simde_mm_set_epi32(INT32_C(         -3), INT32_C(         -4), INT32_C(          2), INT32_C(         -5)) },
    { simde_mm_set_epi32(INT32_C(  542053192), INT32_C(  499863549), INT32_C(  957375358), INT32_C(-1291033589)),
      simde_mm_set_epi32(INT32_C(  427537184), INT32_C(  493530770), INT32_C( -356091799), INT32_C(   29647056)),
      simde_mm_set_epi32(INT32_C(  114516008), INT32_C(    6332779), INT32_C(  245191760), INT32_C(  -16210181)),
      simde_mm_set_epi32(INT32_C(          1), INT32_C(          1), INT32_C(         -2), INT32_C(        -43)) },
    { simde_mm_set_epi32(INT32_C( -193211433), INT32_C( -857989172), INT32_C( -448329300), INT32_C(-1601364212)),
      simde_mm_set_epi32(INT32_C( -284723308), INT32_C( -171790410), INT32_C(  457043765), INT32_C(  -97355006)),
      simde_mm_set_epi32(INT32_C( -193211433), INT32_C( -170827532), INT32_C( -448329300), INT32_C(  -43684116)),
      simde_mm_set_epi32(INT32_C(          0), INT32_C(          4), INT32_C(          0), INT32_C(         16)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i rem;
    simde__m128i r = simde_mm_idivrem_epi32(&rem, test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_i32(r, ==, test_vec[i].r);
    simde_assert_m128i_i32(rem, ==, test_vec[i].rem);
  }

  return 0;
}

static int
test_simde_mm256_idivrem_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i rem;
    simde__m256i r;
  } test_vec[8] = {
    { simde_mm256_set_epi32(INT32_C(-1079516608), INT32_C( -708153743), INT32_C( 1508722402), INT32_C(-2074345640),
                            INT32_C( 1747596798), INT32_C(-2063703989), INT32_C(  527472553), INT32_C(-1403096998)),
      simde_mm256_set_epi32(INT32_C(  172780273), INT32_C(  168508556), INT32_C( -491358722), INT32_C( -230071737),
                            INT32_C(  -93668257), INT32_C( -310201295), INT32_C(  392212716), INT32_C( -285744385)),
      simde_mm256_set_epi32(INT32_C(  -42834970), INT32_C(  -34119519), INT32_C(   34646236), INT32_C(   -3700007),
                            INT32_C(   61568172), INT32_C( -202496219), INT32_C(  135259837), INT32_C( -260119458)),
      simde_mm256_set_epi32(INT32_C(         -6), INT32_C(         -4), INT32_C(         -3), INT32_C(          9),
                            INT32_C(        -18), INT32_C(          6), INT32_C(          1), INT32_C(          4)) },
    { simde_mm256_set_epi32(INT32_C( 1192263444), INT32_C(-2086343723), INT32_C( 1322777130), INT32_C(  163989560),
                            INT32_C( 1492341726), INT32_C(  298608154), INT32_C( 1250819173), INT32_C( -650971253)),
      simde_mm256_set_epi32(INT32_C( -441202718), INT32_C(  294920921), INT32_C( -411581651), INT32_C( -167991823),
                            INT32_C( -396581817), INT32_C(  422762821), INT32_C(   12586973), INT32_C(  182106357)),
      simde_mm256_set_epi32(INT32_C(  309858008), INT32_C(  -21897276), INT32_C(   88032177), INT32_C(  163989560),
                            INT32_C(  302596275), INT32_C(  298608154), INT32_C(    4708846), INT32_C( -104652182)),
      simde_mm256_set_epi32(INT32_C(         -2), INT32_C(         -7), INT32_C(         -3), INT32_C(          0),
                            INT32_C(         -3), INT32_C(          0), INT32_C(         99), INT32_C(         -3)) },
    { simde_mm256_set_epi32(INT32_C(  493161721), INT32_C(-1195115819), INT32_C(  894221337), INT32_C(-1330460172),
                            INT32_C(  492373082), INT32_C(  -13096811), INT32_C(-2087181083), INT32_C( -341007878)),
      simde_mm256_set_epi32(INT32_C(  328620632), INT32_C( -324312655), INT32_C( -184752009), INT32_C( -354760000),
                            INT32_C( -251066163), INT32_C(  395141437), INT32_C( -117766115), INT32_C(  520340456)),
      simde_mm256_set_epi32(INT32_C(  164541089), INT32_C( -222177854), INT32_C(  155213301), INT32_C( -266180172),
                            INT32_C(  241306919), INT32_C(  -13096811), INT32_C(  -85157128), INT32_C( -341007878)),
      simde_mm256_set_epi32(INT32_C(          1), INT32_C(          3), INT32_C(         -4), INT32_C(          3),
                            INT32_C(         -1), INT32_C(          0), INT32_C(         17), INT32_C(          0)) },
    { simde_mm256_set_epi32(INT32_C( 1710148738), INT32_C( 1974123080), INT32_C(-1424367196), INT32_C(  118588227),
                            INT32_C(  542053192), INT32_C(  499863549), INT32_C(  957375358), INT32_C(-1291033589)),
      simde_mm256_set_epi32(INT32_C( -284723308), INT32_C( -171790410), INT32_C(  457043765), INT32_C(  -97355006),
                            INT32_C(  -48302859), INT32_C( -214497293), INT32_C( -112082325), INT32_C( -400341053)),
      simde_mm256_set_epi32(INT32_C(    1808890), INT32_C(   84428570), INT32_C(  -53235901), INT32_C(   21233221),
                            INT32_C(   10721743), INT32_C(   70868963), INT32_C(   60716758), INT32_C(  -90010430)),
      simde_mm256_set_epi32(INT32_C(         -6), INT32_C(        -11), INT32_C(         -3), INT32_C(         -1),
                            INT32_C(        -11), INT32_C(         -2), INT32_C(         -8), INT32_C(          3)) },
    { simde_mm256_set_epi32(INT32_C( 1734496959), INT32_C(  380846712), INT32_C( -941967689), INT32_C( -739443621),
                            INT32_C( 1995198557), INT32_C( -980655097), INT32_C(-1888383043), INT32_C( 1779168063)),
      simde_mm256_set_epi32(INT32_C(  440775120), INT32_C( -129501140), INT32_C( -362589725), INT32_C( -352466550),
                            INT32_C(   67477586), INT32_C(  108492873), INT32_C(  360489056), INT32_C(  254567893)),
      simde_mm256_set_epi32(INT32_C(  412171599), INT32_C(  121844432), INT32_C( -216788239), INT32_C(  -34510521),
                            INT32_C(   38348563), INT32_C(   -4219240), INT32_C(  -85937763), INT32_C(  251760705)),
      simde_mm256_set_epi32(INT32_C(          3), INT32_C(         -2), INT32_C(          2), INT32_C(          2),
                            INT32_C(         29), INT32_C(         -9), INT32_C(         -5), INT32_C(          6)) },
    { simde_mm256_set_epi32(INT32_C( -362876916), INT32_C(-1845390533), INT32_C(  -48621016), INT32_C(  201516689),
                            INT32_C(-1435930720), INT32_C(-1932876068), INT32_C(-1153303869), INT32_C(  562234020)),
      simde_mm256_set_epi32(INT32_C( -166366311), INT32_C(  -85548959), INT32_C(  525546139), INT32_C(  219277873),
                            INT32_C(  295872976), INT32_C( -144152745), INT32_C( -265329050), INT32_C( -202024350)),
      simde_mm256_set_epi32(INT32_C(  -30144294), INT32_C(  -48862394), INT32_C(  -48621016), INT32_C(  201516689),
                            INT32_C( -252438816), INT32_C(  -58890383), INT32_C(  -91987669), INT32_C(  158185320)),
      simde_mm256_set_epi32(INT32_C(          2), INT32_C(         21), INT32_C(          0), INT32_C(          0),
                            INT32_C(         -4), INT32_C(         13), INT32_C(          4), INT32_C(         -2)) },
    { simde_mm256_set_epi32(INT32_C(  910061584), INT32_C( 2002226944), INT32_C( -621963189), INT32_C(  -48343218),
                            INT32_C(  523093293), INT32_C(-1235205724), INT32_C(-2088961787), INT32_C( 1943141679)),
      simde_mm256_set_epi32(INT32_C(  123967721), INT32_C(  -95531607), INT32_C(  228811177), INT32_C(    1270356),
                            INT32_C(  355625346), INT32_C(  -40994931), INT32_C( -379225067), INT32_C(  124491394)),
      simde_mm256_set_epi32(INT32_C(   42287537), INT32_C(   91594804), INT32_C( -164340835), INT32_C(     -69690),
                            INT32_C(  167467947), INT32_C(   -5357794), INT32_C( -192836452), INT32_C(   75770769)),
      simde_mm256_set_epi32(INT32_C(          7), INT32_C(        -20), INT32_C(         -2), INT32_C(        -38),
                            INT32_C(          1), INT32_C(         30), INT32_C(          5), INT32_C(         15)) },
    { simde_mm256_set_epi32(INT32_C( 1755684145), INT32_C(-2061726371), INT32_C(-1050443653), INT32_C(-1299940555),
                            INT32_C(-2116696545), INT32_C( 1493088054), INT32_C( -179829877), INT32_C(  651362699)),
      simde_mm256_set_epi32(INT32_C(  301617823), INT32_C(  343728879), INT32_C(  132913279), INT32_C(  518796827),
                            INT32_C(  -36154638), INT32_C( -532966429), INT32_C(  361195763), INT32_C(  469656308)),
      simde_mm256_set_epi32(INT32_C(  247595030), INT32_C( -343081976), INT32_C( -120050700), INT32_C( -262346901),
                            INT32_C(  -19727541), INT32_C(  427155196), INT32_C( -179829877), INT32_C(  181706391)),
      simde_mm256_set_epi32(INT32_C(          5), INT32_C(         -5), INT32_C(         -7), INT32_C(         -2),
                            INT32_C(         58), INT32_C(         -2), INT32_C(          0), INT32_C(          1)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i rem;
    simde__m256i r = simde_mm256_idivrem_epi32(&rem, test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i32(r, ==, test_vec[i].r);
    simde_assert_m256i_i32(rem, ==, test_vec[i].rem);
  }

  return 0;
}

static int
test_simde_mm_hypot_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 b[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   492.01), SIMDE_FLOAT32_C(   211.12), SIMDE_FLOAT32_C(    12.31), SIMDE_FLOAT32_C(   870.52) },
      { SIMDE_FLOAT32_C(  -363.60), SIMDE_FLOAT32_C(   789.00), SIMDE_FLOAT32_C(   397.20), SIMDE_FLOAT32_C(  -757.25) },
      { SIMDE_FLOAT32_C(   611.78), SIMDE_FLOAT32_C(   816.76), SIMDE_FLOAT32_C(   397.39), SIMDE_FLOAT32_C(  1153.79) } },
    { { SIMDE_FLOAT32_C(  -192.59), SIMDE_FLOAT32_C(  -586.23), SIMDE_FLOAT32_C(   571.12), SIMDE_FLOAT32_C(  -717.05) },
      { SIMDE_FLOAT32_C(  -663.78), SIMDE_FLOAT32_C(    66.94), SIMDE_FLOAT32_C(  -412.69), SIMDE_FLOAT32_C(  -769.47) },
      { SIMDE_FLOAT32_C(   691.15), SIMDE_FLOAT32_C(   590.04), SIMDE_FLOAT32_C(   704.62), SIMDE_FLOAT32_C(  1051.78) } },
    { { SIMDE_FLOAT32_C(  -594.99), SIMDE_FLOAT32_C(  -442.39), SIMDE_FLOAT32_C(  -303.17), SIMDE_FLOAT32_C(   275.57) },
      { SIMDE_FLOAT32_C(   293.68), SIMDE_FLOAT32_C(    44.26), SIMDE_FLOAT32_C(  -780.93), SIMDE_FLOAT32_C(  -309.10) },
      { SIMDE_FLOAT32_C(   663.52), SIMDE_FLOAT32_C(   444.60), SIMDE_FLOAT32_C(   837.71), SIMDE_FLOAT32_C(   414.10) } },
    { { SIMDE_FLOAT32_C(  -878.78), SIMDE_FLOAT32_C(  -647.94), SIMDE_FLOAT32_C(   445.74), SIMDE_FLOAT32_C(   697.72) },
      { SIMDE_FLOAT32_C(    98.72), SIMDE_FLOAT32_C(  -787.29), SIMDE_FLOAT32_C(    -3.77), SIMDE_FLOAT32_C(  -409.27) },
      { SIMDE_FLOAT32_C(   884.31), SIMDE_FLOAT32_C(  1019.63), SIMDE_FLOAT32_C(   445.76), SIMDE_FLOAT32_C(   808.90) } },
    { { SIMDE_FLOAT32_C(   423.83), SIMDE_FLOAT32_C(  -991.46), SIMDE_FLOAT32_C(  -538.75), SIMDE_FLOAT32_C(  -939.77) },
      { SIMDE_FLOAT32_C(   797.54), SIMDE_FLOAT32_C(   858.45), SIMDE_FLOAT32_C(  -697.02), SIMDE_FLOAT32_C(  -395.04) },
      { SIMDE_FLOAT32_C(   903.16), SIMDE_FLOAT32_C(  1311.46), SIMDE_FLOAT32_C(   880.96), SIMDE_FLOAT32_C(  1019.42) } },
    { { SIMDE_FLOAT32_C(  -727.78), SIMDE_FLOAT32_C(   874.10), SIMDE_FLOAT32_C(  -112.10), SIMDE_FLOAT32_C(  -391.56) },
      { SIMDE_FLOAT32_C(   -58.96), SIMDE_FLOAT32_C(   475.22), SIMDE_FLOAT32_C(  -161.04), SIMDE_FLOAT32_C(   346.05) },
      { SIMDE_FLOAT32_C(   730.16), SIMDE_FLOAT32_C(   994.93), SIMDE_FLOAT32_C(   196.21), SIMDE_FLOAT32_C(   522.56) } },
    { { SIMDE_FLOAT32_C(  -967.17), SIMDE_FLOAT32_C(   535.80), SIMDE_FLOAT32_C(  -378.38), SIMDE_FLOAT32_C(   326.51) },
      { SIMDE_FLOAT32_C(  -419.95), SIMDE_FLOAT32_C(  -159.32), SIMDE_FLOAT32_C(  -982.59), SIMDE_FLOAT32_C(  -298.72) },
      { SIMDE_FLOAT32_C(  1054.41), SIMDE_FLOAT32_C(   558.99), SIMDE_FLOAT32_C(  1052.93), SIMDE_FLOAT32_C(   442.54) } },
    { { SIMDE_FLOAT32_C(   192.74), SIMDE_FLOAT32_C(   463.15), SIMDE_FLOAT32_C(  -601.00), SIMDE_FLOAT32_C(  -708.54) },
      { SIMDE_FLOAT32_C(   675.86), SIMDE_FLOAT32_C(   395.23), SIMDE_FLOAT32_C(  -117.81), SIMDE_FLOAT32_C(    99.70) },
      { SIMDE_FLOAT32_C(   702.81), SIMDE_FLOAT32_C(   608.86), SIMDE_FLOAT32_C(   612.44), SIMDE_FLOAT32_C(   715.52) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 b = simde_mm_loadu_ps(test_vec[i].b);
    simde__m128 r = simde_mm_hypot_ps(a, b);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;

}

static int
test_simde_mm_hypot_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 b[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -603.93), SIMDE_FLOAT64_C(   656.94) },
      { SIMDE_FLOAT64_C(   263.86), SIMDE_FLOAT64_C(  -668.26) },
      { SIMDE_FLOAT64_C(   659.06), SIMDE_FLOAT64_C(   937.09) } },
    { { SIMDE_FLOAT64_C(  -573.72), SIMDE_FLOAT64_C(   127.62) },
      { SIMDE_FLOAT64_C(  -494.33), SIMDE_FLOAT64_C(   413.83) },
      { SIMDE_FLOAT64_C(   757.31), SIMDE_FLOAT64_C(   433.06) } },
    { { SIMDE_FLOAT64_C(    92.50), SIMDE_FLOAT64_C(   179.32) },
      { SIMDE_FLOAT64_C(  -379.77), SIMDE_FLOAT64_C(   381.33) },
      { SIMDE_FLOAT64_C(   390.87), SIMDE_FLOAT64_C(   421.39) } },
    { { SIMDE_FLOAT64_C(   344.30), SIMDE_FLOAT64_C(   576.77) },
      { SIMDE_FLOAT64_C(  -663.77), SIMDE_FLOAT64_C(   656.74) },
      { SIMDE_FLOAT64_C(   747.75), SIMDE_FLOAT64_C(   874.05) } },
    { { SIMDE_FLOAT64_C(   499.56), SIMDE_FLOAT64_C(   761.69) },
      { SIMDE_FLOAT64_C(  -752.98), SIMDE_FLOAT64_C(  -522.11) },
      { SIMDE_FLOAT64_C(   903.63), SIMDE_FLOAT64_C(   923.46) } },
    { { SIMDE_FLOAT64_C(   242.72), SIMDE_FLOAT64_C(   412.75) },
      { SIMDE_FLOAT64_C(  -101.50), SIMDE_FLOAT64_C(    96.94) },
      { SIMDE_FLOAT64_C(   263.09), SIMDE_FLOAT64_C(   423.98) } },
    { { SIMDE_FLOAT64_C(  -934.53), SIMDE_FLOAT64_C(  -147.86) },
      { SIMDE_FLOAT64_C(  -959.33), SIMDE_FLOAT64_C(   790.23) },
      { SIMDE_FLOAT64_C(  1339.28), SIMDE_FLOAT64_C(   803.94) } },
    { { SIMDE_FLOAT64_C(   239.33), SIMDE_FLOAT64_C(  -100.41) },
      { SIMDE_FLOAT64_C(  -270.12), SIMDE_FLOAT64_C(   635.40) },
      { SIMDE_FLOAT64_C(   360.89), SIMDE_FLOAT64_C(   643.28) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d b = simde_mm_loadu_pd(test_vec[i].b);
    simde__m128d r = simde_mm_hypot_pd(a, b);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_hypot_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 b[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -777.18), SIMDE_FLOAT32_C(   159.63), SIMDE_FLOAT32_C(   756.34), SIMDE_FLOAT32_C(   -76.33),
        SIMDE_FLOAT32_C(   113.08), SIMDE_FLOAT32_C(   246.24), SIMDE_FLOAT32_C(   841.85), SIMDE_FLOAT32_C(  -845.53) },
      { SIMDE_FLOAT32_C(  -621.65), SIMDE_FLOAT32_C(    72.13), SIMDE_FLOAT32_C(   721.27), SIMDE_FLOAT32_C(  -427.76),
        SIMDE_FLOAT32_C(  -945.55), SIMDE_FLOAT32_C(  -213.25), SIMDE_FLOAT32_C(  -603.55), SIMDE_FLOAT32_C(   373.40) },
      { SIMDE_FLOAT32_C(   995.22), SIMDE_FLOAT32_C(   175.17), SIMDE_FLOAT32_C(  1045.12), SIMDE_FLOAT32_C(   434.52),
        SIMDE_FLOAT32_C(   952.29), SIMDE_FLOAT32_C(   325.74), SIMDE_FLOAT32_C(  1035.85), SIMDE_FLOAT32_C(   924.31) } },
    { { SIMDE_FLOAT32_C(  -731.26), SIMDE_FLOAT32_C(  -820.00), SIMDE_FLOAT32_C(   393.03), SIMDE_FLOAT32_C(  -720.80),
        SIMDE_FLOAT32_C(  -923.20), SIMDE_FLOAT32_C(   -65.81), SIMDE_FLOAT32_C(  -541.82), SIMDE_FLOAT32_C(  -812.46) },
      { SIMDE_FLOAT32_C(   833.72), SIMDE_FLOAT32_C(  -217.64), SIMDE_FLOAT32_C(   806.57), SIMDE_FLOAT32_C(  -582.91),
        SIMDE_FLOAT32_C(   620.23), SIMDE_FLOAT32_C(  -724.63), SIMDE_FLOAT32_C(   373.46), SIMDE_FLOAT32_C(   843.05) },
      { SIMDE_FLOAT32_C(  1108.98), SIMDE_FLOAT32_C(   848.39), SIMDE_FLOAT32_C(   897.23), SIMDE_FLOAT32_C(   927.00),
        SIMDE_FLOAT32_C(  1112.20), SIMDE_FLOAT32_C(   727.61), SIMDE_FLOAT32_C(   658.06), SIMDE_FLOAT32_C(  1170.82) } },
    { { SIMDE_FLOAT32_C(   435.00), SIMDE_FLOAT32_C(   129.80), SIMDE_FLOAT32_C(  -233.28), SIMDE_FLOAT32_C(  -451.92),
        SIMDE_FLOAT32_C(  -623.96), SIMDE_FLOAT32_C(  -391.43), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -245.61) },
      { SIMDE_FLOAT32_C(   680.70), SIMDE_FLOAT32_C(  -576.18), SIMDE_FLOAT32_C(   326.63), SIMDE_FLOAT32_C(   735.15),
        SIMDE_FLOAT32_C(   210.56), SIMDE_FLOAT32_C(   723.09), SIMDE_FLOAT32_C(   108.56), SIMDE_FLOAT32_C(   479.30) },
      { SIMDE_FLOAT32_C(   807.82), SIMDE_FLOAT32_C(   590.62), SIMDE_FLOAT32_C(   401.38), SIMDE_FLOAT32_C(   862.95),
        SIMDE_FLOAT32_C(   658.53), SIMDE_FLOAT32_C(   822.24), SIMDE_FLOAT32_C(   316.64), SIMDE_FLOAT32_C(   538.57) } },
    { { SIMDE_FLOAT32_C(   903.09), SIMDE_FLOAT32_C(  -498.41), SIMDE_FLOAT32_C(   758.50), SIMDE_FLOAT32_C(   979.89),
        SIMDE_FLOAT32_C(   435.78), SIMDE_FLOAT32_C(  -783.32), SIMDE_FLOAT32_C(  -832.57), SIMDE_FLOAT32_C(   269.50) },
      { SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(   973.99), SIMDE_FLOAT32_C(   686.59), SIMDE_FLOAT32_C(  -380.74),
        SIMDE_FLOAT32_C(  -750.64), SIMDE_FLOAT32_C(    60.05), SIMDE_FLOAT32_C(  -537.69), SIMDE_FLOAT32_C(   684.36) },
      { SIMDE_FLOAT32_C(   903.09), SIMDE_FLOAT32_C(  1094.11), SIMDE_FLOAT32_C(  1023.10), SIMDE_FLOAT32_C(  1051.26),
        SIMDE_FLOAT32_C(   867.97), SIMDE_FLOAT32_C(   785.62), SIMDE_FLOAT32_C(   991.10), SIMDE_FLOAT32_C(   735.51) } },
    { { SIMDE_FLOAT32_C(  -810.16), SIMDE_FLOAT32_C(   229.03), SIMDE_FLOAT32_C(  -767.56), SIMDE_FLOAT32_C(  -434.12),
        SIMDE_FLOAT32_C(   837.60), SIMDE_FLOAT32_C(   -65.02), SIMDE_FLOAT32_C(   320.28), SIMDE_FLOAT32_C(   518.30) },
      { SIMDE_FLOAT32_C(   358.80), SIMDE_FLOAT32_C(  -353.09), SIMDE_FLOAT32_C(   253.45), SIMDE_FLOAT32_C(  -430.64),
        SIMDE_FLOAT32_C(  -630.00), SIMDE_FLOAT32_C(  -637.99), SIMDE_FLOAT32_C(  -951.34), SIMDE_FLOAT32_C(  -726.92) },
      { SIMDE_FLOAT32_C(   886.06), SIMDE_FLOAT32_C(   420.86), SIMDE_FLOAT32_C(   808.32), SIMDE_FLOAT32_C(   611.48),
        SIMDE_FLOAT32_C(  1048.08), SIMDE_FLOAT32_C(   641.29), SIMDE_FLOAT32_C(  1003.81), SIMDE_FLOAT32_C(   892.78) } },
    { { SIMDE_FLOAT32_C(  -136.40), SIMDE_FLOAT32_C(   807.17), SIMDE_FLOAT32_C(  -747.03), SIMDE_FLOAT32_C(  -700.62),
        SIMDE_FLOAT32_C(  -976.15), SIMDE_FLOAT32_C(  -579.60), SIMDE_FLOAT32_C(   568.87), SIMDE_FLOAT32_C(    22.88) },
      { SIMDE_FLOAT32_C(  -605.60), SIMDE_FLOAT32_C(   255.46), SIMDE_FLOAT32_C(   642.15), SIMDE_FLOAT32_C(  -356.24),
        SIMDE_FLOAT32_C(  -684.50), SIMDE_FLOAT32_C(  -895.54), SIMDE_FLOAT32_C(  -671.88), SIMDE_FLOAT32_C(  -494.65) },
      { SIMDE_FLOAT32_C(   620.77), SIMDE_FLOAT32_C(   846.63), SIMDE_FLOAT32_C(   985.09), SIMDE_FLOAT32_C(   785.99),
        SIMDE_FLOAT32_C(  1192.23), SIMDE_FLOAT32_C(  1066.74), SIMDE_FLOAT32_C(   880.36), SIMDE_FLOAT32_C(   495.18) } },
    { { SIMDE_FLOAT32_C(   333.49), SIMDE_FLOAT32_C(  -439.45), SIMDE_FLOAT32_C(    71.23), SIMDE_FLOAT32_C(   171.09),
        SIMDE_FLOAT32_C(   495.54), SIMDE_FLOAT32_C(  -608.49), SIMDE_FLOAT32_C(  -310.61), SIMDE_FLOAT32_C(  -145.66) },
      { SIMDE_FLOAT32_C(    38.42), SIMDE_FLOAT32_C(   942.84), SIMDE_FLOAT32_C(   423.70), SIMDE_FLOAT32_C(   408.42),
        SIMDE_FLOAT32_C(  -695.15), SIMDE_FLOAT32_C(   472.36), SIMDE_FLOAT32_C(   681.50), SIMDE_FLOAT32_C(   168.45) },
      { SIMDE_FLOAT32_C(   335.70), SIMDE_FLOAT32_C(  1040.22), SIMDE_FLOAT32_C(   429.65), SIMDE_FLOAT32_C(   442.81),
        SIMDE_FLOAT32_C(   853.69), SIMDE_FLOAT32_C(   770.31), SIMDE_FLOAT32_C(   748.95), SIMDE_FLOAT32_C(   222.69) } },
    { { SIMDE_FLOAT32_C(   279.53), SIMDE_FLOAT32_C(   934.47), SIMDE_FLOAT32_C(   467.83), SIMDE_FLOAT32_C(   303.38),
        SIMDE_FLOAT32_C(  -645.12), SIMDE_FLOAT32_C(    36.70), SIMDE_FLOAT32_C(  -673.74), SIMDE_FLOAT32_C(  -250.73) },
      { SIMDE_FLOAT32_C(  -707.84), SIMDE_FLOAT32_C(   968.41), SIMDE_FLOAT32_C(   393.03), SIMDE_FLOAT32_C(  -392.34),
        SIMDE_FLOAT32_C(  -927.14), SIMDE_FLOAT32_C(   721.15), SIMDE_FLOAT32_C(   113.01), SIMDE_FLOAT32_C(   406.35) },
      { SIMDE_FLOAT32_C(   761.04), SIMDE_FLOAT32_C(  1345.75), SIMDE_FLOAT32_C(   611.01), SIMDE_FLOAT32_C(   495.95),
        SIMDE_FLOAT32_C(  1129.50), SIMDE_FLOAT32_C(   722.08), SIMDE_FLOAT32_C(   683.15), SIMDE_FLOAT32_C(   477.48) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 b = simde_mm256_loadu_ps(test_vec[i].b);
    simde__m256 r = simde_mm256_hypot_ps(a, b);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_hypot_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 b[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -477.45), SIMDE_FLOAT64_C(   593.44), SIMDE_FLOAT64_C(   978.32), SIMDE_FLOAT64_C(  -584.34) },
      { SIMDE_FLOAT64_C(   161.70), SIMDE_FLOAT64_C(   -36.27), SIMDE_FLOAT64_C(   750.17), SIMDE_FLOAT64_C(  -655.19) },
      { SIMDE_FLOAT64_C(   504.09), SIMDE_FLOAT64_C(   594.55), SIMDE_FLOAT64_C(  1232.83), SIMDE_FLOAT64_C(   877.91) } },
    { { SIMDE_FLOAT64_C(  -840.17), SIMDE_FLOAT64_C(  -429.90), SIMDE_FLOAT64_C(   790.20), SIMDE_FLOAT64_C(   -18.28) },
      { SIMDE_FLOAT64_C(   964.56), SIMDE_FLOAT64_C(   136.47), SIMDE_FLOAT64_C(   164.17), SIMDE_FLOAT64_C(   892.62) },
      { SIMDE_FLOAT64_C(  1279.16), SIMDE_FLOAT64_C(   451.04), SIMDE_FLOAT64_C(   807.07), SIMDE_FLOAT64_C(   892.81) } },
    { { SIMDE_FLOAT64_C(   115.18), SIMDE_FLOAT64_C(   353.33), SIMDE_FLOAT64_C(   -41.82), SIMDE_FLOAT64_C(   836.90) },
      { SIMDE_FLOAT64_C(   325.83), SIMDE_FLOAT64_C(   174.90), SIMDE_FLOAT64_C(  -541.27), SIMDE_FLOAT64_C(  -977.07) },
      { SIMDE_FLOAT64_C(   345.59), SIMDE_FLOAT64_C(   394.25), SIMDE_FLOAT64_C(   542.88), SIMDE_FLOAT64_C(  1286.49) } },
    { { SIMDE_FLOAT64_C(   604.56), SIMDE_FLOAT64_C(   980.27), SIMDE_FLOAT64_C(   536.46), SIMDE_FLOAT64_C(   153.38) },
      { SIMDE_FLOAT64_C(  -931.38), SIMDE_FLOAT64_C(  -178.15), SIMDE_FLOAT64_C(  -619.34), SIMDE_FLOAT64_C(  -408.83) },
      { SIMDE_FLOAT64_C(  1110.39), SIMDE_FLOAT64_C(   996.33), SIMDE_FLOAT64_C(   819.37), SIMDE_FLOAT64_C(   436.65) } },
    { { SIMDE_FLOAT64_C(  -584.72), SIMDE_FLOAT64_C(  -641.02), SIMDE_FLOAT64_C(     6.83), SIMDE_FLOAT64_C(   576.98) },
      { SIMDE_FLOAT64_C(   322.71), SIMDE_FLOAT64_C(  -242.99), SIMDE_FLOAT64_C(   921.80), SIMDE_FLOAT64_C(   482.53) },
      { SIMDE_FLOAT64_C(   667.86), SIMDE_FLOAT64_C(   685.53), SIMDE_FLOAT64_C(   921.83), SIMDE_FLOAT64_C(   752.16) } },
    { { SIMDE_FLOAT64_C(   327.10), SIMDE_FLOAT64_C(   712.00), SIMDE_FLOAT64_C(  -535.75), SIMDE_FLOAT64_C(   291.66) },
      { SIMDE_FLOAT64_C(  -151.54), SIMDE_FLOAT64_C(   628.42), SIMDE_FLOAT64_C(   184.28), SIMDE_FLOAT64_C(   963.64) },
      { SIMDE_FLOAT64_C(   360.50), SIMDE_FLOAT64_C(   949.66), SIMDE_FLOAT64_C(   566.56), SIMDE_FLOAT64_C(  1006.81) } },
    { { SIMDE_FLOAT64_C(   -18.25), SIMDE_FLOAT64_C(  -857.54), SIMDE_FLOAT64_C(   800.54), SIMDE_FLOAT64_C(  -692.42) },
      { SIMDE_FLOAT64_C(   317.36), SIMDE_FLOAT64_C(  -740.72), SIMDE_FLOAT64_C(  -669.48), SIMDE_FLOAT64_C(   -78.07) },
      { SIMDE_FLOAT64_C(   317.88), SIMDE_FLOAT64_C(  1133.16), SIMDE_FLOAT64_C(  1043.58), SIMDE_FLOAT64_C(   696.81) } },
    { { SIMDE_FLOAT64_C(  -760.45), SIMDE_FLOAT64_C(   866.98), SIMDE_FLOAT64_C(  -924.70), SIMDE_FLOAT64_C(  -691.83) },
      { SIMDE_FLOAT64_C(  -311.18), SIMDE_FLOAT64_C(  -544.04), SIMDE_FLOAT64_C(  -100.66), SIMDE_FLOAT64_C(   104.10) },
      { SIMDE_FLOAT64_C(   821.66), SIMDE_FLOAT64_C(  1023.54), SIMDE_FLOAT64_C(   930.16), SIMDE_FLOAT64_C(   699.62) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d b = simde_mm256_loadu_pd(test_vec[i].b);
    simde__m256d r = simde_mm256_hypot_pd(a, b);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_hypot_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 b[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -926.16), SIMDE_FLOAT32_C(   -45.76), SIMDE_FLOAT32_C(   907.70), SIMDE_FLOAT32_C(  -928.37),
        SIMDE_FLOAT32_C(   496.55), SIMDE_FLOAT32_C(   566.66), SIMDE_FLOAT32_C(  -501.51), SIMDE_FLOAT32_C(  -575.98),
        SIMDE_FLOAT32_C(  -281.74), SIMDE_FLOAT32_C(  -821.54), SIMDE_FLOAT32_C(   198.58), SIMDE_FLOAT32_C(  -379.20),
        SIMDE_FLOAT32_C(   104.18), SIMDE_FLOAT32_C(  -675.07), SIMDE_FLOAT32_C(  -169.00), SIMDE_FLOAT32_C(   502.70) },
      { SIMDE_FLOAT32_C(     7.03), SIMDE_FLOAT32_C(  -875.48), SIMDE_FLOAT32_C(  -451.63), SIMDE_FLOAT32_C(  -815.00),
        SIMDE_FLOAT32_C(    37.83), SIMDE_FLOAT32_C(  -588.92), SIMDE_FLOAT32_C(  -905.87), SIMDE_FLOAT32_C(   -49.63),
        SIMDE_FLOAT32_C(   813.22), SIMDE_FLOAT32_C(  -962.83), SIMDE_FLOAT32_C(  -486.45), SIMDE_FLOAT32_C(  -367.13),
        SIMDE_FLOAT32_C(  -242.02), SIMDE_FLOAT32_C(   475.59), SIMDE_FLOAT32_C(   -31.20), SIMDE_FLOAT32_C(  -168.18) },
      { SIMDE_FLOAT32_C(   926.19), SIMDE_FLOAT32_C(   876.68), SIMDE_FLOAT32_C(  1013.85), SIMDE_FLOAT32_C(  1235.35),
        SIMDE_FLOAT32_C(   497.99), SIMDE_FLOAT32_C(   817.27), SIMDE_FLOAT32_C(  1035.43), SIMDE_FLOAT32_C(   578.11),
        SIMDE_FLOAT32_C(   860.64), SIMDE_FLOAT32_C(  1265.69), SIMDE_FLOAT32_C(   525.42), SIMDE_FLOAT32_C(   527.80),
        SIMDE_FLOAT32_C(   263.49), SIMDE_FLOAT32_C(   825.78), SIMDE_FLOAT32_C(   171.86), SIMDE_FLOAT32_C(   530.09) } },
    { { SIMDE_FLOAT32_C(  -570.17), SIMDE_FLOAT32_C(  -123.51), SIMDE_FLOAT32_C(   -96.55), SIMDE_FLOAT32_C(   926.38),
        SIMDE_FLOAT32_C(  -556.85), SIMDE_FLOAT32_C(   401.94), SIMDE_FLOAT32_C(  -649.60), SIMDE_FLOAT32_C(   161.41),
        SIMDE_FLOAT32_C(   580.39), SIMDE_FLOAT32_C(   548.98), SIMDE_FLOAT32_C(   782.21), SIMDE_FLOAT32_C(  -315.43),
        SIMDE_FLOAT32_C(   873.91), SIMDE_FLOAT32_C(  -386.79), SIMDE_FLOAT32_C(  -812.72), SIMDE_FLOAT32_C(  -119.05) },
      { SIMDE_FLOAT32_C(  -262.27), SIMDE_FLOAT32_C(  -264.35), SIMDE_FLOAT32_C(    65.94), SIMDE_FLOAT32_C(   775.56),
        SIMDE_FLOAT32_C(   146.72), SIMDE_FLOAT32_C(   160.08), SIMDE_FLOAT32_C(  -274.07), SIMDE_FLOAT32_C(   -40.05),
        SIMDE_FLOAT32_C(   197.24), SIMDE_FLOAT32_C(   239.47), SIMDE_FLOAT32_C(   592.82), SIMDE_FLOAT32_C(   955.23),
        SIMDE_FLOAT32_C(  -284.94), SIMDE_FLOAT32_C(  -438.38), SIMDE_FLOAT32_C(  -212.95), SIMDE_FLOAT32_C(   144.89) },
      { SIMDE_FLOAT32_C(   627.60), SIMDE_FLOAT32_C(   291.78), SIMDE_FLOAT32_C(   116.92), SIMDE_FLOAT32_C(  1208.17),
        SIMDE_FLOAT32_C(   575.85), SIMDE_FLOAT32_C(   432.64), SIMDE_FLOAT32_C(   705.05), SIMDE_FLOAT32_C(   166.30),
        SIMDE_FLOAT32_C(   612.99), SIMDE_FLOAT32_C(   598.94), SIMDE_FLOAT32_C(   981.47), SIMDE_FLOAT32_C(  1005.96),
        SIMDE_FLOAT32_C(   919.19), SIMDE_FLOAT32_C(   584.62), SIMDE_FLOAT32_C(   840.16), SIMDE_FLOAT32_C(   187.53) } },
    { { SIMDE_FLOAT32_C(   438.11), SIMDE_FLOAT32_C(   690.50), SIMDE_FLOAT32_C(    71.27), SIMDE_FLOAT32_C(   881.27),
        SIMDE_FLOAT32_C(    92.44), SIMDE_FLOAT32_C(   421.67), SIMDE_FLOAT32_C(    42.68), SIMDE_FLOAT32_C(  -327.17),
        SIMDE_FLOAT32_C(   -29.36), SIMDE_FLOAT32_C(  -175.11), SIMDE_FLOAT32_C(   357.41), SIMDE_FLOAT32_C(  -155.45),
        SIMDE_FLOAT32_C(   438.11), SIMDE_FLOAT32_C(   544.68), SIMDE_FLOAT32_C(   725.50), SIMDE_FLOAT32_C(  -824.16) },
      { SIMDE_FLOAT32_C(  -719.67), SIMDE_FLOAT32_C(  -208.56), SIMDE_FLOAT32_C(   951.40), SIMDE_FLOAT32_C(   427.05),
        SIMDE_FLOAT32_C(   951.52), SIMDE_FLOAT32_C(  -322.67), SIMDE_FLOAT32_C(  -613.00), SIMDE_FLOAT32_C(   148.76),
        SIMDE_FLOAT32_C(   916.80), SIMDE_FLOAT32_C(   979.82), SIMDE_FLOAT32_C(   103.99), SIMDE_FLOAT32_C(  -368.15),
        SIMDE_FLOAT32_C(  -458.56), SIMDE_FLOAT32_C(   891.04), SIMDE_FLOAT32_C(   776.74), SIMDE_FLOAT32_C(   979.55) },
      { SIMDE_FLOAT32_C(   842.54), SIMDE_FLOAT32_C(   721.31), SIMDE_FLOAT32_C(   954.07), SIMDE_FLOAT32_C(   979.29),
        SIMDE_FLOAT32_C(   956.00), SIMDE_FLOAT32_C(   530.96), SIMDE_FLOAT32_C(   614.48), SIMDE_FLOAT32_C(   359.40),
        SIMDE_FLOAT32_C(   917.27), SIMDE_FLOAT32_C(   995.34), SIMDE_FLOAT32_C(   372.23), SIMDE_FLOAT32_C(   399.62),
        SIMDE_FLOAT32_C(   634.21), SIMDE_FLOAT32_C(  1044.33), SIMDE_FLOAT32_C(  1062.86), SIMDE_FLOAT32_C(  1280.14) } },
    { { SIMDE_FLOAT32_C(   581.54), SIMDE_FLOAT32_C(  -151.99), SIMDE_FLOAT32_C(   860.81), SIMDE_FLOAT32_C(  -326.03),
        SIMDE_FLOAT32_C(  -730.33), SIMDE_FLOAT32_C(   -96.51), SIMDE_FLOAT32_C(   346.80), SIMDE_FLOAT32_C(   240.31),
        SIMDE_FLOAT32_C(   728.39), SIMDE_FLOAT32_C(  -295.79), SIMDE_FLOAT32_C(  -915.13), SIMDE_FLOAT32_C(   166.50),
        SIMDE_FLOAT32_C(  -751.11), SIMDE_FLOAT32_C(   810.37), SIMDE_FLOAT32_C(   342.34), SIMDE_FLOAT32_C(  -470.78) },
      { SIMDE_FLOAT32_C(  -398.19), SIMDE_FLOAT32_C(   293.73), SIMDE_FLOAT32_C(   956.27), SIMDE_FLOAT32_C(  -446.67),
        SIMDE_FLOAT32_C(   971.06), SIMDE_FLOAT32_C(  -656.73), SIMDE_FLOAT32_C(   702.10), SIMDE_FLOAT32_C(   887.86),
        SIMDE_FLOAT32_C(  -676.91), SIMDE_FLOAT32_C(  -193.91), SIMDE_FLOAT32_C(  -480.29), SIMDE_FLOAT32_C(  -135.48),
        SIMDE_FLOAT32_C(  -302.88), SIMDE_FLOAT32_C(  -703.55), SIMDE_FLOAT32_C(  -155.93), SIMDE_FLOAT32_C(  -721.34) },
      { SIMDE_FLOAT32_C(   704.80), SIMDE_FLOAT32_C(   330.72), SIMDE_FLOAT32_C(  1286.64), SIMDE_FLOAT32_C(   553.00),
        SIMDE_FLOAT32_C(  1215.05), SIMDE_FLOAT32_C(   663.78), SIMDE_FLOAT32_C(   783.08), SIMDE_FLOAT32_C(   919.81),
        SIMDE_FLOAT32_C(   994.36), SIMDE_FLOAT32_C(   353.68), SIMDE_FLOAT32_C(  1033.51), SIMDE_FLOAT32_C(   214.66),
        SIMDE_FLOAT32_C(   809.88), SIMDE_FLOAT32_C(  1073.16), SIMDE_FLOAT32_C(   376.18), SIMDE_FLOAT32_C(   861.37) } },
    { { SIMDE_FLOAT32_C(   144.45), SIMDE_FLOAT32_C(  -295.12), SIMDE_FLOAT32_C(   -47.37), SIMDE_FLOAT32_C(   414.12),
        SIMDE_FLOAT32_C(   608.38), SIMDE_FLOAT32_C(  -700.56), SIMDE_FLOAT32_C(  -345.56), SIMDE_FLOAT32_C(   336.76),
        SIMDE_FLOAT32_C(     3.65), SIMDE_FLOAT32_C(  -260.69), SIMDE_FLOAT32_C(  -496.74), SIMDE_FLOAT32_C(   252.54),
        SIMDE_FLOAT32_C(  -450.32), SIMDE_FLOAT32_C(   845.60), SIMDE_FLOAT32_C(   781.76), SIMDE_FLOAT32_C(   151.49) },
      { SIMDE_FLOAT32_C(   139.33), SIMDE_FLOAT32_C(   738.03), SIMDE_FLOAT32_C(   704.82), SIMDE_FLOAT32_C(   110.39),
        SIMDE_FLOAT32_C(  -918.70), SIMDE_FLOAT32_C(   406.92), SIMDE_FLOAT32_C(    -1.75), SIMDE_FLOAT32_C(  -595.61),
        SIMDE_FLOAT32_C(  -787.00), SIMDE_FLOAT32_C(   517.95), SIMDE_FLOAT32_C(   268.91), SIMDE_FLOAT32_C(   -89.87),
        SIMDE_FLOAT32_C(   814.40), SIMDE_FLOAT32_C(  -887.02), SIMDE_FLOAT32_C(   188.79), SIMDE_FLOAT32_C(   -41.15) },
      { SIMDE_FLOAT32_C(   200.70), SIMDE_FLOAT32_C(   794.85), SIMDE_FLOAT32_C(   706.41), SIMDE_FLOAT32_C(   428.58),
        SIMDE_FLOAT32_C(  1101.88), SIMDE_FLOAT32_C(   810.17), SIMDE_FLOAT32_C(   345.56), SIMDE_FLOAT32_C(   684.22),
        SIMDE_FLOAT32_C(   787.01), SIMDE_FLOAT32_C(   579.85), SIMDE_FLOAT32_C(   564.86), SIMDE_FLOAT32_C(   268.05),
        SIMDE_FLOAT32_C(   930.61), SIMDE_FLOAT32_C(  1225.50), SIMDE_FLOAT32_C(   804.23), SIMDE_FLOAT32_C(   156.98) } },
    { { SIMDE_FLOAT32_C(  -182.14), SIMDE_FLOAT32_C(  -858.58), SIMDE_FLOAT32_C(  -627.02), SIMDE_FLOAT32_C(  -573.76),
        SIMDE_FLOAT32_C(  -559.14), SIMDE_FLOAT32_C(    27.42), SIMDE_FLOAT32_C(   763.00), SIMDE_FLOAT32_C(   444.51),
        SIMDE_FLOAT32_C(   766.72), SIMDE_FLOAT32_C(  -733.74), SIMDE_FLOAT32_C(  -302.95), SIMDE_FLOAT32_C(  -683.60),
        SIMDE_FLOAT32_C(  -888.14), SIMDE_FLOAT32_C(  -521.19), SIMDE_FLOAT32_C(   467.89), SIMDE_FLOAT32_C(   251.19) },
      { SIMDE_FLOAT32_C(  -783.16), SIMDE_FLOAT32_C(   172.71), SIMDE_FLOAT32_C(  -638.42), SIMDE_FLOAT32_C(  -701.86),
        SIMDE_FLOAT32_C(  -420.37), SIMDE_FLOAT32_C(   359.83), SIMDE_FLOAT32_C(  -297.47), SIMDE_FLOAT32_C(  -207.37),
        SIMDE_FLOAT32_C(  -122.22), SIMDE_FLOAT32_C(   971.44), SIMDE_FLOAT32_C(   702.76), SIMDE_FLOAT32_C(  -307.82),
        SIMDE_FLOAT32_C(  -915.59), SIMDE_FLOAT32_C(  -108.45), SIMDE_FLOAT32_C(   651.04), SIMDE_FLOAT32_C(   -97.72) },
      { SIMDE_FLOAT32_C(   804.06), SIMDE_FLOAT32_C(   875.78), SIMDE_FLOAT32_C(   894.84), SIMDE_FLOAT32_C(   906.54),
        SIMDE_FLOAT32_C(   699.53), SIMDE_FLOAT32_C(   360.87), SIMDE_FLOAT32_C(   818.94), SIMDE_FLOAT32_C(   490.50),
        SIMDE_FLOAT32_C(   776.40), SIMDE_FLOAT32_C(  1217.40), SIMDE_FLOAT32_C(   765.28), SIMDE_FLOAT32_C(   749.71),
        SIMDE_FLOAT32_C(  1275.58), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   801.73), SIMDE_FLOAT32_C(   269.53) } },
    { { SIMDE_FLOAT32_C(    32.97), SIMDE_FLOAT32_C(  -975.98), SIMDE_FLOAT32_C(   328.52), SIMDE_FLOAT32_C(   473.84),
        SIMDE_FLOAT32_C(    51.43), SIMDE_FLOAT32_C(    91.52), SIMDE_FLOAT32_C(   -81.65), SIMDE_FLOAT32_C(  -181.85),
        SIMDE_FLOAT32_C(   357.78), SIMDE_FLOAT32_C(   615.40), SIMDE_FLOAT32_C(   134.55), SIMDE_FLOAT32_C(   469.64),
        SIMDE_FLOAT32_C(  -905.79), SIMDE_FLOAT32_C(  -397.56), SIMDE_FLOAT32_C(  -279.17), SIMDE_FLOAT32_C(  -688.95) },
      { SIMDE_FLOAT32_C(   775.15), SIMDE_FLOAT32_C(    82.41), SIMDE_FLOAT32_C(  -390.80), SIMDE_FLOAT32_C(  -645.22),
        SIMDE_FLOAT32_C(  -557.76), SIMDE_FLOAT32_C(   311.72), SIMDE_FLOAT32_C(   147.41), SIMDE_FLOAT32_C(   320.02),
        SIMDE_FLOAT32_C(   283.16), SIMDE_FLOAT32_C(  -149.83), SIMDE_FLOAT32_C(  -987.80), SIMDE_FLOAT32_C(   367.57),
        SIMDE_FLOAT32_C(   741.72), SIMDE_FLOAT32_C(   663.24), SIMDE_FLOAT32_C(  -730.15), SIMDE_FLOAT32_C(  -225.30) },
      { SIMDE_FLOAT32_C(   775.85), SIMDE_FLOAT32_C(   979.45), SIMDE_FLOAT32_C(   510.54), SIMDE_FLOAT32_C(   800.52),
        SIMDE_FLOAT32_C(   560.13), SIMDE_FLOAT32_C(   324.88), SIMDE_FLOAT32_C(   168.51), SIMDE_FLOAT32_C(   368.08),
        SIMDE_FLOAT32_C(   456.27), SIMDE_FLOAT32_C(   633.38), SIMDE_FLOAT32_C(   996.92), SIMDE_FLOAT32_C(   596.38),
        SIMDE_FLOAT32_C(  1170.73), SIMDE_FLOAT32_C(   773.27), SIMDE_FLOAT32_C(   781.70), SIMDE_FLOAT32_C(   724.85) } },
    { { SIMDE_FLOAT32_C(   687.25), SIMDE_FLOAT32_C(   598.37), SIMDE_FLOAT32_C(  -751.47), SIMDE_FLOAT32_C(  -261.32),
        SIMDE_FLOAT32_C(  -310.12), SIMDE_FLOAT32_C(   166.88), SIMDE_FLOAT32_C(   556.84), SIMDE_FLOAT32_C(  -952.33),
        SIMDE_FLOAT32_C(  -217.72), SIMDE_FLOAT32_C(  -308.61), SIMDE_FLOAT32_C(   517.31), SIMDE_FLOAT32_C(  -123.51),
        SIMDE_FLOAT32_C(   293.83), SIMDE_FLOAT32_C(  -761.86), SIMDE_FLOAT32_C(   187.55), SIMDE_FLOAT32_C(    68.99) },
      { SIMDE_FLOAT32_C(   320.55), SIMDE_FLOAT32_C(   796.74), SIMDE_FLOAT32_C(   423.77), SIMDE_FLOAT32_C(   762.79),
        SIMDE_FLOAT32_C(   108.47), SIMDE_FLOAT32_C(  -428.82), SIMDE_FLOAT32_C(    82.81), SIMDE_FLOAT32_C(  -608.37),
        SIMDE_FLOAT32_C(   421.35), SIMDE_FLOAT32_C(    95.01), SIMDE_FLOAT32_C(   759.20), SIMDE_FLOAT32_C(   163.07),
        SIMDE_FLOAT32_C(  -241.76), SIMDE_FLOAT32_C(  -970.95), SIMDE_FLOAT32_C(   937.77), SIMDE_FLOAT32_C(  -554.50) },
      { SIMDE_FLOAT32_C(   758.33), SIMDE_FLOAT32_C(   996.41), SIMDE_FLOAT32_C(   862.72), SIMDE_FLOAT32_C(   806.31),
        SIMDE_FLOAT32_C(   328.54), SIMDE_FLOAT32_C(   460.15), SIMDE_FLOAT32_C(   562.96), SIMDE_FLOAT32_C(  1130.06),
        SIMDE_FLOAT32_C(   474.28), SIMDE_FLOAT32_C(   322.90), SIMDE_FLOAT32_C(   918.69), SIMDE_FLOAT32_C(   204.56),
        SIMDE_FLOAT32_C(   380.50), SIMDE_FLOAT32_C(  1234.17), SIMDE_FLOAT32_C(   956.34), SIMDE_FLOAT32_C(   558.78) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 b = simde_mm512_loadu_ps(test_vec[i].b);
    simde__m512 r = simde_mm512_hypot_ps(a, b);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_hypot_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 b[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   367.16), SIMDE_FLOAT32_C(   534.12), SIMDE_FLOAT32_C(   840.53), SIMDE_FLOAT32_C(  -620.21),
        SIMDE_FLOAT32_C(   261.27), SIMDE_FLOAT32_C(   223.14), SIMDE_FLOAT32_C(  -163.58), SIMDE_FLOAT32_C(   267.96),
        SIMDE_FLOAT32_C(  -882.06), SIMDE_FLOAT32_C(  -703.87), SIMDE_FLOAT32_C(   527.51), SIMDE_FLOAT32_C(  -734.80),
        SIMDE_FLOAT32_C(  -828.23), SIMDE_FLOAT32_C(  -822.70), SIMDE_FLOAT32_C(  -911.73), SIMDE_FLOAT32_C(   856.22) },
      UINT8_C(182),
      { SIMDE_FLOAT32_C(   508.95), SIMDE_FLOAT32_C(   401.36), SIMDE_FLOAT32_C(  -896.06), SIMDE_FLOAT32_C(   773.16),
        SIMDE_FLOAT32_C(    -9.93), SIMDE_FLOAT32_C(  -389.05), SIMDE_FLOAT32_C(  -811.06), SIMDE_FLOAT32_C(   179.53),
        SIMDE_FLOAT32_C(  -842.09), SIMDE_FLOAT32_C(    34.81), SIMDE_FLOAT32_C(  -170.09), SIMDE_FLOAT32_C(   888.35),
        SIMDE_FLOAT32_C(  -467.85), SIMDE_FLOAT32_C(   381.00), SIMDE_FLOAT32_C(   255.51), SIMDE_FLOAT32_C(  -933.73) },
      { SIMDE_FLOAT32_C(   221.53), SIMDE_FLOAT32_C(   635.30), SIMDE_FLOAT32_C(   327.54), SIMDE_FLOAT32_C(  -555.33),
        SIMDE_FLOAT32_C(  -528.28), SIMDE_FLOAT32_C(  -404.50), SIMDE_FLOAT32_C(  -437.39), SIMDE_FLOAT32_C(  -232.15),
        SIMDE_FLOAT32_C(  -876.99), SIMDE_FLOAT32_C(  -172.19), SIMDE_FLOAT32_C(   -60.39), SIMDE_FLOAT32_C(  -699.69),
        SIMDE_FLOAT32_C(   -83.92), SIMDE_FLOAT32_C(  -204.17), SIMDE_FLOAT32_C(   701.45), SIMDE_FLOAT32_C(  -574.97) },
      { SIMDE_FLOAT32_C(   367.16), SIMDE_FLOAT32_C(   751.46), SIMDE_FLOAT32_C(   954.05), SIMDE_FLOAT32_C(  -620.21),
        SIMDE_FLOAT32_C(   528.37), SIMDE_FLOAT32_C(   561.23), SIMDE_FLOAT32_C(  -163.58), SIMDE_FLOAT32_C(   293.47),
        SIMDE_FLOAT32_C(  -882.06), SIMDE_FLOAT32_C(  -703.87), SIMDE_FLOAT32_C(   527.51), SIMDE_FLOAT32_C(  -734.80),
        SIMDE_FLOAT32_C(  -828.23), SIMDE_FLOAT32_C(  -822.70), SIMDE_FLOAT32_C(  -911.73), SIMDE_FLOAT32_C(   856.22) } },
    { { SIMDE_FLOAT32_C(  -802.80), SIMDE_FLOAT32_C(   805.39), SIMDE_FLOAT32_C(  -801.81), SIMDE_FLOAT32_C(   187.27),
        SIMDE_FLOAT32_C(  -583.65), SIMDE_FLOAT32_C(  -612.87), SIMDE_FLOAT32_C(  -633.20), SIMDE_FLOAT32_C(  -425.74),
        SIMDE_FLOAT32_C(   421.94), SIMDE_FLOAT32_C(   196.71), SIMDE_FLOAT32_C(  -537.40), SIMDE_FLOAT32_C(   954.08),
        SIMDE_FLOAT32_C(  -422.29), SIMDE_FLOAT32_C(   718.11), SIMDE_FLOAT32_C(  -979.65), SIMDE_FLOAT32_C(   799.24) },
      UINT8_C(  1),
      { SIMDE_FLOAT32_C(   347.90), SIMDE_FLOAT32_C(  -756.09), SIMDE_FLOAT32_C(   825.13), SIMDE_FLOAT32_C(   943.40),
        SIMDE_FLOAT32_C(  -193.47), SIMDE_FLOAT32_C(  -407.03), SIMDE_FLOAT32_C(  -933.59), SIMDE_FLOAT32_C(   634.34),
        SIMDE_FLOAT32_C(   532.59), SIMDE_FLOAT32_C(  -633.28), SIMDE_FLOAT32_C(  -449.58), SIMDE_FLOAT32_C(  -671.58),
        SIMDE_FLOAT32_C(  -931.83), SIMDE_FLOAT32_C(   -24.55), SIMDE_FLOAT32_C(  -474.38), SIMDE_FLOAT32_C(   873.57) },
      { SIMDE_FLOAT32_C(   173.64), SIMDE_FLOAT32_C(   712.89), SIMDE_FLOAT32_C(  -710.09), SIMDE_FLOAT32_C(   560.77),
        SIMDE_FLOAT32_C(  -920.31), SIMDE_FLOAT32_C(  -135.83), SIMDE_FLOAT32_C(   -17.30), SIMDE_FLOAT32_C(   276.39),
        SIMDE_FLOAT32_C(   326.78), SIMDE_FLOAT32_C(   -63.21), SIMDE_FLOAT32_C(   854.10), SIMDE_FLOAT32_C(    44.89),
        SIMDE_FLOAT32_C(   -42.86), SIMDE_FLOAT32_C(   653.34), SIMDE_FLOAT32_C(  -601.70), SIMDE_FLOAT32_C(  -694.96) },
      { SIMDE_FLOAT32_C(   388.83), SIMDE_FLOAT32_C(   805.39), SIMDE_FLOAT32_C(  -801.81), SIMDE_FLOAT32_C(   187.27),
        SIMDE_FLOAT32_C(  -583.65), SIMDE_FLOAT32_C(  -612.87), SIMDE_FLOAT32_C(  -633.20), SIMDE_FLOAT32_C(  -425.74),
        SIMDE_FLOAT32_C(   421.94), SIMDE_FLOAT32_C(   196.71), SIMDE_FLOAT32_C(  -537.40), SIMDE_FLOAT32_C(   954.08),
        SIMDE_FLOAT32_C(  -422.29), SIMDE_FLOAT32_C(   718.11), SIMDE_FLOAT32_C(  -979.65), SIMDE_FLOAT32_C(   799.24) } },
    { { SIMDE_FLOAT32_C(   897.26), SIMDE_FLOAT32_C(  -776.57), SIMDE_FLOAT32_C(  -751.56), SIMDE_FLOAT32_C(  -296.22),
        SIMDE_FLOAT32_C(  -183.60), SIMDE_FLOAT32_C(  -685.15), SIMDE_FLOAT32_C(  -661.88), SIMDE_FLOAT32_C(  -651.01),
        SIMDE_FLOAT32_C(  -318.42), SIMDE_FLOAT32_C(  -111.46), SIMDE_FLOAT32_C(  -322.60), SIMDE_FLOAT32_C(  -250.25),
        SIMDE_FLOAT32_C(   863.99), SIMDE_FLOAT32_C(   203.02), SIMDE_FLOAT32_C(  -376.68), SIMDE_FLOAT32_C(    37.62) },
      UINT8_C( 54),
      { SIMDE_FLOAT32_C(   -86.77), SIMDE_FLOAT32_C(  -401.61), SIMDE_FLOAT32_C(    -4.41), SIMDE_FLOAT32_C(   777.40),
        SIMDE_FLOAT32_C(   581.09), SIMDE_FLOAT32_C(  -728.01), SIMDE_FLOAT32_C(   104.18), SIMDE_FLOAT32_C(  -482.12),
        SIMDE_FLOAT32_C(  -873.91), SIMDE_FLOAT32_C(  -850.93), SIMDE_FLOAT32_C(   475.02), SIMDE_FLOAT32_C(   779.43),
        SIMDE_FLOAT32_C(  -452.63), SIMDE_FLOAT32_C(   780.06), SIMDE_FLOAT32_C(   676.69), SIMDE_FLOAT32_C(  -229.20) },
      { SIMDE_FLOAT32_C(  -971.50), SIMDE_FLOAT32_C(  -619.53), SIMDE_FLOAT32_C(   587.20), SIMDE_FLOAT32_C(  -656.65),
        SIMDE_FLOAT32_C(  -281.40), SIMDE_FLOAT32_C(   936.19), SIMDE_FLOAT32_C(    24.93), SIMDE_FLOAT32_C(   607.14),
        SIMDE_FLOAT32_C(  -386.41), SIMDE_FLOAT32_C(   774.68), SIMDE_FLOAT32_C(   471.12), SIMDE_FLOAT32_C(   816.61),
        SIMDE_FLOAT32_C(  -602.00), SIMDE_FLOAT32_C(  -491.25), SIMDE_FLOAT32_C(  -267.48), SIMDE_FLOAT32_C(   311.23) },
      { SIMDE_FLOAT32_C(   897.26), SIMDE_FLOAT32_C(   738.31), SIMDE_FLOAT32_C(   587.22), SIMDE_FLOAT32_C(  -296.22),
        SIMDE_FLOAT32_C(   645.64), SIMDE_FLOAT32_C(  1185.94), SIMDE_FLOAT32_C(  -661.88), SIMDE_FLOAT32_C(  -651.01),
        SIMDE_FLOAT32_C(  -318.42), SIMDE_FLOAT32_C(  -111.46), SIMDE_FLOAT32_C(  -322.60), SIMDE_FLOAT32_C(  -250.25),
        SIMDE_FLOAT32_C(   863.99), SIMDE_FLOAT32_C(   203.02), SIMDE_FLOAT32_C(  -376.68), SIMDE_FLOAT32_C(    37.62) } },
    { { SIMDE_FLOAT32_C(   107.14), SIMDE_FLOAT32_C(   728.11), SIMDE_FLOAT32_C(    88.63), SIMDE_FLOAT32_C(  -311.77),
        SIMDE_FLOAT32_C(  -999.90), SIMDE_FLOAT32_C(  -807.18), SIMDE_FLOAT32_C(   206.11), SIMDE_FLOAT32_C(  -873.82),
        SIMDE_FLOAT32_C(  -658.11), SIMDE_FLOAT32_C(  -318.87), SIMDE_FLOAT32_C(   905.61), SIMDE_FLOAT32_C(  -110.74),
        SIMDE_FLOAT32_C(  -538.82), SIMDE_FLOAT32_C(   582.30), SIMDE_FLOAT32_C(   660.06), SIMDE_FLOAT32_C(  -510.32) },
      UINT8_C(112),
      { SIMDE_FLOAT32_C(   247.26), SIMDE_FLOAT32_C(  -166.97), SIMDE_FLOAT32_C(  -318.63), SIMDE_FLOAT32_C(   183.45),
        SIMDE_FLOAT32_C(   857.96), SIMDE_FLOAT32_C(  -711.49), SIMDE_FLOAT32_C(   797.04), SIMDE_FLOAT32_C(   632.64),
        SIMDE_FLOAT32_C(   759.63), SIMDE_FLOAT32_C(   613.65), SIMDE_FLOAT32_C(  -969.36), SIMDE_FLOAT32_C(  -731.62),
        SIMDE_FLOAT32_C(  -653.84), SIMDE_FLOAT32_C(   341.87), SIMDE_FLOAT32_C(   375.52), SIMDE_FLOAT32_C(  -925.73) },
      { SIMDE_FLOAT32_C(  -569.50), SIMDE_FLOAT32_C(  -936.25), SIMDE_FLOAT32_C(  -925.63), SIMDE_FLOAT32_C(  -376.68),
        SIMDE_FLOAT32_C(   269.87), SIMDE_FLOAT32_C(  -799.45), SIMDE_FLOAT32_C(   -34.80), SIMDE_FLOAT32_C(   950.99),
        SIMDE_FLOAT32_C(  -893.84), SIMDE_FLOAT32_C(   854.47), SIMDE_FLOAT32_C(  -587.82), SIMDE_FLOAT32_C(   688.47),
        SIMDE_FLOAT32_C(   514.53), SIMDE_FLOAT32_C(   -98.14), SIMDE_FLOAT32_C(   651.24), SIMDE_FLOAT32_C(  -238.21) },
      { SIMDE_FLOAT32_C(   107.14), SIMDE_FLOAT32_C(   728.11), SIMDE_FLOAT32_C(    88.63), SIMDE_FLOAT32_C(  -311.77),
        SIMDE_FLOAT32_C(   899.40), SIMDE_FLOAT32_C(  1070.20), SIMDE_FLOAT32_C(   797.80), SIMDE_FLOAT32_C(  -873.82),
        SIMDE_FLOAT32_C(  -658.11), SIMDE_FLOAT32_C(  -318.87), SIMDE_FLOAT32_C(   905.61), SIMDE_FLOAT32_C(  -110.74),
        SIMDE_FLOAT32_C(  -538.82), SIMDE_FLOAT32_C(   582.30), SIMDE_FLOAT32_C(   660.06), SIMDE_FLOAT32_C(  -510.32) } },
    { { SIMDE_FLOAT32_C(   734.89), SIMDE_FLOAT32_C(  -667.39), SIMDE_FLOAT32_C(   945.23), SIMDE_FLOAT32_C(   592.85),
        SIMDE_FLOAT32_C(  -378.88), SIMDE_FLOAT32_C(   742.27), SIMDE_FLOAT32_C(   225.49), SIMDE_FLOAT32_C(  -619.25),
        SIMDE_FLOAT32_C(   355.91), SIMDE_FLOAT32_C(   256.12), SIMDE_FLOAT32_C(  -350.87), SIMDE_FLOAT32_C(   702.07),
        SIMDE_FLOAT32_C(  -402.01), SIMDE_FLOAT32_C(  -975.35), SIMDE_FLOAT32_C(   776.35), SIMDE_FLOAT32_C(    28.49) },
      UINT8_C( 29),
      { SIMDE_FLOAT32_C(   850.71), SIMDE_FLOAT32_C(   651.81), SIMDE_FLOAT32_C(   358.27), SIMDE_FLOAT32_C(  -948.74),
        SIMDE_FLOAT32_C(  -382.99), SIMDE_FLOAT32_C(   309.27), SIMDE_FLOAT32_C(  -842.57), SIMDE_FLOAT32_C(  -528.52),
        SIMDE_FLOAT32_C(   721.45), SIMDE_FLOAT32_C(   845.89), SIMDE_FLOAT32_C(   986.00), SIMDE_FLOAT32_C(  -376.69),
        SIMDE_FLOAT32_C(   497.14), SIMDE_FLOAT32_C(  -252.21), SIMDE_FLOAT32_C(  -641.80), SIMDE_FLOAT32_C(   829.75) },
      { SIMDE_FLOAT32_C(  -306.98), SIMDE_FLOAT32_C(   951.05), SIMDE_FLOAT32_C(  -549.13), SIMDE_FLOAT32_C(  -564.71),
        SIMDE_FLOAT32_C(   176.53), SIMDE_FLOAT32_C(  -168.38), SIMDE_FLOAT32_C(   791.20), SIMDE_FLOAT32_C(  -567.34),
        SIMDE_FLOAT32_C(   480.75), SIMDE_FLOAT32_C(   493.27), SIMDE_FLOAT32_C(    30.65), SIMDE_FLOAT32_C(   505.41),
        SIMDE_FLOAT32_C(   269.62), SIMDE_FLOAT32_C(  -940.86), SIMDE_FLOAT32_C(   593.82), SIMDE_FLOAT32_C(   120.33) },
      { SIMDE_FLOAT32_C(   904.40), SIMDE_FLOAT32_C(  -667.39), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(  1104.09),
        SIMDE_FLOAT32_C(   421.72), SIMDE_FLOAT32_C(   742.27), SIMDE_FLOAT32_C(   225.49), SIMDE_FLOAT32_C(  -619.25),
        SIMDE_FLOAT32_C(   355.91), SIMDE_FLOAT32_C(   256.12), SIMDE_FLOAT32_C(  -350.87), SIMDE_FLOAT32_C(   702.07),
        SIMDE_FLOAT32_C(  -402.01), SIMDE_FLOAT32_C(  -975.35), SIMDE_FLOAT32_C(   776.35), SIMDE_FLOAT32_C(    28.49) } },
    { { SIMDE_FLOAT32_C(   710.95), SIMDE_FLOAT32_C(   -47.91), SIMDE_FLOAT32_C(   171.59), SIMDE_FLOAT32_C(  -672.04),
        SIMDE_FLOAT32_C(  -738.64), SIMDE_FLOAT32_C(   329.02), SIMDE_FLOAT32_C(  -200.57), SIMDE_FLOAT32_C(   982.81),
        SIMDE_FLOAT32_C(   174.91), SIMDE_FLOAT32_C(  -214.56), SIMDE_FLOAT32_C(  -393.88), SIMDE_FLOAT32_C(  -327.95),
        SIMDE_FLOAT32_C(   533.22), SIMDE_FLOAT32_C(   -35.69), SIMDE_FLOAT32_C(  -498.20), SIMDE_FLOAT32_C(  -773.76) },
      UINT8_C(210),
      { SIMDE_FLOAT32_C(   -47.34), SIMDE_FLOAT32_C(  -338.47), SIMDE_FLOAT32_C(  -908.10), SIMDE_FLOAT32_C(   784.28),
        SIMDE_FLOAT32_C(  -547.27), SIMDE_FLOAT32_C(  -475.45), SIMDE_FLOAT32_C(   265.03), SIMDE_FLOAT32_C(   946.00),
        SIMDE_FLOAT32_C(   555.20), SIMDE_FLOAT32_C(  -229.56), SIMDE_FLOAT32_C(   215.62), SIMDE_FLOAT32_C(   614.34),
        SIMDE_FLOAT32_C(  -635.74), SIMDE_FLOAT32_C(  -664.05), SIMDE_FLOAT32_C(   325.29), SIMDE_FLOAT32_C(   316.35) },
      { SIMDE_FLOAT32_C(   507.54), SIMDE_FLOAT32_C(   653.24), SIMDE_FLOAT32_C(   577.71), SIMDE_FLOAT32_C(  -163.44),
        SIMDE_FLOAT32_C(  -547.32), SIMDE_FLOAT32_C(   560.52), SIMDE_FLOAT32_C(  -988.53), SIMDE_FLOAT32_C(   238.11),
        SIMDE_FLOAT32_C(  -833.36), SIMDE_FLOAT32_C(  -316.48), SIMDE_FLOAT32_C(  -228.66), SIMDE_FLOAT32_C(   130.95),
        SIMDE_FLOAT32_C(   185.32), SIMDE_FLOAT32_C(    -2.42), SIMDE_FLOAT32_C(  -953.69), SIMDE_FLOAT32_C(  -862.02) },
      { SIMDE_FLOAT32_C(   710.95), SIMDE_FLOAT32_C(   735.72), SIMDE_FLOAT32_C(   171.59), SIMDE_FLOAT32_C(  -672.04),
        SIMDE_FLOAT32_C(   773.99), SIMDE_FLOAT32_C(   329.02), SIMDE_FLOAT32_C(  1023.44), SIMDE_FLOAT32_C(   975.51),
        SIMDE_FLOAT32_C(   174.91), SIMDE_FLOAT32_C(  -214.56), SIMDE_FLOAT32_C(  -393.88), SIMDE_FLOAT32_C(  -327.95),
        SIMDE_FLOAT32_C(   533.22), SIMDE_FLOAT32_C(   -35.69), SIMDE_FLOAT32_C(  -498.20), SIMDE_FLOAT32_C(  -773.76) } },
    { { SIMDE_FLOAT32_C(   659.11), SIMDE_FLOAT32_C(  -861.79), SIMDE_FLOAT32_C(   922.26), SIMDE_FLOAT32_C(  -888.16),
        SIMDE_FLOAT32_C(  -337.24), SIMDE_FLOAT32_C(   187.30), SIMDE_FLOAT32_C(  -942.16), SIMDE_FLOAT32_C(  -782.04),
        SIMDE_FLOAT32_C(   957.74), SIMDE_FLOAT32_C(   273.45), SIMDE_FLOAT32_C(   832.30), SIMDE_FLOAT32_C(  -678.00),
        SIMDE_FLOAT32_C(   609.40), SIMDE_FLOAT32_C(   157.59), SIMDE_FLOAT32_C(   638.35), SIMDE_FLOAT32_C(   116.94) },
      UINT8_C(122),
      { SIMDE_FLOAT32_C(   216.06), SIMDE_FLOAT32_C(   953.51), SIMDE_FLOAT32_C(   263.51), SIMDE_FLOAT32_C(  -223.42),
        SIMDE_FLOAT32_C(   964.98), SIMDE_FLOAT32_C(  -498.37), SIMDE_FLOAT32_C(   -56.78), SIMDE_FLOAT32_C(  -351.50),
        SIMDE_FLOAT32_C(   272.97), SIMDE_FLOAT32_C(  -925.83), SIMDE_FLOAT32_C(   833.82), SIMDE_FLOAT32_C(  -729.45),
        SIMDE_FLOAT32_C(  -879.52), SIMDE_FLOAT32_C(   971.80), SIMDE_FLOAT32_C(   929.66), SIMDE_FLOAT32_C(  -741.31) },
      { SIMDE_FLOAT32_C(   894.07), SIMDE_FLOAT32_C(  -958.51), SIMDE_FLOAT32_C(   -78.55), SIMDE_FLOAT32_C(    81.37),
        SIMDE_FLOAT32_C(  -900.67), SIMDE_FLOAT32_C(   139.42), SIMDE_FLOAT32_C(    39.11), SIMDE_FLOAT32_C(   372.78),
        SIMDE_FLOAT32_C(   -28.28), SIMDE_FLOAT32_C(   361.11), SIMDE_FLOAT32_C(   -17.81), SIMDE_FLOAT32_C(  -870.69),
        SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(  -900.87), SIMDE_FLOAT32_C(   -59.85), SIMDE_FLOAT32_C(  -784.48) },
      { SIMDE_FLOAT32_C(   659.11), SIMDE_FLOAT32_C(  1352.01), SIMDE_FLOAT32_C(   922.26), SIMDE_FLOAT32_C(   237.78),
        SIMDE_FLOAT32_C(  1320.00), SIMDE_FLOAT32_C(   517.50), SIMDE_FLOAT32_C(    68.95), SIMDE_FLOAT32_C(  -782.04),
        SIMDE_FLOAT32_C(   957.74), SIMDE_FLOAT32_C(   273.45), SIMDE_FLOAT32_C(   832.30), SIMDE_FLOAT32_C(  -678.00),
        SIMDE_FLOAT32_C(   609.40), SIMDE_FLOAT32_C(   157.59), SIMDE_FLOAT32_C(   638.35), SIMDE_FLOAT32_C(   116.94) } },
    { { SIMDE_FLOAT32_C(  -947.37), SIMDE_FLOAT32_C(  -796.34), SIMDE_FLOAT32_C(    -7.90), SIMDE_FLOAT32_C(  -982.39),
        SIMDE_FLOAT32_C(  -294.71), SIMDE_FLOAT32_C(   935.32), SIMDE_FLOAT32_C(  -333.88), SIMDE_FLOAT32_C(   978.25),
        SIMDE_FLOAT32_C(  -990.51), SIMDE_FLOAT32_C(  -500.06), SIMDE_FLOAT32_C(  -751.20), SIMDE_FLOAT32_C(  -870.03),
        SIMDE_FLOAT32_C(  -528.26), SIMDE_FLOAT32_C(  -821.55), SIMDE_FLOAT32_C(  -611.34), SIMDE_FLOAT32_C(  -634.19) },
      UINT8_C(234),
      { SIMDE_FLOAT32_C(   310.12), SIMDE_FLOAT32_C(   447.18), SIMDE_FLOAT32_C(  -680.72), SIMDE_FLOAT32_C(  -550.47),
        SIMDE_FLOAT32_C(  -513.72), SIMDE_FLOAT32_C(   692.06), SIMDE_FLOAT32_C(   421.25), SIMDE_FLOAT32_C(   847.39),
        SIMDE_FLOAT32_C(  -325.76), SIMDE_FLOAT32_C(   550.57), SIMDE_FLOAT32_C(  -153.15), SIMDE_FLOAT32_C(  -226.63),
        SIMDE_FLOAT32_C(  -509.29), SIMDE_FLOAT32_C(    62.37), SIMDE_FLOAT32_C(  -173.99), SIMDE_FLOAT32_C(  -305.63) },
      { SIMDE_FLOAT32_C(  -945.53), SIMDE_FLOAT32_C(  -156.38), SIMDE_FLOAT32_C(   399.66), SIMDE_FLOAT32_C(   989.79),
        SIMDE_FLOAT32_C(   509.74), SIMDE_FLOAT32_C(   377.91), SIMDE_FLOAT32_C(   999.28), SIMDE_FLOAT32_C(  -990.32),
        SIMDE_FLOAT32_C(   626.71), SIMDE_FLOAT32_C(  -870.75), SIMDE_FLOAT32_C(  -518.58), SIMDE_FLOAT32_C(   805.16),
        SIMDE_FLOAT32_C(  -482.08), SIMDE_FLOAT32_C(  -152.77), SIMDE_FLOAT32_C(  -974.89), SIMDE_FLOAT32_C(   828.03) },
      { SIMDE_FLOAT32_C(  -947.37), SIMDE_FLOAT32_C(   473.73), SIMDE_FLOAT32_C(    -7.90), SIMDE_FLOAT32_C(  1132.56),
        SIMDE_FLOAT32_C(  -294.71), SIMDE_FLOAT32_C(   788.52), SIMDE_FLOAT32_C(  1084.44), SIMDE_FLOAT32_C(  1303.38),
        SIMDE_FLOAT32_C(  -990.51), SIMDE_FLOAT32_C(  -500.06), SIMDE_FLOAT32_C(  -751.20), SIMDE_FLOAT32_C(  -870.03),
        SIMDE_FLOAT32_C(  -528.26), SIMDE_FLOAT32_C(  -821.55), SIMDE_FLOAT32_C(  -611.34), SIMDE_FLOAT32_C(  -634.19) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 b = simde_mm512_loadu_ps(test_vec[i].b);
    simde__m512 r = simde_mm512_mask_hypot_ps(src, test_vec[i].k, a, b);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_hypot_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 b[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   275.20), SIMDE_FLOAT64_C(   366.24), SIMDE_FLOAT64_C(   966.11), SIMDE_FLOAT64_C(  -937.96),
        SIMDE_FLOAT64_C(   570.22), SIMDE_FLOAT64_C(    -7.21), SIMDE_FLOAT64_C(   612.58), SIMDE_FLOAT64_C(  -184.69) },
      { SIMDE_FLOAT64_C(  -503.58), SIMDE_FLOAT64_C(   256.83), SIMDE_FLOAT64_C(    80.98), SIMDE_FLOAT64_C(  -364.25),
        SIMDE_FLOAT64_C(   598.02), SIMDE_FLOAT64_C(  -961.08), SIMDE_FLOAT64_C(   560.19), SIMDE_FLOAT64_C(  -553.76) },
      { SIMDE_FLOAT64_C(   573.87), SIMDE_FLOAT64_C(   447.32), SIMDE_FLOAT64_C(   969.50), SIMDE_FLOAT64_C(  1006.20),
        SIMDE_FLOAT64_C(   826.30), SIMDE_FLOAT64_C(   961.11), SIMDE_FLOAT64_C(   830.10), SIMDE_FLOAT64_C(   583.75) } },
    { { SIMDE_FLOAT64_C(  -373.20), SIMDE_FLOAT64_C(   625.48), SIMDE_FLOAT64_C(   871.64), SIMDE_FLOAT64_C(  -503.55),
        SIMDE_FLOAT64_C(  -900.28), SIMDE_FLOAT64_C(    58.59), SIMDE_FLOAT64_C(  -493.99), SIMDE_FLOAT64_C(   103.21) },
      { SIMDE_FLOAT64_C(   916.41), SIMDE_FLOAT64_C(    70.36), SIMDE_FLOAT64_C(  -720.02), SIMDE_FLOAT64_C(  -164.66),
        SIMDE_FLOAT64_C(   487.58), SIMDE_FLOAT64_C(  -677.71), SIMDE_FLOAT64_C(  -865.62), SIMDE_FLOAT64_C(  -237.21) },
      { SIMDE_FLOAT64_C(   989.49), SIMDE_FLOAT64_C(   629.42), SIMDE_FLOAT64_C(  1130.57), SIMDE_FLOAT64_C(   529.79),
        SIMDE_FLOAT64_C(  1023.84), SIMDE_FLOAT64_C(   680.24), SIMDE_FLOAT64_C(   996.66), SIMDE_FLOAT64_C(   258.69) } },
    { { SIMDE_FLOAT64_C(   688.53), SIMDE_FLOAT64_C(  -899.51), SIMDE_FLOAT64_C(  -175.18), SIMDE_FLOAT64_C(   258.75),
        SIMDE_FLOAT64_C(    93.28), SIMDE_FLOAT64_C(  -562.60), SIMDE_FLOAT64_C(  -925.94), SIMDE_FLOAT64_C(   589.69) },
      { SIMDE_FLOAT64_C(   694.23), SIMDE_FLOAT64_C(   155.04), SIMDE_FLOAT64_C(  -774.56), SIMDE_FLOAT64_C(   292.25),
        SIMDE_FLOAT64_C(   193.96), SIMDE_FLOAT64_C(   785.64), SIMDE_FLOAT64_C(   738.49), SIMDE_FLOAT64_C(   820.76) },
      { SIMDE_FLOAT64_C(   977.77), SIMDE_FLOAT64_C(   912.77), SIMDE_FLOAT64_C(   794.12), SIMDE_FLOAT64_C(   390.34),
        SIMDE_FLOAT64_C(   215.22), SIMDE_FLOAT64_C(   966.31), SIMDE_FLOAT64_C(  1184.37), SIMDE_FLOAT64_C(  1010.63) } },
    { { SIMDE_FLOAT64_C(   411.12), SIMDE_FLOAT64_C(   610.13), SIMDE_FLOAT64_C(  -682.79), SIMDE_FLOAT64_C(   510.84),
        SIMDE_FLOAT64_C(  -331.28), SIMDE_FLOAT64_C(  -176.78), SIMDE_FLOAT64_C(  -385.95), SIMDE_FLOAT64_C(  -414.87) },
      { SIMDE_FLOAT64_C(   893.58), SIMDE_FLOAT64_C(  -105.97), SIMDE_FLOAT64_C(   420.47), SIMDE_FLOAT64_C(   381.16),
        SIMDE_FLOAT64_C(   216.32), SIMDE_FLOAT64_C(   554.85), SIMDE_FLOAT64_C(  -856.05), SIMDE_FLOAT64_C(   -95.14) },
      { SIMDE_FLOAT64_C(   983.62), SIMDE_FLOAT64_C(   619.26), SIMDE_FLOAT64_C(   801.87), SIMDE_FLOAT64_C(   637.37),
        SIMDE_FLOAT64_C(   395.65), SIMDE_FLOAT64_C(   582.33), SIMDE_FLOAT64_C(   939.03), SIMDE_FLOAT64_C(   425.64) } },
    { { SIMDE_FLOAT64_C(   655.34), SIMDE_FLOAT64_C(   -31.23), SIMDE_FLOAT64_C(  -836.39), SIMDE_FLOAT64_C(  -251.38),
        SIMDE_FLOAT64_C(   406.17), SIMDE_FLOAT64_C(  -762.33), SIMDE_FLOAT64_C(  -661.69), SIMDE_FLOAT64_C(   100.40) },
      { SIMDE_FLOAT64_C(   392.71), SIMDE_FLOAT64_C(  -436.24), SIMDE_FLOAT64_C(  -607.35), SIMDE_FLOAT64_C(  -413.33),
        SIMDE_FLOAT64_C(  -650.61), SIMDE_FLOAT64_C(  -868.86), SIMDE_FLOAT64_C(  -592.57), SIMDE_FLOAT64_C(   760.51) },
      { SIMDE_FLOAT64_C(   764.00), SIMDE_FLOAT64_C(   437.36), SIMDE_FLOAT64_C(  1033.65), SIMDE_FLOAT64_C(   483.77),
        SIMDE_FLOAT64_C(   766.99), SIMDE_FLOAT64_C(  1155.88), SIMDE_FLOAT64_C(   888.24), SIMDE_FLOAT64_C(   767.11) } },
    { { SIMDE_FLOAT64_C(   741.27), SIMDE_FLOAT64_C(  -275.37), SIMDE_FLOAT64_C(   271.35), SIMDE_FLOAT64_C(  -590.01),
        SIMDE_FLOAT64_C(   547.85), SIMDE_FLOAT64_C(   885.41), SIMDE_FLOAT64_C(    -4.88), SIMDE_FLOAT64_C(   441.42) },
      { SIMDE_FLOAT64_C(  -220.56), SIMDE_FLOAT64_C(  -584.41), SIMDE_FLOAT64_C(  -177.42), SIMDE_FLOAT64_C(   995.76),
        SIMDE_FLOAT64_C(   970.44), SIMDE_FLOAT64_C(   -33.47), SIMDE_FLOAT64_C(   -99.38), SIMDE_FLOAT64_C(   625.78) },
      { SIMDE_FLOAT64_C(   773.39), SIMDE_FLOAT64_C(   646.04), SIMDE_FLOAT64_C(   324.20), SIMDE_FLOAT64_C(  1157.43),
        SIMDE_FLOAT64_C(  1114.40), SIMDE_FLOAT64_C(   886.04), SIMDE_FLOAT64_C(    99.50), SIMDE_FLOAT64_C(   765.80) } },
    { { SIMDE_FLOAT64_C(   935.30), SIMDE_FLOAT64_C(    64.23), SIMDE_FLOAT64_C(  -625.60), SIMDE_FLOAT64_C(   341.47),
        SIMDE_FLOAT64_C(   301.89), SIMDE_FLOAT64_C(  -287.29), SIMDE_FLOAT64_C(  -558.13), SIMDE_FLOAT64_C(  -305.40) },
      { SIMDE_FLOAT64_C(   276.47), SIMDE_FLOAT64_C(  -165.48), SIMDE_FLOAT64_C(   281.27), SIMDE_FLOAT64_C(   625.86),
        SIMDE_FLOAT64_C(   -34.34), SIMDE_FLOAT64_C(   688.70), SIMDE_FLOAT64_C(   386.37), SIMDE_FLOAT64_C(  -293.08) },
      { SIMDE_FLOAT64_C(   975.31), SIMDE_FLOAT64_C(   177.51), SIMDE_FLOAT64_C(   685.92), SIMDE_FLOAT64_C(   712.95),
        SIMDE_FLOAT64_C(   303.84), SIMDE_FLOAT64_C(   746.22), SIMDE_FLOAT64_C(   678.82), SIMDE_FLOAT64_C(   423.28) } },
    { { SIMDE_FLOAT64_C(  -586.67), SIMDE_FLOAT64_C(  -342.28), SIMDE_FLOAT64_C(   116.91), SIMDE_FLOAT64_C(   961.18),
        SIMDE_FLOAT64_C(  -456.87), SIMDE_FLOAT64_C(  -887.97), SIMDE_FLOAT64_C(   402.60), SIMDE_FLOAT64_C(   322.57) },
      { SIMDE_FLOAT64_C(  -472.39), SIMDE_FLOAT64_C(  -774.82), SIMDE_FLOAT64_C(   318.33), SIMDE_FLOAT64_C(  -501.95),
        SIMDE_FLOAT64_C(   191.71), SIMDE_FLOAT64_C(  -781.04), SIMDE_FLOAT64_C(  -876.17), SIMDE_FLOAT64_C(   127.01) },
      { SIMDE_FLOAT64_C(   753.22), SIMDE_FLOAT64_C(   847.05), SIMDE_FLOAT64_C(   339.12), SIMDE_FLOAT64_C(  1084.35),
        SIMDE_FLOAT64_C(   495.46), SIMDE_FLOAT64_C(  1182.59), SIMDE_FLOAT64_C(   964.24), SIMDE_FLOAT64_C(   346.67) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d b = simde_mm512_loadu_pd(test_vec[i].b);
    simde__m512d r = simde_mm512_hypot_pd(a, b);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_hypot_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 b[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -431.95), SIMDE_FLOAT64_C(  -237.84), SIMDE_FLOAT64_C(   748.51), SIMDE_FLOAT64_C(   841.10),
        SIMDE_FLOAT64_C(  -673.54), SIMDE_FLOAT64_C(   668.62), SIMDE_FLOAT64_C(   514.70), SIMDE_FLOAT64_C(  -656.78) },
      UINT8_C(201),
      { SIMDE_FLOAT64_C(   160.07), SIMDE_FLOAT64_C(  -729.81), SIMDE_FLOAT64_C(   -33.18), SIMDE_FLOAT64_C(   130.28),
        SIMDE_FLOAT64_C(   345.30), SIMDE_FLOAT64_C(  -333.34), SIMDE_FLOAT64_C(  -285.62), SIMDE_FLOAT64_C(  -843.08) },
      { SIMDE_FLOAT64_C(  -705.31), SIMDE_FLOAT64_C(  -528.34), SIMDE_FLOAT64_C(   222.02), SIMDE_FLOAT64_C(  -760.66),
        SIMDE_FLOAT64_C(  -344.72), SIMDE_FLOAT64_C(  -209.64), SIMDE_FLOAT64_C(  -687.68), SIMDE_FLOAT64_C(    52.34) },
      { SIMDE_FLOAT64_C(   723.25), SIMDE_FLOAT64_C(  -237.84), SIMDE_FLOAT64_C(   748.51), SIMDE_FLOAT64_C(   771.74),
        SIMDE_FLOAT64_C(  -673.54), SIMDE_FLOAT64_C(   668.62), SIMDE_FLOAT64_C(   744.64), SIMDE_FLOAT64_C(   844.70) } },
    { { SIMDE_FLOAT64_C(   859.76), SIMDE_FLOAT64_C(   134.54), SIMDE_FLOAT64_C(  -771.62), SIMDE_FLOAT64_C(  -408.76),
        SIMDE_FLOAT64_C(   106.34), SIMDE_FLOAT64_C(  -575.90), SIMDE_FLOAT64_C(   159.29), SIMDE_FLOAT64_C(   868.50) },
      UINT8_C(223),
      { SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(  -805.04), SIMDE_FLOAT64_C(   841.23), SIMDE_FLOAT64_C(  -484.91),
        SIMDE_FLOAT64_C(  -461.82), SIMDE_FLOAT64_C(   403.45), SIMDE_FLOAT64_C(   675.17), SIMDE_FLOAT64_C(  -191.63) },
      { SIMDE_FLOAT64_C(  -629.72), SIMDE_FLOAT64_C(  -194.56), SIMDE_FLOAT64_C(  -846.33), SIMDE_FLOAT64_C(    36.94),
        SIMDE_FLOAT64_C(   519.83), SIMDE_FLOAT64_C(  -689.41), SIMDE_FLOAT64_C(   331.63), SIMDE_FLOAT64_C(   991.49) },
      { SIMDE_FLOAT64_C(   629.72), SIMDE_FLOAT64_C(   828.22), SIMDE_FLOAT64_C(  1193.29), SIMDE_FLOAT64_C(   486.31),
        SIMDE_FLOAT64_C(   695.34), SIMDE_FLOAT64_C(  -575.90), SIMDE_FLOAT64_C(   752.22), SIMDE_FLOAT64_C(  1009.84) } },
    { { SIMDE_FLOAT64_C(   532.61), SIMDE_FLOAT64_C(   570.97), SIMDE_FLOAT64_C(  -353.24), SIMDE_FLOAT64_C(  -677.03),
        SIMDE_FLOAT64_C(   883.29), SIMDE_FLOAT64_C(   699.10), SIMDE_FLOAT64_C(  -817.27), SIMDE_FLOAT64_C(    17.83) },
      UINT8_C(222),
      { SIMDE_FLOAT64_C(  -226.03), SIMDE_FLOAT64_C(  -875.83), SIMDE_FLOAT64_C(  -648.42), SIMDE_FLOAT64_C(   933.26),
        SIMDE_FLOAT64_C(   992.67), SIMDE_FLOAT64_C(  -475.82), SIMDE_FLOAT64_C(   -66.35), SIMDE_FLOAT64_C(  -812.37) },
      { SIMDE_FLOAT64_C(  -634.58), SIMDE_FLOAT64_C(   448.74), SIMDE_FLOAT64_C(  -274.19), SIMDE_FLOAT64_C(   768.87),
        SIMDE_FLOAT64_C(   123.91), SIMDE_FLOAT64_C(   534.18), SIMDE_FLOAT64_C(  -860.86), SIMDE_FLOAT64_C(   929.35) },
      { SIMDE_FLOAT64_C(   532.61), SIMDE_FLOAT64_C(   984.10), SIMDE_FLOAT64_C(   704.01), SIMDE_FLOAT64_C(  1209.19),
        SIMDE_FLOAT64_C(  1000.37), SIMDE_FLOAT64_C(   699.10), SIMDE_FLOAT64_C(   863.41), SIMDE_FLOAT64_C(  1234.36) } },
    { { SIMDE_FLOAT64_C(   687.85), SIMDE_FLOAT64_C(   176.08), SIMDE_FLOAT64_C(   449.18), SIMDE_FLOAT64_C(   998.45),
        SIMDE_FLOAT64_C(  -492.29), SIMDE_FLOAT64_C(   440.66), SIMDE_FLOAT64_C(   531.06), SIMDE_FLOAT64_C(  -921.32) },
      UINT8_C( 88),
      { SIMDE_FLOAT64_C(   854.03), SIMDE_FLOAT64_C(   961.97), SIMDE_FLOAT64_C(   786.53), SIMDE_FLOAT64_C(  -963.25),
        SIMDE_FLOAT64_C(   -20.20), SIMDE_FLOAT64_C(   714.01), SIMDE_FLOAT64_C(  -189.28), SIMDE_FLOAT64_C(   103.97) },
      { SIMDE_FLOAT64_C(  -934.41), SIMDE_FLOAT64_C(  -256.02), SIMDE_FLOAT64_C(    96.64), SIMDE_FLOAT64_C(  -410.23),
        SIMDE_FLOAT64_C(   677.63), SIMDE_FLOAT64_C(   284.27), SIMDE_FLOAT64_C(   -44.81), SIMDE_FLOAT64_C(   126.37) },
      { SIMDE_FLOAT64_C(   687.85), SIMDE_FLOAT64_C(   176.08), SIMDE_FLOAT64_C(   449.18), SIMDE_FLOAT64_C(  1046.97),
        SIMDE_FLOAT64_C(   677.93), SIMDE_FLOAT64_C(   440.66), SIMDE_FLOAT64_C(   194.51), SIMDE_FLOAT64_C(  -921.32) } },
    { { SIMDE_FLOAT64_C(  -989.92), SIMDE_FLOAT64_C(  -275.94), SIMDE_FLOAT64_C(  -749.72), SIMDE_FLOAT64_C(   544.27),
        SIMDE_FLOAT64_C(  -136.80), SIMDE_FLOAT64_C(  -820.37), SIMDE_FLOAT64_C(   232.12), SIMDE_FLOAT64_C(  -960.72) },
      UINT8_C( 98),
      { SIMDE_FLOAT64_C(   230.57), SIMDE_FLOAT64_C(  -453.01), SIMDE_FLOAT64_C(    69.47), SIMDE_FLOAT64_C(  -238.38),
        SIMDE_FLOAT64_C(  -374.34), SIMDE_FLOAT64_C(   156.90), SIMDE_FLOAT64_C(  -384.35), SIMDE_FLOAT64_C(  -412.37) },
      { SIMDE_FLOAT64_C(   -56.57), SIMDE_FLOAT64_C(  -347.60), SIMDE_FLOAT64_C(   567.43), SIMDE_FLOAT64_C(  -342.56),
        SIMDE_FLOAT64_C(   463.12), SIMDE_FLOAT64_C(  -328.60), SIMDE_FLOAT64_C(  -276.97), SIMDE_FLOAT64_C(  -792.90) },
      { SIMDE_FLOAT64_C(  -989.92), SIMDE_FLOAT64_C(   571.00), SIMDE_FLOAT64_C(  -749.72), SIMDE_FLOAT64_C(   544.27),
        SIMDE_FLOAT64_C(  -136.80), SIMDE_FLOAT64_C(   364.14), SIMDE_FLOAT64_C(   473.75), SIMDE_FLOAT64_C(  -960.72) } },
    { { SIMDE_FLOAT64_C(   768.04), SIMDE_FLOAT64_C(   312.80), SIMDE_FLOAT64_C(   884.73), SIMDE_FLOAT64_C(    52.31),
        SIMDE_FLOAT64_C(  -732.01), SIMDE_FLOAT64_C(    11.11), SIMDE_FLOAT64_C(    62.39), SIMDE_FLOAT64_C(    -7.95) },
      UINT8_C(156),
      { SIMDE_FLOAT64_C(  -393.34), SIMDE_FLOAT64_C(   855.25), SIMDE_FLOAT64_C(   441.02), SIMDE_FLOAT64_C(   838.78),
        SIMDE_FLOAT64_C(   894.53), SIMDE_FLOAT64_C(    69.83), SIMDE_FLOAT64_C(    69.35), SIMDE_FLOAT64_C(  -558.49) },
      { SIMDE_FLOAT64_C(  -860.69), SIMDE_FLOAT64_C(   830.97), SIMDE_FLOAT64_C(    67.18), SIMDE_FLOAT64_C(   296.21),
        SIMDE_FLOAT64_C(  -553.38), SIMDE_FLOAT64_C(   654.81), SIMDE_FLOAT64_C(  -760.36), SIMDE_FLOAT64_C(    99.02) },
      { SIMDE_FLOAT64_C(   768.04), SIMDE_FLOAT64_C(   312.80), SIMDE_FLOAT64_C(   446.11), SIMDE_FLOAT64_C(   889.55),
        SIMDE_FLOAT64_C(  1051.86), SIMDE_FLOAT64_C(    11.11), SIMDE_FLOAT64_C(    62.39), SIMDE_FLOAT64_C(   567.20) } },
    { { SIMDE_FLOAT64_C(   222.24), SIMDE_FLOAT64_C(  -102.92), SIMDE_FLOAT64_C(  -437.85), SIMDE_FLOAT64_C(   893.64),
        SIMDE_FLOAT64_C(   620.10), SIMDE_FLOAT64_C(  -230.75), SIMDE_FLOAT64_C(   661.68), SIMDE_FLOAT64_C(   -67.10) },
      UINT8_C( 62),
      { SIMDE_FLOAT64_C(  -286.01), SIMDE_FLOAT64_C(   200.89), SIMDE_FLOAT64_C(   665.09), SIMDE_FLOAT64_C(   776.38),
        SIMDE_FLOAT64_C(  -807.06), SIMDE_FLOAT64_C(   -73.52), SIMDE_FLOAT64_C(  -616.96), SIMDE_FLOAT64_C(  -951.82) },
      { SIMDE_FLOAT64_C(  -632.50), SIMDE_FLOAT64_C(  -778.18), SIMDE_FLOAT64_C(   942.71), SIMDE_FLOAT64_C(   437.33),
        SIMDE_FLOAT64_C(   291.17), SIMDE_FLOAT64_C(  -615.78), SIMDE_FLOAT64_C(   576.64), SIMDE_FLOAT64_C(   122.14) },
      { SIMDE_FLOAT64_C(   222.24), SIMDE_FLOAT64_C(   803.69), SIMDE_FLOAT64_C(  1153.71), SIMDE_FLOAT64_C(   891.08),
        SIMDE_FLOAT64_C(   857.98), SIMDE_FLOAT64_C(   620.15), SIMDE_FLOAT64_C(   661.68), SIMDE_FLOAT64_C(   -67.10) } },
    { { SIMDE_FLOAT64_C(   451.40), SIMDE_FLOAT64_C(  -127.16), SIMDE_FLOAT64_C(   568.75), SIMDE_FLOAT64_C(   106.22),
        SIMDE_FLOAT64_C(   112.48), SIMDE_FLOAT64_C(  -332.22), SIMDE_FLOAT64_C(  -671.54), SIMDE_FLOAT64_C(  -990.45) },
      UINT8_C(133),
      { SIMDE_FLOAT64_C(  -777.90), SIMDE_FLOAT64_C(   629.66), SIMDE_FLOAT64_C(   999.17), SIMDE_FLOAT64_C(   883.78),
        SIMDE_FLOAT64_C(  -437.44), SIMDE_FLOAT64_C(  -346.84), SIMDE_FLOAT64_C(  -402.24), SIMDE_FLOAT64_C(   763.45) },
      { SIMDE_FLOAT64_C(  -681.75), SIMDE_FLOAT64_C(  -625.86), SIMDE_FLOAT64_C(   956.39), SIMDE_FLOAT64_C(   244.73),
        SIMDE_FLOAT64_C(  -242.82), SIMDE_FLOAT64_C(  -995.43), SIMDE_FLOAT64_C(   612.23), SIMDE_FLOAT64_C(   -21.00) },
      { SIMDE_FLOAT64_C(  1034.37), SIMDE_FLOAT64_C(  -127.16), SIMDE_FLOAT64_C(  1383.12), SIMDE_FLOAT64_C(   106.22),
        SIMDE_FLOAT64_C(   112.48), SIMDE_FLOAT64_C(  -332.22), SIMDE_FLOAT64_C(  -671.54), SIMDE_FLOAT64_C(   763.74) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d b = simde_mm512_loadu_pd(test_vec[i].b);
    simde__m512d r = simde_mm512_mask_hypot_pd(src, test_vec[i].k, a, b);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_invcbrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -830.78), SIMDE_FLOAT32_C(   407.78), SIMDE_FLOAT32_C(    34.12), SIMDE_FLOAT32_C(  -431.04) },
      { SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.13) } },
    { { SIMDE_FLOAT32_C(  -838.35), SIMDE_FLOAT32_C(  -741.30), SIMDE_FLOAT32_C(   354.85), SIMDE_FLOAT32_C(  -840.30) },
      { SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.11) } },
    { { SIMDE_FLOAT32_C(  -332.67), SIMDE_FLOAT32_C(   463.71), SIMDE_FLOAT32_C(  -606.20), SIMDE_FLOAT32_C(  -312.79) },
      { SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -0.15) } },
    { { SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(  -815.81), SIMDE_FLOAT32_C(  -819.10), SIMDE_FLOAT32_C(  -853.90) },
      { SIMDE_FLOAT32_C(    -1.55), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.11) } },
    { { SIMDE_FLOAT32_C(  -112.18), SIMDE_FLOAT32_C(    14.21), SIMDE_FLOAT32_C(   387.92), SIMDE_FLOAT32_C(  -952.65) },
      { SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.10) } },
    { { SIMDE_FLOAT32_C(  -492.35), SIMDE_FLOAT32_C(   204.52), SIMDE_FLOAT32_C(  -434.43), SIMDE_FLOAT32_C(   455.92) },
      { SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.13) } },
    { { SIMDE_FLOAT32_C(  -372.57), SIMDE_FLOAT32_C(  -697.63), SIMDE_FLOAT32_C(  -993.40), SIMDE_FLOAT32_C(    96.43) },
      { SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.22) } },
    { { SIMDE_FLOAT32_C(  -450.23), SIMDE_FLOAT32_C(   393.40), SIMDE_FLOAT32_C(   531.72), SIMDE_FLOAT32_C(  -281.01) },
      { SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.15) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_invcbrt_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_invcbrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -362.46), SIMDE_FLOAT64_C(   897.33) },
      { SIMDE_FLOAT64_C(    -0.14), SIMDE_FLOAT64_C(     0.10) } },
    { { SIMDE_FLOAT64_C(  -324.66), SIMDE_FLOAT64_C(  -116.25) },
      { SIMDE_FLOAT64_C(    -0.15), SIMDE_FLOAT64_C(    -0.20) } },
    { { SIMDE_FLOAT64_C(  -229.39), SIMDE_FLOAT64_C(  -924.64) },
      { SIMDE_FLOAT64_C(    -0.16), SIMDE_FLOAT64_C(    -0.10) } },
    { { SIMDE_FLOAT64_C(   619.01), SIMDE_FLOAT64_C(  -919.66) },
      { SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(    -0.10) } },
    { { SIMDE_FLOAT64_C(  -996.99), SIMDE_FLOAT64_C(  -352.60) },
      { SIMDE_FLOAT64_C(    -0.10), SIMDE_FLOAT64_C(    -0.14) } },
    { { SIMDE_FLOAT64_C(  -639.25), SIMDE_FLOAT64_C(    29.93) },
      { SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     0.32) } },
    { { SIMDE_FLOAT64_C(  -468.42), SIMDE_FLOAT64_C(   775.98) },
      { SIMDE_FLOAT64_C(    -0.13), SIMDE_FLOAT64_C(     0.11) } },
    { { SIMDE_FLOAT64_C(  -721.32), SIMDE_FLOAT64_C(   122.22) },
      { SIMDE_FLOAT64_C(    -0.11), SIMDE_FLOAT64_C(     0.20) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_invcbrt_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_invcbrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    91.84), SIMDE_FLOAT32_C(  -751.70), SIMDE_FLOAT32_C(    15.02), SIMDE_FLOAT32_C(  -388.95),
        SIMDE_FLOAT32_C(    99.77), SIMDE_FLOAT32_C(   919.81), SIMDE_FLOAT32_C(    65.75), SIMDE_FLOAT32_C(  -859.67) },
      { SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(    -0.14),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.11) } },
    { { SIMDE_FLOAT32_C(  -294.11), SIMDE_FLOAT32_C(    51.33), SIMDE_FLOAT32_C(  -783.32), SIMDE_FLOAT32_C(  -179.27),
        SIMDE_FLOAT32_C(  -759.73), SIMDE_FLOAT32_C(  -346.33), SIMDE_FLOAT32_C(   701.43), SIMDE_FLOAT32_C(    29.88) },
      { SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.18),
        SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.32) } },
    { { SIMDE_FLOAT32_C(  -448.16), SIMDE_FLOAT32_C(  -516.54), SIMDE_FLOAT32_C(  -452.98), SIMDE_FLOAT32_C(   948.25),
        SIMDE_FLOAT32_C(   387.51), SIMDE_FLOAT32_C(   585.82), SIMDE_FLOAT32_C(  -920.12), SIMDE_FLOAT32_C(   -81.56) },
      { SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.10),
        SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(    -0.23) } },
    { { SIMDE_FLOAT32_C(  -341.26), SIMDE_FLOAT32_C(  -436.41), SIMDE_FLOAT32_C(   422.76), SIMDE_FLOAT32_C(  -782.86),
        SIMDE_FLOAT32_C(  -131.30), SIMDE_FLOAT32_C(  -313.86), SIMDE_FLOAT32_C(   339.30), SIMDE_FLOAT32_C(   960.53) },
      { SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(    -0.11),
        SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.10) } },
    { { SIMDE_FLOAT32_C(   -65.56), SIMDE_FLOAT32_C(  -645.68), SIMDE_FLOAT32_C(  -428.41), SIMDE_FLOAT32_C(  -965.79),
        SIMDE_FLOAT32_C(  -725.86), SIMDE_FLOAT32_C(   637.33), SIMDE_FLOAT32_C(  -825.46), SIMDE_FLOAT32_C(   -19.97) },
      { SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(    -0.10),
        SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.37) } },
    { { SIMDE_FLOAT32_C(  -311.34), SIMDE_FLOAT32_C(  -608.78), SIMDE_FLOAT32_C(   800.75), SIMDE_FLOAT32_C(   -71.07),
        SIMDE_FLOAT32_C(    44.89), SIMDE_FLOAT32_C(   502.19), SIMDE_FLOAT32_C(   958.81), SIMDE_FLOAT32_C(   596.72) },
      { SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.24),
        SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.12) } },
    { { SIMDE_FLOAT32_C(   985.65), SIMDE_FLOAT32_C(  -494.17), SIMDE_FLOAT32_C(   544.98), SIMDE_FLOAT32_C(   373.15),
        SIMDE_FLOAT32_C(  -908.35), SIMDE_FLOAT32_C(   624.86), SIMDE_FLOAT32_C(  -708.41), SIMDE_FLOAT32_C(  -249.62) },
      { SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.14),
        SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.16) } },
    { { SIMDE_FLOAT32_C(  -811.55), SIMDE_FLOAT32_C(   714.36), SIMDE_FLOAT32_C(   -32.48), SIMDE_FLOAT32_C(    57.15),
        SIMDE_FLOAT32_C(  -599.50), SIMDE_FLOAT32_C(  -693.18), SIMDE_FLOAT32_C(    17.68), SIMDE_FLOAT32_C(   334.94) },
      { SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.14) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_invcbrt_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_invcbrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -253.42), SIMDE_FLOAT64_C(  -775.86), SIMDE_FLOAT64_C(     7.55), SIMDE_FLOAT64_C(   246.09) },
      { SIMDE_FLOAT64_C(    -0.16), SIMDE_FLOAT64_C(    -0.11), SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(     0.16) } },
    { { SIMDE_FLOAT64_C(  -201.99), SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(  -584.03), SIMDE_FLOAT64_C(  -671.92) },
      { SIMDE_FLOAT64_C(    -0.17), SIMDE_FLOAT64_C(     1.22), SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(    -0.11) } },
    { { SIMDE_FLOAT64_C(   851.57), SIMDE_FLOAT64_C(   459.01), SIMDE_FLOAT64_C(   394.56), SIMDE_FLOAT64_C(   866.29) },
      { SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.10) } },
    { { SIMDE_FLOAT64_C(   645.75), SIMDE_FLOAT64_C(   575.99), SIMDE_FLOAT64_C(    41.51), SIMDE_FLOAT64_C(  -177.11) },
      { SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(     0.29), SIMDE_FLOAT64_C(    -0.18) } },
    { { SIMDE_FLOAT64_C(  -632.82), SIMDE_FLOAT64_C(   815.53), SIMDE_FLOAT64_C(   -21.43), SIMDE_FLOAT64_C(  -406.93) },
      { SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -0.36), SIMDE_FLOAT64_C(    -0.13) } },
    { { SIMDE_FLOAT64_C(   471.99), SIMDE_FLOAT64_C(  -996.82), SIMDE_FLOAT64_C(  -716.04), SIMDE_FLOAT64_C(  -550.05) },
      { SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(    -0.10), SIMDE_FLOAT64_C(    -0.11), SIMDE_FLOAT64_C(    -0.12) } },
    { { SIMDE_FLOAT64_C(   564.26), SIMDE_FLOAT64_C(  -164.60), SIMDE_FLOAT64_C(  -303.42), SIMDE_FLOAT64_C(  -304.34) },
      { SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(    -0.18), SIMDE_FLOAT64_C(    -0.15), SIMDE_FLOAT64_C(    -0.15) } },
    { { SIMDE_FLOAT64_C(   749.99), SIMDE_FLOAT64_C(   564.62), SIMDE_FLOAT64_C(  -957.88), SIMDE_FLOAT64_C(  -503.43) },
      { SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(    -0.10), SIMDE_FLOAT64_C(    -0.13) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_invcbrt_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_invsqrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   963.10), SIMDE_FLOAT32_C(   544.41), SIMDE_FLOAT32_C(   741.04), SIMDE_FLOAT32_C(   478.93) },
      { SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(   289.81), SIMDE_FLOAT32_C(   489.84), SIMDE_FLOAT32_C(   576.93), SIMDE_FLOAT32_C(   960.27) },
      { SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03) } },
    { { SIMDE_FLOAT32_C(   308.08), SIMDE_FLOAT32_C(    66.08), SIMDE_FLOAT32_C(   486.27), SIMDE_FLOAT32_C(   318.16) },
      { SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.06) } },
    { { SIMDE_FLOAT32_C(   848.25), SIMDE_FLOAT32_C(   887.84), SIMDE_FLOAT32_C(   814.84), SIMDE_FLOAT32_C(   533.08) },
      { SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04) } },
    { { SIMDE_FLOAT32_C(   476.90), SIMDE_FLOAT32_C(   887.49), SIMDE_FLOAT32_C(   751.34), SIMDE_FLOAT32_C(   508.49) },
      { SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04) } },
    { { SIMDE_FLOAT32_C(   679.70), SIMDE_FLOAT32_C(   603.84), SIMDE_FLOAT32_C(   905.34), SIMDE_FLOAT32_C(    39.88) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.16) } },
    { { SIMDE_FLOAT32_C(   629.17), SIMDE_FLOAT32_C(   401.81), SIMDE_FLOAT32_C(   823.42), SIMDE_FLOAT32_C(   435.02) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(   727.18), SIMDE_FLOAT32_C(   800.47), SIMDE_FLOAT32_C(    32.70), SIMDE_FLOAT32_C(   690.28) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.04) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_invsqrt_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_invsqrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   387.27), SIMDE_FLOAT64_C(   266.58) },
      { SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.06) } },
    { { SIMDE_FLOAT64_C(   629.96), SIMDE_FLOAT64_C(   591.67) },
      { SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(   185.36), SIMDE_FLOAT64_C(   529.90) },
      { SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(   429.91), SIMDE_FLOAT64_C(   539.03) },
      { SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(   626.90), SIMDE_FLOAT64_C(   833.69) },
      { SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.03) } },
    { { SIMDE_FLOAT64_C(   722.07), SIMDE_FLOAT64_C(   296.55) },
      { SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.06) } },
    { { SIMDE_FLOAT64_C(   474.49), SIMDE_FLOAT64_C(   271.22) },
      { SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.06) } },
    { { SIMDE_FLOAT64_C(   980.81), SIMDE_FLOAT64_C(   981.24) },
      { SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.03) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_invsqrt_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_invsqrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   523.53), SIMDE_FLOAT32_C(   456.96), SIMDE_FLOAT32_C(   204.64), SIMDE_FLOAT32_C(   395.38),
        SIMDE_FLOAT32_C(   112.91), SIMDE_FLOAT32_C(   473.53), SIMDE_FLOAT32_C(   965.22), SIMDE_FLOAT32_C(   423.85) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(   834.19), SIMDE_FLOAT32_C(   352.97), SIMDE_FLOAT32_C(   156.12), SIMDE_FLOAT32_C(   635.31),
        SIMDE_FLOAT32_C(   962.63), SIMDE_FLOAT32_C(   823.80), SIMDE_FLOAT32_C(   454.23), SIMDE_FLOAT32_C(   413.73) },
      { SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(   443.70), SIMDE_FLOAT32_C(   770.20), SIMDE_FLOAT32_C(   506.36), SIMDE_FLOAT32_C(    13.18),
        SIMDE_FLOAT32_C(   957.34), SIMDE_FLOAT32_C(   388.10), SIMDE_FLOAT32_C(   124.63), SIMDE_FLOAT32_C(     5.64) },
      { SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.28),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.42) } },
    { { SIMDE_FLOAT32_C(   141.65), SIMDE_FLOAT32_C(   772.61), SIMDE_FLOAT32_C(   451.36), SIMDE_FLOAT32_C(   350.31),
        SIMDE_FLOAT32_C(    74.48), SIMDE_FLOAT32_C(   384.43), SIMDE_FLOAT32_C(   380.41), SIMDE_FLOAT32_C(   598.01) },
      { SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.04) } },
    { { SIMDE_FLOAT32_C(   841.39), SIMDE_FLOAT32_C(   585.05), SIMDE_FLOAT32_C(   993.40), SIMDE_FLOAT32_C(   954.30),
        SIMDE_FLOAT32_C(    58.58), SIMDE_FLOAT32_C(   958.61), SIMDE_FLOAT32_C(   378.15), SIMDE_FLOAT32_C(   892.77) },
      { SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.03) } },
    { { SIMDE_FLOAT32_C(   311.58), SIMDE_FLOAT32_C(   534.27), SIMDE_FLOAT32_C(   528.07), SIMDE_FLOAT32_C(   274.21),
        SIMDE_FLOAT32_C(   358.06), SIMDE_FLOAT32_C(   982.30), SIMDE_FLOAT32_C(   687.94), SIMDE_FLOAT32_C(   801.76) },
      { SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04) } },
    { { SIMDE_FLOAT32_C(   752.50), SIMDE_FLOAT32_C(   194.30), SIMDE_FLOAT32_C(   814.95), SIMDE_FLOAT32_C(   709.84),
        SIMDE_FLOAT32_C(   582.40), SIMDE_FLOAT32_C(   939.58), SIMDE_FLOAT32_C(   715.48), SIMDE_FLOAT32_C(   724.05) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04) } },
    { { SIMDE_FLOAT32_C(   712.19), SIMDE_FLOAT32_C(   166.84), SIMDE_FLOAT32_C(    74.36), SIMDE_FLOAT32_C(   786.67),
        SIMDE_FLOAT32_C(   551.27), SIMDE_FLOAT32_C(   454.77), SIMDE_FLOAT32_C(   384.69), SIMDE_FLOAT32_C(   392.66) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.05) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_invsqrt_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_invsqrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    35.16), SIMDE_FLOAT64_C(   340.96), SIMDE_FLOAT64_C(    60.32), SIMDE_FLOAT64_C(   560.44) },
      { SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(   259.52), SIMDE_FLOAT64_C(   415.50), SIMDE_FLOAT64_C(   716.63), SIMDE_FLOAT64_C(   444.07) },
      { SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.05) } },
    { { SIMDE_FLOAT64_C(   714.85), SIMDE_FLOAT64_C(    53.22), SIMDE_FLOAT64_C(   199.06), SIMDE_FLOAT64_C(   714.03) },
      { SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(   807.60), SIMDE_FLOAT64_C(    19.21), SIMDE_FLOAT64_C(   401.27), SIMDE_FLOAT64_C(   275.62) },
      { SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.06) } },
    { { SIMDE_FLOAT64_C(    69.48), SIMDE_FLOAT64_C(   716.42), SIMDE_FLOAT64_C(   754.51), SIMDE_FLOAT64_C(   517.80) },
      { SIMDE_FLOAT64_C(     0.12), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(   294.75), SIMDE_FLOAT64_C(   671.92), SIMDE_FLOAT64_C(   712.33), SIMDE_FLOAT64_C(   826.45) },
      { SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.03) } },
    { { SIMDE_FLOAT64_C(    47.66), SIMDE_FLOAT64_C(   965.47), SIMDE_FLOAT64_C(   318.45), SIMDE_FLOAT64_C(   190.50) },
      { SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.07) } },
    { { SIMDE_FLOAT64_C(    58.25), SIMDE_FLOAT64_C(   429.76), SIMDE_FLOAT64_C(   771.19), SIMDE_FLOAT64_C(    93.42) },
      { SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.10) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_invsqrt_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_invsqrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    22.96), SIMDE_FLOAT32_C(   915.74), SIMDE_FLOAT32_C(    22.13), SIMDE_FLOAT32_C(   201.67),
        SIMDE_FLOAT32_C(   223.81), SIMDE_FLOAT32_C(   949.13), SIMDE_FLOAT32_C(    18.28), SIMDE_FLOAT32_C(   237.29),
        SIMDE_FLOAT32_C(    95.68), SIMDE_FLOAT32_C(   358.07), SIMDE_FLOAT32_C(   974.18), SIMDE_FLOAT32_C(   343.28),
        SIMDE_FLOAT32_C(   900.66), SIMDE_FLOAT32_C(   905.83), SIMDE_FLOAT32_C(   810.45), SIMDE_FLOAT32_C(   409.74) },
      { SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.07),
        SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(   332.59), SIMDE_FLOAT32_C(   299.68), SIMDE_FLOAT32_C(   414.08), SIMDE_FLOAT32_C(   229.81),
        SIMDE_FLOAT32_C(   905.70), SIMDE_FLOAT32_C(   204.12), SIMDE_FLOAT32_C(   480.98), SIMDE_FLOAT32_C(   846.82),
        SIMDE_FLOAT32_C(   367.27), SIMDE_FLOAT32_C(   670.54), SIMDE_FLOAT32_C(   936.86), SIMDE_FLOAT32_C(   972.95),
        SIMDE_FLOAT32_C(   695.70), SIMDE_FLOAT32_C(   781.82), SIMDE_FLOAT32_C(   825.14), SIMDE_FLOAT32_C(   718.66) },
      { SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.07),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04) } },
    { { SIMDE_FLOAT32_C(   697.56), SIMDE_FLOAT32_C(   847.27), SIMDE_FLOAT32_C(   920.33), SIMDE_FLOAT32_C(   921.36),
        SIMDE_FLOAT32_C(   796.40), SIMDE_FLOAT32_C(   938.61), SIMDE_FLOAT32_C(   158.65), SIMDE_FLOAT32_C(   892.08),
        SIMDE_FLOAT32_C(   296.69), SIMDE_FLOAT32_C(   132.83), SIMDE_FLOAT32_C(   235.36), SIMDE_FLOAT32_C(   197.35),
        SIMDE_FLOAT32_C(    38.67), SIMDE_FLOAT32_C(    45.81), SIMDE_FLOAT32_C(   607.10), SIMDE_FLOAT32_C(   371.26) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.07),
        SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(   345.49), SIMDE_FLOAT32_C(    21.18), SIMDE_FLOAT32_C(   601.07), SIMDE_FLOAT32_C(   251.19),
        SIMDE_FLOAT32_C(   225.29), SIMDE_FLOAT32_C(    82.05), SIMDE_FLOAT32_C(    98.01), SIMDE_FLOAT32_C(   592.56),
        SIMDE_FLOAT32_C(   752.59), SIMDE_FLOAT32_C(    34.87), SIMDE_FLOAT32_C(   565.51), SIMDE_FLOAT32_C(   448.29),
        SIMDE_FLOAT32_C(   816.69), SIMDE_FLOAT32_C(   390.65), SIMDE_FLOAT32_C(   166.96), SIMDE_FLOAT32_C(   514.24) },
      { SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.04) } },
    { { SIMDE_FLOAT32_C(   237.92), SIMDE_FLOAT32_C(    87.29), SIMDE_FLOAT32_C(   435.61), SIMDE_FLOAT32_C(    34.32),
        SIMDE_FLOAT32_C(    25.90), SIMDE_FLOAT32_C(   594.25), SIMDE_FLOAT32_C(   926.40), SIMDE_FLOAT32_C(   322.59),
        SIMDE_FLOAT32_C(   727.09), SIMDE_FLOAT32_C(   161.76), SIMDE_FLOAT32_C(   519.95), SIMDE_FLOAT32_C(   765.75),
        SIMDE_FLOAT32_C(   207.57), SIMDE_FLOAT32_C(   127.04), SIMDE_FLOAT32_C(   137.01), SIMDE_FLOAT32_C(   553.06) },
      { SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.17),
        SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.04) } },
    { { SIMDE_FLOAT32_C(   148.22), SIMDE_FLOAT32_C(   738.08), SIMDE_FLOAT32_C(   804.24), SIMDE_FLOAT32_C(   373.51),
        SIMDE_FLOAT32_C(   820.13), SIMDE_FLOAT32_C(   902.25), SIMDE_FLOAT32_C(   966.07), SIMDE_FLOAT32_C(   572.72),
        SIMDE_FLOAT32_C(   937.12), SIMDE_FLOAT32_C(   531.58), SIMDE_FLOAT32_C(    21.01), SIMDE_FLOAT32_C(   753.81),
        SIMDE_FLOAT32_C(   922.24), SIMDE_FLOAT32_C(   187.97), SIMDE_FLOAT32_C(   268.05), SIMDE_FLOAT32_C(   160.16) },
      { SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.08) } },
    { { SIMDE_FLOAT32_C(   275.26), SIMDE_FLOAT32_C(   703.65), SIMDE_FLOAT32_C(   194.48), SIMDE_FLOAT32_C(   301.16),
        SIMDE_FLOAT32_C(   297.91), SIMDE_FLOAT32_C(   120.89), SIMDE_FLOAT32_C(   623.76), SIMDE_FLOAT32_C(    25.00),
        SIMDE_FLOAT32_C(   282.65), SIMDE_FLOAT32_C(   143.70), SIMDE_FLOAT32_C(   790.75), SIMDE_FLOAT32_C(   490.22),
        SIMDE_FLOAT32_C(   270.74), SIMDE_FLOAT32_C(   927.76), SIMDE_FLOAT32_C(    43.28), SIMDE_FLOAT32_C(   418.96) },
      { SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.20),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     0.05) } },
    { { SIMDE_FLOAT32_C(   665.84), SIMDE_FLOAT32_C(   847.52), SIMDE_FLOAT32_C(   792.47), SIMDE_FLOAT32_C(   485.97),
        SIMDE_FLOAT32_C(   749.77), SIMDE_FLOAT32_C(   758.54), SIMDE_FLOAT32_C(    58.69), SIMDE_FLOAT32_C(   686.89),
        SIMDE_FLOAT32_C(   290.13), SIMDE_FLOAT32_C(    79.70), SIMDE_FLOAT32_C(   440.70), SIMDE_FLOAT32_C(   212.36),
        SIMDE_FLOAT32_C(   267.67), SIMDE_FLOAT32_C(   708.75), SIMDE_FLOAT32_C(   372.52), SIMDE_FLOAT32_C(   542.93) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.07),
        SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.04) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_invsqrt_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_invsqrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   624.14), SIMDE_FLOAT32_C(   819.60), SIMDE_FLOAT32_C(   672.51), SIMDE_FLOAT32_C(   550.11),
        SIMDE_FLOAT32_C(   812.34), SIMDE_FLOAT32_C(   166.77), SIMDE_FLOAT32_C(    70.17), SIMDE_FLOAT32_C(   377.64),
        SIMDE_FLOAT32_C(   183.00), SIMDE_FLOAT32_C(   818.17), SIMDE_FLOAT32_C(   404.48), SIMDE_FLOAT32_C(   187.86),
        SIMDE_FLOAT32_C(   392.86), SIMDE_FLOAT32_C(   212.92), SIMDE_FLOAT32_C(   867.57), SIMDE_FLOAT32_C(   410.64) },
      UINT8_C(  3),
      { SIMDE_FLOAT32_C(    33.63), SIMDE_FLOAT32_C(    77.51), SIMDE_FLOAT32_C(   932.62), SIMDE_FLOAT32_C(   356.45),
        SIMDE_FLOAT32_C(   533.80), SIMDE_FLOAT32_C(   680.31), SIMDE_FLOAT32_C(   975.45), SIMDE_FLOAT32_C(   578.12),
        SIMDE_FLOAT32_C(   558.84), SIMDE_FLOAT32_C(   281.04), SIMDE_FLOAT32_C(   747.18), SIMDE_FLOAT32_C(   909.72),
        SIMDE_FLOAT32_C(   312.02), SIMDE_FLOAT32_C(   748.71), SIMDE_FLOAT32_C(   533.86), SIMDE_FLOAT32_C(   131.63) },
      { SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(   672.51), SIMDE_FLOAT32_C(   550.11),
        SIMDE_FLOAT32_C(   812.34), SIMDE_FLOAT32_C(   166.77), SIMDE_FLOAT32_C(    70.17), SIMDE_FLOAT32_C(   377.64),
        SIMDE_FLOAT32_C(   183.00), SIMDE_FLOAT32_C(   818.17), SIMDE_FLOAT32_C(   404.48), SIMDE_FLOAT32_C(   187.86),
        SIMDE_FLOAT32_C(   392.86), SIMDE_FLOAT32_C(   212.92), SIMDE_FLOAT32_C(   867.57), SIMDE_FLOAT32_C(   410.64) } },
    { { SIMDE_FLOAT32_C(   421.22), SIMDE_FLOAT32_C(    83.97), SIMDE_FLOAT32_C(   943.97), SIMDE_FLOAT32_C(   587.99),
        SIMDE_FLOAT32_C(   154.14), SIMDE_FLOAT32_C(   321.61), SIMDE_FLOAT32_C(   770.98), SIMDE_FLOAT32_C(   972.32),
        SIMDE_FLOAT32_C(   726.09), SIMDE_FLOAT32_C(   958.84), SIMDE_FLOAT32_C(   365.17), SIMDE_FLOAT32_C(   939.01),
        SIMDE_FLOAT32_C(   826.41), SIMDE_FLOAT32_C(   775.81), SIMDE_FLOAT32_C(   236.82), SIMDE_FLOAT32_C(   860.05) },
      UINT8_C( 38),
      { SIMDE_FLOAT32_C(   169.44), SIMDE_FLOAT32_C(   216.49), SIMDE_FLOAT32_C(   387.13), SIMDE_FLOAT32_C(   849.74),
        SIMDE_FLOAT32_C(   191.94), SIMDE_FLOAT32_C(   965.24), SIMDE_FLOAT32_C(   408.58), SIMDE_FLOAT32_C(   472.98),
        SIMDE_FLOAT32_C(   712.43), SIMDE_FLOAT32_C(   318.30), SIMDE_FLOAT32_C(   785.00), SIMDE_FLOAT32_C(   461.13),
        SIMDE_FLOAT32_C(   852.16), SIMDE_FLOAT32_C(   916.63), SIMDE_FLOAT32_C(   882.35), SIMDE_FLOAT32_C(   936.13) },
      { SIMDE_FLOAT32_C(   421.22), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(   587.99),
        SIMDE_FLOAT32_C(   154.14), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(   770.98), SIMDE_FLOAT32_C(   972.32),
        SIMDE_FLOAT32_C(   726.09), SIMDE_FLOAT32_C(   958.84), SIMDE_FLOAT32_C(   365.17), SIMDE_FLOAT32_C(   939.01),
        SIMDE_FLOAT32_C(   826.41), SIMDE_FLOAT32_C(   775.81), SIMDE_FLOAT32_C(   236.82), SIMDE_FLOAT32_C(   860.05) } },
    { { SIMDE_FLOAT32_C(   860.60), SIMDE_FLOAT32_C(   470.34), SIMDE_FLOAT32_C(    90.27), SIMDE_FLOAT32_C(   182.21),
        SIMDE_FLOAT32_C(   241.32), SIMDE_FLOAT32_C(    62.59), SIMDE_FLOAT32_C(   908.29), SIMDE_FLOAT32_C(   200.16),
        SIMDE_FLOAT32_C(   427.77), SIMDE_FLOAT32_C(   847.30), SIMDE_FLOAT32_C(    26.58), SIMDE_FLOAT32_C(   203.58),
        SIMDE_FLOAT32_C(    84.12), SIMDE_FLOAT32_C(   886.63), SIMDE_FLOAT32_C(    56.91), SIMDE_FLOAT32_C(   253.56) },
      UINT8_C( 27),
      { SIMDE_FLOAT32_C(   444.03), SIMDE_FLOAT32_C(   103.30), SIMDE_FLOAT32_C(   295.06), SIMDE_FLOAT32_C(   409.28),
        SIMDE_FLOAT32_C(   511.88), SIMDE_FLOAT32_C(   768.04), SIMDE_FLOAT32_C(   121.70), SIMDE_FLOAT32_C(   830.18),
        SIMDE_FLOAT32_C(   553.04), SIMDE_FLOAT32_C(   582.83), SIMDE_FLOAT32_C(   682.34), SIMDE_FLOAT32_C(   469.67),
        SIMDE_FLOAT32_C(   465.19), SIMDE_FLOAT32_C(   618.47), SIMDE_FLOAT32_C(   330.27), SIMDE_FLOAT32_C(   935.53) },
      { SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    90.27), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    62.59), SIMDE_FLOAT32_C(   908.29), SIMDE_FLOAT32_C(   200.16),
        SIMDE_FLOAT32_C(   427.77), SIMDE_FLOAT32_C(   847.30), SIMDE_FLOAT32_C(    26.58), SIMDE_FLOAT32_C(   203.58),
        SIMDE_FLOAT32_C(    84.12), SIMDE_FLOAT32_C(   886.63), SIMDE_FLOAT32_C(    56.91), SIMDE_FLOAT32_C(   253.56) } },
    { { SIMDE_FLOAT32_C(   708.74), SIMDE_FLOAT32_C(   512.48), SIMDE_FLOAT32_C(   176.85), SIMDE_FLOAT32_C(   771.33),
        SIMDE_FLOAT32_C(   420.77), SIMDE_FLOAT32_C(   377.02), SIMDE_FLOAT32_C(   199.10), SIMDE_FLOAT32_C(   268.07),
        SIMDE_FLOAT32_C(   403.59), SIMDE_FLOAT32_C(   402.68), SIMDE_FLOAT32_C(   352.19), SIMDE_FLOAT32_C(   290.22),
        SIMDE_FLOAT32_C(   459.59), SIMDE_FLOAT32_C(   605.74), SIMDE_FLOAT32_C(   393.34), SIMDE_FLOAT32_C(   903.62) },
      UINT8_C(  7),
      { SIMDE_FLOAT32_C(   688.40), SIMDE_FLOAT32_C(   312.89), SIMDE_FLOAT32_C(   220.93), SIMDE_FLOAT32_C(   456.44),
        SIMDE_FLOAT32_C(   434.59), SIMDE_FLOAT32_C(    51.11), SIMDE_FLOAT32_C(     9.48), SIMDE_FLOAT32_C(    17.43),
        SIMDE_FLOAT32_C(   733.45), SIMDE_FLOAT32_C(   479.15), SIMDE_FLOAT32_C(   482.62), SIMDE_FLOAT32_C(   351.92),
        SIMDE_FLOAT32_C(   809.42), SIMDE_FLOAT32_C(   418.14), SIMDE_FLOAT32_C(    60.66), SIMDE_FLOAT32_C(   321.90) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(   771.33),
        SIMDE_FLOAT32_C(   420.77), SIMDE_FLOAT32_C(   377.02), SIMDE_FLOAT32_C(   199.10), SIMDE_FLOAT32_C(   268.07),
        SIMDE_FLOAT32_C(   403.59), SIMDE_FLOAT32_C(   402.68), SIMDE_FLOAT32_C(   352.19), SIMDE_FLOAT32_C(   290.22),
        SIMDE_FLOAT32_C(   459.59), SIMDE_FLOAT32_C(   605.74), SIMDE_FLOAT32_C(   393.34), SIMDE_FLOAT32_C(   903.62) } },
    { { SIMDE_FLOAT32_C(   594.99), SIMDE_FLOAT32_C(   832.00), SIMDE_FLOAT32_C(   742.67), SIMDE_FLOAT32_C(   972.01),
        SIMDE_FLOAT32_C(    31.10), SIMDE_FLOAT32_C(    10.74), SIMDE_FLOAT32_C(   375.60), SIMDE_FLOAT32_C(   433.77),
        SIMDE_FLOAT32_C(   362.92), SIMDE_FLOAT32_C(   665.82), SIMDE_FLOAT32_C(   893.36), SIMDE_FLOAT32_C(   968.67),
        SIMDE_FLOAT32_C(    59.16), SIMDE_FLOAT32_C(   796.98), SIMDE_FLOAT32_C(   677.71), SIMDE_FLOAT32_C(   747.56) },
      UINT8_C(104),
      { SIMDE_FLOAT32_C(   898.63), SIMDE_FLOAT32_C(   203.99), SIMDE_FLOAT32_C(   544.46), SIMDE_FLOAT32_C(   949.74),
        SIMDE_FLOAT32_C(   213.47), SIMDE_FLOAT32_C(   561.89), SIMDE_FLOAT32_C(   683.19), SIMDE_FLOAT32_C(   692.63),
        SIMDE_FLOAT32_C(    44.51), SIMDE_FLOAT32_C(    35.11), SIMDE_FLOAT32_C(   502.05), SIMDE_FLOAT32_C(   462.65),
        SIMDE_FLOAT32_C(    95.77), SIMDE_FLOAT32_C(   823.95), SIMDE_FLOAT32_C(    57.64), SIMDE_FLOAT32_C(   927.76) },
      { SIMDE_FLOAT32_C(   594.99), SIMDE_FLOAT32_C(   832.00), SIMDE_FLOAT32_C(   742.67), SIMDE_FLOAT32_C(     0.03),
        SIMDE_FLOAT32_C(    31.10), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(   433.77),
        SIMDE_FLOAT32_C(   362.92), SIMDE_FLOAT32_C(   665.82), SIMDE_FLOAT32_C(   893.36), SIMDE_FLOAT32_C(   968.67),
        SIMDE_FLOAT32_C(    59.16), SIMDE_FLOAT32_C(   796.98), SIMDE_FLOAT32_C(   677.71), SIMDE_FLOAT32_C(   747.56) } },
    { { SIMDE_FLOAT32_C(   566.62), SIMDE_FLOAT32_C(    29.65), SIMDE_FLOAT32_C(   958.86), SIMDE_FLOAT32_C(   577.36),
        SIMDE_FLOAT32_C(   405.26), SIMDE_FLOAT32_C(   392.63), SIMDE_FLOAT32_C(   940.29), SIMDE_FLOAT32_C(    71.08),
        SIMDE_FLOAT32_C(   285.99), SIMDE_FLOAT32_C(   908.95), SIMDE_FLOAT32_C(   130.24), SIMDE_FLOAT32_C(    82.97),
        SIMDE_FLOAT32_C(   586.66), SIMDE_FLOAT32_C(   877.80), SIMDE_FLOAT32_C(   192.84), SIMDE_FLOAT32_C(   485.30) },
      UINT8_C( 59),
      { SIMDE_FLOAT32_C(   737.31), SIMDE_FLOAT32_C(   435.04), SIMDE_FLOAT32_C(   295.27), SIMDE_FLOAT32_C(   299.20),
        SIMDE_FLOAT32_C(   118.23), SIMDE_FLOAT32_C(   987.89), SIMDE_FLOAT32_C(   343.70), SIMDE_FLOAT32_C(   153.34),
        SIMDE_FLOAT32_C(   489.94), SIMDE_FLOAT32_C(   806.35), SIMDE_FLOAT32_C(   249.11), SIMDE_FLOAT32_C(   313.90),
        SIMDE_FLOAT32_C(   864.00), SIMDE_FLOAT32_C(   176.87), SIMDE_FLOAT32_C(   880.52), SIMDE_FLOAT32_C(   893.65) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(   958.86), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(   940.29), SIMDE_FLOAT32_C(    71.08),
        SIMDE_FLOAT32_C(   285.99), SIMDE_FLOAT32_C(   908.95), SIMDE_FLOAT32_C(   130.24), SIMDE_FLOAT32_C(    82.97),
        SIMDE_FLOAT32_C(   586.66), SIMDE_FLOAT32_C(   877.80), SIMDE_FLOAT32_C(   192.84), SIMDE_FLOAT32_C(   485.30) } },
    { { SIMDE_FLOAT32_C(   135.73), SIMDE_FLOAT32_C(   457.88), SIMDE_FLOAT32_C(   298.91), SIMDE_FLOAT32_C(   528.36),
        SIMDE_FLOAT32_C(   398.17), SIMDE_FLOAT32_C(   369.99), SIMDE_FLOAT32_C(   814.36), SIMDE_FLOAT32_C(   307.12),
        SIMDE_FLOAT32_C(   500.23), SIMDE_FLOAT32_C(   897.33), SIMDE_FLOAT32_C(   893.78), SIMDE_FLOAT32_C(   378.03),
        SIMDE_FLOAT32_C(    90.17), SIMDE_FLOAT32_C(   379.08), SIMDE_FLOAT32_C(   459.82), SIMDE_FLOAT32_C(   827.48) },
      UINT8_C(163),
      { SIMDE_FLOAT32_C(   755.09), SIMDE_FLOAT32_C(   126.67), SIMDE_FLOAT32_C(   932.35), SIMDE_FLOAT32_C(   742.98),
        SIMDE_FLOAT32_C(   470.38), SIMDE_FLOAT32_C(    85.68), SIMDE_FLOAT32_C(   232.93), SIMDE_FLOAT32_C(   276.73),
        SIMDE_FLOAT32_C(   334.79), SIMDE_FLOAT32_C(   546.82), SIMDE_FLOAT32_C(   140.73), SIMDE_FLOAT32_C(   511.66),
        SIMDE_FLOAT32_C(   427.34), SIMDE_FLOAT32_C(    34.38), SIMDE_FLOAT32_C(   647.39), SIMDE_FLOAT32_C(   885.22) },
      { SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(   298.91), SIMDE_FLOAT32_C(   528.36),
        SIMDE_FLOAT32_C(   398.17), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(   814.36), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(   500.23), SIMDE_FLOAT32_C(   897.33), SIMDE_FLOAT32_C(   893.78), SIMDE_FLOAT32_C(   378.03),
        SIMDE_FLOAT32_C(    90.17), SIMDE_FLOAT32_C(   379.08), SIMDE_FLOAT32_C(   459.82), SIMDE_FLOAT32_C(   827.48) } },
    { { SIMDE_FLOAT32_C(   333.29), SIMDE_FLOAT32_C(   175.75), SIMDE_FLOAT32_C(   283.39), SIMDE_FLOAT32_C(   703.28),
        SIMDE_FLOAT32_C(   990.11), SIMDE_FLOAT32_C(   590.51), SIMDE_FLOAT32_C(   203.51), SIMDE_FLOAT32_C(   887.44),
        SIMDE_FLOAT32_C(   484.30), SIMDE_FLOAT32_C(   581.54), SIMDE_FLOAT32_C(   977.62), SIMDE_FLOAT32_C(   863.38),
        SIMDE_FLOAT32_C(    41.36), SIMDE_FLOAT32_C(   805.09), SIMDE_FLOAT32_C(   677.49), SIMDE_FLOAT32_C(   796.45) },
      UINT8_C(166),
      { SIMDE_FLOAT32_C(   609.84), SIMDE_FLOAT32_C(   539.43), SIMDE_FLOAT32_C(   402.14), SIMDE_FLOAT32_C(   695.53),
        SIMDE_FLOAT32_C(   772.36), SIMDE_FLOAT32_C(   678.87), SIMDE_FLOAT32_C(    30.32), SIMDE_FLOAT32_C(   319.18),
        SIMDE_FLOAT32_C(   819.60), SIMDE_FLOAT32_C(   541.97), SIMDE_FLOAT32_C(   746.52), SIMDE_FLOAT32_C(   853.98),
        SIMDE_FLOAT32_C(   189.36), SIMDE_FLOAT32_C(   631.74), SIMDE_FLOAT32_C(   187.26), SIMDE_FLOAT32_C(   365.12) },
      { SIMDE_FLOAT32_C(   333.29), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(   703.28),
        SIMDE_FLOAT32_C(   990.11), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(   203.51), SIMDE_FLOAT32_C(     0.06),
        SIMDE_FLOAT32_C(   484.30), SIMDE_FLOAT32_C(   581.54), SIMDE_FLOAT32_C(   977.62), SIMDE_FLOAT32_C(   863.38),
        SIMDE_FLOAT32_C(    41.36), SIMDE_FLOAT32_C(   805.09), SIMDE_FLOAT32_C(   677.49), SIMDE_FLOAT32_C(   796.45) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_invsqrt_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_invsqrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    38.73), SIMDE_FLOAT64_C(    19.20), SIMDE_FLOAT64_C(   260.68), SIMDE_FLOAT64_C(   258.52),
        SIMDE_FLOAT64_C(   136.00), SIMDE_FLOAT64_C(   121.97), SIMDE_FLOAT64_C(   936.95), SIMDE_FLOAT64_C(   333.67) },
      { SIMDE_FLOAT64_C(     0.16), SIMDE_FLOAT64_C(     0.23), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.06),
        SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.05) } },
    { { SIMDE_FLOAT64_C(   609.86), SIMDE_FLOAT64_C(   837.14), SIMDE_FLOAT64_C(   372.68), SIMDE_FLOAT64_C(   549.80),
        SIMDE_FLOAT64_C(   402.57), SIMDE_FLOAT64_C(   960.80), SIMDE_FLOAT64_C(   489.90), SIMDE_FLOAT64_C(   885.65) },
      { SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.04),
        SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.03) } },
    { { SIMDE_FLOAT64_C(   875.53), SIMDE_FLOAT64_C(   411.92), SIMDE_FLOAT64_C(   548.19), SIMDE_FLOAT64_C(   708.42),
        SIMDE_FLOAT64_C(   455.90), SIMDE_FLOAT64_C(   110.13), SIMDE_FLOAT64_C(    88.56), SIMDE_FLOAT64_C(   499.24) },
      { SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.04),
        SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(   161.32), SIMDE_FLOAT64_C(   442.19), SIMDE_FLOAT64_C(   573.08), SIMDE_FLOAT64_C(   621.10),
        SIMDE_FLOAT64_C(   338.32), SIMDE_FLOAT64_C(   172.08), SIMDE_FLOAT64_C(   822.98), SIMDE_FLOAT64_C(   377.05) },
      { SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.04),
        SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.05) } },
    { { SIMDE_FLOAT64_C(   191.28), SIMDE_FLOAT64_C(    83.66), SIMDE_FLOAT64_C(   635.57), SIMDE_FLOAT64_C(   327.28),
        SIMDE_FLOAT64_C(   205.63), SIMDE_FLOAT64_C(   572.53), SIMDE_FLOAT64_C(   660.94), SIMDE_FLOAT64_C(   815.49) },
      { SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.06),
        SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(   409.67), SIMDE_FLOAT64_C(    33.63), SIMDE_FLOAT64_C(   365.30), SIMDE_FLOAT64_C(   812.24),
        SIMDE_FLOAT64_C(   994.43), SIMDE_FLOAT64_C(   855.19), SIMDE_FLOAT64_C(   697.89), SIMDE_FLOAT64_C(   869.96) },
      { SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.04),
        SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.03) } },
    { { SIMDE_FLOAT64_C(   267.11), SIMDE_FLOAT64_C(   246.07), SIMDE_FLOAT64_C(   578.38), SIMDE_FLOAT64_C(   723.01),
        SIMDE_FLOAT64_C(   356.21), SIMDE_FLOAT64_C(   666.94), SIMDE_FLOAT64_C(   222.25), SIMDE_FLOAT64_C(   517.53) },
      { SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.04),
        SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.04) } },
    { { SIMDE_FLOAT64_C(   109.13), SIMDE_FLOAT64_C(   795.33), SIMDE_FLOAT64_C(   138.62), SIMDE_FLOAT64_C(   447.45),
        SIMDE_FLOAT64_C(   967.41), SIMDE_FLOAT64_C(   961.61), SIMDE_FLOAT64_C(   824.50), SIMDE_FLOAT64_C(   158.69) },
      { SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     0.05),
        SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.08) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_invsqrt_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_invsqrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   784.96), SIMDE_FLOAT64_C(   815.29), SIMDE_FLOAT64_C(   578.00), SIMDE_FLOAT64_C(   693.34),
        SIMDE_FLOAT64_C(   899.84), SIMDE_FLOAT64_C(   476.45), SIMDE_FLOAT64_C(   558.50), SIMDE_FLOAT64_C(   745.07) },
      UINT8_C( 77),
      { SIMDE_FLOAT64_C(   864.69), SIMDE_FLOAT64_C(   953.84), SIMDE_FLOAT64_C(   134.83), SIMDE_FLOAT64_C(   167.75),
        SIMDE_FLOAT64_C(   474.65), SIMDE_FLOAT64_C(   536.52), SIMDE_FLOAT64_C(   563.54), SIMDE_FLOAT64_C(   963.69) },
      { SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(   815.29), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     0.08),
        SIMDE_FLOAT64_C(   899.84), SIMDE_FLOAT64_C(   476.45), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(   745.07) } },
    { { SIMDE_FLOAT64_C(   410.86), SIMDE_FLOAT64_C(   470.77), SIMDE_FLOAT64_C(   329.50), SIMDE_FLOAT64_C(    65.82),
        SIMDE_FLOAT64_C(   510.47), SIMDE_FLOAT64_C(   748.64), SIMDE_FLOAT64_C(   130.13), SIMDE_FLOAT64_C(   819.32) },
      UINT8_C(180),
      { SIMDE_FLOAT64_C(   969.69), SIMDE_FLOAT64_C(   176.66), SIMDE_FLOAT64_C(   270.39), SIMDE_FLOAT64_C(    73.35),
        SIMDE_FLOAT64_C(   618.94), SIMDE_FLOAT64_C(    55.36), SIMDE_FLOAT64_C(   888.64), SIMDE_FLOAT64_C(   196.94) },
      { SIMDE_FLOAT64_C(   410.86), SIMDE_FLOAT64_C(   470.77), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(    65.82),
        SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     0.13), SIMDE_FLOAT64_C(   130.13), SIMDE_FLOAT64_C(     0.07) } },
    { { SIMDE_FLOAT64_C(   748.70), SIMDE_FLOAT64_C(   788.48), SIMDE_FLOAT64_C(   673.39), SIMDE_FLOAT64_C(   307.20),
        SIMDE_FLOAT64_C(   533.54), SIMDE_FLOAT64_C(   118.92), SIMDE_FLOAT64_C(   171.90), SIMDE_FLOAT64_C(   487.39) },
      UINT8_C( 67),
      { SIMDE_FLOAT64_C(   339.65), SIMDE_FLOAT64_C(   962.04), SIMDE_FLOAT64_C(   790.27), SIMDE_FLOAT64_C(   903.19),
        SIMDE_FLOAT64_C(   925.73), SIMDE_FLOAT64_C(   201.14), SIMDE_FLOAT64_C(   373.95), SIMDE_FLOAT64_C(   255.23) },
      { SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(   673.39), SIMDE_FLOAT64_C(   307.20),
        SIMDE_FLOAT64_C(   533.54), SIMDE_FLOAT64_C(   118.92), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(   487.39) } },
    { { SIMDE_FLOAT64_C(   266.96), SIMDE_FLOAT64_C(   884.43), SIMDE_FLOAT64_C(     3.88), SIMDE_FLOAT64_C(   397.10),
        SIMDE_FLOAT64_C(   703.75), SIMDE_FLOAT64_C(   335.69), SIMDE_FLOAT64_C(   366.79), SIMDE_FLOAT64_C(   880.41) },
      UINT8_C(138),
      { SIMDE_FLOAT64_C(   440.13), SIMDE_FLOAT64_C(   499.35), SIMDE_FLOAT64_C(   661.44), SIMDE_FLOAT64_C(   328.77),
        SIMDE_FLOAT64_C(   696.29), SIMDE_FLOAT64_C(   410.14), SIMDE_FLOAT64_C(   117.25), SIMDE_FLOAT64_C(   369.69) },
      { SIMDE_FLOAT64_C(   266.96), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     3.88), SIMDE_FLOAT64_C(     0.06),
        SIMDE_FLOAT64_C(   703.75), SIMDE_FLOAT64_C(   335.69), SIMDE_FLOAT64_C(   366.79), SIMDE_FLOAT64_C(     0.05) } },
    { { SIMDE_FLOAT64_C(   717.34), SIMDE_FLOAT64_C(   650.79), SIMDE_FLOAT64_C(   488.60), SIMDE_FLOAT64_C(   889.24),
        SIMDE_FLOAT64_C(   138.18), SIMDE_FLOAT64_C(   742.35), SIMDE_FLOAT64_C(   228.88), SIMDE_FLOAT64_C(   100.22) },
      UINT8_C(  3),
      { SIMDE_FLOAT64_C(   132.07), SIMDE_FLOAT64_C(    25.94), SIMDE_FLOAT64_C(   733.76), SIMDE_FLOAT64_C(   506.02),
        SIMDE_FLOAT64_C(   281.17), SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(   390.45), SIMDE_FLOAT64_C(   285.05) },
      { SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(   488.60), SIMDE_FLOAT64_C(   889.24),
        SIMDE_FLOAT64_C(   138.18), SIMDE_FLOAT64_C(   742.35), SIMDE_FLOAT64_C(   228.88), SIMDE_FLOAT64_C(   100.22) } },
    { { SIMDE_FLOAT64_C(   397.82), SIMDE_FLOAT64_C(    94.20), SIMDE_FLOAT64_C(   620.74), SIMDE_FLOAT64_C(   764.60),
        SIMDE_FLOAT64_C(   974.61), SIMDE_FLOAT64_C(   226.82), SIMDE_FLOAT64_C(   204.74), SIMDE_FLOAT64_C(   473.96) },
      UINT8_C(205),
      { SIMDE_FLOAT64_C(   533.51), SIMDE_FLOAT64_C(   170.26), SIMDE_FLOAT64_C(   298.40), SIMDE_FLOAT64_C(   650.76),
        SIMDE_FLOAT64_C(   539.94), SIMDE_FLOAT64_C(    15.74), SIMDE_FLOAT64_C(   301.54), SIMDE_FLOAT64_C(    28.54) },
      { SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(    94.20), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.04),
        SIMDE_FLOAT64_C(   974.61), SIMDE_FLOAT64_C(   226.82), SIMDE_FLOAT64_C(     0.06), SIMDE_FLOAT64_C(     0.19) } },
    { { SIMDE_FLOAT64_C(   904.98), SIMDE_FLOAT64_C(   439.72), SIMDE_FLOAT64_C(   770.90), SIMDE_FLOAT64_C(   133.86),
        SIMDE_FLOAT64_C(   539.94), SIMDE_FLOAT64_C(   303.52), SIMDE_FLOAT64_C(   265.93), SIMDE_FLOAT64_C(   565.88) },
      UINT8_C( 41),
      { SIMDE_FLOAT64_C(   771.96), SIMDE_FLOAT64_C(   847.05), SIMDE_FLOAT64_C(    38.01), SIMDE_FLOAT64_C(   162.41),
        SIMDE_FLOAT64_C(   132.10), SIMDE_FLOAT64_C(   435.83), SIMDE_FLOAT64_C(   256.61), SIMDE_FLOAT64_C(   752.84) },
      { SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(   439.72), SIMDE_FLOAT64_C(   770.90), SIMDE_FLOAT64_C(     0.08),
        SIMDE_FLOAT64_C(   539.94), SIMDE_FLOAT64_C(     0.05), SIMDE_FLOAT64_C(   265.93), SIMDE_FLOAT64_C(   565.88) } },
    { { SIMDE_FLOAT64_C(   200.43), SIMDE_FLOAT64_C(   231.22), SIMDE_FLOAT64_C(   979.66), SIMDE_FLOAT64_C(   405.17),
        SIMDE_FLOAT64_C(   705.18), SIMDE_FLOAT64_C(   867.92), SIMDE_FLOAT64_C(   938.68), SIMDE_FLOAT64_C(   875.43) },
      UINT8_C( 32),
      { SIMDE_FLOAT64_C(   589.43), SIMDE_FLOAT64_C(   415.38), SIMDE_FLOAT64_C(   182.05), SIMDE_FLOAT64_C(   890.98),
        SIMDE_FLOAT64_C(   443.92), SIMDE_FLOAT64_C(    87.03), SIMDE_FLOAT64_C(   330.70), SIMDE_FLOAT64_C(   214.82) },
      { SIMDE_FLOAT64_C(   200.43), SIMDE_FLOAT64_C(   231.22), SIMDE_FLOAT64_C(   979.66), SIMDE_FLOAT64_C(   405.17),
        SIMDE_FLOAT64_C(   705.18), SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(   938.68), SIMDE_FLOAT64_C(   875.43) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_invsqrt_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_log_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  4068.94), SIMDE_FLOAT32_C(  5195.06), SIMDE_FLOAT32_C(  1228.12), SIMDE_FLOAT32_C(  6733.16)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     8.31), SIMDE_FLOAT32_C(     8.56), SIMDE_FLOAT32_C(     7.11), SIMDE_FLOAT32_C(     8.81)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  7486.55), SIMDE_FLOAT32_C(  8351.20), SIMDE_FLOAT32_C(  3512.77), SIMDE_FLOAT32_C(  5170.29)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     8.92), SIMDE_FLOAT32_C(     9.03), SIMDE_FLOAT32_C(     8.16), SIMDE_FLOAT32_C(     8.55)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  9127.65), SIMDE_FLOAT32_C(  7111.03), SIMDE_FLOAT32_C(  3652.77), SIMDE_FLOAT32_C(  7338.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     8.87), SIMDE_FLOAT32_C(     8.20), SIMDE_FLOAT32_C(     8.90)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  1609.14), SIMDE_FLOAT32_C(  1569.36), SIMDE_FLOAT32_C(  5423.87), SIMDE_FLOAT32_C(  7857.29)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     7.38), SIMDE_FLOAT32_C(     7.36), SIMDE_FLOAT32_C(     8.60), SIMDE_FLOAT32_C(     8.97)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  3474.63), SIMDE_FLOAT32_C(   695.25), SIMDE_FLOAT32_C(  2912.29), SIMDE_FLOAT32_C(  8484.34)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     8.15), SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(     7.98), SIMDE_FLOAT32_C(     9.05)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  2775.95), SIMDE_FLOAT32_C(  5142.35), SIMDE_FLOAT32_C(  3079.83), SIMDE_FLOAT32_C(   381.82)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     7.93), SIMDE_FLOAT32_C(     8.55), SIMDE_FLOAT32_C(     8.03), SIMDE_FLOAT32_C(     5.94)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  6306.54), SIMDE_FLOAT32_C(  3937.29), SIMDE_FLOAT32_C(   117.23), SIMDE_FLOAT32_C(  1696.00)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     8.75), SIMDE_FLOAT32_C(     8.28), SIMDE_FLOAT32_C(     4.76), SIMDE_FLOAT32_C(     7.44)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  5890.98), SIMDE_FLOAT32_C(  2746.67), SIMDE_FLOAT32_C(  6166.85), SIMDE_FLOAT32_C(  8435.45)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     8.68), SIMDE_FLOAT32_C(     7.92), SIMDE_FLOAT32_C(     8.73), SIMDE_FLOAT32_C(     9.04)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_log_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_log_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 1228.12), SIMDE_FLOAT64_C( 6733.16)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    7.11), SIMDE_FLOAT64_C(    8.81)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 4068.94), SIMDE_FLOAT64_C( 5195.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    8.31), SIMDE_FLOAT64_C(    8.56)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 3512.77), SIMDE_FLOAT64_C( 5170.29)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    8.16), SIMDE_FLOAT64_C(    8.55)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 7486.55), SIMDE_FLOAT64_C( 8351.20)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    8.92), SIMDE_FLOAT64_C(    9.03)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 3652.77), SIMDE_FLOAT64_C( 7338.80)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    8.20), SIMDE_FLOAT64_C(    8.90)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 9127.65), SIMDE_FLOAT64_C( 7111.03)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    9.12), SIMDE_FLOAT64_C(    8.87)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 5423.87), SIMDE_FLOAT64_C( 7857.29)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    8.60), SIMDE_FLOAT64_C(    8.97)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 1609.14), SIMDE_FLOAT64_C( 1569.36)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    7.38), SIMDE_FLOAT64_C(    7.36)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_log_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_log_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  7486.55), SIMDE_FLOAT32_C(  8351.20),
                         SIMDE_FLOAT32_C(  3512.77), SIMDE_FLOAT32_C(  5170.29),
                         SIMDE_FLOAT32_C(  4068.94), SIMDE_FLOAT32_C(  5195.06),
                         SIMDE_FLOAT32_C(  1228.12), SIMDE_FLOAT32_C(  6733.16)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     8.92), SIMDE_FLOAT32_C(     9.03),
                         SIMDE_FLOAT32_C(     8.16), SIMDE_FLOAT32_C(     8.55),
                         SIMDE_FLOAT32_C(     8.31), SIMDE_FLOAT32_C(     8.56),
                         SIMDE_FLOAT32_C(     7.11), SIMDE_FLOAT32_C(     8.81)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  1609.14), SIMDE_FLOAT32_C(  1569.36),
                         SIMDE_FLOAT32_C(  5423.87), SIMDE_FLOAT32_C(  7857.29),
                         SIMDE_FLOAT32_C(  9127.65), SIMDE_FLOAT32_C(  7111.03),
                         SIMDE_FLOAT32_C(  3652.77), SIMDE_FLOAT32_C(  7338.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     7.38), SIMDE_FLOAT32_C(     7.36),
                         SIMDE_FLOAT32_C(     8.60), SIMDE_FLOAT32_C(     8.97),
                         SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     8.87),
                         SIMDE_FLOAT32_C(     8.20), SIMDE_FLOAT32_C(     8.90)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  2775.95), SIMDE_FLOAT32_C(  5142.35),
                         SIMDE_FLOAT32_C(  3079.83), SIMDE_FLOAT32_C(   381.82),
                         SIMDE_FLOAT32_C(  3474.63), SIMDE_FLOAT32_C(   695.25),
                         SIMDE_FLOAT32_C(  2912.29), SIMDE_FLOAT32_C(  8484.34)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     7.93), SIMDE_FLOAT32_C(     8.55),
                         SIMDE_FLOAT32_C(     8.03), SIMDE_FLOAT32_C(     5.94),
                         SIMDE_FLOAT32_C(     8.15), SIMDE_FLOAT32_C(     6.54),
                         SIMDE_FLOAT32_C(     7.98), SIMDE_FLOAT32_C(     9.05)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  5890.98), SIMDE_FLOAT32_C(  2746.67),
                         SIMDE_FLOAT32_C(  6166.85), SIMDE_FLOAT32_C(  8435.45),
                         SIMDE_FLOAT32_C(  6306.54), SIMDE_FLOAT32_C(  3937.29),
                         SIMDE_FLOAT32_C(   117.23), SIMDE_FLOAT32_C(  1696.00)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     8.68), SIMDE_FLOAT32_C(     7.92),
                         SIMDE_FLOAT32_C(     8.73), SIMDE_FLOAT32_C(     9.04),
                         SIMDE_FLOAT32_C(     8.75), SIMDE_FLOAT32_C(     8.28),
                         SIMDE_FLOAT32_C(     4.76), SIMDE_FLOAT32_C(     7.44)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  1148.23), SIMDE_FLOAT32_C(  7217.40),
                         SIMDE_FLOAT32_C(  2082.02), SIMDE_FLOAT32_C(  6902.28),
                         SIMDE_FLOAT32_C(  1146.40), SIMDE_FLOAT32_C(  9969.51),
                         SIMDE_FLOAT32_C(  5140.40), SIMDE_FLOAT32_C(  9206.03)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     7.05), SIMDE_FLOAT32_C(     8.88),
                         SIMDE_FLOAT32_C(     7.64), SIMDE_FLOAT32_C(     8.84),
                         SIMDE_FLOAT32_C(     7.04), SIMDE_FLOAT32_C(     9.21),
                         SIMDE_FLOAT32_C(     8.54), SIMDE_FLOAT32_C(     9.13)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  3060.52), SIMDE_FLOAT32_C(  6979.60),
                         SIMDE_FLOAT32_C(  8279.36), SIMDE_FLOAT32_C(  6696.04),
                         SIMDE_FLOAT32_C(  7661.76), SIMDE_FLOAT32_C(  3680.04),
                         SIMDE_FLOAT32_C(  8903.22), SIMDE_FLOAT32_C(  4846.05)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     8.03), SIMDE_FLOAT32_C(     8.85),
                         SIMDE_FLOAT32_C(     9.02), SIMDE_FLOAT32_C(     8.81),
                         SIMDE_FLOAT32_C(     8.94), SIMDE_FLOAT32_C(     8.21),
                         SIMDE_FLOAT32_C(     9.09), SIMDE_FLOAT32_C(     8.49)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  3981.75), SIMDE_FLOAT32_C(  4596.36),
                         SIMDE_FLOAT32_C(  6683.64), SIMDE_FLOAT32_C(   276.11),
                         SIMDE_FLOAT32_C(  1262.07), SIMDE_FLOAT32_C(  1163.84),
                         SIMDE_FLOAT32_C(  2229.06), SIMDE_FLOAT32_C(  6994.08)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     8.29), SIMDE_FLOAT32_C(     8.43),
                         SIMDE_FLOAT32_C(     8.81), SIMDE_FLOAT32_C(     5.62),
                         SIMDE_FLOAT32_C(     7.14), SIMDE_FLOAT32_C(     7.06),
                         SIMDE_FLOAT32_C(     7.71), SIMDE_FLOAT32_C(     8.85)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  7348.31), SIMDE_FLOAT32_C(  8400.08),
                         SIMDE_FLOAT32_C(  4256.55), SIMDE_FLOAT32_C(  9093.31),
                         SIMDE_FLOAT32_C(  9550.14), SIMDE_FLOAT32_C(  8002.34),
                         SIMDE_FLOAT32_C(  8956.15), SIMDE_FLOAT32_C(  6271.53)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     8.90), SIMDE_FLOAT32_C(     9.04),
                         SIMDE_FLOAT32_C(     8.36), SIMDE_FLOAT32_C(     9.12),
                         SIMDE_FLOAT32_C(     9.16), SIMDE_FLOAT32_C(     8.99),
                         SIMDE_FLOAT32_C(     9.10), SIMDE_FLOAT32_C(     8.74)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_log_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_log_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 4068.94), SIMDE_FLOAT64_C( 5195.06),
                         SIMDE_FLOAT64_C( 1228.12), SIMDE_FLOAT64_C( 6733.16)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    8.31), SIMDE_FLOAT64_C(    8.56),
                         SIMDE_FLOAT64_C(    7.11), SIMDE_FLOAT64_C(    8.81)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 7486.55), SIMDE_FLOAT64_C( 8351.20),
                         SIMDE_FLOAT64_C( 3512.77), SIMDE_FLOAT64_C( 5170.29)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    8.92), SIMDE_FLOAT64_C(    9.03),
                         SIMDE_FLOAT64_C(    8.16), SIMDE_FLOAT64_C(    8.55)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 9127.65), SIMDE_FLOAT64_C( 7111.03),
                         SIMDE_FLOAT64_C( 3652.77), SIMDE_FLOAT64_C( 7338.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    9.12), SIMDE_FLOAT64_C(    8.87),
                         SIMDE_FLOAT64_C(    8.20), SIMDE_FLOAT64_C(    8.90)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 1609.14), SIMDE_FLOAT64_C( 1569.36),
                         SIMDE_FLOAT64_C( 5423.87), SIMDE_FLOAT64_C( 7857.29)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    7.38), SIMDE_FLOAT64_C(    7.36),
                         SIMDE_FLOAT64_C(    8.60), SIMDE_FLOAT64_C(    8.97)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 3474.63), SIMDE_FLOAT64_C(  695.25),
                         SIMDE_FLOAT64_C( 2912.29), SIMDE_FLOAT64_C( 8484.34)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    8.15), SIMDE_FLOAT64_C(    6.54),
                         SIMDE_FLOAT64_C(    7.98), SIMDE_FLOAT64_C(    9.05)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 2775.95), SIMDE_FLOAT64_C( 5142.35),
                         SIMDE_FLOAT64_C( 3079.83), SIMDE_FLOAT64_C(  381.82)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    7.93), SIMDE_FLOAT64_C(    8.55),
                         SIMDE_FLOAT64_C(    8.03), SIMDE_FLOAT64_C(    5.94)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 6306.54), SIMDE_FLOAT64_C( 3937.29),
                         SIMDE_FLOAT64_C(  117.23), SIMDE_FLOAT64_C( 1696.00)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    8.75), SIMDE_FLOAT64_C(    8.28),
                         SIMDE_FLOAT64_C(    4.76), SIMDE_FLOAT64_C(    7.44)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 5890.98), SIMDE_FLOAT64_C( 2746.67),
                         SIMDE_FLOAT64_C( 6166.85), SIMDE_FLOAT64_C( 8435.45)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    8.68), SIMDE_FLOAT64_C(    7.92),
                         SIMDE_FLOAT64_C(    8.73), SIMDE_FLOAT64_C(    9.04)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_log_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_log_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  1609.14), SIMDE_FLOAT32_C(  1569.36), SIMDE_FLOAT32_C(  5423.87), SIMDE_FLOAT32_C(  7857.29),
                         SIMDE_FLOAT32_C(  9127.65), SIMDE_FLOAT32_C(  7111.03), SIMDE_FLOAT32_C(  3652.77), SIMDE_FLOAT32_C(  7338.80),
                         SIMDE_FLOAT32_C(  7486.55), SIMDE_FLOAT32_C(  8351.20), SIMDE_FLOAT32_C(  3512.77), SIMDE_FLOAT32_C(  5170.29),
                         SIMDE_FLOAT32_C(  4068.94), SIMDE_FLOAT32_C(  5195.06), SIMDE_FLOAT32_C(  1228.12), SIMDE_FLOAT32_C(  6733.16)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.38), SIMDE_FLOAT32_C(     7.36), SIMDE_FLOAT32_C(     8.60), SIMDE_FLOAT32_C(     8.97),
                         SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     8.87), SIMDE_FLOAT32_C(     8.20), SIMDE_FLOAT32_C(     8.90),
                         SIMDE_FLOAT32_C(     8.92), SIMDE_FLOAT32_C(     9.03), SIMDE_FLOAT32_C(     8.16), SIMDE_FLOAT32_C(     8.55),
                         SIMDE_FLOAT32_C(     8.31), SIMDE_FLOAT32_C(     8.56), SIMDE_FLOAT32_C(     7.11), SIMDE_FLOAT32_C(     8.81)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  5890.98), SIMDE_FLOAT32_C(  2746.67), SIMDE_FLOAT32_C(  6166.85), SIMDE_FLOAT32_C(  8435.45),
                         SIMDE_FLOAT32_C(  6306.54), SIMDE_FLOAT32_C(  3937.29), SIMDE_FLOAT32_C(   117.23), SIMDE_FLOAT32_C(  1696.00),
                         SIMDE_FLOAT32_C(  2775.95), SIMDE_FLOAT32_C(  5142.35), SIMDE_FLOAT32_C(  3079.83), SIMDE_FLOAT32_C(   381.82),
                         SIMDE_FLOAT32_C(  3474.63), SIMDE_FLOAT32_C(   695.25), SIMDE_FLOAT32_C(  2912.29), SIMDE_FLOAT32_C(  8484.34)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     8.68), SIMDE_FLOAT32_C(     7.92), SIMDE_FLOAT32_C(     8.73), SIMDE_FLOAT32_C(     9.04),
                         SIMDE_FLOAT32_C(     8.75), SIMDE_FLOAT32_C(     8.28), SIMDE_FLOAT32_C(     4.76), SIMDE_FLOAT32_C(     7.44),
                         SIMDE_FLOAT32_C(     7.93), SIMDE_FLOAT32_C(     8.55), SIMDE_FLOAT32_C(     8.03), SIMDE_FLOAT32_C(     5.94),
                         SIMDE_FLOAT32_C(     8.15), SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(     7.98), SIMDE_FLOAT32_C(     9.05)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  3060.52), SIMDE_FLOAT32_C(  6979.60), SIMDE_FLOAT32_C(  8279.36), SIMDE_FLOAT32_C(  6696.04),
                         SIMDE_FLOAT32_C(  7661.76), SIMDE_FLOAT32_C(  3680.04), SIMDE_FLOAT32_C(  8903.22), SIMDE_FLOAT32_C(  4846.05),
                         SIMDE_FLOAT32_C(  1148.23), SIMDE_FLOAT32_C(  7217.40), SIMDE_FLOAT32_C(  2082.02), SIMDE_FLOAT32_C(  6902.28),
                         SIMDE_FLOAT32_C(  1146.40), SIMDE_FLOAT32_C(  9969.51), SIMDE_FLOAT32_C(  5140.40), SIMDE_FLOAT32_C(  9206.03)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     8.03), SIMDE_FLOAT32_C(     8.85), SIMDE_FLOAT32_C(     9.02), SIMDE_FLOAT32_C(     8.81),
                         SIMDE_FLOAT32_C(     8.94), SIMDE_FLOAT32_C(     8.21), SIMDE_FLOAT32_C(     9.09), SIMDE_FLOAT32_C(     8.49),
                         SIMDE_FLOAT32_C(     7.05), SIMDE_FLOAT32_C(     8.88), SIMDE_FLOAT32_C(     7.64), SIMDE_FLOAT32_C(     8.84),
                         SIMDE_FLOAT32_C(     7.04), SIMDE_FLOAT32_C(     9.21), SIMDE_FLOAT32_C(     8.54), SIMDE_FLOAT32_C(     9.13)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  7348.31), SIMDE_FLOAT32_C(  8400.08), SIMDE_FLOAT32_C(  4256.55), SIMDE_FLOAT32_C(  9093.31),
                         SIMDE_FLOAT32_C(  9550.14), SIMDE_FLOAT32_C(  8002.34), SIMDE_FLOAT32_C(  8956.15), SIMDE_FLOAT32_C(  6271.53),
                         SIMDE_FLOAT32_C(  3981.75), SIMDE_FLOAT32_C(  4596.36), SIMDE_FLOAT32_C(  6683.64), SIMDE_FLOAT32_C(   276.11),
                         SIMDE_FLOAT32_C(  1262.07), SIMDE_FLOAT32_C(  1163.84), SIMDE_FLOAT32_C(  2229.06), SIMDE_FLOAT32_C(  6994.08)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     8.90), SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(     8.36), SIMDE_FLOAT32_C(     9.12),
                         SIMDE_FLOAT32_C(     9.16), SIMDE_FLOAT32_C(     8.99), SIMDE_FLOAT32_C(     9.10), SIMDE_FLOAT32_C(     8.74),
                         SIMDE_FLOAT32_C(     8.29), SIMDE_FLOAT32_C(     8.43), SIMDE_FLOAT32_C(     8.81), SIMDE_FLOAT32_C(     5.62),
                         SIMDE_FLOAT32_C(     7.14), SIMDE_FLOAT32_C(     7.06), SIMDE_FLOAT32_C(     7.71), SIMDE_FLOAT32_C(     8.85)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  4105.04), SIMDE_FLOAT32_C(  8793.93), SIMDE_FLOAT32_C(  6623.12), SIMDE_FLOAT32_C(  6717.40),
                         SIMDE_FLOAT32_C(   628.43), SIMDE_FLOAT32_C(  1010.42), SIMDE_FLOAT32_C(  3357.32), SIMDE_FLOAT32_C(  2370.85),
                         SIMDE_FLOAT32_C(  4038.44), SIMDE_FLOAT32_C(   886.73), SIMDE_FLOAT32_C(  7806.81), SIMDE_FLOAT32_C(  8278.35),
                         SIMDE_FLOAT32_C(  4645.43), SIMDE_FLOAT32_C(  7716.73), SIMDE_FLOAT32_C(  5603.27), SIMDE_FLOAT32_C(  4142.45)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     8.32), SIMDE_FLOAT32_C(     9.08), SIMDE_FLOAT32_C(     8.80), SIMDE_FLOAT32_C(     8.81),
                         SIMDE_FLOAT32_C(     6.44), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(     8.12), SIMDE_FLOAT32_C(     7.77),
                         SIMDE_FLOAT32_C(     8.30), SIMDE_FLOAT32_C(     6.79), SIMDE_FLOAT32_C(     8.96), SIMDE_FLOAT32_C(     9.02),
                         SIMDE_FLOAT32_C(     8.44), SIMDE_FLOAT32_C(     8.95), SIMDE_FLOAT32_C(     8.63), SIMDE_FLOAT32_C(     8.33)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  8450.59), SIMDE_FLOAT32_C(  9203.26), SIMDE_FLOAT32_C(  4894.53), SIMDE_FLOAT32_C(  2042.18),
                         SIMDE_FLOAT32_C(  2755.53), SIMDE_FLOAT32_C(  8657.47), SIMDE_FLOAT32_C(  7528.93), SIMDE_FLOAT32_C(  8118.50),
                         SIMDE_FLOAT32_C(  9155.11), SIMDE_FLOAT32_C(  5703.37), SIMDE_FLOAT32_C(  9886.80), SIMDE_FLOAT32_C(   469.19),
                         SIMDE_FLOAT32_C(  6656.71), SIMDE_FLOAT32_C(  5499.67), SIMDE_FLOAT32_C(  7314.76), SIMDE_FLOAT32_C(  1309.05)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(     9.13), SIMDE_FLOAT32_C(     8.50), SIMDE_FLOAT32_C(     7.62),
                         SIMDE_FLOAT32_C(     7.92), SIMDE_FLOAT32_C(     9.07), SIMDE_FLOAT32_C(     8.93), SIMDE_FLOAT32_C(     9.00),
                         SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     8.65), SIMDE_FLOAT32_C(     9.20), SIMDE_FLOAT32_C(     6.15),
                         SIMDE_FLOAT32_C(     8.80), SIMDE_FLOAT32_C(     8.61), SIMDE_FLOAT32_C(     8.90), SIMDE_FLOAT32_C(     7.18)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  1154.54), SIMDE_FLOAT32_C(  9110.29), SIMDE_FLOAT32_C(  2130.97), SIMDE_FLOAT32_C(    11.83),
                         SIMDE_FLOAT32_C(  3312.02), SIMDE_FLOAT32_C(  9618.20), SIMDE_FLOAT32_C(  6468.19), SIMDE_FLOAT32_C(  1159.42),
                         SIMDE_FLOAT32_C(  2118.90), SIMDE_FLOAT32_C(  4661.80), SIMDE_FLOAT32_C(  8551.88), SIMDE_FLOAT32_C(  9887.44),
                         SIMDE_FLOAT32_C(  1217.92), SIMDE_FLOAT32_C(  7124.06), SIMDE_FLOAT32_C(  5136.26), SIMDE_FLOAT32_C(  4524.23)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.05), SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     7.66), SIMDE_FLOAT32_C(     2.47),
                         SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     9.17), SIMDE_FLOAT32_C(     8.77), SIMDE_FLOAT32_C(     7.06),
                         SIMDE_FLOAT32_C(     7.66), SIMDE_FLOAT32_C(     8.45), SIMDE_FLOAT32_C(     9.05), SIMDE_FLOAT32_C(     9.20),
                         SIMDE_FLOAT32_C(     7.10), SIMDE_FLOAT32_C(     8.87), SIMDE_FLOAT32_C(     8.54), SIMDE_FLOAT32_C(     8.42)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  2809.03), SIMDE_FLOAT32_C(  3201.22), SIMDE_FLOAT32_C(  1237.85), SIMDE_FLOAT32_C(  4831.67),
                         SIMDE_FLOAT32_C(  9663.28), SIMDE_FLOAT32_C(  5036.36), SIMDE_FLOAT32_C(  3363.90), SIMDE_FLOAT32_C(  4374.02),
                         SIMDE_FLOAT32_C(  4087.77), SIMDE_FLOAT32_C(  5199.67), SIMDE_FLOAT32_C(  7554.25), SIMDE_FLOAT32_C(  6973.34),
                         SIMDE_FLOAT32_C(  5071.68), SIMDE_FLOAT32_C(  3476.37), SIMDE_FLOAT32_C(  9581.30), SIMDE_FLOAT32_C(  1516.57)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.94), SIMDE_FLOAT32_C(     8.07), SIMDE_FLOAT32_C(     7.12), SIMDE_FLOAT32_C(     8.48),
                         SIMDE_FLOAT32_C(     9.18), SIMDE_FLOAT32_C(     8.52), SIMDE_FLOAT32_C(     8.12), SIMDE_FLOAT32_C(     8.38),
                         SIMDE_FLOAT32_C(     8.32), SIMDE_FLOAT32_C(     8.56), SIMDE_FLOAT32_C(     8.93), SIMDE_FLOAT32_C(     8.85),
                         SIMDE_FLOAT32_C(     8.53), SIMDE_FLOAT32_C(     8.15), SIMDE_FLOAT32_C(     9.17), SIMDE_FLOAT32_C(     7.32)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_log_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_log_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  2746.67), SIMDE_FLOAT32_C(  8435.45), SIMDE_FLOAT32_C(  3937.29), SIMDE_FLOAT32_C(  1696.00),
                         SIMDE_FLOAT32_C(  5142.35), SIMDE_FLOAT32_C(   381.82), SIMDE_FLOAT32_C(   695.25), SIMDE_FLOAT32_C(  8484.34),
                         SIMDE_FLOAT32_C(  1569.36), SIMDE_FLOAT32_C(  7857.29), SIMDE_FLOAT32_C(  7111.03), SIMDE_FLOAT32_C(  7338.80),
                         SIMDE_FLOAT32_C(  8351.20), SIMDE_FLOAT32_C(  5170.29), SIMDE_FLOAT32_C(  5195.06), SIMDE_FLOAT32_C(  6733.16)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  5890.98), SIMDE_FLOAT32_C(  6166.85), SIMDE_FLOAT32_C(  6306.54), SIMDE_FLOAT32_C(   117.23),
                         SIMDE_FLOAT32_C(  2775.95), SIMDE_FLOAT32_C(  3079.83), SIMDE_FLOAT32_C(  3474.63), SIMDE_FLOAT32_C(  2912.29),
                         SIMDE_FLOAT32_C(  1609.14), SIMDE_FLOAT32_C(  5423.87), SIMDE_FLOAT32_C(  9127.65), SIMDE_FLOAT32_C(  3652.77),
                         SIMDE_FLOAT32_C(  7486.55), SIMDE_FLOAT32_C(  3512.77), SIMDE_FLOAT32_C(  4068.94), SIMDE_FLOAT32_C(  1228.12)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     8.68), SIMDE_FLOAT32_C(  8435.45), SIMDE_FLOAT32_C(     8.75), SIMDE_FLOAT32_C(  1696.00),
                         SIMDE_FLOAT32_C(  5142.35), SIMDE_FLOAT32_C(   381.82), SIMDE_FLOAT32_C(   695.25), SIMDE_FLOAT32_C(     7.98),
                         SIMDE_FLOAT32_C(     7.38), SIMDE_FLOAT32_C(     8.60), SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     8.20),
                         SIMDE_FLOAT32_C(     8.92), SIMDE_FLOAT32_C(  5170.29), SIMDE_FLOAT32_C(     8.31), SIMDE_FLOAT32_C(  6733.16)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  7348.31), SIMDE_FLOAT32_C(  4256.55), SIMDE_FLOAT32_C(  9550.14), SIMDE_FLOAT32_C(  8956.15),
                         SIMDE_FLOAT32_C(  3981.75), SIMDE_FLOAT32_C(  6683.64), SIMDE_FLOAT32_C(  1262.07), SIMDE_FLOAT32_C(  2229.06),
                         SIMDE_FLOAT32_C(  3060.52), SIMDE_FLOAT32_C(  8279.36), SIMDE_FLOAT32_C(  7661.76), SIMDE_FLOAT32_C(  8903.22),
                         SIMDE_FLOAT32_C(  1148.23), SIMDE_FLOAT32_C(  2082.02), SIMDE_FLOAT32_C(  1146.40), SIMDE_FLOAT32_C(  5140.40)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  4142.45), SIMDE_FLOAT32_C(  8400.08), SIMDE_FLOAT32_C(  9093.31), SIMDE_FLOAT32_C(  8002.34),
                         SIMDE_FLOAT32_C(  6271.53), SIMDE_FLOAT32_C(  4596.36), SIMDE_FLOAT32_C(   276.11), SIMDE_FLOAT32_C(  1163.84),
                         SIMDE_FLOAT32_C(  6994.08), SIMDE_FLOAT32_C(  6979.60), SIMDE_FLOAT32_C(  6696.04), SIMDE_FLOAT32_C(  3680.04),
                         SIMDE_FLOAT32_C(  4846.05), SIMDE_FLOAT32_C(  7217.40), SIMDE_FLOAT32_C(  6902.28), SIMDE_FLOAT32_C(  9969.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     8.33), SIMDE_FLOAT32_C(  4256.55), SIMDE_FLOAT32_C(  9550.14), SIMDE_FLOAT32_C(  8956.15),
                         SIMDE_FLOAT32_C(     8.74), SIMDE_FLOAT32_C(     8.43), SIMDE_FLOAT32_C(     5.62), SIMDE_FLOAT32_C(     7.06),
                         SIMDE_FLOAT32_C(     8.85), SIMDE_FLOAT32_C(  8279.36), SIMDE_FLOAT32_C(     8.81), SIMDE_FLOAT32_C(     8.21),
                         SIMDE_FLOAT32_C(     8.49), SIMDE_FLOAT32_C(     8.88), SIMDE_FLOAT32_C(  1146.40), SIMDE_FLOAT32_C(     9.21)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  4524.23), SIMDE_FLOAT32_C(  9203.26), SIMDE_FLOAT32_C(  2042.18), SIMDE_FLOAT32_C(  8657.47),
                         SIMDE_FLOAT32_C(  8118.50), SIMDE_FLOAT32_C(  5703.37), SIMDE_FLOAT32_C(   469.19), SIMDE_FLOAT32_C(  5499.67),
                         SIMDE_FLOAT32_C(  1309.05), SIMDE_FLOAT32_C(  8793.93), SIMDE_FLOAT32_C(  6717.40), SIMDE_FLOAT32_C(  1010.42),
                         SIMDE_FLOAT32_C(  2370.85), SIMDE_FLOAT32_C(   886.73), SIMDE_FLOAT32_C(  8278.35), SIMDE_FLOAT32_C(  7716.73)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  5136.26), SIMDE_FLOAT32_C(  8450.59), SIMDE_FLOAT32_C(  4894.53), SIMDE_FLOAT32_C(  2755.53),
                         SIMDE_FLOAT32_C(  7528.93), SIMDE_FLOAT32_C(  9155.11), SIMDE_FLOAT32_C(  9886.80), SIMDE_FLOAT32_C(  6656.71),
                         SIMDE_FLOAT32_C(  7314.76), SIMDE_FLOAT32_C(  4105.04), SIMDE_FLOAT32_C(  6623.12), SIMDE_FLOAT32_C(   628.43),
                         SIMDE_FLOAT32_C(  3357.32), SIMDE_FLOAT32_C(  4038.44), SIMDE_FLOAT32_C(  7806.81), SIMDE_FLOAT32_C(  4645.43)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  4524.23), SIMDE_FLOAT32_C(     9.04), SIMDE_FLOAT32_C(  2042.18), SIMDE_FLOAT32_C(  8657.47),
                         SIMDE_FLOAT32_C(  8118.50), SIMDE_FLOAT32_C(  5703.37), SIMDE_FLOAT32_C(   469.19), SIMDE_FLOAT32_C(     8.80),
                         SIMDE_FLOAT32_C(     8.90), SIMDE_FLOAT32_C(  8793.93), SIMDE_FLOAT32_C(     8.80), SIMDE_FLOAT32_C(  1010.42),
                         SIMDE_FLOAT32_C(  2370.85), SIMDE_FLOAT32_C(     8.30), SIMDE_FLOAT32_C(  8278.35), SIMDE_FLOAT32_C(  7716.73)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  3256.50), SIMDE_FLOAT32_C(  2809.03), SIMDE_FLOAT32_C(  1237.85), SIMDE_FLOAT32_C(  9663.28),
                         SIMDE_FLOAT32_C(  3363.90), SIMDE_FLOAT32_C(  4087.77), SIMDE_FLOAT32_C(  7554.25), SIMDE_FLOAT32_C(  5071.68),
                         SIMDE_FLOAT32_C(  9581.30), SIMDE_FLOAT32_C(  1154.54), SIMDE_FLOAT32_C(  2130.97), SIMDE_FLOAT32_C(  3312.02),
                         SIMDE_FLOAT32_C(  6468.19), SIMDE_FLOAT32_C(  2118.90), SIMDE_FLOAT32_C(  8551.88), SIMDE_FLOAT32_C(  1217.92)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  9486.33), SIMDE_FLOAT32_C(  4010.56), SIMDE_FLOAT32_C(  3201.22), SIMDE_FLOAT32_C(  4831.67),
                         SIMDE_FLOAT32_C(  5036.36), SIMDE_FLOAT32_C(  4374.02), SIMDE_FLOAT32_C(  5199.67), SIMDE_FLOAT32_C(  6973.34),
                         SIMDE_FLOAT32_C(  3476.37), SIMDE_FLOAT32_C(  1516.57), SIMDE_FLOAT32_C(  9110.29), SIMDE_FLOAT32_C(    11.83),
                         SIMDE_FLOAT32_C(  9618.20), SIMDE_FLOAT32_C(  1159.42), SIMDE_FLOAT32_C(  4661.80), SIMDE_FLOAT32_C(  9887.44)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  3256.50), SIMDE_FLOAT32_C(  2809.03), SIMDE_FLOAT32_C(  1237.85), SIMDE_FLOAT32_C(  9663.28),
                         SIMDE_FLOAT32_C(     8.52), SIMDE_FLOAT32_C(  4087.77), SIMDE_FLOAT32_C(  7554.25), SIMDE_FLOAT32_C(  5071.68),
                         SIMDE_FLOAT32_C(  9581.30), SIMDE_FLOAT32_C(  1154.54), SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     2.47),
                         SIMDE_FLOAT32_C(     9.17), SIMDE_FLOAT32_C(  2118.90), SIMDE_FLOAT32_C(     8.45), SIMDE_FLOAT32_C(     9.20)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  4921.97), SIMDE_FLOAT32_C(  1314.36), SIMDE_FLOAT32_C(  3425.34), SIMDE_FLOAT32_C(  5889.62),
                         SIMDE_FLOAT32_C(  6729.66), SIMDE_FLOAT32_C(  9443.57), SIMDE_FLOAT32_C(  9578.53), SIMDE_FLOAT32_C(  5667.58),
                         SIMDE_FLOAT32_C(  7424.68), SIMDE_FLOAT32_C(  2009.69), SIMDE_FLOAT32_C(  1044.67), SIMDE_FLOAT32_C(  1170.36),
                         SIMDE_FLOAT32_C(  6106.86), SIMDE_FLOAT32_C(  1058.19), SIMDE_FLOAT32_C(  1124.78), SIMDE_FLOAT32_C(  7203.19)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  7482.85), SIMDE_FLOAT32_C(  9575.95), SIMDE_FLOAT32_C(  1407.98), SIMDE_FLOAT32_C(  5799.87),
                         SIMDE_FLOAT32_C(   694.94), SIMDE_FLOAT32_C(  7133.07), SIMDE_FLOAT32_C(  9660.54), SIMDE_FLOAT32_C(  5551.82),
                         SIMDE_FLOAT32_C(  9134.21), SIMDE_FLOAT32_C(  4616.24), SIMDE_FLOAT32_C(  6187.92), SIMDE_FLOAT32_C(  3107.51),
                         SIMDE_FLOAT32_C(  1991.62), SIMDE_FLOAT32_C(  1882.51), SIMDE_FLOAT32_C(   287.66), SIMDE_FLOAT32_C(  7377.56)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  4921.97), SIMDE_FLOAT32_C(     9.17), SIMDE_FLOAT32_C(  3425.34), SIMDE_FLOAT32_C(     8.67),
                         SIMDE_FLOAT32_C(  6729.66), SIMDE_FLOAT32_C(     8.87), SIMDE_FLOAT32_C(     9.18), SIMDE_FLOAT32_C(     8.62),
                         SIMDE_FLOAT32_C(  7424.68), SIMDE_FLOAT32_C(  2009.69), SIMDE_FLOAT32_C(  1044.67), SIMDE_FLOAT32_C(  1170.36),
                         SIMDE_FLOAT32_C(  6106.86), SIMDE_FLOAT32_C(  1058.19), SIMDE_FLOAT32_C(     5.66), SIMDE_FLOAT32_C(  7203.19)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  9415.27), SIMDE_FLOAT32_C(   963.59), SIMDE_FLOAT32_C(  4649.74), SIMDE_FLOAT32_C(  1078.30),
                         SIMDE_FLOAT32_C(  5462.61), SIMDE_FLOAT32_C(  6033.01), SIMDE_FLOAT32_C(  9173.00), SIMDE_FLOAT32_C(  4672.02),
                         SIMDE_FLOAT32_C(  3569.65), SIMDE_FLOAT32_C(  3935.68), SIMDE_FLOAT32_C(  3408.08), SIMDE_FLOAT32_C(  8917.42),
                         SIMDE_FLOAT32_C(  1855.90), SIMDE_FLOAT32_C(  7781.74), SIMDE_FLOAT32_C(  7197.17), SIMDE_FLOAT32_C(  7170.16)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.74), SIMDE_FLOAT32_C(  2968.36), SIMDE_FLOAT32_C(  1281.72), SIMDE_FLOAT32_C(  1177.11),
                         SIMDE_FLOAT32_C(  8949.44), SIMDE_FLOAT32_C(  5024.17), SIMDE_FLOAT32_C(   907.29), SIMDE_FLOAT32_C(  5805.32),
                         SIMDE_FLOAT32_C(  7896.24), SIMDE_FLOAT32_C(  4941.12), SIMDE_FLOAT32_C(  3457.39), SIMDE_FLOAT32_C(  1402.13),
                         SIMDE_FLOAT32_C(  6670.00), SIMDE_FLOAT32_C(  6373.56), SIMDE_FLOAT32_C(   415.89), SIMDE_FLOAT32_C(  2550.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  9415.27), SIMDE_FLOAT32_C(     8.00), SIMDE_FLOAT32_C(     7.16), SIMDE_FLOAT32_C(  1078.30),
                         SIMDE_FLOAT32_C(     9.10), SIMDE_FLOAT32_C(  6033.01), SIMDE_FLOAT32_C(     6.81), SIMDE_FLOAT32_C(     8.67),
                         SIMDE_FLOAT32_C(  3569.65), SIMDE_FLOAT32_C(  3935.68), SIMDE_FLOAT32_C(  3408.08), SIMDE_FLOAT32_C(  8917.42),
                         SIMDE_FLOAT32_C(  1855.90), SIMDE_FLOAT32_C(     8.76), SIMDE_FLOAT32_C(  7197.17), SIMDE_FLOAT32_C(  7170.16)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  7648.13), SIMDE_FLOAT32_C(  4875.56), SIMDE_FLOAT32_C(   161.12), SIMDE_FLOAT32_C(  8194.68),
                         SIMDE_FLOAT32_C(  7254.51), SIMDE_FLOAT32_C(  1142.29), SIMDE_FLOAT32_C(  5528.96), SIMDE_FLOAT32_C(  7950.51),
                         SIMDE_FLOAT32_C(  5154.57), SIMDE_FLOAT32_C(  8176.75), SIMDE_FLOAT32_C(  4580.00), SIMDE_FLOAT32_C(  5400.22),
                         SIMDE_FLOAT32_C(  1452.71), SIMDE_FLOAT32_C(  8039.28), SIMDE_FLOAT32_C(  6972.90), SIMDE_FLOAT32_C(   554.46)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  5093.74), SIMDE_FLOAT32_C(  9045.23), SIMDE_FLOAT32_C(  5720.26), SIMDE_FLOAT32_C(  2861.39),
                         SIMDE_FLOAT32_C(  6541.39), SIMDE_FLOAT32_C(  4114.75), SIMDE_FLOAT32_C(  2711.17), SIMDE_FLOAT32_C(  8391.22),
                         SIMDE_FLOAT32_C(  5330.27), SIMDE_FLOAT32_C(  3661.45), SIMDE_FLOAT32_C(  5586.41), SIMDE_FLOAT32_C(  2116.00),
                         SIMDE_FLOAT32_C(  4808.04), SIMDE_FLOAT32_C(  3749.32), SIMDE_FLOAT32_C(  4730.38), SIMDE_FLOAT32_C(  5459.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  7648.13), SIMDE_FLOAT32_C(  4875.56), SIMDE_FLOAT32_C(   161.12), SIMDE_FLOAT32_C(  8194.68),
                         SIMDE_FLOAT32_C(  7254.51), SIMDE_FLOAT32_C(  1142.29), SIMDE_FLOAT32_C(     7.91), SIMDE_FLOAT32_C(     9.03),
                         SIMDE_FLOAT32_C(     8.58), SIMDE_FLOAT32_C(  8176.75), SIMDE_FLOAT32_C(     8.63), SIMDE_FLOAT32_C(     7.66),
                         SIMDE_FLOAT32_C(     8.48), SIMDE_FLOAT32_C(  8039.28), SIMDE_FLOAT32_C(  6972.90), SIMDE_FLOAT32_C(     8.61)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  1058.07), SIMDE_FLOAT32_C(  6652.15), SIMDE_FLOAT32_C(  2532.95), SIMDE_FLOAT32_C(  9113.62),
                         SIMDE_FLOAT32_C(  9783.41), SIMDE_FLOAT32_C(  9773.08), SIMDE_FLOAT32_C(  9127.47), SIMDE_FLOAT32_C(   918.64),
                         SIMDE_FLOAT32_C(  3953.30), SIMDE_FLOAT32_C(   333.95), SIMDE_FLOAT32_C(  1356.49), SIMDE_FLOAT32_C(  2899.69),
                         SIMDE_FLOAT32_C(  5501.59), SIMDE_FLOAT32_C(  5515.77), SIMDE_FLOAT32_C(  7198.84), SIMDE_FLOAT32_C(  3978.34)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   792.83), SIMDE_FLOAT32_C(  4929.19), SIMDE_FLOAT32_C(  9124.38), SIMDE_FLOAT32_C(  8968.13),
                         SIMDE_FLOAT32_C(  1316.26), SIMDE_FLOAT32_C(  3447.13), SIMDE_FLOAT32_C(  8644.35), SIMDE_FLOAT32_C(  3246.39),
                         SIMDE_FLOAT32_C(  5304.47), SIMDE_FLOAT32_C(  5549.07), SIMDE_FLOAT32_C(  8579.68), SIMDE_FLOAT32_C(  3747.01),
                         SIMDE_FLOAT32_C(  9720.69), SIMDE_FLOAT32_C(  6809.26), SIMDE_FLOAT32_C(  4934.63), SIMDE_FLOAT32_C(  9263.02)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  1058.07), SIMDE_FLOAT32_C(  6652.15), SIMDE_FLOAT32_C(     9.12), SIMDE_FLOAT32_C(     9.10),
                         SIMDE_FLOAT32_C(  9783.41), SIMDE_FLOAT32_C(  9773.08), SIMDE_FLOAT32_C(  9127.47), SIMDE_FLOAT32_C(     8.09),
                         SIMDE_FLOAT32_C(     8.58), SIMDE_FLOAT32_C(   333.95), SIMDE_FLOAT32_C(     9.06), SIMDE_FLOAT32_C(  2899.69),
                         SIMDE_FLOAT32_C(     9.18), SIMDE_FLOAT32_C(  5515.77), SIMDE_FLOAT32_C(  7198.84), SIMDE_FLOAT32_C(     9.13)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_log_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_log_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 7486.55), SIMDE_FLOAT64_C( 8351.20),
                         SIMDE_FLOAT64_C( 3512.77), SIMDE_FLOAT64_C( 5170.29),
                         SIMDE_FLOAT64_C( 4068.94), SIMDE_FLOAT64_C( 5195.06),
                         SIMDE_FLOAT64_C( 1228.12), SIMDE_FLOAT64_C( 6733.16)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    8.92), SIMDE_FLOAT64_C(    9.03),
                         SIMDE_FLOAT64_C(    8.16), SIMDE_FLOAT64_C(    8.55),
                         SIMDE_FLOAT64_C(    8.31), SIMDE_FLOAT64_C(    8.56),
                         SIMDE_FLOAT64_C(    7.11), SIMDE_FLOAT64_C(    8.81)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 1609.14), SIMDE_FLOAT64_C( 1569.36),
                         SIMDE_FLOAT64_C( 5423.87), SIMDE_FLOAT64_C( 7857.29),
                         SIMDE_FLOAT64_C( 9127.65), SIMDE_FLOAT64_C( 7111.03),
                         SIMDE_FLOAT64_C( 3652.77), SIMDE_FLOAT64_C( 7338.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.38), SIMDE_FLOAT64_C(    7.36),
                         SIMDE_FLOAT64_C(    8.60), SIMDE_FLOAT64_C(    8.97),
                         SIMDE_FLOAT64_C(    9.12), SIMDE_FLOAT64_C(    8.87),
                         SIMDE_FLOAT64_C(    8.20), SIMDE_FLOAT64_C(    8.90)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 2775.95), SIMDE_FLOAT64_C( 5142.35),
                         SIMDE_FLOAT64_C( 3079.83), SIMDE_FLOAT64_C(  381.82),
                         SIMDE_FLOAT64_C( 3474.63), SIMDE_FLOAT64_C(  695.25),
                         SIMDE_FLOAT64_C( 2912.29), SIMDE_FLOAT64_C( 8484.34)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.93), SIMDE_FLOAT64_C(    8.55),
                         SIMDE_FLOAT64_C(    8.03), SIMDE_FLOAT64_C(    5.94),
                         SIMDE_FLOAT64_C(    8.15), SIMDE_FLOAT64_C(    6.54),
                         SIMDE_FLOAT64_C(    7.98), SIMDE_FLOAT64_C(    9.05)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 5890.98), SIMDE_FLOAT64_C( 2746.67),
                         SIMDE_FLOAT64_C( 6166.85), SIMDE_FLOAT64_C( 8435.45),
                         SIMDE_FLOAT64_C( 6306.54), SIMDE_FLOAT64_C( 3937.29),
                         SIMDE_FLOAT64_C(  117.23), SIMDE_FLOAT64_C( 1696.00)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    8.68), SIMDE_FLOAT64_C(    7.92),
                         SIMDE_FLOAT64_C(    8.73), SIMDE_FLOAT64_C(    9.04),
                         SIMDE_FLOAT64_C(    8.75), SIMDE_FLOAT64_C(    8.28),
                         SIMDE_FLOAT64_C(    4.76), SIMDE_FLOAT64_C(    7.44)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 1148.23), SIMDE_FLOAT64_C( 7217.40),
                         SIMDE_FLOAT64_C( 2082.02), SIMDE_FLOAT64_C( 6902.28),
                         SIMDE_FLOAT64_C( 1146.40), SIMDE_FLOAT64_C( 9969.51),
                         SIMDE_FLOAT64_C( 5140.40), SIMDE_FLOAT64_C( 9206.03)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.05), SIMDE_FLOAT64_C(    8.88),
                         SIMDE_FLOAT64_C(    7.64), SIMDE_FLOAT64_C(    8.84),
                         SIMDE_FLOAT64_C(    7.04), SIMDE_FLOAT64_C(    9.21),
                         SIMDE_FLOAT64_C(    8.54), SIMDE_FLOAT64_C(    9.13)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 3060.52), SIMDE_FLOAT64_C( 6979.60),
                         SIMDE_FLOAT64_C( 8279.36), SIMDE_FLOAT64_C( 6696.04),
                         SIMDE_FLOAT64_C( 7661.76), SIMDE_FLOAT64_C( 3680.04),
                         SIMDE_FLOAT64_C( 8903.22), SIMDE_FLOAT64_C( 4846.05)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    8.03), SIMDE_FLOAT64_C(    8.85),
                         SIMDE_FLOAT64_C(    9.02), SIMDE_FLOAT64_C(    8.81),
                         SIMDE_FLOAT64_C(    8.94), SIMDE_FLOAT64_C(    8.21),
                         SIMDE_FLOAT64_C(    9.09), SIMDE_FLOAT64_C(    8.49)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 3981.75), SIMDE_FLOAT64_C( 4596.36),
                         SIMDE_FLOAT64_C( 6683.64), SIMDE_FLOAT64_C(  276.11),
                         SIMDE_FLOAT64_C( 1262.07), SIMDE_FLOAT64_C( 1163.84),
                         SIMDE_FLOAT64_C( 2229.06), SIMDE_FLOAT64_C( 6994.08)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    8.29), SIMDE_FLOAT64_C(    8.43),
                         SIMDE_FLOAT64_C(    8.81), SIMDE_FLOAT64_C(    5.62),
                         SIMDE_FLOAT64_C(    7.14), SIMDE_FLOAT64_C(    7.06),
                         SIMDE_FLOAT64_C(    7.71), SIMDE_FLOAT64_C(    8.85)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 7348.31), SIMDE_FLOAT64_C( 8400.08),
                         SIMDE_FLOAT64_C( 4256.55), SIMDE_FLOAT64_C( 9093.31),
                         SIMDE_FLOAT64_C( 9550.14), SIMDE_FLOAT64_C( 8002.34),
                         SIMDE_FLOAT64_C( 8956.15), SIMDE_FLOAT64_C( 6271.53)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    8.90), SIMDE_FLOAT64_C(    9.04),
                         SIMDE_FLOAT64_C(    8.36), SIMDE_FLOAT64_C(    9.12),
                         SIMDE_FLOAT64_C(    9.16), SIMDE_FLOAT64_C(    8.99),
                         SIMDE_FLOAT64_C(    9.10), SIMDE_FLOAT64_C(    8.74)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_log_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_log_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 1569.36), SIMDE_FLOAT64_C( 7857.29),
                         SIMDE_FLOAT64_C( 7111.03), SIMDE_FLOAT64_C( 7338.80),
                         SIMDE_FLOAT64_C( 8351.20), SIMDE_FLOAT64_C( 5170.29),
                         SIMDE_FLOAT64_C( 5195.06), SIMDE_FLOAT64_C( 6733.16)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 1609.14), SIMDE_FLOAT64_C( 5423.87),
                         SIMDE_FLOAT64_C( 9127.65), SIMDE_FLOAT64_C( 3652.77),
                         SIMDE_FLOAT64_C( 7486.55), SIMDE_FLOAT64_C( 3512.77),
                         SIMDE_FLOAT64_C( 4068.94), SIMDE_FLOAT64_C( 1228.12)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.38), SIMDE_FLOAT64_C( 7857.29),
                         SIMDE_FLOAT64_C( 7111.03), SIMDE_FLOAT64_C( 7338.80),
                         SIMDE_FLOAT64_C(    8.92), SIMDE_FLOAT64_C( 5170.29),
                         SIMDE_FLOAT64_C(    8.31), SIMDE_FLOAT64_C(    7.11)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 5890.98), SIMDE_FLOAT64_C( 6166.85),
                         SIMDE_FLOAT64_C( 6306.54), SIMDE_FLOAT64_C(  117.23),
                         SIMDE_FLOAT64_C( 2775.95), SIMDE_FLOAT64_C( 3079.83),
                         SIMDE_FLOAT64_C( 3474.63), SIMDE_FLOAT64_C( 2912.29)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 9206.03), SIMDE_FLOAT64_C( 2746.67),
                         SIMDE_FLOAT64_C( 8435.45), SIMDE_FLOAT64_C( 3937.29),
                         SIMDE_FLOAT64_C( 1696.00), SIMDE_FLOAT64_C( 5142.35),
                         SIMDE_FLOAT64_C(  381.82), SIMDE_FLOAT64_C(  695.25)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    9.13), SIMDE_FLOAT64_C(    7.92),
                         SIMDE_FLOAT64_C(    9.04), SIMDE_FLOAT64_C(  117.23),
                         SIMDE_FLOAT64_C( 2775.95), SIMDE_FLOAT64_C(    8.55),
                         SIMDE_FLOAT64_C( 3474.63), SIMDE_FLOAT64_C(    6.54)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 6994.08), SIMDE_FLOAT64_C( 6979.60),
                         SIMDE_FLOAT64_C( 6696.04), SIMDE_FLOAT64_C( 3680.04),
                         SIMDE_FLOAT64_C( 4846.05), SIMDE_FLOAT64_C( 7217.40),
                         SIMDE_FLOAT64_C( 6902.28), SIMDE_FLOAT64_C( 9969.51)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 2229.06), SIMDE_FLOAT64_C( 3060.52),
                         SIMDE_FLOAT64_C( 8279.36), SIMDE_FLOAT64_C( 7661.76),
                         SIMDE_FLOAT64_C( 8903.22), SIMDE_FLOAT64_C( 1148.23),
                         SIMDE_FLOAT64_C( 2082.02), SIMDE_FLOAT64_C( 1146.40)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.71), SIMDE_FLOAT64_C(    8.03),
                         SIMDE_FLOAT64_C(    9.02), SIMDE_FLOAT64_C(    8.94),
                         SIMDE_FLOAT64_C(    9.09), SIMDE_FLOAT64_C(    7.05),
                         SIMDE_FLOAT64_C( 6902.28), SIMDE_FLOAT64_C(    7.04)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 5603.27), SIMDE_FLOAT64_C( 7348.31),
                         SIMDE_FLOAT64_C( 4256.55), SIMDE_FLOAT64_C( 9550.14),
                         SIMDE_FLOAT64_C( 8956.15), SIMDE_FLOAT64_C( 3981.75),
                         SIMDE_FLOAT64_C( 6683.64), SIMDE_FLOAT64_C( 1262.07)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 7716.73), SIMDE_FLOAT64_C( 4142.45),
                         SIMDE_FLOAT64_C( 8400.08), SIMDE_FLOAT64_C( 9093.31),
                         SIMDE_FLOAT64_C( 8002.34), SIMDE_FLOAT64_C( 6271.53),
                         SIMDE_FLOAT64_C( 4596.36), SIMDE_FLOAT64_C(  276.11)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 5603.27), SIMDE_FLOAT64_C(    8.33),
                         SIMDE_FLOAT64_C( 4256.55), SIMDE_FLOAT64_C(    9.12),
                         SIMDE_FLOAT64_C(    8.99), SIMDE_FLOAT64_C(    8.74),
                         SIMDE_FLOAT64_C( 6683.64), SIMDE_FLOAT64_C(    5.62)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 5499.67), SIMDE_FLOAT64_C( 1309.05),
                         SIMDE_FLOAT64_C( 8793.93), SIMDE_FLOAT64_C( 6717.40),
                         SIMDE_FLOAT64_C( 1010.42), SIMDE_FLOAT64_C( 2370.85),
                         SIMDE_FLOAT64_C(  886.73), SIMDE_FLOAT64_C( 8278.35)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 6656.71), SIMDE_FLOAT64_C( 7314.76),
                         SIMDE_FLOAT64_C( 4105.04), SIMDE_FLOAT64_C( 6623.12),
                         SIMDE_FLOAT64_C(  628.43), SIMDE_FLOAT64_C( 3357.32),
                         SIMDE_FLOAT64_C( 4038.44), SIMDE_FLOAT64_C( 7806.81)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    8.80), SIMDE_FLOAT64_C( 1309.05),
                         SIMDE_FLOAT64_C( 8793.93), SIMDE_FLOAT64_C(    8.80),
                         SIMDE_FLOAT64_C( 1010.42), SIMDE_FLOAT64_C( 2370.85),
                         SIMDE_FLOAT64_C(  886.73), SIMDE_FLOAT64_C(    8.96)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 1217.92), SIMDE_FLOAT64_C( 5136.26),
                         SIMDE_FLOAT64_C( 8450.59), SIMDE_FLOAT64_C( 4894.53),
                         SIMDE_FLOAT64_C( 2755.53), SIMDE_FLOAT64_C( 7528.93),
                         SIMDE_FLOAT64_C( 9155.11), SIMDE_FLOAT64_C( 9886.80)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 9887.44), SIMDE_FLOAT64_C( 7124.06),
                         SIMDE_FLOAT64_C( 4524.23), SIMDE_FLOAT64_C( 9203.26),
                         SIMDE_FLOAT64_C( 2042.18), SIMDE_FLOAT64_C( 8657.47),
                         SIMDE_FLOAT64_C( 8118.50), SIMDE_FLOAT64_C( 5703.37)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 1217.92), SIMDE_FLOAT64_C(    8.87),
                         SIMDE_FLOAT64_C( 8450.59), SIMDE_FLOAT64_C( 4894.53),
                         SIMDE_FLOAT64_C(    7.62), SIMDE_FLOAT64_C( 7528.93),
                         SIMDE_FLOAT64_C(    9.00), SIMDE_FLOAT64_C(    8.65)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 6973.34), SIMDE_FLOAT64_C( 3476.37),
                         SIMDE_FLOAT64_C( 1516.57), SIMDE_FLOAT64_C( 9110.29),
                         SIMDE_FLOAT64_C(   11.83), SIMDE_FLOAT64_C( 9618.20),
                         SIMDE_FLOAT64_C( 1159.42), SIMDE_FLOAT64_C( 4661.80)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 7554.25), SIMDE_FLOAT64_C( 5071.68),
                         SIMDE_FLOAT64_C( 9581.30), SIMDE_FLOAT64_C( 1154.54),
                         SIMDE_FLOAT64_C( 2130.97), SIMDE_FLOAT64_C( 3312.02),
                         SIMDE_FLOAT64_C( 6468.19), SIMDE_FLOAT64_C( 2118.90)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 6973.34), SIMDE_FLOAT64_C(    8.53),
                         SIMDE_FLOAT64_C( 1516.57), SIMDE_FLOAT64_C(    7.05),
                         SIMDE_FLOAT64_C(    7.66), SIMDE_FLOAT64_C(    8.11),
                         SIMDE_FLOAT64_C( 1159.42), SIMDE_FLOAT64_C(    7.66)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 7377.56), SIMDE_FLOAT64_C( 9683.23),
                         SIMDE_FLOAT64_C( 3256.50), SIMDE_FLOAT64_C( 2809.03),
                         SIMDE_FLOAT64_C( 1237.85), SIMDE_FLOAT64_C( 9663.28),
                         SIMDE_FLOAT64_C( 3363.90), SIMDE_FLOAT64_C( 4087.77)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 1124.78), SIMDE_FLOAT64_C( 7203.19),
                         SIMDE_FLOAT64_C( 9486.33), SIMDE_FLOAT64_C( 4010.56),
                         SIMDE_FLOAT64_C( 3201.22), SIMDE_FLOAT64_C( 4831.67),
                         SIMDE_FLOAT64_C( 5036.36), SIMDE_FLOAT64_C( 4374.02)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.03), SIMDE_FLOAT64_C(    8.88),
                         SIMDE_FLOAT64_C( 3256.50), SIMDE_FLOAT64_C(    8.30),
                         SIMDE_FLOAT64_C( 1237.85), SIMDE_FLOAT64_C(    8.48),
                         SIMDE_FLOAT64_C( 3363.90), SIMDE_FLOAT64_C(    8.38)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_log_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_log1p_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    75.94), SIMDE_FLOAT32_C(     8.83), SIMDE_FLOAT32_C(    79.72), SIMDE_FLOAT32_C(    43.97) },
      { SIMDE_FLOAT32_C(     4.34), SIMDE_FLOAT32_C(     2.29), SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     3.81) } },
    { { SIMDE_FLOAT32_C(    40.77), SIMDE_FLOAT32_C(    95.32), SIMDE_FLOAT32_C(    68.75), SIMDE_FLOAT32_C(    17.84) },
      { SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(     4.57), SIMDE_FLOAT32_C(     4.24), SIMDE_FLOAT32_C(     2.94) } },
    { { SIMDE_FLOAT32_C(    87.84), SIMDE_FLOAT32_C(     9.10), SIMDE_FLOAT32_C(    51.15), SIMDE_FLOAT32_C(    49.38) },
      { SIMDE_FLOAT32_C(     4.49), SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(     3.92) } },
    { { SIMDE_FLOAT32_C(    72.43), SIMDE_FLOAT32_C(    10.89), SIMDE_FLOAT32_C(    17.62), SIMDE_FLOAT32_C(    49.42) },
      { SIMDE_FLOAT32_C(     4.30), SIMDE_FLOAT32_C(     2.48), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     3.92) } },
    { { SIMDE_FLOAT32_C(    61.53), SIMDE_FLOAT32_C(     6.26), SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(    56.29) },
      { SIMDE_FLOAT32_C(     4.14), SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     1.53), SIMDE_FLOAT32_C(     4.05) } },
    { { SIMDE_FLOAT32_C(    33.37), SIMDE_FLOAT32_C(    28.79), SIMDE_FLOAT32_C(    10.52), SIMDE_FLOAT32_C(    86.16) },
      { SIMDE_FLOAT32_C(     3.54), SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(     4.47) } },
    { { SIMDE_FLOAT32_C(    75.88), SIMDE_FLOAT32_C(    38.85), SIMDE_FLOAT32_C(    41.92), SIMDE_FLOAT32_C(    15.06) },
      { SIMDE_FLOAT32_C(     4.34), SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(     3.76), SIMDE_FLOAT32_C(     2.78) } },
    { { SIMDE_FLOAT32_C(    49.93), SIMDE_FLOAT32_C(    45.63), SIMDE_FLOAT32_C(    11.83), SIMDE_FLOAT32_C(    25.87) },
      { SIMDE_FLOAT32_C(     3.93), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(     2.55), SIMDE_FLOAT32_C(     3.29) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_log1p_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_log1p_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    71.66), SIMDE_FLOAT64_C(    23.63) },
      { SIMDE_FLOAT64_C(     4.29), SIMDE_FLOAT64_C(     3.20) } },
    { { SIMDE_FLOAT64_C(    39.38), SIMDE_FLOAT64_C(    45.82) },
      { SIMDE_FLOAT64_C(     3.70), SIMDE_FLOAT64_C(     3.85) } },
    { { SIMDE_FLOAT64_C(    26.23), SIMDE_FLOAT64_C(    40.67) },
      { SIMDE_FLOAT64_C(     3.30), SIMDE_FLOAT64_C(     3.73) } },
    { { SIMDE_FLOAT64_C(    88.01), SIMDE_FLOAT64_C(     4.27) },
      { SIMDE_FLOAT64_C(     4.49), SIMDE_FLOAT64_C(     1.66) } },
    { { SIMDE_FLOAT64_C(     8.61), SIMDE_FLOAT64_C(    48.32) },
      { SIMDE_FLOAT64_C(     2.26), SIMDE_FLOAT64_C(     3.90) } },
    { { SIMDE_FLOAT64_C(    83.85), SIMDE_FLOAT64_C(    77.45) },
      { SIMDE_FLOAT64_C(     4.44), SIMDE_FLOAT64_C(     4.36) } },
    { { SIMDE_FLOAT64_C(    28.87), SIMDE_FLOAT64_C(     9.70) },
      { SIMDE_FLOAT64_C(     3.40), SIMDE_FLOAT64_C(     2.37) } },
    { { SIMDE_FLOAT64_C(    59.45), SIMDE_FLOAT64_C(    89.65) },
      { SIMDE_FLOAT64_C(     4.10), SIMDE_FLOAT64_C(     4.51) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_log1p_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_log1p_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     7.73), SIMDE_FLOAT32_C(    44.58), SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(    37.39),
        SIMDE_FLOAT32_C(    81.72), SIMDE_FLOAT32_C(    97.03), SIMDE_FLOAT32_C(    32.40), SIMDE_FLOAT32_C(    46.21) },
      { SIMDE_FLOAT32_C(     2.17), SIMDE_FLOAT32_C(     3.82), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     3.65),
        SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     4.59), SIMDE_FLOAT32_C(     3.51), SIMDE_FLOAT32_C(     3.85) } },
    { { SIMDE_FLOAT32_C(    68.19), SIMDE_FLOAT32_C(    59.69), SIMDE_FLOAT32_C(    65.16), SIMDE_FLOAT32_C(    49.14),
        SIMDE_FLOAT32_C(    16.80), SIMDE_FLOAT32_C(    22.15), SIMDE_FLOAT32_C(    15.49), SIMDE_FLOAT32_C(    40.38) },
      { SIMDE_FLOAT32_C(     4.24), SIMDE_FLOAT32_C(     4.11), SIMDE_FLOAT32_C(     4.19), SIMDE_FLOAT32_C(     3.91),
        SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(     3.14), SIMDE_FLOAT32_C(     2.80), SIMDE_FLOAT32_C(     3.72) } },
    { { SIMDE_FLOAT32_C(    30.77), SIMDE_FLOAT32_C(    61.57), SIMDE_FLOAT32_C(    50.60), SIMDE_FLOAT32_C(    43.40),
        SIMDE_FLOAT32_C(    79.43), SIMDE_FLOAT32_C(    23.65), SIMDE_FLOAT32_C(    55.47), SIMDE_FLOAT32_C(    29.32) },
      { SIMDE_FLOAT32_C(     3.46), SIMDE_FLOAT32_C(     4.14), SIMDE_FLOAT32_C(     3.94), SIMDE_FLOAT32_C(     3.79),
        SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     3.20), SIMDE_FLOAT32_C(     4.03), SIMDE_FLOAT32_C(     3.41) } },
    { { SIMDE_FLOAT32_C(    54.13), SIMDE_FLOAT32_C(    82.81), SIMDE_FLOAT32_C(    78.99), SIMDE_FLOAT32_C(    50.88),
        SIMDE_FLOAT32_C(     5.92), SIMDE_FLOAT32_C(    42.82), SIMDE_FLOAT32_C(    53.24), SIMDE_FLOAT32_C(    13.65) },
      { SIMDE_FLOAT32_C(     4.01), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     4.38), SIMDE_FLOAT32_C(     3.95),
        SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     3.78), SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(     2.68) } },
    { { SIMDE_FLOAT32_C(    87.40), SIMDE_FLOAT32_C(    54.33), SIMDE_FLOAT32_C(    51.04), SIMDE_FLOAT32_C(    69.12),
        SIMDE_FLOAT32_C(    51.36), SIMDE_FLOAT32_C(    83.44), SIMDE_FLOAT32_C(    15.34), SIMDE_FLOAT32_C(    19.54) },
      { SIMDE_FLOAT32_C(     4.48), SIMDE_FLOAT32_C(     4.01), SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(     4.25),
        SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     4.44), SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(     3.02) } },
    { { SIMDE_FLOAT32_C(    43.13), SIMDE_FLOAT32_C(    80.50), SIMDE_FLOAT32_C(    68.69), SIMDE_FLOAT32_C(    59.93),
        SIMDE_FLOAT32_C(     2.65), SIMDE_FLOAT32_C(    84.18), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    33.43) },
      { SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     4.40), SIMDE_FLOAT32_C(     4.24), SIMDE_FLOAT32_C(     4.11),
        SIMDE_FLOAT32_C(     1.29), SIMDE_FLOAT32_C(     4.44), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     3.54) } },
    { { SIMDE_FLOAT32_C(    45.75), SIMDE_FLOAT32_C(    50.91), SIMDE_FLOAT32_C(    76.83), SIMDE_FLOAT32_C(    25.17),
        SIMDE_FLOAT32_C(    74.56), SIMDE_FLOAT32_C(    32.30), SIMDE_FLOAT32_C(    54.49), SIMDE_FLOAT32_C(    28.69) },
      { SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(     4.35), SIMDE_FLOAT32_C(     3.26),
        SIMDE_FLOAT32_C(     4.32), SIMDE_FLOAT32_C(     3.51), SIMDE_FLOAT32_C(     4.02), SIMDE_FLOAT32_C(     3.39) } },
    { { SIMDE_FLOAT32_C(    15.11), SIMDE_FLOAT32_C(    33.49), SIMDE_FLOAT32_C(    79.56), SIMDE_FLOAT32_C(    21.03),
        SIMDE_FLOAT32_C(    76.31), SIMDE_FLOAT32_C(    32.80), SIMDE_FLOAT32_C(    34.68), SIMDE_FLOAT32_C(    63.71) },
      { SIMDE_FLOAT32_C(     2.78), SIMDE_FLOAT32_C(     3.54), SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     3.09),
        SIMDE_FLOAT32_C(     4.35), SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(     3.57), SIMDE_FLOAT32_C(     4.17) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_log1p_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_log1p_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    82.81), SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(    54.22), SIMDE_FLOAT64_C(    13.29) },
      { SIMDE_FLOAT64_C(     4.43), SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     4.01), SIMDE_FLOAT64_C(     2.66) } },
    { { SIMDE_FLOAT64_C(    34.27), SIMDE_FLOAT64_C(    86.02), SIMDE_FLOAT64_C(    66.74), SIMDE_FLOAT64_C(    46.61) },
      { SIMDE_FLOAT64_C(     3.56), SIMDE_FLOAT64_C(     4.47), SIMDE_FLOAT64_C(     4.22), SIMDE_FLOAT64_C(     3.86) } },
    { { SIMDE_FLOAT64_C(    95.48), SIMDE_FLOAT64_C(    40.65), SIMDE_FLOAT64_C(    39.71), SIMDE_FLOAT64_C(    33.88) },
      { SIMDE_FLOAT64_C(     4.57), SIMDE_FLOAT64_C(     3.73), SIMDE_FLOAT64_C(     3.71), SIMDE_FLOAT64_C(     3.55) } },
    { { SIMDE_FLOAT64_C(    25.60), SIMDE_FLOAT64_C(    96.16), SIMDE_FLOAT64_C(    45.65), SIMDE_FLOAT64_C(    11.33) },
      { SIMDE_FLOAT64_C(     3.28), SIMDE_FLOAT64_C(     4.58), SIMDE_FLOAT64_C(     3.84), SIMDE_FLOAT64_C(     2.51) } },
    { { SIMDE_FLOAT64_C(    12.09), SIMDE_FLOAT64_C(    86.42), SIMDE_FLOAT64_C(    87.72), SIMDE_FLOAT64_C(    82.93) },
      { SIMDE_FLOAT64_C(     2.57), SIMDE_FLOAT64_C(     4.47), SIMDE_FLOAT64_C(     4.49), SIMDE_FLOAT64_C(     4.43) } },
    { { SIMDE_FLOAT64_C(    74.51), SIMDE_FLOAT64_C(    10.22), SIMDE_FLOAT64_C(    42.74), SIMDE_FLOAT64_C(    42.04) },
      { SIMDE_FLOAT64_C(     4.32), SIMDE_FLOAT64_C(     2.42), SIMDE_FLOAT64_C(     3.78), SIMDE_FLOAT64_C(     3.76) } },
    { { SIMDE_FLOAT64_C(    56.03), SIMDE_FLOAT64_C(    46.45), SIMDE_FLOAT64_C(    79.57), SIMDE_FLOAT64_C(    53.99) },
      { SIMDE_FLOAT64_C(     4.04), SIMDE_FLOAT64_C(     3.86), SIMDE_FLOAT64_C(     4.39), SIMDE_FLOAT64_C(     4.01) } },
    { { SIMDE_FLOAT64_C(    65.41), SIMDE_FLOAT64_C(    86.99), SIMDE_FLOAT64_C(    98.63), SIMDE_FLOAT64_C(    48.22) },
      { SIMDE_FLOAT64_C(     4.20), SIMDE_FLOAT64_C(     4.48), SIMDE_FLOAT64_C(     4.60), SIMDE_FLOAT64_C(     3.90) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_log1p_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_log1p_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    56.49), SIMDE_FLOAT32_C(    45.26), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     8.51),
        SIMDE_FLOAT32_C(    84.43), SIMDE_FLOAT32_C(    90.20), SIMDE_FLOAT32_C(    58.37), SIMDE_FLOAT32_C(    91.03),
        SIMDE_FLOAT32_C(    16.56), SIMDE_FLOAT32_C(    42.47), SIMDE_FLOAT32_C(    30.02), SIMDE_FLOAT32_C(     0.38),
        SIMDE_FLOAT32_C(    36.91), SIMDE_FLOAT32_C(    32.16), SIMDE_FLOAT32_C(    13.56), SIMDE_FLOAT32_C(    95.86) },
      { SIMDE_FLOAT32_C(     4.05), SIMDE_FLOAT32_C(     3.83), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     2.25),
        SIMDE_FLOAT32_C(     4.45), SIMDE_FLOAT32_C(     4.51), SIMDE_FLOAT32_C(     4.08), SIMDE_FLOAT32_C(     4.52),
        SIMDE_FLOAT32_C(     2.87), SIMDE_FLOAT32_C(     3.77), SIMDE_FLOAT32_C(     3.43), SIMDE_FLOAT32_C(     0.32),
        SIMDE_FLOAT32_C(     3.64), SIMDE_FLOAT32_C(     3.50), SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     4.57) } },
    { { SIMDE_FLOAT32_C(    15.15), SIMDE_FLOAT32_C(    66.91), SIMDE_FLOAT32_C(    89.77), SIMDE_FLOAT32_C(    66.71),
        SIMDE_FLOAT32_C(    24.15), SIMDE_FLOAT32_C(    55.93), SIMDE_FLOAT32_C(    84.52), SIMDE_FLOAT32_C(    55.70),
        SIMDE_FLOAT32_C(    44.08), SIMDE_FLOAT32_C(    33.97), SIMDE_FLOAT32_C(    77.87), SIMDE_FLOAT32_C(    36.54),
        SIMDE_FLOAT32_C(    89.83), SIMDE_FLOAT32_C(    75.19), SIMDE_FLOAT32_C(    48.64), SIMDE_FLOAT32_C(    46.32) },
      { SIMDE_FLOAT32_C(     2.78), SIMDE_FLOAT32_C(     4.22), SIMDE_FLOAT32_C(     4.51), SIMDE_FLOAT32_C(     4.22),
        SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(     4.04), SIMDE_FLOAT32_C(     4.45), SIMDE_FLOAT32_C(     4.04),
        SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     4.37), SIMDE_FLOAT32_C(     3.63),
        SIMDE_FLOAT32_C(     4.51), SIMDE_FLOAT32_C(     4.33), SIMDE_FLOAT32_C(     3.90), SIMDE_FLOAT32_C(     3.86) } },
    { { SIMDE_FLOAT32_C(    20.45), SIMDE_FLOAT32_C(    48.85), SIMDE_FLOAT32_C(    54.83), SIMDE_FLOAT32_C(     4.88),
        SIMDE_FLOAT32_C(    39.05), SIMDE_FLOAT32_C(    13.20), SIMDE_FLOAT32_C(    95.91), SIMDE_FLOAT32_C(    55.62),
        SIMDE_FLOAT32_C(    55.68), SIMDE_FLOAT32_C(    25.92), SIMDE_FLOAT32_C(    55.99), SIMDE_FLOAT32_C(    92.58),
        SIMDE_FLOAT32_C(    58.09), SIMDE_FLOAT32_C(    69.55), SIMDE_FLOAT32_C(    88.44), SIMDE_FLOAT32_C(    73.24) },
      { SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     3.91), SIMDE_FLOAT32_C(     4.02), SIMDE_FLOAT32_C(     1.77),
        SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(     2.65), SIMDE_FLOAT32_C(     4.57), SIMDE_FLOAT32_C(     4.04),
        SIMDE_FLOAT32_C(     4.04), SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     4.04), SIMDE_FLOAT32_C(     4.54),
        SIMDE_FLOAT32_C(     4.08), SIMDE_FLOAT32_C(     4.26), SIMDE_FLOAT32_C(     4.49), SIMDE_FLOAT32_C(     4.31) } },
    { { SIMDE_FLOAT32_C(    36.47), SIMDE_FLOAT32_C(    78.21), SIMDE_FLOAT32_C(    39.95), SIMDE_FLOAT32_C(    60.62),
        SIMDE_FLOAT32_C(    34.14), SIMDE_FLOAT32_C(    24.47), SIMDE_FLOAT32_C(    16.32), SIMDE_FLOAT32_C(    78.22),
        SIMDE_FLOAT32_C(    58.44), SIMDE_FLOAT32_C(    94.19), SIMDE_FLOAT32_C(    14.75), SIMDE_FLOAT32_C(    48.27),
        SIMDE_FLOAT32_C(    69.38), SIMDE_FLOAT32_C(    63.39), SIMDE_FLOAT32_C(    94.60), SIMDE_FLOAT32_C(    89.83) },
      { SIMDE_FLOAT32_C(     3.62), SIMDE_FLOAT32_C(     4.37), SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     4.12),
        SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(     3.24), SIMDE_FLOAT32_C(     2.85), SIMDE_FLOAT32_C(     4.37),
        SIMDE_FLOAT32_C(     4.08), SIMDE_FLOAT32_C(     4.56), SIMDE_FLOAT32_C(     2.76), SIMDE_FLOAT32_C(     3.90),
        SIMDE_FLOAT32_C(     4.25), SIMDE_FLOAT32_C(     4.16), SIMDE_FLOAT32_C(     4.56), SIMDE_FLOAT32_C(     4.51) } },
    { { SIMDE_FLOAT32_C(    12.25), SIMDE_FLOAT32_C(    49.43), SIMDE_FLOAT32_C(    94.71), SIMDE_FLOAT32_C(    51.30),
        SIMDE_FLOAT32_C(    62.63), SIMDE_FLOAT32_C(    90.62), SIMDE_FLOAT32_C(     6.92), SIMDE_FLOAT32_C(    18.31),
        SIMDE_FLOAT32_C(    16.54), SIMDE_FLOAT32_C(    62.91), SIMDE_FLOAT32_C(    10.89), SIMDE_FLOAT32_C(    74.63),
        SIMDE_FLOAT32_C(    32.47), SIMDE_FLOAT32_C(    99.33), SIMDE_FLOAT32_C(    47.86), SIMDE_FLOAT32_C(    68.94) },
      { SIMDE_FLOAT32_C(     2.58), SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(     4.56), SIMDE_FLOAT32_C(     3.96),
        SIMDE_FLOAT32_C(     4.15), SIMDE_FLOAT32_C(     4.52), SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(     2.96),
        SIMDE_FLOAT32_C(     2.86), SIMDE_FLOAT32_C(     4.16), SIMDE_FLOAT32_C(     2.48), SIMDE_FLOAT32_C(     4.33),
        SIMDE_FLOAT32_C(     3.51), SIMDE_FLOAT32_C(     4.61), SIMDE_FLOAT32_C(     3.89), SIMDE_FLOAT32_C(     4.25) } },
    { { SIMDE_FLOAT32_C(    77.54), SIMDE_FLOAT32_C(    87.82), SIMDE_FLOAT32_C(    29.55), SIMDE_FLOAT32_C(    11.68),
        SIMDE_FLOAT32_C(    12.29), SIMDE_FLOAT32_C(    45.87), SIMDE_FLOAT32_C(    89.89), SIMDE_FLOAT32_C(    70.73),
        SIMDE_FLOAT32_C(    40.05), SIMDE_FLOAT32_C(     4.64), SIMDE_FLOAT32_C(    19.00), SIMDE_FLOAT32_C(     9.43),
        SIMDE_FLOAT32_C(    68.04), SIMDE_FLOAT32_C(    13.59), SIMDE_FLOAT32_C(    99.26), SIMDE_FLOAT32_C(    80.28) },
      { SIMDE_FLOAT32_C(     4.36), SIMDE_FLOAT32_C(     4.49), SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     2.54),
        SIMDE_FLOAT32_C(     2.59), SIMDE_FLOAT32_C(     3.85), SIMDE_FLOAT32_C(     4.51), SIMDE_FLOAT32_C(     4.27),
        SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     2.34),
        SIMDE_FLOAT32_C(     4.23), SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     4.61), SIMDE_FLOAT32_C(     4.40) } },
    { { SIMDE_FLOAT32_C(    63.02), SIMDE_FLOAT32_C(    93.97), SIMDE_FLOAT32_C(    31.58), SIMDE_FLOAT32_C(    25.65),
        SIMDE_FLOAT32_C(    84.59), SIMDE_FLOAT32_C(    38.50), SIMDE_FLOAT32_C(    43.96), SIMDE_FLOAT32_C(     1.13),
        SIMDE_FLOAT32_C(     1.41), SIMDE_FLOAT32_C(    54.85), SIMDE_FLOAT32_C(    75.76), SIMDE_FLOAT32_C(    33.88),
        SIMDE_FLOAT32_C(    54.18), SIMDE_FLOAT32_C(    23.62), SIMDE_FLOAT32_C(     2.81), SIMDE_FLOAT32_C(    31.72) },
      { SIMDE_FLOAT32_C(     4.16), SIMDE_FLOAT32_C(     4.55), SIMDE_FLOAT32_C(     3.48), SIMDE_FLOAT32_C(     3.28),
        SIMDE_FLOAT32_C(     4.45), SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     4.02), SIMDE_FLOAT32_C(     4.34), SIMDE_FLOAT32_C(     3.55),
        SIMDE_FLOAT32_C(     4.01), SIMDE_FLOAT32_C(     3.20), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     3.49) } },
    { { SIMDE_FLOAT32_C(    11.44), SIMDE_FLOAT32_C(    32.37), SIMDE_FLOAT32_C(    43.39), SIMDE_FLOAT32_C(    23.72),
        SIMDE_FLOAT32_C(    78.23), SIMDE_FLOAT32_C(    33.28), SIMDE_FLOAT32_C(    94.45), SIMDE_FLOAT32_C(    18.29),
        SIMDE_FLOAT32_C(    37.93), SIMDE_FLOAT32_C(    13.45), SIMDE_FLOAT32_C(    27.72), SIMDE_FLOAT32_C(     5.96),
        SIMDE_FLOAT32_C(    27.05), SIMDE_FLOAT32_C(    26.98), SIMDE_FLOAT32_C(    86.25), SIMDE_FLOAT32_C(    90.07) },
      { SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(     3.51), SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     3.21),
        SIMDE_FLOAT32_C(     4.37), SIMDE_FLOAT32_C(     3.53), SIMDE_FLOAT32_C(     4.56), SIMDE_FLOAT32_C(     2.96),
        SIMDE_FLOAT32_C(     3.66), SIMDE_FLOAT32_C(     2.67), SIMDE_FLOAT32_C(     3.36), SIMDE_FLOAT32_C(     1.94),
        SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     4.47), SIMDE_FLOAT32_C(     4.51) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_log1p_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_log1p_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    12.54), SIMDE_FLOAT32_C(    63.14), SIMDE_FLOAT32_C(    41.17), SIMDE_FLOAT32_C(    60.95),
        SIMDE_FLOAT32_C(     4.09), SIMDE_FLOAT32_C(    68.78), SIMDE_FLOAT32_C(    40.84), SIMDE_FLOAT32_C(    68.42),
        SIMDE_FLOAT32_C(    63.18), SIMDE_FLOAT32_C(    48.47), SIMDE_FLOAT32_C(    50.42), SIMDE_FLOAT32_C(    37.77),
        SIMDE_FLOAT32_C(    19.29), SIMDE_FLOAT32_C(    67.41), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    31.94) },
      UINT8_C(226),
      { SIMDE_FLOAT32_C(    39.12), SIMDE_FLOAT32_C(    35.10), SIMDE_FLOAT32_C(     9.96), SIMDE_FLOAT32_C(     0.10),
        SIMDE_FLOAT32_C(    31.24), SIMDE_FLOAT32_C(    60.86), SIMDE_FLOAT32_C(    46.96), SIMDE_FLOAT32_C(    34.48),
        SIMDE_FLOAT32_C(    76.57), SIMDE_FLOAT32_C(    78.00), SIMDE_FLOAT32_C(    14.95), SIMDE_FLOAT32_C(    17.36),
        SIMDE_FLOAT32_C(    66.84), SIMDE_FLOAT32_C(     3.16), SIMDE_FLOAT32_C(    29.89), SIMDE_FLOAT32_C(    29.98) },
      { SIMDE_FLOAT32_C(    12.54), SIMDE_FLOAT32_C(     3.59), SIMDE_FLOAT32_C(    41.17), SIMDE_FLOAT32_C(    60.95),
        SIMDE_FLOAT32_C(     4.09), SIMDE_FLOAT32_C(     4.12), SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     3.57),
        SIMDE_FLOAT32_C(    63.18), SIMDE_FLOAT32_C(    48.47), SIMDE_FLOAT32_C(    50.42), SIMDE_FLOAT32_C(    37.77),
        SIMDE_FLOAT32_C(    19.29), SIMDE_FLOAT32_C(    67.41), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    31.94) } },
    { { SIMDE_FLOAT32_C(    44.33), SIMDE_FLOAT32_C(    90.84), SIMDE_FLOAT32_C(    34.08), SIMDE_FLOAT32_C(    13.10),
        SIMDE_FLOAT32_C(    31.68), SIMDE_FLOAT32_C(     2.49), SIMDE_FLOAT32_C(    76.28), SIMDE_FLOAT32_C(    80.15),
        SIMDE_FLOAT32_C(    52.91), SIMDE_FLOAT32_C(    14.05), SIMDE_FLOAT32_C(    99.44), SIMDE_FLOAT32_C(    20.32),
        SIMDE_FLOAT32_C(    14.75), SIMDE_FLOAT32_C(    31.39), SIMDE_FLOAT32_C(    83.76), SIMDE_FLOAT32_C(    53.87) },
      UINT8_C(211),
      { SIMDE_FLOAT32_C(    93.72), SIMDE_FLOAT32_C(    53.97), SIMDE_FLOAT32_C(    97.73), SIMDE_FLOAT32_C(    54.58),
        SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    32.21), SIMDE_FLOAT32_C(    31.15), SIMDE_FLOAT32_C(    78.93),
        SIMDE_FLOAT32_C(    47.16), SIMDE_FLOAT32_C(    48.50), SIMDE_FLOAT32_C(    45.77), SIMDE_FLOAT32_C(    50.32),
        SIMDE_FLOAT32_C(    78.40), SIMDE_FLOAT32_C(    75.75), SIMDE_FLOAT32_C(    94.65), SIMDE_FLOAT32_C(    69.24) },
      { SIMDE_FLOAT32_C(     4.55), SIMDE_FLOAT32_C(     4.01), SIMDE_FLOAT32_C(    34.08), SIMDE_FLOAT32_C(    13.10),
        SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     2.49), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     4.38),
        SIMDE_FLOAT32_C(    52.91), SIMDE_FLOAT32_C(    14.05), SIMDE_FLOAT32_C(    99.44), SIMDE_FLOAT32_C(    20.32),
        SIMDE_FLOAT32_C(    14.75), SIMDE_FLOAT32_C(    31.39), SIMDE_FLOAT32_C(    83.76), SIMDE_FLOAT32_C(    53.87) } },
    { { SIMDE_FLOAT32_C(     9.83), SIMDE_FLOAT32_C(     7.75), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    12.32),
        SIMDE_FLOAT32_C(    84.03), SIMDE_FLOAT32_C(    81.06), SIMDE_FLOAT32_C(    65.23), SIMDE_FLOAT32_C(    98.08),
        SIMDE_FLOAT32_C(    80.51), SIMDE_FLOAT32_C(    85.55), SIMDE_FLOAT32_C(    12.83), SIMDE_FLOAT32_C(    11.90),
        SIMDE_FLOAT32_C(    69.31), SIMDE_FLOAT32_C(    66.70), SIMDE_FLOAT32_C(    78.39), SIMDE_FLOAT32_C(    63.03) },
         UINT8_MAX,
      { SIMDE_FLOAT32_C(    76.12), SIMDE_FLOAT32_C(    17.61), SIMDE_FLOAT32_C(    21.60), SIMDE_FLOAT32_C(     8.33),
        SIMDE_FLOAT32_C(    48.76), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(    55.49), SIMDE_FLOAT32_C(    97.26),
        SIMDE_FLOAT32_C(    46.29), SIMDE_FLOAT32_C(     5.81), SIMDE_FLOAT32_C(    75.66), SIMDE_FLOAT32_C(    22.04),
        SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    44.89), SIMDE_FLOAT32_C(    31.87), SIMDE_FLOAT32_C(     8.21) },
      { SIMDE_FLOAT32_C(     4.35), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     3.12), SIMDE_FLOAT32_C(     2.23),
        SIMDE_FLOAT32_C(     3.91), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     4.03), SIMDE_FLOAT32_C(     4.59),
        SIMDE_FLOAT32_C(    80.51), SIMDE_FLOAT32_C(    85.55), SIMDE_FLOAT32_C(    12.83), SIMDE_FLOAT32_C(    11.90),
        SIMDE_FLOAT32_C(    69.31), SIMDE_FLOAT32_C(    66.70), SIMDE_FLOAT32_C(    78.39), SIMDE_FLOAT32_C(    63.03) } },
    { { SIMDE_FLOAT32_C(    45.81), SIMDE_FLOAT32_C(    44.19), SIMDE_FLOAT32_C(    92.24), SIMDE_FLOAT32_C(    26.87),
        SIMDE_FLOAT32_C(     9.42), SIMDE_FLOAT32_C(    90.33), SIMDE_FLOAT32_C(     7.38), SIMDE_FLOAT32_C(    94.98),
        SIMDE_FLOAT32_C(     3.16), SIMDE_FLOAT32_C(    19.28), SIMDE_FLOAT32_C(    64.29), SIMDE_FLOAT32_C(    69.86),
        SIMDE_FLOAT32_C(    97.67), SIMDE_FLOAT32_C(    27.32), SIMDE_FLOAT32_C(    90.53), SIMDE_FLOAT32_C(    73.79) },
      UINT8_C(154),
      { SIMDE_FLOAT32_C(    12.13), SIMDE_FLOAT32_C(    82.12), SIMDE_FLOAT32_C(    93.69), SIMDE_FLOAT32_C(    12.65),
        SIMDE_FLOAT32_C(    37.62), SIMDE_FLOAT32_C(    90.95), SIMDE_FLOAT32_C(    58.94), SIMDE_FLOAT32_C(    43.43),
        SIMDE_FLOAT32_C(    66.61), SIMDE_FLOAT32_C(    80.98), SIMDE_FLOAT32_C(    43.89), SIMDE_FLOAT32_C(    11.51),
        SIMDE_FLOAT32_C(    12.84), SIMDE_FLOAT32_C(    52.10), SIMDE_FLOAT32_C(    57.32), SIMDE_FLOAT32_C(    57.03) },
      { SIMDE_FLOAT32_C(    45.81), SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(    92.24), SIMDE_FLOAT32_C(     2.61),
        SIMDE_FLOAT32_C(     3.65), SIMDE_FLOAT32_C(    90.33), SIMDE_FLOAT32_C(     7.38), SIMDE_FLOAT32_C(     3.79),
        SIMDE_FLOAT32_C(     3.16), SIMDE_FLOAT32_C(    19.28), SIMDE_FLOAT32_C(    64.29), SIMDE_FLOAT32_C(    69.86),
        SIMDE_FLOAT32_C(    97.67), SIMDE_FLOAT32_C(    27.32), SIMDE_FLOAT32_C(    90.53), SIMDE_FLOAT32_C(    73.79) } },
    { { SIMDE_FLOAT32_C(    44.35), SIMDE_FLOAT32_C(    84.19), SIMDE_FLOAT32_C(    66.46), SIMDE_FLOAT32_C(    34.67),
        SIMDE_FLOAT32_C(    91.58), SIMDE_FLOAT32_C(    61.43), SIMDE_FLOAT32_C(    37.83), SIMDE_FLOAT32_C(    10.85),
        SIMDE_FLOAT32_C(    25.72), SIMDE_FLOAT32_C(     7.69), SIMDE_FLOAT32_C(     8.52), SIMDE_FLOAT32_C(    53.04),
        SIMDE_FLOAT32_C(    98.23), SIMDE_FLOAT32_C(    82.31), SIMDE_FLOAT32_C(    97.98), SIMDE_FLOAT32_C(    10.35) },
      UINT8_C( 99),
      { SIMDE_FLOAT32_C(    91.67), SIMDE_FLOAT32_C(    23.00), SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(    82.62),
        SIMDE_FLOAT32_C(    81.94), SIMDE_FLOAT32_C(    45.48), SIMDE_FLOAT32_C(    49.24), SIMDE_FLOAT32_C(    62.91),
        SIMDE_FLOAT32_C(    89.37), SIMDE_FLOAT32_C(    60.75), SIMDE_FLOAT32_C(    75.76), SIMDE_FLOAT32_C(    41.47),
        SIMDE_FLOAT32_C(    18.07), SIMDE_FLOAT32_C(    32.79), SIMDE_FLOAT32_C(    85.82), SIMDE_FLOAT32_C(     2.26) },
      { SIMDE_FLOAT32_C(     4.53), SIMDE_FLOAT32_C(     3.18), SIMDE_FLOAT32_C(    66.46), SIMDE_FLOAT32_C(    34.67),
        SIMDE_FLOAT32_C(    91.58), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(    10.85),
        SIMDE_FLOAT32_C(    25.72), SIMDE_FLOAT32_C(     7.69), SIMDE_FLOAT32_C(     8.52), SIMDE_FLOAT32_C(    53.04),
        SIMDE_FLOAT32_C(    98.23), SIMDE_FLOAT32_C(    82.31), SIMDE_FLOAT32_C(    97.98), SIMDE_FLOAT32_C(    10.35) } },
    { { SIMDE_FLOAT32_C(    99.25), SIMDE_FLOAT32_C(    20.49), SIMDE_FLOAT32_C(    93.84), SIMDE_FLOAT32_C(    60.68),
        SIMDE_FLOAT32_C(    58.33), SIMDE_FLOAT32_C(     4.69), SIMDE_FLOAT32_C(    86.40), SIMDE_FLOAT32_C(    66.02),
        SIMDE_FLOAT32_C(    13.21), SIMDE_FLOAT32_C(    39.45), SIMDE_FLOAT32_C(    64.25), SIMDE_FLOAT32_C(    95.52),
        SIMDE_FLOAT32_C(    37.43), SIMDE_FLOAT32_C(    74.60), SIMDE_FLOAT32_C(    59.95), SIMDE_FLOAT32_C(    29.10) },
      UINT8_C( 67),
      { SIMDE_FLOAT32_C(    62.00), SIMDE_FLOAT32_C(    11.72), SIMDE_FLOAT32_C(    79.53), SIMDE_FLOAT32_C(     7.47),
        SIMDE_FLOAT32_C(    60.96), SIMDE_FLOAT32_C(    42.45), SIMDE_FLOAT32_C(    96.84), SIMDE_FLOAT32_C(    21.71),
        SIMDE_FLOAT32_C(    18.20), SIMDE_FLOAT32_C(    38.31), SIMDE_FLOAT32_C(    39.77), SIMDE_FLOAT32_C(    50.99),
        SIMDE_FLOAT32_C(    24.13), SIMDE_FLOAT32_C(    42.03), SIMDE_FLOAT32_C(    50.24), SIMDE_FLOAT32_C(    44.62) },
      { SIMDE_FLOAT32_C(     4.14), SIMDE_FLOAT32_C(     2.54), SIMDE_FLOAT32_C(    93.84), SIMDE_FLOAT32_C(    60.68),
        SIMDE_FLOAT32_C(    58.33), SIMDE_FLOAT32_C(     4.69), SIMDE_FLOAT32_C(     4.58), SIMDE_FLOAT32_C(    66.02),
        SIMDE_FLOAT32_C(    13.21), SIMDE_FLOAT32_C(    39.45), SIMDE_FLOAT32_C(    64.25), SIMDE_FLOAT32_C(    95.52),
        SIMDE_FLOAT32_C(    37.43), SIMDE_FLOAT32_C(    74.60), SIMDE_FLOAT32_C(    59.95), SIMDE_FLOAT32_C(    29.10) } },
    { { SIMDE_FLOAT32_C(    35.87), SIMDE_FLOAT32_C(    10.92), SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(    40.56),
        SIMDE_FLOAT32_C(    97.33), SIMDE_FLOAT32_C(    68.97), SIMDE_FLOAT32_C(    53.77), SIMDE_FLOAT32_C(    36.78),
        SIMDE_FLOAT32_C(    33.22), SIMDE_FLOAT32_C(    49.29), SIMDE_FLOAT32_C(    74.20), SIMDE_FLOAT32_C(     7.81),
        SIMDE_FLOAT32_C(     9.24), SIMDE_FLOAT32_C(     3.30), SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(    71.23) },
      UINT8_C(113),
      { SIMDE_FLOAT32_C(    84.95), SIMDE_FLOAT32_C(    78.70), SIMDE_FLOAT32_C(    75.98), SIMDE_FLOAT32_C(    27.40),
        SIMDE_FLOAT32_C(    75.54), SIMDE_FLOAT32_C(    97.69), SIMDE_FLOAT32_C(    45.60), SIMDE_FLOAT32_C(    13.85),
        SIMDE_FLOAT32_C(    37.46), SIMDE_FLOAT32_C(    96.59), SIMDE_FLOAT32_C(    37.98), SIMDE_FLOAT32_C(    79.49),
        SIMDE_FLOAT32_C(    46.83), SIMDE_FLOAT32_C(    82.60), SIMDE_FLOAT32_C(    15.36), SIMDE_FLOAT32_C(    57.76) },
      { SIMDE_FLOAT32_C(     4.45), SIMDE_FLOAT32_C(    10.92), SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(    40.56),
        SIMDE_FLOAT32_C(     4.34), SIMDE_FLOAT32_C(     4.59), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(    36.78),
        SIMDE_FLOAT32_C(    33.22), SIMDE_FLOAT32_C(    49.29), SIMDE_FLOAT32_C(    74.20), SIMDE_FLOAT32_C(     7.81),
        SIMDE_FLOAT32_C(     9.24), SIMDE_FLOAT32_C(     3.30), SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(    71.23) } },
    { { SIMDE_FLOAT32_C(    85.55), SIMDE_FLOAT32_C(    55.92), SIMDE_FLOAT32_C(    55.08), SIMDE_FLOAT32_C(    54.52),
        SIMDE_FLOAT32_C(     9.69), SIMDE_FLOAT32_C(    91.86), SIMDE_FLOAT32_C(    87.73), SIMDE_FLOAT32_C(    58.97),
        SIMDE_FLOAT32_C(    66.07), SIMDE_FLOAT32_C(    95.55), SIMDE_FLOAT32_C(    68.21), SIMDE_FLOAT32_C(    69.37),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(    39.44), SIMDE_FLOAT32_C(    84.39), SIMDE_FLOAT32_C(    85.91) },
      UINT8_C(146),
      { SIMDE_FLOAT32_C(    60.37), SIMDE_FLOAT32_C(    13.30), SIMDE_FLOAT32_C(    93.69), SIMDE_FLOAT32_C(    58.06),
        SIMDE_FLOAT32_C(    58.90), SIMDE_FLOAT32_C(     7.54), SIMDE_FLOAT32_C(    95.52), SIMDE_FLOAT32_C(    55.49),
        SIMDE_FLOAT32_C(    45.52), SIMDE_FLOAT32_C(    75.01), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(    28.12),
        SIMDE_FLOAT32_C(    90.38), SIMDE_FLOAT32_C(    60.08), SIMDE_FLOAT32_C(    13.67), SIMDE_FLOAT32_C(    46.30) },
      { SIMDE_FLOAT32_C(    85.55), SIMDE_FLOAT32_C(     2.66), SIMDE_FLOAT32_C(    55.08), SIMDE_FLOAT32_C(    54.52),
        SIMDE_FLOAT32_C(     4.09), SIMDE_FLOAT32_C(    91.86), SIMDE_FLOAT32_C(    87.73), SIMDE_FLOAT32_C(     4.03),
        SIMDE_FLOAT32_C(    66.07), SIMDE_FLOAT32_C(    95.55), SIMDE_FLOAT32_C(    68.21), SIMDE_FLOAT32_C(    69.37),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(    39.44), SIMDE_FLOAT32_C(    84.39), SIMDE_FLOAT32_C(    85.91) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_log1p_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_log1p_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    80.88), SIMDE_FLOAT64_C(    97.21), SIMDE_FLOAT64_C(    22.72), SIMDE_FLOAT64_C(    88.57),
        SIMDE_FLOAT64_C(     7.11), SIMDE_FLOAT64_C(    33.20), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(    68.60) },
      { SIMDE_FLOAT64_C(     4.41), SIMDE_FLOAT64_C(     4.59), SIMDE_FLOAT64_C(     3.17), SIMDE_FLOAT64_C(     4.50),
        SIMDE_FLOAT64_C(     2.09), SIMDE_FLOAT64_C(     3.53), SIMDE_FLOAT64_C(     0.65), SIMDE_FLOAT64_C(     4.24) } },
    { { SIMDE_FLOAT64_C(    18.60), SIMDE_FLOAT64_C(    97.22), SIMDE_FLOAT64_C(    36.01), SIMDE_FLOAT64_C(     5.77),
        SIMDE_FLOAT64_C(    37.64), SIMDE_FLOAT64_C(     8.06), SIMDE_FLOAT64_C(    89.11), SIMDE_FLOAT64_C(    35.34) },
      { SIMDE_FLOAT64_C(     2.98), SIMDE_FLOAT64_C(     4.59), SIMDE_FLOAT64_C(     3.61), SIMDE_FLOAT64_C(     1.91),
        SIMDE_FLOAT64_C(     3.65), SIMDE_FLOAT64_C(     2.20), SIMDE_FLOAT64_C(     4.50), SIMDE_FLOAT64_C(     3.59) } },
    { { SIMDE_FLOAT64_C(    29.67), SIMDE_FLOAT64_C(    90.68), SIMDE_FLOAT64_C(    39.64), SIMDE_FLOAT64_C(    62.60),
        SIMDE_FLOAT64_C(    75.54), SIMDE_FLOAT64_C(    10.18), SIMDE_FLOAT64_C(    92.73), SIMDE_FLOAT64_C(    94.58) },
      { SIMDE_FLOAT64_C(     3.42), SIMDE_FLOAT64_C(     4.52), SIMDE_FLOAT64_C(     3.70), SIMDE_FLOAT64_C(     4.15),
        SIMDE_FLOAT64_C(     4.34), SIMDE_FLOAT64_C(     2.41), SIMDE_FLOAT64_C(     4.54), SIMDE_FLOAT64_C(     4.56) } },
    { { SIMDE_FLOAT64_C(    76.16), SIMDE_FLOAT64_C(     5.81), SIMDE_FLOAT64_C(    62.23), SIMDE_FLOAT64_C(     5.12),
        SIMDE_FLOAT64_C(    77.73), SIMDE_FLOAT64_C(    84.72), SIMDE_FLOAT64_C(    14.00), SIMDE_FLOAT64_C(    58.61) },
      { SIMDE_FLOAT64_C(     4.35), SIMDE_FLOAT64_C(     1.92), SIMDE_FLOAT64_C(     4.15), SIMDE_FLOAT64_C(     1.81),
        SIMDE_FLOAT64_C(     4.37), SIMDE_FLOAT64_C(     4.45), SIMDE_FLOAT64_C(     2.71), SIMDE_FLOAT64_C(     4.09) } },
    { { SIMDE_FLOAT64_C(    81.93), SIMDE_FLOAT64_C(    36.72), SIMDE_FLOAT64_C(    47.19), SIMDE_FLOAT64_C(    89.04),
        SIMDE_FLOAT64_C(    69.92), SIMDE_FLOAT64_C(    48.10), SIMDE_FLOAT64_C(    57.64), SIMDE_FLOAT64_C(    88.52) },
      { SIMDE_FLOAT64_C(     4.42), SIMDE_FLOAT64_C(     3.63), SIMDE_FLOAT64_C(     3.88), SIMDE_FLOAT64_C(     4.50),
        SIMDE_FLOAT64_C(     4.26), SIMDE_FLOAT64_C(     3.89), SIMDE_FLOAT64_C(     4.07), SIMDE_FLOAT64_C(     4.49) } },
    { { SIMDE_FLOAT64_C(    45.32), SIMDE_FLOAT64_C(    93.65), SIMDE_FLOAT64_C(    94.30), SIMDE_FLOAT64_C(    82.96),
        SIMDE_FLOAT64_C(     1.71), SIMDE_FLOAT64_C(    83.41), SIMDE_FLOAT64_C(    18.30), SIMDE_FLOAT64_C(    31.38) },
      { SIMDE_FLOAT64_C(     3.84), SIMDE_FLOAT64_C(     4.55), SIMDE_FLOAT64_C(     4.56), SIMDE_FLOAT64_C(     4.43),
        SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     4.44), SIMDE_FLOAT64_C(     2.96), SIMDE_FLOAT64_C(     3.48) } },
    { { SIMDE_FLOAT64_C(    74.09), SIMDE_FLOAT64_C(    57.95), SIMDE_FLOAT64_C(    93.98), SIMDE_FLOAT64_C(    49.63),
        SIMDE_FLOAT64_C(    68.12), SIMDE_FLOAT64_C(    86.71), SIMDE_FLOAT64_C(    44.21), SIMDE_FLOAT64_C(    44.28) },
      { SIMDE_FLOAT64_C(     4.32), SIMDE_FLOAT64_C(     4.08), SIMDE_FLOAT64_C(     4.55), SIMDE_FLOAT64_C(     3.92),
        SIMDE_FLOAT64_C(     4.24), SIMDE_FLOAT64_C(     4.47), SIMDE_FLOAT64_C(     3.81), SIMDE_FLOAT64_C(     3.81) } },
    { { SIMDE_FLOAT64_C(    92.51), SIMDE_FLOAT64_C(     6.45), SIMDE_FLOAT64_C(    49.40), SIMDE_FLOAT64_C(    70.25),
        SIMDE_FLOAT64_C(    91.16), SIMDE_FLOAT64_C(    63.40), SIMDE_FLOAT64_C(    28.86), SIMDE_FLOAT64_C(    73.09) },
      { SIMDE_FLOAT64_C(     4.54), SIMDE_FLOAT64_C(     2.01), SIMDE_FLOAT64_C(     3.92), SIMDE_FLOAT64_C(     4.27),
        SIMDE_FLOAT64_C(     4.52), SIMDE_FLOAT64_C(     4.17), SIMDE_FLOAT64_C(     3.40), SIMDE_FLOAT64_C(     4.31) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_log1p_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_log1p_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    52.74), SIMDE_FLOAT64_C(    77.28), SIMDE_FLOAT64_C(    34.34), SIMDE_FLOAT64_C(    52.30),
        SIMDE_FLOAT64_C(    78.12), SIMDE_FLOAT64_C(    51.61), SIMDE_FLOAT64_C(     6.35), SIMDE_FLOAT64_C(    45.83) },
      UINT8_C( 39),
      { SIMDE_FLOAT64_C(    43.10), SIMDE_FLOAT64_C(    47.48), SIMDE_FLOAT64_C(    21.67), SIMDE_FLOAT64_C(    82.04),
        SIMDE_FLOAT64_C(    40.45), SIMDE_FLOAT64_C(    94.76), SIMDE_FLOAT64_C(    61.37), SIMDE_FLOAT64_C(    11.74) },
      { SIMDE_FLOAT64_C(     3.79), SIMDE_FLOAT64_C(     3.88), SIMDE_FLOAT64_C(     3.12), SIMDE_FLOAT64_C(    52.30),
        SIMDE_FLOAT64_C(    78.12), SIMDE_FLOAT64_C(     4.56), SIMDE_FLOAT64_C(     6.35), SIMDE_FLOAT64_C(    45.83) } },
    { { SIMDE_FLOAT64_C(     1.10), SIMDE_FLOAT64_C(    18.75), SIMDE_FLOAT64_C(     3.08), SIMDE_FLOAT64_C(    98.55),
        SIMDE_FLOAT64_C(    92.65), SIMDE_FLOAT64_C(    11.89), SIMDE_FLOAT64_C(    24.76), SIMDE_FLOAT64_C(    36.96) },
      UINT8_C(244),
      { SIMDE_FLOAT64_C(    46.12), SIMDE_FLOAT64_C(    85.44), SIMDE_FLOAT64_C(     4.83), SIMDE_FLOAT64_C(    24.72),
        SIMDE_FLOAT64_C(    98.67), SIMDE_FLOAT64_C(    57.57), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(    33.01) },
      { SIMDE_FLOAT64_C(     1.10), SIMDE_FLOAT64_C(    18.75), SIMDE_FLOAT64_C(     1.76), SIMDE_FLOAT64_C(    98.55),
        SIMDE_FLOAT64_C(     4.60), SIMDE_FLOAT64_C(     4.07), SIMDE_FLOAT64_C(     1.10), SIMDE_FLOAT64_C(     3.53) } },
    { { SIMDE_FLOAT64_C(     9.87), SIMDE_FLOAT64_C(    80.12), SIMDE_FLOAT64_C(    84.62), SIMDE_FLOAT64_C(    16.22),
        SIMDE_FLOAT64_C(    25.95), SIMDE_FLOAT64_C(    41.00), SIMDE_FLOAT64_C(    59.31), SIMDE_FLOAT64_C(    73.43) },
      UINT8_C( 77),
      { SIMDE_FLOAT64_C(    41.35), SIMDE_FLOAT64_C(    13.88), SIMDE_FLOAT64_C(    57.44), SIMDE_FLOAT64_C(     2.72),
        SIMDE_FLOAT64_C(    25.62), SIMDE_FLOAT64_C(    58.53), SIMDE_FLOAT64_C(    21.47), SIMDE_FLOAT64_C(    28.69) },
      { SIMDE_FLOAT64_C(     3.75), SIMDE_FLOAT64_C(    80.12), SIMDE_FLOAT64_C(     4.07), SIMDE_FLOAT64_C(     1.31),
        SIMDE_FLOAT64_C(    25.95), SIMDE_FLOAT64_C(    41.00), SIMDE_FLOAT64_C(     3.11), SIMDE_FLOAT64_C(    73.43) } },
    { { SIMDE_FLOAT64_C(    57.09), SIMDE_FLOAT64_C(    14.11), SIMDE_FLOAT64_C(    40.58), SIMDE_FLOAT64_C(    81.85),
        SIMDE_FLOAT64_C(    51.08), SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(    27.97), SIMDE_FLOAT64_C(    36.52) },
      UINT8_C(164),
      { SIMDE_FLOAT64_C(    52.69), SIMDE_FLOAT64_C(    35.19), SIMDE_FLOAT64_C(    62.99), SIMDE_FLOAT64_C(    54.69),
        SIMDE_FLOAT64_C(    68.20), SIMDE_FLOAT64_C(    72.85), SIMDE_FLOAT64_C(    34.81), SIMDE_FLOAT64_C(    52.82) },
      { SIMDE_FLOAT64_C(    57.09), SIMDE_FLOAT64_C(    14.11), SIMDE_FLOAT64_C(     4.16), SIMDE_FLOAT64_C(    81.85),
        SIMDE_FLOAT64_C(    51.08), SIMDE_FLOAT64_C(     4.30), SIMDE_FLOAT64_C(    27.97), SIMDE_FLOAT64_C(     3.99) } },
    { { SIMDE_FLOAT64_C(    89.07), SIMDE_FLOAT64_C(    60.76), SIMDE_FLOAT64_C(    93.82), SIMDE_FLOAT64_C(    48.38),
        SIMDE_FLOAT64_C(    34.19), SIMDE_FLOAT64_C(    56.49), SIMDE_FLOAT64_C(    89.74), SIMDE_FLOAT64_C(    48.07) },
      UINT8_C( 13),
      { SIMDE_FLOAT64_C(    92.46), SIMDE_FLOAT64_C(    73.68), SIMDE_FLOAT64_C(    72.46), SIMDE_FLOAT64_C(    13.92),
        SIMDE_FLOAT64_C(     2.38), SIMDE_FLOAT64_C(    29.55), SIMDE_FLOAT64_C(    28.03), SIMDE_FLOAT64_C(    42.96) },
      { SIMDE_FLOAT64_C(     4.54), SIMDE_FLOAT64_C(    60.76), SIMDE_FLOAT64_C(     4.30), SIMDE_FLOAT64_C(     2.70),
        SIMDE_FLOAT64_C(    34.19), SIMDE_FLOAT64_C(    56.49), SIMDE_FLOAT64_C(    89.74), SIMDE_FLOAT64_C(    48.07) } },
    { { SIMDE_FLOAT64_C(    11.40), SIMDE_FLOAT64_C(    79.11), SIMDE_FLOAT64_C(    43.54), SIMDE_FLOAT64_C(    39.37),
        SIMDE_FLOAT64_C(    15.63), SIMDE_FLOAT64_C(    48.95), SIMDE_FLOAT64_C(    92.06), SIMDE_FLOAT64_C(    50.82) },
      UINT8_C( 26),
      { SIMDE_FLOAT64_C(    46.75), SIMDE_FLOAT64_C(    19.02), SIMDE_FLOAT64_C(    84.79), SIMDE_FLOAT64_C(    81.56),
        SIMDE_FLOAT64_C(    71.83), SIMDE_FLOAT64_C(    73.86), SIMDE_FLOAT64_C(    42.33), SIMDE_FLOAT64_C(    65.65) },
      { SIMDE_FLOAT64_C(    11.40), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(    43.54), SIMDE_FLOAT64_C(     4.41),
        SIMDE_FLOAT64_C(     4.29), SIMDE_FLOAT64_C(    48.95), SIMDE_FLOAT64_C(    92.06), SIMDE_FLOAT64_C(    50.82) } },
    { { SIMDE_FLOAT64_C(    22.25), SIMDE_FLOAT64_C(    76.52), SIMDE_FLOAT64_C(    22.14), SIMDE_FLOAT64_C(    11.98),
        SIMDE_FLOAT64_C(    24.58), SIMDE_FLOAT64_C(    36.07), SIMDE_FLOAT64_C(     4.44), SIMDE_FLOAT64_C(    98.27) },
      UINT8_C(254),
      { SIMDE_FLOAT64_C(    18.36), SIMDE_FLOAT64_C(     0.64), SIMDE_FLOAT64_C(    38.07), SIMDE_FLOAT64_C(    46.40),
        SIMDE_FLOAT64_C(    43.60), SIMDE_FLOAT64_C(    49.47), SIMDE_FLOAT64_C(    25.51), SIMDE_FLOAT64_C(    87.14) },
      { SIMDE_FLOAT64_C(    22.25), SIMDE_FLOAT64_C(     0.49), SIMDE_FLOAT64_C(     3.67), SIMDE_FLOAT64_C(     3.86),
        SIMDE_FLOAT64_C(     3.80), SIMDE_FLOAT64_C(     3.92), SIMDE_FLOAT64_C(     3.28), SIMDE_FLOAT64_C(     4.48) } },
    { { SIMDE_FLOAT64_C(    88.84), SIMDE_FLOAT64_C(    41.14), SIMDE_FLOAT64_C(    36.09), SIMDE_FLOAT64_C(    80.90),
        SIMDE_FLOAT64_C(    91.96), SIMDE_FLOAT64_C(    48.03), SIMDE_FLOAT64_C(    27.65), SIMDE_FLOAT64_C(    10.98) },
      UINT8_C(171),
      { SIMDE_FLOAT64_C(     9.21), SIMDE_FLOAT64_C(    82.81), SIMDE_FLOAT64_C(     6.69), SIMDE_FLOAT64_C(    51.54),
        SIMDE_FLOAT64_C(    48.46), SIMDE_FLOAT64_C(    28.94), SIMDE_FLOAT64_C(    28.06), SIMDE_FLOAT64_C(    70.60) },
      { SIMDE_FLOAT64_C(     2.32), SIMDE_FLOAT64_C(     4.43), SIMDE_FLOAT64_C(    36.09), SIMDE_FLOAT64_C(     3.96),
        SIMDE_FLOAT64_C(    91.96), SIMDE_FLOAT64_C(     3.40), SIMDE_FLOAT64_C(    27.65), SIMDE_FLOAT64_C(     4.27) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_log1p_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_log2_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   631.47), SIMDE_FLOAT32_C(   844.23), SIMDE_FLOAT32_C(   439.63), SIMDE_FLOAT32_C(    13.01) },
      { SIMDE_FLOAT32_C(     9.30), SIMDE_FLOAT32_C(     9.72), SIMDE_FLOAT32_C(     8.78), SIMDE_FLOAT32_C(     3.70) } },
    { { SIMDE_FLOAT32_C(    66.81), SIMDE_FLOAT32_C(    88.82), SIMDE_FLOAT32_C(   350.44), SIMDE_FLOAT32_C(   636.52) },
      { SIMDE_FLOAT32_C(     6.06), SIMDE_FLOAT32_C(     6.47), SIMDE_FLOAT32_C(     8.45), SIMDE_FLOAT32_C(     9.31) } },
    { { SIMDE_FLOAT32_C(   636.53), SIMDE_FLOAT32_C(   411.53), SIMDE_FLOAT32_C(   396.60), SIMDE_FLOAT32_C(   131.18) },
      { SIMDE_FLOAT32_C(     9.31), SIMDE_FLOAT32_C(     8.68), SIMDE_FLOAT32_C(     8.63), SIMDE_FLOAT32_C(     7.04) } },
    { { SIMDE_FLOAT32_C(   749.84), SIMDE_FLOAT32_C(   385.14), SIMDE_FLOAT32_C(   384.93), SIMDE_FLOAT32_C(   165.27) },
      { SIMDE_FLOAT32_C(     9.55), SIMDE_FLOAT32_C(     8.59), SIMDE_FLOAT32_C(     8.59), SIMDE_FLOAT32_C(     7.37) } },
    { { SIMDE_FLOAT32_C(   246.49), SIMDE_FLOAT32_C(   520.56), SIMDE_FLOAT32_C(   778.62), SIMDE_FLOAT32_C(    71.34) },
      { SIMDE_FLOAT32_C(     7.95), SIMDE_FLOAT32_C(     9.02), SIMDE_FLOAT32_C(     9.60), SIMDE_FLOAT32_C(     6.16) } },
    { { SIMDE_FLOAT32_C(   946.80), SIMDE_FLOAT32_C(   380.92), SIMDE_FLOAT32_C(   894.84), SIMDE_FLOAT32_C(   902.24) },
      { SIMDE_FLOAT32_C(     9.89), SIMDE_FLOAT32_C(     8.57), SIMDE_FLOAT32_C(     9.81), SIMDE_FLOAT32_C(     9.82) } },
    { { SIMDE_FLOAT32_C(   574.27), SIMDE_FLOAT32_C(   214.93), SIMDE_FLOAT32_C(   953.03), SIMDE_FLOAT32_C(   638.26) },
      { SIMDE_FLOAT32_C(     9.17), SIMDE_FLOAT32_C(     7.75), SIMDE_FLOAT32_C(     9.90), SIMDE_FLOAT32_C(     9.32) } },
    { { SIMDE_FLOAT32_C(   991.13), SIMDE_FLOAT32_C(   188.32), SIMDE_FLOAT32_C(   949.37), SIMDE_FLOAT32_C(   622.60) },
      { SIMDE_FLOAT32_C(     9.95), SIMDE_FLOAT32_C(     7.56), SIMDE_FLOAT32_C(     9.89), SIMDE_FLOAT32_C(     9.28) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_log2_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_log2_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   982.90), SIMDE_FLOAT64_C(   619.50) },
      { SIMDE_FLOAT64_C(     9.94), SIMDE_FLOAT64_C(     9.27) } },
    { { SIMDE_FLOAT64_C(   102.39), SIMDE_FLOAT64_C(   923.09) },
      { SIMDE_FLOAT64_C(     6.68), SIMDE_FLOAT64_C(     9.85) } },
    { { SIMDE_FLOAT64_C(   243.48), SIMDE_FLOAT64_C(   494.45) },
      { SIMDE_FLOAT64_C(     7.93), SIMDE_FLOAT64_C(     8.95) } },
    { { SIMDE_FLOAT64_C(    45.35), SIMDE_FLOAT64_C(   416.91) },
      { SIMDE_FLOAT64_C(     5.50), SIMDE_FLOAT64_C(     8.70) } },
    { { SIMDE_FLOAT64_C(   259.45), SIMDE_FLOAT64_C(   290.22) },
      { SIMDE_FLOAT64_C(     8.02), SIMDE_FLOAT64_C(     8.18) } },
    { { SIMDE_FLOAT64_C(   923.80), SIMDE_FLOAT64_C(   970.52) },
      { SIMDE_FLOAT64_C(     9.85), SIMDE_FLOAT64_C(     9.92) } },
    { { SIMDE_FLOAT64_C(   646.50), SIMDE_FLOAT64_C(   264.22) },
      { SIMDE_FLOAT64_C(     9.34), SIMDE_FLOAT64_C(     8.05) } },
    { { SIMDE_FLOAT64_C(   634.41), SIMDE_FLOAT64_C(   510.63) },
      { SIMDE_FLOAT64_C(     9.31), SIMDE_FLOAT64_C(     9.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_log2_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_log2_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   889.40), SIMDE_FLOAT32_C(   779.21), SIMDE_FLOAT32_C(   198.92), SIMDE_FLOAT32_C(   945.28),
        SIMDE_FLOAT32_C(    42.71), SIMDE_FLOAT32_C(   341.50), SIMDE_FLOAT32_C(   958.60), SIMDE_FLOAT32_C(   736.56) },
      { SIMDE_FLOAT32_C(     9.80), SIMDE_FLOAT32_C(     9.61), SIMDE_FLOAT32_C(     7.64), SIMDE_FLOAT32_C(     9.88),
        SIMDE_FLOAT32_C(     5.42), SIMDE_FLOAT32_C(     8.42), SIMDE_FLOAT32_C(     9.90), SIMDE_FLOAT32_C(     9.52) } },
    { { SIMDE_FLOAT32_C(    74.89), SIMDE_FLOAT32_C(   979.36), SIMDE_FLOAT32_C(   587.94), SIMDE_FLOAT32_C(   960.37),
        SIMDE_FLOAT32_C(   497.73), SIMDE_FLOAT32_C(   286.82), SIMDE_FLOAT32_C(   507.33), SIMDE_FLOAT32_C(   616.64) },
      { SIMDE_FLOAT32_C(     6.23), SIMDE_FLOAT32_C(     9.94), SIMDE_FLOAT32_C(     9.20), SIMDE_FLOAT32_C(     9.91),
        SIMDE_FLOAT32_C(     8.96), SIMDE_FLOAT32_C(     8.16), SIMDE_FLOAT32_C(     8.99), SIMDE_FLOAT32_C(     9.27) } },
    { { SIMDE_FLOAT32_C(   307.44), SIMDE_FLOAT32_C(   437.70), SIMDE_FLOAT32_C(   685.73), SIMDE_FLOAT32_C(   291.17),
        SIMDE_FLOAT32_C(   840.55), SIMDE_FLOAT32_C(   438.07), SIMDE_FLOAT32_C(   676.25), SIMDE_FLOAT32_C(   160.97) },
      { SIMDE_FLOAT32_C(     8.26), SIMDE_FLOAT32_C(     8.77), SIMDE_FLOAT32_C(     9.42), SIMDE_FLOAT32_C(     8.19),
        SIMDE_FLOAT32_C(     9.72), SIMDE_FLOAT32_C(     8.78), SIMDE_FLOAT32_C(     9.40), SIMDE_FLOAT32_C(     7.33) } },
    { { SIMDE_FLOAT32_C(   788.67), SIMDE_FLOAT32_C(   843.13), SIMDE_FLOAT32_C(   381.11), SIMDE_FLOAT32_C(   499.16),
        SIMDE_FLOAT32_C(   309.83), SIMDE_FLOAT32_C(   369.53), SIMDE_FLOAT32_C(   957.38), SIMDE_FLOAT32_C(   199.23) },
      { SIMDE_FLOAT32_C(     9.62), SIMDE_FLOAT32_C(     9.72), SIMDE_FLOAT32_C(     8.57), SIMDE_FLOAT32_C(     8.96),
        SIMDE_FLOAT32_C(     8.28), SIMDE_FLOAT32_C(     8.53), SIMDE_FLOAT32_C(     9.90), SIMDE_FLOAT32_C(     7.64) } },
    { { SIMDE_FLOAT32_C(   148.75), SIMDE_FLOAT32_C(   156.30), SIMDE_FLOAT32_C(   144.51), SIMDE_FLOAT32_C(   191.45),
        SIMDE_FLOAT32_C(   497.81), SIMDE_FLOAT32_C(   103.11), SIMDE_FLOAT32_C(   928.02), SIMDE_FLOAT32_C(   572.70) },
      { SIMDE_FLOAT32_C(     7.22), SIMDE_FLOAT32_C(     7.29), SIMDE_FLOAT32_C(     7.18), SIMDE_FLOAT32_C(     7.58),
        SIMDE_FLOAT32_C(     8.96), SIMDE_FLOAT32_C(     6.69), SIMDE_FLOAT32_C(     9.86), SIMDE_FLOAT32_C(     9.16) } },
    { { SIMDE_FLOAT32_C(    82.46), SIMDE_FLOAT32_C(   515.95), SIMDE_FLOAT32_C(   533.07), SIMDE_FLOAT32_C(   580.19),
        SIMDE_FLOAT32_C(   802.77), SIMDE_FLOAT32_C(    40.40), SIMDE_FLOAT32_C(   196.83), SIMDE_FLOAT32_C(   110.21) },
      { SIMDE_FLOAT32_C(     6.37), SIMDE_FLOAT32_C(     9.01), SIMDE_FLOAT32_C(     9.06), SIMDE_FLOAT32_C(     9.18),
        SIMDE_FLOAT32_C(     9.65), SIMDE_FLOAT32_C(     5.34), SIMDE_FLOAT32_C(     7.62), SIMDE_FLOAT32_C(     6.78) } },
    { { SIMDE_FLOAT32_C(   478.10), SIMDE_FLOAT32_C(   882.57), SIMDE_FLOAT32_C(   401.38), SIMDE_FLOAT32_C(   318.65),
        SIMDE_FLOAT32_C(   320.63), SIMDE_FLOAT32_C(    77.63), SIMDE_FLOAT32_C(   479.61), SIMDE_FLOAT32_C(   109.31) },
      { SIMDE_FLOAT32_C(     8.90), SIMDE_FLOAT32_C(     9.79), SIMDE_FLOAT32_C(     8.65), SIMDE_FLOAT32_C(     8.32),
        SIMDE_FLOAT32_C(     8.32), SIMDE_FLOAT32_C(     6.28), SIMDE_FLOAT32_C(     8.91), SIMDE_FLOAT32_C(     6.77) } },
    { { SIMDE_FLOAT32_C(   920.76), SIMDE_FLOAT32_C(   860.72), SIMDE_FLOAT32_C(   608.46), SIMDE_FLOAT32_C(   230.59),
        SIMDE_FLOAT32_C(   230.26), SIMDE_FLOAT32_C(   565.84), SIMDE_FLOAT32_C(   429.82), SIMDE_FLOAT32_C(   379.00) },
      { SIMDE_FLOAT32_C(     9.85), SIMDE_FLOAT32_C(     9.75), SIMDE_FLOAT32_C(     9.25), SIMDE_FLOAT32_C(     7.85),
        SIMDE_FLOAT32_C(     7.85), SIMDE_FLOAT32_C(     9.14), SIMDE_FLOAT32_C(     8.75), SIMDE_FLOAT32_C(     8.57) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_log2_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_log2_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   760.38), SIMDE_FLOAT64_C(   341.71), SIMDE_FLOAT64_C(   933.93), SIMDE_FLOAT64_C(   964.91) },
      { SIMDE_FLOAT64_C(     9.57), SIMDE_FLOAT64_C(     8.42), SIMDE_FLOAT64_C(     9.87), SIMDE_FLOAT64_C(     9.91) } },
    { { SIMDE_FLOAT64_C(   115.25), SIMDE_FLOAT64_C(    77.12), SIMDE_FLOAT64_C(   667.61), SIMDE_FLOAT64_C(   365.22) },
      { SIMDE_FLOAT64_C(     6.85), SIMDE_FLOAT64_C(     6.27), SIMDE_FLOAT64_C(     9.38), SIMDE_FLOAT64_C(     8.51) } },
    { { SIMDE_FLOAT64_C(   679.91), SIMDE_FLOAT64_C(   892.57), SIMDE_FLOAT64_C(   787.62), SIMDE_FLOAT64_C(   588.83) },
      { SIMDE_FLOAT64_C(     9.41), SIMDE_FLOAT64_C(     9.80), SIMDE_FLOAT64_C(     9.62), SIMDE_FLOAT64_C(     9.20) } },
    { { SIMDE_FLOAT64_C(    30.55), SIMDE_FLOAT64_C(   713.90), SIMDE_FLOAT64_C(   332.19), SIMDE_FLOAT64_C(   616.75) },
      { SIMDE_FLOAT64_C(     4.93), SIMDE_FLOAT64_C(     9.48), SIMDE_FLOAT64_C(     8.38), SIMDE_FLOAT64_C(     9.27) } },
    { { SIMDE_FLOAT64_C(   183.75), SIMDE_FLOAT64_C(   550.51), SIMDE_FLOAT64_C(   693.58), SIMDE_FLOAT64_C(   893.18) },
      { SIMDE_FLOAT64_C(     7.52), SIMDE_FLOAT64_C(     9.10), SIMDE_FLOAT64_C(     9.44), SIMDE_FLOAT64_C(     9.80) } },
    { { SIMDE_FLOAT64_C(   430.95), SIMDE_FLOAT64_C(   320.69), SIMDE_FLOAT64_C(   576.89), SIMDE_FLOAT64_C(   863.61) },
      { SIMDE_FLOAT64_C(     8.75), SIMDE_FLOAT64_C(     8.33), SIMDE_FLOAT64_C(     9.17), SIMDE_FLOAT64_C(     9.75) } },
    { { SIMDE_FLOAT64_C(   830.18), SIMDE_FLOAT64_C(   881.23), SIMDE_FLOAT64_C(   596.73), SIMDE_FLOAT64_C(   514.46) },
      { SIMDE_FLOAT64_C(     9.70), SIMDE_FLOAT64_C(     9.78), SIMDE_FLOAT64_C(     9.22), SIMDE_FLOAT64_C(     9.01) } },
    { { SIMDE_FLOAT64_C(   253.95), SIMDE_FLOAT64_C(   753.04), SIMDE_FLOAT64_C(   535.98), SIMDE_FLOAT64_C(    14.32) },
      { SIMDE_FLOAT64_C(     7.99), SIMDE_FLOAT64_C(     9.56), SIMDE_FLOAT64_C(     9.07), SIMDE_FLOAT64_C(     3.84) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_log2_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_log2_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   483.98), SIMDE_FLOAT32_C(   550.49), SIMDE_FLOAT32_C(   612.79), SIMDE_FLOAT32_C(   652.36),
        SIMDE_FLOAT32_C(   702.86), SIMDE_FLOAT32_C(   993.84), SIMDE_FLOAT32_C(   608.42), SIMDE_FLOAT32_C(   923.16),
        SIMDE_FLOAT32_C(   531.91), SIMDE_FLOAT32_C(   675.78), SIMDE_FLOAT32_C(   571.39), SIMDE_FLOAT32_C(   422.11),
        SIMDE_FLOAT32_C(   520.20), SIMDE_FLOAT32_C(   536.40), SIMDE_FLOAT32_C(   462.32), SIMDE_FLOAT32_C(   841.06) },
      { SIMDE_FLOAT32_C(     8.92), SIMDE_FLOAT32_C(     9.10), SIMDE_FLOAT32_C(     9.26), SIMDE_FLOAT32_C(     9.35),
        SIMDE_FLOAT32_C(     9.46), SIMDE_FLOAT32_C(     9.96), SIMDE_FLOAT32_C(     9.25), SIMDE_FLOAT32_C(     9.85),
        SIMDE_FLOAT32_C(     9.06), SIMDE_FLOAT32_C(     9.40), SIMDE_FLOAT32_C(     9.16), SIMDE_FLOAT32_C(     8.72),
        SIMDE_FLOAT32_C(     9.02), SIMDE_FLOAT32_C(     9.07), SIMDE_FLOAT32_C(     8.85), SIMDE_FLOAT32_C(     9.72) } },
    { { SIMDE_FLOAT32_C(   513.13), SIMDE_FLOAT32_C(   741.74), SIMDE_FLOAT32_C(   931.43), SIMDE_FLOAT32_C(   670.23),
        SIMDE_FLOAT32_C(   393.50), SIMDE_FLOAT32_C(   862.99), SIMDE_FLOAT32_C(   343.67), SIMDE_FLOAT32_C(   818.00),
        SIMDE_FLOAT32_C(   637.20), SIMDE_FLOAT32_C(   123.18), SIMDE_FLOAT32_C(   888.07), SIMDE_FLOAT32_C(   327.64),
        SIMDE_FLOAT32_C(   438.36), SIMDE_FLOAT32_C(   579.84), SIMDE_FLOAT32_C(   783.89), SIMDE_FLOAT32_C(   922.33) },
      { SIMDE_FLOAT32_C(     9.00), SIMDE_FLOAT32_C(     9.53), SIMDE_FLOAT32_C(     9.86), SIMDE_FLOAT32_C(     9.39),
        SIMDE_FLOAT32_C(     8.62), SIMDE_FLOAT32_C(     9.75), SIMDE_FLOAT32_C(     8.42), SIMDE_FLOAT32_C(     9.68),
        SIMDE_FLOAT32_C(     9.32), SIMDE_FLOAT32_C(     6.94), SIMDE_FLOAT32_C(     9.79), SIMDE_FLOAT32_C(     8.36),
        SIMDE_FLOAT32_C(     8.78), SIMDE_FLOAT32_C(     9.18), SIMDE_FLOAT32_C(     9.61), SIMDE_FLOAT32_C(     9.85) } },
    { { SIMDE_FLOAT32_C(   130.33), SIMDE_FLOAT32_C(   396.68), SIMDE_FLOAT32_C(   574.70), SIMDE_FLOAT32_C(   833.19),
        SIMDE_FLOAT32_C(   390.52), SIMDE_FLOAT32_C(   183.11), SIMDE_FLOAT32_C(   756.35), SIMDE_FLOAT32_C(   922.43),
        SIMDE_FLOAT32_C(   858.89), SIMDE_FLOAT32_C(   327.75), SIMDE_FLOAT32_C(   344.53), SIMDE_FLOAT32_C(   379.09),
        SIMDE_FLOAT32_C(   864.14), SIMDE_FLOAT32_C(   806.85), SIMDE_FLOAT32_C(   220.15), SIMDE_FLOAT32_C(   377.27) },
      { SIMDE_FLOAT32_C(     7.03), SIMDE_FLOAT32_C(     8.63), SIMDE_FLOAT32_C(     9.17), SIMDE_FLOAT32_C(     9.70),
        SIMDE_FLOAT32_C(     8.61), SIMDE_FLOAT32_C(     7.52), SIMDE_FLOAT32_C(     9.56), SIMDE_FLOAT32_C(     9.85),
        SIMDE_FLOAT32_C(     9.75), SIMDE_FLOAT32_C(     8.36), SIMDE_FLOAT32_C(     8.43), SIMDE_FLOAT32_C(     8.57),
        SIMDE_FLOAT32_C(     9.76), SIMDE_FLOAT32_C(     9.66), SIMDE_FLOAT32_C(     7.78), SIMDE_FLOAT32_C(     8.56) } },
    { { SIMDE_FLOAT32_C(   548.60), SIMDE_FLOAT32_C(   151.58), SIMDE_FLOAT32_C(    47.50), SIMDE_FLOAT32_C(   942.10),
        SIMDE_FLOAT32_C(    14.58), SIMDE_FLOAT32_C(   391.17), SIMDE_FLOAT32_C(   760.10), SIMDE_FLOAT32_C(   651.77),
        SIMDE_FLOAT32_C(   514.35), SIMDE_FLOAT32_C(   648.17), SIMDE_FLOAT32_C(   979.41), SIMDE_FLOAT32_C(   952.70),
        SIMDE_FLOAT32_C(   228.00), SIMDE_FLOAT32_C(   763.30), SIMDE_FLOAT32_C(   875.04), SIMDE_FLOAT32_C(   358.34) },
      { SIMDE_FLOAT32_C(     9.10), SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(     5.57), SIMDE_FLOAT32_C(     9.88),
        SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     8.61), SIMDE_FLOAT32_C(     9.57), SIMDE_FLOAT32_C(     9.35),
        SIMDE_FLOAT32_C(     9.01), SIMDE_FLOAT32_C(     9.34), SIMDE_FLOAT32_C(     9.94), SIMDE_FLOAT32_C(     9.90),
        SIMDE_FLOAT32_C(     7.83), SIMDE_FLOAT32_C(     9.58), SIMDE_FLOAT32_C(     9.77), SIMDE_FLOAT32_C(     8.49) } },
    { { SIMDE_FLOAT32_C(   159.99), SIMDE_FLOAT32_C(   449.73), SIMDE_FLOAT32_C(   191.53), SIMDE_FLOAT32_C(   550.50),
        SIMDE_FLOAT32_C(   632.84), SIMDE_FLOAT32_C(   947.88), SIMDE_FLOAT32_C(   472.93), SIMDE_FLOAT32_C(   491.73),
        SIMDE_FLOAT32_C(   275.62), SIMDE_FLOAT32_C(   817.47), SIMDE_FLOAT32_C(   870.83), SIMDE_FLOAT32_C(   139.76),
        SIMDE_FLOAT32_C(   624.32), SIMDE_FLOAT32_C(    90.98), SIMDE_FLOAT32_C(   517.04), SIMDE_FLOAT32_C(   172.92) },
      { SIMDE_FLOAT32_C(     7.32), SIMDE_FLOAT32_C(     8.81), SIMDE_FLOAT32_C(     7.58), SIMDE_FLOAT32_C(     9.10),
        SIMDE_FLOAT32_C(     9.31), SIMDE_FLOAT32_C(     9.89), SIMDE_FLOAT32_C(     8.89), SIMDE_FLOAT32_C(     8.94),
        SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     9.68), SIMDE_FLOAT32_C(     9.77), SIMDE_FLOAT32_C(     7.13),
        SIMDE_FLOAT32_C(     9.29), SIMDE_FLOAT32_C(     6.51), SIMDE_FLOAT32_C(     9.01), SIMDE_FLOAT32_C(     7.43) } },
    { { SIMDE_FLOAT32_C(   242.56), SIMDE_FLOAT32_C(   564.54), SIMDE_FLOAT32_C(   115.01), SIMDE_FLOAT32_C(   257.14),
        SIMDE_FLOAT32_C(   955.71), SIMDE_FLOAT32_C(   875.12), SIMDE_FLOAT32_C(   908.91), SIMDE_FLOAT32_C(   470.05),
        SIMDE_FLOAT32_C(   523.28), SIMDE_FLOAT32_C(   888.32), SIMDE_FLOAT32_C(   422.76), SIMDE_FLOAT32_C(   751.29),
        SIMDE_FLOAT32_C(   651.63), SIMDE_FLOAT32_C(   297.79), SIMDE_FLOAT32_C(   109.62), SIMDE_FLOAT32_C(   811.61) },
      { SIMDE_FLOAT32_C(     7.92), SIMDE_FLOAT32_C(     9.14), SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     8.01),
        SIMDE_FLOAT32_C(     9.90), SIMDE_FLOAT32_C(     9.77), SIMDE_FLOAT32_C(     9.83), SIMDE_FLOAT32_C(     8.88),
        SIMDE_FLOAT32_C(     9.03), SIMDE_FLOAT32_C(     9.79), SIMDE_FLOAT32_C(     8.72), SIMDE_FLOAT32_C(     9.55),
        SIMDE_FLOAT32_C(     9.35), SIMDE_FLOAT32_C(     8.22), SIMDE_FLOAT32_C(     6.78), SIMDE_FLOAT32_C(     9.66) } },
    { { SIMDE_FLOAT32_C(   747.52), SIMDE_FLOAT32_C(   301.15), SIMDE_FLOAT32_C(   362.12), SIMDE_FLOAT32_C(   380.36),
        SIMDE_FLOAT32_C(   249.03), SIMDE_FLOAT32_C(   835.05), SIMDE_FLOAT32_C(   872.10), SIMDE_FLOAT32_C(   524.65),
        SIMDE_FLOAT32_C(   652.52), SIMDE_FLOAT32_C(   742.92), SIMDE_FLOAT32_C(   664.41), SIMDE_FLOAT32_C(   276.84),
        SIMDE_FLOAT32_C(   833.90), SIMDE_FLOAT32_C(   181.45), SIMDE_FLOAT32_C(   449.75), SIMDE_FLOAT32_C(    76.46) },
      { SIMDE_FLOAT32_C(     9.55), SIMDE_FLOAT32_C(     8.23), SIMDE_FLOAT32_C(     8.50), SIMDE_FLOAT32_C(     8.57),
        SIMDE_FLOAT32_C(     7.96), SIMDE_FLOAT32_C(     9.71), SIMDE_FLOAT32_C(     9.77), SIMDE_FLOAT32_C(     9.04),
        SIMDE_FLOAT32_C(     9.35), SIMDE_FLOAT32_C(     9.54), SIMDE_FLOAT32_C(     9.38), SIMDE_FLOAT32_C(     8.11),
        SIMDE_FLOAT32_C(     9.70), SIMDE_FLOAT32_C(     7.50), SIMDE_FLOAT32_C(     8.81), SIMDE_FLOAT32_C(     6.26) } },
    { { SIMDE_FLOAT32_C(   745.98), SIMDE_FLOAT32_C(   564.77), SIMDE_FLOAT32_C(   333.60), SIMDE_FLOAT32_C(   701.69),
        SIMDE_FLOAT32_C(   439.88), SIMDE_FLOAT32_C(   242.51), SIMDE_FLOAT32_C(   171.74), SIMDE_FLOAT32_C(   963.17),
        SIMDE_FLOAT32_C(   130.83), SIMDE_FLOAT32_C(   594.50), SIMDE_FLOAT32_C(   714.46), SIMDE_FLOAT32_C(   782.46),
        SIMDE_FLOAT32_C(   892.29), SIMDE_FLOAT32_C(   824.08), SIMDE_FLOAT32_C(   594.07), SIMDE_FLOAT32_C(   639.81) },
      { SIMDE_FLOAT32_C(     9.54), SIMDE_FLOAT32_C(     9.14), SIMDE_FLOAT32_C(     8.38), SIMDE_FLOAT32_C(     9.45),
        SIMDE_FLOAT32_C(     8.78), SIMDE_FLOAT32_C(     7.92), SIMDE_FLOAT32_C(     7.42), SIMDE_FLOAT32_C(     9.91),
        SIMDE_FLOAT32_C(     7.03), SIMDE_FLOAT32_C(     9.22), SIMDE_FLOAT32_C(     9.48), SIMDE_FLOAT32_C(     9.61),
        SIMDE_FLOAT32_C(     9.80), SIMDE_FLOAT32_C(     9.69), SIMDE_FLOAT32_C(     9.21), SIMDE_FLOAT32_C(     9.32) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_log2_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_log2_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   951.54), SIMDE_FLOAT32_C(   999.54), SIMDE_FLOAT32_C(   334.61), SIMDE_FLOAT32_C(   345.31),
        SIMDE_FLOAT32_C(   632.13), SIMDE_FLOAT32_C(   486.36), SIMDE_FLOAT32_C(   855.38), SIMDE_FLOAT32_C(   575.68),
        SIMDE_FLOAT32_C(   586.36), SIMDE_FLOAT32_C(   821.37), SIMDE_FLOAT32_C(   638.17), SIMDE_FLOAT32_C(   965.64),
        SIMDE_FLOAT32_C(   565.55), SIMDE_FLOAT32_C(   416.08), SIMDE_FLOAT32_C(   543.83), SIMDE_FLOAT32_C(   785.84) },
      UINT8_C( 38),
      { SIMDE_FLOAT32_C(   694.42), SIMDE_FLOAT32_C(    92.26), SIMDE_FLOAT32_C(   723.42), SIMDE_FLOAT32_C(   203.15),
        SIMDE_FLOAT32_C(   315.73), SIMDE_FLOAT32_C(   806.95), SIMDE_FLOAT32_C(   395.41), SIMDE_FLOAT32_C(   157.52),
        SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(   504.24), SIMDE_FLOAT32_C(   237.89), SIMDE_FLOAT32_C(   806.42),
        SIMDE_FLOAT32_C(   668.52), SIMDE_FLOAT32_C(   921.63), SIMDE_FLOAT32_C(   757.96), SIMDE_FLOAT32_C(   668.06) },
      { SIMDE_FLOAT32_C(   951.54), SIMDE_FLOAT32_C(     6.53), SIMDE_FLOAT32_C(     9.50), SIMDE_FLOAT32_C(   345.31),
        SIMDE_FLOAT32_C(   632.13), SIMDE_FLOAT32_C(     9.66), SIMDE_FLOAT32_C(   855.38), SIMDE_FLOAT32_C(   575.68),
        SIMDE_FLOAT32_C(   586.36), SIMDE_FLOAT32_C(   821.37), SIMDE_FLOAT32_C(   638.17), SIMDE_FLOAT32_C(   965.64),
        SIMDE_FLOAT32_C(   565.55), SIMDE_FLOAT32_C(   416.08), SIMDE_FLOAT32_C(   543.83), SIMDE_FLOAT32_C(   785.84) } },
    { { SIMDE_FLOAT32_C(   256.24), SIMDE_FLOAT32_C(   103.27), SIMDE_FLOAT32_C(   300.20), SIMDE_FLOAT32_C(   742.60),
        SIMDE_FLOAT32_C(   958.65), SIMDE_FLOAT32_C(   875.88), SIMDE_FLOAT32_C(   328.96), SIMDE_FLOAT32_C(   780.02),
        SIMDE_FLOAT32_C(   514.05), SIMDE_FLOAT32_C(   294.61), SIMDE_FLOAT32_C(   345.57), SIMDE_FLOAT32_C(   930.14),
        SIMDE_FLOAT32_C(   838.44), SIMDE_FLOAT32_C(   131.42), SIMDE_FLOAT32_C(    65.69), SIMDE_FLOAT32_C(   532.86) },
      UINT8_C(234),
      { SIMDE_FLOAT32_C(   789.11), SIMDE_FLOAT32_C(   736.01), SIMDE_FLOAT32_C(   539.40), SIMDE_FLOAT32_C(   596.06),
        SIMDE_FLOAT32_C(   131.42), SIMDE_FLOAT32_C(   696.92), SIMDE_FLOAT32_C(   597.63), SIMDE_FLOAT32_C(   635.66),
        SIMDE_FLOAT32_C(   934.80), SIMDE_FLOAT32_C(   404.05), SIMDE_FLOAT32_C(   304.18), SIMDE_FLOAT32_C(   856.43),
        SIMDE_FLOAT32_C(   162.01), SIMDE_FLOAT32_C(   972.25), SIMDE_FLOAT32_C(   112.67), SIMDE_FLOAT32_C(   265.28) },
      { SIMDE_FLOAT32_C(   256.24), SIMDE_FLOAT32_C(     9.52), SIMDE_FLOAT32_C(   300.20), SIMDE_FLOAT32_C(     9.22),
        SIMDE_FLOAT32_C(   958.65), SIMDE_FLOAT32_C(     9.44), SIMDE_FLOAT32_C(     9.22), SIMDE_FLOAT32_C(     9.31),
        SIMDE_FLOAT32_C(   514.05), SIMDE_FLOAT32_C(   294.61), SIMDE_FLOAT32_C(   345.57), SIMDE_FLOAT32_C(   930.14),
        SIMDE_FLOAT32_C(   838.44), SIMDE_FLOAT32_C(   131.42), SIMDE_FLOAT32_C(    65.69), SIMDE_FLOAT32_C(   532.86) } },
    { { SIMDE_FLOAT32_C(   272.44), SIMDE_FLOAT32_C(   855.27), SIMDE_FLOAT32_C(   223.93), SIMDE_FLOAT32_C(   148.32),
        SIMDE_FLOAT32_C(   184.23), SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(   662.37), SIMDE_FLOAT32_C(   478.84),
        SIMDE_FLOAT32_C(   349.52), SIMDE_FLOAT32_C(   592.51), SIMDE_FLOAT32_C(   317.28), SIMDE_FLOAT32_C(   480.94),
        SIMDE_FLOAT32_C(   658.20), SIMDE_FLOAT32_C(   850.14), SIMDE_FLOAT32_C(   704.61), SIMDE_FLOAT32_C(   447.31) },
      UINT8_C(189),
      { SIMDE_FLOAT32_C(   244.01), SIMDE_FLOAT32_C(    43.37), SIMDE_FLOAT32_C(   717.57), SIMDE_FLOAT32_C(   940.93),
        SIMDE_FLOAT32_C(   641.00), SIMDE_FLOAT32_C(   353.24), SIMDE_FLOAT32_C(   875.73), SIMDE_FLOAT32_C(    45.05),
        SIMDE_FLOAT32_C(   657.42), SIMDE_FLOAT32_C(   732.16), SIMDE_FLOAT32_C(   207.05), SIMDE_FLOAT32_C(   629.67),
        SIMDE_FLOAT32_C(   844.83), SIMDE_FLOAT32_C(   472.33), SIMDE_FLOAT32_C(   902.11), SIMDE_FLOAT32_C(   700.10) },
      { SIMDE_FLOAT32_C(     7.93), SIMDE_FLOAT32_C(   855.27), SIMDE_FLOAT32_C(     9.49), SIMDE_FLOAT32_C(     9.88),
        SIMDE_FLOAT32_C(     9.32), SIMDE_FLOAT32_C(     8.46), SIMDE_FLOAT32_C(   662.37), SIMDE_FLOAT32_C(     5.49),
        SIMDE_FLOAT32_C(   349.52), SIMDE_FLOAT32_C(   592.51), SIMDE_FLOAT32_C(   317.28), SIMDE_FLOAT32_C(   480.94),
        SIMDE_FLOAT32_C(   658.20), SIMDE_FLOAT32_C(   850.14), SIMDE_FLOAT32_C(   704.61), SIMDE_FLOAT32_C(   447.31) } },
    { { SIMDE_FLOAT32_C(   696.26), SIMDE_FLOAT32_C(    50.44), SIMDE_FLOAT32_C(   884.33), SIMDE_FLOAT32_C(   700.20),
        SIMDE_FLOAT32_C(   712.81), SIMDE_FLOAT32_C(   363.17), SIMDE_FLOAT32_C(    49.73), SIMDE_FLOAT32_C(   305.32),
        SIMDE_FLOAT32_C(   680.45), SIMDE_FLOAT32_C(   530.67), SIMDE_FLOAT32_C(   963.52), SIMDE_FLOAT32_C(   530.59),
        SIMDE_FLOAT32_C(   235.28), SIMDE_FLOAT32_C(   410.84), SIMDE_FLOAT32_C(   116.75), SIMDE_FLOAT32_C(   479.29) },
      UINT8_C(235),
      { SIMDE_FLOAT32_C(   834.32), SIMDE_FLOAT32_C(   420.22), SIMDE_FLOAT32_C(    95.21), SIMDE_FLOAT32_C(   187.56),
        SIMDE_FLOAT32_C(   295.95), SIMDE_FLOAT32_C(   140.25), SIMDE_FLOAT32_C(   844.98), SIMDE_FLOAT32_C(    28.11),
        SIMDE_FLOAT32_C(   347.31), SIMDE_FLOAT32_C(   474.66), SIMDE_FLOAT32_C(   872.94), SIMDE_FLOAT32_C(   819.64),
        SIMDE_FLOAT32_C(   376.77), SIMDE_FLOAT32_C(   573.04), SIMDE_FLOAT32_C(   515.89), SIMDE_FLOAT32_C(   427.21) },
      { SIMDE_FLOAT32_C(     9.70), SIMDE_FLOAT32_C(     8.72), SIMDE_FLOAT32_C(   884.33), SIMDE_FLOAT32_C(     7.55),
        SIMDE_FLOAT32_C(   712.81), SIMDE_FLOAT32_C(     7.13), SIMDE_FLOAT32_C(     9.72), SIMDE_FLOAT32_C(     4.81),
        SIMDE_FLOAT32_C(   680.45), SIMDE_FLOAT32_C(   530.67), SIMDE_FLOAT32_C(   963.52), SIMDE_FLOAT32_C(   530.59),
        SIMDE_FLOAT32_C(   235.28), SIMDE_FLOAT32_C(   410.84), SIMDE_FLOAT32_C(   116.75), SIMDE_FLOAT32_C(   479.29) } },
    { { SIMDE_FLOAT32_C(   457.38), SIMDE_FLOAT32_C(   216.10), SIMDE_FLOAT32_C(   140.02), SIMDE_FLOAT32_C(   820.55),
        SIMDE_FLOAT32_C(   265.82), SIMDE_FLOAT32_C(   445.34), SIMDE_FLOAT32_C(   501.00), SIMDE_FLOAT32_C(   796.49),
        SIMDE_FLOAT32_C(   408.86), SIMDE_FLOAT32_C(    31.60), SIMDE_FLOAT32_C(    31.77), SIMDE_FLOAT32_C(   819.70),
        SIMDE_FLOAT32_C(   148.34), SIMDE_FLOAT32_C(   511.06), SIMDE_FLOAT32_C(   273.91), SIMDE_FLOAT32_C(   982.67) },
      UINT8_C(170),
      { SIMDE_FLOAT32_C(   369.11), SIMDE_FLOAT32_C(   170.23), SIMDE_FLOAT32_C(   227.24), SIMDE_FLOAT32_C(   509.37),
        SIMDE_FLOAT32_C(    15.21), SIMDE_FLOAT32_C(   255.36), SIMDE_FLOAT32_C(   856.67), SIMDE_FLOAT32_C(   489.87),
        SIMDE_FLOAT32_C(   128.30), SIMDE_FLOAT32_C(   676.31), SIMDE_FLOAT32_C(   866.64), SIMDE_FLOAT32_C(   701.34),
        SIMDE_FLOAT32_C(   192.20), SIMDE_FLOAT32_C(   293.84), SIMDE_FLOAT32_C(   158.72), SIMDE_FLOAT32_C(   408.30) },
      { SIMDE_FLOAT32_C(   457.38), SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(   140.02), SIMDE_FLOAT32_C(     8.99),
        SIMDE_FLOAT32_C(   265.82), SIMDE_FLOAT32_C(     8.00), SIMDE_FLOAT32_C(   501.00), SIMDE_FLOAT32_C(     8.94),
        SIMDE_FLOAT32_C(   408.86), SIMDE_FLOAT32_C(    31.60), SIMDE_FLOAT32_C(    31.77), SIMDE_FLOAT32_C(   819.70),
        SIMDE_FLOAT32_C(   148.34), SIMDE_FLOAT32_C(   511.06), SIMDE_FLOAT32_C(   273.91), SIMDE_FLOAT32_C(   982.67) } },
    { { SIMDE_FLOAT32_C(   433.86), SIMDE_FLOAT32_C(   979.27), SIMDE_FLOAT32_C(   674.13), SIMDE_FLOAT32_C(   879.20),
        SIMDE_FLOAT32_C(   480.27), SIMDE_FLOAT32_C(   470.62), SIMDE_FLOAT32_C(   288.06), SIMDE_FLOAT32_C(   511.87),
        SIMDE_FLOAT32_C(   502.39), SIMDE_FLOAT32_C(   107.76), SIMDE_FLOAT32_C(   660.21), SIMDE_FLOAT32_C(    13.45),
        SIMDE_FLOAT32_C(   381.67), SIMDE_FLOAT32_C(   642.88), SIMDE_FLOAT32_C(   944.74), SIMDE_FLOAT32_C(   750.78) },
      UINT8_C( 15),
      { SIMDE_FLOAT32_C(   171.98), SIMDE_FLOAT32_C(   260.15), SIMDE_FLOAT32_C(   828.32), SIMDE_FLOAT32_C(   427.33),
        SIMDE_FLOAT32_C(   116.82), SIMDE_FLOAT32_C(   318.18), SIMDE_FLOAT32_C(   555.63), SIMDE_FLOAT32_C(   793.13),
        SIMDE_FLOAT32_C(   184.82), SIMDE_FLOAT32_C(   256.97), SIMDE_FLOAT32_C(   985.33), SIMDE_FLOAT32_C(   478.66),
        SIMDE_FLOAT32_C(   415.69), SIMDE_FLOAT32_C(   393.63), SIMDE_FLOAT32_C(   912.52), SIMDE_FLOAT32_C(   394.96) },
      { SIMDE_FLOAT32_C(     7.43), SIMDE_FLOAT32_C(     8.02), SIMDE_FLOAT32_C(     9.69), SIMDE_FLOAT32_C(     8.74),
        SIMDE_FLOAT32_C(   480.27), SIMDE_FLOAT32_C(   470.62), SIMDE_FLOAT32_C(   288.06), SIMDE_FLOAT32_C(   511.87),
        SIMDE_FLOAT32_C(   502.39), SIMDE_FLOAT32_C(   107.76), SIMDE_FLOAT32_C(   660.21), SIMDE_FLOAT32_C(    13.45),
        SIMDE_FLOAT32_C(   381.67), SIMDE_FLOAT32_C(   642.88), SIMDE_FLOAT32_C(   944.74), SIMDE_FLOAT32_C(   750.78) } },
    { { SIMDE_FLOAT32_C(    67.76), SIMDE_FLOAT32_C(   791.72), SIMDE_FLOAT32_C(   875.23), SIMDE_FLOAT32_C(   538.38),
        SIMDE_FLOAT32_C(    79.78), SIMDE_FLOAT32_C(   387.09), SIMDE_FLOAT32_C(    40.77), SIMDE_FLOAT32_C(   187.54),
        SIMDE_FLOAT32_C(    47.31), SIMDE_FLOAT32_C(    54.22), SIMDE_FLOAT32_C(   569.20), SIMDE_FLOAT32_C(   690.18),
        SIMDE_FLOAT32_C(   998.96), SIMDE_FLOAT32_C(   319.98), SIMDE_FLOAT32_C(   503.29), SIMDE_FLOAT32_C(   170.94) },
      UINT8_C( 81),
      { SIMDE_FLOAT32_C(   331.60), SIMDE_FLOAT32_C(   598.27), SIMDE_FLOAT32_C(   696.95), SIMDE_FLOAT32_C(   649.79),
        SIMDE_FLOAT32_C(   153.90), SIMDE_FLOAT32_C(   490.08), SIMDE_FLOAT32_C(   834.61), SIMDE_FLOAT32_C(   410.88),
        SIMDE_FLOAT32_C(   475.41), SIMDE_FLOAT32_C(   313.27), SIMDE_FLOAT32_C(   826.57), SIMDE_FLOAT32_C(   869.04),
        SIMDE_FLOAT32_C(   225.79), SIMDE_FLOAT32_C(   221.52), SIMDE_FLOAT32_C(   936.81), SIMDE_FLOAT32_C(    17.51) },
      { SIMDE_FLOAT32_C(     8.37), SIMDE_FLOAT32_C(   791.72), SIMDE_FLOAT32_C(   875.23), SIMDE_FLOAT32_C(   538.38),
        SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(   387.09), SIMDE_FLOAT32_C(     9.70), SIMDE_FLOAT32_C(   187.54),
        SIMDE_FLOAT32_C(    47.31), SIMDE_FLOAT32_C(    54.22), SIMDE_FLOAT32_C(   569.20), SIMDE_FLOAT32_C(   690.18),
        SIMDE_FLOAT32_C(   998.96), SIMDE_FLOAT32_C(   319.98), SIMDE_FLOAT32_C(   503.29), SIMDE_FLOAT32_C(   170.94) } },
    { { SIMDE_FLOAT32_C(    96.75), SIMDE_FLOAT32_C(   475.18), SIMDE_FLOAT32_C(    97.29), SIMDE_FLOAT32_C(   483.84),
        SIMDE_FLOAT32_C(   515.95), SIMDE_FLOAT32_C(   284.83), SIMDE_FLOAT32_C(   531.15), SIMDE_FLOAT32_C(   570.17),
        SIMDE_FLOAT32_C(   854.03), SIMDE_FLOAT32_C(   221.33), SIMDE_FLOAT32_C(   569.13), SIMDE_FLOAT32_C(   174.01),
        SIMDE_FLOAT32_C(   724.62), SIMDE_FLOAT32_C(   740.06), SIMDE_FLOAT32_C(   754.14), SIMDE_FLOAT32_C(    56.23) },
      UINT8_C(124),
      { SIMDE_FLOAT32_C(   451.09), SIMDE_FLOAT32_C(   706.02), SIMDE_FLOAT32_C(   492.24), SIMDE_FLOAT32_C(   941.16),
        SIMDE_FLOAT32_C(   540.62), SIMDE_FLOAT32_C(   903.11), SIMDE_FLOAT32_C(   416.57), SIMDE_FLOAT32_C(   853.89),
        SIMDE_FLOAT32_C(   729.68), SIMDE_FLOAT32_C(   285.62), SIMDE_FLOAT32_C(    79.69), SIMDE_FLOAT32_C(   951.20),
        SIMDE_FLOAT32_C(   222.42), SIMDE_FLOAT32_C(    97.20), SIMDE_FLOAT32_C(    47.95), SIMDE_FLOAT32_C(   697.61) },
      { SIMDE_FLOAT32_C(    96.75), SIMDE_FLOAT32_C(   475.18), SIMDE_FLOAT32_C(     8.94), SIMDE_FLOAT32_C(     9.88),
        SIMDE_FLOAT32_C(     9.08), SIMDE_FLOAT32_C(     9.82), SIMDE_FLOAT32_C(     8.70), SIMDE_FLOAT32_C(   570.17),
        SIMDE_FLOAT32_C(   854.03), SIMDE_FLOAT32_C(   221.33), SIMDE_FLOAT32_C(   569.13), SIMDE_FLOAT32_C(   174.01),
        SIMDE_FLOAT32_C(   724.62), SIMDE_FLOAT32_C(   740.06), SIMDE_FLOAT32_C(   754.14), SIMDE_FLOAT32_C(    56.23) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_log2_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_log2_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   119.65), SIMDE_FLOAT64_C(   209.41), SIMDE_FLOAT64_C(   983.20), SIMDE_FLOAT64_C(   968.40),
        SIMDE_FLOAT64_C(   158.45), SIMDE_FLOAT64_C(   611.79), SIMDE_FLOAT64_C(   202.67), SIMDE_FLOAT64_C(    73.75) },
      { SIMDE_FLOAT64_C(     6.90), SIMDE_FLOAT64_C(     7.71), SIMDE_FLOAT64_C(     9.94), SIMDE_FLOAT64_C(     9.92),
        SIMDE_FLOAT64_C(     7.31), SIMDE_FLOAT64_C(     9.26), SIMDE_FLOAT64_C(     7.66), SIMDE_FLOAT64_C(     6.20) } },
    { { SIMDE_FLOAT64_C(   875.12), SIMDE_FLOAT64_C(   357.46), SIMDE_FLOAT64_C(   960.14), SIMDE_FLOAT64_C(   477.36),
        SIMDE_FLOAT64_C(   185.60), SIMDE_FLOAT64_C(   437.48), SIMDE_FLOAT64_C(   656.75), SIMDE_FLOAT64_C(   468.11) },
      { SIMDE_FLOAT64_C(     9.77), SIMDE_FLOAT64_C(     8.48), SIMDE_FLOAT64_C(     9.91), SIMDE_FLOAT64_C(     8.90),
        SIMDE_FLOAT64_C(     7.54), SIMDE_FLOAT64_C(     8.77), SIMDE_FLOAT64_C(     9.36), SIMDE_FLOAT64_C(     8.87) } },
    { { SIMDE_FLOAT64_C(   538.86), SIMDE_FLOAT64_C(   465.92), SIMDE_FLOAT64_C(   597.15), SIMDE_FLOAT64_C(   858.12),
        SIMDE_FLOAT64_C(   110.06), SIMDE_FLOAT64_C(   149.17), SIMDE_FLOAT64_C(    41.30), SIMDE_FLOAT64_C(   954.56) },
      { SIMDE_FLOAT64_C(     9.07), SIMDE_FLOAT64_C(     8.86), SIMDE_FLOAT64_C(     9.22), SIMDE_FLOAT64_C(     9.75),
        SIMDE_FLOAT64_C(     6.78), SIMDE_FLOAT64_C(     7.22), SIMDE_FLOAT64_C(     5.37), SIMDE_FLOAT64_C(     9.90) } },
    { { SIMDE_FLOAT64_C(   919.40), SIMDE_FLOAT64_C(    93.55), SIMDE_FLOAT64_C(   761.38), SIMDE_FLOAT64_C(   128.98),
        SIMDE_FLOAT64_C(   873.27), SIMDE_FLOAT64_C(   719.89), SIMDE_FLOAT64_C(   554.57), SIMDE_FLOAT64_C(   992.93) },
      { SIMDE_FLOAT64_C(     9.84), SIMDE_FLOAT64_C(     6.55), SIMDE_FLOAT64_C(     9.57), SIMDE_FLOAT64_C(     7.01),
        SIMDE_FLOAT64_C(     9.77), SIMDE_FLOAT64_C(     9.49), SIMDE_FLOAT64_C(     9.12), SIMDE_FLOAT64_C(     9.96) } },
    { { SIMDE_FLOAT64_C(   929.29), SIMDE_FLOAT64_C(   537.77), SIMDE_FLOAT64_C(   961.32), SIMDE_FLOAT64_C(    87.74),
        SIMDE_FLOAT64_C(   149.55), SIMDE_FLOAT64_C(   164.00), SIMDE_FLOAT64_C(   161.49), SIMDE_FLOAT64_C(    24.67) },
      { SIMDE_FLOAT64_C(     9.86), SIMDE_FLOAT64_C(     9.07), SIMDE_FLOAT64_C(     9.91), SIMDE_FLOAT64_C(     6.46),
        SIMDE_FLOAT64_C(     7.22), SIMDE_FLOAT64_C(     7.36), SIMDE_FLOAT64_C(     7.34), SIMDE_FLOAT64_C(     4.62) } },
    { { SIMDE_FLOAT64_C(   521.46), SIMDE_FLOAT64_C(   121.63), SIMDE_FLOAT64_C(   502.03), SIMDE_FLOAT64_C(   707.07),
        SIMDE_FLOAT64_C(   559.11), SIMDE_FLOAT64_C(   158.78), SIMDE_FLOAT64_C(   175.18), SIMDE_FLOAT64_C(    97.96) },
      { SIMDE_FLOAT64_C(     9.03), SIMDE_FLOAT64_C(     6.93), SIMDE_FLOAT64_C(     8.97), SIMDE_FLOAT64_C(     9.47),
        SIMDE_FLOAT64_C(     9.13), SIMDE_FLOAT64_C(     7.31), SIMDE_FLOAT64_C(     7.45), SIMDE_FLOAT64_C(     6.61) } },
    { { SIMDE_FLOAT64_C(   624.70), SIMDE_FLOAT64_C(   772.32), SIMDE_FLOAT64_C(   956.08), SIMDE_FLOAT64_C(   734.75),
        SIMDE_FLOAT64_C(   921.49), SIMDE_FLOAT64_C(   997.38), SIMDE_FLOAT64_C(   689.31), SIMDE_FLOAT64_C(   840.89) },
      { SIMDE_FLOAT64_C(     9.29), SIMDE_FLOAT64_C(     9.59), SIMDE_FLOAT64_C(     9.90), SIMDE_FLOAT64_C(     9.52),
        SIMDE_FLOAT64_C(     9.85), SIMDE_FLOAT64_C(     9.96), SIMDE_FLOAT64_C(     9.43), SIMDE_FLOAT64_C(     9.72) } },
    { { SIMDE_FLOAT64_C(    90.93), SIMDE_FLOAT64_C(   450.70), SIMDE_FLOAT64_C(   969.87), SIMDE_FLOAT64_C(   964.20),
        SIMDE_FLOAT64_C(   170.58), SIMDE_FLOAT64_C(   524.44), SIMDE_FLOAT64_C(   957.13), SIMDE_FLOAT64_C(    99.88) },
      { SIMDE_FLOAT64_C(     6.51), SIMDE_FLOAT64_C(     8.82), SIMDE_FLOAT64_C(     9.92), SIMDE_FLOAT64_C(     9.91),
        SIMDE_FLOAT64_C(     7.41), SIMDE_FLOAT64_C(     9.03), SIMDE_FLOAT64_C(     9.90), SIMDE_FLOAT64_C(     6.64) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_log2_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_log2_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   996.08), SIMDE_FLOAT64_C(    61.08), SIMDE_FLOAT64_C(   921.49), SIMDE_FLOAT64_C(   192.89),
        SIMDE_FLOAT64_C(   553.14), SIMDE_FLOAT64_C(    14.27), SIMDE_FLOAT64_C(   408.18), SIMDE_FLOAT64_C(   275.52) },
      UINT8_C( 23),
      { SIMDE_FLOAT64_C(   470.67), SIMDE_FLOAT64_C(   327.03), SIMDE_FLOAT64_C(   335.52), SIMDE_FLOAT64_C(   992.77),
        SIMDE_FLOAT64_C(   465.65), SIMDE_FLOAT64_C(   524.14), SIMDE_FLOAT64_C(   178.22), SIMDE_FLOAT64_C(   860.48) },
      { SIMDE_FLOAT64_C(     8.88), SIMDE_FLOAT64_C(     8.35), SIMDE_FLOAT64_C(     8.39), SIMDE_FLOAT64_C(   192.89),
        SIMDE_FLOAT64_C(     8.86), SIMDE_FLOAT64_C(    14.27), SIMDE_FLOAT64_C(   408.18), SIMDE_FLOAT64_C(   275.52) } },
    { { SIMDE_FLOAT64_C(   594.48), SIMDE_FLOAT64_C(   196.19), SIMDE_FLOAT64_C(   493.93), SIMDE_FLOAT64_C(   252.94),
        SIMDE_FLOAT64_C(   940.21), SIMDE_FLOAT64_C(   104.98), SIMDE_FLOAT64_C(   946.96), SIMDE_FLOAT64_C(   783.58) },
      UINT8_C(251),
      { SIMDE_FLOAT64_C(   815.52), SIMDE_FLOAT64_C(   353.82), SIMDE_FLOAT64_C(   583.31), SIMDE_FLOAT64_C(   335.41),
        SIMDE_FLOAT64_C(   693.48), SIMDE_FLOAT64_C(   579.39), SIMDE_FLOAT64_C(   396.49), SIMDE_FLOAT64_C(   614.97) },
      { SIMDE_FLOAT64_C(     9.67), SIMDE_FLOAT64_C(     8.47), SIMDE_FLOAT64_C(   493.93), SIMDE_FLOAT64_C(     8.39),
        SIMDE_FLOAT64_C(     9.44), SIMDE_FLOAT64_C(     9.18), SIMDE_FLOAT64_C(     8.63), SIMDE_FLOAT64_C(     9.26) } },
    { { SIMDE_FLOAT64_C(   772.28), SIMDE_FLOAT64_C(   949.63), SIMDE_FLOAT64_C(   629.24), SIMDE_FLOAT64_C(   180.46),
        SIMDE_FLOAT64_C(   225.15), SIMDE_FLOAT64_C(   527.05), SIMDE_FLOAT64_C(   651.14), SIMDE_FLOAT64_C(   552.19) },
      UINT8_C(241),
      { SIMDE_FLOAT64_C(   643.90), SIMDE_FLOAT64_C(    17.84), SIMDE_FLOAT64_C(   386.72), SIMDE_FLOAT64_C(   822.12),
        SIMDE_FLOAT64_C(   878.32), SIMDE_FLOAT64_C(   981.20), SIMDE_FLOAT64_C(    18.32), SIMDE_FLOAT64_C(   372.25) },
      { SIMDE_FLOAT64_C(     9.33), SIMDE_FLOAT64_C(   949.63), SIMDE_FLOAT64_C(   629.24), SIMDE_FLOAT64_C(   180.46),
        SIMDE_FLOAT64_C(     9.78), SIMDE_FLOAT64_C(     9.94), SIMDE_FLOAT64_C(     4.19), SIMDE_FLOAT64_C(     8.54) } },
    { { SIMDE_FLOAT64_C(   234.14), SIMDE_FLOAT64_C(   958.52), SIMDE_FLOAT64_C(   477.23), SIMDE_FLOAT64_C(   181.10),
        SIMDE_FLOAT64_C(   742.10), SIMDE_FLOAT64_C(   235.40), SIMDE_FLOAT64_C(   996.62), SIMDE_FLOAT64_C(    95.92) },
      UINT8_C( 71),
      { SIMDE_FLOAT64_C(   332.03), SIMDE_FLOAT64_C(   789.40), SIMDE_FLOAT64_C(   398.10), SIMDE_FLOAT64_C(   728.52),
        SIMDE_FLOAT64_C(   404.38), SIMDE_FLOAT64_C(   170.38), SIMDE_FLOAT64_C(   678.16), SIMDE_FLOAT64_C(    33.62) },
      { SIMDE_FLOAT64_C(     8.38), SIMDE_FLOAT64_C(     9.62), SIMDE_FLOAT64_C(     8.64), SIMDE_FLOAT64_C(   181.10),
        SIMDE_FLOAT64_C(   742.10), SIMDE_FLOAT64_C(   235.40), SIMDE_FLOAT64_C(     9.41), SIMDE_FLOAT64_C(    95.92) } },
    { { SIMDE_FLOAT64_C(   350.85), SIMDE_FLOAT64_C(   903.31), SIMDE_FLOAT64_C(   560.67), SIMDE_FLOAT64_C(     1.98),
        SIMDE_FLOAT64_C(   455.50), SIMDE_FLOAT64_C(   423.25), SIMDE_FLOAT64_C(   645.89), SIMDE_FLOAT64_C(   473.34) },
      UINT8_C(167),
      { SIMDE_FLOAT64_C(   468.01), SIMDE_FLOAT64_C(   351.66), SIMDE_FLOAT64_C(   791.16), SIMDE_FLOAT64_C(   486.32),
        SIMDE_FLOAT64_C(   723.90), SIMDE_FLOAT64_C(    25.30), SIMDE_FLOAT64_C(   444.84), SIMDE_FLOAT64_C(   201.13) },
      { SIMDE_FLOAT64_C(     8.87), SIMDE_FLOAT64_C(     8.46), SIMDE_FLOAT64_C(     9.63), SIMDE_FLOAT64_C(     1.98),
        SIMDE_FLOAT64_C(   455.50), SIMDE_FLOAT64_C(     4.66), SIMDE_FLOAT64_C(   645.89), SIMDE_FLOAT64_C(     7.65) } },
    { { SIMDE_FLOAT64_C(   206.40), SIMDE_FLOAT64_C(   186.94), SIMDE_FLOAT64_C(   436.54), SIMDE_FLOAT64_C(   203.02),
        SIMDE_FLOAT64_C(   282.87), SIMDE_FLOAT64_C(   255.25), SIMDE_FLOAT64_C(   535.05), SIMDE_FLOAT64_C(    72.27) },
      UINT8_C(195),
      { SIMDE_FLOAT64_C(   263.57), SIMDE_FLOAT64_C(   476.64), SIMDE_FLOAT64_C(   823.73), SIMDE_FLOAT64_C(   941.73),
        SIMDE_FLOAT64_C(   510.26), SIMDE_FLOAT64_C(   174.57), SIMDE_FLOAT64_C(   845.04), SIMDE_FLOAT64_C(    70.93) },
      { SIMDE_FLOAT64_C(     8.04), SIMDE_FLOAT64_C(     8.90), SIMDE_FLOAT64_C(   436.54), SIMDE_FLOAT64_C(   203.02),
        SIMDE_FLOAT64_C(   282.87), SIMDE_FLOAT64_C(   255.25), SIMDE_FLOAT64_C(     9.72), SIMDE_FLOAT64_C(     6.15) } },
    { { SIMDE_FLOAT64_C(   176.55), SIMDE_FLOAT64_C(   300.54), SIMDE_FLOAT64_C(   494.17), SIMDE_FLOAT64_C(   822.44),
        SIMDE_FLOAT64_C(   773.88), SIMDE_FLOAT64_C(   304.14), SIMDE_FLOAT64_C(   290.45), SIMDE_FLOAT64_C(   125.54) },
      UINT8_C( 79),
      { SIMDE_FLOAT64_C(   776.77), SIMDE_FLOAT64_C(   849.44), SIMDE_FLOAT64_C(   120.60), SIMDE_FLOAT64_C(   221.61),
        SIMDE_FLOAT64_C(    50.57), SIMDE_FLOAT64_C(   326.99), SIMDE_FLOAT64_C(   408.55), SIMDE_FLOAT64_C(   487.11) },
      { SIMDE_FLOAT64_C(     9.60), SIMDE_FLOAT64_C(     9.73), SIMDE_FLOAT64_C(     6.91), SIMDE_FLOAT64_C(     7.79),
        SIMDE_FLOAT64_C(   773.88), SIMDE_FLOAT64_C(   304.14), SIMDE_FLOAT64_C(     8.67), SIMDE_FLOAT64_C(   125.54) } },
    { { SIMDE_FLOAT64_C(   530.01), SIMDE_FLOAT64_C(   691.42), SIMDE_FLOAT64_C(   742.35), SIMDE_FLOAT64_C(    65.06),
        SIMDE_FLOAT64_C(   763.69), SIMDE_FLOAT64_C(   395.70), SIMDE_FLOAT64_C(   328.63), SIMDE_FLOAT64_C(   240.33) },
      UINT8_C( 12),
      { SIMDE_FLOAT64_C(   270.37), SIMDE_FLOAT64_C(   750.59), SIMDE_FLOAT64_C(   394.00), SIMDE_FLOAT64_C(   115.41),
        SIMDE_FLOAT64_C(   821.52), SIMDE_FLOAT64_C(   570.56), SIMDE_FLOAT64_C(   415.95), SIMDE_FLOAT64_C(   315.69) },
      { SIMDE_FLOAT64_C(   530.01), SIMDE_FLOAT64_C(   691.42), SIMDE_FLOAT64_C(     8.62), SIMDE_FLOAT64_C(     6.85),
        SIMDE_FLOAT64_C(   763.69), SIMDE_FLOAT64_C(   395.70), SIMDE_FLOAT64_C(   328.63), SIMDE_FLOAT64_C(   240.33) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_log2_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_log10_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  4068.94), SIMDE_FLOAT32_C(  5195.06), SIMDE_FLOAT32_C(  1228.12), SIMDE_FLOAT32_C(  6733.16)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(     3.83)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  7486.55), SIMDE_FLOAT32_C(  8351.20), SIMDE_FLOAT32_C(  3512.77), SIMDE_FLOAT32_C(  5170.29)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     3.71)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  9127.65), SIMDE_FLOAT32_C(  7111.03), SIMDE_FLOAT32_C(  3652.77), SIMDE_FLOAT32_C(  7338.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     3.85), SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(     3.87)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  1609.14), SIMDE_FLOAT32_C(  1569.36), SIMDE_FLOAT32_C(  5423.87), SIMDE_FLOAT32_C(  7857.29)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     3.20), SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(     3.90)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  3474.63), SIMDE_FLOAT32_C(   695.25), SIMDE_FLOAT32_C(  2912.29), SIMDE_FLOAT32_C(  8484.34)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.54), SIMDE_FLOAT32_C(     2.84), SIMDE_FLOAT32_C(     3.46), SIMDE_FLOAT32_C(     3.93)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  2775.95), SIMDE_FLOAT32_C(  5142.35), SIMDE_FLOAT32_C(  3079.83), SIMDE_FLOAT32_C(   381.82)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.44), SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     3.49), SIMDE_FLOAT32_C(     2.58)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  6306.54), SIMDE_FLOAT32_C(  3937.29), SIMDE_FLOAT32_C(   117.23), SIMDE_FLOAT32_C(  1696.00)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(     3.23)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  5890.98), SIMDE_FLOAT32_C(  2746.67), SIMDE_FLOAT32_C(  6166.85), SIMDE_FLOAT32_C(  8435.45)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.77), SIMDE_FLOAT32_C(     3.44), SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     3.93)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_log10_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_log10_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 1228.12), SIMDE_FLOAT64_C( 6733.16)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.09), SIMDE_FLOAT64_C(    3.83)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 4068.94), SIMDE_FLOAT64_C( 5195.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.61), SIMDE_FLOAT64_C(    3.72)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 3512.77), SIMDE_FLOAT64_C( 5170.29)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.55), SIMDE_FLOAT64_C(    3.71)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 7486.55), SIMDE_FLOAT64_C( 8351.20)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.87), SIMDE_FLOAT64_C(    3.92)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 3652.77), SIMDE_FLOAT64_C( 7338.80)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.56), SIMDE_FLOAT64_C(    3.87)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 9127.65), SIMDE_FLOAT64_C( 7111.03)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.96), SIMDE_FLOAT64_C(    3.85)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 5423.87), SIMDE_FLOAT64_C( 7857.29)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.73), SIMDE_FLOAT64_C(    3.90)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( 1609.14), SIMDE_FLOAT64_C( 1569.36)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.21), SIMDE_FLOAT64_C(    3.20)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_log10_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_log10_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  7486.55), SIMDE_FLOAT32_C(  8351.20),
                         SIMDE_FLOAT32_C(  3512.77), SIMDE_FLOAT32_C(  5170.29),
                         SIMDE_FLOAT32_C(  4068.94), SIMDE_FLOAT32_C(  5195.06),
                         SIMDE_FLOAT32_C(  1228.12), SIMDE_FLOAT32_C(  6733.16)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     3.92),
                         SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     3.71),
                         SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     3.72),
                         SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(     3.83)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  1609.14), SIMDE_FLOAT32_C(  1569.36),
                         SIMDE_FLOAT32_C(  5423.87), SIMDE_FLOAT32_C(  7857.29),
                         SIMDE_FLOAT32_C(  9127.65), SIMDE_FLOAT32_C(  7111.03),
                         SIMDE_FLOAT32_C(  3652.77), SIMDE_FLOAT32_C(  7338.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     3.20),
                         SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(     3.90),
                         SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     3.85),
                         SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(     3.87)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  2775.95), SIMDE_FLOAT32_C(  5142.35),
                         SIMDE_FLOAT32_C(  3079.83), SIMDE_FLOAT32_C(   381.82),
                         SIMDE_FLOAT32_C(  3474.63), SIMDE_FLOAT32_C(   695.25),
                         SIMDE_FLOAT32_C(  2912.29), SIMDE_FLOAT32_C(  8484.34)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.44), SIMDE_FLOAT32_C(     3.71),
                         SIMDE_FLOAT32_C(     3.49), SIMDE_FLOAT32_C(     2.58),
                         SIMDE_FLOAT32_C(     3.54), SIMDE_FLOAT32_C(     2.84),
                         SIMDE_FLOAT32_C(     3.46), SIMDE_FLOAT32_C(     3.93)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  5890.98), SIMDE_FLOAT32_C(  2746.67),
                         SIMDE_FLOAT32_C(  6166.85), SIMDE_FLOAT32_C(  8435.45),
                         SIMDE_FLOAT32_C(  6306.54), SIMDE_FLOAT32_C(  3937.29),
                         SIMDE_FLOAT32_C(   117.23), SIMDE_FLOAT32_C(  1696.00)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.77), SIMDE_FLOAT32_C(     3.44),
                         SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     3.93),
                         SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(     3.60),
                         SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(     3.23)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  1148.23), SIMDE_FLOAT32_C(  7217.40),
                         SIMDE_FLOAT32_C(  2082.02), SIMDE_FLOAT32_C(  6902.28),
                         SIMDE_FLOAT32_C(  1146.40), SIMDE_FLOAT32_C(  9969.51),
                         SIMDE_FLOAT32_C(  5140.40), SIMDE_FLOAT32_C(  9206.03)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.06), SIMDE_FLOAT32_C(     3.86),
                         SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(     3.84),
                         SIMDE_FLOAT32_C(     3.06), SIMDE_FLOAT32_C(     4.00),
                         SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     3.96)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  3060.52), SIMDE_FLOAT32_C(  6979.60),
                         SIMDE_FLOAT32_C(  8279.36), SIMDE_FLOAT32_C(  6696.04),
                         SIMDE_FLOAT32_C(  7661.76), SIMDE_FLOAT32_C(  3680.04),
                         SIMDE_FLOAT32_C(  8903.22), SIMDE_FLOAT32_C(  4846.05)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.49), SIMDE_FLOAT32_C(     3.84),
                         SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(     3.83),
                         SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(     3.57),
                         SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(     3.69)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  3981.75), SIMDE_FLOAT32_C(  4596.36),
                         SIMDE_FLOAT32_C(  6683.64), SIMDE_FLOAT32_C(   276.11),
                         SIMDE_FLOAT32_C(  1262.07), SIMDE_FLOAT32_C(  1163.84),
                         SIMDE_FLOAT32_C(  2229.06), SIMDE_FLOAT32_C(  6994.08)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     3.66),
                         SIMDE_FLOAT32_C(     3.83), SIMDE_FLOAT32_C(     2.44),
                         SIMDE_FLOAT32_C(     3.10), SIMDE_FLOAT32_C(     3.07),
                         SIMDE_FLOAT32_C(     3.35), SIMDE_FLOAT32_C(     3.84)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  7348.31), SIMDE_FLOAT32_C(  8400.08),
                         SIMDE_FLOAT32_C(  4256.55), SIMDE_FLOAT32_C(  9093.31),
                         SIMDE_FLOAT32_C(  9550.14), SIMDE_FLOAT32_C(  8002.34),
                         SIMDE_FLOAT32_C(  8956.15), SIMDE_FLOAT32_C(  6271.53)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     3.92),
                         SIMDE_FLOAT32_C(     3.63), SIMDE_FLOAT32_C(     3.96),
                         SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(     3.90),
                         SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(     3.80)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_log10_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_log10_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 4068.94), SIMDE_FLOAT64_C( 5195.06),
                         SIMDE_FLOAT64_C( 1228.12), SIMDE_FLOAT64_C( 6733.16)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.61), SIMDE_FLOAT64_C(    3.72),
                         SIMDE_FLOAT64_C(    3.09), SIMDE_FLOAT64_C(    3.83)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 7486.55), SIMDE_FLOAT64_C( 8351.20),
                         SIMDE_FLOAT64_C( 3512.77), SIMDE_FLOAT64_C( 5170.29)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.87), SIMDE_FLOAT64_C(    3.92),
                         SIMDE_FLOAT64_C(    3.55), SIMDE_FLOAT64_C(    3.71)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 9127.65), SIMDE_FLOAT64_C( 7111.03),
                         SIMDE_FLOAT64_C( 3652.77), SIMDE_FLOAT64_C( 7338.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.96), SIMDE_FLOAT64_C(    3.85),
                         SIMDE_FLOAT64_C(    3.56), SIMDE_FLOAT64_C(    3.87)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 1609.14), SIMDE_FLOAT64_C( 1569.36),
                         SIMDE_FLOAT64_C( 5423.87), SIMDE_FLOAT64_C( 7857.29)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.21), SIMDE_FLOAT64_C(    3.20),
                         SIMDE_FLOAT64_C(    3.73), SIMDE_FLOAT64_C(    3.90)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 3474.63), SIMDE_FLOAT64_C(  695.25),
                         SIMDE_FLOAT64_C( 2912.29), SIMDE_FLOAT64_C( 8484.34)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.54), SIMDE_FLOAT64_C(    2.84),
                         SIMDE_FLOAT64_C(    3.46), SIMDE_FLOAT64_C(    3.93)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 2775.95), SIMDE_FLOAT64_C( 5142.35),
                         SIMDE_FLOAT64_C( 3079.83), SIMDE_FLOAT64_C(  381.82)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.44), SIMDE_FLOAT64_C(    3.71),
                         SIMDE_FLOAT64_C(    3.49), SIMDE_FLOAT64_C(    2.58)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 6306.54), SIMDE_FLOAT64_C( 3937.29),
                         SIMDE_FLOAT64_C(  117.23), SIMDE_FLOAT64_C( 1696.00)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.80), SIMDE_FLOAT64_C(    3.60),
                         SIMDE_FLOAT64_C(    2.07), SIMDE_FLOAT64_C(    3.23)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( 5890.98), SIMDE_FLOAT64_C( 2746.67),
                         SIMDE_FLOAT64_C( 6166.85), SIMDE_FLOAT64_C( 8435.45)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.77), SIMDE_FLOAT64_C(    3.44),
                         SIMDE_FLOAT64_C(    3.79), SIMDE_FLOAT64_C(    3.93)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_log10_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_log10_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  1609.14), SIMDE_FLOAT32_C(  1569.36), SIMDE_FLOAT32_C(  5423.87), SIMDE_FLOAT32_C(  7857.29),
                         SIMDE_FLOAT32_C(  9127.65), SIMDE_FLOAT32_C(  7111.03), SIMDE_FLOAT32_C(  3652.77), SIMDE_FLOAT32_C(  7338.80),
                         SIMDE_FLOAT32_C(  7486.55), SIMDE_FLOAT32_C(  8351.20), SIMDE_FLOAT32_C(  3512.77), SIMDE_FLOAT32_C(  5170.29),
                         SIMDE_FLOAT32_C(  4068.94), SIMDE_FLOAT32_C(  5195.06), SIMDE_FLOAT32_C(  1228.12), SIMDE_FLOAT32_C(  6733.16)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     3.20), SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(     3.90),
                         SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     3.85), SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(     3.87),
                         SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     3.71),
                         SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(     3.83)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  5890.98), SIMDE_FLOAT32_C(  2746.67), SIMDE_FLOAT32_C(  6166.85), SIMDE_FLOAT32_C(  8435.45),
                         SIMDE_FLOAT32_C(  6306.54), SIMDE_FLOAT32_C(  3937.29), SIMDE_FLOAT32_C(   117.23), SIMDE_FLOAT32_C(  1696.00),
                         SIMDE_FLOAT32_C(  2775.95), SIMDE_FLOAT32_C(  5142.35), SIMDE_FLOAT32_C(  3079.83), SIMDE_FLOAT32_C(   381.82),
                         SIMDE_FLOAT32_C(  3474.63), SIMDE_FLOAT32_C(   695.25), SIMDE_FLOAT32_C(  2912.29), SIMDE_FLOAT32_C(  8484.34)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.77), SIMDE_FLOAT32_C(     3.44), SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     3.93),
                         SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(     3.23),
                         SIMDE_FLOAT32_C(     3.44), SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     3.49), SIMDE_FLOAT32_C(     2.58),
                         SIMDE_FLOAT32_C(     3.54), SIMDE_FLOAT32_C(     2.84), SIMDE_FLOAT32_C(     3.46), SIMDE_FLOAT32_C(     3.93)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  3060.52), SIMDE_FLOAT32_C(  6979.60), SIMDE_FLOAT32_C(  8279.36), SIMDE_FLOAT32_C(  6696.04),
                         SIMDE_FLOAT32_C(  7661.76), SIMDE_FLOAT32_C(  3680.04), SIMDE_FLOAT32_C(  8903.22), SIMDE_FLOAT32_C(  4846.05),
                         SIMDE_FLOAT32_C(  1148.23), SIMDE_FLOAT32_C(  7217.40), SIMDE_FLOAT32_C(  2082.02), SIMDE_FLOAT32_C(  6902.28),
                         SIMDE_FLOAT32_C(  1146.40), SIMDE_FLOAT32_C(  9969.51), SIMDE_FLOAT32_C(  5140.40), SIMDE_FLOAT32_C(  9206.03)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.49), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(     3.83),
                         SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(     3.57), SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(     3.69),
                         SIMDE_FLOAT32_C(     3.06), SIMDE_FLOAT32_C(     3.86), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(     3.84),
                         SIMDE_FLOAT32_C(     3.06), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     3.96)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  7348.31), SIMDE_FLOAT32_C(  8400.08), SIMDE_FLOAT32_C(  4256.55), SIMDE_FLOAT32_C(  9093.31),
                         SIMDE_FLOAT32_C(  9550.14), SIMDE_FLOAT32_C(  8002.34), SIMDE_FLOAT32_C(  8956.15), SIMDE_FLOAT32_C(  6271.53),
                         SIMDE_FLOAT32_C(  3981.75), SIMDE_FLOAT32_C(  4596.36), SIMDE_FLOAT32_C(  6683.64), SIMDE_FLOAT32_C(   276.11),
                         SIMDE_FLOAT32_C(  1262.07), SIMDE_FLOAT32_C(  1163.84), SIMDE_FLOAT32_C(  2229.06), SIMDE_FLOAT32_C(  6994.08)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(     3.63), SIMDE_FLOAT32_C(     3.96),
                         SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(     3.90), SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(     3.80),
                         SIMDE_FLOAT32_C(     3.60), SIMDE_FLOAT32_C(     3.66), SIMDE_FLOAT32_C(     3.83), SIMDE_FLOAT32_C(     2.44),
                         SIMDE_FLOAT32_C(     3.10), SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     3.35), SIMDE_FLOAT32_C(     3.84)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  4105.04), SIMDE_FLOAT32_C(  8793.93), SIMDE_FLOAT32_C(  6623.12), SIMDE_FLOAT32_C(  6717.40),
                         SIMDE_FLOAT32_C(   628.43), SIMDE_FLOAT32_C(  1010.42), SIMDE_FLOAT32_C(  3357.32), SIMDE_FLOAT32_C(  2370.85),
                         SIMDE_FLOAT32_C(  4038.44), SIMDE_FLOAT32_C(   886.73), SIMDE_FLOAT32_C(  7806.81), SIMDE_FLOAT32_C(  8278.35),
                         SIMDE_FLOAT32_C(  4645.43), SIMDE_FLOAT32_C(  7716.73), SIMDE_FLOAT32_C(  5603.27), SIMDE_FLOAT32_C(  4142.45)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     3.94), SIMDE_FLOAT32_C(     3.82), SIMDE_FLOAT32_C(     3.83),
                         SIMDE_FLOAT32_C(     2.80), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     3.53), SIMDE_FLOAT32_C(     3.37),
                         SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(     3.89), SIMDE_FLOAT32_C(     3.92),
                         SIMDE_FLOAT32_C(     3.67), SIMDE_FLOAT32_C(     3.89), SIMDE_FLOAT32_C(     3.75), SIMDE_FLOAT32_C(     3.62)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  8450.59), SIMDE_FLOAT32_C(  9203.26), SIMDE_FLOAT32_C(  4894.53), SIMDE_FLOAT32_C(  2042.18),
                         SIMDE_FLOAT32_C(  2755.53), SIMDE_FLOAT32_C(  8657.47), SIMDE_FLOAT32_C(  7528.93), SIMDE_FLOAT32_C(  8118.50),
                         SIMDE_FLOAT32_C(  9155.11), SIMDE_FLOAT32_C(  5703.37), SIMDE_FLOAT32_C(  9886.80), SIMDE_FLOAT32_C(   469.19),
                         SIMDE_FLOAT32_C(  6656.71), SIMDE_FLOAT32_C(  5499.67), SIMDE_FLOAT32_C(  7314.76), SIMDE_FLOAT32_C(  1309.05)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.93), SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(     3.31),
                         SIMDE_FLOAT32_C(     3.44), SIMDE_FLOAT32_C(     3.94), SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(     3.91),
                         SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     3.76), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     2.67),
                         SIMDE_FLOAT32_C(     3.82), SIMDE_FLOAT32_C(     3.74), SIMDE_FLOAT32_C(     3.86), SIMDE_FLOAT32_C(     3.12)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  1154.54), SIMDE_FLOAT32_C(  9110.29), SIMDE_FLOAT32_C(  2130.97), SIMDE_FLOAT32_C(    11.83),
                         SIMDE_FLOAT32_C(  3312.02), SIMDE_FLOAT32_C(  9618.20), SIMDE_FLOAT32_C(  6468.19), SIMDE_FLOAT32_C(  1159.42),
                         SIMDE_FLOAT32_C(  2118.90), SIMDE_FLOAT32_C(  4661.80), SIMDE_FLOAT32_C(  8551.88), SIMDE_FLOAT32_C(  9887.44),
                         SIMDE_FLOAT32_C(  1217.92), SIMDE_FLOAT32_C(  7124.06), SIMDE_FLOAT32_C(  5136.26), SIMDE_FLOAT32_C(  4524.23)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.06), SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     1.07),
                         SIMDE_FLOAT32_C(     3.52), SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     3.06),
                         SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     3.67), SIMDE_FLOAT32_C(     3.93), SIMDE_FLOAT32_C(     4.00),
                         SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(     3.85), SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     3.66)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  2809.03), SIMDE_FLOAT32_C(  3201.22), SIMDE_FLOAT32_C(  1237.85), SIMDE_FLOAT32_C(  4831.67),
                         SIMDE_FLOAT32_C(  9663.28), SIMDE_FLOAT32_C(  5036.36), SIMDE_FLOAT32_C(  3363.90), SIMDE_FLOAT32_C(  4374.02),
                         SIMDE_FLOAT32_C(  4087.77), SIMDE_FLOAT32_C(  5199.67), SIMDE_FLOAT32_C(  7554.25), SIMDE_FLOAT32_C(  6973.34),
                         SIMDE_FLOAT32_C(  5071.68), SIMDE_FLOAT32_C(  3476.37), SIMDE_FLOAT32_C(  9581.30), SIMDE_FLOAT32_C(  1516.57)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.45), SIMDE_FLOAT32_C(     3.51), SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(     3.68),
                         SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(     3.53), SIMDE_FLOAT32_C(     3.64),
                         SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(     3.84),
                         SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     3.54), SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(     3.18)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_log10_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_log10_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  2746.67), SIMDE_FLOAT32_C(  8435.45), SIMDE_FLOAT32_C(  3937.29), SIMDE_FLOAT32_C(  1696.00),
                         SIMDE_FLOAT32_C(  5142.35), SIMDE_FLOAT32_C(   381.82), SIMDE_FLOAT32_C(   695.25), SIMDE_FLOAT32_C(  8484.34),
                         SIMDE_FLOAT32_C(  1569.36), SIMDE_FLOAT32_C(  7857.29), SIMDE_FLOAT32_C(  7111.03), SIMDE_FLOAT32_C(  7338.80),
                         SIMDE_FLOAT32_C(  8351.20), SIMDE_FLOAT32_C(  5170.29), SIMDE_FLOAT32_C(  5195.06), SIMDE_FLOAT32_C(  6733.16)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  5890.98), SIMDE_FLOAT32_C(  6166.85), SIMDE_FLOAT32_C(  6306.54), SIMDE_FLOAT32_C(   117.23),
                         SIMDE_FLOAT32_C(  2775.95), SIMDE_FLOAT32_C(  3079.83), SIMDE_FLOAT32_C(  3474.63), SIMDE_FLOAT32_C(  2912.29),
                         SIMDE_FLOAT32_C(  1609.14), SIMDE_FLOAT32_C(  5423.87), SIMDE_FLOAT32_C(  9127.65), SIMDE_FLOAT32_C(  3652.77),
                         SIMDE_FLOAT32_C(  7486.55), SIMDE_FLOAT32_C(  3512.77), SIMDE_FLOAT32_C(  4068.94), SIMDE_FLOAT32_C(  1228.12)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.77), SIMDE_FLOAT32_C(  8435.45), SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(  1696.00),
                         SIMDE_FLOAT32_C(  5142.35), SIMDE_FLOAT32_C(   381.82), SIMDE_FLOAT32_C(   695.25), SIMDE_FLOAT32_C(     3.46),
                         SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     3.56),
                         SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(  5170.29), SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(  6733.16)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  7348.31), SIMDE_FLOAT32_C(  4256.55), SIMDE_FLOAT32_C(  9550.14), SIMDE_FLOAT32_C(  8956.15),
                         SIMDE_FLOAT32_C(  3981.75), SIMDE_FLOAT32_C(  6683.64), SIMDE_FLOAT32_C(  1262.07), SIMDE_FLOAT32_C(  2229.06),
                         SIMDE_FLOAT32_C(  3060.52), SIMDE_FLOAT32_C(  8279.36), SIMDE_FLOAT32_C(  7661.76), SIMDE_FLOAT32_C(  8903.22),
                         SIMDE_FLOAT32_C(  1148.23), SIMDE_FLOAT32_C(  2082.02), SIMDE_FLOAT32_C(  1146.40), SIMDE_FLOAT32_C(  5140.40)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  4142.45), SIMDE_FLOAT32_C(  8400.08), SIMDE_FLOAT32_C(  9093.31), SIMDE_FLOAT32_C(  8002.34),
                         SIMDE_FLOAT32_C(  6271.53), SIMDE_FLOAT32_C(  4596.36), SIMDE_FLOAT32_C(   276.11), SIMDE_FLOAT32_C(  1163.84),
                         SIMDE_FLOAT32_C(  6994.08), SIMDE_FLOAT32_C(  6979.60), SIMDE_FLOAT32_C(  6696.04), SIMDE_FLOAT32_C(  3680.04),
                         SIMDE_FLOAT32_C(  4846.05), SIMDE_FLOAT32_C(  7217.40), SIMDE_FLOAT32_C(  6902.28), SIMDE_FLOAT32_C(  9969.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.62), SIMDE_FLOAT32_C(  4256.55), SIMDE_FLOAT32_C(  9550.14), SIMDE_FLOAT32_C(  8956.15),
                         SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(     3.66), SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(     3.07),
                         SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(  8279.36), SIMDE_FLOAT32_C(     3.83), SIMDE_FLOAT32_C(     3.57),
                         SIMDE_FLOAT32_C(     3.69), SIMDE_FLOAT32_C(     3.86), SIMDE_FLOAT32_C(  1146.40), SIMDE_FLOAT32_C(     4.00)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  4524.23), SIMDE_FLOAT32_C(  9203.26), SIMDE_FLOAT32_C(  2042.18), SIMDE_FLOAT32_C(  8657.47),
                         SIMDE_FLOAT32_C(  8118.50), SIMDE_FLOAT32_C(  5703.37), SIMDE_FLOAT32_C(   469.19), SIMDE_FLOAT32_C(  5499.67),
                         SIMDE_FLOAT32_C(  1309.05), SIMDE_FLOAT32_C(  8793.93), SIMDE_FLOAT32_C(  6717.40), SIMDE_FLOAT32_C(  1010.42),
                         SIMDE_FLOAT32_C(  2370.85), SIMDE_FLOAT32_C(   886.73), SIMDE_FLOAT32_C(  8278.35), SIMDE_FLOAT32_C(  7716.73)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  5136.26), SIMDE_FLOAT32_C(  8450.59), SIMDE_FLOAT32_C(  4894.53), SIMDE_FLOAT32_C(  2755.53),
                         SIMDE_FLOAT32_C(  7528.93), SIMDE_FLOAT32_C(  9155.11), SIMDE_FLOAT32_C(  9886.80), SIMDE_FLOAT32_C(  6656.71),
                         SIMDE_FLOAT32_C(  7314.76), SIMDE_FLOAT32_C(  4105.04), SIMDE_FLOAT32_C(  6623.12), SIMDE_FLOAT32_C(   628.43),
                         SIMDE_FLOAT32_C(  3357.32), SIMDE_FLOAT32_C(  4038.44), SIMDE_FLOAT32_C(  7806.81), SIMDE_FLOAT32_C(  4645.43)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  4524.23), SIMDE_FLOAT32_C(     3.93), SIMDE_FLOAT32_C(  2042.18), SIMDE_FLOAT32_C(  8657.47),
                         SIMDE_FLOAT32_C(  8118.50), SIMDE_FLOAT32_C(  5703.37), SIMDE_FLOAT32_C(   469.19), SIMDE_FLOAT32_C(     3.82),
                         SIMDE_FLOAT32_C(     3.86), SIMDE_FLOAT32_C(  8793.93), SIMDE_FLOAT32_C(     3.82), SIMDE_FLOAT32_C(  1010.42),
                         SIMDE_FLOAT32_C(  2370.85), SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(  8278.35), SIMDE_FLOAT32_C(  7716.73)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  3256.50), SIMDE_FLOAT32_C(  2809.03), SIMDE_FLOAT32_C(  1237.85), SIMDE_FLOAT32_C(  9663.28),
                         SIMDE_FLOAT32_C(  3363.90), SIMDE_FLOAT32_C(  4087.77), SIMDE_FLOAT32_C(  7554.25), SIMDE_FLOAT32_C(  5071.68),
                         SIMDE_FLOAT32_C(  9581.30), SIMDE_FLOAT32_C(  1154.54), SIMDE_FLOAT32_C(  2130.97), SIMDE_FLOAT32_C(  3312.02),
                         SIMDE_FLOAT32_C(  6468.19), SIMDE_FLOAT32_C(  2118.90), SIMDE_FLOAT32_C(  8551.88), SIMDE_FLOAT32_C(  1217.92)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  9486.33), SIMDE_FLOAT32_C(  4010.56), SIMDE_FLOAT32_C(  3201.22), SIMDE_FLOAT32_C(  4831.67),
                         SIMDE_FLOAT32_C(  5036.36), SIMDE_FLOAT32_C(  4374.02), SIMDE_FLOAT32_C(  5199.67), SIMDE_FLOAT32_C(  6973.34),
                         SIMDE_FLOAT32_C(  3476.37), SIMDE_FLOAT32_C(  1516.57), SIMDE_FLOAT32_C(  9110.29), SIMDE_FLOAT32_C(    11.83),
                         SIMDE_FLOAT32_C(  9618.20), SIMDE_FLOAT32_C(  1159.42), SIMDE_FLOAT32_C(  4661.80), SIMDE_FLOAT32_C(  9887.44)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  3256.50), SIMDE_FLOAT32_C(  2809.03), SIMDE_FLOAT32_C(  1237.85), SIMDE_FLOAT32_C(  9663.28),
                         SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(  4087.77), SIMDE_FLOAT32_C(  7554.25), SIMDE_FLOAT32_C(  5071.68),
                         SIMDE_FLOAT32_C(  9581.30), SIMDE_FLOAT32_C(  1154.54), SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     1.07),
                         SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(  2118.90), SIMDE_FLOAT32_C(     3.67), SIMDE_FLOAT32_C(     4.00)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  4921.97), SIMDE_FLOAT32_C(  1314.36), SIMDE_FLOAT32_C(  3425.34), SIMDE_FLOAT32_C(  5889.62),
                         SIMDE_FLOAT32_C(  6729.66), SIMDE_FLOAT32_C(  9443.57), SIMDE_FLOAT32_C(  9578.53), SIMDE_FLOAT32_C(  5667.58),
                         SIMDE_FLOAT32_C(  7424.68), SIMDE_FLOAT32_C(  2009.69), SIMDE_FLOAT32_C(  1044.67), SIMDE_FLOAT32_C(  1170.36),
                         SIMDE_FLOAT32_C(  6106.86), SIMDE_FLOAT32_C(  1058.19), SIMDE_FLOAT32_C(  1124.78), SIMDE_FLOAT32_C(  7203.19)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  7482.85), SIMDE_FLOAT32_C(  9575.95), SIMDE_FLOAT32_C(  1407.98), SIMDE_FLOAT32_C(  5799.87),
                         SIMDE_FLOAT32_C(   694.94), SIMDE_FLOAT32_C(  7133.07), SIMDE_FLOAT32_C(  9660.54), SIMDE_FLOAT32_C(  5551.82),
                         SIMDE_FLOAT32_C(  9134.21), SIMDE_FLOAT32_C(  4616.24), SIMDE_FLOAT32_C(  6187.92), SIMDE_FLOAT32_C(  3107.51),
                         SIMDE_FLOAT32_C(  1991.62), SIMDE_FLOAT32_C(  1882.51), SIMDE_FLOAT32_C(   287.66), SIMDE_FLOAT32_C(  7377.56)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  4921.97), SIMDE_FLOAT32_C(     3.98), SIMDE_FLOAT32_C(  3425.34), SIMDE_FLOAT32_C(     3.76),
                         SIMDE_FLOAT32_C(  6729.66), SIMDE_FLOAT32_C(     3.85), SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(     3.74),
                         SIMDE_FLOAT32_C(  7424.68), SIMDE_FLOAT32_C(  2009.69), SIMDE_FLOAT32_C(  1044.67), SIMDE_FLOAT32_C(  1170.36),
                         SIMDE_FLOAT32_C(  6106.86), SIMDE_FLOAT32_C(  1058.19), SIMDE_FLOAT32_C(     2.46), SIMDE_FLOAT32_C(  7203.19)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  9415.27), SIMDE_FLOAT32_C(   963.59), SIMDE_FLOAT32_C(  4649.74), SIMDE_FLOAT32_C(  1078.30),
                         SIMDE_FLOAT32_C(  5462.61), SIMDE_FLOAT32_C(  6033.01), SIMDE_FLOAT32_C(  9173.00), SIMDE_FLOAT32_C(  4672.02),
                         SIMDE_FLOAT32_C(  3569.65), SIMDE_FLOAT32_C(  3935.68), SIMDE_FLOAT32_C(  3408.08), SIMDE_FLOAT32_C(  8917.42),
                         SIMDE_FLOAT32_C(  1855.90), SIMDE_FLOAT32_C(  7781.74), SIMDE_FLOAT32_C(  7197.17), SIMDE_FLOAT32_C(  7170.16)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.74), SIMDE_FLOAT32_C(  2968.36), SIMDE_FLOAT32_C(  1281.72), SIMDE_FLOAT32_C(  1177.11),
                         SIMDE_FLOAT32_C(  8949.44), SIMDE_FLOAT32_C(  5024.17), SIMDE_FLOAT32_C(   907.29), SIMDE_FLOAT32_C(  5805.32),
                         SIMDE_FLOAT32_C(  7896.24), SIMDE_FLOAT32_C(  4941.12), SIMDE_FLOAT32_C(  3457.39), SIMDE_FLOAT32_C(  1402.13),
                         SIMDE_FLOAT32_C(  6670.00), SIMDE_FLOAT32_C(  6373.56), SIMDE_FLOAT32_C(   415.89), SIMDE_FLOAT32_C(  2550.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  9415.27), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(  1078.30),
                         SIMDE_FLOAT32_C(     3.95), SIMDE_FLOAT32_C(  6033.01), SIMDE_FLOAT32_C(     2.96), SIMDE_FLOAT32_C(     3.76),
                         SIMDE_FLOAT32_C(  3569.65), SIMDE_FLOAT32_C(  3935.68), SIMDE_FLOAT32_C(  3408.08), SIMDE_FLOAT32_C(  8917.42),
                         SIMDE_FLOAT32_C(  1855.90), SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(  7197.17), SIMDE_FLOAT32_C(  7170.16)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  7648.13), SIMDE_FLOAT32_C(  4875.56), SIMDE_FLOAT32_C(   161.12), SIMDE_FLOAT32_C(  8194.68),
                         SIMDE_FLOAT32_C(  7254.51), SIMDE_FLOAT32_C(  1142.29), SIMDE_FLOAT32_C(  5528.96), SIMDE_FLOAT32_C(  7950.51),
                         SIMDE_FLOAT32_C(  5154.57), SIMDE_FLOAT32_C(  8176.75), SIMDE_FLOAT32_C(  4580.00), SIMDE_FLOAT32_C(  5400.22),
                         SIMDE_FLOAT32_C(  1452.71), SIMDE_FLOAT32_C(  8039.28), SIMDE_FLOAT32_C(  6972.90), SIMDE_FLOAT32_C(   554.46)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  5093.74), SIMDE_FLOAT32_C(  9045.23), SIMDE_FLOAT32_C(  5720.26), SIMDE_FLOAT32_C(  2861.39),
                         SIMDE_FLOAT32_C(  6541.39), SIMDE_FLOAT32_C(  4114.75), SIMDE_FLOAT32_C(  2711.17), SIMDE_FLOAT32_C(  8391.22),
                         SIMDE_FLOAT32_C(  5330.27), SIMDE_FLOAT32_C(  3661.45), SIMDE_FLOAT32_C(  5586.41), SIMDE_FLOAT32_C(  2116.00),
                         SIMDE_FLOAT32_C(  4808.04), SIMDE_FLOAT32_C(  3749.32), SIMDE_FLOAT32_C(  4730.38), SIMDE_FLOAT32_C(  5459.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  7648.13), SIMDE_FLOAT32_C(  4875.56), SIMDE_FLOAT32_C(   161.12), SIMDE_FLOAT32_C(  8194.68),
                         SIMDE_FLOAT32_C(  7254.51), SIMDE_FLOAT32_C(  1142.29), SIMDE_FLOAT32_C(     3.43), SIMDE_FLOAT32_C(     3.92),
                         SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(  8176.75), SIMDE_FLOAT32_C(     3.75), SIMDE_FLOAT32_C(     3.33),
                         SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(  8039.28), SIMDE_FLOAT32_C(  6972.90), SIMDE_FLOAT32_C(     3.74)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  1058.07), SIMDE_FLOAT32_C(  6652.15), SIMDE_FLOAT32_C(  2532.95), SIMDE_FLOAT32_C(  9113.62),
                         SIMDE_FLOAT32_C(  9783.41), SIMDE_FLOAT32_C(  9773.08), SIMDE_FLOAT32_C(  9127.47), SIMDE_FLOAT32_C(   918.64),
                         SIMDE_FLOAT32_C(  3953.30), SIMDE_FLOAT32_C(   333.95), SIMDE_FLOAT32_C(  1356.49), SIMDE_FLOAT32_C(  2899.69),
                         SIMDE_FLOAT32_C(  5501.59), SIMDE_FLOAT32_C(  5515.77), SIMDE_FLOAT32_C(  7198.84), SIMDE_FLOAT32_C(  3978.34)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   792.83), SIMDE_FLOAT32_C(  4929.19), SIMDE_FLOAT32_C(  9124.38), SIMDE_FLOAT32_C(  8968.13),
                         SIMDE_FLOAT32_C(  1316.26), SIMDE_FLOAT32_C(  3447.13), SIMDE_FLOAT32_C(  8644.35), SIMDE_FLOAT32_C(  3246.39),
                         SIMDE_FLOAT32_C(  5304.47), SIMDE_FLOAT32_C(  5549.07), SIMDE_FLOAT32_C(  8579.68), SIMDE_FLOAT32_C(  3747.01),
                         SIMDE_FLOAT32_C(  9720.69), SIMDE_FLOAT32_C(  6809.26), SIMDE_FLOAT32_C(  4934.63), SIMDE_FLOAT32_C(  9263.02)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  1058.07), SIMDE_FLOAT32_C(  6652.15), SIMDE_FLOAT32_C(     3.96), SIMDE_FLOAT32_C(     3.95),
                         SIMDE_FLOAT32_C(  9783.41), SIMDE_FLOAT32_C(  9773.08), SIMDE_FLOAT32_C(  9127.47), SIMDE_FLOAT32_C(     3.51),
                         SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(   333.95), SIMDE_FLOAT32_C(     3.93), SIMDE_FLOAT32_C(  2899.69),
                         SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(  5515.77), SIMDE_FLOAT32_C(  7198.84), SIMDE_FLOAT32_C(     3.97)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_log10_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_log10_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
   { simde_mm512_set_pd(SIMDE_FLOAT64_C( 7486.55), SIMDE_FLOAT64_C( 8351.20),
                         SIMDE_FLOAT64_C( 3512.77), SIMDE_FLOAT64_C( 5170.29),
                         SIMDE_FLOAT64_C( 4068.94), SIMDE_FLOAT64_C( 5195.06),
                         SIMDE_FLOAT64_C( 1228.12), SIMDE_FLOAT64_C( 6733.16)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.87), SIMDE_FLOAT64_C(    3.92),
                         SIMDE_FLOAT64_C(    3.55), SIMDE_FLOAT64_C(    3.71),
                         SIMDE_FLOAT64_C(    3.61), SIMDE_FLOAT64_C(    3.72),
                         SIMDE_FLOAT64_C(    3.09), SIMDE_FLOAT64_C(    3.83)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 1609.14), SIMDE_FLOAT64_C( 1569.36),
                         SIMDE_FLOAT64_C( 5423.87), SIMDE_FLOAT64_C( 7857.29),
                         SIMDE_FLOAT64_C( 9127.65), SIMDE_FLOAT64_C( 7111.03),
                         SIMDE_FLOAT64_C( 3652.77), SIMDE_FLOAT64_C( 7338.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.21), SIMDE_FLOAT64_C(    3.20),
                         SIMDE_FLOAT64_C(    3.73), SIMDE_FLOAT64_C(    3.90),
                         SIMDE_FLOAT64_C(    3.96), SIMDE_FLOAT64_C(    3.85),
                         SIMDE_FLOAT64_C(    3.56), SIMDE_FLOAT64_C(    3.87)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 2775.95), SIMDE_FLOAT64_C( 5142.35),
                         SIMDE_FLOAT64_C( 3079.83), SIMDE_FLOAT64_C(  381.82),
                         SIMDE_FLOAT64_C( 3474.63), SIMDE_FLOAT64_C(  695.25),
                         SIMDE_FLOAT64_C( 2912.29), SIMDE_FLOAT64_C( 8484.34)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.44), SIMDE_FLOAT64_C(    3.71),
                         SIMDE_FLOAT64_C(    3.49), SIMDE_FLOAT64_C(    2.58),
                         SIMDE_FLOAT64_C(    3.54), SIMDE_FLOAT64_C(    2.84),
                         SIMDE_FLOAT64_C(    3.46), SIMDE_FLOAT64_C(    3.93)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 5890.98), SIMDE_FLOAT64_C( 2746.67),
                         SIMDE_FLOAT64_C( 6166.85), SIMDE_FLOAT64_C( 8435.45),
                         SIMDE_FLOAT64_C( 6306.54), SIMDE_FLOAT64_C( 3937.29),
                         SIMDE_FLOAT64_C(  117.23), SIMDE_FLOAT64_C( 1696.00)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.77), SIMDE_FLOAT64_C(    3.44),
                         SIMDE_FLOAT64_C(    3.79), SIMDE_FLOAT64_C(    3.93),
                         SIMDE_FLOAT64_C(    3.80), SIMDE_FLOAT64_C(    3.60),
                         SIMDE_FLOAT64_C(    2.07), SIMDE_FLOAT64_C(    3.23)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 1148.23), SIMDE_FLOAT64_C( 7217.40),
                         SIMDE_FLOAT64_C( 2082.02), SIMDE_FLOAT64_C( 6902.28),
                         SIMDE_FLOAT64_C( 1146.40), SIMDE_FLOAT64_C( 9969.51),
                         SIMDE_FLOAT64_C( 5140.40), SIMDE_FLOAT64_C( 9206.03)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.06), SIMDE_FLOAT64_C(    3.86),
                         SIMDE_FLOAT64_C(    3.32), SIMDE_FLOAT64_C(    3.84),
                         SIMDE_FLOAT64_C(    3.06), SIMDE_FLOAT64_C(    4.00),
                         SIMDE_FLOAT64_C(    3.71), SIMDE_FLOAT64_C(    3.96)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 3060.52), SIMDE_FLOAT64_C( 6979.60),
                         SIMDE_FLOAT64_C( 8279.36), SIMDE_FLOAT64_C( 6696.04),
                         SIMDE_FLOAT64_C( 7661.76), SIMDE_FLOAT64_C( 3680.04),
                         SIMDE_FLOAT64_C( 8903.22), SIMDE_FLOAT64_C( 4846.05)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.49), SIMDE_FLOAT64_C(    3.84),
                         SIMDE_FLOAT64_C(    3.92), SIMDE_FLOAT64_C(    3.83),
                         SIMDE_FLOAT64_C(    3.88), SIMDE_FLOAT64_C(    3.57),
                         SIMDE_FLOAT64_C(    3.95), SIMDE_FLOAT64_C(    3.69)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 3981.75), SIMDE_FLOAT64_C( 4596.36),
                         SIMDE_FLOAT64_C( 6683.64), SIMDE_FLOAT64_C(  276.11),
                         SIMDE_FLOAT64_C( 1262.07), SIMDE_FLOAT64_C( 1163.84),
                         SIMDE_FLOAT64_C( 2229.06), SIMDE_FLOAT64_C( 6994.08)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.60), SIMDE_FLOAT64_C(    3.66),
                         SIMDE_FLOAT64_C(    3.83), SIMDE_FLOAT64_C(    2.44),
                         SIMDE_FLOAT64_C(    3.10), SIMDE_FLOAT64_C(    3.07),
                         SIMDE_FLOAT64_C(    3.35), SIMDE_FLOAT64_C(    3.84)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 7348.31), SIMDE_FLOAT64_C( 8400.08),
                         SIMDE_FLOAT64_C( 4256.55), SIMDE_FLOAT64_C( 9093.31),
                         SIMDE_FLOAT64_C( 9550.14), SIMDE_FLOAT64_C( 8002.34),
                         SIMDE_FLOAT64_C( 8956.15), SIMDE_FLOAT64_C( 6271.53)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.87), SIMDE_FLOAT64_C(    3.92),
                         SIMDE_FLOAT64_C(    3.63), SIMDE_FLOAT64_C(    3.96),
                         SIMDE_FLOAT64_C(    3.98), SIMDE_FLOAT64_C(    3.90),
                         SIMDE_FLOAT64_C(    3.95), SIMDE_FLOAT64_C(    3.80)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_log10_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_log10_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 1569.36), SIMDE_FLOAT64_C( 7857.29),
                         SIMDE_FLOAT64_C( 7111.03), SIMDE_FLOAT64_C( 7338.80),
                         SIMDE_FLOAT64_C( 8351.20), SIMDE_FLOAT64_C( 5170.29),
                         SIMDE_FLOAT64_C( 5195.06), SIMDE_FLOAT64_C( 6733.16)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 1609.14), SIMDE_FLOAT64_C( 5423.87),
                         SIMDE_FLOAT64_C( 9127.65), SIMDE_FLOAT64_C( 3652.77),
                         SIMDE_FLOAT64_C( 7486.55), SIMDE_FLOAT64_C( 3512.77),
                         SIMDE_FLOAT64_C( 4068.94), SIMDE_FLOAT64_C( 1228.12)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.21), SIMDE_FLOAT64_C( 7857.29),
                         SIMDE_FLOAT64_C( 7111.03), SIMDE_FLOAT64_C( 7338.80),
                         SIMDE_FLOAT64_C(    3.87), SIMDE_FLOAT64_C( 5170.29),
                         SIMDE_FLOAT64_C(    3.61), SIMDE_FLOAT64_C(    3.09)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 5890.98), SIMDE_FLOAT64_C( 6166.85),
                         SIMDE_FLOAT64_C( 6306.54), SIMDE_FLOAT64_C(  117.23),
                         SIMDE_FLOAT64_C( 2775.95), SIMDE_FLOAT64_C( 3079.83),
                         SIMDE_FLOAT64_C( 3474.63), SIMDE_FLOAT64_C( 2912.29)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 9206.03), SIMDE_FLOAT64_C( 2746.67),
                         SIMDE_FLOAT64_C( 8435.45), SIMDE_FLOAT64_C( 3937.29),
                         SIMDE_FLOAT64_C( 1696.00), SIMDE_FLOAT64_C( 5142.35),
                         SIMDE_FLOAT64_C(  381.82), SIMDE_FLOAT64_C(  695.25)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.96), SIMDE_FLOAT64_C(    3.44),
                         SIMDE_FLOAT64_C(    3.93), SIMDE_FLOAT64_C(  117.23),
                         SIMDE_FLOAT64_C( 2775.95), SIMDE_FLOAT64_C(    3.71),
                         SIMDE_FLOAT64_C( 3474.63), SIMDE_FLOAT64_C(    2.84)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 6994.08), SIMDE_FLOAT64_C( 6979.60),
                         SIMDE_FLOAT64_C( 6696.04), SIMDE_FLOAT64_C( 3680.04),
                         SIMDE_FLOAT64_C( 4846.05), SIMDE_FLOAT64_C( 7217.40),
                         SIMDE_FLOAT64_C( 6902.28), SIMDE_FLOAT64_C( 9969.51)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 2229.06), SIMDE_FLOAT64_C( 3060.52),
                         SIMDE_FLOAT64_C( 8279.36), SIMDE_FLOAT64_C( 7661.76),
                         SIMDE_FLOAT64_C( 8903.22), SIMDE_FLOAT64_C( 1148.23),
                         SIMDE_FLOAT64_C( 2082.02), SIMDE_FLOAT64_C( 1146.40)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.35), SIMDE_FLOAT64_C(    3.49),
                         SIMDE_FLOAT64_C(    3.92), SIMDE_FLOAT64_C(    3.88),
                         SIMDE_FLOAT64_C(    3.95), SIMDE_FLOAT64_C(    3.06),
                         SIMDE_FLOAT64_C( 6902.28), SIMDE_FLOAT64_C(    3.06)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 5603.27), SIMDE_FLOAT64_C( 7348.31),
                         SIMDE_FLOAT64_C( 4256.55), SIMDE_FLOAT64_C( 9550.14),
                         SIMDE_FLOAT64_C( 8956.15), SIMDE_FLOAT64_C( 3981.75),
                         SIMDE_FLOAT64_C( 6683.64), SIMDE_FLOAT64_C( 1262.07)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 7716.73), SIMDE_FLOAT64_C( 4142.45),
                         SIMDE_FLOAT64_C( 8400.08), SIMDE_FLOAT64_C( 9093.31),
                         SIMDE_FLOAT64_C( 8002.34), SIMDE_FLOAT64_C( 6271.53),
                         SIMDE_FLOAT64_C( 4596.36), SIMDE_FLOAT64_C(  276.11)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 5603.27), SIMDE_FLOAT64_C(    3.62),
                         SIMDE_FLOAT64_C( 4256.55), SIMDE_FLOAT64_C(    3.96),
                         SIMDE_FLOAT64_C(    3.90), SIMDE_FLOAT64_C(    3.80),
                         SIMDE_FLOAT64_C( 6683.64), SIMDE_FLOAT64_C(    2.44)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 5499.67), SIMDE_FLOAT64_C( 1309.05),
                         SIMDE_FLOAT64_C( 8793.93), SIMDE_FLOAT64_C( 6717.40),
                         SIMDE_FLOAT64_C( 1010.42), SIMDE_FLOAT64_C( 2370.85),
                         SIMDE_FLOAT64_C(  886.73), SIMDE_FLOAT64_C( 8278.35)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 6656.71), SIMDE_FLOAT64_C( 7314.76),
                         SIMDE_FLOAT64_C( 4105.04), SIMDE_FLOAT64_C( 6623.12),
                         SIMDE_FLOAT64_C(  628.43), SIMDE_FLOAT64_C( 3357.32),
                         SIMDE_FLOAT64_C( 4038.44), SIMDE_FLOAT64_C( 7806.81)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.82), SIMDE_FLOAT64_C( 1309.05),
                         SIMDE_FLOAT64_C( 8793.93), SIMDE_FLOAT64_C(    3.82),
                         SIMDE_FLOAT64_C( 1010.42), SIMDE_FLOAT64_C( 2370.85),
                         SIMDE_FLOAT64_C(  886.73), SIMDE_FLOAT64_C(    3.89)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 1217.92), SIMDE_FLOAT64_C( 5136.26),
                         SIMDE_FLOAT64_C( 8450.59), SIMDE_FLOAT64_C( 4894.53),
                         SIMDE_FLOAT64_C( 2755.53), SIMDE_FLOAT64_C( 7528.93),
                         SIMDE_FLOAT64_C( 9155.11), SIMDE_FLOAT64_C( 9886.80)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 9887.44), SIMDE_FLOAT64_C( 7124.06),
                         SIMDE_FLOAT64_C( 4524.23), SIMDE_FLOAT64_C( 9203.26),
                         SIMDE_FLOAT64_C( 2042.18), SIMDE_FLOAT64_C( 8657.47),
                         SIMDE_FLOAT64_C( 8118.50), SIMDE_FLOAT64_C( 5703.37)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 1217.92), SIMDE_FLOAT64_C(    3.85),
                         SIMDE_FLOAT64_C( 8450.59), SIMDE_FLOAT64_C( 4894.53),
                         SIMDE_FLOAT64_C(    3.31), SIMDE_FLOAT64_C( 7528.93),
                         SIMDE_FLOAT64_C(    3.91), SIMDE_FLOAT64_C(    3.76)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 6973.34), SIMDE_FLOAT64_C( 3476.37),
                         SIMDE_FLOAT64_C( 1516.57), SIMDE_FLOAT64_C( 9110.29),
                         SIMDE_FLOAT64_C(   11.83), SIMDE_FLOAT64_C( 9618.20),
                         SIMDE_FLOAT64_C( 1159.42), SIMDE_FLOAT64_C( 4661.80)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 7554.25), SIMDE_FLOAT64_C( 5071.68),
                         SIMDE_FLOAT64_C( 9581.30), SIMDE_FLOAT64_C( 1154.54),
                         SIMDE_FLOAT64_C( 2130.97), SIMDE_FLOAT64_C( 3312.02),
                         SIMDE_FLOAT64_C( 6468.19), SIMDE_FLOAT64_C( 2118.90)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 6973.34), SIMDE_FLOAT64_C(    3.71),
                         SIMDE_FLOAT64_C( 1516.57), SIMDE_FLOAT64_C(    3.06),
                         SIMDE_FLOAT64_C(    3.33), SIMDE_FLOAT64_C(    3.52),
                         SIMDE_FLOAT64_C( 1159.42), SIMDE_FLOAT64_C(    3.33)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( 7377.56), SIMDE_FLOAT64_C( 9683.23),
                         SIMDE_FLOAT64_C( 3256.50), SIMDE_FLOAT64_C( 2809.03),
                         SIMDE_FLOAT64_C( 1237.85), SIMDE_FLOAT64_C( 9663.28),
                         SIMDE_FLOAT64_C( 3363.90), SIMDE_FLOAT64_C( 4087.77)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( 1124.78), SIMDE_FLOAT64_C( 7203.19),
                         SIMDE_FLOAT64_C( 9486.33), SIMDE_FLOAT64_C( 4010.56),
                         SIMDE_FLOAT64_C( 3201.22), SIMDE_FLOAT64_C( 4831.67),
                         SIMDE_FLOAT64_C( 5036.36), SIMDE_FLOAT64_C( 4374.02)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.05), SIMDE_FLOAT64_C(    3.86),
                         SIMDE_FLOAT64_C( 3256.50), SIMDE_FLOAT64_C(    3.60),
                         SIMDE_FLOAT64_C( 1237.85), SIMDE_FLOAT64_C(    3.68),
                         SIMDE_FLOAT64_C( 3363.90), SIMDE_FLOAT64_C(    3.64)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_log10_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_logb_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    52.75), SIMDE_FLOAT32_C(    12.37), SIMDE_FLOAT32_C(    32.32), SIMDE_FLOAT32_C(    26.90) },
      { SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     4.00) } },
    { { SIMDE_FLOAT32_C(    28.49), SIMDE_FLOAT32_C(    18.47), SIMDE_FLOAT32_C(    63.22), SIMDE_FLOAT32_C(    55.89) },
      { SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    55.03), SIMDE_FLOAT32_C(    53.88), SIMDE_FLOAT32_C(    60.21), SIMDE_FLOAT32_C(    98.39) },
      { SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00) } },
    { { SIMDE_FLOAT32_C(    48.09), SIMDE_FLOAT32_C(    71.36), SIMDE_FLOAT32_C(    70.54), SIMDE_FLOAT32_C(    16.55) },
      { SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     4.00) } },
    { { SIMDE_FLOAT32_C(    80.97), SIMDE_FLOAT32_C(     4.96), SIMDE_FLOAT32_C(    37.49), SIMDE_FLOAT32_C(    46.77) },
      { SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    90.48), SIMDE_FLOAT32_C(    58.54), SIMDE_FLOAT32_C(    37.33), SIMDE_FLOAT32_C(    31.14) },
      { SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     4.00) } },
    { { SIMDE_FLOAT32_C(    72.20), SIMDE_FLOAT32_C(    35.18), SIMDE_FLOAT32_C(    41.35), SIMDE_FLOAT32_C(    41.72) },
      { SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    30.55), SIMDE_FLOAT32_C(    90.31), SIMDE_FLOAT32_C(    81.30), SIMDE_FLOAT32_C(    83.30) },
      { SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_logb_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_logb_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    42.51), SIMDE_FLOAT64_C(    67.09) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(    79.25), SIMDE_FLOAT64_C(    26.02) },
      { SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     4.00) } },
    { { SIMDE_FLOAT64_C(    47.58), SIMDE_FLOAT64_C(    12.11) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     3.00) } },
    { { SIMDE_FLOAT64_C(    67.84), SIMDE_FLOAT64_C(    75.08) },
      { SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(     6.25), SIMDE_FLOAT64_C(    48.99) },
      { SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     5.00) } },
    { { SIMDE_FLOAT64_C(    74.95), SIMDE_FLOAT64_C(    97.10) },
      { SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(     9.84), SIMDE_FLOAT64_C(    31.53) },
      { SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     4.00) } },
    { { SIMDE_FLOAT64_C(    85.29), SIMDE_FLOAT64_C(    31.26) },
      { SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     4.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_logb_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_logb_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(    14.78), SIMDE_FLOAT32_C(     3.51), SIMDE_FLOAT32_C(    41.15),
        SIMDE_FLOAT32_C(    36.54), SIMDE_FLOAT32_C(    70.74), SIMDE_FLOAT32_C(    85.77), SIMDE_FLOAT32_C(    73.18) },
      { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00) } },
    { { SIMDE_FLOAT32_C(     8.54), SIMDE_FLOAT32_C(    76.06), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     1.66),
        SIMDE_FLOAT32_C(    10.98), SIMDE_FLOAT32_C(    98.59), SIMDE_FLOAT32_C(    85.97), SIMDE_FLOAT32_C(    34.95) },
      { SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    50.04), SIMDE_FLOAT32_C(    79.93), SIMDE_FLOAT32_C(    79.22), SIMDE_FLOAT32_C(    75.66),
        SIMDE_FLOAT32_C(    78.73), SIMDE_FLOAT32_C(    98.52), SIMDE_FLOAT32_C(    71.74), SIMDE_FLOAT32_C(    29.91) },
      { SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     4.00) } },
    { { SIMDE_FLOAT32_C(    36.91), SIMDE_FLOAT32_C(    76.48), SIMDE_FLOAT32_C(    92.50), SIMDE_FLOAT32_C(    91.82),
        SIMDE_FLOAT32_C(    48.28), SIMDE_FLOAT32_C(    85.39), SIMDE_FLOAT32_C(    15.78), SIMDE_FLOAT32_C(    51.62) },
      { SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    19.29), SIMDE_FLOAT32_C(    92.76), SIMDE_FLOAT32_C(    36.71),
        SIMDE_FLOAT32_C(    90.02), SIMDE_FLOAT32_C(    78.53), SIMDE_FLOAT32_C(     9.89), SIMDE_FLOAT32_C(    98.56) },
      { SIMDE_FLOAT32_C(    -3.00), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00) } },
    { { SIMDE_FLOAT32_C(    54.59), SIMDE_FLOAT32_C(    13.36), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(    65.57),
        SIMDE_FLOAT32_C(    11.95), SIMDE_FLOAT32_C(    86.19), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(    61.99) },
      { SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(    -3.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    66.13), SIMDE_FLOAT32_C(    79.73), SIMDE_FLOAT32_C(    37.65), SIMDE_FLOAT32_C(    44.86),
        SIMDE_FLOAT32_C(    78.25), SIMDE_FLOAT32_C(     9.39), SIMDE_FLOAT32_C(    74.76), SIMDE_FLOAT32_C(    15.16) },
      { SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     3.00) } },
    { { SIMDE_FLOAT32_C(    85.87), SIMDE_FLOAT32_C(    67.26), SIMDE_FLOAT32_C(     6.97), SIMDE_FLOAT32_C(    34.15),
        SIMDE_FLOAT32_C(    52.65), SIMDE_FLOAT32_C(    22.75), SIMDE_FLOAT32_C(    85.77), SIMDE_FLOAT32_C(    52.82) },
      { SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_logb_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_logb_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    12.13), SIMDE_FLOAT64_C(    86.21), SIMDE_FLOAT64_C(    41.78), SIMDE_FLOAT64_C(     6.77) },
      { SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     2.00) } },
    { { SIMDE_FLOAT64_C(     9.71), SIMDE_FLOAT64_C(    21.14), SIMDE_FLOAT64_C(    79.78), SIMDE_FLOAT64_C(    24.32) },
      { SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     4.00) } },
    { { SIMDE_FLOAT64_C(    11.31), SIMDE_FLOAT64_C(    66.21), SIMDE_FLOAT64_C(    43.11), SIMDE_FLOAT64_C(    34.90) },
      { SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     5.00) } },
    { { SIMDE_FLOAT64_C(    20.79), SIMDE_FLOAT64_C(    71.26), SIMDE_FLOAT64_C(    78.76), SIMDE_FLOAT64_C(    61.13) },
      { SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     5.00) } },
    { { SIMDE_FLOAT64_C(    36.20), SIMDE_FLOAT64_C(     5.13), SIMDE_FLOAT64_C(    45.05), SIMDE_FLOAT64_C(    35.23) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     5.00) } },
    { { SIMDE_FLOAT64_C(    73.81), SIMDE_FLOAT64_C(    52.97), SIMDE_FLOAT64_C(    18.59), SIMDE_FLOAT64_C(    15.62) },
      { SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     3.00) } },
    { { SIMDE_FLOAT64_C(    69.75), SIMDE_FLOAT64_C(    24.82), SIMDE_FLOAT64_C(    30.54), SIMDE_FLOAT64_C(    67.55) },
      { SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(    11.30), SIMDE_FLOAT64_C(    38.09), SIMDE_FLOAT64_C(    44.42), SIMDE_FLOAT64_C(    23.43) },
      { SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     4.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_logb_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_logb_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    21.10), SIMDE_FLOAT32_C(    11.56), SIMDE_FLOAT32_C(     9.28), SIMDE_FLOAT32_C(    74.19),
        SIMDE_FLOAT32_C(    63.11), SIMDE_FLOAT32_C(    46.70), SIMDE_FLOAT32_C(     5.76), SIMDE_FLOAT32_C(    81.08),
        SIMDE_FLOAT32_C(    64.90), SIMDE_FLOAT32_C(    46.85), SIMDE_FLOAT32_C(    89.59), SIMDE_FLOAT32_C(    87.79),
        SIMDE_FLOAT32_C(    91.37), SIMDE_FLOAT32_C(    41.43), SIMDE_FLOAT32_C(    25.79), SIMDE_FLOAT32_C(    88.74) },
      { SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     6.00) } },
    { { SIMDE_FLOAT32_C(    11.74), SIMDE_FLOAT32_C(    71.01), SIMDE_FLOAT32_C(    59.27), SIMDE_FLOAT32_C(     4.58),
        SIMDE_FLOAT32_C(     8.70), SIMDE_FLOAT32_C(    79.13), SIMDE_FLOAT32_C(    97.09), SIMDE_FLOAT32_C(    48.86),
        SIMDE_FLOAT32_C(    12.81), SIMDE_FLOAT32_C(    63.88), SIMDE_FLOAT32_C(    81.17), SIMDE_FLOAT32_C(    72.37),
        SIMDE_FLOAT32_C(     6.60), SIMDE_FLOAT32_C(    41.15), SIMDE_FLOAT32_C(     9.63), SIMDE_FLOAT32_C(    27.69) },
      { SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     4.00) } },
    { { SIMDE_FLOAT32_C(    52.70), SIMDE_FLOAT32_C(    18.90), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(    15.81),
        SIMDE_FLOAT32_C(    65.61), SIMDE_FLOAT32_C(     7.64), SIMDE_FLOAT32_C(    96.89), SIMDE_FLOAT32_C(    30.50),
        SIMDE_FLOAT32_C(    54.49), SIMDE_FLOAT32_C(    86.48), SIMDE_FLOAT32_C(    18.30), SIMDE_FLOAT32_C(    45.86),
        SIMDE_FLOAT32_C(    27.91), SIMDE_FLOAT32_C(    44.09), SIMDE_FLOAT32_C(    34.59), SIMDE_FLOAT32_C(    39.65) },
      { SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     3.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     4.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    15.10), SIMDE_FLOAT32_C(    93.86), SIMDE_FLOAT32_C(    44.23), SIMDE_FLOAT32_C(    23.80),
        SIMDE_FLOAT32_C(    72.99), SIMDE_FLOAT32_C(    41.32), SIMDE_FLOAT32_C(    72.65), SIMDE_FLOAT32_C(    85.79),
        SIMDE_FLOAT32_C(     5.20), SIMDE_FLOAT32_C(    53.82), SIMDE_FLOAT32_C(    58.16), SIMDE_FLOAT32_C(    11.80),
        SIMDE_FLOAT32_C(    94.97), SIMDE_FLOAT32_C(    67.79), SIMDE_FLOAT32_C(    39.49), SIMDE_FLOAT32_C(    47.67) },
      { SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     4.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     3.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    86.69), SIMDE_FLOAT32_C(    41.37), SIMDE_FLOAT32_C(    63.48), SIMDE_FLOAT32_C(    52.30),
        SIMDE_FLOAT32_C(    49.01), SIMDE_FLOAT32_C(    60.37), SIMDE_FLOAT32_C(    82.80), SIMDE_FLOAT32_C(     3.50),
        SIMDE_FLOAT32_C(    46.85), SIMDE_FLOAT32_C(     1.10), SIMDE_FLOAT32_C(    49.36), SIMDE_FLOAT32_C(    74.76),
        SIMDE_FLOAT32_C(    45.19), SIMDE_FLOAT32_C(    83.95), SIMDE_FLOAT32_C(    14.42), SIMDE_FLOAT32_C(    60.29) },
      { SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    77.81), SIMDE_FLOAT32_C(    58.65), SIMDE_FLOAT32_C(    84.09), SIMDE_FLOAT32_C(    50.80),
        SIMDE_FLOAT32_C(    99.97), SIMDE_FLOAT32_C(    56.74), SIMDE_FLOAT32_C(    36.60), SIMDE_FLOAT32_C(     5.17),
        SIMDE_FLOAT32_C(    10.56), SIMDE_FLOAT32_C(    94.76), SIMDE_FLOAT32_C(    16.97), SIMDE_FLOAT32_C(     5.53),
        SIMDE_FLOAT32_C(    62.55), SIMDE_FLOAT32_C(    56.46), SIMDE_FLOAT32_C(    53.21), SIMDE_FLOAT32_C(    49.24) },
      { SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    97.83), SIMDE_FLOAT32_C(    16.69), SIMDE_FLOAT32_C(     1.54), SIMDE_FLOAT32_C(    46.83),
        SIMDE_FLOAT32_C(    77.05), SIMDE_FLOAT32_C(    84.34), SIMDE_FLOAT32_C(    50.33), SIMDE_FLOAT32_C(    23.90),
        SIMDE_FLOAT32_C(    85.44), SIMDE_FLOAT32_C(    99.69), SIMDE_FLOAT32_C(    98.67), SIMDE_FLOAT32_C(    30.63),
        SIMDE_FLOAT32_C(    83.65), SIMDE_FLOAT32_C(    13.08), SIMDE_FLOAT32_C(    90.93), SIMDE_FLOAT32_C(    61.46) },
      { SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     4.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     4.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00) } },
    { { SIMDE_FLOAT32_C(    71.73), SIMDE_FLOAT32_C(    75.01), SIMDE_FLOAT32_C(    12.26), SIMDE_FLOAT32_C(    71.69),
        SIMDE_FLOAT32_C(    31.76), SIMDE_FLOAT32_C(    48.85), SIMDE_FLOAT32_C(    76.86), SIMDE_FLOAT32_C(    42.32),
        SIMDE_FLOAT32_C(    43.61), SIMDE_FLOAT32_C(    93.83), SIMDE_FLOAT32_C(    47.85), SIMDE_FLOAT32_C(     6.16),
        SIMDE_FLOAT32_C(    50.28), SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(    55.40), SIMDE_FLOAT32_C(    48.11) },
      { SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_logb_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_logb_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(    59.92), SIMDE_FLOAT32_C(    53.26), SIMDE_FLOAT32_C(     9.59), SIMDE_FLOAT32_C(     7.55),
        SIMDE_FLOAT32_C(    46.15), SIMDE_FLOAT32_C(    64.62), SIMDE_FLOAT32_C(    71.46), SIMDE_FLOAT32_C(    14.44),
        SIMDE_FLOAT32_C(    20.71), SIMDE_FLOAT32_C(    37.36), SIMDE_FLOAT32_C(    74.54), SIMDE_FLOAT32_C(    71.98),
        SIMDE_FLOAT32_C(     5.60), SIMDE_FLOAT32_C(    24.56), SIMDE_FLOAT32_C(    41.64), SIMDE_FLOAT32_C(    65.45) },
      UINT8_C( 74),
      { SIMDE_FLOAT32_C(    94.52), SIMDE_FLOAT32_C(    66.49), SIMDE_FLOAT32_C(    56.15), SIMDE_FLOAT32_C(    82.67),
        SIMDE_FLOAT32_C(    41.42), SIMDE_FLOAT32_C(    98.41), SIMDE_FLOAT32_C(    74.30), SIMDE_FLOAT32_C(    60.40),
        SIMDE_FLOAT32_C(    20.04), SIMDE_FLOAT32_C(    51.01), SIMDE_FLOAT32_C(     8.26), SIMDE_FLOAT32_C(    26.15),
        SIMDE_FLOAT32_C(    61.43), SIMDE_FLOAT32_C(    26.22), SIMDE_FLOAT32_C(    86.06), SIMDE_FLOAT32_C(    14.69) },
      { SIMDE_FLOAT32_C(    59.92), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     9.59), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(    46.15), SIMDE_FLOAT32_C(    64.62), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(    14.44),
        SIMDE_FLOAT32_C(    20.71), SIMDE_FLOAT32_C(    37.36), SIMDE_FLOAT32_C(    74.54), SIMDE_FLOAT32_C(    71.98),
        SIMDE_FLOAT32_C(     5.60), SIMDE_FLOAT32_C(    24.56), SIMDE_FLOAT32_C(    41.64), SIMDE_FLOAT32_C(    65.45) } },
    { { SIMDE_FLOAT32_C(    35.81), SIMDE_FLOAT32_C(    93.61), SIMDE_FLOAT32_C(    60.84), SIMDE_FLOAT32_C(     0.43),
        SIMDE_FLOAT32_C(    65.08), SIMDE_FLOAT32_C(    75.28), SIMDE_FLOAT32_C(    21.13), SIMDE_FLOAT32_C(     2.43),
        SIMDE_FLOAT32_C(    49.82), SIMDE_FLOAT32_C(    93.11), SIMDE_FLOAT32_C(     8.03), SIMDE_FLOAT32_C(    74.37),
        SIMDE_FLOAT32_C(    34.75), SIMDE_FLOAT32_C(    73.48), SIMDE_FLOAT32_C(    66.83), SIMDE_FLOAT32_C(    29.26) },
      UINT8_C(187),
      { SIMDE_FLOAT32_C(    22.98), SIMDE_FLOAT32_C(    11.94), SIMDE_FLOAT32_C(    81.39), SIMDE_FLOAT32_C(    21.39),
        SIMDE_FLOAT32_C(    86.23), SIMDE_FLOAT32_C(    41.79), SIMDE_FLOAT32_C(    41.43), SIMDE_FLOAT32_C(    37.25),
        SIMDE_FLOAT32_C(    50.05), SIMDE_FLOAT32_C(    67.58), SIMDE_FLOAT32_C(    98.68), SIMDE_FLOAT32_C(    76.27),
        SIMDE_FLOAT32_C(    53.64), SIMDE_FLOAT32_C(    13.37), SIMDE_FLOAT32_C(    12.08), SIMDE_FLOAT32_C(    47.25) },
      { SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(    60.84), SIMDE_FLOAT32_C(     4.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(    21.13), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(    49.82), SIMDE_FLOAT32_C(    93.11), SIMDE_FLOAT32_C(     8.03), SIMDE_FLOAT32_C(    74.37),
        SIMDE_FLOAT32_C(    34.75), SIMDE_FLOAT32_C(    73.48), SIMDE_FLOAT32_C(    66.83), SIMDE_FLOAT32_C(    29.26) } },
    { { SIMDE_FLOAT32_C(    74.20), SIMDE_FLOAT32_C(    12.51), SIMDE_FLOAT32_C(    12.33), SIMDE_FLOAT32_C(    49.48),
        SIMDE_FLOAT32_C(    33.65), SIMDE_FLOAT32_C(    14.76), SIMDE_FLOAT32_C(    99.30), SIMDE_FLOAT32_C(    26.76),
        SIMDE_FLOAT32_C(    22.79), SIMDE_FLOAT32_C(    73.68), SIMDE_FLOAT32_C(    61.50), SIMDE_FLOAT32_C(    96.27),
        SIMDE_FLOAT32_C(    40.51), SIMDE_FLOAT32_C(    90.77), SIMDE_FLOAT32_C(    36.25), SIMDE_FLOAT32_C(    63.49) },
      UINT8_C(162),
      { SIMDE_FLOAT32_C(    17.64), SIMDE_FLOAT32_C(    84.88), SIMDE_FLOAT32_C(    88.94), SIMDE_FLOAT32_C(    59.43),
        SIMDE_FLOAT32_C(    26.31), SIMDE_FLOAT32_C(    26.18), SIMDE_FLOAT32_C(     9.49), SIMDE_FLOAT32_C(    93.89),
        SIMDE_FLOAT32_C(    24.86), SIMDE_FLOAT32_C(    85.76), SIMDE_FLOAT32_C(    47.53), SIMDE_FLOAT32_C(    38.23),
        SIMDE_FLOAT32_C(    97.84), SIMDE_FLOAT32_C(    94.78), SIMDE_FLOAT32_C(    12.43), SIMDE_FLOAT32_C(    10.35) },
      { SIMDE_FLOAT32_C(    74.20), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(    12.33), SIMDE_FLOAT32_C(    49.48),
        SIMDE_FLOAT32_C(    33.65), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(    99.30), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(    22.79), SIMDE_FLOAT32_C(    73.68), SIMDE_FLOAT32_C(    61.50), SIMDE_FLOAT32_C(    96.27),
        SIMDE_FLOAT32_C(    40.51), SIMDE_FLOAT32_C(    90.77), SIMDE_FLOAT32_C(    36.25), SIMDE_FLOAT32_C(    63.49) } },
    { { SIMDE_FLOAT32_C(     7.11), SIMDE_FLOAT32_C(    61.92), SIMDE_FLOAT32_C(    44.00), SIMDE_FLOAT32_C(    21.88),
        SIMDE_FLOAT32_C(    61.22), SIMDE_FLOAT32_C(    70.75), SIMDE_FLOAT32_C(    44.67), SIMDE_FLOAT32_C(    34.90),
        SIMDE_FLOAT32_C(    32.26), SIMDE_FLOAT32_C(    40.94), SIMDE_FLOAT32_C(    75.40), SIMDE_FLOAT32_C(    23.02),
        SIMDE_FLOAT32_C(    77.19), SIMDE_FLOAT32_C(    38.89), SIMDE_FLOAT32_C(    25.73), SIMDE_FLOAT32_C(    94.83) },
      UINT8_C(143),
      { SIMDE_FLOAT32_C(    14.67), SIMDE_FLOAT32_C(    54.26), SIMDE_FLOAT32_C(    50.08), SIMDE_FLOAT32_C(    40.85),
        SIMDE_FLOAT32_C(    63.75), SIMDE_FLOAT32_C(    43.97), SIMDE_FLOAT32_C(    65.71), SIMDE_FLOAT32_C(    49.51),
        SIMDE_FLOAT32_C(    91.50), SIMDE_FLOAT32_C(     3.94), SIMDE_FLOAT32_C(    47.35), SIMDE_FLOAT32_C(    86.29),
        SIMDE_FLOAT32_C(    16.37), SIMDE_FLOAT32_C(    57.70), SIMDE_FLOAT32_C(    93.40), SIMDE_FLOAT32_C(    78.29) },
      { SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(    61.22), SIMDE_FLOAT32_C(    70.75), SIMDE_FLOAT32_C(    44.67), SIMDE_FLOAT32_C(     5.00),
        SIMDE_FLOAT32_C(    32.26), SIMDE_FLOAT32_C(    40.94), SIMDE_FLOAT32_C(    75.40), SIMDE_FLOAT32_C(    23.02),
        SIMDE_FLOAT32_C(    77.19), SIMDE_FLOAT32_C(    38.89), SIMDE_FLOAT32_C(    25.73), SIMDE_FLOAT32_C(    94.83) } },
    { { SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(    15.27), SIMDE_FLOAT32_C(    39.51), SIMDE_FLOAT32_C(    72.45),
        SIMDE_FLOAT32_C(    59.94), SIMDE_FLOAT32_C(    74.41), SIMDE_FLOAT32_C(     4.71), SIMDE_FLOAT32_C(     0.89),
        SIMDE_FLOAT32_C(    49.81), SIMDE_FLOAT32_C(    27.73), SIMDE_FLOAT32_C(    78.08), SIMDE_FLOAT32_C(    88.70),
        SIMDE_FLOAT32_C(    53.46), SIMDE_FLOAT32_C(    72.91), SIMDE_FLOAT32_C(    12.47), SIMDE_FLOAT32_C(    68.13) },
      UINT8_C(127),
      { SIMDE_FLOAT32_C(    62.56), SIMDE_FLOAT32_C(     8.97), SIMDE_FLOAT32_C(    90.92), SIMDE_FLOAT32_C(     6.53),
        SIMDE_FLOAT32_C(    74.69), SIMDE_FLOAT32_C(    40.42), SIMDE_FLOAT32_C(    98.03), SIMDE_FLOAT32_C(    78.63),
        SIMDE_FLOAT32_C(    87.77), SIMDE_FLOAT32_C(    84.32), SIMDE_FLOAT32_C(    95.00), SIMDE_FLOAT32_C(    45.47),
        SIMDE_FLOAT32_C(    77.72), SIMDE_FLOAT32_C(    73.29), SIMDE_FLOAT32_C(    47.17), SIMDE_FLOAT32_C(    92.99) },
      { SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(     0.89),
        SIMDE_FLOAT32_C(    49.81), SIMDE_FLOAT32_C(    27.73), SIMDE_FLOAT32_C(    78.08), SIMDE_FLOAT32_C(    88.70),
        SIMDE_FLOAT32_C(    53.46), SIMDE_FLOAT32_C(    72.91), SIMDE_FLOAT32_C(    12.47), SIMDE_FLOAT32_C(    68.13) } },
    { { SIMDE_FLOAT32_C(    12.80), SIMDE_FLOAT32_C(    19.62), SIMDE_FLOAT32_C(    52.94), SIMDE_FLOAT32_C(    87.20),
        SIMDE_FLOAT32_C(    24.32), SIMDE_FLOAT32_C(    53.82), SIMDE_FLOAT32_C(    37.01), SIMDE_FLOAT32_C(    52.06),
        SIMDE_FLOAT32_C(    31.90), SIMDE_FLOAT32_C(    25.71), SIMDE_FLOAT32_C(     5.52), SIMDE_FLOAT32_C(     4.81),
        SIMDE_FLOAT32_C(    38.19), SIMDE_FLOAT32_C(    73.64), SIMDE_FLOAT32_C(    31.98), SIMDE_FLOAT32_C(     0.74) },
      UINT8_C( 81),
      { SIMDE_FLOAT32_C(    22.90), SIMDE_FLOAT32_C(     7.28), SIMDE_FLOAT32_C(    57.30), SIMDE_FLOAT32_C(    63.32),
        SIMDE_FLOAT32_C(     5.31), SIMDE_FLOAT32_C(    35.93), SIMDE_FLOAT32_C(    51.08), SIMDE_FLOAT32_C(    89.63),
        SIMDE_FLOAT32_C(    30.93), SIMDE_FLOAT32_C(    96.55), SIMDE_FLOAT32_C(    67.35), SIMDE_FLOAT32_C(     4.22),
        SIMDE_FLOAT32_C(    43.72), SIMDE_FLOAT32_C(    60.34), SIMDE_FLOAT32_C(    17.01), SIMDE_FLOAT32_C(    63.33) },
      { SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(    19.62), SIMDE_FLOAT32_C(    52.94), SIMDE_FLOAT32_C(    87.20),
        SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(    53.82), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(    52.06),
        SIMDE_FLOAT32_C(    31.90), SIMDE_FLOAT32_C(    25.71), SIMDE_FLOAT32_C(     5.52), SIMDE_FLOAT32_C(     4.81),
        SIMDE_FLOAT32_C(    38.19), SIMDE_FLOAT32_C(    73.64), SIMDE_FLOAT32_C(    31.98), SIMDE_FLOAT32_C(     0.74) } },
    { { SIMDE_FLOAT32_C(    13.27), SIMDE_FLOAT32_C(     4.22), SIMDE_FLOAT32_C(    87.66), SIMDE_FLOAT32_C(    67.10),
        SIMDE_FLOAT32_C(    41.23), SIMDE_FLOAT32_C(    39.71), SIMDE_FLOAT32_C(    99.00), SIMDE_FLOAT32_C(    66.95),
        SIMDE_FLOAT32_C(    45.23), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     5.13), SIMDE_FLOAT32_C(    18.87),
        SIMDE_FLOAT32_C(    35.79), SIMDE_FLOAT32_C(     5.88), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(    58.69) },
      UINT8_C(192),
      { SIMDE_FLOAT32_C(    58.78), SIMDE_FLOAT32_C(    22.00), SIMDE_FLOAT32_C(    18.46), SIMDE_FLOAT32_C(    94.71),
        SIMDE_FLOAT32_C(    73.09), SIMDE_FLOAT32_C(     8.09), SIMDE_FLOAT32_C(    25.64), SIMDE_FLOAT32_C(    69.64),
        SIMDE_FLOAT32_C(    75.44), SIMDE_FLOAT32_C(    29.86), SIMDE_FLOAT32_C(    13.36), SIMDE_FLOAT32_C(    35.77),
        SIMDE_FLOAT32_C(    46.87), SIMDE_FLOAT32_C(    76.69), SIMDE_FLOAT32_C(    49.05), SIMDE_FLOAT32_C(    51.09) },
      { SIMDE_FLOAT32_C(    13.27), SIMDE_FLOAT32_C(     4.22), SIMDE_FLOAT32_C(    87.66), SIMDE_FLOAT32_C(    67.10),
        SIMDE_FLOAT32_C(    41.23), SIMDE_FLOAT32_C(    39.71), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     6.00),
        SIMDE_FLOAT32_C(    45.23), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     5.13), SIMDE_FLOAT32_C(    18.87),
        SIMDE_FLOAT32_C(    35.79), SIMDE_FLOAT32_C(     5.88), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(    58.69) } },
    { { SIMDE_FLOAT32_C(    64.34), SIMDE_FLOAT32_C(    16.14), SIMDE_FLOAT32_C(    92.32), SIMDE_FLOAT32_C(     4.06),
        SIMDE_FLOAT32_C(    15.14), SIMDE_FLOAT32_C(    59.27), SIMDE_FLOAT32_C(    49.28), SIMDE_FLOAT32_C(    18.96),
        SIMDE_FLOAT32_C(    64.40), SIMDE_FLOAT32_C(    68.15), SIMDE_FLOAT32_C(    54.75), SIMDE_FLOAT32_C(    70.28),
        SIMDE_FLOAT32_C(    69.63), SIMDE_FLOAT32_C(    13.43), SIMDE_FLOAT32_C(    83.43), SIMDE_FLOAT32_C(    28.42) },
      UINT8_C( 42),
      { SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(    23.13), SIMDE_FLOAT32_C(     8.52), SIMDE_FLOAT32_C(     9.98),
        SIMDE_FLOAT32_C(    48.77), SIMDE_FLOAT32_C(    78.16), SIMDE_FLOAT32_C(    85.41), SIMDE_FLOAT32_C(    78.63),
        SIMDE_FLOAT32_C(    91.52), SIMDE_FLOAT32_C(    21.19), SIMDE_FLOAT32_C(    25.50), SIMDE_FLOAT32_C(    68.21),
        SIMDE_FLOAT32_C(    70.23), SIMDE_FLOAT32_C(    76.59), SIMDE_FLOAT32_C(    32.55), SIMDE_FLOAT32_C(    86.38) },
      { SIMDE_FLOAT32_C(    64.34), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(    92.32), SIMDE_FLOAT32_C(     3.00),
        SIMDE_FLOAT32_C(    15.14), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(    49.28), SIMDE_FLOAT32_C(    18.96),
        SIMDE_FLOAT32_C(    64.40), SIMDE_FLOAT32_C(    68.15), SIMDE_FLOAT32_C(    54.75), SIMDE_FLOAT32_C(    70.28),
        SIMDE_FLOAT32_C(    69.63), SIMDE_FLOAT32_C(    13.43), SIMDE_FLOAT32_C(    83.43), SIMDE_FLOAT32_C(    28.42) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_logb_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_logb_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    61.06), SIMDE_FLOAT64_C(    56.07), SIMDE_FLOAT64_C(     3.95), SIMDE_FLOAT64_C(    60.43),
        SIMDE_FLOAT64_C(    57.40), SIMDE_FLOAT64_C(    69.53), SIMDE_FLOAT64_C(    29.03), SIMDE_FLOAT64_C(    89.93) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     5.00),
        SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(    49.22), SIMDE_FLOAT64_C(     9.42), SIMDE_FLOAT64_C(    73.55), SIMDE_FLOAT64_C(    15.48),
        SIMDE_FLOAT64_C(    60.82), SIMDE_FLOAT64_C(    84.59), SIMDE_FLOAT64_C(     3.74), SIMDE_FLOAT64_C(    54.66) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     3.00),
        SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     5.00) } },
    { { SIMDE_FLOAT64_C(    33.37), SIMDE_FLOAT64_C(    75.87), SIMDE_FLOAT64_C(    58.52), SIMDE_FLOAT64_C(    48.59),
        SIMDE_FLOAT64_C(    90.24), SIMDE_FLOAT64_C(    63.58), SIMDE_FLOAT64_C(    62.75), SIMDE_FLOAT64_C(    73.90) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     5.00),
        SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(    18.87), SIMDE_FLOAT64_C(    24.32), SIMDE_FLOAT64_C(    24.02), SIMDE_FLOAT64_C(    25.17),
        SIMDE_FLOAT64_C(    77.02), SIMDE_FLOAT64_C(    14.07), SIMDE_FLOAT64_C(     2.94), SIMDE_FLOAT64_C(    38.08) },
      { SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     4.00),
        SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     5.00) } },
    { { SIMDE_FLOAT64_C(    70.14), SIMDE_FLOAT64_C(     6.89), SIMDE_FLOAT64_C(    98.50), SIMDE_FLOAT64_C(    27.53),
        SIMDE_FLOAT64_C(    76.42), SIMDE_FLOAT64_C(    27.53), SIMDE_FLOAT64_C(    17.47), SIMDE_FLOAT64_C(    25.65) },
      { SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     4.00),
        SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     4.00) } },
    { { SIMDE_FLOAT64_C(    36.95), SIMDE_FLOAT64_C(    91.02), SIMDE_FLOAT64_C(    41.13), SIMDE_FLOAT64_C(    97.76),
        SIMDE_FLOAT64_C(    75.61), SIMDE_FLOAT64_C(    44.87), SIMDE_FLOAT64_C(    52.42), SIMDE_FLOAT64_C(     8.99) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00),
        SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     3.00) } },
    { { SIMDE_FLOAT64_C(    20.74), SIMDE_FLOAT64_C(    10.94), SIMDE_FLOAT64_C(    57.58), SIMDE_FLOAT64_C(    10.98),
        SIMDE_FLOAT64_C(    74.52), SIMDE_FLOAT64_C(    20.32), SIMDE_FLOAT64_C(    84.88), SIMDE_FLOAT64_C(    93.39) },
      { SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     3.00),
        SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(    44.64), SIMDE_FLOAT64_C(     8.90), SIMDE_FLOAT64_C(    18.56), SIMDE_FLOAT64_C(    21.66),
        SIMDE_FLOAT64_C(    22.97), SIMDE_FLOAT64_C(    21.51), SIMDE_FLOAT64_C(    59.73), SIMDE_FLOAT64_C(    93.10) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     4.00),
        SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_logb_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_logb_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    44.91), SIMDE_FLOAT64_C(    88.38), SIMDE_FLOAT64_C(    45.58), SIMDE_FLOAT64_C(    12.77),
        SIMDE_FLOAT64_C(    31.32), SIMDE_FLOAT64_C(    50.43), SIMDE_FLOAT64_C(    60.04), SIMDE_FLOAT64_C(     3.47) },
      UINT8_C(214),
      { SIMDE_FLOAT64_C(    86.80), SIMDE_FLOAT64_C(    42.80), SIMDE_FLOAT64_C(    69.48), SIMDE_FLOAT64_C(    71.71),
        SIMDE_FLOAT64_C(    94.56), SIMDE_FLOAT64_C(    31.31), SIMDE_FLOAT64_C(    74.51), SIMDE_FLOAT64_C(    72.92) },
      { SIMDE_FLOAT64_C(    44.91), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(    12.77),
        SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(    50.43), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(    29.96), SIMDE_FLOAT64_C(    29.49), SIMDE_FLOAT64_C(    88.44), SIMDE_FLOAT64_C(    26.63),
        SIMDE_FLOAT64_C(    15.97), SIMDE_FLOAT64_C(    77.55), SIMDE_FLOAT64_C(    47.96), SIMDE_FLOAT64_C(    96.03) },
      UINT8_C( 76),
      { SIMDE_FLOAT64_C(    85.66), SIMDE_FLOAT64_C(    58.61), SIMDE_FLOAT64_C(    61.13), SIMDE_FLOAT64_C(    28.12),
        SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     6.05), SIMDE_FLOAT64_C(    16.50), SIMDE_FLOAT64_C(    45.67) },
      { SIMDE_FLOAT64_C(    29.96), SIMDE_FLOAT64_C(    29.49), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     4.00),
        SIMDE_FLOAT64_C(    15.97), SIMDE_FLOAT64_C(    77.55), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(    96.03) } },
    { { SIMDE_FLOAT64_C(    18.81), SIMDE_FLOAT64_C(    47.82), SIMDE_FLOAT64_C(    96.10), SIMDE_FLOAT64_C(    78.86),
        SIMDE_FLOAT64_C(    51.29), SIMDE_FLOAT64_C(     7.80), SIMDE_FLOAT64_C(    65.66), SIMDE_FLOAT64_C(    94.09) },
      UINT8_C( 98),
      { SIMDE_FLOAT64_C(    37.37), SIMDE_FLOAT64_C(    88.65), SIMDE_FLOAT64_C(     8.59), SIMDE_FLOAT64_C(    11.88),
        SIMDE_FLOAT64_C(    61.57), SIMDE_FLOAT64_C(    38.54), SIMDE_FLOAT64_C(    41.37), SIMDE_FLOAT64_C(    50.02) },
      { SIMDE_FLOAT64_C(    18.81), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(    96.10), SIMDE_FLOAT64_C(    78.86),
        SIMDE_FLOAT64_C(    51.29), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(    94.09) } },
    { { SIMDE_FLOAT64_C(    65.18), SIMDE_FLOAT64_C(    57.34), SIMDE_FLOAT64_C(    27.56), SIMDE_FLOAT64_C(    13.13),
        SIMDE_FLOAT64_C(    53.38), SIMDE_FLOAT64_C(    10.85), SIMDE_FLOAT64_C(    98.80), SIMDE_FLOAT64_C(    11.98) },
      UINT8_C(227),
      { SIMDE_FLOAT64_C(    26.92), SIMDE_FLOAT64_C(    12.07), SIMDE_FLOAT64_C(    78.04), SIMDE_FLOAT64_C(    43.42),
        SIMDE_FLOAT64_C(    57.74), SIMDE_FLOAT64_C(    96.85), SIMDE_FLOAT64_C(    91.25), SIMDE_FLOAT64_C(    53.84) },
      { SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(    27.56), SIMDE_FLOAT64_C(    13.13),
        SIMDE_FLOAT64_C(    53.38), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     5.00) } },
    { { SIMDE_FLOAT64_C(    75.71), SIMDE_FLOAT64_C(    42.54), SIMDE_FLOAT64_C(    61.63), SIMDE_FLOAT64_C(    41.37),
        SIMDE_FLOAT64_C(    36.63), SIMDE_FLOAT64_C(    38.91), SIMDE_FLOAT64_C(    78.74), SIMDE_FLOAT64_C(    25.28) },
      UINT8_C(133),
      { SIMDE_FLOAT64_C(    90.62), SIMDE_FLOAT64_C(    86.86), SIMDE_FLOAT64_C(    86.04), SIMDE_FLOAT64_C(    31.99),
        SIMDE_FLOAT64_C(    36.87), SIMDE_FLOAT64_C(    51.22), SIMDE_FLOAT64_C(    89.34), SIMDE_FLOAT64_C(    64.43) },
      { SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(    42.54), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(    41.37),
        SIMDE_FLOAT64_C(    36.63), SIMDE_FLOAT64_C(    38.91), SIMDE_FLOAT64_C(    78.74), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(    64.36), SIMDE_FLOAT64_C(    42.71), SIMDE_FLOAT64_C(    75.29), SIMDE_FLOAT64_C(    63.15),
        SIMDE_FLOAT64_C(    54.70), SIMDE_FLOAT64_C(    47.28), SIMDE_FLOAT64_C(    90.08), SIMDE_FLOAT64_C(    66.76) },
      UINT8_C(185),
      { SIMDE_FLOAT64_C(    33.50), SIMDE_FLOAT64_C(    24.50), SIMDE_FLOAT64_C(    22.16), SIMDE_FLOAT64_C(    24.75),
        SIMDE_FLOAT64_C(    78.34), SIMDE_FLOAT64_C(    97.87), SIMDE_FLOAT64_C(    67.29), SIMDE_FLOAT64_C(    39.97) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(    42.71), SIMDE_FLOAT64_C(    75.29), SIMDE_FLOAT64_C(     4.00),
        SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(    90.08), SIMDE_FLOAT64_C(     5.00) } },
    { { SIMDE_FLOAT64_C(    39.24), SIMDE_FLOAT64_C(     3.92), SIMDE_FLOAT64_C(    78.88), SIMDE_FLOAT64_C(    17.98),
        SIMDE_FLOAT64_C(    29.20), SIMDE_FLOAT64_C(    26.38), SIMDE_FLOAT64_C(     8.60), SIMDE_FLOAT64_C(    16.06) },
      UINT8_C(216),
      { SIMDE_FLOAT64_C(    40.59), SIMDE_FLOAT64_C(    52.93), SIMDE_FLOAT64_C(    63.64), SIMDE_FLOAT64_C(    29.93),
        SIMDE_FLOAT64_C(    17.36), SIMDE_FLOAT64_C(    28.00), SIMDE_FLOAT64_C(    72.65), SIMDE_FLOAT64_C(    92.65) },
      { SIMDE_FLOAT64_C(    39.24), SIMDE_FLOAT64_C(     3.92), SIMDE_FLOAT64_C(    78.88), SIMDE_FLOAT64_C(     4.00),
        SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(    26.38), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     6.00) } },
    { { SIMDE_FLOAT64_C(    91.15), SIMDE_FLOAT64_C(    27.34), SIMDE_FLOAT64_C(    39.93), SIMDE_FLOAT64_C(    81.23),
        SIMDE_FLOAT64_C(    94.10), SIMDE_FLOAT64_C(    65.24), SIMDE_FLOAT64_C(    14.73), SIMDE_FLOAT64_C(    18.60) },
      UINT8_C(111),
      { SIMDE_FLOAT64_C(    39.48), SIMDE_FLOAT64_C(    96.94), SIMDE_FLOAT64_C(    85.27), SIMDE_FLOAT64_C(     6.77),
        SIMDE_FLOAT64_C(    36.91), SIMDE_FLOAT64_C(    24.51), SIMDE_FLOAT64_C(    10.68), SIMDE_FLOAT64_C(    15.79) },
      { SIMDE_FLOAT64_C(     5.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     6.00), SIMDE_FLOAT64_C(     2.00),
        SIMDE_FLOAT64_C(    94.10), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(    18.60) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_logb_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_nearbyint_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -914.49), SIMDE_FLOAT32_C(   460.45), SIMDE_FLOAT32_C(  -816.31), SIMDE_FLOAT32_C(   969.94),
        SIMDE_FLOAT32_C(  -904.29), SIMDE_FLOAT32_C(  -267.48), SIMDE_FLOAT32_C(  -362.84), SIMDE_FLOAT32_C(   -10.93),
        SIMDE_FLOAT32_C(  -124.62), SIMDE_FLOAT32_C(   667.93), SIMDE_FLOAT32_C(   512.15), SIMDE_FLOAT32_C(   -37.80),
        SIMDE_FLOAT32_C(   894.40), SIMDE_FLOAT32_C(   135.20), SIMDE_FLOAT32_C(  -763.47), SIMDE_FLOAT32_C(  -593.20) },
      { SIMDE_FLOAT32_C(  -914.00), SIMDE_FLOAT32_C(   460.00), SIMDE_FLOAT32_C(  -816.00), SIMDE_FLOAT32_C(   970.00),
        SIMDE_FLOAT32_C(  -904.00), SIMDE_FLOAT32_C(  -267.00), SIMDE_FLOAT32_C(  -363.00), SIMDE_FLOAT32_C(   -11.00),
        SIMDE_FLOAT32_C(  -125.00), SIMDE_FLOAT32_C(   668.00), SIMDE_FLOAT32_C(   512.00), SIMDE_FLOAT32_C(   -38.00),
        SIMDE_FLOAT32_C(   894.00), SIMDE_FLOAT32_C(   135.00), SIMDE_FLOAT32_C(  -763.00), SIMDE_FLOAT32_C(  -593.00) } },
    { { SIMDE_FLOAT32_C(  -849.14), SIMDE_FLOAT32_C(   852.22), SIMDE_FLOAT32_C(  -400.69), SIMDE_FLOAT32_C(   171.29),
        SIMDE_FLOAT32_C(   508.23), SIMDE_FLOAT32_C(  -765.53), SIMDE_FLOAT32_C(  -382.38), SIMDE_FLOAT32_C(  -765.99),
        SIMDE_FLOAT32_C(   -92.44), SIMDE_FLOAT32_C(   141.65), SIMDE_FLOAT32_C(   748.46), SIMDE_FLOAT32_C(    28.81),
        SIMDE_FLOAT32_C(  -715.24), SIMDE_FLOAT32_C(  -786.64), SIMDE_FLOAT32_C(   -54.59), SIMDE_FLOAT32_C(  -629.74) },
      { SIMDE_FLOAT32_C(  -849.00), SIMDE_FLOAT32_C(   852.00), SIMDE_FLOAT32_C(  -401.00), SIMDE_FLOAT32_C(   171.00),
        SIMDE_FLOAT32_C(   508.00), SIMDE_FLOAT32_C(  -766.00), SIMDE_FLOAT32_C(  -382.00), SIMDE_FLOAT32_C(  -766.00),
        SIMDE_FLOAT32_C(   -92.00), SIMDE_FLOAT32_C(   142.00), SIMDE_FLOAT32_C(   748.00), SIMDE_FLOAT32_C(    29.00),
        SIMDE_FLOAT32_C(  -715.00), SIMDE_FLOAT32_C(  -787.00), SIMDE_FLOAT32_C(   -55.00), SIMDE_FLOAT32_C(  -630.00) } },
    { { SIMDE_FLOAT32_C(   673.81), SIMDE_FLOAT32_C(   129.11), SIMDE_FLOAT32_C(  -659.80), SIMDE_FLOAT32_C(   769.52),
        SIMDE_FLOAT32_C(   861.62), SIMDE_FLOAT32_C(   -22.64), SIMDE_FLOAT32_C(  -241.41), SIMDE_FLOAT32_C(  -263.00),
        SIMDE_FLOAT32_C(  -354.71), SIMDE_FLOAT32_C(  -729.27), SIMDE_FLOAT32_C(   699.19), SIMDE_FLOAT32_C(  -460.31),
        SIMDE_FLOAT32_C(   405.93), SIMDE_FLOAT32_C(   935.73), SIMDE_FLOAT32_C(   -53.51), SIMDE_FLOAT32_C(   556.79) },
      { SIMDE_FLOAT32_C(   674.00), SIMDE_FLOAT32_C(   129.00), SIMDE_FLOAT32_C(  -660.00), SIMDE_FLOAT32_C(   770.00),
        SIMDE_FLOAT32_C(   862.00), SIMDE_FLOAT32_C(   -23.00), SIMDE_FLOAT32_C(  -241.00), SIMDE_FLOAT32_C(  -263.00),
        SIMDE_FLOAT32_C(  -355.00), SIMDE_FLOAT32_C(  -729.00), SIMDE_FLOAT32_C(   699.00), SIMDE_FLOAT32_C(  -460.00),
        SIMDE_FLOAT32_C(   406.00), SIMDE_FLOAT32_C(   936.00), SIMDE_FLOAT32_C(   -54.00), SIMDE_FLOAT32_C(   557.00) } },
    { { SIMDE_FLOAT32_C(   787.95), SIMDE_FLOAT32_C(   545.80), SIMDE_FLOAT32_C(  -271.92), SIMDE_FLOAT32_C(   296.18),
        SIMDE_FLOAT32_C(   780.27), SIMDE_FLOAT32_C(   345.70), SIMDE_FLOAT32_C(   530.19), SIMDE_FLOAT32_C(  -312.17),
        SIMDE_FLOAT32_C(  -512.65), SIMDE_FLOAT32_C(   278.65), SIMDE_FLOAT32_C(   716.64), SIMDE_FLOAT32_C(  -227.89),
        SIMDE_FLOAT32_C(   492.01), SIMDE_FLOAT32_C(  -337.94), SIMDE_FLOAT32_C(   142.37), SIMDE_FLOAT32_C(   165.82) },
      { SIMDE_FLOAT32_C(   788.00), SIMDE_FLOAT32_C(   546.00), SIMDE_FLOAT32_C(  -272.00), SIMDE_FLOAT32_C(   296.00),
        SIMDE_FLOAT32_C(   780.00), SIMDE_FLOAT32_C(   346.00), SIMDE_FLOAT32_C(   530.00), SIMDE_FLOAT32_C(  -312.00),
        SIMDE_FLOAT32_C(  -513.00), SIMDE_FLOAT32_C(   279.00), SIMDE_FLOAT32_C(   717.00), SIMDE_FLOAT32_C(  -228.00),
        SIMDE_FLOAT32_C(   492.00), SIMDE_FLOAT32_C(  -338.00), SIMDE_FLOAT32_C(   142.00), SIMDE_FLOAT32_C(   166.00) } },
    { { SIMDE_FLOAT32_C(   791.16), SIMDE_FLOAT32_C(   482.57), SIMDE_FLOAT32_C(   -64.66), SIMDE_FLOAT32_C(   652.78),
        SIMDE_FLOAT32_C(  -540.07), SIMDE_FLOAT32_C(   693.92), SIMDE_FLOAT32_C(  -610.22), SIMDE_FLOAT32_C(   105.21),
        SIMDE_FLOAT32_C(   964.66), SIMDE_FLOAT32_C(  -911.03), SIMDE_FLOAT32_C(   644.90), SIMDE_FLOAT32_C(   370.59),
        SIMDE_FLOAT32_C(  -975.30), SIMDE_FLOAT32_C(  -408.60), SIMDE_FLOAT32_C(   -72.62), SIMDE_FLOAT32_C(   812.65) },
      { SIMDE_FLOAT32_C(   791.00), SIMDE_FLOAT32_C(   483.00), SIMDE_FLOAT32_C(   -65.00), SIMDE_FLOAT32_C(   653.00),
        SIMDE_FLOAT32_C(  -540.00), SIMDE_FLOAT32_C(   694.00), SIMDE_FLOAT32_C(  -610.00), SIMDE_FLOAT32_C(   105.00),
        SIMDE_FLOAT32_C(   965.00), SIMDE_FLOAT32_C(  -911.00), SIMDE_FLOAT32_C(   645.00), SIMDE_FLOAT32_C(   371.00),
        SIMDE_FLOAT32_C(  -975.00), SIMDE_FLOAT32_C(  -409.00), SIMDE_FLOAT32_C(   -73.00), SIMDE_FLOAT32_C(   813.00) } },
    { { SIMDE_FLOAT32_C(  -862.80), SIMDE_FLOAT32_C(   655.47), SIMDE_FLOAT32_C(   108.83), SIMDE_FLOAT32_C(   917.47),
        SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(  -360.98), SIMDE_FLOAT32_C(  -394.70), SIMDE_FLOAT32_C(   488.51),
        SIMDE_FLOAT32_C(   917.67), SIMDE_FLOAT32_C(  -678.06), SIMDE_FLOAT32_C(  -739.38), SIMDE_FLOAT32_C(   409.68),
        SIMDE_FLOAT32_C(   -16.00), SIMDE_FLOAT32_C(   402.99), SIMDE_FLOAT32_C(  -424.50), SIMDE_FLOAT32_C(  -224.84) },
      { SIMDE_FLOAT32_C(  -863.00), SIMDE_FLOAT32_C(   655.00), SIMDE_FLOAT32_C(   109.00), SIMDE_FLOAT32_C(   917.00),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(  -361.00), SIMDE_FLOAT32_C(  -395.00), SIMDE_FLOAT32_C(   489.00),
        SIMDE_FLOAT32_C(   918.00), SIMDE_FLOAT32_C(  -678.00), SIMDE_FLOAT32_C(  -739.00), SIMDE_FLOAT32_C(   410.00),
        SIMDE_FLOAT32_C(   -16.00), SIMDE_FLOAT32_C(   403.00), SIMDE_FLOAT32_C(  -424.00), SIMDE_FLOAT32_C(  -225.00) } },
    { { SIMDE_FLOAT32_C(  -114.44), SIMDE_FLOAT32_C(   510.83), SIMDE_FLOAT32_C(  -572.05), SIMDE_FLOAT32_C(   345.49),
        SIMDE_FLOAT32_C(   204.76), SIMDE_FLOAT32_C(  -182.27), SIMDE_FLOAT32_C(  -549.30), SIMDE_FLOAT32_C(   169.42),
        SIMDE_FLOAT32_C(   -93.30), SIMDE_FLOAT32_C(  -904.39), SIMDE_FLOAT32_C(  -459.99), SIMDE_FLOAT32_C(   -68.59),
        SIMDE_FLOAT32_C(  -313.00), SIMDE_FLOAT32_C(   467.39), SIMDE_FLOAT32_C(  -255.94), SIMDE_FLOAT32_C(  -175.80) },
      { SIMDE_FLOAT32_C(  -114.00), SIMDE_FLOAT32_C(   511.00), SIMDE_FLOAT32_C(  -572.00), SIMDE_FLOAT32_C(   345.00),
        SIMDE_FLOAT32_C(   205.00), SIMDE_FLOAT32_C(  -182.00), SIMDE_FLOAT32_C(  -549.00), SIMDE_FLOAT32_C(   169.00),
        SIMDE_FLOAT32_C(   -93.00), SIMDE_FLOAT32_C(  -904.00), SIMDE_FLOAT32_C(  -460.00), SIMDE_FLOAT32_C(   -69.00),
        SIMDE_FLOAT32_C(  -313.00), SIMDE_FLOAT32_C(   467.00), SIMDE_FLOAT32_C(  -256.00), SIMDE_FLOAT32_C(  -176.00) } },
    { { SIMDE_FLOAT32_C(   122.86), SIMDE_FLOAT32_C(   852.89), SIMDE_FLOAT32_C(  -258.33), SIMDE_FLOAT32_C(  -875.98),
        SIMDE_FLOAT32_C(  -508.09), SIMDE_FLOAT32_C(   346.97), SIMDE_FLOAT32_C(   612.54), SIMDE_FLOAT32_C(  -590.42),
        SIMDE_FLOAT32_C(   668.92), SIMDE_FLOAT32_C(   873.16), SIMDE_FLOAT32_C(   819.25), SIMDE_FLOAT32_C(  -347.08),
        SIMDE_FLOAT32_C(   276.15), SIMDE_FLOAT32_C(  -605.25), SIMDE_FLOAT32_C(   428.08), SIMDE_FLOAT32_C(  -838.29) },
      { SIMDE_FLOAT32_C(   123.00), SIMDE_FLOAT32_C(   853.00), SIMDE_FLOAT32_C(  -258.00), SIMDE_FLOAT32_C(  -876.00),
        SIMDE_FLOAT32_C(  -508.00), SIMDE_FLOAT32_C(   347.00), SIMDE_FLOAT32_C(   613.00), SIMDE_FLOAT32_C(  -590.00),
        SIMDE_FLOAT32_C(   669.00), SIMDE_FLOAT32_C(   873.00), SIMDE_FLOAT32_C(   819.00), SIMDE_FLOAT32_C(  -347.00),
        SIMDE_FLOAT32_C(   276.00), SIMDE_FLOAT32_C(  -605.00), SIMDE_FLOAT32_C(   428.00), SIMDE_FLOAT32_C(  -838.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_nearbyint_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_nearbyint_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   307.14), SIMDE_FLOAT32_C(   482.19), SIMDE_FLOAT32_C(   518.39), SIMDE_FLOAT32_C(   499.59),
        SIMDE_FLOAT32_C(  -143.12), SIMDE_FLOAT32_C(   912.60), SIMDE_FLOAT32_C(   782.99), SIMDE_FLOAT32_C(  -267.99),
        SIMDE_FLOAT32_C(  -349.63), SIMDE_FLOAT32_C(  -394.28), SIMDE_FLOAT32_C(  -735.06), SIMDE_FLOAT32_C(   319.94),
        SIMDE_FLOAT32_C(  -352.44), SIMDE_FLOAT32_C(   639.50), SIMDE_FLOAT32_C(  -238.81), SIMDE_FLOAT32_C(   516.17) },
      UINT8_C(107),
      { SIMDE_FLOAT32_C(  -887.79), SIMDE_FLOAT32_C(   742.81), SIMDE_FLOAT32_C(  -913.42), SIMDE_FLOAT32_C(  -611.43),
        SIMDE_FLOAT32_C(   204.70), SIMDE_FLOAT32_C(   940.63), SIMDE_FLOAT32_C(  -825.04), SIMDE_FLOAT32_C(    37.94),
        SIMDE_FLOAT32_C(   967.28), SIMDE_FLOAT32_C(  -950.31), SIMDE_FLOAT32_C(  -916.12), SIMDE_FLOAT32_C(   338.61),
        SIMDE_FLOAT32_C(  -151.13), SIMDE_FLOAT32_C(  -229.02), SIMDE_FLOAT32_C(  -354.25), SIMDE_FLOAT32_C(  -668.94) },
      { SIMDE_FLOAT32_C(  -888.00), SIMDE_FLOAT32_C(   743.00), SIMDE_FLOAT32_C(   518.39), SIMDE_FLOAT32_C(  -611.00),
        SIMDE_FLOAT32_C(  -143.12), SIMDE_FLOAT32_C(   941.00), SIMDE_FLOAT32_C(  -825.00), SIMDE_FLOAT32_C(  -267.99),
        SIMDE_FLOAT32_C(  -349.63), SIMDE_FLOAT32_C(  -394.28), SIMDE_FLOAT32_C(  -735.06), SIMDE_FLOAT32_C(   319.94),
        SIMDE_FLOAT32_C(  -352.44), SIMDE_FLOAT32_C(   639.50), SIMDE_FLOAT32_C(  -238.81), SIMDE_FLOAT32_C(   516.17) } },
    { { SIMDE_FLOAT32_C(  -710.63), SIMDE_FLOAT32_C(  -854.67), SIMDE_FLOAT32_C(   187.94), SIMDE_FLOAT32_C(  -798.03),
        SIMDE_FLOAT32_C(   928.32), SIMDE_FLOAT32_C(   919.94), SIMDE_FLOAT32_C(  -147.65), SIMDE_FLOAT32_C(  -465.96),
        SIMDE_FLOAT32_C(  -815.12), SIMDE_FLOAT32_C(  -827.71), SIMDE_FLOAT32_C(   181.60), SIMDE_FLOAT32_C(   824.38),
        SIMDE_FLOAT32_C(   -66.52), SIMDE_FLOAT32_C(  -302.23), SIMDE_FLOAT32_C(  -118.38), SIMDE_FLOAT32_C(    45.69) },
      UINT8_C(170),
      { SIMDE_FLOAT32_C(   -31.81), SIMDE_FLOAT32_C(   434.25), SIMDE_FLOAT32_C(   645.28), SIMDE_FLOAT32_C(   -91.18),
        SIMDE_FLOAT32_C(   609.22), SIMDE_FLOAT32_C(  -316.78), SIMDE_FLOAT32_C(  -123.90), SIMDE_FLOAT32_C(   658.90),
        SIMDE_FLOAT32_C(  -232.89), SIMDE_FLOAT32_C(  -785.30), SIMDE_FLOAT32_C(  -492.22), SIMDE_FLOAT32_C(   538.09),
        SIMDE_FLOAT32_C(  -139.55), SIMDE_FLOAT32_C(  -161.16), SIMDE_FLOAT32_C(   827.46), SIMDE_FLOAT32_C(     5.78) },
      { SIMDE_FLOAT32_C(  -710.63), SIMDE_FLOAT32_C(   434.00), SIMDE_FLOAT32_C(   187.94), SIMDE_FLOAT32_C(   -91.00),
        SIMDE_FLOAT32_C(   928.32), SIMDE_FLOAT32_C(  -317.00), SIMDE_FLOAT32_C(  -147.65), SIMDE_FLOAT32_C(   659.00),
        SIMDE_FLOAT32_C(  -815.12), SIMDE_FLOAT32_C(  -827.71), SIMDE_FLOAT32_C(   181.60), SIMDE_FLOAT32_C(   824.38),
        SIMDE_FLOAT32_C(   -66.52), SIMDE_FLOAT32_C(  -302.23), SIMDE_FLOAT32_C(  -118.38), SIMDE_FLOAT32_C(    45.69) } },
    { { SIMDE_FLOAT32_C(  -973.23), SIMDE_FLOAT32_C(  -970.57), SIMDE_FLOAT32_C(   -65.89), SIMDE_FLOAT32_C(   946.72),
        SIMDE_FLOAT32_C(  -118.22), SIMDE_FLOAT32_C(   468.15), SIMDE_FLOAT32_C(  -868.40), SIMDE_FLOAT32_C(    54.07),
        SIMDE_FLOAT32_C(  -350.25), SIMDE_FLOAT32_C(   955.97), SIMDE_FLOAT32_C(   987.55), SIMDE_FLOAT32_C(   347.52),
        SIMDE_FLOAT32_C(  -162.41), SIMDE_FLOAT32_C(    33.24), SIMDE_FLOAT32_C(   788.11), SIMDE_FLOAT32_C(   805.78) },
      UINT8_C(147),
      { SIMDE_FLOAT32_C(   433.39), SIMDE_FLOAT32_C(  -285.40), SIMDE_FLOAT32_C(  -923.29), SIMDE_FLOAT32_C(  -883.39),
        SIMDE_FLOAT32_C(   590.69), SIMDE_FLOAT32_C(   735.61), SIMDE_FLOAT32_C(  -116.28), SIMDE_FLOAT32_C(   805.40),
        SIMDE_FLOAT32_C(  -756.61), SIMDE_FLOAT32_C(  -578.19), SIMDE_FLOAT32_C(  -334.15), SIMDE_FLOAT32_C(    82.23),
        SIMDE_FLOAT32_C(  -750.73), SIMDE_FLOAT32_C(   671.63), SIMDE_FLOAT32_C(   109.00), SIMDE_FLOAT32_C(  -721.30) },
      { SIMDE_FLOAT32_C(   433.00), SIMDE_FLOAT32_C(  -285.00), SIMDE_FLOAT32_C(   -65.89), SIMDE_FLOAT32_C(   946.72),
        SIMDE_FLOAT32_C(   591.00), SIMDE_FLOAT32_C(   468.15), SIMDE_FLOAT32_C(  -868.40), SIMDE_FLOAT32_C(   805.00),
        SIMDE_FLOAT32_C(  -350.25), SIMDE_FLOAT32_C(   955.97), SIMDE_FLOAT32_C(   987.55), SIMDE_FLOAT32_C(   347.52),
        SIMDE_FLOAT32_C(  -162.41), SIMDE_FLOAT32_C(    33.24), SIMDE_FLOAT32_C(   788.11), SIMDE_FLOAT32_C(   805.78) } },
    { { SIMDE_FLOAT32_C(  -394.26), SIMDE_FLOAT32_C(    55.71), SIMDE_FLOAT32_C(   160.48), SIMDE_FLOAT32_C(  -926.11),
        SIMDE_FLOAT32_C(   187.31), SIMDE_FLOAT32_C(  -785.45), SIMDE_FLOAT32_C(  -276.36), SIMDE_FLOAT32_C(   143.28),
        SIMDE_FLOAT32_C(  -797.89), SIMDE_FLOAT32_C(  -928.84), SIMDE_FLOAT32_C(   980.87), SIMDE_FLOAT32_C(   235.35),
        SIMDE_FLOAT32_C(   859.27), SIMDE_FLOAT32_C(   786.65), SIMDE_FLOAT32_C(   702.84), SIMDE_FLOAT32_C(   292.65) },
      UINT8_C(  5),
      { SIMDE_FLOAT32_C(   779.55), SIMDE_FLOAT32_C(   409.26), SIMDE_FLOAT32_C(  -908.05), SIMDE_FLOAT32_C(   515.17),
        SIMDE_FLOAT32_C(  -707.02), SIMDE_FLOAT32_C(   897.34), SIMDE_FLOAT32_C(   758.56), SIMDE_FLOAT32_C(  -285.21),
        SIMDE_FLOAT32_C(  -436.81), SIMDE_FLOAT32_C(  -159.22), SIMDE_FLOAT32_C(   -35.94), SIMDE_FLOAT32_C(  -765.18),
        SIMDE_FLOAT32_C(   949.78), SIMDE_FLOAT32_C(   242.76), SIMDE_FLOAT32_C(  -159.44), SIMDE_FLOAT32_C(     5.49) },
      { SIMDE_FLOAT32_C(   780.00), SIMDE_FLOAT32_C(    55.71), SIMDE_FLOAT32_C(  -908.00), SIMDE_FLOAT32_C(  -926.11),
        SIMDE_FLOAT32_C(   187.31), SIMDE_FLOAT32_C(  -785.45), SIMDE_FLOAT32_C(  -276.36), SIMDE_FLOAT32_C(   143.28),
        SIMDE_FLOAT32_C(  -797.89), SIMDE_FLOAT32_C(  -928.84), SIMDE_FLOAT32_C(   980.87), SIMDE_FLOAT32_C(   235.35),
        SIMDE_FLOAT32_C(   859.27), SIMDE_FLOAT32_C(   786.65), SIMDE_FLOAT32_C(   702.84), SIMDE_FLOAT32_C(   292.65) } },
    { { SIMDE_FLOAT32_C(  -596.76), SIMDE_FLOAT32_C(   -85.56), SIMDE_FLOAT32_C(  -807.20), SIMDE_FLOAT32_C(  -382.21),
        SIMDE_FLOAT32_C(   638.08), SIMDE_FLOAT32_C(   336.09), SIMDE_FLOAT32_C(  -180.10), SIMDE_FLOAT32_C(   709.25),
        SIMDE_FLOAT32_C(   316.96), SIMDE_FLOAT32_C(  -944.76), SIMDE_FLOAT32_C(   568.51), SIMDE_FLOAT32_C(   103.62),
        SIMDE_FLOAT32_C(   758.08), SIMDE_FLOAT32_C(  -138.83), SIMDE_FLOAT32_C(   604.87), SIMDE_FLOAT32_C(   537.64) },
      UINT8_C(  9),
      { SIMDE_FLOAT32_C(   696.82), SIMDE_FLOAT32_C(    52.80), SIMDE_FLOAT32_C(  -436.59), SIMDE_FLOAT32_C(   594.16),
        SIMDE_FLOAT32_C(  -188.64), SIMDE_FLOAT32_C(   278.20), SIMDE_FLOAT32_C(  -842.65), SIMDE_FLOAT32_C(   652.14),
        SIMDE_FLOAT32_C(  -757.74), SIMDE_FLOAT32_C(  -607.83), SIMDE_FLOAT32_C(   601.92), SIMDE_FLOAT32_C(   485.02),
        SIMDE_FLOAT32_C(   232.73), SIMDE_FLOAT32_C(  -392.58), SIMDE_FLOAT32_C(   888.25), SIMDE_FLOAT32_C(  -852.82) },
      { SIMDE_FLOAT32_C(   697.00), SIMDE_FLOAT32_C(   -85.56), SIMDE_FLOAT32_C(  -807.20), SIMDE_FLOAT32_C(   594.00),
        SIMDE_FLOAT32_C(   638.08), SIMDE_FLOAT32_C(   336.09), SIMDE_FLOAT32_C(  -180.10), SIMDE_FLOAT32_C(   709.25),
        SIMDE_FLOAT32_C(   316.96), SIMDE_FLOAT32_C(  -944.76), SIMDE_FLOAT32_C(   568.51), SIMDE_FLOAT32_C(   103.62),
        SIMDE_FLOAT32_C(   758.08), SIMDE_FLOAT32_C(  -138.83), SIMDE_FLOAT32_C(   604.87), SIMDE_FLOAT32_C(   537.64) } },
    { { SIMDE_FLOAT32_C(  -199.78), SIMDE_FLOAT32_C(  -493.96), SIMDE_FLOAT32_C(   785.26), SIMDE_FLOAT32_C(  -863.69),
        SIMDE_FLOAT32_C(   325.94), SIMDE_FLOAT32_C(   494.50), SIMDE_FLOAT32_C(   453.27), SIMDE_FLOAT32_C(   381.18),
        SIMDE_FLOAT32_C(    63.02), SIMDE_FLOAT32_C(  -443.12), SIMDE_FLOAT32_C(   139.26), SIMDE_FLOAT32_C(   924.18),
        SIMDE_FLOAT32_C(  -838.25), SIMDE_FLOAT32_C(  -323.10), SIMDE_FLOAT32_C(  -805.38), SIMDE_FLOAT32_C(   858.57) },
      UINT8_C(245),
      { SIMDE_FLOAT32_C(  -241.97), SIMDE_FLOAT32_C(   452.73), SIMDE_FLOAT32_C(  -458.94), SIMDE_FLOAT32_C(  -963.77),
        SIMDE_FLOAT32_C(   610.08), SIMDE_FLOAT32_C(  -806.80), SIMDE_FLOAT32_C(  -721.51), SIMDE_FLOAT32_C(  -997.75),
        SIMDE_FLOAT32_C(   795.12), SIMDE_FLOAT32_C(   763.51), SIMDE_FLOAT32_C(   234.98), SIMDE_FLOAT32_C(  -597.47),
        SIMDE_FLOAT32_C(   651.76), SIMDE_FLOAT32_C(   382.16), SIMDE_FLOAT32_C(   202.75), SIMDE_FLOAT32_C(  -842.20) },
      { SIMDE_FLOAT32_C(  -242.00), SIMDE_FLOAT32_C(  -493.96), SIMDE_FLOAT32_C(  -459.00), SIMDE_FLOAT32_C(  -863.69),
        SIMDE_FLOAT32_C(   610.00), SIMDE_FLOAT32_C(  -807.00), SIMDE_FLOAT32_C(  -722.00), SIMDE_FLOAT32_C(  -998.00),
        SIMDE_FLOAT32_C(    63.02), SIMDE_FLOAT32_C(  -443.12), SIMDE_FLOAT32_C(   139.26), SIMDE_FLOAT32_C(   924.18),
        SIMDE_FLOAT32_C(  -838.25), SIMDE_FLOAT32_C(  -323.10), SIMDE_FLOAT32_C(  -805.38), SIMDE_FLOAT32_C(   858.57) } },
    { { SIMDE_FLOAT32_C(   167.42), SIMDE_FLOAT32_C(   339.06), SIMDE_FLOAT32_C(   483.74), SIMDE_FLOAT32_C(  -338.08),
        SIMDE_FLOAT32_C(  -207.67), SIMDE_FLOAT32_C(  -135.08), SIMDE_FLOAT32_C(   724.94), SIMDE_FLOAT32_C(   349.21),
        SIMDE_FLOAT32_C(  -995.82), SIMDE_FLOAT32_C(   649.12), SIMDE_FLOAT32_C(   510.96), SIMDE_FLOAT32_C(  -318.92),
        SIMDE_FLOAT32_C(   843.74), SIMDE_FLOAT32_C(   369.53), SIMDE_FLOAT32_C(  -589.22), SIMDE_FLOAT32_C(  -398.24) },
      UINT8_C( 64),
      { SIMDE_FLOAT32_C(   -48.16), SIMDE_FLOAT32_C(  -362.01), SIMDE_FLOAT32_C(  -567.67), SIMDE_FLOAT32_C(   145.04),
        SIMDE_FLOAT32_C(   -83.52), SIMDE_FLOAT32_C(  -565.41), SIMDE_FLOAT32_C(   -59.84), SIMDE_FLOAT32_C(  -320.01),
        SIMDE_FLOAT32_C(   669.57), SIMDE_FLOAT32_C(   342.69), SIMDE_FLOAT32_C(  -668.25), SIMDE_FLOAT32_C(    51.73),
        SIMDE_FLOAT32_C(  -454.56), SIMDE_FLOAT32_C(  -510.45), SIMDE_FLOAT32_C(  -780.86), SIMDE_FLOAT32_C(   884.50) },
      { SIMDE_FLOAT32_C(   167.42), SIMDE_FLOAT32_C(   339.06), SIMDE_FLOAT32_C(   483.74), SIMDE_FLOAT32_C(  -338.08),
        SIMDE_FLOAT32_C(  -207.67), SIMDE_FLOAT32_C(  -135.08), SIMDE_FLOAT32_C(   -60.00), SIMDE_FLOAT32_C(   349.21),
        SIMDE_FLOAT32_C(  -995.82), SIMDE_FLOAT32_C(   649.12), SIMDE_FLOAT32_C(   510.96), SIMDE_FLOAT32_C(  -318.92),
        SIMDE_FLOAT32_C(   843.74), SIMDE_FLOAT32_C(   369.53), SIMDE_FLOAT32_C(  -589.22), SIMDE_FLOAT32_C(  -398.24) } },
    { { SIMDE_FLOAT32_C(   973.29), SIMDE_FLOAT32_C(  -118.94), SIMDE_FLOAT32_C(  -323.17), SIMDE_FLOAT32_C(  -161.78),
        SIMDE_FLOAT32_C(  -394.00), SIMDE_FLOAT32_C(  -973.95), SIMDE_FLOAT32_C(  -157.60), SIMDE_FLOAT32_C(  -744.88),
        SIMDE_FLOAT32_C(   537.01), SIMDE_FLOAT32_C(   523.48), SIMDE_FLOAT32_C(  -901.15), SIMDE_FLOAT32_C(   -93.46),
        SIMDE_FLOAT32_C(   934.26), SIMDE_FLOAT32_C(  -299.38), SIMDE_FLOAT32_C(   728.79), SIMDE_FLOAT32_C(  -113.90) },
      UINT8_C( 86),
      { SIMDE_FLOAT32_C(  -838.87), SIMDE_FLOAT32_C(  -968.86), SIMDE_FLOAT32_C(  -744.90), SIMDE_FLOAT32_C(  -404.28),
        SIMDE_FLOAT32_C(   -28.71), SIMDE_FLOAT32_C(   -64.91), SIMDE_FLOAT32_C(  -734.71), SIMDE_FLOAT32_C(  -686.02),
        SIMDE_FLOAT32_C(   266.84), SIMDE_FLOAT32_C(   317.01), SIMDE_FLOAT32_C(  -140.57), SIMDE_FLOAT32_C(   756.39),
        SIMDE_FLOAT32_C(   536.16), SIMDE_FLOAT32_C(  -256.07), SIMDE_FLOAT32_C(   729.69), SIMDE_FLOAT32_C(  -582.78) },
      { SIMDE_FLOAT32_C(   973.29), SIMDE_FLOAT32_C(  -969.00), SIMDE_FLOAT32_C(  -745.00), SIMDE_FLOAT32_C(  -161.78),
        SIMDE_FLOAT32_C(   -29.00), SIMDE_FLOAT32_C(  -973.95), SIMDE_FLOAT32_C(  -735.00), SIMDE_FLOAT32_C(  -744.88),
        SIMDE_FLOAT32_C(   537.01), SIMDE_FLOAT32_C(   523.48), SIMDE_FLOAT32_C(  -901.15), SIMDE_FLOAT32_C(   -93.46),
        SIMDE_FLOAT32_C(   934.26), SIMDE_FLOAT32_C(  -299.38), SIMDE_FLOAT32_C(   728.79), SIMDE_FLOAT32_C(  -113.90) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_nearbyint_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_nearbyint_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -456.37), SIMDE_FLOAT64_C(   239.71), SIMDE_FLOAT64_C(  -214.46), SIMDE_FLOAT64_C(  -228.66),
        SIMDE_FLOAT64_C(  -452.56), SIMDE_FLOAT64_C(  -734.09), SIMDE_FLOAT64_C(   235.92), SIMDE_FLOAT64_C(   143.86) },
      { SIMDE_FLOAT64_C(  -456.00), SIMDE_FLOAT64_C(   240.00), SIMDE_FLOAT64_C(  -214.00), SIMDE_FLOAT64_C(  -229.00),
        SIMDE_FLOAT64_C(  -453.00), SIMDE_FLOAT64_C(  -734.00), SIMDE_FLOAT64_C(   236.00), SIMDE_FLOAT64_C(   144.00) } },
    { { SIMDE_FLOAT64_C(  -285.31), SIMDE_FLOAT64_C(   -86.71), SIMDE_FLOAT64_C(   920.29), SIMDE_FLOAT64_C(  -690.90),
        SIMDE_FLOAT64_C(  -912.99), SIMDE_FLOAT64_C(  -452.36), SIMDE_FLOAT64_C(  -958.90), SIMDE_FLOAT64_C(  -103.11) },
      { SIMDE_FLOAT64_C(  -285.00), SIMDE_FLOAT64_C(   -87.00), SIMDE_FLOAT64_C(   920.00), SIMDE_FLOAT64_C(  -691.00),
        SIMDE_FLOAT64_C(  -913.00), SIMDE_FLOAT64_C(  -452.00), SIMDE_FLOAT64_C(  -959.00), SIMDE_FLOAT64_C(  -103.00) } },
    { { SIMDE_FLOAT64_C(  -186.33), SIMDE_FLOAT64_C(  -533.97), SIMDE_FLOAT64_C(   740.01), SIMDE_FLOAT64_C(  -835.54),
        SIMDE_FLOAT64_C(   905.55), SIMDE_FLOAT64_C(   918.31), SIMDE_FLOAT64_C(   254.16), SIMDE_FLOAT64_C(  -207.74) },
      { SIMDE_FLOAT64_C(  -186.00), SIMDE_FLOAT64_C(  -534.00), SIMDE_FLOAT64_C(   740.00), SIMDE_FLOAT64_C(  -836.00),
        SIMDE_FLOAT64_C(   906.00), SIMDE_FLOAT64_C(   918.00), SIMDE_FLOAT64_C(   254.00), SIMDE_FLOAT64_C(  -208.00) } },
    { { SIMDE_FLOAT64_C(   -15.89), SIMDE_FLOAT64_C(   697.49), SIMDE_FLOAT64_C(  -777.91), SIMDE_FLOAT64_C(  -743.01),
        SIMDE_FLOAT64_C(   145.93), SIMDE_FLOAT64_C(   408.99), SIMDE_FLOAT64_C(  -288.89), SIMDE_FLOAT64_C(   689.55) },
      { SIMDE_FLOAT64_C(   -16.00), SIMDE_FLOAT64_C(   697.00), SIMDE_FLOAT64_C(  -778.00), SIMDE_FLOAT64_C(  -743.00),
        SIMDE_FLOAT64_C(   146.00), SIMDE_FLOAT64_C(   409.00), SIMDE_FLOAT64_C(  -289.00), SIMDE_FLOAT64_C(   690.00) } },
    { { SIMDE_FLOAT64_C(  -351.30), SIMDE_FLOAT64_C(   496.65), SIMDE_FLOAT64_C(  -539.11), SIMDE_FLOAT64_C(   196.13),
        SIMDE_FLOAT64_C(   762.55), SIMDE_FLOAT64_C(   696.81), SIMDE_FLOAT64_C(  -660.01), SIMDE_FLOAT64_C(  -522.75) },
      { SIMDE_FLOAT64_C(  -351.00), SIMDE_FLOAT64_C(   497.00), SIMDE_FLOAT64_C(  -539.00), SIMDE_FLOAT64_C(   196.00),
        SIMDE_FLOAT64_C(   763.00), SIMDE_FLOAT64_C(   697.00), SIMDE_FLOAT64_C(  -660.00), SIMDE_FLOAT64_C(  -523.00) } },
    { { SIMDE_FLOAT64_C(  -389.90), SIMDE_FLOAT64_C(  -739.72), SIMDE_FLOAT64_C(  -213.65), SIMDE_FLOAT64_C(  -302.89),
        SIMDE_FLOAT64_C(  -192.08), SIMDE_FLOAT64_C(  -172.55), SIMDE_FLOAT64_C(   594.00), SIMDE_FLOAT64_C(   621.59) },
      { SIMDE_FLOAT64_C(  -390.00), SIMDE_FLOAT64_C(  -740.00), SIMDE_FLOAT64_C(  -214.00), SIMDE_FLOAT64_C(  -303.00),
        SIMDE_FLOAT64_C(  -192.00), SIMDE_FLOAT64_C(  -173.00), SIMDE_FLOAT64_C(   594.00), SIMDE_FLOAT64_C(   622.00) } },
    { { SIMDE_FLOAT64_C(   293.48), SIMDE_FLOAT64_C(   334.01), SIMDE_FLOAT64_C(   786.05), SIMDE_FLOAT64_C(   199.03),
        SIMDE_FLOAT64_C(   252.33), SIMDE_FLOAT64_C(    40.22), SIMDE_FLOAT64_C(   991.29), SIMDE_FLOAT64_C(  -763.57) },
      { SIMDE_FLOAT64_C(   293.00), SIMDE_FLOAT64_C(   334.00), SIMDE_FLOAT64_C(   786.00), SIMDE_FLOAT64_C(   199.00),
        SIMDE_FLOAT64_C(   252.00), SIMDE_FLOAT64_C(    40.00), SIMDE_FLOAT64_C(   991.00), SIMDE_FLOAT64_C(  -764.00) } },
    { { SIMDE_FLOAT64_C(  -262.29), SIMDE_FLOAT64_C(  -786.62), SIMDE_FLOAT64_C(  -506.58), SIMDE_FLOAT64_C(   883.63),
        SIMDE_FLOAT64_C(   622.37), SIMDE_FLOAT64_C(   204.53), SIMDE_FLOAT64_C(   573.19), SIMDE_FLOAT64_C(  -728.93) },
      { SIMDE_FLOAT64_C(  -262.00), SIMDE_FLOAT64_C(  -787.00), SIMDE_FLOAT64_C(  -507.00), SIMDE_FLOAT64_C(   884.00),
        SIMDE_FLOAT64_C(   622.00), SIMDE_FLOAT64_C(   205.00), SIMDE_FLOAT64_C(   573.00), SIMDE_FLOAT64_C(  -729.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_nearbyint_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_nearbyint_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
            { { SIMDE_FLOAT64_C(   381.89), SIMDE_FLOAT64_C(  -277.28), SIMDE_FLOAT64_C(  -400.11), SIMDE_FLOAT64_C(  -872.84),
        SIMDE_FLOAT64_C(   -66.17), SIMDE_FLOAT64_C(  -250.32), SIMDE_FLOAT64_C(  -214.38), SIMDE_FLOAT64_C(   965.87) },
      UINT8_C(106),
      { SIMDE_FLOAT64_C(   141.85), SIMDE_FLOAT64_C(    88.38), SIMDE_FLOAT64_C(  -374.04), SIMDE_FLOAT64_C(   906.38),
        SIMDE_FLOAT64_C(   851.98), SIMDE_FLOAT64_C(  -170.13), SIMDE_FLOAT64_C(  -142.10), SIMDE_FLOAT64_C(  -367.42) },
      { SIMDE_FLOAT64_C(   381.89), SIMDE_FLOAT64_C(    88.00), SIMDE_FLOAT64_C(  -400.11), SIMDE_FLOAT64_C(   906.00),
        SIMDE_FLOAT64_C(   -66.17), SIMDE_FLOAT64_C(  -170.00), SIMDE_FLOAT64_C(  -142.00), SIMDE_FLOAT64_C(   965.87) } },
    { { SIMDE_FLOAT64_C(    49.27), SIMDE_FLOAT64_C(   950.21), SIMDE_FLOAT64_C(   214.00), SIMDE_FLOAT64_C(   575.74),
        SIMDE_FLOAT64_C(  -350.82), SIMDE_FLOAT64_C(   512.95), SIMDE_FLOAT64_C(  -227.13), SIMDE_FLOAT64_C(  -609.67) },
      UINT8_C( 61),
      { SIMDE_FLOAT64_C(   586.44), SIMDE_FLOAT64_C(   381.99), SIMDE_FLOAT64_C(   608.18), SIMDE_FLOAT64_C(   184.92),
        SIMDE_FLOAT64_C(  -474.55), SIMDE_FLOAT64_C(    -9.93), SIMDE_FLOAT64_C(   907.64), SIMDE_FLOAT64_C(   125.34) },
      { SIMDE_FLOAT64_C(   586.00), SIMDE_FLOAT64_C(   950.21), SIMDE_FLOAT64_C(   608.00), SIMDE_FLOAT64_C(   185.00),
        SIMDE_FLOAT64_C(  -475.00), SIMDE_FLOAT64_C(   -10.00), SIMDE_FLOAT64_C(  -227.13), SIMDE_FLOAT64_C(  -609.67) } },
    { { SIMDE_FLOAT64_C(   117.23), SIMDE_FLOAT64_C(  -158.52), SIMDE_FLOAT64_C(   875.02), SIMDE_FLOAT64_C(   902.85),
        SIMDE_FLOAT64_C(  -192.66), SIMDE_FLOAT64_C(  -256.64), SIMDE_FLOAT64_C(    44.70), SIMDE_FLOAT64_C(   895.72) },
      UINT8_C(180),
      { SIMDE_FLOAT64_C(   -48.92), SIMDE_FLOAT64_C(   747.70), SIMDE_FLOAT64_C(  -800.80), SIMDE_FLOAT64_C(   808.98),
        SIMDE_FLOAT64_C(  -619.73), SIMDE_FLOAT64_C(   248.47), SIMDE_FLOAT64_C(   759.18), SIMDE_FLOAT64_C(   594.28) },
      { SIMDE_FLOAT64_C(   117.23), SIMDE_FLOAT64_C(  -158.52), SIMDE_FLOAT64_C(  -801.00), SIMDE_FLOAT64_C(   902.85),
        SIMDE_FLOAT64_C(  -620.00), SIMDE_FLOAT64_C(   248.00), SIMDE_FLOAT64_C(    44.70), SIMDE_FLOAT64_C(   594.00) } },
    { { SIMDE_FLOAT64_C(  -175.78), SIMDE_FLOAT64_C(  -591.64), SIMDE_FLOAT64_C(   107.22), SIMDE_FLOAT64_C(   597.09),
        SIMDE_FLOAT64_C(  -201.31), SIMDE_FLOAT64_C(  -742.21), SIMDE_FLOAT64_C(   183.53), SIMDE_FLOAT64_C(  -819.31) },
      UINT8_C(241),
      { SIMDE_FLOAT64_C(  -631.55), SIMDE_FLOAT64_C(  -293.87), SIMDE_FLOAT64_C(  -143.96), SIMDE_FLOAT64_C(  -723.91),
        SIMDE_FLOAT64_C(   831.47), SIMDE_FLOAT64_C(   973.27), SIMDE_FLOAT64_C(   117.57), SIMDE_FLOAT64_C(   706.49) },
      { SIMDE_FLOAT64_C(  -632.00), SIMDE_FLOAT64_C(  -591.64), SIMDE_FLOAT64_C(   107.22), SIMDE_FLOAT64_C(   597.09),
        SIMDE_FLOAT64_C(   831.00), SIMDE_FLOAT64_C(   973.00), SIMDE_FLOAT64_C(   118.00), SIMDE_FLOAT64_C(   706.00) } },
    { { SIMDE_FLOAT64_C(   876.13), SIMDE_FLOAT64_C(   924.91), SIMDE_FLOAT64_C(  -550.14), SIMDE_FLOAT64_C(   -79.17),
        SIMDE_FLOAT64_C(   820.63), SIMDE_FLOAT64_C(   819.19), SIMDE_FLOAT64_C(   871.91), SIMDE_FLOAT64_C(   568.33) },
      UINT8_C(250),
      { SIMDE_FLOAT64_C(   680.89), SIMDE_FLOAT64_C(   948.60), SIMDE_FLOAT64_C(   266.86), SIMDE_FLOAT64_C(   440.07),
        SIMDE_FLOAT64_C(   542.88), SIMDE_FLOAT64_C(  -908.92), SIMDE_FLOAT64_C(   848.43), SIMDE_FLOAT64_C(  -349.90) },
      { SIMDE_FLOAT64_C(   876.13), SIMDE_FLOAT64_C(   949.00), SIMDE_FLOAT64_C(  -550.14), SIMDE_FLOAT64_C(   440.00),
        SIMDE_FLOAT64_C(   543.00), SIMDE_FLOAT64_C(  -909.00), SIMDE_FLOAT64_C(   848.00), SIMDE_FLOAT64_C(  -350.00) } },
    { { SIMDE_FLOAT64_C(   688.16), SIMDE_FLOAT64_C(  -352.87), SIMDE_FLOAT64_C(   -92.11), SIMDE_FLOAT64_C(  -128.31),
        SIMDE_FLOAT64_C(  -172.19), SIMDE_FLOAT64_C(  -226.14), SIMDE_FLOAT64_C(   240.14), SIMDE_FLOAT64_C(   533.94) },
      UINT8_C( 61),
      { SIMDE_FLOAT64_C(   516.23), SIMDE_FLOAT64_C(   365.42), SIMDE_FLOAT64_C(   603.18), SIMDE_FLOAT64_C(  -366.20),
        SIMDE_FLOAT64_C(    71.91), SIMDE_FLOAT64_C(   479.30), SIMDE_FLOAT64_C(  -441.29), SIMDE_FLOAT64_C(   521.77) },
      { SIMDE_FLOAT64_C(   516.00), SIMDE_FLOAT64_C(  -352.87), SIMDE_FLOAT64_C(   603.00), SIMDE_FLOAT64_C(  -366.00),
        SIMDE_FLOAT64_C(    72.00), SIMDE_FLOAT64_C(   479.00), SIMDE_FLOAT64_C(   240.14), SIMDE_FLOAT64_C(   533.94) } },
    { { SIMDE_FLOAT64_C(  -599.87), SIMDE_FLOAT64_C(  -620.66), SIMDE_FLOAT64_C(   340.95), SIMDE_FLOAT64_C(  -727.96),
        SIMDE_FLOAT64_C(   947.67), SIMDE_FLOAT64_C(   359.34), SIMDE_FLOAT64_C(   952.92), SIMDE_FLOAT64_C(   896.27) },
      UINT8_C( 22),
      { SIMDE_FLOAT64_C(   392.99), SIMDE_FLOAT64_C(   439.14), SIMDE_FLOAT64_C(  -282.72), SIMDE_FLOAT64_C(   241.43),
        SIMDE_FLOAT64_C(  -910.76), SIMDE_FLOAT64_C(  -594.56), SIMDE_FLOAT64_C(   888.55), SIMDE_FLOAT64_C(    -2.87) },
      { SIMDE_FLOAT64_C(  -599.87), SIMDE_FLOAT64_C(   439.00), SIMDE_FLOAT64_C(  -283.00), SIMDE_FLOAT64_C(  -727.96),
        SIMDE_FLOAT64_C(  -911.00), SIMDE_FLOAT64_C(   359.34), SIMDE_FLOAT64_C(   952.92), SIMDE_FLOAT64_C(   896.27) } },
    { { SIMDE_FLOAT64_C(   277.14), SIMDE_FLOAT64_C(  -283.64), SIMDE_FLOAT64_C(   770.99), SIMDE_FLOAT64_C(  -482.72),
        SIMDE_FLOAT64_C(  -749.69), SIMDE_FLOAT64_C(   400.90), SIMDE_FLOAT64_C(  -966.49), SIMDE_FLOAT64_C(   615.72) },
      UINT8_C(173),
      { SIMDE_FLOAT64_C(  -332.68), SIMDE_FLOAT64_C(  -312.37), SIMDE_FLOAT64_C(  -516.63), SIMDE_FLOAT64_C(   226.03),
        SIMDE_FLOAT64_C(  -790.60), SIMDE_FLOAT64_C(  -116.50), SIMDE_FLOAT64_C(   605.37), SIMDE_FLOAT64_C(   550.35) },
      { SIMDE_FLOAT64_C(  -333.00), SIMDE_FLOAT64_C(  -283.64), SIMDE_FLOAT64_C(  -517.00), SIMDE_FLOAT64_C(   226.00),
        SIMDE_FLOAT64_C(  -749.69), SIMDE_FLOAT64_C(  -116.00), SIMDE_FLOAT64_C(  -966.49), SIMDE_FLOAT64_C(   550.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_nearbyint_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_pow_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 b[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     3.20), SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     1.48) },
      { SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     3.01), SIMDE_FLOAT32_C(     3.83) },
      { SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(     2.30), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     4.49) } },
    { { SIMDE_FLOAT32_C(     4.49), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(     4.20), SIMDE_FLOAT32_C(     3.10) },
      { SIMDE_FLOAT32_C(     2.65), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     1.17) },
      { SIMDE_FLOAT32_C(    53.51), SIMDE_FLOAT32_C(     1.23), SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(     3.76) } },
    { { SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     4.91), SIMDE_FLOAT32_C(     4.05), SIMDE_FLOAT32_C(     0.12) },
      { SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     2.91), SIMDE_FLOAT32_C(     4.46) },
      { SIMDE_FLOAT32_C(     4.61), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    58.57), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(     3.77), SIMDE_FLOAT32_C(     3.76), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     2.56) },
      { SIMDE_FLOAT32_C(     1.47), SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     4.67) },
      { SIMDE_FLOAT32_C(     7.03), SIMDE_FLOAT32_C(    89.10), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    80.63) } },
    { { SIMDE_FLOAT32_C(     2.81), SIMDE_FLOAT32_C(     4.23), SIMDE_FLOAT32_C(     1.15), SIMDE_FLOAT32_C(     3.31) },
      { SIMDE_FLOAT32_C(     4.79), SIMDE_FLOAT32_C(     4.15), SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     4.28) },
      { SIMDE_FLOAT32_C(   141.03), SIMDE_FLOAT32_C(   397.48), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(   167.83) } },
    { { SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(     3.04) },
      { SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     2.43), SIMDE_FLOAT32_C(     4.21), SIMDE_FLOAT32_C(     0.52) },
      { SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     2.04), SIMDE_FLOAT32_C(    38.49), SIMDE_FLOAT32_C(     1.78) } },
    { { SIMDE_FLOAT32_C(     2.34), SIMDE_FLOAT32_C(     3.26), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     3.65) },
      { SIMDE_FLOAT32_C(     3.26), SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(     2.03) },
      { SIMDE_FLOAT32_C(    15.98), SIMDE_FLOAT32_C(    66.36), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    13.85) } },
    { { SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     3.31), SIMDE_FLOAT32_C(     4.59), SIMDE_FLOAT32_C(     3.78) },
      { SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(     3.45), SIMDE_FLOAT32_C(     4.50) },
      { SIMDE_FLOAT32_C(     4.12), SIMDE_FLOAT32_C(    31.41), SIMDE_FLOAT32_C(   191.98), SIMDE_FLOAT32_C(   396.93) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 b = simde_mm_loadu_ps(test_vec[i].b);
    simde__m128 r = simde_mm_pow_ps(a, b);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_pow_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 b[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     6.86), SIMDE_FLOAT64_C(     4.28) },
      { SIMDE_FLOAT64_C(     2.99), SIMDE_FLOAT64_C(     7.45) },
      { SIMDE_FLOAT64_C(   316.67), SIMDE_FLOAT64_C( 50612.30) } },
    { { SIMDE_FLOAT64_C(     7.72), SIMDE_FLOAT64_C(     8.36) },
      { SIMDE_FLOAT64_C(     4.17), SIMDE_FLOAT64_C(     1.82) },
      { SIMDE_FLOAT64_C(  5027.64), SIMDE_FLOAT64_C(    47.69) } },
    { { SIMDE_FLOAT64_C(     9.11), SIMDE_FLOAT64_C(     6.23) },
      { SIMDE_FLOAT64_C(     1.26), SIMDE_FLOAT64_C(     4.65) },
      { SIMDE_FLOAT64_C(    16.18), SIMDE_FLOAT64_C(  4947.31) } },
    { { SIMDE_FLOAT64_C(     2.75), SIMDE_FLOAT64_C(     7.48) },
      { SIMDE_FLOAT64_C(     0.85), SIMDE_FLOAT64_C(     0.71) },
      { SIMDE_FLOAT64_C(     2.36), SIMDE_FLOAT64_C(     4.17) } },
    { { SIMDE_FLOAT64_C(     5.91), SIMDE_FLOAT64_C(     7.19) },
      { SIMDE_FLOAT64_C(     1.19), SIMDE_FLOAT64_C(     5.92) },
      { SIMDE_FLOAT64_C(     8.28), SIMDE_FLOAT64_C(117987.24) } },
    { { SIMDE_FLOAT64_C(     5.42), SIMDE_FLOAT64_C(     3.06) },
      { SIMDE_FLOAT64_C(     9.46), SIMDE_FLOAT64_C(     0.23) },
      { SIMDE_FLOAT64_C(8782805.21), SIMDE_FLOAT64_C(     1.29) } },
    { { SIMDE_FLOAT64_C(     6.88), SIMDE_FLOAT64_C(     9.69) },
      { SIMDE_FLOAT64_C(     2.44), SIMDE_FLOAT64_C(     7.03) },
      { SIMDE_FLOAT64_C(   110.59), SIMDE_FLOAT64_C(8587290.46) } },
    { { SIMDE_FLOAT64_C(     9.85), SIMDE_FLOAT64_C(     1.85) },
      { SIMDE_FLOAT64_C(     1.77), SIMDE_FLOAT64_C(     6.71) },
      { SIMDE_FLOAT64_C(    57.33), SIMDE_FLOAT64_C(    62.05) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d b = simde_mm_loadu_pd(test_vec[i].b);
    simde__m128d r = simde_mm_pow_pd(a, b);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_pow_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 b[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     4.47), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     4.19), SIMDE_FLOAT32_C(     4.26) },
      { SIMDE_FLOAT32_C(     4.92), SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     2.15),
        SIMDE_FLOAT32_C(     3.59), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     3.42) },
      { SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(   328.62), SIMDE_FLOAT32_C(     0.55),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(   228.13), SIMDE_FLOAT32_C(   142.10) } },
    { { SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     4.06), SIMDE_FLOAT32_C(     2.24), SIMDE_FLOAT32_C(     3.04),
        SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(     2.02) },
      { SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     3.82), SIMDE_FLOAT32_C(     0.42),
        SIMDE_FLOAT32_C(     3.50), SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     4.31) },
      { SIMDE_FLOAT32_C(     2.26), SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(    21.77), SIMDE_FLOAT32_C(     1.60),
        SIMDE_FLOAT32_C(     1.15), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(    20.70) } },
    { { SIMDE_FLOAT32_C(     2.29), SIMDE_FLOAT32_C(     4.91), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     2.94),
        SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     4.26), SIMDE_FLOAT32_C(     2.20), SIMDE_FLOAT32_C(     0.66) },
      { SIMDE_FLOAT32_C(     1.59), SIMDE_FLOAT32_C(     1.07), SIMDE_FLOAT32_C(     2.81), SIMDE_FLOAT32_C(     0.18),
        SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     0.55) },
      { SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(     5.49), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.21),
        SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    10.16), SIMDE_FLOAT32_C(    17.22), SIMDE_FLOAT32_C(     0.80) } },
    { { SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     3.59), SIMDE_FLOAT32_C(     1.70),
        SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     1.70) },
      { SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     2.12), SIMDE_FLOAT32_C(     4.04),
        SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     3.35), SIMDE_FLOAT32_C(     1.21) },
      { SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    15.02), SIMDE_FLOAT32_C(     8.53),
        SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    81.53), SIMDE_FLOAT32_C(     1.90) } },
    { { SIMDE_FLOAT32_C(     2.46), SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     4.15), SIMDE_FLOAT32_C(     3.21),
        SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     4.27) },
      { SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     1.68), SIMDE_FLOAT32_C(     4.45), SIMDE_FLOAT32_C(     4.25),
        SIMDE_FLOAT32_C(     3.28), SIMDE_FLOAT32_C(     3.06), SIMDE_FLOAT32_C(     4.80), SIMDE_FLOAT32_C(     3.94) },
      { SIMDE_FLOAT32_C(     8.83), SIMDE_FLOAT32_C(     7.89), SIMDE_FLOAT32_C(   562.75), SIMDE_FLOAT32_C(   142.12),
        SIMDE_FLOAT32_C(    25.37), SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(   662.24), SIMDE_FLOAT32_C(   304.71) } },
    { { SIMDE_FLOAT32_C(     3.90), SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     4.98),
        SIMDE_FLOAT32_C(     3.46), SIMDE_FLOAT32_C(     4.35), SIMDE_FLOAT32_C(     1.68), SIMDE_FLOAT32_C(     3.99) },
      { SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     0.81),
        SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     3.82) },
      { SIMDE_FLOAT32_C(    13.10), SIMDE_FLOAT32_C(   103.46), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     3.67),
        SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(     7.49), SIMDE_FLOAT32_C(     2.85), SIMDE_FLOAT32_C(   197.57) } },
    { { SIMDE_FLOAT32_C(     4.79), SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     2.03), SIMDE_FLOAT32_C(     2.47),
        SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     4.95) },
      { SIMDE_FLOAT32_C(     2.58), SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     4.20), SIMDE_FLOAT32_C(     0.86),
        SIMDE_FLOAT32_C(     4.24), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     4.80), SIMDE_FLOAT32_C(     3.14) },
      { SIMDE_FLOAT32_C(    56.92), SIMDE_FLOAT32_C(     1.22), SIMDE_FLOAT32_C(    19.57), SIMDE_FLOAT32_C(     2.18),
        SIMDE_FLOAT32_C(    51.20), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    13.89), SIMDE_FLOAT32_C(   151.73) } },
    { { SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     3.12), SIMDE_FLOAT32_C(     0.84),
        SIMDE_FLOAT32_C(     4.79), SIMDE_FLOAT32_C(     4.80), SIMDE_FLOAT32_C(     4.84), SIMDE_FLOAT32_C(     1.67) },
      { SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     2.87), SIMDE_FLOAT32_C(     2.48), SIMDE_FLOAT32_C(     4.96),
        SIMDE_FLOAT32_C(     4.24), SIMDE_FLOAT32_C(     4.50), SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     4.03) },
      { SIMDE_FLOAT32_C(    23.23), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(    16.81), SIMDE_FLOAT32_C(     0.42),
        SIMDE_FLOAT32_C(   766.69), SIMDE_FLOAT32_C(  1163.02), SIMDE_FLOAT32_C(   394.06), SIMDE_FLOAT32_C(     7.90) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 b = simde_mm256_loadu_ps(test_vec[i].b);
    simde__m256 r = simde_mm256_pow_ps(a, b);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_pow_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 b[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     7.17), SIMDE_FLOAT64_C(     4.56), SIMDE_FLOAT64_C(     5.81), SIMDE_FLOAT64_C(     1.86) },
      { SIMDE_FLOAT64_C(     7.20), SIMDE_FLOAT64_C(     2.88), SIMDE_FLOAT64_C(     6.56), SIMDE_FLOAT64_C(     0.87) },
      { SIMDE_FLOAT64_C(1444567.77), SIMDE_FLOAT64_C(    79.03), SIMDE_FLOAT64_C(103037.53), SIMDE_FLOAT64_C(     1.72) } },
    { { SIMDE_FLOAT64_C(     6.39), SIMDE_FLOAT64_C(     1.20), SIMDE_FLOAT64_C(     4.73), SIMDE_FLOAT64_C(     0.14) },
      { SIMDE_FLOAT64_C(     9.00), SIMDE_FLOAT64_C(     7.96), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.99) },
      { SIMDE_FLOAT64_C(17762648.57), SIMDE_FLOAT64_C(     4.27), SIMDE_FLOAT64_C(     1.17), SIMDE_FLOAT64_C(     0.14) } },
    { { SIMDE_FLOAT64_C(     1.05), SIMDE_FLOAT64_C(     6.55), SIMDE_FLOAT64_C(     5.85), SIMDE_FLOAT64_C(     2.38) },
      { SIMDE_FLOAT64_C(     7.70), SIMDE_FLOAT64_C(     1.92), SIMDE_FLOAT64_C(     2.76), SIMDE_FLOAT64_C(     9.17) },
      { SIMDE_FLOAT64_C(     1.46), SIMDE_FLOAT64_C(    36.91), SIMDE_FLOAT64_C(   131.02), SIMDE_FLOAT64_C(  2839.30) } },
    { { SIMDE_FLOAT64_C(     1.22), SIMDE_FLOAT64_C(     3.47), SIMDE_FLOAT64_C(     2.69), SIMDE_FLOAT64_C(     4.53) },
      { SIMDE_FLOAT64_C(     8.94), SIMDE_FLOAT64_C(     7.35), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     6.10) },
      { SIMDE_FLOAT64_C(     5.92), SIMDE_FLOAT64_C(  9363.14), SIMDE_FLOAT64_C(     1.94), SIMDE_FLOAT64_C( 10050.76) } },
    { { SIMDE_FLOAT64_C(     1.91), SIMDE_FLOAT64_C(     6.48), SIMDE_FLOAT64_C(     7.96), SIMDE_FLOAT64_C(     9.11) },
      { SIMDE_FLOAT64_C(     9.36), SIMDE_FLOAT64_C(     4.52), SIMDE_FLOAT64_C(     9.98), SIMDE_FLOAT64_C(     5.75) },
      { SIMDE_FLOAT64_C(   427.04), SIMDE_FLOAT64_C(  4659.28), SIMDE_FLOAT64_C(979743556.72), SIMDE_FLOAT64_C(329026.34) } },
    { { SIMDE_FLOAT64_C(     5.73), SIMDE_FLOAT64_C(     4.71), SIMDE_FLOAT64_C(     5.89), SIMDE_FLOAT64_C(     4.73) },
      { SIMDE_FLOAT64_C(     2.67), SIMDE_FLOAT64_C(     5.99), SIMDE_FLOAT64_C(     5.71), SIMDE_FLOAT64_C(     3.72) },
      { SIMDE_FLOAT64_C(   105.75), SIMDE_FLOAT64_C( 10749.67), SIMDE_FLOAT64_C( 24966.54), SIMDE_FLOAT64_C(   323.95) } },
    { { SIMDE_FLOAT64_C(     2.54), SIMDE_FLOAT64_C(     1.56), SIMDE_FLOAT64_C(     6.10), SIMDE_FLOAT64_C(     0.24) },
      { SIMDE_FLOAT64_C(     3.48), SIMDE_FLOAT64_C(     8.87), SIMDE_FLOAT64_C(     9.41), SIMDE_FLOAT64_C(     4.71) },
      { SIMDE_FLOAT64_C(    25.63), SIMDE_FLOAT64_C(    51.64), SIMDE_FLOAT64_C(24544475.02), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(     2.33), SIMDE_FLOAT64_C(     2.10), SIMDE_FLOAT64_C(     9.23), SIMDE_FLOAT64_C(     1.27) },
      { SIMDE_FLOAT64_C(     9.45), SIMDE_FLOAT64_C(     9.90), SIMDE_FLOAT64_C(     7.37), SIMDE_FLOAT64_C(     1.37) },
      { SIMDE_FLOAT64_C(  2961.51), SIMDE_FLOAT64_C(  1548.71), SIMDE_FLOAT64_C(12987828.24), SIMDE_FLOAT64_C(     1.39) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d b = simde_mm256_loadu_pd(test_vec[i].b);
    simde__m256d r = simde_mm256_pow_pd(a, b);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_pow_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 b[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     2.61), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(     3.22),
        SIMDE_FLOAT32_C(     4.10), SIMDE_FLOAT32_C(     4.95), SIMDE_FLOAT32_C(     4.92), SIMDE_FLOAT32_C(     4.08),
        SIMDE_FLOAT32_C(     4.52), SIMDE_FLOAT32_C(     2.15), SIMDE_FLOAT32_C(     4.72), SIMDE_FLOAT32_C(     1.20),
        SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     4.45), SIMDE_FLOAT32_C(     4.31), SIMDE_FLOAT32_C(     1.66) },
      { SIMDE_FLOAT32_C(     4.65), SIMDE_FLOAT32_C(     2.28), SIMDE_FLOAT32_C(     1.70), SIMDE_FLOAT32_C(     3.82),
        SIMDE_FLOAT32_C(     1.78), SIMDE_FLOAT32_C(     2.73), SIMDE_FLOAT32_C(     1.38), SIMDE_FLOAT32_C(     3.44),
        SIMDE_FLOAT32_C(     1.09), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     3.09), SIMDE_FLOAT32_C(     3.82),
        SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     1.67), SIMDE_FLOAT32_C(     5.00) },
      { SIMDE_FLOAT32_C(    86.57), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     9.25), SIMDE_FLOAT32_C(    87.10),
        SIMDE_FLOAT32_C(    12.32), SIMDE_FLOAT32_C(    78.75), SIMDE_FLOAT32_C(     9.01), SIMDE_FLOAT32_C(   126.09),
        SIMDE_FLOAT32_C(     5.18), SIMDE_FLOAT32_C(     3.82), SIMDE_FLOAT32_C(   120.92), SIMDE_FLOAT32_C(     2.01),
        SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    35.45), SIMDE_FLOAT32_C(    11.47), SIMDE_FLOAT32_C(    12.60) } },
    { { SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(     2.57),
        SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     3.14), SIMDE_FLOAT32_C(     1.64), SIMDE_FLOAT32_C(     4.84),
        SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     1.08),
        SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     2.74), SIMDE_FLOAT32_C(     0.46) },
      { SIMDE_FLOAT32_C(     2.63), SIMDE_FLOAT32_C(     4.43), SIMDE_FLOAT32_C(     4.28), SIMDE_FLOAT32_C(     4.41),
        SIMDE_FLOAT32_C(     2.17), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     2.85), SIMDE_FLOAT32_C(     3.26),
        SIMDE_FLOAT32_C(     2.41), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     2.08), SIMDE_FLOAT32_C(     4.80),
        SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     3.75), SIMDE_FLOAT32_C(     4.79), SIMDE_FLOAT32_C(     1.80) },
      { SIMDE_FLOAT32_C(    26.37), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(   149.15), SIMDE_FLOAT32_C(    64.24),
        SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     2.13), SIMDE_FLOAT32_C(     4.10), SIMDE_FLOAT32_C(   170.84),
        SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     1.34), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     1.45),
        SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(   124.98), SIMDE_FLOAT32_C(     0.25) } },
    { { SIMDE_FLOAT32_C(     4.12), SIMDE_FLOAT32_C(     3.01), SIMDE_FLOAT32_C(     4.36), SIMDE_FLOAT32_C(     4.44),
        SIMDE_FLOAT32_C(     1.15), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     4.28), SIMDE_FLOAT32_C(     1.43),
        SIMDE_FLOAT32_C(     2.36), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(     3.17),
        SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     3.64), SIMDE_FLOAT32_C(     3.30) },
      { SIMDE_FLOAT32_C(     4.68), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     2.71), SIMDE_FLOAT32_C(     1.85),
        SIMDE_FLOAT32_C(     3.57), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.98),
        SIMDE_FLOAT32_C(     1.49), SIMDE_FLOAT32_C(     2.19), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     4.82),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     1.62), SIMDE_FLOAT32_C(     0.05) },
      { SIMDE_FLOAT32_C(   754.60), SIMDE_FLOAT32_C(    24.97), SIMDE_FLOAT32_C(    54.08), SIMDE_FLOAT32_C(    15.76),
        SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.17), SIMDE_FLOAT32_C(     1.42),
        SIMDE_FLOAT32_C(     3.59), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(   260.08),
        SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     8.11), SIMDE_FLOAT32_C(     1.06) } },
    { { SIMDE_FLOAT32_C(     3.58), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     4.49), SIMDE_FLOAT32_C(     4.73),
        SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     3.77), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     4.35),
        SIMDE_FLOAT32_C(     4.09), SIMDE_FLOAT32_C(     3.67), SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(     4.75),
        SIMDE_FLOAT32_C(     3.91), SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     3.05), SIMDE_FLOAT32_C(     3.59) },
      { SIMDE_FLOAT32_C(     4.07), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     2.64),
        SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     3.62), SIMDE_FLOAT32_C(     2.81),
        SIMDE_FLOAT32_C(     2.73), SIMDE_FLOAT32_C(     4.40), SIMDE_FLOAT32_C(     2.63), SIMDE_FLOAT32_C(     3.67),
        SIMDE_FLOAT32_C(     4.97), SIMDE_FLOAT32_C(     4.25), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     3.55) },
      { SIMDE_FLOAT32_C(   179.60), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(    60.48),
        SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(     1.71), SIMDE_FLOAT32_C(    62.25),
        SIMDE_FLOAT32_C(    46.77), SIMDE_FLOAT32_C(   305.16), SIMDE_FLOAT32_C(    11.37), SIMDE_FLOAT32_C(   304.41),
        SIMDE_FLOAT32_C(   877.24), SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(    63.33), SIMDE_FLOAT32_C(    93.45) } },
    { { SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     2.21),
        SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     4.45), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     1.07),
        SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(     4.08), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     2.03),
        SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     4.31) },
      { SIMDE_FLOAT32_C(     4.63), SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     0.94),
        SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     3.75), SIMDE_FLOAT32_C(     4.34),
        SIMDE_FLOAT32_C(     4.98), SIMDE_FLOAT32_C(     1.38), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     4.95),
        SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     3.50), SIMDE_FLOAT32_C(     0.85) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     3.44), SIMDE_FLOAT32_C(    10.32), SIMDE_FLOAT32_C(     2.11),
        SIMDE_FLOAT32_C(     2.98), SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(     5.30), SIMDE_FLOAT32_C(     1.34),
        SIMDE_FLOAT32_C(   284.41), SIMDE_FLOAT32_C(     6.96), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    33.27),
        SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(    10.39), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     3.46) } },
    { { SIMDE_FLOAT32_C(     4.94), SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(     3.06), SIMDE_FLOAT32_C(     1.92),
        SIMDE_FLOAT32_C(     1.23), SIMDE_FLOAT32_C(     4.63), SIMDE_FLOAT32_C(     2.99), SIMDE_FLOAT32_C(     4.35),
        SIMDE_FLOAT32_C(     3.71), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     1.38), SIMDE_FLOAT32_C(     3.95),
        SIMDE_FLOAT32_C(     2.68), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     3.26), SIMDE_FLOAT32_C(     2.31) },
      { SIMDE_FLOAT32_C(     3.05), SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     3.25), SIMDE_FLOAT32_C(     4.65),
        SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(     0.78),
        SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     4.00),
        SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     4.23), SIMDE_FLOAT32_C(     4.85), SIMDE_FLOAT32_C(     3.66) },
      { SIMDE_FLOAT32_C(   130.58), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(    37.90), SIMDE_FLOAT32_C(    20.77),
        SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(    21.44), SIMDE_FLOAT32_C(    79.05), SIMDE_FLOAT32_C(     3.15),
        SIMDE_FLOAT32_C(    84.04), SIMDE_FLOAT32_C(    14.32), SIMDE_FLOAT32_C(     1.27), SIMDE_FLOAT32_C(   243.44),
        SIMDE_FLOAT32_C(    39.14), SIMDE_FLOAT32_C(    18.37), SIMDE_FLOAT32_C(   308.39), SIMDE_FLOAT32_C(    21.42) } },
    { { SIMDE_FLOAT32_C(     1.02), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     2.25),
        SIMDE_FLOAT32_C(     2.54), SIMDE_FLOAT32_C(     3.57), SIMDE_FLOAT32_C(     1.60), SIMDE_FLOAT32_C(     1.25),
        SIMDE_FLOAT32_C(     2.37), SIMDE_FLOAT32_C(     2.98), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     4.97), SIMDE_FLOAT32_C(     3.46), SIMDE_FLOAT32_C(     2.36), SIMDE_FLOAT32_C(     3.02) },
      { SIMDE_FLOAT32_C(     3.68), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     2.67), SIMDE_FLOAT32_C(     4.48),
        SIMDE_FLOAT32_C(     2.62), SIMDE_FLOAT32_C(     1.66), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     3.65), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     2.37),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     4.85), SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C(     1.23) },
      { SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     1.94), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(    37.82),
        SIMDE_FLOAT32_C(    11.50), SIMDE_FLOAT32_C(     8.27), SIMDE_FLOAT32_C(     1.12), SIMDE_FLOAT32_C(     1.25),
        SIMDE_FLOAT32_C(    23.33), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     1.40), SIMDE_FLOAT32_C(   411.64), SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     3.89) } },
    { { SIMDE_FLOAT32_C(     2.77), SIMDE_FLOAT32_C(     1.62), SIMDE_FLOAT32_C(     3.48), SIMDE_FLOAT32_C(     0.31),
        SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(     2.56),
        SIMDE_FLOAT32_C(     3.06), SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(     2.61), SIMDE_FLOAT32_C(     3.03),
        SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     4.97), SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     3.90) },
      { SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     3.72), SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(     3.21),
        SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     3.63), SIMDE_FLOAT32_C(     4.20), SIMDE_FLOAT32_C(     4.03),
        SIMDE_FLOAT32_C(     4.61), SIMDE_FLOAT32_C(     4.20), SIMDE_FLOAT32_C(     1.41), SIMDE_FLOAT32_C(     4.82),
        SIMDE_FLOAT32_C(     4.05), SIMDE_FLOAT32_C(     2.44), SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     1.82) },
      { SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(     6.02), SIMDE_FLOAT32_C(    67.69), SIMDE_FLOAT32_C(     0.02),
        SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     6.47), SIMDE_FLOAT32_C(    44.18),
        SIMDE_FLOAT32_C(   173.45), SIMDE_FLOAT32_C(    10.74), SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(   209.20),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    50.02), SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(    11.91) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 b = simde_mm512_loadu_ps(test_vec[i].b);
    simde__m512 r = simde_mm512_pow_ps(a, b);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_pow_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 b[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     3.50), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.87), SIMDE_FLOAT32_C(     2.52),
        SIMDE_FLOAT32_C(     3.74), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     4.86),
        SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(     3.89),
        SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     3.85) },
      UINT8_C( 81),
      { SIMDE_FLOAT32_C(     3.43), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     0.44),
        SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(     3.30), SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     3.99),
        SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(     2.70), SIMDE_FLOAT32_C(     1.54),
        SIMDE_FLOAT32_C(     2.63), SIMDE_FLOAT32_C(     4.93), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     3.48) },
      { SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(     2.57), SIMDE_FLOAT32_C(     2.22), SIMDE_FLOAT32_C(     1.87),
        SIMDE_FLOAT32_C(     3.53), SIMDE_FLOAT32_C(     2.08), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     4.18),
        SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(     4.38), SIMDE_FLOAT32_C(     2.79),
        SIMDE_FLOAT32_C(     2.71), SIMDE_FLOAT32_C(     3.23), SIMDE_FLOAT32_C(     4.72), SIMDE_FLOAT32_C(     1.13) },
      { SIMDE_FLOAT32_C(     9.08), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     1.87), SIMDE_FLOAT32_C(     2.52),
        SIMDE_FLOAT32_C(    73.63), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(    46.47), SIMDE_FLOAT32_C(     4.86),
        SIMDE_FLOAT32_C(     1.16), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(     3.89),
        SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     3.79), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     3.85) } },
    { { SIMDE_FLOAT32_C(     3.40), SIMDE_FLOAT32_C(     2.28), SIMDE_FLOAT32_C(     1.57), SIMDE_FLOAT32_C(     1.78),
        SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     3.46),
        SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     3.46), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     4.59),
        SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     0.19) },
      UINT8_C(140),
      { SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     2.05), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     2.37),
        SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(     4.70), SIMDE_FLOAT32_C(     4.16), SIMDE_FLOAT32_C(     4.71), SIMDE_FLOAT32_C(     2.93),
        SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     1.15) },
      { SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     2.54),
        SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     4.51), SIMDE_FLOAT32_C(     2.34),
        SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     4.10), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.24),
        SIMDE_FLOAT32_C(     2.17), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     2.85), SIMDE_FLOAT32_C(     2.46) },
      { SIMDE_FLOAT32_C(     3.40), SIMDE_FLOAT32_C(     2.28), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(     8.95),
        SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     5.06),
        SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     3.46), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     4.59),
        SIMDE_FLOAT32_C(     3.39), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     0.19) } },
    { { SIMDE_FLOAT32_C(     2.97), SIMDE_FLOAT32_C(     3.99), SIMDE_FLOAT32_C(     4.84), SIMDE_FLOAT32_C(     3.05),
        SIMDE_FLOAT32_C(     4.32), SIMDE_FLOAT32_C(     1.21), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     4.03),
        SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     4.77), SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     4.25),
        SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     3.03) },
      UINT8_C(179),
      { SIMDE_FLOAT32_C(     2.13), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(     2.31),
        SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     2.61), SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(     4.17),
        SIMDE_FLOAT32_C(     3.34), SIMDE_FLOAT32_C(     2.74), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(     4.26),
        SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(     4.58) },
      { SIMDE_FLOAT32_C(     3.65), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     3.90), SIMDE_FLOAT32_C(     4.86),
        SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     2.93), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.11),
        SIMDE_FLOAT32_C(     4.89), SIMDE_FLOAT32_C(     4.48), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     3.17),
        SIMDE_FLOAT32_C(     4.88), SIMDE_FLOAT32_C(     3.76), SIMDE_FLOAT32_C(     4.57), SIMDE_FLOAT32_C(     2.00) },
      { SIMDE_FLOAT32_C(    15.80), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     4.84), SIMDE_FLOAT32_C(     3.05),
        SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    16.62), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     1.17),
        SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     4.77), SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     4.25),
        SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     3.29), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     3.03) } },
    { { SIMDE_FLOAT32_C(     4.33), SIMDE_FLOAT32_C(     4.84), SIMDE_FLOAT32_C(     4.31), SIMDE_FLOAT32_C(     4.40),
        SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(     3.58), SIMDE_FLOAT32_C(     0.79),
        SIMDE_FLOAT32_C(     4.55), SIMDE_FLOAT32_C(     4.92), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.14),
        SIMDE_FLOAT32_C(     3.74), SIMDE_FLOAT32_C(     2.29), SIMDE_FLOAT32_C(     4.73), SIMDE_FLOAT32_C(     2.39) },
      UINT8_C( 97),
      { SIMDE_FLOAT32_C(     3.63), SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     1.56),
        SIMDE_FLOAT32_C(     2.48), SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     1.96),
        SIMDE_FLOAT32_C(     3.76), SIMDE_FLOAT32_C(     4.62), SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     2.51),
        SIMDE_FLOAT32_C(     4.19), SIMDE_FLOAT32_C(     3.83), SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(     4.03) },
      { SIMDE_FLOAT32_C(     3.15), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(     4.97),
        SIMDE_FLOAT32_C(     4.82), SIMDE_FLOAT32_C(     2.27), SIMDE_FLOAT32_C(     4.52), SIMDE_FLOAT32_C(     4.75),
        SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     4.66), SIMDE_FLOAT32_C(     3.48), SIMDE_FLOAT32_C(     4.61),
        SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     2.19), SIMDE_FLOAT32_C(     3.02) },
      { SIMDE_FLOAT32_C(    58.04), SIMDE_FLOAT32_C(     4.84), SIMDE_FLOAT32_C(     4.31), SIMDE_FLOAT32_C(     4.40),
        SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(    12.38), SIMDE_FLOAT32_C(     5.36), SIMDE_FLOAT32_C(     0.79),
        SIMDE_FLOAT32_C(     4.55), SIMDE_FLOAT32_C(     4.92), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.14),
        SIMDE_FLOAT32_C(     3.74), SIMDE_FLOAT32_C(     2.29), SIMDE_FLOAT32_C(     4.73), SIMDE_FLOAT32_C(     2.39) } },
    { { SIMDE_FLOAT32_C(     3.11), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     4.58), SIMDE_FLOAT32_C(     0.59),
        SIMDE_FLOAT32_C(     3.13), SIMDE_FLOAT32_C(     1.02), SIMDE_FLOAT32_C(     2.55), SIMDE_FLOAT32_C(     1.89),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     4.38), SIMDE_FLOAT32_C(     4.40), SIMDE_FLOAT32_C(     4.83),
        SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(     3.86), SIMDE_FLOAT32_C(     1.37) },
      UINT8_C( 99),
      { SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     2.61),
        SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     2.06), SIMDE_FLOAT32_C(     4.94), SIMDE_FLOAT32_C(     0.51),
        SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     4.55), SIMDE_FLOAT32_C(     4.90), SIMDE_FLOAT32_C(     1.41),
        SIMDE_FLOAT32_C(     1.73), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     4.52), SIMDE_FLOAT32_C(     1.84) },
      { SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     4.97), SIMDE_FLOAT32_C(     3.52),
        SIMDE_FLOAT32_C(     2.66), SIMDE_FLOAT32_C(     1.86), SIMDE_FLOAT32_C(     4.17), SIMDE_FLOAT32_C(     2.05),
        SIMDE_FLOAT32_C(     1.27), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     2.51),
        SIMDE_FLOAT32_C(     2.86), SIMDE_FLOAT32_C(     1.63), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     3.20) },
      { SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     1.03), SIMDE_FLOAT32_C(     4.58), SIMDE_FLOAT32_C(     0.59),
        SIMDE_FLOAT32_C(     3.13), SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(   781.34), SIMDE_FLOAT32_C(     1.89),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     4.38), SIMDE_FLOAT32_C(     4.40), SIMDE_FLOAT32_C(     4.83),
        SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(     3.86), SIMDE_FLOAT32_C(     1.37) } },
    { { SIMDE_FLOAT32_C(     2.96), SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     3.81),
        SIMDE_FLOAT32_C(     4.36), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     4.32), SIMDE_FLOAT32_C(     4.90),
        SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     4.22), SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     2.03),
        SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     4.64) },
      UINT8_C(216),
      { SIMDE_FLOAT32_C(     3.84), SIMDE_FLOAT32_C(     3.16), SIMDE_FLOAT32_C(     3.61), SIMDE_FLOAT32_C(     0.70),
        SIMDE_FLOAT32_C(     2.33), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     1.32),
        SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     4.48), SIMDE_FLOAT32_C(     4.19), SIMDE_FLOAT32_C(     2.54),
        SIMDE_FLOAT32_C(     4.47), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     1.78) },
      { SIMDE_FLOAT32_C(     3.20), SIMDE_FLOAT32_C(     4.31), SIMDE_FLOAT32_C(     1.14), SIMDE_FLOAT32_C(     3.95),
        SIMDE_FLOAT32_C(     3.63), SIMDE_FLOAT32_C(     1.05), SIMDE_FLOAT32_C(     4.25), SIMDE_FLOAT32_C(     2.86),
        SIMDE_FLOAT32_C(     2.35), SIMDE_FLOAT32_C(     1.28), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     3.18),
        SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     4.64), SIMDE_FLOAT32_C(     4.13), SIMDE_FLOAT32_C(     3.99) },
      { SIMDE_FLOAT32_C(     2.96), SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.24),
        SIMDE_FLOAT32_C(    21.55), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    17.84), SIMDE_FLOAT32_C(     2.21),
        SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     4.22), SIMDE_FLOAT32_C(     1.31), SIMDE_FLOAT32_C(     2.03),
        SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     3.87), SIMDE_FLOAT32_C(     4.64) } },
    { { SIMDE_FLOAT32_C(     2.80), SIMDE_FLOAT32_C(     2.73), SIMDE_FLOAT32_C(     4.69), SIMDE_FLOAT32_C(     0.13),
        SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(     1.66), SIMDE_FLOAT32_C(     1.45), SIMDE_FLOAT32_C(     4.29),
        SIMDE_FLOAT32_C(     1.13), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     0.60),
        SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     2.34), SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(     1.23) },
      UINT8_C(247),
      { SIMDE_FLOAT32_C(     3.53), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     4.57),
        SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     3.14), SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     0.70),
        SIMDE_FLOAT32_C(     3.14), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     2.78),
        SIMDE_FLOAT32_C(     4.23), SIMDE_FLOAT32_C(     4.84), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.97) },
      { SIMDE_FLOAT32_C(     4.52), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     1.18),
        SIMDE_FLOAT32_C(     2.17), SIMDE_FLOAT32_C(     4.64), SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     2.80),
        SIMDE_FLOAT32_C(     1.48), SIMDE_FLOAT32_C(     2.92), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     3.81),
        SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     2.06), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     3.83) },
      { SIMDE_FLOAT32_C(   299.19), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.13),
        SIMDE_FLOAT32_C(    25.15), SIMDE_FLOAT32_C(   202.19), SIMDE_FLOAT32_C(     4.57), SIMDE_FLOAT32_C(     0.37),
        SIMDE_FLOAT32_C(     1.13), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     1.83), SIMDE_FLOAT32_C(     0.60),
        SIMDE_FLOAT32_C(     3.03), SIMDE_FLOAT32_C(     2.34), SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(     1.23) } },
    { { SIMDE_FLOAT32_C(     2.23), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     3.40), SIMDE_FLOAT32_C(     1.66),
        SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     2.36), SIMDE_FLOAT32_C(     2.02),
        SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     4.81), SIMDE_FLOAT32_C(     4.67),
        SIMDE_FLOAT32_C(     3.04), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     1.63), SIMDE_FLOAT32_C(     2.57) },
      UINT8_C(207),
      { SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     3.75), SIMDE_FLOAT32_C(     3.27), SIMDE_FLOAT32_C(     1.62),
        SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(     1.08), SIMDE_FLOAT32_C(     3.10), SIMDE_FLOAT32_C(     3.97),
        SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(     4.27), SIMDE_FLOAT32_C(     3.96),
        SIMDE_FLOAT32_C(     2.36), SIMDE_FLOAT32_C(     3.10), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(     3.10) },
      { SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     2.85), SIMDE_FLOAT32_C(     1.98), SIMDE_FLOAT32_C(     1.82),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     4.00), SIMDE_FLOAT32_C(     2.26), SIMDE_FLOAT32_C(     3.41),
        SIMDE_FLOAT32_C(     3.81), SIMDE_FLOAT32_C(     1.92), SIMDE_FLOAT32_C(     1.46), SIMDE_FLOAT32_C(     4.20),
        SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(     4.02), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.54) },
      { SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(    43.25), SIMDE_FLOAT32_C(    10.44), SIMDE_FLOAT32_C(     2.41),
        SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    12.90), SIMDE_FLOAT32_C(   110.12),
        SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     4.81), SIMDE_FLOAT32_C(     4.67),
        SIMDE_FLOAT32_C(     3.04), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     1.63), SIMDE_FLOAT32_C(     2.57) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 b = simde_mm512_loadu_ps(test_vec[i].b);
    simde__m512 r = simde_mm512_mask_pow_ps(src, test_vec[i].k, a, b);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_pow_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 b[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     4.13), SIMDE_FLOAT64_C(     8.81), SIMDE_FLOAT64_C(     2.27), SIMDE_FLOAT64_C(     5.77),
        SIMDE_FLOAT64_C(     3.43), SIMDE_FLOAT64_C(     9.71), SIMDE_FLOAT64_C(     1.86), SIMDE_FLOAT64_C(     0.10) },
      { SIMDE_FLOAT64_C(     8.72), SIMDE_FLOAT64_C(     9.17), SIMDE_FLOAT64_C(     7.13), SIMDE_FLOAT64_C(     1.02),
        SIMDE_FLOAT64_C(     3.40), SIMDE_FLOAT64_C(     5.53), SIMDE_FLOAT64_C(     2.12), SIMDE_FLOAT64_C(     0.29) },
      { SIMDE_FLOAT64_C(235008.98), SIMDE_FLOAT64_C(462838076.60), SIMDE_FLOAT64_C(   345.51), SIMDE_FLOAT64_C(     5.98),
        SIMDE_FLOAT64_C(    66.07), SIMDE_FLOAT64_C(287953.49), SIMDE_FLOAT64_C(     3.73), SIMDE_FLOAT64_C(     0.51) } },
    { { SIMDE_FLOAT64_C(     4.06), SIMDE_FLOAT64_C(     3.82), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     7.27),
        SIMDE_FLOAT64_C(     4.30), SIMDE_FLOAT64_C(     3.31), SIMDE_FLOAT64_C(     6.31), SIMDE_FLOAT64_C(     8.11) },
      { SIMDE_FLOAT64_C(     1.51), SIMDE_FLOAT64_C(     1.05), SIMDE_FLOAT64_C(     6.76), SIMDE_FLOAT64_C(     9.20),
        SIMDE_FLOAT64_C(     5.39), SIMDE_FLOAT64_C(     5.09), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(     9.52) },
      { SIMDE_FLOAT64_C(     8.30), SIMDE_FLOAT64_C(     4.08), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(84356116.88),
        SIMDE_FLOAT64_C(  2596.54), SIMDE_FLOAT64_C(   442.51), SIMDE_FLOAT64_C(     1.37), SIMDE_FLOAT64_C(450690633.16) } },
    { { SIMDE_FLOAT64_C(     3.90), SIMDE_FLOAT64_C(     2.44), SIMDE_FLOAT64_C(     5.29), SIMDE_FLOAT64_C(     7.33),
        SIMDE_FLOAT64_C(     2.15), SIMDE_FLOAT64_C(     7.16), SIMDE_FLOAT64_C(     7.43), SIMDE_FLOAT64_C(     0.86) },
      { SIMDE_FLOAT64_C(     6.32), SIMDE_FLOAT64_C(     4.56), SIMDE_FLOAT64_C(     1.88), SIMDE_FLOAT64_C(     9.72),
        SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(     4.00), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     4.15) },
      { SIMDE_FLOAT64_C(  5439.12), SIMDE_FLOAT64_C(    58.41), SIMDE_FLOAT64_C(    22.91), SIMDE_FLOAT64_C(256336608.20),
        SIMDE_FLOAT64_C(     1.07), SIMDE_FLOAT64_C(  2628.16), SIMDE_FLOAT64_C(     1.02), SIMDE_FLOAT64_C(     0.53) } },
    { { SIMDE_FLOAT64_C(     7.82), SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(     1.42), SIMDE_FLOAT64_C(     2.12),
        SIMDE_FLOAT64_C(     3.99), SIMDE_FLOAT64_C(     7.73), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     5.49) },
      { SIMDE_FLOAT64_C(     8.77), SIMDE_FLOAT64_C(     6.98), SIMDE_FLOAT64_C(     4.70), SIMDE_FLOAT64_C(     4.16),
        SIMDE_FLOAT64_C(     2.08), SIMDE_FLOAT64_C(     4.87), SIMDE_FLOAT64_C(     3.68), SIMDE_FLOAT64_C(     5.98) },
      { SIMDE_FLOAT64_C(68143309.86), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     5.20), SIMDE_FLOAT64_C(    22.78),
        SIMDE_FLOAT64_C(    17.78), SIMDE_FLOAT64_C( 21156.03), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C( 26463.22) } },
    { { SIMDE_FLOAT64_C(     7.30), SIMDE_FLOAT64_C(     8.98), SIMDE_FLOAT64_C(     3.31), SIMDE_FLOAT64_C(     9.45),
        SIMDE_FLOAT64_C(     6.13), SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(     0.31), SIMDE_FLOAT64_C(     2.46) },
      { SIMDE_FLOAT64_C(     5.30), SIMDE_FLOAT64_C(     2.18), SIMDE_FLOAT64_C(     2.18), SIMDE_FLOAT64_C(     5.38),
        SIMDE_FLOAT64_C(     6.18), SIMDE_FLOAT64_C(     2.19), SIMDE_FLOAT64_C(     9.53), SIMDE_FLOAT64_C(     4.00) },
      { SIMDE_FLOAT64_C( 37636.67), SIMDE_FLOAT64_C(   119.71), SIMDE_FLOAT64_C(    13.59), SIMDE_FLOAT64_C(176938.82),
        SIMDE_FLOAT64_C( 73536.97), SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    36.62) } },
    { { SIMDE_FLOAT64_C(     2.87), SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(     6.12), SIMDE_FLOAT64_C(     6.85),
        SIMDE_FLOAT64_C(     8.67), SIMDE_FLOAT64_C(     6.34), SIMDE_FLOAT64_C(     2.35), SIMDE_FLOAT64_C(     7.45) },
      { SIMDE_FLOAT64_C(     3.33), SIMDE_FLOAT64_C(     7.04), SIMDE_FLOAT64_C(     1.61), SIMDE_FLOAT64_C(     5.40),
        SIMDE_FLOAT64_C(     1.91), SIMDE_FLOAT64_C(     5.30), SIMDE_FLOAT64_C(     1.38), SIMDE_FLOAT64_C(     9.21) },
      { SIMDE_FLOAT64_C(    33.48), SIMDE_FLOAT64_C(     0.70), SIMDE_FLOAT64_C(    18.48), SIMDE_FLOAT64_C( 32563.35),
        SIMDE_FLOAT64_C(    61.89), SIMDE_FLOAT64_C( 17826.79), SIMDE_FLOAT64_C(     3.25), SIMDE_FLOAT64_C(107785234.77) } },
    { { SIMDE_FLOAT64_C(     4.27), SIMDE_FLOAT64_C(     4.69), SIMDE_FLOAT64_C(     8.66), SIMDE_FLOAT64_C(     0.40),
        SIMDE_FLOAT64_C(     5.42), SIMDE_FLOAT64_C(     8.96), SIMDE_FLOAT64_C(     2.86), SIMDE_FLOAT64_C(     0.72) },
      { SIMDE_FLOAT64_C(     1.15), SIMDE_FLOAT64_C(     5.04), SIMDE_FLOAT64_C(     6.10), SIMDE_FLOAT64_C(     7.33),
        SIMDE_FLOAT64_C(     7.23), SIMDE_FLOAT64_C(     5.63), SIMDE_FLOAT64_C(     1.33), SIMDE_FLOAT64_C(     0.10) },
      { SIMDE_FLOAT64_C(     5.31), SIMDE_FLOAT64_C(  2413.85), SIMDE_FLOAT64_C(523430.64), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(202681.84), SIMDE_FLOAT64_C(229876.25), SIMDE_FLOAT64_C(     4.05), SIMDE_FLOAT64_C(     0.97) } },
    { { SIMDE_FLOAT64_C(     6.58), SIMDE_FLOAT64_C(     7.45), SIMDE_FLOAT64_C(     6.95), SIMDE_FLOAT64_C(     5.25),
        SIMDE_FLOAT64_C(     3.79), SIMDE_FLOAT64_C(     9.30), SIMDE_FLOAT64_C(     2.70), SIMDE_FLOAT64_C(     7.12) },
      { SIMDE_FLOAT64_C(     6.34), SIMDE_FLOAT64_C(     4.32), SIMDE_FLOAT64_C(     2.52), SIMDE_FLOAT64_C(     8.25),
        SIMDE_FLOAT64_C(     9.61), SIMDE_FLOAT64_C(     3.90), SIMDE_FLOAT64_C(     7.46), SIMDE_FLOAT64_C(     3.88) },
      { SIMDE_FLOAT64_C(154011.15), SIMDE_FLOAT64_C(  5857.54), SIMDE_FLOAT64_C(   132.37), SIMDE_FLOAT64_C(873603.27),
        SIMDE_FLOAT64_C(363682.84), SIMDE_FLOAT64_C(  5985.27), SIMDE_FLOAT64_C(  1651.86), SIMDE_FLOAT64_C(  2030.59) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d b = simde_mm512_loadu_pd(test_vec[i].b);
    simde__m512d r = simde_mm512_pow_pd(a, b);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_pow_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 b[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     1.66), SIMDE_FLOAT64_C(     8.31), SIMDE_FLOAT64_C(     9.30),
        SIMDE_FLOAT64_C(     8.14), SIMDE_FLOAT64_C(     3.76), SIMDE_FLOAT64_C(     2.75), SIMDE_FLOAT64_C(     2.84) },
      UINT8_C(150),
      { SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     1.52), SIMDE_FLOAT64_C(     1.22), SIMDE_FLOAT64_C(     1.57),
        SIMDE_FLOAT64_C(     9.30), SIMDE_FLOAT64_C(     5.53), SIMDE_FLOAT64_C(     8.36), SIMDE_FLOAT64_C(     5.48) },
      { SIMDE_FLOAT64_C(     1.36), SIMDE_FLOAT64_C(     7.20), SIMDE_FLOAT64_C(     8.45), SIMDE_FLOAT64_C(     1.77),
        SIMDE_FLOAT64_C(     6.75), SIMDE_FLOAT64_C(     6.44), SIMDE_FLOAT64_C(     2.61), SIMDE_FLOAT64_C(     0.07) },
      { SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(    20.38), SIMDE_FLOAT64_C(     5.37), SIMDE_FLOAT64_C(     9.30),
        SIMDE_FLOAT64_C(3445560.68), SIMDE_FLOAT64_C(     3.76), SIMDE_FLOAT64_C(     2.75), SIMDE_FLOAT64_C(     1.13) } },
    { { SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(     8.62), SIMDE_FLOAT64_C(     9.49), SIMDE_FLOAT64_C(     2.94),
        SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     3.49), SIMDE_FLOAT64_C(     2.24) },
      UINT8_C(147),
      { SIMDE_FLOAT64_C(     2.79), SIMDE_FLOAT64_C(     0.38), SIMDE_FLOAT64_C(     5.06), SIMDE_FLOAT64_C(     5.54),
        SIMDE_FLOAT64_C(     3.22), SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(     6.09), SIMDE_FLOAT64_C(     4.74) },
      { SIMDE_FLOAT64_C(     1.96), SIMDE_FLOAT64_C(     7.66), SIMDE_FLOAT64_C(     4.04), SIMDE_FLOAT64_C(     7.49),
        SIMDE_FLOAT64_C(     6.02), SIMDE_FLOAT64_C(     9.52), SIMDE_FLOAT64_C(     8.85), SIMDE_FLOAT64_C(     3.22) },
      { SIMDE_FLOAT64_C(     7.47), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     9.49), SIMDE_FLOAT64_C(     2.94),
        SIMDE_FLOAT64_C(  1141.02), SIMDE_FLOAT64_C(     3.00), SIMDE_FLOAT64_C(     3.49), SIMDE_FLOAT64_C(   149.97) } },
    { { SIMDE_FLOAT64_C(     7.97), SIMDE_FLOAT64_C(     0.62), SIMDE_FLOAT64_C(     9.97), SIMDE_FLOAT64_C(     4.41),
        SIMDE_FLOAT64_C(     3.23), SIMDE_FLOAT64_C(     0.04), SIMDE_FLOAT64_C(     5.21), SIMDE_FLOAT64_C(     1.85) },
      UINT8_C(167),
      { SIMDE_FLOAT64_C(     8.14), SIMDE_FLOAT64_C(     2.43), SIMDE_FLOAT64_C(     2.53), SIMDE_FLOAT64_C(     1.63),
        SIMDE_FLOAT64_C(     4.67), SIMDE_FLOAT64_C(     3.83), SIMDE_FLOAT64_C(     4.42), SIMDE_FLOAT64_C(     5.05) },
      { SIMDE_FLOAT64_C(     8.89), SIMDE_FLOAT64_C(     9.96), SIMDE_FLOAT64_C(     8.27), SIMDE_FLOAT64_C(     9.63),
        SIMDE_FLOAT64_C(     6.05), SIMDE_FLOAT64_C(     3.01), SIMDE_FLOAT64_C(     1.59), SIMDE_FLOAT64_C(     3.71) },
      { SIMDE_FLOAT64_C(124580755.27), SIMDE_FLOAT64_C(  6928.49), SIMDE_FLOAT64_C(  2156.78), SIMDE_FLOAT64_C(     4.41),
        SIMDE_FLOAT64_C(     3.23), SIMDE_FLOAT64_C(    56.94), SIMDE_FLOAT64_C(     5.21), SIMDE_FLOAT64_C(   406.64) } },
    { { SIMDE_FLOAT64_C(     7.05), SIMDE_FLOAT64_C(     9.08), SIMDE_FLOAT64_C(     9.73), SIMDE_FLOAT64_C(     6.57),
        SIMDE_FLOAT64_C(     7.92), SIMDE_FLOAT64_C(     2.94), SIMDE_FLOAT64_C(     4.54), SIMDE_FLOAT64_C(     8.54) },
      UINT8_C(148),
      { SIMDE_FLOAT64_C(     8.95), SIMDE_FLOAT64_C(     1.77), SIMDE_FLOAT64_C(     2.95), SIMDE_FLOAT64_C(     4.15),
        SIMDE_FLOAT64_C(     3.63), SIMDE_FLOAT64_C(     2.48), SIMDE_FLOAT64_C(     2.30), SIMDE_FLOAT64_C(     6.06) },
      { SIMDE_FLOAT64_C(     5.01), SIMDE_FLOAT64_C(     3.93), SIMDE_FLOAT64_C(     0.73), SIMDE_FLOAT64_C(     8.84),
        SIMDE_FLOAT64_C(     8.35), SIMDE_FLOAT64_C(     5.77), SIMDE_FLOAT64_C(     7.74), SIMDE_FLOAT64_C(     8.32) },
      { SIMDE_FLOAT64_C(     7.05), SIMDE_FLOAT64_C(     9.08), SIMDE_FLOAT64_C(     2.20), SIMDE_FLOAT64_C(     6.57),
        SIMDE_FLOAT64_C( 47339.14), SIMDE_FLOAT64_C(     2.94), SIMDE_FLOAT64_C(     4.54), SIMDE_FLOAT64_C(3237220.14) } },
    { { SIMDE_FLOAT64_C(     4.04), SIMDE_FLOAT64_C(     7.37), SIMDE_FLOAT64_C(     4.37), SIMDE_FLOAT64_C(     7.05),
        SIMDE_FLOAT64_C(     8.95), SIMDE_FLOAT64_C(     8.08), SIMDE_FLOAT64_C(     4.10), SIMDE_FLOAT64_C(     8.03) },
      UINT8_C(201),
      { SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     5.95), SIMDE_FLOAT64_C(     0.75), SIMDE_FLOAT64_C(     5.20),
        SIMDE_FLOAT64_C(     4.50), SIMDE_FLOAT64_C(     3.66), SIMDE_FLOAT64_C(     4.15), SIMDE_FLOAT64_C(     6.27) },
      { SIMDE_FLOAT64_C(     6.61), SIMDE_FLOAT64_C(     8.31), SIMDE_FLOAT64_C(     9.90), SIMDE_FLOAT64_C(     9.09),
        SIMDE_FLOAT64_C(     0.60), SIMDE_FLOAT64_C(     5.95), SIMDE_FLOAT64_C(     4.10), SIMDE_FLOAT64_C(     4.53) },
      { SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     7.37), SIMDE_FLOAT64_C(     4.37), SIMDE_FLOAT64_C(3224559.49),
        SIMDE_FLOAT64_C(     8.95), SIMDE_FLOAT64_C(     8.08), SIMDE_FLOAT64_C(   341.98), SIMDE_FLOAT64_C(  4089.05) } },
    { { SIMDE_FLOAT64_C(     6.68), SIMDE_FLOAT64_C(     2.94), SIMDE_FLOAT64_C(     2.89), SIMDE_FLOAT64_C(     2.45),
        SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(     1.20), SIMDE_FLOAT64_C(     6.49), SIMDE_FLOAT64_C(     8.05) },
      UINT8_C( 44),
      { SIMDE_FLOAT64_C(     3.53), SIMDE_FLOAT64_C(     7.00), SIMDE_FLOAT64_C(     3.65), SIMDE_FLOAT64_C(     7.63),
        SIMDE_FLOAT64_C(     5.03), SIMDE_FLOAT64_C(     1.45), SIMDE_FLOAT64_C(     8.30), SIMDE_FLOAT64_C(     0.98) },
      { SIMDE_FLOAT64_C(     2.20), SIMDE_FLOAT64_C(     3.50), SIMDE_FLOAT64_C(     5.47), SIMDE_FLOAT64_C(     5.86),
        SIMDE_FLOAT64_C(     7.66), SIMDE_FLOAT64_C(     1.74), SIMDE_FLOAT64_C(     2.46), SIMDE_FLOAT64_C(     5.96) },
      { SIMDE_FLOAT64_C(     6.68), SIMDE_FLOAT64_C(     2.94), SIMDE_FLOAT64_C(  1190.53), SIMDE_FLOAT64_C(148454.65),
        SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(     1.91), SIMDE_FLOAT64_C(     6.49), SIMDE_FLOAT64_C(     8.05) } },
    { { SIMDE_FLOAT64_C(     1.64), SIMDE_FLOAT64_C(     1.55), SIMDE_FLOAT64_C(     6.56), SIMDE_FLOAT64_C(     7.59),
        SIMDE_FLOAT64_C(     5.66), SIMDE_FLOAT64_C(     1.10), SIMDE_FLOAT64_C(     4.27), SIMDE_FLOAT64_C(     8.60) },
      UINT8_C(119),
      { SIMDE_FLOAT64_C(     6.72), SIMDE_FLOAT64_C(     9.28), SIMDE_FLOAT64_C(     5.18), SIMDE_FLOAT64_C(     3.21),
        SIMDE_FLOAT64_C(     7.32), SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     6.75), SIMDE_FLOAT64_C(     4.32) },
      { SIMDE_FLOAT64_C(     4.41), SIMDE_FLOAT64_C(     4.38), SIMDE_FLOAT64_C(     9.35), SIMDE_FLOAT64_C(     5.86),
        SIMDE_FLOAT64_C(     2.68), SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(     8.06), SIMDE_FLOAT64_C(     6.18) },
      { SIMDE_FLOAT64_C(  4453.47), SIMDE_FLOAT64_C( 17292.59), SIMDE_FLOAT64_C(4775108.60), SIMDE_FLOAT64_C(     7.59),
        SIMDE_FLOAT64_C(   207.44), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(4832684.12), SIMDE_FLOAT64_C(     8.60) } },
    { { SIMDE_FLOAT64_C(     5.80), SIMDE_FLOAT64_C(     3.91), SIMDE_FLOAT64_C(     3.84), SIMDE_FLOAT64_C(     7.54),
        SIMDE_FLOAT64_C(     6.38), SIMDE_FLOAT64_C(     9.80), SIMDE_FLOAT64_C(     9.18), SIMDE_FLOAT64_C(     7.93) },
      UINT8_C(224),
      { SIMDE_FLOAT64_C(     6.78), SIMDE_FLOAT64_C(     3.59), SIMDE_FLOAT64_C(     7.46), SIMDE_FLOAT64_C(     1.05),
        SIMDE_FLOAT64_C(     2.19), SIMDE_FLOAT64_C(     1.44), SIMDE_FLOAT64_C(     7.77), SIMDE_FLOAT64_C(     1.46) },
      { SIMDE_FLOAT64_C(     6.62), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     8.79), SIMDE_FLOAT64_C(     7.38),
        SIMDE_FLOAT64_C(     7.73), SIMDE_FLOAT64_C(     3.11), SIMDE_FLOAT64_C(     1.78), SIMDE_FLOAT64_C(     2.11) },
      { SIMDE_FLOAT64_C(     5.80), SIMDE_FLOAT64_C(     3.91), SIMDE_FLOAT64_C(     3.84), SIMDE_FLOAT64_C(     7.54),
        SIMDE_FLOAT64_C(     6.38), SIMDE_FLOAT64_C(     3.11), SIMDE_FLOAT64_C(    38.45), SIMDE_FLOAT64_C(     2.22) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d b = simde_mm512_loadu_pd(test_vec[i].b);
    simde__m512d r = simde_mm512_mask_pow_pd(src, test_vec[i].k, a, b);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_rem_epi8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_mm_set_epi8(INT8_C( 104), INT8_C(  42), INT8_C(  53), INT8_C(  -2),
                        INT8_C(-124), INT8_C(  -2), INT8_C(  96), INT8_C(  75),
                        INT8_C(  31), INT8_C( 112), INT8_C(-105), INT8_C( -87),
                        INT8_C( -84), INT8_C(  94), INT8_C( 112), INT8_C(  90)),
      simde_mm_set_epi8(INT8_C( -65), INT8_C( -89), INT8_C( -30), INT8_C(  64),
                        INT8_C( -43), INT8_C( -54), INT8_C( 110), INT8_C( 113),
                        INT8_C(  89), INT8_C( -19), INT8_C(  70), INT8_C( -30),
                        INT8_C(-124), INT8_C(  91), INT8_C(  -1), INT8_C(  88)),
      simde_mm_set_epi8(INT8_C(  39), INT8_C(  42), INT8_C(  23), INT8_C(  -2),
                        INT8_C( -38), INT8_C(  -2), INT8_C(  96), INT8_C(  75),
                        INT8_C(  31), INT8_C(  17), INT8_C( -35), INT8_C( -27),
                        INT8_C( -84), INT8_C(   3), INT8_C(   0), INT8_C(   2)) },
    { simde_mm_set_epi8(INT8_C( -23), INT8_C( -86), INT8_C( -15), INT8_C( 126),
                        INT8_C( -74), INT8_C(  10), INT8_C( -48), INT8_C( -58),
                        INT8_C(  93), INT8_C(-126), INT8_C( -61), INT8_C( -79),
                        INT8_C( -69), INT8_C( -33), INT8_C(-117), INT8_C(  -3)),
      simde_mm_set_epi8(INT8_C(  41), INT8_C(  49), INT8_C( -85), INT8_C( -58),
                        INT8_C(  40), INT8_C(  44), INT8_C( -14), INT8_C(  51),
                        INT8_C(-118), INT8_C( -39), INT8_C( -41), INT8_C(  -7),
                        INT8_C( -55), INT8_C(  37), INT8_C(-119), INT8_C(  29)),
      simde_mm_set_epi8(INT8_C( -23), INT8_C( -37), INT8_C( -15), INT8_C(  10),
                        INT8_C( -34), INT8_C(  10), INT8_C(  -6), INT8_C(  -7),
                        INT8_C(  93), INT8_C(  -9), INT8_C( -20), INT8_C(  -2),
                        INT8_C( -14), INT8_C( -33), INT8_C(-117), INT8_C(  -3)) },
    { simde_mm_set_epi8(INT8_C(  88), INT8_C( -13), INT8_C(  83), INT8_C( -34),
                        INT8_C(  17), INT8_C( -52), INT8_C( 102), INT8_C(  26),
                        INT8_C(  74), INT8_C(-115), INT8_C(  -4), INT8_C( 101),
                        INT8_C( -39), INT8_C(  50), INT8_C(  -9), INT8_C(-117)),
      simde_mm_set_epi8(INT8_C(  71), INT8_C(  16), INT8_C( 127), INT8_C(  20),
                        INT8_C(-125), INT8_C( -92), INT8_C( -21), INT8_C( -43),
                        INT8_C(  78), INT8_C( -41), INT8_C(  -6), INT8_C(  42),
                        INT8_C(   9), INT8_C( -58), INT8_C(  72), INT8_C(  56)),
      simde_mm_set_epi8(INT8_C(  17), INT8_C( -13), INT8_C(  83), INT8_C( -14),
                        INT8_C(  17), INT8_C( -52), INT8_C(  18), INT8_C(  26),
                        INT8_C(  74), INT8_C( -33), INT8_C(  -4), INT8_C(  17),
                        INT8_C(  -3), INT8_C(  50), INT8_C(  -9), INT8_C(  -5)) },
    { simde_mm_set_epi8(INT8_C( -95), INT8_C( 114), INT8_C(-111), INT8_C(  28),
                        INT8_C( 100), INT8_C( -53), INT8_C( 101), INT8_C(  21),
                        INT8_C(   3), INT8_C(   0), INT8_C(  63), INT8_C( 116),
                        INT8_C(  43), INT8_C( 106), INT8_C( -29), INT8_C( -44)),
      simde_mm_set_epi8(INT8_C(-106), INT8_C( -49), INT8_C(  31), INT8_C(-118),
                        INT8_C(  70), INT8_C(  80), INT8_C(-117), INT8_C( 103),
                        INT8_C( -99), INT8_C( -33), INT8_C(  12), INT8_C( -74),
                        INT8_C( -41), INT8_C( -14), INT8_C(-105), INT8_C( -57)),
      simde_mm_set_epi8(INT8_C( -95), INT8_C(  16), INT8_C( -18), INT8_C(  28),
                        INT8_C(  30), INT8_C( -53), INT8_C( 101), INT8_C(  21),
                        INT8_C(   3), INT8_C(   0), INT8_C(   3), INT8_C(  42),
                        INT8_C(   2), INT8_C(   8), INT8_C( -29), INT8_C( -44)) },
    { simde_mm_set_epi8(INT8_C(  29), INT8_C(  89), INT8_C(   4), INT8_C(  90),
                        INT8_C(  -1), INT8_C(  56), INT8_C(  40), INT8_C(-107),
                        INT8_C(-125), INT8_C(-104), INT8_C(  36), INT8_C( -27),
                        INT8_C( -21), INT8_C( -84), INT8_C( -95), INT8_C(  -6)),
      simde_mm_set_epi8(INT8_C(  29), INT8_C( 101), INT8_C(  12), INT8_C(  -7),
                        INT8_C( -72), INT8_C( -61), INT8_C(  -6), INT8_C( -43),
                        INT8_C(  53), INT8_C(  76), INT8_C( -68), INT8_C(  25),
                        INT8_C( -80), INT8_C( -78), INT8_C( -55), INT8_C( -12)),
      simde_mm_set_epi8(INT8_C(   0), INT8_C(  89), INT8_C(   4), INT8_C(   6),
                        INT8_C(  -1), INT8_C(  56), INT8_C(   4), INT8_C( -21),
                        INT8_C( -19), INT8_C( -28), INT8_C(  36), INT8_C(  -2),
                        INT8_C( -21), INT8_C(  -6), INT8_C( -40), INT8_C(  -6)) },
    { simde_mm_set_epi8(INT8_C( -60), INT8_C(  36), INT8_C(  35), INT8_C(  54),
                        INT8_C(  94), INT8_C(  53), INT8_C(-124), INT8_C(  -9),
                        INT8_C( -29), INT8_C( -20), INT8_C(  32), INT8_C( 119),
                        INT8_C( 124), INT8_C(  15), INT8_C(  15), INT8_C( -94)),
      simde_mm_set_epi8(INT8_C(  78), INT8_C(  89), INT8_C( 105), INT8_C(  98),
                        INT8_C( -78), INT8_C( -83), INT8_C(-122), INT8_C( -57),
                        INT8_C( -45), INT8_C( -13), INT8_C( -95), INT8_C( -36),
                        INT8_C( -85), INT8_C( 107), INT8_C(  43), INT8_C(   1)),
      simde_mm_set_epi8(INT8_C( -60), INT8_C(  36), INT8_C(  35), INT8_C(  54),
                        INT8_C(  16), INT8_C(  53), INT8_C(  -2), INT8_C(  -9),
                        INT8_C( -29), INT8_C(  -7), INT8_C(  32), INT8_C(  11),
                        INT8_C(  39), INT8_C(  15), INT8_C(  15), INT8_C(   0)) },
    { simde_mm_set_epi8(INT8_C(  32), INT8_C(  79), INT8_C(  19), INT8_C(  72),
                        INT8_C(  29), INT8_C( -53), INT8_C(  79), INT8_C(  -3),
                        INT8_C(  57), INT8_C(  16), INT8_C(  99), INT8_C( 126),
                        INT8_C( -77), INT8_C(  12), INT8_C( 100), INT8_C(  11)),
      simde_mm_set_epi8(INT8_C( 101), INT8_C( -18), INT8_C( -52), INT8_C(-126),
                        INT8_C( 117), INT8_C( -86), INT8_C( -70), INT8_C(  72),
                        INT8_C( -85), INT8_C(  25), INT8_C( -31), INT8_C( -92),
                        INT8_C(   7), INT8_C(  17), INT8_C(-125), INT8_C(  67)),
      simde_mm_set_epi8(INT8_C(  32), INT8_C(   7), INT8_C(  19), INT8_C(  72),
                        INT8_C(  29), INT8_C( -53), INT8_C(   9), INT8_C(  -3),
                        INT8_C(  57), INT8_C(  16), INT8_C(   6), INT8_C(  34),
                        INT8_C(   0), INT8_C(  12), INT8_C( 100), INT8_C(  11)) },
    { simde_mm_set_epi8(INT8_C( -12), INT8_C( 123), INT8_C( -45), INT8_C( -41),
                        INT8_C( -52), INT8_C( -36), INT8_C(  31), INT8_C( -52),
                        INT8_C( -27), INT8_C(  71), INT8_C(   9), INT8_C( -84),
                        INT8_C( -96), INT8_C(-115), INT8_C(  31), INT8_C(  12)),
      simde_mm_set_epi8(INT8_C( -68), INT8_C(  29), INT8_C( -34), INT8_C(  81),
                        INT8_C( -41), INT8_C(  10), INT8_C( -66), INT8_C( -37),
                        INT8_C( 108), INT8_C(  -9), INT8_C( -68), INT8_C( -41),
                        INT8_C( -24), INT8_C( -55), INT8_C( -20), INT8_C(   9)),
      simde_mm_set_epi8(INT8_C( -12), INT8_C(   7), INT8_C( -11), INT8_C( -41),
                        INT8_C( -11), INT8_C(  -6), INT8_C(  31), INT8_C( -15),
                        INT8_C( -27), INT8_C(   8), INT8_C(   9), INT8_C(  -2),
                        INT8_C(   0), INT8_C(  -5), INT8_C(  11), INT8_C(   3)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_rem_epi8(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_i8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_rem_epi16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_mm_set_epi16(INT16_C( 26666), INT16_C( 13822), INT16_C(-31490), INT16_C( 24651),
                         INT16_C(  8048), INT16_C(-26711), INT16_C(-21410), INT16_C( 28762)),
      simde_mm_set_epi16(INT16_C(-16473), INT16_C( -7616), INT16_C(-10806), INT16_C( 28273),
                         INT16_C( 23021), INT16_C( 18146), INT16_C(-31653), INT16_C(  -168)),
      simde_mm_set_epi16(INT16_C( 10193), INT16_C(  6206), INT16_C( -9878), INT16_C( 24651),
                         INT16_C(  8048), INT16_C( -8565), INT16_C(-21410), INT16_C(    34)) },
    { simde_mm_set_epi16(INT16_C( -5718), INT16_C( -3714), INT16_C(-18934), INT16_C(-12090),
                         INT16_C( 23938), INT16_C(-15439), INT16_C(-17441), INT16_C(-29699)),
      simde_mm_set_epi16(INT16_C( 10545), INT16_C(-21562), INT16_C( 10284), INT16_C( -3533),
                         INT16_C(-29991), INT16_C(-10247), INT16_C(-14043), INT16_C(-30435)),
      simde_mm_set_epi16(INT16_C( -5718), INT16_C( -3714), INT16_C( -8650), INT16_C( -1491),
                         INT16_C( 23938), INT16_C( -5192), INT16_C( -3398), INT16_C(-29699)) },
    { simde_mm_set_epi16(INT16_C( 22771), INT16_C( 21470), INT16_C(  4556), INT16_C( 26138),
                         INT16_C( 19085), INT16_C(  -923), INT16_C( -9934), INT16_C( -2165)),
      simde_mm_set_epi16(INT16_C( 18192), INT16_C( 32532), INT16_C(-31836), INT16_C( -5163),
                         INT16_C( 20183), INT16_C( -1494), INT16_C(  2502), INT16_C( 18488)),
      simde_mm_set_epi16(INT16_C(  4579), INT16_C( 21470), INT16_C(  4556), INT16_C(   323),
                         INT16_C( 19085), INT16_C(  -923), INT16_C( -2428), INT16_C( -2165)) },
    { simde_mm_set_epi16(INT16_C(-24206), INT16_C(-28388), INT16_C( 25803), INT16_C( 25877),
                         INT16_C(   768), INT16_C( 16244), INT16_C( 11114), INT16_C( -7212)),
      simde_mm_set_epi16(INT16_C(-26929), INT16_C(  8074), INT16_C( 18000), INT16_C(-29849),
                         INT16_C(-25121), INT16_C(  3254), INT16_C(-10254), INT16_C(-26681)),
      simde_mm_set_epi16(INT16_C(-24206), INT16_C( -4166), INT16_C(  7803), INT16_C( 25877),
                         INT16_C(   768), INT16_C(  3228), INT16_C(   860), INT16_C( -7212)) },
    { simde_mm_set_epi16(INT16_C(  7513), INT16_C(  1114), INT16_C(  -200), INT16_C( 10389),
                         INT16_C(-31848), INT16_C(  9445), INT16_C( -5204), INT16_C(-24070)),
      simde_mm_set_epi16(INT16_C(  7525), INT16_C(  3321), INT16_C(-18237), INT16_C( -1323),
                         INT16_C( 13644), INT16_C(-17383), INT16_C(-20302), INT16_C(-13836)),
      simde_mm_set_epi16(INT16_C(  7513), INT16_C(  1114), INT16_C(  -200), INT16_C(  1128),
                         INT16_C( -4560), INT16_C(  9445), INT16_C( -5204), INT16_C(-10234)) },
    { simde_mm_set_epi16(INT16_C(-15324), INT16_C(  9014), INT16_C( 24117), INT16_C(-31497),
                         INT16_C( -7188), INT16_C(  8311), INT16_C( 31759), INT16_C(  4002)),
      simde_mm_set_epi16(INT16_C( 20057), INT16_C( 26978), INT16_C(-19795), INT16_C(-31033),
                         INT16_C(-11277), INT16_C(-24100), INT16_C(-21653), INT16_C( 11009)),
      simde_mm_set_epi16(INT16_C(-15324), INT16_C(  9014), INT16_C(  4322), INT16_C(  -464),
                         INT16_C( -7188), INT16_C(  8311), INT16_C( 10106), INT16_C(  4002)) },
    { simde_mm_set_epi16(INT16_C(  8271), INT16_C(  4936), INT16_C(  7627), INT16_C( 20477),
                         INT16_C( 14608), INT16_C( 25470), INT16_C(-19700), INT16_C( 25611)),
      simde_mm_set_epi16(INT16_C( 26094), INT16_C(-13182), INT16_C( 30122), INT16_C(-17848),
                         INT16_C(-21735), INT16_C( -7772), INT16_C(  1809), INT16_C(-31933)),
      simde_mm_set_epi16(INT16_C(  8271), INT16_C(  4936), INT16_C(  7627), INT16_C(  2629),
                         INT16_C( 14608), INT16_C(  2154), INT16_C( -1610), INT16_C( 25611)) },
    { simde_mm_set_epi16(INT16_C( -2949), INT16_C(-11305), INT16_C(-13092), INT16_C(  8140),
                         INT16_C( -6841), INT16_C(  2476), INT16_C(-24435), INT16_C(  7948)),
      simde_mm_set_epi16(INT16_C(-17379), INT16_C( -8623), INT16_C(-10486), INT16_C(-16677),
                         INT16_C( 27895), INT16_C(-17193), INT16_C( -5943), INT16_C( -5111)),
      simde_mm_set_epi16(INT16_C( -2949), INT16_C( -2682), INT16_C( -2606), INT16_C(  8140),
                         INT16_C( -6841), INT16_C(  2476), INT16_C(  -663), INT16_C(  2837)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_rem_epi16(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_i16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_rem_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_mm_set_epi32(INT32_C( 1747596798), INT32_C(-2063703989), INT32_C(  527472553), INT32_C(-1403096998)),
      simde_mm_set_epi32(INT32_C(-1079516608), INT32_C( -708153743), INT32_C( 1508722402), INT32_C(-2074345640)),
      simde_mm_set_epi32(INT32_C(  668080190), INT32_C( -647396503), INT32_C(  527472553), INT32_C(-1403096998)) },
    { simde_mm_set_epi32(INT32_C( -374673026), INT32_C(-1240805178), INT32_C( 1568850865), INT32_C(-1142977539)),
      simde_mm_set_epi32(INT32_C(  691121094), INT32_C(  674034227), INT32_C(-1965434887), INT32_C( -920286947)),
      simde_mm_set_epi32(INT32_C( -374673026), INT32_C( -566770951), INT32_C( 1568850865), INT32_C( -222690592)) },
    { simde_mm_set_epi32(INT32_C( 1492341726), INT32_C(  298608154), INT32_C( 1250819173), INT32_C( -650971253)),
      simde_mm_set_epi32(INT32_C( 1192263444), INT32_C(-2086343723), INT32_C( 1322777130), INT32_C(  163989560)),
      simde_mm_set_epi32(INT32_C(  300078282), INT32_C(  298608154), INT32_C( 1250819173), INT32_C( -159002573)) },
    { simde_mm_set_epi32(INT32_C(-1586327268), INT32_C( 1691051285), INT32_C(   50347892), INT32_C(  728425428)),
      simde_mm_set_epi32(INT32_C(-1764810870), INT32_C( 1179683687), INT32_C(-1646326602), INT32_C( -671967289)),
      simde_mm_set_epi32(INT32_C(-1586327268), INT32_C(  511367598), INT32_C(   50347892), INT32_C(   56458139)) },
    { simde_mm_set_epi32(INT32_C(  492373082), INT32_C(  -13096811), INT32_C(-2087181083), INT32_C( -341007878)),
      simde_mm_set_epi32(INT32_C(  493161721), INT32_C(-1195115819), INT32_C(  894221337), INT32_C(-1330460172)),
      simde_mm_set_epi32(INT32_C(  492373082), INT32_C(  -13096811), INT32_C( -298738409), INT32_C( -341007878)) },
    { simde_mm_set_epi32(INT32_C(-1004264650), INT32_C( 1580565751), INT32_C( -471064457), INT32_C( 2081361826)),
      simde_mm_set_epi32(INT32_C( 1314482530), INT32_C(-1297250617), INT32_C( -739008036), INT32_C(-1419039999)),
      simde_mm_set_epi32(INT32_C(-1004264650), INT32_C(  283315134), INT32_C( -471064457), INT32_C(  662321827)) },
    { simde_mm_set_epi32(INT32_C(  542053192), INT32_C(  499863549), INT32_C(  957375358), INT32_C(-1291033589)),
      simde_mm_set_epi32(INT32_C( 1710148738), INT32_C( 1974123080), INT32_C(-1424367196), INT32_C(  118588227)),
      simde_mm_set_epi32(INT32_C(  542053192), INT32_C(  499863549), INT32_C(  957375358), INT32_C( -105151319)) },
    { simde_mm_set_epi32(INT32_C( -193211433), INT32_C( -857989172), INT32_C( -448329300), INT32_C(-1601364212)),
      simde_mm_set_epi32(INT32_C(-1138893231), INT32_C( -687161637), INT32_C( 1828175063), INT32_C( -389420023)),
      simde_mm_set_epi32(INT32_C( -193211433), INT32_C( -170827535), INT32_C( -448329300), INT32_C(  -43684120)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_rem_epi32(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_i32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_rem_epi64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_mm_set_epi64x(INT64_C( 7505871096235581515), INT64_C( 2265477367564496986)),
      simde_mm_set_epi64x(INT64_C(-4636488523262038415), INT64_C( 6479913377553186648)),
      simde_mm_set_epi64x(INT64_C( 2869382572973543100), INT64_C( 2265477367564496986)) },
    { simde_mm_set_epi64x(INT64_C(-1609208390309195578), INT64_C( 6738163160628300797)),
      simde_mm_set_epi64x(INT64_C( 2968342496979776051), INT64_C(-8441478558707775203)),
      simde_mm_set_epi64x(INT64_C(-1609208390309195578), INT64_C( 6738163160628300797)) },
    { simde_mm_set_epi64x(INT64_C( 6409558907924801050), INT64_C( 5372227444888762251)),
      simde_mm_set_epi64x(INT64_C( 5120732502404950997), INT64_C( 5681284513410730040)),
      simde_mm_set_epi64x(INT64_C( 1288826405519850053), INT64_C( 5372227444888762251)) },
    { simde_mm_set_epi64x(INT64_C(-6813223735121976043), INT64_C(  216242550290965460)),
      simde_mm_set_epi64x(INT64_C(-7579804969095623833), INT64_C(-7070918910501808185)),
      simde_mm_set_epi64x(INT64_C(-6813223735121976043), INT64_C(  216242550290965460)) },
    { simde_mm_set_epi64x(INT64_C( 2114726288902596757), INT64_C(-8964374488360902150)),
      simde_mm_set_epi64x(INT64_C( 2118113466433927893), INT64_C( 3840651400764901876)),
      simde_mm_set_epi64x(INT64_C( 2114726288902596757), INT64_C(-1283071686831098398)) },
    { simde_mm_set_epi64x(INT64_C(-4313283826698320649), INT64_C(-2023206435041636446)),
      simde_mm_set_epi64x(INT64_C( 5645659480511055559), INT64_C(-3174015343225263359)),
      simde_mm_set_epi64x(INT64_C(-4313283826698320649), INT64_C(-2023206435041636446)) },
    { simde_mm_set_epi64x(INT64_C( 2328100732832272381), INT64_C( 4111895855610225675)),
      simde_mm_set_epi64x(INT64_C( 7345032902979795528), INT64_C(-6117610524196633789)),
      simde_mm_set_epi64x(INT64_C( 2328100732832272381), INT64_C( 4111895855610225675)) },
    { simde_mm_set_epi64x(INT64_C( -829836782511317044), INT64_C(-1925559678644969716)),
      simde_mm_set_epi64x(INT64_C(-4891509177172967717), INT64_C( 7851952110853286921)),
      simde_mm_set_epi64x(INT64_C( -829836782511317044), INT64_C(-1925559678644969716)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_rem_epi64(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_i64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_rem_epu8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_x_mm_set_epu8(UINT8_C(104), UINT8_C( 42), UINT8_C( 53), UINT8_C(254),
                          UINT8_C(132), UINT8_C(254), UINT8_C( 96), UINT8_C( 75),
                          UINT8_C( 31), UINT8_C(112), UINT8_C(151), UINT8_C(169),
                          UINT8_C(172), UINT8_C( 94), UINT8_C(112), UINT8_C( 90)),
      simde_x_mm_set_epu8(UINT8_C(191), UINT8_C(167), UINT8_C(226), UINT8_C( 64),
                          UINT8_C(213), UINT8_C(202), UINT8_C(110), UINT8_C(113),
                          UINT8_C( 89), UINT8_C(237), UINT8_C( 70), UINT8_C(226),
                          UINT8_C(132), UINT8_C( 91), UINT8_C(255), UINT8_C( 88)),
      simde_x_mm_set_epu8(UINT8_C(104), UINT8_C( 42), UINT8_C( 53), UINT8_C( 62),
                          UINT8_C(132), UINT8_C( 52), UINT8_C( 96), UINT8_C( 75),
                          UINT8_C( 31), UINT8_C(112), UINT8_C( 11), UINT8_C(169),
                          UINT8_C( 40), UINT8_C(  3), UINT8_C(112), UINT8_C(  2)) },
    { simde_x_mm_set_epu8(UINT8_C(233), UINT8_C(170), UINT8_C(241), UINT8_C(126),
                          UINT8_C(182), UINT8_C( 10), UINT8_C(208), UINT8_C(198),
                          UINT8_C( 93), UINT8_C(130), UINT8_C(195), UINT8_C(177),
                          UINT8_C(187), UINT8_C(223), UINT8_C(139), UINT8_C(253)),
      simde_x_mm_set_epu8(UINT8_C( 41), UINT8_C( 49), UINT8_C(171), UINT8_C(198),
                          UINT8_C( 40), UINT8_C( 44), UINT8_C(242), UINT8_C( 51),
                          UINT8_C(138), UINT8_C(217), UINT8_C(215), UINT8_C(249),
                          UINT8_C(201), UINT8_C( 37), UINT8_C(137), UINT8_C( 29)),
      simde_x_mm_set_epu8(UINT8_C( 28), UINT8_C( 23), UINT8_C( 70), UINT8_C(126),
                          UINT8_C( 22), UINT8_C( 10), UINT8_C(208), UINT8_C( 45),
                          UINT8_C( 93), UINT8_C(130), UINT8_C(195), UINT8_C(177),
                          UINT8_C(187), UINT8_C(  1), UINT8_C(  2), UINT8_C( 21)) },
    { simde_x_mm_set_epu8(UINT8_C( 88), UINT8_C(243), UINT8_C( 83), UINT8_C(222),
                          UINT8_C( 17), UINT8_C(204), UINT8_C(102), UINT8_C( 26),
                          UINT8_C( 74), UINT8_C(141), UINT8_C(252), UINT8_C(101),
                          UINT8_C(217), UINT8_C( 50), UINT8_C(247), UINT8_C(139)),
      simde_x_mm_set_epu8(UINT8_C( 71), UINT8_C( 16), UINT8_C(127), UINT8_C( 20),
                          UINT8_C(131), UINT8_C(164), UINT8_C(235), UINT8_C(213),
                          UINT8_C( 78), UINT8_C(215), UINT8_C(250), UINT8_C( 42),
                          UINT8_C(  9), UINT8_C(198), UINT8_C( 72), UINT8_C( 56)),
      simde_x_mm_set_epu8(UINT8_C( 17), UINT8_C(  3), UINT8_C( 83), UINT8_C(  2),
                          UINT8_C( 17), UINT8_C( 40), UINT8_C(102), UINT8_C( 26),
                          UINT8_C( 74), UINT8_C(141), UINT8_C(  2), UINT8_C( 17),
                          UINT8_C(  1), UINT8_C( 50), UINT8_C( 31), UINT8_C( 27)) },
    { simde_x_mm_set_epu8(UINT8_C(161), UINT8_C(114), UINT8_C(145), UINT8_C( 28),
                          UINT8_C(100), UINT8_C(203), UINT8_C(101), UINT8_C( 21),
                          UINT8_C(  3), UINT8_C(  0), UINT8_C( 63), UINT8_C(116),
                          UINT8_C( 43), UINT8_C(106), UINT8_C(227), UINT8_C(212)),
      simde_x_mm_set_epu8(UINT8_C(150), UINT8_C(207), UINT8_C( 31), UINT8_C(138),
                          UINT8_C( 70), UINT8_C( 80), UINT8_C(139), UINT8_C(103),
                          UINT8_C(157), UINT8_C(223), UINT8_C( 12), UINT8_C(182),
                          UINT8_C(215), UINT8_C(242), UINT8_C(151), UINT8_C(199)),
      simde_x_mm_set_epu8(UINT8_C( 11), UINT8_C(114), UINT8_C( 21), UINT8_C( 28),
                          UINT8_C( 30), UINT8_C( 43), UINT8_C(101), UINT8_C( 21),
                          UINT8_C(  3), UINT8_C(  0), UINT8_C(  3), UINT8_C(116),
                          UINT8_C( 43), UINT8_C(106), UINT8_C( 76), UINT8_C( 13)) },
    { simde_x_mm_set_epu8(UINT8_C( 29), UINT8_C( 89), UINT8_C(  4), UINT8_C( 90),
                          UINT8_C(255), UINT8_C( 56), UINT8_C( 40), UINT8_C(149),
                          UINT8_C(131), UINT8_C(152), UINT8_C( 36), UINT8_C(229),
                          UINT8_C(235), UINT8_C(172), UINT8_C(161), UINT8_C(250)),
      simde_x_mm_set_epu8(UINT8_C( 29), UINT8_C(101), UINT8_C( 12), UINT8_C(249),
                          UINT8_C(184), UINT8_C(195), UINT8_C(250), UINT8_C(213),
                          UINT8_C( 53), UINT8_C( 76), UINT8_C(188), UINT8_C( 25),
                          UINT8_C(176), UINT8_C(178), UINT8_C(201), UINT8_C(244)),
      simde_x_mm_set_epu8(UINT8_C(  0), UINT8_C( 89), UINT8_C(  4), UINT8_C( 90),
                          UINT8_C( 71), UINT8_C( 56), UINT8_C( 40), UINT8_C(149),
                          UINT8_C( 25), UINT8_C(  0), UINT8_C( 36), UINT8_C(  4),
                          UINT8_C( 59), UINT8_C(172), UINT8_C(161), UINT8_C(  6)) },
    { simde_x_mm_set_epu8(UINT8_C(196), UINT8_C( 36), UINT8_C( 35), UINT8_C( 54),
                          UINT8_C( 94), UINT8_C( 53), UINT8_C(132), UINT8_C(247),
                          UINT8_C(227), UINT8_C(236), UINT8_C( 32), UINT8_C(119),
                          UINT8_C(124), UINT8_C( 15), UINT8_C( 15), UINT8_C(162)),
      simde_x_mm_set_epu8(UINT8_C( 78), UINT8_C( 89), UINT8_C(105), UINT8_C( 98),
                          UINT8_C(178), UINT8_C(173), UINT8_C(134), UINT8_C(199),
                          UINT8_C(211), UINT8_C(243), UINT8_C(161), UINT8_C(220),
                          UINT8_C(171), UINT8_C(107), UINT8_C( 43), UINT8_C(  1)),
      simde_x_mm_set_epu8(UINT8_C( 40), UINT8_C( 36), UINT8_C( 35), UINT8_C( 54),
                          UINT8_C( 94), UINT8_C( 53), UINT8_C(132), UINT8_C( 48),
                          UINT8_C( 16), UINT8_C(236), UINT8_C( 32), UINT8_C(119),
                          UINT8_C(124), UINT8_C( 15), UINT8_C( 15), UINT8_C(  0)) },
    { simde_x_mm_set_epu8(UINT8_C( 32), UINT8_C( 79), UINT8_C( 19), UINT8_C( 72),
                          UINT8_C( 29), UINT8_C(203), UINT8_C( 79), UINT8_C(253),
                          UINT8_C( 57), UINT8_C( 16), UINT8_C( 99), UINT8_C(126),
                          UINT8_C(179), UINT8_C( 12), UINT8_C(100), UINT8_C( 11)),
      simde_x_mm_set_epu8(UINT8_C(101), UINT8_C(238), UINT8_C(204), UINT8_C(130),
                          UINT8_C(117), UINT8_C(170), UINT8_C(186), UINT8_C( 72),
                          UINT8_C(171), UINT8_C( 25), UINT8_C(225), UINT8_C(164),
                          UINT8_C(  7), UINT8_C( 17), UINT8_C(131), UINT8_C( 67)),
      simde_x_mm_set_epu8(UINT8_C( 32), UINT8_C( 79), UINT8_C( 19), UINT8_C( 72),
                          UINT8_C( 29), UINT8_C( 33), UINT8_C( 79), UINT8_C( 37),
                          UINT8_C( 57), UINT8_C( 16), UINT8_C( 99), UINT8_C(126),
                          UINT8_C(  4), UINT8_C( 12), UINT8_C(100), UINT8_C( 11)) },
    { simde_x_mm_set_epu8(UINT8_C(244), UINT8_C(123), UINT8_C(211), UINT8_C(215),
                          UINT8_C(204), UINT8_C(220), UINT8_C( 31), UINT8_C(204),
                          UINT8_C(229), UINT8_C( 71), UINT8_C(  9), UINT8_C(172),
                          UINT8_C(160), UINT8_C(141), UINT8_C( 31), UINT8_C( 12)),
      simde_x_mm_set_epu8(UINT8_C(188), UINT8_C( 29), UINT8_C(222), UINT8_C( 81),
                          UINT8_C(215), UINT8_C( 10), UINT8_C(190), UINT8_C(219),
                          UINT8_C(108), UINT8_C(247), UINT8_C(188), UINT8_C(215),
                          UINT8_C(232), UINT8_C(201), UINT8_C(236), UINT8_C(  9)),
      simde_x_mm_set_epu8(UINT8_C( 56), UINT8_C(  7), UINT8_C(211), UINT8_C( 53),
                          UINT8_C(204), UINT8_C(  0), UINT8_C( 31), UINT8_C(204),
                          UINT8_C( 13), UINT8_C( 71), UINT8_C(  9), UINT8_C(172),
                          UINT8_C(160), UINT8_C(141), UINT8_C( 31), UINT8_C(  3)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_rem_epu8(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_u8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_rem_epu16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_x_mm_set_epu16(UINT16_C(26666), UINT16_C(13822), UINT16_C(34046), UINT16_C(24651),
                           UINT16_C( 8048), UINT16_C(38825), UINT16_C(44126), UINT16_C(28762)),
      simde_x_mm_set_epu16(UINT16_C(49063), UINT16_C(57920), UINT16_C(54730), UINT16_C(28273),
                           UINT16_C(23021), UINT16_C(18146), UINT16_C(33883), UINT16_C(65368)),
      simde_x_mm_set_epu16(UINT16_C(26666), UINT16_C(13822), UINT16_C(34046), UINT16_C(24651),
                           UINT16_C( 8048), UINT16_C( 2533), UINT16_C(10243), UINT16_C(28762)) },
    { simde_x_mm_set_epu16(UINT16_C(59818), UINT16_C(61822), UINT16_C(46602), UINT16_C(53446),
                           UINT16_C(23938), UINT16_C(50097), UINT16_C(48095), UINT16_C(35837)),
      simde_x_mm_set_epu16(UINT16_C(10545), UINT16_C(43974), UINT16_C(10284), UINT16_C(62003),
                           UINT16_C(35545), UINT16_C(55289), UINT16_C(51493), UINT16_C(35101)),
      simde_x_mm_set_epu16(UINT16_C( 7093), UINT16_C(17848), UINT16_C( 5466), UINT16_C(53446),
                           UINT16_C(23938), UINT16_C(50097), UINT16_C(48095), UINT16_C(  736)) },
    { simde_x_mm_set_epu16(UINT16_C(22771), UINT16_C(21470), UINT16_C( 4556), UINT16_C(26138),
                           UINT16_C(19085), UINT16_C(64613), UINT16_C(55602), UINT16_C(63371)),
      simde_x_mm_set_epu16(UINT16_C(18192), UINT16_C(32532), UINT16_C(33700), UINT16_C(60373),
                           UINT16_C(20183), UINT16_C(64042), UINT16_C( 2502), UINT16_C(18488)),
      simde_x_mm_set_epu16(UINT16_C( 4579), UINT16_C(21470), UINT16_C( 4556), UINT16_C(26138),
                           UINT16_C(19085), UINT16_C(  571), UINT16_C(  558), UINT16_C( 7907)) },
    { simde_x_mm_set_epu16(UINT16_C(41330), UINT16_C(37148), UINT16_C(25803), UINT16_C(25877),
                           UINT16_C(  768), UINT16_C(16244), UINT16_C(11114), UINT16_C(58324)),
      simde_x_mm_set_epu16(UINT16_C(38607), UINT16_C( 8074), UINT16_C(18000), UINT16_C(35687),
                           UINT16_C(40415), UINT16_C( 3254), UINT16_C(55282), UINT16_C(38855)),
      simde_x_mm_set_epu16(UINT16_C( 2723), UINT16_C( 4852), UINT16_C( 7803), UINT16_C(25877),
                           UINT16_C(  768), UINT16_C( 3228), UINT16_C(11114), UINT16_C(19469)) },
    { simde_x_mm_set_epu16(UINT16_C( 7513), UINT16_C( 1114), UINT16_C(65336), UINT16_C(10389),
                           UINT16_C(33688), UINT16_C( 9445), UINT16_C(60332), UINT16_C(41466)),
      simde_x_mm_set_epu16(UINT16_C( 7525), UINT16_C( 3321), UINT16_C(47299), UINT16_C(64213),
                           UINT16_C(13644), UINT16_C(48153), UINT16_C(45234), UINT16_C(51700)),
      simde_x_mm_set_epu16(UINT16_C( 7513), UINT16_C( 1114), UINT16_C(18037), UINT16_C(10389),
                           UINT16_C( 6400), UINT16_C( 9445), UINT16_C(15098), UINT16_C(41466)) },
    { simde_x_mm_set_epu16(UINT16_C(50212), UINT16_C( 9014), UINT16_C(24117), UINT16_C(34039),
                           UINT16_C(58348), UINT16_C( 8311), UINT16_C(31759), UINT16_C( 4002)),
      simde_x_mm_set_epu16(UINT16_C(20057), UINT16_C(26978), UINT16_C(45741), UINT16_C(34503),
                           UINT16_C(54259), UINT16_C(41436), UINT16_C(43883), UINT16_C(11009)),
      simde_x_mm_set_epu16(UINT16_C(10098), UINT16_C( 9014), UINT16_C(24117), UINT16_C(34039),
                           UINT16_C( 4089), UINT16_C( 8311), UINT16_C(31759), UINT16_C( 4002)) },
    { simde_x_mm_set_epu16(UINT16_C( 8271), UINT16_C( 4936), UINT16_C( 7627), UINT16_C(20477),
                           UINT16_C(14608), UINT16_C(25470), UINT16_C(45836), UINT16_C(25611)),
      simde_x_mm_set_epu16(UINT16_C(26094), UINT16_C(52354), UINT16_C(30122), UINT16_C(47688),
                           UINT16_C(43801), UINT16_C(57764), UINT16_C( 1809), UINT16_C(33603)),
      simde_x_mm_set_epu16(UINT16_C( 8271), UINT16_C( 4936), UINT16_C( 7627), UINT16_C(20477),
                           UINT16_C(14608), UINT16_C(25470), UINT16_C(  611), UINT16_C(25611)) },
    { simde_x_mm_set_epu16(UINT16_C(62587), UINT16_C(54231), UINT16_C(52444), UINT16_C( 8140),
                           UINT16_C(58695), UINT16_C( 2476), UINT16_C(41101), UINT16_C( 7948)),
      simde_x_mm_set_epu16(UINT16_C(48157), UINT16_C(56913), UINT16_C(55050), UINT16_C(48859),
                           UINT16_C(27895), UINT16_C(48343), UINT16_C(59593), UINT16_C(60425)),
      simde_x_mm_set_epu16(UINT16_C(14430), UINT16_C(54231), UINT16_C(52444), UINT16_C( 8140),
                           UINT16_C( 2905), UINT16_C( 2476), UINT16_C(41101), UINT16_C( 7948)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_rem_epu16(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_u16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_rem_epu32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_x_mm_set_epu32(UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C(2891870298)),
      simde_x_mm_set_epu32(UINT32_C(3215450688), UINT32_C(3586813553), UINT32_C(1508722402), UINT32_C(2220621656)),
      simde_x_mm_set_epu32(UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C( 671248642)) },
    { simde_x_mm_set_epu32(UINT32_C(3920294270), UINT32_C(3054162118), UINT32_C(1568850865), UINT32_C(3151989757)),
      simde_x_mm_set_epu32(UINT32_C( 691121094), UINT32_C( 674034227), UINT32_C(2329532409), UINT32_C(3374680349)),
      simde_x_mm_set_epu32(UINT32_C( 464688800), UINT32_C( 358025210), UINT32_C(1568850865), UINT32_C(3151989757)) },
    { simde_x_mm_set_epu32(UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(1250819173), UINT32_C(3643996043)),
      simde_x_mm_set_epu32(UINT32_C(1192263444), UINT32_C(2208623573), UINT32_C(1322777130), UINT32_C( 163989560)),
      simde_x_mm_set_epu32(UINT32_C( 300078282), UINT32_C( 298608154), UINT32_C(1250819173), UINT32_C(  36225723)) },
    { simde_x_mm_set_epu32(UINT32_C(2708640028), UINT32_C(1691051285), UINT32_C(  50347892), UINT32_C( 728425428)),
      simde_x_mm_set_epu32(UINT32_C(2530156426), UINT32_C(1179683687), UINT32_C(2648640694), UINT32_C(3623000007)),
      simde_x_mm_set_epu32(UINT32_C( 178483602), UINT32_C( 511367598), UINT32_C(  50347892), UINT32_C( 728425428)) },
    { simde_x_mm_set_epu32(UINT32_C( 492373082), UINT32_C(4281870485), UINT32_C(2207786213), UINT32_C(3953959418)),
      simde_x_mm_set_epu32(UINT32_C( 493161721), UINT32_C(3099851477), UINT32_C( 894221337), UINT32_C(2964507124)),
      simde_x_mm_set_epu32(UINT32_C( 492373082), UINT32_C(1182019008), UINT32_C( 419343539), UINT32_C( 989452294)) },
    { simde_x_mm_set_epu32(UINT32_C(3290702646), UINT32_C(1580565751), UINT32_C(3823902839), UINT32_C(2081361826)),
      simde_x_mm_set_epu32(UINT32_C(1314482530), UINT32_C(2997716679), UINT32_C(3555959260), UINT32_C(2875927297)),
      simde_x_mm_set_epu32(UINT32_C( 661737586), UINT32_C(1580565751), UINT32_C( 267943579), UINT32_C(2081361826)) },
    { simde_x_mm_set_epu32(UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358), UINT32_C(3003933707)),
      simde_x_mm_set_epu32(UINT32_C(1710148738), UINT32_C(1974123080), UINT32_C(2870600100), UINT32_C( 118588227)),
      simde_x_mm_set_epu32(UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358), UINT32_C(  39228032)) },
    { simde_x_mm_set_epu32(UINT32_C(4101755863), UINT32_C(3436978124), UINT32_C(3846637996), UINT32_C(2693603084)),
      simde_x_mm_set_epu32(UINT32_C(3156074065), UINT32_C(3607805659), UINT32_C(1828175063), UINT32_C(3905547273)),
      simde_x_mm_set_epu32(UINT32_C( 945681798), UINT32_C(3436978124), UINT32_C( 190287870), UINT32_C(2693603084)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_rem_epu32(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_u32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm_rem_epu64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i r;
  } test_vec[8] = {
    { simde_x_mm_set_epu64x(UINT64_C( 7505871096235581515), UINT64_C( 2265477367564496986)),
      simde_x_mm_set_epu64x(UINT64_C(13810255550447513201), UINT64_C( 6479913377553186648)),
      simde_x_mm_set_epu64x(UINT64_C( 7505871096235581515), UINT64_C( 2265477367564496986)) },
    { simde_x_mm_set_epu64x(UINT64_C(16837535683400356038), UINT64_C( 6738163160628300797)),
      simde_x_mm_set_epu64x(UINT64_C( 2968342496979776051), UINT64_C(10005265515001776413)),
      simde_x_mm_set_epu64x(UINT64_C( 1995823198501475783), UINT64_C( 6738163160628300797)) },
    { simde_x_mm_set_epu64x(UINT64_C( 6409558907924801050), UINT64_C( 5372227444888762251)),
      simde_x_mm_set_epu64x(UINT64_C( 5120732502404950997), UINT64_C( 5681284513410730040)),
      simde_x_mm_set_epu64x(UINT64_C( 1288826405519850053), UINT64_C( 5372227444888762251)) },
    { simde_x_mm_set_epu64x(UINT64_C(11633520338587575573), UINT64_C(  216242550290965460)),
      simde_x_mm_set_epu64x(UINT64_C(10866939104613927783), UINT64_C(11375825163207743431)),
      simde_x_mm_set_epu64x(UINT64_C(  766581233973647790), UINT64_C(  216242550290965460)) },
    { simde_x_mm_set_epu64x(UINT64_C( 2114726288902596757), UINT64_C( 9482369585348649466)),
      simde_x_mm_set_epu64x(UINT64_C( 2118113466433927893), UINT64_C( 3840651400764901876)),
      simde_x_mm_set_epu64x(UINT64_C( 2114726288902596757), UINT64_C( 1801066783818845714)) },
    { simde_x_mm_set_epu64x(UINT64_C(14133460247011230967), UINT64_C(16423537638667915170)),
      simde_x_mm_set_epu64x(UINT64_C( 5645659480511055559), UINT64_C(15272728730484288257)),
      simde_x_mm_set_epu64x(UINT64_C( 2842141285989119849), UINT64_C( 1150808908183626913)) },
    { simde_x_mm_set_epu64x(UINT64_C( 2328100732832272381), UINT64_C( 4111895855610225675)),
      simde_x_mm_set_epu64x(UINT64_C( 7345032902979795528), UINT64_C(12329133549512917827)),
      simde_x_mm_set_epu64x(UINT64_C( 2328100732832272381), UINT64_C( 4111895855610225675)) },
    { simde_x_mm_set_epu64x(UINT64_C(17616907291198234572), UINT64_C(16521184395064581900)),
      simde_x_mm_set_epu64x(UINT64_C(13555234896536583899), UINT64_C( 7851952110853286921)),
      simde_x_mm_set_epu64x(UINT64_C( 4061672394661650673), UINT64_C(  817280173358008058)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i r = simde_mm_rem_epu64(test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_u64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_rem_epi8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_mm256_set_epi8(INT8_C( -65), INT8_C( -89), INT8_C( -30), INT8_C(  64),
                           INT8_C( -43), INT8_C( -54), INT8_C( 110), INT8_C( 113),
                           INT8_C(  89), INT8_C( -19), INT8_C(  70), INT8_C( -30),
                           INT8_C(-124), INT8_C(  91), INT8_C(  -1), INT8_C(  88),
                           INT8_C( 104), INT8_C(  42), INT8_C(  53), INT8_C(  -2),
                           INT8_C(-124), INT8_C(  -2), INT8_C(  96), INT8_C(  75),
                           INT8_C(  31), INT8_C( 112), INT8_C(-105), INT8_C( -87),
                           INT8_C( -84), INT8_C(  94), INT8_C( 112), INT8_C(  90)),
      simde_mm256_set_epi8(INT8_C( 121), INT8_C(  85), INT8_C(-103), INT8_C( 116),
                           INT8_C( -38), INT8_C(  21), INT8_C( 101), INT8_C( 122),
                           INT8_C(  10), INT8_C( -25), INT8_C(  54), INT8_C(  71),
                           INT8_C(-100), INT8_C(-107), INT8_C( -12), INT8_C(  84),
                           INT8_C(-108), INT8_C(  85), INT8_C( -86), INT8_C( -72),
                           INT8_C(  94), INT8_C(-102), INT8_C( -27), INT8_C(  11),
                           INT8_C(  70), INT8_C( -77), INT8_C( 121), INT8_C( -99),
                           INT8_C(  -2), INT8_C(  70), INT8_C(  49), INT8_C( 125)),
      simde_mm256_set_epi8(INT8_C( -65), INT8_C(  -4), INT8_C( -30), INT8_C(  64),
                           INT8_C(  -5), INT8_C( -12), INT8_C(   9), INT8_C( 113),
                           INT8_C(   9), INT8_C( -19), INT8_C(  16), INT8_C( -30),
                           INT8_C( -24), INT8_C(  91), INT8_C(  -1), INT8_C(   4),
                           INT8_C( 104), INT8_C(  42), INT8_C(  53), INT8_C(  -2),
                           INT8_C( -30), INT8_C(  -2), INT8_C(  15), INT8_C(   9),
                           INT8_C(  31), INT8_C(  35), INT8_C(-105), INT8_C( -87),
                           INT8_C(   0), INT8_C(  24), INT8_C(  14), INT8_C(  90)) },
    { simde_mm256_set_epi8(INT8_C(  78), INT8_C(  89), INT8_C( 105), INT8_C(  98),
                           INT8_C( -78), INT8_C( -83), INT8_C(-122), INT8_C( -57),
                           INT8_C( -45), INT8_C( -13), INT8_C( -95), INT8_C( -36),
                           INT8_C( -85), INT8_C( 107), INT8_C(  43), INT8_C(   1),
                           INT8_C( -60), INT8_C(  36), INT8_C(  35), INT8_C(  54),
                           INT8_C(  94), INT8_C(  53), INT8_C(-124), INT8_C(  -9),
                           INT8_C( -29), INT8_C( -20), INT8_C(  32), INT8_C( 119),
                           INT8_C( 124), INT8_C(  15), INT8_C(  15), INT8_C( -94)),
      simde_mm256_set_epi8(INT8_C( -61), INT8_C(  49), INT8_C(  14), INT8_C( -86),
                           INT8_C( -53), INT8_C( -89), INT8_C(   3), INT8_C( -41),
                           INT8_C(  63), INT8_C(  -8), INT8_C(  55), INT8_C( -37),
                           INT8_C( -35), INT8_C(-121), INT8_C(  61), INT8_C( -65),
                           INT8_C( -47), INT8_C(  91), INT8_C(  87), INT8_C(-119),
                           INT8_C(  87), INT8_C(  76), INT8_C(  44), INT8_C(-116),
                           INT8_C(   2), INT8_C( -56), INT8_C(  36), INT8_C( -61),
                           INT8_C( -56), INT8_C( 125), INT8_C(  -2), INT8_C(-117)),
      simde_mm256_set_epi8(INT8_C(  17), INT8_C(  40), INT8_C(   7), INT8_C(  12),
                           INT8_C( -25), INT8_C( -83), INT8_C(  -2), INT8_C( -16),
                           INT8_C( -45), INT8_C(  -5), INT8_C( -40), INT8_C( -36),
                           INT8_C( -15), INT8_C( 107), INT8_C(  43), INT8_C(   1),
                           INT8_C( -13), INT8_C(  36), INT8_C(  35), INT8_C(  54),
                           INT8_C(   7), INT8_C(  53), INT8_C( -36), INT8_C(  -9),
                           INT8_C(  -1), INT8_C( -20), INT8_C(  32), INT8_C(  58),
                           INT8_C(  12), INT8_C(  15), INT8_C(   1), INT8_C( -94)) },
    { simde_mm256_set_epi8(INT8_C( -22), INT8_C(  94), INT8_C( -16), INT8_C(  12),
                           INT8_C(-110), INT8_C(   1), INT8_C(-109), INT8_C(  59),
                           INT8_C(  -3), INT8_C(  26), INT8_C(  26), INT8_C(  40),
                           INT8_C(  12), INT8_C(   2), INT8_C( -26), INT8_C(-111),
                           INT8_C( -86), INT8_C( 105), INT8_C( 111), INT8_C( -96),
                           INT8_C(-116), INT8_C( -54), INT8_C( -90), INT8_C( -36),
                           INT8_C( -69), INT8_C(  65), INT8_C(  -6), INT8_C( -61),
                           INT8_C(  33), INT8_C(-125), INT8_C(   2), INT8_C( -92)),
      simde_mm256_set_epi8(INT8_C( -79), INT8_C( -35), INT8_C(  -5), INT8_C( -75),
                           INT8_C( -97), INT8_C( -74), INT8_C(  11), INT8_C(  11),
                           INT8_C(  39), INT8_C(  37), INT8_C(  39), INT8_C( -48),
                           INT8_C(-120), INT8_C( -76), INT8_C( -41), INT8_C(-117),
                           INT8_C(-112), INT8_C(-128), INT8_C( -53), INT8_C( -50),
                           INT8_C( -83), INT8_C(  36), INT8_C(-123), INT8_C( -81),
                           INT8_C( -25), INT8_C(   7), INT8_C( -20), INT8_C(  68),
                           INT8_C( -63), INT8_C( -35), INT8_C(  27), INT8_C(   8)),
      simde_mm256_set_epi8(INT8_C( -22), INT8_C(  24), INT8_C(  -1), INT8_C(  12),
                           INT8_C( -13), INT8_C(   1), INT8_C( -10), INT8_C(   4),
                           INT8_C(  -3), INT8_C(  26), INT8_C(  26), INT8_C(  40),
                           INT8_C(  12), INT8_C(   2), INT8_C( -26), INT8_C(-111),
                           INT8_C( -86), INT8_C( 105), INT8_C(   5), INT8_C( -46),
                           INT8_C( -33), INT8_C( -18), INT8_C( -90), INT8_C( -36),
                           INT8_C( -19), INT8_C(   2), INT8_C(  -6), INT8_C( -61),
                           INT8_C(  33), INT8_C( -20), INT8_C(   2), INT8_C(  -4)) },
    { simde_mm256_set_epi8(INT8_C(  71), INT8_C( -23), INT8_C(  74), INT8_C( 125),
                           INT8_C(  81), INT8_C( -13), INT8_C(-117), INT8_C( -66),
                           INT8_C(  31), INT8_C( -80), INT8_C(  97), INT8_C(  -3),
                           INT8_C( 123), INT8_C( -80), INT8_C( -40), INT8_C( 108),
                           INT8_C(  -9), INT8_C(  97), INT8_C(  75), INT8_C( -53),
                           INT8_C(-128), INT8_C( -18), INT8_C(  79), INT8_C(-115),
                           INT8_C(  86), INT8_C(  29), INT8_C( -93), INT8_C( -49),
                           INT8_C( 111), INT8_C(  -7), INT8_C(-117), INT8_C( -47)),
      simde_mm256_set_epi8(INT8_C( 120), INT8_C( 127), INT8_C(  28), INT8_C(  95),
                           INT8_C( -81), INT8_C( -33), INT8_C( 119), INT8_C( -42),
                           INT8_C( -36), INT8_C( 102), INT8_C(  86), INT8_C(  22),
                           INT8_C( 119), INT8_C( -49), INT8_C(  12), INT8_C( -73),
                           INT8_C( -84), INT8_C( -14), INT8_C( -83), INT8_C(  -7),
                           INT8_C(  52), INT8_C( 108), INT8_C(-128), INT8_C( -53),
                           INT8_C(  85), INT8_C(-121), INT8_C( -29), INT8_C(  35),
                           INT8_C( -69), INT8_C(  24), INT8_C(  -6), INT8_C( -37)),
      simde_mm256_set_epi8(INT8_C(  71), INT8_C( -23), INT8_C(  18), INT8_C(  30),
                           INT8_C(   0), INT8_C( -13), INT8_C(-117), INT8_C( -24),
                           INT8_C(  31), INT8_C( -80), INT8_C(  11), INT8_C(  -3),
                           INT8_C(   4), INT8_C( -31), INT8_C(  -4), INT8_C(  35),
                           INT8_C(  -9), INT8_C(  13), INT8_C(  75), INT8_C(  -4),
                           INT8_C( -24), INT8_C( -18), INT8_C(  79), INT8_C(  -9),
                           INT8_C(   1), INT8_C(  29), INT8_C(  -6), INT8_C( -14),
                           INT8_C(  42), INT8_C(  -7), INT8_C(  -3), INT8_C( -10)) },
    { simde_mm256_set_epi8(INT8_C( -72), INT8_C(  63), INT8_C(  95), INT8_C( -92),
                           INT8_C(  65), INT8_C(  71), INT8_C( -82), INT8_C(  88),
                           INT8_C( -73), INT8_C(-114), INT8_C(  98), INT8_C(  14),
                           INT8_C(  25), INT8_C( -83), INT8_C(  87), INT8_C(   2),
                           INT8_C( -65), INT8_C(-113), INT8_C(-104), INT8_C(   2),
                           INT8_C( 126), INT8_C(   0), INT8_C( -94), INT8_C(  57),
                           INT8_C( -11), INT8_C(  36), INT8_C( -17), INT8_C(  54),
                           INT8_C(  33), INT8_C( -91), INT8_C( -57), INT8_C(  84)),
      simde_mm256_set_epi8(INT8_C( -82), INT8_C(  60), INT8_C(-124), INT8_C( -48),
                           INT8_C(  58), INT8_C( -78), INT8_C( 116), INT8_C( -16),
                           INT8_C(  37), INT8_C(-125), INT8_C( 100), INT8_C( -79),
                           INT8_C(  19), INT8_C( 102), INT8_C(  81), INT8_C(  86),
                           INT8_C(  25), INT8_C(  43), INT8_C(  51), INT8_C(-116),
                           INT8_C(   9), INT8_C(  40), INT8_C( -29), INT8_C(  75),
                           INT8_C( -48), INT8_C( -97), INT8_C( -81), INT8_C( 109),
                           INT8_C( -26), INT8_C(  87), INT8_C(  -2), INT8_C( -40)),
      simde_mm256_set_epi8(INT8_C( -72), INT8_C(   3), INT8_C(  95), INT8_C( -44),
                           INT8_C(   7), INT8_C(  71), INT8_C( -82), INT8_C(   8),
                           INT8_C( -36), INT8_C(-114), INT8_C(  98), INT8_C(  14),
                           INT8_C(   6), INT8_C( -83), INT8_C(   6), INT8_C(   2),
                           INT8_C( -15), INT8_C( -27), INT8_C(  -2), INT8_C(   2),
                           INT8_C(   0), INT8_C(   0), INT8_C(  -7), INT8_C(  57),
                           INT8_C( -11), INT8_C(  36), INT8_C( -17), INT8_C(  54),
                           INT8_C(   7), INT8_C(  -4), INT8_C(  -1), INT8_C(   4)) },
    { simde_mm256_set_epi8(INT8_C(  54), INT8_C(  43), INT8_C( 109), INT8_C( -69),
                           INT8_C(-118), INT8_C(  62), INT8_C( -34), INT8_C(-102),
                           INT8_C( 123), INT8_C(  21), INT8_C(  -9), INT8_C(  99),
                           INT8_C(  37), INT8_C(  48), INT8_C( 116), INT8_C( -23),
                           INT8_C(  95), INT8_C(  -5), INT8_C(-109), INT8_C( 109),
                           INT8_C( -51), INT8_C( -50), INT8_C(  57), INT8_C(  17),
                           INT8_C( 121), INT8_C(  25), INT8_C(   3), INT8_C(  55),
                           INT8_C( -78), INT8_C(-127), INT8_C(-107), INT8_C( -49)),
      simde_mm256_set_epi8(INT8_C(-125), INT8_C(  42), INT8_C(-105), INT8_C( -46),
                           INT8_C(  12), INT8_C( -93), INT8_C(-118), INT8_C( -49),
                           INT8_C(  43), INT8_C(  57), INT8_C(  61), INT8_C(  62),
                           INT8_C(  81), INT8_C( -72), INT8_C(   6), INT8_C(  93),
                           INT8_C( -89), INT8_C(   1), INT8_C(-111), INT8_C(   9),
                           INT8_C(   4), INT8_C(  17), INT8_C(  10), INT8_C( 101),
                           INT8_C( -70), INT8_C( -75), INT8_C(-101), INT8_C( -13),
                           INT8_C( -67), INT8_C( -65), INT8_C( -34), INT8_C( -51)),
      simde_mm256_set_epi8(INT8_C(  54), INT8_C(   1), INT8_C(   4), INT8_C( -23),
                           INT8_C( -10), INT8_C(  62), INT8_C( -34), INT8_C(  -4),
                           INT8_C(  37), INT8_C(  21), INT8_C(  -9), INT8_C(  37),
                           INT8_C(  37), INT8_C(  48), INT8_C(   2), INT8_C( -23),
                           INT8_C(   6), INT8_C(   0), INT8_C(-109), INT8_C(   1),
                           INT8_C(  -3), INT8_C( -16), INT8_C(   7), INT8_C(  17),
                           INT8_C(  51), INT8_C(  25), INT8_C(   3), INT8_C(   3),
                           INT8_C( -11), INT8_C( -62), INT8_C(  -5), INT8_C( -49)) },
    { simde_mm256_set_epi8(INT8_C(  23), INT8_C(-124), INT8_C( 106), INT8_C( 109),
                           INT8_C(-121), INT8_C( -53), INT8_C(  98), INT8_C( 120),
                           INT8_C( 101), INT8_C(  52), INT8_C(  82), INT8_C(  44),
                           INT8_C(-114), INT8_C(  14), INT8_C(  99), INT8_C( -11),
                           INT8_C(   8), INT8_C(-116), INT8_C(-115), INT8_C( 123),
                           INT8_C( -37), INT8_C( -93), INT8_C( -60), INT8_C( -23),
                           INT8_C(  34), INT8_C( -71), INT8_C( -28), INT8_C( 108),
                           INT8_C(  95), INT8_C( -20), INT8_C(  97), INT8_C(  41)),
      simde_mm256_set_epi8(INT8_C( 125), INT8_C( -27), INT8_C( -53), INT8_C(  45),
                           INT8_C(  24), INT8_C(   5), INT8_C(  90), INT8_C(  83),
                           INT8_C(-111), INT8_C(  85), INT8_C(-100), INT8_C( -92),
                           INT8_C(-107), INT8_C( -55), INT8_C(  48), INT8_C(  -1),
                           INT8_C(  41), INT8_C(  42), INT8_C(  94), INT8_C(-127),
                           INT8_C(-121), INT8_C(   8), INT8_C(  12), INT8_C( -53),
                           INT8_C(-128), INT8_C( -54), INT8_C(-108), INT8_C(  -4),
                           INT8_C( 104), INT8_C( -48), INT8_C(  98), INT8_C( -94)),
      simde_mm256_set_epi8(INT8_C(  23), INT8_C( -16), INT8_C(   0), INT8_C(  19),
                           INT8_C(  -1), INT8_C(  -3), INT8_C(   8), INT8_C(  37),
                           INT8_C( 101), INT8_C(  52), INT8_C(  82), INT8_C(  44),
                           INT8_C(  -7), INT8_C(  14), INT8_C(   3), INT8_C(   0),
                           INT8_C(   8), INT8_C( -32), INT8_C( -21), INT8_C( 123),
                           INT8_C( -37), INT8_C(  -5), INT8_C(   0), INT8_C( -23),
                           INT8_C(  34), INT8_C( -17), INT8_C( -28), INT8_C(   0),
                           INT8_C(  95), INT8_C( -20), INT8_C(  97), INT8_C(  41)) },
    { simde_mm256_set_epi8(INT8_C( -94), INT8_C(  31), INT8_C( -88), INT8_C(  17),
                           INT8_C(  50), INT8_C( 110), INT8_C( -25), INT8_C( -40),
                           INT8_C(  94), INT8_C(  20), INT8_C( -93), INT8_C( -73),
                           INT8_C( -99), INT8_C(  16), INT8_C(  91), INT8_C(  54),
                           INT8_C(  62), INT8_C(  81), INT8_C( -97), INT8_C(-105),
                           INT8_C(  57), INT8_C(  12), INT8_C( 118), INT8_C(  33),
                           INT8_C( -76), INT8_C(-117), INT8_C(   1), INT8_C(   5),
                           INT8_C(  78), INT8_C(  13), INT8_C(  93), INT8_C(-101)),
      simde_mm256_set_epi8(INT8_C( -63), INT8_C( -26), INT8_C(  93), INT8_C(  23),
                           INT8_C( -63), INT8_C(  52), INT8_C( -33), INT8_C( -81),
                           INT8_C( -51), INT8_C(  45), INT8_C( -90), INT8_C(  24),
                           INT8_C(  71), INT8_C( -22), INT8_C( -95), INT8_C(-114),
                           INT8_C( -72), INT8_C( -38), INT8_C( -66), INT8_C( -44),
                           INT8_C( 116), INT8_C( -97), INT8_C(  44), INT8_C(  55),
                           INT8_C( -43), INT8_C(-123), INT8_C(  60), INT8_C(   3),
                           INT8_C(  58), INT8_C(  -1), INT8_C( 125), INT8_C( -67)),
      simde_mm256_set_epi8(INT8_C( -31), INT8_C(   5), INT8_C( -88), INT8_C(  17),
                           INT8_C(  50), INT8_C(   6), INT8_C( -25), INT8_C( -40),
                           INT8_C(  43), INT8_C(  20), INT8_C(  -3), INT8_C(  -1),
                           INT8_C( -28), INT8_C(  16), INT8_C(  91), INT8_C(  54),
                           INT8_C(  62), INT8_C(   5), INT8_C( -31), INT8_C( -17),
                           INT8_C(  57), INT8_C(  12), INT8_C(  30), INT8_C(  33),
                           INT8_C( -33), INT8_C(-117), INT8_C(   1), INT8_C(   2),
                           INT8_C(  20), INT8_C(   0), INT8_C(  93), INT8_C( -34)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_rem_epi8(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_rem_epi16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_mm256_set_epi16(INT16_C(-16473), INT16_C( -7616), INT16_C(-10806), INT16_C( 28273),
                            INT16_C( 23021), INT16_C( 18146), INT16_C(-31653), INT16_C(  -168),
                            INT16_C( 26666), INT16_C( 13822), INT16_C(-31490), INT16_C( 24651),
                            INT16_C(  8048), INT16_C(-26711), INT16_C(-21410), INT16_C( 28762)),
      simde_mm256_set_epi16(INT16_C( 10545), INT16_C(-21562), INT16_C( 10284), INT16_C( -3533),
                            INT16_C(-29991), INT16_C(-10247), INT16_C(-14043), INT16_C(-30435),
                            INT16_C( -5718), INT16_C( -3714), INT16_C(-18934), INT16_C(-12090),
                            INT16_C( 23938), INT16_C(-15439), INT16_C(-17441), INT16_C(-29699)),
      simde_mm256_set_epi16(INT16_C( -5928), INT16_C( -7616), INT16_C(  -522), INT16_C(     9),
                            INT16_C( 23021), INT16_C(  7899), INT16_C( -3567), INT16_C(  -168),
                            INT16_C(  3794), INT16_C(  2680), INT16_C(-12556), INT16_C(   471),
                            INT16_C(  8048), INT16_C(-11272), INT16_C( -3969), INT16_C( 28762)) },
    { simde_mm256_set_epi16(INT16_C( 18192), INT16_C( 32532), INT16_C(-31836), INT16_C( -5163),
                            INT16_C( 20183), INT16_C( -1494), INT16_C(  2502), INT16_C( 18488),
                            INT16_C( 22771), INT16_C( 21470), INT16_C(  4556), INT16_C( 26138),
                            INT16_C( 19085), INT16_C(  -923), INT16_C( -9934), INT16_C( -2165)),
      simde_mm256_set_epi16(INT16_C(-26929), INT16_C(  8074), INT16_C( 18000), INT16_C(-29849),
                            INT16_C(-25121), INT16_C(  3254), INT16_C(-10254), INT16_C(-26681),
                            INT16_C(-24206), INT16_C(-28388), INT16_C( 25803), INT16_C( 25877),
                            INT16_C(   768), INT16_C( 16244), INT16_C( 11114), INT16_C( -7212)),
      simde_mm256_set_epi16(INT16_C( 18192), INT16_C(   236), INT16_C(-13836), INT16_C( -5163),
                            INT16_C( 20183), INT16_C( -1494), INT16_C(  2502), INT16_C( 18488),
                            INT16_C( 22771), INT16_C( 21470), INT16_C(  4556), INT16_C(   261),
                            INT16_C(   653), INT16_C(  -923), INT16_C( -9934), INT16_C( -2165)) },
    { simde_mm256_set_epi16(INT16_C(  7525), INT16_C(  3321), INT16_C(-18237), INT16_C( -1323),
                            INT16_C( 13644), INT16_C(-17383), INT16_C(-20302), INT16_C(-13836),
                            INT16_C(  7513), INT16_C(  1114), INT16_C(  -200), INT16_C( 10389),
                            INT16_C(-31848), INT16_C(  9445), INT16_C( -5204), INT16_C(-24070)),
      simde_mm256_set_epi16(INT16_C( 20057), INT16_C( 26978), INT16_C(-19795), INT16_C(-31033),
                            INT16_C(-11277), INT16_C(-24100), INT16_C(-21653), INT16_C( 11009),
                            INT16_C(-15324), INT16_C(  9014), INT16_C( 24117), INT16_C(-31497),
                            INT16_C( -7188), INT16_C(  8311), INT16_C( 31759), INT16_C(  4002)),
      simde_mm256_set_epi16(INT16_C(  7525), INT16_C(  3321), INT16_C(-18237), INT16_C( -1323),
                            INT16_C(  2367), INT16_C(-17383), INT16_C(-20302), INT16_C( -2827),
                            INT16_C(  7513), INT16_C(  1114), INT16_C(  -200), INT16_C( 10389),
                            INT16_C( -3096), INT16_C(  1134), INT16_C( -5204), INT16_C(   -58)) },
    { simde_mm256_set_epi16(INT16_C( 26094), INT16_C(-13182), INT16_C( 30122), INT16_C(-17848),
                            INT16_C(-21735), INT16_C( -7772), INT16_C(  1809), INT16_C(-31933),
                            INT16_C(  8271), INT16_C(  4936), INT16_C(  7627), INT16_C( 20477),
                            INT16_C( 14608), INT16_C( 25470), INT16_C(-19700), INT16_C( 25611)),
      simde_mm256_set_epi16(INT16_C(-17379), INT16_C( -8623), INT16_C(-10486), INT16_C(-16677),
                            INT16_C( 27895), INT16_C(-17193), INT16_C( -5943), INT16_C( -5111),
                            INT16_C( -2949), INT16_C(-11305), INT16_C(-13092), INT16_C(  8140),
                            INT16_C( -6841), INT16_C(  2476), INT16_C(-24435), INT16_C(  7948)),
      simde_mm256_set_epi16(INT16_C(  8715), INT16_C( -4559), INT16_C(  9150), INT16_C( -1171),
                            INT16_C(-21735), INT16_C( -7772), INT16_C(  1809), INT16_C( -1267),
                            INT16_C(  2373), INT16_C(  4936), INT16_C(  7627), INT16_C(  4197),
                            INT16_C(   926), INT16_C(   710), INT16_C(-19700), INT16_C(  1767)) },
    { simde_mm256_set_epi16(INT16_C( 26466), INT16_C( 21183), INT16_C(  5811), INT16_C( 17016),
                            INT16_C(-14374), INT16_C(-18761), INT16_C(-11284), INT16_C(  -933),
                            INT16_C( 30444), INT16_C( 20573), INT16_C(-14964), INT16_C( 25607),
                            INT16_C(-28815), INT16_C(-28739), INT16_C( 27147), INT16_C( -3265)),
      simde_mm256_set_epi16(INT16_C( 26902), INT16_C(-14525), INT16_C( -7905), INT16_C( -8015),
                            INT16_C(-22131), INT16_C( 18318), INT16_C(-21513), INT16_C(  9770),
                            INT16_C(  4118), INT16_C(-32437), INT16_C(  6621), INT16_C( -7897),
                            INT16_C( 22002), INT16_C(-32381), INT16_C( 15537), INT16_C(-26793)),
      simde_mm256_set_epi16(INT16_C( 26466), INT16_C(  6658), INT16_C(  5811), INT16_C(   986),
                            INT16_C(-14374), INT16_C(  -443), INT16_C(-11284), INT16_C(  -933),
                            INT16_C(  1618), INT16_C( 20573), INT16_C( -1722), INT16_C(  1916),
                            INT16_C( -6813), INT16_C(-28739), INT16_C( 11610), INT16_C( -3265)) },
    { simde_mm256_set_epi16(INT16_C( -5538), INT16_C( -4084), INT16_C(-28159), INT16_C(-27845),
                            INT16_C(  -742), INT16_C(  6696), INT16_C(  3074), INT16_C( -6511),
                            INT16_C(-21911), INT16_C( 28576), INT16_C(-29494), INT16_C(-22820),
                            INT16_C(-17599), INT16_C( -1341), INT16_C(  8579), INT16_C(   676)),
      simde_mm256_set_epi16(INT16_C(-10155), INT16_C(-12697), INT16_C( -5222), INT16_C(-32377),
                            INT16_C( 32076), INT16_C(-13716), INT16_C( 13383), INT16_C(-22332),
                            INT16_C( 18058), INT16_C(-22719), INT16_C( -8799), INT16_C(-25251),
                            INT16_C(-16195), INT16_C(-26213), INT16_C(-12331), INT16_C( 27016)),
      simde_mm256_set_epi16(INT16_C( -5538), INT16_C( -4084), INT16_C( -2049), INT16_C(-27845),
                            INT16_C(  -742), INT16_C(  6696), INT16_C(  3074), INT16_C( -6511),
                            INT16_C( -3853), INT16_C(  5857), INT16_C( -3097), INT16_C(-22820),
                            INT16_C( -1404), INT16_C( -1341), INT16_C(  8579), INT16_C(   676)) },
    { simde_mm256_set_epi16(INT16_C( 13886), INT16_C( 28688), INT16_C( 30551), INT16_C(-28928),
                            INT16_C( -9491), INT16_C(-26549), INT16_C(  -738), INT16_C( 22350),
                            INT16_C(  7981), INT16_C(-15059), INT16_C(-18848), INT16_C( 16804),
                            INT16_C(-31876), INT16_C( -1787), INT16_C( 29649), INT16_C(  -721)),
      simde_mm256_set_epi16(INT16_C(  7566), INT16_C( 25511), INT16_C( -5831), INT16_C( 13989),
                            INT16_C( 13965), INT16_C(-31065), INT16_C(    77), INT16_C(-30384),
                            INT16_C( 21705), INT16_C(-23032), INT16_C( -2503), INT16_C( -8652),
                            INT16_C(-23147), INT16_C( -4009), INT16_C(  7598), INT16_C( 23051)),
      simde_mm256_set_epi16(INT16_C(  6320), INT16_C(  3177), INT16_C(  1396), INT16_C(  -950),
                            INT16_C( -9491), INT16_C(-26549), INT16_C(   -45), INT16_C( 22350),
                            INT16_C(  7981), INT16_C(-15059), INT16_C( -1327), INT16_C(  8152),
                            INT16_C( -8729), INT16_C( -1787), INT16_C(  6855), INT16_C(  -721)) },
    { simde_mm256_set_epi16(INT16_C( 26789), INT16_C(-25295), INT16_C(-31460), INT16_C(-29347),
                            INT16_C(-16029), INT16_C(-32645), INT16_C(-19836), INT16_C( 31541),
                            INT16_C(-32299), INT16_C(-14817), INT16_C( 22782), INT16_C(-18634),
                            INT16_C( -2744), INT16_C(   907), INT16_C(  9939), INT16_C(   395)),
      simde_mm256_set_epi16(INT16_C( 18409), INT16_C( 19069), INT16_C( 20979), INT16_C(-29762),
                            INT16_C(  8112), INT16_C( 25085), INT16_C( 31664), INT16_C(-10132),
                            INT16_C( -2207), INT16_C( 19403), INT16_C(-32530), INT16_C( 20365),
                            INT16_C( 22045), INT16_C(-23601), INT16_C( 28665), INT16_C(-29743)),
      simde_mm256_set_epi16(INT16_C(  8380), INT16_C( -6226), INT16_C(-10481), INT16_C(-29347),
                            INT16_C( -7917), INT16_C( -7560), INT16_C(-19836), INT16_C(  1145),
                            INT16_C( -1401), INT16_C(-14817), INT16_C( 22782), INT16_C(-18634),
                            INT16_C( -2744), INT16_C(   907), INT16_C(  9939), INT16_C(   395)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_rem_epi16(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_rem_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_mm256_set_epi32(INT32_C(-1079516608), INT32_C( -708153743), INT32_C( 1508722402), INT32_C(-2074345640),
                            INT32_C( 1747596798), INT32_C(-2063703989), INT32_C(  527472553), INT32_C(-1403096998)),
      simde_mm256_set_epi32(INT32_C(  691121094), INT32_C(  674034227), INT32_C(-1965434887), INT32_C( -920286947),
                            INT32_C( -374673026), INT32_C(-1240805178), INT32_C( 1568850865), INT32_C(-1142977539)),
      simde_mm256_set_epi32(INT32_C( -388395514), INT32_C(  -34119516), INT32_C( 1508722402), INT32_C( -233771746),
                            INT32_C(  248904694), INT32_C( -822898811), INT32_C(  527472553), INT32_C( -260119459)) },
    { simde_mm256_set_epi32(INT32_C( 1192263444), INT32_C(-2086343723), INT32_C( 1322777130), INT32_C(  163989560),
                            INT32_C( 1492341726), INT32_C(  298608154), INT32_C( 1250819173), INT32_C( -650971253)),
      simde_mm256_set_epi32(INT32_C(-1764810870), INT32_C( 1179683687), INT32_C(-1646326602), INT32_C( -671967289),
                            INT32_C(-1586327268), INT32_C( 1691051285), INT32_C(   50347892), INT32_C(  728425428)),
      simde_mm256_set_epi32(INT32_C( 1192263444), INT32_C( -906660036), INT32_C( 1322777130), INT32_C(  163989560),
                            INT32_C( 1492341726), INT32_C(  298608154), INT32_C(   42469765), INT32_C( -650971253)) },
    { simde_mm256_set_epi32(INT32_C(  493161721), INT32_C(-1195115819), INT32_C(  894221337), INT32_C(-1330460172),
                            INT32_C(  492373082), INT32_C(  -13096811), INT32_C(-2087181083), INT32_C( -341007878)),
      simde_mm256_set_epi32(INT32_C( 1314482530), INT32_C(-1297250617), INT32_C( -739008036), INT32_C(-1419039999),
                            INT32_C(-1004264650), INT32_C( 1580565751), INT32_C( -471064457), INT32_C( 2081361826)),
      simde_mm256_set_epi32(INT32_C(  493161721), INT32_C(-1195115819), INT32_C(  155213301), INT32_C(-1330460172),
                            INT32_C(  492373082), INT32_C(  -13096811), INT32_C( -202923255), INT32_C( -341007878)) },
    { simde_mm256_set_epi32(INT32_C( 1710148738), INT32_C( 1974123080), INT32_C(-1424367196), INT32_C(  118588227),
                            INT32_C(  542053192), INT32_C(  499863549), INT32_C(  957375358), INT32_C(-1291033589)),
      simde_mm256_set_epi32(INT32_C(-1138893231), INT32_C( -687161637), INT32_C( 1828175063), INT32_C( -389420023),
                            INT32_C( -193211433), INT32_C( -857989172), INT32_C( -448329300), INT32_C(-1601364212)),
      simde_mm256_set_epi32(INT32_C(  571255507), INT32_C(  599799806), INT32_C(-1424367196), INT32_C(  118588227),
                            INT32_C(  155630326), INT32_C(  499863549), INT32_C(   60716758), INT32_C(-1291033589)) },
    { simde_mm256_set_epi32(INT32_C( 1734496959), INT32_C(  380846712), INT32_C( -941967689), INT32_C( -739443621),
                            INT32_C( 1995198557), INT32_C( -980655097), INT32_C(-1888383043), INT32_C( 1779168063)),
      simde_mm256_set_epi32(INT32_C( 1763100483), INT32_C( -518004559), INT32_C(-1450358898), INT32_C(-1409866198),
                            INT32_C(  269910347), INT32_C(  433971495), INT32_C( 1441956227), INT32_C( 1018271575)),
      simde_mm256_set_epi32(INT32_C( 1734496959), INT32_C(  380846712), INT32_C( -941967689), INT32_C( -739443621),
                            INT32_C(  105826128), INT32_C( -112712107), INT32_C( -446426816), INT32_C(  760896488)) },
    { simde_mm256_set_epi32(INT32_C( -362876916), INT32_C(-1845390533), INT32_C(  -48621016), INT32_C(  201516689),
                            INT32_C(-1435930720), INT32_C(-1932876068), INT32_C(-1153303869), INT32_C(  562234020)),
      simde_mm256_set_epi32(INT32_C( -665465241), INT32_C( -342195833), INT32_C( 2102184556), INT32_C(  877111492),
                            INT32_C( 1183491905), INT32_C( -576610979), INT32_C(-1061316197), INT32_C( -808097400)),
      simde_mm256_set_epi32(INT32_C( -362876916), INT32_C( -134411368), INT32_C(  -48621016), INT32_C(  201516689),
                            INT32_C( -252438815), INT32_C( -203043131), INT32_C(  -91987672), INT32_C(  562234020)) },
    { simde_mm256_set_epi32(INT32_C(  910061584), INT32_C( 2002226944), INT32_C( -621963189), INT32_C(  -48343218),
                            INT32_C(  523093293), INT32_C(-1235205724), INT32_C(-2088961787), INT32_C( 1943141679)),
      simde_mm256_set_epi32(INT32_C(  495870887), INT32_C( -382126427), INT32_C(  915244711), INT32_C(    5081424),
                            INT32_C( 1422501384), INT32_C( -163979724), INT32_C(-1516900265), INT32_C(  497965579)),
      simde_mm256_set_epi32(INT32_C(  414190697), INT32_C(   91594809), INT32_C( -621963189), INT32_C(   -2610402),
                            INT32_C(  523093293), INT32_C(  -87347656), INT32_C( -572061522), INT32_C(  449244942)) },
    { simde_mm256_set_epi32(INT32_C( 1755684145), INT32_C(-2061726371), INT32_C(-1050443653), INT32_C(-1299940555),
                            INT32_C(-2116696545), INT32_C( 1493088054), INT32_C( -179829877), INT32_C(  651362699)),
      simde_mm256_set_epi32(INT32_C( 1206471293), INT32_C( 1374915518), INT32_C(  531653117), INT32_C( 2075187308),
                            INT32_C( -144618549), INT32_C(-2131865715), INT32_C( 1444783055), INT32_C( 1878625233)),
      simde_mm256_set_epi32(INT32_C(  549212852), INT32_C( -686810853), INT32_C( -518790536), INT32_C(-1299940555),
                            INT32_C(  -92036859), INT32_C( 1493088054), INT32_C( -179829877), INT32_C(  651362699)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_rem_epi32(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_rem_epi64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_mm256_set_epi64x(INT64_C(-4636488523262038415), INT64_C( 6479913377553186648),
                             INT64_C( 7505871096235581515), INT64_C( 2265477367564496986)),
      simde_mm256_set_epi64x(INT64_C( 2968342496979776051), INT64_C(-8441478558707775203),
                             INT64_C(-1609208390309195578), INT64_C( 6738163160628300797)),
      simde_mm256_set_epi64x(INT64_C(-1668146026282262364), INT64_C( 6479913377553186648),
                             INT64_C( 1069037534998799203), INT64_C( 2265477367564496986)) },
    { simde_mm256_set_epi64x(INT64_C( 5120732502404950997), INT64_C( 5681284513410730040),
                             INT64_C( 6409558907924801050), INT64_C( 5372227444888762251)),
      simde_mm256_set_epi64x(INT64_C(-7579804969095623833), INT64_C(-7070918910501808185),
                             INT64_C(-6813223735121976043), INT64_C(  216242550290965460)),
      simde_mm256_set_epi64x(INT64_C( 5120732502404950997), INT64_C( 5681284513410730040),
                             INT64_C( 6409558907924801050), INT64_C(  182406237905591211)) },
    { simde_mm256_set_epi64x(INT64_C( 2118113466433927893), INT64_C( 3840651400764901876),
                             INT64_C( 2114726288902596757), INT64_C(-8964374488360902150)),
      simde_mm256_set_epi64x(INT64_C( 5645659480511055559), INT64_C(-3174015343225263359),
                             INT64_C(-4313283826698320649), INT64_C(-2023206435041636446)),
      simde_mm256_set_epi64x(INT64_C( 2118113466433927893), INT64_C(  666636057539638517),
                             INT64_C( 2114726288902596757), INT64_C( -871548748194356366)) },
    { simde_mm256_set_epi64x(INT64_C( 7345032902979795528), INT64_C(-6117610524196633789),
                             INT64_C( 2328100732832272381), INT64_C( 4111895855610225675)),
      simde_mm256_set_epi64x(INT64_C(-4891509177172967717), INT64_C( 7851952110853286921),
                             INT64_C( -829836782511317044), INT64_C(-1925559678644969716)),
      simde_mm256_set_epi64x(INT64_C( 2453523725806827811), INT64_C(-6117610524196633789),
                             INT64_C(  668427167809638293), INT64_C(  260776498320286243)) },
    { simde_mm256_set_epi64x(INT64_C( 7449607714297299576), INT64_C(-4045720414588175269),
                             INT64_C( 8569312554655704071), INT64_C(-8110543410226793665)),
      simde_mm256_set_epi64x(INT64_C( 7572458917823766705), INT64_C(-6229244031487498710),
                             INT64_C( 1159256113650983207), INT64_C( 6193154838246823767)),
      simde_mm256_set_epi64x(INT64_C( 7449607714297299576), INT64_C(-4045720414588175269),
                             INT64_C(  454519759098821622), INT64_C(-1917388571979969898)) },
    { simde_mm256_set_epi64x(INT64_C(-1558544484243762373), INT64_C( -208825673416776047),
                             INT64_C(-6167275479359641892), INT64_C(-4953402399143034204)),
      simde_mm256_set_epi64x(INT64_C(-2858151442766986873), INT64_C( 9028813919053392068),
                             INT64_C( 5083059030774095197), INT64_C(-4558318353343223416)),
      simde_mm256_set_epi64x(INT64_C(-1558544484243762373), INT64_C( -208825673416776047),
                             INT64_C(-1084216448585546695), INT64_C( -395084045799810788)) },
    { simde_mm256_set_epi64x(INT64_C( 3908684742628183808), INT64_C(-2671311551824242866),
                             INT64_C( 2246668589251707300), INT64_C(-8972022555815576273)),
      simde_mm256_set_epi64x(INT64_C( 2129749246616352421), INT64_C( 3930946101587052880),
                             INT64_C( 6109596926925725236), INT64_C(-6515037028970767861)),
      simde_mm256_set_epi64x(INT64_C( 1778935496011831387), INT64_C(-2671311551824242866),
                             INT64_C( 2246668589251707300), INT64_C(-2456985526844808412)) },
    { simde_mm256_set_epi64x(INT64_C( 7540605987113962845), INT64_C(-4511621132930745547),
                             INT64_C(-9091142434838104266), INT64_C( -772363439907339893)),
      simde_mm256_set_epi64x(INT64_C( 5181754748372749246), INT64_C( 2283432752406648940),
                             INT64_C( -621131936186871923), INT64_C( 6205295972918594513)),
      simde_mm256_set_epi64x(INT64_C( 2358851238741213599), INT64_C(-2228188380524096607),
                             INT64_C( -395295328221897344), INT64_C( -772363439907339893)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_rem_epi64(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_i64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_rem_epu8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_x_mm256_set_epu8(UINT8_C(191), UINT8_C(167), UINT8_C(226), UINT8_C( 64),
                             UINT8_C(213), UINT8_C(202), UINT8_C(110), UINT8_C(113),
                             UINT8_C( 89), UINT8_C(237), UINT8_C( 70), UINT8_C(226),
                             UINT8_C(132), UINT8_C( 91), UINT8_C(255), UINT8_C( 88),
                             UINT8_C(104), UINT8_C( 42), UINT8_C( 53), UINT8_C(254),
                             UINT8_C(132), UINT8_C(254), UINT8_C( 96), UINT8_C( 75),
                             UINT8_C( 31), UINT8_C(112), UINT8_C(151), UINT8_C(169),
                             UINT8_C(172), UINT8_C( 94), UINT8_C(112), UINT8_C( 90)),
      simde_x_mm256_set_epu8(UINT8_C(121), UINT8_C( 85), UINT8_C(153), UINT8_C(116),
                             UINT8_C(218), UINT8_C( 21), UINT8_C(101), UINT8_C(122),
                             UINT8_C( 10), UINT8_C(231), UINT8_C( 54), UINT8_C( 71),
                             UINT8_C(156), UINT8_C(149), UINT8_C(244), UINT8_C( 84),
                             UINT8_C(148), UINT8_C( 85), UINT8_C(170), UINT8_C(184),
                             UINT8_C( 94), UINT8_C(154), UINT8_C(229), UINT8_C( 11),
                             UINT8_C( 70), UINT8_C(179), UINT8_C(121), UINT8_C(157),
                             UINT8_C(254), UINT8_C( 70), UINT8_C( 49), UINT8_C(125)),
      simde_x_mm256_set_epu8(UINT8_C( 70), UINT8_C( 82), UINT8_C( 73), UINT8_C( 64),
                             UINT8_C(213), UINT8_C( 13), UINT8_C(  9), UINT8_C(113),
                             UINT8_C(  9), UINT8_C(  6), UINT8_C( 16), UINT8_C( 13),
                             UINT8_C(132), UINT8_C( 91), UINT8_C( 11), UINT8_C(  4),
                             UINT8_C(104), UINT8_C( 42), UINT8_C( 53), UINT8_C( 70),
                             UINT8_C( 38), UINT8_C(100), UINT8_C( 96), UINT8_C(  9),
                             UINT8_C( 31), UINT8_C(112), UINT8_C( 30), UINT8_C( 12),
                             UINT8_C(172), UINT8_C( 24), UINT8_C( 14), UINT8_C( 90)) },
    { simde_x_mm256_set_epu8(UINT8_C( 78), UINT8_C( 89), UINT8_C(105), UINT8_C( 98),
                             UINT8_C(178), UINT8_C(173), UINT8_C(134), UINT8_C(199),
                             UINT8_C(211), UINT8_C(243), UINT8_C(161), UINT8_C(220),
                             UINT8_C(171), UINT8_C(107), UINT8_C( 43), UINT8_C(  1),
                             UINT8_C(196), UINT8_C( 36), UINT8_C( 35), UINT8_C( 54),
                             UINT8_C( 94), UINT8_C( 53), UINT8_C(132), UINT8_C(247),
                             UINT8_C(227), UINT8_C(236), UINT8_C( 32), UINT8_C(119),
                             UINT8_C(124), UINT8_C( 15), UINT8_C( 15), UINT8_C(162)),
      simde_x_mm256_set_epu8(UINT8_C(195), UINT8_C( 49), UINT8_C( 14), UINT8_C(170),
                             UINT8_C(203), UINT8_C(167), UINT8_C(  3), UINT8_C(215),
                             UINT8_C( 63), UINT8_C(248), UINT8_C( 55), UINT8_C(219),
                             UINT8_C(221), UINT8_C(135), UINT8_C( 61), UINT8_C(191),
                             UINT8_C(209), UINT8_C( 91), UINT8_C( 87), UINT8_C(137),
                             UINT8_C( 87), UINT8_C( 76), UINT8_C( 44), UINT8_C(140),
                             UINT8_C(  2), UINT8_C(200), UINT8_C( 36), UINT8_C(195),
                             UINT8_C(200), UINT8_C(125), UINT8_C(254), UINT8_C(139)),
      simde_x_mm256_set_epu8(UINT8_C( 78), UINT8_C( 40), UINT8_C(  7), UINT8_C( 98),
                             UINT8_C(178), UINT8_C(  6), UINT8_C(  2), UINT8_C(199),
                             UINT8_C( 22), UINT8_C(243), UINT8_C( 51), UINT8_C(  1),
                             UINT8_C(171), UINT8_C(107), UINT8_C( 43), UINT8_C(  1),
                             UINT8_C(196), UINT8_C( 36), UINT8_C( 35), UINT8_C( 54),
                             UINT8_C(  7), UINT8_C( 53), UINT8_C(  0), UINT8_C(107),
                             UINT8_C(  1), UINT8_C( 36), UINT8_C( 32), UINT8_C(119),
                             UINT8_C(124), UINT8_C( 15), UINT8_C( 15), UINT8_C( 23)) },
    { simde_x_mm256_set_epu8(UINT8_C(234), UINT8_C( 94), UINT8_C(240), UINT8_C( 12),
                             UINT8_C(146), UINT8_C(  1), UINT8_C(147), UINT8_C( 59),
                             UINT8_C(253), UINT8_C( 26), UINT8_C( 26), UINT8_C( 40),
                             UINT8_C( 12), UINT8_C(  2), UINT8_C(230), UINT8_C(145),
                             UINT8_C(170), UINT8_C(105), UINT8_C(111), UINT8_C(160),
                             UINT8_C(140), UINT8_C(202), UINT8_C(166), UINT8_C(220),
                             UINT8_C(187), UINT8_C( 65), UINT8_C(250), UINT8_C(195),
                             UINT8_C( 33), UINT8_C(131), UINT8_C(  2), UINT8_C(164)),
      simde_x_mm256_set_epu8(UINT8_C(177), UINT8_C(221), UINT8_C(251), UINT8_C(181),
                             UINT8_C(159), UINT8_C(182), UINT8_C( 11), UINT8_C( 11),
                             UINT8_C( 39), UINT8_C( 37), UINT8_C( 39), UINT8_C(208),
                             UINT8_C(136), UINT8_C(180), UINT8_C(215), UINT8_C(139),
                             UINT8_C(144), UINT8_C(128), UINT8_C(203), UINT8_C(206),
                             UINT8_C(173), UINT8_C( 36), UINT8_C(133), UINT8_C(175),
                             UINT8_C(231), UINT8_C(  7), UINT8_C(236), UINT8_C( 68),
                             UINT8_C(193), UINT8_C(221), UINT8_C( 27), UINT8_C(  8)),
      simde_x_mm256_set_epu8(UINT8_C( 57), UINT8_C( 94), UINT8_C(240), UINT8_C( 12),
                             UINT8_C(146), UINT8_C(  1), UINT8_C(  4), UINT8_C(  4),
                             UINT8_C( 19), UINT8_C( 26), UINT8_C( 26), UINT8_C( 40),
                             UINT8_C( 12), UINT8_C(  2), UINT8_C( 15), UINT8_C(  6),
                             UINT8_C( 26), UINT8_C(105), UINT8_C(111), UINT8_C(160),
                             UINT8_C(140), UINT8_C( 22), UINT8_C( 33), UINT8_C( 45),
                             UINT8_C(187), UINT8_C(  2), UINT8_C( 14), UINT8_C( 59),
                             UINT8_C( 33), UINT8_C(131), UINT8_C(  2), UINT8_C(  4)) },
    { simde_x_mm256_set_epu8(UINT8_C( 71), UINT8_C(233), UINT8_C( 74), UINT8_C(125),
                             UINT8_C( 81), UINT8_C(243), UINT8_C(139), UINT8_C(190),
                             UINT8_C( 31), UINT8_C(176), UINT8_C( 97), UINT8_C(253),
                             UINT8_C(123), UINT8_C(176), UINT8_C(216), UINT8_C(108),
                             UINT8_C(247), UINT8_C( 97), UINT8_C( 75), UINT8_C(203),
                             UINT8_C(128), UINT8_C(238), UINT8_C( 79), UINT8_C(141),
                             UINT8_C( 86), UINT8_C( 29), UINT8_C(163), UINT8_C(207),
                             UINT8_C(111), UINT8_C(249), UINT8_C(139), UINT8_C(209)),
      simde_x_mm256_set_epu8(UINT8_C(120), UINT8_C(127), UINT8_C( 28), UINT8_C( 95),
                             UINT8_C(175), UINT8_C(223), UINT8_C(119), UINT8_C(214),
                             UINT8_C(220), UINT8_C(102), UINT8_C( 86), UINT8_C( 22),
                             UINT8_C(119), UINT8_C(207), UINT8_C( 12), UINT8_C(183),
                             UINT8_C(172), UINT8_C(242), UINT8_C(173), UINT8_C(249),
                             UINT8_C( 52), UINT8_C(108), UINT8_C(128), UINT8_C(203),
                             UINT8_C( 85), UINT8_C(135), UINT8_C(227), UINT8_C( 35),
                             UINT8_C(187), UINT8_C( 24), UINT8_C(250), UINT8_C(219)),
      simde_x_mm256_set_epu8(UINT8_C( 71), UINT8_C(106), UINT8_C( 18), UINT8_C( 30),
                             UINT8_C( 81), UINT8_C( 20), UINT8_C( 20), UINT8_C(190),
                             UINT8_C( 31), UINT8_C( 74), UINT8_C( 11), UINT8_C( 11),
                             UINT8_C(  4), UINT8_C(176), UINT8_C(  0), UINT8_C(108),
                             UINT8_C( 75), UINT8_C( 97), UINT8_C( 75), UINT8_C(203),
                             UINT8_C( 24), UINT8_C( 22), UINT8_C( 79), UINT8_C(141),
                             UINT8_C(  1), UINT8_C( 29), UINT8_C(163), UINT8_C( 32),
                             UINT8_C(111), UINT8_C(  9), UINT8_C(139), UINT8_C(209)) },
    { simde_x_mm256_set_epu8(UINT8_C(184), UINT8_C( 63), UINT8_C( 95), UINT8_C(164),
                             UINT8_C( 65), UINT8_C( 71), UINT8_C(174), UINT8_C( 88),
                             UINT8_C(183), UINT8_C(142), UINT8_C( 98), UINT8_C( 14),
                             UINT8_C( 25), UINT8_C(173), UINT8_C( 87), UINT8_C(  2),
                             UINT8_C(191), UINT8_C(143), UINT8_C(152), UINT8_C(  2),
                             UINT8_C(126), UINT8_C(  0), UINT8_C(162), UINT8_C( 57),
                             UINT8_C(245), UINT8_C( 36), UINT8_C(239), UINT8_C( 54),
                             UINT8_C( 33), UINT8_C(165), UINT8_C(199), UINT8_C( 84)),
      simde_x_mm256_set_epu8(UINT8_C(174), UINT8_C( 60), UINT8_C(132), UINT8_C(208),
                             UINT8_C( 58), UINT8_C(178), UINT8_C(116), UINT8_C(240),
                             UINT8_C( 37), UINT8_C(131), UINT8_C(100), UINT8_C(177),
                             UINT8_C( 19), UINT8_C(102), UINT8_C( 81), UINT8_C( 86),
                             UINT8_C( 25), UINT8_C( 43), UINT8_C( 51), UINT8_C(140),
                             UINT8_C(  9), UINT8_C( 40), UINT8_C(227), UINT8_C( 75),
                             UINT8_C(208), UINT8_C(159), UINT8_C(175), UINT8_C(109),
                             UINT8_C(230), UINT8_C( 87), UINT8_C(254), UINT8_C(216)),
      simde_x_mm256_set_epu8(UINT8_C( 10), UINT8_C(  3), UINT8_C( 95), UINT8_C(164),
                             UINT8_C(  7), UINT8_C( 71), UINT8_C( 58), UINT8_C( 88),
                             UINT8_C( 35), UINT8_C( 11), UINT8_C( 98), UINT8_C( 14),
                             UINT8_C(  6), UINT8_C( 71), UINT8_C(  6), UINT8_C(  2),
                             UINT8_C( 16), UINT8_C( 14), UINT8_C( 50), UINT8_C(  2),
                             UINT8_C(  0), UINT8_C(  0), UINT8_C(162), UINT8_C( 57),
                             UINT8_C( 37), UINT8_C( 36), UINT8_C( 64), UINT8_C( 54),
                             UINT8_C( 33), UINT8_C( 78), UINT8_C(199), UINT8_C( 84)) },
    { simde_x_mm256_set_epu8(UINT8_C( 54), UINT8_C( 43), UINT8_C(109), UINT8_C(187),
                             UINT8_C(138), UINT8_C( 62), UINT8_C(222), UINT8_C(154),
                             UINT8_C(123), UINT8_C( 21), UINT8_C(247), UINT8_C( 99),
                             UINT8_C( 37), UINT8_C( 48), UINT8_C(116), UINT8_C(233),
                             UINT8_C( 95), UINT8_C(251), UINT8_C(147), UINT8_C(109),
                             UINT8_C(205), UINT8_C(206), UINT8_C( 57), UINT8_C( 17),
                             UINT8_C(121), UINT8_C( 25), UINT8_C(  3), UINT8_C( 55),
                             UINT8_C(178), UINT8_C(129), UINT8_C(149), UINT8_C(207)),
      simde_x_mm256_set_epu8(UINT8_C(131), UINT8_C( 42), UINT8_C(151), UINT8_C(210),
                             UINT8_C( 12), UINT8_C(163), UINT8_C(138), UINT8_C(207),
                             UINT8_C( 43), UINT8_C( 57), UINT8_C( 61), UINT8_C( 62),
                             UINT8_C( 81), UINT8_C(184), UINT8_C(  6), UINT8_C( 93),
                             UINT8_C(167), UINT8_C(  1), UINT8_C(145), UINT8_C(  9),
                             UINT8_C(  4), UINT8_C( 17), UINT8_C( 10), UINT8_C(101),
                             UINT8_C(186), UINT8_C(181), UINT8_C(155), UINT8_C(243),
                             UINT8_C(189), UINT8_C(191), UINT8_C(222), UINT8_C(205)),
      simde_x_mm256_set_epu8(UINT8_C( 54), UINT8_C(  1), UINT8_C(109), UINT8_C(187),
                             UINT8_C(  6), UINT8_C( 62), UINT8_C( 84), UINT8_C(154),
                             UINT8_C( 37), UINT8_C( 21), UINT8_C(  3), UINT8_C( 37),
                             UINT8_C( 37), UINT8_C( 48), UINT8_C(  2), UINT8_C( 47),
                             UINT8_C( 95), UINT8_C(  0), UINT8_C(  2), UINT8_C(  1),
                             UINT8_C(  1), UINT8_C(  2), UINT8_C(  7), UINT8_C( 17),
                             UINT8_C(121), UINT8_C( 25), UINT8_C(  3), UINT8_C( 55),
                             UINT8_C(178), UINT8_C(129), UINT8_C(149), UINT8_C(  2)) },
    { simde_x_mm256_set_epu8(UINT8_C( 23), UINT8_C(132), UINT8_C(106), UINT8_C(109),
                             UINT8_C(135), UINT8_C(203), UINT8_C( 98), UINT8_C(120),
                             UINT8_C(101), UINT8_C( 52), UINT8_C( 82), UINT8_C( 44),
                             UINT8_C(142), UINT8_C( 14), UINT8_C( 99), UINT8_C(245),
                             UINT8_C(  8), UINT8_C(140), UINT8_C(141), UINT8_C(123),
                             UINT8_C(219), UINT8_C(163), UINT8_C(196), UINT8_C(233),
                             UINT8_C( 34), UINT8_C(185), UINT8_C(228), UINT8_C(108),
                             UINT8_C( 95), UINT8_C(236), UINT8_C( 97), UINT8_C( 41)),
      simde_x_mm256_set_epu8(UINT8_C(125), UINT8_C(229), UINT8_C(203), UINT8_C( 45),
                             UINT8_C( 24), UINT8_C(  5), UINT8_C( 90), UINT8_C( 83),
                             UINT8_C(145), UINT8_C( 85), UINT8_C(156), UINT8_C(164),
                             UINT8_C(149), UINT8_C(201), UINT8_C( 48), UINT8_C(255),
                             UINT8_C( 41), UINT8_C( 42), UINT8_C( 94), UINT8_C(129),
                             UINT8_C(135), UINT8_C(  8), UINT8_C( 12), UINT8_C(203),
                             UINT8_C(128), UINT8_C(202), UINT8_C(148), UINT8_C(252),
                             UINT8_C(104), UINT8_C(208), UINT8_C( 98), UINT8_C(162)),
      simde_x_mm256_set_epu8(UINT8_C( 23), UINT8_C(132), UINT8_C(106), UINT8_C( 19),
                             UINT8_C( 15), UINT8_C(  3), UINT8_C(  8), UINT8_C( 37),
                             UINT8_C(101), UINT8_C( 52), UINT8_C( 82), UINT8_C( 44),
                             UINT8_C(142), UINT8_C( 14), UINT8_C(  3), UINT8_C(245),
                             UINT8_C(  8), UINT8_C( 14), UINT8_C( 47), UINT8_C(123),
                             UINT8_C( 84), UINT8_C(  3), UINT8_C(  4), UINT8_C( 30),
                             UINT8_C( 34), UINT8_C(185), UINT8_C( 80), UINT8_C(108),
                             UINT8_C( 95), UINT8_C( 28), UINT8_C( 97), UINT8_C( 41)) },
    { simde_x_mm256_set_epu8(UINT8_C(162), UINT8_C( 31), UINT8_C(168), UINT8_C( 17),
                             UINT8_C( 50), UINT8_C(110), UINT8_C(231), UINT8_C(216),
                             UINT8_C( 94), UINT8_C( 20), UINT8_C(163), UINT8_C(183),
                             UINT8_C(157), UINT8_C( 16), UINT8_C( 91), UINT8_C( 54),
                             UINT8_C( 62), UINT8_C( 81), UINT8_C(159), UINT8_C(151),
                             UINT8_C( 57), UINT8_C( 12), UINT8_C(118), UINT8_C( 33),
                             UINT8_C(180), UINT8_C(139), UINT8_C(  1), UINT8_C(  5),
                             UINT8_C( 78), UINT8_C( 13), UINT8_C( 93), UINT8_C(155)),
      simde_x_mm256_set_epu8(UINT8_C(193), UINT8_C(230), UINT8_C( 93), UINT8_C( 23),
                             UINT8_C(193), UINT8_C( 52), UINT8_C(223), UINT8_C(175),
                             UINT8_C(205), UINT8_C( 45), UINT8_C(166), UINT8_C( 24),
                             UINT8_C( 71), UINT8_C(234), UINT8_C(161), UINT8_C(142),
                             UINT8_C(184), UINT8_C(218), UINT8_C(190), UINT8_C(212),
                             UINT8_C(116), UINT8_C(159), UINT8_C( 44), UINT8_C( 55),
                             UINT8_C(213), UINT8_C(133), UINT8_C( 60), UINT8_C(  3),
                             UINT8_C( 58), UINT8_C(255), UINT8_C(125), UINT8_C(189)),
      simde_x_mm256_set_epu8(UINT8_C(162), UINT8_C( 31), UINT8_C( 75), UINT8_C( 17),
                             UINT8_C( 50), UINT8_C(  6), UINT8_C(  8), UINT8_C( 41),
                             UINT8_C( 94), UINT8_C( 20), UINT8_C(163), UINT8_C( 15),
                             UINT8_C( 15), UINT8_C( 16), UINT8_C( 91), UINT8_C( 54),
                             UINT8_C( 62), UINT8_C( 81), UINT8_C(159), UINT8_C(151),
                             UINT8_C( 57), UINT8_C( 12), UINT8_C( 30), UINT8_C( 33),
                             UINT8_C(180), UINT8_C(  6), UINT8_C(  1), UINT8_C(  2),
                             UINT8_C( 20), UINT8_C( 13), UINT8_C( 93), UINT8_C(155)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_rem_epu8(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_u8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_rem_epu16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_x_mm256_set_epu16(UINT16_C( 49063), UINT16_C( 57920), UINT16_C( 54730), UINT16_C( 28273),
                              UINT16_C( 23021), UINT16_C( 18146), UINT16_C( 33883), UINT16_C( 65368),
                              UINT16_C( 26666), UINT16_C( 13822), UINT16_C( 34046), UINT16_C( 24651),
                              UINT16_C(  8048), UINT16_C( 38825), UINT16_C( 44126), UINT16_C( 28762)),
      simde_x_mm256_set_epu16(UINT16_C( 10545), UINT16_C( 43974), UINT16_C( 10284), UINT16_C( 62003),
                              UINT16_C( 35545), UINT16_C( 55289), UINT16_C( 51493), UINT16_C( 35101),
                              UINT16_C( 59818), UINT16_C( 61822), UINT16_C( 46602), UINT16_C( 53446),
                              UINT16_C( 23938), UINT16_C( 50097), UINT16_C( 48095), UINT16_C( 35837)),
      simde_x_mm256_set_epu16(UINT16_C(  6883), UINT16_C( 13946), UINT16_C(  3310), UINT16_C( 28273),
                              UINT16_C( 23021), UINT16_C( 18146), UINT16_C( 33883), UINT16_C( 30267),
                              UINT16_C( 26666), UINT16_C( 13822), UINT16_C( 34046), UINT16_C( 24651),
                              UINT16_C(  8048), UINT16_C( 38825), UINT16_C( 44126), UINT16_C( 28762)) },
    { simde_x_mm256_set_epu16(UINT16_C( 18192), UINT16_C( 32532), UINT16_C( 33700), UINT16_C( 60373),
                              UINT16_C( 20183), UINT16_C( 64042), UINT16_C(  2502), UINT16_C( 18488),
                              UINT16_C( 22771), UINT16_C( 21470), UINT16_C(  4556), UINT16_C( 26138),
                              UINT16_C( 19085), UINT16_C( 64613), UINT16_C( 55602), UINT16_C( 63371)),
      simde_x_mm256_set_epu16(UINT16_C( 38607), UINT16_C(  8074), UINT16_C( 18000), UINT16_C( 35687),
                              UINT16_C( 40415), UINT16_C(  3254), UINT16_C( 55282), UINT16_C( 38855),
                              UINT16_C( 41330), UINT16_C( 37148), UINT16_C( 25803), UINT16_C( 25877),
                              UINT16_C(   768), UINT16_C( 16244), UINT16_C( 11114), UINT16_C( 58324)),
      simde_x_mm256_set_epu16(UINT16_C( 18192), UINT16_C(   236), UINT16_C( 15700), UINT16_C( 24686),
                              UINT16_C( 20183), UINT16_C(  2216), UINT16_C(  2502), UINT16_C( 18488),
                              UINT16_C( 22771), UINT16_C( 21470), UINT16_C(  4556), UINT16_C(   261),
                              UINT16_C(   653), UINT16_C( 15881), UINT16_C(    32), UINT16_C(  5047)) },
    { simde_x_mm256_set_epu16(UINT16_C(  7525), UINT16_C(  3321), UINT16_C( 47299), UINT16_C( 64213),
                              UINT16_C( 13644), UINT16_C( 48153), UINT16_C( 45234), UINT16_C( 51700),
                              UINT16_C(  7513), UINT16_C(  1114), UINT16_C( 65336), UINT16_C( 10389),
                              UINT16_C( 33688), UINT16_C(  9445), UINT16_C( 60332), UINT16_C( 41466)),
      simde_x_mm256_set_epu16(UINT16_C( 20057), UINT16_C( 26978), UINT16_C( 45741), UINT16_C( 34503),
                              UINT16_C( 54259), UINT16_C( 41436), UINT16_C( 43883), UINT16_C( 11009),
                              UINT16_C( 50212), UINT16_C(  9014), UINT16_C( 24117), UINT16_C( 34039),
                              UINT16_C( 58348), UINT16_C(  8311), UINT16_C( 31759), UINT16_C(  4002)),
      simde_x_mm256_set_epu16(UINT16_C(  7525), UINT16_C(  3321), UINT16_C(  1558), UINT16_C( 29710),
                              UINT16_C( 13644), UINT16_C(  6717), UINT16_C(  1351), UINT16_C(  7664),
                              UINT16_C(  7513), UINT16_C(  1114), UINT16_C( 17102), UINT16_C( 10389),
                              UINT16_C( 33688), UINT16_C(  1134), UINT16_C( 28573), UINT16_C(  1446)) },
    { simde_x_mm256_set_epu16(UINT16_C( 26094), UINT16_C( 52354), UINT16_C( 30122), UINT16_C( 47688),
                              UINT16_C( 43801), UINT16_C( 57764), UINT16_C(  1809), UINT16_C( 33603),
                              UINT16_C(  8271), UINT16_C(  4936), UINT16_C(  7627), UINT16_C( 20477),
                              UINT16_C( 14608), UINT16_C( 25470), UINT16_C( 45836), UINT16_C( 25611)),
      simde_x_mm256_set_epu16(UINT16_C( 48157), UINT16_C( 56913), UINT16_C( 55050), UINT16_C( 48859),
                              UINT16_C( 27895), UINT16_C( 48343), UINT16_C( 59593), UINT16_C( 60425),
                              UINT16_C( 62587), UINT16_C( 54231), UINT16_C( 52444), UINT16_C(  8140),
                              UINT16_C( 58695), UINT16_C(  2476), UINT16_C( 41101), UINT16_C(  7948)),
      simde_x_mm256_set_epu16(UINT16_C( 26094), UINT16_C( 52354), UINT16_C( 30122), UINT16_C( 47688),
                              UINT16_C( 15906), UINT16_C(  9421), UINT16_C(  1809), UINT16_C( 33603),
                              UINT16_C(  8271), UINT16_C(  4936), UINT16_C(  7627), UINT16_C(  4197),
                              UINT16_C( 14608), UINT16_C(   710), UINT16_C(  4735), UINT16_C(  1767)) },
    { simde_x_mm256_set_epu16(UINT16_C( 26466), UINT16_C( 21183), UINT16_C(  5811), UINT16_C( 17016),
                              UINT16_C( 51162), UINT16_C( 46775), UINT16_C( 54252), UINT16_C( 64603),
                              UINT16_C( 30444), UINT16_C( 20573), UINT16_C( 50572), UINT16_C( 25607),
                              UINT16_C( 36721), UINT16_C( 36797), UINT16_C( 27147), UINT16_C( 62271)),
      simde_x_mm256_set_epu16(UINT16_C( 26902), UINT16_C( 51011), UINT16_C( 57631), UINT16_C( 57521),
                              UINT16_C( 43405), UINT16_C( 18318), UINT16_C( 44023), UINT16_C(  9770),
                              UINT16_C(  4118), UINT16_C( 33099), UINT16_C(  6621), UINT16_C( 57639),
                              UINT16_C( 22002), UINT16_C( 33155), UINT16_C( 15537), UINT16_C( 38743)),
      simde_x_mm256_set_epu16(UINT16_C( 26466), UINT16_C( 21183), UINT16_C(  5811), UINT16_C( 17016),
                              UINT16_C(  7757), UINT16_C( 10139), UINT16_C( 10229), UINT16_C(  5983),
                              UINT16_C(  1618), UINT16_C( 20573), UINT16_C(  4225), UINT16_C( 25607),
                              UINT16_C( 14719), UINT16_C(  3642), UINT16_C( 11610), UINT16_C( 23528)) },
    { simde_x_mm256_set_epu16(UINT16_C( 59998), UINT16_C( 61452), UINT16_C( 37377), UINT16_C( 37691),
                              UINT16_C( 64794), UINT16_C(  6696), UINT16_C(  3074), UINT16_C( 59025),
                              UINT16_C( 43625), UINT16_C( 28576), UINT16_C( 36042), UINT16_C( 42716),
                              UINT16_C( 47937), UINT16_C( 64195), UINT16_C(  8579), UINT16_C(   676)),
      simde_x_mm256_set_epu16(UINT16_C( 55381), UINT16_C( 52839), UINT16_C( 60314), UINT16_C( 33159),
                              UINT16_C( 32076), UINT16_C( 51820), UINT16_C( 13383), UINT16_C( 43204),
                              UINT16_C( 18058), UINT16_C( 42817), UINT16_C( 56737), UINT16_C( 40285),
                              UINT16_C( 49341), UINT16_C( 39323), UINT16_C( 53205), UINT16_C( 27016)),
      simde_x_mm256_set_epu16(UINT16_C(  4617), UINT16_C(  8613), UINT16_C( 37377), UINT16_C(  4532),
                              UINT16_C(   642), UINT16_C(  6696), UINT16_C(  3074), UINT16_C( 15821),
                              UINT16_C(  7509), UINT16_C( 28576), UINT16_C( 36042), UINT16_C(  2431),
                              UINT16_C( 47937), UINT16_C( 24872), UINT16_C(  8579), UINT16_C(   676)) },
    { simde_x_mm256_set_epu16(UINT16_C( 13886), UINT16_C( 28688), UINT16_C( 30551), UINT16_C( 36608),
                              UINT16_C( 56045), UINT16_C( 38987), UINT16_C( 64798), UINT16_C( 22350),
                              UINT16_C(  7981), UINT16_C( 50477), UINT16_C( 46688), UINT16_C( 16804),
                              UINT16_C( 33660), UINT16_C( 63749), UINT16_C( 29649), UINT16_C( 64815)),
      simde_x_mm256_set_epu16(UINT16_C(  7566), UINT16_C( 25511), UINT16_C( 59705), UINT16_C( 13989),
                              UINT16_C( 13965), UINT16_C( 34471), UINT16_C(    77), UINT16_C( 35152),
                              UINT16_C( 21705), UINT16_C( 42504), UINT16_C( 63033), UINT16_C( 56884),
                              UINT16_C( 42389), UINT16_C( 61527), UINT16_C(  7598), UINT16_C( 23051)),
      simde_x_mm256_set_epu16(UINT16_C(  6320), UINT16_C(  3177), UINT16_C( 30551), UINT16_C(  8630),
                              UINT16_C(   185), UINT16_C(  4516), UINT16_C(    41), UINT16_C( 22350),
                              UINT16_C(  7981), UINT16_C(  7973), UINT16_C( 46688), UINT16_C( 16804),
                              UINT16_C( 33660), UINT16_C(  2222), UINT16_C(  6855), UINT16_C( 18713)) },
    { simde_x_mm256_set_epu16(UINT16_C( 26789), UINT16_C( 40241), UINT16_C( 34076), UINT16_C( 36189),
                              UINT16_C( 49507), UINT16_C( 32891), UINT16_C( 45700), UINT16_C( 31541),
                              UINT16_C( 33237), UINT16_C( 50719), UINT16_C( 22782), UINT16_C( 46902),
                              UINT16_C( 62792), UINT16_C(   907), UINT16_C(  9939), UINT16_C(   395)),
      simde_x_mm256_set_epu16(UINT16_C( 18409), UINT16_C( 19069), UINT16_C( 20979), UINT16_C( 35774),
                              UINT16_C(  8112), UINT16_C( 25085), UINT16_C( 31664), UINT16_C( 55404),
                              UINT16_C( 63329), UINT16_C( 19403), UINT16_C( 33006), UINT16_C( 20365),
                              UINT16_C( 22045), UINT16_C( 41935), UINT16_C( 28665), UINT16_C( 35793)),
      simde_x_mm256_set_epu16(UINT16_C(  8380), UINT16_C(  2103), UINT16_C( 13097), UINT16_C(   415),
                              UINT16_C(   835), UINT16_C(  7806), UINT16_C( 14036), UINT16_C( 31541),
                              UINT16_C( 33237), UINT16_C( 11913), UINT16_C( 22782), UINT16_C(  6172),
                              UINT16_C( 18702), UINT16_C(   907), UINT16_C(  9939), UINT16_C(   395)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_rem_epu16(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_u16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_rem_epu32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_x_mm256_set_epu32(UINT32_C(3215450688), UINT32_C(3586813553), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C(2891870298)),
      simde_x_mm256_set_epu32(UINT32_C( 691121094), UINT32_C( 674034227), UINT32_C(2329532409), UINT32_C(3374680349),
                              UINT32_C(3920294270), UINT32_C(3054162118), UINT32_C(1568850865), UINT32_C(3151989757)),
      simde_x_mm256_set_epu32(UINT32_C( 450966312), UINT32_C( 216642418), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C(2891870298)) },
    { simde_x_mm256_set_epu32(UINT32_C(1192263444), UINT32_C(2208623573), UINT32_C(1322777130), UINT32_C( 163989560),
                              UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(1250819173), UINT32_C(3643996043)),
      simde_x_mm256_set_epu32(UINT32_C(2530156426), UINT32_C(1179683687), UINT32_C(2648640694), UINT32_C(3623000007),
                              UINT32_C(2708640028), UINT32_C(1691051285), UINT32_C(  50347892), UINT32_C( 728425428)),
      simde_x_mm256_set_epu32(UINT32_C(1192263444), UINT32_C(1028939886), UINT32_C(1322777130), UINT32_C( 163989560),
                              UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(  42469765), UINT32_C(   1868903)) },
    { simde_x_mm256_set_epu32(UINT32_C( 493161721), UINT32_C(3099851477), UINT32_C( 894221337), UINT32_C(2964507124),
                              UINT32_C( 492373082), UINT32_C(4281870485), UINT32_C(2207786213), UINT32_C(3953959418)),
      simde_x_mm256_set_epu32(UINT32_C(1314482530), UINT32_C(2997716679), UINT32_C(3555959260), UINT32_C(2875927297),
                              UINT32_C(3290702646), UINT32_C(1580565751), UINT32_C(3823902839), UINT32_C(2081361826)),
      simde_x_mm256_set_epu32(UINT32_C( 493161721), UINT32_C( 102134798), UINT32_C( 894221337), UINT32_C(  88579827),
                              UINT32_C( 492373082), UINT32_C(1120738983), UINT32_C(2207786213), UINT32_C(1872597592)) },
    { simde_x_mm256_set_epu32(UINT32_C(1710148738), UINT32_C(1974123080), UINT32_C(2870600100), UINT32_C( 118588227),
                              UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358), UINT32_C(3003933707)),
      simde_x_mm256_set_epu32(UINT32_C(3156074065), UINT32_C(3607805659), UINT32_C(1828175063), UINT32_C(3905547273),
                              UINT32_C(4101755863), UINT32_C(3436978124), UINT32_C(3846637996), UINT32_C(2693603084)),
      simde_x_mm256_set_epu32(UINT32_C(1710148738), UINT32_C(1974123080), UINT32_C(1042425037), UINT32_C( 118588227),
                              UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358), UINT32_C( 310330623)) },
    { simde_x_mm256_set_epu32(UINT32_C(1734496959), UINT32_C( 380846712), UINT32_C(3352999607), UINT32_C(3555523675),
                              UINT32_C(1995198557), UINT32_C(3314312199), UINT32_C(2406584253), UINT32_C(1779168063)),
      simde_x_mm256_set_epu32(UINT32_C(1763100483), UINT32_C(3776962737), UINT32_C(2844608398), UINT32_C(2885101098),
                              UINT32_C( 269910347), UINT32_C( 433971495), UINT32_C(1441956227), UINT32_C(1018271575)),
      simde_x_mm256_set_epu32(UINT32_C(1734496959), UINT32_C( 380846712), UINT32_C( 508391209), UINT32_C( 670422577),
                              UINT32_C( 105826128), UINT32_C( 276511734), UINT32_C( 964628026), UINT32_C( 760896488)) },
    { simde_x_mm256_set_epu32(UINT32_C(3932090380), UINT32_C(2449576763), UINT32_C(4246346280), UINT32_C( 201516689),
                              UINT32_C(2859036576), UINT32_C(2362091228), UINT32_C(3141663427), UINT32_C( 562234020)),
      simde_x_mm256_set_epu32(UINT32_C(3629502055), UINT32_C(3952771463), UINT32_C(2102184556), UINT32_C( 877111492),
                              UINT32_C(1183491905), UINT32_C(3718356317), UINT32_C(3233651099), UINT32_C(3486869896)),
      simde_x_mm256_set_epu32(UINT32_C( 302588325), UINT32_C(2449576763), UINT32_C(  41977168), UINT32_C( 201516689),
                              UINT32_C( 492052766), UINT32_C(2362091228), UINT32_C(3141663427), UINT32_C( 562234020)) },
    { simde_x_mm256_set_epu32(UINT32_C( 910061584), UINT32_C(2002226944), UINT32_C(3673004107), UINT32_C(4246624078),
                              UINT32_C( 523093293), UINT32_C(3059761572), UINT32_C(2206005509), UINT32_C(1943141679)),
      simde_x_mm256_set_epu32(UINT32_C( 495870887), UINT32_C(3912840869), UINT32_C( 915244711), UINT32_C(   5081424),
                              UINT32_C(1422501384), UINT32_C(4130987572), UINT32_C(2778067031), UINT32_C( 497965579)),
      simde_x_mm256_set_epu32(UINT32_C( 414190697), UINT32_C(2002226944), UINT32_C(  12025263), UINT32_C(   3635038),
                              UINT32_C( 523093293), UINT32_C(3059761572), UINT32_C(2206005509), UINT32_C( 449244942)) },
    { simde_x_mm256_set_epu32(UINT32_C(1755684145), UINT32_C(2233240925), UINT32_C(3244523643), UINT32_C(2995026741),
                              UINT32_C(2178270751), UINT32_C(1493088054), UINT32_C(4115137419), UINT32_C( 651362699)),
      simde_x_mm256_set_epu32(UINT32_C(1206471293), UINT32_C(1374915518), UINT32_C( 531653117), UINT32_C(2075187308),
                              UINT32_C(4150348747), UINT32_C(2163101581), UINT32_C(1444783055), UINT32_C(1878625233)),
      simde_x_mm256_set_epu32(UINT32_C( 549212852), UINT32_C( 858325407), UINT32_C(  54604941), UINT32_C( 919839433),
                              UINT32_C(2178270751), UINT32_C(1493088054), UINT32_C(1225571309), UINT32_C( 651362699)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_rem_epu32(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_u32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_mask_rem_epu32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i src;
    simde__mmask16 k;
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu32(UINT32_C( 691121094), UINT32_C( 674034227), UINT32_C(2329532409), UINT32_C(3374680349),
                              UINT32_C(3920294270), UINT32_C(3054162118), UINT32_C(1568850865), UINT32_C(3151989757),
                              UINT32_C(3215450688), UINT32_C(3586813553), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C(2891870298)),
      UINT16_C(63371),
      simde_x_mm512_set_epu32(UINT32_C(3953959418), UINT32_C(2530156426), UINT32_C(1179683687), UINT32_C(2648640694),
                              UINT32_C(3623000007), UINT32_C(2708640028), UINT32_C(1691051285), UINT32_C(  50347892),
                              UINT32_C( 728425428), UINT32_C(1192263444), UINT32_C(2208623573), UINT32_C(1322777130),
                              UINT32_C( 163989560), UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(1250819173)),
      simde_x_mm512_set_epu32(UINT32_C(3003933707), UINT32_C(1314482530), UINT32_C(2997716679), UINT32_C(3555959260),
                              UINT32_C(2875927297), UINT32_C(3290702646), UINT32_C(1580565751), UINT32_C(3823902839),
                              UINT32_C(2081361826), UINT32_C( 493161721), UINT32_C(3099851477), UINT32_C( 894221337),
                              UINT32_C(2964507124), UINT32_C( 492373082), UINT32_C(4281870485), UINT32_C(2207786213)),
      simde_x_mm512_set_epu32(UINT32_C( 950025711), UINT32_C(1215673896), UINT32_C(1179683687), UINT32_C(2648640694),
                              UINT32_C(3920294270), UINT32_C(2708640028), UINT32_C( 110485534), UINT32_C(  50347892),
                              UINT32_C( 728425428), UINT32_C(3586813553), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C( 163989560), UINT32_C(2231263307), UINT32_C( 298608154), UINT32_C(1250819173)) },
    { simde_x_mm512_set_epu32(UINT32_C(1779168063), UINT32_C(3156074065), UINT32_C(3607805659), UINT32_C(1828175063),
                              UINT32_C(3905547273), UINT32_C(4101755863), UINT32_C(3436978124), UINT32_C(3846637996),
                              UINT32_C(2693603084), UINT32_C(1710148738), UINT32_C(1974123080), UINT32_C(2870600100),
                              UINT32_C( 118588227), UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358)),
      UINT16_C(36797),
      simde_x_mm512_set_epu32(UINT32_C(3141663427), UINT32_C( 562234020), UINT32_C(1763100483), UINT32_C(3776962737),
                              UINT32_C(2844608398), UINT32_C(2885101098), UINT32_C( 269910347), UINT32_C( 433971495),
                              UINT32_C(1441956227), UINT32_C(1018271575), UINT32_C(1734496959), UINT32_C( 380846712),
                              UINT32_C(3352999607), UINT32_C(3555523675), UINT32_C(1995198557), UINT32_C(3314312199)),
      simde_x_mm512_set_epu32(UINT32_C(2206005509), UINT32_C(1943141679), UINT32_C(3629502055), UINT32_C(3952771463),
                              UINT32_C(2102184556), UINT32_C( 877111492), UINT32_C(1183491905), UINT32_C(3718356317),
                              UINT32_C(3233651099), UINT32_C(3486869896), UINT32_C(3932090380), UINT32_C(2449576763),
                              UINT32_C(4246346280), UINT32_C( 201516689), UINT32_C(2859036576), UINT32_C(2362091228)),
      simde_x_mm512_set_epu32(UINT32_C( 935657918), UINT32_C(3156074065), UINT32_C(3607805659), UINT32_C(1828175063),
                              UINT32_C( 742423842), UINT32_C( 253766622), UINT32_C( 269910347), UINT32_C( 433971495),
                              UINT32_C(1441956227), UINT32_C(1710148738), UINT32_C(1734496959), UINT32_C( 380846712),
                              UINT32_C(3352999607), UINT32_C( 129739962), UINT32_C( 499863549), UINT32_C( 952220971)) },
    { simde_x_mm512_set_epu32(UINT32_C(4115137419), UINT32_C( 651362699), UINT32_C( 495870887), UINT32_C(3912840869),
                              UINT32_C( 915244711), UINT32_C(   5081424), UINT32_C(1422501384), UINT32_C(4130987572),
                              UINT32_C(2778067031), UINT32_C( 497965579), UINT32_C( 910061584), UINT32_C(2002226944),
                              UINT32_C(3673004107), UINT32_C(4246624078), UINT32_C( 523093293), UINT32_C(3059761572)),
      UINT16_C(46902),
      simde_x_mm512_set_epu32(UINT32_C(4074346392), UINT32_C(1398655610), UINT32_C(1722520923), UINT32_C(1206471293),
                              UINT32_C(1374915518), UINT32_C( 531653117), UINT32_C(2075187308), UINT32_C(4150348747),
                              UINT32_C(2163101581), UINT32_C(1444783055), UINT32_C(1878625233), UINT32_C(1755684145),
                              UINT32_C(2233240925), UINT32_C(3244523643), UINT32_C(2995026741), UINT32_C(2178270751)),
      simde_x_mm512_set_epu32(UINT32_C(3188873807), UINT32_C(1982658188), UINT32_C( 863153207), UINT32_C(2657690668),
                              UINT32_C( 448681074), UINT32_C(1334667053), UINT32_C( 502667641), UINT32_C( 855395764),
                              UINT32_C(2622874348), UINT32_C( 808531712), UINT32_C( 454488139), UINT32_C( 123547093),
                              UINT32_C( 483090439), UINT32_C(3168637539), UINT32_C(3093747107), UINT32_C(4158916667)),
      simde_x_mm512_set_epu32(UINT32_C( 885472585), UINT32_C( 651362699), UINT32_C( 859367716), UINT32_C(1206471293),
                              UINT32_C( 915244711), UINT32_C( 531653117), UINT32_C(  64516744), UINT32_C( 728765691),
                              UINT32_C(2778067031), UINT32_C( 497965579), UINT32_C(  60672677), UINT32_C(  26024843),
                              UINT32_C(3673004107), UINT32_C(  75886104), UINT32_C(2995026741), UINT32_C(3059761572)) },
    { simde_x_mm512_set_epu32(UINT32_C(2113970745), UINT32_C(4112838454), UINT32_C( 564512596), UINT32_C( 604721400),
                              UINT32_C(1471174399), UINT32_C(2491026588), UINT32_C(2529574367), UINT32_C( 298473775),
                              UINT32_C(2890366559), UINT32_C(3063632375), UINT32_C(4055983958), UINT32_C(4149169500),
                              UINT32_C(4113948134), UINT32_C(2384487126), UINT32_C(2434207126), UINT32_C(3923111671)),
      UINT16_C(38914),
      simde_x_mm512_set_epu32(UINT32_C(1533151625), UINT32_C(2122196136), UINT32_C(1690360675), UINT32_C(1484935627),
                              UINT32_C(1463758672), UINT32_C( 602211615), UINT32_C(3830002991), UINT32_C(2864741101),
                              UINT32_C( 797104998), UINT32_C(2737423319), UINT32_C(3342229886), UINT32_C( 178625368),
                              UINT32_C(3091160996), UINT32_C(1095216728), UINT32_C(3079561742), UINT32_C( 430790402)),
      simde_x_mm512_set_epu32(UINT32_C(4043825594), UINT32_C(1274901810), UINT32_C( 413860084), UINT32_C( 550494320),
                              UINT32_C(1997049765), UINT32_C( 505563651), UINT32_C( 463125220), UINT32_C(3843753777),
                              UINT32_C(2346173843), UINT32_C(2157864934), UINT32_C(2591157969), UINT32_C( 389679318),
                              UINT32_C(3939775129), UINT32_C(2493364907), UINT32_C(2006619059), UINT32_C(3391409164)),
      simde_x_mm512_set_epu32(UINT32_C(1533151625), UINT32_C(4112838454), UINT32_C( 564512596), UINT32_C( 383946987),
                              UINT32_C(1463758672), UINT32_C(2491026588), UINT32_C(2529574367), UINT32_C( 298473775),
                              UINT32_C(2890366559), UINT32_C(3063632375), UINT32_C(4055983958), UINT32_C(4149169500),
                              UINT32_C(4113948134), UINT32_C(2384487126), UINT32_C(1072942683), UINT32_C(3923111671)) },
    { simde_x_mm512_set_epu32(UINT32_C(1572579389), UINT32_C(3511888959), UINT32_C(2399346014), UINT32_C(1967093325),
                              UINT32_C( 908815803), UINT32_C(2319376026), UINT32_C(2065037155), UINT32_C( 623932649),
                              UINT32_C(1610322797), UINT32_C(3452844305), UINT32_C(2031682359), UINT32_C(2994836943),
                              UINT32_C(2344919086), UINT32_C( 238137788), UINT32_C(1978166020), UINT32_C(  76768592)),
      UINT16_C(  883),
      simde_x_mm512_set_epu32(UINT32_C(3284847806), UINT32_C(3884897233), UINT32_C(2094036024), UINT32_C(2456834182),
                              UINT32_C(  69201629), UINT32_C(1228958503), UINT32_C(3519587969), UINT32_C(2809504529),
                              UINT32_C(3115789449), UINT32_C(1767270276), UINT32_C( 490610321), UINT32_C(1164436618),
                              UINT32_C(2374669797), UINT32_C(3604002618), UINT32_C(3414719029), UINT32_C(2289333019)),
      simde_x_mm512_set_epu32(UINT32_C(2383307765), UINT32_C( 143428987), UINT32_C(3684943081), UINT32_C( 582607980),
                              UINT32_C(1609326889), UINT32_C(1245407235), UINT32_C(4175005098), UINT32_C(2362914327),
                              UINT32_C(2924553042), UINT32_C(2369006988), UINT32_C(2119408419), UINT32_C(3091878410),
                              UINT32_C(3978436943), UINT32_C(1708684203), UINT32_C(1202455481), UINT32_C(2187745469)),
      simde_x_mm512_set_epu32(UINT32_C(1572579389), UINT32_C(3511888959), UINT32_C(2399346014), UINT32_C(1967093325),
                              UINT32_C( 908815803), UINT32_C(2319376026), UINT32_C(3519587969), UINT32_C( 446590202),
                              UINT32_C(1610322797), UINT32_C(1767270276), UINT32_C( 490610321), UINT32_C(1164436618),
                              UINT32_C(2344919086), UINT32_C( 238137788), UINT32_C(1009808067), UINT32_C( 101587550)) },
    { simde_x_mm512_set_epu32(UINT32_C(2117071873), UINT32_C(2857077767), UINT32_C(3918893192), UINT32_C(1087893388),
                              UINT32_C(3851784011), UINT32_C(3914271744), UINT32_C( 565328458), UINT32_C(4201942548),
                              UINT32_C(1480532604), UINT32_C(4197506536), UINT32_C(3712719696), UINT32_C(3920217826),
                              UINT32_C(1394313506), UINT32_C( 394553965), UINT32_C(2278253176), UINT32_C(1697927724)),
      UINT16_C(12254),
      simde_x_mm512_set_epu32(UINT32_C(  56443211), UINT32_C(2258452653), UINT32_C(3784696472), UINT32_C(1139427205),
                              UINT32_C(1090384090), UINT32_C(2389735891), UINT32_C(2215607313), UINT32_C(3817672405),
                              UINT32_C(3621770268), UINT32_C(2071747620), UINT32_C(3852178197), UINT32_C(3693632585),
                              UINT32_C( 319530416), UINT32_C(2179954815), UINT32_C(3793236393), UINT32_C( 340519338)),
      simde_x_mm512_set_epu32(UINT32_C(1219537084), UINT32_C(1349635715), UINT32_C( 732887738), UINT32_C(2566325375),
                              UINT32_C(2906533885), UINT32_C(1765754685), UINT32_C(2719983633), UINT32_C( 846129112),
                              UINT32_C(1578410935), UINT32_C(2635094838), UINT32_C(1045536663), UINT32_C( 957117985),
                              UINT32_C(3029008645), UINT32_C(1309498779), UINT32_C(3293951997), UINT32_C(1022360677)),
      simde_x_mm512_set_epu32(UINT32_C(2117071873), UINT32_C(2857077767), UINT32_C( 120257782), UINT32_C(1087893388),
                              UINT32_C(1090384090), UINT32_C( 623981206), UINT32_C(2215607313), UINT32_C( 433155957),
                              UINT32_C( 464948398), UINT32_C(2071747620), UINT32_C(3712719696), UINT32_C( 822278630),
                              UINT32_C( 319530416), UINT32_C( 870456036), UINT32_C( 499284396), UINT32_C(1697927724)) },
    { simde_x_mm512_set_epu32(UINT32_C(3990081318), UINT32_C( 991545752), UINT32_C(4151932359), UINT32_C( 843112042),
                              UINT32_C(4067412513), UINT32_C(2124182542), UINT32_C(2768721208), UINT32_C(2302989914),
                              UINT32_C(1224533822), UINT32_C(3475606100), UINT32_C(3610957044), UINT32_C(2556046111),
                              UINT32_C(3035396524), UINT32_C(3603101367), UINT32_C(3321443925), UINT32_C(  45581573)),
      UINT16_C(42669),
      simde_x_mm512_set_epu32(UINT32_C(4138167693), UINT32_C(3221954957), UINT32_C(2164435171), UINT32_C( 397240391),
                              UINT32_C( 200936922), UINT32_C(3263986987), UINT32_C(2536604122), UINT32_C(3629380929),
                              UINT32_C( 453331046), UINT32_C(1704580573), UINT32_C(1606190487), UINT32_C(3209309249),
                              UINT32_C(2959497652), UINT32_C(3926896735), UINT32_C(2875407663), UINT32_C(2069966669)),
      simde_x_mm512_set_epu32(UINT32_C(1379668640), UINT32_C(  66581512), UINT32_C(3737665499), UINT32_C( 304428974),
                              UINT32_C(2686704508), UINT32_C( 532978979), UINT32_C( 946958552), UINT32_C(2383642627),
                              UINT32_C(2176874140), UINT32_C( 283691898), UINT32_C(3848894665), UINT32_C(3836186002),
                              UINT32_C(1951055651), UINT32_C( 765387914), UINT32_C( 822559116), UINT32_C(   7445617)),
      simde_x_mm512_set_epu32(UINT32_C(1378830413), UINT32_C( 991545752), UINT32_C(2164435171), UINT32_C( 843112042),
                              UINT32_C(4067412513), UINT32_C(  66113113), UINT32_C( 642687018), UINT32_C(2302989914),
                              UINT32_C( 453331046), UINT32_C(3475606100), UINT32_C(1606190487), UINT32_C(2556046111),
                              UINT32_C(1008442001), UINT32_C(  99957165), UINT32_C(3321443925), UINT32_C(     85143)) },
    { simde_x_mm512_set_epu32(UINT32_C(2313028370), UINT32_C( 869237081), UINT32_C(4104913762), UINT32_C(2825691966),
                              UINT32_C(3577866502), UINT32_C(2991894408), UINT32_C(2172048625), UINT32_C(1617119933),
                              UINT32_C(1521363431), UINT32_C( 553638116), UINT32_C(1036201367), UINT32_C(3107033445),
                              UINT32_C(3882811410), UINT32_C(3534384353), UINT32_C(3871215839), UINT32_C(1273589632)),
      UINT16_C(35103),
      simde_x_mm512_set_epu32(UINT32_C(2458371652), UINT32_C( 260676470), UINT32_C(1724614860), UINT32_C(4150452663),
                              UINT32_C(3816336716), UINT32_C(2208212235), UINT32_C( 932145867), UINT32_C(2432594561),
                              UINT32_C(1756892633), UINT32_C( 382632965), UINT32_C(1295078740), UINT32_C(3299165262),
                              UINT32_C( 152308919), UINT32_C(3943411788), UINT32_C(  31813624), UINT32_C( 807463845)),
      simde_x_mm512_set_epu32(UINT32_C( 615301803), UINT32_C( 382786341), UINT32_C(1852603705), UINT32_C(1998007730),
                              UINT32_C( 231325888), UINT32_C(1842039329), UINT32_C( 968682756), UINT32_C( 316335394),
                              UINT32_C(2223585202), UINT32_C(3491781959), UINT32_C(2167971796), UINT32_C(1587647099),
                              UINT32_C(2966608712), UINT32_C( 320339033), UINT32_C( 282380179), UINT32_C(4186865204)),
      simde_x_mm512_set_epu32(UINT32_C( 612466243), UINT32_C( 869237081), UINT32_C(4104913762), UINT32_C(2825691966),
                              UINT32_C( 115122508), UINT32_C(2991894408), UINT32_C(2172048625), UINT32_C( 218246803),
                              UINT32_C(1521363431), UINT32_C( 553638116), UINT32_C(1036201367), UINT32_C( 123871064),
                              UINT32_C( 152308919), UINT32_C(  99343392), UINT32_C(  31813624), UINT32_C( 807463845)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_mask_rem_epu32(test_vec[i].src, test_vec[i].k, test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm256_rem_epu64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i r;
  } test_vec[8] = {
    { simde_x_mm256_set_epu64x(UINT64_C(13810255550447513201), UINT64_C( 6479913377553186648),
                               UINT64_C( 7505871096235581515), UINT64_C( 2265477367564496986)),
      simde_x_mm256_set_epu64x(UINT64_C( 2968342496979776051), UINT64_C(10005265515001776413),
                               UINT64_C(16837535683400356038), UINT64_C( 6738163160628300797)),
      simde_x_mm256_set_epu64x(UINT64_C( 1936885562528408997), UINT64_C( 6479913377553186648),
                               UINT64_C( 7505871096235581515), UINT64_C( 2265477367564496986)) },
    { simde_x_mm256_set_epu64x(UINT64_C( 5120732502404950997), UINT64_C( 5681284513410730040),
                               UINT64_C( 6409558907924801050), UINT64_C( 5372227444888762251)),
      simde_x_mm256_set_epu64x(UINT64_C(10866939104613927783), UINT64_C(11375825163207743431),
                               UINT64_C(11633520338587575573), UINT64_C(  216242550290965460)),
      simde_x_mm256_set_epu64x(UINT64_C( 5120732502404950997), UINT64_C( 5681284513410730040),
                               UINT64_C( 6409558907924801050), UINT64_C(  182406237905591211)) },
    { simde_x_mm256_set_epu64x(UINT64_C( 2118113466433927893), UINT64_C( 3840651400764901876),
                               UINT64_C( 2114726288902596757), UINT64_C( 9482369585348649466)),
      simde_x_mm256_set_epu64x(UINT64_C( 5645659480511055559), UINT64_C(15272728730484288257),
                               UINT64_C(14133460247011230967), UINT64_C(16423537638667915170)),
      simde_x_mm256_set_epu64x(UINT64_C( 2118113466433927893), UINT64_C( 3840651400764901876),
                               UINT64_C( 2114726288902596757), UINT64_C( 9482369585348649466)) },
    { simde_x_mm256_set_epu64x(UINT64_C( 7345032902979795528), UINT64_C(12329133549512917827),
                               UINT64_C( 2328100732832272381), UINT64_C( 4111895855610225675)),
      simde_x_mm256_set_epu64x(UINT64_C(13555234896536583899), UINT64_C( 7851952110853286921),
                               UINT64_C(17616907291198234572), UINT64_C(16521184395064581900)),
      simde_x_mm256_set_epu64x(UINT64_C( 7345032902979795528), UINT64_C( 4477181438659630906),
                               UINT64_C( 2328100732832272381), UINT64_C( 4111895855610225675)) },
    { simde_x_mm256_set_epu64x(UINT64_C( 7449607714297299576), UINT64_C(14401023659121376347),
                               UINT64_C( 8569312554655704071), UINT64_C(10336200663482757951)),
      simde_x_mm256_set_epu64x(UINT64_C( 7572458917823766705), UINT64_C(12217500042222052906),
                               UINT64_C( 1159256113650983207), UINT64_C( 6193154838246823767)),
      simde_x_mm256_set_epu64x(UINT64_C( 7449607714297299576), UINT64_C( 2183523616899323441),
                               UINT64_C(  454519759098821622), UINT64_C( 4143045825235934184)) },
    { simde_x_mm256_set_epu64x(UINT64_C(16888199589465789243), UINT64_C(18237918400292775569),
                               UINT64_C(12279468594349909724), UINT64_C(13493341674566517412)),
      simde_x_mm256_set_epu64x(UINT64_C(15588592630942564743), UINT64_C( 9028813919053392068),
                               UINT64_C( 5083059030774095197), UINT64_C(13888425720366328200)),
      simde_x_mm256_set_epu64x(UINT64_C( 1299606958523224500), UINT64_C(  180290562185991433),
                               UINT64_C( 2113350532801719330), UINT64_C(13493341674566517412)) },
    { simde_x_mm256_set_epu64x(UINT64_C( 3908684742628183808), UINT64_C(15775432521885308750),
                               UINT64_C( 2246668589251707300), UINT64_C( 9474721517893975343)),
      simde_x_mm256_set_epu64x(UINT64_C( 2129749246616352421), UINT64_C( 3930946101587052880),
                               UINT64_C( 6109596926925725236), UINT64_C(11931707044738783755)),
      simde_x_mm256_set_epu64x(UINT64_C( 1778935496011831387), UINT64_C(   51648115537097230),
                               UINT64_C( 2246668589251707300), UINT64_C( 9474721517893975343)) },
    { simde_x_mm256_set_epu64x(UINT64_C( 7540605987113962845), UINT64_C(13935122940778806069),
                               UINT64_C( 9355601638871447350), UINT64_C(17674380633802211723)),
      simde_x_mm256_set_epu64x(UINT64_C( 5181754748372749246), UINT64_C( 2283432752406648940),
                               UINT64_C(17825612137522679693), UINT64_C( 6205295972918594513)),
      simde_x_mm256_set_epu64x(UINT64_C( 2358851238741213599), UINT64_C(  234526426338912429),
                               UINT64_C( 9355601638871447350), UINT64_C( 5263788687965022697)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i r = simde_mm256_rem_epu64(test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_u64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_rem_epi8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi8(INT8_C(  41), INT8_C(  49), INT8_C( -85), INT8_C( -58),
                           INT8_C(  40), INT8_C(  44), INT8_C( -14), INT8_C(  51),
                           INT8_C(-118), INT8_C( -39), INT8_C( -41), INT8_C(  -7),
                           INT8_C( -55), INT8_C(  37), INT8_C(-119), INT8_C(  29),
                           INT8_C( -23), INT8_C( -86), INT8_C( -15), INT8_C( 126),
                           INT8_C( -74), INT8_C(  10), INT8_C( -48), INT8_C( -58),
                           INT8_C(  93), INT8_C(-126), INT8_C( -61), INT8_C( -79),
                           INT8_C( -69), INT8_C( -33), INT8_C(-117), INT8_C(  -3),
                           INT8_C( -65), INT8_C( -89), INT8_C( -30), INT8_C(  64),
                           INT8_C( -43), INT8_C( -54), INT8_C( 110), INT8_C( 113),
                           INT8_C(  89), INT8_C( -19), INT8_C(  70), INT8_C( -30),
                           INT8_C(-124), INT8_C(  91), INT8_C(  -1), INT8_C(  88),
                           INT8_C( 104), INT8_C(  42), INT8_C(  53), INT8_C(  -2),
                           INT8_C(-124), INT8_C(  -2), INT8_C(  96), INT8_C(  75),
                           INT8_C(  31), INT8_C( 112), INT8_C(-105), INT8_C( -87),
                           INT8_C( -84), INT8_C(  94), INT8_C( 112), INT8_C(  90)),
      simde_mm512_set_epi8(INT8_C( -61), INT8_C(  49), INT8_C(  14), INT8_C( -86),
                           INT8_C( -53), INT8_C( -89), INT8_C(   3), INT8_C( -41),
                           INT8_C(  63), INT8_C(  -8), INT8_C(  55), INT8_C( -37),
                           INT8_C( -35), INT8_C(-121), INT8_C(  61), INT8_C( -65),
                           INT8_C( -47), INT8_C(  91), INT8_C(  87), INT8_C(-119),
                           INT8_C(  87), INT8_C(  76), INT8_C(  44), INT8_C(-116),
                           INT8_C(   2), INT8_C( -56), INT8_C(  36), INT8_C( -61),
                           INT8_C( -56), INT8_C( 125), INT8_C(  -2), INT8_C(-117),
                           INT8_C( -30), INT8_C(  71), INT8_C(  92), INT8_C(-127),
                           INT8_C( -74), INT8_C( 119), INT8_C(  -9), INT8_C(  34),
                           INT8_C( 121), INT8_C(  85), INT8_C(-103), INT8_C( 116),
                           INT8_C( -38), INT8_C(  21), INT8_C( 101), INT8_C( 122),
                           INT8_C(  10), INT8_C( -25), INT8_C(  54), INT8_C(  71),
                           INT8_C(-100), INT8_C(-107), INT8_C( -12), INT8_C(  84),
                           INT8_C(-108), INT8_C(  85), INT8_C( -86), INT8_C( -72),
                           INT8_C(  94), INT8_C(-102), INT8_C( -27), INT8_C(  11)),
      simde_mm512_set_epi8(INT8_C(  41), INT8_C(   0), INT8_C(  -1), INT8_C( -58),
                           INT8_C(  40), INT8_C(  44), INT8_C(  -2), INT8_C(  10),
                           INT8_C( -55), INT8_C(  -7), INT8_C( -41), INT8_C(  -7),
                           INT8_C( -20), INT8_C(  37), INT8_C( -58), INT8_C(  29),
                           INT8_C( -23), INT8_C( -86), INT8_C( -15), INT8_C(   7),
                           INT8_C( -74), INT8_C(  10), INT8_C(  -4), INT8_C( -58),
                           INT8_C(   1), INT8_C( -14), INT8_C( -25), INT8_C( -18),
                           INT8_C( -13), INT8_C( -33), INT8_C(  -1), INT8_C(  -3),
                           INT8_C(  -5), INT8_C( -18), INT8_C( -30), INT8_C(  64),
                           INT8_C( -43), INT8_C( -54), INT8_C(   2), INT8_C(  11),
                           INT8_C(  89), INT8_C( -19), INT8_C(  70), INT8_C( -30),
                           INT8_C( -10), INT8_C(   7), INT8_C(  -1), INT8_C(  88),
                           INT8_C(   4), INT8_C(  17), INT8_C(  53), INT8_C(  -2),
                           INT8_C( -24), INT8_C(  -2), INT8_C(   0), INT8_C(  75),
                           INT8_C(  31), INT8_C(  27), INT8_C( -19), INT8_C( -15),
                           INT8_C( -84), INT8_C(  94), INT8_C(   4), INT8_C(   2)) },
    { simde_mm512_set_epi8(INT8_C( -40), INT8_C(  85), INT8_C( -50), INT8_C( 103),
                           INT8_C( -21), INT8_C(-102), INT8_C(-127), INT8_C(-121),
                           INT8_C( 125), INT8_C(  76), INT8_C( -54), INT8_C( 108),
                           INT8_C(  52), INT8_C(  71), INT8_C( -88), INT8_C( -60),
                           INT8_C(  70), INT8_C(-118), INT8_C( -89), INT8_C(  65),
                           INT8_C( -35), INT8_C( -95), INT8_C( -99), INT8_C(  93),
                           INT8_C( -64), INT8_C( -67), INT8_C(-103), INT8_C(-101),
                           INT8_C( -49), INT8_C( -43), INT8_C( 105), INT8_C(-120),
                           INT8_C( -22), INT8_C(  94), INT8_C( -16), INT8_C(  12),
                           INT8_C(-110), INT8_C(   1), INT8_C(-109), INT8_C(  59),
                           INT8_C(  -3), INT8_C(  26), INT8_C(  26), INT8_C(  40),
                           INT8_C(  12), INT8_C(   2), INT8_C( -26), INT8_C(-111),
                           INT8_C( -86), INT8_C( 105), INT8_C( 111), INT8_C( -96),
                           INT8_C(-116), INT8_C( -54), INT8_C( -90), INT8_C( -36),
                           INT8_C( -69), INT8_C(  65), INT8_C(  -6), INT8_C( -61),
                           INT8_C(  33), INT8_C(-125), INT8_C(   2), INT8_C( -92)),
      simde_mm512_set_epi8(INT8_C( 120), INT8_C( 127), INT8_C(  28), INT8_C(  95),
                           INT8_C( -81), INT8_C( -33), INT8_C( 119), INT8_C( -42),
                           INT8_C( -36), INT8_C( 102), INT8_C(  86), INT8_C(  22),
                           INT8_C( 119), INT8_C( -49), INT8_C(  12), INT8_C( -73),
                           INT8_C( -84), INT8_C( -14), INT8_C( -83), INT8_C(  -7),
                           INT8_C(  52), INT8_C( 108), INT8_C(-128), INT8_C( -53),
                           INT8_C(  85), INT8_C(-121), INT8_C( -29), INT8_C(  35),
                           INT8_C( -69), INT8_C(  24), INT8_C(  -6), INT8_C( -37),
                           INT8_C(  -3), INT8_C(  62), INT8_C( 125), INT8_C( -20),
                           INT8_C(  75), INT8_C(  13), INT8_C(  79), INT8_C(  81),
                           INT8_C( -79), INT8_C( -35), INT8_C(  -5), INT8_C( -75),
                           INT8_C( -97), INT8_C( -74), INT8_C(  11), INT8_C(  11),
                           INT8_C(  39), INT8_C(  37), INT8_C(  39), INT8_C( -48),
                           INT8_C(-120), INT8_C( -76), INT8_C( -41), INT8_C(-117),
                           INT8_C(-112), INT8_C(-128), INT8_C( -53), INT8_C( -50),
                           INT8_C( -83), INT8_C(  36), INT8_C(-123), INT8_C( -81)),
      simde_mm512_set_epi8(INT8_C( -40), INT8_C(  85), INT8_C( -22), INT8_C(   8),
                           INT8_C( -21), INT8_C(  -3), INT8_C(  -8), INT8_C( -37),
                           INT8_C(  17), INT8_C(  76), INT8_C( -54), INT8_C(  20),
                           INT8_C(  52), INT8_C(  22), INT8_C(  -4), INT8_C( -60),
                           INT8_C(  70), INT8_C(  -6), INT8_C(  -6), INT8_C(   2),
                           INT8_C( -35), INT8_C( -95), INT8_C( -99), INT8_C(  40),
                           INT8_C( -64), INT8_C( -67), INT8_C( -16), INT8_C( -31),
                           INT8_C( -49), INT8_C( -19), INT8_C(   3), INT8_C(  -9),
                           INT8_C(  -1), INT8_C(  32), INT8_C( -16), INT8_C(  12),
                           INT8_C( -35), INT8_C(   1), INT8_C( -30), INT8_C(  59),
                           INT8_C(  -3), INT8_C(  26), INT8_C(   1), INT8_C(  40),
                           INT8_C(  12), INT8_C(   2), INT8_C(  -4), INT8_C(  -1),
                           INT8_C(  -8), INT8_C(  31), INT8_C(  33), INT8_C(   0),
                           INT8_C(-116), INT8_C( -54), INT8_C(  -8), INT8_C( -36),
                           INT8_C( -69), INT8_C(  65), INT8_C(  -6), INT8_C( -11),
                           INT8_C(  33), INT8_C( -17), INT8_C(   2), INT8_C( -11)) },
    { simde_mm512_set_epi8(INT8_C(  87), INT8_C(  63), INT8_C(  47), INT8_C(  80),
                           INT8_C(  35), INT8_C( -27), INT8_C(   5), INT8_C(  31),
                           INT8_C( -28), INT8_C(  73), INT8_C(  53), INT8_C(  47),
                           INT8_C( -86), INT8_C( -64), INT8_C( 122), INT8_C( -19),
                           INT8_C(  47), INT8_C(-126), INT8_C( -37), INT8_C( 102),
                           INT8_C( -93), INT8_C(  41), INT8_C( -61), INT8_C( -41),
                           INT8_C( -57), INT8_C(  54), INT8_C(  97), INT8_C( 126),
                           INT8_C(  10), INT8_C( -91), INT8_C(-101), INT8_C(  88),
                           INT8_C( -72), INT8_C(  63), INT8_C(  95), INT8_C( -92),
                           INT8_C(  65), INT8_C(  71), INT8_C( -82), INT8_C(  88),
                           INT8_C( -73), INT8_C(-114), INT8_C(  98), INT8_C(  14),
                           INT8_C(  25), INT8_C( -83), INT8_C(  87), INT8_C(   2),
                           INT8_C( -65), INT8_C(-113), INT8_C(-104), INT8_C(   2),
                           INT8_C( 126), INT8_C(   0), INT8_C( -94), INT8_C(  57),
                           INT8_C( -11), INT8_C(  36), INT8_C( -17), INT8_C(  54),
                           INT8_C(  33), INT8_C( -91), INT8_C( -57), INT8_C(  84)),
      simde_mm512_set_epi8(INT8_C(-125), INT8_C(  42), INT8_C(-105), INT8_C( -46),
                           INT8_C(  12), INT8_C( -93), INT8_C(-118), INT8_C( -49),
                           INT8_C(  43), INT8_C(  57), INT8_C(  61), INT8_C(  62),
                           INT8_C(  81), INT8_C( -72), INT8_C(   6), INT8_C(  93),
                           INT8_C( -89), INT8_C(   1), INT8_C(-111), INT8_C(   9),
                           INT8_C(   4), INT8_C(  17), INT8_C(  10), INT8_C( 101),
                           INT8_C( -70), INT8_C( -75), INT8_C(-101), INT8_C( -13),
                           INT8_C( -67), INT8_C( -65), INT8_C( -34), INT8_C( -51),
                           INT8_C(  59), INT8_C(  26), INT8_C( -29), INT8_C( 105),
                           INT8_C( -19), INT8_C(-111), INT8_C( -73), INT8_C(  79),
                           INT8_C( -82), INT8_C(  60), INT8_C(-124), INT8_C( -48),
                           INT8_C(  58), INT8_C( -78), INT8_C( 116), INT8_C( -16),
                           INT8_C(  37), INT8_C(-125), INT8_C( 100), INT8_C( -79),
                           INT8_C(  19), INT8_C( 102), INT8_C(  81), INT8_C(  86),
                           INT8_C(  25), INT8_C(  43), INT8_C(  51), INT8_C(-116),
                           INT8_C(   9), INT8_C(  40), INT8_C( -29), INT8_C(  75)),
      simde_mm512_set_epi8(INT8_C(  87), INT8_C(  21), INT8_C(  47), INT8_C(  34),
                           INT8_C(  11), INT8_C( -27), INT8_C(   5), INT8_C(  31),
                           INT8_C( -28), INT8_C(  16), INT8_C(  53), INT8_C(  47),
                           INT8_C(  -5), INT8_C( -64), INT8_C(   2), INT8_C( -19),
                           INT8_C(  47), INT8_C(   0), INT8_C( -37), INT8_C(   3),
                           INT8_C(  -1), INT8_C(   7), INT8_C(  -1), INT8_C( -41),
                           INT8_C( -57), INT8_C(  54), INT8_C(  97), INT8_C(   9),
                           INT8_C(  10), INT8_C( -26), INT8_C( -33), INT8_C(  37),
                           INT8_C( -13), INT8_C(  11), INT8_C(   8), INT8_C( -92),
                           INT8_C(   8), INT8_C(  71), INT8_C(  -9), INT8_C(   9),
                           INT8_C( -73), INT8_C( -54), INT8_C(  98), INT8_C(  14),
                           INT8_C(  25), INT8_C(  -5), INT8_C(  87), INT8_C(   2),
                           INT8_C( -28), INT8_C(-113), INT8_C(  -4), INT8_C(   2),
                           INT8_C(  12), INT8_C(   0), INT8_C( -13), INT8_C(  57),
                           INT8_C( -11), INT8_C(  36), INT8_C( -17), INT8_C(  54),
                           INT8_C(   6), INT8_C( -11), INT8_C( -28), INT8_C(   9)) },
    { simde_mm512_set_epi8(INT8_C( -23), INT8_C(  79), INT8_C(  12), INT8_C(   0),
                           INT8_C(  33), INT8_C( -78), INT8_C(  58), INT8_C(  74),
                           INT8_C(  -6), INT8_C( 116), INT8_C(-114), INT8_C(  20),
                           INT8_C(  88), INT8_C(  63), INT8_C(  34), INT8_C( 124),
                           INT8_C(  -6), INT8_C(  48), INT8_C( -35), INT8_C( -24),
                           INT8_C( -35), INT8_C(  75), INT8_C(-101), INT8_C(  80),
                           INT8_C( -23), INT8_C( -87), INT8_C( -58), INT8_C( -30),
                           INT8_C(  83), INT8_C(  27), INT8_C(-119), INT8_C(  34),
                           INT8_C(  23), INT8_C(-124), INT8_C( 106), INT8_C( 109),
                           INT8_C(-121), INT8_C( -53), INT8_C(  98), INT8_C( 120),
                           INT8_C( 101), INT8_C(  52), INT8_C(  82), INT8_C(  44),
                           INT8_C(-114), INT8_C(  14), INT8_C(  99), INT8_C( -11),
                           INT8_C(   8), INT8_C(-116), INT8_C(-115), INT8_C( 123),
                           INT8_C( -37), INT8_C( -93), INT8_C( -60), INT8_C( -23),
                           INT8_C(  34), INT8_C( -71), INT8_C( -28), INT8_C( 108),
                           INT8_C(  95), INT8_C( -20), INT8_C(  97), INT8_C(  41)),
      simde_mm512_set_epi8(INT8_C( -63), INT8_C( -26), INT8_C(  93), INT8_C(  23),
                           INT8_C( -63), INT8_C(  52), INT8_C( -33), INT8_C( -81),
                           INT8_C( -51), INT8_C(  45), INT8_C( -90), INT8_C(  24),
                           INT8_C(  71), INT8_C( -22), INT8_C( -95), INT8_C(-114),
                           INT8_C( -72), INT8_C( -38), INT8_C( -66), INT8_C( -44),
                           INT8_C( 116), INT8_C( -97), INT8_C(  44), INT8_C(  55),
                           INT8_C( -43), INT8_C(-123), INT8_C(  60), INT8_C(   3),
                           INT8_C(  58), INT8_C(  -1), INT8_C( 125), INT8_C( -67),
                           INT8_C(-111), INT8_C(  88), INT8_C(  55), INT8_C( -74),
                           INT8_C(  23), INT8_C( -95), INT8_C(-123), INT8_C(  27),
                           INT8_C( 125), INT8_C( -27), INT8_C( -53), INT8_C(  45),
                           INT8_C(  24), INT8_C(   5), INT8_C(  90), INT8_C(  83),
                           INT8_C(-111), INT8_C(  85), INT8_C(-100), INT8_C( -92),
                           INT8_C(-107), INT8_C( -55), INT8_C(  48), INT8_C(  -1),
                           INT8_C(  41), INT8_C(  42), INT8_C(  94), INT8_C(-127),
                           INT8_C(-121), INT8_C(   8), INT8_C(  12), INT8_C( -53)),
      simde_mm512_set_epi8(INT8_C( -23), INT8_C(   1), INT8_C(  12), INT8_C(   0),
                           INT8_C(  33), INT8_C( -26), INT8_C(  25), INT8_C(  74),
                           INT8_C(  -6), INT8_C(  26), INT8_C( -24), INT8_C(  20),
                           INT8_C(  17), INT8_C(  19), INT8_C(  34), INT8_C(  10),
                           INT8_C(  -6), INT8_C(  10), INT8_C( -35), INT8_C( -24),
                           INT8_C( -35), INT8_C(  75), INT8_C( -13), INT8_C(  25),
                           INT8_C( -23), INT8_C( -87), INT8_C( -58), INT8_C(   0),
                           INT8_C(  25), INT8_C(   0), INT8_C(-119), INT8_C(  34),
                           INT8_C(  23), INT8_C( -36), INT8_C(  51), INT8_C(  35),
                           INT8_C(  -6), INT8_C( -53), INT8_C(  98), INT8_C(  12),
                           INT8_C( 101), INT8_C(  25), INT8_C(  29), INT8_C(  44),
                           INT8_C( -18), INT8_C(   4), INT8_C(   9), INT8_C( -11),
                           INT8_C(   8), INT8_C( -31), INT8_C( -15), INT8_C(  31),
                           INT8_C( -37), INT8_C( -38), INT8_C( -12), INT8_C(   0),
                           INT8_C(  34), INT8_C( -29), INT8_C( -28), INT8_C( 108),
                           INT8_C(  95), INT8_C(  -4), INT8_C(   1), INT8_C(  41)) },
    { simde_mm512_set_epi8(INT8_C(-114), INT8_C(  19), INT8_C(-128), INT8_C(   3),
                           INT8_C(-127), INT8_C( -64), INT8_C( 118), INT8_C(-100),
                           INT8_C(  16), INT8_C( -24), INT8_C( -53), INT8_C( 122),
                           INT8_C( -27), INT8_C( 105), INT8_C( 120), INT8_C( -55),
                           INT8_C( -28), INT8_C( -89), INT8_C(-115), INT8_C(-110),
                           INT8_C( 116), INT8_C(  74), INT8_C( -65), INT8_C(  35),
                           INT8_C(  45), INT8_C( -98), INT8_C( -28), INT8_C(-118),
                           INT8_C(  49), INT8_C(   7), INT8_C(  65), INT8_C(-116),
                           INT8_C(   0), INT8_C( 113), INT8_C(-100), INT8_C( 113),
                           INT8_C( -10), INT8_C( -89), INT8_C( 109), INT8_C(-115),
                           INT8_C( -64), INT8_C(  11), INT8_C(  33), INT8_C(-115),
                           INT8_C(-127), INT8_C(   2), INT8_C( -88), INT8_C( -29),
                           INT8_C(  23), INT8_C( -83), INT8_C( 104), INT8_C(  71),
                           INT8_C(  11), INT8_C(  -6), INT8_C(  13), INT8_C( -38),
                           INT8_C( -62), INT8_C(-116), INT8_C( 125), INT8_C(  43),
                           INT8_C(-105), INT8_C(  49), INT8_C(-127), INT8_C( -38)),
      simde_mm512_set_epi8(INT8_C(   8), INT8_C(  25), INT8_C(-109), INT8_C( -36),
                           INT8_C( -83), INT8_C(-118), INT8_C(  38), INT8_C(-106),
                           INT8_C(  35), INT8_C(  43), INT8_C( -91), INT8_C( -71),
                           INT8_C(  50), INT8_C(  64), INT8_C( -95), INT8_C(-124),
                           INT8_C( -94), INT8_C(  50), INT8_C( -57), INT8_C(  84),
                           INT8_C(  -5), INT8_C( -56), INT8_C( -39), INT8_C(  19),
                           INT8_C( -76), INT8_C( -60), INT8_C( -10), INT8_C(  76),
                           INT8_C(  55), INT8_C( -52), INT8_C(-117), INT8_C(  75),
                           INT8_C(   1), INT8_C(  89), INT8_C(-123), INT8_C( -44),
                           INT8_C( -50), INT8_C(  55), INT8_C( -52), INT8_C( 120),
                           INT8_C(  37), INT8_C( -97), INT8_C(-110), INT8_C( -39),
                           INT8_C( -30), INT8_C( -66), INT8_C(-122), INT8_C(   8),
                           INT8_C( 113), INT8_C(  61), INT8_C( 103), INT8_C( 100),
                           INT8_C(  23), INT8_C( -27), INT8_C(-110), INT8_C(  97),
                           INT8_C(  95), INT8_C(  32), INT8_C(-120), INT8_C(  91),
                           INT8_C(  46), INT8_C(  -4), INT8_C( -93), INT8_C(  88)),
      simde_mm512_set_epi8(INT8_C(  -2), INT8_C(  19), INT8_C( -19), INT8_C(   3),
                           INT8_C( -44), INT8_C( -64), INT8_C(   4), INT8_C(-100),
                           INT8_C(  16), INT8_C( -24), INT8_C( -53), INT8_C(  51),
                           INT8_C( -27), INT8_C(  41), INT8_C(  25), INT8_C( -55),
                           INT8_C( -28), INT8_C( -39), INT8_C(  -1), INT8_C( -26),
                           INT8_C(   1), INT8_C(  18), INT8_C( -26), INT8_C(  16),
                           INT8_C(  45), INT8_C( -38), INT8_C(  -8), INT8_C( -42),
                           INT8_C(  49), INT8_C(   7), INT8_C(  65), INT8_C( -41),
                           INT8_C(   0), INT8_C(  24), INT8_C(-100), INT8_C(  25),
                           INT8_C( -10), INT8_C( -34), INT8_C(   5), INT8_C(-115),
                           INT8_C( -27), INT8_C(  11), INT8_C(  33), INT8_C( -37),
                           INT8_C(  -7), INT8_C(   2), INT8_C( -88), INT8_C(  -5),
                           INT8_C(  23), INT8_C( -22), INT8_C(   1), INT8_C(  71),
                           INT8_C(  11), INT8_C(  -6), INT8_C(  13), INT8_C( -38),
                           INT8_C( -62), INT8_C( -20), INT8_C(   5), INT8_C(  43),
                           INT8_C( -13), INT8_C(   1), INT8_C( -34), INT8_C( -38)) },
    { simde_mm512_set_epi8(INT8_C(  46), INT8_C(  43), INT8_C( -10), INT8_C( -99),
                           INT8_C(  80), INT8_C(-102), INT8_C(  27), INT8_C( 118),
                           INT8_C( -80), INT8_C( -40), INT8_C(  46), INT8_C(-114),
                           INT8_C( -58), INT8_C(  -8), INT8_C(  88), INT8_C(  29),
                           INT8_C( -80), INT8_C(  25), INT8_C( 101), INT8_C(  54),
                           INT8_C( 103), INT8_C( 120), INT8_C(  94), INT8_C(  16),
                           INT8_C( -59), INT8_C( -51), INT8_C(  71), INT8_C( -10),
                           INT8_C( -98), INT8_C( -80), INT8_C( -38), INT8_C(  43),
                           INT8_C( -21), INT8_C(  -7), INT8_C( 116), INT8_C(-119),
                           INT8_C(  89), INT8_C( -44), INT8_C(-124), INT8_C(  56),
                           INT8_C( -26), INT8_C(-119), INT8_C(  66), INT8_C(  41),
                           INT8_C(  44), INT8_C(  35), INT8_C( -67), INT8_C(-101),
                           INT8_C( 125), INT8_C(-126), INT8_C( 123), INT8_C( 117),
                           INT8_C( 123), INT8_C( 127), INT8_C(-105), INT8_C(  60),
                           INT8_C(-103), INT8_C( -71), INT8_C(  -6), INT8_C( 100),
                           INT8_C(  83), INT8_C( 112), INT8_C(  33), INT8_C(-116)),
      simde_mm512_set_epi8(INT8_C(  36), INT8_C(  33), INT8_C(  42), INT8_C(  75),
                           INT8_C( -77), INT8_C( -84), INT8_C( 126), INT8_C( -85),
                           INT8_C( 110), INT8_C(-106), INT8_C( 107), INT8_C( -76),
                           INT8_C(-122), INT8_C(  73), INT8_C( -49), INT8_C(  15),
                           INT8_C( -15), INT8_C( 103), INT8_C( 103), INT8_C(-106),
                           INT8_C( 103), INT8_C(  58), INT8_C( 104), INT8_C(  35),
                           INT8_C(  -7), INT8_C(  79), INT8_C( 113), INT8_C(  97),
                           INT8_C( -67), INT8_C( -59), INT8_C( -82), INT8_C( -34),
                           INT8_C( -32), INT8_C( 104), INT8_C( 123), INT8_C( 124),
                           INT8_C(  49), INT8_C( -30), INT8_C(  37), INT8_C(  22),
                           INT8_C( 105), INT8_C( -99), INT8_C( 110), INT8_C(  52),
                           INT8_C(  -2), INT8_C( 103), INT8_C( -94), INT8_C( -46),
                           INT8_C( -54), INT8_C(  39), INT8_C( -63), INT8_C(-105),
                           INT8_C( -73), INT8_C(  73), INT8_C(  97), INT8_C( -69),
                           INT8_C( 102), INT8_C( -61), INT8_C(  68), INT8_C( -66),
                           INT8_C(  65), INT8_C(  60), INT8_C( -91), INT8_C( 126)),
      simde_mm512_set_epi8(INT8_C(  10), INT8_C(  10), INT8_C( -10), INT8_C( -24),
                           INT8_C(   3), INT8_C( -18), INT8_C(  27), INT8_C(  33),
                           INT8_C( -80), INT8_C( -40), INT8_C(  46), INT8_C( -38),
                           INT8_C( -58), INT8_C(  -8), INT8_C(  39), INT8_C(  14),
                           INT8_C(  -5), INT8_C(  25), INT8_C( 101), INT8_C(  54),
                           INT8_C(   0), INT8_C(   4), INT8_C(  94), INT8_C(  16),
                           INT8_C(  -3), INT8_C( -51), INT8_C(  71), INT8_C( -10),
                           INT8_C( -31), INT8_C( -21), INT8_C( -38), INT8_C(   9),
                           INT8_C( -21), INT8_C(  -7), INT8_C( 116), INT8_C(-119),
                           INT8_C(  40), INT8_C( -14), INT8_C( -13), INT8_C(  12),
                           INT8_C( -26), INT8_C( -20), INT8_C(  66), INT8_C(  41),
                           INT8_C(   0), INT8_C(  35), INT8_C( -67), INT8_C(  -9),
                           INT8_C(  17), INT8_C(  -9), INT8_C(  60), INT8_C(  12),
                           INT8_C(  50), INT8_C(  54), INT8_C(  -8), INT8_C(  60),
                           INT8_C(  -1), INT8_C( -10), INT8_C(  -6), INT8_C(  34),
                           INT8_C(  18), INT8_C(  52), INT8_C(  33), INT8_C(-116)) },
    { simde_mm512_set_epi8(INT8_C( -16), INT8_C( -87), INT8_C(   8), INT8_C(  54),
                           INT8_C(  66), INT8_C(  99), INT8_C(  14), INT8_C(  32),
                           INT8_C(-108), INT8_C(  92), INT8_C( 122), INT8_C( -56),
                           INT8_C( -64), INT8_C( -70), INT8_C( -31), INT8_C(  52),
                           INT8_C( -74), INT8_C( -12), INT8_C(  -3), INT8_C( -28),
                           INT8_C(-115), INT8_C( -28), INT8_C(-108), INT8_C( -88),
                           INT8_C( -25), INT8_C( 107), INT8_C(  47), INT8_C( -51),
                           INT8_C( 126), INT8_C(   7), INT8_C( -74), INT8_C( -11),
                           INT8_C( -91), INT8_C( -70), INT8_C( -43), INT8_C(  84),
                           INT8_C(  19), INT8_C(-125), INT8_C(  54), INT8_C(  13),
                           INT8_C( -71), INT8_C( -74), INT8_C(  72), INT8_C(  61),
                           INT8_C( 125), INT8_C( 104), INT8_C(-109), INT8_C(  11),
                           INT8_C(  89), INT8_C( -52), INT8_C(  62), INT8_C( -93),
                           INT8_C( -58), INT8_C( -94), INT8_C( -51), INT8_C(   9),
                           INT8_C( -74), INT8_C( 123), INT8_C(  65), INT8_C( -48),
                           INT8_C(-111), INT8_C( -77), INT8_C(  34), INT8_C( -61)),
      simde_mm512_set_epi8(INT8_C(-115), INT8_C( 103), INT8_C( 116), INT8_C(  12),
                           INT8_C( -82), INT8_C( -30), INT8_C( -63), INT8_C( -81),
                           INT8_C(-101), INT8_C( -82), INT8_C(  73), INT8_C(   6),
                           INT8_C(-115), INT8_C(-116), INT8_C(  -2), INT8_C( -63),
                           INT8_C( 100), INT8_C(-105), INT8_C(  14), INT8_C(  19),
                           INT8_C(  38), INT8_C( 115), INT8_C( -55), INT8_C( 118),
                           INT8_C(  74), INT8_C( -70), INT8_C(  89), INT8_C( -73),
                           INT8_C(  65), INT8_C(-118), INT8_C(  64), INT8_C(  90),
                           INT8_C(-104), INT8_C( -15), INT8_C( -27), INT8_C( -38),
                           INT8_C( 126), INT8_C(  38), INT8_C( -97), INT8_C(  27),
                           INT8_C( -92), INT8_C( -57), INT8_C(  25), INT8_C(  -3),
                           INT8_C( -75), INT8_C( 104), INT8_C(   6), INT8_C( -73),
                           INT8_C(  36), INT8_C( -53), INT8_C(-118), INT8_C(-111),
                           INT8_C( 116), INT8_C(-101), INT8_C( -38), INT8_C(  24),
                           INT8_C( -51), INT8_C( -18), INT8_C( -14), INT8_C(  26),
                           INT8_C( -30), INT8_C(  76), INT8_C( -30), INT8_C( -42)),
      simde_mm512_set_epi8(INT8_C( -16), INT8_C( -87), INT8_C(   8), INT8_C(   6),
                           INT8_C(  66), INT8_C(   9), INT8_C(  14), INT8_C(  32),
                           INT8_C(  -7), INT8_C(  10), INT8_C(  49), INT8_C(  -2),
                           INT8_C( -64), INT8_C( -70), INT8_C(  -1), INT8_C(  52),
                           INT8_C( -74), INT8_C( -12), INT8_C(  -3), INT8_C(  -9),
                           INT8_C(  -1), INT8_C( -28), INT8_C( -53), INT8_C( -88),
                           INT8_C( -25), INT8_C(  37), INT8_C(  47), INT8_C( -51),
                           INT8_C(  61), INT8_C(   7), INT8_C( -10), INT8_C( -11),
                           INT8_C( -91), INT8_C( -10), INT8_C( -16), INT8_C(   8),
                           INT8_C(  19), INT8_C( -11), INT8_C(  54), INT8_C(  13),
                           INT8_C( -71), INT8_C( -17), INT8_C(  22), INT8_C(   1),
                           INT8_C(  50), INT8_C(   0), INT8_C(  -1), INT8_C(  11),
                           INT8_C(  17), INT8_C( -52), INT8_C(  62), INT8_C( -93),
                           INT8_C( -58), INT8_C( -94), INT8_C( -13), INT8_C(   9),
                           INT8_C( -23), INT8_C(  15), INT8_C(   9), INT8_C( -22),
                           INT8_C( -21), INT8_C(  -1), INT8_C(   4), INT8_C( -19)) },
    { simde_mm512_set_epi8(INT8_C( -59), INT8_C(  52), INT8_C(-111), INT8_C(  20),
                           INT8_C(  26), INT8_C( -78), INT8_C( 121), INT8_C(  16),
                           INT8_C(  45), INT8_C( -27), INT8_C(  11), INT8_C( -26),
                           INT8_C(  53), INT8_C(   2), INT8_C( -22), INT8_C(   7),
                           INT8_C( -49), INT8_C(-110), INT8_C( -87), INT8_C( -23),
                           INT8_C( -50), INT8_C( 116), INT8_C(  55), INT8_C(-100),
                           INT8_C( -76), INT8_C(  91), INT8_C(  56), INT8_C(-110),
                           INT8_C(  55), INT8_C(-119), INT8_C( -56), INT8_C(  76),
                           INT8_C(  43), INT8_C( -11), INT8_C(-118), INT8_C(   3),
                           INT8_C( -43), INT8_C(-100), INT8_C( -90), INT8_C( -22),
                           INT8_C( -57), INT8_C(   2), INT8_C(  86), INT8_C(  72),
                           INT8_C(  93), INT8_C(  -2), INT8_C( -66), INT8_C( 121),
                           INT8_C( 119), INT8_C(  75), INT8_C( -97), INT8_C(  76),
                           INT8_C(  70), INT8_C( -38), INT8_C(  17), INT8_C( -17),
                           INT8_C(  43), INT8_C(-104), INT8_C( -34), INT8_C(  80),
                           INT8_C( -59), INT8_C( 113), INT8_C( 112), INT8_C(  81)),
      simde_mm512_set_epi8(INT8_C( -63), INT8_C( -94), INT8_C( -78), INT8_C(  36),
                           INT8_C( -78), INT8_C(  86), INT8_C(  79), INT8_C( -89),
                           INT8_C( -77), INT8_C(  45), INT8_C(  18), INT8_C( -25),
                           INT8_C( 113), INT8_C( 127), INT8_C( -45), INT8_C( -75),
                           INT8_C( 121), INT8_C( -85), INT8_C(  76), INT8_C(-121),
                           INT8_C(  15), INT8_C(-123), INT8_C(  -9), INT8_C(  32),
                           INT8_C( -75), INT8_C( -88), INT8_C( -20), INT8_C(  99),
                           INT8_C(  85), INT8_C(-105), INT8_C(  36), INT8_C(  99),
                           INT8_C( 101), INT8_C(  42), INT8_C(  63), INT8_C(  96),
                           INT8_C( -46), INT8_C( -58), INT8_C( -54), INT8_C( 105),
                           INT8_C( -42), INT8_C(  74), INT8_C( -57), INT8_C(  17),
                           INT8_C( -22), INT8_C(  22), INT8_C(-122), INT8_C( 112),
                           INT8_C(  62), INT8_C(-115), INT8_C(-100), INT8_C(  91),
                           INT8_C(  99), INT8_C(  24), INT8_C( -58), INT8_C(-125),
                           INT8_C(  88), INT8_C(-120), INT8_C(  61), INT8_C(  94),
                           INT8_C( -67), INT8_C( -43), INT8_C(  -7), INT8_C(-125)),
      simde_mm512_set_epi8(INT8_C( -59), INT8_C(  52), INT8_C( -33), INT8_C(  20),
                           INT8_C(  26), INT8_C( -78), INT8_C(  42), INT8_C(  16),
                           INT8_C(  45), INT8_C( -27), INT8_C(  11), INT8_C(  -1),
                           INT8_C(  53), INT8_C(   2), INT8_C( -22), INT8_C(   7),
                           INT8_C( -49), INT8_C( -25), INT8_C( -11), INT8_C( -23),
                           INT8_C(  -5), INT8_C( 116), INT8_C(   1), INT8_C(  -4),
                           INT8_C(  -1), INT8_C(   3), INT8_C(  16), INT8_C( -11),
                           INT8_C(  55), INT8_C( -14), INT8_C( -20), INT8_C(  76),
                           INT8_C(  43), INT8_C( -11), INT8_C( -55), INT8_C(   3),
                           INT8_C( -43), INT8_C( -42), INT8_C( -36), INT8_C( -22),
                           INT8_C( -15), INT8_C(   2), INT8_C(  29), INT8_C(   4),
                           INT8_C(   5), INT8_C(  -2), INT8_C( -66), INT8_C(   9),
                           INT8_C(  57), INT8_C(  75), INT8_C( -97), INT8_C(  76),
                           INT8_C(  70), INT8_C( -14), INT8_C(  17), INT8_C( -17),
                           INT8_C(  43), INT8_C(-104), INT8_C( -34), INT8_C(  80),
                           INT8_C( -59), INT8_C(  27), INT8_C(   0), INT8_C(  81)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_rem_epi8(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_rem_epi16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi16(INT16_C( 10545), INT16_C(-21562), INT16_C( 10284), INT16_C( -3533),
                            INT16_C(-29991), INT16_C(-10247), INT16_C(-14043), INT16_C(-30435),
                            INT16_C( -5718), INT16_C( -3714), INT16_C(-18934), INT16_C(-12090),
                            INT16_C( 23938), INT16_C(-15439), INT16_C(-17441), INT16_C(-29699),
                            INT16_C(-16473), INT16_C( -7616), INT16_C(-10806), INT16_C( 28273),
                            INT16_C( 23021), INT16_C( 18146), INT16_C(-31653), INT16_C(  -168),
                            INT16_C( 26666), INT16_C( 13822), INT16_C(-31490), INT16_C( 24651),
                            INT16_C(  8048), INT16_C(-26711), INT16_C(-21410), INT16_C( 28762)),
      simde_mm512_set_epi16(INT16_C(-26929), INT16_C(  8074), INT16_C( 18000), INT16_C(-29849),
                            INT16_C(-25121), INT16_C(  3254), INT16_C(-10254), INT16_C(-26681),
                            INT16_C(-24206), INT16_C(-28388), INT16_C( 25803), INT16_C( 25877),
                            INT16_C(   768), INT16_C( 16244), INT16_C( 11114), INT16_C( -7212),
                            INT16_C( 18192), INT16_C( 32532), INT16_C(-31836), INT16_C( -5163),
                            INT16_C( 20183), INT16_C( -1494), INT16_C(  2502), INT16_C( 18488),
                            INT16_C( 22771), INT16_C( 21470), INT16_C(  4556), INT16_C( 26138),
                            INT16_C( 19085), INT16_C(  -923), INT16_C( -9934), INT16_C( -2165)),
      simde_mm512_set_epi16(INT16_C( 10545), INT16_C( -5414), INT16_C( 10284), INT16_C( -3533),
                            INT16_C( -4870), INT16_C(  -485), INT16_C( -3789), INT16_C( -3754),
                            INT16_C( -5718), INT16_C( -3714), INT16_C(-18934), INT16_C(-12090),
                            INT16_C(   130), INT16_C(-15439), INT16_C( -6327), INT16_C(  -851),
                            INT16_C(-16473), INT16_C( -7616), INT16_C(-10806), INT16_C(  2458),
                            INT16_C(  2838), INT16_C(   218), INT16_C( -1629), INT16_C(  -168),
                            INT16_C(  3895), INT16_C( 13822), INT16_C( -4154), INT16_C( 24651),
                            INT16_C(  8048), INT16_C(  -867), INT16_C( -1542), INT16_C(   617)) },
    { simde_mm512_set_epi16(INT16_C( 20057), INT16_C( 26978), INT16_C(-19795), INT16_C(-31033),
                            INT16_C(-11277), INT16_C(-24100), INT16_C(-21653), INT16_C( 11009),
                            INT16_C(-15324), INT16_C(  9014), INT16_C( 24117), INT16_C(-31497),
                            INT16_C( -7188), INT16_C(  8311), INT16_C( 31759), INT16_C(  4002),
                            INT16_C(  7525), INT16_C(  3321), INT16_C(-18237), INT16_C( -1323),
                            INT16_C( 13644), INT16_C(-17383), INT16_C(-20302), INT16_C(-13836),
                            INT16_C(  7513), INT16_C(  1114), INT16_C(  -200), INT16_C( 10389),
                            INT16_C(-31848), INT16_C(  9445), INT16_C( -5204), INT16_C(-24070)),
      simde_mm512_set_epi16(INT16_C(-17379), INT16_C( -8623), INT16_C(-10486), INT16_C(-16677),
                            INT16_C( 27895), INT16_C(-17193), INT16_C( -5943), INT16_C( -5111),
                            INT16_C( -2949), INT16_C(-11305), INT16_C(-13092), INT16_C(  8140),
                            INT16_C( -6841), INT16_C(  2476), INT16_C(-24435), INT16_C(  7948),
                            INT16_C( 26094), INT16_C(-13182), INT16_C( 30122), INT16_C(-17848),
                            INT16_C(-21735), INT16_C( -7772), INT16_C(  1809), INT16_C(-31933),
                            INT16_C(  8271), INT16_C(  4936), INT16_C(  7627), INT16_C( 20477),
                            INT16_C( 14608), INT16_C( 25470), INT16_C(-19700), INT16_C( 25611)),
      simde_mm512_set_epi16(INT16_C(  2678), INT16_C(  1109), INT16_C( -9309), INT16_C(-14356),
                            INT16_C(-11277), INT16_C( -6907), INT16_C( -3824), INT16_C(   787),
                            INT16_C(  -579), INT16_C(  9014), INT16_C( 11025), INT16_C( -7077),
                            INT16_C(  -347), INT16_C(   883), INT16_C(  7324), INT16_C(  4002),
                            INT16_C(  7525), INT16_C(  3321), INT16_C(-18237), INT16_C( -1323),
                            INT16_C( 13644), INT16_C( -1839), INT16_C(  -403), INT16_C(-13836),
                            INT16_C(  7513), INT16_C(  1114), INT16_C(  -200), INT16_C( 10389),
                            INT16_C( -2632), INT16_C(  9445), INT16_C( -5204), INT16_C(-24070)) },
    { simde_mm512_set_epi16(INT16_C( 26902), INT16_C(-14525), INT16_C( -7905), INT16_C( -8015),
                            INT16_C(-22131), INT16_C( 18318), INT16_C(-21513), INT16_C(  9770),
                            INT16_C(  4118), INT16_C(-32437), INT16_C(  6621), INT16_C( -7897),
                            INT16_C( 22002), INT16_C(-32381), INT16_C( 15537), INT16_C(-26793),
                            INT16_C( 26466), INT16_C( 21183), INT16_C(  5811), INT16_C( 17016),
                            INT16_C(-14374), INT16_C(-18761), INT16_C(-11284), INT16_C(  -933),
                            INT16_C( 30444), INT16_C( 20573), INT16_C(-14964), INT16_C( 25607),
                            INT16_C(-28815), INT16_C(-28739), INT16_C( 27147), INT16_C( -3265)),
      simde_mm512_set_epi16(INT16_C(-10155), INT16_C(-12697), INT16_C( -5222), INT16_C(-32377),
                            INT16_C( 32076), INT16_C(-13716), INT16_C( 13383), INT16_C(-22332),
                            INT16_C( 18058), INT16_C(-22719), INT16_C( -8799), INT16_C(-25251),
                            INT16_C(-16195), INT16_C(-26213), INT16_C(-12331), INT16_C( 27016),
                            INT16_C( -5538), INT16_C( -4084), INT16_C(-28159), INT16_C(-27845),
                            INT16_C(  -742), INT16_C(  6696), INT16_C(  3074), INT16_C( -6511),
                            INT16_C(-21911), INT16_C( 28576), INT16_C(-29494), INT16_C(-22820),
                            INT16_C(-17599), INT16_C( -1341), INT16_C(  8579), INT16_C(   676)),
      simde_mm512_set_epi16(INT16_C(  6592), INT16_C( -1828), INT16_C( -2683), INT16_C( -8015),
                            INT16_C(-22131), INT16_C(  4602), INT16_C( -8130), INT16_C(  9770),
                            INT16_C(  4118), INT16_C( -9718), INT16_C(  6621), INT16_C( -7897),
                            INT16_C(  5807), INT16_C( -6168), INT16_C(  3206), INT16_C(-26793),
                            INT16_C(  4314), INT16_C(   763), INT16_C(  5811), INT16_C( 17016),
                            INT16_C(  -276), INT16_C( -5369), INT16_C( -2062), INT16_C(  -933),
                            INT16_C(  8533), INT16_C( 20573), INT16_C(-14964), INT16_C(  2787),
                            INT16_C(-11216), INT16_C(  -578), INT16_C(  1410), INT16_C(  -561)) },
    { simde_mm512_set_epi16(INT16_C(  7566), INT16_C( 25511), INT16_C( -5831), INT16_C( 13989),
                            INT16_C( 13965), INT16_C(-31065), INT16_C(    77), INT16_C(-30384),
                            INT16_C( 21705), INT16_C(-23032), INT16_C( -2503), INT16_C( -8652),
                            INT16_C(-23147), INT16_C( -4009), INT16_C(  7598), INT16_C( 23051),
                            INT16_C( 13886), INT16_C( 28688), INT16_C( 30551), INT16_C(-28928),
                            INT16_C( -9491), INT16_C(-26549), INT16_C(  -738), INT16_C( 22350),
                            INT16_C(  7981), INT16_C(-15059), INT16_C(-18848), INT16_C( 16804),
                            INT16_C(-31876), INT16_C( -1787), INT16_C( 29649), INT16_C(  -721)),
      simde_mm512_set_epi16(INT16_C( 18409), INT16_C( 19069), INT16_C( 20979), INT16_C(-29762),
                            INT16_C(  8112), INT16_C( 25085), INT16_C( 31664), INT16_C(-10132),
                            INT16_C( -2207), INT16_C( 19403), INT16_C(-32530), INT16_C( 20365),
                            INT16_C( 22045), INT16_C(-23601), INT16_C( 28665), INT16_C(-29743),
                            INT16_C( 26789), INT16_C(-25295), INT16_C(-31460), INT16_C(-29347),
                            INT16_C(-16029), INT16_C(-32645), INT16_C(-19836), INT16_C( 31541),
                            INT16_C(-32299), INT16_C(-14817), INT16_C( 22782), INT16_C(-18634),
                            INT16_C( -2744), INT16_C(   907), INT16_C(  9939), INT16_C(   395)),
      simde_mm512_set_epi16(INT16_C(  7566), INT16_C(  6442), INT16_C( -5831), INT16_C( 13989),
                            INT16_C(  5853), INT16_C( -5980), INT16_C(    77), INT16_C(-10120),
                            INT16_C(  1842), INT16_C( -3629), INT16_C( -2503), INT16_C( -8652),
                            INT16_C( -1102), INT16_C( -4009), INT16_C(  7598), INT16_C( 23051),
                            INT16_C( 13886), INT16_C(  3393), INT16_C( 30551), INT16_C(-28928),
                            INT16_C( -9491), INT16_C(-26549), INT16_C(  -738), INT16_C( 22350),
                            INT16_C(  7981), INT16_C(  -242), INT16_C(-18848), INT16_C( 16804),
                            INT16_C( -1692), INT16_C(  -880), INT16_C(  9771), INT16_C(  -326)) },
    { simde_mm512_set_epi16(INT16_C(-24983), INT16_C(  9260), INT16_C(  6846), INT16_C( 21618),
                            INT16_C( 20365), INT16_C( 26413), INT16_C(  7670), INT16_C(  6521),
                            INT16_C( 13052), INT16_C( 19892), INT16_C(-25515), INT16_C( -7444),
                            INT16_C( 12337), INT16_C( 14080), INT16_C(  6934), INT16_C( -4021),
                            INT16_C(  1885), INT16_C( 11733), INT16_C(  7371), INT16_C( 24583),
                            INT16_C(-17187), INT16_C(-28061), INT16_C(-18330), INT16_C(-10845),
                            INT16_C( -2076), INT16_C(  2107), INT16_C( -3367), INT16_C(-26728),
                            INT16_C( 21341), INT16_C(-13702), INT16_C( 26283), INT16_C(-27301)),
      simde_mm512_set_epi16(INT16_C(  9227), INT16_C( 20728), INT16_C( 22448), INT16_C( 22271),
                            INT16_C(-27526), INT16_C(  3228), INT16_C(-26938), INT16_C( 15839),
                            INT16_C(  4554), INT16_C( 22831), INT16_C(-21433), INT16_C( 32351),
                            INT16_C(-18789), INT16_C( 20983), INT16_C( -3647), INT16_C( 26454),
                            INT16_C( -2225), INT16_C( 19804), INT16_C( -2763), INT16_C( -8730),
                            INT16_C(-29152), INT16_C( 25302), INT16_C(-28393), INT16_C(  3478),
                            INT16_C( -5675), INT16_C( -4361), INT16_C(-16878), INT16_C( 23119),
                            INT16_C( 30252), INT16_C( -2420), INT16_C( 13170), INT16_C(-21449)),
      simde_mm512_set_epi16(INT16_C( -6529), INT16_C(  9260), INT16_C(  6846), INT16_C( 21618),
                            INT16_C( 20365), INT16_C(   589), INT16_C(  7670), INT16_C(  6521),
                            INT16_C(  3944), INT16_C( 19892), INT16_C( -4082), INT16_C( -7444),
                            INT16_C( 12337), INT16_C( 14080), INT16_C(  3287), INT16_C( -4021),
                            INT16_C(  1885), INT16_C( 11733), INT16_C(  1845), INT16_C(  7123),
                            INT16_C(-17187), INT16_C( -2759), INT16_C(-18330), INT16_C(  -411),
                            INT16_C( -2076), INT16_C(  2107), INT16_C( -3367), INT16_C( -3609),
                            INT16_C( 21341), INT16_C( -1602), INT16_C( 13113), INT16_C( -5852)) },
    { simde_mm512_set_epi16(INT16_C( 22335), INT16_C( 12112), INT16_C(  9189), INT16_C(  1311),
                            INT16_C( -7095), INT16_C( 13615), INT16_C(-21824), INT16_C( 31469),
                            INT16_C( 12162), INT16_C( -9370), INT16_C(-23767), INT16_C(-15401),
                            INT16_C(-14538), INT16_C( 24958), INT16_C(  2725), INT16_C(-25768),
                            INT16_C(-18369), INT16_C( 24484), INT16_C( 16711), INT16_C(-20904),
                            INT16_C(-18546), INT16_C( 25102), INT16_C(  6573), INT16_C( 22274),
                            INT16_C(-16497), INT16_C(-26622), INT16_C( 32256), INT16_C(-24007),
                            INT16_C( -2780), INT16_C( -4298), INT16_C(  8613), INT16_C(-14508)),
      simde_mm512_set_epi16(INT16_C( 30472), INT16_C(-28763), INT16_C(  7714), INT16_C( 18947),
                            INT16_C(  7066), INT16_C(-17692), INT16_C( -6885), INT16_C(  1841),
                            INT16_C(-29737), INT16_C(-14957), INT16_C(-32610), INT16_C( 26598),
                            INT16_C(-25999), INT16_C( -4399), INT16_C(  5946), INT16_C(  2262),
                            INT16_C( -5420), INT16_C( 12953), INT16_C(-27491), INT16_C(-17749),
                            INT16_C( 30618), INT16_C(-27725), INT16_C(-13788), INT16_C(-13300),
                            INT16_C( 23394), INT16_C(  2441), INT16_C( 32382), INT16_C(  9384),
                            INT16_C( 25792), INT16_C( -9373), INT16_C( 22658), INT16_C( 20939)),
      simde_mm512_set_epi16(INT16_C( 22335), INT16_C( 12112), INT16_C(  1475), INT16_C(  1311),
                            INT16_C(   -29), INT16_C( 13615), INT16_C( -1169), INT16_C(   172),
                            INT16_C( 12162), INT16_C( -9370), INT16_C(-23767), INT16_C(-15401),
                            INT16_C(-14538), INT16_C(  2963), INT16_C(  2725), INT16_C(  -886),
                            INT16_C( -2109), INT16_C( 11531), INT16_C( 16711), INT16_C( -3155),
                            INT16_C(-18546), INT16_C( 25102), INT16_C(  6573), INT16_C(  8974),
                            INT16_C(-16497), INT16_C( -2212), INT16_C( 32256), INT16_C( -5239),
                            INT16_C( -2780), INT16_C( -4298), INT16_C(  8613), INT16_C(-14508)) },
    { simde_mm512_set_epi16(INT16_C( 13867), INT16_C( 28091), INT16_C(-30146), INT16_C( -8550),
                            INT16_C( 31509), INT16_C( -2205), INT16_C(  9520), INT16_C( 29929),
                            INT16_C( 24571), INT16_C(-27795), INT16_C(-12850), INT16_C( 14609),
                            INT16_C( 31001), INT16_C(   823), INT16_C(-19839), INT16_C(-27185),
                            INT16_C(-29756), INT16_C(-24530), INT16_C(  3633), INT16_C(-20036),
                            INT16_C( 30184), INT16_C( 27396), INT16_C(  1171), INT16_C( 25936),
                            INT16_C( -3833), INT16_C( -7750), INT16_C( 19453), INT16_C( 30002),
                            INT16_C(  6315), INT16_C(   244), INT16_C(  8399), INT16_C( -8080)),
      simde_mm512_set_epi16(INT16_C( 18752), INT16_C( 27431), INT16_C(-11832), INT16_C(-22911),
                            INT16_C(-22667), INT16_C(-23791), INT16_C(-17993), INT16_C( 11401),
                            INT16_C( 26966), INT16_C( 26500), INT16_C(  7486), INT16_C(  7825),
                            INT16_C( 17767), INT16_C( -7030), INT16_C(-29302), INT16_C(-27163),
                            INT16_C(-10544), INT16_C(-18630), INT16_C(-13432), INT16_C( 31285),
                            INT16_C(-30604), INT16_C( 29467), INT16_C(-31755), INT16_C(   883),
                            INT16_C( 23995), INT16_C(-22467), INT16_C(-11949), INT16_C( 11327),
                            INT16_C(-28925), INT16_C(  7518), INT16_C( 30015), INT16_C( 30285)),
      simde_mm512_set_epi16(INT16_C( 13867), INT16_C(   660), INT16_C( -6482), INT16_C( -8550),
                            INT16_C(  8842), INT16_C( -2205), INT16_C(  9520), INT16_C(  7127),
                            INT16_C( 24571), INT16_C( -1295), INT16_C( -5364), INT16_C(  6784),
                            INT16_C( 13234), INT16_C(   823), INT16_C(-19839), INT16_C(   -22),
                            INT16_C( -8668), INT16_C( -5900), INT16_C(  3633), INT16_C(-20036),
                            INT16_C( 30184), INT16_C( 27396), INT16_C(  1171), INT16_C(   329),
                            INT16_C( -3833), INT16_C( -7750), INT16_C(  7504), INT16_C(  7348),
                            INT16_C(  6315), INT16_C(   244), INT16_C(  8399), INT16_C( -8080)) },
    { simde_mm512_set_epi16(INT16_C( 19003), INT16_C( 26627), INT16_C( -1831), INT16_C(-31318),
                            INT16_C(-29481), INT16_C( 13847), INT16_C(-20911), INT16_C(  9042),
                            INT16_C(-29388), INT16_C( 11660), INT16_C( 32339), INT16_C(-25821),
                            INT16_C(-18358), INT16_C( 21002), INT16_C( -4830), INT16_C(  8527),
                            INT16_C( 26072), INT16_C( 29611), INT16_C( 18348), INT16_C(   953),
                            INT16_C(-32154), INT16_C( 22717), INT16_C(-15414), INT16_C(-13122),
                            INT16_C( -6258), INT16_C(-11311), INT16_C( 31952), INT16_C( 29752),
                            INT16_C(-28048), INT16_C( 20614), INT16_C(  1055), INT16_C( -4387)),
      simde_mm512_set_epi16(INT16_C( -5809), INT16_C(  3072), INT16_C(  8626), INT16_C( 14922),
                            INT16_C( -1420), INT16_C(-29164), INT16_C( 22591), INT16_C(  8828),
                            INT16_C( -1488), INT16_C( -8728), INT16_C( -8885), INT16_C(-25776),
                            INT16_C( -5719), INT16_C(-14622), INT16_C( 21275), INT16_C(-30430),
                            INT16_C(  6020), INT16_C( 27245), INT16_C(-30773), INT16_C( 25208),
                            INT16_C( 25908), INT16_C( 21036), INT16_C(-29170), INT16_C( 25589),
                            INT16_C(  2188), INT16_C(-29317), INT16_C( -9309), INT16_C(-15127),
                            INT16_C(  8889), INT16_C( -7060), INT16_C( 24556), INT16_C( 24873)),
      simde_mm512_set_epi16(INT16_C(  1576), INT16_C(  2051), INT16_C( -1831), INT16_C( -1474),
                            INT16_C( -1081), INT16_C( 13847), INT16_C(-20911), INT16_C(   214),
                            INT16_C( -1116), INT16_C(  2932), INT16_C(  5684), INT16_C(   -45),
                            INT16_C( -1201), INT16_C(  6380), INT16_C( -4830), INT16_C(  8527),
                            INT16_C(  1992), INT16_C(  2366), INT16_C( 18348), INT16_C(   953),
                            INT16_C( -6246), INT16_C(  1681), INT16_C(-15414), INT16_C(-13122),
                            INT16_C( -1882), INT16_C(-11311), INT16_C(  4025), INT16_C( 14625),
                            INT16_C( -1381), INT16_C(  6494), INT16_C(  1055), INT16_C( -4387)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_rem_epi16(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_rem_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi32(INT32_C(  691121094), INT32_C(  674034227), INT32_C(-1965434887), INT32_C( -920286947),
                            INT32_C( -374673026), INT32_C(-1240805178), INT32_C( 1568850865), INT32_C(-1142977539),
                            INT32_C(-1079516608), INT32_C( -708153743), INT32_C( 1508722402), INT32_C(-2074345640),
                            INT32_C( 1747596798), INT32_C(-2063703989), INT32_C(  527472553), INT32_C(-1403096998)),
      simde_mm512_set_epi32(INT32_C(-1764810870), INT32_C( 1179683687), INT32_C(-1646326602), INT32_C( -671967289),
                            INT32_C(-1586327268), INT32_C( 1691051285), INT32_C(   50347892), INT32_C(  728425428),
                            INT32_C( 1192263444), INT32_C(-2086343723), INT32_C( 1322777130), INT32_C(  163989560),
                            INT32_C( 1492341726), INT32_C(  298608154), INT32_C( 1250819173), INT32_C( -650971253)),
      simde_mm512_set_epi32(INT32_C(  691121094), INT32_C(  674034227), INT32_C( -319108285), INT32_C( -248319658),
                            INT32_C( -374673026), INT32_C(-1240805178), INT32_C(    8066213), INT32_C( -414552111),
                            INT32_C(-1079516608), INT32_C( -708153743), INT32_C(  185945272), INT32_C( -106470920),
                            INT32_C(  255255072), INT32_C( -272055065), INT32_C(  527472553), INT32_C( -101154492)) },
    { simde_mm512_set_epi32(INT32_C( 1314482530), INT32_C(-1297250617), INT32_C( -739008036), INT32_C(-1419039999),
                            INT32_C(-1004264650), INT32_C( 1580565751), INT32_C( -471064457), INT32_C( 2081361826),
                            INT32_C(  493161721), INT32_C(-1195115819), INT32_C(  894221337), INT32_C(-1330460172),
                            INT32_C(  492373082), INT32_C(  -13096811), INT32_C(-2087181083), INT32_C( -341007878)),
      simde_mm512_set_epi32(INT32_C(-1138893231), INT32_C( -687161637), INT32_C( 1828175063), INT32_C( -389420023),
                            INT32_C( -193211433), INT32_C( -857989172), INT32_C( -448329300), INT32_C(-1601364212),
                            INT32_C( 1710148738), INT32_C( 1974123080), INT32_C(-1424367196), INT32_C(  118588227),
                            INT32_C(  542053192), INT32_C(  499863549), INT32_C(  957375358), INT32_C(-1291033589)),
      simde_mm512_set_epi32(INT32_C(  175589299), INT32_C( -610088980), INT32_C( -739008036), INT32_C( -250779930),
                            INT32_C(  -38207485), INT32_C(  722576579), INT32_C(  -22735157), INT32_C(  479997614),
                            INT32_C(  493161721), INT32_C(-1195115819), INT32_C(  894221337), INT32_C(  -25989675),
                            INT32_C(  492373082), INT32_C(  -13096811), INT32_C( -172430367), INT32_C( -341007878)) },
    { simde_mm512_set_epi32(INT32_C( 1763100483), INT32_C( -518004559), INT32_C(-1450358898), INT32_C(-1409866198),
                            INT32_C(  269910347), INT32_C(  433971495), INT32_C( 1441956227), INT32_C( 1018271575),
                            INT32_C( 1734496959), INT32_C(  380846712), INT32_C( -941967689), INT32_C( -739443621),
                            INT32_C( 1995198557), INT32_C( -980655097), INT32_C(-1888383043), INT32_C( 1779168063)),
      simde_mm512_set_epi32(INT32_C( -665465241), INT32_C( -342195833), INT32_C( 2102184556), INT32_C(  877111492),
                            INT32_C( 1183491905), INT32_C( -576610979), INT32_C(-1061316197), INT32_C( -808097400),
                            INT32_C( -362876916), INT32_C(-1845390533), INT32_C(  -48621016), INT32_C(  201516689),
                            INT32_C(-1435930720), INT32_C(-1932876068), INT32_C(-1153303869), INT32_C(  562234020)),
      simde_mm512_set_epi32(INT32_C(  432170001), INT32_C( -175808726), INT32_C(-1450358898), INT32_C( -532754706),
                            INT32_C(  269910347), INT32_C(  433971495), INT32_C(  380640030), INT32_C(  210174175),
                            INT32_C(  282989295), INT32_C(  380846712), INT32_C(  -18168385), INT32_C( -134893554),
                            INT32_C(  559267837), INT32_C( -980655097), INT32_C( -735079174), INT32_C(   92466003)) },
    { simde_mm512_set_epi32(INT32_C(  495870887), INT32_C( -382126427), INT32_C(  915244711), INT32_C(    5081424),
                            INT32_C( 1422501384), INT32_C( -163979724), INT32_C(-1516900265), INT32_C(  497965579),
                            INT32_C(  910061584), INT32_C( 2002226944), INT32_C( -621963189), INT32_C(  -48343218),
                            INT32_C(  523093293), INT32_C(-1235205724), INT32_C(-2088961787), INT32_C( 1943141679)),
      simde_mm512_set_epi32(INT32_C( 1206471293), INT32_C( 1374915518), INT32_C(  531653117), INT32_C( 2075187308),
                            INT32_C( -144618549), INT32_C(-2131865715), INT32_C( 1444783055), INT32_C( 1878625233),
                            INT32_C( 1755684145), INT32_C(-2061726371), INT32_C(-1050443653), INT32_C(-1299940555),
                            INT32_C(-2116696545), INT32_C( 1493088054), INT32_C( -179829877), INT32_C(  651362699)),
      simde_mm512_set_epi32(INT32_C(  495870887), INT32_C( -382126427), INT32_C(  383591594), INT32_C(    5081424),
                            INT32_C(  120934443), INT32_C( -163979724), INT32_C(  -72117210), INT32_C(  497965579),
                            INT32_C(  910061584), INT32_C( 2002226944), INT32_C( -621963189), INT32_C(  -48343218),
                            INT32_C(  523093293), INT32_C(-1235205724), INT32_C( -110833140), INT32_C(  640416281)) },
    { simde_mm512_set_epi32(INT32_C(-1637276628), INT32_C(  448681074), INT32_C( 1334667053), INT32_C(  502667641),
                            INT32_C(  855395764), INT32_C(-1672092948), INT32_C(  808531712), INT32_C(  454488139),
                            INT32_C(  123547093), INT32_C(  483090439), INT32_C(-1126329757), INT32_C(-1201220189),
                            INT32_C( -136050629), INT32_C( -220620904), INT32_C( 1398655610), INT32_C( 1722520923)),
      simde_mm512_set_epi32(INT32_C(  604721400), INT32_C( 1471174399), INT32_C(-1803940708), INT32_C(-1765392929),
                            INT32_C(  298473775), INT32_C(-1404600737), INT32_C(-1231334921), INT32_C( -238983338),
                            INT32_C( -145797796), INT32_C( -181019162), INT32_C(-1910480170), INT32_C(-1860760170),
                            INT32_C( -371855625), INT32_C(-1106093489), INT32_C( 1982658188), INT32_C(  863153207)),
      simde_mm512_set_epi32(INT32_C( -427833828), INT32_C(  448681074), INT32_C( 1334667053), INT32_C(  502667641),
                            INT32_C(  258448214), INT32_C( -267492211), INT32_C(  808531712), INT32_C(  215504801),
                            INT32_C(  123547093), INT32_C(  121052115), INT32_C(-1126329757), INT32_C(-1201220189),
                            INT32_C( -136050629), INT32_C( -220620904), INT32_C( 1398655610), INT32_C(  859367716)) },
    { simde_mm512_set_epi32(INT32_C( 1463758672), INT32_C(  602211615), INT32_C( -464964305), INT32_C(-1430226195),
                            INT32_C(  797104998), INT32_C(-1557543977), INT32_C( -952737410), INT32_C(  178625368),
                            INT32_C(-1203806300), INT32_C( 1095216728), INT32_C(-1215405554), INT32_C(  430790402),
                            INT32_C(-1081108478), INT32_C( 2113970745), INT32_C( -182128842), INT32_C(  564512596)),
      simde_mm512_set_epi32(INT32_C( 1997049765), INT32_C(  505563651), INT32_C(  463125220), INT32_C( -451213519),
                            INT32_C(-1948793453), INT32_C(-2137102362), INT32_C(-1703809327), INT32_C(  389679318),
                            INT32_C( -355192167), INT32_C(-1801602389), INT32_C( 2006619059), INT32_C( -903558132),
                            INT32_C( 1533151625), INT32_C( 2122196136), INT32_C( 1690360675), INT32_C( 1484935627)),
      simde_mm512_set_epi32(INT32_C( 1463758672), INT32_C(   96647964), INT32_C(   -1839085), INT32_C(  -76585638),
                            INT32_C(  797104998), INT32_C(-1557543977), INT32_C( -952737410), INT32_C(  178625368),
                            INT32_C( -138229799), INT32_C( 1095216728), INT32_C(-1215405554), INT32_C(  430790402),
                            INT32_C(-1081108478), INT32_C( 2113970745), INT32_C( -182128842), INT32_C(  564512596)) },
    { simde_mm512_set_epi32(INT32_C(  908815803), INT32_C(-1975591270), INT32_C( 2065037155), INT32_C(  623932649),
                            INT32_C( 1610322797), INT32_C( -842122991), INT32_C( 2031682359), INT32_C(-1300130353),
                            INT32_C(-1950048210), INT32_C(  238137788), INT32_C( 1978166020), INT32_C(   76768592),
                            INT32_C( -251141702), INT32_C( 1274901810), INT32_C(  413860084), INT32_C(  550494320)),
      simde_mm512_set_epi32(INT32_C( 1228958503), INT32_C( -775379327), INT32_C(-1485462767), INT32_C(-1179177847),
                            INT32_C( 1767270276), INT32_C(  490610321), INT32_C( 1164436618), INT32_C(-1920297499),
                            INT32_C( -690964678), INT32_C( -880248267), INT32_C(-2005634277), INT32_C(-2081094797),
                            INT32_C( 1572579389), INT32_C( -783078337), INT32_C(-1895621282), INT32_C( 1967093325)),
      simde_mm512_set_epi32(INT32_C(  908815803), INT32_C( -424832616), INT32_C(  579574388), INT32_C(  623932649),
                            INT32_C( 1610322797), INT32_C( -351512670), INT32_C(  867245741), INT32_C(-1300130353),
                            INT32_C( -568118854), INT32_C(  238137788), INT32_C( 1978166020), INT32_C(   76768592),
                            INT32_C( -251141702), INT32_C(  491823473), INT32_C(  413860084), INT32_C(  550494320)) },
    { simde_mm512_set_epi32(INT32_C( 1245407235), INT32_C( -119962198), INT32_C(-1932052969), INT32_C(-1370414254),
                            INT32_C(-1925960308), INT32_C( 2119408419), INT32_C(-1203088886), INT32_C( -316530353),
                            INT32_C( 1708684203), INT32_C( 1202455481), INT32_C(-2107221827), INT32_C(-1010119490),
                            INT32_C( -410070063), INT32_C( 2094036024), INT32_C(-1838133114), INT32_C(   69201629)),
      simde_mm512_set_epi32(INT32_C( -380695552), INT32_C(  565328458), INT32_C(  -93024748), INT32_C( 1480532604),
                            INT32_C(  -97460760), INT32_C( -582247600), INT32_C( -374749470), INT32_C( 1394313506),
                            INT32_C(  394553965), INT32_C(-2016714120), INT32_C( 1697927724), INT32_C(-1911659531),
                            INT32_C(  143428987), INT32_C( -610024215), INT32_C(  582607980), INT32_C( 1609326889)),
      simde_mm512_set_epi32(INT32_C(  103320579), INT32_C( -119962198), INT32_C(  -71558009), INT32_C(-1370414254),
                            INT32_C(  -74205868), INT32_C(  372665619), INT32_C(  -78840476), INT32_C( -316530353),
                            INT32_C(  130468343), INT32_C( 1202455481), INT32_C( -409294103), INT32_C(-1010119490),
                            INT32_C( -123212089), INT32_C(  263963379), INT32_C(  -90309174), INT32_C(   69201629)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_rem_epi32(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_mask_rem_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i src;
    simde__mmask16 k;
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi32(INT32_C(  691121094), INT32_C(  674034227), INT32_C(-1965434887), INT32_C( -920286947),
                            INT32_C( -374673026), INT32_C(-1240805178), INT32_C( 1568850865), INT32_C(-1142977539),
                            INT32_C(-1079516608), INT32_C( -708153743), INT32_C( 1508722402), INT32_C(-2074345640),
                            INT32_C( 1747596798), INT32_C(-2063703989), INT32_C(  527472553), INT32_C(-1403096998)),
      UINT16_C(63371),
      simde_mm512_set_epi32(INT32_C( -341007878), INT32_C(-1764810870), INT32_C( 1179683687), INT32_C(-1646326602),
                            INT32_C( -671967289), INT32_C(-1586327268), INT32_C( 1691051285), INT32_C(   50347892),
                            INT32_C(  728425428), INT32_C( 1192263444), INT32_C(-2086343723), INT32_C( 1322777130),
                            INT32_C(  163989560), INT32_C( 1492341726), INT32_C(  298608154), INT32_C( 1250819173)),
      simde_mm512_set_epi32(INT32_C(-1291033589), INT32_C( 1314482530), INT32_C(-1297250617), INT32_C( -739008036),
                            INT32_C(-1419039999), INT32_C(-1004264650), INT32_C( 1580565751), INT32_C( -471064457),
                            INT32_C( 2081361826), INT32_C(  493161721), INT32_C(-1195115819), INT32_C(  894221337),
                            INT32_C(-1330460172), INT32_C(  492373082), INT32_C(  -13096811), INT32_C(-2087181083)),
      simde_mm512_set_epi32(INT32_C( -341007878), INT32_C( -450328340), INT32_C( 1179683687), INT32_C( -168310530),
                            INT32_C( -374673026), INT32_C( -582062618), INT32_C(  110485534), INT32_C(   50347892),
                            INT32_C(  728425428), INT32_C( -708153743), INT32_C( 1508722402), INT32_C(-2074345640),
                            INT32_C(  163989560), INT32_C(-2063703989), INT32_C(   10478312), INT32_C( 1250819173)) },
    { simde_mm512_set_epi32(INT32_C( 1779168063), INT32_C(-1138893231), INT32_C( -687161637), INT32_C( 1828175063),
                            INT32_C( -389420023), INT32_C( -193211433), INT32_C( -857989172), INT32_C( -448329300),
                            INT32_C(-1601364212), INT32_C( 1710148738), INT32_C( 1974123080), INT32_C(-1424367196),
                            INT32_C(  118588227), INT32_C(  542053192), INT32_C(  499863549), INT32_C(  957375358)),
      UINT16_C(36797),
      simde_mm512_set_epi32(INT32_C(-1153303869), INT32_C(  562234020), INT32_C( 1763100483), INT32_C( -518004559),
                            INT32_C(-1450358898), INT32_C(-1409866198), INT32_C(  269910347), INT32_C(  433971495),
                            INT32_C( 1441956227), INT32_C( 1018271575), INT32_C( 1734496959), INT32_C(  380846712),
                            INT32_C( -941967689), INT32_C( -739443621), INT32_C( 1995198557), INT32_C( -980655097)),
      simde_mm512_set_epi32(INT32_C(-2088961787), INT32_C( 1943141679), INT32_C( -665465241), INT32_C( -342195833),
                            INT32_C( 2102184556), INT32_C(  877111492), INT32_C( 1183491905), INT32_C( -576610979),
                            INT32_C(-1061316197), INT32_C( -808097400), INT32_C( -362876916), INT32_C(-1845390533),
                            INT32_C(  -48621016), INT32_C(  201516689), INT32_C(-1435930720), INT32_C(-1932876068)),
      simde_mm512_set_epi32(INT32_C(-1153303869), INT32_C(-1138893231), INT32_C( -687161637), INT32_C( 1828175063),
                            INT32_C(-1450358898), INT32_C( -532754706), INT32_C(  269910347), INT32_C(  433971495),
                            INT32_C(  380640030), INT32_C( 1710148738), INT32_C(  282989295), INT32_C(  380846712),
                            INT32_C(  -18168385), INT32_C( -134893554), INT32_C(  499863549), INT32_C( -980655097)) },
    { simde_mm512_set_epi32(INT32_C( -179829877), INT32_C(  651362699), INT32_C(  495870887), INT32_C( -382126427),
                            INT32_C(  915244711), INT32_C(    5081424), INT32_C( 1422501384), INT32_C( -163979724),
                            INT32_C(-1516900265), INT32_C(  497965579), INT32_C(  910061584), INT32_C( 2002226944),
                            INT32_C( -621963189), INT32_C(  -48343218), INT32_C(  523093293), INT32_C(-1235205724)),
      UINT16_C(46902),
      simde_mm512_set_epi32(INT32_C( -220620904), INT32_C( 1398655610), INT32_C( 1722520923), INT32_C( 1206471293),
                            INT32_C( 1374915518), INT32_C(  531653117), INT32_C( 2075187308), INT32_C( -144618549),
                            INT32_C(-2131865715), INT32_C( 1444783055), INT32_C( 1878625233), INT32_C( 1755684145),
                            INT32_C(-2061726371), INT32_C(-1050443653), INT32_C(-1299940555), INT32_C(-2116696545)),
      simde_mm512_set_epi32(INT32_C(-1106093489), INT32_C( 1982658188), INT32_C(  863153207), INT32_C(-1637276628),
                            INT32_C(  448681074), INT32_C( 1334667053), INT32_C(  502667641), INT32_C(  855395764),
                            INT32_C(-1672092948), INT32_C(  808531712), INT32_C(  454488139), INT32_C(  123547093),
                            INT32_C(  483090439), INT32_C(-1126329757), INT32_C(-1201220189), INT32_C( -136050629)),
      simde_mm512_set_epi32(INT32_C( -220620904), INT32_C(  651362699), INT32_C(  859367716), INT32_C( 1206471293),
                            INT32_C(  915244711), INT32_C(  531653117), INT32_C(   64516744), INT32_C( -144618549),
                            INT32_C(-1516900265), INT32_C(  497965579), INT32_C(   60672677), INT32_C(   26024843),
                            INT32_C( -621963189), INT32_C(-1050443653), INT32_C(  -98720366), INT32_C(-1235205724)) },
    { simde_mm512_set_epi32(INT32_C( 2113970745), INT32_C( -182128842), INT32_C(  564512596), INT32_C(  604721400),
                            INT32_C( 1471174399), INT32_C(-1803940708), INT32_C(-1765392929), INT32_C(  298473775),
                            INT32_C(-1404600737), INT32_C(-1231334921), INT32_C( -238983338), INT32_C( -145797796),
                            INT32_C( -181019162), INT32_C(-1910480170), INT32_C(-1860760170), INT32_C( -371855625)),
      UINT16_C(38914),
      simde_mm512_set_epi32(INT32_C( 1533151625), INT32_C( 2122196136), INT32_C( 1690360675), INT32_C( 1484935627),
                            INT32_C( 1463758672), INT32_C(  602211615), INT32_C( -464964305), INT32_C(-1430226195),
                            INT32_C(  797104998), INT32_C(-1557543977), INT32_C( -952737410), INT32_C(  178625368),
                            INT32_C(-1203806300), INT32_C( 1095216728), INT32_C(-1215405554), INT32_C(  430790402)),
      simde_mm512_set_epi32(INT32_C( -251141702), INT32_C( 1274901810), INT32_C(  413860084), INT32_C(  550494320),
                            INT32_C( 1997049765), INT32_C(  505563651), INT32_C(  463125220), INT32_C( -451213519),
                            INT32_C(-1948793453), INT32_C(-2137102362), INT32_C(-1703809327), INT32_C(  389679318),
                            INT32_C( -355192167), INT32_C(-1801602389), INT32_C( 2006619059), INT32_C( -903558132)),
      simde_mm512_set_epi32(INT32_C(   26301413), INT32_C( -182128842), INT32_C(  564512596), INT32_C(  383946987),
                            INT32_C( 1463758672), INT32_C(-1803940708), INT32_C(-1765392929), INT32_C(  298473775),
                            INT32_C(-1404600737), INT32_C(-1231334921), INT32_C( -238983338), INT32_C( -145797796),
                            INT32_C( -181019162), INT32_C(-1910480170), INT32_C(-1215405554), INT32_C( -371855625)) },
    { simde_mm512_set_epi32(INT32_C( 1572579389), INT32_C( -783078337), INT32_C(-1895621282), INT32_C( 1967093325),
                            INT32_C(  908815803), INT32_C(-1975591270), INT32_C( 2065037155), INT32_C(  623932649),
                            INT32_C( 1610322797), INT32_C( -842122991), INT32_C( 2031682359), INT32_C(-1300130353),
                            INT32_C(-1950048210), INT32_C(  238137788), INT32_C( 1978166020), INT32_C(   76768592)),
      UINT16_C(  883),
      simde_mm512_set_epi32(INT32_C(-1010119490), INT32_C( -410070063), INT32_C( 2094036024), INT32_C(-1838133114),
                            INT32_C(   69201629), INT32_C( 1228958503), INT32_C( -775379327), INT32_C(-1485462767),
                            INT32_C(-1179177847), INT32_C( 1767270276), INT32_C(  490610321), INT32_C( 1164436618),
                            INT32_C(-1920297499), INT32_C( -690964678), INT32_C( -880248267), INT32_C(-2005634277)),
      simde_mm512_set_epi32(INT32_C(-1911659531), INT32_C(  143428987), INT32_C( -610024215), INT32_C(  582607980),
                            INT32_C( 1609326889), INT32_C( 1245407235), INT32_C( -119962198), INT32_C(-1932052969),
                            INT32_C(-1370414254), INT32_C(-1925960308), INT32_C( 2119408419), INT32_C(-1203088886),
                            INT32_C( -316530353), INT32_C( 1708684203), INT32_C( 1202455481), INT32_C(-2107221827)),
      simde_mm512_set_epi32(INT32_C( 1572579389), INT32_C( -783078337), INT32_C(-1895621282), INT32_C( 1967093325),
                            INT32_C(  908815803), INT32_C(-1975591270), INT32_C(  -55606139), INT32_C(-1485462767),
                            INT32_C( 1610322797), INT32_C( 1767270276), INT32_C(  490610321), INT32_C( 1164436618),
                            INT32_C(-1950048210), INT32_C(  238137788), INT32_C( -880248267), INT32_C(-2005634277)) },
    { simde_mm512_set_epi32(INT32_C( 2117071873), INT32_C(-1437889529), INT32_C( -376074104), INT32_C( 1087893388),
                            INT32_C( -443183285), INT32_C( -380695552), INT32_C(  565328458), INT32_C(  -93024748),
                            INT32_C( 1480532604), INT32_C(  -97460760), INT32_C( -582247600), INT32_C( -374749470),
                            INT32_C( 1394313506), INT32_C(  394553965), INT32_C(-2016714120), INT32_C( 1697927724)),
      UINT16_C(12254),
      simde_mm512_set_epi32(INT32_C(   56443211), INT32_C(-2036514643), INT32_C( -510270824), INT32_C( 1139427205),
                            INT32_C( 1090384090), INT32_C(-1905231405), INT32_C(-2079359983), INT32_C( -477294891),
                            INT32_C( -673197028), INT32_C( 2071747620), INT32_C( -442789099), INT32_C( -601334711),
                            INT32_C(  319530416), INT32_C(-2115012481), INT32_C( -501730903), INT32_C(  340519338)),
      simde_mm512_set_epi32(INT32_C( 1219537084), INT32_C( 1349635715), INT32_C(  732887738), INT32_C(-1728641921),
                            INT32_C(-1388433411), INT32_C( 1765754685), INT32_C(-1574983663), INT32_C(  846129112),
                            INT32_C( 1578410935), INT32_C(-1659872458), INT32_C( 1045536663), INT32_C(  957117985),
                            INT32_C(-1265958651), INT32_C( 1309498779), INT32_C(-1001015299), INT32_C( 1022360677)),
      simde_mm512_set_epi32(INT32_C( 2117071873), INT32_C(-1437889529), INT32_C( -510270824), INT32_C( 1087893388),
                            INT32_C( 1090384090), INT32_C( -139476720), INT32_C( -504376320), INT32_C( -477294891),
                            INT32_C( -673197028), INT32_C(  411875162), INT32_C( -582247600), INT32_C( -601334711),
                            INT32_C(  319530416), INT32_C( -805513702), INT32_C( -501730903), INT32_C( 1697927724)) },
    { simde_mm512_set_epi32(INT32_C( -304885978), INT32_C(  991545752), INT32_C( -143034937), INT32_C(  843112042),
                            INT32_C( -227554783), INT32_C( 2124182542), INT32_C(-1526246088), INT32_C(-1991977382),
                            INT32_C( 1224533822), INT32_C( -819361196), INT32_C( -684010252), INT32_C(-1738921185),
                            INT32_C(-1259570772), INT32_C( -691865929), INT32_C( -973523371), INT32_C(   45581573)),
      UINT16_C(42669),
      simde_mm512_set_epi32(INT32_C( -156799603), INT32_C(-1073012339), INT32_C(-2130532125), INT32_C(  397240391),
                            INT32_C(  200936922), INT32_C(-1030980309), INT32_C(-1758363174), INT32_C( -665586367),
                            INT32_C(  453331046), INT32_C( 1704580573), INT32_C( 1606190487), INT32_C(-1085658047),
                            INT32_C(-1335469644), INT32_C( -368070561), INT32_C(-1419559633), INT32_C( 2069966669)),
      simde_mm512_set_epi32(INT32_C( 1379668640), INT32_C(   66581512), INT32_C( -557301797), INT32_C(  304428974),
                            INT32_C(-1608262788), INT32_C(  532978979), INT32_C(  946958552), INT32_C(-1911324669),
                            INT32_C(-2118093156), INT32_C(  283691898), INT32_C( -446072631), INT32_C( -458781294),
                            INT32_C( 1951055651), INT32_C(  765387914), INT32_C(  822559116), INT32_C(    7445617)),
      simde_mm512_set_epi32(INT32_C( -156799603), INT32_C(  991545752), INT32_C( -458626734), INT32_C(  843112042),
                            INT32_C( -227554783), INT32_C( -498001330), INT32_C( -811404622), INT32_C(-1991977382),
                            INT32_C(  453331046), INT32_C( -819361196), INT32_C(  267972594), INT32_C(-1738921185),
                            INT32_C(-1335469644), INT32_C( -368070561), INT32_C( -973523371), INT32_C(      85143)) },
    { simde_mm512_set_epi32(INT32_C(-1981938926), INT32_C(  869237081), INT32_C( -190053534), INT32_C(-1469275330),
                            INT32_C( -717100794), INT32_C(-1303072888), INT32_C(-2122918671), INT32_C( 1617119933),
                            INT32_C( 1521363431), INT32_C(  553638116), INT32_C( 1036201367), INT32_C(-1187933851),
                            INT32_C( -412155886), INT32_C( -760582943), INT32_C( -423751457), INT32_C( 1273589632)),
      UINT16_C(35103),
      simde_mm512_set_epi32(INT32_C(-1836595644), INT32_C(  260676470), INT32_C( 1724614860), INT32_C( -144514633),
                            INT32_C( -478630580), INT32_C(-2086755061), INT32_C(  932145867), INT32_C(-1862372735),
                            INT32_C( 1756892633), INT32_C(  382632965), INT32_C( 1295078740), INT32_C( -995802034),
                            INT32_C(  152308919), INT32_C( -351555508), INT32_C(   31813624), INT32_C(  807463845)),
      simde_mm512_set_epi32(INT32_C(  615301803), INT32_C(  382786341), INT32_C( 1852603705), INT32_C( 1998007730),
                            INT32_C(  231325888), INT32_C( 1842039329), INT32_C(  968682756), INT32_C(  316335394),
                            INT32_C(-2071382094), INT32_C( -803185337), INT32_C(-2126995500), INT32_C( 1587647099),
                            INT32_C(-1328358584), INT32_C(  320339033), INT32_C(  282380179), INT32_C( -108102092)),
      simde_mm512_set_epi32(INT32_C( -605992038), INT32_C(  869237081), INT32_C( -190053534), INT32_C(-1469275330),
                            INT32_C(  -15978804), INT32_C(-1303072888), INT32_C(-2122918671), INT32_C( -280695765),
                            INT32_C( 1521363431), INT32_C(  553638116), INT32_C( 1036201367), INT32_C( -995802034),
                            INT32_C(  152308919), INT32_C(  -31216475), INT32_C(   31813624), INT32_C(   50749201)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_mask_rem_epi32(test_vec[i].src, test_vec[i].k, test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_rem_epi64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_mm512_set_epi64(INT64_C( 2968342496979776051), INT64_C(-8441478558707775203),
                            INT64_C(-1609208390309195578), INT64_C( 6738163160628300797),
                            INT64_C(-4636488523262038415), INT64_C( 6479913377553186648),
                            INT64_C( 7505871096235581515), INT64_C( 2265477367564496986)),
      simde_mm512_set_epi64(INT64_C(-7579804969095623833), INT64_C(-7070918910501808185),
                            INT64_C(-6813223735121976043), INT64_C(  216242550290965460),
                            INT64_C( 5120732502404950997), INT64_C( 5681284513410730040),
                            INT64_C( 6409558907924801050), INT64_C( 5372227444888762251)),
      simde_mm512_set_epi64(INT64_C( 2968342496979776051), INT64_C(-1370559648205967018),
                            INT64_C(-1609208390309195578), INT64_C(   34644101608371537),
                            INT64_C(-4636488523262038415), INT64_C(  798628864142456608),
                            INT64_C( 1096312188310780465), INT64_C( 2265477367564496986)) },
    { simde_mm512_set_epi64(INT64_C( 5645659480511055559), INT64_C(-3174015343225263359),
                            INT64_C(-4313283826698320649), INT64_C(-2023206435041636446),
                            INT64_C( 2118113466433927893), INT64_C( 3840651400764901876),
                            INT64_C( 2114726288902596757), INT64_C(-8964374488360902150)),
      simde_mm512_set_epi64(INT64_C(-4891509177172967717), INT64_C( 7851952110853286921),
                            INT64_C( -829836782511317044), INT64_C(-1925559678644969716),
                            INT64_C( 7345032902979795528), INT64_C(-6117610524196633789),
                            INT64_C( 2328100732832272381), INT64_C( 4111895855610225675)),
      simde_mm512_set_epi64(INT64_C(  754150303338087842), INT64_C(-3174015343225263359),
                            INT64_C( -164099914141735429), INT64_C(  -97646756396666730),
                            INT64_C( 2118113466433927893), INT64_C( 3840651400764901876),
                            INT64_C( 2114726288902596757), INT64_C( -740582777140450800)) },
    { simde_mm512_set_epi64(INT64_C( 7572458917823766705), INT64_C(-6229244031487498710),
                            INT64_C( 1159256113650983207), INT64_C( 6193154838246823767),
                            INT64_C( 7449607714297299576), INT64_C(-4045720414588175269),
                            INT64_C( 8569312554655704071), INT64_C(-8110543410226793665)),
      simde_mm512_set_epi64(INT64_C(-2858151442766986873), INT64_C( 9028813919053392068),
                            INT64_C( 5083059030774095197), INT64_C(-4558318353343223416),
                            INT64_C(-1558544484243762373), INT64_C( -208825673416776047),
                            INT64_C(-6167275479359641892), INT64_C(-4953402399143034204)),
      simde_mm512_set_epi64(INT64_C( 1856156032289792959), INT64_C(-6229244031487498710),
                            INT64_C( 1159256113650983207), INT64_C( 1634836484903600351),
                            INT64_C( 1215429777322250084), INT64_C(  -78032619669430376),
                            INT64_C( 2402037075296062179), INT64_C(-3157141011083759461)) },
    { simde_mm512_set_epi64(INT64_C( 2129749246616352421), INT64_C( 3930946101587052880),
                            INT64_C( 6109596926925725236), INT64_C(-6515037028970767861),
                            INT64_C( 3908684742628183808), INT64_C(-2671311551824242866),
                            INT64_C( 2246668589251707300), INT64_C(-8972022555815576273)),
      simde_mm512_set_epi64(INT64_C( 5181754748372749246), INT64_C( 2283432752406648940),
                            INT64_C( -621131936186871923), INT64_C( 6205295972918594513),
                            INT64_C( 7540605987113962845), INT64_C(-4511621132930745547),
                            INT64_C(-9091142434838104266), INT64_C( -772363439907339893)),
      simde_mm512_set_epi64(INT64_C( 2129749246616352421), INT64_C( 1647513349180403940),
                            INT64_C(  519409501243877929), INT64_C( -309741056052173348),
                            INT64_C( 3908684742628183808), INT64_C(-2671311551824242866),
                            INT64_C( 2246668589251707300), INT64_C( -476024716834837450)) },
    { simde_mm512_set_epi64(INT64_C(-7032049571316476814), INT64_C( 5732351344186366329),
                            INT64_C( 3673896834139808492), INT64_C( 3472617261273378891),
                            INT64_C(  530630724433960967), INT64_C(-4837549467732879965),
                            INT64_C( -584332998080882792), INT64_C( 6007180105039451483)),
      simde_mm512_set_epi64(INT64_C( 2597258637662508799), INT64_C(-7747866342253511201),
                            INT64_C( 1281935105229028959), INT64_C(-5288543212061759658),
                            INT64_C( -626196761534931482), INT64_C(-8205449847372313194),
                            INT64_C(-1597107745019766193), INT64_C( 8515452077469772855)),
      simde_mm512_set_epi64(INT64_C(-1837532295991459216), INT64_C( 5732351344186366329),
                            INT64_C( 1110026623681750574), INT64_C( 3472617261273378891),
                            INT64_C(  530630724433960967), INT64_C(-4837549467732879965),
                            INT64_C( -584332998080882792), INT64_C( 6007180105039451483)) },
    { simde_mm512_set_epi64(INT64_C( 6286795626078602527), INT64_C(-1997006480917628179),
                            INT64_C( 3423539900625568727), INT64_C(-4091976017447117992),
                            INT64_C(-5170308688123548072), INT64_C(-5220127105375971582),
                            INT64_C(-4643325554324364743), INT64_C( -782237419483838636)),
      simde_mm512_set_epi64(INT64_C( 8577263429665049091), INT64_C( 1989107677696558897),
                            INT64_C(-8370004145136048154), INT64_C(-7317805337695090474),
                            INT64_C(-1525538738567005525), INT64_C( 8618363237326703628),
                            INT64_C( 6584836091306452136), INT64_C( 7260043819054420427)),
      simde_mm512_set_epi64(INT64_C( 6286795626078602527), INT64_C(   -7898803221069282),
                            INT64_C( 3423539900625568727), INT64_C(-4091976017447117992),
                            INT64_C( -593692472422531497), INT64_C(-5220127105375971582),
                            INT64_C(-4643325554324364743), INT64_C( -782237419483838636)) },
    { simde_mm512_set_epi64(INT64_C( 3903334154292354714), INT64_C( 8869267046373815529),
                            INT64_C( 6916283752571091217), INT64_C( 8726009290759968207),
                            INT64_C(-8375393287335202372), INT64_C( 8496158362035250512),
                            INT64_C(-1078645395476875982), INT64_C( 1777515526450307184)),
      simde_mm512_set_epi64(INT64_C( 5278336582045705857), INT64_C(-6380014000574878583),
                            INT64_C( 7590368039103504017), INT64_C( 5001217194949514725),
                            INT64_C(-2967670691286451659), INT64_C(-8614133625237732493),
                            INT64_C( 6754177049630551103), INT64_C(-8141631409824500147)),
      simde_mm512_set_epi64(INT64_C( 3903334154292354714), INT64_C( 2489253045798936946),
                            INT64_C( 6916283752571091217), INT64_C( 3724792095810453482),
                            INT64_C(-2440051904762299054), INT64_C( 8496158362035250512),
                            INT64_C(-1078645395476875982), INT64_C( 1777515526450307184)) },
    { simde_mm512_set_epi64(INT64_C( 5348983348701791658), INT64_C(-8298104313070148782),
                            INT64_C(-8271936534134678749), INT64_C(-5167227415572635313),
                            INT64_C( 7338742772279280569), INT64_C(-9050448829097521986),
                            INT64_C(-1761237507559623624), INT64_C(-7894721610255438115)),
      simde_mm512_set_epi64(INT64_C(-1635074945007338934), INT64_C( -399538248898108804),
                            INT64_C( -418590773130585264), INT64_C(-1609536716449019614),
                            INT64_C( 1694596378460381816), INT64_C( 7292544047935022069),
                            INT64_C(  616022812148352233), INT64_C( 2502282222097948969)),
      simde_mm512_set_epi64(INT64_C(  443758513679774856), INT64_C( -307339335107972702),
                            INT64_C( -318711844653558733), INT64_C( -338617266225576471),
                            INT64_C(  560357258437753305), INT64_C(-1757904781162499917),
                            INT64_C( -529191883262919158), INT64_C( -387874943961591208)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_rem_epi64(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_i64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_rem_epu8(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu8(UINT8_C( 41), UINT8_C( 49), UINT8_C(171), UINT8_C(198),
                             UINT8_C( 40), UINT8_C( 44), UINT8_C(242), UINT8_C( 51),
                             UINT8_C(138), UINT8_C(217), UINT8_C(215), UINT8_C(249),
                             UINT8_C(201), UINT8_C( 37), UINT8_C(137), UINT8_C( 29),
                             UINT8_C(233), UINT8_C(170), UINT8_C(241), UINT8_C(126),
                             UINT8_C(182), UINT8_C( 10), UINT8_C(208), UINT8_C(198),
                             UINT8_C( 93), UINT8_C(130), UINT8_C(195), UINT8_C(177),
                             UINT8_C(187), UINT8_C(223), UINT8_C(139), UINT8_C(253),
                             UINT8_C(191), UINT8_C(167), UINT8_C(226), UINT8_C( 64),
                             UINT8_C(213), UINT8_C(202), UINT8_C(110), UINT8_C(113),
                             UINT8_C( 89), UINT8_C(237), UINT8_C( 70), UINT8_C(226),
                             UINT8_C(132), UINT8_C( 91), UINT8_C(255), UINT8_C( 88),
                             UINT8_C(104), UINT8_C( 42), UINT8_C( 53), UINT8_C(254),
                             UINT8_C(132), UINT8_C(254), UINT8_C( 96), UINT8_C( 75),
                             UINT8_C( 31), UINT8_C(112), UINT8_C(151), UINT8_C(169),
                             UINT8_C(172), UINT8_C( 94), UINT8_C(112), UINT8_C( 90)),
      simde_x_mm512_set_epu8(UINT8_C(195), UINT8_C( 49), UINT8_C( 14), UINT8_C(170),
                             UINT8_C(203), UINT8_C(167), UINT8_C(  3), UINT8_C(215),
                             UINT8_C( 63), UINT8_C(248), UINT8_C( 55), UINT8_C(219),
                             UINT8_C(221), UINT8_C(135), UINT8_C( 61), UINT8_C(191),
                             UINT8_C(209), UINT8_C( 91), UINT8_C( 87), UINT8_C(137),
                             UINT8_C( 87), UINT8_C( 76), UINT8_C( 44), UINT8_C(140),
                             UINT8_C(  2), UINT8_C(200), UINT8_C( 36), UINT8_C(195),
                             UINT8_C(200), UINT8_C(125), UINT8_C(254), UINT8_C(139),
                             UINT8_C(226), UINT8_C( 71), UINT8_C( 92), UINT8_C(129),
                             UINT8_C(182), UINT8_C(119), UINT8_C(247), UINT8_C( 34),
                             UINT8_C(121), UINT8_C( 85), UINT8_C(153), UINT8_C(116),
                             UINT8_C(218), UINT8_C( 21), UINT8_C(101), UINT8_C(122),
                             UINT8_C( 10), UINT8_C(231), UINT8_C( 54), UINT8_C( 71),
                             UINT8_C(156), UINT8_C(149), UINT8_C(244), UINT8_C( 84),
                             UINT8_C(148), UINT8_C( 85), UINT8_C(170), UINT8_C(184),
                             UINT8_C( 94), UINT8_C(154), UINT8_C(229), UINT8_C( 11)),
      simde_x_mm512_set_epu8(UINT8_C( 41), UINT8_C(  0), UINT8_C(  3), UINT8_C( 28),
                             UINT8_C( 40), UINT8_C( 44), UINT8_C(  2), UINT8_C( 51),
                             UINT8_C( 12), UINT8_C(217), UINT8_C( 50), UINT8_C( 30),
                             UINT8_C(201), UINT8_C( 37), UINT8_C( 15), UINT8_C( 29),
                             UINT8_C( 24), UINT8_C( 79), UINT8_C( 67), UINT8_C(126),
                             UINT8_C(  8), UINT8_C( 10), UINT8_C( 32), UINT8_C( 58),
                             UINT8_C(  1), UINT8_C(130), UINT8_C( 15), UINT8_C(177),
                             UINT8_C(187), UINT8_C( 98), UINT8_C(139), UINT8_C(114),
                             UINT8_C(191), UINT8_C( 25), UINT8_C( 42), UINT8_C( 64),
                             UINT8_C( 31), UINT8_C( 83), UINT8_C(110), UINT8_C( 11),
                             UINT8_C( 89), UINT8_C( 67), UINT8_C( 70), UINT8_C(110),
                             UINT8_C(132), UINT8_C(  7), UINT8_C( 53), UINT8_C( 88),
                             UINT8_C(  4), UINT8_C( 42), UINT8_C( 53), UINT8_C( 41),
                             UINT8_C(132), UINT8_C(105), UINT8_C( 96), UINT8_C( 75),
                             UINT8_C( 31), UINT8_C( 27), UINT8_C(151), UINT8_C(169),
                             UINT8_C( 78), UINT8_C( 94), UINT8_C(112), UINT8_C(  2)) },
    { simde_x_mm512_set_epu8(UINT8_C(216), UINT8_C( 85), UINT8_C(206), UINT8_C(103),
                             UINT8_C(235), UINT8_C(154), UINT8_C(129), UINT8_C(135),
                             UINT8_C(125), UINT8_C( 76), UINT8_C(202), UINT8_C(108),
                             UINT8_C( 52), UINT8_C( 71), UINT8_C(168), UINT8_C(196),
                             UINT8_C( 70), UINT8_C(138), UINT8_C(167), UINT8_C( 65),
                             UINT8_C(221), UINT8_C(161), UINT8_C(157), UINT8_C( 93),
                             UINT8_C(192), UINT8_C(189), UINT8_C(153), UINT8_C(155),
                             UINT8_C(207), UINT8_C(213), UINT8_C(105), UINT8_C(136),
                             UINT8_C(234), UINT8_C( 94), UINT8_C(240), UINT8_C( 12),
                             UINT8_C(146), UINT8_C(  1), UINT8_C(147), UINT8_C( 59),
                             UINT8_C(253), UINT8_C( 26), UINT8_C( 26), UINT8_C( 40),
                             UINT8_C( 12), UINT8_C(  2), UINT8_C(230), UINT8_C(145),
                             UINT8_C(170), UINT8_C(105), UINT8_C(111), UINT8_C(160),
                             UINT8_C(140), UINT8_C(202), UINT8_C(166), UINT8_C(220),
                             UINT8_C(187), UINT8_C( 65), UINT8_C(250), UINT8_C(195),
                             UINT8_C( 33), UINT8_C(131), UINT8_C(  2), UINT8_C(164)),
      simde_x_mm512_set_epu8(UINT8_C(120), UINT8_C(127), UINT8_C( 28), UINT8_C( 95),
                             UINT8_C(175), UINT8_C(223), UINT8_C(119), UINT8_C(214),
                             UINT8_C(220), UINT8_C(102), UINT8_C( 86), UINT8_C( 22),
                             UINT8_C(119), UINT8_C(207), UINT8_C( 12), UINT8_C(183),
                             UINT8_C(172), UINT8_C(242), UINT8_C(173), UINT8_C(249),
                             UINT8_C( 52), UINT8_C(108), UINT8_C(128), UINT8_C(203),
                             UINT8_C( 85), UINT8_C(135), UINT8_C(227), UINT8_C( 35),
                             UINT8_C(187), UINT8_C( 24), UINT8_C(250), UINT8_C(219),
                             UINT8_C(253), UINT8_C( 62), UINT8_C(125), UINT8_C(236),
                             UINT8_C( 75), UINT8_C( 13), UINT8_C( 79), UINT8_C( 81),
                             UINT8_C(177), UINT8_C(221), UINT8_C(251), UINT8_C(181),
                             UINT8_C(159), UINT8_C(182), UINT8_C( 11), UINT8_C( 11),
                             UINT8_C( 39), UINT8_C( 37), UINT8_C( 39), UINT8_C(208),
                             UINT8_C(136), UINT8_C(180), UINT8_C(215), UINT8_C(139),
                             UINT8_C(144), UINT8_C(128), UINT8_C(203), UINT8_C(206),
                             UINT8_C(173), UINT8_C( 36), UINT8_C(133), UINT8_C(175)),
      simde_x_mm512_set_epu8(UINT8_C( 96), UINT8_C( 85), UINT8_C( 10), UINT8_C(  8),
                             UINT8_C( 60), UINT8_C(154), UINT8_C( 10), UINT8_C(135),
                             UINT8_C(125), UINT8_C( 76), UINT8_C( 30), UINT8_C( 20),
                             UINT8_C( 52), UINT8_C( 71), UINT8_C(  0), UINT8_C( 13),
                             UINT8_C( 70), UINT8_C(138), UINT8_C(167), UINT8_C( 65),
                             UINT8_C( 13), UINT8_C( 53), UINT8_C( 29), UINT8_C( 93),
                             UINT8_C( 22), UINT8_C( 54), UINT8_C(153), UINT8_C( 15),
                             UINT8_C( 20), UINT8_C( 21), UINT8_C(105), UINT8_C(136),
                             UINT8_C(234), UINT8_C( 32), UINT8_C(115), UINT8_C( 12),
                             UINT8_C( 71), UINT8_C(  1), UINT8_C( 68), UINT8_C( 59),
                             UINT8_C( 76), UINT8_C( 26), UINT8_C( 26), UINT8_C( 40),
                             UINT8_C( 12), UINT8_C(  2), UINT8_C( 10), UINT8_C(  2),
                             UINT8_C( 14), UINT8_C( 31), UINT8_C( 33), UINT8_C(160),
                             UINT8_C(  4), UINT8_C( 22), UINT8_C(166), UINT8_C( 81),
                             UINT8_C( 43), UINT8_C( 65), UINT8_C( 47), UINT8_C(195),
                             UINT8_C( 33), UINT8_C( 23), UINT8_C(  2), UINT8_C(164)) },
    { simde_x_mm512_set_epu8(UINT8_C( 87), UINT8_C( 63), UINT8_C( 47), UINT8_C( 80),
                             UINT8_C( 35), UINT8_C(229), UINT8_C(  5), UINT8_C( 31),
                             UINT8_C(228), UINT8_C( 73), UINT8_C( 53), UINT8_C( 47),
                             UINT8_C(170), UINT8_C(192), UINT8_C(122), UINT8_C(237),
                             UINT8_C( 47), UINT8_C(130), UINT8_C(219), UINT8_C(102),
                             UINT8_C(163), UINT8_C( 41), UINT8_C(195), UINT8_C(215),
                             UINT8_C(199), UINT8_C( 54), UINT8_C( 97), UINT8_C(126),
                             UINT8_C( 10), UINT8_C(165), UINT8_C(155), UINT8_C( 88),
                             UINT8_C(184), UINT8_C( 63), UINT8_C( 95), UINT8_C(164),
                             UINT8_C( 65), UINT8_C( 71), UINT8_C(174), UINT8_C( 88),
                             UINT8_C(183), UINT8_C(142), UINT8_C( 98), UINT8_C( 14),
                             UINT8_C( 25), UINT8_C(173), UINT8_C( 87), UINT8_C(  2),
                             UINT8_C(191), UINT8_C(143), UINT8_C(152), UINT8_C(  2),
                             UINT8_C(126), UINT8_C(  0), UINT8_C(162), UINT8_C( 57),
                             UINT8_C(245), UINT8_C( 36), UINT8_C(239), UINT8_C( 54),
                             UINT8_C( 33), UINT8_C(165), UINT8_C(199), UINT8_C( 84)),
      simde_x_mm512_set_epu8(UINT8_C(131), UINT8_C( 42), UINT8_C(151), UINT8_C(210),
                             UINT8_C( 12), UINT8_C(163), UINT8_C(138), UINT8_C(207),
                             UINT8_C( 43), UINT8_C( 57), UINT8_C( 61), UINT8_C( 62),
                             UINT8_C( 81), UINT8_C(184), UINT8_C(  6), UINT8_C( 93),
                             UINT8_C(167), UINT8_C(  1), UINT8_C(145), UINT8_C(  9),
                             UINT8_C(  4), UINT8_C( 17), UINT8_C( 10), UINT8_C(101),
                             UINT8_C(186), UINT8_C(181), UINT8_C(155), UINT8_C(243),
                             UINT8_C(189), UINT8_C(191), UINT8_C(222), UINT8_C(205),
                             UINT8_C( 59), UINT8_C( 26), UINT8_C(227), UINT8_C(105),
                             UINT8_C(237), UINT8_C(145), UINT8_C(183), UINT8_C( 79),
                             UINT8_C(174), UINT8_C( 60), UINT8_C(132), UINT8_C(208),
                             UINT8_C( 58), UINT8_C(178), UINT8_C(116), UINT8_C(240),
                             UINT8_C( 37), UINT8_C(131), UINT8_C(100), UINT8_C(177),
                             UINT8_C( 19), UINT8_C(102), UINT8_C( 81), UINT8_C( 86),
                             UINT8_C( 25), UINT8_C( 43), UINT8_C( 51), UINT8_C(140),
                             UINT8_C(  9), UINT8_C( 40), UINT8_C(227), UINT8_C( 75)),
      simde_x_mm512_set_epu8(UINT8_C( 87), UINT8_C( 21), UINT8_C( 47), UINT8_C( 80),
                             UINT8_C( 11), UINT8_C( 66), UINT8_C(  5), UINT8_C( 31),
                             UINT8_C( 13), UINT8_C( 16), UINT8_C( 53), UINT8_C( 47),
                             UINT8_C(  8), UINT8_C(  8), UINT8_C(  2), UINT8_C( 51),
                             UINT8_C( 47), UINT8_C(  0), UINT8_C( 74), UINT8_C(  3),
                             UINT8_C(  3), UINT8_C(  7), UINT8_C(  5), UINT8_C( 13),
                             UINT8_C( 13), UINT8_C( 54), UINT8_C( 97), UINT8_C(126),
                             UINT8_C( 10), UINT8_C(165), UINT8_C(155), UINT8_C( 88),
                             UINT8_C(  7), UINT8_C( 11), UINT8_C( 95), UINT8_C( 59),
                             UINT8_C( 65), UINT8_C( 71), UINT8_C(174), UINT8_C(  9),
                             UINT8_C(  9), UINT8_C( 22), UINT8_C( 98), UINT8_C( 14),
                             UINT8_C( 25), UINT8_C(173), UINT8_C( 87), UINT8_C(  2),
                             UINT8_C(  6), UINT8_C( 12), UINT8_C( 52), UINT8_C(  2),
                             UINT8_C( 12), UINT8_C(  0), UINT8_C(  0), UINT8_C( 57),
                             UINT8_C( 20), UINT8_C( 36), UINT8_C( 35), UINT8_C( 54),
                             UINT8_C(  6), UINT8_C(  5), UINT8_C(199), UINT8_C(  9)) },
    { simde_x_mm512_set_epu8(UINT8_C(233), UINT8_C( 79), UINT8_C( 12), UINT8_C(  0),
                             UINT8_C( 33), UINT8_C(178), UINT8_C( 58), UINT8_C( 74),
                             UINT8_C(250), UINT8_C(116), UINT8_C(142), UINT8_C( 20),
                             UINT8_C( 88), UINT8_C( 63), UINT8_C( 34), UINT8_C(124),
                             UINT8_C(250), UINT8_C( 48), UINT8_C(221), UINT8_C(232),
                             UINT8_C(221), UINT8_C( 75), UINT8_C(155), UINT8_C( 80),
                             UINT8_C(233), UINT8_C(169), UINT8_C(198), UINT8_C(226),
                             UINT8_C( 83), UINT8_C( 27), UINT8_C(137), UINT8_C( 34),
                             UINT8_C( 23), UINT8_C(132), UINT8_C(106), UINT8_C(109),
                             UINT8_C(135), UINT8_C(203), UINT8_C( 98), UINT8_C(120),
                             UINT8_C(101), UINT8_C( 52), UINT8_C( 82), UINT8_C( 44),
                             UINT8_C(142), UINT8_C( 14), UINT8_C( 99), UINT8_C(245),
                             UINT8_C(  8), UINT8_C(140), UINT8_C(141), UINT8_C(123),
                             UINT8_C(219), UINT8_C(163), UINT8_C(196), UINT8_C(233),
                             UINT8_C( 34), UINT8_C(185), UINT8_C(228), UINT8_C(108),
                             UINT8_C( 95), UINT8_C(236), UINT8_C( 97), UINT8_C( 41)),
      simde_x_mm512_set_epu8(UINT8_C(193), UINT8_C(230), UINT8_C( 93), UINT8_C( 23),
                             UINT8_C(193), UINT8_C( 52), UINT8_C(223), UINT8_C(175),
                             UINT8_C(205), UINT8_C( 45), UINT8_C(166), UINT8_C( 24),
                             UINT8_C( 71), UINT8_C(234), UINT8_C(161), UINT8_C(142),
                             UINT8_C(184), UINT8_C(218), UINT8_C(190), UINT8_C(212),
                             UINT8_C(116), UINT8_C(159), UINT8_C( 44), UINT8_C( 55),
                             UINT8_C(213), UINT8_C(133), UINT8_C( 60), UINT8_C(  3),
                             UINT8_C( 58), UINT8_C(255), UINT8_C(125), UINT8_C(189),
                             UINT8_C(145), UINT8_C( 88), UINT8_C( 55), UINT8_C(182),
                             UINT8_C( 23), UINT8_C(161), UINT8_C(133), UINT8_C( 27),
                             UINT8_C(125), UINT8_C(229), UINT8_C(203), UINT8_C( 45),
                             UINT8_C( 24), UINT8_C(  5), UINT8_C( 90), UINT8_C( 83),
                             UINT8_C(145), UINT8_C( 85), UINT8_C(156), UINT8_C(164),
                             UINT8_C(149), UINT8_C(201), UINT8_C( 48), UINT8_C(255),
                             UINT8_C( 41), UINT8_C( 42), UINT8_C( 94), UINT8_C(129),
                             UINT8_C(135), UINT8_C(  8), UINT8_C( 12), UINT8_C(203)),
      simde_x_mm512_set_epu8(UINT8_C( 40), UINT8_C( 79), UINT8_C( 12), UINT8_C(  0),
                             UINT8_C( 33), UINT8_C( 22), UINT8_C( 58), UINT8_C( 74),
                             UINT8_C( 45), UINT8_C( 26), UINT8_C(142), UINT8_C( 20),
                             UINT8_C( 17), UINT8_C( 63), UINT8_C( 34), UINT8_C(124),
                             UINT8_C( 66), UINT8_C( 48), UINT8_C( 31), UINT8_C( 20),
                             UINT8_C(105), UINT8_C( 75), UINT8_C( 23), UINT8_C( 25),
                             UINT8_C( 20), UINT8_C( 36), UINT8_C( 18), UINT8_C(  1),
                             UINT8_C( 25), UINT8_C( 27), UINT8_C( 12), UINT8_C( 34),
                             UINT8_C( 23), UINT8_C( 44), UINT8_C( 51), UINT8_C(109),
                             UINT8_C( 20), UINT8_C( 42), UINT8_C( 98), UINT8_C( 12),
                             UINT8_C(101), UINT8_C( 52), UINT8_C( 82), UINT8_C( 44),
                             UINT8_C( 22), UINT8_C(  4), UINT8_C(  9), UINT8_C( 79),
                             UINT8_C(  8), UINT8_C( 55), UINT8_C(141), UINT8_C(123),
                             UINT8_C( 70), UINT8_C(163), UINT8_C(  4), UINT8_C(233),
                             UINT8_C( 34), UINT8_C( 17), UINT8_C( 40), UINT8_C(108),
                             UINT8_C( 95), UINT8_C(  4), UINT8_C(  1), UINT8_C( 41)) },
    { simde_x_mm512_set_epu8(UINT8_C(142), UINT8_C( 19), UINT8_C(128), UINT8_C(  3),
                             UINT8_C(129), UINT8_C(192), UINT8_C(118), UINT8_C(156),
                             UINT8_C( 16), UINT8_C(232), UINT8_C(203), UINT8_C(122),
                             UINT8_C(229), UINT8_C(105), UINT8_C(120), UINT8_C(201),
                             UINT8_C(228), UINT8_C(167), UINT8_C(141), UINT8_C(146),
                             UINT8_C(116), UINT8_C( 74), UINT8_C(191), UINT8_C( 35),
                             UINT8_C( 45), UINT8_C(158), UINT8_C(228), UINT8_C(138),
                             UINT8_C( 49), UINT8_C(  7), UINT8_C( 65), UINT8_C(140),
                             UINT8_C(  0), UINT8_C(113), UINT8_C(156), UINT8_C(113),
                             UINT8_C(246), UINT8_C(167), UINT8_C(109), UINT8_C(141),
                             UINT8_C(192), UINT8_C( 11), UINT8_C( 33), UINT8_C(141),
                             UINT8_C(129), UINT8_C(  2), UINT8_C(168), UINT8_C(227),
                             UINT8_C( 23), UINT8_C(173), UINT8_C(104), UINT8_C( 71),
                             UINT8_C( 11), UINT8_C(250), UINT8_C( 13), UINT8_C(218),
                             UINT8_C(194), UINT8_C(140), UINT8_C(125), UINT8_C( 43),
                             UINT8_C(151), UINT8_C( 49), UINT8_C(129), UINT8_C(218)),
      simde_x_mm512_set_epu8(UINT8_C(  8), UINT8_C( 25), UINT8_C(147), UINT8_C(220),
                             UINT8_C(173), UINT8_C(138), UINT8_C( 38), UINT8_C(150),
                             UINT8_C( 35), UINT8_C( 43), UINT8_C(165), UINT8_C(185),
                             UINT8_C( 50), UINT8_C( 64), UINT8_C(161), UINT8_C(132),
                             UINT8_C(162), UINT8_C( 50), UINT8_C(199), UINT8_C( 84),
                             UINT8_C(251), UINT8_C(200), UINT8_C(217), UINT8_C( 19),
                             UINT8_C(180), UINT8_C(196), UINT8_C(246), UINT8_C( 76),
                             UINT8_C( 55), UINT8_C(204), UINT8_C(139), UINT8_C( 75),
                             UINT8_C(  1), UINT8_C( 89), UINT8_C(133), UINT8_C(212),
                             UINT8_C(206), UINT8_C( 55), UINT8_C(204), UINT8_C(120),
                             UINT8_C( 37), UINT8_C(159), UINT8_C(146), UINT8_C(217),
                             UINT8_C(226), UINT8_C(190), UINT8_C(134), UINT8_C(  8),
                             UINT8_C(113), UINT8_C( 61), UINT8_C(103), UINT8_C(100),
                             UINT8_C( 23), UINT8_C(229), UINT8_C(146), UINT8_C( 97),
                             UINT8_C( 95), UINT8_C( 32), UINT8_C(136), UINT8_C( 91),
                             UINT8_C( 46), UINT8_C(252), UINT8_C(163), UINT8_C( 88)),
      simde_x_mm512_set_epu8(UINT8_C(  6), UINT8_C( 19), UINT8_C(128), UINT8_C(  3),
                             UINT8_C(129), UINT8_C( 54), UINT8_C(  4), UINT8_C(  6),
                             UINT8_C( 16), UINT8_C( 17), UINT8_C( 38), UINT8_C(122),
                             UINT8_C( 29), UINT8_C( 41), UINT8_C(120), UINT8_C( 69),
                             UINT8_C( 66), UINT8_C( 17), UINT8_C(141), UINT8_C( 62),
                             UINT8_C(116), UINT8_C( 74), UINT8_C(191), UINT8_C( 16),
                             UINT8_C( 45), UINT8_C(158), UINT8_C(228), UINT8_C( 62),
                             UINT8_C( 49), UINT8_C(  7), UINT8_C( 65), UINT8_C( 65),
                             UINT8_C(  0), UINT8_C( 24), UINT8_C( 23), UINT8_C(113),
                             UINT8_C( 40), UINT8_C(  2), UINT8_C(109), UINT8_C( 21),
                             UINT8_C(  7), UINT8_C( 11), UINT8_C( 33), UINT8_C(141),
                             UINT8_C(129), UINT8_C(  2), UINT8_C( 34), UINT8_C(  3),
                             UINT8_C( 23), UINT8_C( 51), UINT8_C(  1), UINT8_C( 71),
                             UINT8_C( 11), UINT8_C( 21), UINT8_C( 13), UINT8_C( 24),
                             UINT8_C(  4), UINT8_C( 12), UINT8_C(125), UINT8_C( 43),
                             UINT8_C( 13), UINT8_C( 49), UINT8_C(129), UINT8_C( 42)) },
    { simde_x_mm512_set_epu8(UINT8_C( 46), UINT8_C( 43), UINT8_C(246), UINT8_C(157),
                             UINT8_C( 80), UINT8_C(154), UINT8_C( 27), UINT8_C(118),
                             UINT8_C(176), UINT8_C(216), UINT8_C( 46), UINT8_C(142),
                             UINT8_C(198), UINT8_C(248), UINT8_C( 88), UINT8_C( 29),
                             UINT8_C(176), UINT8_C( 25), UINT8_C(101), UINT8_C( 54),
                             UINT8_C(103), UINT8_C(120), UINT8_C( 94), UINT8_C( 16),
                             UINT8_C(197), UINT8_C(205), UINT8_C( 71), UINT8_C(246),
                             UINT8_C(158), UINT8_C(176), UINT8_C(218), UINT8_C( 43),
                             UINT8_C(235), UINT8_C(249), UINT8_C(116), UINT8_C(137),
                             UINT8_C( 89), UINT8_C(212), UINT8_C(132), UINT8_C( 56),
                             UINT8_C(230), UINT8_C(137), UINT8_C( 66), UINT8_C( 41),
                             UINT8_C( 44), UINT8_C( 35), UINT8_C(189), UINT8_C(155),
                             UINT8_C(125), UINT8_C(130), UINT8_C(123), UINT8_C(117),
                             UINT8_C(123), UINT8_C(127), UINT8_C(151), UINT8_C( 60),
                             UINT8_C(153), UINT8_C(185), UINT8_C(250), UINT8_C(100),
                             UINT8_C( 83), UINT8_C(112), UINT8_C( 33), UINT8_C(140)),
      simde_x_mm512_set_epu8(UINT8_C( 36), UINT8_C( 33), UINT8_C( 42), UINT8_C( 75),
                             UINT8_C(179), UINT8_C(172), UINT8_C(126), UINT8_C(171),
                             UINT8_C(110), UINT8_C(150), UINT8_C(107), UINT8_C(180),
                             UINT8_C(134), UINT8_C( 73), UINT8_C(207), UINT8_C( 15),
                             UINT8_C(241), UINT8_C(103), UINT8_C(103), UINT8_C(150),
                             UINT8_C(103), UINT8_C( 58), UINT8_C(104), UINT8_C( 35),
                             UINT8_C(249), UINT8_C( 79), UINT8_C(113), UINT8_C( 97),
                             UINT8_C(189), UINT8_C(197), UINT8_C(174), UINT8_C(222),
                             UINT8_C(224), UINT8_C(104), UINT8_C(123), UINT8_C(124),
                             UINT8_C( 49), UINT8_C(226), UINT8_C( 37), UINT8_C( 22),
                             UINT8_C(105), UINT8_C(157), UINT8_C(110), UINT8_C( 52),
                             UINT8_C(254), UINT8_C(103), UINT8_C(162), UINT8_C(210),
                             UINT8_C(202), UINT8_C( 39), UINT8_C(193), UINT8_C(151),
                             UINT8_C(183), UINT8_C( 73), UINT8_C( 97), UINT8_C(187),
                             UINT8_C(102), UINT8_C(195), UINT8_C( 68), UINT8_C(190),
                             UINT8_C( 65), UINT8_C( 60), UINT8_C(165), UINT8_C(126)),
      simde_x_mm512_set_epu8(UINT8_C( 10), UINT8_C( 10), UINT8_C( 36), UINT8_C(  7),
                             UINT8_C( 80), UINT8_C(154), UINT8_C( 27), UINT8_C(118),
                             UINT8_C( 66), UINT8_C( 66), UINT8_C( 46), UINT8_C(142),
                             UINT8_C( 64), UINT8_C( 29), UINT8_C( 88), UINT8_C( 14),
                             UINT8_C(176), UINT8_C( 25), UINT8_C(101), UINT8_C( 54),
                             UINT8_C(  0), UINT8_C(  4), UINT8_C( 94), UINT8_C( 16),
                             UINT8_C(197), UINT8_C( 47), UINT8_C( 71), UINT8_C( 52),
                             UINT8_C(158), UINT8_C(176), UINT8_C( 44), UINT8_C( 43),
                             UINT8_C( 11), UINT8_C( 41), UINT8_C(116), UINT8_C( 13),
                             UINT8_C( 40), UINT8_C(212), UINT8_C( 21), UINT8_C( 12),
                             UINT8_C( 20), UINT8_C(137), UINT8_C( 66), UINT8_C( 41),
                             UINT8_C( 44), UINT8_C( 35), UINT8_C( 27), UINT8_C(155),
                             UINT8_C(125), UINT8_C( 13), UINT8_C(123), UINT8_C(117),
                             UINT8_C(123), UINT8_C( 54), UINT8_C( 54), UINT8_C( 60),
                             UINT8_C( 51), UINT8_C(185), UINT8_C( 46), UINT8_C(100),
                             UINT8_C( 18), UINT8_C( 52), UINT8_C( 33), UINT8_C( 14)) },
    { simde_x_mm512_set_epu8(UINT8_C(240), UINT8_C(169), UINT8_C(  8), UINT8_C( 54),
                             UINT8_C( 66), UINT8_C( 99), UINT8_C( 14), UINT8_C( 32),
                             UINT8_C(148), UINT8_C( 92), UINT8_C(122), UINT8_C(200),
                             UINT8_C(192), UINT8_C(186), UINT8_C(225), UINT8_C( 52),
                             UINT8_C(182), UINT8_C(244), UINT8_C(253), UINT8_C(228),
                             UINT8_C(141), UINT8_C(228), UINT8_C(148), UINT8_C(168),
                             UINT8_C(231), UINT8_C(107), UINT8_C( 47), UINT8_C(205),
                             UINT8_C(126), UINT8_C(  7), UINT8_C(182), UINT8_C(245),
                             UINT8_C(165), UINT8_C(186), UINT8_C(213), UINT8_C( 84),
                             UINT8_C( 19), UINT8_C(131), UINT8_C( 54), UINT8_C( 13),
                             UINT8_C(185), UINT8_C(182), UINT8_C( 72), UINT8_C( 61),
                             UINT8_C(125), UINT8_C(104), UINT8_C(147), UINT8_C( 11),
                             UINT8_C( 89), UINT8_C(204), UINT8_C( 62), UINT8_C(163),
                             UINT8_C(198), UINT8_C(162), UINT8_C(205), UINT8_C(  9),
                             UINT8_C(182), UINT8_C(123), UINT8_C( 65), UINT8_C(208),
                             UINT8_C(145), UINT8_C(179), UINT8_C( 34), UINT8_C(195)),
      simde_x_mm512_set_epu8(UINT8_C(141), UINT8_C(103), UINT8_C(116), UINT8_C( 12),
                             UINT8_C(174), UINT8_C(226), UINT8_C(193), UINT8_C(175),
                             UINT8_C(155), UINT8_C(174), UINT8_C( 73), UINT8_C(  6),
                             UINT8_C(141), UINT8_C(140), UINT8_C(254), UINT8_C(193),
                             UINT8_C(100), UINT8_C(151), UINT8_C( 14), UINT8_C( 19),
                             UINT8_C( 38), UINT8_C(115), UINT8_C(201), UINT8_C(118),
                             UINT8_C( 74), UINT8_C(186), UINT8_C( 89), UINT8_C(183),
                             UINT8_C( 65), UINT8_C(138), UINT8_C( 64), UINT8_C( 90),
                             UINT8_C(152), UINT8_C(241), UINT8_C(229), UINT8_C(218),
                             UINT8_C(126), UINT8_C( 38), UINT8_C(159), UINT8_C( 27),
                             UINT8_C(164), UINT8_C(199), UINT8_C( 25), UINT8_C(253),
                             UINT8_C(181), UINT8_C(104), UINT8_C(  6), UINT8_C(183),
                             UINT8_C( 36), UINT8_C(203), UINT8_C(138), UINT8_C(145),
                             UINT8_C(116), UINT8_C(155), UINT8_C(218), UINT8_C( 24),
                             UINT8_C(205), UINT8_C(238), UINT8_C(242), UINT8_C( 26),
                             UINT8_C(226), UINT8_C( 76), UINT8_C(226), UINT8_C(214)),
      simde_x_mm512_set_epu8(UINT8_C( 99), UINT8_C( 66), UINT8_C(  8), UINT8_C(  6),
                             UINT8_C( 66), UINT8_C( 99), UINT8_C( 14), UINT8_C( 32),
                             UINT8_C(148), UINT8_C( 92), UINT8_C( 49), UINT8_C(  2),
                             UINT8_C( 51), UINT8_C( 46), UINT8_C(225), UINT8_C( 52),
                             UINT8_C( 82), UINT8_C( 93), UINT8_C(  1), UINT8_C(  0),
                             UINT8_C( 27), UINT8_C(113), UINT8_C(148), UINT8_C( 50),
                             UINT8_C(  9), UINT8_C(107), UINT8_C( 47), UINT8_C( 22),
                             UINT8_C( 61), UINT8_C(  7), UINT8_C( 54), UINT8_C( 65),
                             UINT8_C( 13), UINT8_C(186), UINT8_C(213), UINT8_C( 84),
                             UINT8_C( 19), UINT8_C( 17), UINT8_C( 54), UINT8_C( 13),
                             UINT8_C( 21), UINT8_C(182), UINT8_C( 22), UINT8_C( 61),
                             UINT8_C(125), UINT8_C(  0), UINT8_C(  3), UINT8_C( 11),
                             UINT8_C( 17), UINT8_C(  1), UINT8_C( 62), UINT8_C( 18),
                             UINT8_C( 82), UINT8_C(  7), UINT8_C(205), UINT8_C(  9),
                             UINT8_C(182), UINT8_C(123), UINT8_C( 65), UINT8_C(  0),
                             UINT8_C(145), UINT8_C( 27), UINT8_C( 34), UINT8_C(195)) },
    { simde_x_mm512_set_epu8(UINT8_C(197), UINT8_C( 52), UINT8_C(145), UINT8_C( 20),
                             UINT8_C( 26), UINT8_C(178), UINT8_C(121), UINT8_C( 16),
                             UINT8_C( 45), UINT8_C(229), UINT8_C( 11), UINT8_C(230),
                             UINT8_C( 53), UINT8_C(  2), UINT8_C(234), UINT8_C(  7),
                             UINT8_C(207), UINT8_C(146), UINT8_C(169), UINT8_C(233),
                             UINT8_C(206), UINT8_C(116), UINT8_C( 55), UINT8_C(156),
                             UINT8_C(180), UINT8_C( 91), UINT8_C( 56), UINT8_C(146),
                             UINT8_C( 55), UINT8_C(137), UINT8_C(200), UINT8_C( 76),
                             UINT8_C( 43), UINT8_C(245), UINT8_C(138), UINT8_C(  3),
                             UINT8_C(213), UINT8_C(156), UINT8_C(166), UINT8_C(234),
                             UINT8_C(199), UINT8_C(  2), UINT8_C( 86), UINT8_C( 72),
                             UINT8_C( 93), UINT8_C(254), UINT8_C(190), UINT8_C(121),
                             UINT8_C(119), UINT8_C( 75), UINT8_C(159), UINT8_C( 76),
                             UINT8_C( 70), UINT8_C(218), UINT8_C( 17), UINT8_C(239),
                             UINT8_C( 43), UINT8_C(152), UINT8_C(222), UINT8_C( 80),
                             UINT8_C(197), UINT8_C(113), UINT8_C(112), UINT8_C( 81)),
      simde_x_mm512_set_epu8(UINT8_C(193), UINT8_C(162), UINT8_C(178), UINT8_C( 36),
                             UINT8_C(178), UINT8_C( 86), UINT8_C( 79), UINT8_C(167),
                             UINT8_C(179), UINT8_C( 45), UINT8_C( 18), UINT8_C(231),
                             UINT8_C(113), UINT8_C(127), UINT8_C(211), UINT8_C(181),
                             UINT8_C(121), UINT8_C(171), UINT8_C( 76), UINT8_C(135),
                             UINT8_C( 15), UINT8_C(133), UINT8_C(247), UINT8_C( 32),
                             UINT8_C(181), UINT8_C(168), UINT8_C(236), UINT8_C( 99),
                             UINT8_C( 85), UINT8_C(151), UINT8_C( 36), UINT8_C( 99),
                             UINT8_C(101), UINT8_C( 42), UINT8_C( 63), UINT8_C( 96),
                             UINT8_C(210), UINT8_C(198), UINT8_C(202), UINT8_C(105),
                             UINT8_C(214), UINT8_C( 74), UINT8_C(199), UINT8_C( 17),
                             UINT8_C(234), UINT8_C( 22), UINT8_C(134), UINT8_C(112),
                             UINT8_C( 62), UINT8_C(141), UINT8_C(156), UINT8_C( 91),
                             UINT8_C( 99), UINT8_C( 24), UINT8_C(198), UINT8_C(131),
                             UINT8_C( 88), UINT8_C(136), UINT8_C( 61), UINT8_C( 94),
                             UINT8_C(189), UINT8_C(213), UINT8_C(249), UINT8_C(131)),
      simde_x_mm512_set_epu8(UINT8_C(  4), UINT8_C( 52), UINT8_C(145), UINT8_C( 20),
                             UINT8_C( 26), UINT8_C(  6), UINT8_C( 42), UINT8_C( 16),
                             UINT8_C( 45), UINT8_C(  4), UINT8_C( 11), UINT8_C(230),
                             UINT8_C( 53), UINT8_C(  2), UINT8_C( 23), UINT8_C(  7),
                             UINT8_C( 86), UINT8_C(146), UINT8_C( 17), UINT8_C( 98),
                             UINT8_C( 11), UINT8_C(116), UINT8_C( 55), UINT8_C( 28),
                             UINT8_C(180), UINT8_C( 91), UINT8_C( 56), UINT8_C( 47),
                             UINT8_C( 55), UINT8_C(137), UINT8_C( 20), UINT8_C( 76),
                             UINT8_C( 43), UINT8_C( 35), UINT8_C( 12), UINT8_C(  3),
                             UINT8_C(  3), UINT8_C(156), UINT8_C(166), UINT8_C( 24),
                             UINT8_C(199), UINT8_C(  2), UINT8_C( 86), UINT8_C(  4),
                             UINT8_C( 93), UINT8_C( 12), UINT8_C( 56), UINT8_C(  9),
                             UINT8_C( 57), UINT8_C( 75), UINT8_C(  3), UINT8_C( 76),
                             UINT8_C( 70), UINT8_C(  2), UINT8_C( 17), UINT8_C(108),
                             UINT8_C( 43), UINT8_C( 16), UINT8_C( 39), UINT8_C( 80),
                             UINT8_C(  8), UINT8_C(113), UINT8_C(112), UINT8_C( 81)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_rem_epu8(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u8(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_rem_epu16(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu16(UINT16_C( 10545), UINT16_C( 43974), UINT16_C( 10284), UINT16_C( 62003),
                              UINT16_C( 35545), UINT16_C( 55289), UINT16_C( 51493), UINT16_C( 35101),
                              UINT16_C( 59818), UINT16_C( 61822), UINT16_C( 46602), UINT16_C( 53446),
                              UINT16_C( 23938), UINT16_C( 50097), UINT16_C( 48095), UINT16_C( 35837),
                              UINT16_C( 49063), UINT16_C( 57920), UINT16_C( 54730), UINT16_C( 28273),
                              UINT16_C( 23021), UINT16_C( 18146), UINT16_C( 33883), UINT16_C( 65368),
                              UINT16_C( 26666), UINT16_C( 13822), UINT16_C( 34046), UINT16_C( 24651),
                              UINT16_C(  8048), UINT16_C( 38825), UINT16_C( 44126), UINT16_C( 28762)),
      simde_x_mm512_set_epu16(UINT16_C( 38607), UINT16_C(  8074), UINT16_C( 18000), UINT16_C( 35687),
                              UINT16_C( 40415), UINT16_C(  3254), UINT16_C( 55282), UINT16_C( 38855),
                              UINT16_C( 41330), UINT16_C( 37148), UINT16_C( 25803), UINT16_C( 25877),
                              UINT16_C(   768), UINT16_C( 16244), UINT16_C( 11114), UINT16_C( 58324),
                              UINT16_C( 18192), UINT16_C( 32532), UINT16_C( 33700), UINT16_C( 60373),
                              UINT16_C( 20183), UINT16_C( 64042), UINT16_C(  2502), UINT16_C( 18488),
                              UINT16_C( 22771), UINT16_C( 21470), UINT16_C(  4556), UINT16_C( 26138),
                              UINT16_C( 19085), UINT16_C( 64613), UINT16_C( 55602), UINT16_C( 63371)),
      simde_x_mm512_set_epu16(UINT16_C( 10545), UINT16_C(  3604), UINT16_C( 10284), UINT16_C( 26316),
                              UINT16_C( 35545), UINT16_C(  3225), UINT16_C( 51493), UINT16_C( 35101),
                              UINT16_C( 18488), UINT16_C( 24674), UINT16_C( 20799), UINT16_C(  1692),
                              UINT16_C(   130), UINT16_C(  1365), UINT16_C(  3639), UINT16_C( 35837),
                              UINT16_C( 12679), UINT16_C( 25388), UINT16_C( 21030), UINT16_C( 28273),
                              UINT16_C(  2838), UINT16_C( 18146), UINT16_C(  1357), UINT16_C(  9904),
                              UINT16_C(  3895), UINT16_C( 13822), UINT16_C(  2154), UINT16_C( 24651),
                              UINT16_C(  8048), UINT16_C( 38825), UINT16_C( 44126), UINT16_C( 28762)) },
    { simde_x_mm512_set_epu16(UINT16_C( 20057), UINT16_C( 26978), UINT16_C( 45741), UINT16_C( 34503),
                              UINT16_C( 54259), UINT16_C( 41436), UINT16_C( 43883), UINT16_C( 11009),
                              UINT16_C( 50212), UINT16_C(  9014), UINT16_C( 24117), UINT16_C( 34039),
                              UINT16_C( 58348), UINT16_C(  8311), UINT16_C( 31759), UINT16_C(  4002),
                              UINT16_C(  7525), UINT16_C(  3321), UINT16_C( 47299), UINT16_C( 64213),
                              UINT16_C( 13644), UINT16_C( 48153), UINT16_C( 45234), UINT16_C( 51700),
                              UINT16_C(  7513), UINT16_C(  1114), UINT16_C( 65336), UINT16_C( 10389),
                              UINT16_C( 33688), UINT16_C(  9445), UINT16_C( 60332), UINT16_C( 41466)),
      simde_x_mm512_set_epu16(UINT16_C( 48157), UINT16_C( 56913), UINT16_C( 55050), UINT16_C( 48859),
                              UINT16_C( 27895), UINT16_C( 48343), UINT16_C( 59593), UINT16_C( 60425),
                              UINT16_C( 62587), UINT16_C( 54231), UINT16_C( 52444), UINT16_C(  8140),
                              UINT16_C( 58695), UINT16_C(  2476), UINT16_C( 41101), UINT16_C(  7948),
                              UINT16_C( 26094), UINT16_C( 52354), UINT16_C( 30122), UINT16_C( 47688),
                              UINT16_C( 43801), UINT16_C( 57764), UINT16_C(  1809), UINT16_C( 33603),
                              UINT16_C(  8271), UINT16_C(  4936), UINT16_C(  7627), UINT16_C( 20477),
                              UINT16_C( 14608), UINT16_C( 25470), UINT16_C( 45836), UINT16_C( 25611)),
      simde_x_mm512_set_epu16(UINT16_C( 20057), UINT16_C( 26978), UINT16_C( 45741), UINT16_C( 34503),
                              UINT16_C( 26364), UINT16_C( 41436), UINT16_C( 43883), UINT16_C( 11009),
                              UINT16_C( 50212), UINT16_C(  9014), UINT16_C( 24117), UINT16_C(  1479),
                              UINT16_C( 58348), UINT16_C(   883), UINT16_C( 31759), UINT16_C(  4002),
                              UINT16_C(  7525), UINT16_C(  3321), UINT16_C( 17177), UINT16_C( 16525),
                              UINT16_C( 13644), UINT16_C( 48153), UINT16_C(     9), UINT16_C( 18097),
                              UINT16_C(  7513), UINT16_C(  1114), UINT16_C(  4320), UINT16_C( 10389),
                              UINT16_C(  4472), UINT16_C(  9445), UINT16_C( 14496), UINT16_C( 15855)) },
    { simde_x_mm512_set_epu16(UINT16_C( 26902), UINT16_C( 51011), UINT16_C( 57631), UINT16_C( 57521),
                              UINT16_C( 43405), UINT16_C( 18318), UINT16_C( 44023), UINT16_C(  9770),
                              UINT16_C(  4118), UINT16_C( 33099), UINT16_C(  6621), UINT16_C( 57639),
                              UINT16_C( 22002), UINT16_C( 33155), UINT16_C( 15537), UINT16_C( 38743),
                              UINT16_C( 26466), UINT16_C( 21183), UINT16_C(  5811), UINT16_C( 17016),
                              UINT16_C( 51162), UINT16_C( 46775), UINT16_C( 54252), UINT16_C( 64603),
                              UINT16_C( 30444), UINT16_C( 20573), UINT16_C( 50572), UINT16_C( 25607),
                              UINT16_C( 36721), UINT16_C( 36797), UINT16_C( 27147), UINT16_C( 62271)),
      simde_x_mm512_set_epu16(UINT16_C( 55381), UINT16_C( 52839), UINT16_C( 60314), UINT16_C( 33159),
                              UINT16_C( 32076), UINT16_C( 51820), UINT16_C( 13383), UINT16_C( 43204),
                              UINT16_C( 18058), UINT16_C( 42817), UINT16_C( 56737), UINT16_C( 40285),
                              UINT16_C( 49341), UINT16_C( 39323), UINT16_C( 53205), UINT16_C( 27016),
                              UINT16_C( 59998), UINT16_C( 61452), UINT16_C( 37377), UINT16_C( 37691),
                              UINT16_C( 64794), UINT16_C(  6696), UINT16_C(  3074), UINT16_C( 59025),
                              UINT16_C( 43625), UINT16_C( 28576), UINT16_C( 36042), UINT16_C( 42716),
                              UINT16_C( 47937), UINT16_C( 64195), UINT16_C(  8579), UINT16_C(   676)),
      simde_x_mm512_set_epu16(UINT16_C( 26902), UINT16_C( 51011), UINT16_C( 57631), UINT16_C( 24362),
                              UINT16_C( 11329), UINT16_C( 18318), UINT16_C(  3874), UINT16_C(  9770),
                              UINT16_C(  4118), UINT16_C( 33099), UINT16_C(  6621), UINT16_C( 17354),
                              UINT16_C( 22002), UINT16_C( 33155), UINT16_C( 15537), UINT16_C( 11727),
                              UINT16_C( 26466), UINT16_C( 21183), UINT16_C(  5811), UINT16_C( 17016),
                              UINT16_C( 51162), UINT16_C(  6599), UINT16_C(  1994), UINT16_C(  5578),
                              UINT16_C( 30444), UINT16_C( 20573), UINT16_C( 14530), UINT16_C( 25607),
                              UINT16_C( 36721), UINT16_C( 36797), UINT16_C(  1410), UINT16_C(    79)) },
    { simde_x_mm512_set_epu16(UINT16_C(  7566), UINT16_C( 25511), UINT16_C( 59705), UINT16_C( 13989),
                              UINT16_C( 13965), UINT16_C( 34471), UINT16_C(    77), UINT16_C( 35152),
                              UINT16_C( 21705), UINT16_C( 42504), UINT16_C( 63033), UINT16_C( 56884),
                              UINT16_C( 42389), UINT16_C( 61527), UINT16_C(  7598), UINT16_C( 23051),
                              UINT16_C( 13886), UINT16_C( 28688), UINT16_C( 30551), UINT16_C( 36608),
                              UINT16_C( 56045), UINT16_C( 38987), UINT16_C( 64798), UINT16_C( 22350),
                              UINT16_C(  7981), UINT16_C( 50477), UINT16_C( 46688), UINT16_C( 16804),
                              UINT16_C( 33660), UINT16_C( 63749), UINT16_C( 29649), UINT16_C( 64815)),
      simde_x_mm512_set_epu16(UINT16_C( 18409), UINT16_C( 19069), UINT16_C( 20979), UINT16_C( 35774),
                              UINT16_C(  8112), UINT16_C( 25085), UINT16_C( 31664), UINT16_C( 55404),
                              UINT16_C( 63329), UINT16_C( 19403), UINT16_C( 33006), UINT16_C( 20365),
                              UINT16_C( 22045), UINT16_C( 41935), UINT16_C( 28665), UINT16_C( 35793),
                              UINT16_C( 26789), UINT16_C( 40241), UINT16_C( 34076), UINT16_C( 36189),
                              UINT16_C( 49507), UINT16_C( 32891), UINT16_C( 45700), UINT16_C( 31541),
                              UINT16_C( 33237), UINT16_C( 50719), UINT16_C( 22782), UINT16_C( 46902),
                              UINT16_C( 62792), UINT16_C(   907), UINT16_C(  9939), UINT16_C(   395)),
      simde_x_mm512_set_epu16(UINT16_C(  7566), UINT16_C(  6442), UINT16_C( 17747), UINT16_C( 13989),
                              UINT16_C(  5853), UINT16_C(  9386), UINT16_C(    77), UINT16_C( 35152),
                              UINT16_C( 21705), UINT16_C(  3698), UINT16_C( 30027), UINT16_C( 16154),
                              UINT16_C( 20344), UINT16_C( 19592), UINT16_C(  7598), UINT16_C( 23051),
                              UINT16_C( 13886), UINT16_C( 28688), UINT16_C( 30551), UINT16_C(   419),
                              UINT16_C(  6538), UINT16_C(  6096), UINT16_C( 19098), UINT16_C( 22350),
                              UINT16_C(  7981), UINT16_C( 50477), UINT16_C(  1124), UINT16_C( 16804),
                              UINT16_C( 33660), UINT16_C(   259), UINT16_C(  9771), UINT16_C(    35)) },
    { simde_x_mm512_set_epu16(UINT16_C( 40553), UINT16_C(  9260), UINT16_C(  6846), UINT16_C( 21618),
                              UINT16_C( 20365), UINT16_C( 26413), UINT16_C(  7670), UINT16_C(  6521),
                              UINT16_C( 13052), UINT16_C( 19892), UINT16_C( 40021), UINT16_C( 58092),
                              UINT16_C( 12337), UINT16_C( 14080), UINT16_C(  6934), UINT16_C( 61515),
                              UINT16_C(  1885), UINT16_C( 11733), UINT16_C(  7371), UINT16_C( 24583),
                              UINT16_C( 48349), UINT16_C( 37475), UINT16_C( 47206), UINT16_C( 54691),
                              UINT16_C( 63460), UINT16_C(  2107), UINT16_C( 62169), UINT16_C( 38808),
                              UINT16_C( 21341), UINT16_C( 51834), UINT16_C( 26283), UINT16_C( 38235)),
      simde_x_mm512_set_epu16(UINT16_C(  9227), UINT16_C( 20728), UINT16_C( 22448), UINT16_C( 22271),
                              UINT16_C( 38010), UINT16_C(  3228), UINT16_C( 38598), UINT16_C( 15839),
                              UINT16_C(  4554), UINT16_C( 22831), UINT16_C( 44103), UINT16_C( 32351),
                              UINT16_C( 46747), UINT16_C( 20983), UINT16_C( 61889), UINT16_C( 26454),
                              UINT16_C( 63311), UINT16_C( 19804), UINT16_C( 62773), UINT16_C( 56806),
                              UINT16_C( 36384), UINT16_C( 25302), UINT16_C( 37143), UINT16_C(  3478),
                              UINT16_C( 59861), UINT16_C( 61175), UINT16_C( 48658), UINT16_C( 23119),
                              UINT16_C( 30252), UINT16_C( 63116), UINT16_C( 13170), UINT16_C( 44087)),
      simde_x_mm512_set_epu16(UINT16_C(  3645), UINT16_C(  9260), UINT16_C(  6846), UINT16_C( 21618),
                              UINT16_C( 20365), UINT16_C(   589), UINT16_C(  7670), UINT16_C(  6521),
                              UINT16_C(  3944), UINT16_C( 19892), UINT16_C( 40021), UINT16_C( 25741),
                              UINT16_C( 12337), UINT16_C( 14080), UINT16_C(  6934), UINT16_C(  8607),
                              UINT16_C(  1885), UINT16_C( 11733), UINT16_C(  7371), UINT16_C( 24583),
                              UINT16_C( 11965), UINT16_C( 12173), UINT16_C( 10063), UINT16_C(  2521),
                              UINT16_C(  3599), UINT16_C(  2107), UINT16_C( 13511), UINT16_C( 15689),
                              UINT16_C( 21341), UINT16_C( 51834), UINT16_C( 13113), UINT16_C( 38235)) },
    { simde_x_mm512_set_epu16(UINT16_C( 22335), UINT16_C( 12112), UINT16_C(  9189), UINT16_C(  1311),
                              UINT16_C( 58441), UINT16_C( 13615), UINT16_C( 43712), UINT16_C( 31469),
                              UINT16_C( 12162), UINT16_C( 56166), UINT16_C( 41769), UINT16_C( 50135),
                              UINT16_C( 50998), UINT16_C( 24958), UINT16_C(  2725), UINT16_C( 39768),
                              UINT16_C( 47167), UINT16_C( 24484), UINT16_C( 16711), UINT16_C( 44632),
                              UINT16_C( 46990), UINT16_C( 25102), UINT16_C(  6573), UINT16_C( 22274),
                              UINT16_C( 49039), UINT16_C( 38914), UINT16_C( 32256), UINT16_C( 41529),
                              UINT16_C( 62756), UINT16_C( 61238), UINT16_C(  8613), UINT16_C( 51028)),
      simde_x_mm512_set_epu16(UINT16_C( 30472), UINT16_C( 36773), UINT16_C(  7714), UINT16_C( 18947),
                              UINT16_C(  7066), UINT16_C( 47844), UINT16_C( 58651), UINT16_C(  1841),
                              UINT16_C( 35799), UINT16_C( 50579), UINT16_C( 32926), UINT16_C( 26598),
                              UINT16_C( 39537), UINT16_C( 61137), UINT16_C(  5946), UINT16_C(  2262),
                              UINT16_C( 60116), UINT16_C( 12953), UINT16_C( 38045), UINT16_C( 47787),
                              UINT16_C( 30618), UINT16_C( 37811), UINT16_C( 51748), UINT16_C( 52236),
                              UINT16_C( 23394), UINT16_C(  2441), UINT16_C( 32382), UINT16_C(  9384),
                              UINT16_C( 25792), UINT16_C( 56163), UINT16_C( 22658), UINT16_C( 20939)),
      simde_x_mm512_set_epu16(UINT16_C( 22335), UINT16_C( 12112), UINT16_C(  1475), UINT16_C(  1311),
                              UINT16_C(  1913), UINT16_C( 13615), UINT16_C( 43712), UINT16_C(   172),
                              UINT16_C( 12162), UINT16_C(  5587), UINT16_C(  8843), UINT16_C( 23537),
                              UINT16_C( 11461), UINT16_C( 24958), UINT16_C(  2725), UINT16_C(  1314),
                              UINT16_C( 47167), UINT16_C( 11531), UINT16_C( 16711), UINT16_C( 44632),
                              UINT16_C( 16372), UINT16_C( 25102), UINT16_C(  6573), UINT16_C( 22274),
                              UINT16_C(  2251), UINT16_C(  2299), UINT16_C( 32256), UINT16_C(  3993),
                              UINT16_C( 11172), UINT16_C(  5075), UINT16_C(  8613), UINT16_C(  9150)) },
    { simde_x_mm512_set_epu16(UINT16_C( 13867), UINT16_C( 28091), UINT16_C( 35390), UINT16_C( 56986),
                              UINT16_C( 31509), UINT16_C( 63331), UINT16_C(  9520), UINT16_C( 29929),
                              UINT16_C( 24571), UINT16_C( 37741), UINT16_C( 52686), UINT16_C( 14609),
                              UINT16_C( 31001), UINT16_C(   823), UINT16_C( 45697), UINT16_C( 38351),
                              UINT16_C( 35780), UINT16_C( 41006), UINT16_C(  3633), UINT16_C( 45500),
                              UINT16_C( 30184), UINT16_C( 27396), UINT16_C(  1171), UINT16_C( 25936),
                              UINT16_C( 61703), UINT16_C( 57786), UINT16_C( 19453), UINT16_C( 30002),
                              UINT16_C(  6315), UINT16_C(   244), UINT16_C(  8399), UINT16_C( 57456)),
      simde_x_mm512_set_epu16(UINT16_C( 18752), UINT16_C( 27431), UINT16_C( 53704), UINT16_C( 42625),
                              UINT16_C( 42869), UINT16_C( 41745), UINT16_C( 47543), UINT16_C( 11401),
                              UINT16_C( 26966), UINT16_C( 26500), UINT16_C(  7486), UINT16_C(  7825),
                              UINT16_C( 17767), UINT16_C( 58506), UINT16_C( 36234), UINT16_C( 38373),
                              UINT16_C( 54992), UINT16_C( 46906), UINT16_C( 52104), UINT16_C( 31285),
                              UINT16_C( 34932), UINT16_C( 29467), UINT16_C( 33781), UINT16_C(   883),
                              UINT16_C( 23995), UINT16_C( 43069), UINT16_C( 53587), UINT16_C( 11327),
                              UINT16_C( 36611), UINT16_C(  7518), UINT16_C( 30015), UINT16_C( 30285)),
      simde_x_mm512_set_epu16(UINT16_C( 13867), UINT16_C(   660), UINT16_C( 35390), UINT16_C( 14361),
                              UINT16_C( 31509), UINT16_C( 21586), UINT16_C(  9520), UINT16_C(  7127),
                              UINT16_C( 24571), UINT16_C( 11241), UINT16_C(   284), UINT16_C(  6784),
                              UINT16_C( 13234), UINT16_C(   823), UINT16_C(  9463), UINT16_C( 38351),
                              UINT16_C( 35780), UINT16_C( 41006), UINT16_C(  3633), UINT16_C( 14215),
                              UINT16_C( 30184), UINT16_C( 27396), UINT16_C(  1171), UINT16_C(   329),
                              UINT16_C( 13713), UINT16_C( 14717), UINT16_C( 19453), UINT16_C(  7348),
                              UINT16_C(  6315), UINT16_C(   244), UINT16_C(  8399), UINT16_C( 27171)) },
    { simde_x_mm512_set_epu16(UINT16_C( 19003), UINT16_C( 26627), UINT16_C( 63705), UINT16_C( 34218),
                              UINT16_C( 36055), UINT16_C( 13847), UINT16_C( 44625), UINT16_C(  9042),
                              UINT16_C( 36148), UINT16_C( 11660), UINT16_C( 32339), UINT16_C( 39715),
                              UINT16_C( 47178), UINT16_C( 21002), UINT16_C( 60706), UINT16_C(  8527),
                              UINT16_C( 26072), UINT16_C( 29611), UINT16_C( 18348), UINT16_C(   953),
                              UINT16_C( 33382), UINT16_C( 22717), UINT16_C( 50122), UINT16_C( 52414),
                              UINT16_C( 59278), UINT16_C( 54225), UINT16_C( 31952), UINT16_C( 29752),
                              UINT16_C( 37488), UINT16_C( 20614), UINT16_C(  1055), UINT16_C( 61149)),
      simde_x_mm512_set_epu16(UINT16_C( 59727), UINT16_C(  3072), UINT16_C(  8626), UINT16_C( 14922),
                              UINT16_C( 64116), UINT16_C( 36372), UINT16_C( 22591), UINT16_C(  8828),
                              UINT16_C( 64048), UINT16_C( 56808), UINT16_C( 56651), UINT16_C( 39760),
                              UINT16_C( 59817), UINT16_C( 50914), UINT16_C( 21275), UINT16_C( 35106),
                              UINT16_C(  6020), UINT16_C( 27245), UINT16_C( 34763), UINT16_C( 25208),
                              UINT16_C( 25908), UINT16_C( 21036), UINT16_C( 36366), UINT16_C( 25589),
                              UINT16_C(  2188), UINT16_C( 36219), UINT16_C( 56227), UINT16_C( 50409),
                              UINT16_C(  8889), UINT16_C( 58476), UINT16_C( 24556), UINT16_C( 24873)),
      simde_x_mm512_set_epu16(UINT16_C( 19003), UINT16_C(  2051), UINT16_C(  3323), UINT16_C(  4374),
                              UINT16_C( 36055), UINT16_C( 13847), UINT16_C( 22034), UINT16_C(   214),
                              UINT16_C( 36148), UINT16_C( 11660), UINT16_C( 32339), UINT16_C( 39715),
                              UINT16_C( 47178), UINT16_C( 21002), UINT16_C( 18156), UINT16_C(  8527),
                              UINT16_C(  1992), UINT16_C(  2366), UINT16_C( 18348), UINT16_C(   953),
                              UINT16_C(  7474), UINT16_C(  1681), UINT16_C( 13756), UINT16_C(  1236),
                              UINT16_C(   202), UINT16_C( 18006), UINT16_C( 31952), UINT16_C( 29752),
                              UINT16_C(  1932), UINT16_C( 20614), UINT16_C(  1055), UINT16_C( 11403)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_rem_epu16(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u16(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_rem_epu32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu32(UINT32_C( 691121094), UINT32_C( 674034227), UINT32_C(2329532409), UINT32_C(3374680349),
                              UINT32_C(3920294270), UINT32_C(3054162118), UINT32_C(1568850865), UINT32_C(3151989757),
                              UINT32_C(3215450688), UINT32_C(3586813553), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C(2891870298)),
      simde_x_mm512_set_epu32(UINT32_C(2530156426), UINT32_C(1179683687), UINT32_C(2648640694), UINT32_C(3623000007),
                              UINT32_C(2708640028), UINT32_C(1691051285), UINT32_C(  50347892), UINT32_C( 728425428),
                              UINT32_C(1192263444), UINT32_C(2208623573), UINT32_C(1322777130), UINT32_C( 163989560),
                              UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(1250819173), UINT32_C(3643996043)),
      simde_x_mm512_set_epu32(UINT32_C( 691121094), UINT32_C( 674034227), UINT32_C(2329532409), UINT32_C(3374680349),
                              UINT32_C(1211654242), UINT32_C(1363110833), UINT32_C(   8066213), UINT32_C( 238288045),
                              UINT32_C( 830923800), UINT32_C(1378189980), UINT32_C( 185945272), UINT32_C(  88757376),
                              UINT32_C( 255255072), UINT32_C( 141006229), UINT32_C( 527472553), UINT32_C(2891870298)) },
    { simde_x_mm512_set_epu32(UINT32_C(1314482530), UINT32_C(2997716679), UINT32_C(3555959260), UINT32_C(2875927297),
                              UINT32_C(3290702646), UINT32_C(1580565751), UINT32_C(3823902839), UINT32_C(2081361826),
                              UINT32_C( 493161721), UINT32_C(3099851477), UINT32_C( 894221337), UINT32_C(2964507124),
                              UINT32_C( 492373082), UINT32_C(4281870485), UINT32_C(2207786213), UINT32_C(3953959418)),
      simde_x_mm512_set_epu32(UINT32_C(3156074065), UINT32_C(3607805659), UINT32_C(1828175063), UINT32_C(3905547273),
                              UINT32_C(4101755863), UINT32_C(3436978124), UINT32_C(3846637996), UINT32_C(2693603084),
                              UINT32_C(1710148738), UINT32_C(1974123080), UINT32_C(2870600100), UINT32_C( 118588227),
                              UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358), UINT32_C(3003933707)),
      simde_x_mm512_set_epu32(UINT32_C(1314482530), UINT32_C(2997716679), UINT32_C(1727784197), UINT32_C(2875927297),
                              UINT32_C(3290702646), UINT32_C(1580565751), UINT32_C(3823902839), UINT32_C(2081361826),
                              UINT32_C( 493161721), UINT32_C(1125728397), UINT32_C( 894221337), UINT32_C( 118389676),
                              UINT32_C( 492373082), UINT32_C( 282962093), UINT32_C( 293035497), UINT32_C( 950025711)) },
    { simde_x_mm512_set_epu32(UINT32_C(1763100483), UINT32_C(3776962737), UINT32_C(2844608398), UINT32_C(2885101098),
                              UINT32_C( 269910347), UINT32_C( 433971495), UINT32_C(1441956227), UINT32_C(1018271575),
                              UINT32_C(1734496959), UINT32_C( 380846712), UINT32_C(3352999607), UINT32_C(3555523675),
                              UINT32_C(1995198557), UINT32_C(3314312199), UINT32_C(2406584253), UINT32_C(1779168063)),
      simde_x_mm512_set_epu32(UINT32_C(3629502055), UINT32_C(3952771463), UINT32_C(2102184556), UINT32_C( 877111492),
                              UINT32_C(1183491905), UINT32_C(3718356317), UINT32_C(3233651099), UINT32_C(3486869896),
                              UINT32_C(3932090380), UINT32_C(2449576763), UINT32_C(4246346280), UINT32_C( 201516689),
                              UINT32_C(2859036576), UINT32_C(2362091228), UINT32_C(3141663427), UINT32_C( 562234020)),
      simde_x_mm512_set_epu32(UINT32_C(1763100483), UINT32_C(3776962737), UINT32_C( 742423842), UINT32_C( 253766622),
                              UINT32_C( 269910347), UINT32_C( 433971495), UINT32_C(1441956227), UINT32_C(1018271575),
                              UINT32_C(1734496959), UINT32_C( 380846712), UINT32_C(3352999607), UINT32_C( 129739962),
                              UINT32_C(1995198557), UINT32_C( 952220971), UINT32_C(2406584253), UINT32_C(  92466003)) },
    { simde_x_mm512_set_epu32(UINT32_C( 495870887), UINT32_C(3912840869), UINT32_C( 915244711), UINT32_C(   5081424),
                              UINT32_C(1422501384), UINT32_C(4130987572), UINT32_C(2778067031), UINT32_C( 497965579),
                              UINT32_C( 910061584), UINT32_C(2002226944), UINT32_C(3673004107), UINT32_C(4246624078),
                              UINT32_C( 523093293), UINT32_C(3059761572), UINT32_C(2206005509), UINT32_C(1943141679)),
      simde_x_mm512_set_epu32(UINT32_C(1206471293), UINT32_C(1374915518), UINT32_C( 531653117), UINT32_C(2075187308),
                              UINT32_C(4150348747), UINT32_C(2163101581), UINT32_C(1444783055), UINT32_C(1878625233),
                              UINT32_C(1755684145), UINT32_C(2233240925), UINT32_C(3244523643), UINT32_C(2995026741),
                              UINT32_C(2178270751), UINT32_C(1493088054), UINT32_C(4115137419), UINT32_C( 651362699)),
      simde_x_mm512_set_epu32(UINT32_C( 495870887), UINT32_C(1163009833), UINT32_C( 383591594), UINT32_C(   5081424),
                              UINT32_C(1422501384), UINT32_C(1967885991), UINT32_C(1333283976), UINT32_C( 497965579),
                              UINT32_C( 910061584), UINT32_C(2002226944), UINT32_C( 428480464), UINT32_C(1251597337),
                              UINT32_C( 523093293), UINT32_C(  73585464), UINT32_C(2206005509), UINT32_C( 640416281)) },
    { simde_x_mm512_set_epu32(UINT32_C(2657690668), UINT32_C( 448681074), UINT32_C(1334667053), UINT32_C( 502667641),
                              UINT32_C( 855395764), UINT32_C(2622874348), UINT32_C( 808531712), UINT32_C( 454488139),
                              UINT32_C( 123547093), UINT32_C( 483090439), UINT32_C(3168637539), UINT32_C(3093747107),
                              UINT32_C(4158916667), UINT32_C(4074346392), UINT32_C(1398655610), UINT32_C(1722520923)),
      simde_x_mm512_set_epu32(UINT32_C( 604721400), UINT32_C(1471174399), UINT32_C(2491026588), UINT32_C(2529574367),
                              UINT32_C( 298473775), UINT32_C(2890366559), UINT32_C(3063632375), UINT32_C(4055983958),
                              UINT32_C(4149169500), UINT32_C(4113948134), UINT32_C(2384487126), UINT32_C(2434207126),
                              UINT32_C(3923111671), UINT32_C(3188873807), UINT32_C(1982658188), UINT32_C( 863153207)),
      simde_x_mm512_set_epu32(UINT32_C( 238805068), UINT32_C( 448681074), UINT32_C(1334667053), UINT32_C( 502667641),
                              UINT32_C( 258448214), UINT32_C(2622874348), UINT32_C( 808531712), UINT32_C( 454488139),
                              UINT32_C( 123547093), UINT32_C( 483090439), UINT32_C( 784150413), UINT32_C( 659539981),
                              UINT32_C( 235804996), UINT32_C( 885472585), UINT32_C(1398655610), UINT32_C( 859367716)) },
    { simde_x_mm512_set_epu32(UINT32_C(1463758672), UINT32_C( 602211615), UINT32_C(3830002991), UINT32_C(2864741101),
                              UINT32_C( 797104998), UINT32_C(2737423319), UINT32_C(3342229886), UINT32_C( 178625368),
                              UINT32_C(3091160996), UINT32_C(1095216728), UINT32_C(3079561742), UINT32_C( 430790402),
                              UINT32_C(3213858818), UINT32_C(2113970745), UINT32_C(4112838454), UINT32_C( 564512596)),
      simde_x_mm512_set_epu32(UINT32_C(1997049765), UINT32_C( 505563651), UINT32_C( 463125220), UINT32_C(3843753777),
                              UINT32_C(2346173843), UINT32_C(2157864934), UINT32_C(2591157969), UINT32_C( 389679318),
                              UINT32_C(3939775129), UINT32_C(2493364907), UINT32_C(2006619059), UINT32_C(3391409164),
                              UINT32_C(1533151625), UINT32_C(2122196136), UINT32_C(1690360675), UINT32_C(1484935627)),
      simde_x_mm512_set_epu32(UINT32_C(1463758672), UINT32_C(  96647964), UINT32_C( 125001231), UINT32_C(2864741101),
                              UINT32_C( 797104998), UINT32_C( 579558385), UINT32_C( 751071917), UINT32_C( 178625368),
                              UINT32_C(3091160996), UINT32_C(1095216728), UINT32_C(1072942683), UINT32_C( 430790402),
                              UINT32_C( 147555568), UINT32_C(2113970745), UINT32_C( 732117104), UINT32_C( 564512596)) },
    { simde_x_mm512_set_epu32(UINT32_C( 908815803), UINT32_C(2319376026), UINT32_C(2065037155), UINT32_C( 623932649),
                              UINT32_C(1610322797), UINT32_C(3452844305), UINT32_C(2031682359), UINT32_C(2994836943),
                              UINT32_C(2344919086), UINT32_C( 238137788), UINT32_C(1978166020), UINT32_C(  76768592),
                              UINT32_C(4043825594), UINT32_C(1274901810), UINT32_C( 413860084), UINT32_C( 550494320)),
      simde_x_mm512_set_epu32(UINT32_C(1228958503), UINT32_C(3519587969), UINT32_C(2809504529), UINT32_C(3115789449),
                              UINT32_C(1767270276), UINT32_C( 490610321), UINT32_C(1164436618), UINT32_C(2374669797),
                              UINT32_C(3604002618), UINT32_C(3414719029), UINT32_C(2289333019), UINT32_C(2213872499),
                              UINT32_C(1572579389), UINT32_C(3511888959), UINT32_C(2399346014), UINT32_C(1967093325)),
      simde_x_mm512_set_epu32(UINT32_C( 908815803), UINT32_C(2319376026), UINT32_C(2065037155), UINT32_C( 623932649),
                              UINT32_C(1610322797), UINT32_C(  18572058), UINT32_C( 867245741), UINT32_C( 620167146),
                              UINT32_C(2344919086), UINT32_C( 238137788), UINT32_C(1978166020), UINT32_C(  76768592),
                              UINT32_C( 898666816), UINT32_C(1274901810), UINT32_C( 413860084), UINT32_C( 550494320)) },
    { simde_x_mm512_set_epu32(UINT32_C(1245407235), UINT32_C(4175005098), UINT32_C(2362914327), UINT32_C(2924553042),
                              UINT32_C(2369006988), UINT32_C(2119408419), UINT32_C(3091878410), UINT32_C(3978436943),
                              UINT32_C(1708684203), UINT32_C(1202455481), UINT32_C(2187745469), UINT32_C(3284847806),
                              UINT32_C(3884897233), UINT32_C(2094036024), UINT32_C(2456834182), UINT32_C(  69201629)),
      simde_x_mm512_set_epu32(UINT32_C(3914271744), UINT32_C( 565328458), UINT32_C(4201942548), UINT32_C(1480532604),
                              UINT32_C(4197506536), UINT32_C(3712719696), UINT32_C(3920217826), UINT32_C(1394313506),
                              UINT32_C( 394553965), UINT32_C(2278253176), UINT32_C(1697927724), UINT32_C(2383307765),
                              UINT32_C( 143428987), UINT32_C(3684943081), UINT32_C( 582607980), UINT32_C(1609326889)),
      simde_x_mm512_set_epu32(UINT32_C(1245407235), UINT32_C( 217705892), UINT32_C(2362914327), UINT32_C(1444020438),
                              UINT32_C(2369006988), UINT32_C(2119408419), UINT32_C(3091878410), UINT32_C(1189809931),
                              UINT32_C( 130468343), UINT32_C(1202455481), UINT32_C( 489817745), UINT32_C( 901540041),
                              UINT32_C(  12314584), UINT32_C(2094036024), UINT32_C( 126402262), UINT32_C(  69201629)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_rem_epu32(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u32(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_rem_epu64(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512i a;
    simde__m512i b;
    simde__m512i r;
  } test_vec[8] = {
    { simde_x_mm512_set_epu64(UINT64_C( 2968342496979776051), UINT64_C(10005265515001776413),
                              UINT64_C(16837535683400356038), UINT64_C( 6738163160628300797),
                              UINT64_C(13810255550447513201), UINT64_C( 6479913377553186648),
                              UINT64_C( 7505871096235581515), UINT64_C( 2265477367564496986)),
      simde_x_mm512_set_epu64(UINT64_C(10866939104613927783), UINT64_C(11375825163207743431),
                              UINT64_C(11633520338587575573), UINT64_C(  216242550290965460),
                              UINT64_C( 5120732502404950997), UINT64_C( 5681284513410730040),
                              UINT64_C( 6409558907924801050), UINT64_C( 5372227444888762251)),
      simde_x_mm512_set_epu64(UINT64_C( 2968342496979776051), UINT64_C(10005265515001776413),
                              UINT64_C( 5204015344812780465), UINT64_C(   34644101608371537),
                              UINT64_C( 3568790545637611207), UINT64_C(  798628864142456608),
                              UINT64_C( 1096312188310780465), UINT64_C( 2265477367564496986)) },
    { simde_x_mm512_set_epu64(UINT64_C( 5645659480511055559), UINT64_C(15272728730484288257),
                              UINT64_C(14133460247011230967), UINT64_C(16423537638667915170),
                              UINT64_C( 2118113466433927893), UINT64_C( 3840651400764901876),
                              UINT64_C( 2114726288902596757), UINT64_C( 9482369585348649466)),
      simde_x_mm512_set_epu64(UINT64_C(13555234896536583899), UINT64_C( 7851952110853286921),
                              UINT64_C(17616907291198234572), UINT64_C(16521184395064581900),
                              UINT64_C( 7345032902979795528), UINT64_C(12329133549512917827),
                              UINT64_C( 2328100732832272381), UINT64_C( 4111895855610225675)),
      simde_x_mm512_set_epu64(UINT64_C( 5645659480511055559), UINT64_C( 7420776619631001336),
                              UINT64_C(14133460247011230967), UINT64_C(16423537638667915170),
                              UINT64_C( 2118113466433927893), UINT64_C( 3840651400764901876),
                              UINT64_C( 2114726288902596757), UINT64_C( 1258577874128198116)) },
    { simde_x_mm512_set_epu64(UINT64_C( 7572458917823766705), UINT64_C(12217500042222052906),
                              UINT64_C( 1159256113650983207), UINT64_C( 6193154838246823767),
                              UINT64_C( 7449607714297299576), UINT64_C(14401023659121376347),
                              UINT64_C( 8569312554655704071), UINT64_C(10336200663482757951)),
      simde_x_mm512_set_epu64(UINT64_C(15588592630942564743), UINT64_C( 9028813919053392068),
                              UINT64_C( 5083059030774095197), UINT64_C(13888425720366328200),
                              UINT64_C(16888199589465789243), UINT64_C(18237918400292775569),
                              UINT64_C(12279468594349909724), UINT64_C(13493341674566517412)),
      simde_x_mm512_set_epu64(UINT64_C( 7572458917823766705), UINT64_C( 3188686123168660838),
                              UINT64_C( 1159256113650983207), UINT64_C( 6193154838246823767),
                              UINT64_C( 7449607714297299576), UINT64_C(14401023659121376347),
                              UINT64_C( 8569312554655704071), UINT64_C(10336200663482757951)) },
    { simde_x_mm512_set_epu64(UINT64_C( 2129749246616352421), UINT64_C( 3930946101587052880),
                              UINT64_C( 6109596926925725236), UINT64_C(11931707044738783755),
                              UINT64_C( 3908684742628183808), UINT64_C(15775432521885308750),
                              UINT64_C( 2246668589251707300), UINT64_C( 9474721517893975343)),
      simde_x_mm512_set_epu64(UINT64_C( 5181754748372749246), UINT64_C( 2283432752406648940),
                              UINT64_C(17825612137522679693), UINT64_C( 6205295972918594513),
                              UINT64_C( 7540605987113962845), UINT64_C(13935122940778806069),
                              UINT64_C( 9355601638871447350), UINT64_C(17674380633802211723)),
      simde_x_mm512_set_epu64(UINT64_C( 2129749246616352421), UINT64_C( 1647513349180403940),
                              UINT64_C( 6109596926925725236), UINT64_C( 5726411071820189242),
                              UINT64_C( 3908684742628183808), UINT64_C( 1840309581106502681),
                              UINT64_C( 2246668589251707300), UINT64_C( 9474721517893975343)) },
    { simde_x_mm512_set_epu64(UINT64_C(11414694502393074802), UINT64_C( 5732351344186366329),
                              UINT64_C( 3673896834139808492), UINT64_C( 3472617261273378891),
                              UINT64_C(  530630724433960967), UINT64_C(13609194605976671651),
                              UINT64_C(17862411075628668824), UINT64_C( 6007180105039451483)),
      simde_x_mm512_set_epu64(UINT64_C( 2597258637662508799), UINT64_C(10698877731456040415),
                              UINT64_C( 1281935105229028959), UINT64_C(13158200861647791958),
                              UINT64_C(17820547312174620134), UINT64_C(10241294226337238422),
                              UINT64_C(16849636328689785423), UINT64_C( 8515452077469772855)),
      simde_x_mm512_set_epu64(UINT64_C( 1025659951743039606), UINT64_C( 5732351344186366329),
                              UINT64_C( 1110026623681750574), UINT64_C( 3472617261273378891),
                              UINT64_C(  530630724433960967), UINT64_C( 3367900379639433229),
                              UINT64_C( 1012774746938883401), UINT64_C( 6007180105039451483)) },
    { simde_x_mm512_set_epu64(UINT64_C( 6286795626078602527), UINT64_C(16449737592791923437),
                              UINT64_C( 3423539900625568727), UINT64_C(14354768056262433624),
                              UINT64_C(13276435385586003544), UINT64_C(13226616968333580034),
                              UINT64_C(13803418519385186873), UINT64_C(17664506654225712980)),
      simde_x_mm512_set_epu64(UINT64_C( 8577263429665049091), UINT64_C( 1989107677696558897),
                              UINT64_C(10076739928573503462), UINT64_C(11128938736014461142),
                              UINT64_C(16921205335142546091), UINT64_C( 8618363237326703628),
                              UINT64_C( 6584836091306452136), UINT64_C( 7260043819054420427)),
      simde_x_mm512_set_epu64(UINT64_C( 6286795626078602527), UINT64_C(  536876171219452261),
                              UINT64_C( 3423539900625568727), UINT64_C( 3225829320247972482),
                              UINT64_C(13276435385586003544), UINT64_C( 4608253731006876406),
                              UINT64_C(  633746336772282601), UINT64_C( 3144419016116872126)) },
    { simde_x_mm512_set_epu64(UINT64_C( 3903334154292354714), UINT64_C( 8869267046373815529),
                              UINT64_C( 6916283752571091217), UINT64_C( 8726009290759968207),
                              UINT64_C(10071350786374349244), UINT64_C( 8496158362035250512),
                              UINT64_C(17368098678232675634), UINT64_C( 1777515526450307184)),
      simde_x_mm512_set_epu64(UINT64_C( 5278336582045705857), UINT64_C(12066730073134673033),
                              UINT64_C( 7590368039103504017), UINT64_C( 5001217194949514725),
                              UINT64_C(15479073382423099957), UINT64_C( 9832610448471819123),
                              UINT64_C( 6754177049630551103), UINT64_C(10305112663885051469)),
      simde_x_mm512_set_epu64(UINT64_C( 3903334154292354714), UINT64_C( 8869267046373815529),
                              UINT64_C( 6916283752571091217), UINT64_C( 3724792095810453482),
                              UINT64_C(10071350786374349244), UINT64_C( 8496158362035250512),
                              UINT64_C( 3859744578971573428), UINT64_C( 1777515526450307184)) },
    { simde_x_mm512_set_epu64(UINT64_C( 5348983348701791658), UINT64_C(10148639760639402834),
                              UINT64_C(10174807539574872867), UINT64_C(13279516658136916303),
                              UINT64_C( 7338742772279280569), UINT64_C( 9396295244612029630),
                              UINT64_C(16685506566149927992), UINT64_C(10552022463454113501)),
      simde_x_mm512_set_epu64(UINT64_C(16811669128702212682), UINT64_C(18047205824811442812),
                              UINT64_C(18028153300578966352), UINT64_C(16837207357260532002),
                              UINT64_C( 1694596378460381816), UINT64_C( 7292544047935022069),
                              UINT64_C(  616022812148352233), UINT64_C( 2502282222097948969)),
      simde_x_mm512_set_epu64(UINT64_C( 5348983348701791658), UINT64_C(10148639760639402834),
                              UINT64_C(10174807539574872867), UINT64_C(13279516658136916303),
                              UINT64_C(  560357258437753305), UINT64_C( 2103751196677007561),
                              UINT64_C(   52890638144417701), UINT64_C(  542893575062317625)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512i r = simde_mm512_rem_epu64(test_vec[i].a, test_vec[i].b);
    simde_assert_m512i_u64(r, ==, test_vec[i].r);
  }

  return 0;
}

static int
test_simde_mm512_recip_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -838.19), SIMDE_FLOAT32_C(  -143.82), SIMDE_FLOAT32_C(  -921.01), SIMDE_FLOAT32_C(   206.87),
        SIMDE_FLOAT32_C(  -588.92), SIMDE_FLOAT32_C(  -497.03), SIMDE_FLOAT32_C(  -701.44), SIMDE_FLOAT32_C(  -106.77),
        SIMDE_FLOAT32_C(   464.17), SIMDE_FLOAT32_C(   464.85), SIMDE_FLOAT32_C(   819.12), SIMDE_FLOAT32_C(   908.79),
        SIMDE_FLOAT32_C(   -61.04), SIMDE_FLOAT32_C(   -36.34), SIMDE_FLOAT32_C(   -38.98), SIMDE_FLOAT32_C(  -132.37) },
      { SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.01),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    -0.01) } },
    { { SIMDE_FLOAT32_C(  -324.68), SIMDE_FLOAT32_C(   773.13), SIMDE_FLOAT32_C(  -941.14), SIMDE_FLOAT32_C(   753.16),
        SIMDE_FLOAT32_C(  -838.44), SIMDE_FLOAT32_C(  -965.63), SIMDE_FLOAT32_C(   698.21), SIMDE_FLOAT32_C(  -608.98),
        SIMDE_FLOAT32_C(   -35.12), SIMDE_FLOAT32_C(   227.88), SIMDE_FLOAT32_C(  -531.46), SIMDE_FLOAT32_C(   933.01),
        SIMDE_FLOAT32_C(   160.30), SIMDE_FLOAT32_C(   700.78), SIMDE_FLOAT32_C(  -193.29), SIMDE_FLOAT32_C(   322.12) },
      { SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00),
        SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(  -443.04), SIMDE_FLOAT32_C(  -114.30), SIMDE_FLOAT32_C(  -471.01), SIMDE_FLOAT32_C(   -31.96),
        SIMDE_FLOAT32_C(   388.67), SIMDE_FLOAT32_C(  -172.45), SIMDE_FLOAT32_C(   861.27), SIMDE_FLOAT32_C(  -147.16),
        SIMDE_FLOAT32_C(  -707.59), SIMDE_FLOAT32_C(   680.39), SIMDE_FLOAT32_C(  -238.37), SIMDE_FLOAT32_C(   231.37),
        SIMDE_FLOAT32_C(  -355.96), SIMDE_FLOAT32_C(   722.66), SIMDE_FLOAT32_C(  -901.00), SIMDE_FLOAT32_C(   319.36) },
      { SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.03),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.01),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00) } },
    { { SIMDE_FLOAT32_C(   495.79), SIMDE_FLOAT32_C(  -842.14), SIMDE_FLOAT32_C(    72.53), SIMDE_FLOAT32_C(   657.34),
        SIMDE_FLOAT32_C(  -807.78), SIMDE_FLOAT32_C(  -229.27), SIMDE_FLOAT32_C(  -951.64), SIMDE_FLOAT32_C(   157.10),
        SIMDE_FLOAT32_C(   998.62), SIMDE_FLOAT32_C(  -483.10), SIMDE_FLOAT32_C(    90.12), SIMDE_FLOAT32_C(   158.92),
        SIMDE_FLOAT32_C(  -782.32), SIMDE_FLOAT32_C(   896.82), SIMDE_FLOAT32_C(  -518.96), SIMDE_FLOAT32_C(  -225.36) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.01),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.01),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00) } },
    { { SIMDE_FLOAT32_C(  -217.48), SIMDE_FLOAT32_C(    10.04), SIMDE_FLOAT32_C(   742.68), SIMDE_FLOAT32_C(  -828.81),
        SIMDE_FLOAT32_C(   837.59), SIMDE_FLOAT32_C(   603.95), SIMDE_FLOAT32_C(    24.04), SIMDE_FLOAT32_C(  -870.01),
        SIMDE_FLOAT32_C(   284.34), SIMDE_FLOAT32_C(   785.67), SIMDE_FLOAT32_C(   361.36), SIMDE_FLOAT32_C(   928.38),
        SIMDE_FLOAT32_C(   508.33), SIMDE_FLOAT32_C(   460.36), SIMDE_FLOAT32_C(   247.75), SIMDE_FLOAT32_C(     4.11) },
      { SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.24) } },
    { { SIMDE_FLOAT32_C(   618.21), SIMDE_FLOAT32_C(  -679.72), SIMDE_FLOAT32_C(  -338.54), SIMDE_FLOAT32_C(   810.43),
        SIMDE_FLOAT32_C(    91.01), SIMDE_FLOAT32_C(  -290.18), SIMDE_FLOAT32_C(   -32.46), SIMDE_FLOAT32_C(    89.63),
        SIMDE_FLOAT32_C(   226.71), SIMDE_FLOAT32_C(  -942.35), SIMDE_FLOAT32_C(  -751.45), SIMDE_FLOAT32_C(   444.40),
        SIMDE_FLOAT32_C(   954.48), SIMDE_FLOAT32_C(  -270.41), SIMDE_FLOAT32_C(  -780.96), SIMDE_FLOAT32_C(  -263.00) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.01),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00) } },
    { { SIMDE_FLOAT32_C(   739.63), SIMDE_FLOAT32_C(   961.72), SIMDE_FLOAT32_C(   -91.80), SIMDE_FLOAT32_C(   577.21),
        SIMDE_FLOAT32_C(   565.67), SIMDE_FLOAT32_C(   932.23), SIMDE_FLOAT32_C(   707.21), SIMDE_FLOAT32_C(  -149.99),
        SIMDE_FLOAT32_C(   717.90), SIMDE_FLOAT32_C(    68.56), SIMDE_FLOAT32_C(  -221.60), SIMDE_FLOAT32_C(   226.23),
        SIMDE_FLOAT32_C(  -471.08), SIMDE_FLOAT32_C(  -973.85), SIMDE_FLOAT32_C(  -769.66), SIMDE_FLOAT32_C(  -852.87) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.01),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00) } },
    { { SIMDE_FLOAT32_C(  -653.58), SIMDE_FLOAT32_C(  -108.21), SIMDE_FLOAT32_C(   957.57), SIMDE_FLOAT32_C(   437.43),
        SIMDE_FLOAT32_C(   601.61), SIMDE_FLOAT32_C(   -74.89), SIMDE_FLOAT32_C(  -472.94), SIMDE_FLOAT32_C(  -171.67),
        SIMDE_FLOAT32_C(   -17.24), SIMDE_FLOAT32_C(  -224.39), SIMDE_FLOAT32_C(  -727.28), SIMDE_FLOAT32_C(   -62.76),
        SIMDE_FLOAT32_C(   505.21), SIMDE_FLOAT32_C(  -508.24), SIMDE_FLOAT32_C(   674.24), SIMDE_FLOAT32_C(   244.83) },
      { SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.01),
        SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.02),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_recip_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_recip_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
         { { SIMDE_FLOAT32_C(  -559.02), SIMDE_FLOAT32_C(  -653.98), SIMDE_FLOAT32_C(  -629.51), SIMDE_FLOAT32_C(   712.50),
        SIMDE_FLOAT32_C(   485.85), SIMDE_FLOAT32_C(   827.80), SIMDE_FLOAT32_C(   553.84), SIMDE_FLOAT32_C(  -702.08),
        SIMDE_FLOAT32_C(   943.96), SIMDE_FLOAT32_C(  -619.45), SIMDE_FLOAT32_C(  -617.57), SIMDE_FLOAT32_C(   132.09),
        SIMDE_FLOAT32_C(   914.75), SIMDE_FLOAT32_C(  -571.13), SIMDE_FLOAT32_C(   684.78), SIMDE_FLOAT32_C(   888.84) },
      UINT8_C( 30),
      { SIMDE_FLOAT32_C(   989.94), SIMDE_FLOAT32_C(   139.65), SIMDE_FLOAT32_C(   430.34), SIMDE_FLOAT32_C(   509.85),
        SIMDE_FLOAT32_C(  -762.94), SIMDE_FLOAT32_C(  -610.66), SIMDE_FLOAT32_C(  -278.26), SIMDE_FLOAT32_C(   571.59),
        SIMDE_FLOAT32_C(  -698.60), SIMDE_FLOAT32_C(    66.97), SIMDE_FLOAT32_C(   404.01), SIMDE_FLOAT32_C(  -382.91),
        SIMDE_FLOAT32_C(  -808.74), SIMDE_FLOAT32_C(   383.72), SIMDE_FLOAT32_C(    58.06), SIMDE_FLOAT32_C(  -462.73) },
      { SIMDE_FLOAT32_C(  -559.02), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(   827.80), SIMDE_FLOAT32_C(   553.84), SIMDE_FLOAT32_C(  -702.08),
        SIMDE_FLOAT32_C(   943.96), SIMDE_FLOAT32_C(  -619.45), SIMDE_FLOAT32_C(  -617.57), SIMDE_FLOAT32_C(   132.09),
        SIMDE_FLOAT32_C(   914.75), SIMDE_FLOAT32_C(  -571.13), SIMDE_FLOAT32_C(   684.78), SIMDE_FLOAT32_C(   888.84) } },
    { { SIMDE_FLOAT32_C(   754.21), SIMDE_FLOAT32_C(  -229.44), SIMDE_FLOAT32_C(  -976.87), SIMDE_FLOAT32_C(   582.01),
        SIMDE_FLOAT32_C(  -675.60), SIMDE_FLOAT32_C(  -678.95), SIMDE_FLOAT32_C(   525.97), SIMDE_FLOAT32_C(  -295.05),
        SIMDE_FLOAT32_C(  -296.52), SIMDE_FLOAT32_C(  -341.94), SIMDE_FLOAT32_C(  -380.30), SIMDE_FLOAT32_C(   132.35),
        SIMDE_FLOAT32_C(  -657.15), SIMDE_FLOAT32_C(  -491.46), SIMDE_FLOAT32_C(    10.23), SIMDE_FLOAT32_C(  -667.22) },
      UINT8_C(254),
      { SIMDE_FLOAT32_C(  -559.43), SIMDE_FLOAT32_C(   842.63), SIMDE_FLOAT32_C(   885.25), SIMDE_FLOAT32_C(  -170.09),
        SIMDE_FLOAT32_C(  -435.64), SIMDE_FLOAT32_C(   456.84), SIMDE_FLOAT32_C(   131.32), SIMDE_FLOAT32_C(   631.33),
        SIMDE_FLOAT32_C(  -139.15), SIMDE_FLOAT32_C(   748.40), SIMDE_FLOAT32_C(   822.59), SIMDE_FLOAT32_C(  -755.43),
        SIMDE_FLOAT32_C(  -193.54), SIMDE_FLOAT32_C(  -640.14), SIMDE_FLOAT32_C(   998.78), SIMDE_FLOAT32_C(   577.02) },
      { SIMDE_FLOAT32_C(   754.21), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.01),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(  -296.52), SIMDE_FLOAT32_C(  -341.94), SIMDE_FLOAT32_C(  -380.30), SIMDE_FLOAT32_C(   132.35),
        SIMDE_FLOAT32_C(  -657.15), SIMDE_FLOAT32_C(  -491.46), SIMDE_FLOAT32_C(    10.23), SIMDE_FLOAT32_C(  -667.22) } },
    { { SIMDE_FLOAT32_C(  -617.01), SIMDE_FLOAT32_C(   580.79), SIMDE_FLOAT32_C(   901.43), SIMDE_FLOAT32_C(  -295.96),
        SIMDE_FLOAT32_C(   106.76), SIMDE_FLOAT32_C(  -393.62), SIMDE_FLOAT32_C(   407.52), SIMDE_FLOAT32_C(   764.82),
        SIMDE_FLOAT32_C(   226.07), SIMDE_FLOAT32_C(  -460.13), SIMDE_FLOAT32_C(  -892.33), SIMDE_FLOAT32_C(   734.61),
        SIMDE_FLOAT32_C(   550.10), SIMDE_FLOAT32_C(  -559.55), SIMDE_FLOAT32_C(   382.81), SIMDE_FLOAT32_C(   990.67) },
      UINT8_C( 97),
      { SIMDE_FLOAT32_C(   268.05), SIMDE_FLOAT32_C(  -179.42), SIMDE_FLOAT32_C(  -152.56), SIMDE_FLOAT32_C(  -275.11),
        SIMDE_FLOAT32_C(   951.90), SIMDE_FLOAT32_C(  -521.22), SIMDE_FLOAT32_C(   585.74), SIMDE_FLOAT32_C(   700.30),
        SIMDE_FLOAT32_C(  -698.63), SIMDE_FLOAT32_C(   830.31), SIMDE_FLOAT32_C(  -493.24), SIMDE_FLOAT32_C(  -338.77),
        SIMDE_FLOAT32_C(   829.08), SIMDE_FLOAT32_C(  -916.21), SIMDE_FLOAT32_C(    44.23), SIMDE_FLOAT32_C(   409.87) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   580.79), SIMDE_FLOAT32_C(   901.43), SIMDE_FLOAT32_C(  -295.96),
        SIMDE_FLOAT32_C(   106.76), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   764.82),
        SIMDE_FLOAT32_C(   226.07), SIMDE_FLOAT32_C(  -460.13), SIMDE_FLOAT32_C(  -892.33), SIMDE_FLOAT32_C(   734.61),
        SIMDE_FLOAT32_C(   550.10), SIMDE_FLOAT32_C(  -559.55), SIMDE_FLOAT32_C(   382.81), SIMDE_FLOAT32_C(   990.67) } },
    { { SIMDE_FLOAT32_C(   985.22), SIMDE_FLOAT32_C(   748.27), SIMDE_FLOAT32_C(  -483.37), SIMDE_FLOAT32_C(  -408.41),
        SIMDE_FLOAT32_C(   155.79), SIMDE_FLOAT32_C(  -718.54), SIMDE_FLOAT32_C(   817.67), SIMDE_FLOAT32_C(   695.66),
        SIMDE_FLOAT32_C(  -610.87), SIMDE_FLOAT32_C(   552.28), SIMDE_FLOAT32_C(   245.77), SIMDE_FLOAT32_C(  -170.42),
        SIMDE_FLOAT32_C(   -64.91), SIMDE_FLOAT32_C(   236.44), SIMDE_FLOAT32_C(   112.66), SIMDE_FLOAT32_C(  -796.86) },
      UINT8_C(153),
      { SIMDE_FLOAT32_C(   960.10), SIMDE_FLOAT32_C(   -71.97), SIMDE_FLOAT32_C(  -991.08), SIMDE_FLOAT32_C(  -561.12),
        SIMDE_FLOAT32_C(  -486.23), SIMDE_FLOAT32_C(   709.22), SIMDE_FLOAT32_C(  -259.75), SIMDE_FLOAT32_C(  -655.92),
        SIMDE_FLOAT32_C(  -784.01), SIMDE_FLOAT32_C(   401.48), SIMDE_FLOAT32_C(  -826.84), SIMDE_FLOAT32_C(  -700.22),
        SIMDE_FLOAT32_C(  -554.30), SIMDE_FLOAT32_C(   583.03), SIMDE_FLOAT32_C(  -715.01), SIMDE_FLOAT32_C(  -806.03) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(   748.27), SIMDE_FLOAT32_C(  -483.37), SIMDE_FLOAT32_C(    -0.00),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(  -718.54), SIMDE_FLOAT32_C(   817.67), SIMDE_FLOAT32_C(    -0.00),
        SIMDE_FLOAT32_C(  -610.87), SIMDE_FLOAT32_C(   552.28), SIMDE_FLOAT32_C(   245.77), SIMDE_FLOAT32_C(  -170.42),
        SIMDE_FLOAT32_C(   -64.91), SIMDE_FLOAT32_C(   236.44), SIMDE_FLOAT32_C(   112.66), SIMDE_FLOAT32_C(  -796.86) } },
    { { SIMDE_FLOAT32_C(  -900.34), SIMDE_FLOAT32_C(  -123.41), SIMDE_FLOAT32_C(   349.77), SIMDE_FLOAT32_C(  -618.88),
        SIMDE_FLOAT32_C(  -305.75), SIMDE_FLOAT32_C(    45.43), SIMDE_FLOAT32_C(  -229.75), SIMDE_FLOAT32_C(  -753.47),
        SIMDE_FLOAT32_C(  -708.80), SIMDE_FLOAT32_C(   599.82), SIMDE_FLOAT32_C(   181.62), SIMDE_FLOAT32_C(   527.63),
        SIMDE_FLOAT32_C(  -287.52), SIMDE_FLOAT32_C(   384.76), SIMDE_FLOAT32_C(   584.65), SIMDE_FLOAT32_C(  -327.41) },
      UINT8_C( 60),
      { SIMDE_FLOAT32_C(   593.57), SIMDE_FLOAT32_C(   111.46), SIMDE_FLOAT32_C(  -173.43), SIMDE_FLOAT32_C(   302.80),
        SIMDE_FLOAT32_C(   851.71), SIMDE_FLOAT32_C(   170.65), SIMDE_FLOAT32_C(   518.78), SIMDE_FLOAT32_C(   253.19),
        SIMDE_FLOAT32_C(   343.82), SIMDE_FLOAT32_C(   818.56), SIMDE_FLOAT32_C(   698.89), SIMDE_FLOAT32_C(   -73.15),
        SIMDE_FLOAT32_C(  -896.45), SIMDE_FLOAT32_C(   892.87), SIMDE_FLOAT32_C(    26.51), SIMDE_FLOAT32_C(   -19.86) },
      { SIMDE_FLOAT32_C(  -900.34), SIMDE_FLOAT32_C(  -123.41), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(  -229.75), SIMDE_FLOAT32_C(  -753.47),
        SIMDE_FLOAT32_C(  -708.80), SIMDE_FLOAT32_C(   599.82), SIMDE_FLOAT32_C(   181.62), SIMDE_FLOAT32_C(   527.63),
        SIMDE_FLOAT32_C(  -287.52), SIMDE_FLOAT32_C(   384.76), SIMDE_FLOAT32_C(   584.65), SIMDE_FLOAT32_C(  -327.41) } },
    { { SIMDE_FLOAT32_C(   242.63), SIMDE_FLOAT32_C(   407.63), SIMDE_FLOAT32_C(   674.39), SIMDE_FLOAT32_C(  -711.94),
        SIMDE_FLOAT32_C(  -822.12), SIMDE_FLOAT32_C(   920.93), SIMDE_FLOAT32_C(  -420.74), SIMDE_FLOAT32_C(   777.70),
        SIMDE_FLOAT32_C(   102.55), SIMDE_FLOAT32_C(  -893.11), SIMDE_FLOAT32_C(  -509.82), SIMDE_FLOAT32_C(  -512.69),
        SIMDE_FLOAT32_C(   691.54), SIMDE_FLOAT32_C(   162.77), SIMDE_FLOAT32_C(  -199.89), SIMDE_FLOAT32_C(   285.12) },
      UINT8_C( 58),
      { SIMDE_FLOAT32_C(   626.68), SIMDE_FLOAT32_C(  -412.08), SIMDE_FLOAT32_C(  -874.05), SIMDE_FLOAT32_C(  -202.66),
        SIMDE_FLOAT32_C(  -893.30), SIMDE_FLOAT32_C(   379.14), SIMDE_FLOAT32_C(  -858.85), SIMDE_FLOAT32_C(   925.26),
        SIMDE_FLOAT32_C(    78.03), SIMDE_FLOAT32_C(    68.00), SIMDE_FLOAT32_C(  -971.19), SIMDE_FLOAT32_C(   -29.10),
        SIMDE_FLOAT32_C(  -905.49), SIMDE_FLOAT32_C(     8.95), SIMDE_FLOAT32_C(  -786.47), SIMDE_FLOAT32_C(   502.14) },
      { SIMDE_FLOAT32_C(   242.63), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(   674.39), SIMDE_FLOAT32_C(    -0.00),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  -420.74), SIMDE_FLOAT32_C(   777.70),
        SIMDE_FLOAT32_C(   102.55), SIMDE_FLOAT32_C(  -893.11), SIMDE_FLOAT32_C(  -509.82), SIMDE_FLOAT32_C(  -512.69),
        SIMDE_FLOAT32_C(   691.54), SIMDE_FLOAT32_C(   162.77), SIMDE_FLOAT32_C(  -199.89), SIMDE_FLOAT32_C(   285.12) } },
    { { SIMDE_FLOAT32_C(  -316.66), SIMDE_FLOAT32_C(  -498.40), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(  -395.73),
        SIMDE_FLOAT32_C(    80.86), SIMDE_FLOAT32_C(   457.72), SIMDE_FLOAT32_C(   706.82), SIMDE_FLOAT32_C(   187.75),
        SIMDE_FLOAT32_C(   947.90), SIMDE_FLOAT32_C(  -805.87), SIMDE_FLOAT32_C(  -120.71), SIMDE_FLOAT32_C(   110.67),
        SIMDE_FLOAT32_C(    -5.76), SIMDE_FLOAT32_C(  -835.59), SIMDE_FLOAT32_C(   384.91), SIMDE_FLOAT32_C(  -379.07) },
      UINT8_C(169),
      { SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(   418.26), SIMDE_FLOAT32_C(  -140.98), SIMDE_FLOAT32_C(  -110.01),
        SIMDE_FLOAT32_C(   559.41), SIMDE_FLOAT32_C(  -215.72), SIMDE_FLOAT32_C(   968.02), SIMDE_FLOAT32_C(  -372.59),
        SIMDE_FLOAT32_C(  -186.90), SIMDE_FLOAT32_C(   -61.08), SIMDE_FLOAT32_C(  -278.08), SIMDE_FLOAT32_C(   822.05),
        SIMDE_FLOAT32_C(   152.45), SIMDE_FLOAT32_C(  -775.94), SIMDE_FLOAT32_C(  -494.61), SIMDE_FLOAT32_C(   654.05) },
      { SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(  -498.40), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(    -0.01),
        SIMDE_FLOAT32_C(    80.86), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(   706.82), SIMDE_FLOAT32_C(    -0.00),
        SIMDE_FLOAT32_C(   947.90), SIMDE_FLOAT32_C(  -805.87), SIMDE_FLOAT32_C(  -120.71), SIMDE_FLOAT32_C(   110.67),
        SIMDE_FLOAT32_C(    -5.76), SIMDE_FLOAT32_C(  -835.59), SIMDE_FLOAT32_C(   384.91), SIMDE_FLOAT32_C(  -379.07) } },
    { { SIMDE_FLOAT32_C(   904.08), SIMDE_FLOAT32_C(   109.66), SIMDE_FLOAT32_C(  -265.09), SIMDE_FLOAT32_C(   361.80),
        SIMDE_FLOAT32_C(  -183.52), SIMDE_FLOAT32_C(   922.65), SIMDE_FLOAT32_C(   309.70), SIMDE_FLOAT32_C(    10.61),
        SIMDE_FLOAT32_C(  -198.06), SIMDE_FLOAT32_C(  -579.63), SIMDE_FLOAT32_C(  -995.15), SIMDE_FLOAT32_C(   -33.65),
        SIMDE_FLOAT32_C(   805.28), SIMDE_FLOAT32_C(  -374.23), SIMDE_FLOAT32_C(   718.68), SIMDE_FLOAT32_C(   316.13) },
      UINT8_C(232),
      { SIMDE_FLOAT32_C(  -422.30), SIMDE_FLOAT32_C(  -793.87), SIMDE_FLOAT32_C(   603.45), SIMDE_FLOAT32_C(   361.98),
        SIMDE_FLOAT32_C(  -825.85), SIMDE_FLOAT32_C(  -769.14), SIMDE_FLOAT32_C(  -824.92), SIMDE_FLOAT32_C(   113.07),
        SIMDE_FLOAT32_C(   -47.22), SIMDE_FLOAT32_C(   997.13), SIMDE_FLOAT32_C(  -734.48), SIMDE_FLOAT32_C(   176.84),
        SIMDE_FLOAT32_C(  -497.48), SIMDE_FLOAT32_C(   919.57), SIMDE_FLOAT32_C(    80.93), SIMDE_FLOAT32_C(   612.18) },
      { SIMDE_FLOAT32_C(   904.08), SIMDE_FLOAT32_C(   109.66), SIMDE_FLOAT32_C(  -265.09), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(  -183.52), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.01),
        SIMDE_FLOAT32_C(  -198.06), SIMDE_FLOAT32_C(  -579.63), SIMDE_FLOAT32_C(  -995.15), SIMDE_FLOAT32_C(   -33.65),
        SIMDE_FLOAT32_C(   805.28), SIMDE_FLOAT32_C(  -374.23), SIMDE_FLOAT32_C(   718.68), SIMDE_FLOAT32_C(   316.13) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_recip_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_recip_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   428.72), SIMDE_FLOAT64_C(  -458.86), SIMDE_FLOAT64_C(   806.54), SIMDE_FLOAT64_C(   539.23),
        SIMDE_FLOAT64_C(  -146.88), SIMDE_FLOAT64_C(   637.59), SIMDE_FLOAT64_C(   196.11), SIMDE_FLOAT64_C(  -116.19) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(    -0.01) } },
    { { SIMDE_FLOAT64_C(   736.77), SIMDE_FLOAT64_C(  -342.16), SIMDE_FLOAT64_C(  -904.30), SIMDE_FLOAT64_C(   476.08),
        SIMDE_FLOAT64_C(   944.13), SIMDE_FLOAT64_C(   149.78), SIMDE_FLOAT64_C(  -235.14), SIMDE_FLOAT64_C(   736.57) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -510.10), SIMDE_FLOAT64_C(   107.44), SIMDE_FLOAT64_C(  -102.43), SIMDE_FLOAT64_C(   808.81),
        SIMDE_FLOAT64_C(   777.98), SIMDE_FLOAT64_C(  -457.12), SIMDE_FLOAT64_C(  -403.55), SIMDE_FLOAT64_C(  -682.37) },
      { SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00) } },
    { { SIMDE_FLOAT64_C(  -420.25), SIMDE_FLOAT64_C(   346.45), SIMDE_FLOAT64_C(   923.73), SIMDE_FLOAT64_C(  -651.25),
        SIMDE_FLOAT64_C(   204.13), SIMDE_FLOAT64_C(   115.66), SIMDE_FLOAT64_C(  -627.27), SIMDE_FLOAT64_C(  -367.15) },
      { SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00) } },
    { { SIMDE_FLOAT64_C(   656.80), SIMDE_FLOAT64_C(  -820.73), SIMDE_FLOAT64_C(  -827.92), SIMDE_FLOAT64_C(  -490.07),
        SIMDE_FLOAT64_C(   816.86), SIMDE_FLOAT64_C(   368.19), SIMDE_FLOAT64_C(   393.74), SIMDE_FLOAT64_C(   553.62) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -973.97), SIMDE_FLOAT64_C(   489.44), SIMDE_FLOAT64_C(    29.71), SIMDE_FLOAT64_C(   970.16),
        SIMDE_FLOAT64_C(  -360.78), SIMDE_FLOAT64_C(   794.57), SIMDE_FLOAT64_C(   706.74), SIMDE_FLOAT64_C(   129.11) },
      { SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.03), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.01) } },
    { { SIMDE_FLOAT64_C(   -97.99), SIMDE_FLOAT64_C(  -395.69), SIMDE_FLOAT64_C(   -62.07), SIMDE_FLOAT64_C(  -320.01),
        SIMDE_FLOAT64_C(   147.19), SIMDE_FLOAT64_C(   534.38), SIMDE_FLOAT64_C(    -2.39), SIMDE_FLOAT64_C(   726.95) },
      { SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.02), SIMDE_FLOAT64_C(    -0.00),
        SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -0.42), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(  -119.17), SIMDE_FLOAT64_C(   -78.65), SIMDE_FLOAT64_C(  -924.30), SIMDE_FLOAT64_C(  -915.04),
        SIMDE_FLOAT64_C(  -962.99), SIMDE_FLOAT64_C(  -551.57), SIMDE_FLOAT64_C(  -282.19), SIMDE_FLOAT64_C(   693.81) },
      { SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00),
        SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_recip_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_recip_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   209.25), SIMDE_FLOAT64_C(  -726.84), SIMDE_FLOAT64_C(  -123.44), SIMDE_FLOAT64_C(   592.78),
        SIMDE_FLOAT64_C(  -139.26), SIMDE_FLOAT64_C(  -313.25), SIMDE_FLOAT64_C(   562.79), SIMDE_FLOAT64_C(  -134.44) },
      UINT8_C(203),
      { SIMDE_FLOAT64_C(   624.55), SIMDE_FLOAT64_C(  -863.70), SIMDE_FLOAT64_C(   788.13), SIMDE_FLOAT64_C(   415.51),
        SIMDE_FLOAT64_C(  -772.51), SIMDE_FLOAT64_C(  -934.49), SIMDE_FLOAT64_C(  -140.87), SIMDE_FLOAT64_C(  -265.50) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(  -123.44), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(  -139.26), SIMDE_FLOAT64_C(  -313.25), SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(    -0.00) } },
    { { SIMDE_FLOAT64_C(   420.64), SIMDE_FLOAT64_C(  -690.14), SIMDE_FLOAT64_C(   -96.93), SIMDE_FLOAT64_C(  -275.78),
        SIMDE_FLOAT64_C(  -453.21), SIMDE_FLOAT64_C(   875.20), SIMDE_FLOAT64_C(   895.34), SIMDE_FLOAT64_C(  -766.82) },
      UINT8_C(181),
      { SIMDE_FLOAT64_C(   503.15), SIMDE_FLOAT64_C(   966.97), SIMDE_FLOAT64_C(   164.84), SIMDE_FLOAT64_C(  -672.96),
        SIMDE_FLOAT64_C(   332.40), SIMDE_FLOAT64_C(  -625.91), SIMDE_FLOAT64_C(  -399.81), SIMDE_FLOAT64_C(  -791.04) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -690.14), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(  -275.78),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(   895.34), SIMDE_FLOAT64_C(    -0.00) } },
    { { SIMDE_FLOAT64_C(   966.87), SIMDE_FLOAT64_C(   460.94), SIMDE_FLOAT64_C(  -104.29), SIMDE_FLOAT64_C(   529.67),
        SIMDE_FLOAT64_C(  -673.50), SIMDE_FLOAT64_C(   637.76), SIMDE_FLOAT64_C(   154.22), SIMDE_FLOAT64_C(  -537.20) },
      UINT8_C( 88),
      { SIMDE_FLOAT64_C(  -430.27), SIMDE_FLOAT64_C(  -309.71), SIMDE_FLOAT64_C(   491.40), SIMDE_FLOAT64_C(   428.86),
        SIMDE_FLOAT64_C(   424.79), SIMDE_FLOAT64_C(   -87.96), SIMDE_FLOAT64_C(   738.72), SIMDE_FLOAT64_C(  -672.13) },
      { SIMDE_FLOAT64_C(   966.87), SIMDE_FLOAT64_C(   460.94), SIMDE_FLOAT64_C(  -104.29), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   637.76), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -537.20) } },
    { { SIMDE_FLOAT64_C(   636.26), SIMDE_FLOAT64_C(  -714.50), SIMDE_FLOAT64_C(  -796.93), SIMDE_FLOAT64_C(   531.61),
        SIMDE_FLOAT64_C(  -481.32), SIMDE_FLOAT64_C(  -374.02), SIMDE_FLOAT64_C(    34.75), SIMDE_FLOAT64_C(  -514.35) },
      UINT8_C(120),
      { SIMDE_FLOAT64_C(   361.79), SIMDE_FLOAT64_C(   818.05), SIMDE_FLOAT64_C(  -835.08), SIMDE_FLOAT64_C(   961.98),
        SIMDE_FLOAT64_C(  -973.00), SIMDE_FLOAT64_C(  -868.21), SIMDE_FLOAT64_C(   422.92), SIMDE_FLOAT64_C(   -77.29) },
      { SIMDE_FLOAT64_C(   636.26), SIMDE_FLOAT64_C(  -714.50), SIMDE_FLOAT64_C(  -796.93), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -514.35) } },
    { { SIMDE_FLOAT64_C(   661.46), SIMDE_FLOAT64_C(   749.42), SIMDE_FLOAT64_C(  -439.53), SIMDE_FLOAT64_C(  -184.33),
        SIMDE_FLOAT64_C(  -787.78), SIMDE_FLOAT64_C(   986.36), SIMDE_FLOAT64_C(   385.40), SIMDE_FLOAT64_C(   -97.48) },
      UINT8_C(166),
      { SIMDE_FLOAT64_C(  -185.74), SIMDE_FLOAT64_C(  -672.69), SIMDE_FLOAT64_C(  -610.20), SIMDE_FLOAT64_C(  -447.03),
        SIMDE_FLOAT64_C(  -344.82), SIMDE_FLOAT64_C(  -973.94), SIMDE_FLOAT64_C(  -161.52), SIMDE_FLOAT64_C(  -141.75) },
      { SIMDE_FLOAT64_C(   661.46), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(  -184.33),
        SIMDE_FLOAT64_C(  -787.78), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(   385.40), SIMDE_FLOAT64_C(    -0.01) } },
    { { SIMDE_FLOAT64_C(   557.67), SIMDE_FLOAT64_C(   357.15), SIMDE_FLOAT64_C(   484.23), SIMDE_FLOAT64_C(  -407.58),
        SIMDE_FLOAT64_C(   842.80), SIMDE_FLOAT64_C(   275.05), SIMDE_FLOAT64_C(   954.21), SIMDE_FLOAT64_C(   660.85) },
      UINT8_C( 53),
      { SIMDE_FLOAT64_C(   916.20), SIMDE_FLOAT64_C(   687.85), SIMDE_FLOAT64_C(   571.76), SIMDE_FLOAT64_C(   339.11),
        SIMDE_FLOAT64_C(  -389.44), SIMDE_FLOAT64_C(   233.22), SIMDE_FLOAT64_C(    88.53), SIMDE_FLOAT64_C(   171.03) },
      { SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   357.15), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(  -407.58),
        SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(   954.21), SIMDE_FLOAT64_C(   660.85) } },
    { { SIMDE_FLOAT64_C(  -951.11), SIMDE_FLOAT64_C(   300.76), SIMDE_FLOAT64_C(   157.39), SIMDE_FLOAT64_C(   434.29),
        SIMDE_FLOAT64_C(  -796.73), SIMDE_FLOAT64_C(  -364.85), SIMDE_FLOAT64_C(  -751.45), SIMDE_FLOAT64_C(  -469.41) },
      UINT8_C(211),
      { SIMDE_FLOAT64_C(  -198.47), SIMDE_FLOAT64_C(   185.77), SIMDE_FLOAT64_C(    51.02), SIMDE_FLOAT64_C(   640.00),
        SIMDE_FLOAT64_C(  -955.99), SIMDE_FLOAT64_C(  -391.31), SIMDE_FLOAT64_C(    -2.84), SIMDE_FLOAT64_C(   528.24) },
      { SIMDE_FLOAT64_C(    -0.01), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(   157.39), SIMDE_FLOAT64_C(   434.29),
        SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(  -364.85), SIMDE_FLOAT64_C(    -0.35), SIMDE_FLOAT64_C(     0.00) } },
    { { SIMDE_FLOAT64_C(   201.11), SIMDE_FLOAT64_C(  -160.04), SIMDE_FLOAT64_C(  -196.70), SIMDE_FLOAT64_C(   155.32),
        SIMDE_FLOAT64_C(  -499.19), SIMDE_FLOAT64_C(  -756.73), SIMDE_FLOAT64_C(    71.52), SIMDE_FLOAT64_C(  -811.33) },
      UINT8_C(173),
      { SIMDE_FLOAT64_C(  -589.37), SIMDE_FLOAT64_C(  -200.77), SIMDE_FLOAT64_C(    48.24), SIMDE_FLOAT64_C(   499.16),
        SIMDE_FLOAT64_C(   970.26), SIMDE_FLOAT64_C(    97.13), SIMDE_FLOAT64_C(  -200.08), SIMDE_FLOAT64_C(   127.65) },
      { SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(  -160.04), SIMDE_FLOAT64_C(     0.02), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(  -499.19), SIMDE_FLOAT64_C(     0.01), SIMDE_FLOAT64_C(    71.52), SIMDE_FLOAT64_C(     0.01) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_recip_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_rint_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -665.69), SIMDE_FLOAT32_C(  -529.73), SIMDE_FLOAT32_C(  -462.47), SIMDE_FLOAT32_C(   909.14),
        SIMDE_FLOAT32_C(   211.54), SIMDE_FLOAT32_C(    67.95), SIMDE_FLOAT32_C(   -26.51), SIMDE_FLOAT32_C(  -276.52),
        SIMDE_FLOAT32_C(   812.99), SIMDE_FLOAT32_C(   513.31), SIMDE_FLOAT32_C(  -214.67), SIMDE_FLOAT32_C(   502.05),
        SIMDE_FLOAT32_C(    96.51), SIMDE_FLOAT32_C(  -399.31), SIMDE_FLOAT32_C(   783.78), SIMDE_FLOAT32_C(   -69.17) },
      { SIMDE_FLOAT32_C(  -666.00), SIMDE_FLOAT32_C(  -530.00), SIMDE_FLOAT32_C(  -462.00), SIMDE_FLOAT32_C(   909.00),
        SIMDE_FLOAT32_C(   212.00), SIMDE_FLOAT32_C(    68.00), SIMDE_FLOAT32_C(   -27.00), SIMDE_FLOAT32_C(  -277.00),
        SIMDE_FLOAT32_C(   813.00), SIMDE_FLOAT32_C(   513.00), SIMDE_FLOAT32_C(  -215.00), SIMDE_FLOAT32_C(   502.00),
        SIMDE_FLOAT32_C(    97.00), SIMDE_FLOAT32_C(  -399.00), SIMDE_FLOAT32_C(   784.00), SIMDE_FLOAT32_C(   -69.00) } },
    { { SIMDE_FLOAT32_C(  -445.96), SIMDE_FLOAT32_C(   637.70), SIMDE_FLOAT32_C(   890.97), SIMDE_FLOAT32_C(  -578.19),
        SIMDE_FLOAT32_C(   730.74), SIMDE_FLOAT32_C(  -499.66), SIMDE_FLOAT32_C(  -463.47), SIMDE_FLOAT32_C(   -93.74),
        SIMDE_FLOAT32_C(  -617.08), SIMDE_FLOAT32_C(  -340.40), SIMDE_FLOAT32_C(  -933.85), SIMDE_FLOAT32_C(   901.57),
        SIMDE_FLOAT32_C(   629.93), SIMDE_FLOAT32_C(   901.12), SIMDE_FLOAT32_C(   755.15), SIMDE_FLOAT32_C(   964.24) },
      { SIMDE_FLOAT32_C(  -446.00), SIMDE_FLOAT32_C(   638.00), SIMDE_FLOAT32_C(   891.00), SIMDE_FLOAT32_C(  -578.00),
        SIMDE_FLOAT32_C(   731.00), SIMDE_FLOAT32_C(  -500.00), SIMDE_FLOAT32_C(  -463.00), SIMDE_FLOAT32_C(   -94.00),
        SIMDE_FLOAT32_C(  -617.00), SIMDE_FLOAT32_C(  -340.00), SIMDE_FLOAT32_C(  -934.00), SIMDE_FLOAT32_C(   902.00),
        SIMDE_FLOAT32_C(   630.00), SIMDE_FLOAT32_C(   901.00), SIMDE_FLOAT32_C(   755.00), SIMDE_FLOAT32_C(   964.00) } },
    { { SIMDE_FLOAT32_C(  -628.61), SIMDE_FLOAT32_C(  -707.33), SIMDE_FLOAT32_C(   873.38), SIMDE_FLOAT32_C(   582.93),
        SIMDE_FLOAT32_C(   360.62), SIMDE_FLOAT32_C(  -153.12), SIMDE_FLOAT32_C(  -693.59), SIMDE_FLOAT32_C(   173.61),
        SIMDE_FLOAT32_C(  -639.82), SIMDE_FLOAT32_C(    91.74), SIMDE_FLOAT32_C(  -324.34), SIMDE_FLOAT32_C(   456.69),
        SIMDE_FLOAT32_C(   692.43), SIMDE_FLOAT32_C(  -540.56), SIMDE_FLOAT32_C(  -612.48), SIMDE_FLOAT32_C(  -753.53) },
      { SIMDE_FLOAT32_C(  -629.00), SIMDE_FLOAT32_C(  -707.00), SIMDE_FLOAT32_C(   873.00), SIMDE_FLOAT32_C(   583.00),
        SIMDE_FLOAT32_C(   361.00), SIMDE_FLOAT32_C(  -153.00), SIMDE_FLOAT32_C(  -694.00), SIMDE_FLOAT32_C(   174.00),
        SIMDE_FLOAT32_C(  -640.00), SIMDE_FLOAT32_C(    92.00), SIMDE_FLOAT32_C(  -324.00), SIMDE_FLOAT32_C(   457.00),
        SIMDE_FLOAT32_C(   692.00), SIMDE_FLOAT32_C(  -541.00), SIMDE_FLOAT32_C(  -612.00), SIMDE_FLOAT32_C(  -754.00) } },
    { { SIMDE_FLOAT32_C(  -902.86), SIMDE_FLOAT32_C(  -721.51), SIMDE_FLOAT32_C(  -331.72), SIMDE_FLOAT32_C(   827.88),
        SIMDE_FLOAT32_C(  -221.17), SIMDE_FLOAT32_C(   204.81), SIMDE_FLOAT32_C(  -265.86), SIMDE_FLOAT32_C(   161.75),
        SIMDE_FLOAT32_C(   864.41), SIMDE_FLOAT32_C(  -199.71), SIMDE_FLOAT32_C(    63.32), SIMDE_FLOAT32_C(   494.34),
        SIMDE_FLOAT32_C(  -298.59), SIMDE_FLOAT32_C(  -181.53), SIMDE_FLOAT32_C(   458.58), SIMDE_FLOAT32_C(    72.80) },
      { SIMDE_FLOAT32_C(  -903.00), SIMDE_FLOAT32_C(  -722.00), SIMDE_FLOAT32_C(  -332.00), SIMDE_FLOAT32_C(   828.00),
        SIMDE_FLOAT32_C(  -221.00), SIMDE_FLOAT32_C(   205.00), SIMDE_FLOAT32_C(  -266.00), SIMDE_FLOAT32_C(   162.00),
        SIMDE_FLOAT32_C(   864.00), SIMDE_FLOAT32_C(  -200.00), SIMDE_FLOAT32_C(    63.00), SIMDE_FLOAT32_C(   494.00),
        SIMDE_FLOAT32_C(  -299.00), SIMDE_FLOAT32_C(  -182.00), SIMDE_FLOAT32_C(   459.00), SIMDE_FLOAT32_C(    73.00) } },
    { { SIMDE_FLOAT32_C(   111.14), SIMDE_FLOAT32_C(   331.96), SIMDE_FLOAT32_C(  -344.27), SIMDE_FLOAT32_C(  -528.24),
        SIMDE_FLOAT32_C(  -821.17), SIMDE_FLOAT32_C(   -37.86), SIMDE_FLOAT32_C(   645.37), SIMDE_FLOAT32_C(  -460.98),
        SIMDE_FLOAT32_C(  -946.11), SIMDE_FLOAT32_C(  -678.97), SIMDE_FLOAT32_C(   995.71), SIMDE_FLOAT32_C(   746.32),
        SIMDE_FLOAT32_C(  -219.53), SIMDE_FLOAT32_C(  -616.77), SIMDE_FLOAT32_C(   992.79), SIMDE_FLOAT32_C(  -122.39) },
      { SIMDE_FLOAT32_C(   111.00), SIMDE_FLOAT32_C(   332.00), SIMDE_FLOAT32_C(  -344.00), SIMDE_FLOAT32_C(  -528.00),
        SIMDE_FLOAT32_C(  -821.00), SIMDE_FLOAT32_C(   -38.00), SIMDE_FLOAT32_C(   645.00), SIMDE_FLOAT32_C(  -461.00),
        SIMDE_FLOAT32_C(  -946.00), SIMDE_FLOAT32_C(  -679.00), SIMDE_FLOAT32_C(   996.00), SIMDE_FLOAT32_C(   746.00),
        SIMDE_FLOAT32_C(  -220.00), SIMDE_FLOAT32_C(  -617.00), SIMDE_FLOAT32_C(   993.00), SIMDE_FLOAT32_C(  -122.00) } },
    { { SIMDE_FLOAT32_C(  -338.27), SIMDE_FLOAT32_C(  -338.93), SIMDE_FLOAT32_C(  -294.51), SIMDE_FLOAT32_C(   440.55),
        SIMDE_FLOAT32_C(   865.88), SIMDE_FLOAT32_C(   439.63), SIMDE_FLOAT32_C(  -397.70), SIMDE_FLOAT32_C(   730.29),
        SIMDE_FLOAT32_C(  -760.09), SIMDE_FLOAT32_C(   665.63), SIMDE_FLOAT32_C(   224.63), SIMDE_FLOAT32_C(   -58.68),
        SIMDE_FLOAT32_C(  -515.91), SIMDE_FLOAT32_C(  -316.80), SIMDE_FLOAT32_C(  -985.88), SIMDE_FLOAT32_C(   595.23) },
      { SIMDE_FLOAT32_C(  -338.00), SIMDE_FLOAT32_C(  -339.00), SIMDE_FLOAT32_C(  -295.00), SIMDE_FLOAT32_C(   441.00),
        SIMDE_FLOAT32_C(   866.00), SIMDE_FLOAT32_C(   440.00), SIMDE_FLOAT32_C(  -398.00), SIMDE_FLOAT32_C(   730.00),
        SIMDE_FLOAT32_C(  -760.00), SIMDE_FLOAT32_C(   666.00), SIMDE_FLOAT32_C(   225.00), SIMDE_FLOAT32_C(   -59.00),
        SIMDE_FLOAT32_C(  -516.00), SIMDE_FLOAT32_C(  -317.00), SIMDE_FLOAT32_C(  -986.00), SIMDE_FLOAT32_C(   595.00) } },
    { { SIMDE_FLOAT32_C(  -984.84), SIMDE_FLOAT32_C(  -330.15), SIMDE_FLOAT32_C(  -933.01), SIMDE_FLOAT32_C(  -806.00),
        SIMDE_FLOAT32_C(   632.00), SIMDE_FLOAT32_C(   712.36), SIMDE_FLOAT32_C(  -266.98), SIMDE_FLOAT32_C(   685.88),
        SIMDE_FLOAT32_C(  -966.61), SIMDE_FLOAT32_C(  -271.27), SIMDE_FLOAT32_C(   432.20), SIMDE_FLOAT32_C(  -186.14),
        SIMDE_FLOAT32_C(   111.96), SIMDE_FLOAT32_C(   424.99), SIMDE_FLOAT32_C(   691.48), SIMDE_FLOAT32_C(   773.69) },
      { SIMDE_FLOAT32_C(  -985.00), SIMDE_FLOAT32_C(  -330.00), SIMDE_FLOAT32_C(  -933.00), SIMDE_FLOAT32_C(  -806.00),
        SIMDE_FLOAT32_C(   632.00), SIMDE_FLOAT32_C(   712.00), SIMDE_FLOAT32_C(  -267.00), SIMDE_FLOAT32_C(   686.00),
        SIMDE_FLOAT32_C(  -967.00), SIMDE_FLOAT32_C(  -271.00), SIMDE_FLOAT32_C(   432.00), SIMDE_FLOAT32_C(  -186.00),
        SIMDE_FLOAT32_C(   112.00), SIMDE_FLOAT32_C(   425.00), SIMDE_FLOAT32_C(   691.00), SIMDE_FLOAT32_C(   774.00) } },
    { { SIMDE_FLOAT32_C(  -913.94), SIMDE_FLOAT32_C(  -603.03), SIMDE_FLOAT32_C(   214.24), SIMDE_FLOAT32_C(   951.94),
        SIMDE_FLOAT32_C(   836.60), SIMDE_FLOAT32_C(   816.55), SIMDE_FLOAT32_C(   682.23), SIMDE_FLOAT32_C(  -923.49),
        SIMDE_FLOAT32_C(   482.17), SIMDE_FLOAT32_C(   -93.14), SIMDE_FLOAT32_C(    17.84), SIMDE_FLOAT32_C(   966.27),
        SIMDE_FLOAT32_C(   590.07), SIMDE_FLOAT32_C(    31.96), SIMDE_FLOAT32_C(   561.50), SIMDE_FLOAT32_C(   605.23) },
      { SIMDE_FLOAT32_C(  -914.00), SIMDE_FLOAT32_C(  -603.00), SIMDE_FLOAT32_C(   214.00), SIMDE_FLOAT32_C(   952.00),
        SIMDE_FLOAT32_C(   837.00), SIMDE_FLOAT32_C(   817.00), SIMDE_FLOAT32_C(   682.00), SIMDE_FLOAT32_C(  -923.00),
        SIMDE_FLOAT32_C(   482.00), SIMDE_FLOAT32_C(   -93.00), SIMDE_FLOAT32_C(    18.00), SIMDE_FLOAT32_C(   966.00),
        SIMDE_FLOAT32_C(   590.00), SIMDE_FLOAT32_C(    32.00), SIMDE_FLOAT32_C(   562.00), SIMDE_FLOAT32_C(   605.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_rint_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_rint_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -528.78), SIMDE_FLOAT32_C(   785.86), SIMDE_FLOAT32_C(  -381.92), SIMDE_FLOAT32_C(  -860.14),
        SIMDE_FLOAT32_C(   577.18), SIMDE_FLOAT32_C(   -21.79), SIMDE_FLOAT32_C(   -56.29), SIMDE_FLOAT32_C(  -835.30),
        SIMDE_FLOAT32_C(   126.46), SIMDE_FLOAT32_C(  -806.06), SIMDE_FLOAT32_C(  -450.59), SIMDE_FLOAT32_C(  -478.17),
        SIMDE_FLOAT32_C(  -707.43), SIMDE_FLOAT32_C(  -543.19), SIMDE_FLOAT32_C(  -401.16), SIMDE_FLOAT32_C(  -180.42) },
      UINT8_C( 91),
      { SIMDE_FLOAT32_C(   923.27), SIMDE_FLOAT32_C(    86.29), SIMDE_FLOAT32_C(   691.94), SIMDE_FLOAT32_C(   293.32),
        SIMDE_FLOAT32_C(   -23.72), SIMDE_FLOAT32_C(  -199.60), SIMDE_FLOAT32_C(   909.94), SIMDE_FLOAT32_C(   715.72),
        SIMDE_FLOAT32_C(  -312.75), SIMDE_FLOAT32_C(   291.35), SIMDE_FLOAT32_C(  -637.29), SIMDE_FLOAT32_C(  -832.86),
        SIMDE_FLOAT32_C(  -939.64), SIMDE_FLOAT32_C(  -775.32), SIMDE_FLOAT32_C(  -361.64), SIMDE_FLOAT32_C(   846.22) },
      { SIMDE_FLOAT32_C(   923.00), SIMDE_FLOAT32_C(    86.00), SIMDE_FLOAT32_C(  -381.92), SIMDE_FLOAT32_C(   293.00),
        SIMDE_FLOAT32_C(   -24.00), SIMDE_FLOAT32_C(   -21.79), SIMDE_FLOAT32_C(   910.00), SIMDE_FLOAT32_C(  -835.30),
        SIMDE_FLOAT32_C(   126.46), SIMDE_FLOAT32_C(  -806.06), SIMDE_FLOAT32_C(  -450.59), SIMDE_FLOAT32_C(  -478.17),
        SIMDE_FLOAT32_C(  -707.43), SIMDE_FLOAT32_C(  -543.19), SIMDE_FLOAT32_C(  -401.16), SIMDE_FLOAT32_C(  -180.42) } },
    { { SIMDE_FLOAT32_C(  -157.24), SIMDE_FLOAT32_C(  -221.78), SIMDE_FLOAT32_C(   423.40), SIMDE_FLOAT32_C(   820.97),
        SIMDE_FLOAT32_C(   721.93), SIMDE_FLOAT32_C(   588.10), SIMDE_FLOAT32_C(   -52.57), SIMDE_FLOAT32_C(   915.87),
        SIMDE_FLOAT32_C(  -862.49), SIMDE_FLOAT32_C(   469.26), SIMDE_FLOAT32_C(  -791.57), SIMDE_FLOAT32_C(  -405.68),
        SIMDE_FLOAT32_C(  -931.90), SIMDE_FLOAT32_C(    28.01), SIMDE_FLOAT32_C(    16.04), SIMDE_FLOAT32_C(   991.37) },
      UINT8_C( 35),
      { SIMDE_FLOAT32_C(  -292.02), SIMDE_FLOAT32_C(   284.69), SIMDE_FLOAT32_C(    90.57), SIMDE_FLOAT32_C(   508.38),
        SIMDE_FLOAT32_C(   194.63), SIMDE_FLOAT32_C(  -193.71), SIMDE_FLOAT32_C(  -804.38), SIMDE_FLOAT32_C(  -514.01),
        SIMDE_FLOAT32_C(   169.00), SIMDE_FLOAT32_C(  -637.23), SIMDE_FLOAT32_C(  -453.66), SIMDE_FLOAT32_C(   393.68),
        SIMDE_FLOAT32_C(     1.13), SIMDE_FLOAT32_C(  -607.44), SIMDE_FLOAT32_C(  -763.56), SIMDE_FLOAT32_C(   779.35) },
      { SIMDE_FLOAT32_C(  -292.00), SIMDE_FLOAT32_C(   285.00), SIMDE_FLOAT32_C(   423.40), SIMDE_FLOAT32_C(   820.97),
        SIMDE_FLOAT32_C(   721.93), SIMDE_FLOAT32_C(  -194.00), SIMDE_FLOAT32_C(   -52.57), SIMDE_FLOAT32_C(   915.87),
        SIMDE_FLOAT32_C(  -862.49), SIMDE_FLOAT32_C(   469.26), SIMDE_FLOAT32_C(  -791.57), SIMDE_FLOAT32_C(  -405.68),
        SIMDE_FLOAT32_C(  -931.90), SIMDE_FLOAT32_C(    28.01), SIMDE_FLOAT32_C(    16.04), SIMDE_FLOAT32_C(   991.37) } },
    { { SIMDE_FLOAT32_C(   815.97), SIMDE_FLOAT32_C(  -942.60), SIMDE_FLOAT32_C(   501.28), SIMDE_FLOAT32_C(   404.07),
        SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(   417.15), SIMDE_FLOAT32_C(   541.58), SIMDE_FLOAT32_C(  -525.90),
        SIMDE_FLOAT32_C(   625.58), SIMDE_FLOAT32_C(  -864.10), SIMDE_FLOAT32_C(  -457.80), SIMDE_FLOAT32_C(  -346.41),
        SIMDE_FLOAT32_C(   151.94), SIMDE_FLOAT32_C(  -466.43), SIMDE_FLOAT32_C(  -232.11), SIMDE_FLOAT32_C(   859.92) },
      UINT8_C(181),
      { SIMDE_FLOAT32_C(   858.46), SIMDE_FLOAT32_C(   368.30), SIMDE_FLOAT32_C(    12.90), SIMDE_FLOAT32_C(  -335.24),
        SIMDE_FLOAT32_C(   563.92), SIMDE_FLOAT32_C(   498.88), SIMDE_FLOAT32_C(   833.76), SIMDE_FLOAT32_C(   926.69),
        SIMDE_FLOAT32_C(  -954.77), SIMDE_FLOAT32_C(   227.44), SIMDE_FLOAT32_C(   -72.18), SIMDE_FLOAT32_C(  -562.21),
        SIMDE_FLOAT32_C(   463.87), SIMDE_FLOAT32_C(  -292.83), SIMDE_FLOAT32_C(  -746.24), SIMDE_FLOAT32_C(   521.28) },
      { SIMDE_FLOAT32_C(   858.00), SIMDE_FLOAT32_C(  -942.60), SIMDE_FLOAT32_C(    13.00), SIMDE_FLOAT32_C(   404.07),
        SIMDE_FLOAT32_C(   564.00), SIMDE_FLOAT32_C(   499.00), SIMDE_FLOAT32_C(   541.58), SIMDE_FLOAT32_C(   927.00),
        SIMDE_FLOAT32_C(   625.58), SIMDE_FLOAT32_C(  -864.10), SIMDE_FLOAT32_C(  -457.80), SIMDE_FLOAT32_C(  -346.41),
        SIMDE_FLOAT32_C(   151.94), SIMDE_FLOAT32_C(  -466.43), SIMDE_FLOAT32_C(  -232.11), SIMDE_FLOAT32_C(   859.92) } },
    { { SIMDE_FLOAT32_C(  -791.54), SIMDE_FLOAT32_C(   657.83), SIMDE_FLOAT32_C(  -473.89), SIMDE_FLOAT32_C(   625.60),
        SIMDE_FLOAT32_C(   199.41), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(   251.18), SIMDE_FLOAT32_C(   335.31),
        SIMDE_FLOAT32_C(   542.40), SIMDE_FLOAT32_C(   904.77), SIMDE_FLOAT32_C(  -512.75), SIMDE_FLOAT32_C(  -924.03),
        SIMDE_FLOAT32_C(  -327.34), SIMDE_FLOAT32_C(  -652.83), SIMDE_FLOAT32_C(   894.23), SIMDE_FLOAT32_C(  -468.87) },
      UINT8_C(106),
      { SIMDE_FLOAT32_C(   -92.87), SIMDE_FLOAT32_C(   195.88), SIMDE_FLOAT32_C(   279.39), SIMDE_FLOAT32_C(  -593.99),
        SIMDE_FLOAT32_C(    29.64), SIMDE_FLOAT32_C(   206.08), SIMDE_FLOAT32_C(  -548.77), SIMDE_FLOAT32_C(  -742.92),
        SIMDE_FLOAT32_C(  -866.10), SIMDE_FLOAT32_C(  -110.98), SIMDE_FLOAT32_C(   720.95), SIMDE_FLOAT32_C(  -158.93),
        SIMDE_FLOAT32_C(   142.78), SIMDE_FLOAT32_C(   242.22), SIMDE_FLOAT32_C(    49.53), SIMDE_FLOAT32_C(  -199.39) },
      { SIMDE_FLOAT32_C(  -791.54), SIMDE_FLOAT32_C(   196.00), SIMDE_FLOAT32_C(  -473.89), SIMDE_FLOAT32_C(  -594.00),
        SIMDE_FLOAT32_C(   199.41), SIMDE_FLOAT32_C(   206.00), SIMDE_FLOAT32_C(  -549.00), SIMDE_FLOAT32_C(   335.31),
        SIMDE_FLOAT32_C(   542.40), SIMDE_FLOAT32_C(   904.77), SIMDE_FLOAT32_C(  -512.75), SIMDE_FLOAT32_C(  -924.03),
        SIMDE_FLOAT32_C(  -327.34), SIMDE_FLOAT32_C(  -652.83), SIMDE_FLOAT32_C(   894.23), SIMDE_FLOAT32_C(  -468.87) } },
    { { SIMDE_FLOAT32_C(   768.33), SIMDE_FLOAT32_C(  -324.87), SIMDE_FLOAT32_C(  -999.98), SIMDE_FLOAT32_C(  -231.46),
        SIMDE_FLOAT32_C(   926.31), SIMDE_FLOAT32_C(   335.33), SIMDE_FLOAT32_C(  -689.06), SIMDE_FLOAT32_C(   831.09),
        SIMDE_FLOAT32_C(   822.57), SIMDE_FLOAT32_C(  -613.09), SIMDE_FLOAT32_C(  -496.25), SIMDE_FLOAT32_C(  -830.26),
        SIMDE_FLOAT32_C(  -718.86), SIMDE_FLOAT32_C(    34.88), SIMDE_FLOAT32_C(   885.21), SIMDE_FLOAT32_C(   188.27) },
      UINT8_C(197),
      { SIMDE_FLOAT32_C(   164.59), SIMDE_FLOAT32_C(   594.28), SIMDE_FLOAT32_C(   260.41), SIMDE_FLOAT32_C(  -629.33),
        SIMDE_FLOAT32_C(  -954.49), SIMDE_FLOAT32_C(   517.49), SIMDE_FLOAT32_C(  -495.43), SIMDE_FLOAT32_C(   -65.47),
        SIMDE_FLOAT32_C(   238.43), SIMDE_FLOAT32_C(   345.64), SIMDE_FLOAT32_C(  -922.68), SIMDE_FLOAT32_C(  -519.34),
        SIMDE_FLOAT32_C(  -604.83), SIMDE_FLOAT32_C(  -122.08), SIMDE_FLOAT32_C(  -751.01), SIMDE_FLOAT32_C(    70.30) },
      { SIMDE_FLOAT32_C(   165.00), SIMDE_FLOAT32_C(  -324.87), SIMDE_FLOAT32_C(   260.00), SIMDE_FLOAT32_C(  -231.46),
        SIMDE_FLOAT32_C(   926.31), SIMDE_FLOAT32_C(   335.33), SIMDE_FLOAT32_C(  -495.00), SIMDE_FLOAT32_C(   -65.00),
        SIMDE_FLOAT32_C(   822.57), SIMDE_FLOAT32_C(  -613.09), SIMDE_FLOAT32_C(  -496.25), SIMDE_FLOAT32_C(  -830.26),
        SIMDE_FLOAT32_C(  -718.86), SIMDE_FLOAT32_C(    34.88), SIMDE_FLOAT32_C(   885.21), SIMDE_FLOAT32_C(   188.27) } },
    { { SIMDE_FLOAT32_C(  -122.06), SIMDE_FLOAT32_C(    17.53), SIMDE_FLOAT32_C(    -3.38), SIMDE_FLOAT32_C(  -786.73),
        SIMDE_FLOAT32_C(   328.46), SIMDE_FLOAT32_C(  -172.29), SIMDE_FLOAT32_C(  -964.16), SIMDE_FLOAT32_C(   715.37),
        SIMDE_FLOAT32_C(   331.46), SIMDE_FLOAT32_C(  -794.41), SIMDE_FLOAT32_C(   996.51), SIMDE_FLOAT32_C(  -633.66),
        SIMDE_FLOAT32_C(  -909.21), SIMDE_FLOAT32_C(   184.77), SIMDE_FLOAT32_C(  -402.90), SIMDE_FLOAT32_C(   255.39) },
      UINT8_C(  2),
      { SIMDE_FLOAT32_C(   857.51), SIMDE_FLOAT32_C(   626.06), SIMDE_FLOAT32_C(  -175.44), SIMDE_FLOAT32_C(   375.00),
        SIMDE_FLOAT32_C(  -869.37), SIMDE_FLOAT32_C(   759.09), SIMDE_FLOAT32_C(  -386.57), SIMDE_FLOAT32_C(   476.27),
        SIMDE_FLOAT32_C(   836.41), SIMDE_FLOAT32_C(    94.09), SIMDE_FLOAT32_C(   871.44), SIMDE_FLOAT32_C(  -285.67),
        SIMDE_FLOAT32_C(   343.08), SIMDE_FLOAT32_C(   -58.26), SIMDE_FLOAT32_C(   592.27), SIMDE_FLOAT32_C(  -639.39) },
      { SIMDE_FLOAT32_C(  -122.06), SIMDE_FLOAT32_C(   626.00), SIMDE_FLOAT32_C(    -3.38), SIMDE_FLOAT32_C(  -786.73),
        SIMDE_FLOAT32_C(   328.46), SIMDE_FLOAT32_C(  -172.29), SIMDE_FLOAT32_C(  -964.16), SIMDE_FLOAT32_C(   715.37),
        SIMDE_FLOAT32_C(   331.46), SIMDE_FLOAT32_C(  -794.41), SIMDE_FLOAT32_C(   996.51), SIMDE_FLOAT32_C(  -633.66),
        SIMDE_FLOAT32_C(  -909.21), SIMDE_FLOAT32_C(   184.77), SIMDE_FLOAT32_C(  -402.90), SIMDE_FLOAT32_C(   255.39) } },
    { { SIMDE_FLOAT32_C(   938.35), SIMDE_FLOAT32_C(   805.54), SIMDE_FLOAT32_C(   689.07), SIMDE_FLOAT32_C(  -233.94),
        SIMDE_FLOAT32_C(   841.38), SIMDE_FLOAT32_C(   404.44), SIMDE_FLOAT32_C(  -902.48), SIMDE_FLOAT32_C(  -953.03),
        SIMDE_FLOAT32_C(   400.95), SIMDE_FLOAT32_C(  -536.14), SIMDE_FLOAT32_C(  -862.24), SIMDE_FLOAT32_C(  -414.28),
        SIMDE_FLOAT32_C(    60.96), SIMDE_FLOAT32_C(   393.15), SIMDE_FLOAT32_C(   364.77), SIMDE_FLOAT32_C(   -81.52) },
      UINT8_C( 26),
      { SIMDE_FLOAT32_C(  -810.67), SIMDE_FLOAT32_C(  -706.52), SIMDE_FLOAT32_C(   149.83), SIMDE_FLOAT32_C(   948.42),
        SIMDE_FLOAT32_C(   -93.09), SIMDE_FLOAT32_C(  -373.90), SIMDE_FLOAT32_C(   784.83), SIMDE_FLOAT32_C(  -999.00),
        SIMDE_FLOAT32_C(  -502.46), SIMDE_FLOAT32_C(  -500.84), SIMDE_FLOAT32_C(   344.08), SIMDE_FLOAT32_C(   439.27),
        SIMDE_FLOAT32_C(  -908.56), SIMDE_FLOAT32_C(   704.69), SIMDE_FLOAT32_C(   377.63), SIMDE_FLOAT32_C(   896.98) },
      { SIMDE_FLOAT32_C(   938.35), SIMDE_FLOAT32_C(  -707.00), SIMDE_FLOAT32_C(   689.07), SIMDE_FLOAT32_C(   948.00),
        SIMDE_FLOAT32_C(   -93.00), SIMDE_FLOAT32_C(   404.44), SIMDE_FLOAT32_C(  -902.48), SIMDE_FLOAT32_C(  -953.03),
        SIMDE_FLOAT32_C(   400.95), SIMDE_FLOAT32_C(  -536.14), SIMDE_FLOAT32_C(  -862.24), SIMDE_FLOAT32_C(  -414.28),
        SIMDE_FLOAT32_C(    60.96), SIMDE_FLOAT32_C(   393.15), SIMDE_FLOAT32_C(   364.77), SIMDE_FLOAT32_C(   -81.52) } },
    { { SIMDE_FLOAT32_C(   393.76), SIMDE_FLOAT32_C(  -856.31), SIMDE_FLOAT32_C(   738.36), SIMDE_FLOAT32_C(  -201.81),
        SIMDE_FLOAT32_C(  -758.79), SIMDE_FLOAT32_C(   785.33), SIMDE_FLOAT32_C(  -800.86), SIMDE_FLOAT32_C(  -294.93),
        SIMDE_FLOAT32_C(   923.10), SIMDE_FLOAT32_C(  -215.14), SIMDE_FLOAT32_C(   766.03), SIMDE_FLOAT32_C(   316.25),
        SIMDE_FLOAT32_C(  -850.37), SIMDE_FLOAT32_C(  -315.49), SIMDE_FLOAT32_C(  -664.55), SIMDE_FLOAT32_C(  -661.04) },
      UINT8_C(104),
      { SIMDE_FLOAT32_C(   485.29), SIMDE_FLOAT32_C(  -712.62), SIMDE_FLOAT32_C(   884.89), SIMDE_FLOAT32_C(  -888.61),
        SIMDE_FLOAT32_C(  -927.79), SIMDE_FLOAT32_C(   885.89), SIMDE_FLOAT32_C(  -391.08), SIMDE_FLOAT32_C(  -428.63),
        SIMDE_FLOAT32_C(   229.97), SIMDE_FLOAT32_C(  -951.80), SIMDE_FLOAT32_C(  -337.19), SIMDE_FLOAT32_C(   -65.34),
        SIMDE_FLOAT32_C(   425.83), SIMDE_FLOAT32_C(  -440.21), SIMDE_FLOAT32_C(  -671.58), SIMDE_FLOAT32_C(   569.52) },
      { SIMDE_FLOAT32_C(   393.76), SIMDE_FLOAT32_C(  -856.31), SIMDE_FLOAT32_C(   738.36), SIMDE_FLOAT32_C(  -889.00),
        SIMDE_FLOAT32_C(  -758.79), SIMDE_FLOAT32_C(   886.00), SIMDE_FLOAT32_C(  -391.00), SIMDE_FLOAT32_C(  -294.93),
        SIMDE_FLOAT32_C(   923.10), SIMDE_FLOAT32_C(  -215.14), SIMDE_FLOAT32_C(   766.03), SIMDE_FLOAT32_C(   316.25),
        SIMDE_FLOAT32_C(  -850.37), SIMDE_FLOAT32_C(  -315.49), SIMDE_FLOAT32_C(  -664.55), SIMDE_FLOAT32_C(  -661.04) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_rint_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_rint_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -246.76), SIMDE_FLOAT64_C(   995.20), SIMDE_FLOAT64_C(   968.30), SIMDE_FLOAT64_C(   593.75),
        SIMDE_FLOAT64_C(   235.19), SIMDE_FLOAT64_C(    73.30), SIMDE_FLOAT64_C(  -552.80), SIMDE_FLOAT64_C(  -271.48) },
      { SIMDE_FLOAT64_C(  -247.00), SIMDE_FLOAT64_C(   995.00), SIMDE_FLOAT64_C(   968.00), SIMDE_FLOAT64_C(   594.00),
        SIMDE_FLOAT64_C(   235.00), SIMDE_FLOAT64_C(    73.00), SIMDE_FLOAT64_C(  -553.00), SIMDE_FLOAT64_C(  -271.00) } },
    { { SIMDE_FLOAT64_C(  -135.03), SIMDE_FLOAT64_C(  -911.80), SIMDE_FLOAT64_C(  -344.75), SIMDE_FLOAT64_C(  -200.72),
        SIMDE_FLOAT64_C(   333.22), SIMDE_FLOAT64_C(   889.93), SIMDE_FLOAT64_C(   -90.00), SIMDE_FLOAT64_C(   700.69) },
      { SIMDE_FLOAT64_C(  -135.00), SIMDE_FLOAT64_C(  -912.00), SIMDE_FLOAT64_C(  -345.00), SIMDE_FLOAT64_C(  -201.00),
        SIMDE_FLOAT64_C(   333.00), SIMDE_FLOAT64_C(   890.00), SIMDE_FLOAT64_C(   -90.00), SIMDE_FLOAT64_C(   701.00) } },
    { { SIMDE_FLOAT64_C(  -507.88), SIMDE_FLOAT64_C(    21.18), SIMDE_FLOAT64_C(  -600.24), SIMDE_FLOAT64_C(   -90.19),
        SIMDE_FLOAT64_C(  -792.15), SIMDE_FLOAT64_C(   778.81), SIMDE_FLOAT64_C(   116.68), SIMDE_FLOAT64_C(    97.12) },
      { SIMDE_FLOAT64_C(  -508.00), SIMDE_FLOAT64_C(    21.00), SIMDE_FLOAT64_C(  -600.00), SIMDE_FLOAT64_C(   -90.00),
        SIMDE_FLOAT64_C(  -792.00), SIMDE_FLOAT64_C(   779.00), SIMDE_FLOAT64_C(   117.00), SIMDE_FLOAT64_C(    97.00) } },
    { { SIMDE_FLOAT64_C(   426.71), SIMDE_FLOAT64_C(   210.55), SIMDE_FLOAT64_C(  -406.04), SIMDE_FLOAT64_C(   169.01),
        SIMDE_FLOAT64_C(   164.78), SIMDE_FLOAT64_C(  -734.90), SIMDE_FLOAT64_C(  -482.68), SIMDE_FLOAT64_C(   918.02) },
      { SIMDE_FLOAT64_C(   427.00), SIMDE_FLOAT64_C(   211.00), SIMDE_FLOAT64_C(  -406.00), SIMDE_FLOAT64_C(   169.00),
        SIMDE_FLOAT64_C(   165.00), SIMDE_FLOAT64_C(  -735.00), SIMDE_FLOAT64_C(  -483.00), SIMDE_FLOAT64_C(   918.00) } },
    { { SIMDE_FLOAT64_C(  -739.70), SIMDE_FLOAT64_C(  -514.38), SIMDE_FLOAT64_C(   511.78), SIMDE_FLOAT64_C(   495.49),
        SIMDE_FLOAT64_C(   558.92), SIMDE_FLOAT64_C(   958.98), SIMDE_FLOAT64_C(  -775.99), SIMDE_FLOAT64_C(  -576.12) },
      { SIMDE_FLOAT64_C(  -740.00), SIMDE_FLOAT64_C(  -514.00), SIMDE_FLOAT64_C(   512.00), SIMDE_FLOAT64_C(   495.00),
        SIMDE_FLOAT64_C(   559.00), SIMDE_FLOAT64_C(   959.00), SIMDE_FLOAT64_C(  -776.00), SIMDE_FLOAT64_C(  -576.00) } },
    { { SIMDE_FLOAT64_C(  -952.82), SIMDE_FLOAT64_C(  -120.74), SIMDE_FLOAT64_C(   223.17), SIMDE_FLOAT64_C(   380.40),
        SIMDE_FLOAT64_C(  -230.81), SIMDE_FLOAT64_C(  -866.83), SIMDE_FLOAT64_C(    81.08), SIMDE_FLOAT64_C(   261.31) },
      { SIMDE_FLOAT64_C(  -953.00), SIMDE_FLOAT64_C(  -121.00), SIMDE_FLOAT64_C(   223.00), SIMDE_FLOAT64_C(   380.00),
        SIMDE_FLOAT64_C(  -231.00), SIMDE_FLOAT64_C(  -867.00), SIMDE_FLOAT64_C(    81.00), SIMDE_FLOAT64_C(   261.00) } },
    { { SIMDE_FLOAT64_C(   154.35), SIMDE_FLOAT64_C(   480.85), SIMDE_FLOAT64_C(  -828.88), SIMDE_FLOAT64_C(   362.20),
        SIMDE_FLOAT64_C(   259.66), SIMDE_FLOAT64_C(   287.79), SIMDE_FLOAT64_C(  -540.68), SIMDE_FLOAT64_C(  -313.64) },
      { SIMDE_FLOAT64_C(   154.00), SIMDE_FLOAT64_C(   481.00), SIMDE_FLOAT64_C(  -829.00), SIMDE_FLOAT64_C(   362.00),
        SIMDE_FLOAT64_C(   260.00), SIMDE_FLOAT64_C(   288.00), SIMDE_FLOAT64_C(  -541.00), SIMDE_FLOAT64_C(  -314.00) } },
    { { SIMDE_FLOAT64_C(  -501.66), SIMDE_FLOAT64_C(    53.28), SIMDE_FLOAT64_C(   855.37), SIMDE_FLOAT64_C(   663.12),
        SIMDE_FLOAT64_C(   318.39), SIMDE_FLOAT64_C(  -627.30), SIMDE_FLOAT64_C(   581.15), SIMDE_FLOAT64_C(   578.68) },
      { SIMDE_FLOAT64_C(  -502.00), SIMDE_FLOAT64_C(    53.00), SIMDE_FLOAT64_C(   855.00), SIMDE_FLOAT64_C(   663.00),
        SIMDE_FLOAT64_C(   318.00), SIMDE_FLOAT64_C(  -627.00), SIMDE_FLOAT64_C(   581.00), SIMDE_FLOAT64_C(   579.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_rint_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_rint_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -208.54), SIMDE_FLOAT64_C(  -850.79), SIMDE_FLOAT64_C(  -979.95), SIMDE_FLOAT64_C(  -415.72),
        SIMDE_FLOAT64_C(   722.54), SIMDE_FLOAT64_C(  -386.30), SIMDE_FLOAT64_C(   827.55), SIMDE_FLOAT64_C(  -329.72) },
      UINT8_C( 33),
      { SIMDE_FLOAT64_C(  -547.16), SIMDE_FLOAT64_C(   343.76), SIMDE_FLOAT64_C(  -161.57), SIMDE_FLOAT64_C(   958.51),
        SIMDE_FLOAT64_C(   185.76), SIMDE_FLOAT64_C(   479.23), SIMDE_FLOAT64_C(   948.46), SIMDE_FLOAT64_C(   354.63) },
      { SIMDE_FLOAT64_C(  -547.00), SIMDE_FLOAT64_C(  -850.79), SIMDE_FLOAT64_C(  -979.95), SIMDE_FLOAT64_C(  -415.72),
        SIMDE_FLOAT64_C(   722.54), SIMDE_FLOAT64_C(   479.00), SIMDE_FLOAT64_C(   827.55), SIMDE_FLOAT64_C(  -329.72) } },
    { { SIMDE_FLOAT64_C(   164.70), SIMDE_FLOAT64_C(   580.02), SIMDE_FLOAT64_C(   369.11), SIMDE_FLOAT64_C(  -928.66),
        SIMDE_FLOAT64_C(   607.84), SIMDE_FLOAT64_C(   793.55), SIMDE_FLOAT64_C(  -417.32), SIMDE_FLOAT64_C(   -33.65) },
      UINT8_C(142),
      { SIMDE_FLOAT64_C(    85.45), SIMDE_FLOAT64_C(   426.84), SIMDE_FLOAT64_C(  -691.54), SIMDE_FLOAT64_C(   519.42),
        SIMDE_FLOAT64_C(   413.73), SIMDE_FLOAT64_C(    99.92), SIMDE_FLOAT64_C(   668.63), SIMDE_FLOAT64_C(   433.78) },
      { SIMDE_FLOAT64_C(   164.70), SIMDE_FLOAT64_C(   427.00), SIMDE_FLOAT64_C(  -692.00), SIMDE_FLOAT64_C(   519.00),
        SIMDE_FLOAT64_C(   607.84), SIMDE_FLOAT64_C(   793.55), SIMDE_FLOAT64_C(  -417.32), SIMDE_FLOAT64_C(   434.00) } },
    { { SIMDE_FLOAT64_C(   684.20), SIMDE_FLOAT64_C(   391.17), SIMDE_FLOAT64_C(  -952.53), SIMDE_FLOAT64_C(   511.75),
        SIMDE_FLOAT64_C(  -938.55), SIMDE_FLOAT64_C(  -562.45), SIMDE_FLOAT64_C(   964.59), SIMDE_FLOAT64_C(   405.21) },
      UINT8_C(209),
      { SIMDE_FLOAT64_C(   923.10), SIMDE_FLOAT64_C(  -409.02), SIMDE_FLOAT64_C(  -244.78), SIMDE_FLOAT64_C(   871.57),
        SIMDE_FLOAT64_C(   945.61), SIMDE_FLOAT64_C(   919.91), SIMDE_FLOAT64_C(   451.58), SIMDE_FLOAT64_C(   314.71) },
      { SIMDE_FLOAT64_C(   923.00), SIMDE_FLOAT64_C(   391.17), SIMDE_FLOAT64_C(  -952.53), SIMDE_FLOAT64_C(   511.75),
        SIMDE_FLOAT64_C(   946.00), SIMDE_FLOAT64_C(  -562.45), SIMDE_FLOAT64_C(   452.00), SIMDE_FLOAT64_C(   315.00) } },
    { { SIMDE_FLOAT64_C(   991.25), SIMDE_FLOAT64_C(    59.43), SIMDE_FLOAT64_C(   108.26), SIMDE_FLOAT64_C(  -426.07),
        SIMDE_FLOAT64_C(  -974.22), SIMDE_FLOAT64_C(   827.67), SIMDE_FLOAT64_C(   659.39), SIMDE_FLOAT64_C(   452.62) },
      UINT8_C( 74),
      { SIMDE_FLOAT64_C(   178.81), SIMDE_FLOAT64_C(  -133.64), SIMDE_FLOAT64_C(   236.06), SIMDE_FLOAT64_C(  -152.57),
        SIMDE_FLOAT64_C(  -699.87), SIMDE_FLOAT64_C(   -79.74), SIMDE_FLOAT64_C(  -761.39), SIMDE_FLOAT64_C(  -652.39) },
      { SIMDE_FLOAT64_C(   991.25), SIMDE_FLOAT64_C(  -134.00), SIMDE_FLOAT64_C(   108.26), SIMDE_FLOAT64_C(  -153.00),
        SIMDE_FLOAT64_C(  -974.22), SIMDE_FLOAT64_C(   827.67), SIMDE_FLOAT64_C(  -761.00), SIMDE_FLOAT64_C(   452.62) } },
    { { SIMDE_FLOAT64_C(  -567.98), SIMDE_FLOAT64_C(  -699.94), SIMDE_FLOAT64_C(  -214.84), SIMDE_FLOAT64_C(  -603.39),
        SIMDE_FLOAT64_C(   705.27), SIMDE_FLOAT64_C(  -938.85), SIMDE_FLOAT64_C(  -680.29), SIMDE_FLOAT64_C(  -703.75) },
      UINT8_C(254),
      { SIMDE_FLOAT64_C(  -808.72), SIMDE_FLOAT64_C(  -758.15), SIMDE_FLOAT64_C(  -263.72), SIMDE_FLOAT64_C(   642.86),
        SIMDE_FLOAT64_C(   556.57), SIMDE_FLOAT64_C(  -272.47), SIMDE_FLOAT64_C(  -297.71), SIMDE_FLOAT64_C(  -335.17) },
      { SIMDE_FLOAT64_C(  -567.98), SIMDE_FLOAT64_C(  -758.00), SIMDE_FLOAT64_C(  -264.00), SIMDE_FLOAT64_C(   643.00),
        SIMDE_FLOAT64_C(   557.00), SIMDE_FLOAT64_C(  -272.00), SIMDE_FLOAT64_C(  -298.00), SIMDE_FLOAT64_C(  -335.00) } },
    { { SIMDE_FLOAT64_C(   301.46), SIMDE_FLOAT64_C(  -271.93), SIMDE_FLOAT64_C(  -507.50), SIMDE_FLOAT64_C(   -39.16),
        SIMDE_FLOAT64_C(  -819.31), SIMDE_FLOAT64_C(  -371.36), SIMDE_FLOAT64_C(  -860.35), SIMDE_FLOAT64_C(    47.05) },
      UINT8_C(  9),
      { SIMDE_FLOAT64_C(   -12.91), SIMDE_FLOAT64_C(   347.18), SIMDE_FLOAT64_C(  -215.03), SIMDE_FLOAT64_C(   225.69),
        SIMDE_FLOAT64_C(   694.79), SIMDE_FLOAT64_C(   216.99), SIMDE_FLOAT64_C(   525.75), SIMDE_FLOAT64_C(  -520.05) },
      { SIMDE_FLOAT64_C(   -13.00), SIMDE_FLOAT64_C(  -271.93), SIMDE_FLOAT64_C(  -507.50), SIMDE_FLOAT64_C(   226.00),
        SIMDE_FLOAT64_C(  -819.31), SIMDE_FLOAT64_C(  -371.36), SIMDE_FLOAT64_C(  -860.35), SIMDE_FLOAT64_C(    47.05) } },
    { { SIMDE_FLOAT64_C(   613.60), SIMDE_FLOAT64_C(   231.02), SIMDE_FLOAT64_C(  -458.90), SIMDE_FLOAT64_C(   933.31),
        SIMDE_FLOAT64_C(   527.27), SIMDE_FLOAT64_C(   357.46), SIMDE_FLOAT64_C(  -875.42), SIMDE_FLOAT64_C(   769.12) },
      UINT8_C(129),
      { SIMDE_FLOAT64_C(   767.45), SIMDE_FLOAT64_C(   325.69), SIMDE_FLOAT64_C(  -178.73), SIMDE_FLOAT64_C(  -530.26),
        SIMDE_FLOAT64_C(   990.52), SIMDE_FLOAT64_C(  -877.27), SIMDE_FLOAT64_C(   197.81), SIMDE_FLOAT64_C(  -516.98) },
      { SIMDE_FLOAT64_C(   767.00), SIMDE_FLOAT64_C(   231.02), SIMDE_FLOAT64_C(  -458.90), SIMDE_FLOAT64_C(   933.31),
        SIMDE_FLOAT64_C(   527.27), SIMDE_FLOAT64_C(   357.46), SIMDE_FLOAT64_C(  -875.42), SIMDE_FLOAT64_C(  -517.00) } },
    { { SIMDE_FLOAT64_C(    83.57), SIMDE_FLOAT64_C(   378.50), SIMDE_FLOAT64_C(   111.66), SIMDE_FLOAT64_C(   223.22),
        SIMDE_FLOAT64_C(  -574.45), SIMDE_FLOAT64_C(   -23.63), SIMDE_FLOAT64_C(  -789.69), SIMDE_FLOAT64_C(   772.73) },
      UINT8_C(203),
      { SIMDE_FLOAT64_C(   436.00), SIMDE_FLOAT64_C(   467.52), SIMDE_FLOAT64_C(   -21.68), SIMDE_FLOAT64_C(   -38.25),
        SIMDE_FLOAT64_C(   947.47), SIMDE_FLOAT64_C(  -408.08), SIMDE_FLOAT64_C(  -807.23), SIMDE_FLOAT64_C(  -511.43) },
      { SIMDE_FLOAT64_C(   436.00), SIMDE_FLOAT64_C(   468.00), SIMDE_FLOAT64_C(   111.66), SIMDE_FLOAT64_C(   -38.00),
        SIMDE_FLOAT64_C(  -574.45), SIMDE_FLOAT64_C(   -23.63), SIMDE_FLOAT64_C(  -807.00), SIMDE_FLOAT64_C(  -511.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_rint_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_sin_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.87)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     0.48)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.33)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -0.30)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.53)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(    -0.01)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(    -0.88)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.79)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_sin_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_sin_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(    0.87)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.97)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.84), SIMDE_FLOAT64_C(    0.48)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.81), SIMDE_FLOAT64_C(   -0.88)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.33)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.65), SIMDE_FLOAT64_C(    0.94)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.05), SIMDE_FLOAT64_C(   -0.30)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(   -0.95)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_sin_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_sin_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24),
                         SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01),
                         SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.88),
                         SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     0.48),
                         SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.97),
                         SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.87)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13),
                         SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21),
                         SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.95),
                         SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -0.30),
                         SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.94),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.33)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95),
                         SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(    -0.19),
                         SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(    -0.01),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.15),
                         SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.53)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67),
                         SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54),
                         SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(    -0.88)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48),
                         SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -0.49),
                         SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.32),
                         SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.92),
                         SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.67)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92),
                         SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.08),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.10),
                         SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.59)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73),
                         SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.74),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.63),
                         SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(     0.16)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02),
                         SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.41),
                         SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.16)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_sin_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_sin_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.97),
                         SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(    0.87)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.81), SIMDE_FLOAT64_C(   -0.88),
                         SIMDE_FLOAT64_C(   -0.84), SIMDE_FLOAT64_C(    0.48)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.65), SIMDE_FLOAT64_C(    0.94),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.33)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(   -0.95),
                         SIMDE_FLOAT64_C(    0.05), SIMDE_FLOAT64_C(   -0.30)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.33), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(   -0.29), SIMDE_FLOAT64_C(   -0.53)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.96), SIMDE_FLOAT64_C(   -0.19),
                         SIMDE_FLOAT64_C(   -0.69), SIMDE_FLOAT64_C(   -0.01)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.53), SIMDE_FLOAT64_C(    0.89),
                         SIMDE_FLOAT64_C(   -0.47), SIMDE_FLOAT64_C(   -0.88)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(    0.79)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_sin_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_sin_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -0.30),
                         SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     0.48),
                         SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.87)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(    -0.88),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(    -0.01),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.53)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.32),
                         SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.67)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.74),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(     0.16)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(   343.48),
                         SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   655.67),
                         SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(   120.65), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.87),
                         SIMDE_FLOAT32_C(    -0.81), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.80),
                         SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(    -0.96)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -591.56),
                         SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(  -906.16),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -738.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(    -0.81),
                         SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     1.00),
                         SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -0.98),
                         SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(    -0.08)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(   977.49),
                         SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.98),
                         SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(    -0.44),
                         SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(    -0.78)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -696.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.78),
                         SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.45),
                         SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.68)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_sin_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_sin_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(    -0.29),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(   346.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19),
                         SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   254.31), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(   398.82), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   -30.79), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.63),
                         SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.10),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(     0.92)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    99.93),
                         SIMDE_FLOAT32_C(  -738.19), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -448.89),
                         SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(   331.34),
                         SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -874.31),
                         SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   -70.91)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -337.60),
                         SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(  -756.42)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.98),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.44)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -737.13), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(   177.92),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   888.71), SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   133.52),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(  -775.04), SIMDE_FLOAT32_C(   440.64)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(   915.19), SIMDE_FLOAT32_C(  -718.40), SIMDE_FLOAT32_C(   159.97),
                         SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   110.36),
                         SIMDE_FLOAT32_C(   826.84), SIMDE_FLOAT32_C(   -76.75), SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -378.50),
                         SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(  -942.47), SIMDE_FLOAT32_C(   475.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(     0.25),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.39),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(   440.64)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(  -807.28), SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(   834.60), SIMDE_FLOAT32_C(   -65.60),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   556.35), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -964.25), SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(  -743.66), SIMDE_FLOAT32_C(  -764.58),
                         SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(  -818.54), SIMDE_FLOAT32_C(   161.06),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -308.52), SIMDE_FLOAT32_C(  -719.57),
                         SIMDE_FLOAT32_C(   334.00), SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(  -916.82), SIMDE_FLOAT32_C(  -490.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.74),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   105.79), SIMDE_FLOAT32_C(   590.10),
                         SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(   -84.00), SIMDE_FLOAT32_C(    80.04),
                         SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(  -889.11)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(   144.05), SIMDE_FLOAT32_C(  -427.72),
                         SIMDE_FLOAT32_C(   308.28), SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(  -457.77), SIMDE_FLOAT32_C(   678.24),
                         SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   117.28), SIMDE_FLOAT32_C(  -576.80),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   -53.92), SIMDE_FLOAT32_C(    91.94)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.34),
                         SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.95),
                         SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(    -0.74)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(  -493.41), SIMDE_FLOAT32_C(   822.72),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(  -816.27),
                         SIMDE_FLOAT32_C(  -209.34), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(  -204.33)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -841.43), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(   824.88), SIMDE_FLOAT32_C(   793.63),
                         SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -350.72),
                         SIMDE_FLOAT32_C(    60.89), SIMDE_FLOAT32_C(   109.81), SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -250.60),
                         SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   -13.07), SIMDE_FLOAT32_C(   852.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(     0.91),
                         SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(    -0.94)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_sin_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_sin_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.81), SIMDE_FLOAT64_C(   -0.88),
                         SIMDE_FLOAT64_C(   -0.84), SIMDE_FLOAT64_C(    0.48),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.97),
                         SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(    0.87)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(   -0.95),
                         SIMDE_FLOAT64_C(    0.05), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(    0.65), SIMDE_FLOAT64_C(    0.94),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.33)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.96), SIMDE_FLOAT64_C(   -0.19),
                         SIMDE_FLOAT64_C(   -0.69), SIMDE_FLOAT64_C(   -0.01),
                         SIMDE_FLOAT64_C(    0.33), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(   -0.29), SIMDE_FLOAT64_C(   -0.53)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(    0.79),
                         SIMDE_FLOAT64_C(   -0.53), SIMDE_FLOAT64_C(    0.89),
                         SIMDE_FLOAT64_C(   -0.47), SIMDE_FLOAT64_C(   -0.88)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(  993.90),
                         SIMDE_FLOAT64_C(   28.08), SIMDE_FLOAT64_C(  841.21)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.61), SIMDE_FLOAT64_C(   -0.49),
                         SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(   -0.32),
                         SIMDE_FLOAT64_C(    0.86), SIMDE_FLOAT64_C(    0.92),
                         SIMDE_FLOAT64_C(    0.19), SIMDE_FLOAT64_C(   -0.67)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C(  -30.79)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    0.08),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(   -0.08),
                         SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(   -0.10),
                         SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    0.59)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  -80.73),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -767.23),
                         SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C(  398.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.53), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(    0.11), SIMDE_FLOAT64_C(   -0.63),
                         SIMDE_FLOAT64_C(   -0.96), SIMDE_FLOAT64_C(    0.16)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C(  254.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.86), SIMDE_FLOAT64_C(    0.96),
                         SIMDE_FLOAT64_C(   -0.86), SIMDE_FLOAT64_C(   -0.41),
                         SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.16)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_sin_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_sin_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45),
                         SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(    0.81), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(   -0.39)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  841.21), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  687.09), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -660.80), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -923.64), SIMDE_FLOAT64_C( -860.95)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.67), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   -0.19),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C(   -0.15)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  398.82), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  339.21), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  -30.79), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(  993.90)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C( -387.90),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  532.35),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C( -770.35),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C( -770.72)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.96), SIMDE_FLOAT64_C(    1.00),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(   -0.99),
                         SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    0.61),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(    0.86)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(  469.66),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  910.03),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C( -203.65),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -747.59)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C( -171.51),
                         SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(   -0.96),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(    0.96),
                         SIMDE_FLOAT64_C(   -0.41), SIMDE_FLOAT64_C(    0.16),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C(   -0.74)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   99.93), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(  343.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(  655.67)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  331.34), SIMDE_FLOAT64_C(  462.95),
                         SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  324.62),
                         SIMDE_FLOAT64_C( -874.31), SIMDE_FLOAT64_C( -328.54),
                         SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  561.36)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(   -0.86),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(    0.83)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   27.25),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  831.02), SIMDE_FLOAT64_C(  977.36)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  977.49), SIMDE_FLOAT64_C(  424.81),
                         SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  840.65),
                         SIMDE_FLOAT64_C( -591.56), SIMDE_FLOAT64_C(  731.49),
                         SIMDE_FLOAT64_C(  623.70), SIMDE_FLOAT64_C(  140.67)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   -0.64),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(   -0.81), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    0.65)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C( -304.73),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(  822.06),
                         SIMDE_FLOAT64_C( -997.63), SIMDE_FLOAT64_C(  923.64),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(  -67.64)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  510.85), SIMDE_FLOAT64_C(   14.34),
                         SIMDE_FLOAT64_C(  916.26), SIMDE_FLOAT64_C( -769.09),
                         SIMDE_FLOAT64_C( -573.81), SIMDE_FLOAT64_C( -337.60),
                         SIMDE_FLOAT64_C(  293.64), SIMDE_FLOAT64_C( -576.22)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C(    0.98),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(   -0.56),
                         SIMDE_FLOAT64_C(   -0.89), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(    0.97)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  475.51), SIMDE_FLOAT64_C(  936.65),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C( -438.19),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(  932.66),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C( -182.45)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(  440.64),
                         SIMDE_FLOAT64_C(  897.27), SIMDE_FLOAT64_C( -197.89),
                         SIMDE_FLOAT64_C( -359.76), SIMDE_FLOAT64_C(  -33.67),
                         SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -125.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(    0.73),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C(   -0.03),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(   -0.78),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(    0.45)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_sin_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_sind_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(    -0.23)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.56)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.95)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.52)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.39)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.40)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.86)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(    -0.54)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_sind_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_sincos_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 mem[4];
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.66) },
      { SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.85) },
      { SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.75) } },
    { { SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.86) },
      { SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.53) },
      { SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.51) } },
    { { SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.93) },
      { SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.37) },
      { SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.36) } },
    { { SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.64) },
      { SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.88) },
      { SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.77) } },
    { { SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.77) },
      { SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.69) },
      { SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(    -0.64) } },
    { { SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.91) },
      { SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.42) },
      { SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(    -0.41) } },
    { { SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.92) },
      { SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.40) },
      { SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.39) } },
    { { SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.98) },
      { SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.19) },
      { SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.19) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 mem;
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_sincos_ps(&mem, a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
    simde_test_x86_assert_equal_f32x4(mem, simde_mm_loadu_ps(test_vec[i].mem), 1);
  }

  return 0;
}

static int
test_simde_mm_sincos_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 mem[2];
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(     0.90) },
      { SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.45) },
      { SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.43) } },
    { { SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     0.73) },
      { SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(     0.75) },
      { SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(     0.68) } },
    { { SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(     1.00) },
      { SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(     0.01) },
      { SIMDE_FLOAT64_C(     0.82), SIMDE_FLOAT64_C(     0.01) } },
    { { SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     0.79) },
      { SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(    -0.66) },
      { SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(    -0.61) } },
    { { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.76) },
      { SIMDE_FLOAT64_C(    -0.10), SIMDE_FLOAT64_C(     0.71) },
      { SIMDE_FLOAT64_C(    -0.10), SIMDE_FLOAT64_C(     0.65) } },
    { { SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     0.55) },
      { SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(    -0.99) },
      { SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(    -0.84) } },
    { { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.93) },
      { SIMDE_FLOAT64_C(    -0.02), SIMDE_FLOAT64_C(    -0.37) },
      { SIMDE_FLOAT64_C(    -0.02), SIMDE_FLOAT64_C(    -0.36) } },
    { { SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(     0.90) },
      { SIMDE_FLOAT64_C(    -0.27), SIMDE_FLOAT64_C(     0.44) },
      { SIMDE_FLOAT64_C(    -0.27), SIMDE_FLOAT64_C(     0.43) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d mem;
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_sincos_pd(&mem, a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
    simde_test_x86_assert_equal_f64x2(mem, simde_mm_loadu_pd(test_vec[i].mem), 1);
  }

  return 0;
}

static int
test_simde_mm256_sincos_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 mem[8];
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.58),
        SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.86) },
      { SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(     0.95),
        SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.53) },
      { SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(     0.81),
        SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.51) } },
    { { SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.90),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.76) },
      { SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.45),
        SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.71) },
      { SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.43),
        SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.65) } },
    { { SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.90),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.83) },
      { SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -0.45),
        SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.60) },
      { SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.43),
        SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     0.56) } },
    { { SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.97),
        SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.99) },
      { SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.11) },
      { SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.11) } },
    { { SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.94),
        SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.72) },
      { SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.36),
        SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.77) },
      { SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.35),
        SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.70) } },
    { { SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.71),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.83) },
      { SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    -0.78),
        SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.59) },
      { SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    -0.70),
        SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.56) } },
    { { SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.87),
        SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.93) },
      { SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(    -0.51),
        SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(    -0.37) },
      { SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(    -0.49),
        SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.36) } },
    { { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.97),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.67) },
      { SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.83) },
      { SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.26),
        SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.74) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 mem;
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_sincos_ps(&mem, a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
    simde_test_x86_assert_equal_f32x8(mem, simde_mm256_loadu_ps(test_vec[i].mem), 1);
  }

  return 0;
}

static int
test_simde_mm256_sincos_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 mem[4];
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(     0.79) },
      { SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(    -0.66) },
      { SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(    -0.61) } },
    { { SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     0.97) },
      { SIMDE_FLOAT64_C(     0.46), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(     0.25) },
      { SIMDE_FLOAT64_C(     0.44), SIMDE_FLOAT64_C(    -0.00), SIMDE_FLOAT64_C(     0.63), SIMDE_FLOAT64_C(     0.25) } },
    { { SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(     0.86), SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(     0.72) },
      { SIMDE_FLOAT64_C(    -0.65), SIMDE_FLOAT64_C(    -0.53), SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(    -0.77) },
      { SIMDE_FLOAT64_C(    -0.61), SIMDE_FLOAT64_C(    -0.51), SIMDE_FLOAT64_C(    -0.43), SIMDE_FLOAT64_C(    -0.70) } },
    { { SIMDE_FLOAT64_C(     0.69), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.76) },
      { SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(    -0.09), SIMDE_FLOAT64_C(     0.63), SIMDE_FLOAT64_C(     0.70) },
      { SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(    -0.09), SIMDE_FLOAT64_C(     0.59), SIMDE_FLOAT64_C(     0.64) } },
    { { SIMDE_FLOAT64_C(     0.86), SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     0.76) },
      { SIMDE_FLOAT64_C(    -0.53), SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(    -0.16), SIMDE_FLOAT64_C(     0.71) },
      { SIMDE_FLOAT64_C(    -0.51), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(    -0.16), SIMDE_FLOAT64_C(     0.65) } },
    { { SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.87), SIMDE_FLOAT64_C(     0.64), SIMDE_FLOAT64_C(     0.90) },
      { SIMDE_FLOAT64_C(    -0.99), SIMDE_FLOAT64_C(    -0.52), SIMDE_FLOAT64_C(    -0.87), SIMDE_FLOAT64_C(    -0.44) },
      { SIMDE_FLOAT64_C(    -0.84), SIMDE_FLOAT64_C(    -0.50), SIMDE_FLOAT64_C(    -0.76), SIMDE_FLOAT64_C(    -0.43) } },
    { { SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(     1.00) },
      { SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     0.41), SIMDE_FLOAT64_C(    -0.65), SIMDE_FLOAT64_C(     0.05) },
      { SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.40), SIMDE_FLOAT64_C(    -0.61), SIMDE_FLOAT64_C(     0.05) } },
    { { SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(     0.97), SIMDE_FLOAT64_C(     0.98) },
      { SIMDE_FLOAT64_C(    -0.99), SIMDE_FLOAT64_C(     0.65), SIMDE_FLOAT64_C(    -0.26), SIMDE_FLOAT64_C(     0.20) },
      { SIMDE_FLOAT64_C(    -0.84), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(    -0.26), SIMDE_FLOAT64_C(     0.20) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d mem;
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_sincos_pd(&mem, a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
    simde_test_x86_assert_equal_f64x4(mem, simde_mm256_loadu_pd(test_vec[i].mem), 1);
  }

  return 0;
}

static int
test_simde_mm512_sincos_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 mem[16];
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.97),
        SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.57),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.96),
        SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.69) },
      { SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(    -0.25),
        SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.96),
        SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.28),
        SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.81) },
      { SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(    -0.25),
        SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.82),
        SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.28),
        SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.72) } },
    { { SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.94),
        SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.72),
        SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.84) },
      { SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.03),
        SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(    -0.35),
        SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(    -0.77),
        SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.57) },
      { SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(    -0.03),
        SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.34),
        SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.70),
        SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(     0.54) } },
    { { SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.88),
        SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.94),
        SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.73),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     0.81) },
      { SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.50),
        SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.36),
        SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.75),
        SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.63) },
      { SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.48),
        SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.35),
        SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.68),
        SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.59) } },
    { { SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.87),
        SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.86),
        SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.84),
        SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.58) },
      { SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(    -0.52),
        SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.53),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.58),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.95) },
      { SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -0.50),
        SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.51),
        SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.55),
        SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.81) } },
    { { SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.76),
        SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.92),
        SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.79),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.56) },
      { SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.71),
        SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.41),
        SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(     0.66),
        SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.98) },
      { SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.65),
        SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.40),
        SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.61),
        SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.83) } },
    { { SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.96),
        SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.70),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.86),
        SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.80) },
      { SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.29),
        SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.80),
        SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.54),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(     0.64) },
      { SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.29),
        SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.72),
        SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(    -0.51),
        SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.60) } },
    { { SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.63),
        SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.81),
        SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.64),
        SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     0.78) },
      { SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -0.89),
        SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.63),
        SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.87),
        SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.67) },
      { SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(    -0.78),
        SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.59),
        SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.76),
        SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.62) } },
    { { SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.92),
        SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.95),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(     0.72) },
      { SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -0.61), SIMDE_FLOAT32_C(     0.40),
        SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(    -0.01),
        SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.32),
        SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     0.77) },
      { SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(     0.39),
        SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.01),
        SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(    -0.31),
        SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(     0.70) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 mem;
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_sincos_ps(&mem, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
    simde_test_x86_assert_equal_f32x16(mem, simde_mm512_loadu_ps(test_vec[i].mem), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_sincos_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 mem[16];
    const simde_float32 sin_src[16];
    const simde_float32 cos_src[16];
    const simde__mmask16 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.72),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(    -0.79),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.64) },
      { SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.49),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.73),
        SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.21) },
      { SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(    -0.76),
        SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(    -0.79),
        SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.64) },
      UINT16_C( 4890),
      { SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.76),
        SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     0.28),
        SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.04),
        SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.06) },
      { SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.69),
        SIMDE_FLOAT32_C(    -0.12), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.00),
        SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.73),
        SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.21) } },
    { { SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.34),
        SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(    -0.45),
        SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.09) },
      { SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     0.02),
        SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(    -0.37),
        SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.40),
        SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.84) },
      { SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.34),
        SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(    -0.45),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.49),
        SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.09) },
      UINT16_C(18720),
      { SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     0.27),
        SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.35),
        SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.05),
        SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(    -0.01) },
      { SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(     0.02),
        SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     0.16), SIMDE_FLOAT32_C(    -0.37),
        SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.05),
        SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.84) } },
    { { SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.52),
        SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.54),
        SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.36),
        SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.78) },
      { SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.62),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     0.24),
        SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.77) },
      { SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.52),
        SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.54),
        SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.36),
        SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.78) },
      UINT16_C( 4387),
      { SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.95),
        SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(     0.42),
        SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.73),
        SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.11) },
      { SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.62),
        SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     0.24),
        SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.77) } },
    { { SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     1.00),
        SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.98),
        SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.67),
        SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.96) },
      { SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(    -0.77),
        SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     0.34),
        SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(    -0.34),
        SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(    -0.41) },
      { SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(    -0.49),
        SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.74),
        SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.61),
        SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.39) },
      UINT16_C(36556),
      { SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(    -0.09),
        SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(    -0.18),
        SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.84),
        SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(    -0.29) },
      { SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.09),
        SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.18),
        SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.74),
        SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(    -0.29) } },
    { { SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.40),
        SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.67),
        SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.06),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.86) },
      { SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.08),
        SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.38),
        SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.85),
        SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    -0.86) },
      { SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -0.40),
        SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -0.15),
        SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.06),
        SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(    -0.86) },
      UINT16_C(25479),
      { SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.37),
        SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(     0.84),
        SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.17),
        SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.78) },
      { SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.08),
        SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.74),
        SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.85),
        SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(    -0.86) } },
    { { SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.89),
        SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.94),
        SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.62) },
      { SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.53),
        SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.97),
        SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.05),
        SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(     0.20) },
      { SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.77),
        SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -0.82),
        SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(    -0.89), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.29),
        SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.62) },
      UINT16_C( 2690),
      { SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    -0.08),
        SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -0.48),
        SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.34),
        SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(    -0.52) },
      { SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.53),
        SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.46),
        SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(    -0.33),
        SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(     0.20) } },
    { { SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.35),
        SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(     0.81),
        SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.85),
        SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(     0.90) },
      { SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.92),
        SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.52),
        SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.54) },
      { SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(     0.35),
        SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    -0.94),
        SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.85),
        SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(     0.19), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(     0.75) },
      UINT16_C(41670),
      { SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.84),
        SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(    -0.63),
        SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.41),
        SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.44) },
      { SIMDE_FLOAT32_C(     0.55), SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.16),
        SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.59),
        SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.52),
        SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.43) } },
    { { SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.51),
        SIMDE_FLOAT32_C(     0.77), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(     0.79),
        SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(     0.68),
        SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.54) },
      { SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(     0.84),
        SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.43),
        SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(    -0.24),
        SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.11) },
      { SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.51),
        SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.14), SIMDE_FLOAT32_C(     0.79),
        SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(     0.67),
        SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.54) },
      UINT16_C( 7185),
      { SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(     0.64),
        SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(    -0.23), SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(    -0.93),
        SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(    -0.82),
        SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.31) },
      { SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(     0.84),
        SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.43),
        SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(    -0.73),
        SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.11) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 mem;
    simde__m512 sin_src = simde_mm512_loadu_ps(test_vec[i].sin_src);
    simde__m512 cos_src = simde_mm512_loadu_ps(test_vec[i].cos_src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_sincos_ps(&mem, sin_src, cos_src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
    simde_test_x86_assert_equal_f32x16(mem, simde_mm512_loadu_ps(test_vec[i].mem), 1);
  }

  return 0;
}

static int
test_simde_mm512_sincos_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 mem[8];
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.93), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(     0.82), SIMDE_FLOAT64_C(     1.00) },
      { SIMDE_FLOAT64_C(    -0.37), SIMDE_FLOAT64_C(     0.71), SIMDE_FLOAT64_C(     0.37), SIMDE_FLOAT64_C(    -0.02),
        SIMDE_FLOAT64_C(    -0.16), SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(    -0.61), SIMDE_FLOAT64_C(    -0.06) },
      { SIMDE_FLOAT64_C(    -0.36), SIMDE_FLOAT64_C(     0.65), SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(    -0.02),
        SIMDE_FLOAT64_C(    -0.16), SIMDE_FLOAT64_C(    -0.43), SIMDE_FLOAT64_C(    -0.57), SIMDE_FLOAT64_C(    -0.06) } },
    { { SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     0.88), SIMDE_FLOAT64_C(     0.86),
        SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.86) },
      { SIMDE_FLOAT64_C(     0.70), SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(    -0.50), SIMDE_FLOAT64_C(     0.54),
        SIMDE_FLOAT64_C(    -0.62), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(    -0.09), SIMDE_FLOAT64_C(     0.53) },
      { SIMDE_FLOAT64_C(     0.64), SIMDE_FLOAT64_C(     0.21), SIMDE_FLOAT64_C(    -0.48), SIMDE_FLOAT64_C(     0.51),
        SIMDE_FLOAT64_C(    -0.58), SIMDE_FLOAT64_C(     0.62), SIMDE_FLOAT64_C(    -0.09), SIMDE_FLOAT64_C(     0.51) } },
    { { SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(     0.73), SIMDE_FLOAT64_C(     0.86),
        SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(     0.93) },
      { SIMDE_FLOAT64_C(    -0.09), SIMDE_FLOAT64_C(    -0.45), SIMDE_FLOAT64_C(     0.75), SIMDE_FLOAT64_C(    -0.53),
        SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(    -0.99), SIMDE_FLOAT64_C(    -0.76), SIMDE_FLOAT64_C(    -0.37) },
      { SIMDE_FLOAT64_C(    -0.09), SIMDE_FLOAT64_C(    -0.43), SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(    -0.51),
        SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(    -0.84), SIMDE_FLOAT64_C(    -0.69), SIMDE_FLOAT64_C(    -0.36) } },
    { { SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.86), SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(     0.76),
        SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.93) },
      { SIMDE_FLOAT64_C(    -0.95), SIMDE_FLOAT64_C(    -0.54), SIMDE_FLOAT64_C(     0.29), SIMDE_FLOAT64_C(    -0.71),
        SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(    -0.71), SIMDE_FLOAT64_C(     0.63), SIMDE_FLOAT64_C(    -0.38) },
      { SIMDE_FLOAT64_C(    -0.81), SIMDE_FLOAT64_C(    -0.51), SIMDE_FLOAT64_C(     0.29), SIMDE_FLOAT64_C(    -0.65),
        SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(    -0.65), SIMDE_FLOAT64_C(     0.59), SIMDE_FLOAT64_C(    -0.37) } },
    { { SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(     0.99),
        SIMDE_FLOAT64_C(     0.85), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.71), SIMDE_FLOAT64_C(     0.97) },
      { SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.59), SIMDE_FLOAT64_C(    -0.16),
        SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     0.25) },
      { SIMDE_FLOAT64_C(    -0.84), SIMDE_FLOAT64_C(     0.00), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(    -0.16),
        SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(     0.70), SIMDE_FLOAT64_C(     0.25) } },
    { { SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     0.84),
        SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.99) },
      { SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(    -0.71), SIMDE_FLOAT64_C(    -0.21), SIMDE_FLOAT64_C(     0.57),
        SIMDE_FLOAT64_C(     0.95), SIMDE_FLOAT64_C(     0.70), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(    -0.13) },
      { SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(    -0.65), SIMDE_FLOAT64_C(    -0.21), SIMDE_FLOAT64_C(     0.54),
        SIMDE_FLOAT64_C(     0.81), SIMDE_FLOAT64_C(     0.64), SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(    -0.13) } },
    { { SIMDE_FLOAT64_C(     0.73), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     0.94), SIMDE_FLOAT64_C(     0.61),
        SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(     1.00) },
      { SIMDE_FLOAT64_C(    -0.75), SIMDE_FLOAT64_C(    -0.15), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(    -0.91),
        SIMDE_FLOAT64_C(    -0.14), SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(    -0.09) },
      { SIMDE_FLOAT64_C(    -0.68), SIMDE_FLOAT64_C(    -0.15), SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(    -0.79),
        SIMDE_FLOAT64_C(    -0.14), SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(    -0.09) } },
    { { SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(     0.54), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     0.57),
        SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     0.94), SIMDE_FLOAT64_C(     0.76) },
      { SIMDE_FLOAT64_C(    -0.96), SIMDE_FLOAT64_C(    -1.00), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(    -0.97),
        SIMDE_FLOAT64_C(    -0.70), SIMDE_FLOAT64_C(    -0.17), SIMDE_FLOAT64_C(    -0.35), SIMDE_FLOAT64_C(    -0.70) },
      { SIMDE_FLOAT64_C(    -0.82), SIMDE_FLOAT64_C(    -0.84), SIMDE_FLOAT64_C(     0.20), SIMDE_FLOAT64_C(    -0.82),
        SIMDE_FLOAT64_C(    -0.64), SIMDE_FLOAT64_C(    -0.17), SIMDE_FLOAT64_C(    -0.34), SIMDE_FLOAT64_C(    -0.64) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d mem;
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_sincos_pd(&mem, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
    simde_test_x86_assert_equal_f64x8(mem, simde_mm512_loadu_pd(test_vec[i].mem), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_sincos_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 mem[8];
    const simde_float64 sin_src[8];
    const simde_float64 cos_src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(     0.89), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(    -0.11), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(     0.94), SIMDE_FLOAT64_C(     0.05) },
      { SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(     0.35), SIMDE_FLOAT64_C(    -0.35), SIMDE_FLOAT64_C(     0.72),
        SIMDE_FLOAT64_C(    -0.08), SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(    -0.71), SIMDE_FLOAT64_C(    -0.51) },
      { SIMDE_FLOAT64_C(     0.89), SIMDE_FLOAT64_C(     0.69), SIMDE_FLOAT64_C(    -0.11), SIMDE_FLOAT64_C(    -1.00),
        SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(     0.07), SIMDE_FLOAT64_C(     0.05) },
      UINT8_C( 74),
      { SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(    -0.14), SIMDE_FLOAT64_C(     0.55), SIMDE_FLOAT64_C(     0.05),
        SIMDE_FLOAT64_C(     0.80), SIMDE_FLOAT64_C(     0.65), SIMDE_FLOAT64_C(    -0.35), SIMDE_FLOAT64_C(     0.49) },
      { SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(    -0.14), SIMDE_FLOAT64_C(    -0.35), SIMDE_FLOAT64_C(     0.05),
        SIMDE_FLOAT64_C(    -0.08), SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(    -0.34), SIMDE_FLOAT64_C(    -0.51) } },
    { { SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(    -0.44),
        SIMDE_FLOAT64_C(     0.96), SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.87) },
      { SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(    -0.24), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(    -0.66),
        SIMDE_FLOAT64_C(    -0.17), SIMDE_FLOAT64_C(    -0.55), SIMDE_FLOAT64_C(    -0.83), SIMDE_FLOAT64_C(    -0.82) },
      { SIMDE_FLOAT64_C(     0.10), SIMDE_FLOAT64_C(     0.89), SIMDE_FLOAT64_C(     0.09), SIMDE_FLOAT64_C(    -0.44),
        SIMDE_FLOAT64_C(    -0.82), SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(    -0.55), SIMDE_FLOAT64_C(     0.87) },
      UINT8_C( 82),
      { SIMDE_FLOAT64_C(    -0.55), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     0.19), SIMDE_FLOAT64_C(     0.51),
        SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(     0.17), SIMDE_FLOAT64_C(    -0.39), SIMDE_FLOAT64_C(     0.59) },
      { SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(     0.62), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(    -0.66),
        SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(    -0.55), SIMDE_FLOAT64_C(    -0.38), SIMDE_FLOAT64_C(    -0.82) } },
    { { SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(    -0.10),
        SIMDE_FLOAT64_C(     0.59), SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.48) },
      { SIMDE_FLOAT64_C(    -0.28), SIMDE_FLOAT64_C(     0.66), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(    -0.63),
        SIMDE_FLOAT64_C(    -0.68), SIMDE_FLOAT64_C(    -0.12), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     0.08) },
      { SIMDE_FLOAT64_C(    -0.56), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(     0.90), SIMDE_FLOAT64_C(    -0.10),
        SIMDE_FLOAT64_C(     0.59), SIMDE_FLOAT64_C(    -0.92), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(     0.48) },
      UINT8_C( 33),
      { SIMDE_FLOAT64_C(    -0.44), SIMDE_FLOAT64_C(     0.65), SIMDE_FLOAT64_C(    -0.25), SIMDE_FLOAT64_C(     0.00),
        SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     0.45), SIMDE_FLOAT64_C(     0.20) },
      { SIMDE_FLOAT64_C(    -0.43), SIMDE_FLOAT64_C(     0.66), SIMDE_FLOAT64_C(     0.39), SIMDE_FLOAT64_C(    -0.63),
        SIMDE_FLOAT64_C(    -0.68), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     0.08), SIMDE_FLOAT64_C(     0.08) } },
    { { SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.78),
        SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.70), SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(     0.63) },
      { SIMDE_FLOAT64_C(    -0.59), SIMDE_FLOAT64_C(    -0.03), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.58),
        SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.51), SIMDE_FLOAT64_C(    -0.69), SIMDE_FLOAT64_C(     0.24) },
      { SIMDE_FLOAT64_C(    -0.11), SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(     0.56), SIMDE_FLOAT64_C(     0.78),
        SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(    -0.37), SIMDE_FLOAT64_C(    -0.78), SIMDE_FLOAT64_C(     0.17) },
      UINT8_C(225),
      { SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -0.24), SIMDE_FLOAT64_C(    -0.38), SIMDE_FLOAT64_C(     0.11),
        SIMDE_FLOAT64_C(    -0.76), SIMDE_FLOAT64_C(     0.79), SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(     0.89) },
      { SIMDE_FLOAT64_C(     0.11), SIMDE_FLOAT64_C(    -0.03), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(     0.58),
        SIMDE_FLOAT64_C(     0.58), SIMDE_FLOAT64_C(     0.71), SIMDE_FLOAT64_C(     0.62), SIMDE_FLOAT64_C(     0.78) } },
    { { SIMDE_FLOAT64_C(    -0.47), SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(     1.00),
        SIMDE_FLOAT64_C(    -0.93), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(    -0.07), SIMDE_FLOAT64_C(     0.84) },
      { SIMDE_FLOAT64_C(    -0.46), SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(     0.76),
        SIMDE_FLOAT64_C(    -0.88), SIMDE_FLOAT64_C(    -0.39), SIMDE_FLOAT64_C(    -0.82), SIMDE_FLOAT64_C(     0.09) },
      { SIMDE_FLOAT64_C(    -0.47), SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(    -0.96),
        SIMDE_FLOAT64_C(    -0.93), SIMDE_FLOAT64_C(     0.91), SIMDE_FLOAT64_C(    -0.07), SIMDE_FLOAT64_C(    -0.26) },
      UINT8_C(136),
      { SIMDE_FLOAT64_C(    -0.29), SIMDE_FLOAT64_C(    -0.50), SIMDE_FLOAT64_C(    -0.91), SIMDE_FLOAT64_C(    -0.08),
        SIMDE_FLOAT64_C(     0.67), SIMDE_FLOAT64_C(    -0.37), SIMDE_FLOAT64_C(    -0.96), SIMDE_FLOAT64_C(    -0.57) },
      { SIMDE_FLOAT64_C(    -0.46), SIMDE_FLOAT64_C(    -0.33), SIMDE_FLOAT64_C(     0.42), SIMDE_FLOAT64_C(    -0.08),
        SIMDE_FLOAT64_C(    -0.88), SIMDE_FLOAT64_C(    -0.39), SIMDE_FLOAT64_C(    -0.82), SIMDE_FLOAT64_C(    -0.54) } },
    { { SIMDE_FLOAT64_C(     0.98), SIMDE_FLOAT64_C(     0.94), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(    -0.40),
        SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(    -0.30), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.28) },
      { SIMDE_FLOAT64_C(     0.24), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(     0.03),
        SIMDE_FLOAT64_C(    -0.19), SIMDE_FLOAT64_C(    -0.43), SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(     0.49) },
      { SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.61), SIMDE_FLOAT64_C(    -0.40),
        SIMDE_FLOAT64_C(     0.52), SIMDE_FLOAT64_C(    -0.30), SIMDE_FLOAT64_C(     0.14), SIMDE_FLOAT64_C(     0.28) },
      UINT8_C(  3),
      { SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.34), SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(    -0.89),
        SIMDE_FLOAT64_C(    -0.92), SIMDE_FLOAT64_C(    -0.27), SIMDE_FLOAT64_C(    -0.18), SIMDE_FLOAT64_C(    -0.42) },
      { SIMDE_FLOAT64_C(     0.18), SIMDE_FLOAT64_C(     0.33), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(     0.03),
        SIMDE_FLOAT64_C(    -0.19), SIMDE_FLOAT64_C(    -0.43), SIMDE_FLOAT64_C(     0.57), SIMDE_FLOAT64_C(     0.49) } },
    { { SIMDE_FLOAT64_C(     0.89), SIMDE_FLOAT64_C(     0.66), SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(     0.93),
        SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(     0.97) },
      { SIMDE_FLOAT64_C(    -0.18), SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(    -0.75), SIMDE_FLOAT64_C(     0.45),
        SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(    -0.31), SIMDE_FLOAT64_C(    -0.08) },
      { SIMDE_FLOAT64_C(     0.36), SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(     0.74), SIMDE_FLOAT64_C(     0.93),
        SIMDE_FLOAT64_C(     0.30), SIMDE_FLOAT64_C(     0.22), SIMDE_FLOAT64_C(     0.92), SIMDE_FLOAT64_C(    -0.36) },
      UINT8_C(195),
      { SIMDE_FLOAT64_C(    -0.48), SIMDE_FLOAT64_C(    -0.85), SIMDE_FLOAT64_C(     0.53), SIMDE_FLOAT64_C(     0.66),
        SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(    -0.11), SIMDE_FLOAT64_C(    -0.17), SIMDE_FLOAT64_C(    -0.23) },
      { SIMDE_FLOAT64_C(    -0.46), SIMDE_FLOAT64_C(    -0.75), SIMDE_FLOAT64_C(    -0.75), SIMDE_FLOAT64_C(     0.45),
        SIMDE_FLOAT64_C(     0.78), SIMDE_FLOAT64_C(    -0.32), SIMDE_FLOAT64_C(    -0.17), SIMDE_FLOAT64_C(    -0.23) } },
    { { SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(    -0.64),
        SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(    -0.80), SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(     0.56) },
      { SIMDE_FLOAT64_C(    -0.84), SIMDE_FLOAT64_C(    -0.06), SIMDE_FLOAT64_C(    -0.15), SIMDE_FLOAT64_C(    -0.11),
        SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.70), SIMDE_FLOAT64_C(     0.50) },
      { SIMDE_FLOAT64_C(     0.68), SIMDE_FLOAT64_C(     0.15), SIMDE_FLOAT64_C(     0.27), SIMDE_FLOAT64_C(    -0.64),
        SIMDE_FLOAT64_C(     0.84), SIMDE_FLOAT64_C(    -0.80), SIMDE_FLOAT64_C(     0.72), SIMDE_FLOAT64_C(     0.56) },
      UINT8_C(  0),
      { SIMDE_FLOAT64_C(     0.65), SIMDE_FLOAT64_C(    -0.14), SIMDE_FLOAT64_C(     0.16), SIMDE_FLOAT64_C(     0.56),
        SIMDE_FLOAT64_C(     0.49), SIMDE_FLOAT64_C(     0.99), SIMDE_FLOAT64_C(    -0.92), SIMDE_FLOAT64_C(     0.64) },
      { SIMDE_FLOAT64_C(    -0.84), SIMDE_FLOAT64_C(    -0.06), SIMDE_FLOAT64_C(    -0.15), SIMDE_FLOAT64_C(    -0.11),
        SIMDE_FLOAT64_C(     0.76), SIMDE_FLOAT64_C(     0.43), SIMDE_FLOAT64_C(     0.70), SIMDE_FLOAT64_C(     0.50) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d mem;
    simde__m512d sin_src = simde_mm512_loadu_pd(test_vec[i].sin_src);
    simde__m512d cos_src = simde_mm512_loadu_pd(test_vec[i].cos_src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_sincos_pd(&mem, sin_src, cos_src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
    simde_test_x86_assert_equal_f64x8(mem, simde_mm512_loadu_pd(test_vec[i].mem), 1);
  }

  return 0;
}

static int
test_simde_mm_sind_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.56), SIMDE_FLOAT64_C(   -0.23)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.11), SIMDE_FLOAT64_C(    0.63)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.89), SIMDE_FLOAT64_C(    0.56)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(   -0.76)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    0.95)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.96), SIMDE_FLOAT64_C(    0.88)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(   -0.52)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    0.56)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_sind_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_sind_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24),
                         SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01),
                         SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.76),
                         SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.56),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.63),
                         SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(    -0.23)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13),
                         SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21),
                         SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.56),
                         SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.52),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.88),
                         SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.95)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95),
                         SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.48),
                         SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.63),
                         SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.39)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67),
                         SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54),
                         SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(    -0.54),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.54),
                         SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.86)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48),
                         SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.35),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.86)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92),
                         SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(    -0.35),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    -0.51)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73),
                         SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.70),
                         SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.73),
                         SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.63)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02),
                         SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(    -0.64),
                         SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.87),
                         SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(    -0.96)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_sind_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_sind_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.11), SIMDE_FLOAT64_C(    0.63),
                         SIMDE_FLOAT64_C(   -0.56), SIMDE_FLOAT64_C(   -0.23)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(   -0.76),
                         SIMDE_FLOAT64_C(    0.89), SIMDE_FLOAT64_C(    0.56)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.96), SIMDE_FLOAT64_C(    0.88),
                         SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    0.95)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    0.56),
                         SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(   -0.52)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.82), SIMDE_FLOAT64_C(   -0.63),
                         SIMDE_FLOAT64_C(   -0.84), SIMDE_FLOAT64_C(   -0.39)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.48),
                         SIMDE_FLOAT64_C(   -0.41), SIMDE_FLOAT64_C(    0.40)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(    0.54),
                         SIMDE_FLOAT64_C(    0.97), SIMDE_FLOAT64_C(    0.86)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.03), SIMDE_FLOAT64_C(   -1.00),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.54)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_sind_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_sind_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(    -0.52),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.95),
                         SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.56),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(    -0.23)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(    -0.54),
                         SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.86),
                         SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.39)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.47), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(    -0.35),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    -0.51),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.35),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.86)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(    -0.96),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.70),
                         SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(   343.48),
                         SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   655.67),
                         SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(   120.65), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.28),
                         SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(    -0.24),
                         SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.90),
                         SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(    -0.06), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(    -0.15)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -591.56),
                         SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(  -906.16),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -738.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.86), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(    -0.31)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(   977.49),
                         SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.74),
                         SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.98),
                         SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -1.00)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -696.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(    -0.55),
                         SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(    -0.82),
                         SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.28), SIMDE_FLOAT32_C(     0.40)) }

  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_sind_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_sind_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(    -0.84),
                         SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     1.00), SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     1.00),
                         SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(   346.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19),
                         SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   254.31), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(   398.82), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   -30.79), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.70), SIMDE_FLOAT32_C(    -0.73),
                         SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(    -0.51), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    -1.00)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    99.93),
                         SIMDE_FLOAT32_C(  -738.19), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -448.89),
                         SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(   331.34),
                         SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -874.31),
                         SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   -70.91)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    -0.48),
                         SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -337.60),
                         SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(  -756.42)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.98)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -737.13), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(   177.92),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   888.71), SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   133.52),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(  -775.04), SIMDE_FLOAT32_C(   440.64)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(   915.19), SIMDE_FLOAT32_C(  -718.40), SIMDE_FLOAT32_C(   159.97),
                         SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   110.36),
                         SIMDE_FLOAT32_C(   826.84), SIMDE_FLOAT32_C(   -76.75), SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -378.50),
                         SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(  -942.47), SIMDE_FLOAT32_C(   475.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.94),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(   440.64)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(  -807.28), SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(   834.60), SIMDE_FLOAT32_C(   -65.60),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   556.35), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -964.25), SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(  -743.66), SIMDE_FLOAT32_C(  -764.58),
                         SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(  -818.54), SIMDE_FLOAT32_C(   161.06),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -308.52), SIMDE_FLOAT32_C(  -719.57),
                         SIMDE_FLOAT32_C(   334.00), SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(  -916.82), SIMDE_FLOAT32_C(  -490.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(     0.32),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(    -1.00), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   105.79), SIMDE_FLOAT32_C(   590.10),
                         SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(   -84.00), SIMDE_FLOAT32_C(    80.04),
                         SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(  -889.11)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(   144.05), SIMDE_FLOAT32_C(  -427.72),
                         SIMDE_FLOAT32_C(   308.28), SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(  -457.77), SIMDE_FLOAT32_C(   678.24),
                         SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   117.28), SIMDE_FLOAT32_C(  -576.80),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   -53.92), SIMDE_FLOAT32_C(    91.94)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -0.67),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(     1.00)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(  -493.41), SIMDE_FLOAT32_C(   822.72),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(  -816.27),
                         SIMDE_FLOAT32_C(  -209.34), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(  -204.33)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -841.43), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(   824.88), SIMDE_FLOAT32_C(   793.63),
                         SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -350.72),
                         SIMDE_FLOAT32_C(    60.89), SIMDE_FLOAT32_C(   109.81), SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -250.60),
                         SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   -13.07), SIMDE_FLOAT32_C(   852.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(     0.96),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(     0.74)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_sind_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_sind_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(   -0.76),
                         SIMDE_FLOAT64_C(    0.89), SIMDE_FLOAT64_C(    0.56),
                         SIMDE_FLOAT64_C(    0.11), SIMDE_FLOAT64_C(    0.63),
                         SIMDE_FLOAT64_C(   -0.56), SIMDE_FLOAT64_C(   -0.23)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    0.56),
                         SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(   -0.52),
                         SIMDE_FLOAT64_C(    0.96), SIMDE_FLOAT64_C(    0.88),
                         SIMDE_FLOAT64_C(    1.00), SIMDE_FLOAT64_C(    0.95)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.48),
                         SIMDE_FLOAT64_C(   -0.41), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.82), SIMDE_FLOAT64_C(   -0.63),
                         SIMDE_FLOAT64_C(   -0.84), SIMDE_FLOAT64_C(   -0.39)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.03), SIMDE_FLOAT64_C(   -1.00),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.54),
                         SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(    0.54),
                         SIMDE_FLOAT64_C(    0.97), SIMDE_FLOAT64_C(    0.86)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(  993.90),
                         SIMDE_FLOAT64_C(   28.08), SIMDE_FLOAT64_C(  841.21)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(    0.35),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(   -1.00),
                         SIMDE_FLOAT64_C(    0.47), SIMDE_FLOAT64_C(    0.86)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C(  -30.79)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.47), SIMDE_FLOAT64_C(    0.59),
                         SIMDE_FLOAT64_C(   -0.90), SIMDE_FLOAT64_C(   -0.35),
                         SIMDE_FLOAT64_C(    0.13), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.87), SIMDE_FLOAT64_C(   -0.51)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  -80.73),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -767.23),
                         SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C(  398.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(   -0.99),
                         SIMDE_FLOAT64_C(   -0.40), SIMDE_FLOAT64_C(    0.70),
                         SIMDE_FLOAT64_C(   -0.46), SIMDE_FLOAT64_C(   -0.73),
                         SIMDE_FLOAT64_C(    0.25), SIMDE_FLOAT64_C(    0.63)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C(  254.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.94), SIMDE_FLOAT64_C(   -0.64),
                         SIMDE_FLOAT64_C(   -0.52), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(   -0.17), SIMDE_FLOAT64_C(   -0.87),
                         SIMDE_FLOAT64_C(    0.95), SIMDE_FLOAT64_C(   -0.96)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_sind_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_sind_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45),
                         SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(    0.11), SIMDE_FLOAT64_C(   -0.56)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  841.21), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  687.09), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -660.80), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -923.64), SIMDE_FLOAT64_C( -860.95)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.86), SIMDE_FLOAT64_C(   -1.00),
                         SIMDE_FLOAT64_C(   -0.54), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(    0.48),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C(   -0.63)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  398.82), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  339.21), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  -30.79), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(  993.90)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C( -387.90),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  532.35),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C( -770.35),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C( -770.72)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.25), SIMDE_FLOAT64_C(   -0.47),
                         SIMDE_FLOAT64_C(   -0.90), SIMDE_FLOAT64_C(    0.13),
                         SIMDE_FLOAT64_C(    0.87), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(   -0.77)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(  469.66),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  910.03),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C( -203.65),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -747.59)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C( -171.51),
                         SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(   -0.87), SIMDE_FLOAT64_C(   -0.96),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C(    0.70)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   99.93), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(  343.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(  655.67)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  331.34), SIMDE_FLOAT64_C(  462.95),
                         SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  324.62),
                         SIMDE_FLOAT64_C( -874.31), SIMDE_FLOAT64_C( -328.54),
                         SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  561.36)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.48), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(   -0.58),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(   -0.36)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   27.25),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  831.02), SIMDE_FLOAT64_C(  977.36)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  977.49), SIMDE_FLOAT64_C(  424.81),
                         SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  840.65),
                         SIMDE_FLOAT64_C( -591.56), SIMDE_FLOAT64_C(  731.49),
                         SIMDE_FLOAT64_C(  623.70), SIMDE_FLOAT64_C(  140.67)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(    0.90),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(    0.63)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C( -304.73),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(  822.06),
                         SIMDE_FLOAT64_C( -997.63), SIMDE_FLOAT64_C(  923.64),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(  -67.64)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  510.85), SIMDE_FLOAT64_C(   14.34),
                         SIMDE_FLOAT64_C(  916.26), SIMDE_FLOAT64_C( -769.09),
                         SIMDE_FLOAT64_C( -573.81), SIMDE_FLOAT64_C( -337.60),
                         SIMDE_FLOAT64_C(  293.64), SIMDE_FLOAT64_C( -576.22)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C(    0.25),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(   -0.76),
                         SIMDE_FLOAT64_C(    0.56), SIMDE_FLOAT64_C(    0.38),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(    0.59)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  475.51), SIMDE_FLOAT64_C(  936.65),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C( -438.19),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(  932.66),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C( -182.45)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(  440.64),
                         SIMDE_FLOAT64_C(  897.27), SIMDE_FLOAT64_C( -197.89),
                         SIMDE_FLOAT64_C( -359.76), SIMDE_FLOAT64_C(  -33.67),
                         SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -125.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.82), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C(    0.31),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(   -0.55),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(   -0.82)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_sind_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_sinh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     4.79)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     6.05), SIMDE_FLOAT32_C(    16.05), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    60.15)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     5.44), SIMDE_FLOAT32_C(     6.18), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     3.45)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(   115.22), SIMDE_FLOAT32_C(   241.49), SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(    15.73)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     5.12), SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     5.31)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(   471.94), SIMDE_FLOAT32_C(    83.66), SIMDE_FLOAT32_C(     4.19), SIMDE_FLOAT32_C(   101.17)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     3.66), SIMDE_FLOAT32_C(     5.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    19.42), SIMDE_FLOAT32_C(   158.67)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     6.30)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.59), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     2.13), SIMDE_FLOAT32_C(   272.29)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(    -0.67)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(    15.27), SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(    -0.72)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(     0.46)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    41.54), SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(    -1.03), SIMDE_FLOAT32_C(     0.48)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     4.07), SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     4.30), SIMDE_FLOAT32_C(     6.25)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    29.27), SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(    36.84), SIMDE_FLOAT32_C(   259.01)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_sinh_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_sinh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.06), SIMDE_FLOAT64_C(    4.79)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.06), SIMDE_FLOAT64_C(   60.15)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    2.50), SIMDE_FLOAT64_C(    3.47)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    6.05), SIMDE_FLOAT64_C(   16.05)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    2.02), SIMDE_FLOAT64_C(    3.45)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    3.70), SIMDE_FLOAT64_C(   15.73)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    5.44), SIMDE_FLOAT64_C(    6.18)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(  115.22), SIMDE_FLOAT64_C(  241.49)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    2.14), SIMDE_FLOAT64_C(    5.31)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    4.19), SIMDE_FLOAT64_C(  101.17)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    6.85), SIMDE_FLOAT64_C(    5.12)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(  471.94), SIMDE_FLOAT64_C(   83.66)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    3.66), SIMDE_FLOAT64_C(    5.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   19.42), SIMDE_FLOAT64_C(  158.67)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(    0.35)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(    0.36)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_sinh_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_sinh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     5.44), SIMDE_FLOAT32_C(     6.18),
                         SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     3.45),
                         SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(     3.47),
                         SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     4.79)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   115.22), SIMDE_FLOAT32_C(   241.49),
                         SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(    15.73),
                         SIMDE_FLOAT32_C(     6.05), SIMDE_FLOAT32_C(    16.05),
                         SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    60.15)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.35),
                         SIMDE_FLOAT32_C(     3.66), SIMDE_FLOAT32_C(     5.76),
                         SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     5.12),
                         SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     5.31)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.36),
                         SIMDE_FLOAT32_C(    19.42), SIMDE_FLOAT32_C(   158.67),
                         SIMDE_FLOAT32_C(   471.94), SIMDE_FLOAT32_C(    83.66),
                         SIMDE_FLOAT32_C(     4.19), SIMDE_FLOAT32_C(   101.17)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     3.42),
                         SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(    -0.67),
                         SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(    -0.40),
                         SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     6.30)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(    15.27),
                         SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(    -0.72),
                         SIMDE_FLOAT32_C(     3.59), SIMDE_FLOAT32_C(    -0.41),
                         SIMDE_FLOAT32_C(     2.13), SIMDE_FLOAT32_C(   272.29)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     4.07), SIMDE_FLOAT32_C(     1.36),
                         SIMDE_FLOAT32_C(     4.30), SIMDE_FLOAT32_C(     6.25),
                         SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     2.39),
                         SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(     0.46)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    29.27), SIMDE_FLOAT32_C(     1.82),
                         SIMDE_FLOAT32_C(    36.84), SIMDE_FLOAT32_C(   259.01),
                         SIMDE_FLOAT32_C(    41.54), SIMDE_FLOAT32_C(     5.41),
                         SIMDE_FLOAT32_C(    -1.03), SIMDE_FLOAT32_C(     0.48)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     5.21),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     4.94),
                         SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     7.57),
                         SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     6.92)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    91.54),
                         SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    69.88),
                         SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(   969.57),
                         SIMDE_FLOAT32_C(    15.27), SIMDE_FLOAT32_C(   506.16)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.63), SIMDE_FLOAT32_C(     5.00),
                         SIMDE_FLOAT32_C(     6.12), SIMDE_FLOAT32_C(     4.76),
                         SIMDE_FLOAT32_C(     5.59), SIMDE_FLOAT32_C(     2.16),
                         SIMDE_FLOAT32_C(     6.66), SIMDE_FLOAT32_C(     3.17)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(    74.20),
                         SIMDE_FLOAT32_C(   227.43), SIMDE_FLOAT32_C(    58.37),
                         SIMDE_FLOAT32_C(   133.87), SIMDE_FLOAT32_C(     4.28),
                         SIMDE_FLOAT32_C(   390.27), SIMDE_FLOAT32_C(    11.88)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     2.95),
                         SIMDE_FLOAT32_C(     4.75), SIMDE_FLOAT32_C(    -0.76),
                         SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     5.01)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     5.58), SIMDE_FLOAT32_C(     9.53),
                         SIMDE_FLOAT32_C(    57.79), SIMDE_FLOAT32_C(    -0.84),
                         SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(    74.95)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     5.32), SIMDE_FLOAT32_C(     6.22),
                         SIMDE_FLOAT32_C(     2.66), SIMDE_FLOAT32_C(     6.82),
                         SIMDE_FLOAT32_C(     7.21), SIMDE_FLOAT32_C(     5.88),
                         SIMDE_FLOAT32_C(     6.70), SIMDE_FLOAT32_C(     4.39)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   102.19), SIMDE_FLOAT32_C(   251.35),
                         SIMDE_FLOAT32_C(     7.11), SIMDE_FLOAT32_C(   457.99),
                         SIMDE_FLOAT32_C(   676.45), SIMDE_FLOAT32_C(   178.90),
                         SIMDE_FLOAT32_C(   406.20), SIMDE_FLOAT32_C(    40.31)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_sinh_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_sinh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    2.50), SIMDE_FLOAT64_C(    3.47),
                         SIMDE_FLOAT64_C(    0.06), SIMDE_FLOAT64_C(    4.79)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    6.05), SIMDE_FLOAT64_C(   16.05),
                         SIMDE_FLOAT64_C(    0.06), SIMDE_FLOAT64_C(   60.15)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    5.44), SIMDE_FLOAT64_C(    6.18),
                         SIMDE_FLOAT64_C(    2.02), SIMDE_FLOAT64_C(    3.45)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(  115.22), SIMDE_FLOAT64_C(  241.49),
                         SIMDE_FLOAT64_C(    3.70), SIMDE_FLOAT64_C(   15.73)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    6.85), SIMDE_FLOAT64_C(    5.12),
                         SIMDE_FLOAT64_C(    2.14), SIMDE_FLOAT64_C(    5.31)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(  471.94), SIMDE_FLOAT64_C(   83.66),
                         SIMDE_FLOAT64_C(    4.19), SIMDE_FLOAT64_C(  101.17)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(    0.35),
                         SIMDE_FLOAT64_C(    3.66), SIMDE_FLOAT64_C(    5.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(    0.36),
                         SIMDE_FLOAT64_C(   19.42), SIMDE_FLOAT64_C(  158.67)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.99), SIMDE_FLOAT64_C(   -0.40),
                         SIMDE_FLOAT64_C(    1.50), SIMDE_FLOAT64_C(    6.30)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.59), SIMDE_FLOAT64_C(   -0.41),
                         SIMDE_FLOAT64_C(    2.13), SIMDE_FLOAT64_C(  272.29)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.39), SIMDE_FLOAT64_C(    3.42),
                         SIMDE_FLOAT64_C(    1.65), SIMDE_FLOAT64_C(   -0.67)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.88), SIMDE_FLOAT64_C(   15.27),
                         SIMDE_FLOAT64_C(    2.51), SIMDE_FLOAT64_C(   -0.72)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    4.42), SIMDE_FLOAT64_C(    2.39),
                         SIMDE_FLOAT64_C(   -0.90), SIMDE_FLOAT64_C(    0.46)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   41.54), SIMDE_FLOAT64_C(    5.41),
                         SIMDE_FLOAT64_C(   -1.03), SIMDE_FLOAT64_C(    0.48)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    4.07), SIMDE_FLOAT64_C(    1.36),
                         SIMDE_FLOAT64_C(    4.30), SIMDE_FLOAT64_C(    6.25)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   29.27), SIMDE_FLOAT64_C(    1.82),
                         SIMDE_FLOAT64_C(   36.84), SIMDE_FLOAT64_C(  259.01)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_sinh_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_sinh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     3.66), SIMDE_FLOAT32_C(     5.76),
                         SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     5.12), SIMDE_FLOAT32_C(     2.14), SIMDE_FLOAT32_C(     5.31),
                         SIMDE_FLOAT32_C(     5.44), SIMDE_FLOAT32_C(     6.18), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     3.45),
                         SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     4.79)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    19.42), SIMDE_FLOAT32_C(   158.67),
                         SIMDE_FLOAT32_C(   471.94), SIMDE_FLOAT32_C(    83.66), SIMDE_FLOAT32_C(     4.19), SIMDE_FLOAT32_C(   101.17),
                         SIMDE_FLOAT32_C(   115.22), SIMDE_FLOAT32_C(   241.49), SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(    15.73),
                         SIMDE_FLOAT32_C(     6.05), SIMDE_FLOAT32_C(    16.05), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    60.15)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.07), SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     4.30), SIMDE_FLOAT32_C(     6.25),
                         SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(    -0.90), SIMDE_FLOAT32_C(     0.46),
                         SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(    -0.67),
                         SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     1.50), SIMDE_FLOAT32_C(     6.30)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    29.27), SIMDE_FLOAT32_C(     1.82), SIMDE_FLOAT32_C(    36.84), SIMDE_FLOAT32_C(   259.01),
                         SIMDE_FLOAT32_C(    41.54), SIMDE_FLOAT32_C(     5.41), SIMDE_FLOAT32_C(    -1.03), SIMDE_FLOAT32_C(     0.48),
                         SIMDE_FLOAT32_C(     1.88), SIMDE_FLOAT32_C(    15.27), SIMDE_FLOAT32_C(     2.51), SIMDE_FLOAT32_C(    -0.72),
                         SIMDE_FLOAT32_C(     3.59), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     2.13), SIMDE_FLOAT32_C(   272.29)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.63), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     6.12), SIMDE_FLOAT32_C(     4.76),
                         SIMDE_FLOAT32_C(     5.59), SIMDE_FLOAT32_C(     2.16), SIMDE_FLOAT32_C(     6.66), SIMDE_FLOAT32_C(     3.17),
                         SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     5.21), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     4.94),
                         SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     7.57), SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     6.92)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(    74.20), SIMDE_FLOAT32_C(   227.43), SIMDE_FLOAT32_C(    58.37),
                         SIMDE_FLOAT32_C(   133.87), SIMDE_FLOAT32_C(     4.28), SIMDE_FLOAT32_C(   390.27), SIMDE_FLOAT32_C(    11.88),
                         SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    91.54), SIMDE_FLOAT32_C(     0.87), SIMDE_FLOAT32_C(    69.88),
                         SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(   969.57), SIMDE_FLOAT32_C(    15.27), SIMDE_FLOAT32_C(   506.16)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.32), SIMDE_FLOAT32_C(     6.22), SIMDE_FLOAT32_C(     2.66), SIMDE_FLOAT32_C(     6.82),
                         SIMDE_FLOAT32_C(     7.21), SIMDE_FLOAT32_C(     5.88), SIMDE_FLOAT32_C(     6.70), SIMDE_FLOAT32_C(     4.39),
                         SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(     4.75), SIMDE_FLOAT32_C(    -0.76),
                         SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     5.01)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   102.19), SIMDE_FLOAT32_C(   251.35), SIMDE_FLOAT32_C(     7.11), SIMDE_FLOAT32_C(   457.99),
                         SIMDE_FLOAT32_C(   676.45), SIMDE_FLOAT32_C(   178.90), SIMDE_FLOAT32_C(   406.20), SIMDE_FLOAT32_C(    40.31),
                         SIMDE_FLOAT32_C(     5.58), SIMDE_FLOAT32_C(     9.53), SIMDE_FLOAT32_C(    57.79), SIMDE_FLOAT32_C(    -0.84),
                         SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(     1.06), SIMDE_FLOAT32_C(    74.95)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     6.56), SIMDE_FLOAT32_C(     4.70), SIMDE_FLOAT32_C(     4.78),
                         SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     1.04),
                         SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     5.71), SIMDE_FLOAT32_C(     6.12),
                         SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(     5.64), SIMDE_FLOAT32_C(     3.82), SIMDE_FLOAT32_C(     2.56)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     6.24), SIMDE_FLOAT32_C(   353.14), SIMDE_FLOAT32_C(    54.97), SIMDE_FLOAT32_C(    59.55),
                         SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     3.23), SIMDE_FLOAT32_C(     1.24),
                         SIMDE_FLOAT32_C(     5.87), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(   150.93), SIMDE_FLOAT32_C(   227.43),
                         SIMDE_FLOAT32_C(    10.02), SIMDE_FLOAT32_C(   140.73), SIMDE_FLOAT32_C(    22.79), SIMDE_FLOAT32_C(     6.43)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     6.27), SIMDE_FLOAT32_C(     6.91), SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     0.76),
                         SIMDE_FLOAT32_C(     1.37), SIMDE_FLOAT32_C(     6.45), SIMDE_FLOAT32_C(     5.47), SIMDE_FLOAT32_C(     5.98),
                         SIMDE_FLOAT32_C(     6.87), SIMDE_FLOAT32_C(     3.90), SIMDE_FLOAT32_C(     7.50), SIMDE_FLOAT32_C(    -0.60),
                         SIMDE_FLOAT32_C(     4.72), SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(     5.29), SIMDE_FLOAT32_C(     0.13)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   264.24), SIMDE_FLOAT32_C(   501.12), SIMDE_FLOAT32_C(    12.37), SIMDE_FLOAT32_C(     0.84),
                         SIMDE_FLOAT32_C(     1.84), SIMDE_FLOAT32_C(   316.35), SIMDE_FLOAT32_C(   118.73), SIMDE_FLOAT32_C(   197.72),
                         SIMDE_FLOAT32_C(   481.47), SIMDE_FLOAT32_C(    24.69), SIMDE_FLOAT32_C(   904.02), SIMDE_FLOAT32_C(    -0.64),
                         SIMDE_FLOAT32_C(    56.08), SIMDE_FLOAT32_C(    20.83), SIMDE_FLOAT32_C(    99.17), SIMDE_FLOAT32_C(     0.13)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     6.83), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(     1.85), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(     4.56), SIMDE_FLOAT32_C(    -0.00),
                         SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     3.01), SIMDE_FLOAT32_C(     6.35), SIMDE_FLOAT32_C(     7.50),
                         SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(     5.13), SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     2.89)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(   462.59), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(    -1.16),
                         SIMDE_FLOAT32_C(     3.10), SIMDE_FLOAT32_C(   718.27), SIMDE_FLOAT32_C(    47.79), SIMDE_FLOAT32_C(    -0.00),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    10.12), SIMDE_FLOAT32_C(   286.25), SIMDE_FLOAT32_C(   904.02),
                         SIMDE_FLOAT32_C(     0.05), SIMDE_FLOAT32_C(    84.51), SIMDE_FLOAT32_C(    15.27), SIMDE_FLOAT32_C(     8.97)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     3.16),
                         SIMDE_FLOAT32_C(     7.31), SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     2.76),
                         SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     5.50), SIMDE_FLOAT32_C(     5.00),
                         SIMDE_FLOAT32_C(     3.36), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(     0.30)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     2.79), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    11.76),
                         SIMDE_FLOAT32_C(   747.59), SIMDE_FLOAT32_C(    13.95), SIMDE_FLOAT32_C(     3.23), SIMDE_FLOAT32_C(     7.87),
                         SIMDE_FLOAT32_C(     6.17), SIMDE_FLOAT32_C(    16.05), SIMDE_FLOAT32_C(   122.34), SIMDE_FLOAT32_C(    74.20),
                         SIMDE_FLOAT32_C(    14.38), SIMDE_FLOAT32_C(     3.59), SIMDE_FLOAT32_C(   697.05), SIMDE_FLOAT32_C(     0.30)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_sinh_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_sinh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     6.25), SIMDE_FLOAT32_C(     2.39), SIMDE_FLOAT32_C(     0.46),
                         SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     6.30),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     5.76), SIMDE_FLOAT32_C(     5.12), SIMDE_FLOAT32_C(     5.31),
                         SIMDE_FLOAT32_C(     6.18), SIMDE_FLOAT32_C(     3.45), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     4.79)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     4.07), SIMDE_FLOAT32_C(     4.30), SIMDE_FLOAT32_C(     4.42), SIMDE_FLOAT32_C(    -0.90),
                         SIMDE_FLOAT32_C(     1.39), SIMDE_FLOAT32_C(     1.65), SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     1.50),
                         SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     3.66), SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     2.14),
                         SIMDE_FLOAT32_C(     5.44), SIMDE_FLOAT32_C(     2.02), SIMDE_FLOAT32_C(     2.50), SIMDE_FLOAT32_C(     0.06)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    29.27), SIMDE_FLOAT32_C(     6.25), SIMDE_FLOAT32_C(    41.54), SIMDE_FLOAT32_C(     0.46),
                         SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     2.13),
                         SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(    19.42), SIMDE_FLOAT32_C(   471.94), SIMDE_FLOAT32_C(     4.19),
                         SIMDE_FLOAT32_C(   115.22), SIMDE_FLOAT32_C(     3.45), SIMDE_FLOAT32_C(     6.05), SIMDE_FLOAT32_C(     4.79)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.32), SIMDE_FLOAT32_C(     2.66), SIMDE_FLOAT32_C(     7.21), SIMDE_FLOAT32_C(     6.70),
                         SIMDE_FLOAT32_C(     2.42), SIMDE_FLOAT32_C(     4.75), SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.92),
                         SIMDE_FLOAT32_C(     1.63), SIMDE_FLOAT32_C(     6.12), SIMDE_FLOAT32_C(     5.59), SIMDE_FLOAT32_C(     6.66),
                         SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     3.42)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.56), SIMDE_FLOAT32_C(     6.22), SIMDE_FLOAT32_C(     6.82), SIMDE_FLOAT32_C(     5.88),
                         SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(     2.95), SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(     5.01), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(     4.76), SIMDE_FLOAT32_C(     2.16),
                         SIMDE_FLOAT32_C(     3.17), SIMDE_FLOAT32_C(     5.21), SIMDE_FLOAT32_C(     4.94), SIMDE_FLOAT32_C(     7.57)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     6.43), SIMDE_FLOAT32_C(     2.66), SIMDE_FLOAT32_C(     7.21), SIMDE_FLOAT32_C(     6.70),
                         SIMDE_FLOAT32_C(    40.31), SIMDE_FLOAT32_C(     9.53), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     0.00),
                         SIMDE_FLOAT32_C(    74.95), SIMDE_FLOAT32_C(     6.12), SIMDE_FLOAT32_C(    58.37), SIMDE_FLOAT32_C(     4.28),
                         SIMDE_FLOAT32_C(    11.88), SIMDE_FLOAT32_C(    91.54), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(   969.57)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.89), SIMDE_FLOAT32_C(     6.91), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     6.45),
                         SIMDE_FLOAT32_C(     5.98), SIMDE_FLOAT32_C(     3.90), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     3.73),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     6.56), SIMDE_FLOAT32_C(     4.78), SIMDE_FLOAT32_C(    -0.13),
                         SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(     6.12), SIMDE_FLOAT32_C(     5.64)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(     6.27), SIMDE_FLOAT32_C(     3.21), SIMDE_FLOAT32_C(     1.37),
                         SIMDE_FLOAT32_C(     5.47), SIMDE_FLOAT32_C(     6.87), SIMDE_FLOAT32_C(     7.50), SIMDE_FLOAT32_C(     4.72),
                         SIMDE_FLOAT32_C(     5.29), SIMDE_FLOAT32_C(     2.53), SIMDE_FLOAT32_C(     4.70), SIMDE_FLOAT32_C(    -0.46),
                         SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     2.47), SIMDE_FLOAT32_C(     5.71), SIMDE_FLOAT32_C(     3.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     2.89), SIMDE_FLOAT32_C(   264.24), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     6.45),
                         SIMDE_FLOAT32_C(     5.98), SIMDE_FLOAT32_C(     3.90), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    56.08),
                         SIMDE_FLOAT32_C(    99.17), SIMDE_FLOAT32_C(     6.56), SIMDE_FLOAT32_C(    54.97), SIMDE_FLOAT32_C(    -0.13),
                         SIMDE_FLOAT32_C(     1.04), SIMDE_FLOAT32_C(     5.87), SIMDE_FLOAT32_C(     6.12), SIMDE_FLOAT32_C(     5.64)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.80), SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     7.31),
                         SIMDE_FLOAT32_C(     1.89), SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(     5.50), SIMDE_FLOAT32_C(     3.36),
                         SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     1.85),
                         SIMDE_FLOAT32_C(     4.56), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     6.35), SIMDE_FLOAT32_C(     0.05)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.16), SIMDE_FLOAT32_C(     2.45), SIMDE_FLOAT32_C(     1.75), SIMDE_FLOAT32_C(     3.16),
                         SIMDE_FLOAT32_C(     3.33), SIMDE_FLOAT32_C(     2.76), SIMDE_FLOAT32_C(     3.47), SIMDE_FLOAT32_C(     5.00),
                         SIMDE_FLOAT32_C(     1.99), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(     6.83), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(    -0.00), SIMDE_FLOAT32_C(     3.01), SIMDE_FLOAT32_C(     7.50)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     1.80), SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     7.31),
                         SIMDE_FLOAT32_C(    13.95), SIMDE_FLOAT32_C(     2.52), SIMDE_FLOAT32_C(     5.50), SIMDE_FLOAT32_C(     3.36),
                         SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(   462.59), SIMDE_FLOAT32_C(    -1.16),
                         SIMDE_FLOAT32_C(   718.27), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    10.12), SIMDE_FLOAT32_C(   904.02)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.23), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(     4.07),
                         SIMDE_FLOAT32_C(     4.79), SIMDE_FLOAT32_C(     7.12), SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(     3.87),
                         SIMDE_FLOAT32_C(     5.39), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     4.25), SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     5.19)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.44), SIMDE_FLOAT32_C(     7.24), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     3.99),
                         SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     5.13), SIMDE_FLOAT32_C(     7.31), SIMDE_FLOAT32_C(     3.77),
                         SIMDE_FLOAT32_C(     6.86), SIMDE_FLOAT32_C(     2.97), SIMDE_FLOAT32_C(     4.32), SIMDE_FLOAT32_C(     1.67),
                         SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     5.34)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.23), SIMDE_FLOAT32_C(   697.05), SIMDE_FLOAT32_C(     1.95), SIMDE_FLOAT32_C(    27.02),
                         SIMDE_FLOAT32_C(     4.79), SIMDE_FLOAT32_C(    84.51), SIMDE_FLOAT32_C(   747.59), SIMDE_FLOAT32_C(    21.68),
                         SIMDE_FLOAT32_C(     5.39), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     4.25), SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     5.19)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.10), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     3.00), SIMDE_FLOAT32_C(    -0.07),
                         SIMDE_FLOAT32_C(     3.70), SIMDE_FLOAT32_C(     4.19), SIMDE_FLOAT32_C(     6.89), SIMDE_FLOAT32_C(     3.02),
                         SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     6.67),
                         SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     5.69), SIMDE_FLOAT32_C(     5.19), SIMDE_FLOAT32_C(     5.17)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     1.55), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     6.70), SIMDE_FLOAT32_C(     3.32), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(     3.99),
                         SIMDE_FLOAT32_C(     5.79), SIMDE_FLOAT32_C(     3.25), SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     0.21),
                         SIMDE_FLOAT32_C(     4.74), SIMDE_FLOAT32_C(     4.48), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     1.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     7.10), SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(    -0.07),
                         SIMDE_FLOAT32_C(   406.20), SIMDE_FLOAT32_C(     4.19), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(    27.02),
                         SIMDE_FLOAT32_C(     2.07), SIMDE_FLOAT32_C(     2.38), SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     6.67),
                         SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(    44.11), SIMDE_FLOAT32_C(     5.19), SIMDE_FLOAT32_C(     5.17)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.58), SIMDE_FLOAT32_C(     3.19), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     6.05),
                         SIMDE_FLOAT32_C(     5.24), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     3.75), SIMDE_FLOAT32_C(     5.84),
                         SIMDE_FLOAT32_C(     3.43), SIMDE_FLOAT32_C(     6.03), SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     3.64),
                         SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(     5.91), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(    -0.52)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.38), SIMDE_FLOAT32_C(     6.78), SIMDE_FLOAT32_C(     3.92), SIMDE_FLOAT32_C(     1.46),
                         SIMDE_FLOAT32_C(     4.63), SIMDE_FLOAT32_C(     2.54), SIMDE_FLOAT32_C(     1.33), SIMDE_FLOAT32_C(     6.22),
                         SIMDE_FLOAT32_C(     3.58), SIMDE_FLOAT32_C(     2.15), SIMDE_FLOAT32_C(     3.80), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     3.13), SIMDE_FLOAT32_C(     2.22), SIMDE_FLOAT32_C(     3.07), SIMDE_FLOAT32_C(     3.70)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     5.58), SIMDE_FLOAT32_C(     3.19), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     6.05),
                         SIMDE_FLOAT32_C(     5.24), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     1.76), SIMDE_FLOAT32_C(   251.35),
                         SIMDE_FLOAT32_C(    17.92), SIMDE_FLOAT32_C(     6.03), SIMDE_FLOAT32_C(    22.34), SIMDE_FLOAT32_C(     0.92),
                         SIMDE_FLOAT32_C(    11.42), SIMDE_FLOAT32_C(     5.91), SIMDE_FLOAT32_C(     5.00), SIMDE_FLOAT32_C(    20.21)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(     4.72), SIMDE_FLOAT32_C(     1.18), SIMDE_FLOAT32_C(     6.84),
                         SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(     7.40), SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(    -0.21),
                         SIMDE_FLOAT32_C(     2.40), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(     0.17), SIMDE_FLOAT32_C(     1.49),
                         SIMDE_FLOAT32_C(     3.73), SIMDE_FLOAT32_C(     3.74), SIMDE_FLOAT32_C(     5.19), SIMDE_FLOAT32_C(     2.42)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     3.24), SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     6.71),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     1.96), SIMDE_FLOAT32_C(     6.43), SIMDE_FLOAT32_C(     1.79),
                         SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(     3.77), SIMDE_FLOAT32_C(     6.38), SIMDE_FLOAT32_C(     2.22),
                         SIMDE_FLOAT32_C(     7.36), SIMDE_FLOAT32_C(     4.86), SIMDE_FLOAT32_C(     3.24), SIMDE_FLOAT32_C(     6.97)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.09), SIMDE_FLOAT32_C(     4.72), SIMDE_FLOAT32_C(   471.94), SIMDE_FLOAT32_C(   410.28),
                         SIMDE_FLOAT32_C(     7.41), SIMDE_FLOAT32_C(     7.40), SIMDE_FLOAT32_C(     6.85), SIMDE_FLOAT32_C(     2.91),
                         SIMDE_FLOAT32_C(    17.57), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(   294.96), SIMDE_FLOAT32_C(     1.49),
                         SIMDE_FLOAT32_C(   785.92), SIMDE_FLOAT32_C(     3.74), SIMDE_FLOAT32_C(     5.19), SIMDE_FLOAT32_C(   532.11)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_sinh_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_sinh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.44), SIMDE_FLOAT64_C(    6.18),
                         SIMDE_FLOAT64_C(    2.02), SIMDE_FLOAT64_C(    3.45),
                         SIMDE_FLOAT64_C(    2.50), SIMDE_FLOAT64_C(    3.47),
                         SIMDE_FLOAT64_C(    0.06), SIMDE_FLOAT64_C(    4.79)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  115.22), SIMDE_FLOAT64_C(  241.49),
                         SIMDE_FLOAT64_C(    3.70), SIMDE_FLOAT64_C(   15.73),
                         SIMDE_FLOAT64_C(    6.05), SIMDE_FLOAT64_C(   16.05),
                         SIMDE_FLOAT64_C(    0.06), SIMDE_FLOAT64_C(   60.15)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(    0.35),
                         SIMDE_FLOAT64_C(    3.66), SIMDE_FLOAT64_C(    5.76),
                         SIMDE_FLOAT64_C(    6.85), SIMDE_FLOAT64_C(    5.12),
                         SIMDE_FLOAT64_C(    2.14), SIMDE_FLOAT64_C(    5.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(    0.36),
                         SIMDE_FLOAT64_C(   19.42), SIMDE_FLOAT64_C(  158.67),
                         SIMDE_FLOAT64_C(  471.94), SIMDE_FLOAT64_C(   83.66),
                         SIMDE_FLOAT64_C(    4.19), SIMDE_FLOAT64_C(  101.17)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.39), SIMDE_FLOAT64_C(    3.42),
                         SIMDE_FLOAT64_C(    1.65), SIMDE_FLOAT64_C(   -0.67),
                         SIMDE_FLOAT64_C(    1.99), SIMDE_FLOAT64_C(   -0.40),
                         SIMDE_FLOAT64_C(    1.50), SIMDE_FLOAT64_C(    6.30)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.88), SIMDE_FLOAT64_C(   15.27),
                         SIMDE_FLOAT64_C(    2.51), SIMDE_FLOAT64_C(   -0.72),
                         SIMDE_FLOAT64_C(    3.59), SIMDE_FLOAT64_C(   -0.41),
                         SIMDE_FLOAT64_C(    2.13), SIMDE_FLOAT64_C(  272.29)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.07), SIMDE_FLOAT64_C(    1.36),
                         SIMDE_FLOAT64_C(    4.30), SIMDE_FLOAT64_C(    6.25),
                         SIMDE_FLOAT64_C(    4.42), SIMDE_FLOAT64_C(    2.39),
                         SIMDE_FLOAT64_C(   -0.90), SIMDE_FLOAT64_C(    0.46)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   29.27), SIMDE_FLOAT64_C(    1.82),
                         SIMDE_FLOAT64_C(   36.84), SIMDE_FLOAT64_C(  259.01),
                         SIMDE_FLOAT64_C(   41.54), SIMDE_FLOAT64_C(    5.41),
                         SIMDE_FLOAT64_C(   -1.03), SIMDE_FLOAT64_C(    0.48)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.01), SIMDE_FLOAT64_C(    5.21),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(    4.94),
                         SIMDE_FLOAT64_C(   -0.01), SIMDE_FLOAT64_C(    7.57),
                         SIMDE_FLOAT64_C(    3.42), SIMDE_FLOAT64_C(    6.92)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.01), SIMDE_FLOAT64_C(   91.54),
                         SIMDE_FLOAT64_C(    0.87), SIMDE_FLOAT64_C(   69.88),
                         SIMDE_FLOAT64_C(   -0.01), SIMDE_FLOAT64_C(  969.57),
                         SIMDE_FLOAT64_C(   15.27), SIMDE_FLOAT64_C(  506.16)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.63), SIMDE_FLOAT64_C(    5.00),
                         SIMDE_FLOAT64_C(    6.12), SIMDE_FLOAT64_C(    4.76),
                         SIMDE_FLOAT64_C(    5.59), SIMDE_FLOAT64_C(    2.16),
                         SIMDE_FLOAT64_C(    6.66), SIMDE_FLOAT64_C(    3.17)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.45), SIMDE_FLOAT64_C(   74.20),
                         SIMDE_FLOAT64_C(  227.43), SIMDE_FLOAT64_C(   58.37),
                         SIMDE_FLOAT64_C(  133.87), SIMDE_FLOAT64_C(    4.28),
                         SIMDE_FLOAT64_C(  390.27), SIMDE_FLOAT64_C(   11.88)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    2.42), SIMDE_FLOAT64_C(    2.95),
                         SIMDE_FLOAT64_C(    4.75), SIMDE_FLOAT64_C(   -0.76),
                         SIMDE_FLOAT64_C(    0.09), SIMDE_FLOAT64_C(    0.00),
                         SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(    5.01)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.58), SIMDE_FLOAT64_C(    9.53),
                         SIMDE_FLOAT64_C(   57.79), SIMDE_FLOAT64_C(   -0.84),
                         SIMDE_FLOAT64_C(    0.09), SIMDE_FLOAT64_C(    0.00),
                         SIMDE_FLOAT64_C(    1.06), SIMDE_FLOAT64_C(   74.95)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.32), SIMDE_FLOAT64_C(    6.22),
                         SIMDE_FLOAT64_C(    2.66), SIMDE_FLOAT64_C(    6.82),
                         SIMDE_FLOAT64_C(    7.21), SIMDE_FLOAT64_C(    5.88),
                         SIMDE_FLOAT64_C(    6.70), SIMDE_FLOAT64_C(    4.39)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  102.19), SIMDE_FLOAT64_C(  251.35),
                         SIMDE_FLOAT64_C(    7.11), SIMDE_FLOAT64_C(  457.99),
                         SIMDE_FLOAT64_C(  676.45), SIMDE_FLOAT64_C(  178.90),
                         SIMDE_FLOAT64_C(  406.20), SIMDE_FLOAT64_C(   40.31)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_sinh_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_sinh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.35), SIMDE_FLOAT64_C(    5.76),
                         SIMDE_FLOAT64_C(    5.12), SIMDE_FLOAT64_C(    5.31),
                         SIMDE_FLOAT64_C(    6.18), SIMDE_FLOAT64_C(    3.45),
                         SIMDE_FLOAT64_C(    3.47), SIMDE_FLOAT64_C(    4.79)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(    3.66),
                         SIMDE_FLOAT64_C(    6.85), SIMDE_FLOAT64_C(    2.14),
                         SIMDE_FLOAT64_C(    5.44), SIMDE_FLOAT64_C(    2.02),
                         SIMDE_FLOAT64_C(    2.50), SIMDE_FLOAT64_C(    0.06)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(    5.76),
                         SIMDE_FLOAT64_C(    5.12), SIMDE_FLOAT64_C(    5.31),
                         SIMDE_FLOAT64_C(  115.22), SIMDE_FLOAT64_C(    3.45),
                         SIMDE_FLOAT64_C(    6.05), SIMDE_FLOAT64_C(    0.06)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.07), SIMDE_FLOAT64_C(    4.30),
                         SIMDE_FLOAT64_C(    4.42), SIMDE_FLOAT64_C(   -0.90),
                         SIMDE_FLOAT64_C(    1.39), SIMDE_FLOAT64_C(    1.65),
                         SIMDE_FLOAT64_C(    1.99), SIMDE_FLOAT64_C(    1.50)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    6.92), SIMDE_FLOAT64_C(    1.36),
                         SIMDE_FLOAT64_C(    6.25), SIMDE_FLOAT64_C(    2.39),
                         SIMDE_FLOAT64_C(    0.46), SIMDE_FLOAT64_C(    3.42),
                         SIMDE_FLOAT64_C(   -0.67), SIMDE_FLOAT64_C(   -0.40)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  506.16), SIMDE_FLOAT64_C(    1.82),
                         SIMDE_FLOAT64_C(  259.01), SIMDE_FLOAT64_C(   -0.90),
                         SIMDE_FLOAT64_C(    1.39), SIMDE_FLOAT64_C(   15.27),
                         SIMDE_FLOAT64_C(    1.99), SIMDE_FLOAT64_C(   -0.41)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.01), SIMDE_FLOAT64_C(    5.00),
                         SIMDE_FLOAT64_C(    4.76), SIMDE_FLOAT64_C(    2.16),
                         SIMDE_FLOAT64_C(    3.17), SIMDE_FLOAT64_C(    5.21),
                         SIMDE_FLOAT64_C(    4.94), SIMDE_FLOAT64_C(    7.57)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(    1.63),
                         SIMDE_FLOAT64_C(    6.12), SIMDE_FLOAT64_C(    5.59),
                         SIMDE_FLOAT64_C(    6.66), SIMDE_FLOAT64_C(   -0.01),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(   -0.01)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.06), SIMDE_FLOAT64_C(    2.45),
                         SIMDE_FLOAT64_C(  227.43), SIMDE_FLOAT64_C(  133.87),
                         SIMDE_FLOAT64_C(  390.27), SIMDE_FLOAT64_C(   -0.01),
                         SIMDE_FLOAT64_C(    4.94), SIMDE_FLOAT64_C(   -0.01)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.82), SIMDE_FLOAT64_C(    5.32),
                         SIMDE_FLOAT64_C(    2.66), SIMDE_FLOAT64_C(    7.21),
                         SIMDE_FLOAT64_C(    6.70), SIMDE_FLOAT64_C(    2.42),
                         SIMDE_FLOAT64_C(    4.75), SIMDE_FLOAT64_C(    0.09)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.64), SIMDE_FLOAT64_C(    2.56),
                         SIMDE_FLOAT64_C(    6.22), SIMDE_FLOAT64_C(    6.82),
                         SIMDE_FLOAT64_C(    5.88), SIMDE_FLOAT64_C(    4.39),
                         SIMDE_FLOAT64_C(    2.95), SIMDE_FLOAT64_C(   -0.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.82), SIMDE_FLOAT64_C(    6.43),
                         SIMDE_FLOAT64_C(    2.66), SIMDE_FLOAT64_C(  457.99),
                         SIMDE_FLOAT64_C(  178.90), SIMDE_FLOAT64_C(   40.31),
                         SIMDE_FLOAT64_C(    4.75), SIMDE_FLOAT64_C(   -0.84)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.73), SIMDE_FLOAT64_C(    0.13),
                         SIMDE_FLOAT64_C(    6.56), SIMDE_FLOAT64_C(    4.78),
                         SIMDE_FLOAT64_C(   -0.13), SIMDE_FLOAT64_C(    1.04),
                         SIMDE_FLOAT64_C(   -0.24), SIMDE_FLOAT64_C(    6.12)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    4.72), SIMDE_FLOAT64_C(    5.29),
                         SIMDE_FLOAT64_C(    2.53), SIMDE_FLOAT64_C(    4.70),
                         SIMDE_FLOAT64_C(   -0.46), SIMDE_FLOAT64_C(    1.89),
                         SIMDE_FLOAT64_C(    2.47), SIMDE_FLOAT64_C(    5.71)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   56.08), SIMDE_FLOAT64_C(    0.13),
                         SIMDE_FLOAT64_C(    6.56), SIMDE_FLOAT64_C(   54.97),
                         SIMDE_FLOAT64_C(   -0.13), SIMDE_FLOAT64_C(    1.04),
                         SIMDE_FLOAT64_C(   -0.24), SIMDE_FLOAT64_C(  150.93)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.05), SIMDE_FLOAT64_C(    3.42),
                         SIMDE_FLOAT64_C(    6.27), SIMDE_FLOAT64_C(    3.21),
                         SIMDE_FLOAT64_C(    1.37), SIMDE_FLOAT64_C(    5.47),
                         SIMDE_FLOAT64_C(    6.87), SIMDE_FLOAT64_C(    7.50)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    7.50), SIMDE_FLOAT64_C(    5.13),
                         SIMDE_FLOAT64_C(    2.89), SIMDE_FLOAT64_C(    6.91),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    6.45),
                         SIMDE_FLOAT64_C(    5.98), SIMDE_FLOAT64_C(    3.90)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.05), SIMDE_FLOAT64_C(   84.51),
                         SIMDE_FLOAT64_C(    6.27), SIMDE_FLOAT64_C(    3.21),
                         SIMDE_FLOAT64_C(    0.84), SIMDE_FLOAT64_C(    5.47),
                         SIMDE_FLOAT64_C(  197.72), SIMDE_FLOAT64_C(   24.69)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.00), SIMDE_FLOAT64_C(    1.99),
                         SIMDE_FLOAT64_C(    0.30), SIMDE_FLOAT64_C(    6.83),
                         SIMDE_FLOAT64_C(   -0.99), SIMDE_FLOAT64_C(    7.27),
                         SIMDE_FLOAT64_C(   -0.00), SIMDE_FLOAT64_C(    3.01)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.50), SIMDE_FLOAT64_C(    3.36),
                         SIMDE_FLOAT64_C(    7.24), SIMDE_FLOAT64_C(   -0.01),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    1.85),
                         SIMDE_FLOAT64_C(    4.56), SIMDE_FLOAT64_C(    0.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.00), SIMDE_FLOAT64_C(   14.38),
                         SIMDE_FLOAT64_C(    0.30), SIMDE_FLOAT64_C(   -0.01),
                         SIMDE_FLOAT64_C(    0.93), SIMDE_FLOAT64_C(    3.10),
                         SIMDE_FLOAT64_C(   -0.00), SIMDE_FLOAT64_C(    0.92)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    5.34), SIMDE_FLOAT64_C(    7.33),
                         SIMDE_FLOAT64_C(    1.80), SIMDE_FLOAT64_C(    1.42),
                         SIMDE_FLOAT64_C(    0.06), SIMDE_FLOAT64_C(    7.31),
                         SIMDE_FLOAT64_C(    1.89), SIMDE_FLOAT64_C(    2.52)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(    5.19),
                         SIMDE_FLOAT64_C(    7.16), SIMDE_FLOAT64_C(    2.45),
                         SIMDE_FLOAT64_C(    1.75), SIMDE_FLOAT64_C(    3.16),
                         SIMDE_FLOAT64_C(    3.33), SIMDE_FLOAT64_C(    2.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(   89.73),
                         SIMDE_FLOAT64_C(    1.80), SIMDE_FLOAT64_C(    5.75),
                         SIMDE_FLOAT64_C(    0.06), SIMDE_FLOAT64_C(   11.76),
                         SIMDE_FLOAT64_C(    1.89), SIMDE_FLOAT64_C(    7.87)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_sinh_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_svml_ceil_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -169.65), SIMDE_FLOAT32_C(   267.82), SIMDE_FLOAT32_C(   302.20), SIMDE_FLOAT32_C(   -31.93) },
      { SIMDE_FLOAT32_C(  -169.00), SIMDE_FLOAT32_C(   268.00), SIMDE_FLOAT32_C(   303.00), SIMDE_FLOAT32_C(   -31.00) } },
    { { SIMDE_FLOAT32_C(  -142.32), SIMDE_FLOAT32_C(  -661.66), SIMDE_FLOAT32_C(   156.37), SIMDE_FLOAT32_C(   396.69) },
      { SIMDE_FLOAT32_C(  -142.00), SIMDE_FLOAT32_C(  -661.00), SIMDE_FLOAT32_C(   157.00), SIMDE_FLOAT32_C(   397.00) } },
    { { SIMDE_FLOAT32_C(   382.01), SIMDE_FLOAT32_C(   656.47), SIMDE_FLOAT32_C(  -361.06), SIMDE_FLOAT32_C(  -343.68) },
      { SIMDE_FLOAT32_C(   383.00), SIMDE_FLOAT32_C(   657.00), SIMDE_FLOAT32_C(  -361.00), SIMDE_FLOAT32_C(  -343.00) } },
    { { SIMDE_FLOAT32_C(  -331.36), SIMDE_FLOAT32_C(    68.89), SIMDE_FLOAT32_C(   476.92), SIMDE_FLOAT32_C(   -40.59) },
      { SIMDE_FLOAT32_C(  -331.00), SIMDE_FLOAT32_C(    69.00), SIMDE_FLOAT32_C(   477.00), SIMDE_FLOAT32_C(   -40.00) } },
    { { SIMDE_FLOAT32_C(   390.65), SIMDE_FLOAT32_C(  -570.02), SIMDE_FLOAT32_C(  -935.28), SIMDE_FLOAT32_C(   672.43) },
      { SIMDE_FLOAT32_C(   391.00), SIMDE_FLOAT32_C(  -570.00), SIMDE_FLOAT32_C(  -935.00), SIMDE_FLOAT32_C(   673.00) } },
    { { SIMDE_FLOAT32_C(   681.18), SIMDE_FLOAT32_C(  -100.50), SIMDE_FLOAT32_C(   206.11), SIMDE_FLOAT32_C(   943.93) },
      { SIMDE_FLOAT32_C(   682.00), SIMDE_FLOAT32_C(  -100.00), SIMDE_FLOAT32_C(   207.00), SIMDE_FLOAT32_C(   944.00) } },
    { { SIMDE_FLOAT32_C(   786.98), SIMDE_FLOAT32_C(   -51.78), SIMDE_FLOAT32_C(  -481.30), SIMDE_FLOAT32_C(   955.46) },
      { SIMDE_FLOAT32_C(   787.00), SIMDE_FLOAT32_C(   -51.00), SIMDE_FLOAT32_C(  -481.00), SIMDE_FLOAT32_C(   956.00) } },
    { { SIMDE_FLOAT32_C(  -832.82), SIMDE_FLOAT32_C(   115.81), SIMDE_FLOAT32_C(  -954.30), SIMDE_FLOAT32_C(    -2.48) },
      { SIMDE_FLOAT32_C(  -832.00), SIMDE_FLOAT32_C(   116.00), SIMDE_FLOAT32_C(  -954.00), SIMDE_FLOAT32_C(    -2.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_svml_ceil_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_svml_ceil_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   674.99), SIMDE_FLOAT64_C(   114.55) },
      { SIMDE_FLOAT64_C(   675.00), SIMDE_FLOAT64_C(   115.00) } },
    { { SIMDE_FLOAT64_C(    69.63), SIMDE_FLOAT64_C(  -469.97) },
      { SIMDE_FLOAT64_C(    70.00), SIMDE_FLOAT64_C(  -469.00) } },
    { { SIMDE_FLOAT64_C(    28.21), SIMDE_FLOAT64_C(   212.97) },
      { SIMDE_FLOAT64_C(    29.00), SIMDE_FLOAT64_C(   213.00) } },
    { { SIMDE_FLOAT64_C(   763.99), SIMDE_FLOAT64_C(  -272.25) },
      { SIMDE_FLOAT64_C(   764.00), SIMDE_FLOAT64_C(  -272.00) } },
    { { SIMDE_FLOAT64_C(  -938.61), SIMDE_FLOAT64_C(   282.65) },
      { SIMDE_FLOAT64_C(  -938.00), SIMDE_FLOAT64_C(   283.00) } },
    { { SIMDE_FLOAT64_C(  -881.63), SIMDE_FLOAT64_C(   347.00) },
      { SIMDE_FLOAT64_C(  -881.00), SIMDE_FLOAT64_C(   347.00) } },
    { { SIMDE_FLOAT64_C(    95.36), SIMDE_FLOAT64_C(    -9.46) },
      { SIMDE_FLOAT64_C(    96.00), SIMDE_FLOAT64_C(    -9.00) } },
    { { SIMDE_FLOAT64_C(   -56.68), SIMDE_FLOAT64_C(   444.40) },
      { SIMDE_FLOAT64_C(   -56.00), SIMDE_FLOAT64_C(   445.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_svml_ceil_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_svml_ceil_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   -76.72), SIMDE_FLOAT32_C(  -639.26), SIMDE_FLOAT32_C(   440.96), SIMDE_FLOAT32_C(  -729.70),
        SIMDE_FLOAT32_C(   846.93), SIMDE_FLOAT32_C(     5.62), SIMDE_FLOAT32_C(  -834.54), SIMDE_FLOAT32_C(  -216.99) },
      { SIMDE_FLOAT32_C(   -76.00), SIMDE_FLOAT32_C(  -639.00), SIMDE_FLOAT32_C(   441.00), SIMDE_FLOAT32_C(  -729.00),
        SIMDE_FLOAT32_C(   847.00), SIMDE_FLOAT32_C(     6.00), SIMDE_FLOAT32_C(  -834.00), SIMDE_FLOAT32_C(  -216.00) } },
    { { SIMDE_FLOAT32_C(  -602.71), SIMDE_FLOAT32_C(  -551.43), SIMDE_FLOAT32_C(   949.68), SIMDE_FLOAT32_C(  -637.56),
        SIMDE_FLOAT32_C(  -279.53), SIMDE_FLOAT32_C(   553.99), SIMDE_FLOAT32_C(  -582.80), SIMDE_FLOAT32_C(   265.64) },
      { SIMDE_FLOAT32_C(  -602.00), SIMDE_FLOAT32_C(  -551.00), SIMDE_FLOAT32_C(   950.00), SIMDE_FLOAT32_C(  -637.00),
        SIMDE_FLOAT32_C(  -279.00), SIMDE_FLOAT32_C(   554.00), SIMDE_FLOAT32_C(  -582.00), SIMDE_FLOAT32_C(   266.00) } },
    { { SIMDE_FLOAT32_C(   457.99), SIMDE_FLOAT32_C(   385.92), SIMDE_FLOAT32_C(   814.23), SIMDE_FLOAT32_C(  -511.82),
        SIMDE_FLOAT32_C(  -834.29), SIMDE_FLOAT32_C(    45.52), SIMDE_FLOAT32_C(   999.48), SIMDE_FLOAT32_C(  -489.95) },
      { SIMDE_FLOAT32_C(   458.00), SIMDE_FLOAT32_C(   386.00), SIMDE_FLOAT32_C(   815.00), SIMDE_FLOAT32_C(  -511.00),
        SIMDE_FLOAT32_C(  -834.00), SIMDE_FLOAT32_C(    46.00), SIMDE_FLOAT32_C(  1000.00), SIMDE_FLOAT32_C(  -489.00) } },
    { { SIMDE_FLOAT32_C(   499.94), SIMDE_FLOAT32_C(   847.57), SIMDE_FLOAT32_C(   656.49), SIMDE_FLOAT32_C(   169.03),
        SIMDE_FLOAT32_C(  -361.51), SIMDE_FLOAT32_C(   697.36), SIMDE_FLOAT32_C(  -537.79), SIMDE_FLOAT32_C(   561.78) },
      { SIMDE_FLOAT32_C(   500.00), SIMDE_FLOAT32_C(   848.00), SIMDE_FLOAT32_C(   657.00), SIMDE_FLOAT32_C(   170.00),
        SIMDE_FLOAT32_C(  -361.00), SIMDE_FLOAT32_C(   698.00), SIMDE_FLOAT32_C(  -537.00), SIMDE_FLOAT32_C(   562.00) } },
    { { SIMDE_FLOAT32_C(  -941.90), SIMDE_FLOAT32_C(   903.17), SIMDE_FLOAT32_C(   832.08), SIMDE_FLOAT32_C(   905.03),
        SIMDE_FLOAT32_C(   -91.21), SIMDE_FLOAT32_C(   997.54), SIMDE_FLOAT32_C(  -311.96), SIMDE_FLOAT32_C(   306.08) },
      { SIMDE_FLOAT32_C(  -941.00), SIMDE_FLOAT32_C(   904.00), SIMDE_FLOAT32_C(   833.00), SIMDE_FLOAT32_C(   906.00),
        SIMDE_FLOAT32_C(   -91.00), SIMDE_FLOAT32_C(   998.00), SIMDE_FLOAT32_C(  -311.00), SIMDE_FLOAT32_C(   307.00) } },
    { { SIMDE_FLOAT32_C(  -553.88), SIMDE_FLOAT32_C(  -362.28), SIMDE_FLOAT32_C(   668.53), SIMDE_FLOAT32_C(   166.59),
        SIMDE_FLOAT32_C(  -808.29), SIMDE_FLOAT32_C(  -914.27), SIMDE_FLOAT32_C(  -567.77), SIMDE_FLOAT32_C(   649.70) },
      { SIMDE_FLOAT32_C(  -553.00), SIMDE_FLOAT32_C(  -362.00), SIMDE_FLOAT32_C(   669.00), SIMDE_FLOAT32_C(   167.00),
        SIMDE_FLOAT32_C(  -808.00), SIMDE_FLOAT32_C(  -914.00), SIMDE_FLOAT32_C(  -567.00), SIMDE_FLOAT32_C(   650.00) } },
    { { SIMDE_FLOAT32_C(   471.65), SIMDE_FLOAT32_C(  -753.54), SIMDE_FLOAT32_C(  -862.12), SIMDE_FLOAT32_C(   637.36),
        SIMDE_FLOAT32_C(   291.98), SIMDE_FLOAT32_C(  -862.64), SIMDE_FLOAT32_C(  -852.59), SIMDE_FLOAT32_C(  -208.07) },
      { SIMDE_FLOAT32_C(   472.00), SIMDE_FLOAT32_C(  -753.00), SIMDE_FLOAT32_C(  -862.00), SIMDE_FLOAT32_C(   638.00),
        SIMDE_FLOAT32_C(   292.00), SIMDE_FLOAT32_C(  -862.00), SIMDE_FLOAT32_C(  -852.00), SIMDE_FLOAT32_C(  -208.00) } },
    { { SIMDE_FLOAT32_C(   984.93), SIMDE_FLOAT32_C(   803.90), SIMDE_FLOAT32_C(   960.96), SIMDE_FLOAT32_C(  -376.58),
        SIMDE_FLOAT32_C(   501.26), SIMDE_FLOAT32_C(  -576.83), SIMDE_FLOAT32_C(  -814.80), SIMDE_FLOAT32_C(   559.36) },
      { SIMDE_FLOAT32_C(   985.00), SIMDE_FLOAT32_C(   804.00), SIMDE_FLOAT32_C(   961.00), SIMDE_FLOAT32_C(  -376.00),
        SIMDE_FLOAT32_C(   502.00), SIMDE_FLOAT32_C(  -576.00), SIMDE_FLOAT32_C(  -814.00), SIMDE_FLOAT32_C(   560.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_svml_ceil_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_svml_ceil_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -362.72), SIMDE_FLOAT64_C(  -517.27), SIMDE_FLOAT64_C(  -680.39), SIMDE_FLOAT64_C(  -370.55) },
      { SIMDE_FLOAT64_C(  -362.00), SIMDE_FLOAT64_C(  -517.00), SIMDE_FLOAT64_C(  -680.00), SIMDE_FLOAT64_C(  -370.00) } },
    { { SIMDE_FLOAT64_C(  -614.98), SIMDE_FLOAT64_C(   499.96), SIMDE_FLOAT64_C(  -673.46), SIMDE_FLOAT64_C(   813.10) },
      { SIMDE_FLOAT64_C(  -614.00), SIMDE_FLOAT64_C(   500.00), SIMDE_FLOAT64_C(  -673.00), SIMDE_FLOAT64_C(   814.00) } },
    { { SIMDE_FLOAT64_C(  -134.44), SIMDE_FLOAT64_C(   719.80), SIMDE_FLOAT64_C(  -164.15), SIMDE_FLOAT64_C(  -617.21) },
      { SIMDE_FLOAT64_C(  -134.00), SIMDE_FLOAT64_C(   720.00), SIMDE_FLOAT64_C(  -164.00), SIMDE_FLOAT64_C(  -617.00) } },
    { { SIMDE_FLOAT64_C(  -500.24), SIMDE_FLOAT64_C(   381.09), SIMDE_FLOAT64_C(   264.50), SIMDE_FLOAT64_C(   668.11) },
      { SIMDE_FLOAT64_C(  -500.00), SIMDE_FLOAT64_C(   382.00), SIMDE_FLOAT64_C(   265.00), SIMDE_FLOAT64_C(   669.00) } },
    { { SIMDE_FLOAT64_C(   934.75), SIMDE_FLOAT64_C(  -779.04), SIMDE_FLOAT64_C(   549.14), SIMDE_FLOAT64_C(  -476.20) },
      { SIMDE_FLOAT64_C(   935.00), SIMDE_FLOAT64_C(  -779.00), SIMDE_FLOAT64_C(   550.00), SIMDE_FLOAT64_C(  -476.00) } },
    { { SIMDE_FLOAT64_C(   -15.07), SIMDE_FLOAT64_C(   858.66), SIMDE_FLOAT64_C(  -174.63), SIMDE_FLOAT64_C(  -609.29) },
      { SIMDE_FLOAT64_C(   -15.00), SIMDE_FLOAT64_C(   859.00), SIMDE_FLOAT64_C(  -174.00), SIMDE_FLOAT64_C(  -609.00) } },
    { { SIMDE_FLOAT64_C(   -71.58), SIMDE_FLOAT64_C(   432.38), SIMDE_FLOAT64_C(   -26.35), SIMDE_FLOAT64_C(   -67.29) },
      { SIMDE_FLOAT64_C(   -71.00), SIMDE_FLOAT64_C(   433.00), SIMDE_FLOAT64_C(   -26.00), SIMDE_FLOAT64_C(   -67.00) } },
    { { SIMDE_FLOAT64_C(   708.92), SIMDE_FLOAT64_C(   346.09), SIMDE_FLOAT64_C(  -697.36), SIMDE_FLOAT64_C(  -653.80) },
      { SIMDE_FLOAT64_C(   709.00), SIMDE_FLOAT64_C(   347.00), SIMDE_FLOAT64_C(  -697.00), SIMDE_FLOAT64_C(  -653.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_svml_ceil_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_ceil_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -212.12), SIMDE_FLOAT32_C(  -438.18), SIMDE_FLOAT32_C(   403.70), SIMDE_FLOAT32_C(   369.30),
        SIMDE_FLOAT32_C(    75.33), SIMDE_FLOAT32_C(   898.48), SIMDE_FLOAT32_C(     1.19), SIMDE_FLOAT32_C(  -480.16),
        SIMDE_FLOAT32_C(  -450.03), SIMDE_FLOAT32_C(  -382.53), SIMDE_FLOAT32_C(   364.23), SIMDE_FLOAT32_C(   496.15),
        SIMDE_FLOAT32_C(   778.39), SIMDE_FLOAT32_C(  -311.07), SIMDE_FLOAT32_C(   656.92), SIMDE_FLOAT32_C(   -16.90) },
      { SIMDE_FLOAT32_C(  -212.00), SIMDE_FLOAT32_C(  -438.00), SIMDE_FLOAT32_C(   404.00), SIMDE_FLOAT32_C(   370.00),
        SIMDE_FLOAT32_C(    76.00), SIMDE_FLOAT32_C(   899.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(  -480.00),
        SIMDE_FLOAT32_C(  -450.00), SIMDE_FLOAT32_C(  -382.00), SIMDE_FLOAT32_C(   365.00), SIMDE_FLOAT32_C(   497.00),
        SIMDE_FLOAT32_C(   779.00), SIMDE_FLOAT32_C(  -311.00), SIMDE_FLOAT32_C(   657.00), SIMDE_FLOAT32_C(   -16.00) } },
    { { SIMDE_FLOAT32_C(  -112.72), SIMDE_FLOAT32_C(  -813.31), SIMDE_FLOAT32_C(   470.40), SIMDE_FLOAT32_C(  -748.73),
        SIMDE_FLOAT32_C(  -795.37), SIMDE_FLOAT32_C(   -65.01), SIMDE_FLOAT32_C(   904.80), SIMDE_FLOAT32_C(  -706.59),
        SIMDE_FLOAT32_C(    54.57), SIMDE_FLOAT32_C(  -248.19), SIMDE_FLOAT32_C(  -352.77), SIMDE_FLOAT32_C(   334.66),
        SIMDE_FLOAT32_C(   568.34), SIMDE_FLOAT32_C(   976.72), SIMDE_FLOAT32_C(   104.61), SIMDE_FLOAT32_C(  -643.78) },
      { SIMDE_FLOAT32_C(  -112.00), SIMDE_FLOAT32_C(  -813.00), SIMDE_FLOAT32_C(   471.00), SIMDE_FLOAT32_C(  -748.00),
        SIMDE_FLOAT32_C(  -795.00), SIMDE_FLOAT32_C(   -65.00), SIMDE_FLOAT32_C(   905.00), SIMDE_FLOAT32_C(  -706.00),
        SIMDE_FLOAT32_C(    55.00), SIMDE_FLOAT32_C(  -248.00), SIMDE_FLOAT32_C(  -352.00), SIMDE_FLOAT32_C(   335.00),
        SIMDE_FLOAT32_C(   569.00), SIMDE_FLOAT32_C(   977.00), SIMDE_FLOAT32_C(   105.00), SIMDE_FLOAT32_C(  -643.00) } },
    { { SIMDE_FLOAT32_C(  -461.46), SIMDE_FLOAT32_C(  -491.69), SIMDE_FLOAT32_C(   725.52), SIMDE_FLOAT32_C(   613.87),
        SIMDE_FLOAT32_C(  -593.21), SIMDE_FLOAT32_C(  -273.28), SIMDE_FLOAT32_C(  -866.30), SIMDE_FLOAT32_C(   -43.24),
        SIMDE_FLOAT32_C(   344.18), SIMDE_FLOAT32_C(   497.93), SIMDE_FLOAT32_C(  -547.09), SIMDE_FLOAT32_C(   122.57),
        SIMDE_FLOAT32_C(  -813.14), SIMDE_FLOAT32_C(  -890.17), SIMDE_FLOAT32_C(  -894.33), SIMDE_FLOAT32_C(    74.15) },
      { SIMDE_FLOAT32_C(  -461.00), SIMDE_FLOAT32_C(  -491.00), SIMDE_FLOAT32_C(   726.00), SIMDE_FLOAT32_C(   614.00),
        SIMDE_FLOAT32_C(  -593.00), SIMDE_FLOAT32_C(  -273.00), SIMDE_FLOAT32_C(  -866.00), SIMDE_FLOAT32_C(   -43.00),
        SIMDE_FLOAT32_C(   345.00), SIMDE_FLOAT32_C(   498.00), SIMDE_FLOAT32_C(  -547.00), SIMDE_FLOAT32_C(   123.00),
        SIMDE_FLOAT32_C(  -813.00), SIMDE_FLOAT32_C(  -890.00), SIMDE_FLOAT32_C(  -894.00), SIMDE_FLOAT32_C(    75.00) } },
    { { SIMDE_FLOAT32_C(  -703.48), SIMDE_FLOAT32_C(   576.07), SIMDE_FLOAT32_C(   325.42), SIMDE_FLOAT32_C(  -498.84),
        SIMDE_FLOAT32_C(  -488.94), SIMDE_FLOAT32_C(   230.22), SIMDE_FLOAT32_C(  -205.43), SIMDE_FLOAT32_C(   565.63),
        SIMDE_FLOAT32_C(   982.03), SIMDE_FLOAT32_C(   441.80), SIMDE_FLOAT32_C(   -99.71), SIMDE_FLOAT32_C(   550.37),
        SIMDE_FLOAT32_C(   418.51), SIMDE_FLOAT32_C(  -995.10), SIMDE_FLOAT32_C(   906.59), SIMDE_FLOAT32_C(   957.05) },
      { SIMDE_FLOAT32_C(  -703.00), SIMDE_FLOAT32_C(   577.00), SIMDE_FLOAT32_C(   326.00), SIMDE_FLOAT32_C(  -498.00),
        SIMDE_FLOAT32_C(  -488.00), SIMDE_FLOAT32_C(   231.00), SIMDE_FLOAT32_C(  -205.00), SIMDE_FLOAT32_C(   566.00),
        SIMDE_FLOAT32_C(   983.00), SIMDE_FLOAT32_C(   442.00), SIMDE_FLOAT32_C(   -99.00), SIMDE_FLOAT32_C(   551.00),
        SIMDE_FLOAT32_C(   419.00), SIMDE_FLOAT32_C(  -995.00), SIMDE_FLOAT32_C(   907.00), SIMDE_FLOAT32_C(   958.00) } },
    { { SIMDE_FLOAT32_C(  -486.79), SIMDE_FLOAT32_C(   632.11), SIMDE_FLOAT32_C(   570.92), SIMDE_FLOAT32_C(   -80.00),
        SIMDE_FLOAT32_C(  -641.18), SIMDE_FLOAT32_C(   704.62), SIMDE_FLOAT32_C(   876.76), SIMDE_FLOAT32_C(   703.01),
        SIMDE_FLOAT32_C(   202.55), SIMDE_FLOAT32_C(  -670.32), SIMDE_FLOAT32_C(  -174.43), SIMDE_FLOAT32_C(   389.41),
        SIMDE_FLOAT32_C(  -560.49), SIMDE_FLOAT32_C(   -68.76), SIMDE_FLOAT32_C(  -536.44), SIMDE_FLOAT32_C(  -263.97) },
      { SIMDE_FLOAT32_C(  -486.00), SIMDE_FLOAT32_C(   633.00), SIMDE_FLOAT32_C(   571.00), SIMDE_FLOAT32_C(   -80.00),
        SIMDE_FLOAT32_C(  -641.00), SIMDE_FLOAT32_C(   705.00), SIMDE_FLOAT32_C(   877.00), SIMDE_FLOAT32_C(   704.00),
        SIMDE_FLOAT32_C(   203.00), SIMDE_FLOAT32_C(  -670.00), SIMDE_FLOAT32_C(  -174.00), SIMDE_FLOAT32_C(   390.00),
        SIMDE_FLOAT32_C(  -560.00), SIMDE_FLOAT32_C(   -68.00), SIMDE_FLOAT32_C(  -536.00), SIMDE_FLOAT32_C(  -263.00) } },
    { { SIMDE_FLOAT32_C(  -492.69), SIMDE_FLOAT32_C(   788.98), SIMDE_FLOAT32_C(   237.19), SIMDE_FLOAT32_C(    18.37),
        SIMDE_FLOAT32_C(    19.20), SIMDE_FLOAT32_C(  -968.24), SIMDE_FLOAT32_C(  -416.00), SIMDE_FLOAT32_C(     1.23),
        SIMDE_FLOAT32_C(   473.56), SIMDE_FLOAT32_C(   484.29), SIMDE_FLOAT32_C(  -448.40), SIMDE_FLOAT32_C(  -107.93),
        SIMDE_FLOAT32_C(   489.18), SIMDE_FLOAT32_C(  -541.82), SIMDE_FLOAT32_C(  -150.87), SIMDE_FLOAT32_C(  -997.61) },
      { SIMDE_FLOAT32_C(  -492.00), SIMDE_FLOAT32_C(   789.00), SIMDE_FLOAT32_C(   238.00), SIMDE_FLOAT32_C(    19.00),
        SIMDE_FLOAT32_C(    20.00), SIMDE_FLOAT32_C(  -968.00), SIMDE_FLOAT32_C(  -416.00), SIMDE_FLOAT32_C(     2.00),
        SIMDE_FLOAT32_C(   474.00), SIMDE_FLOAT32_C(   485.00), SIMDE_FLOAT32_C(  -448.00), SIMDE_FLOAT32_C(  -107.00),
        SIMDE_FLOAT32_C(   490.00), SIMDE_FLOAT32_C(  -541.00), SIMDE_FLOAT32_C(  -150.00), SIMDE_FLOAT32_C(  -997.00) } },
    { { SIMDE_FLOAT32_C(  -909.71), SIMDE_FLOAT32_C(  -579.96), SIMDE_FLOAT32_C(   -77.61), SIMDE_FLOAT32_C(  -550.89),
        SIMDE_FLOAT32_C(  -875.34), SIMDE_FLOAT32_C(  -200.84), SIMDE_FLOAT32_C(  -847.88), SIMDE_FLOAT32_C(   327.21),
        SIMDE_FLOAT32_C(   128.83), SIMDE_FLOAT32_C(   -22.31), SIMDE_FLOAT32_C(  -283.37), SIMDE_FLOAT32_C(   568.34),
        SIMDE_FLOAT32_C(   908.94), SIMDE_FLOAT32_C(   180.19), SIMDE_FLOAT32_C(  -695.63), SIMDE_FLOAT32_C(  -583.75) },
      { SIMDE_FLOAT32_C(  -909.00), SIMDE_FLOAT32_C(  -579.00), SIMDE_FLOAT32_C(   -77.00), SIMDE_FLOAT32_C(  -550.00),
        SIMDE_FLOAT32_C(  -875.00), SIMDE_FLOAT32_C(  -200.00), SIMDE_FLOAT32_C(  -847.00), SIMDE_FLOAT32_C(   328.00),
        SIMDE_FLOAT32_C(   129.00), SIMDE_FLOAT32_C(   -22.00), SIMDE_FLOAT32_C(  -283.00), SIMDE_FLOAT32_C(   569.00),
        SIMDE_FLOAT32_C(   909.00), SIMDE_FLOAT32_C(   181.00), SIMDE_FLOAT32_C(  -695.00), SIMDE_FLOAT32_C(  -583.00) } },
    { { SIMDE_FLOAT32_C(   -30.83), SIMDE_FLOAT32_C(   541.56), SIMDE_FLOAT32_C(   434.62), SIMDE_FLOAT32_C(   988.37),
        SIMDE_FLOAT32_C(   573.33), SIMDE_FLOAT32_C(  -981.38), SIMDE_FLOAT32_C(   -10.40), SIMDE_FLOAT32_C(    46.89),
        SIMDE_FLOAT32_C(   502.90), SIMDE_FLOAT32_C(   541.19), SIMDE_FLOAT32_C(   938.96), SIMDE_FLOAT32_C(    -7.91),
        SIMDE_FLOAT32_C(   999.37), SIMDE_FLOAT32_C(  -211.91), SIMDE_FLOAT32_C(    -5.52), SIMDE_FLOAT32_C(  -910.34) },
      { SIMDE_FLOAT32_C(   -30.00), SIMDE_FLOAT32_C(   542.00), SIMDE_FLOAT32_C(   435.00), SIMDE_FLOAT32_C(   989.00),
        SIMDE_FLOAT32_C(   574.00), SIMDE_FLOAT32_C(  -981.00), SIMDE_FLOAT32_C(   -10.00), SIMDE_FLOAT32_C(    47.00),
        SIMDE_FLOAT32_C(   503.00), SIMDE_FLOAT32_C(   542.00), SIMDE_FLOAT32_C(   939.00), SIMDE_FLOAT32_C(    -7.00),
        SIMDE_FLOAT32_C(  1000.00), SIMDE_FLOAT32_C(  -211.00), SIMDE_FLOAT32_C(    -5.00), SIMDE_FLOAT32_C(  -910.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_ceil_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_ceil_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   264.66), SIMDE_FLOAT32_C(   621.32), SIMDE_FLOAT32_C(  -827.32), SIMDE_FLOAT32_C(  -154.51),
        SIMDE_FLOAT32_C(   337.38), SIMDE_FLOAT32_C(   187.19), SIMDE_FLOAT32_C(   659.53), SIMDE_FLOAT32_C(   559.33),
        SIMDE_FLOAT32_C(   209.98), SIMDE_FLOAT32_C(   625.49), SIMDE_FLOAT32_C(   656.87), SIMDE_FLOAT32_C(  -793.87),
        SIMDE_FLOAT32_C(   746.37), SIMDE_FLOAT32_C(  -721.16), SIMDE_FLOAT32_C(   184.21), SIMDE_FLOAT32_C(   251.36) },
      UINT8_C(157),
      { SIMDE_FLOAT32_C(   769.49), SIMDE_FLOAT32_C(  -152.19), SIMDE_FLOAT32_C(   746.20), SIMDE_FLOAT32_C(  -444.46),
        SIMDE_FLOAT32_C(  -336.10), SIMDE_FLOAT32_C(  -772.83), SIMDE_FLOAT32_C(   887.52), SIMDE_FLOAT32_C(   966.03),
        SIMDE_FLOAT32_C(   490.22), SIMDE_FLOAT32_C(  -510.29), SIMDE_FLOAT32_C(   -30.50), SIMDE_FLOAT32_C(     1.38),
        SIMDE_FLOAT32_C(  -217.82), SIMDE_FLOAT32_C(    12.97), SIMDE_FLOAT32_C(  -733.96), SIMDE_FLOAT32_C(  -596.50) },
      { SIMDE_FLOAT32_C(   770.00), SIMDE_FLOAT32_C(   621.32), SIMDE_FLOAT32_C(   747.00), SIMDE_FLOAT32_C(  -444.00),
        SIMDE_FLOAT32_C(  -336.00), SIMDE_FLOAT32_C(   187.19), SIMDE_FLOAT32_C(   659.53), SIMDE_FLOAT32_C(   967.00),
        SIMDE_FLOAT32_C(   209.98), SIMDE_FLOAT32_C(   625.49), SIMDE_FLOAT32_C(   656.87), SIMDE_FLOAT32_C(  -793.87),
        SIMDE_FLOAT32_C(   746.37), SIMDE_FLOAT32_C(  -721.16), SIMDE_FLOAT32_C(   184.21), SIMDE_FLOAT32_C(   251.36) } },
    { { SIMDE_FLOAT32_C(   185.65), SIMDE_FLOAT32_C(   111.53), SIMDE_FLOAT32_C(   740.88), SIMDE_FLOAT32_C(  -627.16),
        SIMDE_FLOAT32_C(  -228.94), SIMDE_FLOAT32_C(   300.20), SIMDE_FLOAT32_C(   582.82), SIMDE_FLOAT32_C(  -603.45),
        SIMDE_FLOAT32_C(   -42.93), SIMDE_FLOAT32_C(   788.96), SIMDE_FLOAT32_C(  -857.08), SIMDE_FLOAT32_C(   235.91),
        SIMDE_FLOAT32_C(   -26.83), SIMDE_FLOAT32_C(   394.28), SIMDE_FLOAT32_C(   795.93), SIMDE_FLOAT32_C(  -257.35) },
      UINT8_C(195),
      { SIMDE_FLOAT32_C(   542.13), SIMDE_FLOAT32_C(   298.19), SIMDE_FLOAT32_C(   -94.01), SIMDE_FLOAT32_C(   769.30),
        SIMDE_FLOAT32_C(   185.71), SIMDE_FLOAT32_C(  -127.98), SIMDE_FLOAT32_C(   259.52), SIMDE_FLOAT32_C(   675.42),
        SIMDE_FLOAT32_C(   841.52), SIMDE_FLOAT32_C(  -739.10), SIMDE_FLOAT32_C(  -542.40), SIMDE_FLOAT32_C(  -145.50),
        SIMDE_FLOAT32_C(  -473.06), SIMDE_FLOAT32_C(  -138.90), SIMDE_FLOAT32_C(  -959.85), SIMDE_FLOAT32_C(   638.47) },
      { SIMDE_FLOAT32_C(   543.00), SIMDE_FLOAT32_C(   299.00), SIMDE_FLOAT32_C(   740.88), SIMDE_FLOAT32_C(  -627.16),
        SIMDE_FLOAT32_C(  -228.94), SIMDE_FLOAT32_C(   300.20), SIMDE_FLOAT32_C(   260.00), SIMDE_FLOAT32_C(   676.00),
        SIMDE_FLOAT32_C(   -42.93), SIMDE_FLOAT32_C(   788.96), SIMDE_FLOAT32_C(  -857.08), SIMDE_FLOAT32_C(   235.91),
        SIMDE_FLOAT32_C(   -26.83), SIMDE_FLOAT32_C(   394.28), SIMDE_FLOAT32_C(   795.93), SIMDE_FLOAT32_C(  -257.35) } },
    { { SIMDE_FLOAT32_C(  -398.03), SIMDE_FLOAT32_C(  -587.00), SIMDE_FLOAT32_C(  -590.48), SIMDE_FLOAT32_C(   902.17),
        SIMDE_FLOAT32_C(   995.82), SIMDE_FLOAT32_C(  -193.93), SIMDE_FLOAT32_C(  -140.76), SIMDE_FLOAT32_C(   784.78),
        SIMDE_FLOAT32_C(   -51.01), SIMDE_FLOAT32_C(  -904.84), SIMDE_FLOAT32_C(  -242.06), SIMDE_FLOAT32_C(  -656.73),
        SIMDE_FLOAT32_C(   891.09), SIMDE_FLOAT32_C(   500.60), SIMDE_FLOAT32_C(  -414.64), SIMDE_FLOAT32_C(   433.21) },
      UINT8_C(211),
      { SIMDE_FLOAT32_C(   491.34), SIMDE_FLOAT32_C(   202.51), SIMDE_FLOAT32_C(   984.50), SIMDE_FLOAT32_C(  -636.64),
        SIMDE_FLOAT32_C(  -537.96), SIMDE_FLOAT32_C(   659.92), SIMDE_FLOAT32_C(  -795.12), SIMDE_FLOAT32_C(  -277.06),
        SIMDE_FLOAT32_C(  -882.48), SIMDE_FLOAT32_C(    59.38), SIMDE_FLOAT32_C(   249.88), SIMDE_FLOAT32_C(   -21.39),
        SIMDE_FLOAT32_C(    99.53), SIMDE_FLOAT32_C(  -111.65), SIMDE_FLOAT32_C(   580.58), SIMDE_FLOAT32_C(   512.52) },
      { SIMDE_FLOAT32_C(   492.00), SIMDE_FLOAT32_C(   203.00), SIMDE_FLOAT32_C(  -590.48), SIMDE_FLOAT32_C(   902.17),
        SIMDE_FLOAT32_C(  -537.00), SIMDE_FLOAT32_C(  -193.93), SIMDE_FLOAT32_C(  -795.00), SIMDE_FLOAT32_C(  -277.00),
        SIMDE_FLOAT32_C(   -51.01), SIMDE_FLOAT32_C(  -904.84), SIMDE_FLOAT32_C(  -242.06), SIMDE_FLOAT32_C(  -656.73),
        SIMDE_FLOAT32_C(   891.09), SIMDE_FLOAT32_C(   500.60), SIMDE_FLOAT32_C(  -414.64), SIMDE_FLOAT32_C(   433.21) } },
    { { SIMDE_FLOAT32_C(   297.87), SIMDE_FLOAT32_C(   482.76), SIMDE_FLOAT32_C(   508.34), SIMDE_FLOAT32_C(  -896.06),
        SIMDE_FLOAT32_C(  -658.00), SIMDE_FLOAT32_C(   293.12), SIMDE_FLOAT32_C(    52.94), SIMDE_FLOAT32_C(  -562.84),
        SIMDE_FLOAT32_C(  -948.94), SIMDE_FLOAT32_C(   396.21), SIMDE_FLOAT32_C(  -671.75), SIMDE_FLOAT32_C(   551.66),
        SIMDE_FLOAT32_C(   981.56), SIMDE_FLOAT32_C(   761.46), SIMDE_FLOAT32_C(  -649.56), SIMDE_FLOAT32_C(   472.90) },
      UINT8_C(186),
      { SIMDE_FLOAT32_C(  -665.06), SIMDE_FLOAT32_C(   836.26), SIMDE_FLOAT32_C(   426.01), SIMDE_FLOAT32_C(   994.86),
        SIMDE_FLOAT32_C(  -958.85), SIMDE_FLOAT32_C(  -851.05), SIMDE_FLOAT32_C(  -887.63), SIMDE_FLOAT32_C(   100.52),
        SIMDE_FLOAT32_C(   398.83), SIMDE_FLOAT32_C(    90.99), SIMDE_FLOAT32_C(  -799.95), SIMDE_FLOAT32_C(  -712.82),
        SIMDE_FLOAT32_C(  -328.43), SIMDE_FLOAT32_C(   712.57), SIMDE_FLOAT32_C(   585.05), SIMDE_FLOAT32_C(  -845.67) },
      { SIMDE_FLOAT32_C(   297.87), SIMDE_FLOAT32_C(   837.00), SIMDE_FLOAT32_C(   508.34), SIMDE_FLOAT32_C(   995.00),
        SIMDE_FLOAT32_C(  -958.00), SIMDE_FLOAT32_C(  -851.00), SIMDE_FLOAT32_C(    52.94), SIMDE_FLOAT32_C(   101.00),
        SIMDE_FLOAT32_C(  -948.94), SIMDE_FLOAT32_C(   396.21), SIMDE_FLOAT32_C(  -671.75), SIMDE_FLOAT32_C(   551.66),
        SIMDE_FLOAT32_C(   981.56), SIMDE_FLOAT32_C(   761.46), SIMDE_FLOAT32_C(  -649.56), SIMDE_FLOAT32_C(   472.90) } },
    { { SIMDE_FLOAT32_C(   220.91), SIMDE_FLOAT32_C(   688.99), SIMDE_FLOAT32_C(  -503.67), SIMDE_FLOAT32_C(  -485.97),
        SIMDE_FLOAT32_C(  -258.07), SIMDE_FLOAT32_C(   -66.51), SIMDE_FLOAT32_C(  -434.91), SIMDE_FLOAT32_C(  -861.87),
        SIMDE_FLOAT32_C(   261.74), SIMDE_FLOAT32_C(  -883.26), SIMDE_FLOAT32_C(  -880.31), SIMDE_FLOAT32_C(    23.19),
        SIMDE_FLOAT32_C(  -532.81), SIMDE_FLOAT32_C(   592.60), SIMDE_FLOAT32_C(   987.17), SIMDE_FLOAT32_C(  -197.87) },
      UINT8_C(171),
      { SIMDE_FLOAT32_C(   413.18), SIMDE_FLOAT32_C(  -203.02), SIMDE_FLOAT32_C(   470.01), SIMDE_FLOAT32_C(   562.13),
        SIMDE_FLOAT32_C(   -90.64), SIMDE_FLOAT32_C(  -429.47), SIMDE_FLOAT32_C(   -39.04), SIMDE_FLOAT32_C(  -999.66),
        SIMDE_FLOAT32_C(  -229.42), SIMDE_FLOAT32_C(   248.13), SIMDE_FLOAT32_C(  -328.09), SIMDE_FLOAT32_C(  -516.85),
        SIMDE_FLOAT32_C(  -166.82), SIMDE_FLOAT32_C(  -173.76), SIMDE_FLOAT32_C(   704.07), SIMDE_FLOAT32_C(  -477.83) },
      { SIMDE_FLOAT32_C(   414.00), SIMDE_FLOAT32_C(  -203.00), SIMDE_FLOAT32_C(  -503.67), SIMDE_FLOAT32_C(   563.00),
        SIMDE_FLOAT32_C(  -258.07), SIMDE_FLOAT32_C(  -429.00), SIMDE_FLOAT32_C(  -434.91), SIMDE_FLOAT32_C(  -999.00),
        SIMDE_FLOAT32_C(   261.74), SIMDE_FLOAT32_C(  -883.26), SIMDE_FLOAT32_C(  -880.31), SIMDE_FLOAT32_C(    23.19),
        SIMDE_FLOAT32_C(  -532.81), SIMDE_FLOAT32_C(   592.60), SIMDE_FLOAT32_C(   987.17), SIMDE_FLOAT32_C(  -197.87) } },
    { { SIMDE_FLOAT32_C(   322.58), SIMDE_FLOAT32_C(  -781.90), SIMDE_FLOAT32_C(   264.10), SIMDE_FLOAT32_C(  -743.93),
        SIMDE_FLOAT32_C(  -216.81), SIMDE_FLOAT32_C(   402.23), SIMDE_FLOAT32_C(   517.80), SIMDE_FLOAT32_C(  -100.07),
        SIMDE_FLOAT32_C(   521.92), SIMDE_FLOAT32_C(  -459.00), SIMDE_FLOAT32_C(   367.12), SIMDE_FLOAT32_C(   114.52),
        SIMDE_FLOAT32_C(  -471.84), SIMDE_FLOAT32_C(  -830.76), SIMDE_FLOAT32_C(  -456.62), SIMDE_FLOAT32_C(   941.34) },
      UINT8_C( 19),
      { SIMDE_FLOAT32_C(  -986.61), SIMDE_FLOAT32_C(   503.47), SIMDE_FLOAT32_C(   875.58), SIMDE_FLOAT32_C(  -416.08),
        SIMDE_FLOAT32_C(  -535.57), SIMDE_FLOAT32_C(   875.92), SIMDE_FLOAT32_C(   354.51), SIMDE_FLOAT32_C(   712.56),
        SIMDE_FLOAT32_C(  -452.16), SIMDE_FLOAT32_C(   837.66), SIMDE_FLOAT32_C(  -454.26), SIMDE_FLOAT32_C(   374.08),
        SIMDE_FLOAT32_C(   541.73), SIMDE_FLOAT32_C(    67.91), SIMDE_FLOAT32_C(  -303.34), SIMDE_FLOAT32_C(   759.83) },
      { SIMDE_FLOAT32_C(  -986.00), SIMDE_FLOAT32_C(   504.00), SIMDE_FLOAT32_C(   264.10), SIMDE_FLOAT32_C(  -743.93),
        SIMDE_FLOAT32_C(  -535.00), SIMDE_FLOAT32_C(   402.23), SIMDE_FLOAT32_C(   517.80), SIMDE_FLOAT32_C(  -100.07),
        SIMDE_FLOAT32_C(   521.92), SIMDE_FLOAT32_C(  -459.00), SIMDE_FLOAT32_C(   367.12), SIMDE_FLOAT32_C(   114.52),
        SIMDE_FLOAT32_C(  -471.84), SIMDE_FLOAT32_C(  -830.76), SIMDE_FLOAT32_C(  -456.62), SIMDE_FLOAT32_C(   941.34) } },
    { { SIMDE_FLOAT32_C(  -668.00), SIMDE_FLOAT32_C(   -47.28), SIMDE_FLOAT32_C(  -456.99), SIMDE_FLOAT32_C(   734.23),
        SIMDE_FLOAT32_C(  -529.48), SIMDE_FLOAT32_C(   442.94), SIMDE_FLOAT32_C(   256.15), SIMDE_FLOAT32_C(    11.52),
        SIMDE_FLOAT32_C(  -189.94), SIMDE_FLOAT32_C(  -629.33), SIMDE_FLOAT32_C(   539.68), SIMDE_FLOAT32_C(   -20.70),
        SIMDE_FLOAT32_C(   -85.95), SIMDE_FLOAT32_C(   481.02), SIMDE_FLOAT32_C(   945.52), SIMDE_FLOAT32_C(   -72.56) },
      UINT8_C(158),
      { SIMDE_FLOAT32_C(   821.10), SIMDE_FLOAT32_C(   511.37), SIMDE_FLOAT32_C(   448.92), SIMDE_FLOAT32_C(   697.03),
        SIMDE_FLOAT32_C(  -134.12), SIMDE_FLOAT32_C(   161.48), SIMDE_FLOAT32_C(  -755.14), SIMDE_FLOAT32_C(  -296.46),
        SIMDE_FLOAT32_C(   707.22), SIMDE_FLOAT32_C(   618.95), SIMDE_FLOAT32_C(  -754.73), SIMDE_FLOAT32_C(  -224.87),
        SIMDE_FLOAT32_C(  -684.40), SIMDE_FLOAT32_C(  -994.91), SIMDE_FLOAT32_C(   107.14), SIMDE_FLOAT32_C(   268.32) },
      { SIMDE_FLOAT32_C(  -668.00), SIMDE_FLOAT32_C(   512.00), SIMDE_FLOAT32_C(   449.00), SIMDE_FLOAT32_C(   698.00),
        SIMDE_FLOAT32_C(  -134.00), SIMDE_FLOAT32_C(   442.94), SIMDE_FLOAT32_C(   256.15), SIMDE_FLOAT32_C(  -296.00),
        SIMDE_FLOAT32_C(  -189.94), SIMDE_FLOAT32_C(  -629.33), SIMDE_FLOAT32_C(   539.68), SIMDE_FLOAT32_C(   -20.70),
        SIMDE_FLOAT32_C(   -85.95), SIMDE_FLOAT32_C(   481.02), SIMDE_FLOAT32_C(   945.52), SIMDE_FLOAT32_C(   -72.56) } },
    { { SIMDE_FLOAT32_C(  -451.89), SIMDE_FLOAT32_C(  -158.63), SIMDE_FLOAT32_C(   738.85), SIMDE_FLOAT32_C(   991.05),
        SIMDE_FLOAT32_C(  -902.48), SIMDE_FLOAT32_C(  -249.63), SIMDE_FLOAT32_C(  -198.89), SIMDE_FLOAT32_C(  -531.81),
        SIMDE_FLOAT32_C(  -709.95), SIMDE_FLOAT32_C(   780.40), SIMDE_FLOAT32_C(   382.24), SIMDE_FLOAT32_C(   771.07),
        SIMDE_FLOAT32_C(   725.93), SIMDE_FLOAT32_C(  -690.31), SIMDE_FLOAT32_C(  -244.43), SIMDE_FLOAT32_C(   547.03) },
      UINT8_C(207),
      { SIMDE_FLOAT32_C(  -795.51), SIMDE_FLOAT32_C(   244.06), SIMDE_FLOAT32_C(  -313.07), SIMDE_FLOAT32_C(   365.97),
        SIMDE_FLOAT32_C(   488.92), SIMDE_FLOAT32_C(   390.47), SIMDE_FLOAT32_C(    73.20), SIMDE_FLOAT32_C(   107.87),
        SIMDE_FLOAT32_C(   635.73), SIMDE_FLOAT32_C(   848.33), SIMDE_FLOAT32_C(   423.47), SIMDE_FLOAT32_C(   640.83),
        SIMDE_FLOAT32_C(   -44.53), SIMDE_FLOAT32_C(  -308.21), SIMDE_FLOAT32_C(  -811.07), SIMDE_FLOAT32_C(   796.84) },
      { SIMDE_FLOAT32_C(  -795.00), SIMDE_FLOAT32_C(   245.00), SIMDE_FLOAT32_C(  -313.00), SIMDE_FLOAT32_C(   366.00),
        SIMDE_FLOAT32_C(  -902.48), SIMDE_FLOAT32_C(  -249.63), SIMDE_FLOAT32_C(    74.00), SIMDE_FLOAT32_C(   108.00),
        SIMDE_FLOAT32_C(  -709.95), SIMDE_FLOAT32_C(   780.40), SIMDE_FLOAT32_C(   382.24), SIMDE_FLOAT32_C(   771.07),
        SIMDE_FLOAT32_C(   725.93), SIMDE_FLOAT32_C(  -690.31), SIMDE_FLOAT32_C(  -244.43), SIMDE_FLOAT32_C(   547.03) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_ceil_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_ceil_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   568.62), SIMDE_FLOAT64_C(   832.90), SIMDE_FLOAT64_C(   451.04), SIMDE_FLOAT64_C(   205.98),
        SIMDE_FLOAT64_C(   456.63), SIMDE_FLOAT64_C(   924.23), SIMDE_FLOAT64_C(  -658.88), SIMDE_FLOAT64_C(  -472.23) },
      { SIMDE_FLOAT64_C(   569.00), SIMDE_FLOAT64_C(   833.00), SIMDE_FLOAT64_C(   452.00), SIMDE_FLOAT64_C(   206.00),
        SIMDE_FLOAT64_C(   457.00), SIMDE_FLOAT64_C(   925.00), SIMDE_FLOAT64_C(  -658.00), SIMDE_FLOAT64_C(  -472.00) } },
    { { SIMDE_FLOAT64_C(  -579.06), SIMDE_FLOAT64_C(   724.10), SIMDE_FLOAT64_C(  -922.32), SIMDE_FLOAT64_C(   603.12),
        SIMDE_FLOAT64_C(  -550.68), SIMDE_FLOAT64_C(  -479.10), SIMDE_FLOAT64_C(  -837.50), SIMDE_FLOAT64_C(   925.16) },
      { SIMDE_FLOAT64_C(  -579.00), SIMDE_FLOAT64_C(   725.00), SIMDE_FLOAT64_C(  -922.00), SIMDE_FLOAT64_C(   604.00),
        SIMDE_FLOAT64_C(  -550.00), SIMDE_FLOAT64_C(  -479.00), SIMDE_FLOAT64_C(  -837.00), SIMDE_FLOAT64_C(   926.00) } },
    { { SIMDE_FLOAT64_C(  -415.08), SIMDE_FLOAT64_C(   718.97), SIMDE_FLOAT64_C(  -850.54), SIMDE_FLOAT64_C(   464.10),
        SIMDE_FLOAT64_C(   558.79), SIMDE_FLOAT64_C(   424.83), SIMDE_FLOAT64_C(  -281.91), SIMDE_FLOAT64_C(   440.87) },
      { SIMDE_FLOAT64_C(  -415.00), SIMDE_FLOAT64_C(   719.00), SIMDE_FLOAT64_C(  -850.00), SIMDE_FLOAT64_C(   465.00),
        SIMDE_FLOAT64_C(   559.00), SIMDE_FLOAT64_C(   425.00), SIMDE_FLOAT64_C(  -281.00), SIMDE_FLOAT64_C(   441.00) } },
    { { SIMDE_FLOAT64_C(   834.86), SIMDE_FLOAT64_C(  -787.94), SIMDE_FLOAT64_C(   560.68), SIMDE_FLOAT64_C(  -896.06),
        SIMDE_FLOAT64_C(   -74.24), SIMDE_FLOAT64_C(   400.53), SIMDE_FLOAT64_C(  -101.01), SIMDE_FLOAT64_C(  -505.62) },
      { SIMDE_FLOAT64_C(   835.00), SIMDE_FLOAT64_C(  -787.00), SIMDE_FLOAT64_C(   561.00), SIMDE_FLOAT64_C(  -896.00),
        SIMDE_FLOAT64_C(   -74.00), SIMDE_FLOAT64_C(   401.00), SIMDE_FLOAT64_C(  -101.00), SIMDE_FLOAT64_C(  -505.00) } },
    { { SIMDE_FLOAT64_C(   233.43), SIMDE_FLOAT64_C(  -649.98), SIMDE_FLOAT64_C(   700.36), SIMDE_FLOAT64_C(  -309.94),
        SIMDE_FLOAT64_C(  -725.75), SIMDE_FLOAT64_C(  -958.52), SIMDE_FLOAT64_C(   217.83), SIMDE_FLOAT64_C(  -304.81) },
      { SIMDE_FLOAT64_C(   234.00), SIMDE_FLOAT64_C(  -649.00), SIMDE_FLOAT64_C(   701.00), SIMDE_FLOAT64_C(  -309.00),
        SIMDE_FLOAT64_C(  -725.00), SIMDE_FLOAT64_C(  -958.00), SIMDE_FLOAT64_C(   218.00), SIMDE_FLOAT64_C(  -304.00) } },
    { { SIMDE_FLOAT64_C(   765.58), SIMDE_FLOAT64_C(   295.51), SIMDE_FLOAT64_C(  -701.69), SIMDE_FLOAT64_C(  -785.11),
        SIMDE_FLOAT64_C(   816.41), SIMDE_FLOAT64_C(  -539.19), SIMDE_FLOAT64_C(  -859.95), SIMDE_FLOAT64_C(  -598.68) },
      { SIMDE_FLOAT64_C(   766.00), SIMDE_FLOAT64_C(   296.00), SIMDE_FLOAT64_C(  -701.00), SIMDE_FLOAT64_C(  -785.00),
        SIMDE_FLOAT64_C(   817.00), SIMDE_FLOAT64_C(  -539.00), SIMDE_FLOAT64_C(  -859.00), SIMDE_FLOAT64_C(  -598.00) } },
    { { SIMDE_FLOAT64_C(  -820.22), SIMDE_FLOAT64_C(  -710.49), SIMDE_FLOAT64_C(   865.42), SIMDE_FLOAT64_C(   738.57),
        SIMDE_FLOAT64_C(   714.34), SIMDE_FLOAT64_C(  -416.48), SIMDE_FLOAT64_C(   179.44), SIMDE_FLOAT64_C(   549.20) },
      { SIMDE_FLOAT64_C(  -820.00), SIMDE_FLOAT64_C(  -710.00), SIMDE_FLOAT64_C(   866.00), SIMDE_FLOAT64_C(   739.00),
        SIMDE_FLOAT64_C(   715.00), SIMDE_FLOAT64_C(  -416.00), SIMDE_FLOAT64_C(   180.00), SIMDE_FLOAT64_C(   550.00) } },
    { { SIMDE_FLOAT64_C(  -204.42), SIMDE_FLOAT64_C(  -259.88), SIMDE_FLOAT64_C(   653.14), SIMDE_FLOAT64_C(   721.34),
        SIMDE_FLOAT64_C(  -859.35), SIMDE_FLOAT64_C(  -447.87), SIMDE_FLOAT64_C(  -784.28), SIMDE_FLOAT64_C(   374.08) },
      { SIMDE_FLOAT64_C(  -204.00), SIMDE_FLOAT64_C(  -259.00), SIMDE_FLOAT64_C(   654.00), SIMDE_FLOAT64_C(   722.00),
        SIMDE_FLOAT64_C(  -859.00), SIMDE_FLOAT64_C(  -447.00), SIMDE_FLOAT64_C(  -784.00), SIMDE_FLOAT64_C(   375.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_ceil_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_ceil_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   229.81), SIMDE_FLOAT64_C(   525.51), SIMDE_FLOAT64_C(    -6.46), SIMDE_FLOAT64_C(  -218.89),
        SIMDE_FLOAT64_C(  -607.98), SIMDE_FLOAT64_C(  -552.09), SIMDE_FLOAT64_C(   531.98), SIMDE_FLOAT64_C(   900.69) },
      UINT8_C(198),
      { SIMDE_FLOAT64_C(  -545.02), SIMDE_FLOAT64_C(   596.71), SIMDE_FLOAT64_C(   311.19), SIMDE_FLOAT64_C(  -696.35),
        SIMDE_FLOAT64_C(  -125.03), SIMDE_FLOAT64_C(  -375.13), SIMDE_FLOAT64_C(   455.71), SIMDE_FLOAT64_C(   769.17) },
      { SIMDE_FLOAT64_C(   229.81), SIMDE_FLOAT64_C(   597.00), SIMDE_FLOAT64_C(   312.00), SIMDE_FLOAT64_C(  -218.89),
        SIMDE_FLOAT64_C(  -607.98), SIMDE_FLOAT64_C(  -552.09), SIMDE_FLOAT64_C(   456.00), SIMDE_FLOAT64_C(   770.00) } },
    { { SIMDE_FLOAT64_C(   871.47), SIMDE_FLOAT64_C(  -774.90), SIMDE_FLOAT64_C(   592.74), SIMDE_FLOAT64_C(  -416.66),
        SIMDE_FLOAT64_C(  -243.97), SIMDE_FLOAT64_C(   106.58), SIMDE_FLOAT64_C(  -923.77), SIMDE_FLOAT64_C(  -472.30) },
      UINT8_C(119),
      { SIMDE_FLOAT64_C(  -407.44), SIMDE_FLOAT64_C(  -264.38), SIMDE_FLOAT64_C(   828.67), SIMDE_FLOAT64_C(  -804.49),
        SIMDE_FLOAT64_C(    95.85), SIMDE_FLOAT64_C(    58.48), SIMDE_FLOAT64_C(   721.02), SIMDE_FLOAT64_C(  -910.62) },
      { SIMDE_FLOAT64_C(  -407.00), SIMDE_FLOAT64_C(  -264.00), SIMDE_FLOAT64_C(   829.00), SIMDE_FLOAT64_C(  -416.66),
        SIMDE_FLOAT64_C(    96.00), SIMDE_FLOAT64_C(    59.00), SIMDE_FLOAT64_C(   722.00), SIMDE_FLOAT64_C(  -472.30) } },
    { { SIMDE_FLOAT64_C(   839.59), SIMDE_FLOAT64_C(  -886.96), SIMDE_FLOAT64_C(  -462.70), SIMDE_FLOAT64_C(   371.56),
        SIMDE_FLOAT64_C(  -986.28), SIMDE_FLOAT64_C(   467.93), SIMDE_FLOAT64_C(   826.54), SIMDE_FLOAT64_C(   610.43) },
      UINT8_C( 11),
      { SIMDE_FLOAT64_C(  -869.81), SIMDE_FLOAT64_C(  -514.60), SIMDE_FLOAT64_C(   404.00), SIMDE_FLOAT64_C(   585.90),
        SIMDE_FLOAT64_C(  -745.43), SIMDE_FLOAT64_C(   275.47), SIMDE_FLOAT64_C(   811.00), SIMDE_FLOAT64_C(   847.30) },
      { SIMDE_FLOAT64_C(  -869.00), SIMDE_FLOAT64_C(  -514.00), SIMDE_FLOAT64_C(  -462.70), SIMDE_FLOAT64_C(   586.00),
        SIMDE_FLOAT64_C(  -986.28), SIMDE_FLOAT64_C(   467.93), SIMDE_FLOAT64_C(   826.54), SIMDE_FLOAT64_C(   610.43) } },
    { { SIMDE_FLOAT64_C(   858.82), SIMDE_FLOAT64_C(  -432.97), SIMDE_FLOAT64_C(   -46.12), SIMDE_FLOAT64_C(   935.05),
        SIMDE_FLOAT64_C(    94.73), SIMDE_FLOAT64_C(  -233.07), SIMDE_FLOAT64_C(  -472.39), SIMDE_FLOAT64_C(   830.35) },
      UINT8_C( 12),
      { SIMDE_FLOAT64_C(  -276.88), SIMDE_FLOAT64_C(   -73.80), SIMDE_FLOAT64_C(   654.07), SIMDE_FLOAT64_C(  -555.86),
        SIMDE_FLOAT64_C(    15.59), SIMDE_FLOAT64_C(   493.66), SIMDE_FLOAT64_C(  -442.83), SIMDE_FLOAT64_C(   552.88) },
      { SIMDE_FLOAT64_C(   858.82), SIMDE_FLOAT64_C(  -432.97), SIMDE_FLOAT64_C(   655.00), SIMDE_FLOAT64_C(  -555.00),
        SIMDE_FLOAT64_C(    94.73), SIMDE_FLOAT64_C(  -233.07), SIMDE_FLOAT64_C(  -472.39), SIMDE_FLOAT64_C(   830.35) } },
    { { SIMDE_FLOAT64_C(  -134.77), SIMDE_FLOAT64_C(  -429.10), SIMDE_FLOAT64_C(    20.82), SIMDE_FLOAT64_C(  -308.24),
        SIMDE_FLOAT64_C(  -818.67), SIMDE_FLOAT64_C(   799.94), SIMDE_FLOAT64_C(  -178.05), SIMDE_FLOAT64_C(  -333.27) },
      UINT8_C(157),
      { SIMDE_FLOAT64_C(  -592.15), SIMDE_FLOAT64_C(   -78.71), SIMDE_FLOAT64_C(  -520.59), SIMDE_FLOAT64_C(  -781.15),
        SIMDE_FLOAT64_C(  -231.40), SIMDE_FLOAT64_C(  -661.77), SIMDE_FLOAT64_C(  -214.12), SIMDE_FLOAT64_C(   722.48) },
      { SIMDE_FLOAT64_C(  -592.00), SIMDE_FLOAT64_C(  -429.10), SIMDE_FLOAT64_C(  -520.00), SIMDE_FLOAT64_C(  -781.00),
        SIMDE_FLOAT64_C(  -231.00), SIMDE_FLOAT64_C(   799.94), SIMDE_FLOAT64_C(  -178.05), SIMDE_FLOAT64_C(   723.00) } },
    { { SIMDE_FLOAT64_C(  -726.72), SIMDE_FLOAT64_C(   880.61), SIMDE_FLOAT64_C(  -510.59), SIMDE_FLOAT64_C(  -199.11),
        SIMDE_FLOAT64_C(   710.96), SIMDE_FLOAT64_C(    85.00), SIMDE_FLOAT64_C(   524.01), SIMDE_FLOAT64_C(  -362.83) },
      UINT8_C(189),
      { SIMDE_FLOAT64_C(   968.14), SIMDE_FLOAT64_C(   652.75), SIMDE_FLOAT64_C(  -767.26), SIMDE_FLOAT64_C(  -474.68),
        SIMDE_FLOAT64_C(   205.64), SIMDE_FLOAT64_C(    97.96), SIMDE_FLOAT64_C(    96.22), SIMDE_FLOAT64_C(  -773.55) },
      { SIMDE_FLOAT64_C(   969.00), SIMDE_FLOAT64_C(   880.61), SIMDE_FLOAT64_C(  -767.00), SIMDE_FLOAT64_C(  -474.00),
        SIMDE_FLOAT64_C(   206.00), SIMDE_FLOAT64_C(    98.00), SIMDE_FLOAT64_C(   524.01), SIMDE_FLOAT64_C(  -773.00) } },
    { { SIMDE_FLOAT64_C(   789.73), SIMDE_FLOAT64_C(   277.54), SIMDE_FLOAT64_C(  -973.60), SIMDE_FLOAT64_C(  -388.32),
        SIMDE_FLOAT64_C(   944.27), SIMDE_FLOAT64_C(   230.34), SIMDE_FLOAT64_C(    19.53), SIMDE_FLOAT64_C(  -134.44) },
      UINT8_C( 15),
      { SIMDE_FLOAT64_C(   238.38), SIMDE_FLOAT64_C(   634.16), SIMDE_FLOAT64_C(  -952.02), SIMDE_FLOAT64_C(  -975.74),
        SIMDE_FLOAT64_C(   356.64), SIMDE_FLOAT64_C(  -678.74), SIMDE_FLOAT64_C(   904.87), SIMDE_FLOAT64_C(   846.05) },
      { SIMDE_FLOAT64_C(   239.00), SIMDE_FLOAT64_C(   635.00), SIMDE_FLOAT64_C(  -952.00), SIMDE_FLOAT64_C(  -975.00),
        SIMDE_FLOAT64_C(   944.27), SIMDE_FLOAT64_C(   230.34), SIMDE_FLOAT64_C(    19.53), SIMDE_FLOAT64_C(  -134.44) } },
    { { SIMDE_FLOAT64_C(   122.14), SIMDE_FLOAT64_C(   615.84), SIMDE_FLOAT64_C(   -68.95), SIMDE_FLOAT64_C(  -353.85),
        SIMDE_FLOAT64_C(  -747.00), SIMDE_FLOAT64_C(   670.13), SIMDE_FLOAT64_C(  -385.71), SIMDE_FLOAT64_C(   905.76) },
      UINT8_C( 69),
      { SIMDE_FLOAT64_C(   139.61), SIMDE_FLOAT64_C(   111.39), SIMDE_FLOAT64_C(     0.83), SIMDE_FLOAT64_C(  -764.17),
        SIMDE_FLOAT64_C(   337.85), SIMDE_FLOAT64_C(  -209.44), SIMDE_FLOAT64_C(   513.37), SIMDE_FLOAT64_C(   364.24) },
      { SIMDE_FLOAT64_C(   140.00), SIMDE_FLOAT64_C(   615.84), SIMDE_FLOAT64_C(     1.00), SIMDE_FLOAT64_C(  -353.85),
        SIMDE_FLOAT64_C(  -747.00), SIMDE_FLOAT64_C(   670.13), SIMDE_FLOAT64_C(   514.00), SIMDE_FLOAT64_C(   905.76) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_ceil_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_svml_sqrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   528.60), SIMDE_FLOAT32_C(   322.33), SIMDE_FLOAT32_C(   385.23), SIMDE_FLOAT32_C(   814.87) },
      { SIMDE_FLOAT32_C(    22.99), SIMDE_FLOAT32_C(    17.95), SIMDE_FLOAT32_C(    19.63), SIMDE_FLOAT32_C(    28.55) } },
    { { SIMDE_FLOAT32_C(   587.72), SIMDE_FLOAT32_C(   685.82), SIMDE_FLOAT32_C(   593.20), SIMDE_FLOAT32_C(   733.30) },
      { SIMDE_FLOAT32_C(    24.24), SIMDE_FLOAT32_C(    26.19), SIMDE_FLOAT32_C(    24.36), SIMDE_FLOAT32_C(    27.08) } },
    { { SIMDE_FLOAT32_C(   325.19), SIMDE_FLOAT32_C(   348.73), SIMDE_FLOAT32_C(   342.79), SIMDE_FLOAT32_C(   565.69) },
      { SIMDE_FLOAT32_C(    18.03), SIMDE_FLOAT32_C(    18.67), SIMDE_FLOAT32_C(    18.51), SIMDE_FLOAT32_C(    23.78) } },
    { { SIMDE_FLOAT32_C(   148.43), SIMDE_FLOAT32_C(    85.30), SIMDE_FLOAT32_C(   679.23), SIMDE_FLOAT32_C(   235.95) },
      { SIMDE_FLOAT32_C(    12.18), SIMDE_FLOAT32_C(     9.24), SIMDE_FLOAT32_C(    26.06), SIMDE_FLOAT32_C(    15.36) } },
    { { SIMDE_FLOAT32_C(   741.81), SIMDE_FLOAT32_C(   327.17), SIMDE_FLOAT32_C(   932.33), SIMDE_FLOAT32_C(   431.37) },
      { SIMDE_FLOAT32_C(    27.24), SIMDE_FLOAT32_C(    18.09), SIMDE_FLOAT32_C(    30.53), SIMDE_FLOAT32_C(    20.77) } },
    { { SIMDE_FLOAT32_C(   630.74), SIMDE_FLOAT32_C(   622.98), SIMDE_FLOAT32_C(   345.17), SIMDE_FLOAT32_C(   666.65) },
      { SIMDE_FLOAT32_C(    25.11), SIMDE_FLOAT32_C(    24.96), SIMDE_FLOAT32_C(    18.58), SIMDE_FLOAT32_C(    25.82) } },
    { { SIMDE_FLOAT32_C(    95.65), SIMDE_FLOAT32_C(   585.30), SIMDE_FLOAT32_C(   996.40), SIMDE_FLOAT32_C(   212.96) },
      { SIMDE_FLOAT32_C(     9.78), SIMDE_FLOAT32_C(    24.19), SIMDE_FLOAT32_C(    31.57), SIMDE_FLOAT32_C(    14.59) } },
    { { SIMDE_FLOAT32_C(   691.00), SIMDE_FLOAT32_C(   383.56), SIMDE_FLOAT32_C(   356.19), SIMDE_FLOAT32_C(   219.60) },
      { SIMDE_FLOAT32_C(    26.29), SIMDE_FLOAT32_C(    19.58), SIMDE_FLOAT32_C(    18.87), SIMDE_FLOAT32_C(    14.82) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_svml_sqrt_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_svml_floor_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -368.97), SIMDE_FLOAT32_C(  -986.85), SIMDE_FLOAT32_C(   853.49), SIMDE_FLOAT32_C(    45.17) },
      { SIMDE_FLOAT32_C(  -369.00), SIMDE_FLOAT32_C(  -987.00), SIMDE_FLOAT32_C(   853.00), SIMDE_FLOAT32_C(    45.00) } },
    { { SIMDE_FLOAT32_C(   562.02), SIMDE_FLOAT32_C(  -924.44), SIMDE_FLOAT32_C(  -802.09), SIMDE_FLOAT32_C(    17.88) },
      { SIMDE_FLOAT32_C(   562.00), SIMDE_FLOAT32_C(  -925.00), SIMDE_FLOAT32_C(  -803.00), SIMDE_FLOAT32_C(    17.00) } },
    { { SIMDE_FLOAT32_C(  -773.69), SIMDE_FLOAT32_C(  -929.41), SIMDE_FLOAT32_C(  -376.84), SIMDE_FLOAT32_C(  -575.41) },
      { SIMDE_FLOAT32_C(  -774.00), SIMDE_FLOAT32_C(  -930.00), SIMDE_FLOAT32_C(  -377.00), SIMDE_FLOAT32_C(  -576.00) } },
    { { SIMDE_FLOAT32_C(   694.60), SIMDE_FLOAT32_C(   556.86), SIMDE_FLOAT32_C(   755.76), SIMDE_FLOAT32_C(    -3.15) },
      { SIMDE_FLOAT32_C(   694.00), SIMDE_FLOAT32_C(   556.00), SIMDE_FLOAT32_C(   755.00), SIMDE_FLOAT32_C(    -4.00) } },
    { { SIMDE_FLOAT32_C(  -225.40), SIMDE_FLOAT32_C(   440.47), SIMDE_FLOAT32_C(  -328.64), SIMDE_FLOAT32_C(  -113.66) },
      { SIMDE_FLOAT32_C(  -226.00), SIMDE_FLOAT32_C(   440.00), SIMDE_FLOAT32_C(  -329.00), SIMDE_FLOAT32_C(  -114.00) } },
    { { SIMDE_FLOAT32_C(  -752.27), SIMDE_FLOAT32_C(  -305.67), SIMDE_FLOAT32_C(  -135.72), SIMDE_FLOAT32_C(  -501.04) },
      { SIMDE_FLOAT32_C(  -753.00), SIMDE_FLOAT32_C(  -306.00), SIMDE_FLOAT32_C(  -136.00), SIMDE_FLOAT32_C(  -502.00) } },
    { { SIMDE_FLOAT32_C(   156.35), SIMDE_FLOAT32_C(   898.85), SIMDE_FLOAT32_C(  -988.19), SIMDE_FLOAT32_C(   407.13) },
      { SIMDE_FLOAT32_C(   156.00), SIMDE_FLOAT32_C(   898.00), SIMDE_FLOAT32_C(  -989.00), SIMDE_FLOAT32_C(   407.00) } },
    { { SIMDE_FLOAT32_C(   973.98), SIMDE_FLOAT32_C(   721.39), SIMDE_FLOAT32_C(  -631.24), SIMDE_FLOAT32_C(  -394.99) },
      { SIMDE_FLOAT32_C(   973.00), SIMDE_FLOAT32_C(   721.00), SIMDE_FLOAT32_C(  -632.00), SIMDE_FLOAT32_C(  -395.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_svml_floor_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_svml_floor_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -495.36), SIMDE_FLOAT64_C(   574.97) },
      { SIMDE_FLOAT64_C(  -496.00), SIMDE_FLOAT64_C(   574.00) } },
    { { SIMDE_FLOAT64_C(  -571.90), SIMDE_FLOAT64_C(    -4.02) },
      { SIMDE_FLOAT64_C(  -572.00), SIMDE_FLOAT64_C(    -5.00) } },
    { { SIMDE_FLOAT64_C(  -111.97), SIMDE_FLOAT64_C(  -326.91) },
      { SIMDE_FLOAT64_C(  -112.00), SIMDE_FLOAT64_C(  -327.00) } },
    { { SIMDE_FLOAT64_C(  -366.90), SIMDE_FLOAT64_C(   909.28) },
      { SIMDE_FLOAT64_C(  -367.00), SIMDE_FLOAT64_C(   909.00) } },
    { { SIMDE_FLOAT64_C(  -637.61), SIMDE_FLOAT64_C(   377.44) },
      { SIMDE_FLOAT64_C(  -638.00), SIMDE_FLOAT64_C(   377.00) } },
    { { SIMDE_FLOAT64_C(   358.88), SIMDE_FLOAT64_C(   783.39) },
      { SIMDE_FLOAT64_C(   358.00), SIMDE_FLOAT64_C(   783.00) } },
    { { SIMDE_FLOAT64_C(   137.00), SIMDE_FLOAT64_C(  -315.38) },
      { SIMDE_FLOAT64_C(   137.00), SIMDE_FLOAT64_C(  -316.00) } },
    { { SIMDE_FLOAT64_C(    20.73), SIMDE_FLOAT64_C(  -927.12) },
      { SIMDE_FLOAT64_C(    20.00), SIMDE_FLOAT64_C(  -928.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_svml_floor_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_svml_floor_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -100.83), SIMDE_FLOAT32_C(   -16.23), SIMDE_FLOAT32_C(   689.00), SIMDE_FLOAT32_C(   627.12),
        SIMDE_FLOAT32_C(  -725.64), SIMDE_FLOAT32_C(  -272.67), SIMDE_FLOAT32_C(   477.57), SIMDE_FLOAT32_C(   968.62) },
      { SIMDE_FLOAT32_C(  -101.00), SIMDE_FLOAT32_C(   -17.00), SIMDE_FLOAT32_C(   689.00), SIMDE_FLOAT32_C(   627.00),
        SIMDE_FLOAT32_C(  -726.00), SIMDE_FLOAT32_C(  -273.00), SIMDE_FLOAT32_C(   477.00), SIMDE_FLOAT32_C(   968.00) } },
    { { SIMDE_FLOAT32_C(   259.55), SIMDE_FLOAT32_C(  -892.87), SIMDE_FLOAT32_C(    37.54), SIMDE_FLOAT32_C(  -594.84),
        SIMDE_FLOAT32_C(   992.66), SIMDE_FLOAT32_C(   528.53), SIMDE_FLOAT32_C(   -44.54), SIMDE_FLOAT32_C(   305.85) },
      { SIMDE_FLOAT32_C(   259.00), SIMDE_FLOAT32_C(  -893.00), SIMDE_FLOAT32_C(    37.00), SIMDE_FLOAT32_C(  -595.00),
        SIMDE_FLOAT32_C(   992.00), SIMDE_FLOAT32_C(   528.00), SIMDE_FLOAT32_C(   -45.00), SIMDE_FLOAT32_C(   305.00) } },
    { { SIMDE_FLOAT32_C(   785.51), SIMDE_FLOAT32_C(  -262.72), SIMDE_FLOAT32_C(   566.52), SIMDE_FLOAT32_C(  -760.14),
        SIMDE_FLOAT32_C(   801.95), SIMDE_FLOAT32_C(   597.73), SIMDE_FLOAT32_C(  -180.14), SIMDE_FLOAT32_C(   556.25) },
      { SIMDE_FLOAT32_C(   785.00), SIMDE_FLOAT32_C(  -263.00), SIMDE_FLOAT32_C(   566.00), SIMDE_FLOAT32_C(  -761.00),
        SIMDE_FLOAT32_C(   801.00), SIMDE_FLOAT32_C(   597.00), SIMDE_FLOAT32_C(  -181.00), SIMDE_FLOAT32_C(   556.00) } },
    { { SIMDE_FLOAT32_C(  -337.69), SIMDE_FLOAT32_C(  -509.08), SIMDE_FLOAT32_C(   665.71), SIMDE_FLOAT32_C(   342.73),
        SIMDE_FLOAT32_C(   672.76), SIMDE_FLOAT32_C(  -625.02), SIMDE_FLOAT32_C(   -13.36), SIMDE_FLOAT32_C(  -428.07) },
      { SIMDE_FLOAT32_C(  -338.00), SIMDE_FLOAT32_C(  -510.00), SIMDE_FLOAT32_C(   665.00), SIMDE_FLOAT32_C(   342.00),
        SIMDE_FLOAT32_C(   672.00), SIMDE_FLOAT32_C(  -626.00), SIMDE_FLOAT32_C(   -14.00), SIMDE_FLOAT32_C(  -429.00) } },
    { { SIMDE_FLOAT32_C(   358.75), SIMDE_FLOAT32_C(  -324.36), SIMDE_FLOAT32_C(  -800.95), SIMDE_FLOAT32_C(   633.11),
        SIMDE_FLOAT32_C(   402.96), SIMDE_FLOAT32_C(   676.62), SIMDE_FLOAT32_C(   601.73), SIMDE_FLOAT32_C(  -337.48) },
      { SIMDE_FLOAT32_C(   358.00), SIMDE_FLOAT32_C(  -325.00), SIMDE_FLOAT32_C(  -801.00), SIMDE_FLOAT32_C(   633.00),
        SIMDE_FLOAT32_C(   402.00), SIMDE_FLOAT32_C(   676.00), SIMDE_FLOAT32_C(   601.00), SIMDE_FLOAT32_C(  -338.00) } },
    { { SIMDE_FLOAT32_C(   783.75), SIMDE_FLOAT32_C(  -360.73), SIMDE_FLOAT32_C(    67.67), SIMDE_FLOAT32_C(   776.41),
        SIMDE_FLOAT32_C(  -832.20), SIMDE_FLOAT32_C(  -976.87), SIMDE_FLOAT32_C(    82.26), SIMDE_FLOAT32_C(   953.31) },
      { SIMDE_FLOAT32_C(   783.00), SIMDE_FLOAT32_C(  -361.00), SIMDE_FLOAT32_C(    67.00), SIMDE_FLOAT32_C(   776.00),
        SIMDE_FLOAT32_C(  -833.00), SIMDE_FLOAT32_C(  -977.00), SIMDE_FLOAT32_C(    82.00), SIMDE_FLOAT32_C(   953.00) } },
    { { SIMDE_FLOAT32_C(  -239.59), SIMDE_FLOAT32_C(  -351.22), SIMDE_FLOAT32_C(  -806.83), SIMDE_FLOAT32_C(  -437.64),
        SIMDE_FLOAT32_C(  -753.50), SIMDE_FLOAT32_C(    13.03), SIMDE_FLOAT32_C(  -881.39), SIMDE_FLOAT32_C(   -91.19) },
      { SIMDE_FLOAT32_C(  -240.00), SIMDE_FLOAT32_C(  -352.00), SIMDE_FLOAT32_C(  -807.00), SIMDE_FLOAT32_C(  -438.00),
        SIMDE_FLOAT32_C(  -754.00), SIMDE_FLOAT32_C(    13.00), SIMDE_FLOAT32_C(  -882.00), SIMDE_FLOAT32_C(   -92.00) } },
    { { SIMDE_FLOAT32_C(   503.95), SIMDE_FLOAT32_C(   784.32), SIMDE_FLOAT32_C(  -748.46), SIMDE_FLOAT32_C(   176.71),
        SIMDE_FLOAT32_C(  -840.70), SIMDE_FLOAT32_C(   238.18), SIMDE_FLOAT32_C(   748.64), SIMDE_FLOAT32_C(   518.06) },
      { SIMDE_FLOAT32_C(   503.00), SIMDE_FLOAT32_C(   784.00), SIMDE_FLOAT32_C(  -749.00), SIMDE_FLOAT32_C(   176.00),
        SIMDE_FLOAT32_C(  -841.00), SIMDE_FLOAT32_C(   238.00), SIMDE_FLOAT32_C(   748.00), SIMDE_FLOAT32_C(   518.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_svml_floor_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_svml_floor_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -780.47), SIMDE_FLOAT64_C(  -616.82), SIMDE_FLOAT64_C(  -962.48), SIMDE_FLOAT64_C(   -74.66) },
      { SIMDE_FLOAT64_C(  -781.00), SIMDE_FLOAT64_C(  -617.00), SIMDE_FLOAT64_C(  -963.00), SIMDE_FLOAT64_C(   -75.00) } },
    { { SIMDE_FLOAT64_C(  -359.82), SIMDE_FLOAT64_C(  -704.98), SIMDE_FLOAT64_C(    11.20), SIMDE_FLOAT64_C(   223.91) },
      { SIMDE_FLOAT64_C(  -360.00), SIMDE_FLOAT64_C(  -705.00), SIMDE_FLOAT64_C(    11.00), SIMDE_FLOAT64_C(   223.00) } },
    { { SIMDE_FLOAT64_C(   173.90), SIMDE_FLOAT64_C(   506.89), SIMDE_FLOAT64_C(   153.15), SIMDE_FLOAT64_C(  -180.08) },
      { SIMDE_FLOAT64_C(   173.00), SIMDE_FLOAT64_C(   506.00), SIMDE_FLOAT64_C(   153.00), SIMDE_FLOAT64_C(  -181.00) } },
    { { SIMDE_FLOAT64_C(  -673.54), SIMDE_FLOAT64_C(   252.79), SIMDE_FLOAT64_C(    95.13), SIMDE_FLOAT64_C(  -639.41) },
      { SIMDE_FLOAT64_C(  -674.00), SIMDE_FLOAT64_C(   252.00), SIMDE_FLOAT64_C(    95.00), SIMDE_FLOAT64_C(  -640.00) } },
    { { SIMDE_FLOAT64_C(  -419.46), SIMDE_FLOAT64_C(   418.21), SIMDE_FLOAT64_C(  -778.55), SIMDE_FLOAT64_C(  -706.38) },
      { SIMDE_FLOAT64_C(  -420.00), SIMDE_FLOAT64_C(   418.00), SIMDE_FLOAT64_C(  -779.00), SIMDE_FLOAT64_C(  -707.00) } },
    { { SIMDE_FLOAT64_C(  -178.87), SIMDE_FLOAT64_C(  -923.30), SIMDE_FLOAT64_C(  -302.46), SIMDE_FLOAT64_C(  -406.02) },
      { SIMDE_FLOAT64_C(  -179.00), SIMDE_FLOAT64_C(  -924.00), SIMDE_FLOAT64_C(  -303.00), SIMDE_FLOAT64_C(  -407.00) } },
    { { SIMDE_FLOAT64_C(   447.97), SIMDE_FLOAT64_C(   431.46), SIMDE_FLOAT64_C(  -217.97), SIMDE_FLOAT64_C(   -97.70) },
      { SIMDE_FLOAT64_C(   447.00), SIMDE_FLOAT64_C(   431.00), SIMDE_FLOAT64_C(  -218.00), SIMDE_FLOAT64_C(   -98.00) } },
    { { SIMDE_FLOAT64_C(   148.46), SIMDE_FLOAT64_C(   945.32), SIMDE_FLOAT64_C(  -663.02), SIMDE_FLOAT64_C(   367.98) },
      { SIMDE_FLOAT64_C(   148.00), SIMDE_FLOAT64_C(   945.00), SIMDE_FLOAT64_C(  -664.00), SIMDE_FLOAT64_C(   367.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_svml_floor_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_floor_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   739.06), SIMDE_FLOAT32_C(   515.75), SIMDE_FLOAT32_C(  -321.43), SIMDE_FLOAT32_C(  -796.82),
        SIMDE_FLOAT32_C(  -649.68), SIMDE_FLOAT32_C(  -774.53), SIMDE_FLOAT32_C(   961.31), SIMDE_FLOAT32_C(   113.28),
        SIMDE_FLOAT32_C(   -29.07), SIMDE_FLOAT32_C(  -213.74), SIMDE_FLOAT32_C(  -272.53), SIMDE_FLOAT32_C(  -357.78),
        SIMDE_FLOAT32_C(   211.62), SIMDE_FLOAT32_C(   164.32), SIMDE_FLOAT32_C(  -909.49), SIMDE_FLOAT32_C(   809.56) },
      { SIMDE_FLOAT32_C(   739.00), SIMDE_FLOAT32_C(   515.00), SIMDE_FLOAT32_C(  -322.00), SIMDE_FLOAT32_C(  -797.00),
        SIMDE_FLOAT32_C(  -650.00), SIMDE_FLOAT32_C(  -775.00), SIMDE_FLOAT32_C(   961.00), SIMDE_FLOAT32_C(   113.00),
        SIMDE_FLOAT32_C(   -30.00), SIMDE_FLOAT32_C(  -214.00), SIMDE_FLOAT32_C(  -273.00), SIMDE_FLOAT32_C(  -358.00),
        SIMDE_FLOAT32_C(   211.00), SIMDE_FLOAT32_C(   164.00), SIMDE_FLOAT32_C(  -910.00), SIMDE_FLOAT32_C(   809.00) } },
    { { SIMDE_FLOAT32_C(   405.65), SIMDE_FLOAT32_C(  -257.98), SIMDE_FLOAT32_C(  -364.12), SIMDE_FLOAT32_C(  -228.18),
        SIMDE_FLOAT32_C(   200.69), SIMDE_FLOAT32_C(   614.44), SIMDE_FLOAT32_C(  -198.53), SIMDE_FLOAT32_C(  -756.05),
        SIMDE_FLOAT32_C(  -833.98), SIMDE_FLOAT32_C(   480.36), SIMDE_FLOAT32_C(   574.27), SIMDE_FLOAT32_C(  -408.80),
        SIMDE_FLOAT32_C(   768.69), SIMDE_FLOAT32_C(   342.19), SIMDE_FLOAT32_C(   -17.03), SIMDE_FLOAT32_C(   507.75) },
      { SIMDE_FLOAT32_C(   405.00), SIMDE_FLOAT32_C(  -258.00), SIMDE_FLOAT32_C(  -365.00), SIMDE_FLOAT32_C(  -229.00),
        SIMDE_FLOAT32_C(   200.00), SIMDE_FLOAT32_C(   614.00), SIMDE_FLOAT32_C(  -199.00), SIMDE_FLOAT32_C(  -757.00),
        SIMDE_FLOAT32_C(  -834.00), SIMDE_FLOAT32_C(   480.00), SIMDE_FLOAT32_C(   574.00), SIMDE_FLOAT32_C(  -409.00),
        SIMDE_FLOAT32_C(   768.00), SIMDE_FLOAT32_C(   342.00), SIMDE_FLOAT32_C(   -18.00), SIMDE_FLOAT32_C(   507.00) } },
    { { SIMDE_FLOAT32_C(  -142.06), SIMDE_FLOAT32_C(   661.53), SIMDE_FLOAT32_C(   710.93), SIMDE_FLOAT32_C(   208.26),
        SIMDE_FLOAT32_C(   887.01), SIMDE_FLOAT32_C(   672.24), SIMDE_FLOAT32_C(  -678.46), SIMDE_FLOAT32_C(  -142.06),
        SIMDE_FLOAT32_C(  -541.50), SIMDE_FLOAT32_C(    49.01), SIMDE_FLOAT32_C(   500.16), SIMDE_FLOAT32_C(   670.12),
        SIMDE_FLOAT32_C(  -786.67), SIMDE_FLOAT32_C(   590.66), SIMDE_FLOAT32_C(   479.68), SIMDE_FLOAT32_C(   618.98) },
      { SIMDE_FLOAT32_C(  -143.00), SIMDE_FLOAT32_C(   661.00), SIMDE_FLOAT32_C(   710.00), SIMDE_FLOAT32_C(   208.00),
        SIMDE_FLOAT32_C(   887.00), SIMDE_FLOAT32_C(   672.00), SIMDE_FLOAT32_C(  -679.00), SIMDE_FLOAT32_C(  -143.00),
        SIMDE_FLOAT32_C(  -542.00), SIMDE_FLOAT32_C(    49.00), SIMDE_FLOAT32_C(   500.00), SIMDE_FLOAT32_C(   670.00),
        SIMDE_FLOAT32_C(  -787.00), SIMDE_FLOAT32_C(   590.00), SIMDE_FLOAT32_C(   479.00), SIMDE_FLOAT32_C(   618.00) } },
    { { SIMDE_FLOAT32_C(  -667.32), SIMDE_FLOAT32_C(  -884.44), SIMDE_FLOAT32_C(  -609.20), SIMDE_FLOAT32_C(   533.37),
        SIMDE_FLOAT32_C(   730.00), SIMDE_FLOAT32_C(   192.28), SIMDE_FLOAT32_C(   777.32), SIMDE_FLOAT32_C(   896.02),
        SIMDE_FLOAT32_C(  -327.36), SIMDE_FLOAT32_C(   351.59), SIMDE_FLOAT32_C(  -512.78), SIMDE_FLOAT32_C(  -558.68),
        SIMDE_FLOAT32_C(  -306.22), SIMDE_FLOAT32_C(   470.19), SIMDE_FLOAT32_C(   949.07), SIMDE_FLOAT32_C(   551.72) },
      { SIMDE_FLOAT32_C(  -668.00), SIMDE_FLOAT32_C(  -885.00), SIMDE_FLOAT32_C(  -610.00), SIMDE_FLOAT32_C(   533.00),
        SIMDE_FLOAT32_C(   730.00), SIMDE_FLOAT32_C(   192.00), SIMDE_FLOAT32_C(   777.00), SIMDE_FLOAT32_C(   896.00),
        SIMDE_FLOAT32_C(  -328.00), SIMDE_FLOAT32_C(   351.00), SIMDE_FLOAT32_C(  -513.00), SIMDE_FLOAT32_C(  -559.00),
        SIMDE_FLOAT32_C(  -307.00), SIMDE_FLOAT32_C(   470.00), SIMDE_FLOAT32_C(   949.00), SIMDE_FLOAT32_C(   551.00) } },
    { { SIMDE_FLOAT32_C(   131.72), SIMDE_FLOAT32_C(   660.01), SIMDE_FLOAT32_C(  -240.02), SIMDE_FLOAT32_C(    18.73),
        SIMDE_FLOAT32_C(   332.25), SIMDE_FLOAT32_C(    81.52), SIMDE_FLOAT32_C(   876.67), SIMDE_FLOAT32_C(   790.75),
        SIMDE_FLOAT32_C(  -869.47), SIMDE_FLOAT32_C(   376.83), SIMDE_FLOAT32_C(   460.87), SIMDE_FLOAT32_C(  -656.14),
        SIMDE_FLOAT32_C(   -32.51), SIMDE_FLOAT32_C(   -59.45), SIMDE_FLOAT32_C(   962.84), SIMDE_FLOAT32_C(   300.17) },
      { SIMDE_FLOAT32_C(   131.00), SIMDE_FLOAT32_C(   660.00), SIMDE_FLOAT32_C(  -241.00), SIMDE_FLOAT32_C(    18.00),
        SIMDE_FLOAT32_C(   332.00), SIMDE_FLOAT32_C(    81.00), SIMDE_FLOAT32_C(   876.00), SIMDE_FLOAT32_C(   790.00),
        SIMDE_FLOAT32_C(  -870.00), SIMDE_FLOAT32_C(   376.00), SIMDE_FLOAT32_C(   460.00), SIMDE_FLOAT32_C(  -657.00),
        SIMDE_FLOAT32_C(   -33.00), SIMDE_FLOAT32_C(   -60.00), SIMDE_FLOAT32_C(   962.00), SIMDE_FLOAT32_C(   300.00) } },
    { { SIMDE_FLOAT32_C(    56.12), SIMDE_FLOAT32_C(  -646.35), SIMDE_FLOAT32_C(  -166.46), SIMDE_FLOAT32_C(  -213.88),
        SIMDE_FLOAT32_C(   545.92), SIMDE_FLOAT32_C(  -389.14), SIMDE_FLOAT32_C(  -317.86), SIMDE_FLOAT32_C(  -781.44),
        SIMDE_FLOAT32_C(   962.45), SIMDE_FLOAT32_C(   169.37), SIMDE_FLOAT32_C(  -340.12), SIMDE_FLOAT32_C(  -343.77),
        SIMDE_FLOAT32_C(  -360.44), SIMDE_FLOAT32_C(  -391.05), SIMDE_FLOAT32_C(  -792.05), SIMDE_FLOAT32_C(   771.28) },
      { SIMDE_FLOAT32_C(    56.00), SIMDE_FLOAT32_C(  -647.00), SIMDE_FLOAT32_C(  -167.00), SIMDE_FLOAT32_C(  -214.00),
        SIMDE_FLOAT32_C(   545.00), SIMDE_FLOAT32_C(  -390.00), SIMDE_FLOAT32_C(  -318.00), SIMDE_FLOAT32_C(  -782.00),
        SIMDE_FLOAT32_C(   962.00), SIMDE_FLOAT32_C(   169.00), SIMDE_FLOAT32_C(  -341.00), SIMDE_FLOAT32_C(  -344.00),
        SIMDE_FLOAT32_C(  -361.00), SIMDE_FLOAT32_C(  -392.00), SIMDE_FLOAT32_C(  -793.00), SIMDE_FLOAT32_C(   771.00) } },
    { { SIMDE_FLOAT32_C(  -731.04), SIMDE_FLOAT32_C(   -32.07), SIMDE_FLOAT32_C(  -209.99), SIMDE_FLOAT32_C(   601.21),
        SIMDE_FLOAT32_C(  -950.55), SIMDE_FLOAT32_C(  -333.32), SIMDE_FLOAT32_C(   391.96), SIMDE_FLOAT32_C(  -820.02),
        SIMDE_FLOAT32_C(  -956.49), SIMDE_FLOAT32_C(  -147.17), SIMDE_FLOAT32_C(  -476.16), SIMDE_FLOAT32_C(    11.00),
        SIMDE_FLOAT32_C(   793.38), SIMDE_FLOAT32_C(  -513.32), SIMDE_FLOAT32_C(  -688.82), SIMDE_FLOAT32_C(  -150.50) },
      { SIMDE_FLOAT32_C(  -732.00), SIMDE_FLOAT32_C(   -33.00), SIMDE_FLOAT32_C(  -210.00), SIMDE_FLOAT32_C(   601.00),
        SIMDE_FLOAT32_C(  -951.00), SIMDE_FLOAT32_C(  -334.00), SIMDE_FLOAT32_C(   391.00), SIMDE_FLOAT32_C(  -821.00),
        SIMDE_FLOAT32_C(  -957.00), SIMDE_FLOAT32_C(  -148.00), SIMDE_FLOAT32_C(  -477.00), SIMDE_FLOAT32_C(    11.00),
        SIMDE_FLOAT32_C(   793.00), SIMDE_FLOAT32_C(  -514.00), SIMDE_FLOAT32_C(  -689.00), SIMDE_FLOAT32_C(  -151.00) } },
    { { SIMDE_FLOAT32_C(  -159.67), SIMDE_FLOAT32_C(   144.72), SIMDE_FLOAT32_C(   635.62), SIMDE_FLOAT32_C(  -613.75),
        SIMDE_FLOAT32_C(   755.58), SIMDE_FLOAT32_C(  -682.24), SIMDE_FLOAT32_C(  -395.19), SIMDE_FLOAT32_C(   718.03),
        SIMDE_FLOAT32_C(   487.12), SIMDE_FLOAT32_C(   264.69), SIMDE_FLOAT32_C(  -625.74), SIMDE_FLOAT32_C(  -873.32),
        SIMDE_FLOAT32_C(   873.65), SIMDE_FLOAT32_C(  -417.79), SIMDE_FLOAT32_C(   897.96), SIMDE_FLOAT32_C(  -857.39) },
      { SIMDE_FLOAT32_C(  -160.00), SIMDE_FLOAT32_C(   144.00), SIMDE_FLOAT32_C(   635.00), SIMDE_FLOAT32_C(  -614.00),
        SIMDE_FLOAT32_C(   755.00), SIMDE_FLOAT32_C(  -683.00), SIMDE_FLOAT32_C(  -396.00), SIMDE_FLOAT32_C(   718.00),
        SIMDE_FLOAT32_C(   487.00), SIMDE_FLOAT32_C(   264.00), SIMDE_FLOAT32_C(  -626.00), SIMDE_FLOAT32_C(  -874.00),
        SIMDE_FLOAT32_C(   873.00), SIMDE_FLOAT32_C(  -418.00), SIMDE_FLOAT32_C(   897.00), SIMDE_FLOAT32_C(  -858.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_floor_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_floor_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   838.31), SIMDE_FLOAT32_C(   390.56), SIMDE_FLOAT32_C(  -564.73), SIMDE_FLOAT32_C(   308.39),
        SIMDE_FLOAT32_C(   260.44), SIMDE_FLOAT32_C(  -533.14), SIMDE_FLOAT32_C(  -978.85), SIMDE_FLOAT32_C(  -130.38),
        SIMDE_FLOAT32_C(   176.76), SIMDE_FLOAT32_C(  -227.10), SIMDE_FLOAT32_C(  -128.58), SIMDE_FLOAT32_C(   463.85),
        SIMDE_FLOAT32_C(  -349.81), SIMDE_FLOAT32_C(   938.22), SIMDE_FLOAT32_C(  -414.35), SIMDE_FLOAT32_C(   715.39) },
      UINT8_C( 56),
      { SIMDE_FLOAT32_C(   324.66), SIMDE_FLOAT32_C(  -904.07), SIMDE_FLOAT32_C(   834.59), SIMDE_FLOAT32_C(  -638.12),
        SIMDE_FLOAT32_C(  -994.43), SIMDE_FLOAT32_C(  -322.02), SIMDE_FLOAT32_C(   105.22), SIMDE_FLOAT32_C(  -770.91),
        SIMDE_FLOAT32_C(   604.26), SIMDE_FLOAT32_C(  -988.26), SIMDE_FLOAT32_C(  -580.41), SIMDE_FLOAT32_C(   673.34),
        SIMDE_FLOAT32_C(   425.23), SIMDE_FLOAT32_C(   713.78), SIMDE_FLOAT32_C(   511.64), SIMDE_FLOAT32_C(  -184.21) },
      { SIMDE_FLOAT32_C(   838.31), SIMDE_FLOAT32_C(   390.56), SIMDE_FLOAT32_C(  -564.73), SIMDE_FLOAT32_C(  -639.00),
        SIMDE_FLOAT32_C(  -995.00), SIMDE_FLOAT32_C(  -323.00), SIMDE_FLOAT32_C(  -978.85), SIMDE_FLOAT32_C(  -130.38),
        SIMDE_FLOAT32_C(   176.76), SIMDE_FLOAT32_C(  -227.10), SIMDE_FLOAT32_C(  -128.58), SIMDE_FLOAT32_C(   463.85),
        SIMDE_FLOAT32_C(  -349.81), SIMDE_FLOAT32_C(   938.22), SIMDE_FLOAT32_C(  -414.35), SIMDE_FLOAT32_C(   715.39) } },
    { { SIMDE_FLOAT32_C(  -850.94), SIMDE_FLOAT32_C(  -179.97), SIMDE_FLOAT32_C(  -923.77), SIMDE_FLOAT32_C(  -384.08),
        SIMDE_FLOAT32_C(  -158.82), SIMDE_FLOAT32_C(   -54.15), SIMDE_FLOAT32_C(   792.68), SIMDE_FLOAT32_C(   614.08),
        SIMDE_FLOAT32_C(   817.27), SIMDE_FLOAT32_C(   256.54), SIMDE_FLOAT32_C(  -735.74), SIMDE_FLOAT32_C(   755.49),
        SIMDE_FLOAT32_C(   842.19), SIMDE_FLOAT32_C(   979.66), SIMDE_FLOAT32_C(   610.86), SIMDE_FLOAT32_C(   166.85) },
      UINT8_C(134),
      { SIMDE_FLOAT32_C(   445.45), SIMDE_FLOAT32_C(   528.73), SIMDE_FLOAT32_C(  -918.84), SIMDE_FLOAT32_C(  -876.56),
        SIMDE_FLOAT32_C(  -366.04), SIMDE_FLOAT32_C(  -689.75), SIMDE_FLOAT32_C(   727.70), SIMDE_FLOAT32_C(  -354.31),
        SIMDE_FLOAT32_C(  -270.16), SIMDE_FLOAT32_C(   401.04), SIMDE_FLOAT32_C(  -929.08), SIMDE_FLOAT32_C(  -556.38),
        SIMDE_FLOAT32_C(   -87.32), SIMDE_FLOAT32_C(  -113.29), SIMDE_FLOAT32_C(  -407.33), SIMDE_FLOAT32_C(   732.72) },
      { SIMDE_FLOAT32_C(  -850.94), SIMDE_FLOAT32_C(   528.00), SIMDE_FLOAT32_C(  -919.00), SIMDE_FLOAT32_C(  -384.08),
        SIMDE_FLOAT32_C(  -158.82), SIMDE_FLOAT32_C(   -54.15), SIMDE_FLOAT32_C(   792.68), SIMDE_FLOAT32_C(  -355.00),
        SIMDE_FLOAT32_C(   817.27), SIMDE_FLOAT32_C(   256.54), SIMDE_FLOAT32_C(  -735.74), SIMDE_FLOAT32_C(   755.49),
        SIMDE_FLOAT32_C(   842.19), SIMDE_FLOAT32_C(   979.66), SIMDE_FLOAT32_C(   610.86), SIMDE_FLOAT32_C(   166.85) } },
    { { SIMDE_FLOAT32_C(   -37.05), SIMDE_FLOAT32_C(   208.59), SIMDE_FLOAT32_C(  -426.10), SIMDE_FLOAT32_C(   908.80),
        SIMDE_FLOAT32_C(     1.27), SIMDE_FLOAT32_C(  -812.02), SIMDE_FLOAT32_C(   726.06), SIMDE_FLOAT32_C(  -742.19),
        SIMDE_FLOAT32_C(  -547.76), SIMDE_FLOAT32_C(   481.55), SIMDE_FLOAT32_C(  -900.00), SIMDE_FLOAT32_C(  -568.10),
        SIMDE_FLOAT32_C(    92.41), SIMDE_FLOAT32_C(   266.85), SIMDE_FLOAT32_C(  -492.51), SIMDE_FLOAT32_C(  -462.13) },
      UINT8_C(130),
      { SIMDE_FLOAT32_C(  -411.36), SIMDE_FLOAT32_C(  -338.69), SIMDE_FLOAT32_C(   429.54), SIMDE_FLOAT32_C(  -101.11),
        SIMDE_FLOAT32_C(  -610.99), SIMDE_FLOAT32_C(  -924.77), SIMDE_FLOAT32_C(   628.73), SIMDE_FLOAT32_C(   790.05),
        SIMDE_FLOAT32_C(  -853.85), SIMDE_FLOAT32_C(  -927.65), SIMDE_FLOAT32_C(  -297.26), SIMDE_FLOAT32_C(    32.86),
        SIMDE_FLOAT32_C(  -334.98), SIMDE_FLOAT32_C(  -564.55), SIMDE_FLOAT32_C(   995.81), SIMDE_FLOAT32_C(   873.62) },
      { SIMDE_FLOAT32_C(   -37.05), SIMDE_FLOAT32_C(  -339.00), SIMDE_FLOAT32_C(  -426.10), SIMDE_FLOAT32_C(   908.80),
        SIMDE_FLOAT32_C(     1.27), SIMDE_FLOAT32_C(  -812.02), SIMDE_FLOAT32_C(   726.06), SIMDE_FLOAT32_C(   790.00),
        SIMDE_FLOAT32_C(  -547.76), SIMDE_FLOAT32_C(   481.55), SIMDE_FLOAT32_C(  -900.00), SIMDE_FLOAT32_C(  -568.10),
        SIMDE_FLOAT32_C(    92.41), SIMDE_FLOAT32_C(   266.85), SIMDE_FLOAT32_C(  -492.51), SIMDE_FLOAT32_C(  -462.13) } },
    { { SIMDE_FLOAT32_C(     9.35), SIMDE_FLOAT32_C(   904.61), SIMDE_FLOAT32_C(  -125.11), SIMDE_FLOAT32_C(   197.33),
        SIMDE_FLOAT32_C(   630.67), SIMDE_FLOAT32_C(   132.70), SIMDE_FLOAT32_C(   649.56), SIMDE_FLOAT32_C(   112.22),
        SIMDE_FLOAT32_C(   232.70), SIMDE_FLOAT32_C(  -918.54), SIMDE_FLOAT32_C(  -795.36), SIMDE_FLOAT32_C(  -500.45),
        SIMDE_FLOAT32_C(  -411.05), SIMDE_FLOAT32_C(  -257.49), SIMDE_FLOAT32_C(   295.13), SIMDE_FLOAT32_C(   177.59) },
      UINT8_C(202),
      { SIMDE_FLOAT32_C(  -275.34), SIMDE_FLOAT32_C(  -923.51), SIMDE_FLOAT32_C(   792.83), SIMDE_FLOAT32_C(  -200.11),
        SIMDE_FLOAT32_C(   705.22), SIMDE_FLOAT32_C(   582.88), SIMDE_FLOAT32_C(   -53.96), SIMDE_FLOAT32_C(   777.57),
        SIMDE_FLOAT32_C(  -714.38), SIMDE_FLOAT32_C(   978.91), SIMDE_FLOAT32_C(  -557.41), SIMDE_FLOAT32_C(  -278.93),
        SIMDE_FLOAT32_C(   974.71), SIMDE_FLOAT32_C(  -683.79), SIMDE_FLOAT32_C(   730.42), SIMDE_FLOAT32_C(   879.32) },
      { SIMDE_FLOAT32_C(     9.35), SIMDE_FLOAT32_C(  -924.00), SIMDE_FLOAT32_C(  -125.11), SIMDE_FLOAT32_C(  -201.00),
        SIMDE_FLOAT32_C(   630.67), SIMDE_FLOAT32_C(   132.70), SIMDE_FLOAT32_C(   -54.00), SIMDE_FLOAT32_C(   777.00),
        SIMDE_FLOAT32_C(   232.70), SIMDE_FLOAT32_C(  -918.54), SIMDE_FLOAT32_C(  -795.36), SIMDE_FLOAT32_C(  -500.45),
        SIMDE_FLOAT32_C(  -411.05), SIMDE_FLOAT32_C(  -257.49), SIMDE_FLOAT32_C(   295.13), SIMDE_FLOAT32_C(   177.59) } },
    { { SIMDE_FLOAT32_C(   191.09), SIMDE_FLOAT32_C(   -72.26), SIMDE_FLOAT32_C(   509.99), SIMDE_FLOAT32_C(  -676.21),
        SIMDE_FLOAT32_C(  -422.69), SIMDE_FLOAT32_C(  -377.79), SIMDE_FLOAT32_C(   556.49), SIMDE_FLOAT32_C(  -341.23),
        SIMDE_FLOAT32_C(  -173.15), SIMDE_FLOAT32_C(  -943.96), SIMDE_FLOAT32_C(   247.72), SIMDE_FLOAT32_C(   569.36),
        SIMDE_FLOAT32_C(   351.17), SIMDE_FLOAT32_C(  -574.69), SIMDE_FLOAT32_C(   -26.83), SIMDE_FLOAT32_C(  -924.17) },
      UINT8_C(134),
      { SIMDE_FLOAT32_C(  -234.00), SIMDE_FLOAT32_C(  -124.28), SIMDE_FLOAT32_C(  -792.99), SIMDE_FLOAT32_C(  -651.12),
        SIMDE_FLOAT32_C(   821.76), SIMDE_FLOAT32_C(   984.58), SIMDE_FLOAT32_C(  -365.50), SIMDE_FLOAT32_C(   800.67),
        SIMDE_FLOAT32_C(  -572.83), SIMDE_FLOAT32_C(   355.57), SIMDE_FLOAT32_C(   775.38), SIMDE_FLOAT32_C(  -256.62),
        SIMDE_FLOAT32_C(    85.98), SIMDE_FLOAT32_C(   654.71), SIMDE_FLOAT32_C(   934.47), SIMDE_FLOAT32_C(  -986.27) },
      { SIMDE_FLOAT32_C(   191.09), SIMDE_FLOAT32_C(  -125.00), SIMDE_FLOAT32_C(  -793.00), SIMDE_FLOAT32_C(  -676.21),
        SIMDE_FLOAT32_C(  -422.69), SIMDE_FLOAT32_C(  -377.79), SIMDE_FLOAT32_C(   556.49), SIMDE_FLOAT32_C(   800.00),
        SIMDE_FLOAT32_C(  -173.15), SIMDE_FLOAT32_C(  -943.96), SIMDE_FLOAT32_C(   247.72), SIMDE_FLOAT32_C(   569.36),
        SIMDE_FLOAT32_C(   351.17), SIMDE_FLOAT32_C(  -574.69), SIMDE_FLOAT32_C(   -26.83), SIMDE_FLOAT32_C(  -924.17) } },
    { { SIMDE_FLOAT32_C(   164.70), SIMDE_FLOAT32_C(  -741.74), SIMDE_FLOAT32_C(  -408.96), SIMDE_FLOAT32_C(   786.91),
        SIMDE_FLOAT32_C(   814.76), SIMDE_FLOAT32_C(   249.81), SIMDE_FLOAT32_C(  -386.24), SIMDE_FLOAT32_C(   870.80),
        SIMDE_FLOAT32_C(  -502.47), SIMDE_FLOAT32_C(  -816.88), SIMDE_FLOAT32_C(   221.97), SIMDE_FLOAT32_C(   -77.16),
        SIMDE_FLOAT32_C(   156.29), SIMDE_FLOAT32_C(   297.80), SIMDE_FLOAT32_C(   424.63), SIMDE_FLOAT32_C(   922.29) },
      UINT8_C(198),
      { SIMDE_FLOAT32_C(   631.65), SIMDE_FLOAT32_C(  -728.83), SIMDE_FLOAT32_C(   995.29), SIMDE_FLOAT32_C(   616.23),
        SIMDE_FLOAT32_C(   -94.34), SIMDE_FLOAT32_C(   795.96), SIMDE_FLOAT32_C(  -956.60), SIMDE_FLOAT32_C(  -738.77),
        SIMDE_FLOAT32_C(   571.34), SIMDE_FLOAT32_C(  -213.23), SIMDE_FLOAT32_C(   347.21), SIMDE_FLOAT32_C(   226.05),
        SIMDE_FLOAT32_C(  -278.76), SIMDE_FLOAT32_C(   360.94), SIMDE_FLOAT32_C(  -609.25), SIMDE_FLOAT32_C(   -20.49) },
      { SIMDE_FLOAT32_C(   164.70), SIMDE_FLOAT32_C(  -729.00), SIMDE_FLOAT32_C(   995.00), SIMDE_FLOAT32_C(   786.91),
        SIMDE_FLOAT32_C(   814.76), SIMDE_FLOAT32_C(   249.81), SIMDE_FLOAT32_C(  -957.00), SIMDE_FLOAT32_C(  -739.00),
        SIMDE_FLOAT32_C(  -502.47), SIMDE_FLOAT32_C(  -816.88), SIMDE_FLOAT32_C(   221.97), SIMDE_FLOAT32_C(   -77.16),
        SIMDE_FLOAT32_C(   156.29), SIMDE_FLOAT32_C(   297.80), SIMDE_FLOAT32_C(   424.63), SIMDE_FLOAT32_C(   922.29) } },
    { { SIMDE_FLOAT32_C(   951.98), SIMDE_FLOAT32_C(  -822.34), SIMDE_FLOAT32_C(  -205.73), SIMDE_FLOAT32_C(   201.79),
        SIMDE_FLOAT32_C(  -208.58), SIMDE_FLOAT32_C(  -334.93), SIMDE_FLOAT32_C(   699.32), SIMDE_FLOAT32_C(   -25.46),
        SIMDE_FLOAT32_C(   887.04), SIMDE_FLOAT32_C(  -377.85), SIMDE_FLOAT32_C(  -869.17), SIMDE_FLOAT32_C(   184.84),
        SIMDE_FLOAT32_C(  -953.21), SIMDE_FLOAT32_C(  -946.88), SIMDE_FLOAT32_C(   358.36), SIMDE_FLOAT32_C(   678.43) },
      UINT8_C(118),
      { SIMDE_FLOAT32_C(   353.65), SIMDE_FLOAT32_C(   294.66), SIMDE_FLOAT32_C(   229.95), SIMDE_FLOAT32_C(   149.61),
        SIMDE_FLOAT32_C(   338.06), SIMDE_FLOAT32_C(   491.18), SIMDE_FLOAT32_C(  -279.05), SIMDE_FLOAT32_C(  -875.17),
        SIMDE_FLOAT32_C(  -161.61), SIMDE_FLOAT32_C(   947.00), SIMDE_FLOAT32_C(  -153.92), SIMDE_FLOAT32_C(  -800.67),
        SIMDE_FLOAT32_C(  -662.25), SIMDE_FLOAT32_C(   825.58), SIMDE_FLOAT32_C(  -848.68), SIMDE_FLOAT32_C(  -484.59) },
      { SIMDE_FLOAT32_C(   951.98), SIMDE_FLOAT32_C(   294.00), SIMDE_FLOAT32_C(   229.00), SIMDE_FLOAT32_C(   201.79),
        SIMDE_FLOAT32_C(   338.00), SIMDE_FLOAT32_C(   491.00), SIMDE_FLOAT32_C(  -280.00), SIMDE_FLOAT32_C(   -25.46),
        SIMDE_FLOAT32_C(   887.04), SIMDE_FLOAT32_C(  -377.85), SIMDE_FLOAT32_C(  -869.17), SIMDE_FLOAT32_C(   184.84),
        SIMDE_FLOAT32_C(  -953.21), SIMDE_FLOAT32_C(  -946.88), SIMDE_FLOAT32_C(   358.36), SIMDE_FLOAT32_C(   678.43) } },
    { { SIMDE_FLOAT32_C(  -380.15), SIMDE_FLOAT32_C(   353.11), SIMDE_FLOAT32_C(   306.83), SIMDE_FLOAT32_C(   284.92),
        SIMDE_FLOAT32_C(    52.42), SIMDE_FLOAT32_C(  -718.63), SIMDE_FLOAT32_C(   171.96), SIMDE_FLOAT32_C(   674.58),
        SIMDE_FLOAT32_C(  -587.81), SIMDE_FLOAT32_C(  -643.20), SIMDE_FLOAT32_C(   721.36), SIMDE_FLOAT32_C(  -534.69),
        SIMDE_FLOAT32_C(   715.16), SIMDE_FLOAT32_C(   399.80), SIMDE_FLOAT32_C(  -210.40), SIMDE_FLOAT32_C(    68.81) },
         UINT8_MAX,
      { SIMDE_FLOAT32_C(  -980.45), SIMDE_FLOAT32_C(  -781.58), SIMDE_FLOAT32_C(  -967.49), SIMDE_FLOAT32_C(   510.73),
        SIMDE_FLOAT32_C(   -60.62), SIMDE_FLOAT32_C(  -842.65), SIMDE_FLOAT32_C(  -650.88), SIMDE_FLOAT32_C(  -113.62),
        SIMDE_FLOAT32_C(     3.42), SIMDE_FLOAT32_C(  -451.55), SIMDE_FLOAT32_C(   224.13), SIMDE_FLOAT32_C(  -170.99),
        SIMDE_FLOAT32_C(  -300.23), SIMDE_FLOAT32_C(   739.54), SIMDE_FLOAT32_C(   448.86), SIMDE_FLOAT32_C(  -947.12) },
      { SIMDE_FLOAT32_C(  -981.00), SIMDE_FLOAT32_C(  -782.00), SIMDE_FLOAT32_C(  -968.00), SIMDE_FLOAT32_C(   510.00),
        SIMDE_FLOAT32_C(   -61.00), SIMDE_FLOAT32_C(  -843.00), SIMDE_FLOAT32_C(  -651.00), SIMDE_FLOAT32_C(  -114.00),
        SIMDE_FLOAT32_C(  -587.81), SIMDE_FLOAT32_C(  -643.20), SIMDE_FLOAT32_C(   721.36), SIMDE_FLOAT32_C(  -534.69),
        SIMDE_FLOAT32_C(   715.16), SIMDE_FLOAT32_C(   399.80), SIMDE_FLOAT32_C(  -210.40), SIMDE_FLOAT32_C(    68.81) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_floor_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_floor_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -794.32), SIMDE_FLOAT64_C(   -48.70), SIMDE_FLOAT64_C(   277.79), SIMDE_FLOAT64_C(  -475.80),
        SIMDE_FLOAT64_C(  -876.95), SIMDE_FLOAT64_C(  -924.41), SIMDE_FLOAT64_C(   255.35), SIMDE_FLOAT64_C(  -243.50) },
      { SIMDE_FLOAT64_C(  -795.00), SIMDE_FLOAT64_C(   -49.00), SIMDE_FLOAT64_C(   277.00), SIMDE_FLOAT64_C(  -476.00),
        SIMDE_FLOAT64_C(  -877.00), SIMDE_FLOAT64_C(  -925.00), SIMDE_FLOAT64_C(   255.00), SIMDE_FLOAT64_C(  -244.00) } },
    { { SIMDE_FLOAT64_C(  -620.91), SIMDE_FLOAT64_C(  -173.96), SIMDE_FLOAT64_C(   275.90), SIMDE_FLOAT64_C(  -717.33),
        SIMDE_FLOAT64_C(  -402.37), SIMDE_FLOAT64_C(  -882.40), SIMDE_FLOAT64_C(    45.04), SIMDE_FLOAT64_C(  -141.04) },
      { SIMDE_FLOAT64_C(  -621.00), SIMDE_FLOAT64_C(  -174.00), SIMDE_FLOAT64_C(   275.00), SIMDE_FLOAT64_C(  -718.00),
        SIMDE_FLOAT64_C(  -403.00), SIMDE_FLOAT64_C(  -883.00), SIMDE_FLOAT64_C(    45.00), SIMDE_FLOAT64_C(  -142.00) } },
    { { SIMDE_FLOAT64_C(  -548.52), SIMDE_FLOAT64_C(  -215.27), SIMDE_FLOAT64_C(   977.63), SIMDE_FLOAT64_C(   913.41),
        SIMDE_FLOAT64_C(  -371.07), SIMDE_FLOAT64_C(   460.81), SIMDE_FLOAT64_C(   547.36), SIMDE_FLOAT64_C(  -452.52) },
      { SIMDE_FLOAT64_C(  -549.00), SIMDE_FLOAT64_C(  -216.00), SIMDE_FLOAT64_C(   977.00), SIMDE_FLOAT64_C(   913.00),
        SIMDE_FLOAT64_C(  -372.00), SIMDE_FLOAT64_C(   460.00), SIMDE_FLOAT64_C(   547.00), SIMDE_FLOAT64_C(  -453.00) } },
    { { SIMDE_FLOAT64_C(   -61.27), SIMDE_FLOAT64_C(  -606.40), SIMDE_FLOAT64_C(   310.76), SIMDE_FLOAT64_C(   420.51),
        SIMDE_FLOAT64_C(  -353.71), SIMDE_FLOAT64_C(  -327.75), SIMDE_FLOAT64_C(   663.33), SIMDE_FLOAT64_C(  -148.03) },
      { SIMDE_FLOAT64_C(   -62.00), SIMDE_FLOAT64_C(  -607.00), SIMDE_FLOAT64_C(   310.00), SIMDE_FLOAT64_C(   420.00),
        SIMDE_FLOAT64_C(  -354.00), SIMDE_FLOAT64_C(  -328.00), SIMDE_FLOAT64_C(   663.00), SIMDE_FLOAT64_C(  -149.00) } },
    { { SIMDE_FLOAT64_C(   623.55), SIMDE_FLOAT64_C(   -58.88), SIMDE_FLOAT64_C(   376.17), SIMDE_FLOAT64_C(   746.60),
        SIMDE_FLOAT64_C(    16.71), SIMDE_FLOAT64_C(  -368.49), SIMDE_FLOAT64_C(  -496.90), SIMDE_FLOAT64_C(   395.80) },
      { SIMDE_FLOAT64_C(   623.00), SIMDE_FLOAT64_C(   -59.00), SIMDE_FLOAT64_C(   376.00), SIMDE_FLOAT64_C(   746.00),
        SIMDE_FLOAT64_C(    16.00), SIMDE_FLOAT64_C(  -369.00), SIMDE_FLOAT64_C(  -497.00), SIMDE_FLOAT64_C(   395.00) } },
    { { SIMDE_FLOAT64_C(   457.55), SIMDE_FLOAT64_C(   779.00), SIMDE_FLOAT64_C(   678.47), SIMDE_FLOAT64_C(  -944.81),
        SIMDE_FLOAT64_C(   896.60), SIMDE_FLOAT64_C(  -276.49), SIMDE_FLOAT64_C(   -85.86), SIMDE_FLOAT64_C(  -651.92) },
      { SIMDE_FLOAT64_C(   457.00), SIMDE_FLOAT64_C(   779.00), SIMDE_FLOAT64_C(   678.00), SIMDE_FLOAT64_C(  -945.00),
        SIMDE_FLOAT64_C(   896.00), SIMDE_FLOAT64_C(  -277.00), SIMDE_FLOAT64_C(   -86.00), SIMDE_FLOAT64_C(  -652.00) } },
    { { SIMDE_FLOAT64_C(   508.25), SIMDE_FLOAT64_C(  -108.22), SIMDE_FLOAT64_C(  -738.51), SIMDE_FLOAT64_C(  -862.82),
        SIMDE_FLOAT64_C(  -647.41), SIMDE_FLOAT64_C(   808.85), SIMDE_FLOAT64_C(  -315.34), SIMDE_FLOAT64_C(   291.32) },
      { SIMDE_FLOAT64_C(   508.00), SIMDE_FLOAT64_C(  -109.00), SIMDE_FLOAT64_C(  -739.00), SIMDE_FLOAT64_C(  -863.00),
        SIMDE_FLOAT64_C(  -648.00), SIMDE_FLOAT64_C(   808.00), SIMDE_FLOAT64_C(  -316.00), SIMDE_FLOAT64_C(   291.00) } },
    { { SIMDE_FLOAT64_C(  -797.54), SIMDE_FLOAT64_C(   995.42), SIMDE_FLOAT64_C(  -288.16), SIMDE_FLOAT64_C(  -151.25),
        SIMDE_FLOAT64_C(  -332.32), SIMDE_FLOAT64_C(  -624.84), SIMDE_FLOAT64_C(   700.72), SIMDE_FLOAT64_C(  -708.77) },
      { SIMDE_FLOAT64_C(  -798.00), SIMDE_FLOAT64_C(   995.00), SIMDE_FLOAT64_C(  -289.00), SIMDE_FLOAT64_C(  -152.00),
        SIMDE_FLOAT64_C(  -333.00), SIMDE_FLOAT64_C(  -625.00), SIMDE_FLOAT64_C(   700.00), SIMDE_FLOAT64_C(  -709.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_floor_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_floor_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -886.28), SIMDE_FLOAT64_C(  -614.03), SIMDE_FLOAT64_C(  -883.55), SIMDE_FLOAT64_C(   865.29),
        SIMDE_FLOAT64_C(  -206.48), SIMDE_FLOAT64_C(   -34.33), SIMDE_FLOAT64_C(  -987.38), SIMDE_FLOAT64_C(   127.49) },
      UINT8_C(157),
      { SIMDE_FLOAT64_C(   163.94), SIMDE_FLOAT64_C(   134.54), SIMDE_FLOAT64_C(   245.58), SIMDE_FLOAT64_C(  -615.09),
        SIMDE_FLOAT64_C(    80.28), SIMDE_FLOAT64_C(   -93.17), SIMDE_FLOAT64_C(   181.16), SIMDE_FLOAT64_C(   303.02) },
      { SIMDE_FLOAT64_C(   163.00), SIMDE_FLOAT64_C(  -614.03), SIMDE_FLOAT64_C(   245.00), SIMDE_FLOAT64_C(  -616.00),
        SIMDE_FLOAT64_C(    80.00), SIMDE_FLOAT64_C(   -34.33), SIMDE_FLOAT64_C(  -987.38), SIMDE_FLOAT64_C(   303.00) } },
    { { SIMDE_FLOAT64_C(   377.85), SIMDE_FLOAT64_C(   999.13), SIMDE_FLOAT64_C(  -474.80), SIMDE_FLOAT64_C(   -29.53),
        SIMDE_FLOAT64_C(   777.92), SIMDE_FLOAT64_C(   307.60), SIMDE_FLOAT64_C(   178.13), SIMDE_FLOAT64_C(   680.84) },
      UINT8_C(246),
      { SIMDE_FLOAT64_C(    47.73), SIMDE_FLOAT64_C(   681.42), SIMDE_FLOAT64_C(  -141.66), SIMDE_FLOAT64_C(   574.99),
        SIMDE_FLOAT64_C(  -969.81), SIMDE_FLOAT64_C(   -27.94), SIMDE_FLOAT64_C(   960.96), SIMDE_FLOAT64_C(  -853.36) },
      { SIMDE_FLOAT64_C(   377.85), SIMDE_FLOAT64_C(   681.00), SIMDE_FLOAT64_C(  -142.00), SIMDE_FLOAT64_C(   -29.53),
        SIMDE_FLOAT64_C(  -970.00), SIMDE_FLOAT64_C(   -28.00), SIMDE_FLOAT64_C(   960.00), SIMDE_FLOAT64_C(  -854.00) } },
    { { SIMDE_FLOAT64_C(  -162.66), SIMDE_FLOAT64_C(  -245.52), SIMDE_FLOAT64_C(   112.31), SIMDE_FLOAT64_C(  -150.03),
        SIMDE_FLOAT64_C(   881.98), SIMDE_FLOAT64_C(   426.57), SIMDE_FLOAT64_C(  -986.09), SIMDE_FLOAT64_C(    16.51) },
      UINT8_C( 53),
      { SIMDE_FLOAT64_C(  -601.18), SIMDE_FLOAT64_C(  -903.21), SIMDE_FLOAT64_C(   578.99), SIMDE_FLOAT64_C(   579.98),
        SIMDE_FLOAT64_C(   399.82), SIMDE_FLOAT64_C(   -43.16), SIMDE_FLOAT64_C(   579.10), SIMDE_FLOAT64_C(   925.02) },
      { SIMDE_FLOAT64_C(  -602.00), SIMDE_FLOAT64_C(  -245.52), SIMDE_FLOAT64_C(   578.00), SIMDE_FLOAT64_C(  -150.03),
        SIMDE_FLOAT64_C(   399.00), SIMDE_FLOAT64_C(   -44.00), SIMDE_FLOAT64_C(  -986.09), SIMDE_FLOAT64_C(    16.51) } },
    { { SIMDE_FLOAT64_C(   927.31), SIMDE_FLOAT64_C(   357.02), SIMDE_FLOAT64_C(   232.62), SIMDE_FLOAT64_C(   105.44),
        SIMDE_FLOAT64_C(    37.87), SIMDE_FLOAT64_C(   434.25), SIMDE_FLOAT64_C(  -846.83), SIMDE_FLOAT64_C(  -280.72) },
      UINT8_C(253),
      { SIMDE_FLOAT64_C(   728.16), SIMDE_FLOAT64_C(  -250.53), SIMDE_FLOAT64_C(   264.65), SIMDE_FLOAT64_C(   689.12),
        SIMDE_FLOAT64_C(  -103.89), SIMDE_FLOAT64_C(  -898.01), SIMDE_FLOAT64_C(  -556.40), SIMDE_FLOAT64_C(  -991.58) },
      { SIMDE_FLOAT64_C(   728.00), SIMDE_FLOAT64_C(   357.02), SIMDE_FLOAT64_C(   264.00), SIMDE_FLOAT64_C(   689.00),
        SIMDE_FLOAT64_C(  -104.00), SIMDE_FLOAT64_C(  -899.00), SIMDE_FLOAT64_C(  -557.00), SIMDE_FLOAT64_C(  -992.00) } },
    { { SIMDE_FLOAT64_C(   -48.04), SIMDE_FLOAT64_C(  -674.42), SIMDE_FLOAT64_C(   434.99), SIMDE_FLOAT64_C(   -34.14),
        SIMDE_FLOAT64_C(   342.09), SIMDE_FLOAT64_C(  -892.85), SIMDE_FLOAT64_C(   364.68), SIMDE_FLOAT64_C(   438.89) },
      UINT8_C( 35),
      { SIMDE_FLOAT64_C(   -55.34), SIMDE_FLOAT64_C(  -161.30), SIMDE_FLOAT64_C(  -357.03), SIMDE_FLOAT64_C(  -476.24),
        SIMDE_FLOAT64_C(  -236.28), SIMDE_FLOAT64_C(  -429.72), SIMDE_FLOAT64_C(   880.78), SIMDE_FLOAT64_C(   996.35) },
      { SIMDE_FLOAT64_C(   -56.00), SIMDE_FLOAT64_C(  -162.00), SIMDE_FLOAT64_C(   434.99), SIMDE_FLOAT64_C(   -34.14),
        SIMDE_FLOAT64_C(   342.09), SIMDE_FLOAT64_C(  -430.00), SIMDE_FLOAT64_C(   364.68), SIMDE_FLOAT64_C(   438.89) } },
    { { SIMDE_FLOAT64_C(   675.71), SIMDE_FLOAT64_C(   -81.35), SIMDE_FLOAT64_C(   430.60), SIMDE_FLOAT64_C(   828.89),
        SIMDE_FLOAT64_C(   637.93), SIMDE_FLOAT64_C(   723.19), SIMDE_FLOAT64_C(   557.05), SIMDE_FLOAT64_C(  -612.60) },
      UINT8_C(162),
      { SIMDE_FLOAT64_C(   246.17), SIMDE_FLOAT64_C(   283.52), SIMDE_FLOAT64_C(    89.83), SIMDE_FLOAT64_C(   689.78),
        SIMDE_FLOAT64_C(   291.94), SIMDE_FLOAT64_C(  -958.21), SIMDE_FLOAT64_C(  -984.64), SIMDE_FLOAT64_C(  -273.07) },
      { SIMDE_FLOAT64_C(   675.71), SIMDE_FLOAT64_C(   283.00), SIMDE_FLOAT64_C(   430.60), SIMDE_FLOAT64_C(   828.89),
        SIMDE_FLOAT64_C(   637.93), SIMDE_FLOAT64_C(  -959.00), SIMDE_FLOAT64_C(   557.05), SIMDE_FLOAT64_C(  -274.00) } },
    { { SIMDE_FLOAT64_C(     7.65), SIMDE_FLOAT64_C(   357.45), SIMDE_FLOAT64_C(  -165.92), SIMDE_FLOAT64_C(  -627.67),
        SIMDE_FLOAT64_C(  -203.66), SIMDE_FLOAT64_C(  -479.79), SIMDE_FLOAT64_C(   316.99), SIMDE_FLOAT64_C(   635.04) },
      UINT8_C(211),
      { SIMDE_FLOAT64_C(   840.75), SIMDE_FLOAT64_C(  -601.24), SIMDE_FLOAT64_C(   733.46), SIMDE_FLOAT64_C(   721.53),
        SIMDE_FLOAT64_C(  -604.89), SIMDE_FLOAT64_C(   409.18), SIMDE_FLOAT64_C(  -359.82), SIMDE_FLOAT64_C(   825.71) },
      { SIMDE_FLOAT64_C(   840.00), SIMDE_FLOAT64_C(  -602.00), SIMDE_FLOAT64_C(  -165.92), SIMDE_FLOAT64_C(  -627.67),
        SIMDE_FLOAT64_C(  -605.00), SIMDE_FLOAT64_C(  -479.79), SIMDE_FLOAT64_C(  -360.00), SIMDE_FLOAT64_C(   825.00) } },
    { { SIMDE_FLOAT64_C(   238.07), SIMDE_FLOAT64_C(  -721.89), SIMDE_FLOAT64_C(   548.91), SIMDE_FLOAT64_C(  -204.89),
        SIMDE_FLOAT64_C(  -334.48), SIMDE_FLOAT64_C(  -463.26), SIMDE_FLOAT64_C(  -958.71), SIMDE_FLOAT64_C(   949.03) },
      UINT8_C(120),
      { SIMDE_FLOAT64_C(   731.06), SIMDE_FLOAT64_C(   240.97), SIMDE_FLOAT64_C(   668.36), SIMDE_FLOAT64_C(   746.42),
        SIMDE_FLOAT64_C(   967.90), SIMDE_FLOAT64_C(  -323.99), SIMDE_FLOAT64_C(   103.87), SIMDE_FLOAT64_C(  -198.02) },
      { SIMDE_FLOAT64_C(   238.07), SIMDE_FLOAT64_C(  -721.89), SIMDE_FLOAT64_C(   548.91), SIMDE_FLOAT64_C(   746.00),
        SIMDE_FLOAT64_C(   967.00), SIMDE_FLOAT64_C(  -324.00), SIMDE_FLOAT64_C(   103.00), SIMDE_FLOAT64_C(   949.03) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_floor_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_svml_round_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -381.64), SIMDE_FLOAT32_C(  -952.18), SIMDE_FLOAT32_C(   936.50), SIMDE_FLOAT32_C(  -269.57) },
      { SIMDE_FLOAT32_C(  -382.00), SIMDE_FLOAT32_C(  -952.00), SIMDE_FLOAT32_C(   937.00), SIMDE_FLOAT32_C(  -270.00) } },
    { { SIMDE_FLOAT32_C(   524.01), SIMDE_FLOAT32_C(   820.80), SIMDE_FLOAT32_C(  -576.54), SIMDE_FLOAT32_C(   493.48) },
      { SIMDE_FLOAT32_C(   524.00), SIMDE_FLOAT32_C(   821.00), SIMDE_FLOAT32_C(  -577.00), SIMDE_FLOAT32_C(   493.00) } },
    { { SIMDE_FLOAT32_C(  -183.12), SIMDE_FLOAT32_C(  -410.38), SIMDE_FLOAT32_C(   918.43), SIMDE_FLOAT32_C(   555.31) },
      { SIMDE_FLOAT32_C(  -183.00), SIMDE_FLOAT32_C(  -410.00), SIMDE_FLOAT32_C(   918.00), SIMDE_FLOAT32_C(   555.00) } },
    { { SIMDE_FLOAT32_C(  -777.47), SIMDE_FLOAT32_C(   961.82), SIMDE_FLOAT32_C(   -15.88), SIMDE_FLOAT32_C(  -545.38) },
      { SIMDE_FLOAT32_C(  -777.00), SIMDE_FLOAT32_C(   962.00), SIMDE_FLOAT32_C(   -16.00), SIMDE_FLOAT32_C(  -545.00) } },
    { { SIMDE_FLOAT32_C(   827.92), SIMDE_FLOAT32_C(  -576.14), SIMDE_FLOAT32_C(   188.86), SIMDE_FLOAT32_C(  -194.33) },
      { SIMDE_FLOAT32_C(   828.00), SIMDE_FLOAT32_C(  -576.00), SIMDE_FLOAT32_C(   189.00), SIMDE_FLOAT32_C(  -194.00) } },
    { { SIMDE_FLOAT32_C(  -357.49), SIMDE_FLOAT32_C(   544.93), SIMDE_FLOAT32_C(  -548.96), SIMDE_FLOAT32_C(   982.95) },
      { SIMDE_FLOAT32_C(  -357.00), SIMDE_FLOAT32_C(   545.00), SIMDE_FLOAT32_C(  -549.00), SIMDE_FLOAT32_C(   983.00) } },
    { { SIMDE_FLOAT32_C(  -811.59), SIMDE_FLOAT32_C(   502.24), SIMDE_FLOAT32_C(    18.44), SIMDE_FLOAT32_C(  -985.11) },
      { SIMDE_FLOAT32_C(  -812.00), SIMDE_FLOAT32_C(   502.00), SIMDE_FLOAT32_C(    18.00), SIMDE_FLOAT32_C(  -985.00) } },
    { { SIMDE_FLOAT32_C(  -901.60), SIMDE_FLOAT32_C(     1.79), SIMDE_FLOAT32_C(  -119.54), SIMDE_FLOAT32_C(  -283.24) },
      { SIMDE_FLOAT32_C(  -902.00), SIMDE_FLOAT32_C(     2.00), SIMDE_FLOAT32_C(  -120.00), SIMDE_FLOAT32_C(  -283.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_svml_round_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_svml_round_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -775.87), SIMDE_FLOAT64_C(   258.36) },
      { SIMDE_FLOAT64_C(  -776.00), SIMDE_FLOAT64_C(   258.00) } },
    { { SIMDE_FLOAT64_C(   698.30), SIMDE_FLOAT64_C(   -24.21) },
      { SIMDE_FLOAT64_C(   698.00), SIMDE_FLOAT64_C(   -24.00) } },
    { { SIMDE_FLOAT64_C(  -755.31), SIMDE_FLOAT64_C(  -751.07) },
      { SIMDE_FLOAT64_C(  -755.00), SIMDE_FLOAT64_C(  -751.00) } },
    { { SIMDE_FLOAT64_C(   607.87), SIMDE_FLOAT64_C(  -999.16) },
      { SIMDE_FLOAT64_C(   608.00), SIMDE_FLOAT64_C(  -999.00) } },
    { { SIMDE_FLOAT64_C(  -558.18), SIMDE_FLOAT64_C(  -447.90) },
      { SIMDE_FLOAT64_C(  -558.00), SIMDE_FLOAT64_C(  -448.00) } },
    { { SIMDE_FLOAT64_C(  -159.19), SIMDE_FLOAT64_C(   675.96) },
      { SIMDE_FLOAT64_C(  -159.00), SIMDE_FLOAT64_C(   676.00) } },
    { { SIMDE_FLOAT64_C(  -682.16), SIMDE_FLOAT64_C(   502.15) },
      { SIMDE_FLOAT64_C(  -682.00), SIMDE_FLOAT64_C(   502.00) } },
    { { SIMDE_FLOAT64_C(  -591.87), SIMDE_FLOAT64_C(   775.61) },
      { SIMDE_FLOAT64_C(  -592.00), SIMDE_FLOAT64_C(   776.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_svml_round_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_svml_round_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   380.84), SIMDE_FLOAT32_C(  -788.70), SIMDE_FLOAT32_C(   937.85), SIMDE_FLOAT32_C(   321.73),
        SIMDE_FLOAT32_C(   250.52), SIMDE_FLOAT32_C(  -410.85), SIMDE_FLOAT32_C(  -411.50), SIMDE_FLOAT32_C(  -342.15) },
      { SIMDE_FLOAT32_C(   381.00), SIMDE_FLOAT32_C(  -789.00), SIMDE_FLOAT32_C(   938.00), SIMDE_FLOAT32_C(   322.00),
        SIMDE_FLOAT32_C(   251.00), SIMDE_FLOAT32_C(  -411.00), SIMDE_FLOAT32_C(  -412.00), SIMDE_FLOAT32_C(  -342.00) } },
    { { SIMDE_FLOAT32_C(  -410.55), SIMDE_FLOAT32_C(   648.37), SIMDE_FLOAT32_C(   294.06), SIMDE_FLOAT32_C(   315.36),
        SIMDE_FLOAT32_C(  -375.65), SIMDE_FLOAT32_C(   783.04), SIMDE_FLOAT32_C(  -600.22), SIMDE_FLOAT32_C(  -208.94) },
      { SIMDE_FLOAT32_C(  -411.00), SIMDE_FLOAT32_C(   648.00), SIMDE_FLOAT32_C(   294.00), SIMDE_FLOAT32_C(   315.00),
        SIMDE_FLOAT32_C(  -376.00), SIMDE_FLOAT32_C(   783.00), SIMDE_FLOAT32_C(  -600.00), SIMDE_FLOAT32_C(  -209.00) } },
    { { SIMDE_FLOAT32_C(   628.12), SIMDE_FLOAT32_C(   178.11), SIMDE_FLOAT32_C(  -902.32), SIMDE_FLOAT32_C(  -420.94),
        SIMDE_FLOAT32_C(  -113.02), SIMDE_FLOAT32_C(   352.97), SIMDE_FLOAT32_C(  -796.40), SIMDE_FLOAT32_C(  -795.50) },
      { SIMDE_FLOAT32_C(   628.00), SIMDE_FLOAT32_C(   178.00), SIMDE_FLOAT32_C(  -902.00), SIMDE_FLOAT32_C(  -421.00),
        SIMDE_FLOAT32_C(  -113.00), SIMDE_FLOAT32_C(   353.00), SIMDE_FLOAT32_C(  -796.00), SIMDE_FLOAT32_C(  -796.00) } },
    { { SIMDE_FLOAT32_C(  -712.04), SIMDE_FLOAT32_C(   880.10), SIMDE_FLOAT32_C(   698.48), SIMDE_FLOAT32_C(  -638.58),
        SIMDE_FLOAT32_C(   349.16), SIMDE_FLOAT32_C(   163.60), SIMDE_FLOAT32_C(  -690.90), SIMDE_FLOAT32_C(  -270.00) },
      { SIMDE_FLOAT32_C(  -712.00), SIMDE_FLOAT32_C(   880.00), SIMDE_FLOAT32_C(   698.00), SIMDE_FLOAT32_C(  -639.00),
        SIMDE_FLOAT32_C(   349.00), SIMDE_FLOAT32_C(   164.00), SIMDE_FLOAT32_C(  -691.00), SIMDE_FLOAT32_C(  -270.00) } },
    { { SIMDE_FLOAT32_C(   374.90), SIMDE_FLOAT32_C(  -753.05), SIMDE_FLOAT32_C(  -948.26), SIMDE_FLOAT32_C(  -374.58),
        SIMDE_FLOAT32_C(  -163.90), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   283.27), SIMDE_FLOAT32_C(   425.55) },
      { SIMDE_FLOAT32_C(   375.00), SIMDE_FLOAT32_C(  -753.00), SIMDE_FLOAT32_C(  -948.00), SIMDE_FLOAT32_C(  -375.00),
        SIMDE_FLOAT32_C(  -164.00), SIMDE_FLOAT32_C(  -360.00), SIMDE_FLOAT32_C(   283.00), SIMDE_FLOAT32_C(   426.00) } },
    { { SIMDE_FLOAT32_C(  -711.40), SIMDE_FLOAT32_C(  -422.67), SIMDE_FLOAT32_C(  -259.09), SIMDE_FLOAT32_C(   -87.05),
        SIMDE_FLOAT32_C(  -639.63), SIMDE_FLOAT32_C(   140.69), SIMDE_FLOAT32_C(   704.01), SIMDE_FLOAT32_C(   988.49) },
      { SIMDE_FLOAT32_C(  -711.00), SIMDE_FLOAT32_C(  -423.00), SIMDE_FLOAT32_C(  -259.00), SIMDE_FLOAT32_C(   -87.00),
        SIMDE_FLOAT32_C(  -640.00), SIMDE_FLOAT32_C(   141.00), SIMDE_FLOAT32_C(   704.00), SIMDE_FLOAT32_C(   988.00) } },
    { { SIMDE_FLOAT32_C(  -681.20), SIMDE_FLOAT32_C(   801.69), SIMDE_FLOAT32_C(  -432.45), SIMDE_FLOAT32_C(   205.78),
        SIMDE_FLOAT32_C(   154.66), SIMDE_FLOAT32_C(  -228.84), SIMDE_FLOAT32_C(   410.28), SIMDE_FLOAT32_C(   442.62) },
      { SIMDE_FLOAT32_C(  -681.00), SIMDE_FLOAT32_C(   802.00), SIMDE_FLOAT32_C(  -432.00), SIMDE_FLOAT32_C(   206.00),
        SIMDE_FLOAT32_C(   155.00), SIMDE_FLOAT32_C(  -229.00), SIMDE_FLOAT32_C(   410.00), SIMDE_FLOAT32_C(   443.00) } },
    { { SIMDE_FLOAT32_C(  -348.74), SIMDE_FLOAT32_C(   108.77), SIMDE_FLOAT32_C(   804.05), SIMDE_FLOAT32_C(  -999.58),
        SIMDE_FLOAT32_C(  -727.63), SIMDE_FLOAT32_C(  -886.85), SIMDE_FLOAT32_C(  -269.57), SIMDE_FLOAT32_C(   647.26) },
      { SIMDE_FLOAT32_C(  -349.00), SIMDE_FLOAT32_C(   109.00), SIMDE_FLOAT32_C(   804.00), SIMDE_FLOAT32_C( -1000.00),
        SIMDE_FLOAT32_C(  -728.00), SIMDE_FLOAT32_C(  -887.00), SIMDE_FLOAT32_C(  -270.00), SIMDE_FLOAT32_C(   647.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_svml_round_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_svml_round_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -844.84), SIMDE_FLOAT64_C(  -247.28), SIMDE_FLOAT64_C(   192.26), SIMDE_FLOAT64_C(   426.25) },
      { SIMDE_FLOAT64_C(  -845.00), SIMDE_FLOAT64_C(  -247.00), SIMDE_FLOAT64_C(   192.00), SIMDE_FLOAT64_C(   426.00) } },
    { { SIMDE_FLOAT64_C(   -53.32), SIMDE_FLOAT64_C(  -778.93), SIMDE_FLOAT64_C(  -167.10), SIMDE_FLOAT64_C(  -593.25) },
      { SIMDE_FLOAT64_C(   -53.00), SIMDE_FLOAT64_C(  -779.00), SIMDE_FLOAT64_C(  -167.00), SIMDE_FLOAT64_C(  -593.00) } },
    { { SIMDE_FLOAT64_C(  -450.17), SIMDE_FLOAT64_C(  -606.32), SIMDE_FLOAT64_C(   101.38), SIMDE_FLOAT64_C(  -341.77) },
      { SIMDE_FLOAT64_C(  -450.00), SIMDE_FLOAT64_C(  -606.00), SIMDE_FLOAT64_C(   101.00), SIMDE_FLOAT64_C(  -342.00) } },
    { { SIMDE_FLOAT64_C(  -461.44), SIMDE_FLOAT64_C(   674.51), SIMDE_FLOAT64_C(   145.37), SIMDE_FLOAT64_C(   148.63) },
      { SIMDE_FLOAT64_C(  -461.00), SIMDE_FLOAT64_C(   675.00), SIMDE_FLOAT64_C(   145.00), SIMDE_FLOAT64_C(   149.00) } },
    { { SIMDE_FLOAT64_C(  -693.71), SIMDE_FLOAT64_C(  -933.34), SIMDE_FLOAT64_C(   117.11), SIMDE_FLOAT64_C(    52.36) },
      { SIMDE_FLOAT64_C(  -694.00), SIMDE_FLOAT64_C(  -933.00), SIMDE_FLOAT64_C(   117.00), SIMDE_FLOAT64_C(    52.00) } },
    { { SIMDE_FLOAT64_C(   574.82), SIMDE_FLOAT64_C(  -929.55), SIMDE_FLOAT64_C(   113.17), SIMDE_FLOAT64_C(  -272.97) },
      { SIMDE_FLOAT64_C(   575.00), SIMDE_FLOAT64_C(  -930.00), SIMDE_FLOAT64_C(   113.00), SIMDE_FLOAT64_C(  -273.00) } },
    { { SIMDE_FLOAT64_C(   102.14), SIMDE_FLOAT64_C(  -880.36), SIMDE_FLOAT64_C(   222.01), SIMDE_FLOAT64_C(  -844.37) },
      { SIMDE_FLOAT64_C(   102.00), SIMDE_FLOAT64_C(  -880.00), SIMDE_FLOAT64_C(   222.00), SIMDE_FLOAT64_C(  -844.00) } },
    { { SIMDE_FLOAT64_C(   363.52), SIMDE_FLOAT64_C(  -723.41), SIMDE_FLOAT64_C(   -68.69), SIMDE_FLOAT64_C(   518.69) },
      { SIMDE_FLOAT64_C(   364.00), SIMDE_FLOAT64_C(  -723.00), SIMDE_FLOAT64_C(   -69.00), SIMDE_FLOAT64_C(   519.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_svml_round_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_svml_round_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   838.26), SIMDE_FLOAT64_C(   713.09), SIMDE_FLOAT64_C(    41.96), SIMDE_FLOAT64_C(  -274.12),
        SIMDE_FLOAT64_C(   674.75), SIMDE_FLOAT64_C(   434.35), SIMDE_FLOAT64_C(  -404.30), SIMDE_FLOAT64_C(  -706.45) },
      { SIMDE_FLOAT64_C(   838.00), SIMDE_FLOAT64_C(   713.00), SIMDE_FLOAT64_C(    42.00), SIMDE_FLOAT64_C(  -274.00),
        SIMDE_FLOAT64_C(   675.00), SIMDE_FLOAT64_C(   434.00), SIMDE_FLOAT64_C(  -404.00), SIMDE_FLOAT64_C(  -706.00) } },
    { { SIMDE_FLOAT64_C(   764.33), SIMDE_FLOAT64_C(   549.73), SIMDE_FLOAT64_C(   946.10), SIMDE_FLOAT64_C(   543.69),
        SIMDE_FLOAT64_C(   399.24), SIMDE_FLOAT64_C(   840.23), SIMDE_FLOAT64_C(  -804.12), SIMDE_FLOAT64_C(    92.87) },
      { SIMDE_FLOAT64_C(   764.00), SIMDE_FLOAT64_C(   550.00), SIMDE_FLOAT64_C(   946.00), SIMDE_FLOAT64_C(   544.00),
        SIMDE_FLOAT64_C(   399.00), SIMDE_FLOAT64_C(   840.00), SIMDE_FLOAT64_C(  -804.00), SIMDE_FLOAT64_C(    93.00) } },
    { { SIMDE_FLOAT64_C(  -719.75), SIMDE_FLOAT64_C(  -288.44), SIMDE_FLOAT64_C(    -7.73), SIMDE_FLOAT64_C(   -17.69),
        SIMDE_FLOAT64_C(  -135.39), SIMDE_FLOAT64_C(  -783.16), SIMDE_FLOAT64_C(   -89.69), SIMDE_FLOAT64_C(  -576.47) },
      { SIMDE_FLOAT64_C(  -720.00), SIMDE_FLOAT64_C(  -288.00), SIMDE_FLOAT64_C(    -8.00), SIMDE_FLOAT64_C(   -18.00),
        SIMDE_FLOAT64_C(  -135.00), SIMDE_FLOAT64_C(  -783.00), SIMDE_FLOAT64_C(   -90.00), SIMDE_FLOAT64_C(  -576.00) } },
    { { SIMDE_FLOAT64_C(   729.17), SIMDE_FLOAT64_C(   679.53), SIMDE_FLOAT64_C(  -484.77), SIMDE_FLOAT64_C(   898.47),
        SIMDE_FLOAT64_C(  -408.70), SIMDE_FLOAT64_C(  -621.23), SIMDE_FLOAT64_C(  -109.48), SIMDE_FLOAT64_C(  -570.45) },
      { SIMDE_FLOAT64_C(   729.00), SIMDE_FLOAT64_C(   680.00), SIMDE_FLOAT64_C(  -485.00), SIMDE_FLOAT64_C(   898.00),
        SIMDE_FLOAT64_C(  -409.00), SIMDE_FLOAT64_C(  -621.00), SIMDE_FLOAT64_C(  -109.00), SIMDE_FLOAT64_C(  -570.00) } },
    { { SIMDE_FLOAT64_C(  -908.13), SIMDE_FLOAT64_C(   932.48), SIMDE_FLOAT64_C(   155.44), SIMDE_FLOAT64_C(   766.61),
        SIMDE_FLOAT64_C(   366.83), SIMDE_FLOAT64_C(   751.14), SIMDE_FLOAT64_C(  -939.84), SIMDE_FLOAT64_C(   131.16) },
      { SIMDE_FLOAT64_C(  -908.00), SIMDE_FLOAT64_C(   932.00), SIMDE_FLOAT64_C(   155.00), SIMDE_FLOAT64_C(   767.00),
        SIMDE_FLOAT64_C(   367.00), SIMDE_FLOAT64_C(   751.00), SIMDE_FLOAT64_C(  -940.00), SIMDE_FLOAT64_C(   131.00) } },
    { { SIMDE_FLOAT64_C(   300.87), SIMDE_FLOAT64_C(  -993.74), SIMDE_FLOAT64_C(  -325.15), SIMDE_FLOAT64_C(  -299.89),
        SIMDE_FLOAT64_C(   846.49), SIMDE_FLOAT64_C(  -129.27), SIMDE_FLOAT64_C(   792.98), SIMDE_FLOAT64_C(  -873.26) },
      { SIMDE_FLOAT64_C(   301.00), SIMDE_FLOAT64_C(  -994.00), SIMDE_FLOAT64_C(  -325.00), SIMDE_FLOAT64_C(  -300.00),
        SIMDE_FLOAT64_C(   846.00), SIMDE_FLOAT64_C(  -129.00), SIMDE_FLOAT64_C(   793.00), SIMDE_FLOAT64_C(  -873.00) } },
    { { SIMDE_FLOAT64_C(   582.29), SIMDE_FLOAT64_C(  -214.75), SIMDE_FLOAT64_C(   109.05), SIMDE_FLOAT64_C(  -553.10),
        SIMDE_FLOAT64_C(     2.09), SIMDE_FLOAT64_C(  -980.64), SIMDE_FLOAT64_C(  -129.57), SIMDE_FLOAT64_C(  -268.74) },
      { SIMDE_FLOAT64_C(   582.00), SIMDE_FLOAT64_C(  -215.00), SIMDE_FLOAT64_C(   109.00), SIMDE_FLOAT64_C(  -553.00),
        SIMDE_FLOAT64_C(     2.00), SIMDE_FLOAT64_C(  -981.00), SIMDE_FLOAT64_C(  -130.00), SIMDE_FLOAT64_C(  -269.00) } },
    { { SIMDE_FLOAT64_C(   698.88), SIMDE_FLOAT64_C(   385.66), SIMDE_FLOAT64_C(  -370.28), SIMDE_FLOAT64_C(  -709.82),
        SIMDE_FLOAT64_C(   764.44), SIMDE_FLOAT64_C(   520.25), SIMDE_FLOAT64_C(  -280.27), SIMDE_FLOAT64_C(   856.30) },
      { SIMDE_FLOAT64_C(   699.00), SIMDE_FLOAT64_C(   386.00), SIMDE_FLOAT64_C(  -370.00), SIMDE_FLOAT64_C(  -710.00),
        SIMDE_FLOAT64_C(   764.00), SIMDE_FLOAT64_C(   520.00), SIMDE_FLOAT64_C(  -280.00), SIMDE_FLOAT64_C(   856.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_svml_round_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_svml_round_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   686.15), SIMDE_FLOAT64_C(   113.85), SIMDE_FLOAT64_C(    91.19), SIMDE_FLOAT64_C(   346.08),
        SIMDE_FLOAT64_C(  -785.05), SIMDE_FLOAT64_C(   656.94), SIMDE_FLOAT64_C(   111.39), SIMDE_FLOAT64_C(  -488.16) },
      UINT8_C(184),
      { SIMDE_FLOAT64_C(  -283.66), SIMDE_FLOAT64_C(   587.43), SIMDE_FLOAT64_C(  -235.49), SIMDE_FLOAT64_C(   163.12),
        SIMDE_FLOAT64_C(   571.24), SIMDE_FLOAT64_C(   582.37), SIMDE_FLOAT64_C(  -370.22), SIMDE_FLOAT64_C(   474.92) },
      { SIMDE_FLOAT64_C(   686.15), SIMDE_FLOAT64_C(   113.85), SIMDE_FLOAT64_C(    91.19), SIMDE_FLOAT64_C(   163.00),
        SIMDE_FLOAT64_C(   571.00), SIMDE_FLOAT64_C(   582.00), SIMDE_FLOAT64_C(   111.39), SIMDE_FLOAT64_C(   475.00) } },
    { { SIMDE_FLOAT64_C(   -66.51), SIMDE_FLOAT64_C(  -591.67), SIMDE_FLOAT64_C(   -91.31), SIMDE_FLOAT64_C(   225.56),
        SIMDE_FLOAT64_C(    12.37), SIMDE_FLOAT64_C(  -659.70), SIMDE_FLOAT64_C(  -760.80), SIMDE_FLOAT64_C(   231.33) },
      UINT8_C( 69),
      { SIMDE_FLOAT64_C(   115.84), SIMDE_FLOAT64_C(  -400.68), SIMDE_FLOAT64_C(  -849.91), SIMDE_FLOAT64_C(   -49.83),
        SIMDE_FLOAT64_C(    85.28), SIMDE_FLOAT64_C(   836.24), SIMDE_FLOAT64_C(  -935.98), SIMDE_FLOAT64_C(  -823.53) },
      { SIMDE_FLOAT64_C(   116.00), SIMDE_FLOAT64_C(  -591.67), SIMDE_FLOAT64_C(  -850.00), SIMDE_FLOAT64_C(   225.56),
        SIMDE_FLOAT64_C(    12.37), SIMDE_FLOAT64_C(  -659.70), SIMDE_FLOAT64_C(  -936.00), SIMDE_FLOAT64_C(   231.33) } },
    { { SIMDE_FLOAT64_C(   182.32), SIMDE_FLOAT64_C(  -721.03), SIMDE_FLOAT64_C(   833.41), SIMDE_FLOAT64_C(  -706.29),
        SIMDE_FLOAT64_C(  -209.20), SIMDE_FLOAT64_C(  -511.45), SIMDE_FLOAT64_C(    10.05), SIMDE_FLOAT64_C(  -621.76) },
      UINT8_C(223),
      { SIMDE_FLOAT64_C(  -826.83), SIMDE_FLOAT64_C(   949.47), SIMDE_FLOAT64_C(  -164.57), SIMDE_FLOAT64_C(  -197.05),
        SIMDE_FLOAT64_C(   424.40), SIMDE_FLOAT64_C(   768.92), SIMDE_FLOAT64_C(   211.28), SIMDE_FLOAT64_C(  -666.92) },
      { SIMDE_FLOAT64_C(  -827.00), SIMDE_FLOAT64_C(   949.00), SIMDE_FLOAT64_C(  -165.00), SIMDE_FLOAT64_C(  -197.00),
        SIMDE_FLOAT64_C(   424.00), SIMDE_FLOAT64_C(  -511.45), SIMDE_FLOAT64_C(   211.00), SIMDE_FLOAT64_C(  -667.00) } },
    { { SIMDE_FLOAT64_C(    -5.52), SIMDE_FLOAT64_C(  -776.35), SIMDE_FLOAT64_C(  -326.62), SIMDE_FLOAT64_C(   233.68),
        SIMDE_FLOAT64_C(   454.98), SIMDE_FLOAT64_C(   714.97), SIMDE_FLOAT64_C(  -650.48), SIMDE_FLOAT64_C(  -945.69) },
      UINT8_C(115),
      { SIMDE_FLOAT64_C(   299.69), SIMDE_FLOAT64_C(   139.59), SIMDE_FLOAT64_C(   701.29), SIMDE_FLOAT64_C(   363.71),
        SIMDE_FLOAT64_C(   316.05), SIMDE_FLOAT64_C(  -116.39), SIMDE_FLOAT64_C(   642.67), SIMDE_FLOAT64_C(   149.46) },
      { SIMDE_FLOAT64_C(   300.00), SIMDE_FLOAT64_C(   140.00), SIMDE_FLOAT64_C(  -326.62), SIMDE_FLOAT64_C(   233.68),
        SIMDE_FLOAT64_C(   316.00), SIMDE_FLOAT64_C(  -116.00), SIMDE_FLOAT64_C(   643.00), SIMDE_FLOAT64_C(  -945.69) } },
    { { SIMDE_FLOAT64_C(   177.32), SIMDE_FLOAT64_C(  -566.52), SIMDE_FLOAT64_C(   638.01), SIMDE_FLOAT64_C(  -812.62),
        SIMDE_FLOAT64_C(  -188.29), SIMDE_FLOAT64_C(  -108.94), SIMDE_FLOAT64_C(  -639.45), SIMDE_FLOAT64_C(  -238.81) },
      UINT8_C( 57),
      { SIMDE_FLOAT64_C(   163.50), SIMDE_FLOAT64_C(  -814.42), SIMDE_FLOAT64_C(   495.41), SIMDE_FLOAT64_C(  -625.21),
        SIMDE_FLOAT64_C(  -481.34), SIMDE_FLOAT64_C(  -510.10), SIMDE_FLOAT64_C(  -401.56), SIMDE_FLOAT64_C(   192.04) },
      { SIMDE_FLOAT64_C(   164.00), SIMDE_FLOAT64_C(  -566.52), SIMDE_FLOAT64_C(   638.01), SIMDE_FLOAT64_C(  -625.00),
        SIMDE_FLOAT64_C(  -481.00), SIMDE_FLOAT64_C(  -510.00), SIMDE_FLOAT64_C(  -639.45), SIMDE_FLOAT64_C(  -238.81) } },
    { { SIMDE_FLOAT64_C(   723.58), SIMDE_FLOAT64_C(  -946.57), SIMDE_FLOAT64_C(   -92.99), SIMDE_FLOAT64_C(  -926.90),
        SIMDE_FLOAT64_C(  -892.27), SIMDE_FLOAT64_C(  -227.94), SIMDE_FLOAT64_C(   372.79), SIMDE_FLOAT64_C(   247.32) },
      UINT8_C(253),
      { SIMDE_FLOAT64_C(  -263.51), SIMDE_FLOAT64_C(  -436.63), SIMDE_FLOAT64_C(   356.97), SIMDE_FLOAT64_C(  -620.84),
        SIMDE_FLOAT64_C(   712.84), SIMDE_FLOAT64_C(  -465.71), SIMDE_FLOAT64_C(  -187.36), SIMDE_FLOAT64_C(   350.85) },
      { SIMDE_FLOAT64_C(  -264.00), SIMDE_FLOAT64_C(  -946.57), SIMDE_FLOAT64_C(   357.00), SIMDE_FLOAT64_C(  -621.00),
        SIMDE_FLOAT64_C(   713.00), SIMDE_FLOAT64_C(  -466.00), SIMDE_FLOAT64_C(  -187.00), SIMDE_FLOAT64_C(   351.00) } },
    { { SIMDE_FLOAT64_C(  -278.33), SIMDE_FLOAT64_C(   624.35), SIMDE_FLOAT64_C(  -758.09), SIMDE_FLOAT64_C(    82.22),
        SIMDE_FLOAT64_C(  -614.46), SIMDE_FLOAT64_C(   968.40), SIMDE_FLOAT64_C(  -754.27), SIMDE_FLOAT64_C(  -428.88) },
      UINT8_C( 24),
      { SIMDE_FLOAT64_C(  -379.49), SIMDE_FLOAT64_C(    89.78), SIMDE_FLOAT64_C(   953.71), SIMDE_FLOAT64_C(   218.96),
        SIMDE_FLOAT64_C(  -718.17), SIMDE_FLOAT64_C(   677.29), SIMDE_FLOAT64_C(   272.38), SIMDE_FLOAT64_C(   188.83) },
      { SIMDE_FLOAT64_C(  -278.33), SIMDE_FLOAT64_C(   624.35), SIMDE_FLOAT64_C(  -758.09), SIMDE_FLOAT64_C(   219.00),
        SIMDE_FLOAT64_C(  -718.00), SIMDE_FLOAT64_C(   968.40), SIMDE_FLOAT64_C(  -754.27), SIMDE_FLOAT64_C(  -428.88) } },
    { { SIMDE_FLOAT64_C(   750.39), SIMDE_FLOAT64_C(   380.12), SIMDE_FLOAT64_C(   960.90), SIMDE_FLOAT64_C(   123.18),
        SIMDE_FLOAT64_C(  -372.56), SIMDE_FLOAT64_C(  -565.75), SIMDE_FLOAT64_C(   859.67), SIMDE_FLOAT64_C(   190.81) },
      UINT8_C(196),
      { SIMDE_FLOAT64_C(  -761.17), SIMDE_FLOAT64_C(   -96.36), SIMDE_FLOAT64_C(  -674.48), SIMDE_FLOAT64_C(    51.47),
        SIMDE_FLOAT64_C(  -745.51), SIMDE_FLOAT64_C(    47.19), SIMDE_FLOAT64_C(  -324.18), SIMDE_FLOAT64_C(  -503.60) },
      { SIMDE_FLOAT64_C(   750.39), SIMDE_FLOAT64_C(   380.12), SIMDE_FLOAT64_C(  -674.00), SIMDE_FLOAT64_C(   123.18),
        SIMDE_FLOAT64_C(  -372.56), SIMDE_FLOAT64_C(  -565.75), SIMDE_FLOAT64_C(  -324.00), SIMDE_FLOAT64_C(  -504.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_svml_round_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_svml_sqrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   770.44), SIMDE_FLOAT64_C(   798.21) },
      { SIMDE_FLOAT64_C(    27.76), SIMDE_FLOAT64_C(    28.25) } },
    { { SIMDE_FLOAT64_C(   609.46), SIMDE_FLOAT64_C(   219.02) },
      { SIMDE_FLOAT64_C(    24.69), SIMDE_FLOAT64_C(    14.80) } },
    { { SIMDE_FLOAT64_C(   514.28), SIMDE_FLOAT64_C(   301.39) },
      { SIMDE_FLOAT64_C(    22.68), SIMDE_FLOAT64_C(    17.36) } },
    { { SIMDE_FLOAT64_C(   520.55), SIMDE_FLOAT64_C(   108.95) },
      { SIMDE_FLOAT64_C(    22.82), SIMDE_FLOAT64_C(    10.44) } },
    { { SIMDE_FLOAT64_C(   417.19), SIMDE_FLOAT64_C(   212.16) },
      { SIMDE_FLOAT64_C(    20.43), SIMDE_FLOAT64_C(    14.57) } },
    { { SIMDE_FLOAT64_C(    40.41), SIMDE_FLOAT64_C(   807.43) },
      { SIMDE_FLOAT64_C(     6.36), SIMDE_FLOAT64_C(    28.42) } },
    { { SIMDE_FLOAT64_C(   746.18), SIMDE_FLOAT64_C(   239.87) },
      { SIMDE_FLOAT64_C(    27.32), SIMDE_FLOAT64_C(    15.49) } },
    { { SIMDE_FLOAT64_C(   461.80), SIMDE_FLOAT64_C(   420.17) },
      { SIMDE_FLOAT64_C(    21.49), SIMDE_FLOAT64_C(    20.50) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_svml_sqrt_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_svml_sqrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   935.36), SIMDE_FLOAT32_C(   463.00), SIMDE_FLOAT32_C(   356.54), SIMDE_FLOAT32_C(   614.58),
        SIMDE_FLOAT32_C(   720.00), SIMDE_FLOAT32_C(   747.09), SIMDE_FLOAT32_C(   873.09), SIMDE_FLOAT32_C(   461.84) },
      { SIMDE_FLOAT32_C(    30.58), SIMDE_FLOAT32_C(    21.52), SIMDE_FLOAT32_C(    18.88), SIMDE_FLOAT32_C(    24.79),
        SIMDE_FLOAT32_C(    26.83), SIMDE_FLOAT32_C(    27.33), SIMDE_FLOAT32_C(    29.55), SIMDE_FLOAT32_C(    21.49) } },
    { { SIMDE_FLOAT32_C(   718.30), SIMDE_FLOAT32_C(   297.75), SIMDE_FLOAT32_C(    46.73), SIMDE_FLOAT32_C(   -42.51),
        SIMDE_FLOAT32_C(   207.50), SIMDE_FLOAT32_C(   492.51), SIMDE_FLOAT32_C(    15.08), SIMDE_FLOAT32_C(   719.29) },
      { SIMDE_FLOAT32_C(    26.80), SIMDE_FLOAT32_C(    17.26), SIMDE_FLOAT32_C(     6.84),            SIMDE_MATH_NANF,
        SIMDE_FLOAT32_C(    14.40), SIMDE_FLOAT32_C(    22.19), SIMDE_FLOAT32_C(     3.88), SIMDE_FLOAT32_C(    26.82) } },
    { { SIMDE_FLOAT32_C(   347.10), SIMDE_FLOAT32_C(   575.60), SIMDE_FLOAT32_C(   719.84), SIMDE_FLOAT32_C(   241.71),
        SIMDE_FLOAT32_C(   139.48), SIMDE_FLOAT32_C(   757.17), SIMDE_FLOAT32_C(   132.17), SIMDE_FLOAT32_C(   152.46) },
      { SIMDE_FLOAT32_C(    18.63), SIMDE_FLOAT32_C(    23.99), SIMDE_FLOAT32_C(    26.83), SIMDE_FLOAT32_C(    15.55),
        SIMDE_FLOAT32_C(    11.81), SIMDE_FLOAT32_C(    27.52), SIMDE_FLOAT32_C(    11.50), SIMDE_FLOAT32_C(    12.35) } },
    { { SIMDE_FLOAT32_C(   780.23), SIMDE_FLOAT32_C(   823.65), SIMDE_FLOAT32_C(   290.06), SIMDE_FLOAT32_C(   492.64),
        SIMDE_FLOAT32_C(   944.24), SIMDE_FLOAT32_C(   836.21), SIMDE_FLOAT32_C(   785.55), SIMDE_FLOAT32_C(   879.60) },
      { SIMDE_FLOAT32_C(    27.93), SIMDE_FLOAT32_C(    28.70), SIMDE_FLOAT32_C(    17.03), SIMDE_FLOAT32_C(    22.20),
        SIMDE_FLOAT32_C(    30.73), SIMDE_FLOAT32_C(    28.92), SIMDE_FLOAT32_C(    28.03), SIMDE_FLOAT32_C(    29.66) } },
    { { SIMDE_FLOAT32_C(   299.21), SIMDE_FLOAT32_C(   142.09), SIMDE_FLOAT32_C(   494.18), SIMDE_FLOAT32_C(    19.21),
        SIMDE_FLOAT32_C(   989.19), SIMDE_FLOAT32_C(   367.28), SIMDE_FLOAT32_C(   581.05), SIMDE_FLOAT32_C(   707.48) },
      { SIMDE_FLOAT32_C(    17.30), SIMDE_FLOAT32_C(    11.92), SIMDE_FLOAT32_C(    22.23), SIMDE_FLOAT32_C(     4.38),
        SIMDE_FLOAT32_C(    31.45), SIMDE_FLOAT32_C(    19.16), SIMDE_FLOAT32_C(    24.10), SIMDE_FLOAT32_C(    26.60) } },
    { { SIMDE_FLOAT32_C(   765.03), SIMDE_FLOAT32_C(   727.79), SIMDE_FLOAT32_C(   764.97), SIMDE_FLOAT32_C(   -27.47),
        SIMDE_FLOAT32_C(   220.30), SIMDE_FLOAT32_C(   880.05), SIMDE_FLOAT32_C(   791.82), SIMDE_FLOAT32_C(   667.40) },
      { SIMDE_FLOAT32_C(    27.66), SIMDE_FLOAT32_C(    26.98), SIMDE_FLOAT32_C(    27.66),            SIMDE_MATH_NANF,
        SIMDE_FLOAT32_C(    14.84), SIMDE_FLOAT32_C(    29.67), SIMDE_FLOAT32_C(    28.14), SIMDE_FLOAT32_C(    25.83) } },
    { { SIMDE_FLOAT32_C(   455.65), SIMDE_FLOAT32_C(   511.66), SIMDE_FLOAT32_C(   -90.90), SIMDE_FLOAT32_C(   695.13),
        SIMDE_FLOAT32_C(   268.83), SIMDE_FLOAT32_C(   141.28), SIMDE_FLOAT32_C(   947.59), SIMDE_FLOAT32_C(    49.06) },
      { SIMDE_FLOAT32_C(    21.35), SIMDE_FLOAT32_C(    22.62),            SIMDE_MATH_NANF, SIMDE_FLOAT32_C(    26.37),
        SIMDE_FLOAT32_C(    16.40), SIMDE_FLOAT32_C(    11.89), SIMDE_FLOAT32_C(    30.78), SIMDE_FLOAT32_C(     7.00) } },
    { { SIMDE_FLOAT32_C(   -35.07), SIMDE_FLOAT32_C(   237.65), SIMDE_FLOAT32_C(   641.70), SIMDE_FLOAT32_C(   -90.83),
        SIMDE_FLOAT32_C(    73.86), SIMDE_FLOAT32_C(   427.26), SIMDE_FLOAT32_C(   888.77), SIMDE_FLOAT32_C(   473.07) },
      {            SIMDE_MATH_NANF, SIMDE_FLOAT32_C(    15.42), SIMDE_FLOAT32_C(    25.33),            SIMDE_MATH_NANF,
        SIMDE_FLOAT32_C(     8.59), SIMDE_FLOAT32_C(    20.67), SIMDE_FLOAT32_C(    29.81), SIMDE_FLOAT32_C(    21.75) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_svml_sqrt_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_svml_sqrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   898.02), SIMDE_FLOAT64_C(    77.23), SIMDE_FLOAT64_C(   690.30), SIMDE_FLOAT64_C(   742.27) },
      { SIMDE_FLOAT64_C(    29.97), SIMDE_FLOAT64_C(     8.79), SIMDE_FLOAT64_C(    26.27), SIMDE_FLOAT64_C(    27.24) } },
    { { SIMDE_FLOAT64_C(   301.75), SIMDE_FLOAT64_C(   377.86), SIMDE_FLOAT64_C(    38.07), SIMDE_FLOAT64_C(   270.72) },
      { SIMDE_FLOAT64_C(    17.37), SIMDE_FLOAT64_C(    19.44), SIMDE_FLOAT64_C(     6.17), SIMDE_FLOAT64_C(    16.45) } },
    { { SIMDE_FLOAT64_C(   661.06), SIMDE_FLOAT64_C(   955.80), SIMDE_FLOAT64_C(   540.55), SIMDE_FLOAT64_C(   699.66) },
      { SIMDE_FLOAT64_C(    25.71), SIMDE_FLOAT64_C(    30.92), SIMDE_FLOAT64_C(    23.25), SIMDE_FLOAT64_C(    26.45) } },
    { { SIMDE_FLOAT64_C(    41.79), SIMDE_FLOAT64_C(   429.36), SIMDE_FLOAT64_C(   830.75), SIMDE_FLOAT64_C(   836.32) },
      { SIMDE_FLOAT64_C(     6.46), SIMDE_FLOAT64_C(    20.72), SIMDE_FLOAT64_C(    28.82), SIMDE_FLOAT64_C(    28.92) } },
    { { SIMDE_FLOAT64_C(   153.46), SIMDE_FLOAT64_C(   994.23), SIMDE_FLOAT64_C(   913.53), SIMDE_FLOAT64_C(   889.00) },
      { SIMDE_FLOAT64_C(    12.39), SIMDE_FLOAT64_C(    31.53), SIMDE_FLOAT64_C(    30.22), SIMDE_FLOAT64_C(    29.82) } },
    { { SIMDE_FLOAT64_C(   140.95), SIMDE_FLOAT64_C(    65.36), SIMDE_FLOAT64_C(   968.68), SIMDE_FLOAT64_C(   947.21) },
      { SIMDE_FLOAT64_C(    11.87), SIMDE_FLOAT64_C(     8.08), SIMDE_FLOAT64_C(    31.12), SIMDE_FLOAT64_C(    30.78) } },
    { { SIMDE_FLOAT64_C(   -31.19), SIMDE_FLOAT64_C(   466.94), SIMDE_FLOAT64_C(   225.29), SIMDE_FLOAT64_C(   967.56) },
      {             SIMDE_MATH_NAN, SIMDE_FLOAT64_C(    21.61), SIMDE_FLOAT64_C(    15.01), SIMDE_FLOAT64_C(    31.11) } },
    { { SIMDE_FLOAT64_C(   710.29), SIMDE_FLOAT64_C(   718.44), SIMDE_FLOAT64_C(   305.66), SIMDE_FLOAT64_C(   608.32) },
      { SIMDE_FLOAT64_C(    26.65), SIMDE_FLOAT64_C(    26.80), SIMDE_FLOAT64_C(    17.48), SIMDE_FLOAT64_C(    24.66) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_svml_sqrt_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_svml_sqrt_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
      { { SIMDE_FLOAT32_C(   914.68), SIMDE_FLOAT32_C(   142.36), SIMDE_FLOAT32_C(   751.77), SIMDE_FLOAT32_C(    42.61),
        SIMDE_FLOAT32_C(   433.18), SIMDE_FLOAT32_C(   -95.01), SIMDE_FLOAT32_C(   535.55), SIMDE_FLOAT32_C(   168.98),
        SIMDE_FLOAT32_C(   508.03), SIMDE_FLOAT32_C(   713.68), SIMDE_FLOAT32_C(   502.38), SIMDE_FLOAT32_C(   504.11),
        SIMDE_FLOAT32_C(   643.10), SIMDE_FLOAT32_C(   546.21), SIMDE_FLOAT32_C(   975.24), SIMDE_FLOAT32_C(   770.62) },
      { SIMDE_FLOAT32_C(    30.24), SIMDE_FLOAT32_C(    11.93), SIMDE_FLOAT32_C(    27.42), SIMDE_FLOAT32_C(     6.53),
        SIMDE_FLOAT32_C(    20.81),            SIMDE_MATH_NANF, SIMDE_FLOAT32_C(    23.14), SIMDE_FLOAT32_C(    13.00),
        SIMDE_FLOAT32_C(    22.54), SIMDE_FLOAT32_C(    26.71), SIMDE_FLOAT32_C(    22.41), SIMDE_FLOAT32_C(    22.45),
        SIMDE_FLOAT32_C(    25.36), SIMDE_FLOAT32_C(    23.37), SIMDE_FLOAT32_C(    31.23), SIMDE_FLOAT32_C(    27.76) } },
    { { SIMDE_FLOAT32_C(   799.15), SIMDE_FLOAT32_C(   249.41), SIMDE_FLOAT32_C(   246.93), SIMDE_FLOAT32_C(   -33.60),
        SIMDE_FLOAT32_C(   336.37), SIMDE_FLOAT32_C(   867.92), SIMDE_FLOAT32_C(    50.92), SIMDE_FLOAT32_C(   348.52),
        SIMDE_FLOAT32_C(   870.30), SIMDE_FLOAT32_C(   193.09), SIMDE_FLOAT32_C(   153.59), SIMDE_FLOAT32_C(   803.32),
        SIMDE_FLOAT32_C(   802.44), SIMDE_FLOAT32_C(   360.38), SIMDE_FLOAT32_C(   481.46), SIMDE_FLOAT32_C(   717.12) },
      { SIMDE_FLOAT32_C(    28.27), SIMDE_FLOAT32_C(    15.79), SIMDE_FLOAT32_C(    15.71),            SIMDE_MATH_NANF,
        SIMDE_FLOAT32_C(    18.34), SIMDE_FLOAT32_C(    29.46), SIMDE_FLOAT32_C(     7.14), SIMDE_FLOAT32_C(    18.67),
        SIMDE_FLOAT32_C(    29.50), SIMDE_FLOAT32_C(    13.90), SIMDE_FLOAT32_C(    12.39), SIMDE_FLOAT32_C(    28.34),
        SIMDE_FLOAT32_C(    28.33), SIMDE_FLOAT32_C(    18.98), SIMDE_FLOAT32_C(    21.94), SIMDE_FLOAT32_C(    26.78) } },
    { { SIMDE_FLOAT32_C(   602.74), SIMDE_FLOAT32_C(   233.23), SIMDE_FLOAT32_C(   859.73), SIMDE_FLOAT32_C(    35.92),
        SIMDE_FLOAT32_C(   238.22), SIMDE_FLOAT32_C(   395.29), SIMDE_FLOAT32_C(   304.89), SIMDE_FLOAT32_C(   846.24),
        SIMDE_FLOAT32_C(   108.97), SIMDE_FLOAT32_C(   907.27), SIMDE_FLOAT32_C(   350.35), SIMDE_FLOAT32_C(   852.07),
        SIMDE_FLOAT32_C(   453.48), SIMDE_FLOAT32_C(   325.59), SIMDE_FLOAT32_C(   622.69), SIMDE_FLOAT32_C(   252.63) },
      { SIMDE_FLOAT32_C(    24.55), SIMDE_FLOAT32_C(    15.27), SIMDE_FLOAT32_C(    29.32), SIMDE_FLOAT32_C(     5.99),
        SIMDE_FLOAT32_C(    15.43), SIMDE_FLOAT32_C(    19.88), SIMDE_FLOAT32_C(    17.46), SIMDE_FLOAT32_C(    29.09),
        SIMDE_FLOAT32_C(    10.44), SIMDE_FLOAT32_C(    30.12), SIMDE_FLOAT32_C(    18.72), SIMDE_FLOAT32_C(    29.19),
        SIMDE_FLOAT32_C(    21.30), SIMDE_FLOAT32_C(    18.04), SIMDE_FLOAT32_C(    24.95), SIMDE_FLOAT32_C(    15.89) } },
    { { SIMDE_FLOAT32_C(   675.00), SIMDE_FLOAT32_C(   969.62), SIMDE_FLOAT32_C(   319.04), SIMDE_FLOAT32_C(    11.37),
        SIMDE_FLOAT32_C(   837.54), SIMDE_FLOAT32_C(   469.95), SIMDE_FLOAT32_C(   459.89), SIMDE_FLOAT32_C(   707.84),
        SIMDE_FLOAT32_C(   763.05), SIMDE_FLOAT32_C(   713.48), SIMDE_FLOAT32_C(   511.15), SIMDE_FLOAT32_C(   565.49),
        SIMDE_FLOAT32_C(    73.86), SIMDE_FLOAT32_C(    -7.39), SIMDE_FLOAT32_C(   282.61), SIMDE_FLOAT32_C(   776.60) },
      { SIMDE_FLOAT32_C(    25.98), SIMDE_FLOAT32_C(    31.14), SIMDE_FLOAT32_C(    17.86), SIMDE_FLOAT32_C(     3.37),
        SIMDE_FLOAT32_C(    28.94), SIMDE_FLOAT32_C(    21.68), SIMDE_FLOAT32_C(    21.44), SIMDE_FLOAT32_C(    26.61),
        SIMDE_FLOAT32_C(    27.62), SIMDE_FLOAT32_C(    26.71), SIMDE_FLOAT32_C(    22.61), SIMDE_FLOAT32_C(    23.78),
        SIMDE_FLOAT32_C(     8.59),            SIMDE_MATH_NANF, SIMDE_FLOAT32_C(    16.81), SIMDE_FLOAT32_C(    27.87) } },
    { { SIMDE_FLOAT32_C(   325.84), SIMDE_FLOAT32_C(   142.35), SIMDE_FLOAT32_C(   912.52), SIMDE_FLOAT32_C(   664.06),
        SIMDE_FLOAT32_C(   637.63), SIMDE_FLOAT32_C(   217.41), SIMDE_FLOAT32_C(   510.30), SIMDE_FLOAT32_C(   846.60),
        SIMDE_FLOAT32_C(   124.68), SIMDE_FLOAT32_C(   960.65), SIMDE_FLOAT32_C(   698.67), SIMDE_FLOAT32_C(   678.16),
        SIMDE_FLOAT32_C(   286.24), SIMDE_FLOAT32_C(   321.36), SIMDE_FLOAT32_C(   -69.20), SIMDE_FLOAT32_C(   -38.77) },
      { SIMDE_FLOAT32_C(    18.05), SIMDE_FLOAT32_C(    11.93), SIMDE_FLOAT32_C(    30.21), SIMDE_FLOAT32_C(    25.77),
        SIMDE_FLOAT32_C(    25.25), SIMDE_FLOAT32_C(    14.74), SIMDE_FLOAT32_C(    22.59), SIMDE_FLOAT32_C(    29.10),
        SIMDE_FLOAT32_C(    11.17), SIMDE_FLOAT32_C(    30.99), SIMDE_FLOAT32_C(    26.43), SIMDE_FLOAT32_C(    26.04),
        SIMDE_FLOAT32_C(    16.92), SIMDE_FLOAT32_C(    17.93),            SIMDE_MATH_NANF,            SIMDE_MATH_NANF } },
    { { SIMDE_FLOAT32_C(   290.98), SIMDE_FLOAT32_C(   349.84), SIMDE_FLOAT32_C(    72.60), SIMDE_FLOAT32_C(   128.51),
        SIMDE_FLOAT32_C(   919.79), SIMDE_FLOAT32_C(   632.49), SIMDE_FLOAT32_C(   936.35), SIMDE_FLOAT32_C(   682.84),
        SIMDE_FLOAT32_C(   345.97), SIMDE_FLOAT32_C(   447.51), SIMDE_FLOAT32_C(   248.33), SIMDE_FLOAT32_C(   519.83),
        SIMDE_FLOAT32_C(   540.12), SIMDE_FLOAT32_C(   630.94), SIMDE_FLOAT32_C(   296.43), SIMDE_FLOAT32_C(   965.96) },
      { SIMDE_FLOAT32_C(    17.06), SIMDE_FLOAT32_C(    18.70), SIMDE_FLOAT32_C(     8.52), SIMDE_FLOAT32_C(    11.34),
        SIMDE_FLOAT32_C(    30.33), SIMDE_FLOAT32_C(    25.15), SIMDE_FLOAT32_C(    30.60), SIMDE_FLOAT32_C(    26.13),
        SIMDE_FLOAT32_C(    18.60), SIMDE_FLOAT32_C(    21.15), SIMDE_FLOAT32_C(    15.76), SIMDE_FLOAT32_C(    22.80),
        SIMDE_FLOAT32_C(    23.24), SIMDE_FLOAT32_C(    25.12), SIMDE_FLOAT32_C(    17.22), SIMDE_FLOAT32_C(    31.08) } },
    { { SIMDE_FLOAT32_C(   873.29), SIMDE_FLOAT32_C(   208.95), SIMDE_FLOAT32_C(   630.01), SIMDE_FLOAT32_C(   510.92),
        SIMDE_FLOAT32_C(   526.36), SIMDE_FLOAT32_C(   140.32), SIMDE_FLOAT32_C(   357.53), SIMDE_FLOAT32_C(   751.05),
        SIMDE_FLOAT32_C(   100.97), SIMDE_FLOAT32_C(    56.20), SIMDE_FLOAT32_C(   429.21), SIMDE_FLOAT32_C(   487.20),
        SIMDE_FLOAT32_C(   477.55), SIMDE_FLOAT32_C(   460.01), SIMDE_FLOAT32_C(   548.44), SIMDE_FLOAT32_C(   868.53) },
      { SIMDE_FLOAT32_C(    29.55), SIMDE_FLOAT32_C(    14.46), SIMDE_FLOAT32_C(    25.10), SIMDE_FLOAT32_C(    22.60),
        SIMDE_FLOAT32_C(    22.94), SIMDE_FLOAT32_C(    11.85), SIMDE_FLOAT32_C(    18.91), SIMDE_FLOAT32_C(    27.41),
        SIMDE_FLOAT32_C(    10.05), SIMDE_FLOAT32_C(     7.50), SIMDE_FLOAT32_C(    20.72), SIMDE_FLOAT32_C(    22.07),
        SIMDE_FLOAT32_C(    21.85), SIMDE_FLOAT32_C(    21.45), SIMDE_FLOAT32_C(    23.42), SIMDE_FLOAT32_C(    29.47) } },
    { { SIMDE_FLOAT32_C(   909.84), SIMDE_FLOAT32_C(   721.04), SIMDE_FLOAT32_C(    -2.95), SIMDE_FLOAT32_C(   829.64),
        SIMDE_FLOAT32_C(   353.53), SIMDE_FLOAT32_C(   -66.60), SIMDE_FLOAT32_C(   512.48), SIMDE_FLOAT32_C(   799.49),
        SIMDE_FLOAT32_C(   480.91), SIMDE_FLOAT32_C(   860.80), SIMDE_FLOAT32_C(   319.32), SIMDE_FLOAT32_C(    21.02),
        SIMDE_FLOAT32_C(   491.75), SIMDE_FLOAT32_C(   715.75), SIMDE_FLOAT32_C(   -13.02), SIMDE_FLOAT32_C(   365.04) },
      { SIMDE_FLOAT32_C(    30.16), SIMDE_FLOAT32_C(    26.85),            SIMDE_MATH_NANF, SIMDE_FLOAT32_C(    28.80),
        SIMDE_FLOAT32_C(    18.80),            SIMDE_MATH_NANF, SIMDE_FLOAT32_C(    22.64), SIMDE_FLOAT32_C(    28.28),
        SIMDE_FLOAT32_C(    21.93), SIMDE_FLOAT32_C(    29.34), SIMDE_FLOAT32_C(    17.87), SIMDE_FLOAT32_C(     4.59),
        SIMDE_FLOAT32_C(    22.18), SIMDE_FLOAT32_C(    26.75),            SIMDE_MATH_NANF, SIMDE_FLOAT32_C(    19.11) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_svml_sqrt_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_svml_sqrt_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   375.58), SIMDE_FLOAT64_C(    46.46), SIMDE_FLOAT64_C(   312.83), SIMDE_FLOAT64_C(   445.14),
        SIMDE_FLOAT64_C(   187.32), SIMDE_FLOAT64_C(   952.90), SIMDE_FLOAT64_C(   987.69), SIMDE_FLOAT64_C(   406.24) },
      { SIMDE_FLOAT64_C(    19.38), SIMDE_FLOAT64_C(     6.82), SIMDE_FLOAT64_C(    17.69), SIMDE_FLOAT64_C(    21.10),
        SIMDE_FLOAT64_C(    13.69), SIMDE_FLOAT64_C(    30.87), SIMDE_FLOAT64_C(    31.43), SIMDE_FLOAT64_C(    20.16) } },
    { { SIMDE_FLOAT64_C(   293.47), SIMDE_FLOAT64_C(   304.52), SIMDE_FLOAT64_C(   836.60), SIMDE_FLOAT64_C(   342.20),
        SIMDE_FLOAT64_C(   740.40), SIMDE_FLOAT64_C(   328.94), SIMDE_FLOAT64_C(   360.36), SIMDE_FLOAT64_C(    97.23) },
      { SIMDE_FLOAT64_C(    17.13), SIMDE_FLOAT64_C(    17.45), SIMDE_FLOAT64_C(    28.92), SIMDE_FLOAT64_C(    18.50),
        SIMDE_FLOAT64_C(    27.21), SIMDE_FLOAT64_C(    18.14), SIMDE_FLOAT64_C(    18.98), SIMDE_FLOAT64_C(     9.86) } },
    { { SIMDE_FLOAT64_C(   931.22), SIMDE_FLOAT64_C(   239.31), SIMDE_FLOAT64_C(   533.01), SIMDE_FLOAT64_C(   413.09),
        SIMDE_FLOAT64_C(   -30.52), SIMDE_FLOAT64_C(   220.33), SIMDE_FLOAT64_C(   224.40), SIMDE_FLOAT64_C(   591.21) },
      { SIMDE_FLOAT64_C(    30.52), SIMDE_FLOAT64_C(    15.47), SIMDE_FLOAT64_C(    23.09), SIMDE_FLOAT64_C(    20.32),
                    SIMDE_MATH_NAN, SIMDE_FLOAT64_C(    14.84), SIMDE_FLOAT64_C(    14.98), SIMDE_FLOAT64_C(    24.31) } },
    { { SIMDE_FLOAT64_C(   737.21), SIMDE_FLOAT64_C(   927.12), SIMDE_FLOAT64_C(   685.90), SIMDE_FLOAT64_C(   452.75),
        SIMDE_FLOAT64_C(   896.77), SIMDE_FLOAT64_C(   752.44), SIMDE_FLOAT64_C(   780.06), SIMDE_FLOAT64_C(   272.35) },
      { SIMDE_FLOAT64_C(    27.15), SIMDE_FLOAT64_C(    30.45), SIMDE_FLOAT64_C(    26.19), SIMDE_FLOAT64_C(    21.28),
        SIMDE_FLOAT64_C(    29.95), SIMDE_FLOAT64_C(    27.43), SIMDE_FLOAT64_C(    27.93), SIMDE_FLOAT64_C(    16.50) } },
    { { SIMDE_FLOAT64_C(   898.90), SIMDE_FLOAT64_C(    92.89), SIMDE_FLOAT64_C(   817.49), SIMDE_FLOAT64_C(    86.22),
        SIMDE_FLOAT64_C(    45.79), SIMDE_FLOAT64_C(   805.18), SIMDE_FLOAT64_C(   592.46), SIMDE_FLOAT64_C(   439.26) },
      { SIMDE_FLOAT64_C(    29.98), SIMDE_FLOAT64_C(     9.64), SIMDE_FLOAT64_C(    28.59), SIMDE_FLOAT64_C(     9.29),
        SIMDE_FLOAT64_C(     6.77), SIMDE_FLOAT64_C(    28.38), SIMDE_FLOAT64_C(    24.34), SIMDE_FLOAT64_C(    20.96) } },
    { { SIMDE_FLOAT64_C(   109.70), SIMDE_FLOAT64_C(   429.07), SIMDE_FLOAT64_C(   881.46), SIMDE_FLOAT64_C(   950.09),
        SIMDE_FLOAT64_C(   858.01), SIMDE_FLOAT64_C(   241.82), SIMDE_FLOAT64_C(    47.32), SIMDE_FLOAT64_C(   789.23) },
      { SIMDE_FLOAT64_C(    10.47), SIMDE_FLOAT64_C(    20.71), SIMDE_FLOAT64_C(    29.69), SIMDE_FLOAT64_C(    30.82),
        SIMDE_FLOAT64_C(    29.29), SIMDE_FLOAT64_C(    15.55), SIMDE_FLOAT64_C(     6.88), SIMDE_FLOAT64_C(    28.09) } },
    { { SIMDE_FLOAT64_C(   581.13), SIMDE_FLOAT64_C(   680.33), SIMDE_FLOAT64_C(   202.32), SIMDE_FLOAT64_C(   650.61),
        SIMDE_FLOAT64_C(   -99.34), SIMDE_FLOAT64_C(   526.72), SIMDE_FLOAT64_C(   241.82), SIMDE_FLOAT64_C(   737.87) },
      { SIMDE_FLOAT64_C(    24.11), SIMDE_FLOAT64_C(    26.08), SIMDE_FLOAT64_C(    14.22), SIMDE_FLOAT64_C(    25.51),
                    SIMDE_MATH_NAN, SIMDE_FLOAT64_C(    22.95), SIMDE_FLOAT64_C(    15.55), SIMDE_FLOAT64_C(    27.16) } },
    { { SIMDE_FLOAT64_C(   453.84), SIMDE_FLOAT64_C(   -72.28), SIMDE_FLOAT64_C(   190.62), SIMDE_FLOAT64_C(   350.61),
        SIMDE_FLOAT64_C(   780.16), SIMDE_FLOAT64_C(   -29.31), SIMDE_FLOAT64_C(   722.96), SIMDE_FLOAT64_C(   679.07) },
      { SIMDE_FLOAT64_C(    21.30),             SIMDE_MATH_NAN, SIMDE_FLOAT64_C(    13.81), SIMDE_FLOAT64_C(    18.72),
        SIMDE_FLOAT64_C(    27.93),             SIMDE_MATH_NAN, SIMDE_FLOAT64_C(    26.89), SIMDE_FLOAT64_C(    26.06) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_svml_sqrt_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_tan_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -1.15), SIMDE_FLOAT32_C(     3.76), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     1.76)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     1.87), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -0.54)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.35)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -3.14), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.32)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -0.63)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -0.01)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -1.81)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(    -6.68), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(    -1.31)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_tan_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_tan_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    1.76)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -1.15), SIMDE_FLOAT64_C(    3.76)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.56), SIMDE_FLOAT64_C(   -0.54)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.36), SIMDE_FLOAT64_C(    1.87)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.89), SIMDE_FLOAT64_C(   -0.35)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.86), SIMDE_FLOAT64_C(    2.88)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.05), SIMDE_FLOAT64_C(   -0.32)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.44), SIMDE_FLOAT64_C(   -3.14)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_tan_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_tan_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24),
                         SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01),
                         SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     1.87),
                         SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -0.54),
                         SIMDE_FLOAT32_C(    -1.15), SIMDE_FLOAT32_C(     3.76),
                         SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     1.76)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13),
                         SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21),
                         SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -3.14),
                         SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.32),
                         SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     2.88),
                         SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.35)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95),
                         SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     0.20),
                         SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -0.01),
                         SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.15),
                         SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -0.63)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67),
                         SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54),
                         SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(    -6.68),
                         SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(    -1.31),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     1.91),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -1.81)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48),
                         SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(    -1.66), SIMDE_FLOAT32_C(     2.28),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.91)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92),
                         SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   -11.51), SIMDE_FLOAT32_C(     0.08),
                         SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(     6.62), SIMDE_FLOAT32_C(    -0.10),
                         SIMDE_FLOAT32_C(    21.84), SIMDE_FLOAT32_C(     0.72)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73),
                         SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     1.40),
                         SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     1.11),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.81),
                         SIMDE_FLOAT32_C(    -3.22), SIMDE_FLOAT32_C(    -0.16)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02),
                         SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   123.48), SIMDE_FLOAT32_C(     7.37),
                         SIMDE_FLOAT32_C(    -1.68), SIMDE_FLOAT32_C(    -3.54),
                         SIMDE_FLOAT32_C(    -1.67), SIMDE_FLOAT32_C(     0.45),
                         SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -0.16)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_tan_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_tan_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.15), SIMDE_FLOAT64_C(    3.76),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    1.76)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.36), SIMDE_FLOAT64_C(    1.87),
                         SIMDE_FLOAT64_C(    1.56), SIMDE_FLOAT64_C(   -0.54)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.86), SIMDE_FLOAT64_C(    2.88),
                         SIMDE_FLOAT64_C(    0.89), SIMDE_FLOAT64_C(   -0.35)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.44), SIMDE_FLOAT64_C(   -3.14),
                         SIMDE_FLOAT64_C(   -0.05), SIMDE_FLOAT64_C(   -0.32)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(    0.30), SIMDE_FLOAT64_C(   -0.63)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    3.55), SIMDE_FLOAT64_C(    0.20),
                         SIMDE_FLOAT64_C(   -0.94), SIMDE_FLOAT64_C(   -0.01)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.62), SIMDE_FLOAT64_C(    1.91),
                         SIMDE_FLOAT64_C(    0.53), SIMDE_FLOAT64_C(   -1.81)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -1.19), SIMDE_FLOAT64_C(   -6.68),
                         SIMDE_FLOAT64_C(    1.24), SIMDE_FLOAT64_C(   -1.31)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_tan_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_tan_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -3.14), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.32),
                         SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     2.88), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(    -0.35),
                         SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(     1.87), SIMDE_FLOAT32_C(     1.56), SIMDE_FLOAT32_C(    -0.54),
                         SIMDE_FLOAT32_C(    -1.15), SIMDE_FLOAT32_C(     3.76), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     1.76)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(    -6.68), SIMDE_FLOAT32_C(     1.24), SIMDE_FLOAT32_C(    -1.31),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     1.91), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -1.81),
                         SIMDE_FLOAT32_C(     3.55), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -0.01),
                         SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.30), SIMDE_FLOAT32_C(    -0.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -11.51), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(    -0.88), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(     6.62), SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(    21.84), SIMDE_FLOAT32_C(     0.72),
                         SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(    -1.66), SIMDE_FLOAT32_C(     2.28), SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.91)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   123.48), SIMDE_FLOAT32_C(     7.37), SIMDE_FLOAT32_C(    -1.68), SIMDE_FLOAT32_C(    -3.54),
                         SIMDE_FLOAT32_C(    -1.67), SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.48), SIMDE_FLOAT32_C(    -0.16),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     1.40), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(     1.11),
                         SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(    -0.81), SIMDE_FLOAT32_C(    -3.22), SIMDE_FLOAT32_C(    -0.16)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(   343.48),
                         SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   655.67),
                         SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(   120.65), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(   -10.46), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(     1.73),
                         SIMDE_FLOAT32_C(    -1.39), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     4.02), SIMDE_FLOAT32_C(    -2.46),
                         SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -1.51), SIMDE_FLOAT32_C(    -1.32),
                         SIMDE_FLOAT32_C(     4.39), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     3.22), SIMDE_FLOAT32_C(     3.31)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -591.56),
                         SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(  -906.16),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -738.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.67), SIMDE_FLOAT32_C(    -3.56), SIMDE_FLOAT32_C(     1.26), SIMDE_FLOAT32_C(    -1.37),
                         SIMDE_FLOAT32_C(     0.37), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(   -10.62),
                         SIMDE_FLOAT32_C(   -14.52), SIMDE_FLOAT32_C(    -0.85), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -5.21),
                         SIMDE_FLOAT32_C(    10.17), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     2.15), SIMDE_FLOAT32_C(     0.08)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(   977.49),
                         SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -1.69), SIMDE_FLOAT32_C(     1.97), SIMDE_FLOAT32_C(     5.68),
                         SIMDE_FLOAT32_C(    -8.21), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    10.08), SIMDE_FLOAT32_C(  1691.15),
                         SIMDE_FLOAT32_C(    -3.72), SIMDE_FLOAT32_C(    10.41), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.49),
                         SIMDE_FLOAT32_C(     0.85), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -1.64), SIMDE_FLOAT32_C(    -1.27)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -696.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -16.06), SIMDE_FLOAT32_C(    20.97), SIMDE_FLOAT32_C(    53.90), SIMDE_FLOAT32_C(     1.23),
                         SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(    -0.54), SIMDE_FLOAT32_C(     0.50),
                         SIMDE_FLOAT32_C(    -0.24), SIMDE_FLOAT32_C(    -1.29), SIMDE_FLOAT32_C(    -2.82), SIMDE_FLOAT32_C(    -2.36),
                         SIMDE_FLOAT32_C(    -4.86), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -1.89), SIMDE_FLOAT32_C(     0.92)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_tan_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_tan_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.19), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(     0.30),
                         SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.89),
                         SIMDE_FLOAT32_C(     1.36), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    -1.15), SIMDE_FLOAT32_C(   346.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19),
                         SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   254.31), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(   398.82), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   -30.79), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     3.31), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(     1.40), SIMDE_FLOAT32_C(     1.11), SIMDE_FLOAT32_C(    -0.81),
                         SIMDE_FLOAT32_C(    -0.16), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.10),
                         SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(     2.28)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    99.93),
                         SIMDE_FLOAT32_C(  -738.19), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -448.89),
                         SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(   331.34),
                         SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -874.31),
                         SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   -70.91)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(    -1.67), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    10.17),
                         SIMDE_FLOAT32_C(     2.15), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(     1.69), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -337.60),
                         SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(  -756.42)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(    -1.69), SIMDE_FLOAT32_C(     5.68),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(    10.41), SIMDE_FLOAT32_C(     0.49)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -737.13), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(   177.92),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   888.71), SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   133.52),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(  -775.04), SIMDE_FLOAT32_C(   440.64)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(   915.19), SIMDE_FLOAT32_C(  -718.40), SIMDE_FLOAT32_C(   159.97),
                         SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   110.36),
                         SIMDE_FLOAT32_C(   826.84), SIMDE_FLOAT32_C(   -76.75), SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -378.50),
                         SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(  -942.47), SIMDE_FLOAT32_C(   475.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(     1.51), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -1.38), SIMDE_FLOAT32_C(     0.43),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(   440.64)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(  -807.28), SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(   834.60), SIMDE_FLOAT32_C(   -65.60),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   556.35), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -964.25), SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(  -743.66), SIMDE_FLOAT32_C(  -764.58),
                         SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(  -818.54), SIMDE_FLOAT32_C(   161.06),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -308.52), SIMDE_FLOAT32_C(  -719.57),
                         SIMDE_FLOAT32_C(   334.00), SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(  -916.82), SIMDE_FLOAT32_C(  -490.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(    -1.80), SIMDE_FLOAT32_C(     1.25), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(     4.46), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(     6.40), SIMDE_FLOAT32_C(     1.11),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(     5.52), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   105.79), SIMDE_FLOAT32_C(   590.10),
                         SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(   -84.00), SIMDE_FLOAT32_C(    80.04),
                         SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(  -889.11)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(   144.05), SIMDE_FLOAT32_C(  -427.72),
                         SIMDE_FLOAT32_C(   308.28), SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(  -457.77), SIMDE_FLOAT32_C(   678.24),
                         SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   117.28), SIMDE_FLOAT32_C(  -576.80),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   -53.92), SIMDE_FLOAT32_C(    91.94)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(     1.27), SIMDE_FLOAT32_C(    -0.36),
                         SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(     1.71), SIMDE_FLOAT32_C(     3.04),
                         SIMDE_FLOAT32_C(    -0.83), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(     1.10)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(  -493.41), SIMDE_FLOAT32_C(   822.72),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(  -816.27),
                         SIMDE_FLOAT32_C(  -209.34), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(  -204.33)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -841.43), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(   824.88), SIMDE_FLOAT32_C(   793.63),
                         SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -350.72),
                         SIMDE_FLOAT32_C(    60.89), SIMDE_FLOAT32_C(   109.81), SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -250.60),
                         SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   -13.07), SIMDE_FLOAT32_C(   852.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(    -4.65), SIMDE_FLOAT32_C(    -2.52),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(     2.17),
                         SIMDE_FLOAT32_C(     2.57), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(   -10.91), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(     2.81)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_tan_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_tan_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.36), SIMDE_FLOAT64_C(    1.87),
                         SIMDE_FLOAT64_C(    1.56), SIMDE_FLOAT64_C(   -0.54),
                         SIMDE_FLOAT64_C(   -1.15), SIMDE_FLOAT64_C(    3.76),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    1.76)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.44), SIMDE_FLOAT64_C(   -3.14),
                         SIMDE_FLOAT64_C(   -0.05), SIMDE_FLOAT64_C(   -0.32),
                         SIMDE_FLOAT64_C(   -0.86), SIMDE_FLOAT64_C(    2.88),
                         SIMDE_FLOAT64_C(    0.89), SIMDE_FLOAT64_C(   -0.35)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    3.55), SIMDE_FLOAT64_C(    0.20),
                         SIMDE_FLOAT64_C(   -0.94), SIMDE_FLOAT64_C(   -0.01),
                         SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(   -0.15),
                         SIMDE_FLOAT64_C(    0.30), SIMDE_FLOAT64_C(   -0.63)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.19), SIMDE_FLOAT64_C(   -6.68),
                         SIMDE_FLOAT64_C(    1.24), SIMDE_FLOAT64_C(   -1.31),
                         SIMDE_FLOAT64_C(    0.62), SIMDE_FLOAT64_C(    1.91),
                         SIMDE_FLOAT64_C(    0.53), SIMDE_FLOAT64_C(   -1.81)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(  993.90),
                         SIMDE_FLOAT64_C(   28.08), SIMDE_FLOAT64_C(  841.21)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.78), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.91), SIMDE_FLOAT64_C(    0.34),
                         SIMDE_FLOAT64_C(   -1.66), SIMDE_FLOAT64_C(    2.28),
                         SIMDE_FLOAT64_C(   -0.20), SIMDE_FLOAT64_C(   -0.91)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C(  -30.79)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  -11.51), SIMDE_FLOAT64_C(    0.08),
                         SIMDE_FLOAT64_C(   -0.88), SIMDE_FLOAT64_C(   -0.08),
                         SIMDE_FLOAT64_C(    6.62), SIMDE_FLOAT64_C(   -0.10),
                         SIMDE_FLOAT64_C(   21.83), SIMDE_FLOAT64_C(    0.72)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  -80.73),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -767.23),
                         SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C(  398.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.62), SIMDE_FLOAT64_C(    1.40),
                         SIMDE_FLOAT64_C(    0.65), SIMDE_FLOAT64_C(    1.11),
                         SIMDE_FLOAT64_C(    0.11), SIMDE_FLOAT64_C(   -0.81),
                         SIMDE_FLOAT64_C(   -3.22), SIMDE_FLOAT64_C(   -0.16)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C(  254.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  123.43), SIMDE_FLOAT64_C(    7.37),
                         SIMDE_FLOAT64_C(   -1.68), SIMDE_FLOAT64_C(   -3.54),
                         SIMDE_FLOAT64_C(   -1.67), SIMDE_FLOAT64_C(    0.45),
                         SIMDE_FLOAT64_C(   -0.48), SIMDE_FLOAT64_C(   -0.16)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_tan_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_tan_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45),
                         SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.44), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(    1.36), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   -1.15), SIMDE_FLOAT64_C(   -0.42)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  841.21), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  687.09), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -660.80), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -923.64), SIMDE_FLOAT64_C( -860.95)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.91), SIMDE_FLOAT64_C(   -6.68),
                         SIMDE_FLOAT64_C(   -1.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(    0.20),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C(   -0.15)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  398.82), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  339.21), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  -30.79), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(  993.90)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C( -387.90),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  532.35),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C( -770.35),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C( -770.72)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -3.22), SIMDE_FLOAT64_C(  -11.51),
                         SIMDE_FLOAT64_C(   -0.88), SIMDE_FLOAT64_C(    6.62),
                         SIMDE_FLOAT64_C(   21.83), SIMDE_FLOAT64_C(   -0.78),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(   -1.66)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(  469.66),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  910.03),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C( -203.65),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -747.59)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C( -171.51),
                         SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(    3.31),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(   -3.54),
                         SIMDE_FLOAT64_C(    0.45), SIMDE_FLOAT64_C(   -0.16),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C(    1.11)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   99.93), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(  343.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(  655.67)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  331.34), SIMDE_FLOAT64_C(  462.95),
                         SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  324.62),
                         SIMDE_FLOAT64_C( -874.31), SIMDE_FLOAT64_C( -328.54),
                         SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  561.36)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   10.17), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(    1.69),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(   -1.51)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   27.25),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  831.02), SIMDE_FLOAT64_C(  977.36)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  977.49), SIMDE_FLOAT64_C(  424.81),
                         SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  840.65),
                         SIMDE_FLOAT64_C( -591.56), SIMDE_FLOAT64_C(  731.49),
                         SIMDE_FLOAT64_C(  623.70), SIMDE_FLOAT64_C(  140.67)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(    0.83),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(   -1.37), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  -10.62), SIMDE_FLOAT64_C(   -0.85)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C( -304.73),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(  822.06),
                         SIMDE_FLOAT64_C( -997.63), SIMDE_FLOAT64_C(  923.64),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(  -67.64)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  510.85), SIMDE_FLOAT64_C(   14.34),
                         SIMDE_FLOAT64_C(  916.26), SIMDE_FLOAT64_C( -769.09),
                         SIMDE_FLOAT64_C( -573.81), SIMDE_FLOAT64_C( -337.60),
                         SIMDE_FLOAT64_C(  293.64), SIMDE_FLOAT64_C( -576.22)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C(   -4.86),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C(    1.97), SIMDE_FLOAT64_C(   -8.21),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(   -3.73)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  475.51), SIMDE_FLOAT64_C(  936.65),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C( -438.19),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(  932.66),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C( -182.45)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(  440.64),
                         SIMDE_FLOAT64_C(  897.27), SIMDE_FLOAT64_C( -197.89),
                         SIMDE_FLOAT64_C( -359.76), SIMDE_FLOAT64_C(  -33.67),
                         SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -125.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    1.35), SIMDE_FLOAT64_C(    1.07),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(    1.23),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(    0.50)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_tan_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_tand_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.24)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -1.18), SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     0.68)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -3.60), SIMDE_FLOAT32_C(     1.90), SIMDE_FLOAT32_C(  -104.17), SIMDE_FLOAT32_C(    -3.12)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    10.92), SIMDE_FLOAT32_C(     0.61)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -0.43)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(   -11.01), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(    -0.44)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -4.18), SIMDE_FLOAT32_C(     1.68)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    85.51), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(    -0.65)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_tand_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_tand_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.24)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.11), SIMDE_FLOAT64_C(    0.81)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    1.93), SIMDE_FLOAT64_C(    0.68)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.92), SIMDE_FLOAT64_C(   -1.18)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm_set_pd(SIMDE_FLOAT64_C( -104.17), SIMDE_FLOAT64_C(   -3.12)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -3.60), SIMDE_FLOAT64_C(    1.90)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   10.92), SIMDE_FLOAT64_C(    0.61)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(    0.67)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_tand_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_tand_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24),
                         SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01),
                         SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -1.18),
                         SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.24)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13),
                         SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21),
                         SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.67),
                         SIMDE_FLOAT32_C(    10.92), SIMDE_FLOAT32_C(     0.61),
                         SIMDE_FLOAT32_C(    -3.60), SIMDE_FLOAT32_C(     1.90),
                         SIMDE_FLOAT32_C(  -104.17), SIMDE_FLOAT32_C(    -3.12)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47),
                         SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95),
                         SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(   -11.01), SIMDE_FLOAT32_C(     0.54),
                         SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(    -0.44),
                         SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.81),
                         SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -0.43)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67),
                         SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54),
                         SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    85.51),
                         SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(    -0.65),
                         SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(    -0.64),
                         SIMDE_FLOAT32_C(    -4.18), SIMDE_FLOAT32_C(     1.68)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48),
                         SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90),
                         SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -1.21), SIMDE_FLOAT32_C(     8.75),
                         SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(     0.37),
                         SIMDE_FLOAT32_C(    -1.22), SIMDE_FLOAT32_C(   -14.67),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -1.65)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92),
                         SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.72),
                         SIMDE_FLOAT32_C(    -2.06), SIMDE_FLOAT32_C(    -0.38),
                         SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(    -9.50),
                         SIMDE_FLOAT32_C(     1.78), SIMDE_FLOAT32_C(    -0.60)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73),
                         SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -6.13),
                         SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(    -1.08),
                         SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.80)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02),
                         SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -2.80), SIMDE_FLOAT32_C(    -0.84),
                         SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -6.57),
                         SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     1.77),
                         SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     3.56)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_tand_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_tand_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.11), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.24)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.92), SIMDE_FLOAT64_C(   -1.18),
                         SIMDE_FLOAT64_C(    1.93), SIMDE_FLOAT64_C(    0.68)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -3.60), SIMDE_FLOAT64_C(    1.90),
                         SIMDE_FLOAT64_C( -104.17), SIMDE_FLOAT64_C(   -3.12)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   10.92), SIMDE_FLOAT64_C(    0.61)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    1.42), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -0.43)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(  -11.01), SIMDE_FLOAT64_C(    0.54),
                         SIMDE_FLOAT64_C(   -0.45), SIMDE_FLOAT64_C(   -0.44)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    6.54), SIMDE_FLOAT64_C(   -0.64),
                         SIMDE_FLOAT64_C(   -4.18), SIMDE_FLOAT64_C(    1.68)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(   85.51),
                         SIMDE_FLOAT64_C(    1.35), SIMDE_FLOAT64_C(   -0.65)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_tand_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_tand_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   571.46),
                         SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(  -269.45), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(    34.06),
                         SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(  -754.38), SIMDE_FLOAT32_C(   346.63)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    10.92), SIMDE_FLOAT32_C(     0.61),
                         SIMDE_FLOAT32_C(    -3.60), SIMDE_FLOAT32_C(     1.90), SIMDE_FLOAT32_C(  -104.17), SIMDE_FLOAT32_C(    -3.12),
                         SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -1.18), SIMDE_FLOAT32_C(     1.93), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.24)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   687.09),
                         SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -976.55), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -923.64),
                         SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(  -417.54), SIMDE_FLOAT32_C(   696.87)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(    85.51), SIMDE_FLOAT32_C(     1.35), SIMDE_FLOAT32_C(    -0.65),
                         SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -4.18), SIMDE_FLOAT32_C(     1.68),
                         SIMDE_FLOAT32_C(   -11.01), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(    -0.44),
                         SIMDE_FLOAT32_C(     1.42), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(    -1.57), SIMDE_FLOAT32_C(    -0.43)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   339.21),
                         SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(  -263.99), SIMDE_FLOAT32_C(   780.64), SIMDE_FLOAT32_C(   -30.79),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(   380.46),
                         SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   993.90), SIMDE_FLOAT32_C(    28.08), SIMDE_FLOAT32_C(   841.21)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(    -2.06), SIMDE_FLOAT32_C(    -0.38),
                         SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(    -9.50), SIMDE_FLOAT32_C(     1.78), SIMDE_FLOAT32_C(    -0.60),
                         SIMDE_FLOAT32_C(    -1.21), SIMDE_FLOAT32_C(     8.75), SIMDE_FLOAT32_C(    -0.95), SIMDE_FLOAT32_C(     0.37),
                         SIMDE_FLOAT32_C(    -1.22), SIMDE_FLOAT32_C(   -14.67), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -1.65)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   818.66),
                         SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   600.47), SIMDE_FLOAT32_C(   791.23), SIMDE_FLOAT32_C(   254.31),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -944.78),
                         SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -767.23), SIMDE_FLOAT32_C(  -554.19), SIMDE_FLOAT32_C(   398.82)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -2.80), SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(    -6.57),
                         SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     1.77), SIMDE_FLOAT32_C(     2.94), SIMDE_FLOAT32_C(     3.56),
                         SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -6.13), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(    -0.99),
                         SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(    -1.08), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.80)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(   343.48),
                         SIMDE_FLOAT32_C(  -874.31), SIMDE_FLOAT32_C(  -797.92), SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -525.83),
                         SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   655.67),
                         SIMDE_FLOAT32_C(   -70.91), SIMDE_FLOAT32_C(   543.35), SIMDE_FLOAT32_C(   120.65), SIMDE_FLOAT32_C(  -171.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(    -0.30),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -4.67), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.25),
                         SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(     4.46), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(    -2.08),
                         SIMDE_FLOAT32_C(    -2.89), SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    -1.69), SIMDE_FLOAT32_C(     0.15)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -591.56),
                         SIMDE_FLOAT32_C(  -448.89), SIMDE_FLOAT32_C(   731.49), SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   623.70),
                         SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(  -906.16),
                         SIMDE_FLOAT32_C(   331.34), SIMDE_FLOAT32_C(    99.93), SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -738.19)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -1.69), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -1.26),
                         SIMDE_FLOAT32_C(   -51.61), SIMDE_FLOAT32_C(     0.20), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     9.06),
                         SIMDE_FLOAT32_C(    -2.60), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     4.46), SIMDE_FLOAT32_C(    -0.11),
                         SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(    -5.71), SIMDE_FLOAT32_C(    -4.35), SIMDE_FLOAT32_C(    -0.33)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(  -337.60), SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -768.12),
                         SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(   977.49),
                         SIMDE_FLOAT32_C(  -756.42), SIMDE_FLOAT32_C(   424.81), SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   -95.15)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -1.15), SIMDE_FLOAT32_C(    -4.68), SIMDE_FLOAT32_C(    -0.67), SIMDE_FLOAT32_C(     7.46),
                         SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -2.28), SIMDE_FLOAT32_C(    -1.12),
                         SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(    -2.43), SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     4.51),
                         SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     2.13), SIMDE_FLOAT32_C(     0.52), SIMDE_FLOAT32_C(    11.10)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(   932.66), SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -125.20),
                         SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(    14.34), SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -696.69)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -4.78), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.67),
                         SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     1.42),
                         SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.56), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     1.44), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(     0.43)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_tand_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_tand_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -450.67), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(  -212.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(   696.87),
                         SIMDE_FLOAT32_C(  -686.13), SIMDE_FLOAT32_C(   571.46), SIMDE_FLOAT32_C(   422.21), SIMDE_FLOAT32_C(   467.76),
                         SIMDE_FLOAT32_C(   670.24), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    39.01), SIMDE_FLOAT32_C(   346.63)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   178.20), SIMDE_FLOAT32_C(   233.37), SIMDE_FLOAT32_C(   261.31), SIMDE_FLOAT32_C(  -976.55),
                         SIMDE_FLOAT32_C(  -444.81), SIMDE_FLOAT32_C(  -384.03), SIMDE_FLOAT32_C(  -305.07), SIMDE_FLOAT32_C(  -417.54),
                         SIMDE_FLOAT32_C(  -678.17), SIMDE_FLOAT32_C(    84.77), SIMDE_FLOAT32_C(   825.53), SIMDE_FLOAT32_C(  -269.45),
                         SIMDE_FLOAT32_C(   497.31), SIMDE_FLOAT32_C(  -297.45), SIMDE_FLOAT32_C(  -186.21), SIMDE_FLOAT32_C(  -754.38)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(   687.09), SIMDE_FLOAT32_C(     6.54), SIMDE_FLOAT32_C(  -660.80),
                         SIMDE_FLOAT32_C(    28.47), SIMDE_FLOAT32_C(  -923.64), SIMDE_FLOAT32_C(  -860.95), SIMDE_FLOAT32_C(    -1.57),
                         SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    10.92), SIMDE_FLOAT32_C(    -3.60), SIMDE_FLOAT32_C(  -104.17),
                         SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    34.06), SIMDE_FLOAT32_C(    -0.11), SIMDE_FLOAT32_C(   346.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   469.66), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(  -203.65), SIMDE_FLOAT32_C(   336.73), SIMDE_FLOAT32_C(  -747.59), SIMDE_FLOAT32_C(  -554.19),
                         SIMDE_FLOAT32_C(  -387.90), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(   532.35), SIMDE_FLOAT32_C(   780.64),
                         SIMDE_FLOAT32_C(  -770.35), SIMDE_FLOAT32_C(  -583.60), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(    28.08)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -171.51), SIMDE_FLOAT32_C(   680.02), SIMDE_FLOAT32_C(   818.66), SIMDE_FLOAT32_C(   600.47),
                         SIMDE_FLOAT32_C(   254.31), SIMDE_FLOAT32_C(   -80.73), SIMDE_FLOAT32_C(  -944.78), SIMDE_FLOAT32_C(  -767.23),
                         SIMDE_FLOAT32_C(   398.82), SIMDE_FLOAT32_C(   395.92), SIMDE_FLOAT32_C(   339.21), SIMDE_FLOAT32_C(  -263.99),
                         SIMDE_FLOAT32_C(   -30.79), SIMDE_FLOAT32_C(   443.48), SIMDE_FLOAT32_C(   380.46), SIMDE_FLOAT32_C(   993.90)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.15), SIMDE_FLOAT32_C(  -148.69), SIMDE_FLOAT32_C(   910.03), SIMDE_FLOAT32_C(   791.23),
                         SIMDE_FLOAT32_C(     3.56), SIMDE_FLOAT32_C(    -6.13), SIMDE_FLOAT32_C(    -0.99), SIMDE_FLOAT32_C(    -1.08),
                         SIMDE_FLOAT32_C(     0.80), SIMDE_FLOAT32_C(   655.87), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -9.50),
                         SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     8.75), SIMDE_FLOAT32_C(  -770.72), SIMDE_FLOAT32_C(   -14.67)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(   840.65), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    99.93),
                         SIMDE_FLOAT32_C(  -738.19), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(   343.48), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(  -822.65), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    27.25), SIMDE_FLOAT32_C(   690.12), SIMDE_FLOAT32_C(   -21.09), SIMDE_FLOAT32_C(  -448.89),
                         SIMDE_FLOAT32_C(   505.79), SIMDE_FLOAT32_C(   831.02), SIMDE_FLOAT32_C(   977.36), SIMDE_FLOAT32_C(   331.34),
                         SIMDE_FLOAT32_C(   462.95), SIMDE_FLOAT32_C(  -178.99), SIMDE_FLOAT32_C(   324.62), SIMDE_FLOAT32_C(  -874.31),
                         SIMDE_FLOAT32_C(  -328.54), SIMDE_FLOAT32_C(  -192.31), SIMDE_FLOAT32_C(   561.36), SIMDE_FLOAT32_C(   -70.91)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -95.15), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(  -591.56), SIMDE_FLOAT32_C(   731.49),
                         SIMDE_FLOAT32_C(   623.70), SIMDE_FLOAT32_C(   140.67), SIMDE_FLOAT32_C(  -906.16), SIMDE_FLOAT32_C(    -0.55),
                         SIMDE_FLOAT32_C(    -4.35), SIMDE_FLOAT32_C(   758.79), SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(  -797.92),
                         SIMDE_FLOAT32_C(  -525.83), SIMDE_FLOAT32_C(    -0.22), SIMDE_FLOAT32_C(   655.67), SIMDE_FLOAT32_C(   543.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(  -327.22), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(  -573.81), SIMDE_FLOAT32_C(  -337.60),
                         SIMDE_FLOAT32_C(   293.64), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(   710.38), SIMDE_FLOAT32_C(  -756.42)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   897.27), SIMDE_FLOAT32_C(  -197.89), SIMDE_FLOAT32_C(  -359.76), SIMDE_FLOAT32_C(   -33.67),
                         SIMDE_FLOAT32_C(     7.27), SIMDE_FLOAT32_C(  -125.20), SIMDE_FLOAT32_C(    39.93), SIMDE_FLOAT32_C(   394.67),
                         SIMDE_FLOAT32_C(  -304.73), SIMDE_FLOAT32_C(  -696.69), SIMDE_FLOAT32_C(   822.06), SIMDE_FLOAT32_C(  -997.63),
                         SIMDE_FLOAT32_C(   923.64), SIMDE_FLOAT32_C(  -768.12), SIMDE_FLOAT32_C(   -67.64), SIMDE_FLOAT32_C(   977.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -348.70), SIMDE_FLOAT32_C(  -438.19), SIMDE_FLOAT32_C(  -752.43), SIMDE_FLOAT32_C(   932.66),
                         SIMDE_FLOAT32_C(     0.13), SIMDE_FLOAT32_C(  -182.45), SIMDE_FLOAT32_C(   510.85), SIMDE_FLOAT32_C(    14.34),
                         SIMDE_FLOAT32_C(   916.26), SIMDE_FLOAT32_C(  -769.09), SIMDE_FLOAT32_C(    -4.68), SIMDE_FLOAT32_C(     7.46),
                         SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(  -576.22), SIMDE_FLOAT32_C(    -2.43), SIMDE_FLOAT32_C(     4.51)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(  -737.13), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(   177.92),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(   888.71), SIMDE_FLOAT32_C(   915.71), SIMDE_FLOAT32_C(   133.52),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(  -775.04), SIMDE_FLOAT32_C(   440.64)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   496.57), SIMDE_FLOAT32_C(   915.19), SIMDE_FLOAT32_C(  -718.40), SIMDE_FLOAT32_C(   159.97),
                         SIMDE_FLOAT32_C(  -861.01), SIMDE_FLOAT32_C(   426.61), SIMDE_FLOAT32_C(   932.11), SIMDE_FLOAT32_C(   110.36),
                         SIMDE_FLOAT32_C(   826.84), SIMDE_FLOAT32_C(   -76.75), SIMDE_FLOAT32_C(   237.58), SIMDE_FLOAT32_C(  -378.50),
                         SIMDE_FLOAT32_C(  -601.68), SIMDE_FLOAT32_C(  -623.50), SIMDE_FLOAT32_C(  -942.47), SIMDE_FLOAT32_C(   475.51)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   -15.61), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(  -314.93), SIMDE_FLOAT32_C(    -0.36),
                         SIMDE_FLOAT32_C(   345.93), SIMDE_FLOAT32_C(     2.31), SIMDE_FLOAT32_C(     0.63), SIMDE_FLOAT32_C(    -2.69),
                         SIMDE_FLOAT32_C(   484.94), SIMDE_FLOAT32_C(  -598.06), SIMDE_FLOAT32_C(  -791.07), SIMDE_FLOAT32_C(  -765.93),
                         SIMDE_FLOAT32_C(   221.37), SIMDE_FLOAT32_C(  -788.36), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(   440.64)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(  -807.28), SIMDE_FLOAT32_C(   -70.05), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(    92.52), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(   834.60), SIMDE_FLOAT32_C(   -65.60),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   556.35), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -964.25), SIMDE_FLOAT32_C(  -406.33), SIMDE_FLOAT32_C(  -743.66), SIMDE_FLOAT32_C(  -764.58),
                         SIMDE_FLOAT32_C(   789.89), SIMDE_FLOAT32_C(     4.83), SIMDE_FLOAT32_C(  -818.54), SIMDE_FLOAT32_C(   161.06),
                         SIMDE_FLOAT32_C(   579.25), SIMDE_FLOAT32_C(   -11.78), SIMDE_FLOAT32_C(  -308.52), SIMDE_FLOAT32_C(  -719.57),
                         SIMDE_FLOAT32_C(   334.00), SIMDE_FLOAT32_C(   274.71), SIMDE_FLOAT32_C(  -916.82), SIMDE_FLOAT32_C(  -490.00)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   883.05), SIMDE_FLOAT32_C(    -1.05), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(  -784.34),
                         SIMDE_FLOAT32_C(     2.73), SIMDE_FLOAT32_C(   206.60), SIMDE_FLOAT32_C(     6.66), SIMDE_FLOAT32_C(    -0.34),
                         SIMDE_FLOAT32_C(  -286.07), SIMDE_FLOAT32_C(  -212.86), SIMDE_FLOAT32_C(  -318.38), SIMDE_FLOAT32_C(   783.48),
                         SIMDE_FLOAT32_C(  -628.82), SIMDE_FLOAT32_C(   -12.14), SIMDE_FLOAT32_C(   439.43), SIMDE_FLOAT32_C(   434.03)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(   105.79), SIMDE_FLOAT32_C(   590.10),
                         SIMDE_FLOAT32_C(    30.91), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(   -84.00), SIMDE_FLOAT32_C(    80.04),
                         SIMDE_FLOAT32_C(  -709.46), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(  -889.11)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    18.75), SIMDE_FLOAT32_C(   809.05), SIMDE_FLOAT32_C(   144.05), SIMDE_FLOAT32_C(  -427.72),
                         SIMDE_FLOAT32_C(   308.28), SIMDE_FLOAT32_C(  -177.05), SIMDE_FLOAT32_C(  -457.77), SIMDE_FLOAT32_C(   678.24),
                         SIMDE_FLOAT32_C(    66.05), SIMDE_FLOAT32_C(  -267.71), SIMDE_FLOAT32_C(   117.28), SIMDE_FLOAT32_C(  -576.80),
                         SIMDE_FLOAT32_C(   -38.39), SIMDE_FLOAT32_C(  -250.14), SIMDE_FLOAT32_C(   -53.92), SIMDE_FLOAT32_C(    91.94)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(   529.63), SIMDE_FLOAT32_C(   -24.89), SIMDE_FLOAT32_C(  -967.78), SIMDE_FLOAT32_C(   638.94),
                         SIMDE_FLOAT32_C(   450.90), SIMDE_FLOAT32_C(  -771.54), SIMDE_FLOAT32_C(     7.33), SIMDE_FLOAT32_C(    -0.89),
                         SIMDE_FLOAT32_C(     2.25), SIMDE_FLOAT32_C(   635.35), SIMDE_FLOAT32_C(    -1.94), SIMDE_FLOAT32_C(    -0.75),
                         SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(   607.86), SIMDE_FLOAT32_C(   394.58), SIMDE_FLOAT32_C(   -29.52)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(  -493.41), SIMDE_FLOAT32_C(   822.72),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(  -816.27),
                         SIMDE_FLOAT32_C(  -209.34), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(  -728.70), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(   100.32), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(  -204.33)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -841.43), SIMDE_FLOAT32_C(   -14.16), SIMDE_FLOAT32_C(   824.88), SIMDE_FLOAT32_C(   793.63),
                         SIMDE_FLOAT32_C(  -736.75), SIMDE_FLOAT32_C(  -310.57), SIMDE_FLOAT32_C(   728.87), SIMDE_FLOAT32_C(  -350.72),
                         SIMDE_FLOAT32_C(    60.89), SIMDE_FLOAT32_C(   109.81), SIMDE_FLOAT32_C(   715.94), SIMDE_FLOAT32_C(  -250.60),
                         SIMDE_FLOAT32_C(   944.14), SIMDE_FLOAT32_C(   361.85), SIMDE_FLOAT32_C(   -13.07), SIMDE_FLOAT32_C(   852.60)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(  -788.39), SIMDE_FLOAT32_C(   330.43), SIMDE_FLOAT32_C(    -3.76), SIMDE_FLOAT32_C(     3.40),
                         SIMDE_FLOAT32_C(   956.68), SIMDE_FLOAT32_C(   954.62), SIMDE_FLOAT32_C(   825.49), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(     1.80), SIMDE_FLOAT32_C(  -933.21), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(  -420.06),
                         SIMDE_FLOAT32_C(     0.97), SIMDE_FLOAT32_C(   103.15), SIMDE_FLOAT32_C(   439.77), SIMDE_FLOAT32_C(    -1.09)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_tand_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_tand_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C(  670.24),
                         SIMDE_FLOAT64_C( -297.45), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C(   39.01),
                         SIMDE_FLOAT64_C( -754.38), SIMDE_FLOAT64_C(  346.63)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.92), SIMDE_FLOAT64_C(   -1.18),
                         SIMDE_FLOAT64_C(    1.93), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C(   -0.11), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.24)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C( -686.13),
                         SIMDE_FLOAT64_C(   84.77), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C(  422.21),
                         SIMDE_FLOAT64_C( -269.45), SIMDE_FLOAT64_C(  467.76)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   10.92), SIMDE_FLOAT64_C(    0.61),
                         SIMDE_FLOAT64_C(   -3.60), SIMDE_FLOAT64_C(    1.90),
                         SIMDE_FLOAT64_C( -104.17), SIMDE_FLOAT64_C(   -3.12)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -384.03), SIMDE_FLOAT64_C( -923.64),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -860.95),
                         SIMDE_FLOAT64_C( -417.54), SIMDE_FLOAT64_C(  696.87)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  -11.01), SIMDE_FLOAT64_C(    0.54),
                         SIMDE_FLOAT64_C(   -0.45), SIMDE_FLOAT64_C(   -0.44),
                         SIMDE_FLOAT64_C(    1.42), SIMDE_FLOAT64_C(    0.81),
                         SIMDE_FLOAT64_C(   -1.57), SIMDE_FLOAT64_C(   -0.43)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  233.37), SIMDE_FLOAT64_C(  687.09),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -976.55), SIMDE_FLOAT64_C( -660.80)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(   85.51),
                         SIMDE_FLOAT64_C(    1.35), SIMDE_FLOAT64_C(   -0.65),
                         SIMDE_FLOAT64_C(    6.54), SIMDE_FLOAT64_C(   -0.64),
                         SIMDE_FLOAT64_C(   -4.18), SIMDE_FLOAT64_C(    1.68)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -770.35), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C(  380.46),
                         SIMDE_FLOAT64_C( -770.72), SIMDE_FLOAT64_C(  993.90),
                         SIMDE_FLOAT64_C(   28.08), SIMDE_FLOAT64_C(  841.21)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.21), SIMDE_FLOAT64_C(    8.75),
                         SIMDE_FLOAT64_C(   -0.95), SIMDE_FLOAT64_C(    0.37),
                         SIMDE_FLOAT64_C(   -1.22), SIMDE_FLOAT64_C(  -14.67),
                         SIMDE_FLOAT64_C(    0.53), SIMDE_FLOAT64_C(   -1.65)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -387.90), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  339.21),
                         SIMDE_FLOAT64_C(  532.35), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C(  -30.79)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.53), SIMDE_FLOAT64_C(    0.72),
                         SIMDE_FLOAT64_C(   -2.06), SIMDE_FLOAT64_C(   -0.38),
                         SIMDE_FLOAT64_C(   -0.13), SIMDE_FLOAT64_C(   -9.50),
                         SIMDE_FLOAT64_C(    1.78), SIMDE_FLOAT64_C(   -0.60)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -203.65), SIMDE_FLOAT64_C(  -80.73),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -944.78),
                         SIMDE_FLOAT64_C( -747.59), SIMDE_FLOAT64_C( -767.23),
                         SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C(  398.82)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(   -6.13),
                         SIMDE_FLOAT64_C(   -0.43), SIMDE_FLOAT64_C(   -0.99),
                         SIMDE_FLOAT64_C(   -0.52), SIMDE_FLOAT64_C(   -1.08),
                         SIMDE_FLOAT64_C(   -0.25), SIMDE_FLOAT64_C(    0.80)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  469.66), SIMDE_FLOAT64_C(  680.02),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  910.03), SIMDE_FLOAT64_C(  600.47),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C(  254.31)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -2.80), SIMDE_FLOAT64_C(   -0.84),
                         SIMDE_FLOAT64_C(    0.61), SIMDE_FLOAT64_C(   -6.57),
                         SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(    1.77),
                         SIMDE_FLOAT64_C(    2.94), SIMDE_FLOAT64_C(    3.56)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_tand_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_tand_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -686.13), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(  670.24), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   39.01), SIMDE_FLOAT64_C(  346.63)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -678.17), SIMDE_FLOAT64_C(   84.77),
                         SIMDE_FLOAT64_C(  825.53), SIMDE_FLOAT64_C( -269.45),
                         SIMDE_FLOAT64_C(  497.31), SIMDE_FLOAT64_C( -297.45),
                         SIMDE_FLOAT64_C( -186.21), SIMDE_FLOAT64_C( -754.38)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(  571.46),
                         SIMDE_FLOAT64_C(  422.21), SIMDE_FLOAT64_C(  467.76),
                         SIMDE_FLOAT64_C(   -0.92), SIMDE_FLOAT64_C(   34.06),
                         SIMDE_FLOAT64_C(   -0.11), SIMDE_FLOAT64_C(   -0.68)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  178.20), SIMDE_FLOAT64_C(  233.37),
                         SIMDE_FLOAT64_C(  261.31), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C( -384.03),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C( -417.54)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  841.21), SIMDE_FLOAT64_C( -450.67),
                         SIMDE_FLOAT64_C(  687.09), SIMDE_FLOAT64_C( -212.54),
                         SIMDE_FLOAT64_C( -660.80), SIMDE_FLOAT64_C(   28.47),
                         SIMDE_FLOAT64_C( -923.64), SIMDE_FLOAT64_C( -860.95)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.65), SIMDE_FLOAT64_C(   85.51),
                         SIMDE_FLOAT64_C(   -0.65), SIMDE_FLOAT64_C( -976.55),
                         SIMDE_FLOAT64_C( -444.81), SIMDE_FLOAT64_C(    0.54),
                         SIMDE_FLOAT64_C( -305.07), SIMDE_FLOAT64_C(    0.81)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  398.82), SIMDE_FLOAT64_C(  395.92),
                         SIMDE_FLOAT64_C(  339.21), SIMDE_FLOAT64_C( -263.99),
                         SIMDE_FLOAT64_C(  -30.79), SIMDE_FLOAT64_C(  443.48),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(  993.90)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -554.19), SIMDE_FLOAT64_C( -387.90),
                         SIMDE_FLOAT64_C(  655.87), SIMDE_FLOAT64_C(  532.35),
                         SIMDE_FLOAT64_C(  780.64), SIMDE_FLOAT64_C( -770.35),
                         SIMDE_FLOAT64_C( -583.60), SIMDE_FLOAT64_C( -770.72)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.25), SIMDE_FLOAT64_C(   -0.53),
                         SIMDE_FLOAT64_C(   -2.06), SIMDE_FLOAT64_C(   -0.13),
                         SIMDE_FLOAT64_C(    1.78), SIMDE_FLOAT64_C(   -1.21),
                         SIMDE_FLOAT64_C(  380.46), SIMDE_FLOAT64_C(   -1.22)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(  469.66),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(  910.03),
                         SIMDE_FLOAT64_C(  791.23), SIMDE_FLOAT64_C( -203.65),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C( -747.59)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  543.35), SIMDE_FLOAT64_C( -171.51),
                         SIMDE_FLOAT64_C(  680.02), SIMDE_FLOAT64_C(  818.66),
                         SIMDE_FLOAT64_C(  600.47), SIMDE_FLOAT64_C(  254.31),
                         SIMDE_FLOAT64_C(  -80.73), SIMDE_FLOAT64_C( -944.78)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  120.65), SIMDE_FLOAT64_C(    0.15),
                         SIMDE_FLOAT64_C( -148.69), SIMDE_FLOAT64_C(   -6.57),
                         SIMDE_FLOAT64_C(    1.77), SIMDE_FLOAT64_C(    3.56),
                         SIMDE_FLOAT64_C(  336.73), SIMDE_FLOAT64_C(   -0.99)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   99.93), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(  343.48),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(  655.67)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  331.34), SIMDE_FLOAT64_C(  462.95),
                         SIMDE_FLOAT64_C( -178.99), SIMDE_FLOAT64_C(  324.62),
                         SIMDE_FLOAT64_C( -874.31), SIMDE_FLOAT64_C( -328.54),
                         SIMDE_FLOAT64_C( -192.31), SIMDE_FLOAT64_C(  561.36)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C( -738.19),
                         SIMDE_FLOAT64_C(  758.79), SIMDE_FLOAT64_C(   -0.71),
                         SIMDE_FLOAT64_C( -797.92), SIMDE_FLOAT64_C( -525.83),
                         SIMDE_FLOAT64_C( -822.65), SIMDE_FLOAT64_C(    0.39)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(   27.25),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C( -448.89), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(  831.02), SIMDE_FLOAT64_C(  977.36)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  977.49), SIMDE_FLOAT64_C(  424.81),
                         SIMDE_FLOAT64_C(  -95.15), SIMDE_FLOAT64_C(  840.65),
                         SIMDE_FLOAT64_C( -591.56), SIMDE_FLOAT64_C(  731.49),
                         SIMDE_FLOAT64_C(  623.70), SIMDE_FLOAT64_C(  140.67)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -756.42), SIMDE_FLOAT64_C(    2.13),
                         SIMDE_FLOAT64_C(  690.12), SIMDE_FLOAT64_C(  -21.09),
                         SIMDE_FLOAT64_C(   -1.26), SIMDE_FLOAT64_C(  505.79),
                         SIMDE_FLOAT64_C(    9.06), SIMDE_FLOAT64_C(   -0.82)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C( -304.73),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(  822.06),
                         SIMDE_FLOAT64_C( -997.63), SIMDE_FLOAT64_C(  923.64),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(  -67.64)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  510.85), SIMDE_FLOAT64_C(   14.34),
                         SIMDE_FLOAT64_C(  916.26), SIMDE_FLOAT64_C( -769.09),
                         SIMDE_FLOAT64_C( -573.81), SIMDE_FLOAT64_C( -337.60),
                         SIMDE_FLOAT64_C(  293.64), SIMDE_FLOAT64_C( -576.22)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(  394.67), SIMDE_FLOAT64_C(    0.26),
                         SIMDE_FLOAT64_C( -696.69), SIMDE_FLOAT64_C(   -1.15),
                         SIMDE_FLOAT64_C(   -0.67), SIMDE_FLOAT64_C(    0.41),
                         SIMDE_FLOAT64_C( -768.12), SIMDE_FLOAT64_C(   -0.73)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(  475.51), SIMDE_FLOAT64_C(  936.65),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C( -438.19),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(  932.66),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C( -182.45)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C( -775.04), SIMDE_FLOAT64_C(  440.64),
                         SIMDE_FLOAT64_C(  897.27), SIMDE_FLOAT64_C( -197.89),
                         SIMDE_FLOAT64_C( -359.76), SIMDE_FLOAT64_C(  -33.67),
                         SIMDE_FLOAT64_C(    7.27), SIMDE_FLOAT64_C( -125.20)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -1.43), SIMDE_FLOAT64_C(    6.07),
                         SIMDE_FLOAT64_C( -348.70), SIMDE_FLOAT64_C(   -0.32),
                         SIMDE_FLOAT64_C( -752.43), SIMDE_FLOAT64_C(   -0.67),
                         SIMDE_FLOAT64_C( -327.22), SIMDE_FLOAT64_C(    1.42)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_tand_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_trunc_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[4];
    const simde_float32 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   -12.21), SIMDE_FLOAT32_C(  -120.46), SIMDE_FLOAT32_C(   116.11), SIMDE_FLOAT32_C(  -957.73) },
      { SIMDE_FLOAT32_C(   -12.00), SIMDE_FLOAT32_C(  -120.00), SIMDE_FLOAT32_C(   116.00), SIMDE_FLOAT32_C(  -957.00) } },
    { { SIMDE_FLOAT32_C(  -970.43), SIMDE_FLOAT32_C(    73.72), SIMDE_FLOAT32_C(   741.23), SIMDE_FLOAT32_C(  -161.72) },
      { SIMDE_FLOAT32_C(  -970.00), SIMDE_FLOAT32_C(    73.00), SIMDE_FLOAT32_C(   741.00), SIMDE_FLOAT32_C(  -161.00) } },
    { { SIMDE_FLOAT32_C(  -669.85), SIMDE_FLOAT32_C(   861.65), SIMDE_FLOAT32_C(   481.06), SIMDE_FLOAT32_C(  -607.16) },
      { SIMDE_FLOAT32_C(  -669.00), SIMDE_FLOAT32_C(   861.00), SIMDE_FLOAT32_C(   481.00), SIMDE_FLOAT32_C(  -607.00) } },
    { { SIMDE_FLOAT32_C(   227.64), SIMDE_FLOAT32_C(  -106.69), SIMDE_FLOAT32_C(   -76.28), SIMDE_FLOAT32_C(   195.74) },
      { SIMDE_FLOAT32_C(   227.00), SIMDE_FLOAT32_C(  -106.00), SIMDE_FLOAT32_C(   -76.00), SIMDE_FLOAT32_C(   195.00) } },
    { { SIMDE_FLOAT32_C(  -755.50), SIMDE_FLOAT32_C(  -618.75), SIMDE_FLOAT32_C(  -293.56), SIMDE_FLOAT32_C(  -686.30) },
      { SIMDE_FLOAT32_C(  -755.00), SIMDE_FLOAT32_C(  -618.00), SIMDE_FLOAT32_C(  -293.00), SIMDE_FLOAT32_C(  -686.00) } },
    { { SIMDE_FLOAT32_C(  -454.44), SIMDE_FLOAT32_C(  -493.17), SIMDE_FLOAT32_C(    45.88), SIMDE_FLOAT32_C(  -307.36) },
      { SIMDE_FLOAT32_C(  -454.00), SIMDE_FLOAT32_C(  -493.00), SIMDE_FLOAT32_C(    45.00), SIMDE_FLOAT32_C(  -307.00) } },
    { { SIMDE_FLOAT32_C(  -593.72), SIMDE_FLOAT32_C(  -346.10), SIMDE_FLOAT32_C(  -356.52), SIMDE_FLOAT32_C(  -727.29) },
      { SIMDE_FLOAT32_C(  -593.00), SIMDE_FLOAT32_C(  -346.00), SIMDE_FLOAT32_C(  -356.00), SIMDE_FLOAT32_C(  -727.00) } },
    { { SIMDE_FLOAT32_C(   304.91), SIMDE_FLOAT32_C(   961.56), SIMDE_FLOAT32_C(   582.51), SIMDE_FLOAT32_C(  -707.29) },
      { SIMDE_FLOAT32_C(   304.00), SIMDE_FLOAT32_C(   961.00), SIMDE_FLOAT32_C(   582.00), SIMDE_FLOAT32_C(  -707.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128 a = simde_mm_loadu_ps(test_vec[i].a);
    simde__m128 r = simde_mm_trunc_ps(a);
    simde_test_x86_assert_equal_f32x4(r, simde_mm_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_trunc_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[2];
    const simde_float64 r[2];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(  -536.40), SIMDE_FLOAT64_C(  -763.02) },
      { SIMDE_FLOAT64_C(  -536.00), SIMDE_FLOAT64_C(  -763.00) } },
    { { SIMDE_FLOAT64_C(  -999.42), SIMDE_FLOAT64_C(  -310.98) },
      { SIMDE_FLOAT64_C(  -999.00), SIMDE_FLOAT64_C(  -310.00) } },
    { { SIMDE_FLOAT64_C(  -951.25), SIMDE_FLOAT64_C(   277.33) },
      { SIMDE_FLOAT64_C(  -951.00), SIMDE_FLOAT64_C(   277.00) } },
    { { SIMDE_FLOAT64_C(   -98.58), SIMDE_FLOAT64_C(  -936.47) },
      { SIMDE_FLOAT64_C(   -98.00), SIMDE_FLOAT64_C(  -936.00) } },
    { { SIMDE_FLOAT64_C(  -124.20), SIMDE_FLOAT64_C(  -990.68) },
      { SIMDE_FLOAT64_C(  -124.00), SIMDE_FLOAT64_C(  -990.00) } },
    { { SIMDE_FLOAT64_C(  -319.44), SIMDE_FLOAT64_C(   434.58) },
      { SIMDE_FLOAT64_C(  -319.00), SIMDE_FLOAT64_C(   434.00) } },
    { { SIMDE_FLOAT64_C(   209.02), SIMDE_FLOAT64_C(   196.07) },
      { SIMDE_FLOAT64_C(   209.00), SIMDE_FLOAT64_C(   196.00) } },
    { { SIMDE_FLOAT64_C(  -740.77), SIMDE_FLOAT64_C(   179.41) },
      { SIMDE_FLOAT64_C(  -740.00), SIMDE_FLOAT64_C(   179.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m128d a = simde_mm_loadu_pd(test_vec[i].a);
    simde__m128d r = simde_mm_trunc_pd(a);
    simde_test_x86_assert_equal_f64x2(r, simde_mm_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_trunc_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[8];
    const simde_float32 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(  -239.01), SIMDE_FLOAT32_C(  -492.80), SIMDE_FLOAT32_C(  -937.05), SIMDE_FLOAT32_C(  -286.30),
        SIMDE_FLOAT32_C(   826.89), SIMDE_FLOAT32_C(   311.87), SIMDE_FLOAT32_C(  -290.83), SIMDE_FLOAT32_C(   155.81) },
      { SIMDE_FLOAT32_C(  -239.00), SIMDE_FLOAT32_C(  -492.00), SIMDE_FLOAT32_C(  -937.00), SIMDE_FLOAT32_C(  -286.00),
        SIMDE_FLOAT32_C(   826.00), SIMDE_FLOAT32_C(   311.00), SIMDE_FLOAT32_C(  -290.00), SIMDE_FLOAT32_C(   155.00) } },
    { { SIMDE_FLOAT32_C(   497.98), SIMDE_FLOAT32_C(   770.36), SIMDE_FLOAT32_C(  -368.92), SIMDE_FLOAT32_C(  -362.61),
        SIMDE_FLOAT32_C(  -693.36), SIMDE_FLOAT32_C(  -206.15), SIMDE_FLOAT32_C(  -571.56), SIMDE_FLOAT32_C(  -305.34) },
      { SIMDE_FLOAT32_C(   497.00), SIMDE_FLOAT32_C(   770.00), SIMDE_FLOAT32_C(  -368.00), SIMDE_FLOAT32_C(  -362.00),
        SIMDE_FLOAT32_C(  -693.00), SIMDE_FLOAT32_C(  -206.00), SIMDE_FLOAT32_C(  -571.00), SIMDE_FLOAT32_C(  -305.00) } },
    { { SIMDE_FLOAT32_C(  -237.16), SIMDE_FLOAT32_C(   968.44), SIMDE_FLOAT32_C(   -77.70), SIMDE_FLOAT32_C(   170.55),
        SIMDE_FLOAT32_C(  -930.56), SIMDE_FLOAT32_C(   755.06), SIMDE_FLOAT32_C(    78.43), SIMDE_FLOAT32_C(  -634.89) },
      { SIMDE_FLOAT32_C(  -237.00), SIMDE_FLOAT32_C(   968.00), SIMDE_FLOAT32_C(   -77.00), SIMDE_FLOAT32_C(   170.00),
        SIMDE_FLOAT32_C(  -930.00), SIMDE_FLOAT32_C(   755.00), SIMDE_FLOAT32_C(    78.00), SIMDE_FLOAT32_C(  -634.00) } },
    { { SIMDE_FLOAT32_C(   107.17), SIMDE_FLOAT32_C(   191.02), SIMDE_FLOAT32_C(  -424.61), SIMDE_FLOAT32_C(  -603.58),
        SIMDE_FLOAT32_C(  -501.82), SIMDE_FLOAT32_C(  -855.61), SIMDE_FLOAT32_C(   927.91), SIMDE_FLOAT32_C(   259.17) },
      { SIMDE_FLOAT32_C(   107.00), SIMDE_FLOAT32_C(   191.00), SIMDE_FLOAT32_C(  -424.00), SIMDE_FLOAT32_C(  -603.00),
        SIMDE_FLOAT32_C(  -501.00), SIMDE_FLOAT32_C(  -855.00), SIMDE_FLOAT32_C(   927.00), SIMDE_FLOAT32_C(   259.00) } },
    { { SIMDE_FLOAT32_C(  -348.41), SIMDE_FLOAT32_C(   990.86), SIMDE_FLOAT32_C(   972.87), SIMDE_FLOAT32_C(  -521.52),
        SIMDE_FLOAT32_C(   302.73), SIMDE_FLOAT32_C(  -317.96), SIMDE_FLOAT32_C(   634.29), SIMDE_FLOAT32_C(  -199.28) },
      { SIMDE_FLOAT32_C(  -348.00), SIMDE_FLOAT32_C(   990.00), SIMDE_FLOAT32_C(   972.00), SIMDE_FLOAT32_C(  -521.00),
        SIMDE_FLOAT32_C(   302.00), SIMDE_FLOAT32_C(  -317.00), SIMDE_FLOAT32_C(   634.00), SIMDE_FLOAT32_C(  -199.00) } },
    { { SIMDE_FLOAT32_C(  -547.60), SIMDE_FLOAT32_C(  -734.63), SIMDE_FLOAT32_C(   438.11), SIMDE_FLOAT32_C(  -240.96),
        SIMDE_FLOAT32_C(    59.22), SIMDE_FLOAT32_C(   866.55), SIMDE_FLOAT32_C(   453.70), SIMDE_FLOAT32_C(   822.06) },
      { SIMDE_FLOAT32_C(  -547.00), SIMDE_FLOAT32_C(  -734.00), SIMDE_FLOAT32_C(   438.00), SIMDE_FLOAT32_C(  -240.00),
        SIMDE_FLOAT32_C(    59.00), SIMDE_FLOAT32_C(   866.00), SIMDE_FLOAT32_C(   453.00), SIMDE_FLOAT32_C(   822.00) } },
    { { SIMDE_FLOAT32_C(   834.99), SIMDE_FLOAT32_C(  -624.00), SIMDE_FLOAT32_C(    -7.39), SIMDE_FLOAT32_C(   904.43),
        SIMDE_FLOAT32_C(  -868.94), SIMDE_FLOAT32_C(  -928.96), SIMDE_FLOAT32_C(  -730.46), SIMDE_FLOAT32_C(   238.23) },
      { SIMDE_FLOAT32_C(   834.00), SIMDE_FLOAT32_C(  -624.00), SIMDE_FLOAT32_C(    -7.00), SIMDE_FLOAT32_C(   904.00),
        SIMDE_FLOAT32_C(  -868.00), SIMDE_FLOAT32_C(  -928.00), SIMDE_FLOAT32_C(  -730.00), SIMDE_FLOAT32_C(   238.00) } },
    { { SIMDE_FLOAT32_C(   262.05), SIMDE_FLOAT32_C(  -155.07), SIMDE_FLOAT32_C(   634.65), SIMDE_FLOAT32_C(   760.24),
        SIMDE_FLOAT32_C(   -10.68), SIMDE_FLOAT32_C(   562.56), SIMDE_FLOAT32_C(    19.41), SIMDE_FLOAT32_C(   640.92) },
      { SIMDE_FLOAT32_C(   262.00), SIMDE_FLOAT32_C(  -155.00), SIMDE_FLOAT32_C(   634.00), SIMDE_FLOAT32_C(   760.00),
        SIMDE_FLOAT32_C(   -10.00), SIMDE_FLOAT32_C(   562.00), SIMDE_FLOAT32_C(    19.00), SIMDE_FLOAT32_C(   640.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256 a = simde_mm256_loadu_ps(test_vec[i].a);
    simde__m256 r = simde_mm256_trunc_ps(a);
    simde_test_x86_assert_equal_f32x8(r, simde_mm256_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm256_trunc_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[4];
    const simde_float64 r[4];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   693.29), SIMDE_FLOAT64_C(   980.27), SIMDE_FLOAT64_C(  -292.17), SIMDE_FLOAT64_C(  -318.62) },
      { SIMDE_FLOAT64_C(   693.00), SIMDE_FLOAT64_C(   980.00), SIMDE_FLOAT64_C(  -292.00), SIMDE_FLOAT64_C(  -318.00) } },
    { { SIMDE_FLOAT64_C(  -733.59), SIMDE_FLOAT64_C(  -256.43), SIMDE_FLOAT64_C(   726.81), SIMDE_FLOAT64_C(   443.36) },
      { SIMDE_FLOAT64_C(  -733.00), SIMDE_FLOAT64_C(  -256.00), SIMDE_FLOAT64_C(   726.00), SIMDE_FLOAT64_C(   443.00) } },
    { { SIMDE_FLOAT64_C(  -589.23), SIMDE_FLOAT64_C(  -428.07), SIMDE_FLOAT64_C(  -734.42), SIMDE_FLOAT64_C(   315.59) },
      { SIMDE_FLOAT64_C(  -589.00), SIMDE_FLOAT64_C(  -428.00), SIMDE_FLOAT64_C(  -734.00), SIMDE_FLOAT64_C(   315.00) } },
    { { SIMDE_FLOAT64_C(   286.91), SIMDE_FLOAT64_C(  -276.33), SIMDE_FLOAT64_C(  -306.67), SIMDE_FLOAT64_C(  -257.37) },
      { SIMDE_FLOAT64_C(   286.00), SIMDE_FLOAT64_C(  -276.00), SIMDE_FLOAT64_C(  -306.00), SIMDE_FLOAT64_C(  -257.00) } },
    { { SIMDE_FLOAT64_C(   -92.17), SIMDE_FLOAT64_C(  -253.48), SIMDE_FLOAT64_C(   663.58), SIMDE_FLOAT64_C(  -246.72) },
      { SIMDE_FLOAT64_C(   -92.00), SIMDE_FLOAT64_C(  -253.00), SIMDE_FLOAT64_C(   663.00), SIMDE_FLOAT64_C(  -246.00) } },
    { { SIMDE_FLOAT64_C(  -825.67), SIMDE_FLOAT64_C(  -678.59), SIMDE_FLOAT64_C(   803.95), SIMDE_FLOAT64_C(   565.59) },
      { SIMDE_FLOAT64_C(  -825.00), SIMDE_FLOAT64_C(  -678.00), SIMDE_FLOAT64_C(   803.00), SIMDE_FLOAT64_C(   565.00) } },
    { { SIMDE_FLOAT64_C(  -428.00), SIMDE_FLOAT64_C(  -167.27), SIMDE_FLOAT64_C(   718.24), SIMDE_FLOAT64_C(   -22.78) },
      { SIMDE_FLOAT64_C(  -428.00), SIMDE_FLOAT64_C(  -167.00), SIMDE_FLOAT64_C(   718.00), SIMDE_FLOAT64_C(   -22.00) } },
    { { SIMDE_FLOAT64_C(  -376.65), SIMDE_FLOAT64_C(  -190.00), SIMDE_FLOAT64_C(   -12.78), SIMDE_FLOAT64_C(  -683.35) },
      { SIMDE_FLOAT64_C(  -376.00), SIMDE_FLOAT64_C(  -190.00), SIMDE_FLOAT64_C(   -12.00), SIMDE_FLOAT64_C(  -683.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m256d a = simde_mm256_loadu_pd(test_vec[i].a);
    simde__m256d r = simde_mm256_trunc_pd(a);
    simde_test_x86_assert_equal_f64x4(r, simde_mm256_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_trunc_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   606.57), SIMDE_FLOAT32_C(   426.10), SIMDE_FLOAT32_C(  -271.64), SIMDE_FLOAT32_C(  -693.93),
        SIMDE_FLOAT32_C(   123.39), SIMDE_FLOAT32_C(  -323.73), SIMDE_FLOAT32_C(  -823.48), SIMDE_FLOAT32_C(   154.72),
        SIMDE_FLOAT32_C(   215.73), SIMDE_FLOAT32_C(   870.22), SIMDE_FLOAT32_C(  -205.21), SIMDE_FLOAT32_C(   262.07),
        SIMDE_FLOAT32_C(   173.72), SIMDE_FLOAT32_C(   310.35), SIMDE_FLOAT32_C(  -516.54), SIMDE_FLOAT32_C(  -500.11) },
      { SIMDE_FLOAT32_C(   606.00), SIMDE_FLOAT32_C(   426.00), SIMDE_FLOAT32_C(  -271.00), SIMDE_FLOAT32_C(  -693.00),
        SIMDE_FLOAT32_C(   123.00), SIMDE_FLOAT32_C(  -323.00), SIMDE_FLOAT32_C(  -823.00), SIMDE_FLOAT32_C(   154.00),
        SIMDE_FLOAT32_C(   215.00), SIMDE_FLOAT32_C(   870.00), SIMDE_FLOAT32_C(  -205.00), SIMDE_FLOAT32_C(   262.00),
        SIMDE_FLOAT32_C(   173.00), SIMDE_FLOAT32_C(   310.00), SIMDE_FLOAT32_C(  -516.00), SIMDE_FLOAT32_C(  -500.00) } },
    { { SIMDE_FLOAT32_C(  -175.79), SIMDE_FLOAT32_C(  -258.58), SIMDE_FLOAT32_C(   -46.96), SIMDE_FLOAT32_C(   515.02),
        SIMDE_FLOAT32_C(   317.58), SIMDE_FLOAT32_C(   852.75), SIMDE_FLOAT32_C(   404.36), SIMDE_FLOAT32_C(    87.35),
        SIMDE_FLOAT32_C(  -977.95), SIMDE_FLOAT32_C(  -929.41), SIMDE_FLOAT32_C(   560.67), SIMDE_FLOAT32_C(    89.12),
        SIMDE_FLOAT32_C(   773.32), SIMDE_FLOAT32_C(   918.64), SIMDE_FLOAT32_C(   751.41), SIMDE_FLOAT32_C(   379.89) },
      { SIMDE_FLOAT32_C(  -175.00), SIMDE_FLOAT32_C(  -258.00), SIMDE_FLOAT32_C(   -46.00), SIMDE_FLOAT32_C(   515.00),
        SIMDE_FLOAT32_C(   317.00), SIMDE_FLOAT32_C(   852.00), SIMDE_FLOAT32_C(   404.00), SIMDE_FLOAT32_C(    87.00),
        SIMDE_FLOAT32_C(  -977.00), SIMDE_FLOAT32_C(  -929.00), SIMDE_FLOAT32_C(   560.00), SIMDE_FLOAT32_C(    89.00),
        SIMDE_FLOAT32_C(   773.00), SIMDE_FLOAT32_C(   918.00), SIMDE_FLOAT32_C(   751.00), SIMDE_FLOAT32_C(   379.00) } },
    { { SIMDE_FLOAT32_C(   344.74), SIMDE_FLOAT32_C(  -520.24), SIMDE_FLOAT32_C(   685.96), SIMDE_FLOAT32_C(  -531.87),
        SIMDE_FLOAT32_C(   156.03), SIMDE_FLOAT32_C(   862.48), SIMDE_FLOAT32_C(   622.85), SIMDE_FLOAT32_C(  -628.23),
        SIMDE_FLOAT32_C(   732.70), SIMDE_FLOAT32_C(  -582.36), SIMDE_FLOAT32_C(   633.84), SIMDE_FLOAT32_C(   -93.59),
        SIMDE_FLOAT32_C(   728.00), SIMDE_FLOAT32_C(  -882.70), SIMDE_FLOAT32_C(   406.31), SIMDE_FLOAT32_C(  -447.79) },
      { SIMDE_FLOAT32_C(   344.00), SIMDE_FLOAT32_C(  -520.00), SIMDE_FLOAT32_C(   685.00), SIMDE_FLOAT32_C(  -531.00),
        SIMDE_FLOAT32_C(   156.00), SIMDE_FLOAT32_C(   862.00), SIMDE_FLOAT32_C(   622.00), SIMDE_FLOAT32_C(  -628.00),
        SIMDE_FLOAT32_C(   732.00), SIMDE_FLOAT32_C(  -582.00), SIMDE_FLOAT32_C(   633.00), SIMDE_FLOAT32_C(   -93.00),
        SIMDE_FLOAT32_C(   728.00), SIMDE_FLOAT32_C(  -882.00), SIMDE_FLOAT32_C(   406.00), SIMDE_FLOAT32_C(  -447.00) } },
    { { SIMDE_FLOAT32_C(  -141.28), SIMDE_FLOAT32_C(  -640.65), SIMDE_FLOAT32_C(  -932.78), SIMDE_FLOAT32_C(  -823.70),
        SIMDE_FLOAT32_C(  -787.91), SIMDE_FLOAT32_C(   471.59), SIMDE_FLOAT32_C(   263.65), SIMDE_FLOAT32_C(  -765.85),
        SIMDE_FLOAT32_C(   542.17), SIMDE_FLOAT32_C(  -175.67), SIMDE_FLOAT32_C(   323.27), SIMDE_FLOAT32_C(   315.49),
        SIMDE_FLOAT32_C(  -257.03), SIMDE_FLOAT32_C(    74.67), SIMDE_FLOAT32_C(  -304.62), SIMDE_FLOAT32_C(  -912.29) },
      { SIMDE_FLOAT32_C(  -141.00), SIMDE_FLOAT32_C(  -640.00), SIMDE_FLOAT32_C(  -932.00), SIMDE_FLOAT32_C(  -823.00),
        SIMDE_FLOAT32_C(  -787.00), SIMDE_FLOAT32_C(   471.00), SIMDE_FLOAT32_C(   263.00), SIMDE_FLOAT32_C(  -765.00),
        SIMDE_FLOAT32_C(   542.00), SIMDE_FLOAT32_C(  -175.00), SIMDE_FLOAT32_C(   323.00), SIMDE_FLOAT32_C(   315.00),
        SIMDE_FLOAT32_C(  -257.00), SIMDE_FLOAT32_C(    74.00), SIMDE_FLOAT32_C(  -304.00), SIMDE_FLOAT32_C(  -912.00) } },
    { { SIMDE_FLOAT32_C(   554.43), SIMDE_FLOAT32_C(  -618.67), SIMDE_FLOAT32_C(  -444.16), SIMDE_FLOAT32_C(  -289.53),
        SIMDE_FLOAT32_C(  -756.19), SIMDE_FLOAT32_C(  -821.31), SIMDE_FLOAT32_C(    82.23), SIMDE_FLOAT32_C(   976.51),
        SIMDE_FLOAT32_C(  -403.66), SIMDE_FLOAT32_C(  -283.93), SIMDE_FLOAT32_C(  -117.08), SIMDE_FLOAT32_C(  -675.67),
        SIMDE_FLOAT32_C(  -166.63), SIMDE_FLOAT32_C(  -710.77), SIMDE_FLOAT32_C(  -123.46), SIMDE_FLOAT32_C(   692.09) },
      { SIMDE_FLOAT32_C(   554.00), SIMDE_FLOAT32_C(  -618.00), SIMDE_FLOAT32_C(  -444.00), SIMDE_FLOAT32_C(  -289.00),
        SIMDE_FLOAT32_C(  -756.00), SIMDE_FLOAT32_C(  -821.00), SIMDE_FLOAT32_C(    82.00), SIMDE_FLOAT32_C(   976.00),
        SIMDE_FLOAT32_C(  -403.00), SIMDE_FLOAT32_C(  -283.00), SIMDE_FLOAT32_C(  -117.00), SIMDE_FLOAT32_C(  -675.00),
        SIMDE_FLOAT32_C(  -166.00), SIMDE_FLOAT32_C(  -710.00), SIMDE_FLOAT32_C(  -123.00), SIMDE_FLOAT32_C(   692.00) } },
    { { SIMDE_FLOAT32_C(  -351.43), SIMDE_FLOAT32_C(   -56.24), SIMDE_FLOAT32_C(   868.39), SIMDE_FLOAT32_C(  -139.33),
        SIMDE_FLOAT32_C(  -584.65), SIMDE_FLOAT32_C(   132.04), SIMDE_FLOAT32_C(    94.81), SIMDE_FLOAT32_C(   957.53),
        SIMDE_FLOAT32_C(   956.37), SIMDE_FLOAT32_C(  -581.92), SIMDE_FLOAT32_C(   273.02), SIMDE_FLOAT32_C(  -300.66),
        SIMDE_FLOAT32_C(   492.75), SIMDE_FLOAT32_C(   968.40), SIMDE_FLOAT32_C(  -212.96), SIMDE_FLOAT32_C(    47.18) },
      { SIMDE_FLOAT32_C(  -351.00), SIMDE_FLOAT32_C(   -56.00), SIMDE_FLOAT32_C(   868.00), SIMDE_FLOAT32_C(  -139.00),
        SIMDE_FLOAT32_C(  -584.00), SIMDE_FLOAT32_C(   132.00), SIMDE_FLOAT32_C(    94.00), SIMDE_FLOAT32_C(   957.00),
        SIMDE_FLOAT32_C(   956.00), SIMDE_FLOAT32_C(  -581.00), SIMDE_FLOAT32_C(   273.00), SIMDE_FLOAT32_C(  -300.00),
        SIMDE_FLOAT32_C(   492.00), SIMDE_FLOAT32_C(   968.00), SIMDE_FLOAT32_C(  -212.00), SIMDE_FLOAT32_C(    47.00) } },
    { { SIMDE_FLOAT32_C(  -650.27), SIMDE_FLOAT32_C(   342.89), SIMDE_FLOAT32_C(   757.65), SIMDE_FLOAT32_C(  -406.46),
        SIMDE_FLOAT32_C(   521.58), SIMDE_FLOAT32_C(  -160.12), SIMDE_FLOAT32_C(  -429.95), SIMDE_FLOAT32_C(  -882.09),
        SIMDE_FLOAT32_C(   555.95), SIMDE_FLOAT32_C(   452.97), SIMDE_FLOAT32_C(  -557.75), SIMDE_FLOAT32_C(  -610.67),
        SIMDE_FLOAT32_C(   742.20), SIMDE_FLOAT32_C(   318.79), SIMDE_FLOAT32_C(  -918.58), SIMDE_FLOAT32_C(  -609.23) },
      { SIMDE_FLOAT32_C(  -650.00), SIMDE_FLOAT32_C(   342.00), SIMDE_FLOAT32_C(   757.00), SIMDE_FLOAT32_C(  -406.00),
        SIMDE_FLOAT32_C(   521.00), SIMDE_FLOAT32_C(  -160.00), SIMDE_FLOAT32_C(  -429.00), SIMDE_FLOAT32_C(  -882.00),
        SIMDE_FLOAT32_C(   555.00), SIMDE_FLOAT32_C(   452.00), SIMDE_FLOAT32_C(  -557.00), SIMDE_FLOAT32_C(  -610.00),
        SIMDE_FLOAT32_C(   742.00), SIMDE_FLOAT32_C(   318.00), SIMDE_FLOAT32_C(  -918.00), SIMDE_FLOAT32_C(  -609.00) } },
    { { SIMDE_FLOAT32_C(  -737.45), SIMDE_FLOAT32_C(   949.82), SIMDE_FLOAT32_C(   251.44), SIMDE_FLOAT32_C(  -322.10),
        SIMDE_FLOAT32_C(    81.86), SIMDE_FLOAT32_C(  -653.75), SIMDE_FLOAT32_C(  -364.57), SIMDE_FLOAT32_C(    38.23),
        SIMDE_FLOAT32_C(  -235.67), SIMDE_FLOAT32_C(   908.45), SIMDE_FLOAT32_C(   737.57), SIMDE_FLOAT32_C(  -742.92),
        SIMDE_FLOAT32_C(   876.84), SIMDE_FLOAT32_C(  -475.39), SIMDE_FLOAT32_C(   304.27), SIMDE_FLOAT32_C(  -773.43) },
      { SIMDE_FLOAT32_C(  -737.00), SIMDE_FLOAT32_C(   949.00), SIMDE_FLOAT32_C(   251.00), SIMDE_FLOAT32_C(  -322.00),
        SIMDE_FLOAT32_C(    81.00), SIMDE_FLOAT32_C(  -653.00), SIMDE_FLOAT32_C(  -364.00), SIMDE_FLOAT32_C(    38.00),
        SIMDE_FLOAT32_C(  -235.00), SIMDE_FLOAT32_C(   908.00), SIMDE_FLOAT32_C(   737.00), SIMDE_FLOAT32_C(  -742.00),
        SIMDE_FLOAT32_C(   876.00), SIMDE_FLOAT32_C(  -475.00), SIMDE_FLOAT32_C(   304.00), SIMDE_FLOAT32_C(  -773.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_trunc_ps(a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_trunc_ps (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float32 src[16];
    const simde__mmask8 k;
    const simde_float32 a[16];
    const simde_float32 r[16];
  } test_vec[] = {
    { { SIMDE_FLOAT32_C(   -49.77), SIMDE_FLOAT32_C(  -686.71), SIMDE_FLOAT32_C(  -757.78), SIMDE_FLOAT32_C(   110.09),
        SIMDE_FLOAT32_C(   324.87), SIMDE_FLOAT32_C(  -371.31), SIMDE_FLOAT32_C(   784.70), SIMDE_FLOAT32_C(   832.26),
        SIMDE_FLOAT32_C(   569.37), SIMDE_FLOAT32_C(   756.19), SIMDE_FLOAT32_C(   204.94), SIMDE_FLOAT32_C(   274.85),
        SIMDE_FLOAT32_C(  -873.98), SIMDE_FLOAT32_C(  -346.20), SIMDE_FLOAT32_C(   -78.53), SIMDE_FLOAT32_C(  -191.48) },
      UINT8_C( 44),
      { SIMDE_FLOAT32_C(   -81.77), SIMDE_FLOAT32_C(  -137.21), SIMDE_FLOAT32_C(   797.93), SIMDE_FLOAT32_C(  -424.41),
        SIMDE_FLOAT32_C(  -278.83), SIMDE_FLOAT32_C(  -767.08), SIMDE_FLOAT32_C(  -764.79), SIMDE_FLOAT32_C(    76.32),
        SIMDE_FLOAT32_C(   979.09), SIMDE_FLOAT32_C(  -188.68), SIMDE_FLOAT32_C(  -648.91), SIMDE_FLOAT32_C(    84.00),
        SIMDE_FLOAT32_C(  -272.96), SIMDE_FLOAT32_C(   552.79), SIMDE_FLOAT32_C(  -965.78), SIMDE_FLOAT32_C(    40.34) },
      { SIMDE_FLOAT32_C(   -49.77), SIMDE_FLOAT32_C(  -686.71), SIMDE_FLOAT32_C(   797.00), SIMDE_FLOAT32_C(  -424.00),
        SIMDE_FLOAT32_C(   324.87), SIMDE_FLOAT32_C(  -767.00), SIMDE_FLOAT32_C(   784.70), SIMDE_FLOAT32_C(   832.26),
        SIMDE_FLOAT32_C(   569.37), SIMDE_FLOAT32_C(   756.19), SIMDE_FLOAT32_C(   204.94), SIMDE_FLOAT32_C(   274.85),
        SIMDE_FLOAT32_C(  -873.98), SIMDE_FLOAT32_C(  -346.20), SIMDE_FLOAT32_C(   -78.53), SIMDE_FLOAT32_C(  -191.48) } },
    { { SIMDE_FLOAT32_C(   795.01), SIMDE_FLOAT32_C(   144.31), SIMDE_FLOAT32_C(  -634.80), SIMDE_FLOAT32_C(  -576.30),
        SIMDE_FLOAT32_C(   -71.00), SIMDE_FLOAT32_C(  -802.54), SIMDE_FLOAT32_C(   993.08), SIMDE_FLOAT32_C(  -314.81),
        SIMDE_FLOAT32_C(   402.40), SIMDE_FLOAT32_C(   267.93), SIMDE_FLOAT32_C(  -188.79), SIMDE_FLOAT32_C(  -943.80),
        SIMDE_FLOAT32_C(  -810.60), SIMDE_FLOAT32_C(   619.74), SIMDE_FLOAT32_C(   857.90), SIMDE_FLOAT32_C(   107.62) },
      UINT8_C(232),
      { SIMDE_FLOAT32_C(   655.83), SIMDE_FLOAT32_C(   683.21), SIMDE_FLOAT32_C(   203.69), SIMDE_FLOAT32_C(   888.75),
        SIMDE_FLOAT32_C(   918.42), SIMDE_FLOAT32_C(  -720.00), SIMDE_FLOAT32_C(   867.84), SIMDE_FLOAT32_C(  -270.26),
        SIMDE_FLOAT32_C(  -368.90), SIMDE_FLOAT32_C(   -48.16), SIMDE_FLOAT32_C(   456.78), SIMDE_FLOAT32_C(  -816.11),
        SIMDE_FLOAT32_C(   -13.93), SIMDE_FLOAT32_C(  -502.88), SIMDE_FLOAT32_C(   978.90), SIMDE_FLOAT32_C(  -869.63) },
      { SIMDE_FLOAT32_C(   795.01), SIMDE_FLOAT32_C(   144.31), SIMDE_FLOAT32_C(  -634.80), SIMDE_FLOAT32_C(   888.00),
        SIMDE_FLOAT32_C(   -71.00), SIMDE_FLOAT32_C(  -720.00), SIMDE_FLOAT32_C(   867.00), SIMDE_FLOAT32_C(  -270.00),
        SIMDE_FLOAT32_C(   402.40), SIMDE_FLOAT32_C(   267.93), SIMDE_FLOAT32_C(  -188.79), SIMDE_FLOAT32_C(  -943.80),
        SIMDE_FLOAT32_C(  -810.60), SIMDE_FLOAT32_C(   619.74), SIMDE_FLOAT32_C(   857.90), SIMDE_FLOAT32_C(   107.62) } },
    { { SIMDE_FLOAT32_C(  -137.68), SIMDE_FLOAT32_C(  -597.40), SIMDE_FLOAT32_C(    59.38), SIMDE_FLOAT32_C(    59.79),
        SIMDE_FLOAT32_C(  -604.32), SIMDE_FLOAT32_C(   744.57), SIMDE_FLOAT32_C(  -537.81), SIMDE_FLOAT32_C(   663.60),
        SIMDE_FLOAT32_C(  -444.21), SIMDE_FLOAT32_C(  -481.61), SIMDE_FLOAT32_C(   853.00), SIMDE_FLOAT32_C(  -824.48),
        SIMDE_FLOAT32_C(  -623.71), SIMDE_FLOAT32_C(   -39.38), SIMDE_FLOAT32_C(  -341.96), SIMDE_FLOAT32_C(  -967.88) },
      UINT8_C( 37),
      { SIMDE_FLOAT32_C(   861.73), SIMDE_FLOAT32_C(   920.87), SIMDE_FLOAT32_C(  -437.74), SIMDE_FLOAT32_C(  -858.26),
        SIMDE_FLOAT32_C(   788.71), SIMDE_FLOAT32_C(   291.99), SIMDE_FLOAT32_C(  -227.16), SIMDE_FLOAT32_C(  -259.44),
        SIMDE_FLOAT32_C(  -251.22), SIMDE_FLOAT32_C(   -43.28), SIMDE_FLOAT32_C(   726.62), SIMDE_FLOAT32_C(   245.90),
        SIMDE_FLOAT32_C(   -64.38), SIMDE_FLOAT32_C(   857.00), SIMDE_FLOAT32_C(  -891.78), SIMDE_FLOAT32_C(   338.22) },
      { SIMDE_FLOAT32_C(   861.00), SIMDE_FLOAT32_C(  -597.40), SIMDE_FLOAT32_C(  -437.00), SIMDE_FLOAT32_C(    59.79),
        SIMDE_FLOAT32_C(  -604.32), SIMDE_FLOAT32_C(   291.00), SIMDE_FLOAT32_C(  -537.81), SIMDE_FLOAT32_C(   663.60),
        SIMDE_FLOAT32_C(  -444.21), SIMDE_FLOAT32_C(  -481.61), SIMDE_FLOAT32_C(   853.00), SIMDE_FLOAT32_C(  -824.48),
        SIMDE_FLOAT32_C(  -623.71), SIMDE_FLOAT32_C(   -39.38), SIMDE_FLOAT32_C(  -341.96), SIMDE_FLOAT32_C(  -967.88) } },
    { { SIMDE_FLOAT32_C(   -83.63), SIMDE_FLOAT32_C(   168.01), SIMDE_FLOAT32_C(   733.90), SIMDE_FLOAT32_C(  -339.05),
        SIMDE_FLOAT32_C(   630.19), SIMDE_FLOAT32_C(   397.50), SIMDE_FLOAT32_C(   216.73), SIMDE_FLOAT32_C(  -851.42),
        SIMDE_FLOAT32_C(   250.50), SIMDE_FLOAT32_C(   392.25), SIMDE_FLOAT32_C(  -475.13), SIMDE_FLOAT32_C(  -788.88),
        SIMDE_FLOAT32_C(  -949.70), SIMDE_FLOAT32_C(  -443.01), SIMDE_FLOAT32_C(  -145.04), SIMDE_FLOAT32_C(   912.03) },
      UINT8_C(240),
      { SIMDE_FLOAT32_C(   417.21), SIMDE_FLOAT32_C(  -946.23), SIMDE_FLOAT32_C(  -733.43), SIMDE_FLOAT32_C(  -290.79),
        SIMDE_FLOAT32_C(  -173.40), SIMDE_FLOAT32_C(     7.13), SIMDE_FLOAT32_C(   457.98), SIMDE_FLOAT32_C(   783.33),
        SIMDE_FLOAT32_C(  -266.25), SIMDE_FLOAT32_C(  -296.12), SIMDE_FLOAT32_C(  -281.05), SIMDE_FLOAT32_C(  -409.26),
        SIMDE_FLOAT32_C(  -187.90), SIMDE_FLOAT32_C(  -942.83), SIMDE_FLOAT32_C(   507.12), SIMDE_FLOAT32_C(   980.11) },
      { SIMDE_FLOAT32_C(   -83.63), SIMDE_FLOAT32_C(   168.01), SIMDE_FLOAT32_C(   733.90), SIMDE_FLOAT32_C(  -339.05),
        SIMDE_FLOAT32_C(  -173.00), SIMDE_FLOAT32_C(     7.00), SIMDE_FLOAT32_C(   457.00), SIMDE_FLOAT32_C(   783.00),
        SIMDE_FLOAT32_C(   250.50), SIMDE_FLOAT32_C(   392.25), SIMDE_FLOAT32_C(  -475.13), SIMDE_FLOAT32_C(  -788.88),
        SIMDE_FLOAT32_C(  -949.70), SIMDE_FLOAT32_C(  -443.01), SIMDE_FLOAT32_C(  -145.04), SIMDE_FLOAT32_C(   912.03) } },
    { { SIMDE_FLOAT32_C(   791.07), SIMDE_FLOAT32_C(  -831.94), SIMDE_FLOAT32_C(   610.30), SIMDE_FLOAT32_C(   188.58),
        SIMDE_FLOAT32_C(   384.80), SIMDE_FLOAT32_C(   758.88), SIMDE_FLOAT32_C(  -560.92), SIMDE_FLOAT32_C(  -222.95),
        SIMDE_FLOAT32_C(  -716.25), SIMDE_FLOAT32_C(  -349.80), SIMDE_FLOAT32_C(  -172.65), SIMDE_FLOAT32_C(  -159.27),
        SIMDE_FLOAT32_C(   505.16), SIMDE_FLOAT32_C(  -260.62), SIMDE_FLOAT32_C(   318.59), SIMDE_FLOAT32_C(   -77.63) },
      UINT8_C( 96),
      { SIMDE_FLOAT32_C(   585.16), SIMDE_FLOAT32_C(   631.57), SIMDE_FLOAT32_C(   619.75), SIMDE_FLOAT32_C(  -407.71),
        SIMDE_FLOAT32_C(    89.55), SIMDE_FLOAT32_C(   403.08), SIMDE_FLOAT32_C(   326.04), SIMDE_FLOAT32_C(   793.43),
        SIMDE_FLOAT32_C(  -877.97), SIMDE_FLOAT32_C(   916.78), SIMDE_FLOAT32_C(  -394.47), SIMDE_FLOAT32_C(  -820.80),
        SIMDE_FLOAT32_C(   423.90), SIMDE_FLOAT32_C(  -414.36), SIMDE_FLOAT32_C(   970.28), SIMDE_FLOAT32_C(   591.96) },
      { SIMDE_FLOAT32_C(   791.07), SIMDE_FLOAT32_C(  -831.94), SIMDE_FLOAT32_C(   610.30), SIMDE_FLOAT32_C(   188.58),
        SIMDE_FLOAT32_C(   384.80), SIMDE_FLOAT32_C(   403.00), SIMDE_FLOAT32_C(   326.00), SIMDE_FLOAT32_C(  -222.95),
        SIMDE_FLOAT32_C(  -716.25), SIMDE_FLOAT32_C(  -349.80), SIMDE_FLOAT32_C(  -172.65), SIMDE_FLOAT32_C(  -159.27),
        SIMDE_FLOAT32_C(   505.16), SIMDE_FLOAT32_C(  -260.62), SIMDE_FLOAT32_C(   318.59), SIMDE_FLOAT32_C(   -77.63) } },
    { { SIMDE_FLOAT32_C(  -804.06), SIMDE_FLOAT32_C(   158.86), SIMDE_FLOAT32_C(   -23.24), SIMDE_FLOAT32_C(   954.82),
        SIMDE_FLOAT32_C(   597.93), SIMDE_FLOAT32_C(   753.81), SIMDE_FLOAT32_C(  -761.43), SIMDE_FLOAT32_C(  -751.86),
        SIMDE_FLOAT32_C(  -418.84), SIMDE_FLOAT32_C(    79.30), SIMDE_FLOAT32_C(   753.29), SIMDE_FLOAT32_C(   320.53),
        SIMDE_FLOAT32_C(  -602.11), SIMDE_FLOAT32_C(  -324.34), SIMDE_FLOAT32_C(  -886.32), SIMDE_FLOAT32_C(   983.05) },
      UINT8_C(109),
      { SIMDE_FLOAT32_C(   733.43), SIMDE_FLOAT32_C(  -424.66), SIMDE_FLOAT32_C(   396.78), SIMDE_FLOAT32_C(   136.51),
        SIMDE_FLOAT32_C(   901.37), SIMDE_FLOAT32_C(   190.22), SIMDE_FLOAT32_C(   258.54), SIMDE_FLOAT32_C(   818.15),
        SIMDE_FLOAT32_C(   795.75), SIMDE_FLOAT32_C(   437.74), SIMDE_FLOAT32_C(   242.05), SIMDE_FLOAT32_C(  -618.61),
        SIMDE_FLOAT32_C(   408.02), SIMDE_FLOAT32_C(  -165.99), SIMDE_FLOAT32_C(  -422.67), SIMDE_FLOAT32_C(  -433.12) },
      { SIMDE_FLOAT32_C(   733.00), SIMDE_FLOAT32_C(   158.86), SIMDE_FLOAT32_C(   396.00), SIMDE_FLOAT32_C(   136.00),
        SIMDE_FLOAT32_C(   597.93), SIMDE_FLOAT32_C(   190.00), SIMDE_FLOAT32_C(   258.00), SIMDE_FLOAT32_C(  -751.86),
        SIMDE_FLOAT32_C(  -418.84), SIMDE_FLOAT32_C(    79.30), SIMDE_FLOAT32_C(   753.29), SIMDE_FLOAT32_C(   320.53),
        SIMDE_FLOAT32_C(  -602.11), SIMDE_FLOAT32_C(  -324.34), SIMDE_FLOAT32_C(  -886.32), SIMDE_FLOAT32_C(   983.05) } },
    { { SIMDE_FLOAT32_C(   810.77), SIMDE_FLOAT32_C(  -467.85), SIMDE_FLOAT32_C(  -835.19), SIMDE_FLOAT32_C(   564.58),
        SIMDE_FLOAT32_C(  -229.28), SIMDE_FLOAT32_C(  -587.05), SIMDE_FLOAT32_C(  -854.26), SIMDE_FLOAT32_C(   850.02),
        SIMDE_FLOAT32_C(  -833.76), SIMDE_FLOAT32_C(   466.27), SIMDE_FLOAT32_C(  -752.09), SIMDE_FLOAT32_C(  -158.10),
        SIMDE_FLOAT32_C(   579.95), SIMDE_FLOAT32_C(  -769.04), SIMDE_FLOAT32_C(   149.13), SIMDE_FLOAT32_C(   313.38) },
      UINT8_C(125),
      { SIMDE_FLOAT32_C(  -454.09), SIMDE_FLOAT32_C(  -550.11), SIMDE_FLOAT32_C(  -292.33), SIMDE_FLOAT32_C(   736.13),
        SIMDE_FLOAT32_C(   708.43), SIMDE_FLOAT32_C(  -474.18), SIMDE_FLOAT32_C(   531.88), SIMDE_FLOAT32_C(   146.17),
        SIMDE_FLOAT32_C(   767.87), SIMDE_FLOAT32_C(   913.26), SIMDE_FLOAT32_C(  -445.81), SIMDE_FLOAT32_C(  -398.12),
        SIMDE_FLOAT32_C(  -509.41), SIMDE_FLOAT32_C(   121.07), SIMDE_FLOAT32_C(  -587.35), SIMDE_FLOAT32_C(    22.74) },
      { SIMDE_FLOAT32_C(  -454.00), SIMDE_FLOAT32_C(  -467.85), SIMDE_FLOAT32_C(  -292.00), SIMDE_FLOAT32_C(   736.00),
        SIMDE_FLOAT32_C(   708.00), SIMDE_FLOAT32_C(  -474.00), SIMDE_FLOAT32_C(   531.00), SIMDE_FLOAT32_C(   850.02),
        SIMDE_FLOAT32_C(  -833.76), SIMDE_FLOAT32_C(   466.27), SIMDE_FLOAT32_C(  -752.09), SIMDE_FLOAT32_C(  -158.10),
        SIMDE_FLOAT32_C(   579.95), SIMDE_FLOAT32_C(  -769.04), SIMDE_FLOAT32_C(   149.13), SIMDE_FLOAT32_C(   313.38) } },
    { { SIMDE_FLOAT32_C(   285.88), SIMDE_FLOAT32_C(   977.23), SIMDE_FLOAT32_C(   793.45), SIMDE_FLOAT32_C(   698.82),
        SIMDE_FLOAT32_C(  -877.03), SIMDE_FLOAT32_C(   643.47), SIMDE_FLOAT32_C(   865.06), SIMDE_FLOAT32_C(   589.25),
        SIMDE_FLOAT32_C(   891.38), SIMDE_FLOAT32_C(  -293.04), SIMDE_FLOAT32_C(   169.20), SIMDE_FLOAT32_C(  -877.66),
        SIMDE_FLOAT32_C(   856.08), SIMDE_FLOAT32_C(  -517.41), SIMDE_FLOAT32_C(   -71.37), SIMDE_FLOAT32_C(  -598.01) },
      UINT8_C(105),
      { SIMDE_FLOAT32_C(   636.30), SIMDE_FLOAT32_C(  -861.88), SIMDE_FLOAT32_C(  -359.09), SIMDE_FLOAT32_C(  -837.88),
        SIMDE_FLOAT32_C(   670.00), SIMDE_FLOAT32_C(   787.08), SIMDE_FLOAT32_C(   929.98), SIMDE_FLOAT32_C(   583.26),
        SIMDE_FLOAT32_C(  -658.72), SIMDE_FLOAT32_C(  -468.14), SIMDE_FLOAT32_C(  -926.15), SIMDE_FLOAT32_C(   462.35),
        SIMDE_FLOAT32_C(   -55.49), SIMDE_FLOAT32_C(    96.59), SIMDE_FLOAT32_C(  -251.77), SIMDE_FLOAT32_C(   -78.25) },
      { SIMDE_FLOAT32_C(   636.00), SIMDE_FLOAT32_C(   977.23), SIMDE_FLOAT32_C(   793.45), SIMDE_FLOAT32_C(  -837.00),
        SIMDE_FLOAT32_C(  -877.03), SIMDE_FLOAT32_C(   787.00), SIMDE_FLOAT32_C(   929.00), SIMDE_FLOAT32_C(   589.25),
        SIMDE_FLOAT32_C(   891.38), SIMDE_FLOAT32_C(  -293.04), SIMDE_FLOAT32_C(   169.20), SIMDE_FLOAT32_C(  -877.66),
        SIMDE_FLOAT32_C(   856.08), SIMDE_FLOAT32_C(  -517.41), SIMDE_FLOAT32_C(   -71.37), SIMDE_FLOAT32_C(  -598.01) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512 src = simde_mm512_loadu_ps(test_vec[i].src);
    simde__m512 a = simde_mm512_loadu_ps(test_vec[i].a);
    simde__m512 r = simde_mm512_mask_trunc_ps(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f32x16(r, simde_mm512_loadu_ps(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_trunc_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(    90.45), SIMDE_FLOAT64_C(   195.98), SIMDE_FLOAT64_C(   -83.38), SIMDE_FLOAT64_C(  -236.26),
        SIMDE_FLOAT64_C(  -941.16), SIMDE_FLOAT64_C(   125.78), SIMDE_FLOAT64_C(  -753.74), SIMDE_FLOAT64_C(  -729.24) },
      { SIMDE_FLOAT64_C(    90.00), SIMDE_FLOAT64_C(   195.00), SIMDE_FLOAT64_C(   -83.00), SIMDE_FLOAT64_C(  -236.00),
        SIMDE_FLOAT64_C(  -941.00), SIMDE_FLOAT64_C(   125.00), SIMDE_FLOAT64_C(  -753.00), SIMDE_FLOAT64_C(  -729.00) } },
    { { SIMDE_FLOAT64_C(   663.53), SIMDE_FLOAT64_C(   196.60), SIMDE_FLOAT64_C(   -90.58), SIMDE_FLOAT64_C(   229.06),
        SIMDE_FLOAT64_C(  -925.87), SIMDE_FLOAT64_C(  -621.28), SIMDE_FLOAT64_C(   631.54), SIMDE_FLOAT64_C(  -475.70) },
      { SIMDE_FLOAT64_C(   663.00), SIMDE_FLOAT64_C(   196.00), SIMDE_FLOAT64_C(   -90.00), SIMDE_FLOAT64_C(   229.00),
        SIMDE_FLOAT64_C(  -925.00), SIMDE_FLOAT64_C(  -621.00), SIMDE_FLOAT64_C(   631.00), SIMDE_FLOAT64_C(  -475.00) } },
    { { SIMDE_FLOAT64_C(   499.40), SIMDE_FLOAT64_C(  -577.93), SIMDE_FLOAT64_C(  -603.42), SIMDE_FLOAT64_C(  -226.68),
        SIMDE_FLOAT64_C(   674.64), SIMDE_FLOAT64_C(  -116.71), SIMDE_FLOAT64_C(   605.38), SIMDE_FLOAT64_C(  -749.41) },
      { SIMDE_FLOAT64_C(   499.00), SIMDE_FLOAT64_C(  -577.00), SIMDE_FLOAT64_C(  -603.00), SIMDE_FLOAT64_C(  -226.00),
        SIMDE_FLOAT64_C(   674.00), SIMDE_FLOAT64_C(  -116.00), SIMDE_FLOAT64_C(   605.00), SIMDE_FLOAT64_C(  -749.00) } },
    { { SIMDE_FLOAT64_C(  -866.90), SIMDE_FLOAT64_C(   273.08), SIMDE_FLOAT64_C(   910.37), SIMDE_FLOAT64_C(  -223.08),
        SIMDE_FLOAT64_C(   229.45), SIMDE_FLOAT64_C(  -919.92), SIMDE_FLOAT64_C(   179.63), SIMDE_FLOAT64_C(  -680.10) },
      { SIMDE_FLOAT64_C(  -866.00), SIMDE_FLOAT64_C(   273.00), SIMDE_FLOAT64_C(   910.00), SIMDE_FLOAT64_C(  -223.00),
        SIMDE_FLOAT64_C(   229.00), SIMDE_FLOAT64_C(  -919.00), SIMDE_FLOAT64_C(   179.00), SIMDE_FLOAT64_C(  -680.00) } },
    { { SIMDE_FLOAT64_C(   276.06), SIMDE_FLOAT64_C(  -903.75), SIMDE_FLOAT64_C(    83.64), SIMDE_FLOAT64_C(   334.90),
        SIMDE_FLOAT64_C(   222.03), SIMDE_FLOAT64_C(   329.90), SIMDE_FLOAT64_C(   605.67), SIMDE_FLOAT64_C(  -114.44) },
      { SIMDE_FLOAT64_C(   276.00), SIMDE_FLOAT64_C(  -903.00), SIMDE_FLOAT64_C(    83.00), SIMDE_FLOAT64_C(   334.00),
        SIMDE_FLOAT64_C(   222.00), SIMDE_FLOAT64_C(   329.00), SIMDE_FLOAT64_C(   605.00), SIMDE_FLOAT64_C(  -114.00) } },
    { { SIMDE_FLOAT64_C(  -473.49), SIMDE_FLOAT64_C(  -484.91), SIMDE_FLOAT64_C(  -885.38), SIMDE_FLOAT64_C(  -399.36),
        SIMDE_FLOAT64_C(  -106.19), SIMDE_FLOAT64_C(   746.15), SIMDE_FLOAT64_C(   124.93), SIMDE_FLOAT64_C(  -606.79) },
      { SIMDE_FLOAT64_C(  -473.00), SIMDE_FLOAT64_C(  -484.00), SIMDE_FLOAT64_C(  -885.00), SIMDE_FLOAT64_C(  -399.00),
        SIMDE_FLOAT64_C(  -106.00), SIMDE_FLOAT64_C(   746.00), SIMDE_FLOAT64_C(   124.00), SIMDE_FLOAT64_C(  -606.00) } },
    { { SIMDE_FLOAT64_C(  -831.78), SIMDE_FLOAT64_C(   521.52), SIMDE_FLOAT64_C(   166.54), SIMDE_FLOAT64_C(   842.86),
        SIMDE_FLOAT64_C(  -595.19), SIMDE_FLOAT64_C(  -228.09), SIMDE_FLOAT64_C(  -906.55), SIMDE_FLOAT64_C(  -462.09) },
      { SIMDE_FLOAT64_C(  -831.00), SIMDE_FLOAT64_C(   521.00), SIMDE_FLOAT64_C(   166.00), SIMDE_FLOAT64_C(   842.00),
        SIMDE_FLOAT64_C(  -595.00), SIMDE_FLOAT64_C(  -228.00), SIMDE_FLOAT64_C(  -906.00), SIMDE_FLOAT64_C(  -462.00) } },
    { { SIMDE_FLOAT64_C(  -955.00), SIMDE_FLOAT64_C(  -996.18), SIMDE_FLOAT64_C(   314.83), SIMDE_FLOAT64_C(   274.44),
        SIMDE_FLOAT64_C(  -916.10), SIMDE_FLOAT64_C(  -505.54), SIMDE_FLOAT64_C(   594.34), SIMDE_FLOAT64_C(   359.96) },
      { SIMDE_FLOAT64_C(  -955.00), SIMDE_FLOAT64_C(  -996.00), SIMDE_FLOAT64_C(   314.00), SIMDE_FLOAT64_C(   274.00),
        SIMDE_FLOAT64_C(  -916.00), SIMDE_FLOAT64_C(  -505.00), SIMDE_FLOAT64_C(   594.00), SIMDE_FLOAT64_C(   359.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_trunc_pd(a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_trunc_pd (SIMDE_MUNIT_TEST_ARGS) {
  static const struct {
    const simde_float64 src[8];
    const simde__mmask8 k;
    const simde_float64 a[8];
    const simde_float64 r[8];
  } test_vec[] = {
    { { SIMDE_FLOAT64_C(   818.03), SIMDE_FLOAT64_C(   444.72), SIMDE_FLOAT64_C(   916.04), SIMDE_FLOAT64_C(  -825.66),
        SIMDE_FLOAT64_C(   941.31), SIMDE_FLOAT64_C(   -37.20), SIMDE_FLOAT64_C(  -948.28), SIMDE_FLOAT64_C(  -408.19) },
      UINT8_C( 90),
      { SIMDE_FLOAT64_C(  -903.02), SIMDE_FLOAT64_C(   326.13), SIMDE_FLOAT64_C(   -77.85), SIMDE_FLOAT64_C(   808.82),
        SIMDE_FLOAT64_C(  -385.32), SIMDE_FLOAT64_C(  -921.95), SIMDE_FLOAT64_C(  -879.51), SIMDE_FLOAT64_C(   447.28) },
      { SIMDE_FLOAT64_C(   818.03), SIMDE_FLOAT64_C(   326.00), SIMDE_FLOAT64_C(   916.04), SIMDE_FLOAT64_C(   808.00),
        SIMDE_FLOAT64_C(  -385.00), SIMDE_FLOAT64_C(   -37.20), SIMDE_FLOAT64_C(  -879.00), SIMDE_FLOAT64_C(  -408.19) } },
    { { SIMDE_FLOAT64_C(  -281.72), SIMDE_FLOAT64_C(   142.99), SIMDE_FLOAT64_C(  -182.68), SIMDE_FLOAT64_C(   -63.76),
        SIMDE_FLOAT64_C(   164.70), SIMDE_FLOAT64_C(  -994.58), SIMDE_FLOAT64_C(   -84.09), SIMDE_FLOAT64_C(   455.69) },
      UINT8_C(145),
      { SIMDE_FLOAT64_C(   892.02), SIMDE_FLOAT64_C(   632.35), SIMDE_FLOAT64_C(   571.19), SIMDE_FLOAT64_C(  -642.67),
        SIMDE_FLOAT64_C(  -756.86), SIMDE_FLOAT64_C(   389.22), SIMDE_FLOAT64_C(   802.05), SIMDE_FLOAT64_C(  -840.82) },
      { SIMDE_FLOAT64_C(   892.00), SIMDE_FLOAT64_C(   142.99), SIMDE_FLOAT64_C(  -182.68), SIMDE_FLOAT64_C(   -63.76),
        SIMDE_FLOAT64_C(  -756.00), SIMDE_FLOAT64_C(  -994.58), SIMDE_FLOAT64_C(   -84.09), SIMDE_FLOAT64_C(  -840.00) } },
    { { SIMDE_FLOAT64_C(   563.57), SIMDE_FLOAT64_C(   743.36), SIMDE_FLOAT64_C(   121.98), SIMDE_FLOAT64_C(   615.28),
        SIMDE_FLOAT64_C(  -664.83), SIMDE_FLOAT64_C(   388.96), SIMDE_FLOAT64_C(   712.26), SIMDE_FLOAT64_C(   661.30) },
      UINT8_C(219),
      { SIMDE_FLOAT64_C(   521.09), SIMDE_FLOAT64_C(  -724.02), SIMDE_FLOAT64_C(  -610.84), SIMDE_FLOAT64_C(   641.58),
        SIMDE_FLOAT64_C(   723.26), SIMDE_FLOAT64_C(   107.43), SIMDE_FLOAT64_C(  -215.43), SIMDE_FLOAT64_C(  -459.42) },
      { SIMDE_FLOAT64_C(   521.00), SIMDE_FLOAT64_C(  -724.00), SIMDE_FLOAT64_C(   121.98), SIMDE_FLOAT64_C(   641.00),
        SIMDE_FLOAT64_C(   723.00), SIMDE_FLOAT64_C(   388.96), SIMDE_FLOAT64_C(  -215.00), SIMDE_FLOAT64_C(  -459.00) } },
    { { SIMDE_FLOAT64_C(  -956.33), SIMDE_FLOAT64_C(   949.27), SIMDE_FLOAT64_C(  -454.00), SIMDE_FLOAT64_C(   -40.42),
        SIMDE_FLOAT64_C(   404.97), SIMDE_FLOAT64_C(  -418.67), SIMDE_FLOAT64_C(  -148.40), SIMDE_FLOAT64_C(    37.32) },
      UINT8_C( 67),
      { SIMDE_FLOAT64_C(   208.93), SIMDE_FLOAT64_C(   280.46), SIMDE_FLOAT64_C(   541.75), SIMDE_FLOAT64_C(    10.98),
        SIMDE_FLOAT64_C(   439.64), SIMDE_FLOAT64_C(   105.31), SIMDE_FLOAT64_C(  -245.66), SIMDE_FLOAT64_C(  -438.38) },
      { SIMDE_FLOAT64_C(   208.00), SIMDE_FLOAT64_C(   280.00), SIMDE_FLOAT64_C(  -454.00), SIMDE_FLOAT64_C(   -40.42),
        SIMDE_FLOAT64_C(   404.97), SIMDE_FLOAT64_C(  -418.67), SIMDE_FLOAT64_C(  -245.00), SIMDE_FLOAT64_C(    37.32) } },
    { { SIMDE_FLOAT64_C(  -279.41), SIMDE_FLOAT64_C(    89.51), SIMDE_FLOAT64_C(   950.57), SIMDE_FLOAT64_C(  -567.14),
        SIMDE_FLOAT64_C(  -249.19), SIMDE_FLOAT64_C(  -738.32), SIMDE_FLOAT64_C(   953.94), SIMDE_FLOAT64_C(    26.79) },
      UINT8_C(166),
      { SIMDE_FLOAT64_C(   595.52), SIMDE_FLOAT64_C(  -249.94), SIMDE_FLOAT64_C(   758.28), SIMDE_FLOAT64_C(  -619.90),
        SIMDE_FLOAT64_C(   290.64), SIMDE_FLOAT64_C(   801.95), SIMDE_FLOAT64_C(  -670.63), SIMDE_FLOAT64_C(   836.64) },
      { SIMDE_FLOAT64_C(  -279.41), SIMDE_FLOAT64_C(  -249.00), SIMDE_FLOAT64_C(   758.00), SIMDE_FLOAT64_C(  -567.14),
        SIMDE_FLOAT64_C(  -249.19), SIMDE_FLOAT64_C(   801.00), SIMDE_FLOAT64_C(   953.94), SIMDE_FLOAT64_C(   836.00) } },
    { { SIMDE_FLOAT64_C(  -238.47), SIMDE_FLOAT64_C(   734.34), SIMDE_FLOAT64_C(  -582.03), SIMDE_FLOAT64_C(   613.13),
        SIMDE_FLOAT64_C(  -228.35), SIMDE_FLOAT64_C(  -429.51), SIMDE_FLOAT64_C(  -177.94), SIMDE_FLOAT64_C(  -947.89) },
      UINT8_C(123),
      { SIMDE_FLOAT64_C(   833.04), SIMDE_FLOAT64_C(   491.75), SIMDE_FLOAT64_C(   217.55), SIMDE_FLOAT64_C(  -412.62),
        SIMDE_FLOAT64_C(  -946.63), SIMDE_FLOAT64_C(   938.15), SIMDE_FLOAT64_C(   676.89), SIMDE_FLOAT64_C(  -996.06) },
      { SIMDE_FLOAT64_C(   833.00), SIMDE_FLOAT64_C(   491.00), SIMDE_FLOAT64_C(  -582.03), SIMDE_FLOAT64_C(  -412.00),
        SIMDE_FLOAT64_C(  -946.00), SIMDE_FLOAT64_C(   938.00), SIMDE_FLOAT64_C(   676.00), SIMDE_FLOAT64_C(  -947.89) } },
    { { SIMDE_FLOAT64_C(  -629.00), SIMDE_FLOAT64_C(  -572.30), SIMDE_FLOAT64_C(  -734.38), SIMDE_FLOAT64_C(  -675.05),
        SIMDE_FLOAT64_C(   454.50), SIMDE_FLOAT64_C(   -83.54), SIMDE_FLOAT64_C(   920.47), SIMDE_FLOAT64_C(  -795.45) },
      UINT8_C(  5),
      { SIMDE_FLOAT64_C(  -699.43), SIMDE_FLOAT64_C(   495.19), SIMDE_FLOAT64_C(  -523.31), SIMDE_FLOAT64_C(  -370.06),
        SIMDE_FLOAT64_C(   331.83), SIMDE_FLOAT64_C(   238.22), SIMDE_FLOAT64_C(  -635.72), SIMDE_FLOAT64_C(   749.81) },
      { SIMDE_FLOAT64_C(  -699.00), SIMDE_FLOAT64_C(  -572.30), SIMDE_FLOAT64_C(  -523.00), SIMDE_FLOAT64_C(  -675.05),
        SIMDE_FLOAT64_C(   454.50), SIMDE_FLOAT64_C(   -83.54), SIMDE_FLOAT64_C(   920.47), SIMDE_FLOAT64_C(  -795.45) } },
    { { SIMDE_FLOAT64_C(  -148.65), SIMDE_FLOAT64_C(   135.93), SIMDE_FLOAT64_C(  -679.70), SIMDE_FLOAT64_C(   673.41),
        SIMDE_FLOAT64_C(   188.04), SIMDE_FLOAT64_C(  -567.46), SIMDE_FLOAT64_C(   506.46), SIMDE_FLOAT64_C(  -320.21) },
      UINT8_C(186),
      { SIMDE_FLOAT64_C(  -906.16), SIMDE_FLOAT64_C(  -266.84), SIMDE_FLOAT64_C(   588.24), SIMDE_FLOAT64_C(   770.73),
        SIMDE_FLOAT64_C(  -262.91), SIMDE_FLOAT64_C(   959.24), SIMDE_FLOAT64_C(  -801.57), SIMDE_FLOAT64_C(     2.71) },
      { SIMDE_FLOAT64_C(  -148.65), SIMDE_FLOAT64_C(  -266.00), SIMDE_FLOAT64_C(  -679.70), SIMDE_FLOAT64_C(   770.00),
        SIMDE_FLOAT64_C(  -262.00), SIMDE_FLOAT64_C(   959.00), SIMDE_FLOAT64_C(   506.46), SIMDE_FLOAT64_C(     2.00) } }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])) ; i++) {
    simde__m512d src = simde_mm512_loadu_pd(test_vec[i].src);
    simde__m512d a = simde_mm512_loadu_pd(test_vec[i].a);
    simde__m512d r = simde_mm512_mask_trunc_pd(src, test_vec[i].k, a);
    simde_test_x86_assert_equal_f64x8(r, simde_mm512_loadu_pd(test_vec[i].r), 1);
  }

  return 0;
}

static int
test_simde_mm_udivrem_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128i a;
    simde__m128i b;
    simde__m128i rem;
    simde__m128i r;
  } test_vec[8] = {
    { simde_x_mm_set_epu32(UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C(2891870298)),
      simde_x_mm_set_epu32(UINT32_C(4025088144), UINT32_C(4117928860), UINT32_C( 377180600), UINT32_C(3776380886)),
      simde_x_mm_set_epu32(UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 150291953), UINT32_C(2891870298)),
      simde_x_mm_set_epu32(UINT32_C(         0), UINT32_C(         0), UINT32_C(         1), UINT32_C(         0)) },
    { simde_x_mm_set_epu32(UINT32_C(3920294270), UINT32_C(3054162118), UINT32_C(1568850865), UINT32_C(3151989757)),
      simde_x_mm_set_epu32(UINT32_C( 172780273), UINT32_C( 168508556), UINT32_C(3803608574), UINT32_C(4064895559)),
      simde_x_mm_set_epu32(UINT32_C( 119128264), UINT32_C(  21008110), UINT32_C(1568850865), UINT32_C(3151989757)),
      simde_x_mm_set_epu32(UINT32_C(        22), UINT32_C(        18), UINT32_C(         0), UINT32_C(         0)) },
    { simde_x_mm_set_epu32(UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(1250819173), UINT32_C(3643996043)),
      simde_x_mm_set_epu32(UINT32_C( 298065861), UINT32_C(3773381365), UINT32_C( 330694282), UINT32_C(  40997390)),
      simde_x_mm_set_epu32(UINT32_C(   2012421), UINT32_C( 298608154), UINT32_C( 258736327), UINT32_C(  36225723)),
      simde_x_mm_set_epu32(UINT32_C(         5), UINT32_C(         0), UINT32_C(         3), UINT32_C(        88)) },
    { simde_x_mm_set_epu32(UINT32_C(2708640028), UINT32_C(1691051285), UINT32_C(  50347892), UINT32_C( 728425428)),
      simde_x_mm_set_epu32(UINT32_C(3853764578), UINT32_C( 294920921), UINT32_C(3883385645), UINT32_C(4126975473)),
      simde_x_mm_set_epu32(UINT32_C(2708640028), UINT32_C( 216446680), UINT32_C(  50347892), UINT32_C( 728425428)),
      simde_x_mm_set_epu32(UINT32_C(         0), UINT32_C(         5), UINT32_C(         0), UINT32_C(         0)) },
    { simde_x_mm_set_epu32(UINT32_C( 492373082), UINT32_C(4281870485), UINT32_C(2207786213), UINT32_C(3953959418)),
      simde_x_mm_set_epu32(UINT32_C( 123290430), UINT32_C(3996188341), UINT32_C( 223555334), UINT32_C(3962352253)),
      simde_x_mm_set_epu32(UINT32_C( 122501792), UINT32_C( 285682144), UINT32_C( 195788207), UINT32_C(3953959418)),
      simde_x_mm_set_epu32(UINT32_C(         3), UINT32_C(         1), UINT32_C(         9), UINT32_C(         0)) },
    { simde_x_mm_set_epu32(UINT32_C(3290702646), UINT32_C(1580565751), UINT32_C(3823902839), UINT32_C(2081361826)),
      simde_x_mm_set_epu32(UINT32_C( 328620632), UINT32_C(3970654641), UINT32_C(4110215287), UINT32_C(3940207296)),
      simde_x_mm_set_epu32(UINT32_C(   4496326), UINT32_C(1580565751), UINT32_C(3823902839), UINT32_C(2081361826)),
      simde_x_mm_set_epu32(UINT32_C(        10), UINT32_C(         0), UINT32_C(         0), UINT32_C(         0)) },
    { simde_x_mm_set_epu32(UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358), UINT32_C(3003933707)),
      simde_x_mm_set_epu32(UINT32_C( 427537184), UINT32_C( 493530770), UINT32_C(3938875497), UINT32_C(  29647056)),
      simde_x_mm_set_epu32(UINT32_C( 114516008), UINT32_C(   6332779), UINT32_C( 957375358), UINT32_C(   9581051)),
      simde_x_mm_set_epu32(UINT32_C(         1), UINT32_C(         1), UINT32_C(         0), UINT32_C(       101)) },
    { simde_x_mm_set_epu32(UINT32_C(4101755863), UINT32_C(3436978124), UINT32_C(3846637996), UINT32_C(2693603084)),
      simde_x_mm_set_epu32(UINT32_C(4010243988), UINT32_C(4123176886), UINT32_C( 457043765), UINT32_C(4197612290)),
      simde_x_mm_set_epu32(UINT32_C(  91511875), UINT32_C(3436978124), UINT32_C( 190287876), UINT32_C(2693603084)),
      simde_x_mm_set_epu32(UINT32_C(         1), UINT32_C(         0), UINT32_C(         8), UINT32_C(         0)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128i rem;
    simde__m128i r = simde_mm_udivrem_epi32(&rem, test_vec[i].a, test_vec[i].b);
    simde_assert_m128i_u32(r, ==, test_vec[i].r);
    simde_assert_m128i_u32(rem, ==, test_vec[i].rem);
  }

  return 0;
}

static int
test_simde_mm_tanh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128 a;
    simde__m128 r;
  } test_vec[8] = {
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     0.34)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(     0.03)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.44)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.52)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.60)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.73)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.58)) },
    { simde_mm_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69)),
      simde_mm_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.60)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128 r = simde_mm_tanh_ps(test_vec[i].a);
    simde_assert_m128_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm_tanh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m128d a;
    simde__m128d r;
  } test_vec[8] = {
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.64), SIMDE_FLOAT64_C(    0.34)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.29), SIMDE_FLOAT64_C(    0.03)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.46), SIMDE_FLOAT64_C(    0.58)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.26), SIMDE_FLOAT64_C(    0.44)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(    0.40)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.52)) },
    { simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69)),
      simde_mm_set_pd(SIMDE_FLOAT64_C(   -0.59), SIMDE_FLOAT64_C(   -0.60)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m128d r = simde_mm_tanh_pd(test_vec[i].a);
    simde_assert_m128d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_tanh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256 a;
    simde__m256 r;
  } test_vec[8] = {
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.58),
                         SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04),
                         SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     0.34)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69),
                         SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42),
                         SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(    -0.60),
                         SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.52),
                         SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.44)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86),
                         SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.73),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.70),
                         SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.60)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45),
                         SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21),
                         SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.21),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.58)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.44),
                         SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.38),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.99),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.84)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.41),
                         SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.36),
                         SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.76),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.69)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.40),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.03)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(     0.38),
                         SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.25),
                         SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -0.03)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.94),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.40)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.74),
                         SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.65),
                         SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.38)) },
    { simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.25)),
      simde_mm256_set_ps(SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.54),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.24)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256 r = simde_mm256_tanh_ps(test_vec[i].a);
    simde_assert_m256_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_tanh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256d a;
    simde__m256d r;
  } test_vec[8] = {
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.64), SIMDE_FLOAT64_C(    0.34)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.46), SIMDE_FLOAT64_C(    0.58),
                         SIMDE_FLOAT64_C(   -0.29), SIMDE_FLOAT64_C(    0.03)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(   -0.26), SIMDE_FLOAT64_C(    0.44)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.59), SIMDE_FLOAT64_C(   -0.60),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.52)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.86),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    0.70)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(   -0.70),
                         SIMDE_FLOAT64_C(   -0.40), SIMDE_FLOAT64_C(    0.60)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.38), SIMDE_FLOAT64_C(   -0.92)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(   -0.41), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.36), SIMDE_FLOAT64_C(   -0.73)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(   -0.66)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.25), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.58)) },
    { simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.69)),
      simde_mm256_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.42),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.60)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256d r = simde_mm256_tanh_pd(test_vec[i].a);
    simde_assert_m256d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_tanh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.57),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.35)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.52),
                         SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.44),
                         SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(     0.03),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     0.34)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.69),
                         SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.98), SIMDE_FLOAT32_C(    -0.66),
                         SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.92),
                         SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.70)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.58),
                         SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.73),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(    -0.40), SIMDE_FLOAT32_C(     0.60)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.26), SIMDE_FLOAT32_C(     0.78), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.38),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.99), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.84)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.37), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(     0.65), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(     0.36),
                         SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.69)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.25),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.94),
                         SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.55), SIMDE_FLOAT32_C(     0.40)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.59), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.24),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.74),
                         SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(    -0.50), SIMDE_FLOAT32_C(     0.38)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.34),
                         SIMDE_FLOAT32_C(    -0.87), SIMDE_FLOAT32_C(    -0.80), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.53),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.54), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.17)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(    -0.66), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -0.49),
                         SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.58),
                         SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.49), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.17)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.59),
                         SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.62),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(    -0.91),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.74)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.53),
                         SIMDE_FLOAT32_C(    -0.42), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.55),
                         SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(     0.75), SIMDE_FLOAT32_C(    -0.72),
                         SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(    -0.63)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(    -0.34), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(     0.98),
                         SIMDE_FLOAT32_C(    -0.76), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.10)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.65), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(    -0.76),
                         SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.28), SIMDE_FLOAT32_C(    -0.65),
                         SIMDE_FLOAT32_C(    -0.52), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.75),
                         SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.10)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.13),
                         SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.70)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.64), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(    -0.13),
                         SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(     0.37),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.60)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_tanh_ps(test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_tanh_ps(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512 src;
    simde__mmask16 k;
    simde__m512 a;
    simde__m512 r;
  } test_vec[8] = {
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.45), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.66),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.70),
                         SIMDE_FLOAT32_C(    -0.69), SIMDE_FLOAT32_C(     0.57), SIMDE_FLOAT32_C(     0.42), SIMDE_FLOAT32_C(     0.47),
                         SIMDE_FLOAT32_C(     0.67), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.35)),
      UINT16_C(41466),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.23), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(    -0.98),
                         SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.38), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.27),
                         SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(    -0.75)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.18), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.66),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(    -0.40),
                         SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.08), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.35)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.47), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(    -0.55),
                         SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.03)),
      UINT16_C(36797),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.60),
                         SIMDE_FLOAT32_C(     0.25), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.40), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.26),
                         SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.99)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.17), SIMDE_FLOAT32_C(    -0.15), SIMDE_FLOAT32_C(     0.91), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.65),
                         SIMDE_FLOAT32_C(     0.38), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.25),
                         SIMDE_FLOAT32_C(    -0.03), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.76)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.84), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(     0.10),
                         SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.34), SIMDE_FLOAT32_C(    -0.80),
                         SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.54)),
      UINT16_C(16804),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.69), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.45),
                         SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(     0.98), SIMDE_FLOAT32_C(     0.33),
                         SIMDE_FLOAT32_C(     0.46), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.32), SIMDE_FLOAT32_C(    -0.87),
                         SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(    -0.07)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.10), SIMDE_FLOAT32_C(     0.60), SIMDE_FLOAT32_C(    -0.59), SIMDE_FLOAT32_C(     0.73),
                         SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.91), SIMDE_FLOAT32_C(     0.32),
                         SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.76), SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.80),
                         SIMDE_FLOAT32_C(    -0.53), SIMDE_FLOAT32_C(    -0.19), SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.54)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(    -0.33), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.57), SIMDE_FLOAT32_C(    -0.34),
                         SIMDE_FLOAT32_C(     0.29), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(     0.71), SIMDE_FLOAT32_C(    -0.76)),
      UINT16_C( 2107),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.90), SIMDE_FLOAT32_C(    -0.20), SIMDE_FLOAT32_C(    -0.36), SIMDE_FLOAT32_C(    -0.03),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.13), SIMDE_FLOAT32_C(     0.04), SIMDE_FLOAT32_C(     0.39),
                         SIMDE_FLOAT32_C(    -0.30), SIMDE_FLOAT32_C(    -0.70), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(    -1.00),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.98)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.35), SIMDE_FLOAT32_C(    -0.44), SIMDE_FLOAT32_C(    -0.75), SIMDE_FLOAT32_C(     0.93),
                         SIMDE_FLOAT32_C(     0.01), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(     0.51), SIMDE_FLOAT32_C(     0.01),
                         SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(    -0.76),
                         SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.58), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(     0.75)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.18),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.89), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(     0.13),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.78), SIMDE_FLOAT32_C(     0.44)),
      UINT16_C(22274),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.50), SIMDE_FLOAT32_C(     0.92), SIMDE_FLOAT32_C(    -0.72), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(    -0.86), SIMDE_FLOAT32_C(     0.43), SIMDE_FLOAT32_C(     0.93), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.24), SIMDE_FLOAT32_C(    -0.38),
                         SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.62), SIMDE_FLOAT32_C(    -0.94), SIMDE_FLOAT32_C(     0.48)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(     0.35), SIMDE_FLOAT32_C(     0.41), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(     0.11),
                         SIMDE_FLOAT32_C(     0.48), SIMDE_FLOAT32_C(    -0.60), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.77),
                         SIMDE_FLOAT32_C(     0.22), SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(     0.44)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -0.81), SIMDE_FLOAT32_C(    -0.07), SIMDE_FLOAT32_C(    -0.78),
                         SIMDE_FLOAT32_C(     0.09), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.07),
                         SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(     0.56), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.43)),
      UINT16_C(27396),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.96), SIMDE_FLOAT32_C(    -0.41), SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.76),
                         SIMDE_FLOAT32_C(     0.79), SIMDE_FLOAT32_C(     0.00), SIMDE_FLOAT32_C(    -0.82), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(     0.58), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(    -0.72),
                         SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.27), SIMDE_FLOAT32_C(    -0.92), SIMDE_FLOAT32_C(    -0.49)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.88), SIMDE_FLOAT32_C(    -0.39), SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(    -0.78),
                         SIMDE_FLOAT32_C(     0.66), SIMDE_FLOAT32_C(     0.21), SIMDE_FLOAT32_C(    -0.68), SIMDE_FLOAT32_C(     0.16),
                         SIMDE_FLOAT32_C(    -0.29), SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.32), SIMDE_FLOAT32_C(     0.78),
                         SIMDE_FLOAT32_C(    -0.63), SIMDE_FLOAT32_C(     0.26), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.43)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.64),
                         SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(     0.03), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(    -0.08), SIMDE_FLOAT32_C(     0.08),
                         SIMDE_FLOAT32_C(    -0.71), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(    -0.89)),
      UINT16_C(  953),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.02), SIMDE_FLOAT32_C(     0.81), SIMDE_FLOAT32_C(     0.14), SIMDE_FLOAT32_C(    -0.43),
                         SIMDE_FLOAT32_C(     0.31), SIMDE_FLOAT32_C(    -0.18), SIMDE_FLOAT32_C(    -0.46), SIMDE_FLOAT32_C(     0.68),
                         SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(    -0.27), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.58),
                         SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(    -0.25), SIMDE_FLOAT32_C(    -0.05), SIMDE_FLOAT32_C(     0.09)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(     0.53), SIMDE_FLOAT32_C(    -0.02), SIMDE_FLOAT32_C(    -0.97), SIMDE_FLOAT32_C(     0.64),
                         SIMDE_FLOAT32_C(     0.45), SIMDE_FLOAT32_C(    -0.77), SIMDE_FLOAT32_C(    -0.43), SIMDE_FLOAT32_C(     0.59),
                         SIMDE_FLOAT32_C(     0.07), SIMDE_FLOAT32_C(     0.64), SIMDE_FLOAT32_C(     0.12), SIMDE_FLOAT32_C(    -0.52),
                         SIMDE_FLOAT32_C(    -0.04), SIMDE_FLOAT32_C(     0.61), SIMDE_FLOAT32_C(     0.39), SIMDE_FLOAT32_C(     0.09)) },
    { simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(    -0.49), SIMDE_FLOAT32_C(     0.82),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.82),
                         SIMDE_FLOAT32_C(    -0.21), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(    -0.73), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(    -0.20)),
      UINT16_C(12713),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.84), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.82), SIMDE_FLOAT32_C(     0.79),
                         SIMDE_FLOAT32_C(    -0.74), SIMDE_FLOAT32_C(    -0.31), SIMDE_FLOAT32_C(     0.73), SIMDE_FLOAT32_C(    -0.35),
                         SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(     0.11), SIMDE_FLOAT32_C(     0.72), SIMDE_FLOAT32_C(    -0.25),
                         SIMDE_FLOAT32_C(     0.94), SIMDE_FLOAT32_C(     0.36), SIMDE_FLOAT32_C(    -0.01), SIMDE_FLOAT32_C(     0.85)),
      simde_mm512_set_ps(SIMDE_FLOAT32_C(    -0.79), SIMDE_FLOAT32_C(     0.33), SIMDE_FLOAT32_C(     0.68), SIMDE_FLOAT32_C(     0.66),
                         SIMDE_FLOAT32_C(     0.96), SIMDE_FLOAT32_C(     0.95), SIMDE_FLOAT32_C(     0.83), SIMDE_FLOAT32_C(    -0.34),
                         SIMDE_FLOAT32_C(     0.06), SIMDE_FLOAT32_C(    -0.93), SIMDE_FLOAT32_C(     0.62), SIMDE_FLOAT32_C(    -0.42),
                         SIMDE_FLOAT32_C(     0.74), SIMDE_FLOAT32_C(     0.10), SIMDE_FLOAT32_C(     0.44), SIMDE_FLOAT32_C(     0.69)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512 r = simde_mm512_mask_tanh_ps(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_tanh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(    0.67),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.35)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.46), SIMDE_FLOAT64_C(    0.58),
                         SIMDE_FLOAT64_C(   -0.29), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.04),
                         SIMDE_FLOAT64_C(   -0.64), SIMDE_FLOAT64_C(    0.34)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(   -0.69),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.27), SIMDE_FLOAT64_C(    0.47)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.59), SIMDE_FLOAT64_C(   -0.60),
                         SIMDE_FLOAT64_C(    0.08), SIMDE_FLOAT64_C(    0.52),
                         SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(   -0.26), SIMDE_FLOAT64_C(    0.44)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.38), SIMDE_FLOAT64_C(   -0.92),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.86),
                         SIMDE_FLOAT64_C(   -0.42), SIMDE_FLOAT64_C(    0.70)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.41), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.36), SIMDE_FLOAT64_C(   -0.73),
                         SIMDE_FLOAT64_C(   -0.30), SIMDE_FLOAT64_C(   -0.70),
                         SIMDE_FLOAT64_C(   -0.40), SIMDE_FLOAT64_C(    0.60)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.69),
                         SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.98), SIMDE_FLOAT64_C(   -0.66)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(   -0.42),
                         SIMDE_FLOAT64_C(    0.23), SIMDE_FLOAT64_C(    0.60),
                         SIMDE_FLOAT64_C(    0.25), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.58)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(    0.38),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(    0.99),
                         SIMDE_FLOAT64_C(    0.03), SIMDE_FLOAT64_C(    0.84)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.65), SIMDE_FLOAT64_C(    0.41),
                         SIMDE_FLOAT64_C(   -0.52), SIMDE_FLOAT64_C(    0.36),
                         SIMDE_FLOAT64_C(   -0.65), SIMDE_FLOAT64_C(    0.76),
                         SIMDE_FLOAT64_C(    0.03), SIMDE_FLOAT64_C(    0.69)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.39), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.34),
                         SIMDE_FLOAT64_C(    0.53), SIMDE_FLOAT64_C(   -0.26),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(   -0.03)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.37), SIMDE_FLOAT64_C(    0.38),
                         SIMDE_FLOAT64_C(    0.58), SIMDE_FLOAT64_C(    0.33),
                         SIMDE_FLOAT64_C(    0.49), SIMDE_FLOAT64_C(   -0.25),
                         SIMDE_FLOAT64_C(    0.65), SIMDE_FLOAT64_C(   -0.03)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.20), SIMDE_FLOAT64_C(   -0.08),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.94),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C(    0.40)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.20), SIMDE_FLOAT64_C(   -0.08),
                         SIMDE_FLOAT64_C(    0.33), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(   -0.64), SIMDE_FLOAT64_C(   -0.65),
                         SIMDE_FLOAT64_C(   -0.50), SIMDE_FLOAT64_C(    0.38)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.47), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(    0.91), SIMDE_FLOAT64_C(    0.60),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(    0.25)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.44), SIMDE_FLOAT64_C(    0.59),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C(    0.72), SIMDE_FLOAT64_C(    0.54),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.24)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_tanh_pd(test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm512_mask_tanh_pd(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m512d src;
    simde__mmask8 k;
    simde__m512d a;
    simde__m512d r;
  } test_vec[8] = {
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.69), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.42), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(    0.67), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(    0.04), SIMDE_FLOAT64_C(    0.35)),
      UINT8_C(139),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.68), SIMDE_FLOAT64_C(    0.08),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(   -0.27),
                         SIMDE_FLOAT64_C(    0.50), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(   -0.75)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.59), SIMDE_FLOAT64_C(    0.57),
                         SIMDE_FLOAT64_C(    0.42), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(    0.46), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(   -0.64)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.18), SIMDE_FLOAT64_C(    0.23),
                         SIMDE_FLOAT64_C(    0.26), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(   -0.38),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.42)),
      UINT8_C(229),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.84), SIMDE_FLOAT64_C(   -0.45),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.21),
                         SIMDE_FLOAT64_C(   -0.66), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.92), SIMDE_FLOAT64_C(   -0.86)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.42),
                         SIMDE_FLOAT64_C(    0.60), SIMDE_FLOAT64_C(   -0.98),
                         SIMDE_FLOAT64_C(   -0.44), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(   -0.31), SIMDE_FLOAT64_C(   -0.70)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.40), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.26),
                         SIMDE_FLOAT64_C(   -0.03), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(    0.99)),
      UINT8_C(253),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.55), SIMDE_FLOAT64_C(   -0.39),
                         SIMDE_FLOAT64_C(    0.66), SIMDE_FLOAT64_C(    0.53),
                         SIMDE_FLOAT64_C(    0.78), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.58), SIMDE_FLOAT64_C(   -0.77)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.50), SIMDE_FLOAT64_C(   -0.37),
                         SIMDE_FLOAT64_C(    0.58), SIMDE_FLOAT64_C(    0.49),
                         SIMDE_FLOAT64_C(    0.65), SIMDE_FLOAT64_C(   -0.65),
                         SIMDE_FLOAT64_C(    0.38), SIMDE_FLOAT64_C(   -0.65)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.12), SIMDE_FLOAT64_C(    0.47),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.91),
                         SIMDE_FLOAT64_C(    0.79), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.75)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.54), SIMDE_FLOAT64_C(   -0.17),
                         SIMDE_FLOAT64_C(    0.68), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(    0.60), SIMDE_FLOAT64_C(    0.25),
                         SIMDE_FLOAT64_C(   -0.08), SIMDE_FLOAT64_C(   -0.94)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.12), SIMDE_FLOAT64_C(   -0.17),
                         SIMDE_FLOAT64_C(   -0.15), SIMDE_FLOAT64_C(    0.68),
                         SIMDE_FLOAT64_C(    0.54), SIMDE_FLOAT64_C(    0.24),
                         SIMDE_FLOAT64_C(    0.34), SIMDE_FLOAT64_C(   -0.74)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.10), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.34),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.53),
                         SIMDE_FLOAT64_C(   -0.82), SIMDE_FLOAT64_C(    0.66)),
      UINT8_C(145),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.33), SIMDE_FLOAT64_C(    0.46),
                         SIMDE_FLOAT64_C(   -0.18), SIMDE_FLOAT64_C(    0.32),
                         SIMDE_FLOAT64_C(   -0.87), SIMDE_FLOAT64_C(   -0.33),
                         SIMDE_FLOAT64_C(   -0.19), SIMDE_FLOAT64_C(    0.56)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.32), SIMDE_FLOAT64_C(   -0.74),
                         SIMDE_FLOAT64_C(    0.76), SIMDE_FLOAT64_C(    0.31),
                         SIMDE_FLOAT64_C(   -0.80), SIMDE_FLOAT64_C(   -0.53),
                         SIMDE_FLOAT64_C(   -0.82), SIMDE_FLOAT64_C(    0.51)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.03),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.02),
                         SIMDE_FLOAT64_C(   -0.45), SIMDE_FLOAT64_C(    0.51),
                         SIMDE_FLOAT64_C(    0.83), SIMDE_FLOAT64_C(    0.98)),
      UINT8_C( 75),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.98), SIMDE_FLOAT64_C(    0.42),
                         SIMDE_FLOAT64_C(   -0.10), SIMDE_FLOAT64_C(    0.84),
                         SIMDE_FLOAT64_C(   -0.59), SIMDE_FLOAT64_C(    0.73),
                         SIMDE_FLOAT64_C(    0.62), SIMDE_FLOAT64_C(    0.14)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.76), SIMDE_FLOAT64_C(    0.40),
                         SIMDE_FLOAT64_C(    0.69), SIMDE_FLOAT64_C(   -0.02),
                         SIMDE_FLOAT64_C(   -0.53), SIMDE_FLOAT64_C(    0.51),
                         SIMDE_FLOAT64_C(    0.55), SIMDE_FLOAT64_C(    0.14)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(   -0.30),
                         SIMDE_FLOAT64_C(   -0.70), SIMDE_FLOAT64_C(    0.82),
                         SIMDE_FLOAT64_C(   -1.00), SIMDE_FLOAT64_C(    0.92),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(   -0.07)),
      UINT8_C( 93),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.51), SIMDE_FLOAT64_C(    0.01),
                         SIMDE_FLOAT64_C(    0.92), SIMDE_FLOAT64_C(   -0.77),
                         SIMDE_FLOAT64_C(   -0.57), SIMDE_FLOAT64_C(   -0.34),
                         SIMDE_FLOAT64_C(    0.29), SIMDE_FLOAT64_C(   -0.58)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.39), SIMDE_FLOAT64_C(    0.01),
                         SIMDE_FLOAT64_C(   -0.70), SIMDE_FLOAT64_C(   -0.65),
                         SIMDE_FLOAT64_C(   -0.52), SIMDE_FLOAT64_C(   -0.33),
                         SIMDE_FLOAT64_C(   -0.77), SIMDE_FLOAT64_C(   -0.52)) },
    { simde_mm512_set_pd(SIMDE_FLOAT64_C(    0.48), SIMDE_FLOAT64_C(    0.94),
                         SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(   -0.44),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(    0.93),
                         SIMDE_FLOAT64_C(   -0.33), SIMDE_FLOAT64_C(   -0.18)),
      UINT8_C(213),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.78), SIMDE_FLOAT64_C(    0.44),
                         SIMDE_FLOAT64_C(    0.90), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(   -0.36), SIMDE_FLOAT64_C(   -0.03),
                         SIMDE_FLOAT64_C(    0.01), SIMDE_FLOAT64_C(   -0.13)),
      simde_mm512_set_pd(SIMDE_FLOAT64_C(   -0.65), SIMDE_FLOAT64_C(    0.41),
                         SIMDE_FLOAT64_C(   -0.35), SIMDE_FLOAT64_C(   -0.20),
                         SIMDE_FLOAT64_C(   -0.75), SIMDE_FLOAT64_C(   -0.03),
                         SIMDE_FLOAT64_C(   -0.33), SIMDE_FLOAT64_C(   -0.13)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m512d r = simde_mm512_mask_tanh_pd(test_vec[i].src, test_vec[i].k, test_vec[i].a);
    simde_assert_m512d_close(r, test_vec[i].r, 1);
  }

  return 0;
}

static int
test_simde_mm256_udivrem_epi32(SIMDE_MUNIT_TEST_ARGS) {
  const struct {
    simde__m256i a;
    simde__m256i b;
    simde__m256i rem;
    simde__m256i r;
  } test_vec[8] = {
    { simde_x_mm256_set_epu32(UINT32_C(3215450688), UINT32_C(3586813553), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 527472553), UINT32_C(2891870298)),
      simde_x_mm256_set_epu32(UINT32_C( 172780273), UINT32_C( 168508556), UINT32_C(3803608574), UINT32_C(4064895559),
                              UINT32_C(4201299039), UINT32_C(3984766001), UINT32_C( 392212716), UINT32_C(4009222911)),
      simde_x_mm256_set_epu32(UINT32_C( 105405774), UINT32_C(  48133877), UINT32_C(1508722402), UINT32_C(2220621656),
                              UINT32_C(1747596798), UINT32_C(2231263307), UINT32_C( 135259837), UINT32_C(2891870298)),
      simde_mm256_set_epi32(INT32_C(         18), INT32_C(         21), INT32_C(          0), INT32_C(          0),
                            INT32_C(          0), INT32_C(          0), INT32_C(          1), INT32_C(          0)) },
    { simde_x_mm256_set_epu32(UINT32_C(1192263444), UINT32_C(2208623573), UINT32_C(1322777130), UINT32_C( 163989560),
                              UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(1250819173), UINT32_C(3643996043)),
      simde_x_mm256_set_epu32(UINT32_C(3853764578), UINT32_C( 294920921), UINT32_C(3883385645), UINT32_C(4126975473),
                              UINT32_C(3898385479), UINT32_C( 422762821), UINT32_C(  12586973), UINT32_C( 182106357)),
      simde_x_mm256_set_epu32(UINT32_C(1192263444), UINT32_C( 144177126), UINT32_C(1322777130), UINT32_C( 163989560),
                              UINT32_C(1492341726), UINT32_C( 298608154), UINT32_C(   4708846), UINT32_C(   1868903)),
      simde_mm256_set_epi32(INT32_C(          0), INT32_C(          7), INT32_C(          0), INT32_C(          0),
                            INT32_C(          0), INT32_C(          0), INT32_C(         99), INT32_C(         20)) },
    { simde_x_mm256_set_epu32(UINT32_C( 493161721), UINT32_C(3099851477), UINT32_C( 894221337), UINT32_C(2964507124),
                              UINT32_C( 492373082), UINT32_C(4281870485), UINT32_C(2207786213), UINT32_C(3953959418)),
      simde_x_mm256_set_epu32(UINT32_C( 328620632), UINT32_C(3970654641), UINT32_C(4110215287), UINT32_C(3940207296),
                              UINT32_C(4043901133), UINT32_C( 395141437), UINT32_C(4177201181), UINT32_C( 520340456)),
      simde_x_mm256_set_epu32(UINT32_C( 164541089), UINT32_C(3099851477), UINT32_C( 894221337), UINT32_C(2964507124),
                              UINT32_C( 492373082), UINT32_C( 330456115), UINT32_C(2207786213), UINT32_C( 311576226)),
      simde_mm256_set_epi32(INT32_C(          1), INT32_C(          0), INT32_C(          0), INT32_C(          0),
                            INT32_C(          0), INT32_C(         10), INT32_C(          0), INT32_C(          7)) },
    { simde_x_mm256_set_epu32(UINT32_C(1710148738), UINT32_C(1974123080), UINT32_C(2870600100), UINT32_C( 118588227),
                              UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358), UINT32_C(3003933707)),
      simde_x_mm256_set_epu32(UINT32_C(4010243988), UINT32_C(4123176886), UINT32_C( 457043765), UINT32_C(4197612290),
                              UINT32_C(4246664437), UINT32_C(4080470003), UINT32_C(4182884971), UINT32_C(3894626243)),
      simde_x_mm256_set_epu32(UINT32_C(1710148738), UINT32_C(1974123080), UINT32_C( 128337510), UINT32_C( 118588227),
                              UINT32_C( 542053192), UINT32_C( 499863549), UINT32_C( 957375358), UINT32_C(3003933707)),
      simde_mm256_set_epi32(INT32_C(          0), INT32_C(          0), INT32_C(          6), INT32_C(          0),
                            INT32_C(          0), INT32_C(          0), INT32_C(          0), INT32_C(          0)) },
    { simde_x_mm256_set_epu32(UINT32_C(1734496959), UINT32_C( 380846712), UINT32_C(3352999607), UINT32_C(3555523675),
                              UINT32_C(1995198557), UINT32_C(3314312199), UINT32_C(2406584253), UINT32_C(1779168063)),
      simde_x_mm256_set_epu32(UINT32_C( 440775120), UINT32_C(4165466156), UINT32_C(3932377571), UINT32_C(3942500746),
                              UINT32_C(  67477586), UINT32_C( 108492873), UINT32_C( 360489056), UINT32_C( 254567893)),
      simde_x_mm256_set_epu32(UINT32_C( 412171599), UINT32_C( 380846712), UINT32_C(3352999607), UINT32_C(3555523675),
                              UINT32_C(  38348563), UINT32_C(  59526009), UINT32_C( 243649917), UINT32_C( 251760705)),
      simde_mm256_set_epi32(INT32_C(          3), INT32_C(          0), INT32_C(          0), INT32_C(          0),
                            INT32_C(         29), INT32_C(         30), INT32_C(          6), INT32_C(          6)) },
    { simde_x_mm256_set_epu32(UINT32_C(3932090380), UINT32_C(2449576763), UINT32_C(4246346280), UINT32_C( 201516689),
                              UINT32_C(2859036576), UINT32_C(2362091228), UINT32_C(3141663427), UINT32_C( 562234020)),
      simde_x_mm256_set_epu32(UINT32_C(4128600985), UINT32_C(4209418337), UINT32_C( 525546139), UINT32_C( 219277873),
                              UINT32_C( 295872976), UINT32_C(4150814551), UINT32_C(4029638246), UINT32_C(4092942946)),
      simde_x_mm256_set_epu32(UINT32_C(3932090380), UINT32_C(2449576763), UINT32_C(  41977168), UINT32_C( 201516689),
                              UINT32_C( 196179792), UINT32_C(2362091228), UINT32_C(3141663427), UINT32_C( 562234020)),
      simde_mm256_set_epi32(INT32_C(          0), INT32_C(          0), INT32_C(          8), INT32_C(          0),
                            INT32_C(          9), INT32_C(          0), INT32_C(          0), INT32_C(          0)) },
    { simde_x_mm256_set_epu32(UINT32_C( 910061584), UINT32_C(2002226944), UINT32_C(3673004107), UINT32_C(4246624078),
                              UINT32_C( 523093293), UINT32_C(3059761572), UINT32_C(2206005509), UINT32_C(1943141679)),
      simde_x_mm256_set_epu32(UINT32_C( 123967721), UINT32_C(4199435689), UINT32_C( 228811177), UINT32_C(   1270356),
                              UINT32_C( 355625346), UINT32_C(4253972365), UINT32_C(3915742229), UINT32_C( 124491394)),
      simde_x_mm256_set_epu32(UINT32_C(  42287537), UINT32_C(2002226944), UINT32_C(  12025275), UINT32_C(   1094326),
                              UINT32_C( 167467947), UINT32_C(3059761572), UINT32_C(2206005509), UINT32_C(  75770769)),
      simde_mm256_set_epi32(INT32_C(          7), INT32_C(          0), INT32_C(         16), INT32_C(       3342),
                            INT32_C(          1), INT32_C(          0), INT32_C(          0), INT32_C(         15)) },
    { simde_x_mm256_set_epu32(UINT32_C(1755684145), UINT32_C(2233240925), UINT32_C(3244523643), UINT32_C(2995026741),
                              UINT32_C(2178270751), UINT32_C(1493088054), UINT32_C(4115137419), UINT32_C( 651362699)),
      simde_x_mm256_set_epu32(UINT32_C( 301617823), UINT32_C( 343728879), UINT32_C( 132913279), UINT32_C( 518796827),
                              UINT32_C(4258812658), UINT32_C(3762000867), UINT32_C( 361195763), UINT32_C( 469656308)),
      simde_x_mm256_set_epu32(UINT32_C( 247595030), UINT32_C( 170867651), UINT32_C(  54604947), UINT32_C( 401042606),
                              UINT32_C(2178270751), UINT32_C(1493088054), UINT32_C( 141984026), UINT32_C( 181706391)),
      simde_mm256_set_epi32(INT32_C(          5), INT32_C(          6), INT32_C(         24), INT32_C(          5),
                            INT32_C(          0), INT32_C(          0), INT32_C(         11), INT32_C(          1)) }
  };

  for (size_t i = 0 ; i < (sizeof(test_vec) / sizeof(test_vec[0])); i++) {
    simde__m256i rem;
    simde__m256i r = simde_mm256_udivrem_epi32(&rem, test_vec[i].a, test_vec[i].b);
    simde_assert_m256i_u32(r, ==, test_vec[i].r);
    simde_assert_m256i_u32(rem, ==, test_vec[i].rem);
  }

  return 0;
}

HEDLEY_DIAGNOSTIC_PUSH
HEDLEY_DIAGNOSTIC_DISABLE_CAST_QUAL

#if HEDLEY_HAS_WARNING("-Wold-style-cast")
  #pragma clang diagnostic ignored "-Wold-style-cast"
#endif
#if HEDLEY_HAS_WARNING("-Wzero-as-null-pointer-constant")
  #pragma clang diagnostic ignored "-Wzero-as-null-pointer-constant"
#endif

SIMDE_TEST_FUNC_LIST_BEGIN
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_acos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_acos_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_acos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_acos_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_acos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_acos_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_acos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_acos_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_acosh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_acosh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_acosh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_acosh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_acosh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_acosh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_acosh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_acosh_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_asin_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_asin_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_asin_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_asin_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_asin_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_asin_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_asin_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_asin_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_asinh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_asinh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_asinh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_asinh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_asinh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_asinh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_asinh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_asinh_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_atan_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_atan_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_atan_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_atan_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_atan_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_atan_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_atan_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_atan_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_atan2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_atan2_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_atan2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_atan2_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_atan2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_atan2_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_atan2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_atan2_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_atanh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_atanh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_atanh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_atanh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_atanh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_atanh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_atanh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_atanh_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cbrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cbrt_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cbrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cbrt_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cbrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cbrt_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cbrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cbrt_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cdfnorm_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cdfnorm_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cdfnorm_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cdfnorm_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cdfnorm_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cdfnorm_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cdfnorm_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cdfnorm_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cdfnorminv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cdfnorminv_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cdfnorminv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cdfnorminv_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cdfnorminv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cdfnorminv_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cdfnorminv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cdfnorminv_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cos_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cos_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cos_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cos_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cosd_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cosd_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cosd_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cosd_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cosd_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cosd_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cosd_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cosd_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cosh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cosh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cosh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cosh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cosh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_cosh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cosh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_cosh_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_cexp_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_cexp_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_clog_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_clog_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_csqrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_csqrt_ps)

  SIMDE_TEST_FUNC_LIST_ENTRY(x_mm_deg2rad_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(x_mm_deg2rad_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(x_mm256_deg2rad_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(x_mm256_deg2rad_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(x_mm512_deg2rad_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(x_mm512_deg2rad_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_div_epi8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_div_epi16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_div_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_div_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_div_epi64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_div_epu8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_div_epu16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_div_epu32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_div_epu64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_div_epi8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_div_epi16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_div_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_div_epi64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_div_epu8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_div_epu16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_div_epu32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_div_epu32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_div_epu64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_div_epi8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_div_epi16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_div_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_div_epi64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_div_epu8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_div_epu16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_div_epu32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_div_epu64)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_erf_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_erf_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_erf_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_erf_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_erf_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_erf_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_erf_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_erf_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_erfinv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_erfinv_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_erfinv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_erfinv_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_erfinv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_erfinv_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_erfinv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_erfinv_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_erfc_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_erfc_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_erfc_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_erfc_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_erfc_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_erfc_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_erfc_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_erfc_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_erfcinv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_erfcinv_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_erfcinv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_erfcinv_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_erfcinv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_erfcinv_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_erfcinv_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_erfcinv_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_exp_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_exp_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_exp_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_exp_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_exp_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_exp_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_exp_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_exp_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_expm1_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_expm1_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_expm1_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_expm1_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_expm1_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_expm1_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_expm1_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_expm1_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_exp2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_exp2_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_exp2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_exp2_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_exp2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_exp2_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_exp2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_exp2_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_exp10_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_exp10_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_exp10_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_exp10_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_exp10_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_exp10_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_exp10_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_exp10_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_idivrem_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_idivrem_epi32)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_hypot_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_hypot_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_hypot_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_hypot_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_hypot_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_hypot_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_hypot_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_hypot_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_invcbrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_invcbrt_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_invcbrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_invcbrt_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_invsqrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_invsqrt_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_invsqrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_invsqrt_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_invsqrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_invsqrt_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_invsqrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_invsqrt_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_log_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_log_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_log_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_log_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_log_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_log_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_log_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_log_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_log1p_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_log1p_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_log1p_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_log1p_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_log1p_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_log1p_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_log1p_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_log1p_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_log2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_log2_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_log2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_log2_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_log2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_log2_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_log2_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_log2_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_log10_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_log10_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_log10_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_log10_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_log10_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_log10_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_log10_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_log10_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_logb_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_logb_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_logb_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_logb_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_logb_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_logb_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_logb_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_logb_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_nearbyint_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_nearbyint_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_nearbyint_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_nearbyint_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_pow_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_pow_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_pow_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_pow_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_pow_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_pow_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_pow_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_pow_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_rem_epi8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_rem_epi16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_rem_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_rem_epi64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_rem_epu8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_rem_epu16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_rem_epu32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_rem_epu64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_rem_epi8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_rem_epi16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_rem_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_rem_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_rem_epi64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_rem_epu8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_rem_epu16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_rem_epu32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_rem_epu32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_rem_epu64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rem_epi8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rem_epi16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rem_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rem_epi64)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rem_epu8)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rem_epu16)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rem_epu32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rem_epu64)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_recip_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_recip_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_recip_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_recip_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rint_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_rint_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_rint_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_rint_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_sin_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_sin_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_sin_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_sin_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_sin_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_sin_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_sin_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_sin_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_sincos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_sincos_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_sincos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_sincos_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_sincos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_sincos_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_sincos_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_sincos_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_sind_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_sind_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_sind_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_sind_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_sind_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_sind_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_sind_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_sind_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_sinh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_sinh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_sinh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_sinh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_sinh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_sinh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_sinh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_sinh_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_svml_ceil_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_svml_ceil_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_svml_ceil_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_svml_ceil_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_ceil_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_ceil_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_ceil_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_ceil_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_svml_floor_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_svml_floor_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_svml_floor_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_svml_floor_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_floor_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_floor_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_floor_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_floor_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_svml_round_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_svml_round_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_svml_round_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_svml_round_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_svml_round_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_svml_round_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_svml_sqrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_svml_sqrt_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_svml_sqrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_svml_sqrt_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_svml_sqrt_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_svml_sqrt_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_tan_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_tan_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_tan_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_tan_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_tan_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_tan_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_tan_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_tan_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_tand_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_tand_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_tand_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_tand_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_tand_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_tand_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_tand_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_tand_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_tanh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_tanh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_tanh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_tanh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_tanh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_tanh_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_tanh_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_tanh_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_trunc_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm_trunc_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_trunc_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_trunc_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_trunc_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_trunc_pd)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_trunc_ps)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm512_mask_trunc_pd)

  SIMDE_TEST_FUNC_LIST_ENTRY(mm_udivrem_epi32)
  SIMDE_TEST_FUNC_LIST_ENTRY(mm256_udivrem_epi32)
SIMDE_TEST_FUNC_LIST_END

#include <test/x86/test-x86-footer.h>
